#!/usr/bin/perl -W
# use strict;

  our $beamline         = '23b';                        # biocat=18ID, gmca=23i, 23o, 23b
  our $undulator        = '';                           # biocat=ID18, gmca=ID23ds,...(?)

  our $shutter          = $beamline.':SH:mp:bo';        # shutter control
  our $shutter_readback = $beamline.':SH:mp:bo:rbk';    # shutter readdback
  our $shutter_open     = 1;                            # Digital I/O status for shutter open
  our $shutter_closed   = 1 - $shutter_open;            # Digital I/O status for shutter closed

# our $HUTCHshutterSTS  = $beamline.':MBSHbm:status';	# endstation shutter status PV: 1=closed, 0=open
  our $HUTCHshutterSTS  = 'PA:23BM:B_BEAM_PRESENT';	# endstation shutter status PV: 1=open,   0=closed
  our $HUTCHshutterOPE  = 'PC:23BM:SBS_OPEN_REQUEST';	# endstation shutter open PV
  our $HUTCHshutterCLO  = 'PC:23BM:SBS_CLOSE_REQUEST';	# endstation shutter close PV
  our $HUTCHshutterOpen = 1;
  our $HUTCHshutterClosed = 1 - $HUTCHshutterOpen;

  our $FEshutter        = 'FE:23:BM:FEshutter';         # write: 1=open, 0=close (no write permissions and not used at GMCA)
# our $FEshutterSTS     = $beamline.':FES:status';      # BM frontend shutter status PV (ACCEL): 1=closed, 0=open
  our $FEshutterSTS     = 'PA:23BM:A_BEAM_PRESENT';	# BM frontend shutter status PV (APS):   1=open,   0=close
  our $FEshutterPOS     = 'EPS:23:BM:PS2:POSITION';	# BM frontend shutter status PV (APS):   0=moving, 1=closed, 2=opened, 3=error
  our $FEshutterOPE     = 'PC:23BM:FES_OPEN_REQUEST';	# BM frontend shutter open PV
  our $FEshutterCLO     = 'PC:23BM:FES_CLOSE_REQUEST';	# BM frontend shutter close PV
  our $FEshutterOpen    = 1;
  our $FEshutterClosed  = 1 - $FEshutterOpen;

  our $struckName       = $beamline.':s1';              # struck name (mcs=BioCAT, s1=GMCA)
  our $struck_ext       = ':mca';                       # struck extension for individual channesl (""=BioCAT, ":mca"=GMCA)
  our $struckChName     = $struckName.$struck_ext;      # struck name (mcs=BioCAT, s1:mca=GMCA)
  our $scanrecName      = $beamline.':2:scan1';         # scanrec name (scan1=BioCAT, 1:scan1=GMCA)
  our $scanrecAbort     = $beamline.':2:AbortScans';    # scanrec abort PV (AbortScans=BioCAT, 1:AbortScans=GMCA)
  our $scanrecPause     = $beamline.':2:scanPause';     # scanrec pause PV (scanPause=BioCAT, 1:scanPause=GMCA)
  our $joergerName      = $beamline.':scaler';          # Joerger scaler name (scaler=BioCAT & GMCA)
# our $bluiceLogPV      = $beamline.':FPE:cmd_input';	# where to write log_note, log_warn and log_error
  our $bluiceLogPV      = $beamline.':FPE:cmd_output256';# where to write log_note, log_warn and log_error
  our $JBLUICE_SERVER   = 'http://10.20.103.207:8001';  # bl3ws2-10g.gmca.aps.anl.gov
  our $robot            = $beamline.':R:';              # sample mounting robot prefix
  our $attenuatorPV     = $beamline.':bo0:ch';		# intensity attenuators channel number prefix in the PV
  our $attenuatrbPV     = $beamline.':rb0:ch';		# intensity attenuators channel number readback prefix in the PV
  our $nAttenuators     = 16;		                # number of intensity attenuators
  our $Attenuators_IN   = 0;				# Digital I/O state of intensity attenuators when IN
  our $axisCameraURL    = '192.168.1.21';		# Axis Video Server URL (sample, on videonet)
  our $axisCameraURL2   = '192.168.1.20';		# Axis Video Camera (hutch)
  our $mysqlHost        = 'bl3upper';			# MySQL host for this beamline
  our $mysqlUser        = 'dhs';			# MySQL user (BluIce)
  our $mysqlPass        = '';				# MySQL pass (BluIce)
  our $mysqlDb          = 'blc2004';			# MySQL DB (BluIce)
  our $HAVE_XYZ         = 1;				# comment out when using crystal XY & gonio St-SH
  our $PLATFORM_IS_64BIT = 1;
  our $CAQTDM           = 1;				# use caqtdm instead of medm
  our $USE_FALCON       = 0;                            # 0=Canberra, 1=Falcon

  if (defined &setEpicsRetry) {&setEpicsRetry();}       # retry no more than 20 times
  if (defined &setEZCA32or64) {&setEZCA32or64();}
