#!/usr/bin/perl -W
#--------------------------------------------------------------
# These are common subs for  most of other scripts
#
#     	             Author: Sergey Stepanov
#
# Version-1:  ..........
# Version-2:  ..........
# Version-3:  2001/11/01
# Version-4:  ..........
# Version-5:  ..........
# Version-6:  ..........
# Version-7:  ..........
# Version-8:  ..........
# Version-9:  ..........
# Version-10: ..........
# Version-11: 2009/09/15  added wait4pv
# Version-12: 2009/09/23  added isMounted
# Version-13: 2010/02/19  added tellBluIce
# Version-14: 2010/02/24  added PezcaGetStringWF and PezcaPutStringWF
# Version-15: 2010/11/04  added disableInterrupts()
# Version-16: 2011/01/06  added popup_note,popup_error to tellBluIce and
#                               switch for Zenity to use BluIce popups
# Version-17: 2011/01/11  added timestamp2epoch
# Version-18: 2012/02/03  added check for zero-length array to many routines.
# Version-19: 2012/02/29  added Which(prg).
# Version-20: 2012/10/26  added split_filename_dirname and replaced win32 detection by $^O
# Version-21: 2014/05/02  added try_load_module()
# Version-22: 2014/10/09  added function_exists();
#                         added calling stack trace to report_exit() and report_only() {needs "use Carp qw(shortmess);"}
# Version-23: 2014/12/04  added wait4topup2happen()
# Version-24: 2015/03/17  added check_Nrunning_and_host_allowed
# Version-25: 2015/05/17  added execute_external_script(\@cmdline) and abort_external_script(;$pid)
# Version-27: 2015/06/09  added zenity_progress($time,$header);
# Version-28: 2016/11/04  added &is_float($str), &is_decimal($str) and &is_int($str)
# Version-29: 2017/06/01  added &is_mounted($dir) and &is_mountpoint($dir)
# Version-30: 2018/01/30  added &try_do($file) and &try_do2($file)
# Version-31: 2018/02/05  added &read_jbluice_config ($key,\$val)
# Version-32: 2018/11/16  added &pt2coord($xpt,\@xarray)
# Version-33: 2018/11/16  added zenity-via-bluice interaction via HTTP with JBluIce server
# Version-34: 2019/11/06  added &getSampleEnv(\%states);
# Version-35: 2020/11/06  added &zenity_list($txt,\@choices,[$default])
# Version-36: 2021/06/22  added &import_elements(\@elements)
# Version-37: 2023/05/09  added &try_load_lib and &try_load_Pezca_local_or_global
#--------------------------------------------------------------
# List of functions:
# ^^^^^^^^^^^^^^^^^
# 1.                          $status = &try_load_module($module,[\@functions,$silent]);
# 2.                          $status = &try_load_lib($libdir,[$silent]);
# 3.                                    &try_load_Pezca_local_or_global($Pezca_path);
# 4.                                    &try_do($file);							#version of "do $file" with error checking (Linux-specific)
# 5.                                    &try_do2($file);						#version of "do $file" with error checking (little less strict than above, but not Linux-specific)
# 6.                         $funcref = &function_exists($funcname);					#after call: if (defined $funcref) {$funcref->(@args);}
# 7.                                    &setEZCA32or64();
# 8.                                    &setEpicsRetry();
# 9.                                    &setEpicsMessages($onoff);					#1=on 0=off
#10.                  ($status,$nelm) = &PezcaGetNelm($pv,['noerr']);
#11.                ($status,$string) = &PezcaGetStringWF($pv,[$nelm,'noerr']);
#12.                          $status = &PezcaPutStringWF($pv,$string,['oldca',$nelm,'group','noerr']);
#13                           $status = &addCaMonitor($pv,[$monitorType,$LUN]);
#14                             $nerr = &addCaMonitors([\@dataPVs,\@stringPVs,$LUN]);
#15.                          $status = &clearCaMonitor($pv,[$LUN]);
#16.                          $status = &resetCaMonitor($pv,[$monitorType,$LUN]);
#17.                            $nerr = &clearAllCaMonitors([$LUN]);
#18.                            $nerr = &resetAllCaMonitors(['list',$LUN]);
#19.                    $statusSTRING = &ezcaStatusCode($status);
#20.                                    &report_exit($program,$status,$pv,[$LUN,$cmd]);
#21.                                    &report_only($program,$status,$pv,[$LUN,$cmd]);
#22.                          $status = &wait4pv($pv,$target,$timeout,$poll,[$delay,$counter,$tolerance,$progress]);	#0=OK, 1=timeout, -1=PV_problem
#23.                        $continue = &ask_to_continue([$msg,$prompt,'noexit']);			#if 'noexit' specified, the function returns 1=continue,0=exit
#24.                         $consent = &ask_to_overwrite($file);
#25.                                    &press_key_continue([$msg,$prompt]);
#26.                                    &press_key_exit([$msg,$exitcode,'info']);
#27.                                    &zenity_info ($message,['nowait']);
#28.                                    &zenity_error ($message,[$stderr,'nowait']);
#29.                                    &zenity_error_exit ($message,[$stderr,'nowait']);
#30.                                    &zenity_proc (\$message,$type,[$stderr,$nowait]);		#$type=info|error
#31.                           $yesno = &zenity_question ($message,[$Yeslbl,$Nolbl]);			#returns: 1=OK/Yes, 0=Cancel/No
#32.                     $choiceIndex = &zenity_list($message,\@choices,[$default]);                    #returns index of choice or -1 if cancel or none selected
#33.                                    &zenity_bring_on_top($title);					#for Linux only, needs wmctrl
#34.                                    &disable_interrupts([$message]);
#35.                         $radians = &asinSS($val);							#uses asin from PDL or Math::Trig, if available
#36.                         $max_val = &maxval(@array);						#replaces List::Util qw(min max);
#37.                         $min_val = &minval(@array);						#replaces List::Util qw(min max);
#38.                         $max_val = &maxvalRef(\@array);						#replaces List::Util qw(min max);
#39.                         $min_val = &minvalRef(\@array);                                            #replaces List::Util qw(min max);
#40.                           $i_min = &findIndexMinRef(\@array);
#41.                           $i_max = &findIndexMaxRef(\@array);
#42.                           $y_max = &findMaxRef(\@array);
#43.                  ($y_min,$y_max) = &findMinMaxRef(\@array,[$npts]);
#44.                ($rmsVal,$rmsDev) = &findRMS(\@array);
#
#45.        ($y_max,$pt_max,$pt_FWHM,
#                 $pt_left,$pt_right,
#         [$x_left,$x_right,$x_peak]) = &findPeak($npts,@array);
#
#46.        ($y_max,$pt_max,$pt_FWHM,
#                 $pt_left,$pt_right,
#         [$x_left,$x_right,$x_peak]) = &findNoisyPeak($npts,@array);
#
#47. ($y_max,$y_min,$pt_max,$pt_min,
#         $pt_FWHM,$pt_left,$pt_right,
#         [$x_left,$x_right,$x_peak]) = &findPeakRef($npts,\@array,[$bkgmode,'smooth|noisy']);
#
#48. ($y_max,$y_min,$pt_max,$pt_min,
#         $pt_FWHM,$pt_left,$pt_right,
#        [$x_left,$x_right,$x_valey]) = &findValleyRef($npts,\@array,[$bkgmode,'smooth|noisy']);
#
#49.             ($x_centroid,[$sum]) = &findCentroidRef($npts,\@arrx,\@arry,[$discriminator]);
#50.             ($x_centroid,[$sum]) = &findCentroidFixRef($npts,$x0,$dx,\@arry,[$discriminator]);
#51.                                    &smoothArray(\@array);
#52.  ($pt_min,$y_min,$pt_max,$y_max) = &arrayStatsRef(\@array);
#53.  ($pt_min,$y_min,$pt_max,$y_max) = &arrayStats($npts,@array);
#54.                     ($name,$ext) = &analyzeFilename($file);
#55.                                    &set_interrupt();
#56.                          $status = &check_interrupt();
#57.                      $scriptName = &makeScriptName([1|'short'|'noext']);					#optionally remove .pl
#58.                       $age_hours = &getFileAgeHours($file);						#returns -1 if file does not exist.
#59.                       $hostEmail = &get_host_email_from_UAP($currentUser,$script);				#uses WebUAP (sql)
#59old.                    $hostEmail = &get_host_email_from_UAP_old($currentUser,$script,$os);			#uses old UAP (file)
#60.                           $yesno = &isMounted($mountPoint);
#61.                          $status = &tellBluIce($msg,[$type]);						#type=note,warn,error,popup_note,popup_error,direct
#62.                          $status = &setSampleEnv(\@commands,['nowait']);
#63.                          $status = &getSampleEnv(\%states);						#fills associative array of states
#64.                 ($status,[$msg]) = &read_data_file ($file,\@xdata,\@ydata,[$xcol,$ycol]); 			#if xcol=0, fills @xdata with pt number
#65.                           $epoch = &timestamp2epoch($timestamp);						#stamp is YYYY-MM-DD HH:MM[:SS] or YYYY/MM/DD HH:MM[:SS]
#66.                          $sorted = &sort_by_x('incr|decr',\@x,@y1,[\@y2,\@y3,...]);			#1=sorted,0=did_not_need,-1=error
#67.                            $npts = &remove_duplicates($filter,$step,\@x,@y1,[\@y2,\@y3,...]);		#the arrays must be sorted over x and 'quasi-regular'
#68.                            $path = &Which($prg);								#returns path to $prg or undef if not found. On win32 tries .exe,.bat,.com
#69.             ($dirname,$filename) = &split_filename_dirname($filespec);
#70.                         $radians = &degrees2rad($degrees);
#71.                         $degrees = &rad2degrees($radians);
#72.                          $status = &wait4topup2happen([$topupInterval,$timeAfterTopup]))
#73.                     $description = &getPVdescription($pv);
#74.                                    &check_Nrunning_and_host_allowed([$scriptname,\@allowed_hosts]);        #works both Linux & Windows, default hosts ws2, ws3.
#75.                        $nrunning = &count_Nrunning_gen([\@processes]);					#count Nrunning on Linux and Windows (generic) 
#76.                        $nrunning = &count_Nrunning_lsof([\@processes]);					#count Nrunning with lsof on Linux
#77.                           $index = &find_first_index($element,\@array);					#returns -1 if nothing found
#78.                           @index = &find_all_index($element,\@array);					#returns empty list if nothing found
#79.                           $index = &find_closest_index($val,\@array);					#finds index of element closest to given value
#80.                          $status = &execute_external_script(\@cmdline);                                    #returns error if cannot execute
#81.                                    &abort_external_script(;$pid);						#uses global $EXTERNAQL_PID if $pid not passed
#82.                          $status = &zenity_progress($time,$header);					#returns 1 if process was interrupted
#83.                            $bool = &is_float($str);							#returns 1/0 (True/False if the value is any float incl. exponents)
#84.                            $bool = &is_decimal($str);							#returns 1/0 (True/False if the value is like xxx.yyy)
#85.                            $bool = &is_int($str);								#returns 1/0 (True/False)
#86.                            $bool = &is_mounted($dir,[$refdir]);						#returns 1/0/-1 (True/False/NofFound)
#87.                            $bool = &is_mountpoint($dir);							#returns 1/0/-1 (True/False/NofFound)
#88.                       $bitstring = &dec2bin($intword,['r[everse]','t[rim]'|$nbits]);			#convert integer into bits string; specify 'reverse' for bits parsing
#89.                         $intword = &bin2dec($bitstring);							#convert bits string into integer
#90.                           $int16 = &uint16_int16($uint16);							#convert from unsigned int16 to signed int16
#91.                           $int32 = &uint32_int32($uint32);							#convert from unsigned int32 to signed int32
#92.                          $status = &read_jbluice_config ($key,\$val);					#read strings from JBluIce Config table
#93.                          $status = &read_jbluice_config_flags ($key,\@flags);				#read strings from JBluIce Config table and split into flags
#94.                          $xcoord = &pt2coord($xpt,\@xarray);                                               #convert non-integer pt into coordinate
#95.                            $host = &gethost(['short');
#96.                                    &sort_hash_array(\@hasharray,$key);
#97.                          $status = &read_entire_file($file,\@lines,[$lock]);				#returns 1=error, 0=OK
#98.                          $status = &open_medm($adl,[$par]);
#99.                             $msg = &call_trace([$LUN]);							#function calling trace (by default prints to STDOUT)
#100.                         $status = &import_elements(\@elements);                   			#fills hash array of the Periodic Table elements (name, code, z, a)
#--------------------------------------------------------------
# use Pezca;							#most functions will need it
# use ENV;							#load the ENV
# use Time::Local;						#needed by timestamp2epoch
# use Carp qw(shortmess);					#may be needed for stack trace in report_exit() and report_only()
# use Tk;							#may be needed by zenity_progress (MS Windows only)
# use Tk::ProgressBar;						#may be needed by zenity_progress (MS Windows only)
# use POSIX ":sys_wait_h";					#may be used by execute_external_script and abort_external_script
# use LWP::UserAgent;						#may be needed by zenity-via-JBluIce

# use warnings::unused;
  use strict;
# no strict "vars";
# no strict "subs";
# no strict "refs";

#=================================================================

#sub try_load_module($module,[\@functions,$silent]) {
sub try_load_module($;\@) {
  my ($script, $module, $funcref, $silent, $i, $e);

  $module  = shift(@_);
  $funcref = shift(@_);
  $silent  = shift(@_);		if (!defined $silent) {$silent = 0;}

  select STDOUT; $|=1;					#set unbuffered output

  if (defined $funcref && @$funcref > 0) {
     for ($i=0; $i<@$funcref; $i++) {
        if ($i == 0) {$module .= ' qw/'.@$funcref[$i];}
        else         {$module .= ' '.@$funcref[$i];}
     }
     $module .= '/';
  }
# print STDOUT 'use '.$module."\n";

  eval('use '.$module);
  ### $@ is set if the string to be eval-ed did not compile or if Perl code
  ### executed during evaluation die()d. In these cases the value of $@ is
  ### the compile error, or the argument to die.
  if ($@) {
     if (! $silent) {
        $e = $@;
        $script = &makeScriptName();
        print STDOUT "\n".'*** '.$script.': module "'.$module.'" failed to load.'."\n".$e."\n\n";
     }
     return 1;
  } else {
     return 0;
  }
}

#=================================================================

#sub try_load_lib($libdir,[$silent]) {
sub try_load_lib($;$) {
  my ($script, $libdir, $silent, $e);

  $libdir = shift(@_);
  $silent  = shift(@_);		if (!defined $silent) {$silent = 0;}

  select STDOUT; $|=1;					#set unbuffered output

# print STDOUT 'use lib "'.$libdir.'"'."\n";

  eval('use lib "'.$libdir.'"');
  ### $@ is set if the string to be eval-ed did not compile or if Perl code
  ### executed during evaluation die()d. In these cases the value of $@ is
  ### the compile error, or the argument to die.
  if ($@) {
     if (! $silent) {
        $e = $@;
        $script = &makeScriptName();
        print STDOUT "\n".'*** '.$script.': failed to load library from dir='.$libdir."\n".$e."\n\n";
        return 1;
     }
  } else {
     return 0;
  }
}

#=================================================================

#sub try_load_Pezca_local_or_global($Pezca_path) {
sub try_load_Pezca_local_or_global($) {
### ---------------------------------------------------
### This is for local Pezca library installation
### (i.e. when we do not have Pezca installed OS-wide):
### ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
### perl Makefile.PL PREFIX=$PEZCA/Pezca
### make
### make install
### - - - - - - - - - - - - - - - 
### A replacement to the "use lib" statement is:
### export PERL5LIB=$PEZCA/Pezca/lib/site_perl
### ---------------------------------------------------
  my ($Pezca_path, @locations, $libdir, $status);
  $status = &try_load_module('Pezca',undef,1);
  if ($status == 0) {return;}		#all good, there is system Pezca installation
  $Pezca_path = shift(@_);
  if ($Pezca_path !~ /\/$/) {$Pezca_path .= '/';}
  if ( -d $Pezca_path) {
     @locations = split(/\n/, `find  "$Pezca_path" -name Pezca.so`);
     foreach $libdir (@locations) {
        $libdir =~ s/Pezca\.so$//;
#       print STDOUT 'so-libdir = '.$libdir."\n";
        if (length($libdir) > 0 && -d $libdir) {
           $status = &try_load_lib($libdir);
           if ($status) {die '*** failed to load lib from '.$libdir."\n";}
        }
     }
     @locations = split(/\n/, `find  "$Pezca_path" -name Pezca.pm`);
     foreach $libdir (@locations) {
        $libdir =~ s/Pezca\.pm$//;
#       print STDOUT 'pm-libdir = '.$libdir."\n";
        if (length($libdir) > 0 && -d $libdir) {
           push (@INC,$libdir);
        }
     }
  }
### Now try to load the Pezca module, either local or global:
  if (&try_load_module('Pezca')) {die '*** Cannot load Pezca';}
}

#=================================================================

#sub try_do($file) {                                                    # Linux-specific!
sub try_do($) {
  my ($file, $st);
  $file = shift(@_);
  if (-e $file) {
     $st = eval `cat $file`;                                            #"cat" is a Linux command.
     if ($@)              {die 'Failed parsing '.$file.': '.$@;}        #'$@' is error message
     elsif ($!)           {die 'Could not eval '.$file.': '.$!;}        #'$!' is error message
#    elsif (!defined $st) {die 'Failed to eval '.$file.': '.$!;}        #this does not work when $file contains subs: then $st is undef
#    elsif (! $st)        {die 'Failed to run '.$file;}                 #this does not work when $file contains subs: then $st is undef
  } else                  {die 'Did not find '.$file;}
}

#=================================================================

#sub try_do2($file) {                                                   #this is less strict than try_do(), but not Linux-specific
sub try_do2($) {
  my ($file, $st);
  $file = shift(@_);
  if (-e $file) {
     $st = do $file;
     if ($@)              {die 'Failed parsing '.$file.': '.$@;}        #'$@' is error message
     elsif ($!)           {die 'Could not do '.$file.': '.$!;}          #'$!' is error message
#    elsif (!defined $st) {die 'Failed to do '.$file.': '.$!;}          #this does not work when $file contains subs: then $st is undef
#    elsif (! $st)        {die 'Failed to run '.$file;}                 #this does not work when $file contains subs: then $st is undef
  } else                  {die 'Did not find '.$file;}
}

#=================================================================

sub function_exists ($) {
  ### This function determines if function exists.
  ### An example use:
  ### $funcref = &function_exists($funcname);
  ### if (defined $funcref) {$funcref->(@args);}	#the way to call function by name
  ### else                  {die $funcname.' is undefined';}
  my $funcname = shift;
  if (defined &{$funcname}) {return \&{$funcname};}
  else                      {return;}
}

#=================================================================

sub setEZCA32or64 () {
  our ($PLATFORM_IS_64BIT, $EZCALONG, $EZCAFLOAT);

### 'PLATFORM_IS_64BIT' is defined in beamline.pl . It may be a problem
### if a beamline contans a mix of 32-bit and 64-bit computers. There
### is a way to autodetect OS bitness using Perl Config module:
#
# use Config;
# my $os = $Config{longsize}*8;
# if ($os eq 64) {$PLATFORM_IS_64BIT = 1;}
# else           {$PLATFORM_IS_64BIT = 0;}

### Another, more home-grown way to determine 'PLATFORM_IS_64BIT':
  if (!defined $PLATFORM_IS_64BIT) {
     #print STDERR '!!! ATTENTION: the PLATFORM is not defined. Guessing from the PATH...  ';
     if ($ENV{PATH} =~ /lib64/ || $ENV{PATH} =~ / \(x86\)/) {		#for Linux & Windows respectively
        $PLATFORM_IS_64BIT = 1;
        #print STDOUT '..platform=64bit'."\n";
     } else {
        $PLATFORM_IS_64BIT = 0;
        #print STDOUT '..platform=32bit'."\n";
     }
  }

  if ($PLATFORM_IS_64BIT) {
### ezcaLong does not work correctly under 64 bit. Ideally, it should be
### replaced by ezcaDouble, but the later may lead to overflowing the
### default buffer of EPICS Channel Access. The later is controlled by the
### EPICS_CA_MAX_ARRAY_BYTES, which defaults to 16384 (see envData.c in
### EPICS base). With Scan Record and Struck scaler arrays of 4000 elements
### and the 16-bytes-length of 64-bit DOUBLE, using ezcaDouble would lead
### to 16*4000=64000 bytes. So, if we want to use ezcaDouble, we have to
### increase EPICS_CA_MAX_ARRAY_BYTES by the factor of two, both at the IOC
### and the clients sides. Otherwice, we need to use ezcaFloat. The later may
### lead to some rounding errors when the numbers are high, but it should be
### good enough for intensity. The only place where rounding is not tolerable
### is where the numbers are planned to be bit-parsed, as in the case of the
### array of excluded cells in polygon rastering. Fortunately, that array is
### short (only 34 elements) and we can safely use ezcaDouble there.
     $EZCALONG  = 'FLOAT';
#    $EZCALONG  = 'DOUBLE';
  } else {
     $EZCALONG  = 'LONG';
  }
  $EZCAFLOAT    = 'FLOAT';
}

#=================================================================

sub setEpicsRetry () {

  our ($nRetry, $report);
  my ($script, $tmout, $retry_count, $status);

  $script = 'setEpicsRetry';
  if (!defined $nRetry) {$nRetry = 20;}
  elsif ($nRetry <= 0)  {$nRetry = 20;}			#the default value
  if (!defined $report) {$report = 0;}
# $report = 1;
  if (!defined &Pezca::GetTimeout || !defined &Pezca::SetRetryCount) {
#    print STDOUT '!!! '.$script.': Pezca is not loaded, skipping...'."\n";
     return;
  }
  if ($report) {
     $tmout = Pezca::GetTimeout();
     $retry_count = Pezca::GetRetryCount();
     if ($retry_count != $nRetry) {
        printf STDOUT "\n".$script.': timeout=%.3fs  retry='.$retry_count.' -> '.$nRetry."\n\n", $tmout;
        $status = Pezca::SetRetryCount($nRetry);
     }
     else {$status = 0;}
  }
  else {$status = Pezca::SetRetryCount($nRetry);}

  if ($status) {
    print STDOUT "\n".'!!! '.$script.': status='.$status.' from "Pezca::SetRetryCount"'."\n\n";
  }
}

#=================================================================

sub setEpicsMessages($) {

  my ($script, $onoff);

  $script = 'setEpicsMessages';
  $onoff  = shift(@_);

  if (defined $onoff) {
     if (!defined &Pezca::AutoErrorMessageOff || !defined &Pezca::AutoErrorMessageOn) {
#       print STDOUT '!!! '.$script.': Pezca is not loaded, skipping...'."\n";
        return;
     }
     if    ($onoff eq '1' || $onoff =~ /^on$/i)  {&Pezca::AutoErrorMessageOn();}
     elsif ($onoff eq '0' || $onoff =~ /^off$/i) {&Pezca::AutoErrorMessageOff();}
     else {print STDOUT '!!! '.$script.': incorrect On/Off flag='.$onoff."\n";}
  } else {print STDOUT '!!! '.$script.': undefined On/Off flag.'."\n";}
  return;
}

#=================================================================

#($status,$nelm) = &PezcaGetNelm ($pv,['noerr']);
sub PezcaGetNelm($;$) {

  my ($script, $pv, $noerr, $status, $nelm);

  $script = 'PezcaGetNelm';
  $pv     = shift(@_);
  $noerr  = shift(@_);						#can be '1' or 'noerr' to suppress error message
  if (!defined $noerr) {$noerr='printerr';}

  $pv =~ s/\.VAL$//;						#truncate VAL if given
### Read max number of elements in the waveform:
  ($status,$nelm) = Pezca::GetDouble($pv.'.NELM');
  if ($status) {
     if ($noerr ne '1' && $noerr !~ /^noerr$/)  {&report_only($script,$status,$pv.'.NELM');}
     return $status,-1;
  }
  if (! &is_int($nelm) || $nelm < 1) {
     print STDOUT "\n".'*** '.$script.': incorrect NELM='.$nelm.' for PV='.$pv."\n\n";
     return -1,-1;
  }
  return 0,$nelm;
}

#=================================================================

#($status,$string) = &PezcaGetStringWF ($pv,[$nelm,'noerr']);
sub PezcaGetStringWF($;$$) {

  my ($script, $pv, $nelm, $noerr, $status, $string, @array, $nchar, $i);

  $script = 'PezcaGetStringWF';
  $pv     = shift(@_);
  $nelm   = shift(@_);						#if not specified, will be asked
  $noerr  = shift(@_);						#can be '1' or 'noerr' to suppress error message
  if (!defined $noerr) {$noerr='printerr';}
  $string = '';

  $pv =~ s/\.VAL$//;						#truncate VAL if given
### Read possible number of elements in the waveform:
  if (!defined $nelm) {
     ($status,$nelm) = &PezcaGetNelm ($pv,$noerr);
     if ($status) {return $status,$string;}
  }
  if (! &is_int($nelm) || $nelm < 1) {
     print STDOUT "\n".'*** '.$script.': incorrect NELM='.$nelm.' for PV='.$pv."\n\n";
     return -1,$string;
  }

### Read the waveform:
  ($status,@array) = Pezca::GetList($pv,'CHAR',$nelm);
  if ($status) {
     if ($noerr ne '1' && $noerr !~ /^noerr$/)  {&report_only($script,$status,$pv);}
     return $status,$string;
  }
  $nchar = @array;
  for ($i=0; $i<$nchar; $i++) {
     if ($array[$i] == 0) {$nchar=$i; last;}
     $array[$i] = chr($array[$i]);				#convert from char codes
  }
  @array = splice(@array,0,$nchar);
  $string = join('',@array);					#merge from array
  $string =~ s/ *$//g;						#remove end-of-the-line spaces
  return $status,$string;
}

#=================================================================

#$status = &PezcaPutStringWF($pv,$string,['oldca',$nelm,'group','noerr']);
sub PezcaPutStringWF($$;$$$) {

  my ($script, $pv, $oldca, $nelm, $group, $noerr);
  my ($status, $string, @array, $nchar, $i);

  $script = 'PezcaPutStringWF';
  $pv     = shift(@_);
  $string = shift(@_);
  $oldca  = shift(@_);		if (!defined $oldca) {$oldca='';}
  $nelm   = shift(@_);
  $group  = shift(@_);		if (!defined $group) {$group=0;}
  $noerr  = shift(@_);						#can be '1' or 'noerr' to suppress error message
  if (!defined $noerr) {$noerr='printerr';}

  $pv =~ s/\.VAL$//;						#truncate VAL if given
### Read possible number of elements in the waveform:
  if (!defined $nelm) {
     ($status,$nelm) = &PezcaGetNelm ($pv);
     if ($status) {return $status;}
  }
  if (! &is_int($nelm) || $nelm < 1) {
     print STDOUT "\n".'*** '.$script.': incorrect NELM='.$nelm.' for PV='.$pv."\n\n";
     return -1;
  }
  $nchar = length($string);
  if ($nchar > $nelm-1) {
     printf STDERR "\n".'*** '.$script.': too long string='.$string.
                   "\n".'*** '.$script.': the string length='.$nchar.' exceeds %d'."\n\n",$nelm-1;
     $string = substr($string,0,$nelm-1);
     $nchar = $nelm-1;
  }
  if ($nchar > 0) {
     @array = split('',$string);				 #split to arrray
     for ($i=0; $i<$nchar; $i++) {$array[$i] = ord($array[$i]);} #convert to char codes
  } else {
#    @array = ();
     @array = 0;						#change of 2014/11/04
  }

### Put the waveform:
  if ($oldca =~ /^oldca$/i) {$status = Pezca::PutListOldCa($pv,'CHAR',@array);}
  else                      {$status = Pezca::PutList     ($pv,'CHAR',@array);}
  if ($group eq '0' || $group =~ /^no/) {
     if ($noerr ne '1' && $noerr !~ /^noerr$/)  {&report_only($script,$status,$pv);}
  } else {
     $status = 0;
  }
  return $status;
}

#=================================================================

# sub addCaMonitor ($pv, [$monitorType,$LUN]) {
sub addCaMonitor ($;$$) {

### $monitorType is an optional argument: double/string

  our (@CaMonitorsArray, @CaMonitorsTypes);
  our ($IGNORE_ERRORS, $USE_ZENITY_FOR_EXIT);
  my ($script, $pv, $monitorType, $LUN);
  my ($status, $dummy, $CaArrayLen, $i, $msg, $debug);

  $script      = 'addCaMonitor';
  $pv          = shift(@_);
  $monitorType = shift(@_);
  $LUN         = shift(@_);
  if (!defined $monitorType)         {$monitorType = 'double';}
  if (!defined $LUN)                 {$LUN = \*STDOUT;}
  if (!defined $IGNORE_ERRORS)       {$IGNORE_ERRORS = 0;}
  if (!defined $USE_ZENITY_FOR_EXIT) {$USE_ZENITY_FOR_EXIT = 0;}
  if (!defined $pv) {
     $msg  = '*** '.$script.': no PV specified.';
     if ($USE_ZENITY_FOR_EXIT) {&zenity_error_exit ($msg);}
     else {&press_key_exit("\n".$msg,1);}
  }
# $debug = 1;					#comment out for normal operations
  if (!defined $debug) {$debug=0;}

  if ((! $monitorType =~ /^double$/i) && (! $monitorType =~ /^string$/i)) {
     $msg  = '*** '.$script.': unknown CA-monitor type='.$monitorType
           . ' -- must be "string" or "double"';
     if ($USE_ZENITY_FOR_EXIT) {&zenity_error_exit ($msg);}
     else {&press_key_exit("\n".$msg,1);}
  }
  if (! @CaMonitorsArray) {
     @CaMonitorsArray = ();			# global array
     @CaMonitorsTypes = ();			# global array
  }
  $CaArrayLen = @CaMonitorsArray;
  if ($debug) {print STDOUT $script.': current CA-monitors array length = '.$CaArrayLen."\n";}

### Check if the monitor was set before:
  for ($i=0; $i<$CaArrayLen; $i++) {
     if ($pv eq $CaMonitorsArray[$i]) {
        if ($debug) {print STDOUT $script.' warning: Monitor on PV='.$pv.' is already set!'."\n";}
#       $status = -1;
        $status = 0;
        return $status;
     }
  }
  if ($monitorType =~ /^double$/i) {
     ($status,$dummy) = Pezca::GetDouble($pv);
     if ($debug) {print STDOUT $script.': pv='.$pv.' status='.$status."\n";}
### This should also clear all monitors (&clearAllCaMonitors):
     if ($status) {
        &report_exit($script, $status, $pv, $LUN);
        return $status;
     }
     $status = Pezca::SetMonitorDouble($pv);
     if ($status) {
        &report_exit($script, $status, $pv, $LUN);
        return $status;
     }
  } else {
     ($status,$dummy) = Pezca::GetString($pv);
     if ($debug) {print STDOUT $script.': pv='.$pv.' status='.$status."\n";}
### This should also clear all monitors (&clearAllCaMonitors):
     if ($status) {
        &report_exit($script, $status, $pv, $LUN);
        return $status;
     }
     $status = Pezca::SetMonitorString($pv);
     if ($status) {
        &report_exit($script, $status, $pv, $LUN);
        return $status;
     }
  }

### Add $pv to the array:
  if ($debug) {print STDOUT $script.': added CA-monitor for pv='.$pv.', type='.$monitorType."\n";}
  $CaMonitorsArray[$CaArrayLen] = $pv;
  $CaMonitorsTypes[$CaArrayLen] = $monitorType;
  return $status;
}

#=================================================================


# sub addCaMonitors([\@dataPVs,\@stringPVs,$LUN]) {	#returns status (number of errors)
sub addCaMonitors (;\@\@$) {

  my ($script, $dataPVsRef,$stringPVsRef, $LUN);
  my ($status, $nerr, $pv, $monitorType);

  $script       = 'addCaMonitors';
  $dataPVsRef   = shift(@_);
  $stringPVsRef = shift(@_);
  $LUN          = shift(@_);
  $nerr = 0;
  if (defined $dataPVsRef) {
     $monitorType = 'double';
     foreach $pv (@$dataPVsRef) {
        $status = &addCaMonitor ($pv,$monitorType,$LUN);
        if ($status) {$nerr++;}
     }
  }
  if (defined $stringPVsRef) {
     $monitorType = 'string';
     foreach $pv (@$stringPVsRef) {
        $status = &addCaMonitor ($pv,$monitorType,$LUN);
        if ($status) {$nerr++;}
     }
  }
  return $nerr;
}

#=================================================================

# sub clearCaMonitor($pv, [$LUN]) {
sub clearCaMonitor ($;$) {

  our (@CaMonitorsArray, @CaMonitorsTypes, $report);
  my ($script, $pv, $LUN, $status, $i);
  my ($CaArrayLen, $monitorType, $statusDescr);

  $script = 'clearCaMonitor';
  $pv     = shift(@_);
  $LUN    = shift(@_);
  if (!defined $LUN)    {$LUN = \*STDOUT;}
  if (!defined $report) {$report = 0;}

  if (! @CaMonitorsArray) {
     @CaMonitorsArray = ();			# global array
     @CaMonitorsTypes = ();			# global array
  }
  $CaArrayLen = @CaMonitorsArray;
# print STDOUT $script.': current CA-monitors array length = '.$CaArrayLen."\n";

### Check if the monitor was set before:
  for ($i=0; $i<$CaArrayLen; $i++) {
     if ($pv eq $CaMonitorsArray[$i]) {
        $monitorType = $CaMonitorsTypes[$i];
        if ($monitorType =~ /^string$/i) {$status = Pezca::ClearMonitorString($pv);}
        else                             {$status = Pezca::ClearMonitorDouble($pv);}
        if ($status) {
           $statusDescr = &ezcaStatusCode($status);
           print $LUN '!!! '.$script.': error='.$status.' ['.$statusDescr.']'
                         .' from EZCA while clearing CA-monitor ['.$pv.']'."\n";
        }
### Remove the element for the array:
        splice (@CaMonitorsArray, $i, 1);
        splice (@CaMonitorsTypes, $i, 1);
#       print $LUN $script.': removed CA-monitor for pv='.$pv.', type='.$monitorType."\n";
        return $status;
     }
  }
### We are here if monitor was not found in the array
  if ($report) {print $LUN $script.' warning: CA-monitor on ['.$pv.'] was not set!'."\n";}
  return -1;
}

#=================================================================

# sub resetCaMonitor ($pv, [$monitorType, $LUN]) {
sub resetCaMonitor ($;$$) {

### $monitorType is an optional argument: double/string

  my ($pv, $monitorType, $LUN, $status);

  $pv          = shift(@_);
  $monitorType = shift(@_);
  $LUN         = shift(@_);
  if (!defined $monitorType) {$monitorType = 'double';}
  if (!defined $LUN)         {$LUN = \*STDOUT;}
  &clearCaMonitor($pv,$LUN);
  $status = &addCaMonitor($pv,$monitorType,$LUN);
  return $status;
}

#=================================================================

# sub clearAllCaMonitors([$LUN]) {
sub clearAllCaMonitors (;$) {

  our (@CaMonitorsArray, @CaMonitorsTypes);
  my ($script, $LUN, $CaArrayLen, $pv, $monitorType);
  my ($i, $status, $statusDescr, $totalErrors, $debug);

  $script = 'clearAllCaMonitors';
  $LUN = shift(@_);
  if (!defined $LUN) {$LUN = \*STDOUT;}

  $totalErrors = 0;
# $debug = 1;					#comment out for normal operations
  if (!defined $debug) {$debug=0;}

  if (@CaMonitorsArray == 0) {
     @CaMonitorsArray = ();			# global array
     @CaMonitorsTypes = ();			# global array
     return $totalErrors;
  }

  $CaArrayLen = @CaMonitorsArray;
  if ($debug) {print STDOUT $script.': current CA-monitors array length = '.$CaArrayLen."\n";}

### Short sleep in case one of the monitors was just added.
### Immediately removing just added monitor may case freezing
### on the Pezca::ClearMonitor function	
  select(undef,undef,undef,0.1);		# sleep(..) -- added 2021/10/01

  for ($i=0; $i<$CaArrayLen; $i++) {
#    if ($i>0) {select(undef,undef,undef,0.1);}	# sleep(..) -- added 2021/10/01 (same purpose as above, but slower)
     $pv          = $CaMonitorsArray[$i];
     $monitorType = $CaMonitorsTypes[$i];
     if ($debug) {print STDOUT $script.': removing CA-monitor for pv='.$pv.', type='.$monitorType;}
     if ($monitorType =~ /^string$/i) {$status = Pezca::ClearMonitorString($pv);}
     else                             {$status = Pezca::ClearMonitorDouble($pv);}
     if ($debug) {print STDOUT ' ... status='.$status."\n";}
     if ($status) {
        $totalErrors++;
        $statusDescr = &ezcaStatusCode($status);
        print $LUN '!!! '.$script.': error='.$status.' ['.$statusDescr.']'
                         .' from EZCA while clearing CA-monitor ['.$pv.']'."\n";
     }
  }
  @CaMonitorsArray = ();			# clear global array
  @CaMonitorsTypes = ();			# clear global array
  return $totalErrors;
}

#=================================================================

# sub resetAllCaMonitors(['list|silent',$LUN]) {		#returns $status (if $IGNORE_ERRORS=1)
sub resetAllCaMonitors (;$$) {

### This routine re-establishes all CA monitors after an IOC reboot.
  our (@CaMonitorsArray, @CaMonitorsTypes);
  my ($script, $scriptMAIN, $list, $LUN);
  my ($CaArrayLen, $pv, $monitorType, $i);
  my ($status, $statusDescr, $statusALL, $current_time);

  $script = 'resetAllCaMonitors';
  $list = shift(@_);  if (!defined $list) {$list='silent';}
  $LUN  = shift(@_);  if (!defined $LUN)  {$LUN = \*STDOUT;}

  if (! @CaMonitorsArray) {
     @CaMonitorsArray = ();			# global array
     @CaMonitorsTypes = ();			# global array
  }

  $statusALL = 0;
  $CaArrayLen = @CaMonitorsArray;
  if ($CaArrayLen > 0) {
     $scriptMAIN = &makeScriptName();
     $current_time = scalar localtime;
     print $LUN "\n".'!!! '.$scriptMAIN.' ['.$current_time.']: Re-establishing all '
                    .$CaArrayLen.' CA monitors. Please wait...'."\n";
     for ($i=0; $i<$CaArrayLen; $i++) {
        $pv          = $CaMonitorsArray[$i];
        $monitorType = $CaMonitorsTypes[$i];
        if ($list =~ /^list$/i) {
           print $LUN 'Reconnecting pv='.$pv.'  type='.$monitorType."\n";
        }
        if ($monitorType =~ /^double$/i) {
           $status = Pezca::ClearMonitorDouble($pv);
           &report_only($script, $status, $pv, $LUN);
           select(undef,undef,undef,0.1);		# sleep(..);
           $status = Pezca::SetMonitorDouble($pv);
           &report_exit($script, $status, $pv, $LUN);
        } else {
           $status = Pezca::ClearMonitorString($pv);
           &report_only($script, $status, $pv, $LUN);
           select(undef,undef,undef,0.1);		# sleep(..);
           $status = Pezca::SetMonitorString($pv);
           &report_exit($script, $status, $pv, $LUN);
        }
        if ($status) {
           $statusALL++;
           $statusDescr = &ezcaStatusCode($status);
           print $LUN '!!! '.$script.': error='.$status.' ['.$statusDescr.'] '
                            .'from EZCA while resetting CA-monitor ['.$pv.']'."\n";
        }
     }
  }
  return $statusALL;
}

#=================================================================

#sub ezcaStatusCode ($status) {
sub ezcaStatusCode($) {

### This routine returns Description for EZCA return Status code.
  my ($status, @statusCodes, $ndescr);

  $status  = shift(@_);
  @statusCodes = ('EZCA_OK',				#0
                  'EZCA_INVALIDARG',			#1
                  'EZCA_FAILEDMALLOC',			#2
                  'EZCA_CAFAILURE',			#3
                  'EZCA_UDFREQ',			#4
                  'EZCA_NOTCONNECTED',			#5
                  'EZCA_NOTIMELYRESPONSE',		#6
                  'EZCA_INGROUP',			#7
                  'EZCA_NOTINGROUP', 			#8
                  'EZCA_ABORTED');			#9 (same as EZCA_NOTIMELYRESPONSE when RetryCount=0)

  $ndescr = @statusCodes;

  if    (!defined $status)                   {return '***_UNDEFINED_EZCA_STATUS';}
  elsif ($status !~ /^[0-9]$/)               {return '***_ILLEGAL_EZCA_STATUS';}
  elsif ($status < 0 || $status > $ndescr-1) {return '***_UNKNOWN_EZCA_STATUS_CODE='.$status;}
  else                                       {return $statusCodes[$status];}
}

#=================================================================

#sub report_exit ($program, $status, $pv, [$LUN,$cmd]) {
sub report_exit ($$$;$$) {

### This routine reports EZCA communication error and exits

  our ($IGNORE_ERRORS, $USE_ZENITY_FOR_EXIT, $EXIT_WITH_SIGNAL_HANDLER);
  my ($program, $status, $pv, $current_time, $statusDescr, $msg, $LUN, $cmd);
  my ($messref);

  $program = shift(@_);
  $status  = shift(@_);
  $pv      = shift(@_);
  $LUN     = shift(@_);
  $cmd     = shift(@_);
  if (!defined $LUN)                      {$LUN = \*STDOUT;}
  if (!defined $IGNORE_ERRORS)            {$IGNORE_ERRORS = 0;}
  if (!defined $USE_ZENITY_FOR_EXIT)      {$USE_ZENITY_FOR_EXIT = 0;}
  if (!defined $EXIT_WITH_SIGNAL_HANDLER) {$EXIT_WITH_SIGNAL_HANDLER = 0;}

  if ($status) {
     $current_time = scalar localtime;
     $statusDescr  = &ezcaStatusCode($status);
     $msg  = '*** '.$program.': '.$current_time."\n";
     $msg .= '*** '.$program.': EZCA error='.$status.' ['.$statusDescr.'] PV=['.$pv.']';
     if (defined $cmd) {$msg .= ' CMD=['.$cmd.']';}
     ### Function "shortmess" is defined in Carp:
     ### use Carp qw(shortmess);
     $messref = &function_exists('shortmess');
     if (defined $messref) {$msg .= "\n".$messref->('shortmess calling stack trace');}
     if ($IGNORE_ERRORS) {
        print $LUN "\n".$msg."\n\n";
        return;
     } else {
        if ($EXIT_WITH_SIGNAL_HANDLER == 0) {
           &press_key_exit($msg,1);
        }
        elsif ($EXIT_WITH_SIGNAL_HANDLER == 1) {
           print $LUN "\n".$msg."\n\n";
           $EXIT_WITH_SIGNAL_HANDLER = 2;	#we are entering signal handler (set the same there to avoid duplicate calls!)
           &signal_handler('quit');
        }
        else {					#we got here from signal handler
           print $LUN "\n".$msg."\n\n";
           return;
        }
     }
  }
}

#=================================================================

#sub report_only ($program, $status, $pv, [$LUN,$cmd]) {
sub report_only ($$$;$$) {

### This routine reports EZCA communication error

  my ($program, $status, $pv, $current_time, $statusDescr, $msg, $LUN, $cmd);
  my ($messref);

  $program = shift(@_);
  $status  = shift(@_);
  $pv      = shift(@_);
  $LUN     = shift(@_);
  $cmd     = shift(@_);
  if (!defined $LUN) {$LUN = \*STDOUT;}
  if ($status) {
     $current_time = scalar localtime;
     $statusDescr  = &ezcaStatusCode($status);
     $msg  = '*** '.$program.': '.$current_time."\n";
     $msg .= '*** '.$program.': EZCA error='.$status.' ['.$statusDescr.'] PV=['.$pv.']';
     if (defined $cmd) {$msg .= ' CMD=['.$cmd.']';}
     ### Function "shortmess" is defined in Carp:
     ### use Carp qw(shortmess);
     $messref = &function_exists('shortmess');
     if (defined $messref) {$msg .= "\n".$messref->('shortmess calling stack trace');}
     print $LUN "\n".$msg."\n\n";
  }
}

#=================================================================

#sub wait4pv ($pv,$target,$timeout,$poll,[$delay,$counter,$tolerance,$progress]) {
sub wait4pv ($$$$;$$$$) {

### This routine waits for a non-string PV to become $val.
### It returns '0' if OK, '1' if timeout and '-1' on PV connection problem.
  my ($script, $pv, $target, $timeout, $poll, $delay, $counter, $tolerance, $progress);
  my ($val, $status, $wait_, $print_, $achieved);

  $script    = 'wait4pv';
  $pv        = shift(@_);						#PV to monitor
  $target    = shift(@_);						#PV target value
  $timeout   = shift(@_);						#waiting timeout (s)
  $poll      = shift(@_);						#pollin interval (s)
  $delay     = shift(@_);  if (!defined $delay)     {$delay=0;}		#starting delay (s)
  $counter   = shift(@_);  if (!defined $counter)   {$counter=1;}	#how many times to check achieved
  $tolerance = shift(@_);  if (!defined $tolerance) {$tolerance=0;}	#value tolerance
  $progress  = shift(@_);  if (!defined $progress)  {$progress=0;}	#interval (s) between progress prints

  if ($counter   < 1) {$counter   = 1;}
  if ($tolerance < 0) {$tolerance = 0;}
  if ($progress  < 0) {$progress  = 0;}

#  print STDOUT 'pv='.$pv
#            .', target='.$target
#            .', timeout='.$timeout
#            .', poll='.$poll
#            .', delay='.$delay
#            .', counter='.$counter
#            .', tolerance='.$tolerance
#            .', progress='.$progress;

  $status = &addCaMonitor($pv);
# if ($status) {return -1;}

  $wait_    = 0;
  $print_   = 0;
  $achieved = 0;
  if ($delay > 0) {
     select(undef,undef,undef,$delay);		# sleep($delay);
     $wait_  += $delay;
     $print_ += $delay;
  }
  do {
    select(undef,undef,undef,$poll);		# sleep($wait_step);
    $wait_  += $poll;
    $print_ += $poll;
    if ($progress > 0 && $print_ > $progress) {
       printf STDOUT $script.': %1.0fs of %1.fs waiting '.$pv
                            .' to become '.$target."\n", $wait_, $timeout;
       $print_ = 0;
    }
    ($status,$val) = Pezca::GetDouble($pv);
    if ($status) {
       $achieved=0;				# presume worst scenario
    } else {
       if (abs($val-$target) <= $tolerance) {$achieved++;}
       else                                 {$achieved=0;};
    }
#   print STDOUT $script.': pv='.$pv
#                       .' target='.$target
#                       .' val='.$val
#                       .' tolerance='.$tolerance
#                       .' counter='.$counter
#                       .' achieved='.$achieved
#                       .' wait_='.$wait_
#                       .' wait_max='.$timeout
#                       ."\n";
  } while ($achieved < $counter && $wait_ <= $timeout);

  if ($wait_ > $timeout) {return 1;}
  else                   {return 0;}

}

#=================================================================

#sub ask_to_continue ([$msg,$prompt,'noexit']) {
sub ask_to_continue (;$$$) {

  our ($NONINTERACTIVE, $USE_ZENITY_FOR_EXIT);
  my ($msg, $prompt, $txt, $noexit);
  $msg    = shift(@_);
  $prompt = shift(@_);
  $noexit = shift(@_);
  if (!defined $noexit)              {$noexit = 'exit';}
  if (!defined $NONINTERACTIVE)      {$NONINTERACTIVE=0;}
  if (!defined $USE_ZENITY_FOR_EXIT) {$USE_ZENITY_FOR_EXIT=0;}

  if ($USE_ZENITY_FOR_EXIT && $NONINTERACTIVE == 0) {
     if (defined $prompt) {$txt = $prompt;}
     else                 {$txt = 'Continue?';}
     if	(defined $msg) {$txt = $msg."\n\n".$txt;}
     $_ = &zenity_question ($txt);				#returns 1 or 0
  } else {
     if (defined $msg)    {print STDOUT "\n".$msg."\n\n";}
     if (defined $prompt) {print STDOUT $prompt;}
     else                 {print STDOUT 'Continue [n]?_';}
     if ($NONINTERACTIVE == 0) {$_=uc <STDIN>;  chomp $_;}
     else                      {$_='Y'; print STDOUT 'Y';}
  }
  if ($_ ne 'Y' && $_ ne '1') {
     print STDOUT ' ....... Exiting..'."\n\n";
     if ($noexit ne 'noexit') {
        &clearAllCaMonitors();
        exit;
     } else {
        return 0;
     }
  } else {
     print STDOUT ' ....... Continuing..'."\n\n";
     return 1;
  }
}

#=================================================================

#sub ask_to_overwrite ($file) {
sub ask_to_overwrite ($) {

  our ($NONINTERACTIVE, $USE_ZENITY_FOR_EXIT);
  my ($file, $yesno);
  if (!defined $NONINTERACTIVE)      {$NONINTERACTIVE      = 0;}
  if (!defined $USE_ZENITY_FOR_EXIT) {$USE_ZENITY_FOR_EXIT = 0;}
  $file = shift(@_);
  if (-e $file) {			#file exists
    if (-f $file) {			#file is file
      if (-w $file) {			#file is writable
         if ($USE_ZENITY_FOR_EXIT) {
            $yesno = &zenity_question('File "'.$file.'" exists. Overwrite?');
            return $yesno;
         } else {
            print STDOUT 'File "'.$file.'" exists. Overwrite [n]?_';
            if ($NONINTERACTIVE == 0) {$_=uc <STDIN>;  chomp $_;}
            else                      {$_='Y'; print STDOUT 'Y';}
            print STDOUT "\n";
            if ($_ eq 'Y') {return 1;}
            else           {return 0;}
         }
      } else {
        return 0;			#file is writable, NO overwrite
      }
    } else {
      return 0;				#file is directory, NO overwrite
    }
  } else {
    return 1;				#no file, OK to create new file
  }
}

#=================================================================

#sub press_key_continue ([$msg,$prompt]) {
sub press_key_continue (;$$) {

  our $NONINTERACTIVE;
  my $msg    = shift(@_);
  my $prompt = shift(@_);
  if (!defined $NONINTERACTIVE) {$NONINTERACTIVE=0;}

  if (defined $msg && length($msg) > 0) {print STDOUT "\n".$msg."\n\n";}
  if ($NONINTERACTIVE == 0) {
     if (defined $prompt) {print STDOUT  $prompt;}
     else                 {print STDOUT  'Hit <ENTER> to continue...';}
     $_=<STDIN>;
     print STDOUT "\n";
  }
}

#=================================================================

#sub press_key_exit ([$msg,$exitcode,'info|error']) {
sub press_key_exit (;$$$) {
  our ($NONINTERACTIVE, $USE_ZENITY_FOR_EXIT);
  my ($msg, $exitcode, $icontype);
  $msg      = shift(@_);
  $exitcode = shift(@_);
  $icontype = shift(@_);					#error/info
  if (!defined $NONINTERACTIVE)      {$NONINTERACTIVE = 0;}
  if (!defined $USE_ZENITY_FOR_EXIT) {$USE_ZENITY_FOR_EXIT = 0;}
  if (!defined $exitcode)	     {$exitcode = 0;}
  if ($exitcode ne '1')		     {$exitcode = 0;}
  if (!defined $icontype)            {$icontype = 'error';}
  if ($icontype !~ /^info$/i)        {$icontype = 'error';}

  &clearAllCaMonitors();

  if ($USE_ZENITY_FOR_EXIT) {
     ### Graphical mode
     if (!defined $msg || length($msg) == 0) {$msg = 'Click OK to exit...';}
     if ($icontype =~ /^info$/i) {&zenity_info ($msg);}
     else                        {&zenity_error ($msg);}
  }  else  {
     ### Text mode
     if (defined $msg && length($msg) > 0) {print STDOUT "\n".$msg."\n";}
     if (!$NONINTERACTIVE) {
        print STDOUT  "\n".'Hit <ENTER> to exit...';
        $_=<STDIN>;
     }
     print STDOUT "\n";
  }
  exit $exitcode;
}

#=================================================================

#sub zenity_info ($message,['nowait']) {
sub zenity_info ($;$) {

### Open graphical window to report a notification
### zenity --info --text='a-a\nb-b' --title='Test' --no-wrap --ellipsize

  my ($msg, $nowait, $stderr);
  $msg    = shift(@_);
  $nowait = shift(@_);
  $stderr = 0;					#output to STDOUT
  &zenity_proc(\$msg,'info',$stderr,$nowait);
}

#=================================================================

#sub zenity_error ($message,[$stderr,$nowait]) {
sub zenity_error ($;$$) {

### Open graphical window to report an error
### zenity --error --text='a-a\nb-b' --title='Test' --no-wrap --ellipsize

  my ($msg, $stderr, $nowait);
  $msg    = shift(@_);
  $stderr = shift(@_);
  $nowait = shift(@_);
  &zenity_proc(\$msg,'error',$stderr,$nowait);
}

#=================================================================

#sub zenity_error_exit ($message,[$stderr,'nowait']) {
sub zenity_error_exit ($;$$) {

### Open graphical window to report an error and exits
### zenity --error --text='a-a\nb-b' --title='Test' --no-wrap --ellipsize

  my ($msg, $stderr, $nowait);
  $msg    = shift(@_);
  $stderr = shift(@_);
  $nowait = shift(@_);
  &zenity_proc(\$msg,'error',$stderr,$nowait);
  &clearAllCaMonitors();
  exit 1;
}

#=================================================================

#sub zenity_proc (\$message,$type,[$stderr,'wait|nowait']) {
sub zenity_proc (\$$;$$) {

### Open graphical window to report an error or post an information
### ($type can be 'error' and 'info')

  our ($ZENITY_VIA_BLUICE, $JBLUICE_SERVER, $bluiceLogPV);
  my ($script, $msg_r, $msg, $type, $stderr, $wait);
  my ($LUN, $prefix, $status, $title, $path, @cmd, $string);
  my (%FORM, $ua, $response);

  $script = &makeScriptName();

  $msg_r  = shift(@_);                                                  #pointer to message
  $type   = shift(@_);                                                  #'error' or 'info'
  $stderr = shift(@_);  if (!defined $stderr) {$stderr = 0;}		#0=stdout or 1=stderr
  $wait   = shift(@_);  if (!defined $wait)   {$wait = 'wait';}		#'nowait' or any meaning 'wait'

  if (!defined $ZENITY_VIA_BLUICE) {$ZENITY_VIA_BLUICE = 0;}
  if (defined $ENV{BLUICE_SERVER}) {$JBLUICE_SERVER = $ENV{BLUICE_SERVER};}

  if ($ZENITY_VIA_BLUICE ne '0') {					#we do not specify JBluIce method, it is auto-selected as http or PV
     if (defined $JBLUICE_SERVER && $JBLUICE_SERVER =~ /^http/) {
        $ZENITY_VIA_BLUICE = 2;
        use if ! defined &LWP::UserAgent, 'LWP::UserAgent';
        $ua = LWP::UserAgent->new(timeout=>'0.2');
        $response = $ua->get($JBLUICE_SERVER.'/check');
        if (! $response->is_success) {					#JbluIce server is not responding
           if (defined $bluiceLogPV) {
              $ZENITY_VIA_BLUICE = 1;                                   #switch to PV method
              $wait = 'nowait';						#cannot wait in this case
           } else {
              $ZENITY_VIA_BLUICE = 0;					#no method to work with JBluIce
           }
        }
     }
     elsif (defined $bluiceLogPV) {
        $ZENITY_VIA_BLUICE = 1;
        $wait = 'nowait';						#cannot wait in this case
     } else {
        $ZENITY_VIA_BLUICE = 0;						#no method to work with JBluIce
     }
     if ($ZENITY_VIA_BLUICE eq '0') {print STDOUT '!!! '.$script.': failed reaching BluIce server for MSG:'."\n".$$msg_r."\n\n";}
  }

  $status = 0;
  $msg = $$msg_r;
  if ($ZENITY_VIA_BLUICE == 2) {
     $FORM{'text'} = $msg;
     if ($wait !~ /^nowait$/i) {
        $FORM{'type'}    = 'question';
        $FORM{'options'} = 'OK';
        $ua->timeout(3600);						#wait up to 1 hour for user response
     } else {
#       if    ($type =~ /^error$/) {$FORM{'type'}='error';}
        if    ($type =~ /^error$/) {$FORM{'type'}='note';}		#the server does not understand error
        elsif ($type =~ /^info$/)  {$FORM{'type'}='note';}
        else                       {$FORM{'type'}='note';}
     }
     $response = $ua->post($JBLUICE_SERVER.'/popup',\%FORM);
     if (! $response->is_success && $wait !~ /^nowait$/i) {$status=1;}  #we do not care about response when 'nowait'
  }
  if ($ZENITY_VIA_BLUICE == 1 || $status) {
     if    ($type =~ /^error$/) {$status = &tellBluIce($msg,'popup_error');}
     elsif ($type =~ /^info$/)  {$status = &tellBluIce($msg,'popup_note');}
  }
  if ($ZENITY_VIA_BLUICE == 0 || $status) {			        #BluIce failed or no BluIce flag
     $msg =~ s/\&/and/g;
     $msg =~ s/\"/\\\"/g;
     $msg =~ s/\'/`/g;
##   $msg =~ s/</\\\"/g;
##   $msg =~ s/>/\\\"/g;
#    $msg =~ s/</\\\</g;
#    $msg =~ s/>/\\\>/g;
     $msg =~ s/</\</g;
     $msg =~ s/>/\>/g;
     if ($^O !~ /Win/i) {
        $msg =~ s/\(/\\\(/g;
        $msg =~ s/\)/\\\)/g;
     }
     $title = $script.'-'.$type;
     $title =~ s/ /_/g;
     @cmd = ('--'.$type,
             '--title='."'".$title."'",
             '--text='."'".$msg."'");
     $path = '/usr/bin/zenity';
     if ($^O !~ /Win/i && -f $path) {					#Linux with zenity installed
### zenity --error --text='bla-bla-aaaaaaaaaaaaaaa ee\rbla' --title='Test' --no-wrap --ellipsize --window-icon='/usr/share/pixmaps/applet-critical.png'
        unshift @cmd, $path;						#prepend with 'path'
        push @cmd, '--no-wrap';						#this helps to reduce excessive white space below text
        push @cmd, '--ellipsize';					#this fixes the high window size with long texts
#       push @cmd, '2>/dev/null';					#redirect stderr
#       push @cmd, '&>/dev/null';					#redirect both stdout and stderr
        push @cmd, ' >/dev/null', '2>&1';				#redirect both stdout and stderr (preferred)
        if ($wait =~ /^nowait$/i) {
           push @cmd, '&';
#          unshift (@cmd,'screen','-d','-m');
           unshift (@cmd,'nohup');
        }
        $string = join(' ',@cmd);					#merge from array
        &zenity_bring_on_top($title);
        $status = system($string);					#only string form allows no-wait
     } else {								#either no zenity or no Linux
        $cmd[2]  =~ s/\n/\\n/g;						#transform new lines
        if    (defined $ENV{PEZCA})     {$path=$ENV{PEZCA}.'/';}
        elsif (defined $ENV{FASTSCANS}) {$path=$ENV{FASTSCANS}.'/'; $path=~s/fastscans$/pezca/i;}
        else                            {$path='';}
        $path =~ s/\\/\//g;
        $path = $path.'zenity.pl';
        if (-f $path) {
           unshift @cmd, 'perl', '-W', $path;				#prepend with 'perl -W path'
           if ($^O !~ /Win/i) {						#Linux
              if ($wait =~ /^nowait$/i) {
                 push @cmd, '&';
#                unshift (@cmd,'screen','-d','-m');
                 unshift (@cmd,'nohup');
              }
              $string = join(' ',@cmd);					#merge from array
              $status = system($string);				#olny string form allows no-wait
           } else {							#Windows
### "The simplest way to start detouched process under WIN32 is the barely
### documented  system-with-a-first-argument-of-1 asynchronous system. It's
### not portable  outside of Win32, but it is roughy analogous to appending
### '&' to the end of a unix command."
### --- See: http://www.perlmonks.org/?node_id=556209
              if ($wait =~ /^nowait$/i) {$status = system(1,@cmd); $status=0;}	#no-wait form (this returns no-sense status
              else                      {$status = system(@cmd);}		#wait form
           }
#          $status = 0;
        } else {
           $status = -1;
        }
     }
     if ($status) {
        $string = join(' ',@cmd);					#merge from array
        print STDOUT "\n".' !!! Error='.$status.' running'."\n".$string."\n\n";
     }
  }

  $LUN = \*STDOUT;
  if ($stderr) {$LUN = \*STDERR;}
  $prefix = '';
  if ($type =~ /^error$/) {$prefix = '*** ';}
  print $LUN "\n".$prefix.$script.':'."\n".$$msg_r."\n\n";
}

#=================================================================

#sub zenity_question ($message,[$Yeslbl,$Nolbl]) {			#return 1/0=yes/no
sub zenity_question ($;$$) {

### Open graphical window to query a choice of 2 options
### zenity --question --text='a-a\nb-b' --ok-label='OK' --cancel-label='Cancel' --title='Test' --no-wrap --ellipsize

  our ($ZENITY_VIA_BLUICE, $JBLUICE_SERVER);
  my ($script, $msg, $Yeslbl, $Nolbl, $title, $path, $yesno);
  my (%FORM, $ua, $response, $status);

  $script = &makeScriptName();

  $msg    = shift(@_);
  $Yeslbl = shift(@_);	if (!defined $Yeslbl) {$Yeslbl = 'OK';}
  $Nolbl  = shift(@_);	if (!defined $Nolbl)  {$Nolbl  = 'Cancel';}

  if (!defined $ZENITY_VIA_BLUICE) {$ZENITY_VIA_BLUICE = 0;}
  if (defined $ENV{BLUICE_SERVER}) {$JBLUICE_SERVER = $ENV{BLUICE_SERVER};}

  if ($ZENITY_VIA_BLUICE ne '0') {					#we do not specify JBluIce method, it is auto-selected as http or PV
     if (defined $JBLUICE_SERVER && $JBLUICE_SERVER =~ /^http/) {
        $ZENITY_VIA_BLUICE = 2;
        use if ! defined &LWP::UserAgent, 'LWP::UserAgent';
        $ua = LWP::UserAgent->new(timeout=>'0.2');
        $response = $ua->get($JBLUICE_SERVER.'/check');
        if (! $response->is_success) {					#JbluIce server is not responding
           $ZENITY_VIA_BLUICE = 0;                                      #no method to work with JBluIce
        }
     } else {
        $ZENITY_VIA_BLUICE = 0;						#no method to work with JBluIce
     }
     if ($ZENITY_VIA_BLUICE eq '0') {print STDOUT '!!! '.$script.': failed reaching BluIce server for MSG:'."\n".$msg."\n\n";}
  }

  $status = 0;
  if ($ZENITY_VIA_BLUICE == 2) {
     $FORM{'text'}    = $msg;
     $FORM{'type'}    = 'question';
     $FORM{'options'} = $Nolbl.','.$Yeslbl;
     $ua->timeout(3600);                                                #wait up to 1 hour for user response
     $response = $ua->post($JBLUICE_SERVER.'/popup',\%FORM);
     if (! $response->is_success) {$status=-1;}                         #something failed
     else {
        $status=$response->decoded_content;                             #0=cancel, 1=OK, -1=error.
        if    ($status eq '1') {$yesno = 1;}
        elsif ($status eq '0') {$yesno = 0;}
        else                   {$status=-1;}
     }
  }
  if ($ZENITY_VIA_BLUICE == 0 || $status == -1) {			#BluIce failed or no BluIce flag
     $title = $script.'-question';
     $title =~ s/ /_/g;
     $path   = '/usr/bin/zenity';
     if ($^O !~ /Win/i && -f $path) {			                #Linux with zenity installed
        ### xmessage -buttons Yes:1,No:0 'bla-bla-bla'
        ### zenity --question --no-wrap --ellipsize --text='bla-bla-bla\rbla' --title='Test' --ok-label='Continue' --cancel-label='Dismiss' --window-icon='/usr/share/pixmaps/applet-critical.png'
        &zenity_bring_on_top($title);
        $yesno = system($path.' --question'
                             .' --no-wrap'
                             .' --ellipsize'
                             .' --ok-label='    ."'" .$Yeslbl ."'"
                             .' --cancel-label='."'" .$Nolbl  ."'"
                             .' --title='       ."'" .$title  ."'"
                             .' --text='        ."'" .$msg    ."'"
                             .' 2>/dev/null');
        if ($yesno ne '0') {$yesno = 0;}		                #Cancel returns '1'
        else               {$yesno = 1;}		                #OK returns '0'
     } else {
        if    (defined $ENV{PEZCA})     {$path=$ENV{PEZCA}."/";}
        elsif (defined $ENV{FASTSCANS}) {$path=$ENV{FASTSCANS}."/"; $path=~s/fastscans$/pezca/i;}
        else                            {$path="";}
        $path =~ s/\\/\//g;
        $msg  =~ s/\n/\\n/g;
        $yesno = `${path}zenity.pl --question --title=\'${title}\' --text=\'${msg}\'`;
        if ($yesno =~ /^OK/i || $yesno =~ /^Yes/i) {$yesno = 1;}
        else                                       {$yesno = 0;}
     }
  }
  ### Replace \r by carriage return:
  $msg =~ s/\\\\n/\n/g;
  $msg =~ s/\\n/\n/g;
  $msg =~ s/\\r/\n/g;
  print STDOUT "\n".$script.":\n".$msg.'  => ';
  if ($yesno == 1) {print STDOUT $Yeslbl."\n";}
  else             {print STDOUT $Nolbl."\n";}
  return $yesno;
}

#=================================================================

#sub zenity_list($message,\@choices,[$default]) {                       #returns index of choice or -1 if cancel or none selected
sub zenity_list ($\@;$) {

### Open graphical window to request a single choice from a list of options
### zenity --list --radiolist --text='bla-bla-bla' --title='Test' --column='Select' --column='Option' TRUE 'aaa' FALSE 'bbb' FALSE 'ccc'

  our ($ZENITY_VIA_BLUICE, $JBLUICE_SERVER);                            #not used yet (need to work on API with Mark)
  my ($script, $subname, $msg, $choices_ref, $default);
  my ($str, $path, $title, $nchoices);
  my ($i, $j, $result);

  $script = &makeScriptName();
  $subname = 'zenity_list';

  $msg         = shift(@_);
  $choices_ref = shift(@_);     $nchoices = @$choices_ref;
  $default     = shift(@_);     if (!defined $default) {$default = -1;}

  if (!defined $ZENITY_VIA_BLUICE) {$ZENITY_VIA_BLUICE = 0;}
  if (defined $ENV{BLUICE_SERVER}) {$JBLUICE_SERVER = $ENV{BLUICE_SERVER};}

  for ($i=0; $i<$nchoices; $i++) {
     for ($j=0; $j<$nchoices; $j++) {
        if ($i != $j && $$choices_ref[$i] eq $$choices_ref[$j]) {
           print STDOUT '*** '.$subname.': duplicate choice "'.$$choices_ref[$i].'"'."\n";
           return -1;
        }
     }     
  }  

  $title = $script.'-question';
  $title =~ s/ /_/g;
  $path   = '/usr/bin/zenity';
  if ($^O !~ /Win/i && -f $path) {			                #Linux with zenity installed
     ### zenity --list --radiolist --text='bla-bla-bla' --title='Test' --column='Select' --column='Option' TRUE 'aaa' FALSE 'bbb' FALSE 'ccc'
     &zenity_bring_on_top($title);
     $str = $path
          . ' --list'
          . ' --radiolist'
          . ' --text='        ."'".$msg."'"
          . ' --title='       ."'".$title."'"
          . ' --column='      ."'Select'"
          . ' --column='      ."'Option'";
     for ($i=0; $i<$nchoices; $i++) {
        if ($i == $default) {$str .= ' TRUE';}
        else                {$str .= ' FALSE';}
        $str .= " '".$$choices_ref[$i]."'";
     }
     $str .= ' 2>/dev/null';
#    print STDOUT $str,"\n";
     $result = `$str`;
     chomp($result);
#    print STDOUT $result,"\n";
     for ($i=0; $i<$nchoices; $i++) {
        $str = $$choices_ref[$i];
        chomp($str);
        if ($result eq $str) {return $i;}
     }
     return -1;
  } else {
     return -1;
  }
}


#=================================================================

#sub zenity_bring_on_top ($title) {
sub zenity_bring_on_top ($) {

### Open graphical window to report an error

  my ($title, $wmctrl, $bash, $sleep);

  $title = shift;
  $wmctrl = '/usr/bin/wmctrl';
  $bash   = '/usr/bin/bash';
  $sleep  = '/bin/sleep';
  if (-e $wmctrl && -e $bash && -e $sleep) {
     system($bash.' -c "'.$sleep.' 2 && '.$wmctrl.' -r '.$title.' -b toggle,above" &');
  }
  return;
}
#=================================================================

#sub disable_interrupts ([$message]) {
sub disable_interrupts  (;$) {
### Disable all interrupts so that the interrupt
### routine is not called several times
  my ($script, $msg);
  $msg = shift(@_);
  if (!defined $msg) {
     $script = &makeScriptName();
     $msg = $script.': further interrupts are disabled.';
  }
  print STDOUT "\n".$msg."\n\n";
  $SIG{HUP}  = 'IGNORE';	#(1) hangup (see signum.h)
  $SIG{INT}  = 'IGNORE';	#(2) interrupt (see signum.h)
  $SIG{QUIT} = 'IGNORE';	#(3) quit (see signum.h)
# $SIG{KILL} = 'IGNORE';	#(9) this is what is sent by kill -9 -- unblockable
  $SIG{TERM} = 'IGNORE';	#(15) terminate -- sent by kill without paramaters
# $SIG{STOP} = 'IGNORE';	#(19) stop -- unblockable
}

#=================================================================

sub asinSS ($) {
### This sub uses standard asin from PDL or Math::Trig, if they are available
  local $a = $_[0];
  if    ($a > 1.)  {$a = 1.;}
  elsif ($a < -1.) {$a = -1.;}
  if (defined &asin) {asin($a);}
  else               {atan2($a,sqrt(1-$a*$a));}
}

#=================================================================

#sub maxval (@array) {
sub maxval (@) {
### This is a home-made replacement for List::Util qw(min max);
  my ($script, $npts, $max_, $i);

  $script = 'maxval';

  $npts = @_;
  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0;
  }
  $max_ = $_[0];
  foreach $i (1 .. $#_) {$max_ = $_[$i] if $_[$i] > $max_;}
  return $max_;
}

#=================================================================

#sub minval (@array) {
sub minval (@) {
### This is a home-made replacement for List::Util qw(min max);
  my ($script, $npts, $min_, $i);

  $script = 'minval';

  $npts = @_;
  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0;
  }
  $min_ = $_[0];
  foreach $i (1 .. $#_) {$min_ = $_[$i] if $_[$i] < $min_;}
  return $min_;
}

#=================================================================

#sub maxvalRef (\@array) {
sub maxvalRef (\@) {
### This is a home-made replacement for List::Util qw(min max);
  my ($script, $npts, $arrayRef, $element, $max_);
  $script   = 'maxvalRef';
  $arrayRef = shift(@_);
  $npts     = @$arrayRef;
  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0;
  }
  $max_ = @$arrayRef[0];
  foreach $element (@$arrayRef) {$max_ = $element if $element > $max_;}
  return $max_;
}

#=================================================================

#sub minvalRef (\@array) {
sub minvalRef (\@) {
### This is a home-made replacement for List::Util qw(min max);
  my ($script, $npts, $arrayRef, $element, $min_);
  $script   = 'minvalRef';
  $arrayRef = shift(@_);
  $npts     = @$arrayRef;
  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0;
  }
  $min_ = @$arrayRef[0];
  foreach $element (@$arrayRef) {$min_ = $element if $element < $min_;}
  return $min_;
}

#=================================================================

#sub findIndexMinRef (\@array) {
sub findIndexMinRef (\@) {
### This script finds index of the array element with min value
  my ($script, $npts, $arrayRef, $a_, $pt, $array_min, $index_min);
  $script   = 'findIndexMinRef';
  $arrayRef = shift(@_);
  $npts     = @$arrayRef;
  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0;
  }
  $array_min = @$arrayRef[0];
  $index_min = 0;
  for ($pt=1; $pt < $npts; $pt++) {
     $a_ = @$arrayRef[$pt];
     if ($a_ < $array_min) {$array_min=$a_; $index_min=$pt;}
  }
  return $index_min;
}

#=================================================================

#sub findIndexMaxRef (\@array) {
sub findIndexMaxRef (\@) {
### This script finds index of the array element with max value
  my ($script, $npts, $arrayRef, $a_, $pt, $array_max, $index_max);
  $script   = 'findIndexMaxRef';
  $arrayRef = shift(@_);
  $npts     = @$arrayRef;
  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0;
  }
  $array_max = @$arrayRef[0];
  $index_max = 0;
  for ($pt=1; $pt < $npts; $pt++) {
     $a_ = @$arrayRef[$pt];
     if ($a_ > $array_max) {$array_max=$a_; $index_max=$pt;}
  }
  return $index_max;
}

#=================================================================

#sub findMaxRef (\@array) {
sub findMaxRef (\@) {
### This subroutine returns ($array_max)
  my ($script, $arrayRef, $array_max);
  $script   = 'findMaxRef';
  $arrayRef = shift(@_);
### Find maximum:
  $array_max = &maxvalRef($arrayRef);
  return $array_max;
}

#=================================================================

#sub findMinMaxRef (\@array,[$npts]) {
sub findMinMaxRef (\@;$) {
### This subroutine returns ($array_min,$array_max)
  my ($script, $npts, $nmax, $arrayRef, $array_max, $array_min, $pt, $a_);
  $script   = 'findMinMaxRef';
  $arrayRef = shift(@_);
  $npts     = shift(@_);
  $nmax     = @$arrayRef;
  if ($nmax <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0,0;
  }
  if (!defined $npts || $npts > $nmax) {$npts = $nmax;}

### Find maximum/minimum:
  $array_max = @$arrayRef[0];		#array maximum value
  $array_min = @$arrayRef[0];		#array minimum value
  for ($pt=1; $pt < $npts; $pt++) {
     $a_ = @$arrayRef[$pt];
     if ($a_ > $array_max) {$array_max = $a_;}
     if ($a_ < $array_min) {$array_min = $a_;}
  }
  return $array_min,$array_max;
}

#=================================================================

# sub findRMS(\@array) {		#returns ($rmsVal,$rmsDev)

sub findRMS (\@) {
### This subroutine returns ($rmsVal,$rmsDev)
  my ($script, $npts, $arrayRef, $rmsVal, $rmsDev, $i, $a_);
  $script   = 'findRMS';
  $arrayRef = shift(@_);
  $npts     = @$arrayRef;
  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0,0;
  }

### Find mean RMS:
  $rmsVal = 0;
  for ($i=0; $i<$npts; $i++) {
     $a_ = @$arrayRef[$i];
     $rmsVal += $a_*$a_;
  }
  $rmsVal = sqrt($rmsVal/$npts);

### Find RMS deviation:
  $rmsDev = 0;
  for ($i=0; $i<$npts; $i++) {
     $a_ = @$arrayRef[$i] - $rmsVal;
     $rmsDev += $a_*$a_;
  }
  $rmsDev = sqrt($rmsDev/$npts);

  return $rmsVal,$rmsDev;
}

#=================================================================

#sub findPeak ($npts, @array) {
sub findPeak ($@) {
### This subroutine returns:
### ($array_max,$pt_max,$pt_FWHM,$pt_left,$pt_right,[$x_left,$x_right,$x_peak])
### where $x_left,$x_right,$x_peak are respective interpolated value (i.e.
### they are a higher precision than $pt_left,$pt_right,$pt_FWHM.
  my ($script, $npts, @array, $narg, $array_max, $pt_max, $pt_FWHM);
  my ($pt_left, $pt_right, $x_left, $x_right, $x_peak);
  my ($half_intensity, $pt, $dy);

  $script = 'findPeak';
  $narg   = @_;
  $npts   = shift(@_);

  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0,0,-1;
  }
  elsif ($npts > $narg-1) {
     printf STDERR "\n".'!!! '.$script.': received %d array elements instead of %d declared!'."\n\n",($narg-1),$npts;
     $npts = $narg-1;
  }

### Find maximum
  $pt_max    = -1;				# array index at maximum value
  $pt_FWHM   = -1;				# peak width in pts (if exists)
  $array_max = -1.E37;				# array maximum value
  for ($pt=0; $pt < $npts; $pt++) {
     $array[$pt] = shift(@_);
     if ($array[$pt] > $array_max) {
              $array_max = $array[$pt];
              $pt_max    = $pt;
     }
  }
  $half_intensity = 0.5*$array_max;
### Climb down to the left of the peak:
  $pt_left = $pt_max;
  while (($array[$pt_left] > $half_intensity) && ($pt_left > 0)) {
     $pt_left--;
  }
### Climb down to the right of the peak:
  $pt_right = $pt_max;
  while (($array[$pt_right] > $half_intensity) && ($pt_right < $npts-1)) {
     $pt_right++;
  }
### If peak found:
  if (($pt_right < $npts-1) && ($pt_left > 0)) {
     $pt_FWHM = $pt_right - $pt_left + 1;
     $x_left = $pt_left;
     $dy = $array[$pt_left+1] - $array[$pt_left];
     if ($dy != 0.) {$x_left += ($half_intensity - $array[$pt_left]) / $dy;}
     $x_right = $pt_right;
     $dy = $array[$pt_right-1] - $array[$pt_right];
     if ($dy != 0.) {$x_right -= ($half_intensity - $array[$pt_right]) / $dy;}
     $x_peak = 0.5 * ($x_left + $x_right);
  } else {
     $pt_FWHM = -1;
     $x_left  = $pt_left;
     $x_right = $pt_right;
     $x_peak  = $pt_max;
  }
  return $array_max,$pt_max,$pt_FWHM,$pt_left,$pt_right,$x_left,$x_right,$x_peak;
}

#=================================================================

#sub findNoisyPeak ($npts, @array) {
sub findNoisyPeak ($@) {
### This subroutine returns:
### ($array_max,$pt_max,$pt_FWHM,$pt_left,$pt_right,[$x_left,$x_right,$x_peak])
### where $x_left,$x_right,$x_peak are respective interpolated value (i.e.
### they are a higher precision than $pt_left,$pt_right,$pt_FWHM.
### +---------------------------------------+
### |ATTENTION: It does not smooth anything!|
### +---------------------------------------+
### All the difference between "normal" and "noisy" peak
### finding is that in "normal" case we climb down the peak
### to find FWHM, while in "noisy" we climb up from the tails.
###
  my ($script, $npts, @array, $narg, $array_max, $pt_max, $pt_FWHM);
  my ($pt_left, $pt_right, $x_left, $x_right, $x_peak);
  my ($half_intensity, $pt, $dy);

  $script = 'findNoisyPeak';
  $narg   = @_;
  $npts   = shift(@_);

  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0,0,-1,0,0,0,0,0;
  }
  elsif ($npts > $narg-1) {
     printf STDERR "\n".'!!! '.$script.': received %d array elements instead of %d declared!'."\n\n",($narg-1),$npts;
     $npts = $narg-1;
  }

  $array_max = -1.E37;			#array maximum value
  $pt_max    = -1;			#array index at maximum value
  $pt_FWHM   = -1;			#peak width in pts (if exists)
### Find maximum:
  for ($pt=0; $pt < $npts; $pt++) {
     $array[$pt] = shift(@_);
     if ($array[$pt] > $array_max) {
        $array_max = $array[$pt];
        $pt_max    = $pt;
     }
  }
  $half_intensity = 0.5*$array_max;
### Climb up on the peak from curve start:
  $pt_left = 0;
  while (($array[$pt_left] < $half_intensity) && ($pt_left < $pt_max)) {
     $pt_left++;
  }
### Climb up on the peak from curve end:
  $pt_right = $npts-1;
  while (($array[$pt_right] < $half_intensity) && ($pt_right > $pt_max)) {
     $pt_right--;
  }
### If peak found:
  if (($pt_right < $npts-1) && ($pt_left > 0)) {
     $pt_FWHM = $pt_right - $pt_left + 1;
     $x_left = $pt_left;
     $dy = $array[$pt_left] - $array[$pt_left-1];
     if ($dy != 0.) {$x_left += ($half_intensity - $array[$pt_left]) / $dy;}
     $x_right = $pt_right;
     $dy = $array[$pt_right] - $array[$pt_right+1];
     if ($dy != 0.) {$x_right -= ($half_intensity - $array[$pt_right]) / $dy;}
     $x_peak = 0.5 * ($x_left + $x_right);
  } else {
     $pt_FWHM = -1;
     $x_left  = $pt_left;
     $x_right = $pt_right;
     $x_peak  = $pt_max;
  }
  return $array_max,$pt_max,$pt_FWHM,$pt_left,$pt_right,$x_left,$x_right,$x_peak;
}

#=================================================================

#sub findPeakRef ($npts, \@array, [$bkgmode,'smooth|noisy']) {
sub findPeakRef ($\@;$$) {
### This subroutine returns:
### ($array_max,$array_min,$pt_max,$pt_min,$pt_FWHM,$pt_left,$pt_right,[$x_left,$x_right,$x_peak])
### where $x_left,$x_right,$x_peak are respective interpolated value (i.e.
### they are a higher precision than $pt_left,$pt_right,$pt_FWHM).
### +---------------------------------------+
### |ATTENTION: It does not smooth anything!|
### +---------------------------------------+
### All the difference between "normal" and "noisy" peak
### finding is that in "normal" case we climb down the peak
### to find FWHM, while in "noisy" we climb up from the tails.
###
  my ($script, $npts, $arrayRef, $bkgmode, $searchmode);
  my ($array_max, $array_min, $pt_max, $pt_min);
  my ($pt_FWHM, $array_span, $a_, $half_intensity);
  my ($pt_left, $pt_right, $x_left, $x_right, $x_peak, $pt, $dy);

  $script     = 'findPeakRef';
  $npts       = shift(@_);
  $arrayRef   = shift(@_);
  $bkgmode    = shift(@_);               		#zero(default) or auto - differs from findValley
  $searchmode = shift(@_);               		#smooth(default) or noisy
  if (!defined $bkgmode)    {$bkgmode    = 'zero';}
  if (!defined $searchmode) {$searchmode = 'smooth';}

  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0,0,-1,-1,0,0,0,0,0;
  }

  $array_max = -1.E37;				        #array maximum value
  $array_min = +1.E37;				        #array minimum value
  $pt_max    = -1;					#array index at maximum value
  $pt_min    = -1;					#array index at minimum value
  $pt_FWHM   = -1;					#peak width in pts (if exists)

### Find maximum/minimum:
  for ($pt=0; $pt < $npts; $pt++) {
     $a_ = @$arrayRef[$pt];
     if ($a_ > $array_max) {$array_max = $a_; $pt_max = $pt;}
     if ($a_ < $array_min) {$array_min = $a_; $pt_min = $pt;}
  }
  if ($array_max < 0) {
     print STDOUT '*** '.$script.': all intensity is negative (max='.$array_max.')'."\n";
     print STDOUT '*** '.$script.': possibly incorrect background data.'."\n";
  }
  elsif ($array_min < 0) {
     print STDOUT '!!! '.$script.': some intensity is negative (min='.$array_min.')'."\n";
     print STDOUT '!!! '.$script.': possibly incorrect background data.'."\n";
  }

  if    ($bkgmode =~ /^zero$/i) {
     if ($array_max >= 0) {$half_intensity = 0.5*$array_max;}
     else {
        $pt_left  = 0;
        $pt_right = $npts-1;
        $x_left = $pt_left;
        $x_right = $pt_right;
        $x_peak = $pt_max;
        return $array_max,$array_min,$pt_max,$pt_min,$pt_FWHM,$pt_left,$pt_right,$x_left,$x_right,$x_peak;
     }
  }
  elsif ($bkgmode =~ /^auto$/i) {
     $array_span = $array_max - $array_min;
     $half_intensity = 0.5*$array_span + $array_min;
  } else {
     &press_key_exit('*** '.$script.': incorrect bkgmode='.$bkgmode,1);
  }

  if ($searchmode =~ /^smooth$/i) {
     ### Climb up on the peak from curve start:
     $pt_left = 0;
     while ((@$arrayRef[$pt_left] < $half_intensity) && ($pt_left < $pt_max)) {$pt_left++;}
     ### Climb up on the peak from curve end:
     $pt_right = $npts-1;
     while ((@$arrayRef[$pt_right] < $half_intensity) && ($pt_right > $pt_max)) {$pt_right--;}
  }          
  elsif ($searchmode =~ /^noisy$/i) {    
     ### Climb up on the peak from curve start:
     $pt_left = 0;
     while ((@$arrayRef[$pt_left]  < $half_intensity) && ($pt_left < $pt_max)) {$pt_left++;}
     ### Climb up on the peak from curve end:
     $pt_right = $npts-1;
     while ((@$arrayRef[$pt_right] < $half_intensity) && ($pt_right > $pt_max)) {$pt_right--;}
  }
  else {
     &press_key_exit('*** '.$script.': incorrect searchmode='.$searchmode,1);
  }
### If peak found:
  if (($pt_right < $npts-1) && ($pt_left > 0)) {
     $pt_FWHM = $pt_right - $pt_left + 1;
     $x_left = $pt_left;
     $dy = @$arrayRef[$pt_left] - @$arrayRef[$pt_left-1];
     if ($dy != 0.) {$x_left += ($half_intensity - @$arrayRef[$pt_left]) / $dy;}
     $x_right = $pt_right;
     $dy = @$arrayRef[$pt_right] - @$arrayRef[$pt_right+1];
     if ($dy != 0.) {$x_right -= ($half_intensity - @$arrayRef[$pt_right]) / $dy;}
     $x_peak = 0.5 * ($x_left + $x_right);
#    print STDOUT $script.":\n";
#    print STDOUT '   Half='.$half_intensity."\n";
#    print STDOUT '    Left='.@$arrayRef[$pt_left].'   '.@$arrayRef[$pt_left-1]."\n";
#    print STDOUT '   Right='.@$arrayRef[$pt_right].'  '.@$arrayRef[$pt_right+1]."\n";
#    print STDOUT '    pt_left='.$pt_left.'   x_left='.$x_left."\n";
#    print STDOUT '   pt_right='.$pt_right.'  x_right='.$x_right."\n";
  } else {
     $x_left  = $pt_left;
     $x_right = $pt_right;
     $x_peak  = $pt_max;
  }
  return $array_max,$array_min,$pt_max,$pt_min,$pt_FWHM,$pt_left,$pt_right,$x_left,$x_right,$x_peak;
}

#=================================================================

#sub findValleyRef ($npts, \@array, [$bkgmode,'smooth|noisy']) {
sub findValleyRef ($\@;$$) {
### This subroutine returns:
### ($array_max,$array_min,$pt_max,$pt_min,$pt_FWHM,$pt_left,$pt_right,[$x_left,$x_right,$x_valley])
### where $x_left,$x_right,$x_valley are respective interpolated value (i.e.
### they are a higher precision than $pt_left,$pt_right,$pt_FWHM).
### +---------------------------------------+
### |ATTENTION: It does not smooth anything!|
### +---------------------------------------+
### All the difference between "normal" and "noisy" valley
### finding is that in "normal" case we climb up from the valley
### to find FWHM, while in "noisy" we climb down from the tails.
###
  my ($script, $npts, $arrayRef, $bkgmode, $searchmode);
  my ($array_max, $array_min, $pt_max, $pt_min);
  my ($pt_FWHM, $pt, $array_span, $a_, $half_valley);
  my ($pt_left, $pt_right, $x_left, $x_right, $x_valley, $dy);

  $script     = 'findValleyRef';
  $npts       = shift(@_);
  $arrayRef   = shift(@_);
  $bkgmode    = shift(@_);               		#zero or auto(default) - differs from findPeak
  $searchmode = shift(@_);               		#smooth(default) or noisy
  if (!defined $bkgmode)    {$bkgmode    = 'auto';}
  if (!defined $searchmode) {$searchmode = 'smooth';}

  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0,0,-1,-1,0,0,0,0,0;
  }

  $array_max = -1.E37;				        #array maximum value
  $array_min = +1.E37;				        #array minimum value
  $pt_max    = -1;					#array index at maximum value
  $pt_min    = -1;					#array index at minimum value
  $pt_FWHM   = -1;					#valley width in pts (if exists)

### Find maximum/minimum:
  for ($pt=0; $pt < $npts; $pt++) {
     $a_ = @$arrayRef[$pt];
     if ($a_ > $array_max) {$array_max = $a_; $pt_max = $pt;}
     if ($a_ < $array_min) {$array_min = $a_; $pt_min = $pt;}
  }
  if ($array_max < 0) {
     print STDOUT '*** '.$script.': all intensity is negative (max='.$array_max.')'."\n";
     print STDOUT '*** '.$script.': possibly incorrect background data.'."\n";
  }
  elsif ($array_min < 0) {
     print STDOUT '!!! '.$script.': some intensity is negative (min='.$array_min.')'."\n";
     print STDOUT '!!! '.$script.': possibly incorrect background data.'."\n";
  }

  if    ($bkgmode =~ /^zero$/i) {
     if ($array_min >= 0) {$half_valley = 2.0*$array_min;}
     else                 {$half_valley = 0.5*$array_min;}
  }
  elsif ($bkgmode =~ /^auto$/i) {
     $array_span = $array_max - $array_min;
     $half_valley = 0.5*$array_span + $array_min;
  } else {
     &press_key_exit('*** '.$script.': incorrect bkgmode='.$bkgmode,1);
  }

  if ($searchmode =~ /^smooth$/i) {
     ### Climb up to the left of the valley:
     $pt_left = $pt_max;
     while ((@$arrayRef[$pt_left] < $half_valley) && ($pt_left > 0)) {$pt_left--;}
     ### Climb up to the right of the valley:
     $pt_right = $pt_max;
     while ((@$arrayRef[$pt_right] < $half_valley) && ($pt_right < $npts-1)) {$pt_right++;}
  }          
  elsif ($searchmode =~ /^noisy$/i) {    
     ### Climb down on the valley from curve start:
     $pt_left = 0;
     while ((@$arrayRef[$pt_left]  > $half_valley) && ($pt_left < $pt_max)) {$pt_left++;}
     ### Climb down on the valley from curve end:
     $pt_right = $npts-1;
     while ((@$arrayRef[$pt_right] > $half_valley) && ($pt_right > $pt_max)) {$pt_right--;}
  }
  else {
     &press_key_exit('*** '.$script.': incorrect searchmode='.$searchmode,1);
  }

### If valley found:
  if (($pt_right < $npts-1) && ($pt_left > 0)) {
     $pt_FWHM = $pt_right - $pt_left + 1;
     $x_left = $pt_left;
     $dy = @$arrayRef[$pt_left] - @$arrayRef[$pt_left-1];
     if ($dy != 0.) {$x_left += ($half_valley - @$arrayRef[$pt_left]) / $dy;}
     $x_right = $pt_right;
     $dy = @$arrayRef[$pt_right] - @$arrayRef[$pt_right+1];
     if ($dy != 0.) {$x_right -= ($half_valley - @$arrayRef[$pt_right]) / $dy;}
     $x_valley = 0.5 * ($x_left + $x_right);
#    print STDOUT $script.":\n";
#    print STDOUT '   Half='.$half_valley."\n";
#    print STDOUT '    Left='.@$arrayRef[$pt_left].'   '.@$arrayRef[$pt_left-1]."\n";
#    print STDOUT '   Right='.@$arrayRef[$pt_right].'  '.@$arrayRef[$pt_right+1]."\n";
#    print STDOUT '    pt_left='.$pt_left.'   x_left='.$x_left."\n";
#    print STDOUT '   pt_right='.$pt_right.'  x_right='.$x_right."\n";
  } else {
     $x_left   = $pt_left;
     $x_right  = $pt_right;
     $x_valley = $pt_min;
  }
  return $array_max,$array_min,$pt_max,$pt_min,$pt_FWHM,$pt_left,$pt_right,$x_left,$x_right,$x_valley;
}

#=================================================================

#($x_centroid,[$sum]) = &findCentroidRef($npts,\@arrx,\@arry,[$discriminator]);
#sub findCentroidRef ($npts, \@arrx, \@arry, [$discriminator]) {
sub findCentroidRef ($\@\@;$) {
### This subroutine returns $x_centroid
### ATTENTION: It does not smooth anything!
  my ($script, $npts, $arrxRef, $arryRef, $discriminator);
  my ($x_centroid, $sumxy, $sumx, $sumy, $i_warn);
  my ($x_, $y_, $pt, $discr, $y_max);
  my ($x1_, $y1_, $sum, $percent);

  $script        = 'findCentroidRef';
  $npts          = shift(@_);
  $arrxRef       = shift(@_);
  $arryRef       = shift(@_);
  $discriminator = shift(@_);
  if (!defined $discriminator) {$discriminator=0;}

  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0,0;
  }

  $x_centroid = -1.E37;
  $sumxy      = 0.0;
  $sumx       = 0.0;
  $sumy       = 0.0;
  $sum        = 0.0;				#integral intensity above discriminator
  $i_warn     = 0;

### Find discriminated intensity level:
  if ($discriminator > 0) {
     $y_max =  -1.E37;
     for ($pt=0; $pt < $npts; $pt++) {
        $y_ = @$arryRef[$pt];
        if ($y_ > $y_max) {$y_max = $y_;}
     }
     $discr = $y_max * $discriminator;
  } else {
     $discr = 0.;
  }

### Find sums:
  for ($pt=0; $pt < $npts; $pt++) {
     $x_ = @$arrxRef[$pt];
     $y_ = @$arryRef[$pt]- $discr;
     if ($y_ < 0) {$i_warn++;  $y_ = 0;}
     $sumx += $x_;
     $sumy  += $y_;
     $sumxy += $x_ * $y_;
     if ($pt > 0) {$sum += 0.5*($y_ + $y1_) * ($x_ - $x1_);}
     $x1_ = $x_;
     $y1_ = $y_;
  }
  if ($npts > 0) {
     if ($sumy != 0.0) {$x_centroid = $sumxy/$sumy;}    #real centroid
     else              {$x_centroid = $sumx/$npts;}     #average x
  }

  if ($i_warn > 0) {
     $percent = (100.*$i_warn) / $npts;
     printf STDOUT '!!! '.$script.': intensity was '.$i_warn.' of '.$npts
                                  .' times (%1.0f%s) below discrimination level ('
                                  .$discriminator.') -- ignored'."\n",
                                   $percent,'%';
   }

  return $x_centroid, $sum;
}

#=================================================================

# ($x_centroid,[$sum]) = &findCentroidFixRef($npts,$x0,$dx,\@arry,[$discriminator]);
#sub findCentroidFixRef($npts,$x0,$dx,\@arry, [$discriminator]) {
sub findCentroidFixRef ($$$\@;$) {
### This subroutine returns $x_centroid for an equally spaced array
### ATTENTION: It does not smooth anything!
  my ($script, $npts, $x0, $dx, $arryRef, $discriminator);
  my ($x_centroid, $sumxy, $sumx, $sumy, $i_warn);
  my ($pt, $x_, $y_, $discr, $y_max);
  my ($y1_, $sum, $percent);

  $script        = 'findCentroidFixRef';
  $npts          = shift(@_);
  $x0            = shift(@_);
  $dx            = shift(@_);
  $arryRef       = shift(@_);
  $discriminator = shift(@_);
  if (!defined $discriminator) {$discriminator=0;}

  if ($npts <= 0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0,0;
  }

  $x_centroid = -1.E37;
  $sumxy      = 0.0;
  $sumx       = 0.0;
  $sumy       = 0.0;
  $sum        = 0.0;
  $i_warn     = 0;

### Find discriminated intensity level:
  if ($discriminator > 0) {
     $y_max =  -1.E37;
     for ($pt=0; $pt < $npts; $pt++) {
        $y_ = @$arryRef[$pt];
        if ($y_ > $y_max) {$y_max = $y_;}
     }
     $discr = $y_max * $discriminator;
  } else {
     $discr = 0.;
  }

### Find sums:
  for ($pt=0; $pt < $npts; $pt++) {
     $x_ = $x0 + $dx*$pt;
     $y_ = @$arryRef[$pt] - $discr;
     if ($y_ < 0) {$i_warn++;  $y_ = 0;}
     $sumx += $x_;
     $sumy  += $y_;
     $sumxy += $x_ * $y_;
     if ($pt > 0) {$sum += 0.5*($y_ + $y1_) * $dx;}
     $y1_ = $y_;
  }
  if ($npts > 0) {
     if ($sumy != 0.0) {$x_centroid = $sumxy/$sumy;}    #real centroid
     else              {$x_centroid = $sumx/$npts;}     #average x
  }

  if ($i_warn > 0) {
     $percent = (100.*$i_warn) / $npts;
     printf STDOUT '!!! '.$script.': intensity was '.$i_warn.' of '.$npts
                                  .' times (%1.0f%s) below discrimination level ('
                                  .$discriminator.') -- ignored'."\n",
                                   $percent,'%';
  }

  return $x_centroid, $sum;
}

#=================================================================

#sub smoothArray (\@array) {
sub smoothArray (\@) {
# May be used by any curve analysis program.
# This routine does weighted array averaging over 7 nearest points.
# The algorithm is due to Alexander Chuzo.

  my ($arref, @w, @q, $d, $i, $j, $k, $nPts);

  $arref = shift(@_);
  @w = (0.005,0.025,0.625,1.,0.625,0.025,0.005);	# weights
  @q = ();						#cached arrx
  $d = 0.0;						#divider

  no strict 'refs';
  $nPts = @$arref;
  if ($nPts < 4) {return;}		    		#nothing to smooth

  for($j=-3; $j<=3; $j++) {                 		#since we are overwriting the array
    if ($j >= 0) {$q[3+$j] = @$arref[$j];}  		#we need to make a cached copy
    else         {$q[3+$j] = 0.0;}
  }

  for ($i=0; $i<$nPts; $i++) {
     @$arref[$i] = 0.0;
     $d          = 0.0;
     for($j=$i-3; $j<=$i+3; $j++) {
       if (($j >= 0) && ($j < $nPts)) {
          $k           = $j-($i-3);
          @$arref[$i] += $w[$k] * $q[$k];
          $d          += $w[$k];
       }
     }
     if ($d != 0.) {@$arref[$i] = @$arref[$i]/$d;}
     else          {@$arref[$i] = 0.0;}

     for ($k=0; $k<6; $k++) {$q[$k]=$q[$k+1];} 		#shift cached copy of arr2 by one element
     if ($i<$nPts-4) {$q[6] = @$arref[$i+4];}  		#add next element to the cache (if exists)
     else            {$q[6] = 0.0;}
  }
  use strict 'refs';
}

#=================================================================

#sub arrayStatsRef (\@array) {
sub arrayStatsRef (\@) {
### This subroutine returns array statistics: ($pt_min,$y_min,$pt_max,$y_max)

  my ($script, $arrayRef, $npts, $y_min, $y_max, $pt_min, $pt_max, $pt);

  $script = 'arrayStatsRef';
  $arrayRef = shift(@_);
  $npts     = @$arrayRef;

  if ($npts <=0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0,0,0,0;
  }

  $y_min    = +1.E37;				#array minimum value
  $y_max    = -1.E37;				#array maximum value
  $pt_min   = -1;				#array index at minimum value
  $pt_max   = -1;				#array index at maximum value
### Find maximum
  for ($pt=0; $pt < $npts; $pt++) {
     if (@$arrayRef[$pt] > $y_max) {$y_max=@$arrayRef[$pt]; $pt_max=$pt;}
     if (@$arrayRef[$pt] < $y_min) {$y_min=@$arrayRef[$pt]; $pt_min=$pt;}
  }
  return $pt_min,$y_min,$pt_max,$y_max;
}

#=================================================================

#sub arrayStats ($npts, @array) {
sub arrayStats ($@) {
# This subroutine returns array statistics: ($pt_min,$y_min,$pt_max,$y_max)

  my ($script, $npts, @array, $narg);
  my ($y_min, $y_max, $pt_min, $pt_max, $pt, $pt1);

  $script = 'arrayStats';
  $narg   = @_;
  $npts   = shift(@_);

  if ($npts <=0) {
     print STDERR "\n".'!!! '.$script.': received zero-length array!'."\n\n";
     return 0, 0, 0, 0;
  }
  elsif ($npts > $narg-1) {
     printf STDERR "\n".'!!! '.$script.': received %d array elements instead of %d declared!'."\n\n",($narg-1),$npts;
     $npts = $narg-1;
  }

  $y_min  = +1.E37;				#array minimum value
  $y_max  = -1.E37;				#array maximum value
  $pt_min = -1;					#array index at minimum value
  $pt_max = -1;					#array index at maximum value
### Find maximum
  for ($pt=0; $pt<$npts; $pt++) {
     $array[$pt] = shift(@_);
     if (!defined $array[$pt]) {
        $pt1 = $pt+1;
        print STDOUT '*** '.$script.': undefined array for pt='.$pt1.' of '.$npts."\n";
     }
     else {
        if ($array[$pt] > $y_max) {$y_max=$array[$pt]; $pt_max=$pt;}
        if ($array[$pt] < $y_min) {$y_min=$array[$pt]; $pt_min=$pt;}
     }
  }
  return $pt_min,$y_min,$pt_max,$y_max;
}

#=================================================================

#sub &analyzeFilename ($file) {			#returns ($name,$ext)
sub analyzeFilename ($) {
### This subroutine returns name and extension of a given file

  my ($file, $name, $ext, @a, $n);

  $file = shift(@_);
  $name = $file;
  $ext  = '';
  @a = split /\./, $file;
  $n = @a;
  if ($n > 1) {
     $ext  = $a[$n-1];
     $name =~ s/\.$ext$//;
  }
  return $name, $ext;
}

#=================================================================

#sub set_interrupt () {
sub set_interrupt () {
### This subroutine sets global interrupt flag
  our $INTERRUPTED = 1;
  print STDOUT "\n".'*** Got Interrupt Signal! ***'."\n";
}

#=================================================================

#sub check_interrupt () {			#returns status=1 if run is interrupted
sub check_interrupt () {
### This subroutine checks global interrupt flag
  our $INTERRUPTED;
  if (defined $INTERRUPTED) {
     if ($INTERRUPTED eq "1") {
        return 1;
     }
  }
  return 0;
}

#=================================================================

#sub makeScriptName ([1|'short'|'noext']) {		#returns truncated script name (no path)
sub makeScriptName (;$) {
  my ($script, $short);
  $short = shift(@_); if (!defined $short) {$short=0;}
# $script = $0;
# $script =~ s/\\/\//g;				#change \ to /
# $script =~ s/^.*\///g;			#remove everything before last "/"
  $script = $0 =~ s/^.*[\/\\]//gr;		#same as above 3 in 1 line: remove everything before last "/" or "\"

  if ($short eq '1'        || 
      $short =~ /^short$/i || 
      $short =~ /^noext/i) {$script =~ s/\.pl$//i;}
  return $script;
}

#=================================================================

#sub getFileAgeHours ($file) {  	#returns $age or -1 if file does not exist
sub getFileAgeHours($) {

  my ($script, $file, $age_hours);

  $script = 'getFileAgeHours';
  $file = shift(@_);
  if (-f $file) {				#if file exists:
     $age_hours = 24 * (-M $file);
     if ($age_hours < 0) {
        print STDOUT '!!! '.$script.': file "'.$file.'" has modification'
                                   .' timestamp in future'."\n";
        return 0;
     } else {
        return $age_hours;
     }
  } else {
     return -1;					#file does not exist
  }
}

#=================================================================

#sub get_host_email_from_UAP ($user,$script) {		#returns host email for current user, uses mySQL DB
sub get_host_email_from_UAP ($$) {
# use DBI;						#mySQL interface is needed
  our ($beamline);					#MySQL pass (see beamline.pl)
  my ($user, $script, $dbhost, $dbase, $dbtable);
  my ($blname, $dbquery, $dsn, $dbh, $sth);
  my ($dbu, $dbp, $nrw, $rwref, $email);

  $user   = shift(@_);
  $script = shift(@_);  $script .= '/get_host_email_from_UAP';

  if (!defined $beamline)  {print STDOUT '!!! '.$script.': undefined beamline!'."\n";   return '';}
  if	($beamline =~ /23i:?$/) {$blname = 'ID-D';}
  elsif ($beamline =~ /23o:?$/) {$blname = 'ID-B';}
  elsif ($beamline =~ /23b:?$/) {$blname = 'BM';}
  else {print STDOUT '!!! '.$script.': no mySQL usr data for beamline='.$beamline.'!'."\n"; return '';}
  
  $dbhost  = 'bl1upper.gmca.aps.anl.gov';		#this is for all 3 beamlines; should we use bl3upper for BM?
  $dbase   = 'gmca_accounts';
  $dbtable = 'accounts';
  $dbu     = 'dhs';					# this is for $dbhost, we cannot use the value from beamline.pl
  $dbp     = '';					# this is for $dbhost, we cannot use the value from beamline.pl
  $dsn     = 'DBI:mysql:host='.$dbhost.';database='.$dbase;
  $dbquery = 'SELECT * FROM '.$dbtable.' where username="'.$user.'" and beamline="'.$blname.'"';

  $dbh = DBI->connect($dsn, $dbu, $dbp, {PrintError=>0,});
  if (!defined $dbh) {
     print STDOUT '!!! '.$script.': Cannot connect to mySQL server '.$dbhost."\n"; 
     return '';
  }
  $sth = $dbh->prepare($dbquery);
  $nrw = $sth->execute();				#nrecords works for MySQL only. For SQLite result is def/undef only.
  if (!defined $nrw || $nrw < 1) {
     print STDOUT '!!! '.$script.': no mySQL data for user '.$user."\n"; 
     $sth->finish();
     $dbh->disconnect();
     return '';
  }
  $rwref = $sth->fetchrow_hashref();
  if (!defined $rwref->{'host_email'}) {
     print STDOUT '!!! '.$script.': no mySQL data for user '.$user."\n"; 
     $sth->finish();
     $dbh->disconnect();
     return '';
  }
  $email = $rwref->{'host_email'};
  $sth->finish();
  $dbh->disconnect();
  print STDOUT $script.': host email = '.$email."\n";
  if ($email =~ /\@anl\.gov/ || $email =~ /\@aps\.anl\.gov/ ) {return $email;}
  else {
     print STDOUT '!!! '.$script.': host email is not "@anl.gov" or "@aps.anl.gov"'."\n";
     return '';
  }
}

#=================================================================

#sub get_host_email_from_UAP_old ($currentUser,$script,$os) {	#returns host email for current user
sub get_host_email_from_UAP_old ($$$) {
  my ($currentUser, $script, $os, $uapDB, $uapHandle, $string);
  $currentUser = shift(@_);
  $script      = shift(@_);
  $os          = shift(@_);
  if ($os !~ /^LINUX$/i) {return '';}
  $uapDB = '/mnt/software/uap/UserAcctFiles/users_info.tcl';
  if (-f $uapDB) {
     $uapHandle = open (UAP,'< '.$uapDB);
     if (!defined $uapHandle) {
        print STDOUT '!!! '.$script.': Cannot open "'.$uapDB.'" for reading!'."\n";
        return '';
     }
#    print STDOUT "Reading \'${uapDB}\'\n";
     while (<UAP>) {
### read line
        $string = $_;
        if ($string !~ /^User ::${currentUser}/) {next;}
        while (<UAP>) {
           $string = $_;
           if ($string =~ /-hostEmail/i) {
              chomp $string;
              $string =~ s/^ *-hostEmail \"//g;
              $string =~ s/\".*$//g;
              print STDOUT $script.': host email = '.$string."\n";
              if ($string =~ /\@anl\.gov/) {return $string;}
              else                         {
                 print STDOUT '!!! '.$script.': host email is not "@anl.gov"'."\n";
                 return '';
              }
           }
           elsif ($string =~ /^User ::/) {
              print STDOUT '!!! '.$script.': no host email for user = '.$currentUser."\n";
              return '';
           }
        }
        print STDOUT '!!! '.$script.': no host email for user = '.$currentUser."\n";
        return '';
     }
     close(UAP);
     print STDOUT '!!! '.$script.': no UAP data for user = '.$currentUser."\n";
     return '';
  } else {
     print STDOUT '!!! '.$script.': cannot find UAP file = '.$uapDB."\n";
     return '';
  }
}

#=================================================================

#sub isMounted ($mountPoint) {	#returns 1(yes) is mount point is mounted; otherwise 0(no)
sub isMounted ($) {
  my ($script, $mountPoint, $procMountFile, $handle, $record, @words, $nwords);

  $script = 'isMounted';
  $mountPoint = shift(@_);
  if (!defined $mountPoint) {
     print STDOUT '!!! '.$script.': unspecified mountpoint'."\n";
     return 1;
  }
  if ($^O !~ /Win/i) {
### Proc filesystem which tracks current mounts on Linux:
     $procMountFile = '/proc/mounts';
     $handle = open (MOUNTLIST, '< '.$procMountFile);
     if (!defined $handle) {
        print STDOUT '!!! '.$script.': cannot open "'.$procMountFile.'" -- '
                    .'presuming mountpoint='.$mountPoint.' is mounted'."\n";
	return 1;
     }
     while (<MOUNTLIST>) {
        $record = $_;
        chomp $record;				# trail CR
        @words = split " ", $record;
        $nwords = @words;
        if ($nwords >=2) {
           if ($words[1] =~ /^$mountPoint$/){
### The mount is found
              close(MOUNTLIST);
              return 1;
           }
        }
     }
### The mount is NOT found
     close(MOUNTLIST);
     return 0;
  }
### Microsoft Windows:
  return 1;
}

#=================================================================

#$status = &tellBluIce ($msg,[$type]);  	#type=note,warn,error,popup_note,popup_error,direct
sub tellBluIce ($;$) {

  our ($beamline, $bluiceLogPV);		#defined in beamline.pl
  our ($ZENITY_VIA_BLUICE);
  my ($script, $msg, $type, $status);
  my ($maxLen, $ll);

  $msg    = shift(@_);
  $type   = shift(@_);
  $script = 'tellBluIce';

  if (!defined $bluiceLogPV) {
     $ZENITY_VIA_BLUICE=0;
     &press_key_exit('*** '.$script.': JBluIce Log PV is undefined',1);
  }
  if ($bluiceLogPV !~ /256$/) {$maxLen = 39;}	#string PV version
  else                        {$maxLen = 255;}	#waveForm PV version

  if    (!defined $type)     {$type='log_note ';}
  elsif (length($type) == 0) {$type='log_note ';}
  if    ($type =~ /err/i)    {
     if ($type =~ /popup/i)  {$type = 'popup_error ';}		#'type' contains both popup & err
     else                    {$type = 'log_error ';}
  }
  elsif ($type =~ /warn/i)   {
     if ($type =~ /popup/i)  {$type = 'popup_error ';}		#'type' contains both popup & warn
     else                    {$type = 'log_warn ';}
  }
  elsif ($type =~ /note/i)   {
     if ($type =~ /popup/i)  {$type = 'popup_note ';}		#'type' contains both popup & note
     else                    {$type = 'log_note ';}
  }
  elsif ($type =~ /direct/i) {$type = '';}
  else  {
     print STDOUT '!!! '.$script.': incorrect message type='.$type.'. Using "log_note".'."\n";
     $type = 'log_note ';
  }
  if ($bluiceLogPV !~ /^${beamline}/) {$bluiceLogPV = $beamline.':'.$bluiceLogPV;}

  $msg = $type.$msg;
  $ll = length ($msg);
  if ($ll > $maxLen) {
     print STDOUT '!!! '.$script.': length of ['.$msg.'] = '.$ll.' > '.$maxLen.'. Truncating.'."\n";
     $msg = substr($msg,0,$maxLen);
  }
  if ($maxLen <= 39) {$status = Pezca::PutString ($bluiceLogPV, $msg);}
  else               {$status = &PezcaPutStringWF($bluiceLogPV, $msg);}
  &report_only($script,$status,$bluiceLogPV);
  return $status;
}

#=================================================================

# $status = &setSampleEnv(\@commands,['nowait']);
sub setSampleEnv(\@;$) {

  our ($beamline, $bluiceLogPV);				#defined in beamline.pl
  our ($ZENITY_VIA_BLUICE);
  my ($script, $cmdRef, $wflag, $cmd, @commands, $PEZCA);
  my ($tell_bluice, $status, $useServer, $prg);
  my ($sampleEnvPV, $start, $poll, $duration);
  my ($timeout, @states, $str, $zenity_via);

  $script = 'setSampleEnv';
  $cmdRef = shift(@_);
  $wflag  = shift(@_);
  if (!defined $wflag)       {$wflag = 'wait';}
  if ($wflag !~ /^nowait$/i) {$wflag = 'wait';}
  if (!defined $ZENITY_VIA_BLUICE) {$ZENITY_VIA_BLUICE=0;}

  $useServer = 1;						#1=server, 0=script
# if ($beamline eq '23i') {$useServer=0;}			#use script
  @commands = ();
  $tell_bluice = 0;
# @states = qw/accepted finished popup failed/;
  @states = qw/finished popup failed/;
  $poll = 0.05;							#server poll interval in ms
  $timeout = 120;						#timeout
  foreach $cmd (@$cmdRef) {
     if ($cmd !~ /^${beamline}:?$/) {push(@commands,$cmd);}
     if ($cmd =~ /^bluice$/i)       {$tell_bluice = 1;}
  }

  if ($useServer) { #-------------------------------------------call-server-
     $sampleEnvPV = $beamline.':bi:sampleEnv';
     $status = &addCaMonitor($sampleEnvPV);
     &report_only($script,$status,$sampleEnvPV);
     $cmd = join(' ',@commands);
     $status = &PezcaPutStringWF($sampleEnvPV,$cmd);		#send command to the server
     if ($status) {
        &report_only($script,$status,$sampleEnvPV);
	return 1;
     }
     if ($wflag =~ /^nowait$/i) {return 0;}
     $start = time();
     do {							#wait for command completion
        select(undef,undef,undef,$poll);			#sleep(..);
        ($status,$str) = &PezcaGetStringWF($sampleEnvPV);
        if ($status) {
           print STDOUT "\n".'*** '.$script.': error='.$status.' reading PV='.$sampleEnvPV."\n\n";
           if ($tell_bluice) {&tellBluIce($script.': error reading '.$sampleEnvPV,'error');}
           return 1;
        }
        if (grep(/^$str$/i, @states)) {
           if ($str =~ /^finished$/i) {					#exit=OK
              return 0;
           } else {							#exit=popup/failed
              if (defined $bluiceLogPV && !$tell_bluice) {		#report popup, if exists & not meant fot BluIce
                 ($status,$str) = &PezcaGetStringWF($bluiceLogPV); 	#read popup message
                 if ($status) {
                    &report_only($script,$status,$bluiceLogPV);
                 } else {
                    $zenity_via = $ZENITY_VIA_BLUICE;
                    $ZENITY_VIA_BLUICE = 0;
                    if ($str =~ /^popup_error/) {
                       $str =~ s/^popup_error//;
                       &zenity_error($str,'nowait');                    #reproduce server error popup
                    }
                    elsif ($str =~ /^popup_note/) {
                       $str =~ s/^popup_note//;
                       &zenity_info($str,'nowait');			#reproduce server note popup
                    }
                    $ZENITY_VIA_BLUICE = $zenity_via;
                 }
              }
              return 1;
           }
        }
        $duration = time() - $start;
     } while ($duration < $timeout);
     $prg = 'SampleEnvSrv';
     print STDOUT "\n".'*** '.$script.': no response from '.$prg.' within timeout='.$timeout."\n\n";
     if ($tell_bluice) {&tellBluIce($script.': timeout from '.$prg,'error');}
     return 1;
  } else { #----------------------------------------------------call-script-
     if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
     elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
     else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
     $PEZCA =~ s/\\/\//g;
     if ($PEZCA !~ /\/$/) {$PEZCA .= '/';}
     $prg = 'sampleEnv.pl';
     unshift (@commands, ('perl', '-W', $PEZCA.$prg, $beamline));
     if ($wflag =~ /^nowait$/i) {
        if ($^O !~ /Win/i) {
           $cmd=join(' ',@commands); 
           system($cmd.' &');
#          unshift (@cmd,'screen','-d','-m');
#          unshift (@cmd,'nohup');
        } else {
           system(1,@commands);
        }
        return 0;
     } else {
        $status = system(@commands);
        if ($status) {
           print STDOUT "\n".'*** '.$script.': failure='.$status.' from '.$prg."\n\n";
           if ($tell_bluice) {&tellBluIce($script.': failure from '.$prg,'error');}
           return 1;
        } else {
           return 0;
        }
     }
  } #-----------------------------------------------------------------------
  return 0;
}

#=================================================================

# $status = &getSampleEnv(\%states);                            #fills states of (Collimator Beamstop Pindiode Backlight SONICC)
sub getSampleEnv(\%) {

  our ($beamline);						#defined in beamline.pl
  my ($script, $states);
  my ($sampleEnvPV, $status, $str);
  my ($poll, $timeout, $start, $duration);
  my (@words, $w, @obj);

  $script = 'getSampleEnv';
  $states = shift(@_);

  %{$states} = ();
  $poll = 0.05;							#server poll interval in ms
  $timeout = 50;						#timeout

  $sampleEnvPV = $beamline.':bi:sampleEnv';
### Request states:
  $status = &PezcaPutStringWF($sampleEnvPV,'getStates');	#send command to the server
  if ($status) {
     &report_only($script,$status,$sampleEnvPV);
     return 1;
  }
### Wait for response
  $status = &addCaMonitor($sampleEnvPV);
  &report_only($script,$status,$sampleEnvPV);
  $start = time();
  do {								#wait for command completion
     select(undef,undef,undef,$poll);				#sleep(..);
     ($status,$str) = &PezcaGetStringWF($sampleEnvPV);
     if ($status) {
        print STDOUT "\n".'*** '.$script.': error='.$status.' reading PV='.$sampleEnvPV."\n\n";
        return 1;
     }
     if ($str =~ /^;/) {
        $str =~ s/^;\s*//;
        @words = split(/\s/,$str);
        foreach $w (@words) {
           @obj = split('=',$w);
           if (@obj != 2) {
              print STDOUT "\n".'*** '.$script.': incorrect val=key in '.$w.' while parsing str=['.$str.']'."\n\n";
              return 1;
           }
           $states->{$obj[0]} = $obj[1];
        }
        $status = &PezcaPutStringWF($sampleEnvPV,'');		#clear PV
        &report_only($script,$status,$sampleEnvPV);
        return 0;
     }
     $duration = time() - $start;
  } while ($duration < $timeout);
  print STDOUT "\n".'*** '.$script.': timeout on getting response to "getStates" from PV='.$sampleEnvPV."\n\n";
  return 1;
}


#=================================================================

#($status,$msg) = &read_data_file ($file,\@xdata,\@ydata,[$xcol,$ycol]);
sub read_data_file ($\@\@;$$) {
  my ($script, $file, $xref, $yref, $xcol, $ycol, $maxcol, $ncol);
  my ($handle, $line, $npts, $string, @arrline, $msg);
  $script = 'read_data_file';
  $file   = shift(@_);
  $xref   = shift(@_);
  $yref   = shift(@_);
  $xcol   = shift(@_); if (!defined $xcol) {$xcol=1;}
  $ycol   = shift(@_); if (!defined $ycol) {$ycol=2;}
  if ($xcol == $ycol) {
     $msg = '*** '.$script.': Xcol='.$xcol.' and Ycol='.$ycol.' are the same';
     return 1,$msg;
  }
  $maxcol = &maxval(($xcol,$ycol));

### Open data file:
  if (-f $file) {
     print STDOUT $script.': Opening file="'.$file.'", xcol='.$xcol.', ycol='.$ycol."\n";
     $handle = open (DAT, '< '.$file);
     if (!defined $handle) {
        $msg = '*** '.$script.': Error opening data file='.$file;
        return 1,$msg;
     }
#    select DAT; $|=1;
  } else {
     $msg = '*** '.$script.': data file: '.$file.' does not exist';
     return 1,$msg;
  }
### Read the file ignoring the comment lines:
  $line = 0;
  $npts = 0;
  while (<DAT>) {						#begin loop over lines in file
     $line++;
     if (!defined $_) {next;}
     $string = $_;						#read line
     chomp $string;						#trail CR
     $string =~ s/\#.*$//g;					#trail comments started by "#" ("."=any char, "*"=any times, "$"=EOL)
     $string =~ s/,/ /g;					#replace comas by spaces
     $string =~ s/\t/ /g;					#replace tabs by spaces
     $string =~ s/ +/ /g;					#compress spaces ("+" means match 1 or more times)
     $string =~ s/^ *//g;					#remove beginning-of-the-line spaces
     $string =~ s/ *$//g;					#remove end-of-the-line spaces
     if (length($string) == 0) {next;}
     @arrline = split ' ', $string;  				#split the line into words
     $ncol = @arrline;
     if ($ncol < $maxcol) {
        $msg = '*** '.$script.': data file '.$file.' has '.$ncol.' < '.$maxcol.' columns at line='.$line;
        return 1,$msg;
     }
     if ($xcol > 0) {$$xref[$npts] = $arrline[$xcol-1];}	#x-data
     else           {$$xref[$npts] = $npts;}            	#x-data
     $$yref[$npts] = $arrline[$ycol-1];				#y-data
     $npts++;
  }								#end of loop over lines in data file
  close(DAT);							#close data file
  return 0;
}	##read_data_file


#=================================================================

#$epoch = &timestamp2epoch($timestamp); 		#expected: YYYY-MM-DD HH:MM[:SS]
sub timestamp2epoch ($) {
  my ($script, $timestamp, $g1, $g2, $epoch);
  my (@ct, $el);					#$sec,$min,$hour,$mday,$mon,$year

  $script = 'timestamp2epoch';
  $timestamp = shift(@_);
  $epoch = 0;
  if ((length($timestamp) != 19 && 			#length(YYYY-MM-DD HH:MM:SS)
       length($timestamp) != 16) || 			#length(YYYY-MM-DD HH:MM)
       $timestamp !~ /[-\/_]/    || 			#e.g. YYYY-MM-DD or YYYY/MM/DD or YYYY_MM_DD
       $timestamp !~ ':'         ||			#e.g. HH:MM
       $timestamp !~ ' ') {				#should contain space between date & time
     print STDOUT '!!! '.$script.': incorrect time stamp ['.$timestamp.']'."\n";
     return $epoch;
  }
  ($g1,$g2) = split ' ', $timestamp;			#split into date & time
  ($ct[5], $ct[4], $ct[3]) = split /[-\/_]/, $g1;	#($year, $month, $mday)
  ($ct[2], $ct[1], $ct[0]) = split ':',      $g2;	#($hour, $min, $sec)
  if (!defined $ct[0]) {$ct[0] = '00';}
  foreach $el (@ct) {
     if (length($el) > 4 || length($el) < 2 || $el !~ /^\d+$/) {
        print STDOUT '!!! '.$script.': incorrect time stamp ['.$timestamp.']'."\n";
        return $epoch;
     }
  }
  $ct[5] -= 1900;
  $ct[4] -= 1;
  if (!defined &function_exists('timelocal')) {
     if (&try_load_module('Time::Local')) {return 0;}
  }
  $epoch = timelocal($ct[0],$ct[1],$ct[2],$ct[3],$ct[4],$ct[5]);	#use Time::Local
  return $epoch;
}


#=================================================================


# $status = &sort_by_x('incr|decr',\@x,@y1,[\@y2,\@y3,...]);
sub sort_by_x ($\@\@;\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@) {
  my ($script, $type, $xref, @yref, $npts, $pt, $j, $j1, $tmp, $nSort, $y);

  $script = 'sort_by_x';
  $type = shift(@_);
  $xref = shift(@_);
  @yref = @_;				#the rest: array of pointers to y-arrays
  $npts = @$xref;			#array length
  if ($type !~ /^incr/i && $type !~ /^decr/) {
     print STDOUT '*** '.$script.': incorrect sorting type='.$type."\n";
     return -1;
  }
  if ($npts < 2) {return 0;}		#no sorting required
  $nSort = 0;
  if ($type =~ /^incr/i) {
#    print STDOUT $script.': sorting data in increasing order...'."\n";
     for ($pt=0; $pt<($npts-1); $pt++) {
        for ($j=0; $j<($npts-1-$pt); $j++) {
           $j1 = $j + 1;
           if (@$xref[$j1] < @$xref[$j]) {
              $nSort++;
              $tmp        = @$xref[$j];
              @$xref[$j]  = @$xref[$j1];
              @$xref[$j1] = $tmp;
              foreach $y (@yref) {
                 $tmp     = @$y[$j];
                 @$y[$j]  = @$y[$j1];
                 @$y[$j1] = $tmp;
              }
           }
        }
     }
  } else {
#    print STDOUT $script.': sorting data in decreasing order...'."\n";
     for ($pt=0; $pt<($npts-1); $pt++) {
        for ($j=0; $j<($npts-1-$pt); $j++) {
           $j1 = $j + 1;
           if (@$xref[$j1] > @$xref[$j]) {
              $nSort++;
              $tmp        = @$xref[$j];
              @$xref[$j]  = @$xref[$j1];
              @$xref[$j1] = $tmp;
              foreach $y (@yref) {
                 $tmp     = @$y[$j];
                 @$y[$j]  = @$y[$j1];
                 @$y[$j1] = $tmp;
              }
           }
        }
     }
  }
  if ($nSort > 0) {
#    print STDOUT '!!! '.$script.': data has been re-ordered.'."\n";
     return 1;
  } else {
#    print STDOUT '!!! '.$script.': data sorting was not required.'."\n";
     return 0;
  }
}


#=================================================================


# $npts = &remove_duplicates($filter,$step,\@x,@y1,[\@y2,\@y3,...]);
sub remove_duplicates($$\@\@;\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@\@) {

### The arrays must be sorted over x and 'quasi-regular', i.e.
### $x(last)-$x(first)/($npts-1) is about an average step

  my ($script, $fltr, $step, $xref, @yref, $npts, $pt, $y, $diff, $nDup);

  $script = 'remove_duplicates';
  $fltr = shift(@_);
  $step = shift(@_);
  $xref = shift(@_);
  @yref = @_;
  $npts = @$xref;
  if ($npts < 2) {return $npts;}			#no duplicates here

  if (!defined $step || $step =~ /^auto$/i) {
     $step = (@$xref[$npts-1] - @$xref[0])/($npts-1);	#presumes 'quasi-regular' data spacing
  }
  if ($fltr <=0 || $fltr > 0.5) {
     print STDOUT '*** '.$script.': duplicates filter='.$fltr.' is not in range [0.0-0.5]'."\n";
     return $npts;
  }
# print STDOUT $script.': filter='.$fltr.'  step='.$step."\n";
  $fltr = $fltr*abs($step);				#points spaced below 20% of step are duplcates

### Remove duplicates:
  $nDup   = 0;
  $pt     = 1;
  while ($pt < $npts) {
     $diff = @$xref[$pt] - @$xref[$pt-1];
     if (abs($diff) < $fltr) {
        @$xref[$pt-1] = 0.5*(@$xref[$pt] + @$xref[$pt-1]);
#       print STDOUT $script.': filter*step='.$fltr.'  diff='.$diff.'  x(average)='.@$xref[$pt-1]."\n";
        splice(@$xref, $pt, 1);				#compress x array
        foreach $y (@yref) {
           @$y[$pt-1] = 0.5*(@$y[$pt] + @$y[$pt-1]);
           splice(@$y, $pt, 1);				#compress data arrays
        }
        $pt--;
        $npts--;
        $nDup++;
     }
     $pt++;
  }
  if ($nDup > 0) {
     print STDOUT '!!! '.$script.': '.$nDup.' duplicate data points removed'
                        .' ('.$npts.' total remaining)'."\n";
  }
  return $npts;
}


#=================================================================

#sub Which ($prg) {	#returns path to $prg or undef if not found. On win32 tries .exe,.bat,.com
sub Which ($) {

  my ($program, $path, @parr, $file, $os, @ext, $ex, $filewin);
  $program = shift;				#get the passed value
  return if (not defined($program));		#return if nothing is provided
  if ($^O !~ /Win/i) {$os = 'unix';}
  else               {$os = 'win32';}
  @ext = ('.exe', '.bat', '.com');
# use ENV;					#load the ENV
  $path = $ENV{PATH};				#load the path
  $path =~ s/\\/\//g;				#replace all \ by /
  if ($os eq 'unix') {$path =~ s/:/;/g;}	#replace all ":"  by ";"
  else               {$path = './;'.$path;}	#on Windows, also add current directory
  $path =~ s/\/;/;/g;				#replace all "/;" by ";"
  @parr = split(/;/,$path);			#now make an array (split by ";"_
  foreach (@parr) {				#loop and find if the file is in one of the paths
    $file = $_ . '/' . $program;		#concatenate the file
    #print STDOUT '  Which: checking '.$file."\n";
    return $file if ((-e $file) && (-f $file));	#return the path if it was found
    if ($os eq 'win32') {
      foreach $ex (@ext) {			#loop over possible extensions
        if ($file !~ /${ex}$/i) {
          $filewin = $file.$ex;
          return $filewin if ((-e $filewin) && (-f $filewin));
        }
      }
    }
  }
  return;
}


#=================================================================

#sub split_filename_dirname ($filespec) {	#returns ($dirname,$filename)
sub split_filename_dirname ($) {
  my ($filespec, $dirname, $filename);
  $filespec = shift(@_);
  if ($filespec =~ m|[/\\]$|) {			#if ends by '\' or '/'
     ($dirname, $filename) = ($filespec, '');
  } else {
    ($dirname, $filename) = ($filespec =~ m|^(.*[/\\])([^/\\]+?)$|);	#split by  '\' or '/'
    if (!defined $dirname && $^O =~ /Win/i) {				#if unsuccessful & Windows
       ($dirname, $filename) = ($filespec =~ m|^([a-zA-Z]\:)(.*)$|);	#split by drive name (e.g. 'c:')
    }
    if (!defined $dirname) {($dirname, $filename) = ('',$filespec);}
  }
  return ($dirname, $filename);
}


#=================================================================

# $radians = &degrees2rad($degrees);		#this is a replacement to deg2rad of Math::Trig
sub degrees2rad($) {
  my ($degrees, $radians, $pi);
  $degrees = shift(@_);
  $pi = 4.*atan2(1.,1.);
  if (!defined $pi) {$pi = 3.1416;}
  $radians = $degrees*$pi/180.;
  return $radians
}


#=================================================================


# $degrees = &rad2degrees($radians);		#this is a replacement to rad2deg of Math::Trig
sub rad2degrees($) {
  my ($degrees, $radians, $pi);
  $radians = shift(@_);
  $pi = 4.*atan2(1.,1.);
  if (!defined $pi) {$pi = 3.1416;}
  $degrees = $radians*180./$pi;
  return $radians
}


#=================================================================


#sub wait4topup2happen([$topupInterval,$timeAfterTopup]) {
sub wait4topup2happen(;$$) {

  my ($script, $topupInterval, $timeAfterTopup, $topupIntervalPV);
  my ($time2topupPV, $time2topup, $time2topup1, $time2topup_scan);
  my ($timeSinceTopupPV, $timeSinceTopup, $timeSinceTopup1);
  my ($topupHappened, $status, $wait_, $wait_report, $wait_step, $report_step);

### This routine waits the APS beam topup to occur (plus $timeAfterTopup)
### to ensure that an operation we are planning will not interfere with
### the topup.

  $script = 'wait4topup2happen';
  $topupInterval  = shift(@_);
  $timeAfterTopup = shift(@_);					#extra time to wait AFTER topup
  if (!defined $timeAfterTopup) {$timeAfterTopup = 0.25;}	#in seconds
  if ($timeAfterTopup < 0)      {$timeAfterTopup =-$timeAfterTopup;}

  $time2topupPV     = 'Mt:TopUpTime2Inject';
  $topupIntervalPV  = 'S:TopUpInterval';
  $timeSinceTopupPV = 'XFD:TimeSinceTopUp';
  $report_step     = 5;

  if (!defined $topupInterval)  {
     ($status, $topupInterval) = Pezca::GetDouble($topupIntervalPV);
     if ($status) {
        &report_only($script, $status, $topupIntervalPV);
        $topupInterval = 120;					#in seconds
        print STDOUT '!!! '.$script.': presuming topup interval is '.$topupInterval.'s.'."\n";
     }
  }
  elsif ($topupInterval < 0)  {
     $topupInterval =-$topupInterval;
  }

  $status = &addCaMonitor($time2topupPV);
  if ($status) {
     &report_only($script, $status, $time2topupPV);
     print STDOUT '*** '.$script.': topups will be ignored'."\n";
     return 1;
  }

  ($status, $time2topup_scan) = Pezca::GetString($time2topupPV.'.SCAN');
  if ($status) {
     &report_only($script, $status, $time2topupPV.'.SCAN');
  }
  elsif ($time2topup_scan =~ /Passive/i) {
     $time2topup_scan = 1;					#take 1second
  }
  elsif ($time2topup_scan !~ /second/i) {
     print STDOUT '!!! '.$script.': unexpected time-to-topup scan rate ['.$time2topup_scan.'].'."\n";
     undef $time2topup_scan;
  }
  else {
     $time2topup_scan =~ s/second//ig;
     $time2topup_scan =~ s/\s+//g;
  }
  if (!defined $time2topup_scan) {
     $time2topup_scan = 1;
     print STDOUT '!!! '.$script.': failed to determine time-to-topup scan rate.'."\n"
                 .'!!! '.$script.': Topup checks will be done every '.$time2topup_scan.'s'."\n";
  }
# print STDOUT $script.': checks will be done every '.$time2topup_scan.'s'."\n";

  $status = &addCaMonitor($timeSinceTopupPV);
  if ($status) {
     &report_only($script, $status, $timeSinceTopupPV);
     undef $timeSinceTopupPV;
  }

  $wait_step       = 2*$time2topup_scan+0.1;		#this must exceed scan_step as otherwice may get $time2popup=$time2popup1
  $time2topup      = 0;
  $time2topup1     = -1;
  $timeSinceTopup  = $topupInterval;
  $timeSinceTopup1 = $topupInterval;
  $wait_           = 0;
  $wait_report     = 0;
  $topupHappened   = 0;

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  do {
     select (undef, undef, undef, $wait_step);
     $wait_       += $wait_step;
     $wait_report += $wait_step;
     if ($wait_report >= $report_step) {
        $wait_report = 0;
#       printf STDOUT '  -- %3.0fs waiting for the APS topup (remaining=%.0fs, max=%.0fs)'."\n", $wait_, $time2topup, $topupInterval;
        printf STDOUT '  -- waiting for the APS topup, remaining time=%.0fs', $time2topup;
        if (defined $timeSinceTopupPV) {
           printf STDOUT ', time since previous=%.0fs'."\n", $timeSinceTopup;
        } else {
           print STDOUT "\n";
        }
     }
     if (defined $timeSinceTopupPV) {
        ($status, $timeSinceTopup) = Pezca::GetDouble($timeSinceTopupPV);
        if ($status) {
           &report_only($script, $status, $timeSinceTopupPV);
           $timeSinceTopup = $topupInterval;
        }
     }
     ($status, $time2topup) = Pezca::GetDouble($time2topupPV);
     if ($status) {
        &report_only($script, $status, $time2topupPV);
        print STDOUT '*** '.$script.': topups will be ignored.'."\n";
        return 1;
     }
     elsif ($topupInterval-$time2topup < 0) {
        printf STDOUT '!!! '.$script.': APS time to topup (%.0fs) exceeds'
                     .' topup interval (%.0fs)'."\n", $time2topup, $topupInterval;
        return 1;
     }
     elsif ($time2topup == $time2topup1) {
        if ($timeSinceTopup == $timeSinceTopup1) {
           printf STDOUT '*** '.$script.': APS topup timers are not running:'
                               .' timers are stuck at %ds to topup and'
                               .' %ds since topup.'."\n", $time2topup,$timeSinceTopup;
           return 1;
        }
        elsif ($time2topup == 0) {
           ### do nothing
        }
#       else {
#          printf STDOUT '!!! '.$script.': APS topup timer may not be running'
#                              .' (unchanged reading at %ds).'."\n", $time2topup;
#       }
     }
     elsif ($time2topup > $time2topup1 && $time2topup1 != -1) {
        $topupHappened = 1;
     }
     $time2topup1     = $time2topup;
     $timeSinceTopup1 = $timeSinceTopup;
  } while ($topupHappened == 0 && $wait_ < $topupInterval+$timeAfterTopup);
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  if ($timeAfterTopup > 0) {select (undef, undef, undef, $timeAfterTopup);}
  if ($wait_ >= $topupInterval) {
     printf STDOUT '*** '.$script.': waiting time (%.0fs) exceeds'
                     .' topup interval (%.0fs)'."\n", $wait_, $topupInterval;
     return 1;
  } else {
     return 0;
  }
}


#=================================================================


#$description = &getPVdescription($pv);
sub getPVdescription($) {

 my ($script, $pv, $desc, $status);

  $script = 'getPVdescription';
  $pv     = shift(@_);
  $pv =~ s/\.VAL$//;
  $pv .= '.DESC';
  ($status, $desc) = Pezca::GetString($pv);
  if ($status) {
     &report_only($script, $status, $pv);
     $desc = '';
  }
  $desc =~ tr/\x20-\x7f//cd;	#trim unreadable characters: only leave those between 32 & 127 (hex 20 & 7f)
  return $desc;
}


#=================================================================


# sub check_Nrunning_and_host_allowed([$scriptname,\@allowed_hosts]){
sub check_Nrunning_and_host_allowed(;$\@){

  our ($beamline, $NONINTERACTIVE, $USE_ZENITY_FOR_EXIT);
  my ($script, $scriptname, $hosts_ref, @hosts_allowed, $nhosts, $host);
  my (@processes, $nprocesses, $msg);

  $script     = 'check_Nrunning_and_host_allowed';
  $scriptname = shift(@_);
  $hosts_ref  = shift(@_);

  if (!defined $scriptname) {$scriptname = &makeScriptName();}
  if (!defined $hosts_ref) {
     if    ($beamline eq '23i') {@hosts_allowed = qw(bl1ws2 bl1ws3);}
     elsif ($beamline eq '23o') {@hosts_allowed = qw(bl2ws2 bl2ws3);}
     elsif ($beamline eq '23b') {@hosts_allowed = qw(bl3ws2 bl3ws3);}
     elsif ($beamline eq '23d') {@hosts_allowed = qw(www px0 sergey x-server);}
     else                       {@hosts_allowed = ();}
  } else {
     @hosts_allowed = @$hosts_ref;
  }
  $nhosts = @hosts_allowed;

  if ($nhosts > 0) {
     ### Protection from running on computers which are not allowed:
     $host = &gethost('short');

     if ($hosts_allowed[0] ne '*') {
        if (! grep(/^$host$/, @hosts_allowed)) {
           $msg = '*** '.$script.': cannot run on '.$host."\n"
                . 'Allowed host(s) for beamline '.$beamline.': '."@{hosts_allowed}";
           if (!defined $USE_ZENITY_FOR_EXIT) {$USE_ZENITY_FOR_EXIT=1;}
           &press_key_exit($msg);
        }
     } else {
        print STDOUT '!!! '.$script.': Skipping allowed hosts check: wildcard allowed hosts list.'."\n";
     }
  } else {
     print STDOUT '!!! '.$script.': Skipping allowed hosts check: empty allowed hosts list.'."\n";
  }

#----------------------------------------------------------------
  $nprocesses = &count_Nrunning_gen(\@processes);
  if ($nprocesses > 1) {
     $msg =  '*** '.$scriptname.': '.$nprocesses.' instances of the script are running'."\n"
                   .' (including the one you are trying to start):'."\n".join("\n",@processes);
     if (!defined $USE_ZENITY_FOR_EXIT) {$USE_ZENITY_FOR_EXIT=1;}
     &press_key_exit($msg);
  }
}


#=================================================================


# sub count_Nrunning_gen(;\@processes) {
sub count_Nrunning_gen(;\@) {                                    #both Linux and Windows

  my ($script, $prg, $cmd, $processes_ref, @processes, $nprocesses);
  my ($msg, @words, $nwords, $i);

  $script = 'check_Nrunning_gen';
  $prg    = &makeScriptName();
  $processes_ref = shift(@_);
#----------------------------------------------------------------
### Protection from running multiple instances:
  if ($^O !~ /Win/i) {                                          #Linux
     $cmd = '/bin/ps -eo user:19,pid,cmd '
          . '| /bin/grep -iv "grep\|edit\|nano\|geany\|emacs\|term\|onsole\|bash\|csh\|zenity\|pezca23\|\<defunct\>" '
#         . '| /bin/grep -iv "grep\|\<defunct\>" '
          . '| /bin/grep '.$prg;
  } else {							#Windows
#    $cmd = 'wmic process where name="perl.exe" get ProcessId,CommandLine,ExecutablePath /format:csv '
     $cmd = 'wmic process where name="perl.exe" get ProcessId,CommandLine /format:csv '
          . '| grep -iv "grep\|zenity" '
          . '| grep -i '.$prg;
  }
# print STDOUT $script.': executing:'."\n".$cmd."\n";
  $msg = `$cmd`;
# print STDOUT $script.': result:'."\n".$msg."\n";
  $msg =~ s/\'//g;						#remove all "'"
  @processes = split(/\n/, $msg);
  $nprocesses = @processes;
  if ($^O !~ /Win/i) {                                          #Linux
     for ($i=0; $i<$nprocesses; $i++) {
        @words = split(/\s+/,$processes[$i]);
        $nwords = @words;
        if ($nwords >= 3 && $words[2] !~ /perl$/ && $words[2] !~ /\Q$prg\E/) {
#          print STDOUT 'splice '.$i,"\n";
           splice @processes,$i,1;
           $i--;
           $nprocesses--;
        }
     }
  }
# print STDOUT $script.': nprocesses='.$nprocesses."\n";
  if (defined $processes_ref) {@$processes_ref = @processes;}
  return $nprocesses;
}


#=================================================================


# sub count_Nrunning_lsof(;\@processes) {                       #Linux only, but more accurate
sub count_Nrunning_lsof(;\@) {

  my ($script, $processes_ref, $prg, $lsof, $wc);
  my ($str, $nprocesses, $use_lsof);

  $script  = 'count_Nrunning_lsof';
  $processes_ref = shift(@_);

  $use_lsof = 1;
  if ($^O =~ /Win/i)   {print STDOUT '!!! '.$script.': cannot run on Windows'; $use_lsof = 0;}
  else {
     $lsof = &Which('lsof'); 
     if (! defined $lsof) {print STDOUT '!!! '.$script.': cannot find lsof'; $use_lsof = 0;}
     $wc = &Which('wc'); 
     if (! defined $wc)   {print STDOUT '!!! '.$script.': cannot find wc';   $use_lsof = 0;}
  }

### Taken from:
### https://stackoverflow.com/questions/1715137/what-is-the-best-way-to-ensure-only-one-instance-of-a-bash-script-is-running
### This is not working on CentOS (seemed to work under Ubuntu)!!! 
  $use_lsof = 0;

  if ($use_lsof) {
     $prg = $0;
     $str = $lsof.' -t "'.$prg.'" 2>/dev/null | '.$wc.' -l 2>/dev/null';
#    print STDOUT $script.': str='.$str."\n";
     $nprocesses = `$str`;
     chomp $nprocesses;
#    print STDOUT $script.': nprocesses='.$nprocesses."\n";
     if ($nprocesses > 0 && defined $processes_ref) {&count_Nrunning_gen($processes_ref);}
  } else {
     $nprocesses = &count_Nrunning_gen($processes_ref);
  }
  return $nprocesses;
}


#=================================================================


#sub find_first_index($element,\@array) {
sub find_first_index($\@) {			#returns index of element in array or -1
  my ($element, $arrayRef, $ind, $narr);

  $element  = shift(@_);
  $arrayRef = shift(@_);

  $narr = @$arrayRef;
  $ind = -1;
  do {$ind++;} while ($ind < $narr && @$arrayRef[$ind] ne $element);
  if ($ind < $narr) {return $ind;} else {return -1;}
}


#=================================================================

#sub find_all_index($element,\@array) {
sub find_all_index($\@) {			#returns index of element in array or -1
  my ($element, $arrayRef, @ind, $ilast);

  $element  = shift(@_);
  $arrayRef = shift(@_);

  $ilast = @$arrayRef-1;
  @ind = grep{@$arrayRef[$_]=~/^$element$/} 0..$ilast;
  return @ind;
}


#=================================================================


#sub find_closest_index($val,\@array) {
sub find_closest_index($\@) {			#returns index of element in array closest to val
  my ($val, $arrayRef, $diff, $d, $ind, $narr, $i);

  $val      = shift(@_);
  $arrayRef = shift(@_);

  $narr = @$arrayRef;
  $diff = 1.e+100;
  $ind = 0;
  for ($i=0; $i<$narr; $i++) {
     $d = abs(@$arrayRef[$i] - $val);
     if ($d < $diff) {$diff=$d; $ind=$i;}
  }
  return $ind;
}


#=================================================================

# sub execute_external_script(\@cmd) {		#perl, -W, $script, $arg1, ...
sub execute_external_script(\@) {
  our ($EXTERNAL_PID, $EXTERNAL_PRG);
  my ($cmdref, $script, $pid, $kid, $WNOHANG);
  my ($code, $arg, @subpid, $not_perl);
  my ($cmd, $msg, @processes, $p_, @words, $nw);

  $cmdref = shift(@_);
  undef $EXTERNAL_PID;
  undef $EXTERNAL_PRG;

### WNOHANG is a constant defined in POSIX:
# use POSIX ":sys_wait_h";
### This constant is actually "1". So instead
### of using POSIX we define a variable:
  $WNOHANG = 1;

  $script = &makeScriptName();

  $pid = fork;
  if (!defined $pid) {
     print STDOUT '*** '.$script.': cannot fork to execute '."@$cmdref\n";
     return 1;
  }
  elsif ($pid == 0) {
     ### Client process: execute external program
     print STDOUT $script.':  ==> Calling:'."\n@$cmdref\n";
     exec(@$cmdref);							#this will completely replace the child with called program, but retain PID
  }
  else {								#monitor the script until it exits (also prepare data for signal handler)
     ### We are in parent process
     ### 1. Find the script name
     foreach $arg (@$cmdref) {						#find the name of executed script
        if ($arg =~ /\.pl$/) {
           $EXTERNAL_PRG = $arg;
           $EXTERNAL_PRG =~ s/^.*\///g;
           last;
        }
     }
     if (!defined $EXTERNAL_PRG) {
        $EXTERNAL_PRG = @$cmdref[0];
        $not_perl = 1;
     }
     ### 2. Find the script PID
     if (@$cmdref[0] =~ /perl$/ || 					#direct call of perl script
         @$cmdref[0] =~ /perl\.exe$/i ||                                #direct call of perl script
         defined $not_perl) {	                                        #we are calling not perl
        $EXTERNAL_PID = $pid;						#this is for aborting by signal handler
     } else {								#script is started in terminal, find the PID of the script
        if ($^O !~ /Win/i) {
           @subpid = `/usr/bin/pgrep -P $pid`;				#find all subprocesses for given parent
        } else {
           @subpid = ();
           $cmd = 'wmic process where name="perl.exe" get ProcessId,CommandLine /format:csv '
           . '| grep -iv "grep\|zenity" '
           . '| grep -i '.$EXTERNAL_PRG;
           $msg = `$cmd`;
           $msg =~ s/\'//g;						#remove all "'"
           @processes = split(/\n/, $msg);
           foreach $p_ (@processes) {
              @words = split(',', $p_);
              $nw = @words;
              if ($words[$nw-1] =~ /^\d+$/) {
                 push(@subpid,$words[$nw-1]);
              } else {
                 print $script.': Unexpected PID='.$words[$nw-1].' for process='.$p_."\n";
              }
           }
        }
        $nw = @subpid;
        if ($nw > 0) {
           $EXTERNAL_PID = $subpid[0];					#take the first found only. Is it right?
           if ($nw != 1) {
              print STDOUT '!!!'.$script.': found '.$nw.' child PIDs for script '.$EXTERNAL_PRG."\n";
           }
        } else {
           $EXTERNAL_PID = $pid;
        }
     }
     ### 3. Wait for child to exit
     do {						#wait the script to finish
        select(undef,undef,undef,0.05);
        $kid = waitpid($pid, $WNOHANG);			#wait for particular child
     } while ($kid == 0);				#this should be "0" while process runs
     $code = $? >> 8;
     undef $EXTERNAL_PID;
     undef $EXTERNAL_PRG;
     if ($code) {
        print STDOUT '!!!'.$script.': execution of ['."@$cmdref".'] finished with status='.$code."\n";
     }
     return $code;
  }
}


#=================================================================

#sub abort_external_script (;$pid) {
sub abort_external_script(;$) {
  our ($EXTERNAL_PID, $EXTERNAL_PRG);
  my  ($script, $pid, $kid, $nkilled, $code);
  my  ($wait_, $wait_step, $wait_max, $WNOHANG);

### WNOHANG is a constant defined in POSIX:
# use POSIX ":sys_wait_h";
### This constant is actually "1". So instead
### of using POSIX we define a variable:
  $WNOHANG = 1;

  $pid = shift(@_);   if (defined $pid) {$EXTERNAL_PID = $pid;}

  if (!defined $EXTERNAL_PID) {return 0;}				#nothing to do
  else                        {$pid = $EXTERNAL_PID;}
  if (!defined $EXTERNAL_PRG ) {$EXTERNAL_PRG = 'external script';}

  $script = &makeScriptName();
  printf STDOUT '!!! '.$script.': killing [%s] with PID=%s'."\n", $EXTERNAL_PRG, $EXTERNAL_PID;

  $nkilled = kill('SIGINT',$EXTERNAL_PID);			#sends to main process
# printf STDOUT '!!! '.$script.': killed %d processes.'."\n", $nkilled;

### The script we are trying to kill may have its own signal handler and
### it may caLL other scripts which it may try to kill on exit. Therefore
# we need to wait until it finishes:
  $wait_     = 0;
  $wait_step = 0.05;
  $wait_max  = 30;
  do {								#wait the script to finish
     select(undef,undef,undef,$wait_step);
     $wait_ += $wait_step;
     $kid = waitpid($pid, $WNOHANG);				#wait for particular child
  } while ($kid == 0 && $wait_ < $wait_max);			#this should be "0" while process runs
  $code = $? >> 8;
  undef $EXTERNAL_PID;
  undef $EXTERNAL_PRG;
  if ($code) {
     print STDOUT '!!!!'.$script.': execution of '.$EXTERNAL_PRG.' finished with status='.$code."\n";
  }
  return $code;
}

#===========================================================================

sub zenity_progress($$) {			# $seconds,$msg

### This routine displays countdown window. On Linux it
### uses zenity, but on Windows pipes do not take command
### line argumnents and then this script requires
### Tk and Tk::ProgressBar to display the progress

  our ($ZENITY_VIA_BLUICE, $JBLUICE_SERVER);
  our ($zenity_pid, $mw, $mw_progress_label);
  our ($mw_progress_bar, $mw_progress_abort);
  my ($script, $waitmax, $wait_, $msg, $zenity, @zenargs);
  my ($start_time, $elapsed, $gui, $percent_remaining);
  my (%FORM, $ua, $response, $status);

  $script = 'zenity_progress';

  $waitmax = shift(@_);		if ($waitmax < 1) {return;}
  $msg     = shift(@_);		$msg =~ s/[\n\r]/ /g;

  $start_time = time();

  if (!defined $ZENITY_VIA_BLUICE) {$ZENITY_VIA_BLUICE = 0;}
  if (defined $ENV{BLUICE_SERVER}) {$JBLUICE_SERVER = $ENV{BLUICE_SERVER};}

  if ($ZENITY_VIA_BLUICE ne '0') {                                      #we do not specify JBluIce method, it is auto-selected as http or PV
     if (defined $JBLUICE_SERVER && $JBLUICE_SERVER =~ /^http/) {
        $ZENITY_VIA_BLUICE = 2;
        use if ! defined &LWP::UserAgent, 'LWP::UserAgent';
        $ua = LWP::UserAgent->new(timeout=>'0.2');
        $response = $ua->get($JBLUICE_SERVER.'/check');
        if (! $response->is_success) {                                  #JbluIce server is not responding
           $ZENITY_VIA_BLUICE = 0;                                      #no method to work with JBluIce
        }
     } else {
        $ZENITY_VIA_BLUICE = 0;                                         #no method to work with JBluIce
     }
     if ($ZENITY_VIA_BLUICE eq '0') {print STDOUT '!!! '.$script.': failed reaching BluIce server for MSG:'."\n".$msg."\n\n";}
  }
  $status = 0;
  if ($ZENITY_VIA_BLUICE == 2) {
     $FORM{'text'}     = $msg;
     $FORM{'type'}     ='progress';
     $FORM{'wait_sec'} = $waitmax;
     $ua->timeout($waitmax+5);                                          #wait a bit more than $waitmax
     $response = $ua->post($JBLUICE_SERVER.'/popup',\%FORM);
     if (! $response->is_success) {$status=-1;}                         #something failed
     else {
        ### Server responds "0" if the user clicks
        ### cancel or "1" if user waits to end:
        $status = $response->decoded_content;                           #0=cancel, 1=OK, -1=error.
        $elapsed = time() - $start_time;
        printf STDOUT $script.': real waiting time = %.0f seconds.'."\n",$elapsed;
        if    ($status eq '1') {return 0;}
        elsif ($status eq '0') {return 1;}
        else                   {$status=-1;}
     }
  }

  if ($ZENITY_VIA_BLUICE == 0 || $status == -1) {			#BluIce failed or no BluIce flag
     if ($^O !~ /Win/i) {$zenity = &Which('zenity');}
     else               {$zenity = &Which('zenity.exe'); undef $zenity;} #cygwin zenity works, but requires X11

     if (defined $zenity) {
### zenity --progress --text="bla-bla-aaaaaaaaaaaaaaa ee\rbla" --title="Beamstop-IN" --percentage=99
        $wait_ = sprintf $msg.'. Remaining time %ds', $waitmax;
        @zenargs = ('--progress'
                   ,'--title=Progress'
                   ,'--text="'.$wait_.'"'
                   ,'--percentage=99'
                   );
        if ($^O !~ /Win/i) {
### The canonical version does not suppress Gtk errors, even with '2>/dev/null'.
### Therefore, we must use Open3, which is a bidirectional pipe catching both
### zenity STDOUT and zenity STDERR.
#          push @zenargs, '2>/dev/null';
#          $zenity_pid = open(ZPIPE, '|-', $zenity, @zenargs);
           use IPC::Open3;
           $zenity_pid = open3(\*ZPIPE, \*CHLD_OUT, \*CHLD_ERR, $zenity, @zenargs);
        } else {
           $zenity_pid = open(ZPIPE, '|-', join(' ',$zenity,@zenargs));		#only string works on Windows
        }
        if (defined $zenity_pid) {                                              #if pipe to zenity opened OK
           select ZPIPE; $|=1;							#set unbuffered output
           $SIG{PIPE} = sub {our $zenity_pid;
                             undef $zenity_pid;
                             close(CHLD_OUT);
                             close(CHLD_ERR);
                            };
           do {
              select(undef,undef,undef,0.5);
              $elapsed = time() - $start_time;
              $wait_ = sprintf $msg.'. Remaining time %ds', ($waitmax-$elapsed);
              $percent_remaining = sprintf '%d', 100*($waitmax-$elapsed)/$waitmax;
              if ($percent_remaining > 99) {$percent_remaining=99;}
              if (waitpid($zenity_pid,1) !=0 ) {undef $zenity_pid;}		#no wait
              if (defined $zenity_pid) {
                 print ZPIPE '# '.$wait_."\n".$percent_remaining."\n";
              } else {
                 print STDOUT $wait_."\n";
              }
           } while ($elapsed < $waitmax && defined $zenity_pid);
           if (defined $zenity_pid) {
              kill('KILL',$zenity_pid);
              close(ZPIPE);
              return 0;
           } else {
              $script = &makeScriptName();
              $wait_ = sprintf '%ds', ($waitmax-$elapsed);
              print STDOUT '!!! '.$script.': Progress aborted '.$wait_.' before completion.'."\n\n";
              return 1;
           }
        } else {                                                                #pipe to zenity did not open
           do {
              select(undef,undef,undef,0.5);
              $elapsed = time() - $start_time;
              $wait_ = sprintf $msg.'. Remaining time %ds', ($waitmax-$elapsed);
              print STDOUT $wait_."\n";
           } while ($elapsed < $waitmax);
        }
     } else {                                                                   #there is no zenity app on the system
#-------------------------------------------------#
        if (!defined &function_exists('Exists')) {
           # use Tk;
           # use Tk::ProgressBar;
           if (&try_load_module('Tk'))              {die 'cannot load Tk';}
           if (&try_load_module('Tk::ProgressBar')) {die 'cannot load Tk::ProgressBar';}
        }
#-------------------------------------------------#
        if (! Exists($mw)) {
           $mw = MainWindow->new(-title => 'Progress');
           $mw->geometry( "600x150" );
           $gui = 0;
        } else {
           $gui = 1;
        }
        $zenity_pid = 1;
        $percent_remaining = 100;
        $wait_ = sprintf $msg.'. Remaining time %ds', $waitmax;
        $mw_progress_label = $mw->Label(-text =>$wait_,
                             )->pack(-padx=>10,-pady=>10);

        $mw_progress_bar = $mw->ProgressBar(
                          -width => 30,
                          -from => 0,
                          -to => 100,
                          -blocks => 10,
                          -gap => 0,
                          -colors => [0, 'blue'],
                          -variable => \$percent_remaining,
                          )->pack(-fill => 'x',-padx=>10,-pady=>10);

        $mw_progress_abort = $mw->Button(-text => 'Abort',
                                -command=> sub {
                                   $mw_progress_label->destroy();
                                   $mw_progress_abort->destroy();
                                   $mw_progress_bar->destroy();
                                   if (!$gui) {$mw->destroy();}
                                   undef $zenity_pid;
                                },
                             )->pack(-padx=>10,-pady=>10);
        do {
           select(undef,undef,undef,0.5);
           $elapsed = time() - $start_time;
           $wait_ = sprintf $msg.'. Remaining time %ds', ($waitmax-$elapsed);
           $percent_remaining = sprintf '%d', 100*($waitmax-$elapsed)/$waitmax;
           $mw_progress_label->configure(-text=>$wait_);
           $mw->update;
#          print STDOUT $wait_."\n";
        } while ($elapsed < $waitmax && defined $zenity_pid);
        if (defined $zenity_pid) {
           $mw_progress_label->destroy();
           $mw_progress_abort->destroy();
           $mw_progress_bar->destroy();
           if (!$gui) {$mw->destroy();}
           undef $zenity_pid;
           return 0;
        } else {
           $script = &makeScriptName();
           $wait_ = sprintf '%ds', ($waitmax-$elapsed);
           print STDOUT '!!! '.$script.': Progress aborted '.$wait_.' before completion.'."\n\n";
           return 1;
        }
        no strict "subs";
        if (!$gui) {MainLoop;}
        use strict "subs";
     }
  }
}

#===========================================================================

sub is_float($) {					# returns 1/0 (True/False)
# use Data::Types;					# may conflict with this module (same names)
  my $str = shift(@_);
  if ($str =~ /^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/) {return 1;}
  else                                                            {return 0;}
}

#===========================================================================

sub is_decimal($) {					# returns 1/0 (True/False)
# use Data::Types;					# may conflict with this module (same names)
  my $str = shift(@_);
### Similar to is_float(), but allows fixed-point decimals only (no exponents)
  if ($str =~ /^[+-]?(?:\d+(?:\.\d*)?|\.\d+)$/) {return 1;}
  else                                          {return 0;}
}

#===========================================================================

sub is_int($) {						# returns 1/0 (True/False)
# use Data::Types;					# may conflict with this module (same names)
  my $str = shift(@_);
  if ($str =~ /^[+-]?\d+$/) {return 1;}
  else                      {return 0;}
}

#===========================================================================

sub is_mounted ($;$) {
  use Cwd 'abs_path';
  my ($dir, $ref, @stat1, @stat2);
# my ($dirlink);
  $dir = shift(@_);
  $ref = shift(@_);
  if (!defined $ref) {$ref='/';}
  if (! -e $dir) {return -1;}
  if (! -e $ref) {return -1;}
# $dirlink = readlink($dir);				#this does not if link is relative because it will return relative path and stat will fail
# if (defined $dirlink) {$dir = $dirlink;}
  $dir   = abs_path($dir);
  $ref   = abs_path($ref);
# ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks)=stat($filename);
  @stat1 = stat($ref);
  @stat2 = stat($dir);
  if ($stat2[0] != $stat1[0]) {return 1;}		#device numbers do not coincide
  else                        {return 0;}
}

#===========================================================================

sub is_mountpoint ($) {
  use Cwd 'abs_path';
  my ($dir, @stat1, @stat2);
# my ($dirlink);
  $dir = shift(@_);
  if (! -e $dir) {return -1;}
# $dirlink = readlink($dir);				#this does not if link is relative because it will return relative path and stat will fail
# if (defined $dirlink) {$dir = $dirlink;}
# ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks)=stat($filename);
  $dir   = abs_path($dir);
  @stat1 = stat($dir.'/.');
  @stat2 = stat($dir.'/..');
  if ($stat2[0] != $stat1[0]) {return 1;}		#device numbers do not coincide
  else                        {return 0;}
}

#===========================================================================

#sub dec2bin ($decimal,['r[everse]','t[rim]'|$nbits]) {
sub dec2bin ($;$$) {
  ### http://www.geos.ed.ac.uk/~bmg/software/Perl%20Books/cookbook/ch02_05.htm
  my ($script, $str, $decimal, $reverse, $nbits);
  $script = 'dec2bin';
  $decimal = shift @_;					#this must be integer
  $reverse = shift @_;					#for later decoding to bits
  $nbits   = shift @_;					#16,24,32, or 'trim'
  if (!defined $reverse) {$reverse = 'n';}
  if (!defined $nbits) {
     if ($reverse =~ /^r/i) {$nbits = 32;}		#normally 24
     else                   {$nbits = 'trim';} 
  }
  if (! &is_int($decimal)) {
     print STDOUT '!!! '.$script.': non-integer number='.$decimal."\n";
  }
# $str = sprintf("%032b", $decimal);			#sprintf and unpack do same
  $str = unpack("B32", pack("N", $decimal));		#sprintf and unpack do same
  if ($nbits =~ /^t/i) {
     $str =~ s/^0+(?=\d)//;				#trim leading zeros
  } else {
     if (&is_int($nbits)) {$str = substr($str,length($str)-$nbits,$nbits);}
     else {print STDOUT '*** '.$script.': unexpected nbits='.$nbits."\n";}
  }
  ### Reversing is required if we want to decode bits: normally when we  
  ### print a number as a binary, higher bits are on the left, but for 
  ### decoding we want lower bits to be on the left (the first characters
  ### of the string):
  if ($reverse =~ /^r/i) {$str = reverse $str;}		#reverse the order of bits
  return $str;
}

#===========================================================================

#sub bin2dec ($binstr) {
sub bin2dec ($) {
  my ($binstr, $decimal);
  $binstr = shift @_;
  ### These two methods convert to unsigned int32:
  ### (they seem to do the same thing)
  ### http://www.geos.ed.ac.uk/~bmg/software/Perl%20Books/cookbook/ch02_05.htm
  $decimal = unpack("N", pack("B32", substr("0" x 32 . $binstr, -32)));
# $decimal = oct('0b' . shift);
  ### Convert from unsigned int32 to signed int32
  $decimal = ($decimal & 0x80000000) ? -((~$decimal & 0xffffffff) + 1) : $decimal; 
  return $decimal;
}

#===========================================================================

sub uint16_int16 ($) {
  ### Convert from unsigned int16 to signed int16
  my $v = shift;
  return ($v & 0x8000) ? -((~$v & 0xffff) + 1) : $v; 
}

#===========================================================================

sub uint32_int32 ($) {
  ### Convert from unsigned int32 to signed int32
  my $v = shift;
  return ($v & 0x80000000) ? -((~$v & 0xffffffff) + 1) : $v; 
}

#=================================================================

#sub read_jbluice_config ($key,\$val) {			#returns &status & $val for given $key in JBluIce Config table
sub read_jbluice_config ($\$) {
# use DBI;						#mySQL interface is needed
  our ($mysqlHost, $mysqlUser, $mysqlPass, $mysqlDb);	#defined in $beamline.pl
  my ($key, $valref, $script, $mysqlTbl);
  my ($keyName, $valName);
  my ($dsn, $dbquery, $dbh, $sth, $nrw, $rwref);

  $key    = shift(@_);
  $valref = shift(@_);  
  $script = 'read_jbluice_config';

  $mysqlTbl = 'Config';					#JBluIce Config table
  $dsn      = 'DBI:mysql:host='.$mysqlHost.';database='.$mysqlDb;
  $keyName  = 'ID';
  $valName  = 'Value';

  $dbquery  = 'SELECT * FROM '.$mysqlTbl.' WHERE '.$keyName.'="'.$key.'"';

  $dbh = DBI->connect($dsn, $mysqlUser, $mysqlPass, {PrintError=>0,});
  if (!defined $dbh) {
     print STDOUT '!!! '.$script.': Cannot connect to mySQL server '.$mysqlHost."\n"; 
     return 1;
  }
  $sth = $dbh->prepare($dbquery);
  $nrw = $sth->execute();				#nrecords works for MySQL only. For SQLite result is def/undef only.
  if (!defined $nrw || $nrw < 1) {
     print STDOUT '!!! '.$script.': no mySQL data for '.$keyName.'='.$key."\n"; 
     $sth->finish();
     $dbh->disconnect();
     return 1;
  }
  $rwref = $sth->fetchrow_hashref();
  $sth->finish();
  $dbh->disconnect();
  if (!defined $rwref->{$valName}) {
     print STDOUT '!!! '.$script.': no mySQL "'.$valName.'" field for '.$keyName.'='.$key."\n"; 
     return 1;
  } else {
     $$valref = $rwref->{$valName};
#    print STDOUT $script.': key='.$key.', data="'.$$valref.'"'."\n";
     return 0;
  }
}

#=================================================================

#sub read_jbluice_config_flags ($key,\@flags) {			#returns &status & @flags for given $key in JBluIce Config table
sub read_jbluice_config_flags ($\@) {
# use DBI;						#mySQL interface is needed
  my ($key, $flagsref, $script, $str, $status);

  $key      = shift(@_);
  $flagsref = shift(@_);  
  $script   = 'read_jbluice_config_flags';

  @$flagsref = ();
  $str = '';
  $status = &read_jbluice_config($key,\$str);
  if ($status) {return $status;}

  @$flagsref = split(/\s+/,$str);
  return 0;  
}

#=================================================================

#sub write_jbluice_config ($key,$val) {			#returns $status
sub write_jbluice_config ($\$) {
# use DBI;						#mySQL interface is needed
  our ($mysqlHost, $mysqlUser, $mysqlPass, $mysqlDb);	#defined in $beamline.pl
  my ($key, $val, $script, $mysqlTbl);
  my ($keyName, $valName, $valold, $status);
  my ($dsn, $dbquery, $dbh, $sth, $nrw, $rwref);

  $key = shift(@_);
  $val = shift(@_);  
  $script = 'write_jbluice_config';

  $mysqlTbl = 'Config';					#JBluIce Config table
  $dsn      = 'DBI:mysql:host='.$mysqlHost.';database='.$mysqlDb;
  $keyName  = 'ID';
  $valName  = 'Value';

  $dbquery  = 'SELECT * FROM '.$mysqlTbl.' WHERE '.$keyName.'="'.$key.'"';

  $dbh = DBI->connect($dsn, $mysqlUser, $mysqlPass, {PrintError=>0,});
  if (!defined $dbh) {
     print STDOUT '!!! '.$script.': Cannot connect to mySQL server '.$mysqlHost."\n"; 
     return 1;
  }
  $sth = $dbh->prepare($dbquery);
  $nrw = $sth->execute();				#nrecords works for MySQL only. For SQLite result is def/undef only.
  if (!defined $nrw || $nrw < 1) {
     print STDOUT '!!! '.$script.': no mySQL data for '.$keyName.'='.$key."\n"; 
     $sth->finish();
     $dbh->disconnect();
     return 1;
  }
  $rwref = $sth->fetchrow_hashref();
  if (!defined $rwref->{$valName}) {
     print STDOUT '!!! '.$script.': no mySQL "'.$valName.'" field for '.$keyName.'='.$key."\n"; 
     $status = 1;
  } else {
     $valold = $rwref->{$valName};
#    print STDOUT $script.': key='.$key.', data="'.$valold.'"'."\n";
     if ($val ne $valold) {
        $dbquery  = 'UPDATE '.$mysqlTbl.' SET '.$valName.'="'.$val.'" WHERE '.$keyName.'="'.$key.'"';
        $sth = $dbh->prepare($dbquery);
        $nrw = $sth->execute();
        if (!defined $nrw || $nrw < 1) {
           print STDOUT '!!! '.$script.': failed to update mySQL data for '.$keyName.'='.$key."\n"
                       .' -- '.$sth->errstr."\n"; 
           $status = 1;
        } else {
           $status = 0;
        }
     } else {
        $status = 0;				#old value was the same as requested
     }
  }
  $sth->finish();
  $dbh->disconnect();
  return $status;
}

#=================================================================

# $xcoord = &pt2coord($xpt,\@xarray);
#sub pt2coord($xpt,\@xarray) {                          #returns $xcoord
sub pt2coord($\@) {                                     #returns $xcoord
# This subroutine converts non-integer (interpolated) point of
# array @xarray into respective coordinate.
  my ($xpt, $xarr_ref, $pmax, $pt0, $pt1, $dx, $xcoord);
  $xpt      = shift(@_);
  $xarr_ref = shift(@_);
 
  $pmax = @$xarr_ref - 1;

  $pt0 = int($xpt);
  if    ($pt0 < 0)     {$pt0 = 0;}
  elsif ($pt0 > $pmax) {$pt0 = $pmax;}
 
  $dx = $xpt - $pt0;
 
  if ($dx >= 0) {$pt1 = $pt0+1;}
  else          {$pt1 = $pt0-1;}
  if    ($pt1 < 0)     {$pt1 = 0;}
  elsif ($pt1 > $pmax) {$pt1 = $pmax;}
  
  $dx = abs($dx);
  $xcoord = @$xarr_ref[$pt0]*(1-$dx) + @$xarr_ref[$pt1]*($dx);
  return $xcoord;
}

#===========================================================================

sub gethost(;$) {					# returns full or short hostname
  my ($host, $shortopt);
# my (@parts);

  $shortopt = shift(@_);
  if (!defined $shortopt)     {$shortopt = 0;}
  if ($shortopt =~ /^short/i) {$shortopt = 1;}
  if ($shortopt ne '1')       {$shortopt = 0;}
  $host = $ENV{HOSTNAME};
  if (!defined $host) {$host=`hostname`; $host=~s/\n//g;}
  if (!defined $host) {$host='unknown';}
  $host = lc($host);
  if ($shortopt) {
     $host  =~ s/\..*//;					#remove everything after first "."
#    @parts = split(/\./,$host);
#    $host  = $parts[0];
  }
  return $host;
}

#==========================================================================

#sub sort_hash_array(\@hasharray,$key) {
sub sort_hash_array(\@$) {
  my ($script, $arref, $key, @sorted);
  $script = 'sort_hash_array';
  $arref  = shift(@_);
  $key    = shift(@_);
  if (defined $$arref[0]->{$key}) {
     @sorted = sort { $a->{$key} <=> $b->{$key} } @$arref;
     @$arref = @sorted;
  } else {
     print STDOUT '*** '.$script.': No hash for key='.$key.' in the array. Skipping sorting.'."\n";
  }
}

#==========================================================================

# sub read_entire_file($file,\@lines,[$lock]) {
sub read_entire_file($\@;$) {
# use Fcntl;					                #not used (defines O_RDONLY constant (O_RDONLY=0,O_WRONLY=1,O_RDWR=2))
  my ($script, $file, $linesref, $lock, $buffer, $status);

  $script = 'read_entire_file';
  $file     = shift(@_);
  $linesref = shift(@_);
  $lock     = shift(@_);	if (!defined $lock) {$lock=0;}

  @$linesref = ();
  $status = sysopen(FH, $file, 0);		                #0 is used instead of O_RDONLY defined in "use Fcntl";
  if (!defined $status) {print STDOUT '*** '.$script.': Could not open '.$file."\n"; return 1;}
  if ($lock) {flock(FH, 2);}
  sysread(FH, $buffer, -s FH);
  close(FH);
  @$linesref = split(/\r?\n/,$buffer);
  return 0;
}

#==========================================================================

# sub &open_medm($adl,[$par]) {					#returns $status
sub open_medm($;$) {
  our ($CAQTDM);						#from beamline.pl
  my ($script, $adl, $par, $medmprg, $medmString, $status);
  
  $script = 'open_medm';
  $adl = shift(@_);
  $par = shift(@_);
  if (!defined $adl) {
     print STDOUT '*** '.$script.': undefined ADL file'."\n"; 
     &call_trace();
     return -1;
  }
  if (!defined $CAQTDM)  {$CAQTDM = 0;}
  if ($CAQTDM == 0) {$medmprg = 'medm';}
  else              {$medmprg = 'startDM'; $adl =~ s/\.adl$/\.ui/;}

  if (! -e $adl) {
     print STDOUT '!!! '.$script.': error finding MEDM screen, adl='.$adl."\n\n";
     return -1;
  }
  $medmString = $medmprg.' -noMsg -attach';
  if (defined $par) {
     $medmString .= ' -macro "'.$par.'"';
  }
# $medmString .= ' -x '.$adl.' &>/dev/null &';				#these two formrs are the same (bash, but not sh)
  $medmString .= ' -x '.$adl.' >/dev/null 2>&1 &';			#these two formrs are the same
# print STDOUT 'Opening '.$medmString.'...'."\n";
# $status = system($medmString);
  $status = system('nohup '.$medmString);
  if ($status) {print STDOUT '!!! '.$script.': error='.$status.' starting '.$medmString."\n\n";}
  else         {select(undef,undef,undef,1);}				#Wait for medm messages
  return $status;
}

#==========================================================================

sub call_trace(;$) {
  my ($i, $j, $LUN, $msg);

  $LUN = shift(@_);
  if (!defined $LUN) {$LUN = \*STDOUT;}

  $msg = 'Function calling trace:'."\n";
  $i = 0;
  do {
     $i++;
     $j = ( caller($i) )[3];
     if (defined $j) {$msg .= '  - '.$j."\n";}
     else            {$msg .= '  - main'."\n";}
  } while (defined $j);
  print $LUN $msg;
  return $msg;
}

#=================================================================

#sub import_elements(\@elements) {			#fills elements data and returns status
sub import_elements(\@) {

# This script imports elements data from elements.dat into a hash array.
# The keywords in the array are ('name', 'code', 'z', 'a').

  my ($PEZCA, $script, $elref, $file, @lines, @words);
  my ($status, $str, $nwords, $msg);
        
  $script = 'import_elements';
  $elref  = shift(@_);
  @$elref = ();
 
  if (!defined $ENV{PEZCA}) {
     $msg = '*** '.$script.': Please define the PEZCA environment.'
                  .' It must point to the PERL scripts repository';
     print STDOUT $msg."\n";
     return -1;
  }
  $PEZCA = $ENV{PEZCA};
  $PEZCA =~ s/\\/\//g;
  if ($PEZCA !~ /\/$/) {$PEZCA .='/';}
  $file = $PEZCA.'Data/elements.dat';

  if (! -e $file) {
     $msg = '*** '.$script.':  Cannot find data file ['.$file.'].';
     print STDOUT $msg."\n";
     return -1;
  }
  print STDOUT $script.': Opening elements file: '.$file."\n";
  $status = &read_entire_file($file,\@lines);
  if ($status) {
     $msg = '*** '.$script.': Error reading data file ['.$file.'].';
     print STDOUT $msg."\n";
     return -1;
  }
  foreach $str (@lines) {
     if ($str =~ /^\#/) {next;}
     $str =~ s/\s+$//;					#remove end-of-line spaces
     $str =~ s/^\s+//;					#remove beg-of-line spaces
     if (length($str) == 0) {next;}
     @words = split(/\s+/,$str);
     $nwords = @words;
     if ($nwords != 4) {
        $msg = '*** '.$script.': incorrect record ['.$str.'] in data file ['.$file.'] -- nWords='.$nwords;
        print STDOUT $msg."\n";
        return -1;
     }
     if (! &is_int($words[2])) {
        $msg = '*** '.$script.': incorrect record ['.$str.'] in data file ['.$file.'] -- non-int Z='.$words[2];
        print STDOUT $msg."\n";
        return -1;
     }
     $words[3] =~ s/[\(\)]//g;
     if (! &is_float($words[3])) {
        $msg = '*** '.$script.': incorrect record ['.$str.'] in data file ['.$file.'] -- non-float a='.$words[3];
        print STDOUT $msg."\n";
        return -1;
     }
     push @$elref, {name=>$words[0], code=>$words[1], z=>$words[2], a=>$words[3]};
  }
  @$elref = sort{$a->{'z'} <=> $b->{'z'}} @$elref;	#sort over Z
  return 0;
}
