#!/usr/bin/perl -W
#--------------------------------------------------------------
#      These are Keithley Signal Amplifier related subs
#              for beamline controls Perl scripts
#
#                   Author: Sergey Stepanov
#
# Version-1.0:  2008/03/20 using Pezca
# Version-1.1:  2009/06/08 added check for set values
#--------------------------------------------------------------
# List of functions:
# ^^^^^^^^^^^^^^^^^
#  1.            &keithley_initialize();
#  2. $channel = &keithley_find_amp($label);
#       $label = &keithley_read_label($channel);
#  4.    $gain = &keithley_read_gain($channel);
#  5.   $igain = &keithley_gain_index($gain);
#  6.  $offset = &keithley_read_offset($channel);
#  7.    $time = &keithley_read_time($channel,['ms|label']);			#by default returns time in ms; if 'label', returns time label string
#  8.  $status = &keithley_set_gain($channel,$gain,['nowait']);
#  9.  $status = &keithley_set_offset($channel,$offset,['nowait']);
# 10.  $status = &keithley_set_time($channel,$time,['nowait','ms|label']);	#by default expects time in ms; if 'label', expects time label string
# 11.  $status = &keithley_wait([$startuptime]);
# 12.  $status = &keithley_set_monitors($channel);
# 13. ($status,$keithleyChannel,$scalerInput) = &getSignalChannels($signalName);
# 14.  $status = &getDarks4KeithleyGains($signalName,\$stamp,\$epoch,
#                                        \$keithleyChannel,\$keithleyLabel,\$keithleyOffset,
#                                        \$scalerInput,\@darksE3toE10);
# 15.  $status = &getDarks4KeithleyList(\@signalNames);				#Gets the list of available signals from mySQL
# 16.  ($status,$gain,$intensity,[$msg]) = &keithleyOptimizeGain($signal,$channel,$scaler,$input,
#                                         [$I_optimum,$I_minimum,
#                                         $iterationsmax,$avoidresonance,$checkonly,
#                                         $tell_bluice,$telltag]);		#$telltag is scriptname for tellBluIce
# 17. ($status,[$msg]) = &keithleyOptimizeGains(\@signals, $scaler,
#                                         [\@I_optimum,\@I_minimum,
#                                         $iterationsmax,$avoidresonance,$checkonly,
#                                         $tell_bluice,$telltag]);              #$telltag is scriptname for tellBluIce
#--------------------------------------------------------------
# use Pezca;
# use DBI;			#for getDarks4KeithleyGains

# use warnings::unused;
  use strict;
# no strict 'vars';
# no strict 'subs';
# no strict 'refs';

#=================================================================

# sub keithley_initialize() {
sub keithley_initialize() {
  our ($KEITHEY_NUMCHANNELS, $beamline, $keithley);
  if (!defined $KEITHEY_NUMCHANNELS) {$KEITHEY_NUMCHANNELS = 7;}
  if (!defined $keithley) {$keithley = $beamline.':keithley';}
}

#=================================================================

# sub keithley_find_amp($label) {
sub keithley_find_amp($) {
### This function returns channel corresponding to given label:

  our ($KEITHEY_NUMCHANNELS, $keithley);
  my ($script, $label, $status, $i, $channel, $desc, $kch);

  $script = 'keithley_find_amp';
  $label  = shift(@_);

  &keithley_initialize();			#initialize Keithley PVs (unlesss they are ready);

  $channel = 0;
  for ($i=1; $i<=$KEITHEY_NUMCHANNELS; $i++) {
     $kch = $keithley.$i;
     ($status, $desc) = Pezca::GetString($kch.'.DESC');
     if ($status) {
        &report_exit($script, $status, $kch.'.DESC');
        $desc = '';
     }
     if ($desc =~ /$label/) {
        if ($channel != 0) {
           print STDOUT '*** '.$script.': label="'.$label.'" applies to multiple Keithley channels:'."\n";
           print STDOUT '*** '.$script.': At least '.$keithley.$channel.' and '.$keithley.$i.' match.'."\n";
           $channel = 0;
           return $channel;
        } else {
           $channel = $i;
        }
     }
  }
  return $channel;
}

#=================================================================

# sub keithley_read_label($channel) {
sub keithley_read_label($) {
### This function returns current Keithley label for a given channel:

  our ($KEITHEY_NUMCHANNELS, $keithley);
  my ($script, $channel, $status, $label, $kch);

  $script  = 'keithley_read_label';
  $channel = shift(@_);

  &keithley_initialize();			#initialize Keithley PVs (unlesss they are ready);

  if ($channel < 1 || $channel > $KEITHEY_NUMCHANNELS) {
     &press_key_exit('*** '.$script.': Keithley channel='.$channel
                    .' is not in range [1-'.$KEITHEY_NUMCHANNELS.']');
  }
  $kch = $keithley.$channel;
  ($status, $label) = Pezca::GetString($kch.'.DESC');
  if ($status) {
     &report_exit($script, $status, $kch.'.DESC');
     $label = '';
  }
  return $label;
}

#=================================================================

# sub keithley_read_gain($channel) {
sub keithley_read_gain($) {
### This function returns current Keithley gain for a given channel:

  our ($KEITHEY_NUMCHANNELS, $keithley);
  my ($script, $channel, $status, $gain, $kch);

  $script  = 'keithley_read_gain';
  $channel = shift(@_);

  &keithley_initialize();			#initialize Keithley PVs (unlesss they are ready);

  if ($channel < 1 || $channel > $KEITHEY_NUMCHANNELS) {
     &press_key_exit('*** '.$script.': Keithley channel='.$channel
                    .' is not in range [1-'.$KEITHEY_NUMCHANNELS.']');
  }
  $kch = $keithley.$channel;
  ($status, $gain) = Pezca::GetString($kch.':ActGain');
  if ($status) {
     &report_exit($script, $status, $kch.':ActGain');
     $gain = 0;
  }
  if    ($gain =~ /^UNKNOWN$/) {$gain = 0;}
  elsif (! &is_float($gain) || $gain <= 0) {
     print STDOUT '*** '.$script.': incorrect gain='.$gain."\n";
     $gain = 1E3;
  }
  return $gain;
}

#=================================================================

# sub keithley_gain_index($gain) {		#convert '1E4 -> 4' and etc.
sub keithley_gain_index($) {
### This function converts Keithley gain to respective power index:

  my ($gain, $igain, $script);

  $script  = 'keithley_gain_index';
  $gain = shift(@_);
  if (!defined $gain) {
     print STDOUT '*** '.$script.': undefined input gain'."\n";
     $igain = 3;
     return $igain;
  }
  elsif ($gain =~ /^UNKNOWN$/) {
     print STDOUT '*** '.$script.': input gain is "UNDENED"'."\n";
     $igain = 3;
     return $igain;
  }
  elsif (! &is_float($gain) || $gain <= 0) {
     print STDOUT '*** '.$script.': incorrect input gain='.$gain."\n";
     $igain = 3;
     return $igain;
  }
  if ($gain > 0) {$igain = int(log($gain)/log(10) + 0.5);}
  else           {$igain = 3;}
  if    ($igain > 10) {$igain = 10;}
  elsif ($igain <  3) {$igain = 3;}
  return $igain;
}

#=================================================================

# sub keithley_read_offset($channel) {
sub keithley_read_offset($) {
### This function returns current Keithley offset for a given channel:

  our ($KEITHEY_NUMCHANNELS, $keithley);
  my ($script, $channel, $status, $offset, $kch);

  $script  = 'keithley_read_offset';
  $channel = shift(@_);

  &keithley_initialize();			#initialize Keithley PVs (unlesss they are ready);

  if ($channel < 1 || $channel > $KEITHEY_NUMCHANNELS) {
     &press_key_exit('*** '.$script.': Keithley channel='.$channel
                    .' is not in range [1-'.$KEITHEY_NUMCHANNELS.']');
  }
  $kch = $keithley.$channel;
  ($status, $offset) = Pezca::GetDouble($kch.':ActOffset');
  if ($status) {
     &report_exit($script, $status, $kch.':ActOffset');
     $offset = 0;
  }
  return $offset;
}

#=================================================================

# sub keithley_read_time($channel,['ms|label']) {			#by default returns time in ms; if 'label', returns time label string
sub keithley_read_time($;$) {
### This function returns current Keithley time for a given channel:

  our ($KEITHEY_NUMCHANNELS, $keithley);
  my ($script, $channel, $format, $status, $ktime, $kch);

  $script  = 'keithley_read_time';
  $channel = shift(@_);
  $format  = shift(@_);
  if (!defined $format || $format ne 'label') {$format = 'ms';}

  &keithley_initialize();			#initialize Keithley PVs (unlesss they are ready);

  if ($channel < 1 || $channel > $KEITHEY_NUMCHANNELS) {
     &press_key_exit('*** '.$script.': Keithley channel='.$channel
                    .' is not in range [1-'.$KEITHEY_NUMCHANNELS.']');
  }
  $kch = $keithley.$channel;
  ($status, $ktime) = Pezca::GetString($kch.':ActTime');
  if ($status) {
     &report_exit($script, $status, $kch.':ActTime');
     $ktime = 0;
  }
# print STDOUT 'ktime='.$ktime."\n";
  if ($format eq 'ms') {
     $ktime =~ s/ //g;
### Recaclulate into milliseconds:
     if    ($ktime =~ /us/) {$ktime =~ s/us//g;  $ktime = $ktime/1000.;}
     elsif ($ktime =~ /ms/) {$ktime =~ s/ms//g;}
     elsif ($ktime =~ /^UNKNOWN$/) {$ktime = 0;}
  }
  return $ktime;
}

#=================================================================

# sub keithley_set_gain($channel,$gain,['nowait']) {
sub keithley_set_gain($$;$) {
### This function sets Keithley gain for a given channel:

  our ($KEITHEY_NUMCHANNELS, $keithley);
  my ($script, $channel, $gain, $gain_rb, $status, $i, $kch);
  my ($wait_max, $wait_step, $wait_, $nowait);

  $script  = 'keithley_set_gain';
  $channel = shift(@_);
  $gain    = shift(@_);
  $nowait  = shift(@_);     if (!defined $nowait) {$nowait=0;}
  if ($nowait =~ /^nowait$/i || $nowait =~ /^1$/) {$nowait=1;}
  else                                            {$nowait=0;}

  &keithley_wait(0);		#initialize Keithley PVs (unlesss they are ready) & wait immediately(0) while Keithley finishes previous operation

  if ($channel < 1 || $channel > $KEITHEY_NUMCHANNELS) {
     &press_key_exit('*** '.$script.': Keithley channel='.$channel
                    .' is not in range [1-'.$KEITHEY_NUMCHANNELS.']');
  }
  $kch = $keithley.$channel;
  if ($gain < 1E3)  {$gain = 1E3;}
  if ($gain > 1E10) {$gain = 1E10;}

  $i = int(log($gain)/log(10) + 0.5) - 3;
  if ($i<0) {$i=0;}
  if ($i>7) {$i=7;}
  if ($nowait) {$status = Pezca::PutDoubleOldCa($kch.':RqsGain',$i);}
  else         {$status = Pezca::PutDouble     ($kch.':RqsGain',$i);}
  if ($status) {
     &report_exit($script, $status, $kch.':RqsGain');
     return $status;
  }

  if ($nowait) {return 0;}

# $status = Pezca::PutDoubleOldCa($keithley.':Set',1);			#this should be set per RqsVal by the server
# if ($status) {&report_only($script, $status, $keithley.':Set'); $status=0;}
  $status = &keithley_wait();
  if ($status == 0) {
     $wait_max   = 10;
     $wait_step  = 0.05;
     $wait_ = 0;
     $status = &addCaMonitor($kch.':ActGain');
     &report_only($script, $status, $kch.':ActGain');
     do {
        select(undef,undef,undef,$wait_step);				#sleep($wait_step);
        $wait_ += $wait_step;
        ($status, $gain_rb) = Pezca::GetDouble($kch.':ActGain');
        if ($status) {
           &report_exit($script, $status, $kch.':ActGain');
           $gain_rb = 0;
        }
     } while ($gain_rb != $i && $wait_ < $wait_max);
     if ($gain_rb != $i) {
        $gain_rb = 10**($gain_rb+3);
        printf STDOUT '*** '.$script.': Keithley "'.$kch.'" gain was not set:'
                            .' Rqs=%2.0e Act=2.0e timeout=%2.0f'."\n",
                            $gain, $gain_rb, $wait_;
        $status = -1;
     }
  }

  return $status;
}

#=================================================================

# sub keithley_set_offset($channel,$offset,['nowait']) {
sub keithley_set_offset($$;$) {
### This function sets Keithley offset for a given channel:

  our ($KEITHEY_NUMCHANNELS, $keithley);
  my ($script, $channel, $offset, $offset_rb, $status, $kch);
  my ($wait_max, $wait_step, $wait_, $nowait);

  $script  = 'keithley_set_offset';
  $channel = shift(@_);
  $offset  = shift(@_);
  $nowait  = shift(@_);     if (!defined $nowait) {$nowait=0;}
  if ($nowait =~ /^nowait$/i || $nowait =~ /^1$/) {$nowait=1;}
  else                                            {$nowait=0;}

  &keithley_wait(0);		#initialize Keithley PVs (unlesss they are ready) & wait immediately(0) while Keithley finishes previous operation

  if ($channel < 1 || $channel > $KEITHEY_NUMCHANNELS) {
     &press_key_exit('*** '.$script.': Keithley channel='.$channel
                    .' is not in range [1-'.$KEITHEY_NUMCHANNELS.']');
  }
  $kch = $keithley.$channel;
  if ($nowait) {$status = Pezca::PutDoubleOldCa($kch.':RqsOffset',$offset);}
  else         {$status = Pezca::PutDouble     ($kch.':RqsOffset',$offset);}
  if ($status) {
     &report_exit($script, $status, $kch.':RqsOffset');
     return $status;
  }

  if ($nowait) {return 0;}

# $status = Pezca::PutDoubleOldCa($keithley.':Set',1);		#this should be set per RqsVal by the server
# if ($status) {&report_only($script, $status, $keithley.':Set'); $status=0;}
  $status = &keithley_wait();
  if ($status == 0) {
     $wait_max   = 10;
     $wait_step  = 0.05;
     $wait_ = 0;
     $status = &addCaMonitor($kch.':ActOffset');
     &report_only($script, $status, $kch.':ActOffset');
     do {
        select(undef,undef,undef,$wait_step);			#sleep($wait_step);
        $wait_ += $wait_step;
        ($status, $offset_rb) = Pezca::GetDouble($kch.':ActOffset');
        if ($status) {
           &report_exit($script, $status, $kch.':ActOffset');
           $offset_rb = 0;
        }
     } while (abs($offset_rb - $offset) > 0.05*(abs($offset_rb) + abs($offset)) && $wait_ < $wait_max);
     if (abs($offset_rb - $offset) > 0.05*(abs($offset_rb) + abs($offset))) {
        print STDOUT '*** '.$script.': Keithley "'.$kch.'" offset was not set:'
                                   .' Rqs='.$offset.', Act='.$offset_rb."\n";
        $status = -1;
     }
  }

  return $status;
}

#=================================================================

# sub keithley_set_time($channel,$time,['nowait','ms|label']) {		#by default expects time in ms; if 'label', expects time label string
sub keithley_set_time($$;$) {
### This function sets Keithley time for a given channel:

  our ($KEITHEY_NUMCHANNELS, $keithley);
  my ($script, $channel, $ktime, $nowait, $format);
  my ($ktime_rb, @timesarray, $ntimes);
  my ($status, $i, $j, $diff, $dd, $kch);
  my ($wait_max, $wait_step, $wait_);

  $script  = 'keithley_set_time';
  $channel = shift(@_);
  $ktime   = shift(@_);
  $nowait  = shift(@_);     
  $format  = shift(@_);
  if (!defined $nowait) {$nowait=0;}
  if ($nowait =~ /^nowait$/i || $nowait =~ /^1$/) {$nowait=1;}
  else                                            {$nowait=0;}
  if (!defined $format || $format ne 'label') {$format = 'ms';}

  @timesarray = (0.01, 0.03, 0.1, 0.3, 1, 3, 10, 30, 100, 300);
  $ntimes = @timesarray;

  &keithley_wait(0);		#initialize Keithley PVs (unlesss they are ready) & wait immediately(0) while Keithley finishes previous operation

  if ($channel < 1 || $channel > $KEITHEY_NUMCHANNELS) {
     &press_key_exit('*** '.$script.': Keithley channel='.$channel
                    .' is not in range [1-'.$KEITHEY_NUMCHANNELS.']');
  }
  $kch = $keithley.$channel;

  if ($format eq 'ms') {
     if ($ktime < $timesarray[0])         {$ktime = $timesarray[0];}
     if ($ktime > $timesarray[$ntimes-1]) {$ktime = $timesarray[$ntimes-1];}
     $diff = 1E10;
     $j = -1;
     for ($i=0; $i<$ntimes; $i++) {
        $dd = abs($ktime-$timesarray[$i]);
        if ($dd < $diff) {$diff=$dd; $j=$i;}
     }
     if ($nowait) {$status = Pezca::PutDoubleOldCa($kch.':RqsTime',$j);}
     else         {$status = Pezca::PutDouble     ($kch.':RqsTime',$j);}
  } else {
     if ($nowait) {$status = Pezca::PutStringOldCa($kch.':RqsTime',$ktime);}
     else         {$status = Pezca::PutString     ($kch.':RqsTime',$ktime);}
  }
  if ($status) {
     &report_exit($script, $status, $kch.':RqsTime');
     return $status;
  }

  if ($nowait) {return 0;}

# $status = Pezca::PutDoubleOldCa($keithley.':Set',1);		#this should be set per RqsVal by the server
# if ($status) {&report_only($script, $status, $keithley.':Set'); $status=0;}
  $status = &keithley_wait();
  if ($status == 0) {
     $wait_max   = 10;
     $wait_step  = 0.05;
     $wait_ = 0;
     if ($format eq 'ms') {
        $status = &addCaMonitor($kch.':ActTime');
        &report_only($script, $status, $kch.':ActTime');
        do {
           select(undef,undef,undef,$wait_step);			#sleep($wait_step);
           $wait_ += $wait_step;
           ($status, $ktime_rb) = Pezca::GetDouble($kch.':ActTime');
           if ($status) {
              &report_exit($script, $status, $kch.':ActTime');
              $ktime_rb = 0;
           }
        } while ($ktime_rb != $j && $wait_ < $wait_max);
        if ($ktime_rb != $j) {
           if ($ktime_rb >= 0 && $ktime_rb <= $ntimes-1) {$ktime_rb = $timesarray[$ktime_rb];}
           print STDOUT '*** '.$script.': Keithley "'.$kch.'" time was not set:'
                                      .' Rqs='.$ktime.', Act='.$ktime_rb."\n";
           $status = -1;
        }
     } else {
        $status = &addCaMonitor($kch.':ActTime','string');
        &report_only($script, $status, $kch.':ActTime');
        do {
           select(undef,undef,undef,$wait_step);			#sleep($wait_step);
           $wait_ += $wait_step;
           ($status, $ktime_rb) = Pezca::GetString($kch.':ActTime');
           if ($status) {
              &report_exit($script, $status, $kch.':ActTime');
              $ktime_rb = 0;
           }
        } while ($ktime_rb != $ktime && $wait_ < $wait_max);
        if ($ktime_rb != $ktime) {
           print STDOUT '*** '.$script.': Keithley "'.$kch.'" time was not set:'
                                      .' Rqs='.$ktime.', Act='.$ktime_rb."\n";
           $status = -1;
        }
     }
  }
  return $status;
}

#=================================================================

# sub keithley_wait([$wait_start]) {
sub keithley_wait(;$) {
### This function waits for Keithley to set new values:

  our ($KEITHEY_NUMCHANNELS, $keithley);
  my ($script, $wait_max, $wait_start, $wait_step, $wait_);
  my ($status, $keithley_status);

  $script     = 'keithley_wait';
  $wait_start = shift(@_);
  if (!defined $wait_start) {$wait_start = 0.20;}
  $wait_max   = 10;
  $wait_step  = 0.05;
  $wait_ = 0;

  &keithley_initialize();			#initialize Keithley PVs (unlesss they are ready);

  $status = &addCaMonitor($keithley.':Set');
  &report_only($script, $status, $keithley.':Set');

  if ($wait_start > 0) {
     select(undef,undef,undef,$wait_start);			#sleep($wait_start);
     $wait_ += $wait_start;
  }

  do {
     ($status,$keithley_status) = Pezca::GetDouble($keithley.':Set');
     if ($keithley_status != 1) {
        select(undef,undef,undef,$wait_step);			#sleep($wait_step);
        $wait_ += $wait_step;
     }
  } while ($keithley_status == 1 && $wait_ < $wait_max);

  if ($wait_ >= $wait_max) {
    print STDOUT '!!! '.$script.': timeout while waiting Keithley "'
                               .$keithley.'" operation to complete'."\n";
    return -1;
  } else {
    return 0;
  }
}

#=================================================================

# sub keithley_set_monitors($channel) {
sub keithley_set_monitors($) {
### This function sets Keithley gain,offset,time monitors for a given channel:

  our ($KEITHEY_NUMCHANNELS, $keithley);
  my ($script, $channel, $status, $status_overall, $kch);

  $script  = 'keithley_set_monitors';
  $channel = shift(@_);

  &keithley_initialize();			#initialize Keithley PVs (unlesss they are ready);

  if ($channel < 1 || $channel > $KEITHEY_NUMCHANNELS) {
     &press_key_exit('*** '.$script.': Keithley channel='.$channel
                    .' is not in range [1-'.$KEITHEY_NUMCHANNELS.']');
  }
  $kch = $keithley.$channel;
  $status_overall = 0;

  $status = &addCaMonitor($kch.':ActGain', 'string');
  &report_only($script, $status, $kch.':ActGain');
  $status_overall += $status;

  $status = &addCaMonitor($kch.':ActTime', 'string');
  &report_only($script, $status, $kch.':ActTime');
  $status_overall += $status;

  $status = &addCaMonitor($kch.':ActOffset');
  &report_only($script, $status, $kch.':ActOffset');
  $status_overall += $status;

  return $status_overall;
}

#=================================================================

# ($status,$keithleyChannel,$scalerInput) = &getSignalChannels($signalName);
sub getSignalChannels($) {

  our ($mysqlHost, $mysqlUser, $mysqlPass, $mysqlDb, $report);
  my ($script, $signalName, $kchannel, $input);
  my ($mysqlTable, $dsn, $dbh, $sth, $nrecords, $hashref);

  $script = 'getSignalChannels';
  $signalName  = shift(@_);

  if (!defined $report) {$report = 0;}

  $mysqlTable = 'struckDarks4KeithleyGains';
  $dsn = 'DBI:mysql:host='.$mysqlHost.';database='.$mysqlDb;
  if ($report) {
     print STDOUT $script.': connecting to mySQL database='.$mysqlDb
                         .' on host='.$mysqlHost."\n";
  }
  $dbh = DBI->connect($dsn,$mysqlUser,$mysqlPass,{PrintError=>0,});
  if (!defined $dbh) {
     print STDOUT '*** '.$script.': cannot connect to mySQL DB='.$mysqlDb
                        .' on '.$mysqlHost."\n";
     return (-1,0,0);
  }
  $sth = $dbh->prepare('SELECT * FROM '.$mysqlTable.' WHERE signalName=?');
  if (!defined $sth) {
     print STDOUT '*** '.$script.': error on mySQL table='.$mysqlTable
                        .' -- '.$dbh->errstr."\n";
     $dbh->disconnect();
     return (-1,0,0);
  }
  $nrecords = $sth->execute($signalName);		#nrecords works for MySQL only. For SQLite result is def/undef only.
  if (!defined $nrecords) {
     print STDOUT '*** '.$script.': Could not execute SELECT for signalName="'
                        .$signalName.'" in table='.$mysqlTable.' -- '.$sth->errstr."\n";
     $sth->finish();
     $dbh->disconnect();
     return (-1,0,0);
  }
  elsif ($nrecords > 1) {
     print STDOUT '*** '.$script.': found multiple='.$nrecords
                        .' records for signalName="'.$signalName.'"'."\n";
     $sth->finish();
     $dbh->disconnect();
     return (-1,0,0);
  }

### Method-1:
  $hashref = $sth->fetchrow_hashref();
  $kchannel = $hashref->{'keithleyChannel'};
  $input    = $hashref->{'struckInput'};
  if ($report) {print STDOUT $script.': Disconnecting from mySQL table='
                                    .$mysqlTable."\n";
  }
  $sth->finish();
  $dbh->disconnect();
  return (0,$kchannel,$input);

### Method=2:
# @val = $sth->fetchrow_array();
# if ($report) {print STDOUT $script.': Disconnecting from mySQL table='
#                                   .$mysqlTable."\n";
# }
# $sth->finish();
# $dbh->disconnect();
# $nval = @val;
# if ($nval != 14) {
#    print STDOUT '*** '.$script.': unexpected number of elements='.$nval
#                .' in the record for signalName="'.$signalName.'"'."\n";
#    return (-1,0,0);
# }
# $kchannel = $val[1];
# $input = $val[2];
# return (0,$kchannel,$input);
}

#=================================================================

# $status = &getDarks4KeithleyGains($signalName,\$stamp,\$epoch,
#                                   \$keithleyChannel,\$keithleyLabel,\$keithleyOffset,
#                                   \$scalerInput,\@darksE3toE10);
sub getDarks4KeithleyGains($\$\$\$\$\$\$\@) {

  our ($mysqlHost, $mysqlUser, $mysqlPass, $mysqlDb, $report);
  my ($script, $signalName, $stampRef, $epochRef);
  my ($kchannelRef, $klabelRef, $koffsetRef, $inputRef, $darksRef);
  my ($mysqlTable, $dsn, $dbh, $sth, $nrecords, @val, $nval, @timestamp);

  $script = 'getDarks4KeithleyGains';
  $signalName  = shift(@_);
  $stampRef    = shift(@_);
  $epochRef    = shift(@_);
  $kchannelRef = shift(@_);
  $klabelRef   = shift(@_);
  $koffsetRef  = shift(@_);
  $inputRef    = shift(@_);
  $darksRef    = shift(@_);

  if (!defined $report) {$report = 0;}

  $mysqlTable = 'struckDarks4KeithleyGains';
  $dsn = 'DBI:mysql:host='.$mysqlHost.';database='.$mysqlDb;
  if ($report) {
     print STDOUT $script.': connecting to mySQL database='.$mysqlDb
                         .' on host='.$mysqlHost."\n";
  }
  $dbh = DBI->connect($dsn,$mysqlUser,$mysqlPass,{PrintError=>0,});
  if (!defined $dbh) {
     print STDOUT '*** '.$script.': cannot connect to mySQL DB='.$mysqlDb
                        .' on '.$mysqlHost."\n";
     return -1;
  }
  $sth = $dbh->prepare('SELECT * FROM '.$mysqlTable.' WHERE signalName=?');
  if (!defined $sth) {
     print STDOUT '*** '.$script.': error on mySQL table='.$mysqlTable
                        .' -- '.$dbh->errstr."\n";
     $dbh->disconnect();
     return -1;
  }
  $nrecords = $sth->execute($signalName);		#nrecords works for MySQL only. For SQLite result is def/undef only.
  if (!defined $nrecords) {
     print STDOUT '*** '.$script.': Could not execute SELECT for signalName="'
                        .$signalName.'" in table='.$mysqlTable.' -- '.$sth->errstr."\n";
     $sth->finish();
     $dbh->disconnect();
     return -1;
  }
  elsif ($nrecords > 1) {
     print STDOUT '*** '.$script.': found multiple='.$nrecords
                        .' records for signalName="'.$signalName.'"'."\n";
     $sth->finish();
     $dbh->disconnect();
     return -1;
  }
  @val = $sth->fetchrow_array();
  if ($report) {print STDOUT $script.': Disconnecting from mySQL table='
                                    .$mysqlTable."\n";
  }
  $sth->finish();
  $dbh->disconnect();
  $nval = @val;
  if ($nval != 14) {
     print STDOUT '*** '.$script.': unexpected number of elements='.$nval
                 .' in the record for signalName="'.$signalName.'"'."\n";
     return -1;
  }
  $$kchannelRef = $val[1];
  $$inputRef    = $val[2];
  $$klabelRef   = $val[3];
  $$koffsetRef  = $val[4];
  @timestamp = split ' ',$val[5];
  $nval = @timestamp;
  if ($nval != 3) {
     print STDOUT '*** '.$script.': unexpected timestamp='.$val[5]
                 .' in the record for signalName="'.$signalName.'"'."\n";
     $$stampRef    = '';
     $$epochRef    = 0;
  } else {
     $$stampRef    = $timestamp[0].' '.$timestamp[1];
     $$epochRef    = $timestamp[2];
  }
  @$darksRef    = ($val[6],$val[7],$val[8],$val[9],$val[10],$val[11],$val[12],$val[13]);
#                   1E3     1E4     1E5     1E6     1E7      1E8      1E9     1E10
  return 0;
}

#=================================================================

# $status = &getDarks4KeithleyList(\@signalNames);
sub getDarks4KeithleyList(\@) {

# Gets the list of available signals from mySQL

  our ($mysqlHost, $mysqlUser, $mysqlPass, $mysqlDb, $report);
  my ($script, $signalsRef);
  my ($mysqlTable, $dsn, $dbh, $sth, $nrecords, $hashref);

  $script = 'getDarks4KeithleyList';
  $signalsRef  = shift(@_);

  if (!defined $report) {$report = 0;}

  $mysqlTable = 'struckDarks4KeithleyGains';
  $dsn = 'DBI:mysql:host='.$mysqlHost.';database='.$mysqlDb;
  if ($report) {
     print STDOUT $script.': connecting to mySQL database='
                         .$mysqlDb.' on host='.$mysqlHost."\n";
  }
  $dbh = DBI->connect($dsn,$mysqlUser,$mysqlPass,{PrintError=>0,});
  if (!defined $dbh) {
     print STDOUT '*** '.$script.': cannot connect to mySQL DB='
                        .$mysqlDb.' on '.$mysqlHost."\n";
     return -1;
  }
  $sth = $dbh->prepare('SELECT * FROM '.$mysqlTable);
  if (!defined $sth) {
     print STDOUT '*** '.$script.': error on mySQL table='
                        .$mysqlTable.' -- '.$dbh->errstr."\n";
     $dbh->disconnect();
     return -1;
  }
  $nrecords = $sth->execute();				#nrecords works for MySQL only. For SQLite result is def/undef only.
  if (!defined $nrecords) {
     print STDOUT '*** '.$script.': Could not execute "SELECT *" on table='
                                .$mysqlTable.' -- '.$sth->errstr."\n";
     $sth->finish();
     $dbh->disconnect();
     return -1;
  }
  while ($hashref = $sth->fetchrow_hashref()) {
    push @$signalsRef, $hashref->{'signalName'};
  }
  if ($report) {print STDOUT $script.': Disconnecting from mySQL table='
                                    .$mysqlTable."\n";
  }
  $sth->finish();
  $dbh->disconnect();
  return 0;
}

#=================================================================

# ($status,$gain,$intensity,[$msg]) = &keithleyOptimizeGain($signal,$channel,$scaler,$input,
#                                         [$I_optimum,$I_minimum]
#                                         [$iterationsmax,$avoidresonance,$checkonly],
#                                         [$tell_bluice,$telltag]);
sub keithleyOptimizeGain($$$$;$$$$$$$) {
  our ($beamline, $shutter, $shutter_readback, $report);
  my ($script, $status, $signal, $kchannel, $scaler, $input, $I_optimum, $I_minimum);
  my ($iterationsmax, $avoidresonance, $checkonly, $tell_bluice, $telltag);
  my ($cnt_time, @I0_, $I0, $hasoverfill, $kgain, $kgain_ini, $igain);
  my ($iteration, $msg, $igain_max, $igain_min, $factor);

  $script = 'keithleyOptimizeGain';
  $signal         = shift(@_);  if (!defined $signal)         {$signal = 'Intensity';}
  $kchannel       = shift(@_);
  $scaler         = shift(@_);
  $input          = shift(@_);
  $I_optimum      = shift(@_);  if (!defined $I_optimum)      {$I_optimum = 4.0 * 1.E5;}
  $I_minimum      = shift(@_);  if (!defined $I_minimum)      {$I_minimum = $I_optimum/1000.;}
  $iterationsmax  = shift(@_);	if (!defined $iterationsmax)  {$iterationsmax=20;}
  $avoidresonance = shift(@_);  if (!defined $avoidresonance) {$avoidresonance=1;}
  $checkonly      = shift(@_);  if (!defined $checkonly)      {$checkonly=0;}
  $tell_bluice    = shift(@_);  if (!defined $tell_bluice)    {$tell_bluice=1;}
  $telltag        = shift(@_);				#script name for reporting to JBluIce

  if (!defined $report) {$report=0;}
  $cnt_time  = 1;					#scaler counting time
  $factor    = 0.1;					# intensity must be placed between $I_optimum and $factor*$I_optimum
  $status    = 0;
  $msg       = '';
  $iteration = 0;
  $igain_max = 9;
  $igain_min = 3;

  if    (length($signal) == 0) 			{$signal = 'Intensity';}
  if    ($avoidresonance =~ /^avoid/i   ||
         $avoidresonance =~ /^true/i    ||
         $avoidresonance =~ /^yes/i     ||
         $avoidresonance =~ /^1/i)		{$avoidresonance=1;}
  elsif ($avoidresonance =~ /^skip/i    ||
         $avoidresonance =~ /^ignore/i  ||
         $avoidresonance =~ /^false/i   ||
         $avoidresonance =~ /^no/i      ||
         $avoidresonance =~ /^0$/i)		{$avoidresonance=0;}
  else {
     print STDOUT '!!! '.$script.': unrecognized avoidresonance parameter='
                        .$avoidresonance.' -- Ignoring!'."\n";
     $avoidresonance = 0;
  }
  if    ($checkonly =~ /^check/i   ||
         $checkonly =~ /^true/i    ||
         $checkonly =~ /^yes/i     ||
         $checkonly =~ /^1/i)		{$checkonly=1;}
  elsif ($checkonly =~ /^skip/i    ||
         $checkonly =~ /^false/i   ||
         $checkonly =~ /^no/i      ||
         $checkonly =~ /^0$/i)		{$checkonly=0;}
  else {
     print STDOUT '!!! '.$script.': unrecognized checkonly parameter='
                        .$checkonly.' -- Ignoring!'."\n";
     $checkonly = 0;
  }
  if    ($tell_bluice =~ /^tell/i   ||
         $tell_bluice =~ /^true/i   ||
         $tell_bluice =~ /^yes/i    ||
         $tell_bluice =~ /^1/i)		{$tell_bluice=1;}
  elsif ($tell_bluice =~ /^skip/i   ||
         $tell_bluice =~ /^false/i  ||
         $tell_bluice =~ /^no/i     ||
         $tell_bluice =~ /^0$/i)	{$tell_bluice=0;}
  else {
     print STDOUT '!!! '.$script.': unrecognized tell_bluice parameter='
                        .$tell_bluice.' -- Ignoring!'."\n";
     $tell_bluice = 0;
  }
  if ($tell_bluice) {if (!defined $telltag) {$telltag = &makeScriptName();}}

  if ($scaler  !~ /^${beamline}:/)    {$scaler    = $beamline.':'.$scaler;}
  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }

### Read initial (current) Keithley gain:
  $kgain_ini = &keithley_read_gain($kchannel);
  $msg =  sprintf ': Starting optimization of gain on Keithley channel='
                  .$kchannel.' (gain_ini=%1.0e)', $kgain_ini;
  if ($tell_bluice) {&tellBluIce($telltag.$msg);}
  if ($report)      {print STDOUT $script.$msg."\n";}

### Set scaler:
  if ($report) {print STDOUT $script.': Setting up scaler "'.$scaler.'" count time='.$cnt_time."s\n";}
  &setJoerger($scaler,$cnt_time);

### Open endstation shutter:
  $status = &getEndStationShutter($beamline);
  if ($status == 0) {					#if closed
     if ($tell_bluice) {&tellBluIce($telltag.': opening endstation shutter');}
     $status = &openEndStationShutter($beamline);
     if ($status) {
        if ($tell_bluice) {&tellBluIce($telltag.': Station shutter failed','error');}
        $msg = '*** '.$script.': failed to open endstation shutter'."\n";
        print STDOUT $msg;
        $status = -1;
        $I0 = 0;
        return ($status, $kgain_ini, $I0, $msg);
     }
  } else {
     $status = 0;					#all is OK, enstation shutter is open
  }

### Measure I0:
  if ($report) {print STDOUT $script.': '.$iteration.'. Measuring initial "'
                           .$signal.'" rate (scaler input='.$input.')'."\n";}
  if (defined $shutter) {
     $status = &openShutter ($shutter,$shutter_readback);
     if ($status) {
        if ($tell_bluice) {&tellBluIce($telltag.': Opening fast shutter failed','error');}
        $msg = '*** '.$script.': failed to open fast shutter'."\n";
        print STDOUT $msg;
        $status = -1;
        $I0 = 0;
        return ($status, $kgain_ini, $I0, $msg);
     }
  }
  @I0_ = &runJoerger ($scaler, $cnt_time, 1, ($input));
  if (defined $shutter) {
     $status = &closeShutter ($shutter,$shutter_readback);
     if ($status) {
        if ($tell_bluice) {&tellBluIce($telltag.': Closing fast shutter failed','error');}
        $msg = '*** '.$script.': failed to close fast shutter'."\n";
        print STDOUT $msg;
     }
  }
  $I0  = $I0_[0];
  $msg = sprintf ': initial '.$signal.' rate=%1.0f', $I0;
  if ($tell_bluice) {&tellBluIce($telltag.$msg);}
  $msg = $script.$msg."\n";
  if ($report) {print STDOUT $msg;}
  $iteration++;

  $status = 0;
  $kgain  = $kgain_ini;
  if ($I0 >= $factor*$I_optimum && $I0 <= $I_optimum) { #-------intensity is OK---------+
     ### We are here if initial intensity is OK:                                        #
     $msg = 'Keithley gain adjustment is not needed.'."\n";                             #
     if ($report) {print STDOUT $script.': '.$signal                                    #
                               .' rate is within normal range. '.$msg;}                 #
  } else { #----------------------------------------------------intensity is not OK-----+
     if ($I0 < $factor*$I_optimum) {                                                    #
        $msg = sprintf ' rate=%1.0f is less than '.$factor.' of optimal=%1.0f'."\n",    #
                                                                $I0, $I_optimum;        #
     }                                                                                  #
     elsif ($I0 > $I_optimum) {                                                         #
        $msg = sprintf ' rate=%1.0f exceeds optimal=%1.0f'."\n",$I0, $I_optimum;        #
     }                                                                                  #
     if ($report) {print STDOUT '!!! '.$script.': '.$signal.$msg."\n";}                 #
     $msg = ': Keithley gain adjustment for '.$signal.' needed';                        #
     if ($tell_bluice) {&tellBluIce($telltag.$msg,'warn');}                             #
     $msg = '!!! '.$script.$msg."\n";                                                   #
     if ($report) {print STDOUT $msg;}                                                  #
     if ($checkonly) { #------------------------------------------------------------+   #
        ### We are here if the "check only" option is passed & bad signal:          #   #
        $msg = sprintf ': '.$signal.' rate=%1.0f is not close to optimal=%1.0f'     #   #
                                                       ."\n", $I0, $I_optimum;      #   #
        if ($tell_bluice) {&tellBluIce($telltag.$msg,'error');}                     #   #
        $msg = '*** '.$script.$msg                                                  #   #
             . '*** '.$script.': Keithley gain adjustment needed'."\n";             #   #
        print STDOUT $msg;                                                          #   #
        $status = 1;                                                                #   #
        $kgain = $kgain_ini;                                                        #   #
     } else { #---------------------------------------------------------------------+   #
        ### Signal is bad. Optimize Keithley gain for I0:                           #   #
        ### (same code as in optimizeFluorescence.pl)                               #   #
        if ($I0 > $I_optimum) {$hasoverfill = 1;}                                   #   #
        else                  {$hasoverfill = 0;}                                   #   #
KEITHLEY:                                                                           #   #
        while(1) { #------------------------------------------------------------+   #   #
           $iteration++;                                                        #   #   #
           if ($iteration > $iterationsmax) {                                   #   #   #
              $msg = ': Iterations limit in optimizing'                         #   #   #
                    .' Keithley gain for '.$signal.' is reached!';              #   #   #
              if ($tell_bluice) {&tellBluIce($telltag.$msg,'error');}           #   #   #
              $msg = '*** '.$script.$msg."\n";                                  #   #   #
              print STDOUT $msg;                                                #   #   #
              $status = -1;                                                     #   #   #
              last KEITHLEY;                                                    #   #   #
           }                                                                    #   #   #
           ### Too low I0:                                                      #   #   #
           if ($I0 < $factor*$I_optimum) {                                      #   #   #
              if ($hasoverfill) {                                               #   #   #
                 ### There is no sense to increase gain because a higher        #   #   #
                 ### gain overfills Keithley:                                   #   #   #
                 $msg = ': Keithley gain adjustment converged';                 #   #   #
                 if ($tell_bluice) {&tellBluIce($telltag.$msg);}                #   #   #
                 $msg = $script.$msg."\n";                                      #   #   #
                 if ($report) {print STDOUT $msg;}                              #   #   #
                 last KEITHLEY;                                                 #   #   #
              }                                                                 #   #   #
              if ($I0<1) {$I0=1;}                                               #   #   #
              $kgain = $kgain_ini * int($I_optimum/$I0);                        #   #   #
              $igain = &keithley_gain_index($kgain);			       	#   #   #e.g. 1E4->4
#             if ($report) {print STDOUT 'kgain='.$kgain                        #   #   #
#                                    .'   igain='.$igain."\n";}                 #   #   #
              if    ($igain > $igain_max) {$igain = $igain_max;}                #   #   #
              elsif ($igain < $igain_min) {$igain = $igain_min;}                #   #   #
              if ($avoidresonance && $igain == 5) {$igain = 6;}		       	#   #   #avoid resonance on pin diodes
              $kgain = 10**$igain;                                              #   #   #
              if ($kgain <= $kgain_ini) {                                       #   #   #
                 ### Calculated increased gain not really increasing:           #   #   #
                 $kgain = $kgain_ini;                                           #   #   #
                 $msg = ': Keithley gain adjustment finished.';                 #   #   #
                 if ($tell_bluice) {&tellBluIce($telltag.$msg);}                #   #   #
                 $msg = $script.$msg."\n";                                      #   #   #
                 if ($report) {print STDOUT $script.': Calculated increased'    #   #   #
                              .' gain not really increasing.'."\n".$msg;}       #   #   #
                 if ($I0 < $I_minimum) {                                        #   #   #
                    $msg = ': too low '.$signal.' rate at max Keithley gain.';  #   #   #
                    if ($tell_bluice) {&tellBluIce($telltag.$msg,'error');}     #   #   #
                    $msg = '*** '.$script.$msg."\n";                            #   #   #
                    print STDOUT $msg;                                          #   #   #
                    $status = 1;                                                #   #   #
                 }                                                              #   #   #
                 last KEITHLEY;                                                 #   #   #
              }                                                                 #   #   #
              $msg = sprintf ': Low '.$signal.' rate.'                          #   #   #
                            .' Setting Keithley gain=%1.0e', $kgain;            #   #   #
              if ($tell_bluice) {&tellBluIce($telltag.$msg);}                   #   #   #
              $msg = $script.$msg."\n";                                         #   #   #
              if ($report) {print STDOUT $msg;}                                 #   #   #
           }                                                                    #   #   #
           ##### Too high I0:                                                   #   #   #
           elsif ($I0 > $I_optimum) {                                           #   #   #
              $hasoverfill = 1;                                                 #   #   #
              $kgain = $kgain_ini / 10;                                         #   #   #
              $igain = &keithley_gain_index($kgain);			       	#   #   #e.g. 1E4->4
              if    ($igain > $igain_max) {$igain = $igain_max;}                #   #   #
              elsif ($igain < $igain_min) {$igain = $igain_min;}                #   #   #
              if ($avoidresonance && $igain == 5) {$igain = 4;}		       	#   #   #avoid resonance on pin diodes
              $kgain = 10**$igain;                                              #   #   #
              if ($kgain >= $kgain_ini) {                                       #   #   #
                 ### Calculated increased gain not really decreasing:           #   #   #
                 $kgain = $kgain_ini;                                           #   #   #
                 $msg = ': Keithley gain adjustment finished';                  #   #   #
                 if ($tell_bluice) {&tellBluIce($telltag.$msg);}                #   #   #
                 $msg = $script.$msg."\n";                                      #   #   #
                 if ($report) {print STDOUT $script.': Calculated decreased'    #   #   #
                                . ' gain not really decreasing.'."\n".$msg;}    #   #   #
                 if ($I0 > 2*$I_optimum) {                                      #   #   #
                    $msg = ': too high '.$signal.' rate at min Keithley gain.'; #   #   #
                    if ($tell_bluice) {&tellBluIce($telltag.$msg,'error');}     #   #   #
                    $msg = '*** '.$script.$msg."\n";                            #   #   #
                    print STDOUT $msg;                                          #   #   #
                    $status = 1;                                                #   #   #
                 }                                                              #   #   #
                 last KEITHLEY;                                                 #   #   #
              }                                                                 #   #   #
              $msg = sprintf ': High '.$signal.' rate.'                         #   #   #
                            .' Setting Keithley gain=%1.0e', $kgain;            #   #   #
              if ($tell_bluice) {&tellBluIce($telltag.$msg);}                   #   #   #
              $msg = $script.$msg."\n";                                         #   #   #
              if ($report) {print STDOUT $msg;}                                 #   #   #
           }                                                                    #   #   #
           else {                                                               #   #   #
              ##### I0 is OK:                                                   #   #   #
              $msg = ': '.$signal.' is brought to normal range.'                #   #   #
                    .' Keithley gain adjustment complete.';                     #   #   #
              if ($tell_bluice) {&tellBluIce($telltag.$msg);}                   #   #   #
              $msg = $script.$msg."\n";                                         #   #   #
              if ($report) {print STDOUT $msg;}                                 #   #   #
              last KEITHLEY;                                                    #   #   #
           }                                                                    #   #   #
           &keithley_set_gain($kchannel, $kgain);                               #   #   #
           $kgain_ini = $kgain;                                                 #   #   #
           ### Re-measure I0:                                                   #   #   #
           if ($report) {print STDOUT $script.': '.$iteration                   #   #   #
                        .' Re-measuring '.$signal.' rate...'."\n";}             #   #   #
           if (defined $shutter) {&openShutter ($shutter,$shutter_readback);}   #   #   #
           @I0_ = &runJoerger ($scaler, $cnt_time, 1, $input);                  #   #   #
           if (defined $shutter) {&closeShutter ($shutter,$shutter_readback);}  #   #   #
           $I0 = $I0_[0];                                                       #   #   #
           $msg = sprintf ': '.$iteration.'. '.$signal.' rate=%1.0f', $I0;      #   #   #
           if ($tell_bluice) {&tellBluIce($telltag.$msg);}                      #   #   #
           $msg = $script.$msg."\n";                                            #   #   #
           if ($report) {printf STDOUT $msg.$script.': ...'.$signal             #   #   #
                                           .' rate=%1.0f'."\n", $I0;}           #   #   #
        } #-----------------:KEITHLEY loop end----------------------------------+   #   #
     } #---------------------adjustmend end-----------------------------------------+   #
  } #------------------------I0 not in range end----------------------------------------+
### Done:
  if ($status == 0) {$msg = 'OK';}
  return ($status, $kgain, $I0, $msg);
}

#=================================================================

# ($status,[$msg]) = &keithleyOptimizeGains(\@signals, $scaler,
#                                         [\@I_optimum,\@I_minimum]
#                                         [$iterationsmax,$avoidresonance,$checkonly],
#                                         [$tell_bluice,$telltag]);
sub keithleyOptimizeGains(\@;\@\@$$$$$) {
  our ($beamline, $shutter, $shutter_readback, $report);
  my ($script, $status, $signalsRef, $scaler, $I_optRef, $I_minRef);
  my ($iterationsmax, $avoidresonance, $checkonly, $tell_bluice, $telltag);
  my ($nsignal, $cnt_time, @kchannel, @input, @I0, @hasoverfill, @optimized);
  my (@I_opt, @I_min, @kgain_ini, $kgain, $igain, $failure);
  my ($iteration, $msg, $igain_max, $igain_min, $factor, $i);

  $script = 'keithleyOptimizeGains';
  $signalsRef     = shift(@_);
  $scaler         = shift(@_);
  $I_optRef       = shift(@_);
  $I_minRef       = shift(@_);
  $iterationsmax  = shift(@_);	if (!defined $iterationsmax)  {$iterationsmax=20;}
  $avoidresonance = shift(@_);  if (!defined $avoidresonance) {$avoidresonance=1;}
  $checkonly      = shift(@_);  if (!defined $checkonly)      {$checkonly=0;}
  $tell_bluice    = shift(@_);  if (!defined $tell_bluice)    {$tell_bluice=1;}
  $telltag        = shift(@_);				#script name for reporting to JBluIce

  if (!defined $report) {$report=0;}
  $cnt_time  = 1;					#scaler counting time
  $factor    = 0.1;					# intensity must be placed between $I_optimum and $factor*$I_optimum
  $status    = 0;
  $msg       = '';
  $iteration = 0;
  $igain_max = 9;
  $igain_min = 3;

  $nsignal = @$signalsRef;

  for ($i=0; $i<$nsignal; $i++) {
     ($status,$kchannel[$i],$input[$i]) = &getSignalChannels($$signalsRef[$i]);
     if ($status) {return $status,'No SQL data for signal='.$$signalsRef[$i];}
  }

  if (defined $$I_optRef[0]) {for ($i=0; $i<$nsignal; $i++) {$I_opt[$i] = $$I_optRef[$i];  }}
  else                       {for ($i=0; $i<$nsignal; $i++) {$I_opt[$i] = 4.0 * 1.E5;      }}
  if (defined $$I_minRef[0]) {for ($i=0; $i<$nsignal; $i++) {$I_min[$i] = $$I_minRef[$i];  }}
  else                       {for ($i=0; $i<$nsignal; $i++) {$I_min[$i] = $I_opt[$i]/1000.;}}

  if    ($avoidresonance =~ /^avoid/i   ||
         $avoidresonance =~ /^true/i    ||
         $avoidresonance =~ /^yes/i     ||
         $avoidresonance =~ /^1/i)		{$avoidresonance=1;}
  elsif ($avoidresonance =~ /^skip/i    ||
         $avoidresonance =~ /^ignore/i  ||
         $avoidresonance =~ /^false/i   ||
         $avoidresonance =~ /^no/i      ||
         $avoidresonance =~ /^0$/i)		{$avoidresonance=0;}
  else {
     print STDOUT '!!! '.$script.': unrecognized avoidresonance parameter='
                        .$avoidresonance.' -- Ignoring!'."\n";
     $avoidresonance = 0;
  }
  if    ($checkonly =~ /^check/i   ||
         $checkonly =~ /^true/i    ||
         $checkonly =~ /^yes/i     ||
         $checkonly =~ /^1/i)		{$checkonly=1;}
  elsif ($checkonly =~ /^skip/i    ||
         $checkonly =~ /^false/i   ||
         $checkonly =~ /^no/i      ||
         $checkonly =~ /^0$/i)		{$checkonly=0;}
  else {
     print STDOUT '!!! '.$script.': unrecognized checkonly parameter='
                        .$checkonly.' -- Ignoring!'."\n";
     $checkonly = 0;
  }
  if    ($tell_bluice =~ /^tell/i   ||
         $tell_bluice =~ /^true/i   ||
         $tell_bluice =~ /^yes/i    ||
         $tell_bluice =~ /^1/i)		{$tell_bluice=1;}
  elsif ($tell_bluice =~ /^skip/i   ||
         $tell_bluice =~ /^false/i  ||
         $tell_bluice =~ /^no/i     ||
         $tell_bluice =~ /^0$/i)	{$tell_bluice=0;}
  else {
     print STDOUT '!!! '.$script.': unrecognized tell_bluice parameter='
                        .$tell_bluice.' -- Ignoring!'."\n";
     $tell_bluice = 0;
  }
  if ($tell_bluice) {if (!defined $telltag) {$telltag = &makeScriptName();}}

  if ($scaler !~ /^${beamline}:/)     {$scaler    = $beamline.':'.$scaler;}
  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }

### Read initial (current) Keithley gains:
  $msg = ': Starting optimization of Keithley gain(s)';
  for ($i=0; $i<$nsignal; $i++) {
     $kgain_ini[$i] = &keithley_read_gain($kchannel[$i]);
     $msg .= sprintf ' ch=%d (ini=%1.0e)', $kchannel[$i], $kgain_ini[$i];
  }
  if ($tell_bluice) {&tellBluIce($telltag.$msg);}
  $msg = $script.$msg."\n";
  if ($report) {print STDOUT $msg;}

### Set scaler:
  if ($report) {print STDOUT $script.': Setting up scaler "'.$scaler.'" count time='.$cnt_time."s\n";}
  &setJoerger($scaler,$cnt_time);

### Open endstation shutter:
  $status = &getEndStationShutter($beamline);
  if ($status == 0) {					#if closed
     if ($tell_bluice) {&tellBluIce($telltag.': opening endstation shutter');}
     $status = &openEndStationShutter($beamline);
     if ($status) {
        if ($tell_bluice) {&tellBluIce($telltag.': Station shutter failed','error');}
        $msg = '*** '.$script.': failed to open endstation shutter'."\n";
        print STDOUT $msg;
        $status = -1;
        return ($status, $msg);
     }
  } else {
     $status = 0;					#all is OK, enstation shutter is open
  }

### Measure I0:
  if ($report) {print STDOUT $script.': '.$iteration.'. Measuring initial intensity rate(s)'."\n";}
  if (defined $shutter) {&openShutter ($shutter,$shutter_readback);}
  @I0 = &runJoergerRef ($scaler, $cnt_time, \@input);
  if (defined $shutter) {&closeShutter ($shutter,$shutter_readback);}
  for ($i=0; $i<$nsignal; $i++) {
     $msg = sprintf ': signal=%s, keithley=%d, scalerInput=%d, iniRate=%1.0f',
                           $$signalsRef[$i], $kchannel[$i], $input[$i], $I0[$i];
     if ($tell_bluice) {&tellBluIce($telltag.$msg);}
     if ($report)      {print STDOUT $script.$msg."\n";}
  }

  $iteration++;

  $status = 0;
  $failure = 0;

  for ($i=0; $i<$nsignal; $i++) { #===============================================================#
     if ($I0[$i] < $factor*$I_opt[$i] || $I0[$i] > $I_opt[$i]) {                                  #
        $optimized[$i]   = 0;                                                                     #
        $status++;                                                                                #
        if ($I0[$i] < $factor*$I_opt[$i]) {                                                       #
           $hasoverfill[$i] = 0;                                                                  #
           $msg = sprintf ' signal=%s rate=%1.0f is less than '.$factor.' of optimal=%1.0f'."\n", #
                                                          $$signalsRef[$i], $I0[$i], $I_opt[$i];  #
        }                                                                                         #
        elsif ($I0[$i] > $I_opt[$i]) {                                                            #
           $hasoverfill[$i] = 1;                                                                  #
           $msg = sprintf ' signal=%s rate=%1.0f exceeds optimal=%1.0f'."\n",                     #
                                                          $$signalsRef[$i], $I0[$i], $I_opt[$i];  #
        }                                                                                         #
        if ($report) {printf STDOUT '!!! '.$script.': signal=%s '.$msg."\n", $$signalsRef[$i];}   #
        $msg = sprintf ': Keithley gain adjustment for %s needed', $$signalsRef[$i];              #
        if ($tell_bluice) {&tellBluIce($telltag.$msg,'warn');}                                    #
        if ($report)      {print STDOUT '!!! '.$script.$msg."\n";}                                #
        if ($checkonly) {                                                                         #
           $msg = sprintf ': signal=%s rate=%1.0f is not close to optimal=%1.0f'."\n",            #
                                                       $$signalsRef[$i], $I0[$i], $I_opt[$i];     #
           if ($tell_bluice) {&tellBluIce($telltag.$msg,'error');}                                #
           if ($report)       {print STDOUT '*** '.$script.$msg;}                                 #
        }                                                                                         #
     } else {                                                                                     #
        $hasoverfill[$i] = 0;                                                                     #
        $optimized[$i] = 1;                                                                       #
     }                                                                                            #
  } #=============================================================================================#

  if ($checkonly && $status) {
     ### We are here if the "check only" option is passed & bad signal:
     $msg = ': Keithley gain adjustment needed for '.$status.' channels of '.$nsignal;
     if ($tell_bluice) {&tellBluIce($telltag.$msg,'error');}
     $msg = '*** '.$script."\n";
     print STDOUT $msg;
     return $status,$msg;
  }
  elsif ($status == 0) {
     $msg = ': All signal rates are in range. No Keithley gain adjustment needed.';
     if ($tell_bluice) {&tellBluIce($telltag.$msg);}
     $msg = $script."\n";
     print STDOUT $msg;
     return $status,$msg;
  }

### Optimize Keithley gain for I0:
### (same code as in optimizeFluorescence.pl)
KEITHLEY:
  while ($status) { #--------------------------status is number of signals to optimize------+
     $iteration++;                                                                          #
     if ($iteration > $iterationsmax) {                                                     #
        $msg = ': Iterations limit in optimizing Keithley gain(s) is reached!';             #
        if ($tell_bluice) {&tellBluIce($telltag.$msg,'error');}                             #
        if ($report) {print STDOUT '*** '.$script.$msg."\n";}                               #
        return $status, '*** '.$script.$msg;                                                #
     }                                                                                      #
     for ($i=0; $i<$nsignal; $i++) { #===================================================+  #
        if ($optimized[$i] == 0) { #--------------------------------------------------+  #  #
           if ($I0[$i] < $factor*$I_opt[$i]) {                                        #  #  #
              ### Too low I0:                                                         #  #  #
              if ($hasoverfill[$i]) {                                                 #  #  #
                 ### There is no sense to increase gain because a higher              #  #  #
                 ### gain overfills Keithley:                                         #  #  #
                 $msg = ': Keithley gain adjustment converged';                       #  #  #
                 if ($tell_bluice) {&tellBluIce($telltag.$msg);}                      #  #  #
                 if ($report) {print STDOUT $script.$msg."\n";}                       #  #  #
                 $optimized[$i] = 1;                                                  #  #  #
                 $status--;                                                           #  #  #
                 next;                                                                #  #  #
              }                                                                       #  #  #
              if ($I0[$i] < 1) {$I0[$i] = 1;}                                         #  #  #
              ### Increase gain:                                                      #  #  #
              $kgain = $kgain_ini[$i] * int($I_opt[$i]/$I0[$i]);                      #  #  #
              $igain = &keithley_gain_index($kgain);				      #  #  #e.g. 1E4->4
#             if ($report) {print STDOUT 'kgain='.$kgain.'   igain='.$igain."\n";}    #  #  #
              if    ($igain > $igain_max) {$igain = $igain_max;}                      #  #  #
              elsif ($igain < $igain_min) {$igain = $igain_min;}                      #  #  #
              if ($avoidresonance && $igain == 5) {$igain = 6;}			      #  #  #avoid pindiode resonance
              $kgain = 10**$igain;                                                    #  #  #
              if ($kgain <= $kgain_ini[$i]) {                                         #  #  #
                 ### Calculated increased gain not really increasing:                 #  #  #
                 $kgain = $kgain_ini[$i];                                             #  #  #
                 $optimized[$i] = 1;                                                  #  #  #
                 $status--;                                                           #  #  #
                 $msg = sprintf ': Keithley gain adjustment for signal=%s finished',  #  #  #
                                                                   $$signalsRef[$i];  #  #  #
                 if ($tell_bluice) {&tellBluIce($telltag.$msg);}                      #  #  #
                 if ($report) {print STDOUT $script.': Calculated increased'          #  #  #
                            .' gain not really increasing.'."\n".$script.$msg."\n";}  #  #  #
                 if ($I0[$i] < $I_min[$i]) {                                          #  #  #
                    $msg = sprintf ': signal=%s rate is too low at max'               #  #  #
                     .' Keithley-%d gain.', $$signalsRef[$i], $kchannel[$i];          #  #  #
                    if ($tell_bluice) {&tellBluIce($telltag.$msg,'error');}           #  #  #
                    $msg = '*** '.$script.$msg."\n";                                  #  #  #
                    print STDOUT $msg;                                                #  #  #
                    $failure++;                                                       #  #  #
                 }                                                                    #  #  #
                 next;                                                                #  #  #
              } else {                                                                #  #  #
                 $msg = sprintf ': Low signal=%s rate. Setting Keithley gain=%1.0e',  #  #  #
                                                       $$signalsRef[$i], $kgain;      #  #  #
                 if ($tell_bluice) {&tellBluIce($telltag.$msg);}                      #  #  #
                 $msg = $script.$msg."\n";                                            #  #  #
                 if ($report) {print STDOUT $msg;}                                    #  #  #
              }                                                                       #  #  #
           }                                                                          #  #  #
           elsif ($I0[$i] > $I_opt[$i]) {                                             #  #  #
              ##### Too high I0:                                                      #  #  #
              $hasoverfill[$i] = 1;                                                   #  #  #
              ### Decrease gain:                                                      #  #  #
              $kgain = $kgain_ini[$i] / 10;                                           #  #  #
              $igain = &keithley_gain_index($kgain);				      #  #  #e.g. 1E4->4
              if    ($igain > $igain_max) {$igain = $igain_max;}                      #  #  #
              elsif ($igain < $igain_min) {$igain = $igain_min;}                      #  #  #
              if ($avoidresonance && $igain == 5) {$igain = 4;}			      #  #  #avoid pindiode resonance
              $kgain = 10**$igain;                                                    #  #  #
              if ($kgain >= $kgain_ini[$i]) {                                         #  #  #
                 ### Calculated increased gain not really decreasing:                 #  #  #
                 $kgain = $kgain_ini[$i];                                             #  #  #
                 $optimized[$i] = 1;                                                  #  #  #
                 $status--;                                                           #  #  #
                 $msg = sprintf ': Keithley gain adjustment for signal=%s finished',  #  #  #
                                                                   $$signalsRef[$i];  #  #  #
                 if ($tell_bluice) {&tellBluIce($telltag.$msg);}                      #  #  #
                 if ($report) {print STDOUT $script.': Calculated decreased'          #  #  #
                           .' gain not really decreasing.'."\n".$script.$msg."\n";}   #  #  #
                 if ($I0[$i] > 2*$I_opt[$i]) {                                        #  #  #
                    $msg = sprintf ': signal=%s rate is too high at min'              #  #  #
                     .' Keithley-%d gain.', $$signalsRef[$i], $kchannel[$i];          #  #  #
                    if ($tell_bluice) {&tellBluIce($telltag.$msg,'error');}           #  #  #
                    $msg = '*** '.$script.$msg."\n";                                  #  #  #
                    print STDOUT $msg;                                                #  #  #
                    $failure++;                                                       #  #  #
                 }                                                                    #  #  #
                 next;                                                                #  #  #
              } else {                                                                #  #  #
                 $msg = sprintf ': High signal=%s rate. Setting Keithley gain=%1.0e', #  #  #
                                                        $$signalsRef[$i], $kgain;     #  #  #
                 if ($tell_bluice) {&tellBluIce($telltag.$msg);}                      #  #  #
                 $msg = $script.$msg."\n";                                            #  #  #
                 if ($report) {print STDOUT $msg;}                                    #  #  #
              }                                                                       #  #  #
           }                                                                          #  #  #
           else {                                                                     #  #  #
              ##### I0 OK:                                                            #  #  #
              $msg = sprintf ': signal=%s is brought to normal range.'                #  #  #
                                         .' Keithley-%d gain adjustment complete.',   #  #  #
                                                  $$signalsRef[$i], $kchannel[$i];    #  #  #
              if ($tell_bluice) {&tellBluIce($telltag.$msg);}                         #  #  #
              print STDOUT $script.$msg."\n";                                         #  #  #
              $optimized[$i] = 1;                                                     #  #  #
              $status--;                                                              #  #  #
              next;                                                                   #  #  #
           }                                                                          #  #  #
           &keithley_set_gain($kchannel[$i], $kgain);                                 #  #  #
           $kgain_ini[$i] = $kgain;                                                   #  #  #
        } #---------------------------------------------------------------------------+  #  #
     } #=================================================================================+  #
     if ($status == 0) {last;}                                                              #
### Re-measure I0:                                                                          #
     if ($report) {printf STDOUT $script.': %d. Re-measuring intensity rates for'           #
                             .' %d signals(s)...'.."\n", $iteration, $status;}              #
     if (defined $shutter) {&openShutter ($shutter,$shutter_readback);}                     #
     @I0 = &runJoergerRef ($scaler, $cnt_time, \@input);                                    #
     if (defined $shutter) {&closeShutter ($shutter,$shutter_readback);}                    #
     for ($i=0; $i<$nsignal; $i++) {  #=====================================+               #
        if ($optimized[$i] == 0) { #-------------------------------------+  #               #
           $msg = sprintf ': %d. signal=%s, keithley=%d, rate=%1.0f',    #  #               #
                $iteration, $$signalsRef[$i], $kchannel[$i], $I0[$i];    #  #               #
           if ($tell_bluice) {&tellBluIce($telltag.$msg);}               #  #               #
           if ($report)      {print STDOUT $script.$msg."\n";}           #  #               #
        } #--------------------------------------------------------------+  #               #
     } #====================================================================+               #
  } #-----------------:KEITHLEY loop end----------------------------------------------------+
### Done:
  if ($status == 0) {$msg = 'OK';}
  return ($status, $msg);
}
