#!/usr/bin/perl -W
#--------------------------------------------------------------
#
# Example: script.pl Data/SHG-example.txt
#
#    This script reads a TXT-formatted SONICC image, converts it into
#    PNG, displays on the screen and shows mouse coordinates in pixels.
#
#                       Author: Sergey Stepanov
#
# Version-1.0:  2012/10/25	initial implementation using PDL::IO::Pic and NetPBM
# Version-1.1:  2012/11/12	added scale factor for images
# Version-1.2:  2015/02/24	added automatic determining if image scaling needs be applied
#--------------------------------------------------------------
  use PDL;			#also needs NetPBM package installed for PDL::IO::Pic
  use PDL::Image2D;		#for scaling images
  use Tk;
  use Tk::widgets qw/JPEG PNG ROText/;
# use Tk::JPEG;
# use Tk::PNG;
# use File::Which;              # exports which() -- not available in ActivePerl-5.10
  use strict;

  our ($NONINTERACTIVE, $USE_ZENITY_FOR_EXIT, $IGNORE_ERRORS);
  our ($BYE_IN_PROGRESS, $SUB_IN_PROGRESS);
  my ($PEZCA, $script, $syntax, $narg, $arg, $i, $imgName, $imgLog);
  my ($mw, $mw0, $f, $image, $imDisplay, $imLabelXY, $imLabelNm, $logBox);
  my ($filetypes, $iniDir, $name, @scaleLabels,$msg);
  my ($nlines, $ncols, $min, $max, $magnify);

  if (!defined $ENV{PEZCA}) {
     die 'Please define the PEZCA environment. It must point to the PERL scripts depository';
  }
  $PEZCA=$ENV{PEZCA};
  $PEZCA =~ s/\\/\//g;
  if ($PEZCA !~ /\/$/) {$PEZCA .='/';}

  do $PEZCA.'Common/common_subs.pl';

  select(STDERR);  $|=1;		# Flush output after each print
  select(STDOUT);  $|=1;		# Flush output after each print

  $NONINTERACTIVE      = 1;
  $USE_ZENITY_FOR_EXIT = 1;
  $IGNORE_ERRORS       = 1;
  $BYE_IN_PROGRESS     = 0;
  $SUB_IN_PROGRESS     = 0;
  $imgLog = 1;				# default intensity scale flag (0=linear,1=log)
  @scaleLabels = qw(Linear Log);

  $script = &makeScriptName();
  $syntax = "\n\n".'Syntax:'."\n".$script.' TXTimage [scale=log|scale=linear]'."\n";

# if (!defined which('pnmtopng')) {
#    &press_key_exit('*** '.$script.': cannot find pnmtopng for converting txt-formatted images.'."\n"
#                   .'Please install NetPBM and ensure that it is in the path.');
# }

  $narg = @ARGV;
  for ($i=0; $i<$narg; $i++) {
     $arg = $ARGV[$i];
     if ($arg =~ /^scale=/i) {
	$arg =~ s/^scale=//i;
	if    ($arg =~ /^log$/i)    {$imgLog = 1;}
	elsif ($arg =~ /^lin$/i)    {$imgLog = 0;}
	elsif ($arg =~ /^linear$/i) {$imgLog = 0;}
	else  {&press_key_exit('*** '.$script.': incorrect cmdline argument '.$ARGV[$i].$syntax);}
     } else {
        if (!defined $imgName) {$imgName = $ARGV[$i];}
	else  {&press_key_exit('*** '.$script.': incorrect cmdline argument '.$ARGV[$i].$syntax);}
     }
  }
  $filetypes = [ ['TXT images', '.txt'],
                 ['All Files',   '*'],];
  if (!defined $imgName) {
     $mw0 = MainWindow->new;			#main window for file open dialog
     $mw0 -> geometry ("1x1");			#hide this window
     $mw0->withdraw();				#hide this window
### See: http://theory.uwinnipeg.ca/perltk/widgets/
     $imgName = $mw0->getOpenFile(-filetypes => $filetypes,
                                  -defaultextension => '.txt',
                                  -initialdir => '.',);
#    $mw0->destroy();					#causes application crashes
  }
  if (!defined $imgName) {&press_key_exit('*** '.$script.': no image file specified.'.$syntax);}
  if (! -f $imgName) {&press_key_exit('*** '.$script.': file='.$imgName.' not found');}

  ($iniDir, $name) = &split_filename_dirname($imgName);
  $imgName = &read_image($imgName);			#read TXTimage & save as PNG
###-------------------------------------------------------------------------
### Create a GUI:
# $mw->deiconify();		#restore prev.hidden main window -- did not work on Linux
# $mw->raise();			#restore prev.hidden main window -- did not work on Linux
# $mw->update();		#restore prev.hidden main window -- did not work on Linux
  $mw = MainWindow->new;
  $f = $mw->Frame->grid(-row=>0, -column=>0, -sticky=>'n');
  $image = $mw->Photo(-file=>$imgName);					#create image object
  $imDisplay = $f->Label(-image=>$image,-cursor=>'tcross',
                         -borderwidth=>5,-relief=>'groove',
			 -background=>'white')->grid;			#pack image widget
  $imLabelNm = $f->Label(-text =>$name)->grid;				#pack name label widget
  $imLabelXY = $f->Label(-text =>'')->grid;				#pack XY label widget
# $imLabelX = $f->Label(-text =>$mw->pointerx)->grid;
# $imLabelY = $f->Label(-text =>$mw->pointery)->grid;
# $imLabelX = $f->Label(-textvariable =>\$mw->pointerx)->grid;
# $imLabelY = $f->Label(-textvariable =>\$mw->pointery)->grid;
  $f = $mw->Frame->grid(-row=>0, -column=>1, -sticky=>'n');
  $f->Button(-text=>'Open', -command=>\&open_another_image,)->grid(-columnspan=>2);
  for ($i=0; $i<2; $i++) {
     $f->Radiobutton(-text=>$scaleLabels[$i],
                     -variable=>\$imgLog,
                     -value   =>$i,
                     -command =>\&change_image_scale,)->grid(-row=>1, -column=>$i);
  }
  $f = $mw->Frame->grid(-row=>1, -column=>0, -columnspan=>2, -sticky=>'n');
  $logBox = $f->Scrolled('ROText', -scrollbars=>'se',-width=>'72',-height=>'5')->grid;

  $msg = 'Opening text image: '.$name."\n"
       . 'Image size: '.$ncols.'x'.$nlines."\n"
       . 'Image intensity: min='.$min.', max='.$max;
  &updateLogBox($msg);

### Set up binding so that, when the leftt mouse button (1)
### is clicked on the image window ($imDisplay), &(showcoords) is
### called and is given the x and y coordinates of the click.
### See: http://oreilly.com/catalog/mastperltk/chapter/ch15.html

# $mw->bind('<1>', [\&showcoords, Ev('x'), Ev('y')]);
# $mw->bind('<Motion>', [\&updatecoords, Ev('x'), Ev('y')]);
  $imDisplay->bind('<1>', [\&showcoords, Ev('x'), Ev('y')]);
  $imDisplay->bind('<Motion>', [\&updatecoords, Ev('x'), Ev('y')]);
  $imDisplay->bind('<Leave>', [\&clearcoords]);
  $mw->bind('<Destroy>', [\&goodbye]);
  $mw->bind('<KeyPress-Escape>', [\&goodbye]);
### This is for testing keys names:
# $mw->bind('<KeyPress>'=>\&print_keysym);

  $mw->resizable( 0, 0 );
  $mw->focus();
  MainLoop;

#==========================================================================
### Called when on chosen event on the main window.
sub showcoords {
  if ($SUB_IN_PROGRESS) {return;}
  my ($widget, $x, $y) = @_;
  $msg = $x.' '.$y;
  $imLabelXY->configure(-text=>$msg);
  $msg = 'Clicked at '.$msg;
  &updateLogBox($msg);
}

#--------------------------------------------------------------------------
sub updatecoords {
  my ($widget, $x, $y) = @_;
  $imLabelXY->configure(-text=>$x.' '.$y);
}

#--------------------------------------------------------------------------
sub clearcoords {
  $imLabelXY->configure(-text=>'');
}

#--------------------------------------------------------------------------
sub updateLogBox ($) {
  my $message = shift(@_);
  if (defined $logBox) {
     $logBox->insert('end', $message."\n");
     $logBox->see('end');
  } else {
     print STDOUT $message."\n";
  }
}

#--------------------------------------------------------------------------
sub goodbye {
  if (defined $BYE_IN_PROGRESS && $BYE_IN_PROGRESS) {return;}
  $BYE_IN_PROGRESS = 1;
  $msg = 'Goodbye!';
# print STDOUT $msg."\n\n";
  &updateLogBox($msg."\n");
  exit;
}

#--------------------------------------------------------------------------
sub print_keysym {
  my($widget) = @_;
  my $e = $widget->XEvent;    # get event object
  my($keysym_text, $keysym_decimal) = ($e->K, $e->N);
  $msg = 'keysym='.$keysym_text.', numeric='.$keysym_decimal;
  &updateLogBox($msg);
}

#--------------------------------------------------------------------------
sub open_another_image {
  if ($SUB_IN_PROGRESS) {return;}
  $SUB_IN_PROGRESS=1;
  my $im = $mw->getOpenFile(-filetypes=>$filetypes,
                             -defaultextension=>'.txt',
                             -initialdir=>$iniDir,);
  if (defined $im) {
     if (! -f $im) {
	$msg = '*** File "'.$im.'" not found.';
        &updateLogBox($msg);
	$SUB_IN_PROGRESS=0;
	return;
     }
     ($iniDir, $name) = &split_filename_dirname($im);
     $imgName = &read_image($im);
     $image->configure(-file=>$imgName);		#create image object
     $imDisplay->configure(-image=>$image);		#update image widget
     $imLabelNm->configure(-text=>$name);		#update name widget
     $mw->focus();
  }
  $SUB_IN_PROGRESS=0;
}

#--------------------------------------------------------------------------
sub change_image_scale {

  if ($SUB_IN_PROGRESS) {return;}
  $SUB_IN_PROGRESS=1;
  my $im = $imgName;
  $im =~ s/\.png$//i;
  if (defined $im) {
     if (! -f $im) {
     	$msg = '*** File "'.$im.'" not found.';
        &updateLogBox($msg);
	$SUB_IN_PROGRESS=0;
	return;
     }
     $msg = 'File '.$name.': switching to scale='.$scaleLabels[$imgLog];
     &updateLogBox($msg);
     $im = &read_image($im);
     $image->configure(-file=>$im);			#create image object
     $imDisplay->configure(-image=>$image);		#pack image widget
     $mw->focus();
  }
  $SUB_IN_PROGRESS=0;
}

#--------------------------------------------------------------------------
sub read_image ($) {
  my ($handle, $status, $im, @buffer, $imgpdl);
  my (@imgtxt, @arrline, $nwrd, $i, $j, $l, $mi, $ma);
  $im = shift(@_);
  $msg = 'Opening text image: '.$name;
  &updateLogBox($msg);
  $handle = open (TXIM, '< '.$im);
  if (!defined $handle) {
     $msg = '*** '.$script.': Error opening '.$im;
     &updateLogBox($msg);
#    &press_key_exit($msg);
     return;
  }
  @buffer = <TXIM>;
  close(TXIM);

  $nlines = @buffer;
  @imgtxt = ();
  $l = 0;
  undef $ncols;
# for ($i=0; $i<$nlines; $i++) {
  for ($i=$nlines-1; $i>=0; $i--) {				#need to invert image vertically
     chomp $buffer[$i];
     if (length($buffer[$i]) == 0) {
#       splice(@buffer,$i,1);					#uncomment if not inverting image
#       $i--;							#uncomment if not inverting image
        $nlines--;
     } else {
        @arrline = split /\s+/, $buffer[$i];  			#split into words
        $nwrd = @arrline;
        if (!defined $ncols) {$ncols = $nwrd;}
	else {
	   if ($ncols ne $nwrd) {
              $msg = '*** '.$script.': Number of columns in image changed '.$ncols.' -> '.$nwrd.' at line='.$nlines;
              &updateLogBox($msg);
#             &press_key_exit($msg);
	      return;
	   }
	}
        for ($j=0; $j<$ncols; $j++) {$imgtxt[$l][$j] = $arrline[$j];}
	$l++;
     }
  }
  $msg = 'Image size: '.$ncols.'x'.$nlines;
  if ($ncols < 500) {$magnify = 2;}
  else              {$magnify = 1;}
  &updateLogBox($msg);
  $imgpdl = pdl (\@imgtxt);					#create piddle
  if ($magnify > 1) {
     $ncols  = $magnify*$ncols;
     $nlines = $magnify*$nlines;
     my $tmppdl = zeroes($ncols,$nlines);
     rescale2d($imgpdl, $tmppdl);
     $imgpdl = $tmppdl;
  }
  ($mi,$ma) = ($min, $max) = minmax($imgpdl);
  $msg = 'Image intensity: min='.$min.', max='.$max;
  &updateLogBox($msg);

  if ($imgLog) {
     $imgpdl = log($imgpdl+0.125);
     ($mi, $ma) = minmax($imgpdl);
  }
# print STDOUT 'Image intensity: min='.$mi.', max='.$ma."\n";
  if ($ma == $mi) {
     $msg = '*** Image "'.$name.'" is fully dark (no data)';
     &updateLogBox($msg);
#    &press_key_exit($msg);
     return;
  }
### Normalize and convert to short (both
### 8-bit & 16 bit work, but the image
### created by netPBM at next step is
### 8-bit in either case):
  $imgpdl = byte((255/($ma-$mi))*($imgpdl-$mi));             	#  8-bit
# $imgpdl = short((32767/($ma-$mi))*($imgpdl-$mi));          	# 16-bit
### Test:
# ($mi, $ma) = minmax($imgpdl);
# print STDOUT 'Image intensity: min='.$mi.', max='.$ma."\n";

###append '.png' (or better substitute txt->png?):
  $im .= '.png';
  $msg = 'Saving image as '.$im;
  &updateLogBox($msg);
### Uses PDL::IO::Pic -- see: http://pdl.perl.org/?docs=IO/Pic&title=PDL::IO::Pic
### Also needs NetPBM package installed (the routine pnmtopng must be in the path):
# $status = wpic($imgpdl,$im);					#save image (needs NetPBM' pnmtopng)
  $status = wim($imgpdl,$im); 					#save image (needs NetPBM' pnmtopng)
# print STDOUT 'status = '.$status."\n";			#status=number of saved images?
  return $im;
}
