#!/usr/bin/perl -W
#----------------------------------------------------
# This script adds scienceDMZ route.
# Syntax: add_dmz_route.pl ip|cidr [--delete]
#----------------------------------------------------
  use strict;
  my ($script, $syntax, @inets, $inet, $str, $dmz, $narg, $ip, $cidr, @cmd);
  my ($dmzmask, $dmzgate, $user, $arg, $action);
  my ($LS, $IFCONFIG, $GREP, $ROUTE);

  select STDERR; $|=1;                          #set unbuffered output
  select STDOUT; $|=1;                          #set unbuffered output

  if ($^O =~ /Win/i) {
     print STDOUT "\n".'This script cannot run on Windows'."\n\n";
     exit;
  }
  if ($> != 0) {
     $user = (getpwuid $>);
     print STDOUT "\n".'User='.$user;
     print STDOUT "\n".'This script should be run as root. Re-launching via sudo...'."\n\n";
     exec('/usr/bin/sudo','-n',$0, @ARGV);
     exit;
  }

  if    (-e '/bin/ls')            {$LS = '/bin/ls';}
  elsif (-e '/usr/bin/ls')        {$LS = '/usr/bin/ls';}
  elsif (-e `which ls`)           {$LS = `which ls`;}
  else                            {die 'ls command not found';}

  if    (-e '/sbin/ifconfig')     {$IFCONFIG = '/sbin/ifconfig';}
  elsif (-e '/usr/sbin/ifconfig') {$IFCONFIG = '/usr/sbin/ifconfig';}
  elsif (-e `which ifconfig`)     {$IFCONFIG = `which ifconfig`;}
  else                            {die 'ifconfig command not found';}

  if    (-e '/bin/grep')          {$GREP = '/bin/grep';}
  elsif (-e '/usr/bin/grep')      {$GREP = '/usr/bin/grep';}
  elsif (-e `which grep`)         {$GREP = `which grep`;}
  else                            {die 'grep command not found';}

  if    (-e '/usr/sbin/route')    {$ROUTE = '/usr/sbin/route';}
  elsif (-e '/sbin/route')        {$ROUTE = '/sbin/route';}
  elsif (-e `which route`)        {$ROUTE = `which route`;}
  else                            {die 'route command not found';}

  $dmzmask = '192.5.180.';
  $dmzgate = $dmzmask.'129';
  $script = $0;
  $script =~ s/\\/\//g;				#change \ to /
  $script =~ s/^.*\///g;			#remove everything before last "/"
  $syntax = "\n".'Syntax: '.$script.' IPaddress-or-CIDR-for-DMZ-network [--delete]'."\n\n";
  $narg = @ARGV;
  if ($narg < 1 || $narg > 2) {print STDOUT $syntax; exit;}
  foreach $arg (@ARGV) {
     if ($arg =~ /^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/) {
        $ip = $arg;
     }
     elsif ($arg =~ /^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\/[0-9]{1,2}$/) {
        $cidr = $arg;
     }
     elsif ($arg =~ /^-{0,2}del(ete)?$/) {
        $action = 'del';
     }
     else {   
        print STDOUT "\n".$script.': incorrect command-line argument "'.$arg.'"'.$syntax; exit;
     }
  }

  if (!defined $action) {$action = 'add';}
  if (!defined $ip && !defined $cidr) {print STDOUT "\n".$script.': no target specified.'.$syntax; exit;}
  if ( defined $ip &&  defined $cidr) {print STDOUT "\n".$script.': ambiguous target.'.$syntax; exit;}
  
# if    (defined $ip)   {print STDOUT 'IP='.  $ip  .'  action='.$action."\n"; exit;}
# elsif (defined $cidr) {print STDOUT 'CIDR='.$cidr.'  action='.$action."\n"; exit;}

  @inets = split(/\n/, `$LS -1 /sys/class/net`);

  foreach $inet (@inets) {
    $str = `$IFCONFIG $inet | $GREP ${dmzmask}.`;
    if (length($str) > 0) {$dmz =  $inet; last;}
  }
  if (!defined $dmz) {
     print STDOUT "\n".'No DMZ network interface found!'."\n\n";
     exit;
  }

  $str = $ROUTE.' '.$action;
  if    (defined $ip)   {$str .= ' -host '.$ip  .' gw '.$dmzgate.' dev '.$dmz;}
  elsif (defined $cidr) {$str .= ' -net ' .$cidr.' gw '.$dmzgate.' dev '.$dmz;}
  @cmd = split(/\s+/,$str);
# print STDOUT "@{cmd}\n";
  print STDOUT "\n".$str."\n";
  system(@cmd);

  $str = '/usr/sbin/ip route list';
  print STDOUT "\n".$str."\n";
  system($str);
  exit;
