#!/usr/bin/perl -W
#--------------------------------------------------------------
# This routine prints attenuation data (foils, thicknesses, exceptions)
# from both BluIce mySQL and Acromag-9440 Digital I/O channel descriptions
#
#                   Author: Sergey Stepanov
#
# Version 1.0 2008/02/15 using Pezca
#--------------------------------------------------------------
  use strict;
  use Pezca;
  use DBI;

  my $PEZCA;

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/attenuation_subs_v2.pl';
  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDOUT; $|=1;			# set unbuffered output

# our $beamline = '23o';
#--------------------------------------------------------------
  our $nAttenuators;
  our (@Attenuators_material, @Attenuators_thickness);
  our (@attenuatorExceptions, $nAttenuatorExceptions);

  my (@Attenuators_material_mySQL, @Attenuators_thickness_mySQL);
  my (@attenuatorExceptions_mySQL, $nAttenuatorExceptions_mySQL);

  my (@Attenuators_material_DESC, @Attenuators_thickness_DESC);
  my (@attenuatorExceptions_DESC, $nAttenuatorExceptions_DESC);

  my ($status, $i, $care);

  $status = &makeAttenuatorDB_mySQL();
  if ($status) {die "Exit status=${status} from makeAttenuatorDB_mySQL\n";}
  @Attenuators_material_mySQL  = @Attenuators_material;
  @Attenuators_thickness_mySQL = @Attenuators_thickness;
  @attenuatorExceptions_mySQL  = @attenuatorExceptions;
  $nAttenuatorExceptions_mySQL = $nAttenuatorExceptions;

  $status = &makeAttenuatorDB_DESC();
  if ($status) {die "Exit status=${status} from makeAttenuatorDB_DESC\n";}
  @Attenuators_material_DESC  = @Attenuators_material;
  @Attenuators_thickness_DESC = @Attenuators_thickness;
  @attenuatorExceptions_DESC  = @attenuatorExceptions;
  $nAttenuatorExceptions_DESC = $nAttenuatorExceptions;

  if (!defined $nAttenuators) {die 'nAttenuators is undefined'."\n";}
  print STDOUT 'Channel       mySQL                DESC'."\n";
  for ($i=0; $i<$nAttenuators; $i++) {
    if ($Attenuators_material_mySQL[$i]  eq $Attenuators_material_DESC[$i] &&
	$Attenuators_thickness_mySQL[$i] == $Attenuators_thickness_DESC[$i]) {
       $care = '';
    } else {
       $care = '***************************';
    }
    printf STDOUT "%02d         (%s, %s)      \t (%s, %s)  %s\n", $i,
     $Attenuators_material_mySQL[$i], $Attenuators_thickness_mySQL[$i],
     $Attenuators_material_DESC[$i], $Attenuators_thickness_DESC[$i],
     $care;
  }
    print STDOUT "\n Exceptions:\n";
    print STDOUT "mySQL ($nAttenuatorExceptions_mySQL): @{attenuatorExceptions_mySQL}\n";
    print STDOUT " DESC ($nAttenuatorExceptions_DESC): @{attenuatorExceptions_DESC}\n";
    print STDOUT "\n\n";
