#!/usr/bin/perl -W
#--------------------------------------------------------------
# This program set attenuation to the value given in the command line.
#
#     		Author: Sergey Stepanov
#
# Version-1.0:  2007/04/03 using Pezca
# Version-1.1:  2009/05/21 some cleanup
#--------------------------------------------------------------
  use Pezca;
  use DBI;
# use warnings::unused;
  use strict;

  our ($beamline, $ATTENUATION_USE_FACTOR);
  my ($PEZCA, $script, $syntax, $narg, $arg, $i, @beamlinelist);
  my ($status, $attenuation, $attenuation_actual);
  my ($attenuationUnits, $bl_specified, $at_specified);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/attenuation_subs_v2.pl';
  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDOUT; $|=1;			# set unbuffered output

  $script  = &makeScriptName();
  $syntax = "\n".'Syntax: '.$script.' [beamline] attenuation[%|x]'."\n";
  @beamlinelist = ('23i', '23o', '23b', '23d');
  $attenuationUnits = '%';

  $narg = @ARGV;

  if ($narg < 1 || $narg > 2) {
     &press_key_exit($syntax);
  }

###------------------------------------------------------------------------
### Parse command line arguments:

  for ($i=0; $i<$narg; $i++) {
     $arg = lc $ARGV[$i];
     if (grep(/^$arg$/, @beamlinelist)) {
	if (defined $bl_specified) {
           &press_key_exit('*** '.$script.': Duplicate beamline parameter in the command line.'.$syntax);
	}
	$bl_specified=1;
        if ($arg ne $beamline) {
           $beamline = $arg;
           print STDOUT "\n".'!!! Cross-operation for beamline '.$beamline."\n\n";
           do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
	}
     }
     else {
	if (defined $at_specified) {
           &press_key_exit('*** Duplicate attenuation parameter in the command line.'.$syntax);
	}
	$at_specified=1;
        $attenuation = $arg;
        if ($attenuation =~ /x$/) {
	  $attenuationUnits = 'x';
	  $attenuation =~ s/x$//;
	}
	elsif ($attenuation =~ /%$/) {
	  $attenuationUnits = '%';
	  $attenuation =~ s/%$//;
	}
        if (!($attenuation =~ /^[+]?[0-9]*[.]?[0-9]*[Ee]?[+-]?[0-9]*$/)) {
           &press_key_exit('*** Incorrect command line: attenuation must be a positive number.'.$syntax);
        }
     }
  }

  if ($attenuationUnits ne '%') {$ATTENUATION_USE_FACTOR = 1;}

  ($status,$attenuation_actual) = &setAttenuation1($attenuation);
  if ($status) {
     print STDOUT '*** '.$script.' failed with status='.$status."\n";
  } else {
     printf STDOUT $script.' succeeded: Rqs=%.3g%s, Act=%.3g%s'."\n",
                  $attenuation, $attenuationUnits, $attenuation_actual, $attenuationUnits;
  }
  &clearAllCaMonitors();
  exit;

