#!/usr/bin/perl -W
# _CALLED_FROM_BLUICE_
#--------------------------------------------------------------
# This script implements automatic sample centering with video
# from Axis/Digital camera/server and XREC from victor lamzin
# (http://www.embl-hamburg.de/xrec)
#
#     	Authors: Sudhir Pothineni and Sergey Stepanov
#
# Version-1.0:  2006/05/16 using Pezca, LWP and PerlMagick
# Version-2.0:  2009/03/25 uses single camera at low zoom and high zoom
# Version-2.1:  2009/09/22 adapted to work with the collimatorCtrl script
# Version-2.2:  2011/01/07 change to output popups via JBluIce (SS)
# Version-2.3   2013/04/24 changed to work with sampleEnvSrv
# Version-2.4   2013/05/03 added crystal-XYZ or gonio-SH switch
# Version-2.5   2014/03/28 changed functions calls to arrays pointers (preparation to Perl-5.16)
# Version-2.6   2014/07/28 changed re-linking gonio: added test for the result & multiple attempts
# Version-2.7   2016/06/27 some cleanup
# Version-2.8   2017/12/04 moved hardcoded parameters (brightness, zoom...) to command line
#--------------------------------------------------------------
# Algorithm (due to Ward Smith):
#
# Insert backlight, remove collimator
#
# for Low res camera:
# 1 take six images with axis_snapshot at Omega 0,60,120,180,240,300 for lowzoom
# & label files image1,2,3,4,5,6
# 2 take nine images with digital camera at Omega 0 40 80 120 160 200 240 280 320
# 2 convert files from .jpg to .png
# 3 run xrec with saved parameter file i.e. xrec input_file output_file
# 4 if success - adjust x,y if not exit
#
#
#--------------------------------------------------------------
  use strict;
  use LWP::UserAgent;           # World-Wide Web library for Perl (libwww-perl)
  use Image::Magick;            # ImageMagick API for Perl
  use DBI;			# MySQL interface (query camera type)
  use File::Spec;		# uncomment only when using display option
# use File::Copy;		# to copy/move files (does not take wildcards)
  use List::Util qw(min max);	# min/max functions (for zooming and cropping)
# use Math::Trig;		# needed for mono Theta <-> Energy calculations
  use Pezca;

  our ($beamline, $HAVE_XYZ);
  our ($IGNORE_ERRORS, $NONINTERACTIVE, $USE_ZENITY_FOR_EXIT, $ZENITY_VIA_BLUICE);
  our ($image, $inpDir, $imageDir, $imgType, $imgDir);
  my ($PEZCA, $script, @beamlinelist, $arg, $tell_bluice);
  my (@anglesLow, @idxLow, @anglesHigh, @idxHigh);
  my ($file, $lowzoom, $highzoom, $zoomAssyHigh, $zoomAssyLow, $zoomAxisHigh, $zoomAxisLow);
  my ($hilo, $xcenter, $ycenter, $radius, $angle, $status, $width, $height);
  my ($found, $xpixel, $ypixel, $zpixel, $video, $clickString, $motionTimeout);
  my ($bkgPrefix, $sleep_interval, $degr2rad, $nangles, $syntax);
  my ($omegaAssyPV, $omegaAxisPV, $pointclickPV);
  my (@ct, $min, $hour, $mday, $monN, $year, $dateStamp, $message);
  my (@argArray, @types, @ips, $type, $ip, $i);
  my ($targetangle, $reliability, $str);
  my ($inpfile, $r2dfile, $r3dfile, $filehandle);
  my (@axesXY, $assyXY, $axisSH, $assySH, $omegaTolerance, $naxes);
  my ($omegaMtrPV, $omegaMtrNo, $rsp, $omegaJogSpeed, $omegaPrgSpeed);
  my ($omegaScalePV, $omegaScale, $virtualMtrNo, $formula, $pcsNo, $need_relink);
  my ($backbright, $ringbright, $ringlightPV, $backlightPV, $have_back, $have_ring);
  my ($filter, $nprocesses, $autocenterCmd, @cmd);
  my ($xrec, $archdir);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/video_subs.pl';
  do $PEZCA.'/Common/motor_subs.pl';
  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDOUT; $|=1;					# set unbuffered output
  select STDERR; $|=1;					# set unbuffered output

  $SIG{'HUP'}  = 'signal_handler';			#(1) hangup (see signum.h)
  $SIG{'INT'}  = 'signal_handler';			#(2) interrupt (see signum.h)
  $SIG{'QUIT'} = 'signal_handler';			#(3) quit (see signum.h)
  $SIG{'KILL'} = 'signal_handler';			#(9) this is what is sent by kill -9 -- unblockable
  $SIG{'TERM'} = 'signal_handler';			#(15) terminate -- sent by kill without paramaters
  $SIG{'STOP'} = 'signal_handler';			#(19) stop -- unblockable
#-----------------------------------------------------------------
  $script = &makeScriptName();

  if ($^O =~ /Win/i) {die $script.': cannot run under Windows.'."\n";}

  printf STDOUT "\n%s\n", scalar(localtime);

  $IGNORE_ERRORS    	= 1;		# do not exit on error
  $NONINTERACTIVE   	= 1;		# do not wait for any answers
  $USE_ZENITY_FOR_EXIT 	= 1;

  @beamlinelist = ('23i:', '23o:', '23b:', '23d:');
  @anglesLow    = (40, 80, 120, 160, 200, 240);
  @idxLow       = (0,  1,   2,   3,  4,  5);
  @anglesHigh   = (40, 80, 120, 160, 200, 240, 280, 320, 360);
  @idxHigh      = (0, 1, 2, 3, 4, 5, 6, 7, 8);

  $inpDir    = './';			# inpfile=${inpDir}${beamline}_xrec_${hilo}.inp
  $imageDir  = './';
  $imgType   = 'png';
# $imgType   = 'tif';			# TIFF make smaller files than PNG
# $bkgPrefix = 'background';		# expansion: ${imageDir}${beamline}_${bkgPrefix}_${hilo}.${imgType}
  $bkgPrefix = '';			# set to '' if no background used
  $sleep_interval = 0.05;
  $motionTimeout  = 10;
  $omegaTolerance = 0.1;                # in degrees
  $tell_bluice    = 0;			# default if no cmdline argument is given

  $degr2rad = 45./atan2(1.,1.);		# atan2(1)=pi/4
  $nangles  = @anglesLow;

  $imgDir   = '/dev/shm/';		# this is tmpfs using RAM
  $archdir  = '/home/gmca/shared/Sudhir/XREC/USER_RUNS/';
# $xrec     = '/home/gmca/shared/Sudhir/XREC/xrec';
  $xrec     = '/mnt/software/px/XREC/xrec';

#---------------------------------------------------------------------------
### Protection from multiple instances:
  $filter = '/bin/ps -eo user:19,pid,cmd | '
           .'/bin/grep -iv "grep\|edit\|term\|onsole" | '
           .'/bin/grep ';
  $nprocesses  = `${filter} -c autocenter_xrec`;
  chomp $nprocesses;
  $nprocesses--;						#ignore this script
  if ($nprocesses > 0) {
     $NONINTERACTIVE      = 1;				#do not wait for any answers
     $USE_ZENITY_FOR_EXIT = 1;
     &zenity_error_exit('*** '.$nprocesses.' instance(s) of '.$script.' not yet finished. Please try again.');
  }
#---------------------------------------------------------------------------
### Read command line:
  $syntax = "\n\n".'Syntax: '.$script.' [beamline] [bluice|nobluice]'."\n"
                  .'[haveringlight|noringlight] [havebacklight|nobacklight]'."\n"
                  .'[backlight=x.y] [ringlight=x.y] [lowzoom=x.y] [highzoom=x.y]'."\n\n";
  foreach $arg (@ARGV) {
     if (grep(/^$arg:?$/, @beamlinelist)) {
        $arg =~ s/:$//;
        if ($arg ne $beamline) {
           $beamline = $arg;
           print STDOUT "\n".'!!! '.$script.': Cross operation for beamline '.$beamline.' requested!'."\n";
           do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
        }
     }
     elsif ($arg =~ /^bluice$/i)        {$tell_bluice = 1;}
     elsif ($arg =~ /^nobluice$/i)      {$tell_bluice = 0;}
     elsif ($arg =~ /^haveringlight$/i) {$have_ring = 1;}
     elsif ($arg =~ /^noringlight$/i)   {$have_ring = 0;}
     elsif ($arg =~ /^havebacklight$/i) {$have_back = 1;}
     elsif ($arg =~ /^nobacklight$/i)   {$have_back = 0;}
     elsif ($arg =~ /^backlight=/i) {
        $arg =~ s/^backlight=//i;
        if (! &is_decimal($arg)) {&press_key_exit('*** '.$script.': backlight brightness='.$arg.' is not decimal.'.$syntax);}
        $backbright = $arg;
     }
     elsif ($arg =~ /^ringlight=/i) {
        $arg =~ s/^ringlight=//i;
        if (! &is_decimal($arg)) {&press_key_exit('*** '.$script.': ringlight brightness='.$arg.' is not decimal.'.$syntax);}
        $ringbright = $arg;
     }
     elsif ($arg =~ /^lowzoom=/i) {
        $arg =~ s/^lowzoom=//i;
        if (! &is_decimal($arg)) {&press_key_exit('*** '.$script.': lowzoom='.$arg.' is not decimal.'.$syntax);}
        $lowzoom = $arg;
     }
     elsif ($arg =~ /^highzoom=/i) {
        $arg =~ s/^highzoom=//i;
        if (! &is_decimal($arg)) {&press_key_exit('*** '.$script.': highzoom='.$arg.' is not decimal.'.$syntax);}
        $highzoom = $arg;
     }
     else {
        &press_key_exit('*** '.$script.': Unknown cmdline argument: '.$arg.$syntax);
     }
  }
#---------------------------------------------------------------------------
  if ($tell_bluice) {$ZENITY_VIA_BLUICE = 1;}


  if (!defined $lowzoom)   {$lowzoom   = 6.0;}
  if (!defined $highzoom)  {$highzoom  = 6.0;}
  if (!defined $have_back) {$have_back = 1;}
  if    ($beamline =~ /^23o/i) {
     if (!defined $backbright) {$backbright = 3.0;}
     if (!defined $ringbright) {$ringbright = 4.0;}
     if (!defined $have_ring)  {$have_ring  = 0;}
  }
  elsif ($beamline =~ /^23i/i) {
     if (!defined $backbright) {$backbright = 2.6;}
     if (!defined $ringbright) {$ringbright = 4.0;}
     if (!defined $have_ring)  {$have_ring  = 0;}
  }
  else {
     if (!defined $backbright) {$backbright = 2.6;} 
     if (!defined $ringbright) {$ringbright = 5.0;}
     if (!defined $have_ring)  {$have_ring  = 1;}
  }

  if ($have_ring == 0) {$ringbright = 0.0;}
  if ($have_back == 0) {$backbright = 0.0;}

  $zoomAssyHigh = $beamline.':TV:V1';
  $zoomAxisHigh = $beamline.':TV:ZM1';
  $zoomAssyLow  = $beamline.':TV:V2';
  $zoomAxisLow 	= $beamline.':TV:ZM2';
  $omegaAssyPV  = $beamline.':GO:Om';
  $omegaAxisPV  = $beamline.':GO:OM';
  $omegaMtrPV   = $beamline.':GO:mom';
  $omegaScalePV = $beamline.':GO:OM:Scale';

  $pointclickPV = $beamline.':bi:pointclick';

  @axesXY       = ($beamline.':GO:CX', $beamline.':GO:CY');
  $assyXY       = $beamline.':GO:Cr';
  if (defined $HAVE_XYZ) {
     push (@axesXY, $beamline.':GO:CZ');
  } else {
     $axisSH    = $beamline.':GO:SH';
     $assySH    = $beamline.':GO:St';
     if ($beamline eq '23b') {$assySH .= 'B';}
  }
  $naxes = @axesXY;

  $ringlightPV  = $beamline.':DAC1_4.VAL';
  $backlightPV  = $beamline.':DAC1_8.VAL';

  if ($beamline eq '23b') {$assySH .= 'B';}

  ($status,$omegaScale) = Pezca::GetDouble($omegaScalePV);
  if ($status) {&report_only($script,$status,$omegaScalePV); $omegaScale=0;}

  $virtualMtrNo = 32;				#virtual motor linked to Omega in pmac20
  $omegaMtrNo   = 5;				#gonio Omega motor#: hardcoded to save time
# $omegaMtrNo   = &getMtrNo($omegaMtrPV);	#we use hardcoded value
  $need_relink  = 0;
  &set_omega_speed_4x();

####xrec

### Move collimator out:
  @cmd = ('OUT', $beamline);
  if ($have_back) {push(@cmd,'backlight='.$backbright);}
  if ($have_ring) {push(@cmd,'backlight='.$ringbright);}
  $status = &setSampleEnv(\@cmd);
  if ($status) {
     if ($tell_bluice) {&tellBluIce('autoCntr: coll-OUT failed','error');}
     &handle_press_key_exit('*** '.$script.': Cannot move collimator out: status='.$status);
  }

  $status = &addCaMonitor ($pointclickPV, 'string');
  &report_only($script, $status, $pointclickPV);

  @ct = localtime(time);		#($sec,$min,$hour,$mday,$mon,$year$,$wday,$yday,$isdst)
  ($min,$hour,$mday,$monN,$year) = ($ct[1],$ct[2],$ct[3],$ct[4]+1,$ct[5]+1900);
# $mon  = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec')[$ct[4]];
# $dateStamp = sprintf '%04d%s%02d_%02d%02d',$year,$mon,$mday,$hour,$min;
  $dateStamp = sprintf '%04d-%02d-%02d_%02d-%02d',$year,$monN,$mday,$hour,$min;
  print STDOUT $script.': Getting camera types from mySQL'."\n";
  $status = &getCameraTypes(\@types,\@ips);		#data order: HiRes/LoRes/Hutch
  if ($status) {&handle_press_key_exit('*** '.$script.': error='.$status.' from getCameraTypes');}

  printf STDOUT "\n%s\n", scalar(localtime);
  $xpixel = 0;
  $radius = 0;

### Loop over cameras:
  foreach $hilo ('LoRes','LoRes','LoRes','HiRes') {
# foreach $hilo ('LoRes') {
# foreach $hilo ('HiRes') {
     ### Take snapshots:
     if ($tell_bluice) {&tellBluIce('autoCentering started ..');}
     # 423 pixels : size of pin length from the tip to copper stem
     if ($hilo =~ /^LoRes$/i) {
        if($xpixel > 423 || $xpixel < 100 || $radius > 230 ) {  }
        else { next; }
     }

     if    ($hilo =~ /^HiRes$/i) {$ip = $ips[0]; $type = $types[0];}
     elsif ($hilo =~ /^LoRes$/i) {$ip = $ips[1]; $type = $types[1];}
     else {&handle_press_key_exit('*** '.$script.': incorrect camera = '.$hilo);}
     $status = &runMotor ($omegaAssyPV, $omegaAxisPV, 0, 'checkAuto');
     if ($status) {
        if ($tell_bluice) {&tellBluIce('autoCenter: Omega failed','error');}
        &handle_press_key_exit('*** '.$script.': Omega failed.',$omegaAssyPV,$omegaAxisPV);
     }
     if ($hilo =~ /^LoRes$/i) {
        $i = 0;
        print STDOUT $script.': Moving LoRes zoom to '.$lowzoom."\n";
        $status = &runMotor ($zoomAssyLow, $zoomAxisLow, $lowzoom, 'checkAuto');
        if ($status) {
           if ($tell_bluice) {&tellBluIce('autoCenter: lowzoom failed','error');}
           &handle_press_key_exit('*** '.$script.': lowzoom failed.');
        }

        print STDOUT $script.': Moving HiRes zoom to '.$highzoom."\n";
        $status = &runMotor ($zoomAssyHigh, $zoomAxisHigh, $highzoom, 'checkAuto');
        if ($status) {
           if ($tell_bluice) {&tellBluIce('autoCenter: highzoom failed','error');}
           &handle_press_key_exit('*** '.$script.': highzoom failed.',$omegaAssyPV,$omegaAxisPV);
        }

	if ($have_back) {
           $status = Pezca::PutDouble($backlightPV, $backbright);
           &report_only($script,$status,$backlightPV);
	}

        foreach $angle (@anglesLow) {
           print STDOUT $script.': Taking image-'.$i."\n";
           ($image, $width, $height) = &takeImage ($dateStamp,$hilo,$i,$ip,$type);

           printf STDOUT $script.': Starting motion of Omega to %3.2f degr.'."\n",$angle;
#          $status = &runMotor ($omegaAssyPV, $omegaAxisPV, $angle, 'checkAuto');
           $status = &startNMotors ($omegaAssyPV, 1, [$omegaAxisPV], [$angle], 'checkAuto');
           if ($status) {
             if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
             &handle_press_key_exit('*** '.$script.': Omega start failed.');
           }
           print STDOUT $script.': Saving image-'.$i."\n";
           &saveImage ($dateStamp,$hilo,$i,$ip,$type,$image);
           printf STDOUT $script.': Waiting for Omega to arrive at %3.2f degr.'."\n",$angle;
#          $status = &trackMotor ($omegaAssyPV, $motionTimeout, $omegaAxisPV);
	   $status = &trackMotorFast ($omegaAssyPV, $motionTimeout, $omegaAxisPV, $angle, $omegaTolerance, 'RunPrg');
           if ($status) {
              if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
              &handle_press_key_exit('*** '.$script.': Omega track failed.',$omegaAssyPV,$omegaAxisPV);
           }
           $i++;
        }
     }		#endof taking images with LoRes camera
     if ($hilo =~ /^HiRes$/i) {
        $i = 0;

        #if ($beamline =~ /^23o/i) {
	$status = Pezca::PutDouble($ringlightPV, 0.0);
	&report_only($script,$status,$ringlightPV);
        #}
	if ($have_back) {
           $status = Pezca::PutDouble($backlightPV, $backbright);
           &report_only($script,$status,$backlightPV);
	}

        foreach $angle (@anglesHigh) {

           print STDOUT $script.': Taking image-'.$i."\n";
           ($image,$width, $height) = &takeImage ($dateStamp,$hilo,$i,$ip,$type);

           printf STDOUT $script.': Starting motion of Omega to %3.2f degr.'."\n",$angle;
#          $status = &runMotor ($omegaAssyPV, $omegaAxisPV, $angle, 'checkAuto');
           $status = &startNMotors ($omegaAssyPV, 1, [$omegaAxisPV], [$angle], 'checkAuto');
           if ( $status ) {
              if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
              &handle_press_key_exit('*** '.$script.': Omega start failed.');
           }
           print STDOUT $script.': Saving image-'.$i."\n";
           &saveImage ($dateStamp,$hilo,$i,$ip,$type,$image);
           printf STDOUT $script.': Waiting for Omega to arrive at %3.2f degr.'."\n",$angle;
#          $status = &trackMotor ($omegaAssyPV, $motionTimeout, $omegaAxisPV);
	   $status = &trackMotorFast ($omegaAssyPV, $motionTimeout, $omegaAxisPV, $angle, $omegaTolerance, 'RunPrg');
           if ($status) {
              if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
              &handle_press_key_exit('*** '.$script.': Omega track failed.');
           }
           $i++;
        }
     }		#endof taking images with HiRes camera
     printf STDOUT "\n%s\n", scalar(localtime);
### Execute XREC:
     $inpfile = $imgDir.$inpDir.$beamline.'_'.$dateStamp.'_'.$hilo.'.inp';
#    $r2dfile = $inpDir.$beamline.'_'.$dateStamp.'_'.$hilo.'.r2d';
     $r3dfile = $imgDir.$inpDir.$beamline.'_'.$dateStamp.'_'.$hilo.'.r3d';
### Settings common for both high-res and low-res cameras:
### Sudhir:
     if ($hilo =~ /^LoRes$/i) {
        @argArray = ( 	'LOOP_POSITION 2',
			'NUM_IMG 6',
			'PREALIGN',
#			'CENTRE_COORD 240',
#			'SET_CROSS',
			'DATA_START',
			'0 '             .$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxLow[0].'.'.$imgType,
			$anglesLow[0].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxLow[1].'.'.$imgType,
			$anglesLow[1].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxLow[2].'.'.$imgType,
			$anglesLow[2].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxLow[3].'.'.$imgType,
			$anglesLow[3].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxLow[4].'.'.$imgType,
			$anglesLow[4].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxLow[5].'.'.$imgType,
			'DATA_END');
     }
     if ($hilo =~ /^HiRes$/i) {
        @argArray = ( 	'LOOP_POSITION 2',
			'NUM_IMG 9',
			'PREALIGN',
#			'CENTRE_COORD 240',
#			'SET_CROSS',
			'DATA_START',
			'0 '              .$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[0].'.'.$imgType,
			$anglesHigh[0].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[1].'.'.$imgType,
			$anglesHigh[1].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[2].'.'.$imgType,
			$anglesHigh[2].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[3].'.'.$imgType,
			$anglesHigh[3].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[4].'.'.$imgType,
			$anglesHigh[4].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[5].'.'.$imgType,
			$anglesHigh[5].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[6].'.'.$imgType,
			$anglesHigh[6].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[7].'.'.$imgType,
			$anglesHigh[7].' '.$imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idxHigh[8].'.'.$imgType,
			'DATA_END');
     }
### If background file has to be used:
### Sudhir:
     if ($bkgPrefix ne '') {
#       push @argArray, 'BACK '.$beamline.'_'.$bkgPrefix.'_'.$overidehilo.'.'.$imgType;
     }
### Settings specific for low-res camera only:
### Sudhir:
     if ($hilo =~ /^LoRes$/i) {
#       push @argArray, 'PREALIGN';
     }
### Settings specific for high-res camera only:
     if ($hilo =~ /^HiRes$/i) {
#       push @argArray, 'PREALIGN';
#       push @argArray, '-cin=\<dark\>';
#       push @argArray, '-lrm=3';
#       push @argArray, '-mdp';
#       push @argArray, '-mme=150';
#       push @argArray, '-tbm=3';
     }
     printf STDOUT "\n%s\n", scalar(localtime);
### Generate INP-file for xrec:
     $filehandle = open (INP,'> '.$inpfile);
     if (!defined $filehandle) {
        &handle_press_key_exit('*** '.$script.': Cannot open "'.$r3dfile.'"');
     }
     select INP; $|=1;			# set unbuffered output
     foreach $arg (@argArray) {print INP $arg."\n";}
     close(INP);
     if ($tell_bluice) {&tellBluIce('XREC processing the images..');}
### Execute xrec:
     $autocenterCmd = $xrec.' '.$inpfile.' '.$r3dfile.' 2>/dev/null';
     $status = system($autocenterCmd);

#    if ($status || !(-e $r2dfile) || !(-e $r3dfile)) {
     if ($status || !(-e $r3dfile)) {
        &handle_press_key_exit('*** '.$script.': XREC failed.');
     }
     ($targetangle,$radius, $xcenter, $ycenter, $reliability) = &readRes3dFile($r3dfile);
     print STDOUT $targetangle.', '.$radius.', '.$xcenter.', '.$ycenter.', '.$reliability."\n";
     printf STDOUT "\n%s\n", scalar(localtime);
#    if ( ! $found ) {
#       if ($tell_bluice) {&tellBluIce('autoCenter: '.$hilo.'-center fail','error');}
#       &handle_press_key_exit('*** '.$script.': XREC did not find the loop!');
#    }
#    $targetangle=-$targetangle;
### Move Omega to angle specified by xrec:
     print STDOUT $script.': Moving Omega to '.$targetangle.' given by XREC'."\n";
#    $targetangle = $anglesLow[$nangles-1] - $targetangle;

     if ($targetangle <  0.)  {$targetangle = $targetangle + 360.;}
     if ($targetangle > 360.) {$targetangle = $targetangle - 360.;}
     $status = &runMotor ($omegaAssyPV, $omegaAxisPV, $targetangle,'checkAuto');
     if ( $status ) {
        if ($tell_bluice) {&tellBluIce('autoCenter: omega failed','error');}
        &handle_press_key_exit('*** '.$script.': Omega failed.');
     }
     if ($tell_bluice) {&tellBluIce('Making Auto centering..');}
     if    ($hilo =~ /^HiRes$/i) {
        $str = sprintf 'hires Reliability: %2.1f%s',$reliability,'%';
        if ($tell_bluice) {&tellBluIce($str);}
     }
     elsif ($hilo  =~ /^LoRes$/i) {
        $str = sprintf 'lowres Reliability: %2.1f%s',$reliability,'%';
        if ($tell_bluice) {&tellBluIce($str);}
     }
### MoveXYhead conventions on cameras: 0=LowRes 1=HiRes  #
     if    ($hilo =~ /^HiRes$/i)  {$video = 1;}
     elsif ($hilo  =~ /^LoRes$/i) {$video = 0;}
     else                         {$video = '';}
#    $xpixel = int($xcenter + $radius * sin($targetangle*$degr2rad));
### offset 22
     if    ($hilo =~ /^HiRes$/i) {
        $xpixel  =  int($ycenter);
     }
     elsif ( $hilo  =~ /^LoRes$/i ) {
        $xpixel  =  int($ycenter); 	  #offset lowest loop size by hampton 25um = 22 pixels horizontal
        if ($ycenter < 1) {$xpixel = 0;}
     }
     else {
        $xpixel  =  int($ycenter);
     }
#    $zpixel  =  int($ycenter);
#    $ypixel = int($xcenter + $radius * cos($targetangle*$degr2rad));
#    $ypixel = int($ycenter);
     $ypixel = int($xcenter+$radius);
     if ($ypixel > $height)          {$ypixel=$height};
     if ($ypixel < 1 || $radius < 1) {$xpixel=0; $ypixel=240;};
     $clickString = $video.' '.$xpixel.' '.$ypixel.' '.$width.' '.$height;

     print STDOUT $script.': Requesting Point & Click Server to process '.$pointclickPV.' '.$clickString."\n";
     $status = Pezca::PutString($pointclickPV,$clickString);
     if ($status) {
        if ($tell_bluice) {&tellBluIce('autoCenter: click failed','error');}
        &handle_press_key_exit('*** '.$script.': click failed.');
     }
### Now wait it to finish (while moveXYhead.pl finishes):
     print STDOUT $script.': Waiting Point & Click Server to finish'."\n";
#    do {
#       select(undef,undef,undef,$sleep_interval);    #short sleep
#       ($status, $clickString) = Pezca::GetString($pointclickPV);
#       if ($status) {                              #Connection lost
#          &report_only($script, $status, $pointclickPV);
#          if ($tell_bluice) {&tellBluIce('autoCenter: click failed','error');}
#          sleep 1;                                    #wait before trying again
#       }
#    } while ($clickString ne '');

     select(undef,undef,undef,0.2);    		    #give P&C server time to start motion
### Track XY positioner motion:
     if ($ypixel != 0 || defined $HAVE_XYZ) {
        $status = &trackNMotors($assyXY,$motionTimeout,$naxes,\@axesXY);
        if ($status) {				    #Report error to BluIce:
           if ($tell_bluice) {&tellBluIce('autoCenter: XY failed','error');}
           &handle_press_key_exit('*** '.$script.': Tracking XY detected timeout.');
        }
        print STDOUT $script.': Finished tracking '.$assyXY."\n";
     }
### Track support horizontal motion:
     if ($xpixel != 0 && !defined $HAVE_XYZ) {
        $status = &trackMotor($assySH,$motionTimeout,$axisSH);
        if ($status) {				    #Report error to BluIce:
           if ($tell_bluice) {&tellBluIce('autoCenter: SH failed','error');}
           &handle_press_key_exit('*** '.$script.': Tracking SH detected timeout.');
        }
        print STDOUT $script.': Finished tracking '.$assySH."\n";
     }
     print STDOUT $script.': Point & Click Server done'."\n";
#    printf STDOUT "\n%s\n", scalar(localtime);
#    mkdir $archdir.$beamline.'_'.$dateStamp.'_'.$hilo.'/';
#    system('/bin/mv cross* '.$archdir.$beamline.'_'.$dateStamp.'_'.$hilo.'/');
  }	# end of the main loop over all cameras (foreach...)

## check reliability 70 is emperical value
  if ($reliability < 70 && $reliability > 0)  {
#    if ($tell_bluice) {&tellBluIce('autoCenter: Centering relibility is low, please check centering','popup_note');}
	if ($tell_bluice) {&tellBluIce('autoCenter: Centering relibility is low, please check centering');}
  }
  if ($reliability <= 0 )  {
#    if ($tell_bluice) {&tellBluIce('autoCenter: No sample found in the field of view, please check','popup_note');}
     if ($tell_bluice) {&tellBluIce('autoCenter: No sample found in the field of view, please check');}
  }
  printf STDOUT $script.': Rotating Omega to 0'."\n";
  $status = &runMotor ($omegaAssyPV, $omegaAxisPV, 0, 'checkAuto');

  &restore_omega();
  &clearAllCaMonitors();
  print STDOUT 'Moving images '.$imgDir.$beamline.'* to '.$archdir."\n";
  system('/usr/bin/sudo -n /bin/mv -f '.$imgDir.$beamline.'* '.$archdir.' 2>/dev/null');
  if ($tell_bluice) {&tellBluIce('autoCenter: finished');}
# printf STDOUT "\n%s\n", scalar(localtime);
  exit 0;

#=================================================================
# save image
sub saveImage ($$$) {
  our ($beamline, $axisURL, $imageDir, $imgType, $imgDir, $image);
  my ($dateStamp, $hilo, $idx, $ip, $type);

  $dateStamp = shift(@_);
  $hilo      = shift(@_);
  $idx       = shift(@_);
  $ip        = shift(@_);
  $type      = shift(@_);
  $image     = shift(@_);

  $file = $imgDir.$beamline.'_'.$dateStamp.'_'.$hilo.$idx.'.'.$imgType;
  print STDOUT $script.': Camera-'.$hilo.': saving image as '.$file."\n";
  $image->Write($file);
}

#=================================================================

# sub takeImage ($dateStamp,$hilo,$idx,$ip,$type) {
sub takeImage ($$$) {
  our ($image, $beamline, $axisURL, $imageDir, $imgType, $imgDir);
  my  ($dateStamp, $hilo, $idx, $ip, $type);
  my  ($compression, $colorlevel, $rotation, $overidehilo);
  my  ($downsample, $display, $status);

  $dateStamp = shift(@_);
  $hilo      = shift(@_);
  $idx       = shift(@_);
  $ip        = shift(@_);
  $type      = shift(@_);

### Camera and image options:
  $compression = 0;		# can be 0, 25, 50, 75, 100
  $colorlevel  = 100;		# can be ??? /0 = grayscale, 100 = full color/
  $rotation    = 0;
  $overidehilo = 'HiRes';

  $downsample = 4;
# $display = 0;
# $status = &saveCameraSnap($file,$hilo,$display,-$downscale,$type,$ip,$compression,$colorlevel,$rotation);
  $status = &getCameraSnap(\$image,$overidehilo,-$downsample,$type,$ip,$compression,$colorlevel,$rotation);
  if ($status) {&handle_press_key_exit('*** '.$script.': saveCameraSnap failed on camera='.$overidehilo.' with status='.$status);}


### Convert the image to grayscale:
### (do we need it?)
# $image->Quantize(colorspace=>'gray');

  $width  = $image->Get('columns');
  $height = $image->Get('rows');

  return $image,$width, $height;
}

#=================================================================

# sub readRes3dFile($r3dfile) {
#sudhir
sub readRes3dFile($) {
  my ($r3dfile, $string, $filehandle, @stringWords);
  my (@stringwords1, @stringwords2, @stringwords3);
  my (@stringwords4, @stringwords5, @stringwords6);
  my ($flag, $found, $xc, $yc, $angle);
# my ($T, $GBA, $QSel, $QCentr, $QImage, $Score, $Comment);

  $r3dfile = shift(@_);
  $string = '';
### c3d 3D result file format:
# ($flag, $found, $xc, $yc, $R, $A0, $Rmse, $NUI, $H, $L, $TILT);
#       0       1     2     3  4   5     6     7     8  9    10  .....
  $filehandle = open (DAT,'< '.$r3dfile);
  print STDOUT $script.': Reading "'.$r3dfile.'"'."\n";
  if (!defined $filehandle) {
     &handle_press_key_exit('*** '.$script.': readRes3dFile cannot open "'.$r3dfile.'"');
  }
# while ( <DAT> ) {
### read line
#    $string = $_;
#    if ($string =~ /^\#/) { next; }		# skip comments
#    chomp($string);
#    @stringWords = split ' ', $string;  	# split into words
#    @stringWords = split "\n", $string;
#    if ($stringWords[0] eq 'TARGET_ANGLE') {last;}
#    close (DAT);
#    &handle_press_key_exit('*** '.$script.': readRes3dFile found incorrect file structure in "'.$r3dfile.'"');
# }
  $string=<DAT>;
  @stringwords1 = split ' ', $string;
  $string=<DAT>;
  @stringwords2 = split ' ', $string;
  $string=<DAT>;
  @stringwords3 = split ' ', $string;
  $string=<DAT>;
  @stringwords4 = split ' ', $string;
  $string=<DAT>;
  @stringwords5 = split ' ', $string;
  $string=<DAT>;
  @stringwords6 = split ' ', $string;

  close(DAT);
# if ($stringWords[0] eq 'TARGET_ANGLE') {
     $targetangle = $stringwords1[1];
# }
# if($stringWords[2] eq 'RADIUS') {
     $radius = $stringwords2[1];
# }
# if ($stringWords[4] eq 'Y_CENTRE') {
     $ypixel = $stringwords3[1];
# }
# if ($stringWords[6] eq 'X_CENTRE') {
     $xpixel = $stringwords4[1];
# }
# if ($stringWords[10] eq 'RELIABILITY') {
     $reliability = $stringwords5[1];
# }
  print STDOUT "\n".'*** '.$stringwords1[0].'  '.$stringwords1[1].'  '.$targetangle."\n";
  print STDOUT "\n".'*** '.$stringwords2[0].'  '.$stringwords2[1].'  '.$radius."\n";
  print STDOUT "\n".'*** '.$stringwords3[0].'  '.$stringwords3[1].'  '.$ypixel."\n";
  print STDOUT "\n".'*** '.$stringwords4[0].'  '.$stringwords4[1].'  '.$xpixel."\n";
  print STDOUT "\n".'*** '.$stringwords5[0].'  '.$stringwords5[1].'  '.$reliability."\n";

  return ($targetangle,$radius, $xpixel, $ypixel, $reliability);
}

#=================================================================

sub set_omega_speed_4x() {

  my ($omegaJobSpeedDeg, $omegaPrgSpeedDeg);
  if (&isMtrOutLinked($omegaAssyPV,$virtualMtrNo) == 1) {
### If the virtual motor is linked to Omega,
### then read the linking equation & unlink:
     $formula = &readPcsRsp ($omegaAssyPV, '#'.$virtualMtrNo.'->');
     print STDOUT $script.': Un-linking Omega from virtual motor (formula='.$formula.')'."\n";
     &unlinkMotor_Pcs($omegaAssyPV, $virtualMtrNo);
     $need_relink = 1;
  }
### Read current Omega jog speed:
  $omegaJogSpeed = &readMtrRsp($omegaMtrPV,'I'.$omegaMtrNo.'22','numeric');
  print STDOUT $script.': Omega motor normal speed = '.$omegaJogSpeed.' cts/msec'."\n";
  if (length($omegaJogSpeed) > 0 && $omegaJogSpeed =~ /^[+]?[0-9]*[.]?[0-9]*$/) {
     $omegaJobSpeedDeg = int ($omegaJogSpeed * $omegaScale * 1000 + 0.5);
     print STDOUT $script.': Omega motor normal speed = '.$omegaJobSpeedDeg.' degr/sec'."\n";
     ### Set Omega program speed 4x normal:
     $omegaPrgSpeed = 4 * $omegaJogSpeed;
     &sendMtrCmd($omegaMtrPV, 'I'.$omegaMtrNo.'16='.$omegaPrgSpeed);
     ### Verify that the program speed has been set:
     # $omegaPrgSpeed = &readMtrRsp($omegaMtrPV, 'I'.$omegaMtrNo.'16','numeric');
     # print STDOUT $script.': Omega motor prg high speed = '.$omegaPrgSpeed.' cts/msec'."\n";
     $omegaPrgSpeedDeg = int($omegaPrgSpeed * $omegaScale * 1000 + 0.5);
     print STDOUT $script.': Setting Omega motor high speed = '.$omegaPrgSpeedDeg.' degr/sec'."\n";
  } else {
     print STDOUT '*** '.$script.': Incorrect Omega motor jog speed = '.$omegaJogSpeed.' cts/msec'."\n"
		 .'*** '.$script.': Omega speed will not be changed!'."\n\n";
  }
}


#=================================================================

sub restore_omega() {

  my ($omegaJobSpeedDeg, $attempt, $max_attempts, $linked, $msg);
  $max_attempts = 3;
  if (defined $omegaAssyPV && defined $omegaAxisPV && defined $omegaPrgSpeed) {
     # print STDOUT $script.': Restoring Omega motor normal speed = '.$omegaJogSpeed.' cts/msec'."\n";
     $omegaJobSpeedDeg = int ($omegaJogSpeed * $omegaScale * 1000 + 0.5);
     print STDOUT $script.': Restoring Omega motor normal speed = '.$omegaJobSpeedDeg.' degr/sec'."\n";
### This should reset the program speed to the jog speed:
     &setAxisHiSpeed($omegaAssyPV,$omegaAxisPV);
### Read the program speed back to verify that everything is restored:
#    $omegaPrgSpeed = &readMtrRsp($omegaMtrPV,'I'.$omegaMtrNo.'16','numeric');
#    print STDOUT $script.': Omega motor prg restored speed = '.$omegaPrgSpeed."\n";
  }
  if (defined $omegaAssyPV && defined $omegaAxisPV) {
     if ($need_relink) {
        if (defined $formula) {
### Omega was initially linked to the
### virtual motor; restore the link:
	   $attempt = 0;
           do {
	      $attempt++;
              print STDOUT $script.': Re-linking Omega to virtual motor & syncing positions (formula='.$formula.")\n";
              &abortAssy($omegaAssyPV);
              &linkMtrOut ($omegaAssyPV, $virtualMtrNo, $formula, 'sync');
              #&sendPcsCmd($omegaAssyPV, '#'.$virtualMtrNo.' homez');			#Zero motor-OUT position
              #$pcsNo=&getPcs($omegaAssyPV);
              #&sendPcsCmd($omegaAssyPV, '&'.$pcsNo.' #'.$virtualMtrNo.'->'.$formula);	#Link using the formula
              #&sendPcsCmd($omegaAssyPV, 'R');						#Sync the positions
	      $linked = &isMtrOutLinked($omegaAssyPV,$virtualMtrNo);
              if ($linked != 1) {
		 $msg = ': Failed re-linking Omega to virtual motor, attempt-'.$attempt.'/'.$max_attempts;
                 print STDOUT '*** '.$script.$msg."\n";
                 if ($tell_bluice) {&tellBluIce('autoCntr'.$msg,'warn');}
	      }
	   } while ( $attempt < $max_attempts && $linked != 1);
        } else {
	   $msg = ': Failed re-linking Omega to virtual motor because formula is undefined';
           print STDOUT '*** '.$script.$msg."\n";
           if ($tell_bluice) {&tellBluIce('autoCntr'.$msg,'error');}
	}
     }
  }
}

#=================================================================

sub handle_press_key_exit($) {

  $message = shift(@_);
  &restore_omega();
  &press_key_exit($message);
}

#=================================================================

sub signal_handler {
  my $signame = shift;
  if ($signame ne 'quit') {print STDOUT "\n".' *** '.$script.': Caught a SIG'.$signame."\n";}
  &restore_omega();
  &clearAllCaMonitors();
  print STDOUT $script.': Done!'."\n\n";
  exit 1;
}
