#!/usr/bin/perl -W

  use strict;
  use Pezca;
  use List::Util;

  our (@monitor_sclr, @monitor_str);
  our ($n_monitor_sclr, $n_monitor_str, $n_max, $n);
  our (@time, @now, $now_day, $now_min, $ct);
  our ($year, $month, $day, $hour, $min);
  our ($filename, $outdir, $outfile, $filehandle);
  our ($error, $datum, $logdir);
  our (@datum_monitor_sclr, @datum_monitor_sclr_old);
  our (@datum_monitor_str, @datum_monitor_str_old);
  our ($sleeptime);

### Watch PVs from all beamlines
  if (defined $ENV{EPICS_CA_ADDR_LIST_FULL}) {$ENV{EPICS_CA_ADDR_LIST}=$ENV{EPICS_CA_ADDR_LIST_FULL};}
  else                                       {die "EPICS_CA_ADDR_LIST_FULL not defined.\n";}

  if (defined $ENV{HOME}) {$logdir = $ENV{HOME}.'/logfiles/';}
  else                    {die "HOME not defined.\n";}

### Set unbuffered output
  select STDOUT; $|=1;
  print STDOUT "\n\n";

### Check for pre-existing script ----------------------------------------------

  my ($filter, $nprocesses, $filehandle2, $filename2);
  $filter = '/bin/ps -eo user:19,pid,cmd | '
         .'/bin/grep -v grep | '
         .'/bin/grep -v edit | '
         .'/bin/grep -v term | '
         .'/bin/grep -v onsole | '
         .'/bin/grep -v pezca23 | '
         .'/bin/grep ';
  $nprocesses  = `${filter} -c beamline_monitor.pl`;
  chomp $nprocesses;
  $nprocesses--;                             #ignore this script
  if ($nprocesses > 0) {
     $filename2 = ${logdir}.'beamline_monitor_error.log';
     $filehandle2 = open (DUMP,">> $filename2");	# open DAT file for writing
     if (!defined $filehandle2) {
        print STDOUT "Could not open file.\n";
        exit;
     }
     ### Set unbuffered output
     select DUMP; $|=1;
     ### Write logfile header;
     $ct = scalar localtime;
     print DUMP "Attempted to start beamline monitor: ${ct}\n";
     print DUMP "Found $nprocesses process(es) already running.\n\n";
     close DUMP;
     exit;
  }

### Open logfile
  &open_file;

### Signal handler
  $SIG{INT}  = 'signal_exit';
  $SIG{QUIT} = 'signal_exit';

### Initialize lists of PVs
  &monitor_list;

### Set sleeptime
  $sleeptime = 0.02;

### Get number of PVs in each array
  $n_monitor_sclr = @monitor_sclr;
  $n_monitor_str  = @monitor_str;
  $n_max          = List::Util::max($n_monitor_sclr,$n_monitor_str);
  for ($n=0; $n<$n_max; $n++) {
     $datum_monitor_sclr[$n] = -999999;
     $datum_monitor_sclr_old[$n] = -999999;
     $datum_monitor_str[$n] = -999999;
     $datum_monitor_str_old[$n] = -999999;
  }
### Set up monitors
  &set_monitors;

### Get data, repeat
  while (1) {
     @now     = localtime(time);					# This is (sec,min,hour,mday,mon,year,wday,yday,isdst)
     $now_day = $now[3];
     if ($now_day != $day) {
        $ct = scalar localtime;
        print DAT "End of day.  File closed at: ${ct}\n\n";
        close DAT;
        for ($n=0; $n<$n_max; $n++) {
           $datum_monitor_sclr[$n] = -999999;
           $datum_monitor_sclr_old[$n] = -999999;
           $datum_monitor_str[$n] = -999999;
           $datum_monitor_str_old[$n] = -999999;
        }
        &open_file;
     }
     for ($n=0; $n<$n_max; $n++) {
        #print STDOUT "$n\n";
        if ($n < $n_monitor_sclr) {
           ($error, $datum_monitor_sclr[$n]) = Pezca::GetDouble($monitor_sclr[$n]);
           $ct = scalar localtime;
           if ($error) {
              print DAT "$ct \t$monitor_sclr[$n] \tERR\n";
              print STDOUT "$ct \t$monitor_sclr[$n] \tERR\n";
           }
           else {
              if ($datum_monitor_sclr[$n] != $datum_monitor_sclr_old[$n]) {
                 print DAT "$ct \t$monitor_sclr[$n] \t$datum_monitor_sclr[$n]\n";
                 $datum_monitor_sclr_old[$n] = $datum_monitor_sclr[$n];
              }
           }
           select(undef,undef,undef,$sleeptime);
        }
        if ($n < $n_monitor_str) {
           ($error, $datum_monitor_str[$n]) = Pezca::GetString($monitor_str[$n]);
           $ct = scalar localtime;
           if ($error) {
              print DAT "$ct \t$monitor_str[$n] \tERR\n";
              print STDOUT "$ct \t$monitor_str[$n] \tERR\n";
           }
           else {
              if ($datum_monitor_str[$n] ne $datum_monitor_str_old[$n]) {
                 print DAT "$ct \t$monitor_str[$n] \t$datum_monitor_str[$n]\n";
                 $datum_monitor_str_old[$n] = $datum_monitor_str[$n];
              }
           }
           select(undef,undef,undef,$sleeptime);
        }
     }
     print STDOUT ".";
  }
  exit;

#---------------------------------------------------------------

sub open_file {

  @time      = localtime(time);					# This is (sec,min,hour,mday,mon,year,wday,yday,isdst)
  $min       = $time[1];
  $day       = $time[3];
  $month     = $time[4] + 1;
  $year      = $time[5] + 1900;
  $outdir    = $logdir;
  $outfile   = 'beamline_monitor';
  $filename  = sprintf "${outdir}${outfile}_%04d%02d%02d.dat",$year,$month,$day;
  $filename  =~ s/ /0/g;
  $filehandle = open (DAT,">> $filename");				# open DAT file for writing
  if (!defined $filehandle) {
     print STDOUT "Could not open file.\n";
     &signal_exit;
  }
  ### Set unbuffered output
  select DAT; $|=1;
  ### Write logfile header;
  $ct = scalar localtime;
  print DAT "Beamline monitor started at: ${ct}\n\n";
  print STDOUT "Now logging data to ${filename}.  Press CTRL+C to quit.\n\n";
}

#---------------------------------------------------------------

sub monitor_list {

  @monitor_sclr = (
     "23ID:FES:status",
     "23ID:BLEPS:status",
     "EPS:23:ID:IP7sp",		# Not sure where this IP is
     "EPS:23:ID:BIV:POSITION",
     "EPS:23:ID:IP6sp",
     "EPS:23:ID:EM:F",
     "EPS:23:ID:IP5sp",
     "EPS:23:ID:FEV:POSITION",
     "EPS:23:ID:CC2:RDY",
     "EPS:23:ID:SS2:POSITION",
     "EPS:23:ID:BLEPS:VACPER",
     "EPS:23:ID:BLEPS:SPER",
     "EPS:23:ID:BLEPS:VAC",
     "EPS:23:ID:SP:IP6_BLSEN",
     "EPS:23:ID:SP:IP5_IP6",
     "EPS:23:ID:FE:PERM",
     "EPS:23:ID:IP6:SetptCnt",
     "EPS:23:ID:BLSEN:Cnt",
     "S:DesiredMode",
     "ACIS:ShutterPermit",
     "S:OrbitCorrection:CC",
     "ID23us:Energy.VAL",
     "ID23us:HarmonicValue",
     "ID23ds:Energy.VAL",
     "ID23ds:HarmonicValue",
     "EPS:23:ID:FEV:POSITION",
     "EPS:23:ID:BIV:POSITION",
     "23ID:VCA1:status",
     "23ID:VCA2:status",
     "23ID:VCA3:status",
     "23ID:VCA4:status",
     "23i:VIB1:status",
     "23i:VIC1:status",
     "23i:VIC2:status",
     "23i:VIC3:status",
     "23i:VID1:status",
     "23o:VOB1:status",
     "23o:VOB2:status",
     "23ID:FES:status",
     "23o:MBSHO:status",
     "23i:IBSH:status",
     "23i:MBSHI:status",
     "23i:WS:ZC:ActPos",
     "23i:WS:ZS:ActPos",
     "23i:WS:YC:ActPos",
     "23i:WS:YS:ActPos",
     "23o:WS:ZC:ActPos",
     "23o:WS:ZS:ActPos",
     "23o:WS:YC:ActPos",
     "23o:WS:YS:ActPos",
     "23i:BP:MOZ:ActPos",
     "23o:BP:MOZ:ActPos",
     "23o:HD:SL:ActPos",
     "23i:CS:ZC:ActPos",
     "23i:CS:ZS:ActPos",
     "23i:CS:YC:ActPos",
     "23i:CS:YS:ActPos",
     "23o:CS:ZC:ActPos",
     "23o:CS:ZS:ActPos",
     "23o:CS:YC:ActPos",
     "23o:CS:YS:ActPos",
     "23i:KB:HL:ActPos",
     "23i:KB:VL:ActPos",
     "23i:BD:HD:ActPos",
     "23i:BD:HA:ActPos",
#    "23i:BD:VD:ActPos",
     "23i:BD:VA:ActPos",
     "23i:BD:VH:ActPos",
     "23i:BD:Dv3:ANGLE",
     "23o:BD:HD:ActPos",
     "23o:BD:HA:ActPos",
#    "23o:BD:VD:ActPos",
     "23o:BD:VA:ActPos",
     "23o:BD:VH:ActPos",
     "23o:BD:Dv3:ANGLE",
     "23i:GO:OM:ActPos",
     "23i:GO:CX:ActPos",
     "23i:GO:CY:ActPos",
     "23i:GO:SH:ActPos",
     "23i:GO:SW:ActPos",
     "23i:GO:SV:ActPos",
     "23o:GO:OM:ActPos",
     "23o:GO:CX:ActPos",
     "23o:GO:CY:ActPos",
     "23o:GO:SH:ActPos",
     "23o:GO:SW:ActPos",
     "23o:GO:SV:ActPos",
     "23i:GS:YW:ActPos",
     "23i:GS:X:ActPos",
#    "23i:GS:PT:ActPos",
     "23i:GS:ZC:ActPos",
     "23i:GS:ZS:ActPos",
     "23i:GS:YC:ActPos",
     "23i:GS:YS:ActPos",
     "23o:GS:YW:ActPos",
     "23o:GS:X:ActPos",
#    "23o:GS:PT:ActPos",
     "23o:GS:ZC:ActPos",
     "23o:GS:ZS:ActPos",
     "23o:GS:YC:ActPos",
     "23o:GS:YS:ActPos",
     "23i:JET:P:ActPos",
#    "23o:JET:P:ActPos",
     "23o:SNC:P:ActPos",
     "23i:RB:RX:ActPos",
     "23i:RB:RY:ActPos",
     "23i:RB:RZ:ActPos",
     "23i:RB:RA:ActPos",
     "23o:RB:RX:ActPos",
     "23o:RB:RY:ActPos",
     "23o:RB:RZ:ActPos",
     "23o:RB:RA:ActPos",
     "23i:VFM:GET-VOUT00",
     "23i:VFM:GET-VOUT01",
     "23i:VFM:GET-VOUT02",
     "23i:VFM:GET-VOUT03",
     "23i:VFM:GET-VOUT04",
     "23i:VFM:GET-VOUT05",
     "23i:VFM:GET-VOUT06",
     "23i:VFM:GET-VOUT07",
     "23i:VFM:GET-VOUT08",
     "23i:VFM:GET-VOUT09",
     "23i:VFM:GET-VOUT10",
     "23i:VFM:GET-VOUT11",
     "23i:VFM:GET-VOUT12",
     "23i:VFM:GET-VOUT13",
     "23i:VFM:GET-VOUT14",
     "23i:VFM:GET-VOUT15",
     "23i:HFM:GET-VOUT00",
     "23i:HFM:GET-VOUT01",
     "23i:HFM:GET-VOUT02",
     "23i:HFM:GET-VOUT03",
     "23i:HFM:GET-VOUT04",
     "23i:HFM:GET-VOUT05",
     "23i:HFM:GET-VOUT06",
     "23i:HFM:GET-VOUT07",
     "23i:HFM:GET-VOUT08",
     "23i:HFM:GET-VOUT09",
     "23i:HFM:GET-VOUT10",
     "23i:HFM:GET-VOUT11",
     "23i:HFM:GET-VOUT12",
     "23i:HFM:GET-VOUT13",
     "23o:VFM:GET-VOUT00",
     "23o:VFM:GET-VOUT01",
     "23o:VFM:GET-VOUT02",
     "23o:VFM:GET-VOUT03",
     "23o:VFM:GET-VOUT04",
     "23o:VFM:GET-VOUT05",
     "23o:VFM:GET-VOUT06",
     "23o:VFM:GET-VOUT07",
     "23o:VFM:GET-VOUT08",
     "23o:VFM:GET-VOUT09",
     "23o:VFM:GET-VOUT10",
     "23o:VFM:GET-VOUT11",
     "23o:VFM:GET-VOUT12",
     "23o:VFM:GET-VOUT13",
     "23o:VFM:GET-VOUT14",
     "23o:VFM:GET-VOUT15",
     "23o:HFM:GET-VOUT00",
     "23o:HFM:GET-VOUT01",
     "23o:HFM:GET-VOUT02",
     "23o:HFM:GET-VOUT03",
     "23o:HFM:GET-VOUT04",
     "23o:HFM:GET-VOUT05",
     "23o:HFM:GET-VOUT06",
     "23o:HFM:GET-VOUT07",
     "23o:HFM:GET-VOUT08",
     "23o:HFM:GET-VOUT09",
     "23o:HFM:GET-VOUT10",
     "23o:HFM:GET-VOUT11",
     "23o:HFM:GET-VOUT12",
     "23o:HFM:GET-VOUT13",
     "23i:DAC2_2.VAL",
     "23i:DAC2_3.VAL",
     "23o:DAC2_2.VAL",
     "23o:DAC2_3.VAL",
     "23i:keithley6:ActGain",
     "23i:keithley6:ActOffset",
     "23i:keithley6:ActTime",
     "23o:keithley6:ActGain",
     "23o:keithley6:ActOffset",
     "23o:keithley6:ActTime",
     "23o:V3000:4",
     "23o:V3121:4",
     "23i:V3266:5",
     "23i:V3266:16",
     "23i:bo0:ch0",
     "23i:bo0:ch1",
     "23i:bo0:ch2",
     "23i:bo0:ch3",
     "23i:bo0:ch4",
     "23i:bo0:ch5",
     "23i:bo0:ch6",
     "23i:bo0:ch7",
     "23i:bo0:ch8",
     "23i:bo0:ch9",
     "23i:bo0:ch10",
     "23i:bo0:ch11",
     "23i:bo0:ch12",
     "23i:bo0:ch13",
     "23i:bo0:ch14",
     "23i:bo0:ch15",
     "23o:bo0:ch0",
     "23o:bo0:ch1",
     "23o:bo0:ch2",
     "23o:bo0:ch3",
     "23o:bo0:ch4",
     "23o:bo0:ch5",
     "23o:bo0:ch6",
     "23o:bo0:ch7",
     "23o:bo0:ch8",
     "23o:bo0:ch9",
     "23o:bo0:ch10",
     "23o:bo0:ch11",
     "23o:bo0:ch12",
     "23o:bo0:ch13",
     "23o:bo0:ch14",
     "23o:bo0:ch15",
     "23o:V3001_mbar",
     "FE:23:ID:IP6",
     "S:SRcurrentAI.VAL",
     "S23us:ID:SrcPt:xAngleM",
     "S23us:ID:SrcPt:xPositionM",
     "S23us:ID:SrcPt:yAngleM",
     "S23us:ID:SrcPt:yPositionM",
     "S23ds:ID:SrcPt:xAngleM",
     "S23ds:ID:SrcPt:xPositionM",
     "S23ds:ID:SrcPt:yAngleM",
     "S23ds:ID:SrcPt:yPositionM",
     "S23ID:P1:mswAve:x:AdjustedCC",
     "S23ID:P1:mswAve:y:AdjustedCC",
     "S23ID:P2:mswAve:x:AdjustedCC",
     "S23ID:P2:mswAve:y:AdjustedCC",
     "23o:V3001_mbar",
     "23o:V3003_mbar",
     "23o:V3005_mbar",
     "23o:V3007_mbar",
     "23o:V3123_mbar",
     "23o:V3125_mbar",
     "23i:V3011_mbar",
     "23i:V3127_mbar",
     "23i:V3267_mbar",
     "23i:V3271_mbar",
     "23i:V3273_mbar",
     "23i:V3275_mbar",
     "23o:V3025_K",		#WBSLout
     "23o:V3027_K",
     "23o:V3031_K",
     "23o:V3033_K",
     "23o:V3035_K",
     "23o:V3037_K",
     "23o:V3047_K",
     "23o:V3045_K",
     "23o:V3041_K",
     "23o:V3043_K",
     "23o:V3051_K",
     "23o:V3053_K",
     "23o:V3055_K",
     "23o:V3057_K",
     "23o:V3061_K",		#DCMout
     "23o:V3063_K",
     "23o:V3065_K",
     "23o:V3067_K",
     "23o:V3071_K",
     "23o:V3073_K",
     "23o:V3075_K",
     "23o:V3077_K",
     "23o:V3021_K",
     "23o:V3023_K",
     "23o:V3015_psi",		#Cryocooler
     "23ID:V3101_K",		#Premask
     "23ID:V3103_K",
     "23ID:V3105_K",
     "23ID:V3107_K",
     "23ID:V3111_K",
     "23ID:V3113_K",
     "23ID:V3115_K",
     "23ID:V3117_K",
     "23i:V3307_K",		#WBSLin
     "23i:V3311_K",
     "23i:V3313_K",
     "23i:V3315_K",
     "23i:V3317_K",
     "23i:V3321_K",
     "23i:V3323_K",
     "23i:V3325_K",
     "23i:V3327_K",
     "23i:V3331_K",
     "23i:V3333_K",
     "23i:V3335_K",
     "23i:V3337_K",
     "23i:V3341_K",
     "23i:V3343_K",
     "23i:V3345_K",
     "23i:V3351_K",		#DCMin
     "23i:V3347_K",
     "23i:V3353_K",
     "23i:V3355_K",
     "23i:V3357_K",
     "23i:V3361_K",
     "23i:V3363_K",
     "23i:V3365_K",
     "23i:V3367_K",
     "23i:V3371_K",
     "23i:V3373_K",
     "23i:V3375_K",
     "23i:V3301_psi",		#Cryocooler
     "23i:MO:E:ActPos",
     "23i:MO:R1:ActPos",
     "23i:MO:P2:ActPos",
     "23o:MO:E:ActPos",
     "23o:MO:R1:ActPos",
     "23o:MO:P2:ActPos",
     "23i:BS:D:ActPos",
     "23i:BS:PT:ActPos",
     "23i:BS:YW:ActPos",
     "23o:BS:D:ActPos",
     "23o:BS:PT:ActPos",
     "23o:BS:YW:ActPos",
     "23i:DAC1_1.VAL",
     "23o:DAC1_1.VAL",
     "23i:ADC_1.VAL",
     "23i:ADC_8.VAL",
     "23o:ADC_1.VAL",
     "23o:ADC_8.VAL"
  );

  @monitor_str = (
     "OPS:message1",
     "OPS:message2",
     "OPS:message3",
     "OPS:message4",
     "OPS:message5",
     "OPS:message6",
     "OPS:message7",
     "OPS:message8",
     "OPS:message9",
     "OPS:message10",
     "OPS:message11",
     "OPS:message12",
     "OPS:message13",
     "OPS:message14",
     "OPS:message15"
  );

}

#---------------------------------------------------------------

sub set_monitors {

  my ($i, $j, $k);
  
  for ($i=0; $i<$n_monitor_sclr; $i++) {
     $error = Pezca::SetMonitorDouble($monitor_sclr[$i]);
     if ($error) {
        print STDOUT "Error setting monitor on: ${monitor_sclr[$i]}!\n";
        print STDOUT "Clearing monitors and exiting...\n";
        for ($j=0; $j<$i; $j++) {
           Pezca::ClearMonitorDouble($monitor_sclr[$j]);
        }
        print STDOUT "Done!\n";
        exit;
     } else {
        print STDOUT "Monitor set on ${monitor_sclr[$i]}\n";
     }
  }
  for ($i=0; $i<$n_monitor_str; $i++) {
     $error = Pezca::SetMonitorString($monitor_str[$i]);
     if ($error) {
        print STDOUT "Error setting monitor on: ${monitor_str[$i]}!\n";
        print STDOUT "Clearing monitors and exiting...\n";
        for ($j=0; $j<$i; $j++) {
           Pezca::ClearMonitorString($monitor_str[$j]);
        }
        for ($k=0; $k<$n_monitor_sclr; $j++) {
           Pezca::ClearMonitorDouble($monitor_sclr[$j]);
        }
        print STDOUT "Done!\n";
        exit;
     } else {
        print STDOUT "Monitor set on ${monitor_str[$i]}\n";
     }
  }
}

#---------------------------------------------------------------

sub clear_monitors {

  my ($i, $j, $error);

  for ($i=0; $i<$n_monitor_sclr; $i++) {
     $error = Pezca::ClearMonitorDouble($monitor_sclr[$i]);
     if ($error) {
        print STDOUT "Error clearing monitor on: ${monitor_sclr[$i]}!\n";
     }
     else {
        print STDOUT "Monitor cleared on ${monitor_sclr[$i]}\n";
     }
  }
  for ($i=0; $i<$n_monitor_str; $i++) {
     $error = Pezca::ClearMonitorString($monitor_str[$i]);
     if ($error) {
        print STDOUT "Error clearing monitor on: ${monitor_str[$i]}!\n";
     }
     else {
        print STDOUT "Monitor cleared on ${monitor_str[$i]}\n";
     }
  }
}

#---------------------------------------------------------------

sub signal_exit {

  $ct = scalar localtime;
  print DAT "Beamline monitor stopped at: ${ct}\n\n";
  close DAT;
  print STDOUT "Exiting!\n";
  print STDOUT "Clearing monitors... ";
  &clear_monitors;
  print STDOUT "Done.\n";
  exit;
}
