#!/usr/bin/perl -W
#---------------------------------------------------------------------------
  use strict;
  use Pezca;

  our $beamline;
  my ($PEZCA, $nkeithley, $maxtime, $status);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

# $beamline = '23b';

  do $PEZCA.'/Common/keithley_subs.pl';
  do $PEZCA.'/Common/common_subs.pl';
  if (defined $beamline) {do $PEZCA.'/Common/beamline_'.$beamline.'.pl';}
  else                   {do $PEZCA.'/Common/beamline.pl';}

  select STDOUT; $|=1;						#set unbuffered output
  $SIG{HUP}  = 'signal_handler';				#(1) hangup (see signum.h)
  $SIG{INT}  = 'signal_handler';				#(2) interrupt (see signum.h)
  $SIG{QUIT} = 'signal_handler';				#(3) quit (see signum.h)
# $SIG{KILL} = 'signal_handler';				#(9) this is what is sent by kill -9 -- unblockable
  $SIG{TERM} = 'signal_handler';				#(15) terminate -- sent by kill without paramaters
# $SIG{STOP} = 'signal_handler';				#(19) stop -- unblockable

  $nkeithley = 7;
  $maxtime = '10ms';
  $maxtime = '1us';
  $status = &check_keithley_gains($nkeithley,$maxtime);
  print STDOUT 'status='.$status."\n";
  &clearAllCaMonitors();
  exit;

#=======================================================================
sub check_keithley_gains($;$) {

  my ($script, $nkeithley, $maxtime, $msg);
  my ($time, $label, $status, $i);

  $script    = 'check_keithley_gains';
  $nkeithley = shift(@_);
  $maxtime   = shift(@_);	if (!defined $maxtime) {$maxtime='10ms';}

  $maxtime =~ s/ //g;
  if    ($maxtime =~ /us$/) {$maxtime =~ s/us//g; $maxtime = $maxtime/1000.;}	#convert to milliseconds
  elsif ($maxtime =~ /ms$/) {$maxtime =~ s/ms//g;}
  elsif ($maxtime =~ /s$/)  {$maxtime =~ s/s//g;  $maxtime = $maxtime*1000.;}	#convert to milliseconds
  else  {print STDOUT '*** '.$script.': incorrect maxtime='.$maxtime."\n"; return -1;}

  $msg = '';
  for ($i=1; $i<=$nkeithley; $i++) {
     $time = &keithley_read_time($i);
     if ($time > $maxtime) {
        $label = &keithley_read_label($i);
        $msg .= sprintf 'keithley-%d integration time='.$time.'ms exceeds max='.$maxtime.'ms -- label=[%s]'."\n", $i, $label;
     }
  }
  if (length($msg) > 0) {&zenity_error($msg,undef,'nowait');}
  return 0;
}

#=======================================================================

sub signal_handler {
# local($sig) = @_;
  my $sig = shift;
  if ($sig ne 'quit') {print STDOUT "\n *** Caught a SIG${sig}\n\n";}
  &clearAllCaMonitors();
  exit;
}

#=======================================================================
