#!/usr/bin/perl -W
#------------------------------------------------------------------------
# This script calls chooch program and captures its STDOUT.
#
# The intercepted STDOUT stream is parsed to decode peak and inflection points.
# It could also be used to drive monochromator to one or the other (not yet implemented).
#
# The main purspose of this script is to produce postscript plots instead of chooch.
# The matter is that chooch uses cgraph for plotting and cgraph may crash
# on certain LDAP accounts.
#
#
#     		Author: Sergey Stepanov
#
# Version-1.0:  2012/11/08
#------------------------------------------------------------------------
  use strict;

  our ($USE_ZENITY_FOR_EXIT, $ZENITY_VIA_BLUICE, $NONINTERACTIVE);
  my ($PEZCA, $script, $syntax, @cmdline, $narg, $arg);
  my ($choochprg, $choochdata, $Edge_element, $Edge_KLMN);
  my ($status, $fileroot, $ext, $outfile, $outpost);
  my (@txt, $nlines, $line, $caption, $ncap);
  my ($filehandle, $header, $pid);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/common_subs.pl';

  select STDOUT; $|=1;					#set unbuffered output
  select STDERR; $|=1;					#set unbuffered output

#################################
  $USE_ZENITY_FOR_EXIT = 1;				#set to '1' for graphical popups
  $ZENITY_VIA_BLUICE   = 1;
  $NONINTERACTIVE      = 1;				#set to one to suppress 'press key to exit'

  $script = &makeScriptName();
  $syntax = "\n\n".'Usage: '.$script.' element=XX edge=XX data=file'."\n";

  $choochprg  = '/mnt/software/px/bin/chooch';
# $choochdata=$PEZCA.'/Data/Pt_1C11_chooch.dat';  $Edge_element='Pt';  $Edge_KLMN='L3';
# $choochdata=$PEZCA.'/Data/SK_Zn1.dat';          $Edge_element='Zn';  $Edge_KLMN='K';
# $choochdata=$PEZCA.'/Data/SK_Zn2.dat';          $Edge_element='Zn';  $Edge_KLMN='K';
# $choochdata=$PEZCA.'/Data/Se_1G11_chooch.dat';  $Edge_element='Se';  $Edge_KLMN='K';
# $choochdata=$PEZCA.'/Data/chooch_01.dat';       $Edge_element='Zn';  $Edge_KLMN='K';
# $choochdata=$PEZCA.'/Data/chooch_Hg1.dat';      $Edge_element='Hg';  $Edge_KLMN='L3';

  if ($^O =~ /Win/i) {
     &press_key_exit ('*** '.$script.': cannot run on Windows (no Chooch).');
  }
  $narg = @ARGV;
  if ($narg > 0) {print STDOUT $script.': started with cmdline = '."@ARGV\n\n";}
  else           {print STDOUT $script.': started with no cmdline arguments.'."\n\n";}

  foreach $arg (@ARGV) {
     if    ($arg =~ /^element=/i)    {$arg =~ s/^element=//i;  $Edge_element = $arg;}
     elsif ($arg =~ /^edge=/i)       {$arg =~ s/^edge=//i;     $Edge_KLMN    = $arg;}
     elsif ($arg =~ /^data=/i)       {$arg =~ s/^data=//i;     $choochdata   = $arg;}
     else {&press_key_exit ('*** '.$script.': incorrect cmdline parameter='.$arg.$syntax);}
  }
  if (!defined $Edge_element) {&press_key_exit ('*** '.$script.': unspecified element.'  .$syntax);}
  if (!defined $Edge_KLMN)    {&press_key_exit ('*** '.$script.': unspecified edge.'     .$syntax);}
  if (!defined $choochdata)   {&press_key_exit ('*** '.$script.': unspecified data file.'.$syntax);}

  print STDOUT $script.': element='.$Edge_element.' edge='.$Edge_KLMN.' data='.$choochdata."\n";

  ($fileroot, $ext) = &analyzeFilename($choochdata);
  $outfile = $fileroot.'.efs';
  $outpost = $fileroot.'.ps';

  if (! -f $choochdata) {
     &press_key_exit ('*** '.$script.': Chooch data file='.$choochdata.' does not exist.');
  }
  if (! -x $choochprg) {
     &press_key_exit ('*** '.$script.': Chooch program='.$choochprg.' cannot be executed.');
  }
  if (-f $outfile) {
     $status = unlink($outfile);
     if ($status == 0) {
        &press_key_exit ('*** '.$script.': could not delete file='.$outfile.', err='.$!);
     }
  }
  if (-f $outpost) {
     $status = unlink($outpost);
     if ($status == 0) {
        &press_key_exit ('*** '.$script.': could not delete file='.$outpost.', err='.$!);
     }
  }

### Execute chooch here:
  @cmdline = ($choochprg,'-x','-e',$Edge_element,'-a',$Edge_KLMN,'-o',$outfile,$choochdata);
  print STDOUT $script.': Starting Chooch:'."\n@cmdline\n";
# $status = system(@cmdline);
  @txt = `@cmdline`;				#this method of execution captures STDOUT (not STDERR)
  $status = $?;
  if ($status) {
     &press_key_exit ('*** '.$script.': calling chooch returned status='.$status);
  }
  $nlines = @txt;
  if ($nlines > 0) {
     print STDOUT "\n".'---------------'."\n@txt\n".'---------------'."\n\n";
  } else {
     &press_key_exit ('*** '.$script.': no STDOUT output from chooch');
  }
  if (! -f $outfile) {
     &press_key_exit ('*** '.$script.': chooch did not produce file='.$outfile);
  }

  &plot_to_postscript();

  print STDOUT $script.': chooch executed OK'."\n";
  print STDOUT $script.': output file='.$outfile."\n";
  print STDOUT $script.': output ps='.$outpost."\n";
  exit;

#-------------------------------------------------------------------
sub plot_to_postscript () {

### Read header from data file:
  $filehandle = open (DAT,'< '.$choochdata);
  if (defined $filehandle) {
     $header = readline(*DAT);
     close(DAT);
     if (defined $header) {chomp $header;}
     else {
        print STDOUT '*** '.$script.': Cannot read data header from '.$choochdata."\n";
	$header ='';
     }
  } else {
     print STDOUT '*** '.$script.': Cannot open '.$choochdata."\n";
     $header ='';
  }
### In Gnuplot a symbol (or group of symbols in{}) after "_" is treated as subscript:
  $header =~ s/_start=/_\{start\}=/i;
  $header =~ s/_end=/_\{end\}=/i;
  $header =~ s/_center=/_\{center\}=/i;
# print STDOUT $script.': header='.$header."\n";

### Read inflection and peak from captured chooch stdout:
  $caption = '';
  $ncap = 0;
  foreach $line (@txt) {
     chomp $line;
     if ($line =~ /^\|      \|  energy  \|/) {
        $line =~ s/\|//g;
	$line =~ s/f\'\'/f\\047\\047/;			#octal code of ('')
	$line =~ s/f\'/f\\047/;				#octal code of (')
	$caption .= "\n".$line;  $ncap++;
     }
     elsif ($line =~ /^\| peak \|/) {
        $line =~ s/\|//g;
	$caption .= "\n".$line;  $ncap++;
     }
     elsif ($line =~ /^\| infl \|/) {
        $line =~ s/\|//g;
	$caption .= "\n".$line;  $ncap++;
     }
     if ($ncap >= 3) {last;}
  }
# if ($ncap > 0) {
#    print STDOUT $script.': chooch peak/inflection findings'."\n".$caption."\n\n";
# }

  $pid = open(GNUPLOT, '|-', 'gnuplot -persist 2>/dev/null');
  if (defined $pid) {
     $SIG{PIPE} = sub {&press_key_exit ('*** '.$script.': gnuplot pipe crashed');};
     select GNUPLOT; $|=1;			# set unbuffered output
     print GNUPLOT 'set terminal postscript landscape enhanced monochrome blacktext'."\n";
     print GNUPLOT 'set output "'.$outpost.'"'."\n";
#    print GNUPLOT 'set terminal x11'."\n";
#    print GNUPLOT 'set mxtics 5'  ."\n";
#    print GNUPLOT 'set mytics 10' ."\n";
#    print GNUPLOT 'unset logscale'."\n";
     print GNUPLOT 'set tics in'   ."\n";
     print GNUPLOT 'set grid'      ."\n";
     print GNUPLOT 'set xlabel "X-ray energy (eV)"'."\n";
     print GNUPLOT 'set ylabel "f\047 and f\047\047 (e)"'."\n";
     print GNUPLOT 'set style data lines'."\n";
     $caption =~ s/\n/\\n/g;
     print GNUPLOT 'set title '.'"'.$header.$caption.'"'."\n";
     print GNUPLOT 'plot "'.$outfile.'" using 1:2 lw 2 lt 1 lc rgb "black" title "",'
                                  .' "" using 1:3 lw 2 lt 1 lc rgb "black" title ""'."\n";
#    print GNUPLOT 'set terminal x11'."\n";			#comment out to skip screen plot
#    print GNUPLOT 'replot'."\n";				#comment out to skip screen plot
#    &press_key_continue('Press enter to close the plot');	#not needed since we opened persistent plot
     print GNUPLOT 'exit'."\n";
     close(GNUPLOT);
  } else {&press_key_exit ('*** '.$script.': Cannot open pipe to gnuplot');}
}
