#!/usr/bin/perl -W
#--------------------------------------------------------------
# This script clears errors on all macrostations of specified PMAC.
#
#     		     Author: Sergey Stepanov
#
# Version 1.00 (2013.05.31): initial release using Pezca
#--------------------------------------------------------------
  use strict;
  use Pezca;

  our ($beamline, $USE_ZENITY_FOR_EXIT);
  my ($PEZCA, $script, $narg, $pmac, $bl, @beamlinelist, @macroList, $nmacro);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

# do $PEZCA.'/Common/motor_subs.pl';
  do $PEZCA.'/Common/home_subs.pl';
  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDOUT; $|=1;			# set unbuffered output
  select STDERR; $|=1;			# set unbuffered output

  $SIG{HUP}  = 'signal_handler';	#(1) hangup (see signum.h)
  $SIG{INT}  = 'signal_handler';	#(2) interrupt (see signum.h)
  $SIG{QUIT} = 'signal_handler';	#(3) quit (see signum.h)
# $SIG{KILL} = 'signal_handler';	#(9) this is what is sent by kill -9 -- unblockable
  $SIG{TERM} = 'signal_handler';	#(15) terminate -- sent by kill without paramaters
# $SIG{STOP} = 'signal_handler';	#(19) stop -- unblockable

  $script = &makeScriptName();
# $script =~ s/\.pl$//i;

### Defaults (may be changed by cmdline options):
  $USE_ZENITY_FOR_EXIT = 0;		#normally you want 0. It is auto-switched to 1 for summary
  @beamlinelist = ('23i:', '23o:', '23b:', '23d:');

  $narg = @ARGV;
  print STDOUT "\n";
  print STDOUT '-- GM/CA @ APS clear PMAC macro stations script:'."\n";
  if ($narg > 0) {print STDOUT '-- CMD: '.$script." @{ARGV}\n";}
  else           {print STDOUT '-- CMD: '.$script.        "\n";}

  if ($narg != 1) {&printSyntax('Please specify pmac name.');}

  $pmac = $ARGV[0];
  if ($pmac !~ /pmac/) {&printSyntax('Specified argument is not pmac name.');}
  foreach $bl (@beamlinelist) {
     if ($pmac =~ /^${bl}/) {
        $bl =~ s/:$//;
 	if ($beamline ne $bl) {
           print STDOUT "\n".'!!! '.$script.': Cross-beamline operation: homing '.$bl.' from '.$beamline."\n\n";
           $beamline = $bl;
	}
        last;
     }
  }
  if ($pmac =~ /^pmac/) {$pmac  = $beamline.':'.$pmac;}
  if ($pmac !~ /:$/)    {$pmac .= ':';}

###-------------------------------------------------------------------------
  @macroList = ();
  &getPmacMacroList ($pmac,\@macroList);
  $nmacro = @macroList;
  if ($nmacro > 0) {
     &ask_to_continue ('This script is going to reset the following macrostations for pmac='.$pmac."\n\n"
                      ."   (@{macroList})\n\n"
	   	      .'Each macrostation with number N will be sent ms$$$N and msclrfN.'."\n\n"
		      .'Please proceed with caution as all motors connected to these macrostations will need re-homing.');
  }
  &clearPmacMacro_Pmac ($pmac);
  &ask_to_continue ('', 'Reset the PMAC itself (send $$$) [n]?');
  &sendPmacCmd($pmac,'$$$');
  &press_key_exit('All done!');

#=================================================================

# sub printSyntax($msg) {
sub printSyntax($) {
  my ($msg);
  $msg  = shift(@_);
  if (length($msg) > 0) {$msg  = $script.': '.$msg;}
  $msg .= "\n".'Syntax: '.$script.' pmac.'."\n";
  &press_key_exit($msg);
}

#=================================================================

sub signal_handler {
# local($sig) = @_;
  my $sig = shift;
  $USE_ZENITY_FOR_EXIT = 0;
  &press_key_exit($script.': *** Caught a SIG'.$sig);
}

