#!/usr/bin/perl -W
#----------------------------------------------------------------
# This script displays passed text in a scrollabe DialogBox window.
#
#		 Author: Sergey Stepanov
#
# Version:	1.00	2020/01/10 initial implementation
# Version:      1.10    2020/06/25 added option to specify windows header
#----------------------------------------------------------------

  use strict;
  use warnings;
  use Tk;
  use Tk::Dialog;
  use Tk::ROText;
# use warnings::unused;

  select STDERR; $|=1;					#set unbuffered output
  select STDOUT; $|=1;					#set unbuffered output

  my ($PEZCA, $script, $syntax, $txt, $hdr2close, $hdrmywin);
  my ($mw, $dialogWin, $rt, $fontBtn, $fontTbl);
  my ($fgcolor, $afgcolor, $bgcolor, $abgcolor);
  my (@lines, $line, $w, $wmax, $scrl, $answer);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;
  if ($PEZCA !~ /\/$/) {$PEZCA .='/';}

  do $PEZCA.'Common/common_subs.pl';

  $script = &makeScriptName();

  if (defined $ARGV[0]) {$txt       = $ARGV[0];}
  if (defined $ARGV[1]) {$hdr2close = $ARGV[1];} else {$hdr2close = 'none';}
  if (defined $ARGV[2]) {$hdrmywin  = $ARGV[2];} else {$hdrmywin  = '';}

  $syntax = "\n\n".'Syntax: '.$script;
  $syntax .= ' "text" ["title_of_window_to_close_on_dialog_closing"] ["window_title"'."\n\n";
  if (!defined $txt) {die $syntax;}
# print STDOUT $txt;
  @lines = split(/\r?\n/,$txt);
  $wmax = 0;
  foreach $line (@lines) {if (length($line) > $wmax) {$wmax=length($line);}}
  $wmax +=5;

  $mw = MainWindow->new(-width=>1,-height=>1,-title=>$script);
  $mw->resizable( 0, 0 );
  $mw->withdraw;

  $fontBtn = $mw->fontCreate(-size=>11, -weight=>'bold');
  $fontTbl = $mw->fontCreate(-size=>11, -weight=>'bold', -family=>'Courier');

  $fgcolor = '#000000';     $afgcolor = '#770000';
  $bgcolor = '#11A8FF';     $abgcolor = '#0097EE';

  if (!defined $hdrmywin || length($hdrmywin) == 0) {$hdrmywin = $script.'-dialog'}
  $dialogWin = $mw->DialogBox(-title=>$hdrmywin,
                              -buttons=>['OK'],
                              -default_button=>'OK',
                              -takefocus=>1,
                              );
  if    ($wmax > 100) {$w = 100;   $scrl='se';}
  elsif ($wmax < 20)  {$w = 20;    $scrl='e';}
  else                {$w = $wmax; $scrl='e';}
  $rt = $dialogWin->ScrlROText(-width=>$w,
                               -height=>40,
                               -scrollbars=>$scrl,
                               -font=>$fontTbl,
                               -wrap=>'none',
                               )->pack();
  if ($scrl eq 'se') {
     $rt->Subwidget("xscrollbar")->configure(-background=>$bgcolor,);
  }
  $rt->Subwidget("yscrollbar")->configure(-background=>$bgcolor,);
  $rt->insert(end => $txt);

  $dialogWin->Subwidget('B_OK')->configure(-font=>$fontBtn,
                                           -background=>$bgcolor, -activebackground=>$abgcolor, 
                                           -foreground=>$fgcolor, -activeforeground=>$afgcolor,
                                           );
  $answer = $dialogWin->Show;
  if ($answer eq 'OK') {
     $dialogWin->destroy();
     if (defined $hdr2close && $hdr2close ne 'none') {system ('wmctrl -F -c "'.$hdr2close.'" &');}
     exit;
  }

  $mw->protocol('WM_DELETE_WINDOW', [\&exit]);
  $mw->bind('<KeyPress-Escape>', [\&exit]);
  $mw->focus();
  MainLoop;
