#!/usr/bin/perl -W
#--------------------------------------------------------------
#
#                Author: Sergey Stepanov
#
#  This script outputs current mono Bragg reflection.
#
#
# Version-1.0:  2021/09/15 using Pezca
#--------------------------------------------------------------
# use Math::Trig;		# needed for mono Theta <-> Energy calculations
  use Pezca;
  use warnings::unused;
  use strict;

  our ($IGNORE_ERRORS, $NONINTERACTIVE);
  our ($USE_ZENITY_FOR_EXIT, $ZENITY_VIA_BLUICE);
  our ($beamline);
  my ($PEZCA, $script, $syntax, @beamlinelist);
  my ($hkl, $narg, $arg, $i, $tell_bluice);
  my ($bln_specified, $blu_specified);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  $IGNORE_ERRORS          = 1;							#GLOBAL: do not break script if error occurred
  $NONINTERACTIVE         = 1;							#GLOBAL: do not wait for any answers
  $USE_ZENITY_FOR_EXIT    = 1;							#GLOBAL: popup exit messages
 
  do $PEZCA.'/Common/mono_subs.pl';
  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDERR; $|=1;								#set unbuffered output
  select STDOUT; $|=1;								#set unbuffered output

  $script = &makeScriptName();
  $syntax = "\n\n".'Syntax: '.$script.' [beamline] [bluice|nobluice]'."\n";
  @beamlinelist = ('23i:', '23o:', '23b:');
# ##############################

  $narg = @ARGV;
  print STDOUT "\n".'Date: '.scalar(localtime)."\n";
  if ($narg > 0) {print STDOUT $script.': started with cmdline = '."@ARGV\n\n";}
  else           {print STDOUT $script.': started with no cmdline arguments.'."\n\n";}
#---------------------------------------------------------------------------
### Scan command line arguments:
  for ($i=0; $i<$narg; $i++) {
     $arg = $ARGV[$i];
     if ($arg =~ /MO:?$/) {$arg =~ s/MO:?$//;}					#this is calling from MEDM screens
     if (grep(/^$arg:?$/, @beamlinelist)) {
        if (!defined $bln_specified) {
           $bln_specified = 1;
           $arg =~ s/:$//;
           if ($arg ne $beamline) {
              print STDOUT "\n".'!!! '.$script.': Cross-beamline operation:'
                             .' controlling '.$arg.' from '.$beamline."\n\n";
              $beamline = $arg;
              do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
           }
        } else {
           &press_key_exit('*** '.$script.': duplicate command line argument-'
                                             .$i.' ['.$ARGV[$i].']!'.$syntax,1);
        }
     }
     elsif ($arg =~ /^bluice$/i || $arg =~ /^nobluice$/i) {
        if (!defined $blu_specified) {
           $blu_specified = 1;
           if    ($arg =~ /^bluice$/i)   {$tell_bluice = 1;}
           elsif ($arg =~ /^nobluice$/i) {$tell_bluice = 0;}
        } else {
           &press_key_exit('*** '.$script.': duplicate command line argument-'
                                             .$i.' ['.$ARGV[$i].']!'.$syntax,1);
        }
     }
     else {
        &press_key_exit('*** '.$script.': unrecognized command line argument-'
                                             .$i.' ['.$ARGV[$i].']!'.$syntax,1);
     }
  }
#---------------------------------------------------------------------------
  if (!defined $tell_bluice) {$tell_bluice = 0;}
  if ($tell_bluice) {$ZENITY_VIA_BLUICE = 1;}

###------------------------------------------------------------------------
  $hkl = &getMonoHKL();
  &zenity_info('Current monochromator Bragg reflection is '.$hkl.'.','nowait');

  &clearAllCaMonitors();
  exit 0;
