#!/usr/bin/perl -W
#---------------------------------------------------------------------------
# This script is called by https://www.gmca.aps.anl.gov/remote/index.php
# and checks if remote IP falls into a range of acceptable IPs in
# /usr/NX/etc/htd_require_ip.cfg
# If it does, the web page shows link to NoMachine Cloud Server login
#
#                         Author: Sergey Stepanov
#
# Version 1.0 2017/06/27  initial implementation
# Version 2.0 2017/08/18  added determining beamline
#---------------------------------------------------------------------------
  use strict;
# use Fcntl;					#not used (defines O_RDONLY constant (O_RDONLY=0,O_WRONLY=1,O_RDWR=2))
# use Net::CIDR::Lite;				#this is loaded below via &try_load_module()
  use Time::HiRes qw(gettimeofday time);	#for time stamps with us resolution

  my ($script, $narg, $ip, $file, $syntax, $cidr, @cidr_list);
  my (@records, $record, @words, $word, @parts, $nparts);
  my ($ipmin, $ipmax, $found, $i, $debug);
  my ($beamline, @beamlines, @ipranges, @iptypes, $nranges);
  my (@blf, @ipf, $nfound);

  select STDERR; $|=1;                          #set unbuffered output
  select STDOUT; $|=1;                          #set unbuffered output

  $script = $0;
  $script =~ s/\\/\//g;				#change \ to /
  $script =~ s/^.*\///g;			#remove everything before last "/"

  &try_load_module('Net::CIDR::Lite');

  $syntax = "\n".'Syntax: '.$script.' ip'."\n\n";
  $narg = @ARGV;
  if ($narg != 1) {die $syntax;}
  $ip = $ARGV[0];
  $file = '/usr/NX/etc/htd_require_ip.cfg';

# $debug = 1;
# $ip = '164.54.103.18';
# $file = $ENV{PEZCA}.'/Data/htd_require_ip.cfg';

  if (! -e $file) {die '*** '.$script.': Cannot access '.$file;}
  &read_entire_file ($file,\@records);
  if (!defined $debug) {$debug = 0;}
  $beamline='ALL';
  @beamlines = ();
  @ipranges  = ();
  @iptypes   = ();
# $cidr = Net::CIDR::Lite->new;
  foreach $record (@records) {
     if ($record =~ /\@BL[123]{1}$/) {
       $beamline = $record;
       $beamline =~ s/^.*\@BL/BL/;
     }
     &strip_comments(\$record);
     if (length($record) == 0) {next;}
     if ($record =~ /^.*Require ip /) {
        $record =~ s/^.*Require ip //i;
        if (length($record) == 0) {next;}
        if ($debug) {print STDOUT "\n".'Record=',$record,"\n";}
        @words = split(/\s+/,$record);
        foreach $word (@words) {
           if ($debug) {print STDOUT 'Word=',$word,"\n";}
#          if    ($word =~ /^[0-9.]+\/{1}[0-9]{1,2}$/) {			#record is CIDR: 1.2.3/24
           if    ($word =~ /^([0-9]{1,3}\.){1,3}[0-9]{1,3}\/{1}[0-9]{1,2}$/) {	#record is CIDR: 1.2.3/24
              if ($debug) {print STDOUT 'Found CIDR=',$word,"\n";}
#             $cidr->add($word);
              push (@beamlines,$beamline);
	      push (@ipranges,$word);
	      push (@iptypes,'cidr'); 
           }
#          elsif ($word =~ /^[0-9.]+$/) {					#record is IP or range: 1.2.3 (which means 1.2.3.1-1.2.3.255)
           elsif ($word =~ /^([0-9]{1,3}\.){1,3}[0-9]{1,3}$/) {			#record is IP or range: 1.2.3 (which means 1.2.3.1-1.2.3.255)
	      @parts = split(/\./,$word);
	      $nparts = @parts;
	      if    ($nparts < 4 && $nparts > 1) {
 	         $ipmin = $ipmax = $word;
 	         for ($i=0; $i<(4-$nparts); $i++) {$ipmin .= '.1'; $ipmax .= '.255';}
                 if ($debug) {print STDOUT 'Found RANGE=',$ipmin.'-'.$ipmax,"\n";}
# 	         $cidr->add_range($ipmin.'-'.$ipmax);
                 push (@beamlines,$beamline);
	         push (@ipranges,$ipmin.'-'.$ipmax);
	         push (@iptypes,'range'); 
	      } 
	      elsif ($nparts == 4) {						#record is single IP 1.2.3.4
                 if ($debug) {print STDOUT 'Found IP=',$word,"\n";}
#	         $cidr->add_ip($word);
                 push (@beamlines,$beamline);
	         push (@ipranges,$word);
	         push (@iptypes,'ip'); 
	      }
	      else {
                 die  '*** '.$script.': Unexpected word=',$word;
	      }
           }
        }
     }
  }
  @blf = ();
  @ipf = ();
  $nranges = @ipranges;
  $nfound = 0;
  for ($i=0; $i<$nranges; $i++) {
     $cidr = Net::CIDR::Lite->new;
     if    ($iptypes[$i] eq 'cidr')  {$cidr->add($ipranges[$i]);}
     elsif ($iptypes[$i] eq 'range') {$cidr->add_range($ipranges[$i]);}
     elsif ($iptypes[$i] eq 'ip')    {$cidr->add_ip($ipranges[$i]);}
     else {die '*** '.$script.': unexpected IP type='.$iptypes[$i];}
     $found = $cidr->find($ip);								#returns 1 or empty string
     if ($found ne '1') {$found = 0;}
     if ($found) {
        $nfound++;
        push(@blf, $beamlines[$i]);
        push(@ipf, $ipranges[$i]);
        if ($debug) {
           if ($nfound == 1) {print STDOUT "\n\n".'================================================'."\n";}
 	   print STDOUT 'found: ip='.$ip.' belongs to ['.$ipranges[$i].'], beamline='.$beamlines[$i]."\n";
	}
     }
  }
  $nfound = @ipf;
  if ($nfound > 0) {
     $beamline = $blf[0];
     for ($i=1; $i<$nfound; $i++) {
        if ($blf[$i] ne $beamline) {$beamline = 'ALL'; last;}
     }
#    print STDERR '[',&ustamp(),'] [:note] [',$script,'] [IP='.$ip.' access OK to beamline='.$beamline.']'."\n";
     print STDOUT '1 ',$beamline;
  } else {
     print STDERR '[',&ustamp(),'] [:warn] [',$script,'] [IP='.$ip.' no any beamline access]'."\n";
     print STDOUT '0 ALL';
  }
  if ($debug) {print STDOUT "\n\n";}

#=================================================================

sub try_load_module($) {
  my $module = shift(@_);
  eval('use '.$module);
  if ($@) {die '*** '.$script.': module "'.$module.'" failed to load.';}
}
#==========================================================================

# sub read_entire_file($file,\@lines) {
sub read_entire_file($\@) {
  my ($file, $linesref, $buffer, $status);

  $file     = shift(@_);
  $linesref = shift(@_);

  @$linesref = ();
  $status = sysopen(FH, $file, 0);		#0 is used instead of O_RDONLY is defined in "use Fcntl";
  if (!defined $status) {die '*** '.$script.': Could not open '.$file;}
  sysread(FH, $buffer, -s FH);
  close FH;
  @$linesref = split(/\r?\n/,$buffer);
}

#==========================================================================

# sub strip_comments(\$string) {
sub strip_comments(\$) {
  my ($str_ref, $delim);
  $str_ref   = shift(@_);
  $$str_ref =~ s/\r?\n//g;						#remove \r -- perhaps this is not working?
  $delim = '#';
### trail comments started by $delim ("." matches any character,
### "*" means match 0 or more times, "$" matches end of string)
  $$str_ref =~ s/\Q${delim}\E.*$//g;
  $$str_ref =~ s/^\s+//g;	#remove beginning-of-the-line spaces and tabs
  $$str_ref =~ s/\s+$//g;	#remove end-of-the-line spaces and tabs
  $$str_ref =~ s/\s+/ /g;	#compress spaces ("+" means match 1 or more times)
}

#==========================================================================

sub ustamp() {
  my (@ct, $epoch, $usecs, $stamp, @words);
# my ($sec,$minute,$hour,$mday,$month,$year,$monthName,$weekday);
  
  ($epoch, $usecs) = gettimeofday();
  $stamp = scalar(localtime($epoch));
  
#  @ct = localtime($epoch)
#  ($sec,$minute,$hour,$mday,$month,$year) = ($ct[0],$ct[1],$ct[2],$ct[3],$ct[4]+1,$ct[5]+1900);
 @words = split(/\s+/,$stamp);
 $words[-2] .= sprintf '.%06d', $usecs;
 $stamp = join(' ',@words);
 return $stamp;
}
