#!/usr/bin/perl -W
#---------------------------------------------------------------------------
# This script is called by https://www.gmca.aps.anl.gov/remote/index.php
# and checks if remote IP falls into a range of acceptable IPs in
# /usr/NX/etc/htd_require_ip.cfg (versions 1-2) or in mySQL (version 3).
# Since version 4 it also returns error status and message.
# If the script finds that remote IP is on the allowed access list, the 
# web page generated by PHP shows link to NoMachine Webplayer logins.
#
# ATTENTION: this script is no longer in use because it has been replaced
# ^^^^^^^^^  by the PHP determine_access() function.
#
#                         Author: Sergey Stepanov
#
# Version 1.0 2017/06/27  initial implementation     (returns 1|0)
# Version 2.0 2017/08/18  added determining beamline (returns 1|0 BL1|BL2|BL3|ALL|NONE)
# Version 3.0 2018/01/29  changed to read MySQL gmca_accounts/server DB instead of htd_require_ip.cfg
#                         (returns 1|0 BL1(d1,d2) BL2(d1,d2) BL3(d1,d2), where d1,d2=1|0 when day,day are allowed|disallowed)
# Version 4.0 2019/05/09  added returning status and error message
#                         (returns 1|0 BL1(d1,d2) BL2(d1,d2) BL3(d1,d2) status "message")
# Version 4.1 2019/07/02  added @noerip
#---------------------------------------------------------------------------
  use strict;
# use Fcntl;					#not used (defines O_RDONLY constant (O_RDONLY=0,O_WRONLY=1,O_RDWR=2))
# use Net::CIDR::Lite;				#this is loaded below via &try_load_module()
# use DBI;					#this is loaded below via &try_load_module()
# use Time::HiRes qw(gettimeofday time);	#for time stamps with us resolution
# use warnings::unused;

  my ($script, $narg, $ip, $syntax, $cidr);
  my ($status, @records, $record, @words, $word, @parts, $nparts);
  my ($ipmin, $ipmax, $found, $i, $j, $debug, $msg);
  my (@beamlines, @day1, @day2, @ipranges, @iptypes, $nranges);
  my (@blf, @d1f, @d2f, @ipf, $nfound, %BL, $b, @homeip, $h);
  my (@noerip, $noerr);

  select STDERR; $|=1;                          #set unbuffered output
  select STDOUT; $|=1;                          #set unbuffered output

  $script = $0;
  $script =~ s/\\/\//g;				#change \ to /
  $script =~ s/^.*\///g;			#remove everything before last "/"

  &try_load_module('Net::CIDR::Lite');
  &try_load_module('DBI');
  &try_load_module('Time::HiRes qw(gettimeofday time)');

  $syntax = "\n".'Syntax: '.$script.' ip'."\n\n";
  $narg = @ARGV;
  if ($narg != 1) {&die_message('Incorrect syntax, expecting: '.$script.' ip');}
  $ip = $ARGV[0];
  if ($ip !~ /^([0-9]{1,3}\.){3}[0-9]{1,3}$/) {
     &die_message('Incorrect IP address='.$ip);
  }
  @words = split(/\./,$ip);
  foreach $word (@words) {
     if ($word > 255) {&die_message('Incorrect IP address='.$ip.', (part='.$word.')');}
  }

# $debug = 1;
# $ip = '164.54.103.18';		#sergey@gmca
# $ip = '164.54.22.109';		#one of the APS Metis virtual desktops
# $ip = '73.8.1.1';			#sergey@home: 67.162.0.0/17  73.0.0.0/8  98.192.0.0/10

  ($status,$msg) = &read_last_ssh_update(\@records);
  if ($status) {&die_message('failure from read_last_ssh_update: '.$msg);}

  if (!defined $debug) {$debug = 0;}

### Add GMCA and Argonne IPs (should be enabled in sshd_config and htd_require_ip.cfg):
### See also respective IP filters (they should match!) in the PHP code:
### WWW/userprogram/guest-wifi.php and WWW_local/index.php
# @homeip = ('164.54.103',      '164.54.0.0/16', '130.202.0.0/16', '146.137.0.0/16');
  @homeip = ('164.54.103.0/24', '164.54.142.10', '164.54.142.11', '164.54.22.38', '164.54.22.100', '130.202.154.0/23');
  @noerip = ('164.54.0.0/16', '130.202.0.0/16', '146.137.0.0/16', '192.5.186.0/24');
  $i = @records;
  for ($j=1; $j<=3; $j++) {
     $records[$i]->{user} = '*any*';
     $records[$i]->{bl}   = 'BL'.$j;
     $records[$i]->{day1} = 1;
     $records[$i]->{day2} = 1;
     $records[$i]->{ips}  = join(',',@homeip);
     $i++;
  }
### Check if the IP is originated from the lab and then do not report "No any beamine access"
  $cidr = Net::CIDR::Lite->new;
  foreach $word (@noerip) {
     $word =~ s/\.\*//g;								#replace 1.2.*.* by 1.2 (for IP range below)
     if    ($word =~ /^([0-9]{1,3}\.){1,3}[0-9]{1,3}\/{1}[0-9]{1,2}$/) {		#record is CIDR: 1.2.3/24
        $cidr->add($word);
     }
     elsif ($word =~ /^([0-9]{1,3}\.){1,3}[0-9]{1,3}$/) {				#record is IP or range: 1.2.3 (which means 1.2.3.1-1.2.3.255)
        @parts = split(/\./,$word);
        $nparts = @parts;
        if    ($nparts < 4 && $nparts > 1) {
           $ipmin = $ipmax = $word;
           for ($i=0; $i<(4-$nparts); $i++) {$ipmin .= '.1'; $ipmax .= '.255';}
           $cidr->add_range($ipmin.'-'.$ipmax);
        } 
        elsif ($nparts == 4) {							#record is single IP 1.2.3.4
           $cidr->add_ip($word);
        }
     }
  }
  $noerr = $cidr->find($ip);								#returns 1 or empty string
  if ($noerr ne '1') {$noerr = 0;}                                                      #by default do report "No any beamine access"
  if ($debug) {
     if ($noerr) {print STDOUT 'Specified IP='.$ip.' is within the lab; no error reporting.'."\n";}
     else        {print STDOUT 'Specified IP='.$ip.' is outside the lab; errors will be reported.'."\n";}
  }
### End of check

  @beamlines = ();
  @day1      = ();
  @day2      = ();
  @ipranges  = ();
  @iptypes   = ();
  foreach $record (@records) {
     @words = split(',',$record->{ips});
     foreach $word (@words) {
        if ($debug) {print STDOUT $record->{bl},': word=',$word,"\n";}
        $word =~ s/\.\*//g;								#replace 1.2.*.* by 1.2 (for IP range below)
        if    ($word =~ /^([0-9]{1,3}\.){1,3}[0-9]{1,3}\/{1}[0-9]{1,2}$/) {		#record is CIDR: 1.2.3/24
           if ($debug) {print STDOUT $record->{bl},': found CIDR=',$word,"\n";}
	   push (@ipranges,$word);
	   push (@iptypes,'cidr'); 
        }
        elsif ($word =~ /^([0-9]{1,3}\.){1,3}[0-9]{1,3}$/) {				#record is IP or range: 1.2.3 (which means 1.2.3.1-1.2.3.255)
	   @parts = split(/\./,$word);
	   $nparts = @parts;
	   if    ($nparts < 4 && $nparts > 1) {
 	      $ipmin = $ipmax = $word;
 	      for ($i=0; $i<(4-$nparts); $i++) {$ipmin .= '.1'; $ipmax .= '.255';}
              if ($debug) {print STDOUT $record->{bl},': found RANGE=',$ipmin.'-'.$ipmax,"\n";}
	      push (@ipranges,$ipmin.'-'.$ipmax);
	      push (@iptypes,'range'); 
	   } 
	   elsif ($nparts == 4) {							#record is single IP 1.2.3.4
              if ($debug) {print STDOUT $record->{bl},': found IP=',$word,"\n";}
	      push (@ipranges,$word);
	      push (@iptypes,'ip'); 
	   }
	   else {
	   }
        }
        else {
           &die_message('Unexpected word=',$word.' in record='.$record->{bl});
	}
        push (@beamlines,$record->{bl});
        push (@day1,     $record->{day1});
        push (@day2,     $record->{day2});
     }
  }
  @blf = ();
  @d1f = ();
  @d2f = ();
  @ipf = ();
  $nranges = @ipranges;
  $nfound = 0;
  for ($i=0; $i<$nranges; $i++) {
     $cidr = Net::CIDR::Lite->new;
     if    ($iptypes[$i] eq 'cidr')  {$cidr->add($ipranges[$i]);}
     elsif ($iptypes[$i] eq 'range') {$cidr->add_range($ipranges[$i]);}
     elsif ($iptypes[$i] eq 'ip')    {$cidr->add_ip($ipranges[$i]);}
     else {&die_message('unexpected IP type='.$iptypes[$i]);}
     $found = $cidr->find($ip);								#returns 1 or empty string
     if ($found ne '1') {$found = 0;}
     if ($found) {
        $nfound++;
        push(@blf, $beamlines[$i]);
        push(@d1f, $day1[$i]);
        push(@d2f, $day2[$i]);
        push(@ipf, $ipranges[$i]);
        if ($debug) {
           if ($nfound == 1) {print STDOUT "\n\n".'================================================'."\n";}
 	   print STDOUT 'found: ip='.$ip.' belongs to ['.$ipranges[$i].'], beamline='.$beamlines[$i].', day1='.$day1[$i].', day2='.$day2[$i]."\n";
	}
     }
  }
  $nfound = @ipf;
  if ($nfound > 0) {
     for ($i=1; $i<=3; $i++) {
        for ($j=1; $j<=2; $j++) { 
           $BL{'BL'.$i}{'DAY'.$j} = 0;
        }
     }
     for ($i=0; $i<$nfound; $i++) {
        $b = $blf[$i];									#BL1/BL2/BL3
        if ($d1f[$i] == 1)         {$BL{$b}{'DAY1'} = 1;}
        if ($d2f[$i] == 1)         {$BL{$b}{'DAY2'} = 1;}
     }
#    if ($noerr == 0) {print STDERR '[',&ustamp(),'] [:note] [',$script,'] [IP='.$ip.' access OK to beamline='.$beamline.']'."\n";}
     print STDOUT '1';
     for ($i=1; $i<=3; $i++) {print STDOUT ' BL',$i,'(',$BL{'BL'.$i}{'DAY1'},',',$BL{'BL'.$i}{'DAY2'},')';}
     print STDOUT ' 0 "ALL OK"';
  } else {
     if ($noerr == 0) {print STDERR '[',&ustamp(),'] [:warn] [',$script,'] [IP='.$ip.' no any beamline access]'."\n";}
     print STDOUT '0 BL1(0,0) BL2(0,0) BL3(0,0) 0 "no any beamline access for this IP"';
  }
  if ($debug) {print STDOUT "\n\n";}
  exit;

#=================================================================

sub try_load_module($) {
  my $module = shift(@_);
  eval('use '.$module);
  if ($@) {&die_message('failed to load module='.$module);}
}

#==========================================================================

# sub read_last_ssh_update(\@records) {			#expects array of hashes, returns status
sub read_last_ssh_update(\@) {
  my ($recref, $dbhost, $dbase, $dbtable);
  my ($dbquery, $dsn, $dbh, $sth);
  my ($dbu, $dbp, $nrw, $sqlid, $sqlkey);
  my ($rawref, @recs, $rec, @fields, $i);
  $recref = shift(@_);

  $dbhost  = 'bl1upper.gmca.aps.anl.gov';		#this is for all 3 beamlines; should we use bl3upper for BM?
  $dbase   = 'gmca_accounts';
  $dbtable = 'server';
  $dbu     = 'dhs';					# this is for $dbhost, we cannot use the value from beamline.pl
  $dbp     = '';					# this is for $dbhost, we cannot use the value from beamline.pl
  $sqlid   = 'id';
  $sqlkey  = 'last_ssh_update';
  $dsn     = 'DBI:mysql:host='.$dbhost.';database='.$dbase;
  $dbquery = 'SELECT * FROM '.$dbtable.' where '.$sqlid.'="'.$sqlkey.'"';
  $dbh = DBI->connect($dsn, $dbu, $dbp, {PrintError=>0,});
  if (!defined $dbh) {
     &die_message('Cannot connect to mySQL server='.$dbhost); 
  }
  $sth = $dbh->prepare($dbquery);
  $nrw = $sth->execute();				#nrecords works for MySQL only. For SQLite result is def/undef only.
  if (!defined $nrw || $nrw < 1) {
     &die_message('no mySQL data for '.$sqlid.'='.$sqlkey);
     $sth->finish();
     $dbh->disconnect();
     return 1
  }
  $rawref = $sth->fetchrow_hashref();
  $sth->finish();
  $dbh->disconnect();
  if (!defined $rawref->{'value'}) {
     &die_message('no mySQL value for '.$sqlid.'='.$sqlkey);
     return 1;
  }

### The value contains records separated by space. Record examples:
# USER,BL,DAY1,DAY2,IP,IP,...
# test160,ID-B,first_day=False,second_day=True,164.54.*.*
# testuseridb2,ID-B,first_day=False,second_day=True,164.54.*.*,67.184.*.*,73.8.*.*
  @recs = split(/\s+/,$rawref->{'value'});
  $i = 0;
  foreach $rec (@recs) {
     if ($debug) {print STDOUT 'SQLREC: '.$rec,"\n";}
     @fields = split(',',$rec);
     $$recref[$i]{user} = shift @fields;
     $$recref[$i]{bl}   = shift @fields;
     if ($$recref[$i]{bl} !~ /^ID-D$/ && 
         $$recref[$i]{bl} !~ /^ID-B$/ && 
         $$recref[$i]{bl} !~ /^BM$/) {
        &die_message('incorrect beamline='.$$recref[$i]{bl}.' in SQL record for user='.$$recref[$i]{user});
        return 1;
     }
     if    ($$recref[$i]{bl} =~ /^ID-D$/) {$$recref[$i]{bl}='BL1';}
     elsif ($$recref[$i]{bl} =~ /^ID-B$/) {$$recref[$i]{bl}='BL2';}
     elsif ($$recref[$i]{bl} =~ /^BM$/)   {$$recref[$i]{bl}='BL3';}

     $$recref[$i]{day1} = shift @fields;
     if ($$recref[$i]{day1} !~ /^first_day=/) {
        &die_message('incorrect day1 field '.$$recref[$i]{day1}.' in SQL record for user='.$$recref[$i]{user});
        return 1;
     }
     $$recref[$i]{day1} =~ s/^first_day=//;
     if    ($$recref[$i]{day1} =~ /^True/i)  {$$recref[$i]{day1}=1;} 
     elsif ($$recref[$i]{day1} =~ /^False/i) {$$recref[$i]{day1}=0;} 
     else {
        &die_message('incorrect day1 value first_day='.$$recref[$i]{day1}.' in SQL record for user='.$$recref[$i]{user});
        return 1;
     }
     $$recref[$i]{day2} = shift @fields;
     if ($$recref[$i]{day2} !~ /^second_day=/) {
        &die_message('incorrect day2 field '.$$recref[$i]{day2}.' in SQL record for user='.$$recref[$i]{user});
        return 1;
     }
     $$recref[$i]{day2} =~ s/^second_day=//;
     if    ($$recref[$i]{day2} =~ /^True/i)  {$$recref[$i]{day2}=1;} 
     elsif ($$recref[$i]{day2} =~ /^False/i) {$$recref[$i]{day2}=0;} 
     else {
        &die_message('incorrect day2 value second_day='.$$recref[$i]{day2}.' in SQL record for user='.$$recref[$i]{user});
        return 1;
     }
     $$recref[$i]{ips} = join(',',@fields);
     $i++;     
  }
  return 0,'';
}

#==========================================================================

sub ustamp() {
  my ($epoch, $usecs, $stamp, @words);
# my ($sec,$minute,$hour,$mday,$month,$year,$monthName,$weekday);
  
  ($epoch, $usecs) = gettimeofday();
  $stamp = scalar(localtime($epoch));
  
#  my @ct = localtime($epoch)
#  ($sec,$minute,$hour,$mday,$month,$year) = ($ct[0],$ct[1],$ct[2],$ct[3],$ct[4]+1,$ct[5]+1900);
 @words = split(/\s+/,$stamp);
 $words[-2] .= sprintf '.%06d', $usecs;
 $stamp = join(' ',@words);
 return $stamp;
}

#==========================================================================

sub die_message($) {
  my ($msg);
  $msg = shift(@_);
  print STDERR '*** '.$script.': '.$msg."\n";
  print STDOUT '0 BL1(0,0) BL2(0,0) BL3(0,0) 1 "'.$msg.'"';
  exit 1;
}
