#!/usr/bin/perl -W
# _CALLED_FROM_BLUICE_     *** OBSOLETE ***
#--------------------------------------------------------------
# This script was called by JBluIce at the beginning of frames
# acquisition. Now it is replaced by the server version sampleEnvSrv.pl.
# If you still need a command-line version (e.g. calling from MEDM),
# the use sampleEnvWrapper.pl which converts command line arguments to
# calls to the server (to sampleEnvSrv.pl).
#
# This script performs the following collective actions:
#
# (1) Normal operation ("in"):
#    - closes fast shutter (unless we are not moving collimator or pin diode)
#    - puts collimator IN
#    - puts backlight out (if requested or conditioned by collision with beamstop)
#    - flips beamstop IN
#    - does nothing to pindiode, but may optionally move pindiode IN or OUT
#    - does nothing to ringlight brightness, but may optionally do whatever requested
#    - does nothing to backlight brightness, but may optionally do whatever requested
# (2) Reverse operation ("out"):
#    - closes fast shutter
#    - puts collimator OUT
#    - flips beamstop OUT (if requested or conditioned by collision with backlight)
#    - puts backlight IN
#    - does nothing to pindiode, but may optionally move pindiode IN or OUT
#    - does nothing to ringlight brightness, but may optionally do whatever requested
#    - does nothing to backlight brightness, but may optionally do whatever requested
#
# The script can also move any individual component or any individual component
# can be excluded from the above sequences.
#
# The results are optionally reported to BluIce.
#
#                        Author: Sergey Stepanov
#
# Version-1.0: 2006/04/14 using Pezca
# Version-2.0: 2006/05/16 added "-reverse" flag, i.e. collimatorOut
# Version-2.1: 2006/12/07 added multiple attempts to move collimator
# Version-2.2: 2007/04/16 added verifying AutoStart at collimator motion
# Version-2.3: 2007/07/06 added skipping motions if collimator is already in place
# Version-3.0: 2007/10/09 adapted to new preset scheme: multiple presets, active preset, both X&Y.
# Version-4.0: 2008/10/20 ringlight brightness controls added; peristant backlight & beamstop added.
# Version-4.1: 2009/07/29 performed code cleanup
# Version-5.0: 2009/09/10 moved backlight on digital IO; added beamstop In/Out readback
# Version-6.0: 2009/09/15 removed calls of beamstopIN script and replaced it with internal beamstop flip controls;
#                         changed many flags and renamed from collimatorIn to collimatorCtrl
# Version-6.1: 2009/09/23 added disabling shutter during the script execution;
#                         moved collimator operation up for faster execution
# Version-6.2: 2009/10/12 shutter may not be closed if we move collimator only;
# Version-6.3: 2009/11/17 added protection against multiple instances
# Version-6.4: 2010/01/05 added backlight brightness controls
# Version-6.5: 2010/01/15 added beamstop distance threshold after which it cannot
#                         not collide with backlight and hence moving one the two
#                         IN does not require removing the other.
# Version-6.6: 2010/01/16 same as v.6.5, but simplified
# Version-6.7: 2010/04/13 added processing of beamstop and backlight not flipping
#                         when someone stands on safety mat
# Version-6.8: 2010/06/08 changed treating collimator=none : always take out
# Version-6.9: 2010/12/14 added SDIS checks for backlight and beamstop
# Version-7.0: 2011/01/06 added global flag for opening popups via BluIce
# Version-7.1: 2011/01/11 added opening guard slits when pinhole is requested
#                         and restoring it back
# Version-7.2: 2011/01/17 changed handling guard slits when switching between pinhole & scatterguard:
#                         guard slits are only opened to the minimum value per each pin and
#                         never closed back.
# Version-7.3: 2011/02/08 added workaround for backlight and beamstop flip: when one
#                         tries to command them after an attemt with SDIS=1, the value
#                         is there, but not in effect CA server ignores requests (thinks
#                         that it is already there. The workaround is to clear the value
#                         (send cmd=rbk) and then send it again.
# Version-8.0: 2013/02/27 added SONICC device. In reality SONICC can only collide with beamstop, but for
#                         safety the script assumes that it may also collide with PinDiode and backlight;
#                         Introduced collision matrix;
#                         Introduced options to move any component individually (the script auto-resolves potential collisions);
#                         Renamed the script to sampleEnv;
# Version-8.1: 2014/03/28 Changed functions calls to arrays pointers (preparation to Perl-5.16)
# Version-8.2: 2014/06/24 Added auto-detection if SONICC was hooked
#--------------------------------------------------------------
  use strict;
  use Pezca;

  our ($beamline, $shutter, $shutter_readback);
  our ($IGNORE_ERRORS, $NONINTERACTIVE, $USE_ZENITY_FOR_EXIT, $ZENITY_VIA_BLUICE);
  my ($PEZCA, $script, $scr, $syntax, $status, $shutter_assy);
  my (@beamlinelist, $i, $ln_specified, $op_specified, $so_specified);
  my ($cl_specified, $bs_specified, $pd_specified, $bl_specified);
  my ($rb_specified, $bb_specified, $tb_specified, $gs_specified);
  my (%collision, %hooked, %move, @obj_all, @obj, %state);
  my ($ringbright_ctrl, $ringbright_hooked, $backbright_ctrl, $backbright_hooked);
  my ($soniccAssy_PV, $soniccAxis_PV, $sonicc_tolerance);
  my ($sonicc_in_PV, $sonicc_out_PV, $sonicc_in_pos, $sonicc_out_pos);
  my ($colAssy_PV, @colAxes_PV, @col_dest_pos, $col_tolerance);
  my (@col_in_PV, @col_out_PV, @col_in_pos, @col_out_pos);
  my ($col_act_preset_PV, $col_active_preset, $col_started);
  my ($gsAssyV_PV, $gsSizeV_PV, $gsAssyH_PV, $gsSizeH_PV);
  my ($gsMinima_PV, $adjust_gs);
  my ($col_start_time, $col_motionTime);
  my ($backlight_cmd_PV, $backlight_rbk_PV, $backlight_act_PV);
  my ($backlight_out_pos, $backlight_in_pos);
  my ($beamstopflip_cmd_PV, $beamstopflip_rbk_PV, $beamstopflip_act_PV);
  my ($beamstopflip_out_pos, $beamstopflip_in_pos);
  my ($backlight_sdis_PV, $beamstopflip_sdis_PV);
  my ($bsAssy_PV, @bsAxes_PV, $bs_tolerance);
  my ($bs_in_PV, $bs_out_PV, $bs_in_pos, $bs_out_pos);
  my ($pinAssy_PV, $pinAxis_PV, $pin_tolerance);
  my ($pin_in_PV, $pin_out_PV, $pin_in_pos, $pin_out_pos);
  my ($bsDistance, $bsDistance_threshold, $bs_threshold_PV);
  my ($ringbright_ctrl_PV, $ringbright_save_PV, $ringbright_val);
  my ($backbright_ctrl_PV, $backbright_save_PV, $backbright_val);
  my ($attemptsMax, $start_time, $end_time, $tell_bluice);
  my ($safetymat_hooked_PV, $safetymat_state_PV, $safetymat_state);
  my ($flip_timeout, $flip_poll, $flip_polldelay, $flip_counter, $flip_tolerance);
  my ($component, $a, $b, $print_collisions, $l_limit, $h_limit);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/shutter_subs.pl';
  do $PEZCA.'/Common/motor_subs.pl';
  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDOUT; $|=1;					#set unbuffered output
  select STDERR; $|=1;					#set unbuffered output

  $SIG{'HUP'}  = 'signal_handler';			#(1) hangup (see signum.h)
  $SIG{'INT'}  = 'signal_handler';			#(2) interrupt (see signum.h)
  $SIG{'QUIT'} = 'signal_handler';			#(3) quit (see signum.h)
# $SIG{'KILL'} = 'signal_handler';			#(9) this is what is sent by kill -9 -- unblockable
  $SIG{'TERM'} = 'signal_handler';			#(15) terminate -- sent by kill without paramaters
# $SIG{'STOP'} = 'signal_handler';			#(19) stop -- unblockable
#-----------------------------------------------------------------
  $IGNORE_ERRORS       = 1;				#don't exit on error
  $NONINTERACTIVE      = 1;				#don't wait for any answers
  $USE_ZENITY_FOR_EXIT = 1;

### Set this to really big value to presume
### that collisions are always possible:
### (note: may be overwritten below by
### the content of BS:D:critical PV)
  $bsDistance_threshold = 35.;				#threshold for colliding with backlight

  $start_time = time();

  $script = &makeScriptName();
  $scr    = $script;
  $scr   =~ s/\.pl$//i;
  @obj_all = qw/Collimator Beamstop Pindiode Backlight SONICC/;

### Defaults (if no command line):
  foreach $a (@obj_all) {$move{$a} = 'none';}
  $adjust_gs       = 1;				#0=no, 1=adjust when collimator/scatterguard is moved in
  $ringbright_ctrl = 0;                 	#0=no_ctrl (default), 1=set a value
  $ringbright_val  = 0;
  $backbright_ctrl = 0;                 	#0=no_ctrl (default), 1=set a value
  $backbright_val  = 0;
  $flip_timeout    = 10;			#in seconds
  $flip_poll       = 0.1;			#polling interval in seconds
  $flip_polldelay  = 0.2;			#initial delay in seconds before starting polling
  $flip_counter    = 1;				#how many times to check that flip occurred
  $flip_tolerance  = 0.01;			#flip destination tolerance
  $tell_bluice     = 1;

  &protect_multirun();						#protect from multiple instances
  &read_commandline();						#read command line parameters

###-----------------------------------------------------------------
  if ($shutter          !~ /^${beamline}/) {$shutter          = $beamline.':'.$shutter;}
  if ($shutter_readback !~ /^${beamline}/) {$shutter_readback = $beamline.':'.$shutter_readback;}
  $shutter_assy = $beamline.':SH:Ps';
  &setAssyAuto ($shutter_assy);
###=============================
### Collimator (motor):
  $colAssy_PV        = $beamline.':COL:St';
  @colAxes_PV        =($beamline.':COL:H',$beamline.':COL:V');
  $col_act_preset_PV = $colAssy_PV.':ActivePreset';
  $col_tolerance     = 0.03;					#max|act-rqs| in mm
  $attemptsMax       = 3;					#how may times to try moving
  $hooked{Collimator} = 1;					#beamline config parameter
# if ($beamline eq '23o') {$hooked{Collimator} = 0;}		#collimator not connected at ID-out
###=============================
### Guard Slits (X&Y sizes - motors):
  $gsAssyV_PV        = $beamline.':GS:Av';
  $gsAssyH_PV        = $beamline.':GS:Ah';
  $gsSizeV_PV        = $beamline.':GS:ZS';
  $gsSizeH_PV        = $beamline.':GS:YS';
  $gsMinima_PV       = $beamline.':bi:guards_minimum';
  $hooked{Guardslits} = 1;						#beamline config parameter
###=============================
### Backlight (digital IO):
  $backlight_cmd_PV  = $beamline.':pmac21:acc65e:bo13';
  $backlight_rbk_PV  = $beamline.':pmac21:acc65e:rb13';
  $backlight_act_PV  = $beamline.':pmac21:acc65e:bi20';
# $backlight_sdis_PV = $backlight_cmd_PV.'.DISA';		#this is direct, but no CA monitor
  $backlight_sdis_PV = $backlight_cmd_PV;
  $backlight_sdis_PV =~ s/:bo/:SDIS/;
  $backlight_out_pos = 0.;
  $backlight_in_pos  = 1.;                              	#after  Sept-2009 (digital IO)
  $hooked{Backlight} = 1;					#beamline config parameter
# if ($beamline eq '23o') {$hooked{Backlight} = 0;}		#there is no backlight at ID-out
###=============================
### Beamstop (digital IO):
  $beamstopflip_cmd_PV  = $beamline.':pmac21:acc65e:bo14';
  $beamstopflip_rbk_PV  = $beamline.':pmac21:acc65e:rb14';
  $beamstopflip_act_PV  = $beamline.':pmac21:acc65e:bi21';
# $beamstopflip_sdis_PV = $beamstopflip_cmd_PV.'.DISA';		#this is direct, but no CA monitor
  $beamstopflip_sdis_PV = $beamstopflip_cmd_PV;
  $beamstopflip_sdis_PV =~ s/:bo/:SDIS/;
  $beamstopflip_out_pos = 0.;
  $beamstopflip_in_pos  = 1.;                              	#after  Sept-2009 (digital IO)
  $bsAssy_PV         = $beamline.':BS:Ps';
  @bsAxes_PV         = ($beamline.':BS:PT',$beamline.':BS:D',$beamline.':BS:YW');
  $bs_in_PV          = $bsAxes_PV[0].':RqsPosMem1';
  $bs_out_PV         = $bsAxes_PV[0].':RqsPosMem2';
  $bs_threshold_PV   = $bsAxes_PV[1].':critical';
  $bs_tolerance      = 0.1; 					#max |act-rqs| in mm
  $hooked{Beamstop}  = 1;					#beamline config parameter
# if ($beamline eq '23o') {$hooked{Beamstop} = 0;}		#there is no beamstop at ID-out
###=============================
### Pin Diode (motor):
  $pinAssy_PV        = $beamline.':PIN:St';
  $pinAxis_PV        = $beamline.':PIN:P';
  $pin_in_PV         = $pinAxis_PV.':RqsPosMem1';
  $pin_out_PV        = $pinAxis_PV.':RqsPosMem2';
  $pin_tolerance     = 0.2; 					#max |act-rqs| in mm
  $hooked{Pindiode}  = 1;					#beamline config parameter
# if ($beamline eq '23b') {$hooked{Pindiode} = 0;}    		#there is no pin diode at BM
###=============================
### SONICC (motor):
# $soniccAssy_PV        = $beamline.':JET:Ps';
# $soniccAxis_PV        = $beamline.':JET:P';
  $soniccAssy_PV        = $beamline.':SNC:Ps';
  $soniccAxis_PV        = $beamline.':SNC:P';
  $sonicc_in_PV         = $soniccAxis_PV.':RqsPosMem1';
  $sonicc_out_PV        = $soniccAxis_PV.':RqsPosMem2';
  $sonicc_tolerance     = 0.2; 					#max |act-rqs| in mm
  $hooked{SONICC}       = 0;					#beamline config parameter
  ### Comment this out when SONICC experiment is over:
  if ($beamline eq '23o') {		#SONICC is only available at 23ID-OUT during SONICC experiments
     ($l_limit,$h_limit) = &checkMotorLimits ($soniccAssy_PV);
     if ($l_limit*$h_limit == 0) {$hooked{SONICC} = 1;}		#if at least one limit is not set, motor is hooked
     else                        {$hooked{SONICC} = 0;}		#if both limits are set, then motor is unhooked
  }
###=============================
### Ringlight brightness:
  $ringbright_ctrl_PV = $beamline.':DAC1_4';
  $ringbright_save_PV = $ringbright_ctrl_PV.'_tweak.O'; 	#max=HOPR
  $hooked{Ringbrightness} = 1;					#beamline config parameter
###=============================
### Backlight brightness:
  $backbright_ctrl_PV = $beamline.':DAC1_8';
  $backbright_save_PV = $backbright_ctrl_PV.'_tweak.O'; 	#max=HOPR
  $hooked{Backbrightness} = 1;					#beamline config parameter
###=============================
### Safety mat:
  $safetymat_hooked_PV = $beamline.':pmac21:acc65e:bi23';
  $safetymat_state_PV  = $beamline.':pmac21:acc65e:bi22';
###=============================
### Check if beamstop may collide with backlight (when it is too upstream)
  if ($hooked{Beamstop}) {
### Read the threshold distance for beamstop colliding with backlight:
     ($status,$bsDistance) = Pezca::GetDouble($bs_threshold_PV);	#using here $bsDistance as temp to protect default $bsDistance_threshold
     if ($status) {&report_only($script,$status,$bs_threshold_PV);}	#in this case default $bsDistance_threshold will be used (see above)
     else         {$bsDistance_threshold = $bsDistance;}
### Read current beamstop distance:
     $bsDistance = &getAxisAct ($bsAxes_PV[1]);
  } else {
     $bsDistance = 0;
  }
###=============================
### Build collision matrix:
  @obj=();
  foreach $a (@obj_all) {if ($hooked{$a}) {push @obj,$a;}}
  foreach $a (@obj) {
  foreach $b (@obj) {
     if ($a ne $b) {$collision{$a}{$b} = 0;}
     else          {$collision{$a}{$b} = 1;}
  }}
  if ($hooked{Backlight}) {
     if ($hooked{Beamstop} && $bsDistance < $bsDistance_threshold) {
        $collision{Beamstop}{Backlight} = 1;
     }
     if ($hooked{Pindiode}) {
        $collision{Pindiode}{Backlight} = 1;
     }
  }
  if ($hooked{SONICC}) {
     if ($hooked{Beamstop})  {$collision{SONICC}{Beamstop}  = 1;}
     if ($hooked{Pindiode})  {$collision{SONICC}{Pindiode}  = 1;}
     if ($hooked{Backlight}) {$collision{SONICC}{Backlight} = 1;}
  }
  foreach $a (@obj) {
  foreach $b (@obj) {
     if ($collision{$a}{$b} == 1) {$collision{$b}{$a} = 1;}
  }}
### Print collisions matrix:
  if (defined $print_collisions) {
     print  STDOUT "\n".'Collisions matrix:'."\n"
                       .'------------------'."\n";
     printf STDOUT ' %-10s',' ';
     foreach $a (@obj) {printf STDOUT ' %-10s',$a;}
     print STDOUT "\n";
     foreach $a (@obj) {
        printf STDOUT '%10s', $a;
        foreach $b (@obj) {printf STDOUT '     %d     ',$collision{$a}{$b};}
        print STDOUT "\n";
     }
     print STDOUT "\n\n";
     &press_key_exit();
  }
###=============================

  &find_IN_and_OUT_collimator_PVs();

### Check if anyone stands on safety mat. Standing on the mat disables
### the Beamstop & Backlight moves (as it disables the digital IO):
  &check_safetymat();

### Find current state of all devices:
  foreach $a (@obj) {&getState($a);}

  $col_started = 0;
### First move all elements that are requested to be moved OUT:
  foreach $a (@obj) {
     if ($hooked{$a} && $move{$a} eq 'out' && $state{$a} ne 'out') {
        &moveObj($a,$move{$a});
     }
  }

### Now move all elements that are requested to be moved IN:
  foreach $a (@obj) {
     if ($hooked{$a} && $move{$a} eq 'in' && $state{$a} ne 'in') {
        ### Find possible collisions:
        foreach $b (@obj) {
           if ($b ne $a && $state{$b} ne 'out' && $collision{$a}{$b} == 1) {
              if ($move{$b} ne 'in') {
                 &message('will move '.$b.' OUT to avoid collision with '.$a,'warn');
                 &moveObj($b,'out');
              } else {
                 &cleanup_zenity_exit('Contradictory request: '.$b.' was requested to move IN, while it'
                                     .' must to be moved OUT in view of potential collision with '.$a);
              }
           }
        }
        &moveObj($a,$move{$a});
     }
  }

### Control ringlight brightness:
  if ($hooked{Ringbrightness} && $ringbright_ctrl) {
     &controlRinglightBrightness();
  }

### Control backlight brightness:
  if ($hooked{Backbrightness} && $backbright_ctrl) {
     &controlBacklightBrightness();
  }

  if ($hooked{Collimator} && $col_started) {
     &finishCollimator('Collimator',$move{Collimator});
  }

  &setAssyAuto ($shutter_assy);
### Print final states of all devices:
  foreach $a (@obj) {print STDOUT $script.': '.$a.' final state='.uc($state{$a}).".\n";}

  $end_time = time();
  print STDOUT  "\n";
  printf STDOUT $script.': Elapsed time=%.1f seconds'."\n",($end_time-$start_time);
  print STDOUT  $script.': job finished'."\n\n";
  &clearAllCaMonitors();
  exit 0;

#=================================================================

sub protect_multirun() {
  my ($filter, $nprocesses);

### Protection from multiple instances:
  if ($^O !~ /Win/i) {
#    $filter = '/bin/ps -ef | '
     $filter = '/bin/ps -eo user:19,pid,cmd | '
              .'/bin/grep -iv "grep\|edit\|term\|onsole\|sampleEnvSrv" | '
              .'/bin/grep ';
     $nprocesses  = `${filter} -c ${scr}`;
     chomp $nprocesses;
     $nprocesses = $nprocesses - 1;				#ignore this script
     if ($nprocesses > 0) {
        $NONINTERACTIVE      = 1;				#do not wait for any answers
        $USE_ZENITY_FOR_EXIT = 1;
        if (grep(/^j?bluice$/i, @ARGV)) {$ZENITY_VIA_BLUICE=1;}
        &zenity_error_exit('*** '.$nprocesses.' instance(s) of '.$script.' not yet finished. Please try again.');
     }
  }
}

#=================================================================

sub read_commandline () {

  my ($i, $arg, $narg);
  $syntax = "\n\n".'Syntax: '.$script.' [beamline] [help|?] [printCollisions]'."\n";
  $syntax .= ' IN|OUT'."\n";
  $syntax .= ' [noCollimator|collimatorIN|collimatorOUT]'."\n";
  $syntax .= ' [noBeamstop|beamstopIN|beamstopOUT]'."\n";
  $syntax .= ' [noBacklight|backlightIN|backlightOUT]'."\n";
  $syntax .= ' [noPindiode|pindiodeIN|pindiodeOUT]'."\n";
  $syntax .= ' [noSonicc|soniccIN|soniccOUT]'."\n";
  $syntax .= ' [ringlight=on|ringlight=value|ringlight=off|ringlight=skip]'."\n";
  $syntax .= ' [backlight=on|backlight=value|backlight=off|backlight=skip]'."\n";
  $syntax .= ' [handleGS|skipGS] [bluice|nobluice]'."\n";

  $narg = @ARGV;
  print STDOUT "\n".$script.': starting with cmdline: ';
  if ($narg > 0) {print STDOUT "@{ARGV}\n\n";}
  else           {print STDOUT '<no arguments>'."\n\n";}

### Make IN/OUT the first command line arguments:
  if ($narg > 1) {
     $i=1;
     do {
        $arg = $ARGV[$i];
        if ($arg =~ /^-{0,2}IN$/i || $arg =~ /^-{0,2}OUT$/i) {
           splice(@ARGV, $i, 1);
           unshift(@ARGV, $arg);
        } else {
           $i++;
        }
     } while ($i<$narg);
  }
### Read command line arguments:
  @beamlinelist = ('23i:', '23o:', '23b:', '23d:');
  for ($i=0; $i<$narg; $i++) {
     $arg = $ARGV[$i];
     $arg =~ s/^-{0,2}//;					#trim leading dashes
### Help requested:
     if ($arg =~ /^help$/i || $arg =~ /^\?$/) {
        &press_key_exit($syntax);				#this will exit automatically if $NONINTERACTIVE=1
     }
### Collision matrix requested:
     elsif ($arg =~ /^printCollisions$/i) {
        if (defined $print_collisions) {
           &press_key_exit('*** '.$script
                 .': Duplicate "printCollisions" parameter in the command line.'
                 .$syntax);					#this will exit automatically if $NONINTERACTIVE=1
        }
        $print_collisions = 1
     }
### Check if the default beamline name is overridden:
     elsif (grep(/^$arg:?$/, @beamlinelist)) {
        if (defined $ln_specified) {
           &press_key_exit('*** '.$script
                 .': Duplicate "beamline" parameter in the command line.'
                 .$syntax);					#this will exit automatically if $NONINTERACTIVE=1
        }
        $ln_specified=1;
        $arg =~ s/:$//;						#trim ':'
        if ($beamline ne $arg) {
           print STDOUT "\n".'!!! '.$script.': Cross-beamline operation: controlling '.$arg.' from '.$beamline."\n\n";
           $beamline = $arg;
           do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
        }
     }
### Syntax = "[in|out|componentIN(OUT)=...]";
     elsif ($arg =~ /^out$/i ||
            $arg =~ /^in$/i) {
        if (defined $op_specified) {
           &press_key_exit('*** '.$script
                 .': Duplicate "IN/OUT" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
        $op_specified=1;
        print STDOUT '--> '.$script.': Operation: '.uc($arg).'!'."\n";
        if    ($arg =~ /^in$/i) {
           $move{Collimator} = 'in';
           $move{Beamstop}   = 'in';
           $move{Backlight}  = 'out';
        }
        elsif ($arg =~ /^out$/i) {
           $move{Collimator} = 'out';
           $move{Beamstop}   = 'out';
           $move{Backlight}  = 'in';
        }
     }
### Syntax = "[noCollimator|collimatorIN|collimatorOUT]";
     elsif ($arg =~ /^noCollimator$/i  ||
            $arg =~ /^collimatorIN$/i  ||
            $arg =~ /^collimatorOUT$/i) {
        if (defined $cl_specified) {
           &press_key_exit('*** '.$script
                 .': Duplicate "collimator" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
        $cl_specified=1;
        if ($arg =~ /^noCollimator$/i) {
           $move{Collimator} = 'none';
           print STDOUT '--> '.$script.': collimator will not be moved unless possible collision!'."\n";
        }
        elsif ($arg =~ /^collimatorOUT$/i || $arg =~ /^collimatorIN$/i) {
           $arg =~ s/collimator//i;
           print STDOUT '--> '.$script.': collimator move-'.uc($arg).' requested!'."\n";
           $move{Beamstop} = lc($arg);
        }
        else {
           &press_key_exit('*** '.$script
                 .': Incorrect "collimator" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
     }
### Syntax = "[noBeamstop|beamstopNormal|beamstopOUT|beamstopIfneeded]";
     elsif ($arg =~ /^nobeamstop$/i  ||
            $arg =~ /^beamstopIN$/i  ||
            $arg =~ /^beamstopOUT$/i) {
        if (defined $bs_specified) {
           &press_key_exit('*** '.$script
                 .': Duplicate "beamstop" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
        $bs_specified=1;
        if ($arg =~ /^noBeamstop$/i) {
           $move{Beamstop} = 'none';
           print STDOUT '--> '.$script.': beamstop will not be flipped unless possible collision!'."\n";
        }
        elsif ($arg =~ /^beamstopOUT$/i || $arg =~ /^beamstopIN$/i) {
           $arg =~ s/beamstop//i;
           print STDOUT '--> '.$script.': beamstop flip-'.uc($arg).' requested!'."\n";
           $move{Beamstop} = lc($arg);
        }
        else {
           &press_key_exit('*** '.$script
                 .': Incorrect "beamstop" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
     }
### Syntax = "[noBacklight|backlightNormal|backlightOUT|backlightIfneeded]";
     elsif ($arg =~ /^nobacklight$/i  ||
            $arg =~ /^backlightIN$/i  ||
            $arg =~ /^backlightOUT$/i) {
        if (defined $bl_specified) {
           &press_key_exit('*** '.$script
                 .': Duplicate "backlight" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
        $bl_specified=1;
        if ($arg =~ /^nobacklight$/i) {
           $move{Backlight} = 'none';
           print STDOUT '--> '.$script.': backlight will not be moved unless possible collision!'."\n";
        }
        elsif ($arg =~ /^backlightOUT$/i || $arg =~ /^backlightIN$/i) {
           $arg =~ s/backlight//i;
           print STDOUT '--> '.$script.': backlight move-'.uc($arg).' requested!'."\n";
           $move{Backlight} = lc($arg);
        }
        else {
           &press_key_exit('*** '.$script
                 .': Incorrect "backlight" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
     }
### Syntax = [pindiodeIN|pindiodeOUT]";
     elsif ($arg =~ /^nopindiode/i   ||
            $arg =~ /^pindiodeIN$/i  ||
            $arg =~ /^pindiodeOUT$/i) {
        if (defined $pd_specified) {
           &press_key_exit('*** '.$script
                 .': Duplicate "pindiode" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
        $pd_specified=1;
        if ($arg =~ /^nopindiode$/i) {
           $move{Pindiode} = 'none';
           print STDOUT '--> '.$script.': pindiode will not be moved unless possible collision!'."\n";
        }
        elsif ($arg =~ /^pindiodeOUT$/i || $arg =~ /^pindiodeIN$/i) {
           $arg =~ s/pindiode//i;
           print STDOUT '--> '.$script.': pindiode move-'.uc($arg).' is requested!'."\n";
           $move{Pindiode} = lc($arg);
        }
        else {
           &press_key_exit('*** '.$script
                 .': Incorrect "pindiode" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
     }
### Syntax = [nosonicc|soniccIN|soniccOUT]";
     elsif ($arg =~ /^nosonicc$/i ||
            $arg =~ /^soniccIN$/i ||
            $arg =~ /^soniccOUT$/i) {
        if (defined $so_specified) {
           &press_key_exit('*** '.$script
                 .': Duplicate "SONICC" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
        $so_specified=1;
        if ($arg =~ /^nosonicc$/i) {
           $move{SONICC} = 'none';
           print STDOUT '--> '.$script.': SONICC will not be moved unless possible collision!'."\n";
        }
        elsif ($arg =~ /^soniccOUT$/i || $arg =~ /^soniccIN$/i) {
           $arg =~ s/sonicc//i;
           print STDOUT '--> '.$script.': SONICC move-'.uc($arg).' is requested!'."\n";
           $move{SONICC} = lc($arg);
        }
        else {
           &press_key_exit('*** '.$script
                 .': Incorrect "SONICC" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
     }
### Syntax = "[ringlight=skip|ringlight=on|ringlight=value|ringlight=off]";
     elsif ($arg =~ /^ringlight=/i) {
        if (defined $rb_specified) {
           &press_key_exit('*** '.$script
                 .': Duplicate "ringlight" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
        $rb_specified=1;
        $arg =~ s/^ringlight=//i;
        $ringbright_ctrl = 1;
        if ($arg =~ /^skip$/i) {
           $ringbright_ctrl = 0;
           print STDOUT '--> '.$script.': ringlight brightness operation disabled!'."\n";
        }
        elsif ($arg =~ /^on$/i) {
           $ringbright_val = undef;  				#take from last saved
           print STDOUT '--> '.$script
                 .': set ringlight ON, read brightness value from EPICS'."\n";
        }
        elsif ($arg =~ /^off$/i) {
           $ringbright_val = 0;
           print STDOUT '--> '.$script
                 .': set ringlight brightness value=0'."\n";
        }
        elsif ($arg =~ /^[+-]?[0-9]*[.]?[0-9]*$/i) {
           $ringbright_val = $arg;
           print STDOUT '--> '.$script
                 .': set ringlight brightness value='.$ringbright_val."\n";
        }
        else {
           &press_key_exit('*** '.$script
                 .': Incorrect "ringlight" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
     }
### Syntax = "[backlight=skip|backlight=on|backlight=value|backlight=off]";
     elsif ($arg =~ /^backlight=/i) {
        if (defined $bb_specified) {
           &press_key_exit('*** '.$script
                 .': Duplicate "backlight" brightness parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
        $bb_specified=1;
        $arg =~ s/^backlight=//i;
        $backbright_ctrl = 1;
        if ($arg =~ /^skip$/i) {
           $backbright_ctrl = 0;
           print STDOUT '--> '.$script
                 .': backlight brightness operation disabled!'."\n";
        }
        elsif ($arg =~ /^on$/i) {
           $backbright_val = undef;  				#take from last saved
           print STDOUT '--> '.$script
                 .': set backlight ON, read brightness value from EPICS'."\n";
        }
        elsif ($arg =~ /^off$/i) {
           $backbright_val = 0;
           print STDOUT '--> '.$script
                 .': set backlight brightness value=0'."\n";
        }
        elsif ($arg =~ /^[+-]?[0-9]*[.]?[0-9]*$/i) {
           $backbright_val = $arg;
           print STDOUT '--> '.$script
                 .': set backlight brightness value='.$backbright_val."\n";
        }
        else {
           &press_key_exit('*** '.$script
                 .': Incorrect "backlight" brightness parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
     }
### Syntax = [handleGS|skipGS]";
     elsif ($arg =~ /^handleGS$/i || $arg =~ /^skipGS$/i) {
        if (defined $gs_specified) {
           &press_key_exit('*** '.$script
                 .': Duplicate "GS handling" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
        $gs_specified=1;
        if ($arg =~ /^handleGS$/i) {
           $adjust_gs = 1;
           print STDOUT '--> '.$script.': GS handling is requested!'."\n";
        }
        elsif ($arg =~ /^skipGS$/i) {
           $adjust_gs = 0;
           print STDOUT '--> '.$script.': GS handling is disabled!'."\n";
        }
     }
### Syntax = [bluice|nobluice]";
     elsif ($arg =~ /^j?bluice$/i || $arg =~ /^noj?bluice$/i) {
        if (defined $tb_specified) {
           &press_key_exit('*** '.$script
                 .': Duplicate "jbluice" parameter in the command line: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
        }
        $tb_specified=1;
        if ($arg =~ /^j?bluice$/i) {
           $tell_bluice = 1;
           print STDOUT '--> '.$script.': reports to JBluIce are requested!'."\n";
        }
        elsif ($arg =~ /^noj?bluice$/i) {
           $tell_bluice = 0;
           print STDOUT '--> '.$script.': reports to JBluIce are disabled!'."\n";
        }
     }
### Incorrect (unrecognized) command line argument:
     else {
        &press_key_exit('*** '.$script
                 .': Incorrect command line argument: '
                 .$ARGV[$i].$syntax);				#this will exit automatically if $NONINTERACTIVE=1
     }
  }

  $ZENITY_VIA_BLUICE = $tell_bluice;
}

#=================================================================

sub find_IN_and_OUT_collimator_PVs() {
  my ($i, $j);
  if ($hooked{Collimator}) {
### Query current active collimator preset:
### 0=out, 1=300um, 2=20um, 3=10um, 4=5um
     ($status,$col_active_preset) = Pezca::GetDouble($col_act_preset_PV);
     &ezca_err($script,$status,$col_act_preset_PV,'. Getting active preset failed');
     if ($col_active_preset < 0 || $col_active_preset > 4) {
        &cleanup_zenity_exit('Unexpected value='.$col_active_preset
                            .' in active preset PV='.$col_act_preset_PV);
     }
     if ($col_active_preset != 0) {				#Collimator=IN (0=out,1=300um,2=20um,3=10um,4=5um)
        $j = $col_active_preset + 1;				#the 'RqsPosMem' PVs are numbered from 1 to 5
### Here: @colAxes_PV=('COL:H','COL:V');
        for ($i=0; $i<2; $i++) {				#loop over H & V
           $col_in_PV[$i] = $colAxes_PV[$i].':RqsPosMem'.$j;	#IN-position PVs
           if    ($colAxes_PV[$i] =~ /COL:V/) {
              $col_out_PV[$i] = $colAxes_PV[$i].':RqsPosMem1';  #OUT-position PV for vert.
           }
           elsif ($colAxes_PV[$i] =~ /COL:H/) {
### Move collimator OUT only vertically: ignore COL:H-out & use COL:H-in:
              $col_out_PV[$i] = $col_in_PV[$i];			#OUT-position PV for horiz.
           }
           else {
              &cleanup_zenity_exit('Unexpected collimator axis PV='.$colAxes_PV[$i]);
           }
        }
     }
     else {							#preset=None
#       &message('Collimator motion disabled because active preset is NONE');
#       $move{Collimator} = 'none';
### Always move Collimator-out:
        for ($i=0; $i<2; $i++) {
           $col_in_PV[$i]  = $colAxes_PV[$i].':RqsPosMem1';
           $col_out_PV[$i] = $col_in_PV[$i];
        }
     }
  }
}

#=================================================================

sub check_safetymat() {
### Check if anyone stands on safety mat. Standing on the mat disables
### the Beamstop & Backlight moves (as it disables the digital IO):
  if ($hooked{Beamstop} || $hooked{Backlight}) {
     ($status,$hooked{Safetymat}) = Pezca::GetDouble($safetymat_hooked_PV);
     if ($status) {
        &ezca_warn($script,$status,$safetymat_hooked_PV,'. Ignoring safety mat');
        $hooked{Safetymat} = 0;
     }
     if ($hooked{Safetymat}) {
        ($status,$safetymat_state) = Pezca::GetDouble($safetymat_state_PV);
        if ($status) {
           &ezca_warn($script,$status,$safetymat_state_PV,'. Ignoring safety mat state');
           $safetymat_state = 0;
        }
        if ($safetymat_state && ($move{Beamstop} ne 'none' || $move{Backlight} ne 'none')) {
           &cleanup_zenity_exit('Operation is disabled by safety mat. '
                               .'Please step away from the mat and repeat');
        }
     }
  }
}

#=================================================================

#sub getState($object) {
sub getState($) {
  my ($obj, @col_act, $bs_act, $beamstopflip_act_pos, $beamstopflip_rbk_pos);
  my ($pin_act_pos, $backlight_act_pos, $backlight_rbk_pos, $sonicc_act_pos, $i);
  $obj = shift(@_);
  if    ($obj eq 'Collimator') { ###########################################
     for ($i=0; $i<2; $i++) {
### Get collimator actual pos (horiz. & vert.):
        $col_act[$i] = &getAxisAct ($colAxes_PV[$i]);
### Get collimator IN/OUT positions:
        if (!defined $col_in_pos[$i]) {
           ($status,$col_in_pos[$i]) = Pezca::GetDouble($col_in_PV[$i]);
           &ezca_err($script,$status,$col_in_PV[$i],'. Getting '.$obj.' settings failed');
        }
        if (!defined $col_out_pos[$i]) {
           if ($col_out_PV[$i] eq $col_in_PV[$i]) {$col_out_pos[$i] = $col_in_pos[$i];}
           else {
              ($status,$col_out_pos[$i]) = Pezca::GetDouble($col_out_PV[$i]);
              &ezca_err($script,$status,$col_out_PV[$i],'. Getting '.$obj.' settings failed');
           }
        }
     }
     if    ((abs($col_act[0]-$col_in_pos[0])  < $col_tolerance) &&
            (abs($col_act[1]-$col_in_pos[1])  < $col_tolerance)) {$state{$obj}='in';}
     elsif ((abs($col_act[0]-$col_out_pos[0]) < $col_tolerance) &&
            (abs($col_act[1]-$col_out_pos[1]) < $col_tolerance)) {$state{$obj}='out';}
     else                                                        {$state{$obj}='unknown';}
  }
  elsif ($obj eq 'Beamstop') { #############################################
 ### Get Beamstop actual position:
     $bs_act = &getAxisAct ($bsAxes_PV[0]);			#BS:PT
### Get beamstop IN vertical motor position:
     if (!defined $bs_in_pos) {
        ($status,$bs_in_pos) = Pezca::GetDouble($bs_in_PV);
        &ezca_err($script,$status,$bs_in_PV,'. Getting '.$obj.' settings failed');
     }
### Read beamstop current flip:
     ($status, $beamstopflip_act_pos) = Pezca::GetDouble($beamstopflip_act_PV);
     &ezca_err($script,$status,$beamstopflip_act_PV,'. Getting '.$obj.' settings failed');
### Read beamstop current flip readback:
     ($status, $beamstopflip_rbk_pos) = Pezca::GetDouble($beamstopflip_rbk_PV);
     &ezca_err($script,$status,$beamstopflip_rbk_PV,' Getting '.$obj.' settings failed');
     if ($beamstopflip_act_pos != $beamstopflip_rbk_pos) {
        &cleanup_zenity_exit('Current '.$obj.' cmd & rbk disagree while getting '.$obj.' settings.');
     }
     if    (abs($beamstopflip_act_pos - $beamstopflip_in_pos)  < 0.01) {$state{$obj}='in';}
     elsif (abs($beamstopflip_act_pos - $beamstopflip_out_pos) < 0.01) {$state{$obj}='out';}
     else                                                              {$state{$obj}='unknown';}
     if (abs($bs_act - $bs_in_pos) > $bs_tolerance) {$state{$obj} .= '/notinplace';}
  }
  elsif ($obj eq 'Pindiode') { #############################################
### Read pin diode current position:
     $pin_act_pos = &getAxisAct ($pinAxis_PV);
### Read pindiode IN/OUT positions:
     if (!defined $pin_in_pos) {
        ($status,$pin_in_pos) = Pezca::GetDouble($pin_in_PV);
        &ezca_err($script,$status,$pin_in_PV,'. Getting '.$obj.' settings failed');
     }
     if (!defined $pin_out_pos) {
        ($status,$pin_out_pos) = Pezca::GetDouble($pin_out_PV);
        &ezca_err($script,$status,$pin_out_PV,'. Getting '.$obj.' settings failed');
     }
     if    (abs($pin_act_pos - $pin_in_pos)  < $pin_tolerance) {$state{$obj}='in';}
     elsif (abs($pin_act_pos - $pin_out_pos) < $pin_tolerance) {$state{$obj}='out';}
     else                                                      {$state{$obj}='unknown';}
  }
  elsif ($obj eq 'Backlight') { ############################################
### Read backlight current position:
     ($status, $backlight_act_pos) = Pezca::GetDouble($backlight_act_PV);
     &ezca_err($script,$status,$backlight_act_PV,'. Getting '.$obj.' settings failed');
### Read backlight current position readback:
     ($status, $backlight_rbk_pos) = Pezca::GetDouble($backlight_rbk_PV);
     &ezca_err($script,$status,$backlight_rbk_PV,'. Getting '.$obj.' settings failed');
     if ($backlight_act_pos != $backlight_rbk_pos) {
        &cleanup_zenity_exit('Current '.$obj.' cmd & rbk disagree while getting '.$obj.' settings.');
     }
     if    (abs($backlight_act_pos - $backlight_in_pos)  < 0.01) {$state{$obj}='in';}
     elsif (abs($backlight_act_pos - $backlight_out_pos) < 0.01) {$state{$obj}='out';}
     else                                                        {$state{$obj}='unknown';}
  }
  elsif ($obj eq 'SONICC') { ###############################################
### Read SONICC current position:
     $sonicc_act_pos = &getAxisAct ($soniccAxis_PV);
### Read pindiode IN/OUT positions:
     if (!defined $sonicc_in_pos) {
        ($status,$sonicc_in_pos) = Pezca::GetDouble($sonicc_in_PV);
        &ezca_err($script,$status,$sonicc_in_PV,'. Getting '.$obj.' settings failed');
     }
     if (!defined $sonicc_out_pos) {
        ($status,$sonicc_out_pos) = Pezca::GetDouble($sonicc_out_PV);
        &ezca_err($script,$status,$sonicc_out_PV,'. Getting '.$obj.' settings failed');
     }
     if    (abs($sonicc_act_pos - $sonicc_in_pos)  < $sonicc_tolerance) {$state{$obj}='in';}
     elsif (abs($sonicc_act_pos - $sonicc_out_pos) < $sonicc_tolerance) {$state{$obj}='out';}
     else                                                               {$state{$obj}='unknown';}
  }
  else { ###################################################################
     &cleanup_zenity_exit('Unknown sample environment object "'.$obj.'"');
  } ########################################################################
  print STDOUT $script.': '.$obj.' initial state='.uc($state{$obj}).".\n";
}

#=================================================================

#sub moveObj($object,$destination) {
sub moveObj($$) {
  my ($obj, $dest);
  $obj  = shift(@_);
  $dest = shift(@_);
  if ($dest ne 'in' && $dest ne 'out') {
     &cleanup_zenity_exit('Unknown destination "'.$dest.'" for sample environment object "'.$obj.'"');
  }
  if    ($obj eq 'Collimator') { ###########################################
     ($col_started,$col_motionTime) = &startCollimator($obj,$dest);
     $col_start_time = time();
     if ($dest eq 'in' && $col_started && $hooked{Guardslits} && $adjust_gs) {
        ###  Check if we need to open/close guard slits:
        &sortoutGuardSlits();
     }
  }
  elsif ($obj eq 'Beamstop') { #############################################
     &moveBeamstop($obj,$dest);
  }
  elsif ($obj eq 'Pindiode') { #############################################
     &movePindiode($obj,$dest);
  }
  elsif ($obj eq 'Backlight') { ############################################
     &moveBacklight($obj,$dest);
  }
  elsif ($obj eq 'SONICC') { ###############################################
     &moveSONICC($obj,$dest);
  }
  else { ###################################################################
     &cleanup_zenity_exit('Unknown sample environment object "'.$obj.'"');
  } ########################################################################
}

#=================================================================

# sub checkIfAssyMovable ($assy,$obj) {
sub checkIfAssyMovable ($$) {
  my ($assy, $obj, $motion, $homed);
  $assy = shift(@_);
  $obj  = shift(@_);

### Verify that motion program is not 'none':
  $motion = &getMotionPrg($assy);
  if ($motion == 0) {
     &cleanup_zenity_exit($obj.' motion is disabled. Cannot move the device');
  }

### Verify that assembly is homed:
  $homed = &getHomed($pinAssy_PV);
  if ($homed == 0) {
     &cleanup_zenity_exit($obj.' is not homed. Cannot move the device');
  }
}

#=================================================================

# sub moveBacklight ($obj,$dest) {
sub moveBacklight ($$) {
  my ($obj, $dest, $backlight_cmd_pos, $backlight_rbk_pos);
  my ($backlight_act_pos, $backlight_sdis);
  my ($attempt, @pvlist, $pv, $val);

  $obj  = shift(@_);
  $dest = shift(@_);

  if    ($dest eq 'in')  {$backlight_cmd_pos = $backlight_in_pos;}
  elsif ($dest eq 'out') {$backlight_cmd_pos = $backlight_out_pos;}
  else {
     &cleanup_zenity_exit('Unknown destination "'.$dest.'" for '.$obj.' while preparing to move.');
  }
  @pvlist = ($backlight_act_PV, $backlight_cmd_PV,
             $beamstopflip_act_PV, $beamstopflip_cmd_PV,
             $bsAxes_PV[0].':ActPos', $bs_threshold_PV,
             $beamline.':PIN:stateCALC');

# &addCaMonitor($backlight_rbk_PV);
# &addCaMonitor($backlight_act_PV);
# &addCaMonitor($backlight_sdis_PV);
# &addCaMonitor($beamstopflip_act_PV);

### If backlight is not where we need it:
  if ($state{$obj} ne $dest) {
     &close_disable_shutter();
     $attempt = 0;
     do {
        $attempt++;
### Start motion:
        &message('Moving '.$obj.' '.uc($dest).', attempt '.$attempt.'/'.$attemptsMax);
        $status = Pezca::PutDouble($backlight_cmd_PV,$backlight_cmd_pos);		#digital I/O
        &ezca_err($script,$status,$backlight_cmd_PV,'. Driving '.$obj.' '.uc($dest).' failed');
### Check if the command was accepted (SDIS?)
        select(undef,undef,undef,.25);							#normally takes 0.15 between cmd &rbk
        ($status, $backlight_rbk_pos) = Pezca::GetDouble($backlight_rbk_PV);
        if ($status) {
           &ezca_warn($script,$status,$backlight_rbk_PV,'. Taking the commanded position');
           $backlight_rbk_pos = $backlight_cmd_pos;
        }
        elsif ($backlight_cmd_pos != $backlight_rbk_pos) {
           &message('Operation "'.$obj.'-'.uc($dest).'" failed:'
                   .' readback did not confirm the command','warn');
           ($status, $backlight_sdis) = Pezca::GetDouble($backlight_sdis_PV);
           if ($status) {
              &ezca_warn($script,$status,$backlight_sdis_PV,'. Presuming SDIS=0');
              $backlight_sdis = 0;
           }
           if ($backlight_sdis) {
              print STDOUT '*** '.$script.': '.$obj.' motion disabled by these conditions (SDIS):'."\n";
              foreach $pv (@pvlist) {
                 ($status, $val) = Pezca::GetDouble($pv);
                 if ($status) {&report_only($script,$status,$pv); $val='undef';}
                 print STDOUT '!!! '.$script.': '.$pv.' = '.$val."\n";
              }
              select(undef,undef,undef,0.5);
           } else {
              &message('Clearing previous attempt to command '.$obj
                      .' motion while it was disabled (SDIS)');
              $status = Pezca::PutDouble($backlight_cmd_PV,$backlight_rbk_pos);
              &report_only($script,$status,$backlight_cmd_PV);
              select(undef,undef,undef,.25);
           }
        }
     } while ($attempt < $attemptsMax && $backlight_cmd_pos != $backlight_rbk_pos);
### Wait for motion to complete:
     if ($attempt >= $attemptsMax) {$status = 1;}
     else {
        $status = &wait4pv($backlight_act_PV, $backlight_cmd_pos,
                           $flip_timeout, $flip_poll, $flip_polldelay,
                           $flip_counter, $flip_tolerance);
     }
     if ($status) {
### Bring the request and the actual in sync (safety mat issue):
        ($status, $backlight_act_pos) = Pezca::GetDouble($backlight_act_PV);
        if (!$status) {Pezca::PutDouble($backlight_cmd_PV,$backlight_act_pos);}	#digital I/O
        &cleanup_zenity_exit('Timeout on driving '.$obj.' '.uc($dest));
     } else {
        select(undef,undef,undef,1.5);     					#sleep a bit extra;
        &message($obj.' has been moved '.uc($dest));
        $state{$obj} = $dest;
     }
  }
}

#=================================================================

# sub moveBeamstop ($obj,$dest) {
sub moveBeamstop ($$) {
  my ($obj, $dest, $beamstopflip_cmd_pos, $beamstopflip_rbk_pos);
  my ($beamstopflip_act_pos, $beamstopflip_sdis);
  my ($bs_act, $attempt, @pvlist, $pv, $val, $msg);

  $obj  = shift(@_);
  $dest = shift(@_);

  if    ($dest eq 'in')  {$beamstopflip_cmd_pos = $beamstopflip_in_pos;}
  elsif ($dest eq 'out') {$beamstopflip_cmd_pos = $beamstopflip_out_pos;}
  else {
     &cleanup_zenity_exit('Unknown destination "'.$dest.'" for '.$obj.' while preparing to move.');
  }
  @pvlist = ($beamstopflip_act_PV, $beamstopflip_cmd_PV,
             $backlight_act_PV, $backlight_cmd_PV,
             $bsAxes_PV[0].':ActPos', $bs_threshold_PV,
             $beamline.':PIN:stateCALC');

# &addCaMonitor($beamstopflip_rbk_PV);
# &addCaMonitor($beamstopflip_act_PV);
# &addCaMonitor($beamstopflip_sdis_PV);
# &addCaMonitor($backlight_act_PV);


  if ($dest eq 'in' && $state{$obj} =~ /notinplace/) {
### Make sure that beamstop *motors* are in the proper position:
     &message('Synchronizing Act->Rqs for '.$obj);
     $status = Pezca::PutDouble($bsAssy_PV.':Sync.PROC',1);
     &ezca_warn($script,$status,$bsAssy_PV.':Sync.PROC','. Cannot sync Act->Rqs');
### Since the beamstop vertical motor is not in place, then move it there:
     &close_disable_shutter();
     $bs_act = &getAxisAct ($bsAxes_PV[0]);
     $msg = sprintf 'Moving '.$obj.' from %.3f to %.3f',$bs_act,$bs_in_pos;
     &message($msg);
     &checkIfAssyMovable ($bsAssy_PV,$obj);
     for ($attempt=1; $attempt<=$attemptsMax; $attempt++) {
        $status = &runMotor ($bsAssy_PV, $bsAxes_PV[0], $bs_in_pos, 'checkAuto');
        if ($status) {
           &message($obj.' move failed (err='.$status.'), attempt '.$attempt.'/'.$attemptsMax.'.','warn');
           next;
  	}
        last;						#operation successful; we can exit the loop.
     }
     if ($status) {
        &cleanup_zenity_exit('All attempts to move '.$obj.' motors to the "IN" position failed');
     }
# } else {
#    $msg = sprintf 'Did not have to move '.$obj.' IN to %.3f (device in place)',$bs_in_pos;
  }
  $state{$obj} =~ s/\/notinplace//i;			#now motor is in place or it does not matter.

### If beamstop flip not where we need it:
  if ($state{$obj} ne $dest) {
     &close_disable_shutter();
     $attempt = 0;
     do {
        $attempt++;
### Start flipping beamstop IN or OUT
        &message('Flipping '.$obj.' '.uc($dest).', attempt '.$attempt.'/'.$attemptsMax);
        $status = Pezca::PutDouble($beamstopflip_cmd_PV,$beamstopflip_cmd_pos);			#digital I/O
        &ezca_err($script,$status,$beamstopflip_cmd_PV,'. Flipping '.$obj.' '.uc($dest).' failed');
### Check if the command was accepted (SDIS?)
        select(undef,undef,undef,.25);							#normally takes 0.15 between cmd &rbk
        ($status, $beamstopflip_rbk_pos) = Pezca::GetDouble($beamstopflip_rbk_PV);
        if ($status) {
           &ezca_warn($script,$status,$beamstopflip_rbk_PV,'. Taking the commanded position');
           $beamstopflip_rbk_pos = $beamstopflip_cmd_pos;
        }
        elsif ($beamstopflip_cmd_pos != $beamstopflip_rbk_pos) {
           &message('Operation "'.$obj.' flip '.uc($dest).'" failed:'
                   .' readback did not confirm the command','warn');
           ($status, $beamstopflip_sdis) = Pezca::GetDouble($beamstopflip_sdis_PV);
           if ($status) {
              &ezca_warn($script,$status,$beamstopflip_sdis_PV,'. Presuming SDIS=0');
              $beamstopflip_sdis = 0;
           }
           if ($beamstopflip_sdis) {
              print STDOUT '*** '.$script.': '.$obj.' flip disabled by these conditions (SDIS):'."\n";
              foreach $pv (@pvlist) {
                 ($status, $val) = Pezca::GetDouble($pv);
                 if ($status) {&report_only($script,$status,$pv); $val='undef';}
                 print STDOUT '!!! '.$script.': '.$pv.' = '.$val."\n";
              }
              select(undef,undef,undef,0.5);
           } else {
              &message('Clearing previous attempt to command '.$obj
                      .' flip while it was disabled (SDIS)');
              $status = Pezca::PutDouble($beamstopflip_cmd_PV,$beamstopflip_rbk_pos);
              &report_only($script,$status,$beamstopflip_cmd_PV);
              select(undef,undef,undef,.25);
           }
        }
     } while ($attempt < $attemptsMax && $beamstopflip_cmd_pos != $beamstopflip_rbk_pos);
### Wait for beamstop flip to finish:
     if ($attempt >= $attemptsMax) {$status = 1;}
     else {
        $status = &wait4pv($beamstopflip_act_PV,$beamstopflip_cmd_pos,
                           $flip_timeout, $flip_poll, $flip_polldelay,
                           $flip_counter, $flip_tolerance);
     }
     if ($status) {
### Bring the request and the actual in sync (safety mat issue):
        ($status, $beamstopflip_act_pos) = Pezca::GetDouble($beamstopflip_act_PV);
        if (!$status) {Pezca::PutDouble($beamstopflip_cmd_PV,$beamstopflip_act_pos);}		#digital I/O
        &cleanup_zenity_exit('Timeout on flipping '.$obj.' '.uc($dest));
     } else {
        select(undef,undef,undef,1.5);     						#sleep a bit extra;
        &message($obj.' has been flipped '.uc($dest));
        $state{$obj} = $dest;
     }
  }
}

#=================================================================

# sub movePindiode($obj,$dest) {
sub movePindiode ($$) {
  my ($obj, $dest, $pin_dest_pos, $pin_act_pos);
  my ($motionTime, $attempt, $msg);

  $obj  = shift(@_);
  $dest = shift(@_);

  if    ($dest eq 'in')  {$pin_dest_pos = $pin_in_pos;}
  elsif ($dest eq 'out') {$pin_dest_pos = $pin_out_pos;}
  else {
     &cleanup_zenity_exit('Unknown destination "'.$dest.'" for '.$obj.' while preparing to move.');
  }

### If pin diode is not where we want it to be:
  if ($state{$obj} ne $dest) {
     &close_disable_shutter();
     $pin_act_pos = &getAxisAct ($pinAxis_PV);
     $msg = sprintf 'Moving '.$obj.' '.uc($dest).' from %.3f to %.3f',$pin_act_pos,$pin_dest_pos;
     &message($msg);
     &checkIfAssyMovable($pinAssy_PV,$obj);
### Start moving pin diode:
     for ($attempt=1; $attempt<=$attemptsMax; $attempt++) {
### Setup motion:
        $status = &configAssy ($pinAssy_PV);
        if ($status) {
           &message($obj.' motor setup err='.$status.', attempt '.$attempt.'/'.$attemptsMax,'warn');
           next;
        }
        $motionTime = &calcMotorTime($pinAssy_PV, $pinAxis_PV, $pin_dest_pos);
#       $status = &runMotor ($pinAssy_PV, $pinAxis_PV, $pin_dest_pos, 'checkAuto');
        $status = &startMotor ($pinAxis_PV, $pin_dest_pos, $pinAssy_PV, 'checkAuto');
        if ($status) {
           &message($obj.' motor start err='.$status.', attempt '.$attempt.'/'.$attemptsMax,'warn');
           next;
        }
### Track motion:
        $status = &trackMotor ($pinAssy_PV, $motionTime, $pinAxis_PV);
        if ($status) {
           &messsage($obj.' motion track err='.$status.', attempt '.$attempt.'/'.$attemptsMax,'warn');
           next;
        }
        last;						#operation successful; we can exit the loop.
     }
     if ($status) {
        &cleanup_zenity_exit('All attempts to move '.$obj.' '.uc($dest).' failed');
     } else {
        &message($obj.' has been moved '.uc($dest));
        $state{$obj} = $dest;
     }
  } else {
     $msg = sprintf 'Did not have to move '.$obj.' '.uc($dest)
                   .' to %.3f (device already in place)', $pin_dest_pos;
     &message($msg)
  }
}

#=================================================================

# sub moveSONICC($obj,$dest) {
sub moveSONICC ($$) {
  my ($obj, $dest, $sonicc_dest_pos, $sonicc_act_pos);
  my ($motionTime, $attempt, $msg);

  $obj  = shift(@_);
  $dest = shift(@_);

  if    ($dest eq 'in')  {$sonicc_dest_pos = $sonicc_in_pos;}
  elsif ($dest eq 'out') {$sonicc_dest_pos = $sonicc_out_pos;}
  else {
     &cleanup_zenity_exit('Unknown destination "'.$dest.'" for '.$obj.' while preparing to move.');
  }

### If SONICC is not where we want it to be:
  if ($state{$obj} ne $dest) {
     &close_disable_shutter();
     $sonicc_act_pos = &getAxisAct ($soniccAxis_PV);
     $msg = sprintf 'Moving '.$obj.' '.uc($dest).' from %.3f to %.3f',$sonicc_act_pos,$sonicc_dest_pos;
     &message($msg);
     &checkIfAssyMovable($soniccAssy_PV,$obj);
### Start moving SONICC:
     for ($attempt=1; $attempt<=$attemptsMax; $attempt++) {
### Setup motion:
        $status = &configAssy ($soniccAssy_PV);
        if ($status) {
           &message($obj.' motor setup err='.$status.', attempt '.$attempt.'/'.$attemptsMax,'warn');
           next;
        }
        $motionTime = &calcMotorTime($soniccAssy_PV, $soniccAxis_PV, $sonicc_dest_pos);
#       $status = &runMotor ($soniccAssy_PV, $soniccAxis_PV, $sonicc_dest_pos, 'checkAuto');
        $status = &startMotor ($soniccAxis_PV, $sonicc_dest_pos, $soniccAssy_PV, 'checkAuto');
        if ($status) {
           &message($obj.' motor start err='.$status.', attempt '.$attempt.'/'.$attemptsMax,'warn');
           next;
        }
### Track motion:
        $status = &trackMotor ($soniccAssy_PV, $motionTime, $soniccAxis_PV);
        if ($status) {
           &message($obj.' motion track err='.$status.', attempt '.$attempt.'/'.$attemptsMax,'warn');
           next;
        }
        last;						#operation successful; we can exit the loop.
     }
     if ($status) {
        &cleanup_zenity_exit('All attempts to move '.$obj.' '.uc($dest).' failed');
     } else {
        &message($obj.' has been moved '.uc($dest));
        $state{$obj} = $dest;
     }
  } else {
     $msg = sprintf 'Did not have to move '.$obj.' '.uc($dest)
                   .' to %.3f (device already in place)', $sonicc_dest_pos;
     &message($msg);
  }
}

#=================================================================

# sub startCollimator($obj,$dest) {
sub startCollimator ($$) {
  my ($obj, $dest, @col_dest_pos, @col_act, $attempt, $motionTime, $msg, $i);
  $obj  = shift(@_);
  $dest = shift(@_);

  if    ($dest eq 'in')  {@col_dest_pos = @col_in_pos;}
  elsif ($dest eq 'out') {@col_dest_pos = @col_out_pos;}
  else {
     &cleanup_zenity_exit('Unknown destination "'.$dest.'" for '.$obj.' while preparing to move.');
  }
### Is motion is needed?
  if ($state{$obj} ne $dest) {
### Get actual pos:
     for ($i=0; $i<2; $i++) {$col_act[$i] = &getAxisAct ($colAxes_PV[$i]);}
     print  STDOUT $script.': Moving '.$obj.'-'.uc($dest).":\n";
     printf STDOUT '   --> from V=%.3f to V=%.3f'."\n", $col_act[1], $col_dest_pos[1];
     printf STDOUT '   --> from H=%.3f to H=%.3f'."\n", $col_act[0], $col_dest_pos[0];

###==========Start collimator motion: Loop over attempts ===================
     &checkIfAssyMovable($colAssy_PV,$obj);
     $motionTime = &waitMotor ($colAssy_PV,5,0);			#give collimator 5s to finish prev.motion
     for ($attempt=1; $attempt<=$attemptsMax; $attempt++) {
### Setup motion:
        $status = &configAssy ($colAssy_PV);
        if ($status) {
           &message($obj.' motor setup err='.$status.', attempt '.$attempt.'/'.$attemptsMax,'warn');
           next;
        }
### Start motion:
        $status = &startNMotors ($colAssy_PV, 2, \@colAxes_PV, \@col_dest_pos, 'checkAuto');
        if ($status) {
           &messsage($obj.' motion start err='.$status.', attempt '.$attempt.'/'.$attemptsMax,'warn');
           next;
        }
        $motionTime = &calcNMotorsTime($colAssy_PV, 2, \@colAxes_PV, \@col_dest_pos);
        last;							#operation successful; we can exit the loop.
     }
###============== End of Loop over attempts to start collimator ============

     if ($status) {
        &cleanup_zenity_exit('All attempts to start '.$obj.'-'.uc($dest).' failed');
     } else {
        &message($obj.' started motion to '.uc($dest).' OK');
        $state{$obj} = 'unknown';
        return 1,$motionTime;
     }
  }
  else  {
     $msg = sprintf 'Did not move '.$obj.' '.uc($dest).' to V=%.3f  H=%.3f'
                   .' as it had been in place', $col_dest_pos[1], $col_dest_pos[0];
     &message($msg);
     return 0,0;
  }
}

#=================================================================

# sub finishCollimator($obj,$dest) {
sub finishCollimator ($$) {
  my ($obj, $dest, $motionTime, $attempt);
  $obj  = shift(@_);
  $dest = shift(@_);

  $col_motionTime -= time() - $col_start_time;
  if ($col_motionTime < 0) {$col_motionTime = 0;}
  $status = &trackNMotors($colAssy_PV, $col_motionTime, 2, \@colAxes_PV);
  if ($status) {
     &message('Initial attempts to move '.$obj.' err='.$status.'. Making '.$attemptsMax.' more attempts','warn');
  } else {
     ### Motion finished OK. Check new state one more time:
     &getState($obj);
     if ($state{$obj} eq $dest) {
        &message($obj.' has been moved '.uc($dest));
        return;
     } else {
        &message($obj.' has not moved '.uc($dest).', trying more');
     }
  }

###==========Start collimator motion: Loop over attempts ====================
  for ($attempt=1; $attempt<=$attemptsMax; $attempt++) {
### Setup motion:
     $status = &configAssy ($colAssy_PV);
     if ($status) {
        &message($obj.' motor setup err='.$status.', attempt '.$attempt.'/'.$attemptsMax,'warn');
        next;
     }
     $motionTime = &calcNMotorsTime($colAssy_PV, 2, \@colAxes_PV, \@col_dest_pos);
### Start motion:
     $status = &startNMotors ($colAssy_PV, 2, \@colAxes_PV, \@col_dest_pos, 'checkAuto');
     if ($status) {
        &message($obj.' motor start err='.$status.', attempt '.$attempt.'/'.$attemptsMax,'warn');
        next;
     }
### Track motion:
     $status = &trackNMotors($colAssy_PV, $motionTime, 2, \@colAxes_PV);
     if ($status) {
        &message($obj.' motor track err='.$status.', attempt '.$attempt.'/'.$attemptsMax,'warn');
        next;
     }
     last;						#operation successful; we can exit the loop.
  }
###============== End of Loop over attempts to move collimator =============

  if ($status) {
     &cleanup_zenity_exit('All attempts to move '.$obj.' '.uc($dest).' failed');
  }
  else {
     &message($obj.' has been moved '.uc($dest));
     $state{$obj} = $dest;
  }
### End of Collimator motion with pre-checks #################
}

#=================================================================

# sub sortoutGuardSlits() { 				#see if we need to open/close guard slits
sub sortoutGuardSlits() {

  my (@gsMinima_um, @pinholes, $nval, $gsSizeV, $gsSizeH, $size, $string, $msg, $i);

  @pinholes = (20, 10, 5);				#in um

### 0=none, 1=300um, 2=20um, 3=10um, 4=5um
  if ($col_active_preset >= 2) { #------------------------------#when pin is IN

     ($status,$string) = Pezca::GetString($gsMinima_PV);
     if ($status) {
        &ezca_warn($script, $status, $gsMinima_PV,'. Ignoring the Guard Slits');
        return;
     }
     elsif (length($string) < 5) {
        &message('"'.$gsMinima_PV.'" contains incorrect record="'.$string.'". Skipping GS','warn');
        return;
     }
     $string  =~ s/,/ /g;
     @gsMinima_um = split ' ', $string;
     $nval = @gsMinima_um;
     if ($nval != 3 && $nval != 1) {
        &message('"'.$gsMinima_PV.'" contains incorrect record="'.$string.'". Skipping GS','warn');
        return;
     }
     if ($nval == 1) {
        $gsMinima_um[1] = $gsMinima_um[0];
        $gsMinima_um[2] = $gsMinima_um[0];
        $nval = 3;
     }
     for ($i=0; $i<$nval; $i++) {
        $gsMinima_um[$i] =~ s/um//;
        if ($gsMinima_um[$i] !~ /^[0-9]*[.]?[0-9]*$/) {
           &message('"'.$gsMinima_PV.'" contains incorrect record="'.$string.'". Skipping GS','warn');
           return;
        }
        elsif ($gsMinima_um[$i] < $pinholes[$i]) {
           &message('Too small values "'.$string.'" in "'.$gsMinima_PV.'". Skipping GS','warn');
           return;
        }

     }

     $size = 0.001*($gsMinima_um[$col_active_preset-2] + 1.0);	#0=none, 1=300um, 2=20um, 3=10um, 4=5um

     $gsSizeV = &getAxisAct ($gsSizeV_PV);
     if ($gsSizeV < $size) {
        &checkIfAssyMovable($gsAssyV_PV,'GS-vertical size');
        ### Open guard slits-vertical:
        $msg = sprintf 'Opening GS-vertical size to %.3fum', 1000*$size;
        &message($msg);
        $status = &startMotor($gsSizeV_PV, $size, $gsAssyV_PV, 'checkauto');
        if ($status) {
           $msg = sprintf 'Failed to open GS-vertical size "'.$gsSizeV_PV.'" to %fum', 1000*$size;
           &message($msg,'warn');
        }
     }

     $gsSizeH = &getAxisAct ($gsSizeH_PV);
     if ($gsSizeH < $size) {
        &checkIfAssyMovable($gsAssyH_PV,'GS-horizontal size');
        ### Open guard slits-horizontal:
        $msg = sprintf 'Opening GS-horizontal size to %.3fum', 1000*$size;
        &message($msg);
        $status = &startMotor($gsSizeH_PV, $size, $gsAssyH_PV, 'checkauto');
        if ($status) {
           $msg = sprintf 'Failed to open GS-horizontal size "'.$gsSizeH_PV.'" to %fum', 1000*$size;
           &message($msg,'warn');
        }
     }

  } elsif ($col_active_preset == 1) { #-------------------------#when scatterguard is IN

### Do nothing -- do not close Guard Slits back

  }                              #------------------------------#end of pin or scatterguard
}

#=================================================================

# sub controlRinglightBrightness() {
sub controlRinglightBrightness () {
  my ($msg);

### Ringlight brightness: determine destination or save current intensity:
  if (!defined $ringbright_val) {				#val not passed via cmd line
### Read destination from previously saved:
     ($status,$ringbright_val) = Pezca::GetDouble($ringbright_save_PV);
     if ($status) {
        &ezca_warn($script,$status,$ringbright_save_PV,'. Presuming value=0');
        $ringbright_val = 0;
     }
  }
  elsif ($ringbright_val == 0) {				#request to dim ringbright
### Read current intensity:
     ($status,$ringbright_val) = Pezca::GetDouble($ringbright_ctrl_PV);
     if ($status) {
        &ezca_warn($script,$status,$ringbright_ctrl_PV,'. Operation skipped');
        $ringbright_ctrl = 0;
     } else {
### Save current intensity for future restoration:
        $status = Pezca::PutDouble($ringbright_save_PV,$ringbright_val);
        &ezca_warn($script,$status,$ringbright_save_PV,'. Ignoring');
     }
     $ringbright_val = 0;
  }

  if ($ringbright_ctrl) {
### Control ringbright:
     $status = Pezca::PutDouble($ringbright_ctrl_PV,$ringbright_val);
     if ($status) {
        &ezca_warn($script,$status,$ringbright_ctrl_PV,'. Failed to set ringlight brightness');
     } else {
        $msg = sprintf 'Ringlight brightness set to %.3f',$ringbright_val;
        &message($msg);
     }
  }
}

#=================================================================

# sub controlBacklightBrightness() {
sub controlBacklightBrightness () {
  my ($msg);

### Backlight brightness: determine destination or save current intensity:
  if (!defined $backbright_val) {				#val not passed via cmd line
### Read destination from previously saved:
     ($status,$backbright_val) = Pezca::GetDouble($backbright_save_PV);
     if ($status) {
        &ezca_warn($script,$status,$backbright_save_PV,'. Presuming saved backlight brightness=0');
        $backbright_val = 0;
     }
  }
  elsif ($backbright_val == 0) {				#request to dim backbright
### Read current intensity:
     ($status,$backbright_val) = Pezca::GetDouble($backbright_ctrl_PV);
     if ($status) {
        &ezca_warn($script,$status,$backbright_ctrl_PV,'. Operation skipped');
        $backbright_ctrl = 0;
     } else {
### Save current intensity for future restoration:
        $status = Pezca::PutDouble($backbright_save_PV,$backbright_val);
        &ezca_warn($script,$status,$backbright_save_PV,'. Current backlight brightness was not saved');
     }
     $backbright_val = 0;
  }

  if ($backbright_ctrl) {
### Control backbright:
     $status = Pezca::PutDouble($backbright_ctrl_PV,$backbright_val);
     if ($status) {
        &ezca_warn($script,$status,$backbright_ctrl_PV,'. Setting backlight brightness failed');
     } else {
        $msg = sprintf 'Backlight brightness set to %.3f',$backbright_val;
        &message($msg);
     }
  }
}

#=================================================================

# sub close_disable_shutter () {
sub close_disable_shutter () {
  if (&getShutter($shutter_readback)) {			#if shutter is open
     &closeShutter($shutter);
     select(undef,undef,undef,0.1);     		#sleep a bit;
  }
  &setAssyManual ($shutter_assy);                       #disable shutter
  if (&getShutter($shutter_readback)) {			#if shutter is open
     &cleanup_zenity_exit('Could not close shutter');
  }
}

#=================================================================

# sub ezca_err ($script, $status, $PV, [$msg]) {
sub ezca_err ($$$;$) {
  my ($script, $status, $PV, $statusDescr, $msg);
  $script = shift(@_);
  $status = shift(@_);
  $PV     = shift(@_);
  $msg    = shift(@_);
  if (!defined $msg) {$msg = '';}
  if ($msg !~ /\.$/) {$msg .= '.';}
  if ($status) {
     $statusDescr  = &ezcaStatusCode($status);
     &cleanup_zenity_exit('EZCA error='.$status.' ['.$statusDescr.'] on PV=['.$PV.']'.$msg);
  }
}

#=================================================================

# sub ezca_warn ($script, $status, $PV, [$msg]) {
sub ezca_warn ($$$;$) {
  my ($script, $status, $PV, $statusDescr, $msg);
  $script = shift(@_);
  $status = shift(@_);
  $PV     = shift(@_);
  $msg    = shift(@_);
  if (!defined $msg) {$msg = '';}
  if ($msg !~ /\.$/) {$msg .= '.';}
  if ($status) {
     $statusDescr = &ezcaStatusCode($status);
     &message('EZCA error='.$status.' ['.$statusDescr.'] on PV=['.$PV.']'.$msg, 'warn');
  }
}

#=================================================================

# sub message ($msg,[$type]) {
sub message ($;$) {
  my ($msg, $type, $prefix);
  $msg  = shift(@_);
  $type = shift(@_);
  if (!defined $type) 	      {$type = 'log_note';}
  if    ($type =~ /^error$/i) {$prefix = '*** ';}
  elsif ($type =~ /^warn/i)   {$prefix = '!!! ';}
  else                        {$prefix = '';}
  if ($msg !~ /\.$/) {$msg .= '.';}
  print STDOUT $prefix.$script.': '.$msg."\n";
  if ($tell_bluice) {&tellBluIce($scr.': '.$msg,$type);}
}

#=================================================================

# sub cleanup_zenity_exit($msg) {
sub cleanup_zenity_exit($) {
  my $msg = shift(@_);
  &setAssyAuto ($shutter_assy);
  &closeShutter($shutter);
  if ($tell_bluice) {&tellBluIce($scr.': '.$msg,'error');}
  #$ZENITY_VIA_BLUICE = 0;
  &zenity_error_exit($msg);
}

#=================================================================

sub signal_handler {
  our ($ABORT_IN_PROCESS);
  my $signame = shift(@_);
  print STDOUT "\n".$script.': *** Caught a SIG'.$signame."\n";

  if (defined $ABORT_IN_PROCESS) {return;}
  $ABORT_IN_PROCESS = 1;
  &abort_external_script();

  &setAssyAuto ($shutter_assy);
  &closeShutter($shutter);
  &clearAllCaMonitors();
  print STDOUT 'Done!'."\n\n";
  exit 1;
}
