#!/usr/bin/perl -W
# _CALLED_FROM_BLUICE_
#--------------------------------------------------------------
# This script is called by JBluIce Assistant and passes commands to
# Sample Environment Server. The purpose of this script is to be called
# from MEDM because MEDM does not support message buttons on waveform
# strings (like those used by Sample Environment Server).
# ATTENTION: the commands are not checked for syntax
#
#     	            	Author: Sergey Stepanov
#
# Version-1.0: 2014/02/07 using Pezca
# Version-1.1: 2019/11/07 added support for the getStates command
#--------------------------------------------------------------
  use strict;
  use Pezca;

  our ($beamline);
  our ($IGNORE_ERRORS, $NONINTERACTIVE, $USE_ZENITY_FOR_EXIT, $ZENITY_VIA_BLUICE);
  my ($PEZCA, $script, $status, @beamlinelist, $string, $sampleEnvPV);
  my (%states, $key, $i, $wantStates);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDOUT; $|=1;					#set unbuffered output
  select STDERR; $|=1;					#set unbuffered output

#-----------------------------------------------------------------
  $IGNORE_ERRORS       = 1;				#don't exit on error
  $NONINTERACTIVE      = 1;				#don't wait for any answers
  $USE_ZENITY_FOR_EXIT = 1;

  $script = &makeScriptName();

  &read_commandline();						#read command line parameters
  $sampleEnvPV = $beamline.':bi:sampleEnv';

  if (grep(/^getStates$/,@ARGV)) {
     $wantStates = 1;
     for $i (reverse 0..$#ARGV) {
        if ($ARGV[$i] =~ /^getStates$/) {splice(@ARGV, $i, 1, ());}
    }
  }          
  if (@ARGV > 0) {
     $string = join(' ',@ARGV);
     print STDOUT $script.': Sending "'.$string.'" to "'.$sampleEnvPV.'"'."\n";
     $status = &PezcaPutStringWF($sampleEnvPV,$string);
     &report_only($script, $status, $sampleEnvPV);
  }
  if (defined $wantStates) {
     print STDOUT $script.': Sending "getStates" to "'.$sampleEnvPV.'"'."\n";
     $status = &getSampleEnv(\%states);
     if (! $status) {
        foreach $key (keys %states) {print STDOUT $key.'='.$states{$key}."\n";}
     } else {
        print STDOUT 'getSampleEnv returned status='.$status."\n";
     }
  }
  &clearAllCaMonitors();
  exit;

#=================================================================

sub read_commandline () {

  my ($i, $arg, $narg, $syntax);
  $syntax = "\n\n".'Syntax: '.$script.' [beamline] @sampleEnvServerCmds'."\n";

  $narg = @ARGV;
  print STDOUT "\n".$script.': starting with cmdline: ';
  if ($narg > 0) {print STDOUT "@{ARGV}\n\n";}
  else           {print STDOUT '<no arguments>'."\n\n";}

### Read command line arguments:
  @beamlinelist = ('23i:', '23o:', '23b:', '23d:');
  for ($i=0; $i<$narg; $i++) {
     $arg = $ARGV[$i];
     $arg =~ s/^-{0,2}//;					#trim leading dashes
### Help requested:
     if ($arg =~ /^help$/i || $arg =~ /^\?$/) {
        &press_key_exit($syntax);				#this will exit automatically if $NONINTERACTIVE=1
     }
### Check if the default beamline name is overridden:
     elsif (grep(/^$arg:?$/, @beamlinelist)) {
        $arg =~ s/:$//;						#trim ':'
	if ($beamline ne $arg) {
           print STDOUT "\n".'!!! '.$script.': Cross-beamline operation: controlling '.$arg.' from '.$beamline."\n\n";
           $beamline = $arg;
           do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
        }
	splice(@ARGV,$i,1); $i--; $narg--;
     }
  }
  if ($narg == 0) {&press_key_exit($script.': No commands specified'.$syntax);}
  return;
}

#=================================================================

