#!/usr/bin/perl -W
#--------------------------------------------------------------
#    This script saves maximum speeds of Gonio and crystal XYZ to mySQL
#
#       		Author: Sergey Stepanov
#
# Version-1.0:  2014/05/12 using Pezca
#--------------------------------------------------------------
  use strict;
  use DBI;						#for recording speeds in mySQL
  use Pezca;

  our ($beamline, $mysqlHost, $mysqlUser, $mysqlPass);
  our ($NONINTERACTIVE, $USE_ZENITY_FOR_EXIT);
  my ($PEZCA, $script, $cmdline, $syntax, $narg, $i, $arg, $status);
  my (@beamlinelist, %pars, $name, $speed);
  my ($mysqlDb, $mysqlTbl, $dsn, $dbh, $sth, $nrecords, $hashref);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

# $beamline = '23i';                    # uncomment for debugging

  do $PEZCA.'/Common/motor_subs.pl';
  do $PEZCA.'/Common/home_subs.pl';
  do $PEZCA.'/Common/common_subs.pl';
  if (defined $beamline) {do $PEZCA.'/Common/beamline_'.$beamline.'.pl';}
  else                   {do $PEZCA.'/Common/beamline.pl';}

  select STDERR; $|=1;			# set unbuffered output
  select STDOUT; $|=1;			# set unbuffered output

  $NONINTERACTIVE = 0;	 		# to exit from script with no prompting
  $USE_ZENITY_FOR_EXIT = 1;
  @beamlinelist   = ('23i', '23o', '23b', '23d');

  $script = &makeScriptName();
  $script =~ s/\.pl$//i;
  $cmdline =  ' [beamline]';
  $syntax = "\n\n".'Syntax: '.$script.'.pl '.$cmdline;

  $narg = @ARGV;

  if ($narg > 0) {
     for ($i=0; $i<$narg; $i++) {
        $arg = lc $ARGV[$i];
        if (grep(/$arg/, @beamlinelist)) {
           if ($beamline ne $arg) {
              $beamline = $arg;
              print STDOUT "\n".'!!! '.$script.': Cross-operation for beamline '.$beamline."\n\n";
              do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
           }
        } else {
           &press_key_exit('*** '.$script.': Incorrect cmdline argument "'.$ARGV[$i].'"'.$syntax);
        }
     }
  }
  $pars{omega}{name}  = 'Gonio omega';
  $pars{omega}{drive} = $beamline.':GO:OM';
  $pars{omega}{motor} = $beamline.':GO:mom';
# $pars{omega}{assy}  = $beamline.':GO:Om';
  $pars{omega}{sql}   = 'speed_limit_gonio_omega';

  $pars{crysx}{name}  = 'Crystal X';
  $pars{crysx}{drive} = $beamline.':GO:CX';
  $pars{crysx}{motor} = $beamline.':GO:mcx';
# $pars{crysx}{assy}  = $beamline.':GO:Cr';
  $pars{crysx}{sql}   = 'speed_limit_sample_x';

  $pars{crysy}{name}  = 'Crystal Y';
  $pars{crysy}{drive} = $beamline.':GO:CY';
  $pars{crysy}{motor} = $beamline.':GO:mcy';
# $pars{crysy}{assy}  = $beamline.':GO:Cr';
  $pars{crysy}{sql}   = 'speed_limit_sample_y';

  $pars{crysz}{name}  = 'Crystal Z';
  $pars{crysz}{drive} = $beamline.':GO:CZ';
  $pars{crysz}{motor} = $beamline.':GO:mcz';
# $pars{crysz}{assy}  = $beamline.':GO:Cr';
  $pars{crysz}{sql}   = 'speed_limit_sample_z';

  $mysqlDb  = 'blc2004';
  $mysqlTbl = 'Config';
  $dsn = 'DBI:mysql:host='.$mysqlHost.';database='.$mysqlDb;
  print STDOUT $script.': logging Gonio & Crystal-XYZ maximum speeds to mySQL'."\n";
  print STDOUT $script.': connecting to mySQL database='.$mysqlDb.' on host='.$mysqlHost."\n\n";
  $dbh = DBI->connect($dsn,$mysqlUser,$mysqlPass,{AutoCommit=>1,PrintError=>0,});
  if (!defined $dbh) {
     &press_key_exit('*** '.$script.': cannot connect to mySQL DB='.$mysqlDb.' on '.$mysqlHost);
  }

  $status = 0;
  foreach $name (keys %pars) {  #----------------------------------------------------
     print STDOUT $script.': Name="'.$pars{$name}{name}.'"'
               .'  Drive='.$pars{$name}{drive}
               .'  Motor='.$pars{$name}{motor}
               ."\n";
     $pars{$name}{jogSpeed} = &getMtrJogSpeed($pars{$name}{motor},\$status);
     if ($status) {next;}
     ($status, $pars{$name}{scale}) = Pezca::GetDouble($pars{$name}{drive}.':Scale');
     if ($status) {
        &report_only($script, $status, $pars{$name}{drive}.':Scale');
        next;
     }
     $pars{$name}{speed} = abs( 1000. * $pars{$name}{scale} * $pars{$name}{jogSpeed} );
     printf STDOUT $script.': Scale=%.6e mtrSpeed=%.4g drvSpeed=%.4g'."\n",
                         $pars{$name}{scale}, $pars{$name}{jogSpeed}, $pars{$name}{speed};
     $pars{$name}{speed} = sprintf '%.4f', $pars{$name}{speed};
     $sth = $dbh->prepare('SELECT * FROM '.$mysqlTbl.' WHERE ID="'.$pars{$name}{sql}.'"');
     $nrecords = $sth->execute();				#nrecords works for MySQL only. For SQLite result is def/undef only.
     if (!defined $nrecords) {
        print STDOUT '*** '.$script.': could not execute SELECT on mySQL table="'.$mysqlTbl
                    .'" for ID="'.$pars{$name}{sql}.'" -- '.$sth->errstr."\n";
        next;
     }
     elsif ($nrecords > 1) {
        print STDOUT '*** '.$script.': multiple ('.$nrecords.') records in mySQL table="'
                    .$mysqlTbl.'" for ID="'.$pars{$name}{sql}.'"'."\n";
        next;
     }
     elsif ($nrecords == 0) {
        print STDOUT '!!! '.$script.': no records in mySQL table="'.$mysqlTbl
                    .'" for ID="'.$pars{$name}{sql}.'".  Inserting.'."\n";
        $nrecords = $dbh->do('INSERT INTO '.$mysqlTbl
             .' (ID,Value,TypeInfo) VALUES ("'.$pars{$name}{sql}.'", "'.$pars{$name}{speed}.'","f4")');
        if (!defined $nrecords) {
           print STDOUT '*** '.$script.': failed to insert record for ID="'
             .$pars{$name}{sql}.'" in mySQL table="'.$mysqlTbl.'"'."\n";
           next;
        }
     }
     elsif ($nrecords == 1) {
        $hashref = $sth->fetchrow_hashref();
        $speed = $hashref->{'Value'};
        if (abs(($pars{$name}{speed}-$speed)/$pars{$name}{speed}) > 0.01) {
           printf STDOUT '!!! '.$script.': value for ID="'.$pars{$name}{sql}
                    .'" in mySQL table="'.$mysqlTbl.'" changed from %.4g to %.4g'
                    .'.  Updating.'."\n", $speed, $pars{$name}{speed};
           $nrecords = $dbh->do('UPDATE '.$mysqlTbl.' SET Value="'.$pars{$name}{speed}
                    .'" WHERE ID="'.$pars{$name}{sql}.'"');
           if (!defined $nrecords) {
              print STDOUT '*** '.$script.': failed to update record for ID="'
                .$pars{$name}{sql}.'" in mySQL table="'.$mysqlTbl.'"'."\n";
              next;
           }
        } else {
           print STDOUT $script.': Value in mySQL table="'.$mysqlTbl
                    .'" for ID="'.$pars{$name}{sql}.'" is up to date.'."\n";
        }
     }
     print STDOUT "\n";
  } #------------------------------------------------------------------------------
  print STDOUT $script.': disconnecting from mySQL table="'.$mysqlTbl.'"'."\n";
  $sth->finish();
  $dbh->disconnect();
  exit;
