#!/usr/bin/perl -W
#--------------------------------------------------------------
#
#          This script saves Transfocator state using 
#         saveTFstate then verifies it with readTFstate
#
#                     Author: Sergey Stepanov
#
# Version-1.0:  2020/11/17	initial implementation
# Version-1.1:  2021/09/15	eliminated center
#--------------------------------------------------------------
  use Pezca;
  use DBI;
  use strict;
  use warnings;

  select STDERR; $|=1;			# set unbuffered output
  select STDOUT; $|=1;			# set unbuffered output

  our $beamline;
  my ($PEZCA, $script, $D, $X, $H, $V, $E, $F, $err);
  my ($status, $msg, $epoch);

  if (!defined $ENV{PEZCA}) {
     die 'Please define the PEZCA environment. It must point to the PERL scripts repository';
  }
  $PEZCA = $ENV{PEZCA};
  $PEZCA =~ s/\\/\//g;
  if ($PEZCA !~ /\/$/) {$PEZCA .='/';}

  if (-e $PEZCA.'Common/common_subs.pl') {
     do $PEZCA.'Common/common_subs.pl';
     do $PEZCA.'Common/crl_subs.pl';
     if (defined $beamline) {do $PEZCA.'/Common/beamline_'.$beamline.'.pl';}
     else                   {do $PEZCA.'/Common/beamline.pl';}
  } else {die '*** Cannot fined PEZCA subs in '.$PEZCA."\n";}

  $script = &makeScriptName('noext');
  
  if ($beamline ne '23i') {
     $msg =  $script.': this script can only run on beamline=23i (23IDD)';
     &zenity_error_exit($msg);
  }

  print STDOUT $script.': saving Transfocator state for beamline='.$beamline."\n";
  $status = &saveTFstate();
  if ($status) {
     $msg =  $script.': status from saveTFstate is '.$status;
     &zenity_error_exit($msg);
  }

  print STDOUT $script.': reading Transfocator state back'."\n";
# $D=0; $X=0; $H=0; $V=0; $E=0; $F=0;
  $status = &readTFstate(\$D,\$X,\$H,\$V,\$E,\$F);
  print STDOUT $script.': status from readTFstate is '.$status."\n";
  $err = 0;
  $msg = '';
  $msg.='D = '; if (defined $D) {$msg.=sprintf '%s',  $D;} else {$msg.='undef'; $err++;} $msg.="\n";
  $msg.='X = '; if (defined $X) {$msg.=sprintf '%.2f',$X;} else {$msg.='undef'; $err++;} $msg.="\n";
  $msg.='H = '; if (defined $H) {$msg.=sprintf '%.2f',$H;} else {$msg.='undef'; $err++;} $msg.="\n";
  $msg.='V = '; if (defined $V) {$msg.=sprintf '%.2f',$V;} else {$msg.='undef'; $err++;} $msg.="\n";
  $msg.='E = '; if (defined $E) {$msg.=sprintf '%.2f',$E;} else {$msg.='undef'; $err++;} $msg.="\n";
  $msg.='F = '; if (defined $F) {$msg.=sprintf '%.2f',$F;} else {$msg.='undef'; $err++;} $msg.="\n";
  if ($err > 0) {
     $msg = $script.': error reading back Transfocator state:'."\n\n".$msg;
     &zenity_error_exit($msg);
  } else {
     print STDOUT "\n".$msg;
     &clearAllCaMonitors();
     exit 0;
  }
### This is not used (test):
# $epoch = time();				#convert current time to epoch
  $epoch = &timestamp2epoch($D);		#convert 'YYYY-MM-DD HH:MM' to epoch
  print STDOUT 'epoch='.$epoch."\n";
  $msg = scalar localtime($epoch);		#date stamp corresponding to epoch
  print STDOUT $msg,"\n";
  exit;
