#!/usr/bin/perl -W
#--------------------------------------------------------------
# This script converts AllowedUsers from /etc/ssh/sshd_config
# to /mnt/software/http_require.cfg
#
#         	Author: Sergey Stepanov
#
# Version-1.0: 2021/03/05 initial implementation (parsing /etc/ssh/sshd_config)
#--------------------------------------------------------------
  use POSIX;
  use strict;
  use warnings::unused;

  my ($script, $key, $sshcfg, $htdcfg, @buf, $fh, $usr);
  my ($string, @remote, @ip, $ipspec, $i, $nremote);

  select STDOUT; $|=1;					        #set unbuffered output
  select STDERR; $|=1;					        #set unbuffered output

  $script = $0;
  $script =~ s/\\/\//g;					        #change \ to /
  $script =~ s/^.*\///g;				        #remove everything before last /
  $script =~ s/\.pl$//i;

  if (geteuid() ne '0') {
### ----------------------------------------------------------
### /etc/sudoers should have
# ALL ALL = (root) NOPASSWD:  /home/gmca/epics/pezca/sshdconf2requireip.pl
### ----------------------------------------------------------
     if (exec('/usr/bin/sudo','-n',$0,@ARGV)) {die $script.': failed restarting with sudo.'."\n";}
  }

  $sshcfg = '/etc/ssh/sshd_config';
  $htdcfg = '/mnt/software/www_local_require_ip.cfg';
  $key    = 'AllowUsers';

### ========================================================================
### Parse /etc/ssh/sshd_config for AllowUsers:
### ========================================================================
  if (! -e $sshcfg) {die $script.': file "'.$sshcfg.'" does not exist.'."\n";}
  if (! -r $sshcfg) {die $script.': file "'.$sshcfg.'" is not readable.'."\n";}

  if (&read_entire_file($sshcfg,\@buf)) {die $script.': cannot open "'.$sshcfg.'" for reading.'."\n";}
  @ip = ();
  foreach $string (@buf) {
     if ($string !~ /^\s*${key}/) {next;}			#skip if line does not contain 'AllowUsers'
     $string =~ s/^#.*$//g;					#remove comment lines
     $string =~ s/DO_NOT_MODIFY_AFTER_HERE//g;			#remove temp marker
     $string =~ s/\t/ /g;					#replace tabs by spaces
     $string =~ s/\s+/ /g;					#compress spaces ("+" means match 1 or more times)
     $string =~ s/^ *//g;					#remove beginning-of-the-line spaces
     $string =~ s/ *$//g;					#remove end-of-the-line spaces
     if ($string !~ /^${key}/) {next;}				#skip if line does not contain 'AllowUsers'
     $string =~ s/^${key} //;					#remove 'AllowUsers'
     @remote = split(/\s+/,$string);
     @remote = sort @remote;
     $nremote = @remote;
     for ($i=0; $i<$nremote; $i++) {
        ($usr,$ipspec) = split/\@/,$remote[$i];
        push (@ip,$ipspec);
     }
  }
  @ip = sort @ip;
  @ip = &unique(@ip);

  $fh = open(DAT, ' > ', $htdcfg);
  if (!defined $fh ) {die 'Cannot open file='.$htdcfg};
# foreach $ipspec (@ip) {print DAT '  Require ip ',$ipspec,"\n";}
  foreach $ipspec (@ip) {print DAT '  Allow from ',$ipspec,"\n";}
  close(DAT);
  exit;

#==========================================================================

# sub read_entire_file($file,\@lines) {
sub read_entire_file($\@) {
# use Fcntl;					#not used (defines O_RDONLY constant (O_RDONLY=0,O_WRONLY=1,O_RDWR=2))
  my ($file, $linesref, $buffer, $status);

  $file     = shift(@_);
  $linesref = shift(@_);

  @$linesref = ();
  $status = sysopen(FH, $file, 0);		#0 is used instead of O_RDONLY defined in "use Fcntl";
  if (!defined $status) {print STDOUT '*** '.$script.': Could not open '.$file."\n"; return 1;}
  sysread(FH, $buffer, -s FH);
  close FH;
  @$linesref = split(/\r?\n/,$buffer);
  return 0;
}

#==========================================================================

sub unique {
  my %seen;
  grep !$seen{$_}++, @_;
}
