#!/usr/bin/perl -W
#--------------------------------------------------------------
# This script measures background intensity in Struck scaler
# channels 1-32. The intensity is measured with closed shutter
# and recorded into struck_Bkg.dat
#
#     		Author: Sergey Stepanov
#
# Version-1.0:  2001/06/22 using Pezca
# Version-2.0:  2006/01/06 using struck_subs_v2.pl
# Version-2.1:  2014/03/28 changed functions calls to arrays pointers (preparation to Perl-5.16)
#--------------------------------------------------------------
  use strict;
# no strict "vars";
# no strict "subs";
# no strict "refs";
  use Pezca;

  our ($beamline, $shutter, $shutter_readback);
  our ($struckName, $struck_ext, $joergerName);
  our (@excludes);					#may be defined externally in beamline.pl
  my ($PEZCA, $script, $dwell, $num_pts, $outfile, @files);
  my ($shutter_ini, $MBSH_ini, $int_ext, $i, $pt, $act_pts);
  my (@timer, @data, @Intensity, $inp, $dwell_ini);
  my ($str, $handle, $status, $nexcludes);
  my ($scalerNo, $scaler, $struck, $ninputs);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/shutter_subs.pl';
  do $PEZCA.'/Common/motor_subs.pl';
  do $PEZCA.'/Common/struck_subs_v2.pl';
  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDERR; $|=1;					#set unbuffered output
  select STDOUT; $|=1;					#set unbuffered output

  $dwell   = 1;						#Struck count time/channel in s
  $num_pts = 10;
  $scalerNo = 1;					#Struck as Joerger
# if (!defined @excludes) {                             #if not defined in beamline.pl
  if (@excludes) {                                      #if not defined in beamline.pl
     if    ($beamline =~ /^23i/) {@excludes = (25, 26, 29, 30, 31, 32);}
     elsif ($beamline =~ /^23o/) {@excludes = (25, 26, 27, 28, 29, 30, 31, 32);}
     elsif ($beamline =~ /^23b/) {@excludes = (4, 5, 6, 7, 8, 9);}
     else                        {@excludes = ();}
  }
  if (!grep(/^1$/,@excludes)) {push (@excludes,1);}	# timer channel
  @excludes = sort(@excludes);
  $outfile = 'struck_Bkg.dat';
###----------------------------------------------------------------
  $script = &makeScriptName();
  $scaler = $joergerName.$scalerNo;
  $struck = $struckName;
  if ($scaler           !~ /^${beamline}:/) {$scaler           = $beamline.':'.$scaler;}
  if ($struck           !~ /^${beamline}:/) {$struck           = $beamline.':'.$struck;}
  if ($shutter          !~ /^${beamline}:/) {$shutter          = $beamline.':'.$shutter;}
  if ($shutter_readback !~ /^${beamline}:/) {$shutter_readback = $beamline.':'.$shutter_readback;}

### Get number of inputs in Struck:
  $ninputs = &StruckNinputs ($struck);
###----------------------------------------------------------------
### get shutter status:
  $shutter_ini = &getShutter($shutter_readback);
  $MBSH_ini    = &getEndStationShutter($beamline);

  if ($shutter_ini == 1) {					#if shutter is initially open
    print STDOUT $script.': closing shutter '.$shutter."\n";
    &closeShutter($shutter,$shutter_readback);
  }
  if ($MBSH_ini == 1) {
     print STDOUT $script.': closing endstation shutter'."\n";
     $status = &closeEndStationShutter($beamline);
     if ($status) {&press_key_exit('*** '.$script.': failed to close endstation shutter');}
  }

### set Struck:
### Get initial Struck dwell:
  $dwell_ini = &getStruckDwell ($struck);

  print STDOUT $script.': setting Struck '.$struck."\n";
  $int_ext = 0;							#internal channel advance
  &setStruck ($struck, $struck_ext, $int_ext, $dwell, $num_pts, $scaler);

  printf STDOUT $script.': running Struck '.$struck." for %.1fs\n",$dwell*$num_pts;
  &runStruck($struck);
  &waitStruck($struck);

  print STDOUT $script.': reading Struck '.$struck."\n";
  ($act_pts, @timer) = &readStruckTimer ($struck, $struck_ext, $num_pts);
  if ($act_pts < $num_pts) {
     print STDOUT "\n".'!!! '.$script.': requested to read='.$num_pts.'pts, found='.$act_pts.'pts'."\n";
     for ($i=$act_pts; $i<$num_pts; $i++) {$timer[$i] = 0;}
  }

  @Intensity=();
  $Intensity[0] = 0;						#internal clock channel
  for ($i=1; $i<$ninputs; $i++) {
     @data = &readStruckInput($struck, $struck_ext, $num_pts, $i+1, \@timer);
     $Intensity[$i] = 0.;
     for ($pt=0; $pt < $num_pts; $pt++) {$Intensity[$i] += $data[$pt];}
     $Intensity[$i] = $Intensity[$i] / $num_pts;
  }

  print STDOUT $script.': excluding offsets for Struck inputs ('."@{excludes})\n";
  $nexcludes = @excludes;
  for ($i=0; $i<$nexcludes; $i++) {
     $inp = $excludes[$i];
     if ($inp >=1 && $inp <= $ninputs) {$Intensity[$inp-1]=0;}
  }

  &setStruckDwell ($struck, $dwell_ini);			#Restore initial Struck dwell:

  if ($MBSH_ini == 1) {
     print STDOUT $script.': opening endstation shutter'."\n";
     $status = &openEndStationShutter($beamline);
     if ($status) {
        print STDOUT '!!! '.$script.': failed to open endstation shutter'."\n";
     }
  }
  if ($shutter_ini == 1) {					#if shutter is initially open
    print STDOUT $script.': opening shutter '.$shutter."\n";
    &openShutter ($shutter,$shutter_readback);
  }

  @files = ($outfile, 'joerger1_Bkg.dat');			# scaler-1 is same as Struck

  foreach $outfile (@files) {
### Open file for writing:
     print STDOUT $script.': opening file ['.$outfile.']'."\n";
     $handle = open (DAT,'> '.$outfile);				#overwrite DAT file
     if (!defined $handle) {
        &press_key_exit('*** '.$script.':  could not open file ['.$outfile.']');
     }
     print STDOUT $script.': writing data'."\n";
     for ($i=0; $i<$ninputs; $i++) {printf DAT '%d'."\n",int($Intensity[$i]);}
     close(DAT);
  }

  &clearAllCaMonitors();
  print STDOUT $script.': ..Finished!'."\n";
  exit;

