#!/usr/bin/perl -W
#--------------------------------------------------------------
# This script compares sample number in PV and SQL and it the SQL
# value is larger, it set the PV from SQL. The script is supposed
# to be executed on the keithley IOC startup to fix the situations
# when EPICS save/restore keeps not the latest value
#
#
#     		Author: Sergey Stepanov
#
# Version-1:  2020/08/07 using Pezca and DBI
#--------------------------------------------------------------
  use strict;
  use Pezca;
  use DBI;

  our ($beamline);
  my ($PEZCA, $script, $syntax, $pv, $sql_val, $pv_val, $status);
  my (@beamlinelist, $bln_specified, $narg, $arg_ini, $arg);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/common_subs.pl';
  do $PEZCA.'/Common/beamline.pl';

  select STDERR; $|=1;			# set unbuffered output
  select STDOUT; $|=1;			# set unbuffered output

  $script = &makeScriptName();
  @beamlinelist  = qw(23i 23o 23b 23d);
  $syntax = "\n".'Syntax: '.$script.' [beamline]'."\n\n";

  $narg = @ARGV;
# if ($narg > 0) {print STDOUT "\n\n".$script.': started with cmdline = '."@ARGV\n\n";}
# else           {print STDOUT "\n\n".$script.': started with no cmdline arguments.'."\n\n";}

  foreach $arg_ini (@ARGV) {
     $arg = $arg_ini;
     if (grep(/^$arg:?$/, @beamlinelist)) {
        if (defined $bln_specified) {
           &press_key_exit ('*** '.$script.': Duplicate beamline parameter in the cmdline.'.$syntax);
        }
        $bln_specified=1;
        $arg =~ s/:$//;						#trim ':'
        if ($arg ne $beamline) {
           print STDOUT "\n".'!!! '.$script.': Cross-beamline operation: controlling '.$arg.' from '.$beamline."\n\n";
           $beamline = $arg;
           do $PEZCA.'/Common/beamline_'.$beamline.'.pl';
        }
     }
     else {
        &press_key_exit ('*** '.$script.': unrecognized argument '.$arg_ini.' in the cmdline.'.$syntax);
     }
  }

### Get the SQL sampleNumber
  ($status,$sql_val) = &get_sql_sampleNumber();
  if ($status) {exit 1;}

### Get the EPICS sampleNumber
  $pv = $beamline.':bi:scr:sampleNumber';
  ($status, $pv_val) = Pezca::GetDouble($pv);
  &report_exit($script,$status,$pv);

### If EPICS value is less than SQL value, fix EPICS:
  if ($pv_val < $sql_val) {
     print STDOUT '!!! '.$script.': PV('.$pv.')='.$pv_val.' is less than SQL(sampleNumber)='.$sql_val.'. Correcting...'."\n";
     $status = Pezca::PutDoubleOldCa($pv,$sql_val);
     &report_exit($script,$status,$pv);
  } else {
     print STDOUT $script.': PV('.$pv.')='.$pv_val.', SQL(sampleNumber)='.$sql_val.'. This is OK.'."\n";
  }
  &clearAllCaMonitors();
  exit;

#=====================================================================

sub get_sql_sampleNumber() {					#returns ($status,$sql_val)
  our ($mysqlHost, $mysqlUser, $mysqlPass);
  my ($script, $mysqlDb, $mysqlTbl);
  my ($dsn, $dbh, $sth);
  my ($maxSample, $key, $hashref);
  
  $script = 'get_sql_sampleNumber';

  $mysqlDb  = 'gmca_data';
  $mysqlTbl = 'ScreeningEvents';
  $key      = 'MAX(sampleNumber)';		#MAX is a function: find max value of sampleNumber

  $dsn = 'DBI:mysql:host='.$mysqlHost.';database='.$mysqlDb;
  $dbh = DBI->connect($dsn,$mysqlUser,$mysqlPass,{PrintError=>0,});
  if (!defined $dbh) {
     print STDOUT '*** '.$script.': Cannot connect to mySQL server '.$mysqlHost."\n";
     return -1;
  }
  $sth = $dbh->prepare('SELECT '.$key.' FROM '.$mysqlTbl);
  $sth->execute;
  $hashref = $sth->fetchrow_hashref();
  if (! defined $hashref->{$key}) {
     print STDOUT '*** '.$script.': No "'.$key.'" key'
                        .' in table='.$mysqlDb
                        .' in DB='.$mysqlTbl
                        .' at mySQL server '.$mysqlHost."\n";
     $sth->finish();
     $dbh->disconnect();
     return -2;
  }
  $maxSample = $hashref->{$key};

  $sth->finish();
  $dbh->disconnect();
  return (0,$maxSample);
}
