#!/usr/bin/perl -W

  use strict;
  use warnings;

  our ($expectations, $expectstamp);

  if (!defined $expectations) {$expectations = __FILE__;}		#currently sourced file
# $expectstamp = localtime((stat($expectations))[9]);			#this may not reflect when the expectations were changed.
# $expectstamp = '2016.10.26';						#put here date/work of last change (@minibeam_ changed) -- see also $hist_date per beamline
# $expectstamp = '2016.11.11';						#put here date/work of last change (i0 & dcm value changed) -- see also $hist_date per beamline
# $expectstamp = '2016.11.15';						#put here date/work of last change (added GS offsets PVs) -- see also $hist_date per beamline
# $expectstamp = '2017.06.01';						#put here date/work of last change (added GS offsets PVs) -- see also $hist_date per beamline
# $expectstamp = '2017.10.10';						#put here date/work of last change (added GS offsets PVs) -- see also $hist_date per beamline
# $expectstamp = '2019.02.19';						#put here date/work of last change (added GS offsets PVs) -- see also $hist_date per beamline
  $expectstamp = '2020.02.10';						#put here date/work of last change (added GS offsets PVs) -- see also $hist_date per beamline

  printf STDOUT "\n".'Sourcing %s (last changed %s)...'."\n", $expectations,$expectstamp;

# ======================================================================================================

sub setup_expectations() {  ############################################################################### begin setup_expectations()

# @desc_name = (pv_name, error message, value_type, values...)
#
# value_type - description (values)
#
#     0      - no expectation (0,0) - USE ONLY FOR MOTORS!
#     1      - single integer value   (x,0)
#     2      - single value with tolerance (x,abs_tol) - USE ONLY FOR MOTORS!
#     3      - range of values (lo,hi)
#     4      - mirror voltages, no expectations (0,0)
#  currently unused:
#     5      - unwanted integer value (x,0)
#     6      - multiple integer values (will try to avoid this)

# ------------------------------------------------------------------------------------------------------
  our ($beamline, $shutter, $shutter_readback);				#these are from beamline.pl
  our ($struckName, $struck_ext, $struckChName, $joergerName);		#these are from beamline.pl
  our ($HAVE_XYZ, $ATTENUATION_USE_FACTOR, $report);       		#these are from beamline.pl

  our ($expectations, $simulate, $gh_assy, $gh_axis);
  our (@mostab_status, @mostab_status_b, @mostab_bpm2_ch, @mostab_use_bpm2);
  our (@beamstop_chk, @shutter_chk, @feedback_chk, @egap_chk);
  our (@wbmtr, $wbsl_distance2source, $wbsl2_distance2source);
  our (@mbmtr, @esmtr, @vals, $att_factor, $minibeam_date);
  our ($kl_pin_channel, $struck_pin_input, $adc_pin_input, $struck_dwell);
  our ($kl_i0_channel,  $struck_i0_input,  $adc_i0_input, $kl_i0_gain_hist);
  our ($adc_scanrate, $scaler, $cnt_time, $historic_aps_cur, $bmstp_pos);
  our (@minibeam_300, @minibeam_20, @minibeam_10, @minibeam_5);
  our ($hist_date, $hist_aps_current, $hist_energy, $hist_dcm_vsum, $hist_att);
  our ($hist_gs_v, $hist_gs_h, $hist_i0, $hist_bmstp);
  our ($hist_vsize, $hist_hsize, $hist_gs_v_mini, $hist_gs_h_mini);
  our ($nkeithley, $maxkeithleytime);
  my ($err, $scalerNo, $i);

  if ($beamline ne '23i' && $beamline ne '23o') {
     die "\n".'Beamline ['.$beamline.'] is unsupported!'."\n\n";
  }

# -----------------------------BEGIN COMMON PV DEFINITIONS

  if (defined $shutter) {
     if ($shutter !~ /^${beamline}:/) {$shutter   = $beamline.':'.$shutter;}
  }
  if (defined $shutter_readback) {
     if ($shutter_readback !~ /^${beamline}:/) {
                                $shutter_readback = $beamline.':'.$shutter_readback;}
  }

  if (defined $HAVE_XYZ) {
     $gh_assy = 'GO:Cr';
     $gh_axis = 'GO:CZ';
  } else {
     $gh_assy = 'GO:St';
     $gh_axis = 'GO:SH';
     if ($beamline eq '23b') {$gh_assy .= 'B';}
  }

### Feedback status
  @mostab_status   = ('mostab:status',                  'Feedback script not running.');
  if ($beamline eq '23i')    {push @mostab_status,   (3,     0.5,     4.5);}
  if ($beamline eq '23o')    {push @mostab_status,   (3,     0.5,     4.5);}

  @mostab_status_b = ('mostab:status',                  'Feedback not in active state.');
  if ($beamline eq '23i')    {push @mostab_status_b, (1,       1,       0);}
  if ($beamline eq '23o')    {push @mostab_status_b, (1,       1,       0);}
          
  @mostab_bpm2_ch  = ('mostab:bpm2_ch',                 'Feedback: unexpected channel for bpm2.');
  if ($beamline eq '23i')    {push @mostab_bpm2_ch,  (1,       3,       0);}
  if ($beamline eq '23o')    {push @mostab_bpm2_ch,  (1,       3,       0);}
     
  @mostab_use_bpm2 = ('mostab:use_bpm2',          	'Feedback: typically, bpm2 not used at ID-D.');
  if ($beamline eq '23i')    {push @mostab_use_bpm2, (1,       0,       0);}
  if ($beamline eq '23o')    {push @mostab_use_bpm2, (1,       0,       0);}

### BluIce runtime settings
  @beamstop_chk    = ('bi:enableBeamstopCheck',   	'Beamstop check should be disabled.');
  if ($beamline eq '23i')    {push @beamstop_chk,    (1,       0,       0);}
  if ($beamline eq '23o')    {push @beamstop_chk,    (1,       0,       0);}

  @shutter_chk     = ('bi:enableEndShutterCheck', 	'Shutter check should be enabled.');
  if ($beamline eq '23i')    {push @shutter_chk,     (1,       1,       0);}
  if ($beamline eq '23o')    {push @shutter_chk,     (1,       1,       0);}

  @feedback_chk    = ('bi:waitForFeedback',       	'Feedback check should be enabled.');
  if ($beamline eq '23i')    {push @feedback_chk,    (1,       1,       0);}
  if ($beamline eq '23o')    {push @feedback_chk,    (1,       1,       0);}

  @egap_chk        = ('bi:enableEnergyGapCheck',  	'Energy gap check should be enabled.');
  if ($beamline eq '23i')    {push @egap_chk,        (1,       1,       0);}
  if ($beamline eq '23o')    {push @egap_chk,        (1,       1,       0);}


### Record motor positions (ActPos)
  $i = 0;
### White Beam Slits (old)
  if ($beamline eq '23i') {
     $wbmtr[$i]    = ['WS:ZC',             	'Consider steering vertically.'];
     if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,   0.000,   0.240); $i++;}         #4 urad steering error
#    if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,   0.000,   0.100); $i++;}         #4 urad steering error

     $wbmtr[$i]    = ['WS:ZS',             	'WBSL V size is non-standard.'];
     if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,   2.400,   0.010); $i++;}
#    if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,   1.000,   0.010); $i++;}

     $wbmtr[$i]    = ['WS:YC',             	'Consider steering horizontally.'];
     if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,   0.000,   0.240); $i++;}         #4 urad steering error
#    if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,   0.000,   0.100); $i++;}         #4 urad steering error

     $wbmtr[$i]    = ['WS:YS',             	'WBSL H size is non-standard.'];
     if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,   3.200,   0.010); $i++;}
#    if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,   2.700,   0.010); $i++;}
  }

### White Beam Slits (new)
  $wbmtr[$i]       = ['WB:V',             	'Consider steering vertically.'];
  if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,   0.000,   0.100); $i++;}         #4 urad steering error
  if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,   0.000,   0.100); $i++;}         #4 urad steering error

  $wbmtr[$i]       = ['WB:SV',             	'WBSL2 V size is non-standard.'];
  if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,   1.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,   1.000,   0.010); $i++;}

  $wbmtr[$i]       = ['WB:H',             	'Consider steering horizontally.'];
  if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,   0.000,   0.100); $i++;}         #4 urad steering error
  if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,   0.000,   0.100); $i++;}         #4 urad steering error

  $wbmtr[$i]       = ['WB:SH',             	'WBSL H size is non-standard.'];
  if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,   2.700,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,   2.700,   0.010); $i++;}



### Monochromator & Mono BPM
  $wbmtr[$i]       = ['MO:Z',              	'CAUTION: DCM vert is non-standard!'];
  if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,   0.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,  -0.250,   0.010); $i++;}

  $wbmtr[$i]       = ['MO:Y',              	'CAUTION: DCM horz is non-standard!'];
  if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,   0.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,   1.000,   0.010); $i++;}

  $wbmtr[$i]       = ['MO:E',              	'Energy is non-standard.'];
  if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,  12.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,  12.000,   0.010); $i++;}

  $wbmtr[$i]       = ['MO:R1',             	'Xtal 1 roll is non-standard.'];
  if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,   0.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,   0.000,   0.010); $i++;}

  $wbmtr[$i]       = ['MO:P2',             	'Mono Crystal-2 pitch'];
  if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (0,  undef,    undef); $i++;}
  if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (0,  undef,    undef); $i++;}

  $wbmtr[$i]       = ['BP:MOZ',            	'BPM_DCM is not on silver foil.'];
  if ($beamline eq '23i')    {push @{$wbmtr[$i]},    (2,   0.000,   4.500); $i++;}
  if ($beamline eq '23o')    {push @{$wbmtr[$i]},    (2,   0.000,   2.500); $i++;}

  if ($beamline eq '23i') {
     $wbmtr[$i]    = ['BP:MZ2',            	'BPM_DCM2 is not on ????? foil.'];
                              push @{$wbmtr[$i]},    (2,   0.000,   4.500); $i++;
  }

### Horizontal deflecting mirrors (IDB only)
  if ($beamline eq '23o') {
     $wbmtr[$i]    = ['HD:SL',             	'HDM lane is non-standard.'];           #HDM values could easily be adjusted with re-alignment
                              push @{$wbmtr[$i]},    (2,  12.000,   0.010); $i++;

     $wbmtr[$i]    = ['HD:HT',             	'HDM tilt is non-standard.'];
                              push @{$wbmtr[$i]},    (2,   0.000,   0.010); $i++;

     $wbmtr[$i]    = ['HD:HR',             	'HDM roll is non-standard.'];
                              push @{$wbmtr[$i]},    (2,   0.000,   0.010); $i++;

     $wbmtr[$i]    = ['HD:UY',             	'HDM1 Y is non-standard.'];
                              push @{$wbmtr[$i]},    (2,  -0.174,   0.010); $i++;

     $wbmtr[$i]    = ['HD:UA',             	'HDM1 angle is non-standard.'];
                              push @{$wbmtr[$i]},    (2,   4.017,   0.010); $i++;

     $wbmtr[$i]    = ['HD:DY',             	'HDM2 Y is non-standard.'];
                              push @{$wbmtr[$i]},    (2,   5.326,   0.010); $i++;

     $wbmtr[$i]    = ['HD:DA',             	'HDM2 angle is non-standard.'];
                              push @{$wbmtr[$i]},    (2,  12.118,   0.010); $i++;
  }

  $i = 0;
### Collimator Slits
  $mbmtr[$i]	   = ['CS:ZC',	                'Monobeam Slit V cntr is non-standard.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,   1.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,  -0.600,   0.010); $i++;}

  $mbmtr[$i]	   = ['CS:ZS',	                'Monobeam Slit V size is non-standard.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,   1.650,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   1.200,   0.010); $i++;}

  $mbmtr[$i]	   = ['CS:YC',	                'Monobeam Slit H cntr is non-standard.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,   0.350,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   0.125,   0.010); $i++;}

  $mbmtr[$i]	   = ['CS:YS',	                'Monobeam Slit H size is non-standard.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,   3.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   3.000,   0.010); $i++;}

### KBM
  $mbmtr[$i]       = ['KB:HY',             	'HFM position is non-standard.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,   0.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   0.000,   0.010); $i++;}

  $mbmtr[$i]       = ['KB:HA',             	'HFM angle is non-standard.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,   3.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   3.000,   0.010); $i++;}

  $mbmtr[$i]       = ['KB:HL',             	'Non-standard lane may cause intensity problems.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,  12.495,   2.505); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   0.000,   2.510); $i++;}

  $mbmtr[$i]       = ['KB:HW',             	'HFM yaw is non-standard.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,   0.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   0.000,   0.010); $i++;}

  $mbmtr[$i]       = ['KB:HR',             	'HFM roll is non-standard.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,   0.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   0.000,   0.010); $i++;}

  $mbmtr[$i]       = ['KB:VZ',             	'VFM position is non-standard.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,   0.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   0.000,   0.010); $i++;}

  $mbmtr[$i]       = ['KB:VA',             	'VFM angle is non-standard. CCD may not track.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,   3.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   3.000,   0.010); $i++;}

  $mbmtr[$i]       = ['KB:VL',             	'Non-standard lane may cause intensity problems.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,  -7.000,   3.000); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   5.670,   2.510); $i++;}

  $mbmtr[$i]       = ['KB:VW',             	'VFM yaw is non-standard.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,   6.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   6.000,   0.010); $i++;}

  $mbmtr[$i]       = ['KB:VR',             	'VFM roll is non-standard.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2,   0.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2,   0.000,   0.010); $i++;}

  $mbmtr[$i]       = ['BP:KBZ',            	'BPM not on blank. Will attenuate beam.'];
  if ($beamline eq '23i')    {push @{$mbmtr[$i]},    (2, -30.500,   2.500); $i++;}
  if ($beamline eq '23o')    {push @{$mbmtr[$i]},    (2, -30.500,   2.500); $i++;}

  $i = 0;
### Beam Delivery
  $esmtr[$i]       = ['BD:VH',             	'Beam delivery vertical height'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['BD:VA',             	'Beam delivery vertical angle'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['BD:HD',             	'Beam delivery horizontal pos.'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['BD:HA',             	'Beam delivery horizontal angle'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
     
### Guard Slits
  $esmtr[$i]       = ['GS:X',              	'Guard slits support X'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
 
  $esmtr[$i]       = ['GS:YW',             	'GS yaw is non-standard.'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (2,   0.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (2,   0.000,   0.010); $i++;}

  $esmtr[$i]       = ['GS:ZC',             	'GS vertical center pos.'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['GS:ZS',             	'GS vertical size'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['GS:YC',             	'GS horizontal center pos.'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['GS:YS',             	'GS horizontal size'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

### CRLs
# $esmtr[$i]       = ['CRL:AP',        	        'CRL Pitch angle'];
# if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

# $esmtr[$i]       = ['CRL:AW',        	        'CRL Yaw angle'];
# if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

# $esmtr[$i]       = ['CRL:XS',        	        'CRL small support horiz. pos.'];
# if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

# $esmtr[$i]       = ['CRL:YS',        	        'CRL small support vert. pos.'];
# if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

# $esmtr[$i]       = ['CRL:P',        	        'CRL Pitch mm'];
# if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

# $esmtr[$i]       = ['CRL:W',        	        'CRL Yaw mm'];
# if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

# $esmtr[$i]       = ['CRL:ZB',        	        'CRL large support downstream pos.'];
# if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

# $esmtr[$i]       = ['CRL:YB',        	        'CRL large support vert.pos.'];
# if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['CRL:Z',        	        'Transforcator support vertical is off'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (2,   4.4,   0.5); $i++;}

  $esmtr[$i]       = ['CRL:PT',        	        'Transforcator support pitch is off'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (2,   5.6,   0.5); $i++;}

  $esmtr[$i]       = ['CRL:RL',        	        'Transforcator support roll is off'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (2,   0.0,   0.5); $i++;}

  $esmtr[$i]       = ['CRL:Y',        	        'Transforcator support horizontal is off'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (2,   2.3,   0.5); $i++;}

  $esmtr[$i]       = ['CRL:YW',        	        'Transforcator support yaw is off'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (2,  -6.5,   0.5); $i++;}

  $esmtr[$i]       = ['CRL:X',        	        'Transforcator translation along beam'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,  undef,   undef); $i++;}

### Shutter
  $esmtr[$i]       = ['SH:P',              	'Fast shutter position'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

### Goniometer
  $esmtr[$i]       = ['GO:SV',             	'Gonio support vertical'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = [$gh_axis,              	'Gonio support horizontal'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['GO:SW',             	'Gonio yaw is non-standard.'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (2,   0.000,   0.010); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (2,   0.000,   0.010); $i++;}

  $esmtr[$i]       = ['GO:OM',             	'Gonio omega angle'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['GO:CX',             	'Gonio crystal X'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['GO:CY',             	'Gonio crystal Y'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

### Beamstop
  $esmtr[$i]       = ['BS:D',              	'Beamstop distance'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['BS:PT',             	'Beamstop pitch'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['BS:YW',             	'Beamstop yaw'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

### On-axis cameras zoom
  $esmtr[$i]       = ['TV:ZM1',            	'Hi-res. camera zoom'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['TV:ZM2',            	'Lo-res. camera zoom'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

### Robot
  $esmtr[$i]       = ['RB:RX',             	'Robot dewar X'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['RB:RZ',             	'Robot dewar Z'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['RB:RA',             	'Robot dewar angle'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['RB:GV',             	'Robot gripper vertical'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['RB:GR',             	'Robot gripper rotation'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['RB:GH',             	'Robot gripper horizontal'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

### Fluorescence detector
  $esmtr[$i]       = ['FLU:D',             	'Fluo.detector distance'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

### Pin Diode
  $esmtr[$i]       = ['PIN:P',             	'Pindiode position'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

### Collimator
  $esmtr[$i]       = ['COL:AP',             	'Collimator pitch angle'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['COL:AY',             	'Collimator yaw angle'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['COL:V',             	'Collimator vert.pos.'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['COL:H',             	'Collimator horiz.pos.'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

### CCD Support
  $esmtr[$i]       = ['CCD:DL',            	'Detector lateral pos.'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['CCD:D',             	'Detector distance'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

  $esmtr[$i]       = ['CCD:2Q',            	'Detector two theta'];
  if ($beamline eq '23i')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$esmtr[$i]},    (0,   undef,   undef); $i++;}

### Record values (DAC, ADC, ...)
  $i = 0;
  $vals[$i]       = ['DAC2_3',            	'Horiz.Mirror Piezo'];
  if ($beamline eq '23i')    {push @{$vals[$i]},     (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$vals[$i]},     (0,   undef,   undef); $i++;}

  $vals[$i]       = ['DAC2_2',            	'Vert.Mirror Piezo'];
  if ($beamline eq '23i')    {push @{$vals[$i]},     (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$vals[$i]},     (0,   undef,   undef); $i++;}

  $vals[$i]       = ['DAC1_4',            	'Ring. Light'];
  if ($beamline eq '23i')    {push @{$vals[$i]},     (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$vals[$i]},     (0,   undef,   undef); $i++;}

  $vals[$i]       = ['DAC1_5',            	'LowRez. Light'];
  if ($beamline eq '23i')    {push @{$vals[$i]},     (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$vals[$i]},     (0,   undef,   undef); $i++;}

  $vals[$i]       = ['DAC1_6',            	'HiRez. Light'];
  if ($beamline eq '23i')    {push @{$vals[$i]},     (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$vals[$i]},     (0,   undef,   undef); $i++;}

  $vals[$i]       = ['DAC1_8',            	'Back Light'];
  if ($beamline eq '23i')    {push @{$vals[$i]},     (0,   undef,   undef); $i++;}
  if ($beamline eq '23o')    {push @{$vals[$i]},     (0,   undef,   undef); $i++;}


### Minibeam measurement arguments
  $ATTENUATION_USE_FACTOR  = 1;						#this must be global
  if ($beamline eq '23i') {$att_factor = 1;}				#attenuation for measuring mini-beam entered as a factor - will be converted to percentage elsewhere
  if ($beamline eq '23o') {$att_factor = 5;}				#attenuation for measuring mini-beam entered as a factor - will be converted to percentage elsewhere
  $adc_i0_input            = 9;
  $adc_pin_input           = 10;
  $kl_i0_gain_hist         = 1e7;
# $kl_i0_channel           = 6;
# $kl_pin_channel          = 3;
# $struck_i0_input         = 18;					#for recording 5um minibeam intensity as a function of time
# $struck_pin_input        = 17;					#for recording 5um minibeam intensity as a function of time
  if (! $simulate) {
     ($err,$kl_i0_channel, $struck_i0_input )=&getSignalChannels('I0');   if ($err) {die "\n".'*** No SQL data for I0'."\n\n";}
     ($err,$kl_pin_channel,$struck_pin_input)=&getSignalChannels('Pin');  if ($err) {die "\n".'*** No SQL data for Pin'."\n\n";}
  } else {
     $kl_i0_channel  = 0; $struck_i0_input  = 0;
     $kl_pin_channel = 0; $struck_pin_input = 0;
  }
  $adc_scanrate            = 9;
  $struck_dwell            = 0.001;					#for recording 5um minibeam intensity as a function of time
  $cnt_time                = 1;						#time of recording intensities with "Joerger"
  $scalerNo                = 1;						#Struck as Joerger
  $scaler = $joergerName.$scalerNo;
  if ($scaler !~ /^${beamline}/) {$scaler = $beamline.':'.$scaler;}
  $nkeithley               = 7;						#number of keithley at beamlines
  $maxkeithleytime         = '10ms';					#high integration time warning threshold

  $wbsl_distance2source = 0;						#distance between X-ray source & WBSL
  if ($beamline eq '23i') {$wbsl_distance2source = 60.313;}		#in meters
  if ($beamline eq '23o') {$wbsl_distance2source = 27.314;}		#in meters
  $wbsl2_distance2source = 0;						#distance between X-ray source & WBSL
  if ($beamline eq '23i') {$wbsl2_distance2source = 27.100;}		#in meters
  if ($beamline eq '23o') {$wbsl2_distance2source = 27.610;}		#in meters

# -----------------------------END COMMON PV DEFINITIONS

  if ($beamline eq '23i') { # --BEGIN OF 23i----------------------------------------------------------------------------------------

### ATTENTION!!!  CHANGE THE HISTORICAL NUMBERS AS NEEDED!!!
###
### Historical numbers (also see historical minibeam intensities in next section)
##   $hist_date        = 'Oct 2011 (dwy)';				#Date of 'historic' measurement
##   $hist_date        = 'Feb 2013 (RFF, 23i)';				#Date of 'historic' measurement
##   $hist_date        = 'Oct 2016 (NV, 23i)';				#Date of 'historic' measurement
##   $hist_date        = 'Nov 2016 (NV, 23i)';				#Date of 'historic' measurement
##   $hist_date        = 'Dec 2016 (NV, 23i)';				#Date of 'historic' measurement
##   $hist_date        = 'Jun 2017 (NV, 23i)';				#Date of 'historic' measurement
##   $hist_date        = 'Oct 2017 (NV, 23i)';				#Date of 'historic' measurement, same as Dec 2016
     $hist_date        = 'Feb 2019 (NV, 23i)';				#Date of 'historic' measurement, same as Dec 2016
     $hist_aps_current = 102;
     $hist_i0          = 5.07;						#9.0,  then 6.75, then 1.25
     $hist_dcm_vsum    = 4.91;						#2.93, then 2.71
     $hist_energy      = 12.0;
     $hist_gs_v        = 2;
     $hist_gs_h        = 2;
     $hist_gs_v_mini   = 0.2;
     $hist_gs_h_mini   = 0.2;
     $hist_att         = 5;
     $hist_bmstp       = 35;
     $hist_vsize       = 66.5;						#focused beamsize in um
     $hist_hsize       =142.8;						#focused beamsize in um

# @arguments = (pinhole_active_preset,     	#0
#               pindiode_gain, 			#1
#               expected_intensity, 		#2
#               warn_level(%), 			#3
#               date_of_measurement);		#4
#                      0    1     2      3         4
#    @minibeam_300  = (1,  1e3,  0.83,  0.2,  $minibeam_date);		#2017.10
#    @minibeam_20   = (2,  1e4,  1.02,  0.2,  $minibeam_date);		#2017.10
#    @minibeam_10   = (3,  1e4,  0.34,  0.2,  $minibeam_date);		#2017.10
#    @minibeam_5    = (4,  1e4,  0.18,  0.2,  $minibeam_date);		#2017.10
     @minibeam_300  = (1,  1e3,  1.05,  0.2,  $minibeam_date);		#2019.02
     @minibeam_20   = (2,  1e4,  3.48,  0.2,  $minibeam_date);		#2019.02
     @minibeam_10   = (3,  1e4,  1.50,  0.2,  $minibeam_date);		#2019.02
     @minibeam_5    = (4,  1e4,  0.56,  0.2,  $minibeam_date);		#2019.02

  }  # --END OF 23i------------------------------------------------------------------------------------------


### ======================================================================================================
### Here begins 23o

  elsif ($beamline eq '23o') { # --BEGIN OF 23o----------------------------------------------------------------------------------------

### ATTENTION!!!  CHANGE THE HISTORICAL NUMBERS AS NEEDED!!!
###
### Historical numbers (also see historical minibeam intensities in next section)
##   $hist_date        = 'Feb 2011 (DY)';	#Date of 'historic' measurement
##   $hist_date        = 'Feb 2013 (RFF, 23o)';	#Date of 'historic' measurement
##   $hist_date        = 'Jan 2014 (NV, 23o)';	#Date of 'historic' measurement
     $hist_date        = 'Oct 2016 (NV, 23o)';	#Date of 'historic' measurement
     $hist_aps_current = 102;
     $hist_i0          = 0.87;			#4.00;
     $hist_dcm_vsum    = 1.39;
##   $hist_i0          = 2.63;			#4.00;
##   $hist_dcm_vsum    = 0.84;
     $hist_energy      = 12.0;
     $hist_gs_v        = 2;
     $hist_gs_h        = 2;
     $hist_gs_v_mini   = 0.2;
     $hist_gs_h_mini   = 0.2;
     $hist_att         = 5;
     $hist_bmstp       = 40;
#    $hist_vsize       = 37.4;						#focused beamsize in um (when mirror was upstream (before 10/2013))
#    $hist_hsize       =129.4;						#focused beamsize in um (when mirror was upstream (before 10/2013))
     $hist_vsize       = 16.22;						#focused beamsize in um (this after mirror moved downstream (10/2013))
     $hist_hsize       = 79.59;						#focused beamsize in um (this after mirror moved downstream (10/2013))

# @arguments = (pinhole_active_preset,     	#0
#               pindiode_gain, 			#1
#               expected_intensity, 		#2
#               warn_level(%), 			#3
#               date_of_measurement);		#4
#                      0    1     2      3         4
     @minibeam_300  = (1,  1e4,  2.59,  0.2,  $minibeam_date);         #after mirror moved downstream (10/2013)
     @minibeam_20   = (2,  1e4,  0.48,  0.2,  $minibeam_date);         #after mirror moved downstream (10/2013)
     @minibeam_10   = (3,  1e4,  0.18,  0.2,  $minibeam_date);         #after mirror moved downstream (10/2013)
     @minibeam_5    = (4,  1e6,  9.97,  0.2,  $minibeam_date);         #after mirror moved downstream (10/2013)

  }  # --END OF 23o------------------------------------------------------------------------------------------

} ######################################################################################################## end of setup_expectations()
