#!/usr/bin/perl -W
#############################################################
# script for finding device names for external HFS+ disks using
# the udisks utility
# Version-1.0 -- Jun 21 2013 -- initial implementation
# Version-2.0 -- Sep 18 2015 -- adapted to udisks2 (CentOS7)
#############################################################

  use strict;

  my ($PEZCA, @devices, @params, @mounted, @words, @lines);
  my (%PAR, $device, $line, $string, $nhfs);
  my (@dm, @uuid, @hfspartitions, $prog, $status);

  select STDOUT; $|=1;				#set unbuffered output
  select STDERR; $|=1;				#set unbuffered output

  if (!defined $ENV{PEZCA}) {die 'PEZCA undefined';}
  $PEZCA = $ENV{PEZCA}; $PEZCA =~ s/\\/\//g;	#replace \ by /
  if ($PEZCA !~ /\/$/) {$PEZCA .= '/';}		#append /
  do $PEZCA.'Common/common_subs.pl';

  @devices = split (/\n/, `/bin/ls -1 /dev/sd* 2>/dev/null`);

  ### find all /dev/sd* devices, which are already mounted:
  @lines = split (/\n/, `/bin/mount | grep "^\/dev\/sd"`);
  @mounted = ();
  foreach $line (@lines) {
     @words = split (/\s+/, $line);
     push (@mounted, $words[0]);
  }
# print STDOUT 'Mounted: '."@mounted\n";

  ### find multipath devices:
  @dm = split (/\n/, `/bin/ls -1 /dev/dm* 2>/dev/null`);
  ### find uuid of multipath devices:
  @uuid = ();
  foreach $device (@dm) {
     &getPAR($device,\%PAR);
     if (defined $PAR{'uuid'}) {push (@uuid, $PAR{'uuid'});}
  }
# print STDOUT 'UUID: '."@uuid\n";

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  @hfspartitions = ();
  foreach  $device (@devices) {

     ### fill udisks info for the device
     &getPAR($device,\%PAR);

     ### verify that device is not a part of multipath devices:
     $string = $PAR{'uuid'};
     if (defined $string) {
        if (grep(/^${string}$/, @uuid)) {next;}		#this device is a part of multipath
     }

     ### check if the device is formatted as HFS or HFS+:
#    print $PAR{'type'},"\n";
     $string = $PAR{'type'};
     if (defined $string && $string =~ /^hfs/) {
        print STDOUT "\n".'Found HFS partition='.$device.', type='.$string."\n";
        ### check if the device in mounted list
        if (grep(/^${device}$/, @mounted)) {
	   print STDOUT '    (partition already mounted, skipping)'."\n";
	} else {
           push (@hfspartitions, $device);
	}
     }
  }
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  $nhfs = @hfspartitions;
  if ($nhfs == 0) {print STDOUT "\n".'No non-mounted HFS/HFS+ partitions found!'."\n\n";}
  else {
     $prog = '/usr/local/bin/chkhfs';
     if (-f $prog) {
        foreach $device (@hfspartitions) {
           $string = $prog.' -f -a --verbose '.$device;
           print STDOUT "\n".'Checking '.$device.' for errors using:'."\n".$string."\n";
	   $status = system($string);
	   print STDOUT "\n".'Check of '.$device.' finished with status='.$status."\n\n";
           ### attempt to mount the device:
           print 'Attempting to mount '.$device.' ...'."\n";
           $string = `/usr/bin/udisks --mount $device`;
           if ($?) {print STDOUT '.. failure='.$?."\n",'Log: ',"\n",$string,"\n";}
        }
     } else {
        print STDOUT "\n".'*** Could not find '.$prog."\n";
     }
  }
  exit 0;

#==========================================================================================
sub getPAR($\%) {
  my ($device, $PARref, @params, $string, @words, $key, $udisks);
  $device = shift(@_);
  $PARref = shift(@_);

  ### fill udisks info for the device
  if    (defined &Which('udisks'))    {@params = split (/\n/, `udisks --show-info $device`);}
# elsif (defined &Which('udisksctl')) {@params = split (/\n/, `udisksctl info -b $device`);}
# else  {die "\n".'Please install udisks (Centos6) or udisks2 (Centos7) or oldudisks (Centos7, nux-desktop repo)'."\n";}
  else  {die "\n".'Please install udisks (Centos6) or oldudisks (Centos7, nux-desktop repo)'."\n";}

  %$PARref = ();
  foreach $string (@params) {
     $string =~ s/^\s+//;				#remove firs two spaces at the beginning of line
     @words = split(/:/,$string);			#we are interested in first ":" only - will merge the rest
     $key = $words[0];
     splice @words,0,1;
     $string = join(':',@words);
     if    (length($string) > 0) {
        $$PARref{$key} = $string;
     }
     elsif ($key =~ /^\/dev\/disk\/by-uuid\//) {
        $string = $key; 
        $string =~ s/^\/dev\/disk\/by-uuid\///;
	$key = 'uuid';
        $$PARref{$key} = $string;
     }
  }
}
