#!/usr/bin/perl -W
#--------------------------------------------------------------
# This script serves as a wrapper for x11vnc server. Its task is
# to start x11vnc only if needed (to ensure there are no multiple
# copies and no competing vino), or to force restarting, or
# shutting down.
#
#                       Author: Sergey Stepanov
#
# Version-1.0: 2012/02/15 initial implementation
# Version-1.1: 2012/02/23 added "exit 1" on error; added checking if server started
# Version-1.2: 2015/09/25 extended to Mate desktop
#--------------------------------------------------------------
  use strict;

  our ($NONINTERACTIVE, $USE_ZENITY_FOR_EXIT);
  my ($PEZCA, $os, $script, $syntax, $narg, $arg, $needstart);
  my ($status, $mode, $display_specified, $bg_specified, $log_specified, $i);
  my ($myUser, $vncserver, $vncpath, $nforeignUser, $logfile, $debug);
  my ($nprocesses, @processlist, $process, @words, @cmd);
  my ($wait_,$wait_step,$wait_max);

  if    (defined $ENV{PEZCA})     {$PEZCA=$ENV{PEZCA};}
  elsif (defined $ENV{FASTSCANS}) {$PEZCA=$ENV{FASTSCANS}; $PEZCA=~s/fastscans$/pezca/i;}
  else {if ($^O !~ /Win/i) {$PEZCA='/home/gmca/epics/pezca';}else{$PEZCA='c:/gmca/pezca';}}
  $PEZCA =~ s/\\/\//g;

  do $PEZCA.'/Common/common_subs.pl';

  select STDOUT; $|=1;                                  #set unbuffered output
  select STDERR; $|=1;                                  #set unbuffered output

  $script = &makeScriptName();
  $syntax = "\n\n".'Syntax: '.$script.' mode=startasneeded|restart|stop [debug=0|1] [x11vncparameters]'."\n";
  $USE_ZENITY_FOR_EXIT = 0;				#set to '1' for graphical popups
  $NONINTERACTIVE      = 1;				#set to one to suppress 'press key to exit'
  $vncserver           = 'x11vnc';
  $vncpath             = '/usr/bin/';
  $wait_max            = 4.;
  $wait_step           = 0.1;
  $debug               = 0;				#default. Debug is not yet used -- reserved for future

  if ($^O !~ /Win/i) {$os = 'LINUX';}
  else               {$os = 'WIN32';}
  if ($os !~ /^LINUX$/) {&press_key_exit ('*** '.$script.': cannot run on '.$os,1);}

  $narg = @ARGV;
# if ($narg > 0) {print STDOUT "\n".$script.': started with cmdline: '."@{ARGV}\n\n";}
# else           {print STDOUT "\n".$script.': started with no cmdline parameters'."\n\n";}
###----------------------------------------------------------------
###             Analyze command line arguments:
###----------------------------------------------------------------
### Possible x11vnc arguments:
### x11vnc -no6 -quiet -shared -rfbport 5901 -rfbauth $passwordfile -wait 100 \
###           -scale $SCALE -allow $CLIENTS -forever -gui tray -display :0 -bg
  for ($i=0; $i<$narg; $i++) {
     $arg = $ARGV[$i];
     if ($arg =~ /^-{0,2}mode=/i) {
        splice(@ARGV,$i,1);
        $narg--;
        $i--;
        $arg =~ s/^-{0,2}mode=//i;
        $mode = lc($arg);
        if ($mode !~ /^startasneeded$/ && $mode !~ /^restart$/ && $mode !~ /^stop$/) {
           &press_key_exit ('*** '.$script.': unknown operation mode='.$mode.$syntax,1);
        }
     }
     elsif ($arg =~ /^-{0,2}debug=/i) {
        splice(@ARGV,$i,1);
        $narg--;
        $i--;
        $arg =~ s/^-{0,2}debug=//i;
        $debug = lc($arg);
        if ($debug ne 0 && $debug ne 1) {
           &press_key_exit ('*** '.$script.': unknown debug option'.$debug.$syntax,1);
        }
     }
     elsif ($arg =~ /^-display$/) {$display_specified=1;}
     elsif ($arg =~ /^-bg$/)      {$bg_specified=1;}
     elsif (($arg =~ /^-o$/  || $arg =~ /^-logfile$/ ||
             $arg =~ /^-oa$/ || $arg =~ /^-logappend$/) && $i < $narg-1) {
	$log_specified = 1;
	$logfile       = $ARGV[$i+1];
     }
  }
  if (!defined $mode) {
     &press_key_exit ('*** '.$script.': operation mode not specified.'.$syntax,1);
  }
  if (!defined $display_specified) {$narg = push(@ARGV, '-display',':0');}
  if (!defined $bg_specified)      {$narg = push(@ARGV, '-bg');}
### -----------------------------------------------------------------

  if    (defined $ENV{USER})     {$myUser = $ENV{USER};}
  elsif (defined $ENV{USERNAME}) {$myUser = $ENV{USERNAME};}
  else {
     &press_key_exit ('*** '.$script.': neither USER nor USERNAME environment defined.',1);
  }
  if (&is_on_console($myUser) != 1) {exit 1;}
### -----------------------------------------------------------------

  if ($mode !~ /^stop$/) {$needstart = 1;}
  else                   {$needstart = 0;}

  $nprocesses = &getprocesslist($vncserver,\@processlist);				#get active instances of x11vnc
  if ($nprocesses > 0) {
     print STDOUT "\n".$script.': found '.$nprocesses.' instances of '.$vncserver.' running'."\n\n";
     print STDOUT "\n@{processlist}\n\n";

     if ($mode =~ /^startasneeded/ && $nprocesses == 1) {$needstart = 0;}

     if ($mode =~ /^stop$/ || $mode =~ /^restart/ || ($mode =~ /^startasneeded/ && $nprocesses > 1)) {
#       print STDOUT $script.': sending "disconnect:all"'."\n";
#	@cmd = ($vncpath.$vncserver,'-R','disconnect:all','-display',':0');
#       if ($log_specified) {push @cmd, ('-logappend',$logfile);}
#	$status = system(@cmd);
        print STDOUT $script.': sending "stop"'."\n";
	@cmd = ($vncpath.$vncserver,'-R','stop','-display',':0');
        if ($log_specified) {push @cmd, ('-logappend',$logfile);}
	$status = system(@cmd);
	sleep 1;
	$nprocesses = &getprocesslist($vncserver,\@processlist);
        $nforeignUser = 0;
	if ($nprocesses > 0) {
           print STDOUT $script.': executing "killall -SIGINT '.$vncpath.$vncserver.'"'."\n";
           $status = system('/usr/bin/killall','-SIGINT',$vncpath.$vncserver);
	   sleep 1;
	   $nprocesses = &getprocesslist($vncserver,\@processlist);
	   if ($nprocesses > 0) {
              foreach $process (@processlist) {
	         ### Example structure of $process:
	         ### gmca                2777 /usr/libexec/clock-applet --oaf-activate .....
                 @words = split ' ', $process;
                 ### Workaround for LDAP users: ps shows their user ID instead of name:
                 ### ($user, $passwd, $uid, $gid, $quota, $comment, $gcos, $homedir, $shell) = getpwuid($uid);
                 ### ($user, $passwd, $uid, $gid, $quota, $comment, $gcos, $homedir, $shell) = getpwnam($username);
                 if ($words[0] =~/^[\d]*$/) {$words[0]=getpwuid($words[0]);}      #only need with "ps -ef"
                 if ($myUser eq $words[0]) {
                    print STDOUT $script.': killing "'.$process.'"'."\n";
                    $status = kill ('SIGINT', $words[1]);
                    if ($status) {$nprocesses--;}
                 } else {
                    $nforeignUser++;
                    print STDOUT '*** '.$script.': root privileges required to kill '.$vncserver
                                       .' pid='.$words[1].' that belongs to user='.$words[0]."\n";
                 }
              }
	   }
        }
	if ($nforeignUser > 0) {
           print STDOUT '*** '.$script.': '.$nforeignUser.' instances of '.$vncserver
                                           .' could not be killed as they belong to different users!'."\n";
	   $needstart = 0;
	   exit 1;
        }
     }
  }
  if ($needstart) {
     print STDOUT $script.': executing "'.$vncpath.$vncserver." @{ARGV}\"\n";
     $status = system($vncpath.$vncserver,@ARGV);
     $wait_ = 0;
     do	{
        select(undef,undef,undef,$wait_step);
	$wait_ += $wait_step;
	$nprocesses = &getprocesslist($vncserver,\@processlist);
     } while ($nprocesses < 1 && $wait_ < $wait_max);
     if ($nprocesses < 1) {&press_key_exit ('*** '.$script.': failed to start '.$vncserver,1);}
     else                 {print STDOUT $script.': '.$vncserver.' started OK'."\n";}
  }
  &press_key_exit ($script.': finished.',0);

#==========================================================================

sub getprocesslist($\@) {					#get active instances of a process
  my ($task, $rqs, $filter, $proclistRef, $nproc);
  $task        = shift(@_);
  $proclistRef = shift(@_);

### <defunct> and /bin/tail may appear with the "-gui tray|icon|full" flag:
  $rqs = '/bin/ps -eo user:19,pid,cmd | /bin/grep';
  $filter = '| /bin/grep -v "grep\|edit\|term\|wrapper\|onsole\|/bin/tail \|\.tray\.\|\<defunct\>"';
  @$proclistRef = split /^/, `${rqs} ${task} ${filter}`;
  $nproc = @$proclistRef;
  return $nproc;
}

#==========================================================================

sub is_on_console($) {
  my ($user, $rec, @records, $line, @words, $nwords, $logged, $gnome, $rqs);
  $user = shift(@_);

  $logged = 0;
  $gnome = 0;
  @records = split /^/, `/usr/bin/who`;
  foreach $rec (@records) {
     @words = split /\s+/, $rec;
     $nwords = @words;
     if    ($nwords == 0)                          {next;}
     elsif ($words[$nwords-1] !~ /^\(:(0\.)?0\)$/) {next;}
     else {
        if ($user eq $words[0]) {$logged = 1; last;}
     }
  }
### This looks for Gnome or Mate only. What about KDE?:
  @records = split /^/, `/bin/ps -eo user:19,pid,cmd | /bin/grep "gnome-session\|mate-session" | /bin/grep -v "grep\|\<defunct\>"`;
  foreach $rec (@records) {
     @words = split /\s+/, $rec;
     $nwords = @words;
     if    ($nwords == 0)                   {next;}
     else {
        if ($user eq $words[0]) {$gnome = 1; last;}
     }
  }
  if    ($logged == 1 && $gnome == 1) {return 1;}
  elsif ($logged == 1 && $gnome == 0) {
     print STDOUT '*** '.$script.': user='.$user.' does not have Gnome or Mate session open on console.'."\n";
     return 0;
  }
  else {
     print STDOUT '*** '.$script.': user='.$user.' is not logged on console.'."\n";
     return -1;
  }
}
