package gov.anl.gmca.analysis;

import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.syslog.Log;

import java.io.*;
import java.util.Arrays;

// actually run the processing pipeline
public class CmdProcessData implements Runnable {
	private static String scriptTemplate = 
			"#!/bin/sh\n" 
			+ "##$ -cwd\n"
			+ "##$ -j y\n"
			+ "##$ -V\n"
			+ "##$ -S /bin/bash \n"
			+ "#SBATCH -o %s.out \n"
			+ "#SBATCH --export=ALL \n"
			+ "##$ -o %s.err \n"
			+ "\n" 
			+ "echo job started at `date \"+%%Y-%%m-%%d %%H:%%M:%%S\"`\n"
			+ "echo \"host: `hostname -s` (`uname`) user: `whoami`\"\n" 
			+ "echo\n" 
			+ "echo\n" 
			+ "\n" 
			+ "%s\n"
			+ "export PATH=/mnt/software/px/miniconda3/bin:$PATH\n"
			+ "#export PYTHONPATH=%s/tools/scripts/quickProcess:$PYTHONPATH\n" 
			+ "export PYTHONPATH=/mnt/software/quickProcess/quickProcess:$PYTHONPATH\n" 
			+ "python -m quickProcess.%s %s\n" 
			+ "\n"
			+ "echo\n" 
			+ "echo\n" 
			+ "echo job finished at `date \"+%%Y-%%m-%%d %%H:%%M:%%S\"`\n";

	private String scriptName;
	private String script;
	private String joblabel;
	private String outdir;
	private boolean block = false;
	private PipelineOptions pipelineOptions;

	public String getJoblabel() {
		return this.joblabel;
	}

	public CmdProcessData(PipelineOptions p, String joblabel, boolean waitFor) {
		this.pipelineOptions = p;
		joblabel = joblabel.replace(' ', '_');
		this.joblabel = joblabel;
		this.block = waitFor;
		// decide which task to start quickProcess.process or quickProcess.strategy
		String[] strategiesPipelines = new String[] { "mosflm_strategy", "labelit_strategy", "dials_strategy",
				"xds_strategy" };
		String task;
		if (Arrays.asList(strategiesPipelines).contains(p.getPipeline())) {
			task = "strategy";
		} else {
			task = "process";
		}

		outdir = p.getOutputDirectory();
		File odir = new File(outdir);
		if (!odir.exists()) {
			odir.mkdirs();
		}
		String cddir = String.format("cd %s", outdir);
		script = String.format(scriptTemplate, String.format("%s/%s", outdir, joblabel),
				String.format("%s/%s", outdir, joblabel), cddir, Main.getAppDir(), task, p.getPipelineOptions());
		this.scriptName = outdir + "/" + joblabel + ".sh";
		try (Writer writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(scriptName), "utf-8"))) {
			writer.write(script);
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public CmdProcessData(PipelineOptions p, String joblabel) {
		this(p, joblabel, false);
	}

	public CmdProcessData(PipelineOptions p) {
		this(p, "reprocess", false);
	}

	@Override
	public void run() {
		Log.note("Run processing job: " + this.scriptName);
		String cmd; 
		if (pipelineOptions.getPipeline().startsWith("kamo_") || block) {
			cmd = "bash " + scriptName;
		} else {
			cmd = "sbatch " + this.scriptName;
			ExecTask.syncExec("chmod 755 " + this.scriptName, "chmod", false);
		}

		ExecTask task = ExecTask.new_ExecTask(cmd, "autoprocess", false);
		
		task.start();
		if (block) {
			System.out.print("wait for result ..."+joblabel);
			task.waitFor();
		}
		
	}

}
