package gov.anl.gmca.analysis;

import gov.anl.gmca.analysis.helpers.MiscUtils;
import gov.anl.gmca.analysis.helpers.ValidateUserInput;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.syslog.Log;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;

public class DB2Html {
	public enum RESULT_PAGE {
		PROCESSING_RESULTS("Summary of Processing Results"), SCREENING_RESULTS("Summary of Screening Results"), URL(
				"generic url");

		private String pageTitle;

		private RESULT_PAGE(String s) {
			pageTitle = s;
		}

		public String getPageTitle() {
			return pageTitle;
		}
	};
		
	private String statesQuery;
	private String resultsQuery;
	private RESULT_PAGE page;
	private String htmlConfig;
	private Result result;
	private String thead="";
	private String tfoot="";
	
	protected final String blankPage = "<div style=\"text-align:center\">\n" 
										+ "  <font color=\"#e0ebeb\">\n"
										+ "<p style=\"font-size:60px\">\n" 
										+ "Data analysis interface\n" 
										+ "</p>\n" 
										+ "</font>\n"
										+ "  <font color=\"#ff9999\"> <p style=\"font-size:50px\"> Ctrl-H for help. Ctrl-X to start process GUI. </p></font>\n"
										+ "</div>\n";

	// use statesQuery for monitoring changes in states
	public DB2Html(RESULT_PAGE page) {
		this.page = page;

		// gmca user can see everything
		String username = System.getProperty("user.name");
		String u1 = String.format(" AND username='%s' ", username);
		String u2 = String.format(" AND p.username='%s' ", username);
		String dateRange = " AND DATE(p.starttime) >= CURDATE() - INTERVAL 3 DAY "; // show results within 10 days
		if (page == RESULT_PAGE.PROCESSING_RESULTS) {
			statesQuery = "/*dataproc NOPRINT*/ select state, count(*) from pipelinestatus p "
					+ " where pipeline in "
					+ "('autoproc', 'gmcaproc', "
					+ "'xia2_dials', 'xia2_xds', 'xia2_dials_aimless', "
					+ "'fastdp', 'kamo_xds', 'kamo_dials')"
					+ u1
					+ dateRange
					+ " group by state order by state;";
			
			resultsQuery = "/*dataproc*/ SELECT p.id, "
					+ "p.samplename as name, "
					+ "p.pipeline, "
					+ "p.imageset, "
					+ "p.state,  "
					+ "d.report_url as Summary, "
					+ "d.wavelength as wav, "
					+ "d.spacegroup as Symm, "
					+ "d.unitcell as Cell, "
					+ "d.highresolution as h_res, " 
					+ "d.rmerge as Rsym, "
					+ "d.rmeas as Rmeas, "
					+ "d.rpim as Rpim, "
					+ "d.isigmai as IsigI, "
					+ "d.isigmai_outer as IsigIo, "
					+ "d.multiplicity as multi, "
					+ "d.completeness as Cmpl,  " 
					+ "d.anom_completeness as a_Cmpl, "
					+ "p.warning, "
					+ "p.logfile, "
					+ "d.table1, " 
					+ "p.elapsedtime, "
					+ "p.imagedir, "
					+ "d.firstframe, "
					+ "d.workdir, "
					+ "d.scale_log, "
					+ "d.truncate_log, "
					+ "d.truncate_mtz, "
					+ "d.run_stats, "
					+ "d.reprocess, "
					+ "d.solve, "
					+ "p.delete  "
					+ "FROM pipelinestatus p left join dataprocessresults d "
					+ "on p.id=d.pipelinestatus_id where p.pipeline in "
					+ "('autoproc', 'gmcaproc', "
					+ "'xia2_dials', 'xia2_xds', 'xia2_dials_aimless', "
					+ "'fastdp', 'kamo_xds', 'kamo_dials') " 
					+ u2
					+ dateRange
					+ "order by p.id desc, d.id desc "
					+ "limit 1200;";

			htmlConfig ="DataProcResults.html";
		} else if (page == RESULT_PAGE.SCREENING_RESULTS) {
			statesQuery = "/*screening NOPRINT*/ select state, count(*) from pipelinestatus p "
					+ " where pipeline in ('mosflm_strategy', 'labelit_strategy', 'dials_strategy', 'xds_strategy')"
					+ u1
					+ dateRange
					+ " group by state order by state;";
			
			resultsQuery = "/*screening*/ SELECT p.id, "
					+ "p.samplename as name, "
					+ "p.pipeline, "
					+ "p.imageset, " 
					+ "p.state,  "
					+ "d.export2run as exp_strategy, "
					+ "d.bravais_lattice as lattice, "
					+ "d.unitcell as Cell, "
					+ "d.spacegroup as Symm, "
					+ "d.resolution_from_spots as h_res, "
					+ "d.mosaicity, "
					+ "d.rmsd, "
					+ "d.score, " 
					+ "d.n_spots, "
					+ "d.osc_start, "
					+ "d.osc_end, "
					+ "d.osc_delta, "
					+ "d.detectordistance as distance, "
					+ "d.completeness_native as cmpl, "
					+ "d.completeness_anomalous as a_cmpl, "
					+ "d.estimated_asu_content_aa as asu_aa, "
					+ "d.index_table, "
					+ "d.xplanlog, "
					+ "d.solvent_content, "
					+ "p.warning, "
					+ "p.logfile, "
					+ "p.elapsedtime, "
					+ "p.imagedir, "
					+ "d.workdir, "
					+ "d.reprocess, "
					+ "p.delete, "
					+ "d.pointgroup_choices as userChoose, "
					+ "d.anomalous, "
					+ "d.referencedata "
					+ "FROM pipelinestatus p left join screenstrategyresults d "
					+ "on p.id=d.pipelinestatus_id where p.pipeline in "
					+ "('mosflm_strategy', 'labelit_strategy', 'dials_strategy', 'xds_strategy') " 
					+ u2
					+ dateRange
					+ "order by p.id desc "
					+ "limit 1200;";

			htmlConfig ="ScreeningResults.html";
		} else {
			// not implemented page mode
			statesQuery = null;
		}
	}
	
	public Result getStates() {
		long startTime = System.currentTimeMillis();
		if (statesQuery == null) {
			return null;
		}
		Result r = Database.exec(DBName.USERDATA, statesQuery);
		long endTime   = System.currentTimeMillis();
		long totalTime = endTime - startTime;
		if (totalTime > 500) {
			Log.warn("Data processing: query state " + totalTime + " ms.");
		}
		return r;
	}
	
	public String html() {
		return generateHtmlWithDataAsJson();
	}

	public Result getResults() {
		long startTime = System.currentTimeMillis();

		result = Database.exec(DBName.USERDATA, resultsQuery);
		long endTime   = System.currentTimeMillis();

		long totalTime = endTime - startTime;
		if (totalTime > 500) {
			Log.warn("Data processing: query data processing results table " + totalTime + " ms.");
		}
		return result;
	}
	
	// construct html file with data as json object instead of html table, to test performance
	private String generateHtmlWithDataAsJson() {
		getResults();

		if (!result.isOK()) {
			Log.warn("failed to query database, tables may not exist.");
			return blankPage;
		}
		
		updateResultInPlace();
		
		String template = getHeader();
		String columns = jsonColumnLabel();
		String jsonArray = MiscUtils.result2json(result, null);

		boolean debug = false;
		if (debug) {
			try {
				String filename = System.getProperty("user.home") + "/" + page.getPageTitle().replaceAll(" ", "") + ".html";
				System.out.println(filename);
				FileWriter fw = new FileWriter(filename);
				fw.write(template.replace("<DATA_DEFINITION>", jsonArray).replace("<COLUMN_DEFINITION>", columns));
				fw.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		String html = template.replace("<COLUMN_DEFINITION>", columns)
								.replace("<DATA_DEFINITION>", jsonArray)
								.replace("<THEAD>", thead)
								.replace("<TFOOT>", tfoot);
		
		if (result.numRows() == 0) {
			html += blankPage;
		}
		return html;
	}

	private String jsonColumnLabel() {
		if (result.numRows() == 0 || !result.isOK()) {
			return "[]"; 
		}
		
		Map<String, String> row0 = result.getRow(0);
		StringBuilder sb = new StringBuilder(); 
		sb.append("[\n");
		StringBuilder columns = new StringBuilder();
		for (String columnLabel: row0.keySet()) {
			String title = columnLabel; // StringUtils.capitalize(columnLabel); // capitalize header/footer
			sb.append("{data: \"");
			sb.append(columnLabel);
			sb.append("\", title: \"");
			sb.append(title);
			sb.append("\"},\n");
			columns.append(String.format("<td><b>%s</b></td>", title));
		}
		sb.append("]");
		thead = ""; //"<thead><tr>" + columns + "</tr></thead>";
		tfoot = "<tfoot><tr>" + columns.toString() + "</tr></tfoot>";
		return sb.toString();
	}
	
	// utility function showing either plain text or html text of a string
	public static String toHtmlButton(String text) {
		return String.format("<button class='dt-button tooltip'>%s</button>", text);
	}
	
	// create a link of button of html link
	/*
	public static String toButtonOfHtmlLink(String link, String linkTitle, String linkText, String buttonTitle) {
		String aref = String.format("<a href='%s' title='%s' target='_blank'>%s</a>", link, linkTitle, linkText);
		String btn = String.format("<button class='dt-button tooltip', title='%s'>%s</button>", buttonTitle, aref);
		return btn;
	}
	 */
	
	// create a link of a html button 
	public static String toLinkOfHtmlButton(String link, String linkTitle, String linkText, String buttonTitle) {
		String btn = String.format("<button class='dt-button tooltip', title='%s'>%s</button>", buttonTitle, linkText);
		String aref = String.format("<a href='%s' title='%s' target='_blank'>%s</a>", link, linkTitle, btn);
		return aref;
	}
	
	private String UserSelectChoices(Map<String, String> row) {
		if (row.get("referencedata") == null 
				|| row.get("referencedata").equals("NA")
				|| row.get("referencedata").equals("")) {
			return htmlSelectPointGroupChoices(row);
		} 
		return htmlSelectXplanlog(row); // multiple strategy
	}

	// in some cases sampleName does not match prefix, get it from imageSet
	private String getPrefix(String imageSet, String sampleName) {
		String[] fields = imageSet.split(":");
		if (fields.length <= 1) return sampleName;
		return fields[fields.length - 1];
	}

	private String htmlSelectPointGroupChoices(Map<String, String> row) {
		String id = row.get("id");
		String sampleName = row.get("name");
		String imageSet = row.get("imageset");
		String pipeline = row.get("pipeline");
		String anomalous = row.get("anomalous");
		String imgdir = row.get("imagedir");
		String workdir = row.get("workdir");
		String defaultSpg = row.get("Symm");

		String prefix = getPrefix(imageSet, sampleName);

		String[] opts = row.get("userChoose").split(";");
		StringBuilder sb = new StringBuilder();

		sb.append("<select name='form' onchange='location = this.value;'>");
		sb.append("<option selected disabled>Choose an alternative space group...</option>");
		for (String opt: opts) {
			String spg = opt.substring(0, opt.indexOf(" "));
			String value = String.format("strategy2:%s:%s:%s:%s:%s:%s:%s:%s", id, prefix, pipeline, imgdir, workdir,
					anomalous, spg, defaultSpg);
			sb.append(String.format("<option value='%s'>%s</option>", value, opt));
		}
		sb.append("</select>");
		return sb.toString();
	}

	// convert xplanlog to select options
	private String htmlSelectXplanlog(Map<String, String> row) {
		String id = row.get("id");
		String xplan = row.get("xplanlog").replaceAll("<pre>", "").replaceAll("</pre>", "");
		String[] opts = xplan.split("\n");

		StringBuilder sb = new StringBuilder();

		sb.append("<select name='form' onchange='location = this.value;'>");
		sb.append("<option selected disabled>Choose a multi-crystal strategy...</option>");
		for (String opt: opts) {
			String[] fields = opt.split("\\s+");
			if (!opt.contains("start_angle") && fields.length == 9 ) {
				String startOsc = fields[1];
				String totalOsc = fields[2];
				Double endOsc = Double.valueOf(startOsc) + Double.valueOf(totalOsc); 
				String startCmpl = fields[5];
				String endCmpl = fields[8];
				String value = String.format("export:%s:%s:%.1f", id, startOsc, endOsc);
				String textOpt = String.format("rot: %sdeg., start angle=%s, compl=%s%% => end cmpl=%s", totalOsc, startOsc, startCmpl, endCmpl);
				sb.append(String.format("<option value='%s'>%s</option>", value, textOpt));
			}
		}
		sb.append("</select>");
		return sb.toString();
	}
	
	// this is probably best done on python side, 
	private void updateResultInPlace() {
		for (Map<String, String> row: result.getRows()) {
			Map<String, String> rowOriginal = new HashMap<String, String>(row);
			for (Map.Entry<String, String> e : row.entrySet()) {
				String columnLabel = e.getKey();
				String fld = e.getValue();
				if (fld == null || fld.equals("") || fld.equals("None")) {
					fld = "NA";
					row.put(columnLabel, fld);
				}
				
				if (columnLabel.equals("reprocess")) {
					if (page == RESULT_PAGE.SCREENING_RESULTS) {
						row.put(columnLabel, String.format("<a href='strategy:%s' target='strategy'>%s</a>", row.get("id"), toHtmlButton("Start")));
					} else {
						row.put(columnLabel, String.format("<a href='process:%s' target='process'>%s</a>", row.get("id"), toHtmlButton("Start")));
					}                                
				} else if (columnLabel.contains("userChoose") && !fld.equals("NA")) { 
					String select = UserSelectChoices(rowOriginal);
					row.put(columnLabel, select);
				} else if (columnLabel.contains("export2run")||columnLabel.contains("export")||columnLabel.contains("exp_strategy")) {
					if (ValidateUserInput.isNumeric(row.get("osc_start"))) {
						String hintText = String.format("<pre>Recommended strategy: symm=%s, start=%s, end=%s, delta=%s, dist=%smm, cmpl=%s%% a_compl=%s%%</pre>", 
								row.get("Symm"), 
								row.get("osc_start"), 
								row.get("osc_end"),
								row.get("osc_delta"),
								row.get("distance"),
								row.get("cmpl"),
								row.get("a_cmpl"));
						String btn1 = String.format("<a href='export:%s:%s:%s:%s:%s' target='export'>%s</a>", 
								row.get("id"), 
								row.get("osc_start"), 
								row.get("osc_end"), 
								row.get("osc_delta"), 
								row.get("distance"),
								toHtmlButton("NewRun"));
						String btn2 = String.format("<a href='strategy:%s' target='strategy'>%s</a>", row.get("id"), toHtmlButton("Reproc"));
						String btnsWithHint= String.format("<div class='tooltip' id='%s' title='%s'>%s</div>", fld, 
								hintText, 
								new StringBuilder().append(btn1).append(btn2).toString());
						row.put(columnLabel, btnsWithHint);
					} else {
						row.put(columnLabel, "NA");
					}
				} else if (columnLabel.contains("delete")) {
					row.put(columnLabel, String.format("<a href='delete:%s' target='_parent'>%s</a>", row.get("id"), toHtmlButton("Delete row")));
				} else if (columnLabel.contains("table1") 
						|| columnLabel.equals("log")
						|| columnLabel.equals("xplanlog") 
						|| columnLabel.contains("index_table") 
						|| columnLabel.contains("warning") 
						|| columnLabel.contains("solvent_content")
						) {
					String tagText = String.format("<pre>%s</pre>", fld.replaceAll("<", "&lt;").replaceAll(">", "&gt;"));
					row.put(columnLabel, tagText);
				} else if (columnLabel.contains("state")) { 
					// if no table1, display warning instead
					String warning = row.get("warning");
					warning = String.format("<pre>%s</pre>", warning.replaceAll("<", "&lt;").replaceAll(">", "&gt;")
							.replaceAll("'", "&#39;").replaceAll("\"", "&#34;"));
					row.put(columnLabel,
							String.format("<div class='tooltip' id='%s' title='%s'>%s</div>", fld, warning, fld));
				} else if ( (columnLabel.contains("report_url") || columnLabel.toLowerCase().contains("summary"))
						&& !fld.equals("NA")) {
					String btn3 = String.format("<a href='process:%s' target='process' title='click to reprocess data'>%s</a>", row.get("id"), toHtmlButton("Reproc"));
					
					if (new File(fld.replace("file://", "")).exists()) {
						String scale_log = row.get("scale_log");

						String btn1 = toLinkOfHtmlButton(fld, fld, "Overview", "click to open detailed graphical pipeline processing summary");
						
						String table1 = row.get("table1");
						if (table1 != null) { // show hint if table1 is available
							table1 = String.format("<pre>%s</pre>", table1.replaceAll("<", "&lt;").replaceAll(">", "&gt;"));
						} else {
							table1 = "table1 not available.";
						}

						String btn2 = toLinkOfHtmlButton(scale_log, "click to open scaling/aimless log", "Scaling", table1);
						
						String mapBtn = "";
						String solveDir = row.get("solve");
						String cootSh = String.format("%s/%s", solveDir, "coot.sh");
						if (new File(cootSh).exists()) {
							mapBtn = toLinkOfHtmlButton("file://"+cootSh, cootSh, "Map", "launch COOT to view map");
						}
						
						row.put(columnLabel, new StringBuilder().append(btn1).append(btn2).append(mapBtn).append(btn3).toString());
					} else {
						row.put(columnLabel, btn3);
					}
				} else if (columnLabel.contains("logfile") && !fld.equals("NA")) {
					row.put(columnLabel, String.format("<a href='%s' title='%s' target='_blank'>%s</a>", fld, fld, toHtmlButton("Open")));
				} else if (columnLabel.contains("imageset")) {
					String txt2Show = fld;
					if (fld.length() >= 20) {
						txt2Show = fld.substring(0,20) + "...";
					}
					row.put(columnLabel, String.format("<div id='%s' title='%s'>%s</div>", txt2Show, fld, txt2Show));
				} else if (columnLabel.contains("imagedir")) {
					if (!fld.equals("NA")) {
						row.put(columnLabel, String.format("<a href=file://%s title='%s' target='_blank'>%s</a>", fld, fld, toHtmlButton("Open")));
					}
				} else if (columnLabel.contains("truncate_log") 
						|| columnLabel.contains("scale_log")
						|| columnLabel.contains("solve")
						|| columnLabel.contains("run_stats"))  {
					if (!fld.equals("NA")) {
						File f = new File(fld);
						if (f.exists()) {
							row.put(columnLabel, String.format("<a href=file://%s title='%s' target='_blank'>%s</a>", fld, fld, toHtmlButton("Open")));
						}
						
						if (columnLabel.contains("scale_log") && row.get("scale_log").contains("/scale-2/")) {
							row.put("pipeline", "gmcaproc-refi");
						}
					}
				} else if (columnLabel.contains("truncate_mtz")) {
					if (!fld.equals("NA")) {
						File f = new File(fld);
						if (f.exists()) {
							row.put(columnLabel, String.format("<a href=file://%s title='%s' target='_blank'>%s</a>", fld, fld, toHtmlButton("Download")));
						}
					}
				} else if (columnLabel.contains("referencedata")) {
					if (!fld.equals("NA")) {
						row.put("pipeline", String.format("%s%s", row.get("pipeline"), "(MCS)"));
					}
				} else if (columnLabel.contains("workdir")) {
					String workdir = fld;
					if (workdir.equals("NA")) {
						String query = String.format("select workdir from pipelinestatus where id=%s;", row.get("id"));
						Result r = Database.exec(DBName.USERDATA, query);
						if (r.numRows() == 1) {
							workdir = r.getRow(0).get("workdir");
						}
 					}
					if (!workdir.equals("NA")) {
						row.put(columnLabel, String.format("<a href=file://%s title='%s' target='_blank'>%s</a>", workdir, workdir, toHtmlButton("Open")));
					}
				}
			}
		}
	}

	// TODO need to refactor this so that the templates can be created only once for all
	private String getHeader() {
		String config = Main.getAppDir() + "/java/src/gov/anl/gmca/analysis/" + this.htmlConfig;
		URL url = DB2Html.class.getResource(this.htmlConfig);
		String header = Main.getAppDir() + "/java/src/gov/anl/gmca/analysis/common.html";
		if (url != null) {
			config = url.getFile();
		}

		if (new File(config).exists()) {
			try {
				String headerHtml = MiscUtils.readFile(header, Charset.defaultCharset());
				return MiscUtils.readFile(config, Charset.defaultCharset()).replace("<COMMON_HTML>", headerHtml);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		Log.warn("config html file does not exist: " + config);
		return "";

	}

}
