package gov.anl.gmca.analysis;

import gov.anl.gmca.analysis.helpers.MiscUtils;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.widgets.Shell;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;

public class KAMO {
	String ImageDirectory = "";
	String OutputDirectory = "";
	String beamline = Util.getBID();
	String sampleName="";
	String sampleNumber="0";
	String pipeline = "";
	Shell shell;
	boolean canRun = false;
	boolean alive = false;
	Thread t;
	
	public KAMO(Shell inshell) {
		ImageDirectory = CTabBrowser.getPrimaryDir();
		
		File datasetsTxt = new File(ImageDirectory + "/datasets.txt");
		if (!datasetsTxt.exists() || !new File(ImageDirectory).canWrite()) {
			String msg = "KAMO (live) expects a file named datasets.txt written by jBluIce in directory: " + ImageDirectory + ", collect some data first.";
			WdgDataProcDialog.NBDisplayMessage(shell, msg);
			canRun = false;
			return;
		}
		
		OutputDirectory = MiscUtils.getProcessDir(ImageDirectory, MiscUtils.analysisNGSubDirectory);
		beamline = Util.getBID();
		sampleName="KAMO";
		sampleNumber="0";
		shell = inshell;
		pipeline = "kamo_xds";
		canRun = true;
	}
	
	public void run() {
		if (!canRun) {
			return;
		}

		if (OutputDirectory == null || OutputDirectory.isEmpty()) {
			OutputDirectory = ImageDirectory + "/process";
		}
		File of = new File(OutputDirectory);
		if (!of.exists()) {
			of.mkdirs();
		}

		PipelineOptions p = new PipelineOptions.PipelineBuilder(ImageDirectory, OutputDirectory)
				.sampleName(sampleName)
				.sampleNumber(sampleNumber)
				.anomalous(false)
				.smallWedges(true)
				.pipeline(pipeline)
				.njobs("1")
				.nproc("16")
				.runner(autoProcessData.RUNNER)
				.beamline(beamline)
				.build();

		SimpleDateFormat datetimeFormatter = new SimpleDateFormat("yyyy-MM-dd-hh-mm-ss");
		String strDate = datetimeFormatter.format(new Date());
		String joblabel = String.format("%s-%s-%s", "process", pipeline, strDate);
		t = new Thread(new CmdProcessData(p, joblabel)); 
		t.start();
		Log.note(String.format("KAMO-live using XDS was launched in %s", OutputDirectory));
		
		String msg = "KAMO XDS started. Please wait until KAMO interface to show up, no need to click the button again.";
		WdgDataProcDialog.NBDisplayMessage(shell, msg);
	}
}
