package gov.anl.gmca.analysis;

import gov.anl.gmca.analysis.helpers.ValidateUserInput;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

// Build pipeline options, builder pattern
// only the toString method of the main class was written by me
// the rest was auto generated by Lombok 
public class PipelineOptions {
	private final String sampleName;
	private final String sampleNumber;
	private final String ImageDirectory;
	private final String start;
	private final String end;
	private final String percent;
	private final String imageSet;
	private final String filenamePrefix;
	private final String pipeline;
	private final String OutputDirectory;
	private final String spacegroup;
	private final String unitcell;
	private final String highResolutionCutoff;
	private final Boolean anomalous;
	private final Boolean smallWedges;
	private final Boolean recursive;
	private final String excludeDirs;
	private final String includeDirs;
	private final String XDScmds;
	private final String refhkl;
	private final String njobs;
	private final String nproc;
	private final String filelist;
	private final String runner;
	private final String beamline;
	private final String xlsFile;
	private final String mountedCrystal;
	private final String timeout;
	
	public static class PipelineBuilder {
		private String sampleName;
		private String sampleNumber;
		private final String ImageDirectory;
		private String start;
		private String end;
		private String percent;
		private String imageSet;
		private String filenamePrefix;
		private String pipeline;
		private final String OutputDirectory;
		private String spacegroup;
		private String unitcell;
		private String highResolutionCutoff;
		private Boolean anomalous;
		private Boolean smallWedges;
		private Boolean recursive;
		private String excludeDirs;
		private String includeDirs;
		private String XDScmds;
		private String refhkl;
		private String njobs;
		private String nproc;
		private String filelist;
		private String runner;
		private String beamline;
		private String xlsFile;
		private String mountedCrystal;
		private String timeout;

		// required parameter 
		public PipelineBuilder(String ImageDirectory, String OutputDirectory) {
			this.ImageDirectory = ImageDirectory;
			this.OutputDirectory = OutputDirectory;
		}
		
		public PipelineBuilder sampleName(final String sampleName) {
			this.sampleName = sampleName;
			return this;
		}

		public PipelineBuilder sampleNumber(final String sampleNumber) {
			this.sampleNumber = sampleNumber;
			return this;
		}

		public PipelineBuilder start(final String start) {
			this.start = start;
			return this;
		}

		public PipelineBuilder end(final String end) {
			this.end = end;
			return this;
		}

		public PipelineBuilder percent(final String percent) {
			this.percent = percent;
			return this;
		}
		
		public PipelineBuilder imageSet(final String imageSet) {
			this.imageSet = imageSet;
			return this;
		}
		
		public PipelineBuilder filenamePrefix(final String filenamePrefix) {
			this.filenamePrefix = filenamePrefix;
			return this;
		}

		public PipelineBuilder pipeline(final String pipeline) {
			this.pipeline = pipeline;
			return this;
		}

		public PipelineBuilder spacegroup(final String spacegroup) {
			this.spacegroup = spacegroup;
			return this;
		}

		public PipelineBuilder unitcell(final String unitcell) {
			this.unitcell = unitcell;
			return this;
		}

		public PipelineBuilder highResolutionCutoff(final String highResolutionCutoff) {
			this.highResolutionCutoff = highResolutionCutoff;
			return this;
		}

		public PipelineBuilder anomalous(final boolean anomalous) {
			this.anomalous = anomalous;
			return this;
		}
		
		public PipelineBuilder smallWedges(final boolean smallWedges) {
			this.smallWedges = smallWedges;
			return this;
		}		

		public PipelineBuilder recursive(final boolean recursive) {
			this.recursive = recursive;
			return this;
		}

		public PipelineBuilder excludeDirs(final String excludeDirs) {
			this.excludeDirs = excludeDirs;
			return this;
		}

		public PipelineBuilder includeDirs(final String includeDirs) {
			this.includeDirs = includeDirs;
			return this;
		}

		public PipelineBuilder xdscmds(final String XDScmds) {
			this.XDScmds = XDScmds;
			return this;
		}
		
		public PipelineBuilder refhkl(final String refhkl) {
			this.refhkl = refhkl;
			return this;
		}

		public PipelineBuilder njobs(final String njobs) {
			this.njobs = njobs;
			return this;
		}

		public PipelineBuilder nproc(final String nproc) {
			this.nproc = nproc;
			return this;
		}

		public PipelineBuilder filelist(final String filelist) {
			this.filelist = filelist;
			return this;
		}

		public PipelineBuilder runner(final String runner) {
			this.runner = runner;
			return this;
		}
		
		public PipelineBuilder beamline(final String beamline) {
			this.beamline = beamline;
			return this;
		}
		
		public PipelineBuilder xlsFile(final String xlsFile) {
			this.xlsFile = xlsFile;
			return this;
		}

		public PipelineBuilder mountedCrystal(final String mountedCrystal) {
			this.mountedCrystal = mountedCrystal;
			return this;
		}

		public PipelineBuilder timeout(final String timeout) {
			this.timeout = timeout;
			return this;
		}
		
		public PipelineOptions build() {
			return new PipelineOptions(this);
		}

		public java.lang.String toString() {
			return "Pipeline.PipelineBuilder(sampleName=" + this.sampleName + ", sampleNumber=" + this.sampleNumber
					+ ", ImageDirectory=" + this.ImageDirectory + ", start=" + this.start 
					+ ", end=" + this.end 
					+ ", percent=" + this.percent
					+ ", imageSet=" + this.imageSet 
					+ ", filenamePrefix=" + this.filenamePrefix + ", pipeline=" + this.pipeline + ", OutputDirectory="
					+ this.OutputDirectory + ", spacegroup=" + this.spacegroup + ", unitcell=" + this.unitcell
					+ ", highResolutionCutoff=" + this.highResolutionCutoff + ", anomalous=" + this.anomalous
					+ ", refhkl=" + this.refhkl + ", njobs=" + this.njobs + ", nproc=" + this.nproc + ", filelist="
					+ this.filelist + ", runner=" + this.runner + ", beamline=" + this.beamline + ")";
		}
	}

	private PipelineOptions(PipelineBuilder b) {
		this.sampleName = b.sampleName;
		this.sampleNumber = b.sampleNumber;
		this.ImageDirectory = b.ImageDirectory;
		this.start = b.start;
		this.end = b.end;
		this.percent = b.percent;
		this.imageSet = b.imageSet;
		this.filenamePrefix = b.filenamePrefix;
		this.pipeline = b.pipeline;
		this.OutputDirectory = b.OutputDirectory;
		this.spacegroup = b.spacegroup;
		this.unitcell = b.unitcell;
		this.highResolutionCutoff = b.highResolutionCutoff;
		this.anomalous = b.anomalous;
		this.smallWedges = b.smallWedges;
		this.recursive = b.recursive;
		this.includeDirs = b.includeDirs;
		this.excludeDirs = b.excludeDirs;
		this.XDScmds = b.XDScmds;
		this.refhkl = b.refhkl;
		this.njobs = b.njobs;
		this.nproc = b.nproc;
		this.filelist = b.filelist;
		this.runner = b.runner;
		this.beamline = b.beamline;
		this.xlsFile = b.xlsFile;
		this.mountedCrystal = b.mountedCrystal;
		this.timeout = b.timeout;
	}

	@Override
	public String toString() {
		List<String> pipelineOptions = new ArrayList<String>();
		pipelineOptions.add("--log2db");
		//pipelineOptions.add("--disable_console_logger");

		if (this.sampleName != null && !this.sampleName.isEmpty()) {
			pipelineOptions.add("--sampleName="+this.sampleName);
		}

		if (this.sampleNumber != null && !this.sampleNumber.isEmpty()) {
			pipelineOptions.add("--sampleNumber="+this.sampleNumber);
		}
		
		pipelineOptions.add("--data_dir=" + this.ImageDirectory);
		pipelineOptions.add("--proc_dir="+this.OutputDirectory);
		
		if (this.start != null && !this.start.isEmpty()) {
			pipelineOptions.add("--start="+this.start);
		}
		
		if (this.end != null && !this.end.isEmpty()) {
			if (ValidateUserInput.isPercentage(this.end)) {
				pipelineOptions.add("--percent="+ ValidateUserInput.getPercentage(this.end));
			} else {
				pipelineOptions.add("--end="+this.end);
			}
		}

		if (this.percent != null && !this.percent.isEmpty()) {
			pipelineOptions.add("--percent="+this.percent);
		}
		
		if (this.imageSet != null && !this.imageSet.isEmpty()) {
			pipelineOptions.add("--imageSet="+this.imageSet);
		}
		
		if (this.filenamePrefix != null && !this.filenamePrefix.isEmpty()) {
			//pipelineOptions.add("--prefix=" + "\"" + this.filenamePrefix + "\"");
			pipelineOptions.add(String.format("--prefix='%s'", this.filenamePrefix));
		}
		
		if (this.pipeline.isEmpty()) {
			pipelineOptions.add("--pipeline=gmcaproc");
		} else if (pipeline.equals("gmcaproc w optimization")) {
			pipelineOptions.add("--pipeline=gmcaproc --gmca_optimize");
		} else if (pipeline.contains("generate XDS.INP only")) {
			pipelineOptions.add("--xdsinp");
		} else {
			pipelineOptions.add("--pipeline=" + this.pipeline);
		}		
		
		if (this.spacegroup != null && !this.spacegroup.isEmpty()) {
			pipelineOptions.add("--space_group="+this.spacegroup);
			if (this.unitcell != null && !this.unitcell.isEmpty()) {
				pipelineOptions.add("--unit_cell=" + "\"" + this.unitcell + "\"");
			}
		}

		if (this.highResolutionCutoff!=null && !this.highResolutionCutoff.isEmpty()) {
			//pipelineOptions.add("--highres="+this.highResolutionCutoff);
			pipelineOptions.add(String.format("--highres=\"%s\"",this.highResolutionCutoff));
		}
		
		if (this.anomalous != null && !this.anomalous) {
			pipelineOptions.add("--native");
		}
		
		if (this.smallWedges != null && this.smallWedges) {
			pipelineOptions.add("--small_wedges");
		}
		
		if (this.recursive != null && this.recursive) {
			pipelineOptions.add("--recursive");
		}
		
		if (this.includeDirs!=null && !this.includeDirs.isEmpty() && this.includeDirs.length() >= 1) {
			pipelineOptions.add(String.format("--include_dirs=\"%s\"",this.includeDirs.replaceAll(",", ";")));
		}

		if (this.excludeDirs!=null && !this.excludeDirs.isEmpty() && this.excludeDirs.length() >= 1) {
			pipelineOptions.add(String.format("--exclude_dirs=\"%s\"",this.excludeDirs.replaceAll(",", ";")));
		}

		if (this.refhkl!=null && !this.refhkl.isEmpty()) {
			File hkl = new File(this.refhkl);
			if (hkl.exists() && (this.refhkl.endsWith(".HKL") 
					|| this.refhkl.endsWith(".ahkl") 
					|| this.refhkl.endsWith(".SCA") 
					|| this.refhkl.endsWith(".sca"))) {
				pipelineOptions.add("--xds_refhkl="+this.refhkl);
			} else {
				throw new IllegalArgumentException("make sure the provided reference hkl exists and valid: " + this.refhkl);
			}
		}

		if (this.filelist!=null && !this.filelist.isEmpty()) {
			File flist = new File(this.filelist);
			if (flist.exists()) {
				pipelineOptions.add("--filelist=" + this.filelist);
			} else {
				throw new IllegalArgumentException("make sure the provided reference hkl exists and valid: " + refhkl);
			}
		}
		
		if (this.runner!=null && !this.runner.isEmpty()) {
			pipelineOptions.add("--runner=" + this.runner);
		}
		
		if (this.njobs!=null && !this.njobs.isEmpty()) {
			pipelineOptions.add("--njobs=" + this.njobs);
		}
		
		if (this.nproc!=null && !this.nproc.isEmpty()) {
			pipelineOptions.add("--nproc=" + this.nproc);
		}
		
		if (this.XDScmds!=null && !this.XDScmds.isEmpty() && this.XDScmds.length() > 10) {
			pipelineOptions.add(String.format("--xdscmds=\"%s\"",this.XDScmds));
		}
	
		if (this.beamline!=null && !this.beamline.isEmpty()) {
			pipelineOptions.add("--beamline=" + this.beamline);
		}
		
		if (this.xlsFile!=null && !this.xlsFile.isEmpty()) {
			//pipelineOptions.add("--xlsFile=" + "\"" + this.xlsFile + "\"");
			pipelineOptions.add(String.format("--xlsFile='%s'", this.xlsFile));
			//pipelineOptions.add(String.format("--xlsFile=\"%s\"", this.xlsFile));
		}

		if (this.mountedCrystal!=null && !this.mountedCrystal.isEmpty()) {
			pipelineOptions.add("--mountedCrystal=" + this.mountedCrystal);
		}
		
		if (this.timeout!=null && !this.timeout.isEmpty()) {
			pipelineOptions.add("--timeout=" + this.timeout);
		}
		
		return String.join(" ", pipelineOptions);
	}
	
	public String getPipelineOptions() {
		return this.toString();
	}
	
	public String getOutputDirectory() {
		return this.OutputDirectory;
	}
	
	public String getPipeline() {
		return this.pipeline;
	}
}
