package gov.anl.gmca.analysis.helpers;

import java.io.File;
import java.io.IOException;
import java.nio.file.*;
import java.util.*;


// given a directory, find image prefixes
public class FindImagePrefixes {
	public Integer nImages = 0;
	public static String[] VALID_IMAGEFILE_SUFFIXES = { ".cbf", ".h5" };

	String directory;
	ArrayList<String> imageFileNames;

	public FindImagePrefixes(String directory) {
		this.directory = directory;
		this.imageFileNames = ImageFileNames();
	}
	
	public ArrayList<String> getImageFileNames() {
		return this.imageFileNames;
	}

	public SortedSet<String> getPrefixes() {
		ArrayList<String> listOfImageFiles = this.imageFileNames;
		if (listOfImageFiles == null || listOfImageFiles.isEmpty()) {
			return null;
		}
		
		SortedSet<String> setOfPrefixes = new TreeSet<>();
		for (String fn : listOfImageFiles) {
			int lastIndex = fn.lastIndexOf('_');
			if (lastIndex > 0) {
				setOfPrefixes.add(fn.substring(0, fn.lastIndexOf('_')));
			}
		}
		
		/*
		for (String fn : listOfImageFiles) {
			int lastIndex = fn.lastIndexOf('.');
			if (lastIndex > 0) {
				setOfPrefixes.add(fn.substring(0, fn.lastIndexOf('.')));
			}
		}
		*/
		
		return setOfPrefixes;
	}

	public String getPrefixHints() {
		String prefixHint = "Warning: no images in the current directory...";
		if (getPrefixes()!= null && getPrefixes().size() > 0) {
			prefixHint = String.join(";", getPrefixes());
		}
		return prefixHint;
	}

	public String getImageListHints() {
		String imageListHint = "Warning: no images in the current directory...";
		ArrayList<String> fl = this.imageFileNames;
		if (fl!= null && fl.size() > 0) {
			if (fl.size() > 10) {
				// took first and last 5 image and show it
				imageListHint = String.join("\n", fl.subList(0, 5));
				imageListHint += "\n......\n";
				imageListHint += String.join("\n", fl.subList(fl.size()-5, fl.size()));
			} else {
				imageListHint = String.join("\n", fl.subList(0, fl.size()));
			}
			imageListHint += "\n\nTotal No. of images: " + fl.size();
		}		
		return imageListHint;
	}
	
	// works ok, but very slower
	private ArrayList<String> ImageFileNames_SLOW() {
		if (directory == null || directory.isEmpty() || !directory.startsWith("/")) {
			return null;
		}
		
		File folder = new File(directory);
		if (!folder.exists() || !folder.canRead()) {
			return null;
		}
		
		File[] listOfFiles = folder.listFiles();
		ArrayList<String> listOfImageFiles = new ArrayList<>();
		
		for (int i = 0; i < listOfFiles.length; i++) {
			if (listOfFiles[i].isFile()) {
				for (String suffix : VALID_IMAGEFILE_SUFFIXES) {
					if (listOfFiles[i].getName().endsWith(suffix)) {
						listOfImageFiles.add(listOfFiles[i].getName());
						break;
					}
				}
			}
		}
		
		nImages = listOfImageFiles.size();
		
		Collections.sort(listOfImageFiles);
		return listOfImageFiles;
	}
	
	// faster performance for large directory
	private ArrayList<String> ImageFileNames() {
		ArrayList<String> imagefiles = listOfImageFiles(directory);
		nImages = imagefiles.size();
		return imagefiles;
	}

	public static ArrayList<String> listOfImageFiles(String directory) {
		if (directory == null || directory.isEmpty() || !directory.startsWith("/")) {
			return null;
		}

		Path folder = FileSystems.getDefault().getPath( directory );
		if (!Files.exists(folder) || !Files.isReadable(folder)) {
			return null;
		}

		ArrayList<String> listOfImageFiles = new ArrayList<>();
		try {
			DirectoryStream<Path> stream = Files.newDirectoryStream(folder);

			for (Path path : stream) {
				Path fn = path.getFileName();
				if (!Files.isDirectory(path)) {
					for (String suffix : VALID_IMAGEFILE_SUFFIXES) {
						if (fn.toString().endsWith(suffix)) {
							listOfImageFiles.add(fn.toString());
							break;
						}
					}
				}
			}
			stream.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		Collections.sort(listOfImageFiles);
		return listOfImageFiles;
	}

	public static ArrayList<String> listOfImageFiles2(String directory) {
		if (directory == null || directory.isEmpty() || !directory.startsWith("/")) {
			return null;
		}

		Path folder = FileSystems.getDefault().getPath( directory );
		if (!Files.exists(folder) || !Files.isReadable(folder)) {
			return null;
		}

		ArrayList<String> listOfImageFiles = new ArrayList<>();

		try {
			DirectoryStream<Path> stream = Files.newDirectoryStream(Paths.get(directory), p -> p.toFile().isFile());

			for (Path path : stream) {
				Path fn = path.getFileName();
				for (String suffix : VALID_IMAGEFILE_SUFFIXES) {
					if (fn.toString().endsWith(suffix)) {
						listOfImageFiles.add(fn.toString());
						break;
					}
				}
			}
			stream.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		Collections.sort(listOfImageFiles);
		return listOfImageFiles;
	}
}
