package gov.anl.gmca.analysis.helpers;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.syslog.Log;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

// this dialog class monitor a thread status, close itself when thread is finished or timeout'ed
// if a callback is defined, then it will be executed after job finishes.

public class ThreadMonitorDialog extends Dialog {
	private Text statusField;
	private Thread thread;
	private boolean blocking = false;
	private int jobStatus = -1;
	private int counter = 0;
	private int maxSeconds = 120;
	private String message = "Message to user";
	private Runnable callback = null;

	public ThreadMonitorDialog(Shell parentShell, Thread thread, int timeout, String message, Runnable callback) {
		super(parentShell);
		this.blocking = false;
		setShellStyle(SWT.MODELESS | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(false);
		this.thread = thread;
		this.message = message;
		this.maxSeconds = timeout;
		this.callback = callback;
	}
	
	public ThreadMonitorDialog(Shell parentShell, Thread thread, int timeout, Runnable callback) {
		this(parentShell, thread, timeout, null, callback);
	}
	
	public ThreadMonitorDialog(Shell parentShell, Thread thread, int timeout, String message) {
		this(parentShell, thread, timeout, message, null);
	}

	public ThreadMonitorDialog(Shell parentShell, Thread thread, int timeout) {
		this(parentShell, thread, timeout, "Message", null);
	}
	
	public ThreadMonitorDialog(Shell parentShell, Thread thread) {
		this(parentShell, thread, 180, "Message", null);
	}
	
	public void setBlocking(boolean block) {
		this.blocking = block;
		setBlockOnOpen(blocking);
	}
	
	public void setMessage(String message) {
		this.message = message;
	}
	
	protected Control createDialogArea(Composite parent) {
		Composite comp = (Composite) super.createDialogArea(parent);

		GridLayout layout = new GridLayout(2, true);
		layout.verticalSpacing = 10;
		layout.marginHeight = 10;
		layout.marginWidth = 10;
		layout.makeColumnsEqualWidth = false;
		comp.setLayout(layout);

		
		Label messageLabel = new Label(comp, SWT.CENTER);
		messageLabel.setText(message);
		messageLabel.setFont(GUI.FONT_12_BOLD);
		GridData griddata = new GridData(GridData.FILL_HORIZONTAL);
		griddata.verticalSpan = 6;
		griddata.horizontalSpan = 2;
		messageLabel.setLayoutData(griddata);
		
		layout.numColumns = 2;
		Label statusLabel = new Label(comp, SWT.RIGHT);
		statusLabel.setText("status: ");

		statusField = new Text(comp, SWT.NONE);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		statusField.setLayoutData(data);
		statusField.setText("calculating...");

		final Display display = parent.getDisplay();

		/* Set up the timer here */
		final Runnable timer = new Runnable() {
			public void run() {
				if (!statusField.isDisposed()) {
					statusField.setText("calculating ..." + counter++ + "sec.");
					if (counter <= maxSeconds) {
						if (thread.isAlive()) {
							display.timerExec(1000, this);
						} else {
							jobStatus = 0;
							statusField.setText("job finished");
							if (callback !=null ) {
								try {
									callback.run();
								} catch (Exception e) {
									Log.warn("failed to execute callback @ ThreadMonitorDialog");
									e.printStackTrace();
								}
							}
							ThreadMonitorDialog.this.close();
						}
					} else {
						jobStatus = 1;
						statusField.setText("timeout reached, job may have failed.");
						ThreadMonitorDialog.this.close();
					}
				}

			}
		};
		display.timerExec(0, timer);
		
		return comp;
	}

	protected void createButtonsForButtonBar(Composite parent) {
		super.createButtonsForButtonBar(parent);
		getButton(IDialogConstants.CANCEL_ID).setVisible(false);
		getButton(IDialogConstants.OK_ID).setText("Abort");
	}

	protected void okPressed() {
		jobStatus = 2;
		this.close();
	}

	public int jobStatus() {
		return jobStatus;
	}
	
	
	public static void main(String...strings) {
		Runnable r = new Runnable() {
			
			@Override
			public void run() {
				try {
					Thread.sleep(10000);
					System.out.println("1000...");
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		};
		Thread thread = new Thread(r);
		thread.start();
		
		final Display display = new Display();
	    final Shell shell = new Shell(display);
	    shell.setLayout(new FillLayout());
	    shell.setText("StackOverflow");
	    
	    Button button = new Button(shell, SWT.PUSH);
	    button.setText("Open dialog");

	    button.addListener(SWT.Selection, new Listener()
	    {
	        @Override
	        public void handleEvent(Event arg0)
	        {
	            new ThreadMonitorDialog(shell, thread, 180).open();
	        }
	    });

		/*
	    ThreadMonitorDialog td = new ThreadMonitorDialog(shell, thread, 180);
	    td.setBlocking(true);
	    int result = td.open();
	    
	    System.out.println("DEBUG " + result + " " + td.jobStatus());
		*/

	    shell.pack();
	    shell.open();
	    while (!shell.isDisposed())
	    {
	        if (!display.readAndDispatch())
	            display.sleep();
	    }
	    display.dispose();
	}
}
