package gov.anl.gmca.analysis.helpers;

import java.io.File;
import java.util.Arrays;

public class ValidateUserInput {

	public ValidateUserInput() {
	}
	
	public static boolean isDirectoryExists(String directoryname) {
		return new File(directoryname).exists();
	}

	public static boolean isEmptyOrNumeric(String field) {
		return field == null || field.isEmpty() || field.matches("[+-]?(?:\\d+(?:\\.\\d*)?|\\.\\d+)");
	}

	public static boolean isEmptyOrInteger(String field) {
		if (field == null || field.isEmpty()) {
			return true;
		}
		
		String value = field.toLowerCase().replaceAll(" ", "").replace("%", "").replace("percent", "");
		return value.matches("\\d+");
	}

	public static boolean isInteger(String field) {
		return field.matches("\\d+");
	}
	
	public static boolean isPercentage(String field) {
		if (field == null || field.isEmpty()) {
			return false;
		}
		
		if (field.toLowerCase().contains("%") || field.toLowerCase().contains("percent")) {
			return true;
		}
		return false;
	}
	
	public static String getPercentage(String field) {
		return field.toLowerCase().replaceAll(" ", "").replaceAll("percent", "").replaceAll("%","");
	}
	
	public static boolean isNumeric(String field) {
		return field != null && field.matches("[+-]?(?:\\d+(?:\\.\\d*)?|\\.\\d+)");
	}

	public static boolean isValidUnitCell(String field) {
		//System.out.println(field);
		String[] fields = field.split("\\s*,\\s*|\\s+");
		if (fields.length != 6) {
			return false;
		}

		for (String el : fields) {
			if (!el.matches("\\d+(\\.\\d+)?")) {
				return false;
			}
		}

		return true;
	}

	public static boolean isValidSpaceGroup(String field) {

		String spg = field.replaceAll(" ", "").toLowerCase();

		if (Arrays.asList(Spacegroup.SOHNCKE_SGS).contains(spg)) {
			return true;
		} else {
			return false;
		}
	}

	/*
	public static void main(String... strings) {
		String a = "1";
		System.out.println(new ValidateUserInput(a).isEmptyOrNumeric());
		a = "1     2 3 4, 5 , 6";
		System.out.println(new ValidateUserInput(a).validatedUnitCell());

		a = "r3a";
		System.out.println(new ValidateUserInput(a).validatedSpaceGroup());

	}
	*/

}
