package gov.anl.gmca.collect;

import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.io.Disk;
import gov.anl.gmca.io.Disk.File;
import gov.anl.gmca.io.Disk.IOException;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.PerfTimer;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;
import java.util.StringTokenizer;

public class DevAdxvFile
{
	protected static final String adxvDirname = System.getProperty("user.home")+"/bluice_temp";
	protected static final String adxvFilename = "adxv_image.txt";
	protected static boolean stop = false;
	protected static boolean listenerSet = false;
	public static final long EIGER_ADXV_DELAY_MS=500;
	
	public static class EigerADXVPath
	{
		public String path;
		public long receivedTime;
		public String getUpdate()
		{
			synchronized(eigerADXVPath)
			{
				long now = System.currentTimeMillis();
				if (now-receivedTime>EIGER_ADXV_DELAY_MS)
				{
					String ret = path;
					path = null;
					return ret;
				}
				return null;
			}
		}
		public void newImage(String path)
		{
			synchronized(eigerADXVPath)
			{
				if (null==this.path)
				{
					receivedTime=System.currentTimeMillis();
					this.path=path;
				}
			}
		}
	}
	protected static EigerADXVPath eigerADXVPath = new EigerADXVPath();
	
	public static String getFilename() { return adxvDirname+"/"+adxvFilename; }
	
	public synchronized static void startListener()
	{
		if (!listenerSet)
		{
    		listenerSet = true;
    		
    		if (DetectorConfig.isEiger())
    		{
    			Thread thread = new Thread(new Runnable(){public void run()
    			{
    				while (true)
    				{
    					String newPath = eigerADXVPath.getUpdate();
    					if (null!=newPath) update(newPath);
    					Util.sleep(200);
    				}
    			}}, "ADXV delayed update");
    			thread.start();
    		}
    		
    		DevFPE.get().imageViewerListener.addListener(new ListenerList.Listener<DevFPE>()
    		{
				@Override public void handleEvent(DevFPE object, Object data)
				{
					if (Client.isMaster())
					{
						if (DetectorConfig.isEiger())
						{
							String imagePath = DevFPE.get().getImageViewerDir()+"/"+DevFPE.get().getImageViewerFile().filename;
							eigerADXVPath.newImage(imagePath);
						}
						else
						{
							update(null);
						}
					}
				}
			});
		}
	}
	
	public synchronized static void update(String imagePath)
	{
		if (!Client.isMaster()) return;
		
		if (null==imagePath) imagePath = DevFPE.get().getImageViewerDir()+"/"+DevFPE.get().getImageViewerFile().filename;
		
		PerfTimer pt = new PerfTimer("adxv_image.txt update: ");
		
		File adxvDir = new File(adxvDirname);
		
		if (!adxvDir.exists())
		{
			if (!adxvDir.mkdir())
			{
				Log.error("Couldn't create "+adxvDir.getAbsolutePath());
				stop=true;
				return;
			}
		}
		
		File adxvFile = new File(adxvDir.getAbsolutePath()+"/"+adxvFilename);
		
		if (adxvFile.exists() && (!adxvFile.canRead() || !adxvFile.canWrite()))
		{
			Log.error("Error reading or writing to file "+adxvFile.getAbsolutePath());
			stop=true;
			return;
		}
		
		int number=0;
		
		boolean fileIsDifferent = true;
		
		try
		{
			if (adxvFile.exists())
			{
				ArrayList<String> contents = Disk.readText(adxvFile.getCanonicalPath());
				if (null == contents || contents.size()==0)
				{
					Log.error("Couldn't determine number from file "+adxvFile.getCanonicalPath());
					//stop=true;
				}
				else
				{
	    			String line = contents.get(0);
	    			StringTokenizer st = new StringTokenizer(line);
	    			if (st.countTokens()==0 || st.countTokens()>2)
	    			{
	    				Log.error("Couldn't determine number from line: "+line);
	    				//stop=true;
	    			}
	    			try
	    			{
	    				number = Integer.parseInt(st.nextToken()) + 1;

	    				String lastfilepath;
	    				if (st.hasMoreTokens())
	    				{
		    				lastfilepath = st.nextToken();
	    				}
	    				else
	    				{
	    					lastfilepath = "";
	    				}
	    				
	    				if (lastfilepath.equals(imagePath))
	    				{
	    					fileIsDifferent = false;
	    				}
	    			}
	    			catch (NumberFormatException nfe) { nfe.printStackTrace(); }
				}
			}			
		}
		catch (IOException ioe)
		{
			Log.error("Error reading adxv file "+adxvFile.getAbsolutePath());
			//stop=true;
		}
		
		if (!stop && fileIsDifferent)
		{
			String newLine = number+" "+imagePath;
			ArrayList<String> newContents = new ArrayList<String>();
			newContents.add(newLine);
			try
			{
				Disk.writeText(adxvFile.getCanonicalPath(), newContents);
			}
			catch (IOException ioe)
			{
				Log.error("Error writing to adxv file "+adxvFile.getAbsolutePath());
				//stop=true;
			}
		}
		//pt.printElapsedMs();
	}
}
