package gov.anl.gmca.collect;

import gov.anl.gmca.common.Limits;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.Epics.EpicsValueMonitor;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;
import gov.anl.gmca.io.PVLookup.PV_MOSTAB;
import gov.anl.gmca.io.PVLookup.PV_RUN_LIMITS;
import gov.anl.gmca.util.ListenerList;
import gov.aps.jca.dbr.DBR;

public class DevCollectSettings
{
	public static ListenerList<DevCollectSettings> llChecks = new ListenerList<DevCollectSettings>(null);

	public static void init()
	{
		EpicsValueMonitor mon = new EpicsValueMonitor() {
			@Override public void valueChanged(String pvName, DBR value) {
				llChecks.notifyListeners();
			}
		};
		Epics.addMonitor(PV_BLUICE.enableEndShutterCheck, mon);
		Epics.addMonitor(PV_BLUICE.enableEnergyGapCheck, mon);
		Epics.addMonitor(PV_BLUICE.waitForFeedback, mon);
		Epics.addMonitor(PV_BLUICE.enableEnergyMove, mon);
		Epics.addMonitor(PV_BLUICE.enableDetectorMove, mon);
		Epics.addMonitor(PV_BLUICE.enableSampleEnvMove, mon);
		Epics.addMonitor(PV_BLUICE.enableCryojetMonitor, mon);
	}

	public static Limits getWedgeLimits()
	{
		double ll = Epics.getDouble(PV_RUN_LIMITS.wedgeSize_DRVL.collectLimit());
		double hl = Epics.getDouble(PV_RUN_LIMITS.wedgeSize_DRVH.collectLimit());
		return new Limits(ll,hl,"wedge size");
	}
	
	public static Limits getCollectExposureLimits()
	{
		double ll = Epics.getDouble(PV_RUN_LIMITS.exposureTime_DRVL.collectLimit());
		double hl = Epics.getDouble(PV_RUN_LIMITS.exposureTime_DRVH.collectLimit());
		return new Limits(ll,"minimum exposure time",hl,"maximum exposure time");
	}
	
	public static Limits getCollectDeltaLimits()
	{
    	double ll = Epics.getDouble(PV_RUN_LIMITS.delta_DRVL.collectLimit());
    	double hl = Epics.getDouble(PV_RUN_LIMITS.delta_DRVH.collectLimit());
    	return new Limits(ll,"minimum delta value",hl,"maximum delta value");
   	}
	
	public static Limits getRasterExposureLimits()
	{
		double ll = Epics.getDouble(PV_RUN_LIMITS.exposureTime_DRVL.rasterLimit());
		double hl = Epics.getDouble(PV_RUN_LIMITS.exposureTime_DRVH.rasterLimit());
		return new Limits(ll,"minimum exposure time",hl,"maximum exposure time");
	}
	
	public static Limits getRasterDeltaLimits()
	{
    	double ll = Epics.getDouble(PV_RUN_LIMITS.delta_DRVL.rasterLimit());
    	double hl = Epics.getDouble(PV_RUN_LIMITS.delta_DRVH.rasterLimit());
    	return new Limits(ll,"minimum delta value",hl,"maximum delta value");
   	}
	
	public static boolean getEndstationCheckEnabled()
	{
		int val = Epics.getInteger(PV_BLUICE.enableEndShutterCheck);
		return (1==val);
	}
	
	public static boolean getEnergyGapCheckEnabled()
	{
		int val = Epics.getInteger(PV_BLUICE.enableEnergyGapCheck);
		return (1==val);
	}

	public static boolean getFeedbackCheckEnabled()
	{
		int val = Epics.getInteger(PV_BLUICE.waitForFeedback);
		return (1==val);
	}

	public static boolean getEnableEnergyMove()
	{
		int val = Epics.getInteger(PV_BLUICE.enableEnergyMove);
		return (1==val);
	}

	public static boolean getEnableDetectorMove()
	{
		int val = Epics.getInteger(PV_BLUICE.enableDetectorMove);
		return (1==val);
	}

	public static boolean getEnableSampleEnvMove()
	{
		int val = Epics.getInteger(PV_BLUICE.enableSampleEnvMove);
		return (1==val);
	}

	public static boolean getEnableCryojetMonitor()
	{
		int val = Epics.getInteger(PV_BLUICE.enableCryojetMonitor);
		return (1==val);
	}

	public static int getMostabStatus()
	{
		return Epics.getInteger(PV_MOSTAB.status);
	}
	
	public static void setMostabStatus(int val)
	{
		Epics.putInteger(PV_MOSTAB.status, val);
	}

	public static String getMostabStatusLine()
	{
		return Epics.getString(PV_MOSTAB.statusline);
	}
}
