package gov.anl.gmca.collect;

import com.mashape.unirest.http.*;
import com.mashape.unirest.http.exceptions.UnirestException;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.CompressionAlgo;
import gov.anl.gmca.io.PVLookup.PV_EIGER;
import gov.anl.gmca.io.PVLookup.StreamHeaderDetail;
import gov.anl.gmca.util.Util;

public class DevEiger
{
	static String[] valueText = new String[]{"\"value\": false", "\"value\": true"};
	static String[] valueTextWithBraces = new String[]{"{\"value\": false}", "{\"value\": true}"};

	public static boolean isAutoSumEnabled(Boolean expectedValue) throws UnirestException
	{
		// send server request
		HttpResponse<String> resp2 = Unirest.get("http://10.42.103.10/detector/api/1.6.0/config/auto_summation")
				.header("Accept", "application/json")
				.asString();

		// parse result
		Boolean result = null;
		if (resp2.getBody().contains(valueText[0])) result = false;
		if (resp2.getBody().contains(valueText[1])) result = true;
		System.out.println("Eiger auto_summation = "+result);

		// if result doesn't match expected, show error
		if (null != expectedValue && expectedValue != result) {
			// set failed
			String msg = "Setting Eiger auto summation failed.  Result=" + resp2.getBody();
			if (DBConfig.serverMode)
				gov.anl.gmca.server.messaging.Popup.syncPopup("note", msg, null, 0);
			else
				Util.popupError("Error", msg);
		}

		return result;
	}

	public static boolean setAutoSumEnabled(boolean enable) throws UnirestException
	{
		// send value to Eiger
		HttpResponse<String> resp = Unirest.put("http://10.42.103.10/detector/api/1.6.0/config/auto_summation")
				.header("Content-Type", "application/json")
				.header("charset", "utf-8")
				.body(enable ? valueTextWithBraces[1] : valueTextWithBraces[0])
				.asString();

		// check the new value is set
		boolean newValue = isAutoSumEnabled(enable);
		return newValue == enable;
	}

	public static String setupForCollection(boolean disableEigerPerImageTrigger, boolean disableAutoSummation)
	{
		if (!DetectorConfig.isEiger()) return null;

		//curl -H "Content-Type: application/json; charset=utf-8 HTTP/1.1" -X PUT -d "{\"value\":true}" http://10.42.103.10/detector/api/1.6.0/config/auto_summation
		Epics.putInteger(PV_EIGER.CompressionAlgo, CompressionAlgo.BSLZ4.ordinal());
		Epics.putInteger(PV_EIGER.StreamHeaderDetail, StreamHeaderDetail.ALL.ordinal());

		int triggerMode = disableEigerPerImageTrigger ? 2 : 3;
		Epics.putInteger(PV_EIGER.TriggerMode, triggerMode);

		try {
			if (isAutoSumEnabled(null) != !disableAutoSummation) {
				setAutoSumEnabled(!disableAutoSummation);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return null;
	}
	
	public static void init()
	{
	}
	
	public static boolean checkStreamWriter()
	{
		return true;
	}
}
