package gov.anl.gmca.collect;

import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.Unirest;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.client_server.ui.DlgStreamWriterView;
import gov.anl.gmca.gui.GUIStatus;
import gov.anl.gmca.users.ModelUserTable;
import gov.anl.gmca.util.Util;

import java.util.HashMap;
import java.util.Map;

public class DevStreamWriter
{
    public static boolean setConfig(String directory, String prefix, int firstFrameLabel, int firstFrameIndex, boolean continuingSeries, double omega_start, double omega_delta, double energy_keV)
    {
        // prepare url
        String host = DBConfig.get("streamwriter_ip");
        int port = (int) DBConfig.getDouble("streamwriter_port");
        int httpStatus = -1;
        String url = String.format("http://%s:%d/set_config", host, port);

        // prepare parameters
        Map<String,Object> fields = new HashMap<>();
        fields.put("dir",directory);
        fields.put("prefix",prefix);
        fields.put("username",ModelUserTable.Client.getUser());
        fields.put("first_frame_label",firstFrameLabel);
        fields.put("counter_start_value",firstFrameIndex);
        fields.put("continue_series",continuingSeries?1:0);
        fields.put("omega_start",Util.printDouble(omega_start,3));
        fields.put("omega_delta",Util.printDouble(omega_delta,3));
        fields.put("energy_keV",Util.printDouble(energy_keV,4));

        // log the request
        String urlAsText = url+" ";
        for (String key : fields.keySet()) urlAsText += key+"="+fields.get(key)+" ";
        System.out.println(urlAsText);
        DlgStreamWriterView.get().cmdSent(urlAsText);

        // send the request
        HttpResponse<String> resp;
        try {
            resp = Unirest.post(url)
                    .fields(fields)
                    .asString();
            httpStatus = resp.getStatus();
            if (GUIStatus.inGUIMode) DlgStreamWriterView.get().cmdReceived("HTTP response code: "+httpStatus);
            if (httpStatus != 200) {
                Util.asyncPopupError("Error", "Error sending command to StreamWriter: " + httpStatus);
                return false;
            }
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            if (GUIStatus.inGUIMode) DlgStreamWriterView.get().cmdReceived("Exception: "+e.getMessage());
            return false;
        }
    }
}
