package gov.anl.gmca.collect;

import gov.anl.gmca.collect.ModelCollectRun.RunState;
import gov.anl.gmca.collect.ModelCollectRunList.CollectState;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.ModelRasterState.RasterState;
import gov.anl.gmca.util.Util;

// does ETA calculations, but PVs are still handled by DevFPE
//
public class ETA
{
	public static class CollectETA
	{
		public int totalSeconds, runCount;
	}
	public static class RasterETA
	{
		public int totalSeconds, cellCount;
	}
	
	public static String getCollectTooltip()
	{
		CollectETA eta = calcCollectETA();
		String runString = eta.runCount+" run"+(eta.runCount>1?"s":"");
		return "ETA for "+runString+" is "+Util.readableSeconds(eta.totalSeconds);
	}
	public static String getRasterTooltip()
	{
		RasterETA eta = calcRasterETA();
		return "ETA for "+eta.cellCount+" cells is "+Util.readableSeconds(eta.totalSeconds);
	}
	
	public static CollectETA calcCollectETA()
	{
		int etaSec = 0, runCount = 0;
		int currentRunIndex = ModelCollectRunList.get().getCurrentRunIndex();
		boolean zeroTab = currentRunIndex==0;
		
		for (ModelCollectRun run : ModelCollectRunList.get().getAllRunsCopy())
		{
			if ((zeroTab && run.getRunIndex()!=0) ||
				(!zeroTab && run.getRunIndex()<currentRunIndex)) continue;
			if (run.getRunState()==RunState.COMPLETE) continue;

			double totalTransitionTime = 0.0;
			gov.anl.gmca.collect.FramePartitioner.FramePartition lastPart = null;
			for (gov.anl.gmca.collect.FramePartitioner.FramePartition part : run.seq().getFramePartitions())
			{
				if (null != lastPart)
				{
					double omegaMove = Math.abs(lastPart.endAngle - part.startAngle);
					double omegaTime = omegaMove / DBConfig.getDouble("speed_limit_gonio_omega");
					double partitionTransitionTime = 0.050;
					totalTransitionTime += omegaTime + partitionTransitionTime;
				}
				lastPart = part;
			}
			
			boolean	isShutterless			= DetectorConfig.isDectris() && DevUserConfig.get().getShutterlessEnable();
			int 	framesThisRun 			= run.seq().getRemainingFrameCount();
			double 	exposureTimePerFrame 	= run.getExposureTime();
			double 	deadTimePerFrame 		= isShutterless ? 0.0 : (DetectorConfig.isDectris() ? 0.10 : 2.5);
			double	startupTime				= 5.0;
			double 	runTotalTime 			= startupTime + (exposureTimePerFrame + deadTimePerFrame) * framesThisRun + totalTransitionTime;
			
			etaSec += runTotalTime;
			runCount++;
		}
		
		CollectETA collectETA = new CollectETA();
		collectETA.totalSeconds = etaSec;
		collectETA.runCount = runCount;
		return collectETA;
	}
	
	public static RasterETA calcRasterETA()
	{
		ModelRasterRun currentRun = ModelRasterState.get().getCurrentRunModel();
		
		int 	cellCount 				= currentRun.grid().getSelectedCellCount() - currentRun.getNextFrame();
		boolean isPilatus				= DetectorConfig.isDectris();
		boolean	isShutterless			= currentRun.isShutterless();
		double 	exposureTimePerFrame 	= currentRun.getExposureTime();
		double 	deadTimePerFrame 		= isShutterless ? 0.0 : isPilatus ? 0.2 : 3.0;
		double	transitionTime			= isShutterless ? 1.0 : 0.0; // row transition time is built into the cell transition time in shuttered mode
		double	totalTransitionTime		= transitionTime * currentRun.grid().getCellCountY();
		double	startupTime				= 5.0;
		double 	runTotalTime 			= startupTime + (exposureTimePerFrame + deadTimePerFrame) * cellCount + totalTransitionTime;
		
		RasterETA rasterETA = new RasterETA();
		rasterETA.totalSeconds = cellCount==0?0:(int)Math.round(runTotalTime);
		rasterETA.cellCount = cellCount;
		return rasterETA;
	}
	
	public static int calcActiveETA()
	{
		if (ModelCollectRunList.get().getCollectState() != CollectState.INACTIVE)
		{
			return calcCollectETA().totalSeconds;
		}
		else if (ModelRasterState.get().getRasterState() != RasterState.INACTIVE)
		{
			return calcRasterETA().totalSeconds;
		}
		else return 0;
	}
}
