package gov.anl.gmca.collect;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.util.Util;

import java.io.FileInputStream;
import java.util.Map;
import java.util.StringTokenizer;

public class HeaderCheck
{
	public static class Values
	{
		public double startAngle_deg, delta_deg, expTime_sec, energy_keV;
	}

	public static String check(String cbfFullPath, Values targetValues)
	{
		if (Main.FAKE_DATA) {
			return "OK";
		}

		Values actualValues = parseDectrisCBFHeader(cbfFullPath);
		if (null==actualValues) {
			return "WARN: Failed to parse CBF header for "+cbfFullPath;
		}

		double headerCheckToleranceDeg = DBConfig.getDouble("header_check_tolerance_deg");
		double headerCheckToleranceSec = DBConfig.getDouble("header_check_tolerance_sec");
		double headerCheckTolerance_keV = DBConfig.getDouble("header_check_tolerance_keV");

		boolean success=true;
		String msg = "";
		if (Math.abs(targetValues.startAngle_deg-actualValues.startAngle_deg)>headerCheckToleranceDeg) {
			success=false;
			msg += String.format("Start angle mismatch: expected=%.2f, header=%.2f. ", targetValues.startAngle_deg, actualValues.startAngle_deg);
		}
		if (Math.abs(targetValues.delta_deg-actualValues.delta_deg)>headerCheckToleranceDeg) {
			success=false;
			msg += String.format("Delta mismatch: expected=%.2f, header=%.2f. ", targetValues.delta_deg, actualValues.delta_deg);
		}
		if (Math.abs(targetValues.expTime_sec-actualValues.expTime_sec)>headerCheckToleranceSec) {
			success=false;
			msg += String.format("Exposure time mismatch: expected=%.2f, header=%.2f. ", targetValues.expTime_sec, actualValues.expTime_sec);
		}
		if (Math.abs(targetValues.energy_keV-actualValues.energy_keV)>headerCheckTolerance_keV) {
			success=false;
			msg += String.format("Energy mismatch: expected=%.2f, header=%.2f. ", targetValues.energy_keV, actualValues.energy_keV);
		}

		Util.printf("Header check for %s: start angle exp=%.2f act=%.2f, delta exp=%.2f act=%.2f, expTime exp=%.2f act=%.2f, energy exp=%.2f act=%.2f %s",
				cbfFullPath.substring(cbfFullPath.lastIndexOf('/')+1),
				targetValues.startAngle_deg,actualValues.startAngle_deg,
				targetValues.delta_deg,actualValues.delta_deg,
				targetValues.expTime_sec,actualValues.expTime_sec,
				targetValues.energy_keV,actualValues.energy_keV,
				success ? "OK" : "ERROR: "+msg);

		return success ? "OK" : msg;
	}

	public static Values parseDectrisCBFHeader(String cbfFullPath)
	{
		/* Example header format:
		# Start_angle 0.2850 deg.
		# Angle_increment 0.0000 deg.
		# Exposure_period 0.1000000 s
		# Wavelength 1.03326 A
		# Detector_distance 0.30000 m
		*/
		Values ret = new Values();
		FileInputStream is = null;
		try {
			is = new FileInputStream(cbfFullPath);
			byte[] first2k = new byte[2000];
			int readBytes = is.read(first2k);
			if (readBytes<2000) {
				System.out.println("ERROR: only read "+readBytes+" bytes from "+cbfFullPath);
				return null;
			}
			String first2ks = new String(first2k);
			StringTokenizer st = new StringTokenizer(first2ks,"\n");
			while (st.hasMoreTokens()) {
				String line = st.nextToken();
				StringTokenizer st2 = new StringTokenizer(line, " ");
				if (st2.countTokens()>=3) {
					if (!st2.nextToken().equals("#")) continue;
					String secondToken = st2.nextToken(), thirdToken = st2.nextToken();
					if (secondToken.equals("Start_angle")) ret.startAngle_deg = Double.parseDouble(thirdToken);
					else if (secondToken.equals("Angle_increment")) ret.delta_deg = Double.parseDouble(thirdToken);
					else if (secondToken.equals("Exposure_period")) ret.expTime_sec = Double.parseDouble(thirdToken);
					else if (secondToken.equals("Wavelength")) ret.energy_keV = 12.398/Double.parseDouble(thirdToken);
				}
			}
		} catch (Exception e) { e.printStackTrace(); return null; }
		finally { if (null!=is) try { is.close(); } catch (Exception e) { e.printStackTrace(); } }
		return ret;
	}
}
