package gov.anl.gmca.collect.graph;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.ui.ModelGraph;
import gov.anl.gmca.util.Pair;
import gov.anl.gmca.util.Util;
import gov.anl.gmca.util.VerticalText;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;

import java.util.ArrayList;

public class SFAxes
{
	private SFGraph graph;
	public String leftYLabel = "", rightYLabel = "";
	private int tickLength = 5;
	private int tickMargin = 2;
	private static final int X_AXIS_MARGIN_PX = 50;
	
	public SFAxes(SFGraph graph)
	{
		this.graph = graph;
	}
	
	public void draw(GC gc)
	{
		gc.setForeground(GUI.COLOR_BLACK);
		gc.setFont(GUI.FONT_9);
		
		// X-axis line
		graph.setXAxisTransform(gc);
		gc.drawLine(0, 0, graph.plotAreaWidth, 0);

		// X-axis units
		String omegaLabel = "Omega start (deg)";
		int omegaWidth = gc.textExtent(omegaLabel).x;
		int omegaHeight = gc.textExtent(omegaLabel).y;
		gc.drawString(omegaLabel, graph.plotAreaWidth/2-omegaWidth/2, graph.xAxisHeight-omegaHeight);
		
		// X-axis ticks
		Pair<ArrayList<Double>, Integer> xTicks = ModelGraph.looseLabel(0, graph.maxAngle, Math.max(3,graph.plotAreaWidth/80));
		ArrayList<Double> tickList = xTicks.a, correctedTickList = new ArrayList<Double>();
		for (Double tick : tickList)
		{
			int x = graph.angleToXPixelCoord(tick, false);
			int xMin = graph.angleToXPixelCoord(graph.minAngle, false);
			int xMax = graph.angleToXPixelCoord(graph.maxAngle, false);
			if (x-xMin > X_AXIS_MARGIN_PX && xMax-x > X_AXIS_MARGIN_PX)
			{
				correctedTickList.add(tick);
			}
		}
		correctedTickList.add(0, graph.minAngle);
		correctedTickList.add(graph.maxAngle);
		
		for (Double tick : correctedTickList)
		{
			int x = graph.angleToXPixelCoord(tick, false);
			gc.drawLine(x, 0, x, tickLength);
			
			String tickString = Util.sprintf("%."+Math.max(2, xTicks.b)+"f",tick);
			int textWidth = gc.textExtent(tickString).x;
			
			gc.drawString(tickString, x-textWidth/2, tickLength+tickMargin);
		}
		
		// Left Y-axis line
		graph.setLeftYAxisTransform(gc);
		gc.drawLine(graph.yAxisWidth, 0, graph.yAxisWidth, graph.plotAreaHeight);
		
		// Left Y-axis units
		leftYLabel = graph.getOutput1().name;
		int textHeight = gc.textExtent(leftYLabel).x+4;
		int spotLabelY = graph.plotAreaHeight/2 + textHeight/2;
		VerticalText.drawString(gc, leftYLabel, 0, spotLabelY, SWT.LEFT, null);
		
		// Left Y-axis ticks
		Pair<ArrayList<Double>, Integer> yTicks = ModelGraph.looseLabel(graph.minParam1, graph.maxParam1, 5);
		for (Double tick : yTicks.a)
		{
			if (tick<graph.minParam1 || tick>graph.maxParam1) continue;
			
			int tickYCoord = graph.outputToYPixelCoord(tick, graph.minParam1, graph.maxParam1, graph.getOutput1().reverseDisplay);
			String text = Util.sprintf("%."+yTicks.b+"f", tick);
			int textWidth = gc.textExtent(text).x;
			textHeight = gc.textExtent(text).y;
			
			gc.drawString(text, graph.yAxisWidth-tickLength-tickMargin-textWidth, tickYCoord-textHeight/2);
			gc.drawLine(graph.yAxisWidth-tickLength, tickYCoord, graph.yAxisWidth, tickYCoord);
		}
		
		// Right Y-axis line
		/*graph.setRightYAxisTransform(gc);
		gc.drawLine(0, 0, 0, graph.plotAreaHeight);
		
		// Right Y-axis units
		rightYLabel = graph.getOutput2().name;
		textHeight = gc.textExtent(rightYLabel).x+4;
		int resLabelY = graph.plotAreaHeight/2 - textHeight/2;
		VerticalText.drawString(gc, rightYLabel, graph.yAxisWidth, resLabelY, SWT.RIGHT, null);
		
		// Right Y-axis ticks
		yTicks = ModelGraph.looseLabel(graph.minParam2, graph.maxParam2, 5);
		for (Double tick : yTicks.a)
		{
			if (tick > graph.maxParam2 || tick < graph.minParam2) continue;
			
			int tickYCoord = graph.outputToYPixelCoord(tick, graph.minParam2, graph.maxParam2, graph.getOutput2().reverseDisplay);
			String text = Util.sprintf("%."+yTicks.b+"f", tick);
			textHeight = gc.textExtent(text).y;
			
			gc.drawString(text, tickLength+tickMargin, tickYCoord-textHeight/2);
			gc.drawLine(0, tickYCoord, tickLength, tickYCoord);
		}*/
	}
}
