package gov.anl.gmca.collect.graph;

import gov.anl.gmca.collect.graph.SFGraph.SpotFinderPoint;
import gov.anl.gmca.gui.GUI;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.MouseTrackListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;

public class SFHover
{
	private SFGraph graph;
	private MouseMoveListener mouseMoveListener;
	private MouseTrackListener mouseTrackListener;
	SpotFinderPoint closestPoint;
	public static enum PointType { SPOT_TOTAL, BRAGG_CANDIDATES, RES_ESTIMATE };
	
	public SFHover(final SFGraph graph)
	{
		this.graph = graph;
		
		mouseMoveListener = new MouseMoveListener()
		{
			@Override public void mouseMove(MouseEvent me)
			{
				updateDisplayText(me.x, me.y);
			}
		};
		graph.wdg.addMouseMoveListener(mouseMoveListener);
		
		mouseTrackListener = new MouseTrackListener()
		{
			@Override public void mouseEnter(MouseEvent me)
			{
				if (graph.points.size()!=0)
				{
					graph.wdg.diffTab.setSFDisplayVisible(true);
					updateDisplayText(me.x, me.y);
					graph.wdg.redraw();
				}
			}
			@Override public void mouseExit(MouseEvent me)
			{
				graph.wdg.diffTab.setSFDisplayVisible(false);
				closestPoint = null;
				graph.wdg.redraw();
			}
			@Override public void mouseHover(MouseEvent me) {}
		};
		graph.wdg.addMouseTrackListener(mouseTrackListener);
	}
	
	private void updateDisplayText(int mx, int my)
	{
		Point graphPos = graph.mousePosToGraphPos(mx,my);
		SpotFinderPoint prevPoint = closestPoint;
		closestPoint = graph.getClosestPt(graphPos);
		if (closestPoint != prevPoint)
		{
			graph.wdg.diffTab.sfDisplay.updateText(closestPoint);
			graph.wdg.redraw();
		}
	}
	
	void drawHoverLine(GC gc)
	{
		if (null == closestPoint) return;
		
		int x = closestPoint.xPlotAreaCoord;
		gc.setLineWidth(1);
		gc.setForeground(GUI.COLOR_BLACK);
		gc.drawLine(x, 0, x, graph.plotAreaHeight);
	}
}
