package gov.anl.gmca.collect.graph;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.ui.TabCollect;
import gov.anl.gmca.collect.viewer.WdgDiffTab;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.sample.ImageViewer;
import gov.anl.gmca.spotfinder.DBImageStats;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_OUTPUT;
import gov.anl.gmca.spotfinder.SFData.SFType;
import gov.anl.gmca.spotfinder.SFLog;
import gov.anl.gmca.syslog.Log;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;
import java.util.HashMap;

// top-level widget for the graph itself, not the title bar
//
public class WdgSFGraph extends Canvas implements PaintListener
{
	private Color bgColor = GUI.COLOR_WHITE;
	WdgDiffTab diffTab;
	private Menu menu;
	private MenuItem itemSaveScreenshot, itemSFParam, itemOpenSpreadsheet;
	private SFGraph sfGraph;
	private static ArrayList<WdgSFGraph> instList = new ArrayList<>();
	
	public static enum GraphSource { COLLECT, SAMPLE };
	
	public static enum LegendItem
	{
		LABEL1(GUI.COLOR_BLUE2, SPOTFINDER_OUTPUT.DIALS_SPOT_TOTAL, true),
		LABEL2(GUI.COLOR_GREEN2, SPOTFINDER_OUTPUT.DIALS_SPOT_TOTAL, false);
		
		public Color plotColor;
		public SPOTFINDER_OUTPUT output;
		public HashMap<GraphSource, Boolean> isVisible = new HashMap<>();
		public HashMap<GraphSource, ArrayList<Label>> label = new HashMap<>();
		
		public void setLabel(GraphSource src, Label label)
		{
			if (null==this.label.get(src)) this.label.put(src, new ArrayList<Label>());
			this.label.get(src).add(label);
			setVisible(src, isVisible.get(src), false);
		}
		
		// clicking widgets will call this method with sendUpdate=true
		// PV updates will pass sendUpdate=false
		// initial construction also passes sendUpdate=false
		public void setVisible(GraphSource src, boolean isVisible, boolean sendUpdate)
		{
			//for (Label l : this.label.get(src))
			//	l.setForeground(isVisible?plotColor:GUI.COLOR_DARK_GRAY);
			//this.isVisible.put(src, isVisible);
			if (sendUpdate) updateDevice(src);
		}

		private void updateDevice(GraphSource src)
		{
			int visibility=0;
			for (LegendItem item : LegendItem.values())
			{
				if (item.isVisible.get(src)) visibility |= 1<<item.ordinal();
			}
			DevSFGraph.setVisible(src, visibility);
		}
		
		LegendItem(Color color, SPOTFINDER_OUTPUT output, boolean isVisible)
		{
			this.plotColor=color; this.output=output;
			for (GraphSource src : GraphSource.values())
			{
				this.isVisible.put(src, isVisible);
			}
		}
	}
	
	public WdgSFGraph(Composite parent, WdgDiffTab diffTab)
	{
		super(parent, SWT.NONE);
		this.diffTab = diffTab;
		instList.add(this);
		
		setBackground(bgColor);
		
		sfGraph = new SFGraph(this);
		
		addMenu();
		addPaintListener(this);
		addControlListener(new ControlListener()
		{
			@Override public void controlResized(ControlEvent arg0)
			{
				redraw();
			}
			@Override public void controlMoved(ControlEvent arg0) {}
		});
	}
	
	private void addMenu()
	{
		menu = new Menu(this);
		
		MenuItem itemDescription = new MenuItem(menu, SWT.PUSH);
		itemDescription.setEnabled(false);
		itemDescription.setText("SpotFinder Graph Menu");
		
		new MenuItem(menu, SWT.SEPARATOR);
		
		itemSaveScreenshot = new MenuItem(menu, SWT.PUSH);
		itemSaveScreenshot.setText("Save screenshot");
		itemSaveScreenshot.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				TabCollect.get().leftPane.diffTab.spotFinderGraph.saveScreenshot(true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		/*itemSFParam = new MenuItem(menu, SWT.PUSH);
		itemSFParam.setText("Change SpotFinder parameters");
		itemSFParam.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				DlgSFOptions.get().open();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});*/
		
		itemOpenSpreadsheet = new MenuItem(menu, SWT.PUSH);
		itemOpenSpreadsheet.setText("Open results spreadsheet");
		itemOpenSpreadsheet.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				SFLog.openLog(SFType.COLLECT, ModelCollectRunList.get().getCurrentRunIndex());
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		setMenu(menu);
		menu.addMenuListener(new MenuListener()
		{
			@Override public void menuShown(MenuEvent e)
			{
				//itemSFParam.setEnabled(Client.isMaster());
			}
			@Override public void menuHidden(MenuEvent arg0) {}
		});
	}
	
	public static void redrawAll()
	{
		for (WdgSFGraph wdg : instList) { wdg.redraw(); }
	}
	
	public void paintToImage(Image target)
	{
		GC targetGC = new GC(target);
		int targetWidth=target.getBounds().width, targetHeight=target.getBounds().height;
		draw(targetGC, targetWidth, targetHeight, true, false);
   		targetGC.dispose();
	}
	
	@Override public void paintControl(PaintEvent pe)
	{
		draw(pe.gc, getBounds().width, getBounds().height, false, true);
	}
	
	public void draw(GC gc, int width, int height, boolean drawHeader, boolean drawHover)
	{
		sfGraph.updateLayout(gc, width, height);
		try
		{
			sfGraph.reloadData();
		}
		catch (Exception e) { e.printStackTrace(); }
		
		if (sfGraph.points.size()==0)
		{
			// gray background
			gc.setBackground(GUI.BGCOLOR);
			gc.fillRectangle(0, 0, width, height);
			
			// black border
			gc.setForeground(GUI.COLOR_BLACK);
			gc.setLineWidth(1);
			gc.drawRectangle(0, 0, width-1, height-1);
			
			// note
			String note = "---";
			Point extent = gc.textExtent(note);
			gc.drawString(note, width/2-extent.x/2, height/2-extent.y/2);

			return;
		}
		
		if (width<=0 || height<=0)
		{
			Log.note("zero size graph");
			return;
		}
		
		// create buffer
		Image buffer = new Image(gc.getDevice(), width, height);
		GC bufGC = new GC(buffer);
		
		// white background
		bufGC.setAntialias(SWT.ON);
		bufGC.setBackground(GUI.COLOR_WHITE);
		bufGC.fillRectangle(0, 0, width, height);
		
		// contents
		sfGraph.axes.draw(bufGC);
		sfGraph.plot.draw(bufGC);
		if (drawHover) sfGraph.hover.drawHoverLine(bufGC);
		
		// black border
		bufGC.setTransform(null);
		bufGC.setForeground(GUI.COLOR_BLACK);
		bufGC.setLineWidth(1);
		bufGC.drawRectangle(0, 0, width-1, height-1);
		
		// draw buffer
		gc.drawImage(buffer, 0, 0);
		bufGC.dispose();
		buffer.dispose();
	}

	public void saveScreenshot(boolean openViewer)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			try
			{
				ModelCollectRun run = ModelCollectRunList.get().getCurrentRunModel();
				String path = run.getFullDirectory()+"/"+run.getFileRoot()+"_"+run.getRunLabel()+"_sf.png";
				// TODO: find a unique filename
				int width = Math.max(600,  getBounds().width);
				int height = Math.max(200, getBounds().height);
				Image target = new Image(GUI.getDisplay(), width, height);
				paintToImage(target);
				ImageLoader loader = new ImageLoader();
				loader.data = new ImageData[]{target.getImageData()};
				loader.save(path, SWT.IMAGE_PNG);
				
				if (openViewer)
				{
					ImageViewer.show(path);
				}
			}
			catch (Exception e) { e.printStackTrace(); }
		}});
	}
}
