package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.DevCollectRunList;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.ModelCollectRunList.CollectState;
import gov.anl.gmca.collect.graph.DevSFGraph;
import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.common.ui.WdgRunTabs;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

public class TabCollect extends Tab
{
	protected static final int diffImageWidth = 489, diffImageHeight = 510;
	
	public	WdgLeftPane			leftPane;
	private WdgMiddleColumn 	middleColumn;
	private WdgRunTabs 			runTabFolder;
	private WdgCollectSequence	wcs;
	public  WdgRunTabs 			getRunTabFolder() { return runTabFolder; }
	
	private static TabCollect inst = null;
	public static TabCollect get() { return inst; }
	
	public TabCollect(Composite parent, int style)
	{
		super(parent,style);
		inst = this;
		
		setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(3,false);
		gl.verticalSpacing=0;
		setLayout(gl);

		leftPane = new WdgLeftPane(this);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.verticalSpan=2;
		leftPane.setLayoutData(gd);
		
		middleColumn = new WdgMiddleColumn(this);
		gd = new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false);
		gd.widthHint=185;
		gd.verticalSpan=1;
		middleColumn.setLayoutData(gd);
		
		runTabFolder = new WdgRunTabs(this, SWT.RIGHT, ModelCollectRunList.get(), DevCollectRunList.get(), new WdgRunTab.Factory());
		wcs = new WdgCollectSequence(this, SWT.NONE);
		updateSequenceSize(false);
	}
	
	@Override public void notifyIsVisible(boolean isVisible)
	{
		leftPane.notifyIsVisible(isVisible);
	}
	
	@Override
	public void connectListeners()
	{
		runTabFolder.connectListeners();
		leftPane.connectListeners();
		middleColumn.connectListeners();
		
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				updateEnableTabs();
			}
		});
		
		runTabFolder.addListener(new WdgRunTabs.Listener()
		{
			@Override public void update()
			{
				updateEnableTabs();
			}
		});
		
		DevSFGraph.init();
	}
	
	protected void updateEnableTabs()
	{
		final boolean enableRunTabs =
		(
			!runTabFolder.getRemoteOpInProgress() &&
			ModelCollectRunList.get().getCollectState() == CollectState.INACTIVE &&
			Client.isMaster()
		);
			
		GUI.asyncExec(new Runnable(){public void run()
		{
			runTabFolder.setEnabled(enableRunTabs);
		}});
	}
	
	public void submitFields()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			middleColumn.collectButtons.btnCollect.setFocus();
		}});
	}
	
	private Boolean seqCurrentlyWide = null;
	
	protected void toggleSequenceSize()
	{
		updateSequenceSize(!seqCurrentlyWide);
	}
	
	private void updateSequenceSize(boolean seqIsWide)
	{
		if (null != seqCurrentlyWide && seqIsWide == seqCurrentlyWide) return;
		
		GridData gd;
		
		if (seqIsWide)
		{
			gd 					= new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false);
			gd.widthHint		= 315;
			gd.verticalSpan		= 1;
			gd.heightHint 		= 332;
			runTabFolder.setLayoutData(gd);
			
			gd					= new GridData(SWT.FILL,SWT.FILL,false,true);
			gd.verticalSpan		= 1;
			gd.horizontalSpan	= 2;
			wcs.setLayoutData(gd);
			
			wcs.group.setText("Run sequence <<");
			wcs.group.setToolTipText("Click below title to restore normal size");
		}
		else
		{
			gd 					= new GridData(SWT.DEFAULT, SWT.FILL, false, true);
			gd.widthHint		= 315;
			gd.verticalSpan		= 2;
			runTabFolder.setLayoutData(gd);
			
			gd					= new GridData(SWT.DEFAULT,SWT.FILL,false,true);
			gd.widthHint		= 185;
			gd.verticalSpan		= 1;
			wcs.setLayoutData(gd);
			
			wcs.group.setText("Run sequence >>");
			wcs.group.setToolTipText("Click below title to expand");
		}

		layout(true);
		wcs.redistributeColumns(seqIsWide);
		seqCurrentlyWide = seqIsWide;
	}
	
	@Override public String getTabName() { return "Collect"; }
}
