package gov.anl.gmca.collect.ui;

import gov.anl.gmca.analysis.TabAnalysisNG;
import gov.anl.gmca.analysis.helpers.ExportStrategy;
import gov.anl.gmca.analysis.helpers.StrategyResults;
import gov.anl.gmca.collect.DevCollectRun;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.RunCalculator.Frame;
import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.common.ui.WdgControlEntry.EntryType;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable;
import gov.anl.gmca.util.Util;
import gov.anl.gmca.util.WidgetFactory;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import java.util.ArrayList;

public class WdgCollectEndPoints extends WdgControl
{
	protected WdgControlEntry startFrameEntry, startGonioEntry, endFrameEntry, endGonioEntry;
	protected Composite startFrameFiller, endFrameFiller, headerFiller;
	protected Button startButton, endButton, strategyCheckbox, btnNewRun;
	protected Label startLabel, endLabel;
	
	protected DevCollectRun runDev;
	protected ModelCollectRun run;
	protected int tabIndex;
	protected static StrategyResults sr = null;

	public WdgCollectEndPoints(Composite parent, int style, ModelCollectRun _run, DevCollectRun _runDev, int tabIndex)
	{
		super(parent, style, null);
		runDev=_runDev;
		run=_run;
		this.tabIndex=tabIndex;
		
		GridLayout gl = new GridLayout(5,false);
		gl.verticalSpacing = 0;
		setLayout(gl);
		setBackground(GUI.BGCOLOR);

		///////////////////////////////////////////////////////////////////////////////////////
		////////////////////////////////////// labels /////////////////////////////////////////
		///////////////////////////////////////////////////////////////////////////////////////

		Composite leftFiller = new Composite(this, SWT.NONE);
		leftFiller.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.DEFAULT, SWT.FILL, false, true, 1, 3);
		gd.widthHint = 7;
		leftFiller.setLayoutData(gd);
		
		Label emptyLabel = new Label(this,SWT.NONE);
		emptyLabel.setBackground(GUI.BGCOLOR);
		gd = new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false);
		gd.widthHint = 60;
		emptyLabel.setLayoutData(gd);
		
		Label frameLabel = WidgetFactory.newLabel(this, SWT.CENTER, "Frame");
		frameLabel.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		Label gonioLabel = WidgetFactory.newLabel(this, SWT.CENTER, "Gonio");
		gonioLabel.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));

		Composite rightFiller = new Composite(this, SWT.NONE);
		rightFiller.setBackground(GUI.BGCOLOR);
		gd = new GridData(SWT.DEFAULT, SWT.FILL, false, true, 1, 3);
		gd.widthHint = 10;
		rightFiller.setLayoutData(gd);
		
		///////////////////////////////////////////////////////////////////////////////////////
		////////////////////////////////// start point ////////////////////////////////////////
		///////////////////////////////////////////////////////////////////////////////////////
		
		startLabel = WidgetFactory.newLabel(this, SWT.RIGHT, (0==tabIndex)?"":"Start: ");
		startLabel.setBackground(GUI.BGCOLOR);
		gd = new GridData(SWT.DEFAULT, SWT.CENTER, false, false);
		gd.widthHint = 60;
		startLabel.setLayoutData(gd);
		
		startFrameEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value) { run.fid.changeStartFrame(((Double)value).intValue()); }
			@Override public Limits getLimits() { return new Limits(1,10000,"start frame"); }
			@Override public String formatValue(Object o) { return Util.sprintf("%03d",((Double)o).intValue()); }
			@Override public Object getModelValue() { return Double.valueOf(run.getStartFrameLabel()); }
		});
		gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		gd.widthHint = 100;
		startFrameEntry.setLayoutData(gd);
		
		startGonioEntry = new WdgControlEntry(this, SWT.NONE, 0==tabIndex?EntryType.RW_COMBO:EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value) { run.fid.changeStartAngle((Double)value); }
			@Override public Limits getLimits() { return DevPmacMotor.get("gonio_omega").getLimits(); }
			@Override public int getDecimals() { return 2; }
			@Override public Object getModelValue() { return run.getStartAngle(); }
			@Override public double[] getFloatChoices()
			{
				double[] allChoices = {run.getStartAngle()-90, run.getStartAngle()+90};
				Limits omegaLimits = DevPmacMotor.get("gonio_omega").getLimits();
				
				ArrayList<Double> validChoices = new ArrayList<Double>();
				for (double choice : allChoices)
				{
					if (choice >= omegaLimits.getLower() && choice <= omegaLimits.getUpper())
					{
						validChoices.add(choice);
					}
				}
				
				double[] ret = new double[validChoices.size()];

				for (int i=0;i<ret.length;i++) ret[i]=validChoices.get(i);
				return ret;
			}
		});
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.widthHint = 110;
		startGonioEntry.setLayoutData(gd);
		
		///////////////////////////////////////////////////////////////////////////////////////
		/////////////////////////////////// end point /////////////////////////////////////////
		///////////////////////////////////////////////////////////////////////////////////////
		
		endLabel = WidgetFactory.newLabel(this, SWT.RIGHT, (0==tabIndex)?"":"End: ");
		gd = new GridData(SWT.DEFAULT, SWT.CENTER, false, false);
		gd.widthHint = 60;
		endLabel.setLayoutData(gd);
		
		if (0 == tabIndex)
		{
    		endFrameEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
    		{
    			@Override public void setValue(Object value) {}
    			@Override public Limits getLimits() { return new Limits(1,10000,"start frame 2"); }
    			@Override public String formatValue(Object o) { return Util.sprintf("%03d",((Double)o).intValue()); }
    			@Override public Object getModelValue() { return Double.valueOf(run.getStartFrameLabel()+2); }
    		});
    		endFrameEntry.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
    		endGonioEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
    		{
    			@Override public void setValue(Object value) { runDev.setStartAngle2((Double)value,true); }
    			@Override public Limits getLimits() { return DevPmacMotor.get("gonio_omega").getLimits(); }
    			@Override public int getDecimals() { return 2; }
    			@Override public Object getModelValue() { return run.getStartAngle2(); }
    		});
    		endGonioEntry.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		}
		else
		{
    		endFrameEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
    		{
    			@Override public void setValue(Object value) { run.fid.changeEndFrame(((Double)value).intValue()); }
    			@Override public Limits getLimits() { return new Limits(1,20000,"end frame"); }
    			@Override public String formatValue(Object o) { return Util.sprintf("%03d",((Double)o).intValue()); }
    			@Override public Object getModelValue() { return Double.valueOf(run.getEndFrameLabel()); }
    		});
    		endFrameEntry.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
    		endGonioEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
    		{
    			@Override public void setValue(Object value) { run.fid.changeEndAngle((Double)value); }
    			@Override public Limits getLimits() { return DevPmacMotor.get("gonio_omega").getLimits(); }
    			@Override public int getDecimals() { return 2; }
    			@Override public Object getModelValue() { return run.getEndAngle(); }
    		});
    		endGonioEntry.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		}
		
		if (0 == run.getRunIndex())
		{
			createStrategySelections();
		}
	}

	private void createStrategySelections() {

		//ui
		Group strategyGroup = new Group(this, SWT.BORDER);
		strategyGroup.setText("Strategy");
		strategyGroup.setFont(GUI.FONT_COLLECT_GROUP);

		GridLayout gl = new GridLayout(5, false);
		strategyGroup.setLayout(gl);
		GridData gd = new GridData(SWT.CENTER, SWT.DEFAULT, false, false, 5, 6);
		strategyGroup.setLayoutData(gd);

		strategyCheckbox = new Button(strategyGroup, SWT.CHECK);
		strategyCheckbox.setText("Collect 2 images 90 apart");
		strategyCheckbox.setToolTipText("Collect two images 90 deg apart for indexing/strategy calculation");
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false, 5, 1);
		gd.widthHint = 210;
		strategyCheckbox.setLayoutData(gd);
		strategyCheckbox.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				boolean enable = strategyCheckbox.getSelection();
				run.fii.changeStrategyEnabled(enable);
			}
		});

		Button btnShowStrategy = new Button(strategyGroup, SWT.NONE);
		btnShowStrategy.setText("Inspect Strategy Results");
		btnShowStrategy.setToolTipText("open analysis tab to check indexing results in details\nor choose a different strategy.");
		btnShowStrategy.setLayoutData(gd);
		btnShowStrategy.setBackground(GUI.COLOR_BLUE_BUTTON);
		btnShowStrategy.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetSelected(SelectionEvent selectionEvent) {
				Log.note("Inspect strategy results in the analysis tab");
				//TabAnalysisNG.newDetached(1);
				TabAnalysisNG.setVisibleTab(1);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent selectionEvent) {

			}
		});

		btnNewRun = new Button(strategyGroup, SWT.NONE);
		btnNewRun.setBackground(GUI.COLOR_BLUE_BUTTON);
		btnNewRun.setText("Create New Run");
		btnNewRun.setToolTipText("Create a new run with strategy results displayed below.");
		btnNewRun.setLayoutData(gd);

		btnNewRun.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetSelected(SelectionEvent selectionEvent) {
				Log.note("Create a new run with strategy results, requested by via Collect tab 0.");
				new ExportStrategy(sr, true);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent selectionEvent) {

			}
		});

		/*
		Button btnAuto = new Button(strategyGroup, SWT.NONE);
		btnAuto.setBackground(GUI.COLOR_BLUE_BUTTON);
		btnAuto.setText("Auto Collect");
		btnAuto.setToolTipText("create a new run & start data collection.");
		btnAuto.setLayoutData(gd);

		btnAuto.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetSelected(SelectionEvent selectionEvent) {
				Log.note("create a new run & start data collection");
				new ExportStrategy(sr, true);
				ModelCollectRunList.get().startCollection();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent selectionEvent) {

			}
		});
		 */

		//		Label jobStatusLabel = new Label(strategyGroup, SWT.WRAP);
		//		jobStatusLabel.setText(Epics.getString(PVLookup.PV_STRATEGY.strategyJobStatus));
		//		jobStatusLabel.setLayoutData(gd);

		Label strategyResultsLabel = new Label(strategyGroup, SWT.WRAP);
		strategyResultsLabel.setBackground(GUI.COLOR_LIGHT_GREEN);
		gd = new GridData(SWT.CENTER,
				SWT.DEFAULT,
				false,
				false,
				5,
				2);
		strategyResultsLabel.setLayoutData(gd);

		String sout = getStrategy();
		Log.note("Found strategy? :" + sout);
		if (sr == null) {
			strategyResultsLabel.setText(Epics.getString(PVLookup.PV_STRATEGY.strategyJobStatus));
		} else {
			strategyResultsLabel.setText(sout);
		}
		strategyResultsLabel.requestLayout();
		btnNewRun.setEnabled(false);

		Epics.EpicsValueMonitor monitor = new Epics.EpicsValueMonitor() {
			@Override
			public void valueChanged(String pvName, DBR value) {
				final String _pvName = pvName;
				final DBR _value = value;

				if (_pvName.equals(PVLookup.PV_STRATEGY.strategyJobStatus.toString())) {
					final String jobStatus = Epics.dbrToString(_value);
					GUI.asyncExec(new Runnable() {
						@Override
						public void run() {
							strategyResultsLabel.setText(jobStatus);
							strategyResultsLabel.requestLayout();
						}
					});
				}

				if (_pvName.equals(PVLookup.PV_STRATEGY.strategyLastXtal.toString())) {
					// ignore PV reset by empty string
					if (Epics.dbrToString(_value).equals("")
							|| Epics.getString(PVLookup.PV_ROBOT.mounted).equals("none")) {
						return;
					}

					GUI.asyncExec(new Runnable() {
						@Override
						public void run() {
							String sout = getStrategy();
							strategyResultsLabel.setText(sout);
							strategyResultsLabel.requestLayout();
							if (sr !=null && ModelUserTable.Client.isMaster()) {
								btnNewRun.setEnabled(true);
							}
						}
					});
				}

				if (_pvName.equals(PVLookup.PV_ROBOT.mounted.toString())) {
					final String mounted = Epics.dbrToString(_value);
					GUI.asyncExec(new Runnable() {
						@Override
						public void run() {
							if (mounted.equals("none") && ModelUserTable.Client.isMaster()) {
								btnNewRun.setEnabled(false);
								sr = null;
								Epics.put(PVLookup.PV_STRATEGY.strategyJobStatus, "Strategy results:");
							}
						}
					});
				}

			}
		};

		Epics.addMonitor(PVLookup.PV_STRATEGY.strategyLastXtal, monitor);
		Epics.addMonitor(PVLookup.PV_STRATEGY.strategyJobStatus, monitor);
		Epics.addMonitor(PVLookup.PV_ROBOT.mounted, monitor);

	}

	private String getStrategy() {
		String failedStrategy = "Failed to obtain strategy results.";
		String mounted = Epics.getString(PVLookup.PV_ROBOT.mounted);

		if (mounted.equals("none")) {
			return failedStrategy + ", no mounted crystal.";
		}

		// get prefix from tab 0
		String prefix = run.getFileRoot();
		if (prefix == null || prefix.length() == 0) {
			Log.warn("getStrategy: unable to get prefix, use EPICS run0 value.");
			prefix = Epics.getString(PVLookup.PV_RUN.fileroot.run(0));
		}

		if (prefix.length() == 0) {
			Log.note("getStrategy: unable to get prefix, aborted.");
			return failedStrategy;
		}

		String strategyCrystal = Epics.getString(PVLookup.PV_STRATEGY.strategyLastXtal);
		if ( prefix != null && strategyCrystal.startsWith(prefix)) {
			sr = new StrategyResults(prefix, null, 4);
			if (sr.getOscStart() != null) {
				return sr.toString();
			}
		}
		return failedStrategy;
	}

	protected boolean enable = false;
	
	@Override
	public void setEnabled(boolean _enable)
	{
		enable = _enable;
		updateWidgetState();
	}

	protected void updateWidgetState()
	{
		if (null!=strategyCheckbox) strategyCheckbox.setEnabled(enable);

		boolean endFrameEnable = (run.getRunIndex()==0) ?false :enable;
		
		if (null!=startFrameEntry) startFrameEntry.setEnabled(enable);
		if (null!=startGonioEntry) startGonioEntry.setEnabled(enable);
		if (null!=endGonioEntry) endGonioEntry.setEnabled(enable);
		if (null!=endFrameEntry) endFrameEntry.setEnabled(endFrameEnable);

		if (run.getRunIndex() == 0 && sr != null) {
			btnNewRun.setEnabled(enable);
		}
	}
	
	public void setStartAngle(double d) { if (null!=startGonioEntry) startGonioEntry.setText(d); 			}
	public void setStartFrame(int i)	{ if (null!=startFrameEntry) startFrameEntry.setText((double)i); 	}
	public void setEndAngle(double d) 	{ if (null!=endGonioEntry) endGonioEntry.setText(d); }
	public void setEndFrame(int i)		{ if (null!=endFrameEntry) endFrameEntry.setText((double)i); }
	
	protected void runChanged()
	{
		setStartAngle(run.getStartAngle());
		setStartFrame(run.getStartFrameLabel());
		
		int endFrameDisplay = (run.getRunIndex()==0&&strategyFrameShown)?run.getStartFrameLabel()+2:run.getEndFrameLabel();
		
		ArrayList<Frame> listCopy = new ArrayList<Frame>(run.seq().getFrameListCopy());
		double lastFrameEndAngle = Double.NaN;
		if (run.getRunIndex()!=0 && run.getSiteCheckEnable() && listCopy.size()!=0)
		{
			int lastIndex = listCopy.size()-1;
			Frame lastFrame = listCopy.get(lastIndex);
			lastFrameEndAngle = lastFrame.angle + run.getDelta();
		}
		
		double endAngleDisplay =
			(run.getRunIndex()==0&&strategyFrameShown)
    			?run.getStartAngle2()
    			:run.getSiteCheckEnable()
					?lastFrameEndAngle
    				:run.getEndAngle();
    			
		setEndFrame(endFrameDisplay);
		setEndAngle(endAngleDisplay);
		
		if (0==run.getRunIndex())
		{
    		boolean strategyEnabled = (run.getRunIndex()==0 && run.getStrategyEnabled());
    		strategyCheckbox.setSelection(strategyEnabled);
    		showStrategyFrame(strategyEnabled);
    		startGonioEntry.reloadChoices();
		}
	}
	
	protected boolean strategyFrameShown=true;
	
	protected void showStrategyFrame(final boolean show)
	{
		if (0 != run.getRunIndex()) return;
		
		System.out.println("showStrategyFrame: "+show);
		
		WdgControl.setControlVisible(endLabel, show);
		WdgControl.setControlVisible(endFrameEntry, show);
		WdgControl.setControlVisible(endGonioEntry, show);
		WdgControl.setControlVisible(endFrameFiller, show);
		
		startLabel.setText(show?"Start 1:":"Start:");
		endLabel.setText(show?"Start 2:":"");
		
		layout(true);
		getParent().layout(true);
	}
	
	@Override public boolean updateModel() { return true; }
}
