package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.DevCollectSettings;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.collect.ModelCollectRun.SetupStep;
import gov.anl.gmca.collect.ModelCollectRun.XDSProcMode;
import gov.anl.gmca.common.EpicsDevice;
import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgControlCheckbox;
import gov.anl.gmca.common.ui.WdgControlDir;
import gov.anl.gmca.common.ui.WdgControlDir.PrefixDir;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.common.ui.WdgControlEntry.EntryType;
import gov.anl.gmca.common.ui.WdgControlEntry.ValidateResult;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevAttenuation;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.hutch.ModelPmacMotor;
import gov.anl.gmca.raster.ModelRasterSiteList;
import gov.anl.gmca.raster.ModelRasterSiteList.Site;
import gov.anl.gmca.sample.DBBeamSize;
import gov.anl.gmca.sample.DBBeamSize.BeamSize;
import gov.anl.gmca.sample.ModelBeamsize;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import gov.anl.gmca.util.WidgetFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import java.util.ArrayList;

public class WdgCollectParams extends Composite
{
	private static final int 			MAX_ENERGY_FIELDS = 3; // 5 or less

	private Combo						beamsizeCombo, processingCombo;
	private WdgControlCheckbox			inverseBeamCheckbox;
	private WdgControlEntry 			wedgeEntry, siteWedgeEntry, xdsProcessingEntry, siteSelectEntry, beamWidthEntry, beamHeightEntry;
	private ModelCollectRun 			run;
	private WdgControlDir				controlDir;
	private Label						inverseBeamLabel, inverseBeamUnits, wedgeLabel, wedgeUnits, siteWedgeLabel, siteWedgeUnits, deltaLabel, deltaUnits;
	private Label 						processingLabel, processingUnits;
	private Label						framesPerSiteLabel, framesPerSiteUnits, xdsProcessingLabel, xdsProcessingUnits, siteSelectLabel, siteSelectUnits;
	private Label						framesPerDirLabel, framesPerDirUnits;
	private WdgControlEntry 			distanceEntry, beamstopEntry, attenuationEntry, deltaEntry, timeEntry, framesPerSiteEntry, framesPerDirEntry;
	private WdgCollectEndPoints 		endPoints;
	private WdgCollectEnergy 			energy1Entry, energy2Entry, energy3Entry, energy4Entry, energy5Entry;
	private ArrayList<WdgCollectEnergy> energyControls = new ArrayList<WdgCollectEnergy>();
	private	int							tabIndex;
	private Label 						siteDeltaLabel,siteSkipLabel,siteDeltaEnableLabel,siteDeltaUnits,siteDeltaEnableUnits,overlapLabel,overlapUnits,siteSkipUnits;
	private WdgControlEntry 			siteDeltaEntry,siteSkipEntry,overlapEntry;
	private WdgCenteredAngle			wdgCenteredAngle;
	private WdgControlCheckbox 			siteDeltaEnableCheckbox;
	private Label						vectorSetupLabel;
	private ListenerList.Listener<ModelRasterSiteList> siteListener;
	
	public WdgCollectParams(Composite parent, final ModelCollectRun run, int tabIndex)
	{
		super(parent, SWT.NONE);
		this.run=run; this.tabIndex=tabIndex;
		
		compSetup();
		
		createPrefixDir();
		createDistance();
		//createBeamstop();
		createAttenuation();
		createSiteSelect();
		createBeamSize();
		//WdgCollectRunTab.createSpacer(this, 5, 5);
		
		createDelta();
		createTime();
		createCenteredAngle();
		createStartEndFrameAngle();
		createFramesPerDir();
		createFPS();
		//createXDSProc();
		createXDSProcDropdown();
		//WdgCollectRunTab.createSpacer(this, 5, 5);
		
		createOverlap();
		createSiteCheck();
		createInvBeam();
		createWedge();
		createEnergies();
		createVectorSetupLabel();
		
		updateVisibility();
	
		connectListeners();
	}
	
	private void connectListeners()
	{
		siteListener = new ListenerList.Listener<ModelRasterSiteList>()
		{
			@Override public void handleEvent(ModelRasterSiteList object, Object data)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					// check for new sites for dropdown
					int siteCount = ModelRasterSiteList.get().getSiteListCopy().size();
					int itemCount = siteSelectEntry.getComboItems().length;
					if (siteCount+1 != itemCount)
					{
						int selectedIndex = siteSelectEntry.getSelection();
						siteSelectEntry.reloadChoices();
						if (selectedIndex >= siteSelectEntry.getComboItems().length) selectedIndex = 0;
						siteSelectEntry.setSelection(selectedIndex);
					}
				}});
			}
		};
		ModelRasterSiteList.get().ll.addListener(siteListener);

		if (!Util.beamSizeCalculated())
		{
			final ListenerList.Listener<ModelBeamsize> beamsizeListener = new ListenerList.Listener<ModelBeamsize>() {
				@Override
				public void handleEvent(ModelBeamsize object, Object data) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							if (null!=beamWidthEntry) beamWidthEntry.updateHighlight();
							if (null!=beamHeightEntry) beamHeightEntry.updateHighlight();
						}
					});
				}
			};
			ModelBeamsize.get().ll.addListener(beamsizeListener);
			addDisposeListener(new DisposeListener()
			{
				@Override public void widgetDisposed(DisposeEvent e)
				{
					ModelBeamsize.get().ll.removeListener(beamsizeListener);
				}
			});
		}

		addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent e)
			{
				ModelRasterSiteList.get().ll.removeListener(siteListener);
			}
		});
	}
	
	private void compSetup()
	{
		setBackground(GUI.BGCOLOR);
		GridLayout paramLayout = new GridLayout(5,false);
		paramLayout.marginWidth = 0;
		paramLayout.marginHeight = 0;
		paramLayout.verticalSpacing = 0;
		setLayout(paramLayout);
	}
	
	private void createPrefixDir()
	{
		controlDir = new WdgControlDir(this, SWT.NONE, null, 0, new WdgControlDir.ModelInterface()
		{
			@Override public void setValue(Object value)
			{
				WdgControlDir.PrefixDir prefixDir = (WdgControlDir.PrefixDir)value;
				run.dev().setFileroot(prefixDir.prefix,false);
				run.dev().setDirectorySuffix(prefixDir.dir,true);
			}
			@Override public PrefixDir getPrefixDir()
			{
				return new WdgControlDir.PrefixDir(run.getFileRoot(), run.getDirectorySuffix());
			}
		});
		controlDir.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 5, 1));
	}
	
	private void createDistance()
	{
		WidgetFactory.newLabel(this, SWT.RIGHT, "Distance:").setLayoutData(WdgRunTab.newGDLeftLabel());
		distanceEntry = new WdgControlEntry(this, SWT.NONE, EntryType.RW_COMBO,  new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.dev().setDistance((Double)value,true);							}
			@Override public Object getModelValue() 		{ return run.getdistance_mm();										}
			@Override public EpicsDevice getDevice()		{ return DevPmacMotor.get("detector_z");							}
			@Override public Object getDeviceValue() 		{ return DevPmacMotor.get("detector_z").getPosition(); 				}
			@Override public Limits getLimits() 			{ return DevPmacMotor.get("detector_z").getLimits(); 				}
			@Override public int getDecimals() 				{ return 2; 														}
			@Override public double[] getFloatChoices()		{ return ModelPmacMotor.getDetectorDistanceOptions();				}
			@Override public boolean doHighlight(double d)  { return Math.abs((Double)getDeviceValue()-d)>0.01; }
		});
		distanceEntry.setLayoutData(WdgRunTab.newGDEntry());
		WidgetFactory.newLabel(this, SWT.LEFT, "mm").setLayoutData(WdgRunTab.newGDRightLabel());
	}
	
	private void createBeamstop()
	{
		WidgetFactory.newLabel(this, SWT.RIGHT, "Beamstop:").setLayoutData(WdgRunTab.newGDLeftLabel());
		beamstopEntry = new WdgControlEntry(this, SWT.NONE, EntryType.RW_COMBO,  new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.dev().setBeamstop_mm((Double)value, true);					}
			@Override public Object getModelValue()			{ return run.getBeamstop_mm();									}
			@Override public EpicsDevice getDevice()		{ return DevPmacMotor.get("beamstop_z");							}
			@Override public Object getDeviceValue() 		{ return DevPmacMotor.get("beamstop_z").getPosition();			}
			@Override public Limits getLimits() 			{ return DevPmacMotor.get("beamstop_z").getLimits();			}
			@Override public double[] getFloatChoices()		{ return new double[]{35,40,45,50,55,60,65,70};					}
			@Override public int getDecimals() 				{ return 3;														}
		});
		beamstopEntry.setLayoutData(WdgRunTab.newGDEntry());
		WidgetFactory.newLabel(this, SWT.LEFT, "mm").setLayoutData(WdgRunTab.newGDRightLabel());
	}
	
	private void createAttenuation()
	{
		WidgetFactory.newLabel(this, SWT.RIGHT, "Atten.:").setLayoutData(WdgRunTab.newGDLeftLabel());
		attenuationEntry = new WdgControlEntry(this, SWT.NONE, EntryType.RW_COMBO,  new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.dev().setAttenuation_factors((Double)value, true);				}
			@Override public Object getModelValue()			{ return run.getAttenuation_factors();								}
			@Override public EpicsDevice getDevice()		{ return DevAttenuation.get();										}
			@Override public Object getDeviceValue() 		{ return DevAttenuation.get().getPosition();						}
			@Override public Limits getLimits() 			{ return DevAttenuation.get().getLimits();							}
			@Override public int getDecimals() 				{ return 2;					 										}
			@Override public double[] getFloatChoices()		{ return new double[]{1,5,10,20,50,100,1000};					}
			@Override public boolean doHighlight(double w)  { double d=(Double)getDeviceValue(); return (Math.abs(w-d)/d)>0.05;	}
		});
		attenuationEntry.setLayoutData(WdgRunTab.newGDEntry());
		WidgetFactory.newLabel(this, SWT.LEFT, "factor").setLayoutData(WdgRunTab.newGDRightLabel());
	}
	
	private void createSiteSelect()
	{
		siteSelectLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Site:", "Raster site"); siteSelectLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		siteSelectEntry = new WdgControlEntry(this, SWT.NONE, EntryType.RO_COMBO, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)
			{
				// get site index from option string
				String s = (String)value;
				int siteIndex=-1;
				if (-1 != s.indexOf('['))
				{
					String indexStr = s.substring(1, s.indexOf(']'));
					siteIndex = Integer.parseInt(indexStr)-1;
				}
				
				// set settings that will be hidden
				run.dev().setOverlap(0, false);
				run.dev().setSiteCheckEnable(false, false);
				run.dev().setFramesPerSite(1, false);
				
				// update raster mode
				ScanMode scanMode = (-1 == siteIndex) ? ScanMode.SINGLE : ScanMode.RASTER;
				SetupStep setupStep = (-1 == siteIndex) ? SetupStep.SITE_SETUP : SetupStep.COLLECT_PARAM;
				run.dev().setScanMode(scanMode, false);
				run.dev().setSetupStep(setupStep, false);
				
				// update selected sites
				int siteCount = ModelRasterSiteList.get().getHighestSiteNumber();
				if (siteCount==-1) siteCount=0;
				ArrayList<Boolean> selectedSites = new ArrayList<Boolean>(siteCount);
				for (int i=0;i<selectedSites.size();i++) { selectedSites.add(i==siteIndex); }
				run.dev().setSelectedSites(selectedSites, true);
			}
			@Override public String[] getChoices()
			{
				ArrayList<Site> siteList = ModelRasterSiteList.get().getSiteListCopy();
				String[] ret = new String[siteList.size()+1];
				ret[0]="None";
				for (int i=0;i<siteList.size();i++)
				{
					ret[i+1]="["+(i+1)+"] "+siteList.get(i).getSource().id;
				}
				return ret;
			}
			@Override public ValidateResult validate(String s)	{ return new ValidateResult(true); }
		});
		siteSelectEntry.setLayoutData(WdgRunTab.newGDEntry());
		siteSelectEntry.setSelection(0);
		siteSelectUnits = WidgetFactory.newLabel(this, SWT.LEFT, ""); siteSelectUnits.setLayoutData(WdgRunTab.newGDRightLabel());
	}

	private void createBeamSizeFields()
	{
		Composite beamSizeComp = new Composite(this, SWT.NONE);
		beamSizeComp.setLayoutData(WdgRunTab.newGDEntry());
		beamSizeComp.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(3,false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		beamSizeComp.setLayout(gl);
		beamWidthEntry = new WdgControlEntry(beamSizeComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.fii.changeBeamWidth_um((Double)value, true); }
			@Override public Object getModelValue() 		{ return run.getBeamWidth_um(); }
			@Override public Object getDeviceValue() 		{ return ModelBeamsize.getCurrentBeamsize().x; }
			@Override public int getDecimals() 				{ return 0; }
			@Override public boolean doHighlight(double d)  { return Math.abs((Double)getDeviceValue()-d)>1.0; }
			@Override public Limits getLimits()
			{
				double lowLimit = 5.0;
				double highLimit = ModelBeamsize.getActualBeamsize_um().x;
				return new Limits(lowLimit, "minimum beam width", highLimit, "maximum beam width");
			}
		});
		beamWidthEntry.setLayoutData(new GridData(SWT.FILL,SWT.DEFAULT,true,false));
		WidgetFactory.newLabel(beamSizeComp, SWT.NONE, "x").setLayoutData(new GridData());
		beamHeightEntry = new WdgControlEntry(beamSizeComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.fii.changeBeamHeight_um((Double)value, true);}
			@Override public Object getModelValue() 		{ return run.getBeamHeight_um(); 		}
			@Override public Object getDeviceValue() 		{ return ModelBeamsize.getCurrentBeamsize().y; }
			@Override public boolean doHighlight(double d)  { return Math.abs((Double)getDeviceValue()-d)>1.0; }
			@Override public int getDecimals() 				{ return 0; 									}
			@Override public Limits getLimits()
			{
				double lowLimit = 5.0;
				double highLimit = ModelBeamsize.getActualBeamsize_um().y;
				return new Limits(lowLimit, "minimum beam height", highLimit, "maximum beam height");
			}
		});
		beamHeightEntry.setLayoutData(new GridData(SWT.FILL,SWT.DEFAULT,true,false));
	}



	private void createBeamSizeDropdown()
	{
		beamsizeCombo = new Combo(this, SWT.READ_ONLY);
		beamsizeCombo.setLayoutData(WdgRunTab.newGDEntry());
		for (int i = 0; i< DBBeamSize.colli_sizes.length; i++) beamsizeCombo.add("");
		updateBeamSizeDisplay();
		beamsizeCombo.select(0);
		beamsizeCombo.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent selectionEvent)
			{
				applyBeamSizeDropdown(false);
			}
			@Override public void widgetDefaultSelected(SelectionEvent selectionEvent) {}
		});
	}

	private void applyBeamSizeDropdown(boolean updateModelOnly)
	{
		int selected = beamsizeCombo.getSelectionIndex();
		int beamsize = DBBeamSize.colli_sizes[selected];
		if (beamsize != run.getBeamWidth_um()) {
			if (updateModelOnly) {
				run.setBeamWidth_um(beamsize);
				run.setBeamHeight_um(beamsize);
			} else {
				run.dev().setBeamWidth_um(beamsize, false);
				run.dev().setBeamHeight_um(beamsize, true);
			}
		}
	}

	private void updateBeamSizeDisplay()
	{
		java.util.List<Double> energies = run.getEnergies();
		if (energies.size()==0) return;

		ArrayList<BeamSize> sizes = DBBeamSize.getBeamSizes(energies.get(0));
		for (int i=0;i<sizes.size();i++) {
			BeamSize size = sizes.get(i);
			String text = Util.sprintf("%d x %d", Math.round(size.width_um), Math.round(size.height_um));
			beamsizeCombo.setItem(i, text);
		}
		updateBeamsizeDropdownSelect();
	}

	private void updateBeamsizeDropdownSelect()
	{
		int colli_index = DBBeamSize.getColliIndex((int)run.getBeamWidth_um());
		if (beamsizeCombo.getSelectionIndex()!=colli_index) beamsizeCombo.select(colli_index);
	}

	private void createBeamSize()
	{
		WidgetFactory.newLabel(this, SWT.RIGHT, "Beam size:").setLayoutData(WdgRunTab.newGDLeftLabel());
		if (Util.beamSizeCalculated())
			createBeamSizeDropdown();
		else
			createBeamSizeFields();
		WidgetFactory.newLabel(this, SWT.LEFT, "\u00B5m").setLayoutData(WdgRunTab.newGDRightLabel());
	}
	
	private void createDelta()
	{
		deltaLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Delta:"); deltaLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		deltaEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value) 	{ run.fid.changeDelta((Double)value); 			}
			@Override public Object getModelValue()			{ return run.getDelta();						}
			@Override public Limits getLimits() 			{ return DevCollectSettings.getCollectDeltaLimits(); 	}
			@Override public int getDecimals() 				{ return 2; 									}
		});
		deltaEntry.setLayoutData(WdgRunTab.newGDEntry());
		deltaUnits = WidgetFactory.newLabel(this, SWT.LEFT, "deg"); deltaUnits.setLayoutData(WdgRunTab.newGDRightLabel());
	}
	
	private void createTime()
	{
		WidgetFactory.newLabel(this, SWT.RIGHT, "Time:").setLayoutData(WdgRunTab.newGDLeftLabel());
		timeEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.dev().setExposureTime((Double)value,true);	}
			@Override public Object getModelValue()			{ return run.getExposureTime();					}
			@Override public Limits getLimits() 			{ return DevCollectSettings.getCollectExposureLimits(); }
			@Override public int getDecimals() 				{ return 4; 									}
		});
		timeEntry.setLayoutData(WdgRunTab.newGDEntry());
		WidgetFactory.newLabel(this, SWT.LEFT, "sec").setLayoutData(WdgRunTab.newGDRightLabel());
	}

	private void createCenteredAngle()
	{
		wdgCenteredAngle = new WdgCenteredAngle(this, run);
		wdgCenteredAngle.setVisible(false);
	}

	private void createStartEndFrameAngle()
	{
		endPoints = new WdgCollectEndPoints(this, SWT.NONE, run, run.dev(), tabIndex);
		endPoints.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 5, 1));
	}

	private void createFramesPerDir()
	{
		framesPerDirLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Frames/dir:"); framesPerDirLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		framesPerDirEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public Object getModelValue()			{ return run.getFramesPerDir();								}
			@Override public Limits getLimits() 			{ return new Limits(1,1000000,"frames per dir"); 			}
			@Override public String formatValue(Object o)	{ return Util.sprintf("%d",((Integer)o).intValue()); 		}
			@Override public void setValue(Object value)	{ run.fid.changeFramesPerSite(((Integer)value).intValue(), true, true);}
		});
		framesPerDirEntry.setLayoutData(WdgRunTab.newGDEntry());
		framesPerDirUnits = WidgetFactory.newLabel(this, SWT.LEFT, "fr.", "frames"); framesPerDirUnits.setLayoutData(WdgRunTab.newGDRightLabel());
	}

	private void createFPS()
	{
		framesPerSiteLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Fr./site:"); framesPerSiteLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		framesPerSiteEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public Object getModelValue()			{ return run.getFramesPerSite();							}
			@Override public Limits getLimits() 			{ return new Limits(1,10000,"frames per site"); 			}
			@Override public String formatValue(Object o)	{ return Util.sprintf("%03d",((Integer)o).intValue()); 		}
			@Override public void setValue(Object value)	{ run.fid.changeFramesPerSite(((Integer)value).intValue(), true, true);}
		});
		framesPerSiteEntry.setLayoutData(WdgRunTab.newGDEntry());
		framesPerSiteUnits = WidgetFactory.newLabel(this, SWT.LEFT, "fr.", "frames"); framesPerSiteUnits.setLayoutData(WdgRunTab.newGDRightLabel());
	}
	
	private void createXDSProc()
	{
		xdsProcessingLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "XDS Proc:", "XDS Processing");
		xdsProcessingLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		xdsProcessingEntry = new WdgControlEntry(this, SWT.NONE, EntryType.RO_COMBO, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)		{ run.dev().setXDSProcMode(XDSProcMode.fromString((String)value),true);	}
			@Override public String[] getChoices()				{ String[] ret=new String[XDSProcMode.values().length]; for (int i=0;i<ret.length;i++) ret[i]=Util.capitalizeFirstLetters(XDSProcMode.values()[i].toString()); return ret; }
			@Override public ValidateResult validate(String s)	{ return new ValidateResult(true); }
		});
		xdsProcessingEntry.setLayoutData(WdgRunTab.newGDEntry());
		xdsProcessingEntry.setBackground(GUI.BGCOLOR);
		xdsProcessingEntry.setForeground(GUI.COLOR_BLACK);
		xdsProcessingEntry.setFont(GUI.FONT_9);
		xdsProcessingUnits = WidgetFactory.newLabel(this, SWT.LEFT, ""); xdsProcessingUnits.setLayoutData(WdgRunTab.newGDRightLabel());
	}

	private void createXDSProcDropdown() {
		processingLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Data Proc:");
		processingLabel.setLayoutData(WdgRunTab.newGDLeftLabel());

		processingCombo = new Combo(this, SWT.READ_ONLY);
		processingCombo.setLayoutData(WdgRunTab.newGDEntry());
		String[] choices=new String[XDSProcMode.values().length];
		for (int i=0;i<choices.length;i++) {
			choices[i]=Util.capitalizeFirstLetters(XDSProcMode.values()[i].toString());
		}
		processingCombo.setItems(choices);
		//processingCombo.setItems(new String[] {"None", "Native", "Anomalous"});

		processingCombo.select(0);
		processingCombo.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent selectionEvent)
			{
				String value = processingCombo.getText();
				run.dev().setXDSProcMode(XDSProcMode.fromString(value),true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent selectionEvent) {}
		});
		processingUnits = WidgetFactory.newLabel(this, SWT.LEFT, "");
		processingUnits.setLayoutData(WdgRunTab.newGDRightLabel());
	}
	
	private void createOverlap()
	{
		overlapLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Overlap:"); overlapLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		overlapEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.dev().setOverlap(((Integer)value).intValue(),true);	}
			@Override public void updateModel(Object value)	{ run.setOverlap(((Integer)value).intValue());			}
			@Override public Object getModelValue()			{ return run.getOverlap();								}
			@Override public Limits getLimits() 			{ return new Limits(0,"minimum overlap value",run.getFramesPerSite(),"selected frames per site"); }
			@Override public String formatValue(Object o) 	{ return Util.sprintf("%03d",((Integer)o).intValue()); 					}
		});
		overlapEntry.setLayoutData(WdgRunTab.newGDEntry());
		overlapUnits = WidgetFactory.newLabel(this, SWT.LEFT, "fr.", "frames"); overlapUnits.setLayoutData(WdgRunTab.newGDRightLabel());
	}
	
	private void createSiteCheck()
	{
		siteDeltaEnableLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Site check:"); siteDeltaEnableLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		siteDeltaEnableCheckbox = new WdgControlCheckbox(this, SWT.NONE, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.fii.changeSiteCheck((Boolean)value, true); 	}
			@Override public void updateModel(Object value)	{ run.setSiteCheckEnable((Boolean)value);			}
			@Override public Object getModelValue()			{ return run.getSiteCheckEnable();					}
		});
		siteDeltaEnableCheckbox.setLayoutData(WdgRunTab.newGDEntry());
		siteDeltaEnableUnits = WidgetFactory.newLabel(this, SWT.LEFT, ""); siteDeltaEnableUnits.setLayoutData(WdgRunTab.newGDRightLabel());

		siteSkipLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Site Skip:"); siteSkipLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		siteSkipEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.dev().setSiteSkip(((Integer)value).intValue(), true);		}
			@Override public void updateModel(Object value)	{ run.setSiteSkip(((Integer)value).intValue());					}
			@Override public Object getModelValue()			{ return run.getSiteSkip();										}
			@Override public Limits getLimits() 			{ return new Limits(0,"minimum site skip",1000,"maximum site skip"); 	}
		});
		siteSkipEntry.setLayoutData(WdgRunTab.newGDEntry());
		siteSkipUnits = WidgetFactory.newLabel(this, SWT.LEFT, "sites", "sites"); siteSkipUnits.setLayoutData(WdgRunTab.newGDRightLabel());
		
		siteDeltaLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Site Delta:"); siteDeltaLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		siteDeltaEntry = new WdgControlEntry(this, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.fii.changeSiteDelta(((Double)value).doubleValue()); 			}
			@Override public void updateModel(Object value)	{ run.setSiteDelta_deg(((Double)value).doubleValue());			}
			@Override public Object getModelValue()			{ return run.getSiteDelta_deg();								}
			@Override public Limits getLimits() 			{ return new Limits(0,"minimum site delta",180,"maximum site delta"); }
			@Override public String formatValue(Object o) 	{ return Util.printDouble((Double)o, 2); 			}
		});
		siteDeltaEntry.setLayoutData(WdgRunTab.newGDEntry());
		siteDeltaUnits = WidgetFactory.newLabel(this, SWT.LEFT, "deg.", "degrees"); siteDeltaUnits.setLayoutData(WdgRunTab.newGDRightLabel());
	}
	
	private void createInvBeam()
	{
		inverseBeamLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Inv. beam:", "Inverse beam"); inverseBeamLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		inverseBeamCheckbox = new WdgControlCheckbox(this, SWT.NONE, new WdgControlEntry.ModelInterface() {
			@Override
			public void setValue(Object value) {
				Boolean inverse = (Boolean) value;
				run.fii.changeInverseBeam(inverse, true);
			}
		});
		inverseBeamCheckbox.setLayoutData(WdgRunTab.newGDEntry());
		inverseBeamUnits = WidgetFactory.newLabel(this, SWT.LEFT, ""); inverseBeamUnits.setLayoutData(WdgRunTab.newGDRightLabel());
	}
	
	private void createWedge()
	{
		wedgeLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Wedge:"); wedgeLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		wedgeEntry = new WdgControlEntry(this, SWT.NONE, EntryType.RW_COMBO, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.fii.changeWedge((Double)value);	}
			@Override public Object getModelValue()			{ return run.getWedgeSize_deg();			}
			@Override public Limits getLimits() 			{ return DevCollectSettings.getWedgeLimits(); }
			@Override public int getDecimals() 				{ return 1; 								}
			@Override public double[] getFloatChoices()
			{
				double[] allOptions = {30,45,60,90,180};
				Limits wedgeLimits = DevCollectSettings.getWedgeLimits();
				
				ArrayList<Double> validOptions = new ArrayList<Double>();
				for (double option : allOptions)
				{
					if (option >= wedgeLimits.getLower() && option <= wedgeLimits.getUpper())
					{
						validOptions.add(option);
					}
				}
				
				double[] ret = new double[validOptions.size()];
				for (int i=0;i<ret.length;i++) ret[i]=validOptions.get(i);
				return ret;
			}
		});
		wedgeEntry.setLayoutData(WdgRunTab.newGDEntry());
		wedgeUnits = WidgetFactory.newLabel(this, SWT.LEFT, "deg"); wedgeUnits.setLayoutData(WdgRunTab.newGDRightLabel());
		
		siteWedgeLabel = WidgetFactory.newLabel(this, SWT.RIGHT, "Wedge:"); siteWedgeLabel.setLayoutData(WdgRunTab.newGDLeftLabel());
		siteWedgeEntry = new WdgControlEntry(this, SWT.NONE, EntryType.RW_COMBO, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.fii.changeWedgeSites((int)Math.round((Integer)value));	}
			@Override public Object getModelValue()			{ return run.getWedgeSize_sites();					}
			@Override public Limits getLimits() 			{ return new Limits(1, run.getSelectedSiteCount(), "site count");	}
			@Override public int getDecimals() 				{ return 0; 										}
			@Override public double[] getFloatChoices()
			{
				int total = run.getSelectedSiteCount();
				ArrayList<Integer> choices = new ArrayList<Integer>();
				
				if (total<10)
				{
					for (int i=1; i<=total; i++) choices.add(i);
				}
				else if (total < 50)
				{
					choices.add(1);
					for (int i=5; i<=total; i+=5) choices.add(i);
				}
				else if (total < 100)
				{
					choices.add(1);
					for (int i=10; i<=total; i+=10) choices.add(i);
				}
				else
				{
					choices.add(1);
					for (int i=50; i<=total; i+=50) choices.add(i);
				}
				
				double[] d_options = new double[choices.size()];
				for (int i=0;i<d_options.length;i++) d_options[i]=choices.get(i);
				return d_options;
			}
		});
		siteWedgeEntry.setLayoutData(WdgRunTab.newGDEntry());
		siteWedgeUnits = WidgetFactory.newLabel(this, SWT.LEFT, "sites"); siteWedgeUnits.setLayoutData(WdgRunTab.newGDRightLabel());
	}
	
	private void createEnergies()
	{
		String energy1labelText = (run.getRunIndex()==0)?"Energy:":"Energy 1:";
		
		Label energy1Label = WidgetFactory.newLabel(this, SWT.RIGHT, energy1labelText);
		energy1Label.setLayoutData(WdgRunTab.newGDLeftLabel());
		energy1Entry = new WdgCollectEnergy(this, SWT.NONE, true, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)		{ updateEnergies(1,value);																}
			@Override public Object getModelValue()				{ return (run.getEnergies().size()>=1)?run.getEnergies().get(0):Double.NaN;				}
			@Override public EpicsDevice getDevice()			{ return DevPmacMotor.get("mo_En_E");													}
			@Override public Object getDeviceValue() 			{ return DevPmacMotor.get("mo_En_E").getPosition();										}
			@Override public Limits getLimits() 				{ return DevPmacMotor.get("mo_En_E").getLimits();										}
			@Override public ValidateResult validate(String s)	{ return WdgCollectEnergy.energyValidate(1,s,DevPmacMotor.get("mo_En_E").getLimits());	}
			@Override public int getDecimals() 					{ return 4; 																			}
			@Override public boolean doHighlight(double w)		{ double d=(Double)getDeviceValue(); return Math.abs(w-d)>=DevPmacMotor.get("mo_En_E").getInPosBand(); }
		});
		energy1Entry.setLayoutData(WdgRunTab.newGDEntry());
		Label energy1Units = WidgetFactory.newLabel(this, SWT.LEFT, "keV");
		energy1Units.setLayoutData(WdgRunTab.newGDRightLabel());
		energy1Entry.setLabels(energy1Label, energy1Units);
		energyControls.add(energy1Entry);
		
		Label energy2Label = WidgetFactory.newLabel(this, SWT.RIGHT, "Energy 2:");
		energy2Label.setLayoutData(WdgRunTab.newGDLeftLabel());
		energy2Entry = new WdgCollectEnergy(this, SWT.NONE, false, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)		{ updateEnergies(2,value);										}
			@Override public Object getModelValue()				{ return (run.getEnergies().size()>=2)?run.getEnergies().get(1):Double.NaN;}
			@Override public EpicsDevice getDevice()			{ return DevPmacMotor.get("mo_En_E");							}
			@Override public Object getDeviceValue() 			{ return DevPmacMotor.get("mo_En_E").getPosition();	}
			@Override public Limits getLimits() 				{ return DevPmacMotor.get("mo_En_E").getLimits();		}
			@Override public ValidateResult validate(String s)	{ return WdgCollectEnergy.energyValidate(2,s,DevPmacMotor.get("mo_En_E").getLimits());	}
			@Override public int getDecimals() 					{ return 4; 													}
			@Override public boolean doHighlight(double w)		{ double d=(Double)getDeviceValue(); return Math.abs(w-d)>0.0002;	}
		});
		energy2Entry.setLayoutData(WdgRunTab.newGDEntry());
		Label energy2Units = WidgetFactory.newLabel(this, SWT.LEFT, "keV");
		energy2Units.setLayoutData(WdgRunTab.newGDRightLabel());
		energy2Entry.setLabels(energy2Label, energy2Units);
		energyControls.add(energy2Entry);
		
		Label energy3Label = WidgetFactory.newLabel(this, SWT.RIGHT, "Energy 3:");
		energy3Label.setLayoutData(WdgRunTab.newGDLeftLabel());
		energy3Entry = new WdgCollectEnergy(this, SWT.NONE, false, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)		{ updateEnergies(3,value);										}
			@Override public Object getModelValue()				{ return (run.getEnergies().size()>=3)?run.getEnergies().get(2):Double.NaN;}
			@Override public EpicsDevice getDevice()			{ return DevPmacMotor.get("mo_En_E");							}
			@Override public Object getDeviceValue() 			{ return DevPmacMotor.get("mo_En_E").getPosition();	}
			@Override public Limits getLimits() 				{ return DevPmacMotor.get("mo_En_E").getLimits();		}
			@Override public ValidateResult validate(String s)	{ return WdgCollectEnergy.energyValidate(3,s,DevPmacMotor.get("mo_En_E").getLimits());	}
			@Override public int getDecimals() 					{ return 4; 													}
			@Override public boolean doHighlight(double w)		{ double d=(Double)getDeviceValue(); return Math.abs(w-d)>0.0002;	}
		});
		energy3Entry.setLayoutData(WdgRunTab.newGDEntry());
		Label energy3Units = WidgetFactory.newLabel(this, SWT.LEFT, "keV");
		energy3Units.setLayoutData(WdgRunTab.newGDRightLabel());
		energy3Entry.setLabels(energy3Label, energy3Units);
		energyControls.add(energy3Entry);

		Label energy4Label = WidgetFactory.newLabel(this, SWT.RIGHT, "Energy 4:");
		energy4Label.setLayoutData(WdgRunTab.newGDLeftLabel());
		energy4Entry = new WdgCollectEnergy(this, SWT.NONE, false, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)		{ updateEnergies(4,value);										}
			@Override public Object getModelValue()				{ return (run.getEnergies().size()>=4)?run.getEnergies().get(3):Double.NaN;}
			@Override public EpicsDevice getDevice()			{ return DevPmacMotor.get("mo_En_E");							}
			@Override public Object getDeviceValue() 			{ return DevPmacMotor.get("mo_En_E").getPosition();	}
			@Override public Limits getLimits() 				{ return DevPmacMotor.get("mo_En_E").getLimits();		}
			@Override public ValidateResult validate(String s)	{ return WdgCollectEnergy.energyValidate(4,s,DevPmacMotor.get("mo_En_E").getLimits());	}
			@Override public int getDecimals() 					{ return 4; 													}
			@Override public boolean doHighlight(double w)		{ double d=(Double)getDeviceValue(); return Math.abs(w-d)>0.0002;	}
		});
		energy4Entry.setLayoutData(WdgRunTab.newGDEntry());
		Label energy4Units = WidgetFactory.newLabel(this, SWT.LEFT, "keV");
		energy4Units.setLayoutData(WdgRunTab.newGDRightLabel());
		energy4Entry.setLabels(energy4Label, energy4Units);
		energyControls.add(energy4Entry);

		Label energy5Label = WidgetFactory.newLabel(this, SWT.RIGHT, "Energy 5:");
		energy5Label.setLayoutData(WdgRunTab.newGDLeftLabel());
		energy5Entry = new WdgCollectEnergy(this, SWT.NONE, false, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)		{ updateEnergies(5,value);										}
			@Override public Object getModelValue()				{ return (run.getEnergies().size()>=5)?run.getEnergies().get(4):Double.NaN;}
			@Override public EpicsDevice getDevice()			{ return DevPmacMotor.get("mo_En_E");							}
			@Override public Object getDeviceValue() 			{ return DevPmacMotor.get("mo_En_E").getPosition();	}
			@Override public Limits getLimits() 				{ return DevPmacMotor.get("mo_En_E").getLimits();		}
			@Override public ValidateResult validate(String s)	{ return WdgCollectEnergy.energyValidate(5,s,DevPmacMotor.get("mo_En_E").getLimits());	}
			@Override public int getDecimals() 					{ return 4; 													}
			@Override public boolean doHighlight(double w)		{ double d=(Double)getDeviceValue(); return Math.abs(w-d)>0.0002;	}
		});
		energy5Entry.setLayoutData(WdgRunTab.newGDEntry());
		Label energy5Units = WidgetFactory.newLabel(this, SWT.LEFT, "keV");
		energy5Units.setLayoutData(WdgRunTab.newGDRightLabel());
		energy5Entry.setLabels(energy5Label, energy5Units);
		energyControls.add(energy5Entry);
	}
	
	private void createVectorSetupLabel()
	{
		vectorSetupLabel = new Label(this, SWT.CENTER);
		vectorSetupLabel.setText("To complete this run's setup,\nclick 'site setup'");
		vectorSetupLabel.setBackground(GUI.BGCOLOR);
		vectorSetupLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 5, 1));
	}
	
	// This method is called from the widgets where energyNumber indicates which widget did the call
	// The energy is an Object because that's what the entry class sends from setValue 
	// energyNumber is 1-based
	protected void updateEnergies(int energyNumber, Object newEnergyObject)
	{
		ArrayList<Double> energies = new ArrayList<Double>(run.getEnergies());
		
		Double newEnergy = null;
		if (newEnergyObject instanceof Double) newEnergy = (Double)newEnergyObject;
		
		if (null == newEnergy || 0.0 == newEnergy)
		{
			if (energyNumber <= energies.size())
			{
				energies.remove(energyNumber-1);
			}
		}
		else if (energyNumber == energies.size()+1)
		{
			energies.add(newEnergy);
		}
		else if (energyNumber <= energies.size())
		{
			energies.set(energyNumber-1, newEnergy);
		}
		else
		{
			System.out.println("ERROR: energies.size()=="+energies.size()+" energyNumber="+energyNumber+" newEnergy="+newEnergy);
			return;
		}
		
		run.fii.changeEnergies(energies, true);
	}

	private static enum VISIBLE_WEDGE_FIELD { NONE, DEG, SITES };
	private Boolean lastSiteFieldsVisible = null, lastFpsVisible = null, lastNoteVisible = null;
	private VISIBLE_WEDGE_FIELD lastVisibleWedgeField = null;
	private Boolean lastShowXDSInverse = null;
	private Boolean[] lastShowEnergies = null;
	private Boolean lastRun0FieldsVisible = null;
	private Boolean lastOverlapVisible = null;
	private Boolean lastShowDelta = null;
	private Boolean lastShowEndpoints = null;
	private Boolean lastShowFramesPerDir = null;
	
	// called in GUI thread
	protected void updateVisibility()
	{
		boolean changeMade = false;
		boolean inSiteModeWithNoSites =
		(
			run.getScanMode() != ScanMode.SINGLE &&
			run.getScanMode() != ScanMode.JET &&
			run.getScanMode() != ScanMode.IMPORT &&
			run.getSetupStep() == SetupStep.SITE_SETUP
		);

		// delta
		Boolean showDelta = run.getScanMode() != ScanMode.JET;
		if (lastShowDelta != showDelta)
		{
			WdgControl.setControlVisible(deltaLabel, 	showDelta);
			WdgControl.setControlVisible(deltaEntry, 	showDelta);
			WdgControl.setControlVisible(deltaUnits, 	showDelta);

			changeMade = true;
			lastShowDelta = showDelta;
		}

		// endpoints
		Boolean showEndpoints = run.getScanMode() != ScanMode.JET;
		if (lastShowEndpoints != showEndpoints)
		{
			WdgControl.setControlVisible(endPoints, showEndpoints);
			changeMade = true;
			lastShowEndpoints = showEndpoints;
		}

		// centered angle
		boolean showCenteredAngle = false;//run.getScanMode()==ScanMode.MESH;
		wdgCenteredAngle.setVisible(showCenteredAngle);

		// frames per dir
		Boolean fpdVisible = run.getScanMode()==ScanMode.JET;
		if (fpdVisible != lastShowFramesPerDir)
		{
			WdgControl.setControlVisible(framesPerDirLabel, fpdVisible);
			WdgControl.setControlVisible(framesPerDirEntry, fpdVisible);
			WdgControl.setControlVisible(framesPerDirUnits, fpdVisible);
			changeMade = true;
			lastShowFramesPerDir = fpdVisible;
		}

		// fps
		boolean fpsVisible =
		(
			(
				inSiteModeWithNoSites &&
				!run.getHelicalMode()
			) || (
				run.getScanMode() != ScanMode.SINGLE &&
				run.getScanMode() != ScanMode.JET &&
				run.getScanMode() != ScanMode.IMPORT &&
				run.getScanMode() != ScanMode.MESH &&
				SetupStep.COLLECT_PARAM == run.getSetupStep() &&
				run.getRunIndex()!=0 &&
				!run.getHelicalMode()
			)
		);
		if (null==lastFpsVisible || lastFpsVisible!=fpsVisible)
		{
			WdgControl.setControlVisible(framesPerSiteLabel, 		fpsVisible);
			WdgControl.setControlVisible(framesPerSiteEntry, 		fpsVisible);
			WdgControl.setControlVisible(framesPerSiteUnits, 		fpsVisible);
			
			changeMade = true;
			lastFpsVisible = fpsVisible;
		}		
		
		// single site dropdown
		boolean run0FieldsVisible = (run.getRunIndex()==0);
		if (null==lastRun0FieldsVisible || lastRun0FieldsVisible != run0FieldsVisible)
		{
			WdgControl.setControlVisible(siteSelectLabel, 				run0FieldsVisible);
			WdgControl.setControlVisible(siteSelectEntry, 				run0FieldsVisible);
			WdgControl.setControlVisible(siteSelectUnits, 				run0FieldsVisible);
			
			changeMade = true;
			lastRun0FieldsVisible = run0FieldsVisible;
		}

		// site check
		boolean siteFieldsVisible =
		(
			run.getScanMode() != ScanMode.SINGLE &&
			run.getScanMode() != ScanMode.JET &&
			run.getScanMode() != ScanMode.IMPORT &&
			run.getScanMode() != ScanMode.MESH &&
			SetupStep.COLLECT_PARAM == run.getSetupStep() &&
			run.getRunIndex()!=0
		);
		boolean helical = run.getHelicalMode();
		if (null==lastSiteFieldsVisible || lastSiteFieldsVisible != siteFieldsVisible)
		{
			WdgControl.setControlVisible(siteDeltaEnableLabel, 		siteFieldsVisible);
			WdgControl.setControlVisible(siteDeltaEnableCheckbox, 	siteFieldsVisible);
			WdgControl.setControlVisible(siteDeltaEnableUnits, 		siteFieldsVisible);
			
			WdgControl.setControlVisible(siteDeltaLabel, 			siteFieldsVisible && !helical);
			WdgControl.setControlVisible(siteDeltaEntry, 			siteFieldsVisible && !helical);
			WdgControl.setControlVisible(siteDeltaUnits, 			siteFieldsVisible && !helical);

			WdgControl.setControlVisible(siteSkipLabel, 			siteFieldsVisible && helical);
			WdgControl.setControlVisible(siteSkipEntry, 			siteFieldsVisible && helical);
			WdgControl.setControlVisible(siteSkipUnits, 			siteFieldsVisible && helical);

			changeMade = true;
			lastSiteFieldsVisible = siteFieldsVisible;
		}

		// overlap
		boolean overlapVisible = (siteFieldsVisible && !run.getHelicalMode());
		if (null==lastOverlapVisible || lastOverlapVisible != overlapVisible)
		{
			WdgControl.setControlVisible(overlapLabel, 				overlapVisible);
			WdgControl.setControlVisible(overlapEntry, 				overlapVisible);
			WdgControl.setControlVisible(overlapUnits, 				overlapVisible);
			
			changeMade = true;
			lastOverlapVisible = overlapVisible;
		}
		
		// wedge
		VISIBLE_WEDGE_FIELD visibleWedgeField;
		if (run.getRunIndex()==0) 						visibleWedgeField = VISIBLE_WEDGE_FIELD.NONE;
		else if (run.getScanMode()==ScanMode.SINGLE) 	visibleWedgeField = VISIBLE_WEDGE_FIELD.DEG;
		else if (run.getScanMode()==ScanMode.IMPORT) 	visibleWedgeField = VISIBLE_WEDGE_FIELD.DEG;
		else if (run.getScanMode()==ScanMode.MESH)	 	visibleWedgeField = VISIBLE_WEDGE_FIELD.DEG;
		else if (run.getScanMode()==ScanMode.JET)	 	visibleWedgeField = VISIBLE_WEDGE_FIELD.NONE;
		else 											visibleWedgeField = VISIBLE_WEDGE_FIELD.SITES;
		if (inSiteModeWithNoSites) 						visibleWedgeField = VISIBLE_WEDGE_FIELD.NONE;
		
		if (null == lastVisibleWedgeField || lastVisibleWedgeField != visibleWedgeField)
		{
			WdgControl.setControlVisible(wedgeLabel, 				(visibleWedgeField == VISIBLE_WEDGE_FIELD.DEG));
			WdgControl.setControlVisible(wedgeEntry, 				(visibleWedgeField == VISIBLE_WEDGE_FIELD.DEG));
			WdgControl.setControlVisible(wedgeUnits, 				(visibleWedgeField == VISIBLE_WEDGE_FIELD.DEG));
			
			WdgControl.setControlVisible(siteWedgeLabel, 			(visibleWedgeField == VISIBLE_WEDGE_FIELD.SITES));
			WdgControl.setControlVisible(siteWedgeEntry, 			(visibleWedgeField == VISIBLE_WEDGE_FIELD.SITES));
			WdgControl.setControlVisible(siteWedgeUnits, 			(visibleWedgeField == VISIBLE_WEDGE_FIELD.SITES));

			changeMade = true;
			lastVisibleWedgeField = visibleWedgeField;
		}

		// xds proc, inverse
		boolean showXDSInverse = run.getRunIndex()!=0;
		if (inSiteModeWithNoSites || run.getScanMode()==ScanMode.JET) showXDSInverse = false;
		if (null == lastShowXDSInverse || lastShowXDSInverse != showXDSInverse)
		{
			//WdgControl.setControlVisible(xdsProcessingLabel, 		showXDSInverse);
			//WdgControl.setControlVisible(xdsProcessingEntry, 		showXDSInverse);
			//WdgControl.setControlVisible(xdsProcessingUnits, 		showXDSInverse);
			processingCombo.setVisible(showXDSInverse);
			processingLabel.setVisible(showXDSInverse);
			processingUnits.setVisible(showXDSInverse);
			
			WdgControl.setControlVisible(inverseBeamLabel, 			showXDSInverse);
			WdgControl.setControlVisible(inverseBeamCheckbox, 		showXDSInverse);
			WdgControl.setControlVisible(inverseBeamUnits, 			showXDSInverse);

			changeMade = true;
			lastShowXDSInverse = showXDSInverse;
		}

		// energies
		ArrayList<Double> energies = run.getEnergies();
		Boolean[] showEnergies = new Boolean[MAX_ENERGY_FIELDS];
		boolean energiesMatch = true;

		// should each energy field be shown?
		// (keep track of if any field visibility changed)
		for (int i=0; i<MAX_ENERGY_FIELDS; i++)
		{
			if (i>0 && run.getRunIndex()==0) 	{ showEnergies[i]=false; }
			else if (inSiteModeWithNoSites||run.getScanMode()==ScanMode.JET) { showEnergies[i]=false; }
			else								{ showEnergies[i] = (energies.size()>=i); }
			
			if (null == lastShowEnergies || showEnergies[i]!=lastShowEnergies[i]) energiesMatch=false;
		}
		
		// if there was a change, apply it
		if (!energiesMatch)
		{
			for (int i=0; i<MAX_ENERGY_FIELDS; i++)
			{
				energyControls.get(i).setVisible(showEnergies[i]);
			}
			lastShowEnergies=showEnergies;
			changeMade = true;
		}

		// vector setup note
		boolean noteVisible = inSiteModeWithNoSites;
		if (null==lastNoteVisible || lastNoteVisible!=noteVisible)
		{
			WdgControl.setControlVisible(vectorSetupLabel, noteVisible);
			
			changeMade = true;
			lastNoteVisible = noteVisible;
		}		
		
		if (changeMade) layout(true);
	}

	public void updateEnableState(boolean gEnable, boolean isVectorSet, boolean isSiteSkipEnabled)
	{
		//xdsProcessingEntry		.setEnabled(gEnable);
		processingCombo			.setEnabled(gEnable);
		inverseBeamCheckbox		.setEnabled(gEnable && !isSiteSkipEnabled);
		wedgeEntry				.setEnabled(gEnable && !isSiteSkipEnabled);
		siteWedgeEntry			.setEnabled(gEnable && !isSiteSkipEnabled);
		if (null!=beamWidthEntry)  beamWidthEntry.setEnabled(gEnable);
		if (null!=beamHeightEntry) beamHeightEntry.setEnabled(gEnable);
		if (null!=beamsizeCombo)   beamsizeCombo.setEnabled(gEnable);
		
		framesPerDirEntry		.setEnabled(gEnable);
		framesPerSiteEntry		.setEnabled(gEnable && !isSiteSkipEnabled);
		overlapEntry			.setEnabled(gEnable && !isSiteSkipEnabled);
		siteDeltaEnableCheckbox	.setEnabled(gEnable);
		siteDeltaEntry			.setEnabled(gEnable && isSiteSkipEnabled);
		siteSkipEntry			.setEnabled(gEnable && isSiteSkipEnabled);
		siteSelectEntry			.setEnabled(gEnable);
		
		controlDir				.setEnabled(gEnable);
		distanceEntry			.setEnabled(gEnable);
		if (null!=beamstopEntry)	beamstopEntry.setEnabled(gEnable);
		attenuationEntry		.setEnabled(gEnable);
		deltaEntry				.setEnabled(gEnable && !isSiteSkipEnabled);
		timeEntry				.setEnabled(gEnable && !isSiteSkipEnabled);
		endPoints				.setEnabled(gEnable && !isSiteSkipEnabled);
		wdgCenteredAngle		.setEnabled(gEnable && !isSiteSkipEnabled);
		energy1Entry			.setEnabled(gEnable && !isSiteSkipEnabled);
		energy2Entry			.setEnabled(gEnable && !isSiteSkipEnabled);
		energy3Entry			.setEnabled(gEnable && !isSiteSkipEnabled);
		energy4Entry			.setEnabled(gEnable && !isSiteSkipEnabled);
		energy5Entry			.setEnabled(gEnable && !isSiteSkipEnabled);
	}
	
	public void runChanged(final ModelCollectRun run)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			//xdsProcessingEntry		.setSelection(run.getXDSProcMode().ordinal());
			processingCombo			.select(run.getXDSProcMode().ordinal());
    		inverseBeamCheckbox		.setSelected(run.getInverseOn());
    		wedgeEntry				.setText(run.getWedgeSize_deg());
    		siteWedgeEntry			.setText(run.getWedgeSize_sites());
    		if (null!=beamWidthEntry) beamWidthEntry			.setText(run.getBeamWidth_um());
    		if (null!=beamHeightEntry) beamHeightEntry			.setText(run.getBeamHeight_um());
    		if (null!=beamsizeCombo) updateBeamSizeDisplay();
    		
    		overlapEntry			.setText(run.getOverlap());
    		siteDeltaEnableCheckbox	.setSelected(run.getSiteCheckEnable());
    		siteDeltaEntry			.setText(run.getSiteDelta_deg());
    		siteSkipEntry			.setText(run.getSiteSkip());
    		
    		controlDir				.setPrefix(run.getFileRoot());
			controlDir				.setDir(run.getDirectorySuffix());
			distanceEntry			.setText(run.getdistance_mm());
			if (null!=beamstopEntry)	beamstopEntry.setText(run.getBeamstop_mm());
			attenuationEntry		.setText(run.getAttenuation_factors());
			deltaEntry				.setText(run.getDelta());
			timeEntry				.setText(run.getExposureTime());
			endPoints				.runChanged();
			wdgCenteredAngle		.runChanged();
			framesPerSiteEntry		.setText(run.getFramesPerSite());
			framesPerDirEntry		.setText(run.getFramesPerDir());

			// selected site in dropdown
			ArrayList<Boolean> selectedSites = run.getSelectedSites();
			boolean siteSelected = false;
			for (int i=0; i<selectedSites.size();i++)
			{
				if (selectedSites.get(i))
				{
					siteSelectEntry.setSelection(i+1);
					siteSelected = true;
					break;
				}
			}
			if (!siteSelected) siteSelectEntry.setSelection(0);

			ArrayList<Double> energies = run.getEnergies();
			for (int i=0; i<MAX_ENERGY_FIELDS; i++)
			{
				if (i>0 && run.getRunIndex()==0) break;
				
				WdgCollectEnergy energyControl = energyControls.get(i);

				if (energies.size()>i)
				{
					energyControl.setText(energies.get(i));
				}
				else if (energies.size()==i)
				{
					energyControl.setText(null);
				}
			}
			
			updateVisibility();
		}});
	}
	
	public boolean updateModel()
	{
		//if (!xdsProcessingEntry.updateModel()) return false;
		if (!inverseBeamCheckbox.updateModel()) return false;
		
		if (run.getScanMode() == ScanMode.SINGLE)
		{
			if (!wedgeEntry.updateModel()) return false;
		}
		else
		{
			if (!wedgeEntry.updateModel()) return false;
		}
		
		if (!overlapEntry.updateModel()) 			return false;
		if (!siteDeltaEnableCheckbox.updateModel())	return false;
		if (!siteDeltaEntry.updateModel())			return false;
		if (!siteSkipEntry.updateModel())			return false;
		if (!framesPerSiteEntry.updateModel()) 		return false;
		if (!framesPerDirEntry.updateModel()) 		return false;
		if (null!=beamWidthEntry && !beamWidthEntry.updateModel())	 		return false;
		if (null!=beamHeightEntry && !beamHeightEntry.updateModel()) 		return false;
		if (null!=beamsizeCombo) applyBeamSizeDropdown(true);

		if (!controlDir.updateModel()) 			return false;
		if (!distanceEntry.updateModel()) 		return false;
		if (null!=beamstopEntry)				if (!beamstopEntry.updateModel()) return false;
		if (!attenuationEntry.updateModel()) 	return false;
		if (!deltaEntry.updateModel()) 			return false;
		if (!timeEntry.updateModel()) 			return false;
		if (!endPoints.updateModel()) 			return false;
		if (!wdgCenteredAngle.updateModel())	return false;
		if (!energy1Entry.updateModel()) 		return false;
		if (!energy2Entry.updateModel()) 		return false;
		if (!energy3Entry.updateModel()) 		return false;
		if (!energy4Entry.updateModel()) 		return false;
		if (!energy5Entry.updateModel()) 		return false;
		
		return true;
	}
}
