package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.Point3D;
import gov.anl.gmca.collect.ModelCollectRun.SetupStep;
import gov.anl.gmca.collect.ui.WdgCommitButton.ButtonAction;
import gov.anl.gmca.collect.ui.WdgSetPoint.SetPointActions;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.hw.Disk;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

public class WdgImportSetup extends Composite implements ModelCollectRun.Listener
{
	private ModelCollectRun run;
	private WdgFileImport wdgFileImport;
	private WdgSetPoint setPoint1, setPoint2, setPoint3, setPoint4;
	private Button btnCalculate;
	private Group crystalListGroup;
	private WdgSiteList wdgCrystalList;
	
	public WdgImportSetup(Composite parent, final ModelCollectRun run)
	{
		super(parent, SWT.NONE);
		this.run=run;
		
		setBackground(GUI.BGCOLOR);
		GridLayout paramLayout = new GridLayout(3,false);
		paramLayout.marginWidth = 0;
		paramLayout.marginHeight = 0;
		paramLayout.verticalSpacing = 0;
		setLayout(paramLayout);
		
		GridData gd;
		
		wdgFileImport = new WdgFileImport(this, run);
		wdgFileImport.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));
		
		GridLayout gl = new GridLayout(5, false);
		setLayout(gl);
		
		SetPointActions pt1Actions = new SetPointActions() {
			@Override public void setPoint(Point3D point) 	{ run.fii.changeFiducial1(point,true); }
			@Override public void unsetPoint() 				{ run.fii.changeFiducial1(null,true); }
			@Override public Point3D getModelValue() 		{ return run.getPt1(); }
		};
		setPoint1 = new WdgSetPoint(this, "Fiducial 1: ", run, pt1Actions);
		setPoint1.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));
		
		SetPointActions pt2Actions = new SetPointActions() {
			@Override public void setPoint(Point3D point) 	{ run.fii.changeFiducial2(point,true); }
			@Override public void unsetPoint() 				{ run.fii.changeFiducial2(null,true); }
			@Override public Point3D getModelValue() 		{ return run.getPt2(); }
		};
		setPoint2 = new WdgSetPoint(this, "Fiducial 2: ", run, pt2Actions);
		setPoint2.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));

		SetPointActions pt3Actions = new SetPointActions() {
			@Override public void setPoint(Point3D point) 	{ run.fii.changeFiducial3(point,true); }
			@Override public void unsetPoint() 				{ run.fii.changeFiducial3(null,true); }
			@Override public Point3D getModelValue() 		{ return run.getPt3(); }
		};
		setPoint3 = new WdgSetPoint(this, "Fiducial 3: ", run, pt3Actions);
		setPoint3.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));
		
		SetPointActions pt4Actions = new SetPointActions() {
			@Override public void setPoint(Point3D point) 	{ run.fii.changeFiducial4(point,true); }
			@Override public void unsetPoint() 				{ run.fii.changeFiducial4(null,true); }
			@Override public Point3D getModelValue() 		{ return run.getPt4(); }
		};
		setPoint4 = new WdgSetPoint(this, "Fiducial 4: ", run, pt4Actions);
		setPoint4.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));
		
		btnCalculate = new Button(this, SWT.PUSH);
		btnCalculate.setText("Calculate");
		btnCalculate.setBackground(GUI.COLOR_LIGHT_GREEN);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.horizontalSpan=3;
		gd.widthHint = 160;
		btnCalculate.setLayoutData(gd);
		btnCalculate.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				ButtonAction action = lastSitesCalculated ? ButtonAction.RETURN : ButtonAction.FINISH;
				run.fii.handleCalculateButton(action);
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		WdgRunTab.createSpacer(this, 3, 3);
		
		crystalListGroup = new Group(this, SWT.NONE);
		crystalListGroup.setText("Crystal selection");
		crystalListGroup.setBackground(getBackground());
		gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = 5;
		crystalListGroup.setLayoutData(gd);
		crystalListGroup.setLayout(new GridLayout(1,false));
		
		wdgCrystalList = new WdgSiteList(crystalListGroup, run, "crystal");
		gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan=5;
		wdgCrystalList.setLayoutData(gd);
		
		run.addListener(this);
	}

	public void updateEnableState(boolean gEnable)
	{
		boolean areSitesCalculated 	= (SetupStep.COLLECT_PARAM == run.getSetupStep());

		wdgFileImport			.setEnabled(gEnable && !areSitesCalculated);
		setPoint1				.setEnabled(gEnable && !areSitesCalculated);
		setPoint2				.setEnabled(gEnable && !areSitesCalculated);
		setPoint3				.setEnabled(gEnable && !areSitesCalculated);
		setPoint4				.setEnabled(gEnable && !areSitesCalculated);
		btnCalculate			.setEnabled(gEnable);
		wdgCrystalList			.setEnabled(gEnable && areSitesCalculated);
		
		if (areSitesCalculated)
		{
			btnCalculate.setEnabled(gEnable);
		}
		else
		{
			
			boolean readyToCalculate =
			(
				Disk.exists(run.getImportPath()) &&
				null != run.getPt1() &&
				null != run.getPt2() &&
				null != run.getPt3() &&
				null != run.getPt4()
			);
			btnCalculate.setEnabled(gEnable && readyToCalculate);
		}
		
		//WdgControl.setControlVisible(crystalListGroup, );
	}

	private boolean lastSitesCalculated = false;
	@Override public void runChanged(ModelCollectRun run)
	{
		boolean areSitesCalculated 	= (SetupStep.COLLECT_PARAM == run.getSetupStep());
		if (lastSitesCalculated != areSitesCalculated)
		{
			GUI.asyncExec(new Runnable(){public void run()
			{
				btnCalculate.setText(areSitesCalculated ? "Edit inputs" : "Calculate");
				btnCalculate.setBackground(areSitesCalculated ? GUI.COLOR_SAMPLE_BG : GUI.COLOR_LIGHT_GREEN);
				lastSitesCalculated = areSitesCalculated;
			}});
		}
	}
}
