package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.SetupStep;
import gov.anl.gmca.gui.GUI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

public class WdgRasterSiteSetup extends Composite
{
	private ModelCollectRun run;
	private Button btnImport, btnAdd;
	private WdgCommitButton commitButton;
	private WdgRasterAllSites siteList;
	private WdgRasterSelectedSites selectedSites;
	
	public WdgRasterSiteSetup(Composite parent, ModelCollectRun run)
	{
		super(parent, SWT.NONE);
		this.run=run;

		setBackground(GUI.BGCOLOR);
		setLayout(new GridLayout(1,false));
		GridData gd;
		
		Group group = new Group(this, SWT.NONE);
		group.setText("Define sites");
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(2,false);
		group.setLayout(gl);
		
		btnImport = new Button(group, SWT.CENTER);
		btnImport.setBackground(GUI.COLOR_BLUE_BUTTON);
		btnImport.setText("Import...");
		btnImport.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		btnImport.setToolTipText("Import a spreadsheet of 2D raster site offsets from the current position");
		
		btnAdd = new Button(group, SWT.CENTER);
		btnAdd.setBackground(GUI.COLOR_BLUE_BUTTON);
		btnAdd.setText("Add current center");
		btnAdd.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		btnAdd.setToolTipText("Add the current sample position as a raster site");
		
		siteList = new WdgRasterAllSites(this, run);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.heightHint = 1;
		siteList.setLayoutData(gd);
		
		selectedSites = new WdgRasterSelectedSites(this, run);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.heightHint = 1;
		selectedSites.setLayoutData(gd);
		
		commitButton = new WdgCommitButton(this, run);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		commitButton.setLayoutData(gd);
		
		connectListeners();
	}

	private void connectListeners()
	{
		btnImport.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				run.importSites();
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		btnAdd.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				run.addSiteFromCurrentCenter();
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	}
	
	public void updateEnableState(boolean gEnable)
	{
		boolean siteSetupComplete = (SetupStep.SITE_SETUP != run.getSetupStep());
		
		btnImport.setEnabled(gEnable && !siteSetupComplete);
		btnAdd.setEnabled(gEnable && !siteSetupComplete);
		commitButton.setEnabled(gEnable && 0 != run.getSelectedSiteCount());
		siteList.setEnabled(gEnable && !siteSetupComplete);
		selectedSites.setEnabled(gEnable && !siteSetupComplete);
	}

	public void runChanged()
	{
		commitButton.runChanged();
		siteList.updateSelected();
		selectedSites.updateSelected();
	}
	
	public boolean updateModel()
	{
		return true;
	}
}
