package gov.anl.gmca.collect.ui;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.Point3D;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.util.WidgetFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgSetPoint extends Composite implements ModelCollectRun.Listener
{
	protected static Color bdPointUnset 	= GUI.COLOR_DARK_BLUE;
	protected static Color bgPointUnset 	= GUI.COLOR_SAMPLE_BG;
	protected static Color fgPointUnset 	= GUI.COLOR_BLACK;
	protected static Color bdPointSet 		= GUI.COLOR_DARK_GREEN;
	protected static Color bgPointSet 		= GUI.COLOR_GREEN;
	protected static Color fgPointSet 		= GUI.COLOR_WHITE;
	protected static String strPointUnset	= "---";
	protected static String strPointSet	= "saved";
	
	private Label label;
	private Button button;
	private boolean isSet;
	private boolean enable = false;
	private SetPointActions actions;
	private ModelCollectRun run;
	private Composite displayBorder, displayComp;
	private Label displayLabel;
	
	public static interface SetPointActions
	{
		public void setPoint(Point3D point);
		public void unsetPoint();
		public Point3D getModelValue();
	}
	
	public WdgSetPoint(Composite parent, String text, ModelCollectRun run, SetPointActions actions)
	{
		super(parent, SWT.NONE);
		this.run=run;
		this.actions=actions;

		// background and layout
		setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(5,false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		gl.verticalSpacing=0;
		setLayout(gl);
		
		// widget 1: left filler
		Composite leftFiller = new Composite(this, SWT.NONE);
		leftFiller.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false, 1, 1);
		gd.widthHint = 10;
		gd.heightHint = 1;
		leftFiller.setLayoutData(gd);
		
		// widget 2: text label
		label = WidgetFactory.newLabel(this, SWT.RIGHT, text);
		gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		label.setLayoutData(gd);
		
		// widget 3: button
		button = WidgetFactory.newButton(this, SWT.NONE, "set");
		gd = new GridData(SWT.DEFAULT, SWT.CENTER, false, false);
		gd.widthHint = 55;
		button.setLayoutData(gd);
		button.setEnabled(false);
		button.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				if (!isSet)
				{
    				double x = DevPmacMotor.get("sample_x").getPosition();
    				double y = DevPmacMotor.get("sample_y").getPosition();
    				double z = DevPmacMotor.get("sample_z").getPosition();
    				Point3D pt = new Point3D(x,y,z);
    				actions.setPoint(pt);
				}
				else
				{
					actions.unsetPoint();
				}
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		// widget 4: set/unset display
		displayBorder = new Composite(this, SWT.NONE);
		gd = new GridData(SWT.DEFAULT, SWT.CENTER, false, false);
		gd.widthHint = 55;
		displayBorder.setLayoutData(gd);
		gl = new GridLayout(1,false);
		gl.marginWidth=1;
		gl.marginHeight=1;
		displayBorder.setLayout(gl);
		
		displayComp = new Composite(displayBorder, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		displayComp.setLayoutData(gd);
		displayComp.setLayout(new GridLayout(1,false));
		
		displayLabel = WidgetFactory.newLabel(displayComp, SWT.CENTER, "");
		displayLabel.setFont(GUI.FONT_COLLECT_VECTORSTATE);
		gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		displayLabel.setLayoutData(gd);
		
		// widget 5: filler
		Composite rightFiller = new Composite(this, SWT.NONE);
		rightFiller.setBackground(GUI.BGCOLOR);
		gd = new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false, 1, 1);
		gd.widthHint = 10;
		gd.heightHint = 1;
		rightFiller.setLayoutData(gd);
		
		// listen for run updates
		run.addListener(this);
	}
	
	protected void updatePointSetDisplay()
	{
		if (null == displayBorder || null == displayComp || null == displayLabel) return;
		
		if (isSet)
		{
			displayBorder.setBackground(bdPointSet);
			displayComp.setBackground(bgPointSet);
			displayLabel.setBackground(bgPointSet);
			displayLabel.setForeground(fgPointSet);
			displayLabel.setText(strPointSet);
		}
		else
		{
			displayBorder.setBackground(bdPointUnset);
			displayComp.setBackground(bgPointUnset);
			displayLabel.setBackground(bgPointUnset);
			displayLabel.setForeground(fgPointUnset);
			displayLabel.setText(strPointUnset);
		}
	}
	
	@Override public void setEnabled(boolean enable)
	{
		this.enable = enable;
		updateWidgetState();
	}
	
	protected void updateWidgetState()
	{
		if (null != button)
		{
			button.setText(isSet?"unset":"set");
			button.setEnabled(enable);
		}
		updatePointSetDisplay();
	}

	@Override public void runChanged(ModelCollectRun run)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			isSet = (null!=actions.getModelValue());
			updateWidgetState();
		}});
	}
}
