package gov.anl.gmca.collect.ui.cam;

import gov.anl.gmca.collect.DBImportSites;
import gov.anl.gmca.collect.DBImportSites.ImportSite;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRun.Point3D;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.RunCalculator.Frame;
import gov.anl.gmca.common.DevRuntimeConfig;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.SwingUtil;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

public class WdgImportCamera extends WdgSiteCamera
{
	private static enum SITE_STATE 
	{ FULLY_COLLECTED, PARTIALLY_COLLECTED, SELECTED, UNSELECTED };

	private long lastSequenceVersion = -1;
	private int currentFrameIndex = -1;
	private int lastRunIndex = -1, lastCollectedSiteIndex = -1;
	private ArrayList<Frame> frameList = new ArrayList<Frame>();
	private static final Color TEXT_BORDER = new Color(128,0,0), TEXT_COLOR = Color.red;
	
	public volatile int fiducialMarkersOffScreen=0, sitesOffScreen=0;
	private int lastFiducialMarkersOffscreen=0, lastSitesOffScreen=0;
	public ListenerList<WdgImportCamera> llMarkerVisibility;
	
	public WdgImportCamera(Composite parent, int style, String _urlString, CameraType camType, CameraSize camSize)
	{
		super(parent, style, _urlString, camType, camSize);
		llMarkerVisibility = new ListenerList<WdgImportCamera>(this);
	}
	
	@Override protected void drawAll()
	{
		try
		{
			updateProgress();
		}
		catch (Exception e) { e.printStackTrace(); }
		
		if (!DevRuntimeConfig.getImportShowSites()) return;
		
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		drawFiducialMarkers();
		drawImportSites();
		updateVisibilityDisplay();
	}
	
	private void drawFiducialMarkers()
	{
		siteSize = DevRuntimeConfig.getImportShowNumbers() ? 22 : 12;
		fiducialMarkersOffScreen=0;
		
		drawFiducialMarker(run.getPt1(), 1);
		drawFiducialMarker(run.getPt2(), 2);
		drawFiducialMarker(run.getPt3(), 3);
		drawFiducialMarker(run.getPt4(), 4);
	}
	
	private void drawFiducialMarker(Point3D pt, int markerNumber)
	{
		if (null==pt) return;
		
		Point screenXY = motorMMtoScreenIntPx(pt);
		
		// check if marker is off screen
		
		if (!g2.getDeviceConfiguration().getBounds().contains(screenXY.x,screenXY.y)) {
			fiducialMarkersOffScreen++;
			return;
		}
		
		outlineSite(screenXY);
		fillSite(Color.cyan, screenXY);
		if (DevRuntimeConfig.getImportShowNumbers()) drawSiteNumber(-1*markerNumber, screenXY);
	}
	
	private void updateProgress()
	{
		ModelCollectRun currentRun = ModelCollectRunList.get().getCurrentRunModel();
		
		// refill table if needed
		if (lastSequenceVersion != currentRun.seq().getVersion() ||	lastRunIndex != currentRun.getRunIndex())
		{
			// get the current frame list
			frameList = currentRun.seq().getFrameListCopy();

			// update variables to reflect a table refresh
			lastSequenceVersion = currentRun.seq().getVersion();
			lastRunIndex = currentRun.getRunIndex();
		}
		
		currentFrameIndex = currentRun.getNextFrame();
	}

	private SITE_STATE calcSiteState(int siteIndex)
	{
		 boolean isSelected = run.isSiteSelected(siteIndex+1);
		
		// collection hasn't started
		if (frameList.size()==0 || currentFrameIndex==0)
		{
			return isSelected ? SITE_STATE.SELECTED : SITE_STATE.UNSELECTED;
		}
		// collection is complete
		else if (currentFrameIndex==frameList.size())
		{
			return isSelected ? SITE_STATE.FULLY_COLLECTED : SITE_STATE.UNSELECTED;
		}
		else
		{
			Frame previousFrame = frameList.get(currentFrameIndex-1);
			Frame currentFrame = frameList.get(currentFrameIndex);
			boolean currentSitePartiallyCollected = previousFrame.siteNumber==currentFrame.siteNumber;

			// frame is in the middle of the site being calculated
			if (currentSitePartiallyCollected && siteIndex == currentFrame.siteNumber)
			{
				return SITE_STATE.PARTIALLY_COLLECTED;
			}
			else if (siteIndex < currentFrame.siteNumber)
			{
				// site was either collected or skipped
				return isSelected ? SITE_STATE.FULLY_COLLECTED : SITE_STATE.UNSELECTED;
			}
			else
			{
				// site is not collected yet
				return isSelected ? SITE_STATE.SELECTED : SITE_STATE.UNSELECTED;
			}
		}
	}
	
	private void drawImportSites()
	{
		ArrayList<ImportSite> sites = DBImportSites.getAllImportSites();
		siteSize = DevRuntimeConfig.getImportShowNumbers() ? 17 : 10;
		sitesOffScreen=0;
		
		for (ImportSite site : sites)
		{
			// calc site position on screen
			Point screenXY = motorMMtoScreenIntPx(site.xyz);
			
			// check if site is off screen
			if (!g2.getDeviceConfiguration().getBounds().contains(screenXY.x,screenXY.y)) {
				sitesOffScreen++;
				continue;
			}
			
			// draw site outline
			outlineSite(screenXY);
			
			// fill the site based on its state
			SITE_STATE siteState = calcSiteState(site.index);
			switch (siteState)
			{
			case FULLY_COLLECTED: 		fillSite(Color.green, screenXY); break;
			case PARTIALLY_COLLECTED:	fillSite(Color.green, Color.orange, screenXY); break;
			case SELECTED: 				fillSite(Color.orange, screenXY); break;
			case UNSELECTED: 			fillSite(Color.gray, screenXY); break;
			}
			
			// draw the site number
			if (DevRuntimeConfig.getImportShowNumbers()) drawSiteNumber(site.index, screenXY);
		}
	}
	
	private void updateVisibilityDisplay()
	{
		/*if (lastFiducialMarkersOffscreen!=fiducialMarkersOffScreen ||
			lastSitesOffScreen!=sitesOffScreen)
		{
			lastFiducialMarkersOffscreen=fiducialMarkersOffScreen;
			lastSitesOffScreen=sitesOffScreen;
			llMarkerVisibility.notifyListeners();
		}*/
		
		int row=0;
		g2.setFont(GUI.FONT_DEFAULT_AWT);
		if (fiducialMarkersOffScreen>0) drawStringAtTopRight(fiducialMarkersOffScreen+" fiducial markers off screen", row++);
		if (sitesOffScreen>0) 			drawStringAtTopRight(sitesOffScreen+" sites off screen", row++);
	}
	
	private void drawStringAtTopRight(String text, int rowIndex)
	{
		Rectangle2D extent = g2.getFontMetrics().getStringBounds(text, g2);
		Rectangle bounds = g2.getDeviceConfiguration().getBounds();
		int lineSpacing=6, marginX=5, marginY=10;
		
		int strX=(int)(bounds.getWidth()-(int)extent.getWidth()-marginX);
		int strY=(int)(rowIndex*extent.getHeight()+lineSpacing+marginY);
		SwingUtil.drawStringWithBorder(g2, text, TEXT_COLOR, TEXT_BORDER, strX, strY);
	}
}
