package gov.anl.gmca.collect.ui.cam;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.ModelRasterCell;
import gov.anl.gmca.raster.ModelRasterHits.Hit;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterRunList;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.ModelRasterState.NumberingParameter;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.raster.cam.DrawStandard;
import gov.anl.gmca.raster.cam.DrawViewImages;
import gov.anl.gmca.raster.cam.ModelRasterVideo.VideoState;
import gov.anl.gmca.raster.cam.SONICCView;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_OUTPUT;
import gov.anl.gmca.util.PerfTimer;
import gov.anl.gmca.util.Point2D;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import java.awt.*;
import java.util.ArrayList;

public class WdgRasterCamera extends gov.anl.gmca.raster.cam.WdgRasterCamera
{
	public WdgRasterCamera(Composite parent, int style, String urlString, CameraType camera)
	{
		super(parent, style, urlString, camera, VideoState.POINT_AND_CLICK);
		addListener(SWT.MouseEnter, new Listener()
		{
			@Override public void handleEvent(Event event)
			{
				displayOnlyHit=-1;
			}
		});
	}

	@Override public void draw(GC gc, int targetWidth, int targetHeight)
	{
		ModelCollectRun run = ModelCollectRunList.get().getCurrentRunModel();
		model.displayParam = SPOTFINDER_OUTPUT.values()[run.getMeshParam()].toString();
		model.displayNumbering = NumberingParameter.values()[run.getRasterNumbering()];

		super.drawSuper(gc, targetWidth, targetHeight);
	}

	@Override public void drawOverlays(Graphics2D g2)
	{
		ModelCollectRun run = ModelCollectRunList.get().getCurrentRunModel();

		try
		{
			model.setDisplayRunIndex(run.getRasterRun());

			ModelRasterRun currentRun = (ModelRasterRun)ModelRasterRunList.get(RasterListType.INTER).getRun(run.getRasterRun());
			if (currentRun.getRunType()== ModelRasterRun.RunType.DIFFRACTION) {
				model.displayParam = SPOTFINDER_OUTPUT.values()[run.getMeshParam()].toString();
				model.displayNumbering = NumberingParameter.values()[run.getRasterNumbering()];
			} else {
				// TODO: support sonicc in collect run mesh settings
				model.displayParam = ModelRasterState.get().getSONICCDisplay().toString().toLowerCase();
				model.displayNumbering = ModelRasterState.NumberingParameter.NONE;
			}

			// save motor positions and px2mm conversions once here so they are consistent while drawing
			model.currentFrameMotorPos = model.cam.getImageMotorPos().toArray();
			model.current_mm2px.x = gov.anl.gmca.raster.cam.WdgRasterCamera.getConversion(CameraType.HIGHRES, SWT.HORIZONTAL);
			model.current_mm2px.y = gov.anl.gmca.raster.cam.WdgRasterCamera.getConversion(CameraType.HIGHRES, SWT.VERTICAL);

			model.drawStandard.drawColors(g2);
			model.drawStandard.drawCellOutlines(g2, false, false, true);
			model.drawStandard.drawGridNumbers(g2);
			drawHitOutlines(g2);
			drawHitNumbers(g2, run);
			model.drawStandard.drawBeamsize(g2);
			model.drawStandard.drawAngleDiffMessage(g2, 30.0);
			drawRasterMismatchMessage(g2, run);
			model.cam.drawCrosshairAndBorder(g2,model.cam.displayWidth,model.cam.displayHeight);
		}
		catch (Exception e) { e.printStackTrace(); }
	}

	private void drawHitOutlines(Graphics2D g2)
	{

	}

	private int displayOnlyHit=-1;
	@Override public void mouseMove(MouseEvent me)
	{
		ModelCollectRun run = ModelCollectRunList.get().getCurrentRunModel();
		displayOnlyHit=getHitNumber(me.x-drawX,me.y-drawY,run);
	}

	public int getHitNumber(int screenX, int screenY, ModelCollectRun run)
	{
		for (Hit hit : lastHitArray) {
			Point2D screenXY = model.cam.motorMMtoCurrentScreenPx(hit.getMotorXYZO());
			double dist = screenXY.dist(new Point2D(screenX,screenY));
			boolean match = dist <= DrawStandard.HIT_RADIUS;
			if (match) return hit.hitNumber;
		}
		return -1;
	}

	private ArrayList<Hit> lastHitArray = new ArrayList<>();
	private void drawHitNumbers(Graphics2D g2, ModelCollectRun run)
	{
		int hit_draw_max = 500, hitIndex=0;
		ModelRasterRun currentRasterRun = (ModelRasterRun)ModelRasterRunList.get(RasterListType.INTER).getRun(run.getRasterRun());
		lastHitArray = run.getHits();
		for (Hit hit : lastHitArray)
		{
			boolean showHit=true, breakLoop=false;
			if (displayOnlyHit>=0) {
				showHit = hit.hitNumber == displayOnlyHit;
				if (showHit) breakLoop = true;
			}
			if (showHit) {
				for (Integer cellNumber : hit.cellList) {
					ModelRasterCell cell = currentRasterRun.grid().getCellCopy(cellNumber);
					DrawViewImages.highlightCell(cell, model, g2);
				}
			}
			if (hitIndex++>hit_draw_max || breakLoop) break;
		}
		for (Hit hit : lastHitArray) {
			Point2D screenXY = model.cam.motorMMtoCurrentScreenPx(hit.getMotorXYZO());
			if (displayOnlyHit<0 || hit.hitNumber==displayOnlyHit) {
				model.drawStandard.drawNumberInCircle(g2, screenXY.round(), Integer.toString(hit.getNumber()), displayOnlyHit<0?GUI.COLOR_PURPLE_AWT:null);
			}
		}
	}

	private void drawRasterMismatchMessage(Graphics2D g2, ModelCollectRun run)
	{
		ModelRasterRun rasterRun = ModelRasterState.get().getCurrentRunModel();
		if (rasterRun.getRunState() == ModelRasterRun.RunState.RASTERING && run.getRasterRun()!=rasterRun.getRunIndex())
		{
			model.drawStandard.drawMessage(g2, "Raster in progress for a different run than is displayed");
		}
	}
}
