package gov.anl.gmca.collect.ui.cam;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.sample.WdgCamera;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;

import java.awt.*;
import java.awt.geom.Rectangle2D;

public abstract class WdgSiteCamera extends WdgCamera
{
	// variables common to site cameras
	Graphics2D g2;
	int siteSize;
	ModelCollectRun run;
	boolean drawNumbers = true;
	Color selectedSiteBG, unselectedSiteBG;

	protected abstract void drawAll();
	
	public WdgSiteCamera(Composite parent, int style, String _urlString, CameraType camType, CameraSize camSize)
	{
		super(parent, style, _urlString, camType, camSize);
	}

	private boolean drawInit()
	{
		selectedSiteBG 		= drawNumbers?GUI.COLOR_BRIGHT_GREEN2_AWT:java.awt.Color.yellow;
		unselectedSiteBG	= java.awt.Color.gray;
		siteSize			= drawNumbers?17:7;
		
		run = ModelCollectRunList.get().getCurrentRunModel();
		if (null == run) return false;

		return true;
	}
	
	@Override public void drawOverlays(Graphics2D g2)
	{
		this.g2 = g2;
		if (!drawInit()) return;

		// draw cameratype-specific elements, then common camera elements
    	drawAll();
    	super.drawOverlays(g2);
	}
	
	void outlineSite(Point siteXY)
	{
		g2.setColor(java.awt.Color.black);
		g2.fillOval((int)(siteXY.x-(siteSize/2))-1, (int)(siteXY.y-(siteSize/2))-1, siteSize+2, siteSize+2);
	}
	
	// negative site index = fiducial marker
	void fillSite(int siteIndex, Point siteXY)
	{
		Color bgColor = null;
		if (siteIndex>=0)
		{
			// get site bg color
			boolean siteSelected = run.isSiteSelected(siteIndex+1);
			bgColor = siteSelected?selectedSiteBG:unselectedSiteBG;
		}
		else
		{
			bgColor = selectedSiteBG;
		}
		
		fillSite(bgColor, siteXY);
	}
	
	void fillSite(Color fillColor, Point siteXY)
	{
		g2.setColor(fillColor);
		g2.fillOval((int)(siteXY.x-(siteSize/2)), (int)(siteXY.y-(siteSize/2)), siteSize, siteSize);
	}
	
	void fillSite(Color firstFillColor, Color secondFillColor, Point siteXY)
	{
		int x = siteXY.x-(siteSize/2);
		int y = siteXY.y-(siteSize/2);
		int dx = siteSize;
		int dy = siteSize;
		
		// top
		g2.setColor(firstFillColor);
		g2.setClip(x,y,dx,dy/2);
		g2.fillOval(x,y,dx,dy);
		
		// bottom
		g2.setColor(secondFillColor);
		g2.setClip(x,y+dy/2+1,dx,dy/2+2);
		g2.fillOval(x, y, dx, dy);
		
		// clear clipping
		g2.setClip(null);
	}
	
	// negative site index = fiducial marker
	void drawSiteNumber(int siteIndex, Point siteXY)
	{
		// draw the site number
		if (drawNumbers)
		{
			// set up for text drawing
			String siteText = siteIndex>=0 ? Integer.toString(siteIndex+1) : "F"+(-1*siteIndex);
			g2.setFont(GUI.FONT_COLLECT_SITENUMBER_AWT);
			Rectangle2D extent = g2.getFontMetrics().getStringBounds(siteText, g2);
			int textWidth = (int)(extent.getWidth());
			int textHeight = (int)(extent.getHeight());
			int textX = (int)(siteXY.x-textWidth/2);
			int textY = (int)(siteXY.y+textHeight/2);
			
			// draw the black text outline
			g2.setColor(java.awt.Color.black);
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			for (int dx=-1; dx<=1; dx+=2)
			for (int dy=-1; dy<=1; dy+=2)
			{
				g2.drawString(siteText, textX+dx, textY+dy);
			}
			
			// draw the white text
			g2.setColor(java.awt.Color.white);
			g2.drawString(siteText, textX, textY);
		}
	}
}
