package gov.anl.gmca.collect.viewer;

import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.util.Point2D;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.MouseWheelListener;

// handles both mouse wheel and right-click zoom
//
public class ListenerZoom implements MouseListener, MouseMoveListener, MouseWheelListener
{
	private WdgDiffViewer viewer;
	private boolean inZoomDrag = false;
	private Point2D zoomPctCoords;
	
	public ListenerZoom(WdgDiffViewer viewer)
	{
		this.viewer = viewer;
		viewer.addMouseListener(this);
		viewer.addMouseMoveListener(this);
		viewer.addMouseWheelListener(this);
	}
	
	@Override public void mouseDown(MouseEvent e)
	{
		if (e.button==3) // 3 is right button
		{
			inZoomDrag=true;
			zoomPctCoords = mouseToPercentXY(e.x, e.y);
			viewer.diffTab.setZoomVisible(true);
			viewer.diffTab.updateZoom(zoomPctCoords);
		}
	}
	
	@Override public void mouseUp(MouseEvent arg0)
	{
		inZoomDrag=false;
		//viewer.diffTab.setZoomVisible(false);
	}
	
	@Override public void mouseMove(MouseEvent e)
	{
		if (inZoomDrag)
		{
			zoomPctCoords = mouseToPercentXY(e.x, e.y);
			viewer.diffTab.updateZoom(zoomPctCoords);
		}
	}
	
	private Point2D mouseToPercentXY(int mouseX, int mouseY)
	{
		return pixelToPercentXY(new Point2D(mouseX, mouseY - viewer.model.titleHeight));
	}
	
	// image viewer parameters are: zoom multiple, percent X/Y for image center
	// viewer canvas parameter is its size in pixels
	//
	// image viewer zoom of 1.0 means the image exactly fills the requested image width
	//
	private Point2D pixelToPercentXY(Point2D pixelXY)
	{
		Point2D canvasSize = new Point2D(viewer.model.imgWidth, viewer.model.imgHeight);
		Point2D imagePos = new Point2D(viewer.model._percentX, viewer.model._percentY);
		
		// To find what imgZoomY is
		// ------------------------
		// defining what imgZoomX/Y are (imgZoomX is what is passed to the image server)
		// (1) imgZoomX = imgWidth/canvasWidth
		// (2) imgZoomY = imgHeight/canvasHeight
		//
		// from (1):
		// (3) imgWidth = imgZoomX*canvasWidth
		//
		// defining imgRatio which we can get from DetectorConfig or loading the image
		// (4) imgRatio = imgWidth/imgHeight
		//
		// from (4):
		// (5) imgWidth = imgRatio*imgHeight
		//
		// substituting imgWidth from (3) in (5):
		// (6) imgRatio*imgHeight = imgZoomX*canvasWidth
		//
		// from (6):
		// (7) imgHeight/canvasWidth = imgZoomX/imgRatio
		//
		// defining canvasRatio:
		// (8) canvasRatio = canvasWidth/canvasHeight
		//
		// from (8):
		// (9) canvasWidth = canvasRatio*canvasHeight
		// 
		// substituting canvasWidth from (9) in (7):
		// (10) imgHeight/(canvasRatio*canvasHeight) = imgZoomX/imgRatio
		//
		// from (10):
		// (11) imgHeight/canvasHeight = (imgZoomX*canvasRatio)/imgRatio
		//
		// and since the left side of (10) is imgZoomY:
		// (12) imgZoomY = (imgZoomX*canvasRatio)/imgRatio
		double canvasRatio = ((double)viewer.model.imgWidth)/((double)viewer.model.imgHeight);
		Point2D imgSizeMM = DetectorConfig.getImageSizeMM(viewer.model.currentImgDetectorID, null);
		double imgRatio = imgSizeMM.x / imgSizeMM.y;
		double imgZoomX = viewer.model.getZoom();
		double imgZoomY = (imgZoomX*canvasRatio)/imgRatio;

		// how far is the pixel from the center, in pixels?
		double dx = (canvasSize.x/2.0) - pixelXY.x;
		double dy = (canvasSize.y/2.0) - pixelXY.y;

		// how far is the pixel from the center, in image percentage?
		double dx_pct = ( dx / canvasSize.x) / imgZoomX;
		double dy_pct = ( dy / canvasSize.y) / imgZoomY;
		
		// subtract the distance from the center from the center's position
		// to get the requested pixel's percentage
		double xpct = imagePos.x-dx_pct;
		double ypct = imagePos.y-dy_pct;
		
		/*String line = Util.sprintf("Canvas (%.2f,%.2f) ImagePos (%.2f,%.2f) Zoom (%.2f,%.2f) Mouse (%.2f,%.2f), Result(%.2f,%.2f)",
									canvasSize.x,canvasSize.y,
									imagePos.x,imagePos.y,
									imgZoomX,imgZoomY,
									pixelXY.x,pixelXY.y,
									xpct,ypct);
		Log.note(line);
		Log.note(Util.sprintf("Canvas = (%d,%d) image = (%.1f,%.1f) zoomX=%.2f zoomY=%.2f",
				viewer.imgWidth,viewer.imgHeight,DetectorConfig.getDimensions().x,DetectorConfig.getDimensions().y, imgZoomX, imgZoomY));*/

		return new Point2D(xpct, ypct);
	}

	@Override public void mouseDoubleClick(MouseEvent arg0) {}

	@Override public void mouseScrolled(MouseEvent e)
	{
		// apply the zoom
		boolean scrollUp = e.count>0;
		viewer.model.setZoom(viewer.model.getZoom() * (scrollUp ? 1.5 : (1.0/1.5)), true);
	
		// update
		viewer.model.requestUpdate();
	}
	
}
