package gov.anl.gmca.collect.viewer;

import gov.anl.gmca.collect.DevAdxvFile;
import gov.anl.gmca.collect.DevFPE;
import gov.anl.gmca.collect.DevFPE.ViewerFileInfo;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.diffimage.*;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.io.Disk;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.ListenerList.Listener;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.graphics.Image;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ModelDiffViewer
{
	private WdgDiffViewer wdg;
	
	// image center
	protected double _percentX = 0.5, _percentY = 0.5;
	public double getPercentX() { return _percentX; }
	public double getPercentY() { return _percentY; }
	public void setPercentX(double percentX) { _percentX = percentX; requestUpdate(); }
	public void setPercentY(double percentY) { _percentY = percentY; requestUpdate(); }
	
	protected String title = new String("Loading image...");
	public String currentFullPath = null;
	protected ViewerFileInfo currentFilename = null;
	
	boolean followFPE = true;
	public boolean getFollowFPE() { return followFPE; }
	public void setFollowFPE(boolean b)
	{
		followFPE = b;
		if (followFPE && (null!=wdg && wdg.isVisible)) _requestReload = true;
	}
	
	private volatile boolean _requestUpdate = false, _requestReload = false;
	public void requestUpdate()
	{
		// with imgsrv, all updates are reloads
		if (DBConfig.getBoolean("imgsrv_enable"))
			_requestReload=true;
		else
			_requestUpdate=true;
	}
	public void requestReload() { _requestReload=true; }
	
	public LoadAdjacentFrame loadAdjacent;

	protected DiffractionImage currentImage = null;
	protected Image imgFront = null, imgBack = null;

	public static int DEFAULT_CONTRAST = DetectorConfig.isDectris() ? 10 : 200;
	protected int _gray = DEFAULT_CONTRAST;
	public double getContrast() { return (double)_gray; }
	public void setContrast(double newContrast) { _gray = (int)newContrast; requestUpdate(); }
	
	String currentImgDetectorID = "";
	
	protected double _zoom = 1.0;
	public double getZoom() { return _zoom; }
	public void setZoom(final double newZoom, boolean updateWidget)
	{
		_zoom = newZoom;
		if (updateWidget && null != wdg.zoomEntry)
		{
			GUI.asyncExec(new Runnable(){public void run()
			{
				wdg.zoomEntry.setText(newZoom);
			}});
		}
		requestUpdate();
	}
	
	// these are saved so they are accessible outside the GUI thread
	protected int imgWidth=0, imgHeight=0;
	public int getWidth() { return imgWidth; }
	public int getHeight() { return imgHeight; }
	protected int titleHeight = 21;
	
	/////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////
	
	public ModelDiffViewer(WdgDiffViewer wdg)
	{
		this.wdg=wdg;
		if (null!=wdg) wdg.model=this;
		
		startImageLoadingThread();
		connectListeners();
		DevAdxvFile.startListener();
	}
	
	int lastSeries=0, lastDataFile=0;
	boolean isNewHDF5DataFileAvailable()
	{
		ModelCollectRun currentRun = ModelCollectRunList.get().getCurrentRunModel();
		if (null==currentRun) return false;
		
		String[] listing = Disk.getDir(currentRun.getFullDirectory());
		if (null==listing) return false;
		
		Pattern dataPattern = Pattern.compile("series_(\\d+)_data_(\\d+).h5");
		Pattern masterPattern = Pattern.compile("series_(\\d+)_master.h5");
		
		// first find the latest master file
		for (String filename : listing)
		{
			Matcher masterMatcher = masterPattern.matcher(filename);
			
			if (masterMatcher.matches())
			{
				String seriesString = masterMatcher.group(1);
				int seriesNumber = Integer.parseInt(seriesString);
				if (seriesNumber>lastSeries)
				{
					lastSeries=seriesNumber;
					lastDataFile=0;
				}
			}
		}

		// now look for the latest data file in that series
		boolean newDataFound=false;
		for (String filename : listing)
		{
			Matcher dataMatcher = dataPattern.matcher(filename);
			if (dataMatcher.matches())
			{
				int seriesNumber = Integer.parseInt(dataMatcher.group(1));
				int dataFileNumber = Integer.parseInt(dataMatcher.group(2));
				
				if (seriesNumber==lastSeries && dataFileNumber>lastDataFile)
				{
					lastDataFile=dataFileNumber;
					newDataFound=true;
				}
			}
		}
		
		if (newDataFound)
		{
			// TODO: handle multiple images per file and multiple datasets per run
			//int nextFrame = lastDataFile; 
			//ModelCollectRunList.get().getCurrentRunModel().dev().setNextFrame(nextFrame);
			
			_requestReload=true;
			return true;
		}
		return false;
	}
	protected boolean updateImage()
	{
		// unset request flag here so if it's set during the execution of this method,
		// it will be called again soon
		_requestUpdate = false;
		
		if (imgWidth<=0 || imgHeight<=0) return false;

		if (_requestReload || null == currentImage)
		{
			_requestReload = false;
			
			if (followFPE)
			{
				currentFilename = DevFPE.get().getImageViewerFile();
				currentFullPath = DevFPE.get().getImageViewerDir()+"/"+currentFilename.filename;
			}
			
			if (0 == DevFPE.get().getImageViewerFile().filename.length() && 0 == DevFPE.get().getImageViewerDir().length())
	 		{
				title = "---";
				if (null!=wdg) GUI.asyncExec(new Runnable(){public void run(){wdg.redraw();}});
				return false;
			}
			
			// TODO: temporary change for Eiger only
			//title = currentFilename.filename+":"+currentFilename.imageIndex;
			title = DevFPE.getPathAfterCommonDir(currentFullPath);
			
			DiffractionImage newImage = null;
			
			if (DBConfig.getBoolean("imgsrv_enable"))
			{
				newImage = new ImageServerImage
				(
					currentFullPath,
					_percentX,
					_percentY,
					_gray,
					_zoom,
					imgWidth,
					imgHeight
				);
			}
			else if (currentFullPath.endsWith(".h5"))
			{
				if (Disk.exists(currentFullPath))
				try
				{
					newImage = new HDF5Image(currentFullPath, 10, currentFilename.imageIndex);
				}
				catch (Exception e) { e.printStackTrace(); }
			}
			else if (currentFullPath.endsWith(".cbf"))
			{
				newImage = new CBFImage(currentFullPath);
			}
			else
			{
				newImage = new MARImage(currentFullPath);
			}

			if (null!=newImage && newImage.hasValidImage())
			{
				if (null == currentImage)
				{
					// set the zoom to default on the first image load
					showFullImage();
				}

				currentImage = newImage;
				//diffTab.imageOptions.setHistogram(ZoomPanContrast.getHistogram(currentImage));
			}
		}
		
		if (null!=currentImage)
		{
			imgBack = ZoomPanContrast.process(currentImage, getPercentX(), getPercentY(), getContrast(), getZoom(), imgWidth, imgHeight);
			currentImgDetectorID = currentImage.detector;
				
			// image is valid, flip the buffers and draw
			Image oldFront=imgFront;
			imgFront= imgBack;
			if (null != oldFront && !oldFront.isDisposed()) oldFront.dispose();
			imgBack=null;
	
			//Log.note("Redrawing diff image, load complete");
			if (null!=wdg) GUI.asyncExec(new Runnable(){public void run(){wdg.redraw();}});
		}
		
		return true;
	}
	
	
	private int currentServer = -1;
	private void connectListeners()
	{
		// last file update listener
		DevAdxvFile.startListener();
		
		DevFPE.get().imageViewerListener.addListener(new Listener<DevFPE>()
		{
			@Override public void handleEvent(DevFPE object, Object data)
			{
				if (null!=wdg && wdg.isVisible && followFPE)
				{
					_requestReload = true;
				}
			}
		});

		DevUserConfig.get().ll.addListener(new ListenerList.Listener<DevUserConfig>()
		{
			@Override public void handleEvent(DevUserConfig config, Object data)
			{
				int i = config.getImgSrv().index;
				if (i!=currentServer) {
					GUI.asyncExec(new Runnable(){public void run()
					{
						requestReload();
					}});
				}
			}
		});
	}
	
	// sets the zoom so the entire image exactly fills the viewer
	// zoom=1 tells imageViewer to fit the width
	// but if the height exceeds the viewer height we need to reduce the zoom
	public void showFullImage()
	{
/*		double viewAreaHeight = imgHeight-titleHeight;
		double viewAreaWidth = imgWidth;
		double viewRatio = viewAreaHeight / viewAreaWidth;
		
		double imageRatio = DetectorConfig.getHeightWidthRatio();

		boolean imageRatioExceedsViewer = imageRatio > viewRatio;
		if (imageRatioExceedsViewer) newZoom = viewRatio/imageRatio;*/
		double newZoom = 1.0;
		setZoom(newZoom, true);
	}
	
	private void startImageLoadingThread()
	{
		// image loading thread
		new Thread("Diff viewer image load")
		{
			@Override public void run()
			{
				while (!Util.atExit())
				{
					// keep trying to update until it works
    				while (true)
    				{
    					try
    					{
    						//PerfTimer pt = new PerfTimer("updateImage(): ");
    						boolean success = updateImage();
    						//pt.printElapsedMs();
        					if (success || null==wdg || !wdg.isVisible) break;
    					}
    					catch (Exception e) { e.printStackTrace(); }
    					Util.sleep(1000);
    				}

    				// wait for a new request
    				while (true)
    				{
    					Util.sleep(100);
    					if (null!=wdg && wdg.isVisible && (_requestUpdate||_requestReload)) break;
    					//if (DetectorConfig.isEiger() && isNewHDF5DataFileAvailable()) break;
    				}
				}
			}
		}.start();
	}
}
