package gov.anl.gmca.collect.viewer;

import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.gui.GUI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgDiffViewer extends Canvas implements Tab.VisibilityListener, PaintListener, ControlListener
{
	ModelDiffViewer model;
	
	WdgDiffTab diffTab;
	WdgControlEntry zoomEntry;
	ListenerZoom lZoom;
	ListenerPan lPan;
	ListenerExternalViewer lExternalViewer;
	ListenerResDisplay lResDisplay;
	protected Label label;
	protected int labelWidth = 80, labelHeight = 22;
	
	//////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////// end variables ////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////
	
	public WdgDiffViewer(Composite parent, WdgDiffTab diffTab, WdgControlEntry zoomEntry, boolean enableMouseControls)
	{
		super(parent,SWT.NONE);
		model = new ModelDiffViewer(this);
		this.diffTab = diffTab;
		this.zoomEntry = zoomEntry;
		model.loadAdjacent = new LoadAdjacentFrame(this);
		
		addPaintListener(this);
		addControlListener(this);
		
		if (enableMouseControls)
		{
			lZoom = new ListenerZoom(this);
			lPan = new ListenerPan(this);
			lExternalViewer = new ListenerExternalViewer(this);
			lResDisplay = new ListenerResDisplay(this);
		}
	}
	
	@Override public void controlResized(ControlEvent e)
	{
		model.imgWidth=getBounds().width;
		model.imgHeight=getBounds().height-model.titleHeight;
		System.out.println("diff resize: "+model.imgWidth+" "+model.imgHeight);
		
		model.requestUpdate();
	}
	
	protected boolean isVisible = false;
	public void notifyIsVisible(boolean _isVisible)
	{
		isVisible = _isVisible;
		if (isVisible) model.requestUpdate();
	}

	@Override public void paintControl(PaintEvent e)
	{
		try
		{
			e.gc.setBackground(GUI.BGCOLOR);
			e.gc.fillRectangle(e.x, e.y, e.width, e.height);
			
			e.gc.setForeground(GUI.COLOR_BLACK);
			e.gc.drawRectangle(0, 0, model.imgWidth-1, model.titleHeight-1);
			
			e.gc.setForeground(GUI.COLOR_BLACK);
			e.gc.setFont(GUI.FONT_DIFFIMAGE_TITLE);
			Point extent = e.gc.textExtent(model.title);
			
			if (e.y <= model.titleHeight)
			{
				int textX = (model.imgWidth/2)-(extent.x/2);
				int textY = (model.titleHeight/2)-(extent.y/2);
				e.gc.drawText(model.title, textX, textY);
			}
	
	        if (model.imgFront != null && !model.imgFront.isDisposed()) 
	        {
	           	e.gc.drawImage(model.imgFront, 0, model.titleHeight);
	        }
		}
		catch (Exception _e) { 
			_e.printStackTrace();
		}
    }

	@Override public void controlMoved(ControlEvent e) {}
}
