package gov.anl.gmca.common;

import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;

// A DevCommand is a Command that should only be run once at a time.
//
// It can be queried as to whether it's currently running, and can only be
// started if it is not currently running.
//
// For DevCommand:
// DO USE isCmdAlive()
// DO NOT USE isAlive() or isRunning()
//
// To add listeners: DevCmdExample.get().ll.addListener(new ListenerList.Listener<Command>(){...});
//
public abstract class DevCommand extends Command
{
	public static Command inst = null;
	public static Command get() { return inst; }
	public static boolean isCmdAlive() { return inst.isAlive(); }
	
	public DevCommand()
	{
		if (null != inst) throw new IllegalStateException("This DevCommand was already created.");
		inst = this;
	}
	
	@Override public synchronized void start()
	{
		while (isAlive())
		{
			Log.warn("Command already running, waiting for termination...");
			Util.sleep(1000);
		}
		super.start();
	}
}
