package gov.anl.gmca.common;

import gov.anl.gmca.util.Util;

public class Limits
{
	public static class LimitsResult
	{
		public boolean isOK()
		{
			return isWithinLimits;
		}
		
		public String getReason()
		{
			if (isOK()) return null;
			return errorMsg;
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////
		////////////////////////////////////// end public interface //////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////
		
		protected boolean isWithinLimits;
		protected String errorMsg;
		
		protected LimitsResult(boolean isWithinLimits, String errorMsg)
		{
			this.isWithinLimits = isWithinLimits;
			this.errorMsg = errorMsg;
		}
		protected LimitsResult(boolean isWithinLimits)
		{
			this.isWithinLimits = isWithinLimits;
			this.errorMsg = null;
		}
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////// end LimitsResult //////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////
	
	public LimitsResult testValue(double value)
	{
		if (value>getUpper()) return new LimitsResult(false, getUpperMessage(value));
		if (value<getLower()) return new LimitsResult(false, getLowerMessage(value));
		return new LimitsResult(true);
	}
	
	public double getUpper()
	{
		if (0==lower && 0==upper) return Double.MAX_VALUE;
		return upper;
	}
	
	public double getLower()
	{
		if (0==lower && 0==upper) return -1.0*Double.MAX_VALUE;
		return lower;
	}
	
	public Limits(double lower, String lowerDescription, double upper, String upperDescription)
	{
		init(lower, createFullLowerReason(lower,lowerDescription), upper, createFullUpperReason(upper,upperDescription));
	}
	
	public Limits(double lower, String lowerString, double upper, String upperString, boolean stringsAreFullReasons)
	{
		if (stringsAreFullReasons)	init(lower, lowerString, upper, upperString);
		else						init(lower, createFullLowerReason(lower,lowerString), upper, createFullUpperReason(upper,upperString));
	}
	
	public Limits(double lower, double upper, String description)
	{
		init(lower, createFullLowerReason(lower,"minimum "+description), upper, createFullUpperReason(upper,"maximum "+description));
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////// end public interface ////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////
	
	// the limits
	protected double upper, lower;
	
	// the full message that goes after the number
	protected String lowerReason, upperReason;
	
	protected String createFullLowerReason(double lower, String lowerDescription)
	{
		return Util.sprintf("is less than the "+lowerDescription+", which is %.04f",lower);
	}
	
	protected String createFullUpperReason(double upper, String upperDescription)
	{
		return Util.sprintf("is greater than the "+upperDescription+", which is %.04f",upper);
	}
	
	protected String getLowerMessage(double value)
	{
		return Util.sprintf("%.04f "+lowerReason,value);
	}
	
	protected String getUpperMessage(double value)
	{
		return Util.sprintf("%.04f "+upperReason,value);
	}
	
	protected void init(double lower, String fullLowerReason, double upper, String fullUpperReason)
	{
		this.lower=lower;
		this.lowerReason=fullLowerReason;
		this.upper=upper;
		this.upperReason=fullUpperReason;
	}
}
