package gov.anl.gmca.common.client_server;

import com.github.wnameless.json.flattener.FlattenMode;
import com.github.wnameless.json.flattener.JsonFlattener;
import com.github.wnameless.json.flattener.JsonifyArrayList;
import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.Unirest;
import com.mashape.unirest.http.exceptions.UnirestException;
import gov.anl.gmca.common.client_server.StateEvents.StateListener;
import gov.anl.gmca.common.ui.NBMessageBox;
import gov.anl.gmca.common.ui.NBProgress;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.users.ModelUserTable;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;

import java.math.BigDecimal;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;

public class Popup
{
    public static void init()
    {
        StateEvents.addListener("messages", new StateListener()
        {
            @Override public void stateChanged(String key, String value) {
                handleMessages(value);
            }
        });
    }

    public static void handleMessages(String messages)
    {
        // parse each message into a Java map
        if (messages.equals("[]")) return;
        Map<String,Object> messageList = JsonFlattener.flattenAsMap(messages);
        for (String key : messageList.keySet()) {

            // convert each message field into its own map
            String messageText = (String) messageList.get(key);
            Map<String, Object> message = new JsonFlattener(messageText).withFlattenMode(FlattenMode.KEEP_ARRAYS).flattenAsMap();

            // get common options
            String type = (String) message.get("type");
            String text = (String) message.get("text");
            String popupID = (String) message.get("popupID");

            if (type.equals("question")) {
                String[] options = (String[]) ((JsonifyArrayList) (message.get("options"))).toArray(new String[]{});
                popupQuestion(text, popupID, options);
            } else if (type.equals("note")) {
                popupNote(text, popupID);
            } else if (type.equals("progress")) {
                int wait_sec = ((BigDecimal)message.get("wait_sec")).intValue();
                popupProgress(text, popupID, wait_sec);
            } else {
                System.out.println("ERROR: Unknown popup type: "+type);
                sendResponse(popupID, -1);
            }
        }
    }

    public static void popupQuestion(String text, String popupID, String[] options)
    {
        NBMessageBox mb = new NBMessageBox(GUI.getShell(), SWT.NONE, false);
        mb.setCustomButtons(options);
        mb.setText("Script message");
        mb.setMessage(text);

        // TODO: merge with progress bar monitor

        AtomicBoolean quitMonitor = new AtomicBoolean(false);

        mb.setRunnableOnSubmit(()->{
            sendResponse(popupID, mb.getReturnValue());
            quitMonitor.set(true);
        });

        new Thread(()->{
            while (!quitMonitor.get()) {
                GUI.asyncExec(()->{
                    if (quitMonitor.get()) return;

                    if (!ModelUserTable.Client.isMaster()) {
                        mb.dialogShell.close();
                        sendResponse(popupID, 0);
                        quitMonitor.set(true);
                    }
                });
                Util.sleep(1000);
            }
        }, "Script question popup monitor").start();

        mb.asyncOpen();
    }

    public static void popupNote(String text, String popupID)
    {
        NBMessageBox mb = new NBMessageBox(GUI.getShell(), SWT.NONE, false);
        mb.setCustomButtons(new String[]{"OK"});
        mb.setText("Script message");
        mb.setMessage(text);
        mb.asyncOpen();
        sendResponse(popupID, 0); // immediate response
    }

    public static void popupProgress(String text, String popupID, int wait_sec)
    {
        NBProgress progress = new NBProgress(GUI.getShell(), SWT.APPLICATION_MODAL);
        progress.setText("Script message");
        progress.setMessage(text);
        progress.setWait_sec(wait_sec);

        // TODO: merge with monitor for question, move to NBMessageBox?
        AtomicBoolean quitMonitor = new AtomicBoolean(false);

        progress.setRunnableOnSubmit(()->{
            sendResponse(popupID, progress.getReturnValue());
            quitMonitor.set(true);
        });

        new Thread(()->{
            while (!quitMonitor.get()) {
                GUI.asyncExec(()->{
                    if (quitMonitor.get()) return;

                    if (!ModelUserTable.Client.isMaster()) {
                        progress.dialogShell.close();
                        sendResponse(popupID, 0);
                        quitMonitor.set(true);
                    }
                });
                Util.sleep(1000);
            }
        }, "Progress popup monitor").start();

        progress.asyncOpen();
    }

    public static void sendResponse(String popupID, int response)
    {
        String responseString = Integer.toString(response);
        try {
            HttpResponse<String> resp = Unirest.post(Util.getServerURL() + "/popup_response")
                    .field("popup_id", popupID)
                    .field("response", responseString)
                    .asString();
            System.out.println("Sent popup response to "+popupID+" result="+responseString);
        } catch (UnirestException e) { e.printStackTrace(); }
    }
}
