package gov.anl.gmca.common.client_server;

import gov.anl.gmca.common.EpicsDevice;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;

// Used by both client and server
//
// Server loads data into the object then serializes to JSON
// Client deserializes to an object and replaces the old one
//
// No methods are contained here, just data.
//
public class StateMap 
{
	public static StateMap current;
	public static Map<String,Object> currentMap;
	public static String getKey(Map<String,Object> map, String key) { Object val = map.get(key); return null==val?"null":val.toString(); }
	
	public interface ServerCmd {}
	public interface ServerObj {}
	
	public static class Client implements ServerObj
	{
		public String clientID;
		public String host;
		public String username;
		public Boolean isMaster;
		public long lastUpdateSec;
	}
	public ArrayList<Client> clientList = new ArrayList<>();

	public ArrayList<String> messages = new ArrayList<>();

	public static class WorkerState implements ServerObj
	{
		public String name;
		public String state;
	}
	public ArrayList<WorkerState> workerStates = new ArrayList<>();

	public static class StateMotor implements ServerObj
	{
		public String motorID;
		public EpicsDevice.State state;
		public Double actPos, rqsPos;
		public Boolean inPos;
		public String error;
		public Double lowerLimit, upperLimit, inPosBand;
		public String units;
	}
	public LinkedHashMap<String, StateMotor> motorList = new LinkedHashMap<>();

	public static class StateOptimizeBeamGraph implements ServerObj
	{
		public int scanFlag, pointCount;
		public double vLow, vHigh;
		public double[] xPoints = new double[1];
		public double[] yPoints = new double[1];
	}
	public static class StateOptimizeBeam implements ServerObj
	{
		public boolean cmdRunning;
		public long lastRunTime_unix;
		public double lastEnergy_keV;
		public StateOptimizeBeamGraph vGraph = new StateOptimizeBeamGraph(), hGraph = new StateOptimizeBeamGraph();
	}
	public StateOptimizeBeam optimizeBeam = new StateOptimizeBeam();
	public static enum OptimizeBeamCommands implements ServerCmd { OptimizeBeam };

	public static class StateAutoCenter implements ServerObj
	{
		public boolean cmdRunning;
		public String mode;
	}
	public StateAutoCenter autoCenter = new StateAutoCenter();

	public static class StateEnterHutch implements ServerObj
	{
		public boolean cmdRunning;
	}
	public StateEnterHutch enterHutch = new StateEnterHutch();
	public static enum AutomationCommands implements ServerCmd { AutoCenter, EnterHutch };

	public static class KBM implements ServerObj
	{
		public double xPos_V, yPos_V;
	}
	public KBM kbm = new KBM();

	public static class CollectRunState
	{
		// general
		public String runStatus;
		public int runLabel;
		public String scanMode;
		public int optionTab;
		public int startFrameLabel;
		public int nextFrame;

		// common parameters
		public String prefix, dir;
		public boolean inverseOn, savedInverseOn;
		public double beamstop_mm;
		public double atten_factor;
		public double delta_deg;
		public double exposure_sec;
		public double distance_mm;
		public double startAngle_deg, endAngle_deg, savedEndAngle_deg;
		public double beamHeight_um, beamWidth_um;

		// strategy
		public boolean strategyEnabled;
		public String xdsProc;
		public double startAngle2_deg;

		// vector
		public double[] pt1, pt2;
		public boolean reverseDir;
		public boolean helical;
		public double stepSize_um;
		public double wedgeSize;
		public int framesPerSite, savedFPS;
		public int overlap, savedOverlap;
		public String setupStep;
		public int potVecSiteCount;
		public ArrayList<Double> energies;
		public ArrayList<Boolean> selectedSites;
		public boolean siteDeltaEnable;
		public double siteDelta_deg;
		public int siteSkip;

		// mesh
		public double meshFilter;
		public boolean meshCombine;
	}
	public ArrayList<CollectRunState> collectRuns = new ArrayList<>();

	public static class CollectState {
		public String collectState, dataDir;
		public int currentRunIndex, runCount;
	}
	public CollectState collect = new CollectState();

	public static class FPEState {
		public String state;
		public int nextFrame;
	}
	public FPEState fpe = new FPEState();

	public static class JetCollectState {
		public String state;
		public String dir, prefix;
		public double expTime_sec;
		public int framesPerDir;
		public int smxProc;
		public int subDir;
	}
	public JetCollectState jetCollect = new JetCollectState();

	public static class ImageViewer {
		public String fullDir, sampleDir, filename;
		public double contrast, zoom;
		public int serverIndex, viewerIndex;
	}
	public ImageViewer imageViewer = new ImageViewer();

	public enum MotorCommands implements ServerCmd
	{
		MotorMove("dest,f3"),
		Abort();
		private String[] args; MotorCommands(String ... arguments) { this.args = arguments; }
	}
	public ArrayList<String> motorCommands = new ArrayList<>();
}
