package gov.anl.gmca.common.ui;

import gov.anl.gmca.gui.GUI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;

import java.util.ArrayList;

public class GTabColumn extends Composite
{
	ArrayList<Label> labels = new ArrayList<Label>();
	private GTabFolder folder;
	
	public GTabColumn(Composite parent)
	{
		super(parent, SWT.NONE);
		folder = (GTabFolder)parent;
		
		GridLayout gl = new GridLayout(1,false);
		gl.marginWidth=0;
		gl.marginHeight=0;
		gl.horizontalSpacing=0;
		gl.verticalSpacing=0;
		setLayout(gl);
	}
	
	public void addTab(String tabText)
	{
		final int newIndex = labels.size();
		
		Label label = new Label(this, SWT.BORDER | SWT.CENTER);
		label.setText(tabText);
		label.addMouseListener(new MouseListener()
		{
			@Override public void mouseDoubleClick(MouseEvent arg0) {}
			@Override public void mouseUp(MouseEvent arg0) {}
			@Override public void mouseDown(MouseEvent me)
			{
				folder.setSelection(newIndex);
				folder.notifyListeners(SWT.MouseDown, new Event());
			}
		});
		labels.add(label);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint = 20;
		label.setLayoutData(gd);
		layout(true);
	}
	
	public void removeTab(int index)
	{
		labels.get(index).dispose();
		labels.remove(index);
		layout(true);
	}
	
	public void setTabText(int index, String text)
	{
		labels.get(index).setText(text);
	}

	public void updateColors()
	{
		boolean enable = folder.getEnabled();
		
		for (int i=0; i<labels.size();i++)
		{
			Label label = labels.get(i);
			boolean selected = i==folder.getSelectionIndex();
			
			Color bgColor, fgColor;
			if (enable) {
				if (selected) {
					fgColor=GUI.COLOR_WHITE;
					bgColor=GUI.COLOR_DARK_BLUE;
				} else {
					fgColor=GUI.COLOR_BLACK;
					bgColor=GUI.BGCOLOR;
			}} else {
				if (selected) {
					fgColor=GUI.COLOR_LIGHT_GRAY;
					bgColor=GUI.COLOR_DARK_BLUE;
				} else {
					fgColor=GUI.COLOR_DARK_GRAY;
					bgColor=GUI.BGCOLOR;
			}}
				
			label.setBackground(bgColor);
			label.setForeground(fgColor);
		}
	}
	
	@Override public void setEnabled(boolean enable)
	{
		super.setEnabled(enable);
		updateColors();
	}
}
