package gov.anl.gmca.common.ui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;

import java.util.ArrayList;

public class GTabFolder extends Composite
{
	private ArrayList<SelectionListener> slList = new ArrayList<SelectionListener>();
	ArrayList<GTabItem> tabList = new ArrayList<GTabItem>();
	private int selectedIndex = 0;
	GTabColumn tabCol;
	private StackLayout stackLayout;
	Composite stackComp;
	
	public GTabFolder(Composite parent, int style)
	{
		super(parent, SWT.NONE);
		
		GridLayout gl = new GridLayout(2,false);
		gl.marginWidth=0;
		gl.marginHeight=0;
		gl.horizontalSpacing=0;
		gl.verticalSpacing=0;
		setLayout(gl);
		
		if (SWT.LEFT == style) { createTabCol(); createStackComp(); }
		else				   { createStackComp(); createTabCol(); }
	}
	
	private void createStackComp()
	{
		stackComp = new Composite(this, SWT.BORDER);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		stackComp.setLayoutData(gd);
		stackLayout = new StackLayout();
		stackComp.setLayout(stackLayout);
	}
	
	private void createTabCol()
	{
		tabCol = new GTabColumn(this);
		GridData gd = new GridData(SWT.DEFAULT, SWT.FILL, false, true);
		gd.widthHint = 40;
		tabCol.setLayoutData(gd);
	}
	
	// returns the index of the newly added item
	int addTab(GTabItem item)
	{
		tabList.add(item);
		tabCol.addTab(item.getText());
		return tabList.size()-1;
	}
	
	void removeTab(int indexToRemove)
	{
		//System.out.println("Removing tab index "+indexToRemove);
		tabCol.removeTab(indexToRemove);
		Control control = tabList.get(indexToRemove).getControl();
		if (null!=control) control.dispose();
		tabList.remove(indexToRemove);
		for (int i=indexToRemove; i<tabList.size(); i++)
		{
			tabList.get(i).index = i;
		}
	}

	public void addSelectionListener(SelectionListener sl)
	{
		slList.add(sl);
	}
	
	private void notifySelectionListeners()
	{
		for (SelectionListener sl : slList)
		{
			Event e = new Event();
			notifyListeners(SWT.Selection, e);
		}
	}
	
	public void setSelection(int index)
	{
		selectedIndex = index;
		tabCol.updateColors();
		
		stackLayout.topControl = tabList.get(index).getControl();
		stackComp.layout();
		
		notifySelectionListeners();
	}
	
	public int getSelectionIndex()
	{
		return selectedIndex;
	}
	
	public int getItemCount()
	{
		return tabList.size();
	}
	
	public GTabItem getItem(int index)
	{
		return tabList.get(index);
	}
	
	@Override public void setEnabled(boolean enable)
	{
		super.setEnabled(enable);
		tabCol.setEnabled(enable);
	}
}
