package gov.anl.gmca.common.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

public abstract class NBDialog
{
	protected Shell parentShell = null;
	public void setParentShell(Shell parentShell) { this.parentShell=parentShell; }
	public Shell dialogShell = null;
	protected ControlListener controlListener = null;
	protected boolean closedOnEscape = false;

	protected String dialogTitle = "";
	protected boolean mouseDown = false;
	protected Integer returnValue = null;
	public Integer getReturnValue() { return returnValue; }
	protected int shellStyle;
	public volatile boolean closeRequest;
	
	protected NBDialog(Shell parentShell, int shellStyle)
	{
		this.shellStyle = shellStyle;
		closeRequest = false;
		this.parentShell = parentShell;
	}
	
	public void setText(String title)
	{
		dialogTitle = title;
	}
	
	protected void createShell()
	{
		dialogShell = new Shell(parentShell, shellStyle);
		dialogShell.setText(dialogTitle);
		
		dialogShell.addTraverseListener(new TraverseListener()
		{
			@Override public void keyTraversed(TraverseEvent te)
			{
				if (te.detail == SWT.TRAVERSE_ESCAPE)
				{
					closedOnEscape = true;
					te.doit = true;
				}
			}
		});
		dialogShell.addShellListener(new ShellListener()
		{
			@Override public void shellActivated(ShellEvent shellEvent) {}
			@Override public void shellClosed(ShellEvent shellEvent) { onDialogClose(); }
			@Override public void shellDeactivated(ShellEvent shellEvent) {}
			@Override public void shellDeiconified(ShellEvent shellEvent) {}
			@Override public void shellIconified(ShellEvent shellEvent) {}
		});
	}

	protected void createWidgets()
	{
		setupLayout();
		createContents(dialogShell);
	}
	
	public void closeDialog()
	{
		if (null != dialogShell && !dialogShell.isDisposed())
		{
			dialogShell.dispose();
		}
	}
	
	protected boolean calledFromGUIThread()
	{
		return (parentShell.getDisplay().getThread()==Thread.currentThread());
	}
	
	private static void centerShell(Shell parentShell, Shell shellToCenter)
	{
		if (null==parentShell || null==shellToCenter) return;
		Rectangle b1 = parentShell.getBounds(), b2 = shellToCenter.getBounds();
		Point parentCenter = new Point(b1.x+b1.width/2, b1.y+b1.height/2);
		Point newCenter = new Point(parentCenter.x-b2.width/2, parentCenter.y-b2.height/2);
		shellToCenter.setLocation(newCenter);
		shellToCenter.setFocus();
	}
	
	public int open()
	{
		asyncOpen();

		// wait for the dialog to close
		if (calledFromGUIThread())
		{
			System.out.println("Called from GUI thread, starting message pump.");
			while (!dialogShell.isDisposed() && null==returnValue)
			{
				if (!GUI.readAndDispatch())
				{
					GUI.getDisplay().sleep();
				}
				if (closeRequest)
				{
					dialogShell.close();
					break;
				}
			}
		}
		else
		{
			System.out.println("Called from a non-GUI thread, blocking.");
			while (null==returnValue)
			{
				Util.safeSyncExec(new Runnable(){public void run()
				{
					GUI.processMessages(10000);
				}});
				Util.sleep(10);

				if (closeRequest)
				{
					Util.safeSyncExec(new Runnable(){public void run()
					{
						dialogShell.close();
					}});
					break;
				}
			}
		}

		return (null==returnValue)?SWT.CANCEL:returnValue;
	}

	public void asyncOpen()
	{
		closeRequest = false;

		// the runnable to create and show the dialog
		Runnable runnable = new Runnable() {
			public void run() {
				createShell();
				createWidgets();

				dialogShell.pack();
				centerShell(parentShell, dialogShell);
				dialogShell.open();
			}
		};

		// call the runnable either directly or through async
		if (calledFromGUIThread()) {
			System.out.println("Called from GUI thread, creating shell directly");
			runnable.run();
		} else {
			System.out.println("Called from non-GUI thread, calling asyncExec");
			GUI.asyncExec(runnable);
		}
	}

	public void close()
	{
		closeRequest = true;
	}
	
	protected void setupLayout()
	{
		dialogShell.setBackgroundMode(SWT.INHERIT_FORCE);
		
		GridLayout gl = new GridLayout(1,false);
		gl.marginWidth=0;
		gl.marginHeight=0;
		dialogShell.setLayout(gl);
	}
	
	abstract protected void createContents(Composite parent);
	protected void onDialogClose() {}
}