package gov.anl.gmca.common.ui;

import gov.anl.gmca.gui.GUI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

// A standard messagebox with an option to ignore further messages
// for a user-selected amount of time
//
// The return value is the time in seconds to wait
public class NBDlgIgnore extends NBDialog
{
	protected String message;
	protected Integer icon;
	protected Combo ignoreCombo;
	protected int tempReturnValue=-1;
	
	protected static final Object[][] choices =
	{
		{ "Don't ignore",	0 		},
		{ "1 minute",		60 		},
		{ "10 minutes",		10*60	},
		{ "1 hour",			60*60	},
		{ "4 hours",		4*60*60 }
	};
	
	public NBDlgIgnore (String title, String message, Integer icon)
	{
	    super(GUI.getShell(), SWT.APPLICATION_MODAL|SWT.DIALOG_TRIM);
		setText(title);
		this.message = message;
		this.icon = icon;
	}
	
	@Override protected void createContents(Composite parent)
	{
        // disable 'x' button in top right corner
        dialogShell.addListener (SWT.Close, new Listener () {
    		public void handleEvent (Event event) {
    			event.doit = false;
    		}
    	});
        
        GridLayout gl = new GridLayout(1,false);
        gl.marginWidth = 10;
        gl.marginHeight = 10;
        gl.verticalSpacing = 10;
        parent.setLayout(gl);
        GridData gd = null;

        Composite messageComp = new Composite(parent, SWT.NONE);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        messageComp.setLayoutData(gd);
        gl = new GridLayout(2,false);
        gl.horizontalSpacing = 20;
        messageComp.setLayout(gl);
        
        Label lblIcon = new Label(messageComp, SWT.NONE);
        if (null != icon) lblIcon.setImage(Display.getDefault().getSystemImage(icon));
        gd = new GridData();
        lblIcon.setLayoutData(gd);
        
        Label lblMessage = new Label(messageComp, SWT.WRAP);
        lblMessage.setText(message);
        gd = new GridData();
        gd.widthHint = 350;
        gd.horizontalAlignment = SWT.FILL;
        gd.grabExcessHorizontalSpace = true;
        lblMessage.setLayoutData(gd);
        
        Composite ignoreComp = new Composite(parent, SWT.NONE);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        ignoreComp.setLayoutData(gd);
        gl = new GridLayout(2,false);
        gl.marginHeight=0;
        ignoreComp.setLayout(gl);

        Label lblIgnore = new Label(ignoreComp, SWT.RIGHT);
        lblIgnore.setText("Ignore this message for: ");
        gd = new GridData(GridData.FILL_HORIZONTAL);
        lblIgnore.setLayoutData(gd);
        
        ignoreCombo = new Combo(ignoreComp, SWT.READ_ONLY);
        for (Object[] choice : choices)
        {
        	String choiceText = (String)choice[0];
        	ignoreCombo.add(choiceText);
        }
        ignoreCombo.select(0);
        
        ignoreCombo.addModifyListener(new ModifyListener()
        {
			@Override public void modifyText(ModifyEvent arg0)
			{
				updateIgnoreTime();
			}
		});
        
        Composite buttonComp = new Composite(parent,SWT.NONE);
        gd = new GridData();
        gd.horizontalSpan=2;
        gd.grabExcessHorizontalSpace=true;
        gd.horizontalAlignment=SWT.FILL;
        buttonComp.setLayoutData(gd);
        gl = new GridLayout(2,false);
        gl.marginHeight=0;
        buttonComp.setLayout(gl);
        
        Composite filler = new Composite(buttonComp, SWT.NONE);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.heightHint=1;
        filler.setLayoutData(gd);
        
        final Button okButton = new Button(buttonComp,SWT.NONE);
        okButton.setText("OK");
        gd = new GridData();
        gd.horizontalAlignment=SWT.RIGHT;
        gd.widthHint = 60;
        okButton.setLayoutData(gd);
        okButton.addSelectionListener(new SelectionListener()
        {
			public void widgetDefaultSelected(SelectionEvent arg0) {}

			public void widgetSelected(SelectionEvent arg0)
			{
				okButton.setFocus();
				updateIgnoreTime();
				returnValue = tempReturnValue; // only set on close
				closeDialog();
			}
		});
        
        dialogShell.setDefaultButton(okButton);
	}
	
	protected void updateIgnoreTime()
	{
		int selection = ignoreCombo.getSelectionIndex();
		if (-1 != selection)
		{
			tempReturnValue = (Integer)(choices[selection][1]);
		}
		System.out.println("Updated secondsToIgnore="+tempReturnValue);
	}
}
