package gov.anl.gmca.common.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.util.Pair;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;

public class NBMessageBox extends NBDialog
{
	public static enum FieldType { TEXT, NONE };
	protected FieldType fieldType = FieldType.NONE;
	protected Text text = null;
	protected String textContents = "";
	
	protected Runnable runOnSubmit = null;
	public void setRunnableOnSubmit(Runnable r) { runOnSubmit = r; }

	protected String dialogMessage = "";
	protected int dialogIconType = SWT.ICON_INFORMATION, style = SWT.NONE;
	
	int[] iconValues =
	{
    	SWT.ICON_CANCEL,
    	SWT.ICON_ERROR,
    	SWT.ICON_INFORMATION,
    	SWT.ICON_QUESTION,
    	SWT.ICON_SEARCH,
    	SWT.ICON_WARNING,
    	SWT.ICON_WORKING
	};
	
	Object[] buttonValues =
	{
		SWT.RETRY,	"Retry",
		SWT.CANCEL, "Cancel",
		SWT.OK, 	"OK",
		SWT.NO, 	"No",
		SWT.YES, 	"Yes"
	};
	
	private String[] customButtons = null;
	public void setCustomButtons(String[] values) { customButtons = values; }
	
	protected ArrayList<Button> buttons = new ArrayList<Button>();
	
	public NBMessageBox(Shell parentShell, int style)
	{
		super(parentShell, SWT.APPLICATION_MODAL|SWT.DIALOG_TRIM);
		this.style = style;
		init();
	}

	public NBMessageBox(Shell parentShell, int style, boolean modal)
	{
		super(parentShell, SWT.DIALOG_TRIM);
		// TODO: change all callers to use one constructor
 		if (modal) { System.out.println("ERROR: For now please use the two-argument constructor for modal dialogs"); System.exit(-1); }
		this.style = style;
		init();
	}

	private void init()
	{
		for (int iconValue : iconValues)
		{
			if (0 != (iconValue & style)) dialogIconType = iconValue;
		}
	}

	public void setMessage(String msg)
	{
		dialogMessage = msg;
	}
	
	public void setFieldType(FieldType fieldType)
	{
		this.fieldType = fieldType;
	}
	
	public String getText()
	{
		if (fieldType != FieldType.TEXT) throw new IllegalStateException("This dialog does not have a text field.");
		return textContents;
	}

	private ArrayList<Pair<Integer,String>> getButtonLabels()
	{
		ArrayList<Pair<Integer,String>> ret = new ArrayList<Pair<Integer,String>>();
		
		if (null != customButtons)
		{
			int index=0;
			for (String label : customButtons)
			{
				ret.add(new Pair<Integer,String>(index++,label));
			}
		}
		else
		{
			for (int i=0; i<buttonValues.length; i+=2)
			{
				int buttonValue = (Integer)buttonValues[i];
				String buttonString = (String)buttonValues[i+1];
				
				if (0 != (style & buttonValue))
				{
					ret.add(new Pair<Integer,String>(buttonValue, buttonString));
				}
			}
			
			if (ret.size()==0)
			{
				style |= SWT.OK;
				ret.add(new Pair<Integer,String>(SWT.OK, "OK"));
			}
		}
		return ret;
	}
	
	protected void createMessageWidget(Composite parent)
	{
		Label label = new Label(parent, SWT.NONE);
		label.setText(dialogMessage);
		label.setFont(GUI.FONT_DEFAULT);
	}
	
	@Override protected void createContents(Composite parent)
	{
		ArrayList<Pair<Integer,String>> buttonLabels = getButtonLabels();
		
		Composite comp = new Composite(parent, SWT.NONE);
		comp.setBackgroundMode(SWT.INHERIT_FORCE);
		GridLayout gl = new GridLayout(2,false);
		gl.marginHeight = 10;
		gl.marginWidth = 10;
		comp.setLayout(gl);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		comp.setLayoutData(gd);
		
		boolean hasTextField = fieldType==FieldType.TEXT;
		
		Label icon = new Label(comp, SWT.NONE);
		icon.setImage(GUI.getDisplay().getSystemImage(dialogIconType));
		icon.setLayoutData(new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false, 1, hasTextField?2:1));
		
		createMessageWidget(comp);
		
		if (fieldType == FieldType.TEXT)
		{
			text = new Text(comp, SWT.BORDER);
			text.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
			text.addTraverseListener(new TraverseListener()
			{
				@Override public void keyTraversed(TraverseEvent te)
				{
					if (te.detail == SWT.TRAVERSE_RETURN)
					{
						submit(SWT.OK);
					}
				}
			});
		}
		
		Composite buttonComp = new Composite(comp, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
		buttonComp.setLayoutData(gd);
		gl = new GridLayout(1+buttonLabels.size(),false);
		buttonComp.setLayout(gl);
		
		Composite spacer = new Composite(buttonComp, SWT.NONE);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 1, 1);
		gd.heightHint = 1;
		spacer.setLayoutData(gd);

		for (final Pair<Integer,String> buttonPair : buttonLabels)
		{
			Button button = new Button(buttonComp, SWT.PUSH);
			button.setText(buttonPair.b);
			button.setFont(GUI.FONT_DEFAULT);
			gd = new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false);
			gd.widthHint = 100;
			button.setLayoutData(gd);
			button.addSelectionListener(new SelectionListener()
			{
				@Override public void widgetDefaultSelected(SelectionEvent se) {}
				@Override public void widgetSelected(SelectionEvent se)
				{
					submit(buttonPair.a);
				}
			});
			button.setSelection(true);
		}
	}

	protected boolean submitCalled = false;
	protected void submit(int value)
	{
		returnValue = value;
		if (null != text) textContents = text.getText();
		closeDialog();
		
		if (null!=runOnSubmit && !submitCalled) { submitCalled=true; runOnSubmit.run(); }
	}
}