package gov.anl.gmca.common.ui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

public class NBProgress extends NBMessageBox
{
    int wait_sec = 0;
    public void setWait_sec(int i) { wait_sec=i; }
    long wait_start_ms = 0;

    private ProgressBar progressBar;

    public NBProgress(Shell parent, int style)
    {
        super(parent, style);
        wait_start_ms = System.currentTimeMillis();
    }

    @Override
    protected void createContents(Composite parent)
    {
        // top-level container
        Composite comp = new Composite(parent, SWT.NONE);
        comp.setBackgroundMode(SWT.INHERIT_FORCE);
        GridLayout gl = new GridLayout(2,false);
        gl.marginHeight = 10;
        gl.marginWidth = 10;
        comp.setLayout(gl);
        GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
        comp.setLayoutData(gd);

        // message
        Label label = new Label(comp, SWT.NONE);
        label.setText(dialogMessage);
        //label.setFont(GUI.FONT_DEFAULT);
        gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
        label.setLayoutData(gd);

        // progress bar
        progressBar = new ProgressBar(comp, SWT.HORIZONTAL);
        progressBar.setMinimum(0);
        progressBar.setMaximum(wait_sec);
        progressBar.setSelection(0);
        gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
        progressBar.setLayoutData(gd);

        // spacer for cancel button
        Composite spacer = new Composite(comp, SWT.NONE);
        gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
        gd.heightHint=1;
        spacer.setLayoutData(gd);

        // cancel button
        Button button = new Button(comp, SWT.PUSH);
        button.setText("Cancel");
        //button.setFont(GUI.FONT_DEFAULT);
        gd = new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false);
        gd.widthHint = 100;
        button.setLayoutData(gd);
        button.addSelectionListener(new SelectionListener()
        {
            @Override public void widgetDefaultSelected(SelectionEvent se) {}
            @Override public void widgetSelected(SelectionEvent se)
            {
                submit(0);
            }
        });
        button.setSelection(true);

        // start timer
        dialogShell.getDisplay().timerExec(1000, ()->updateProgress());
    }

    public void updateProgress()
    {
        if (submitCalled) return;
        try
        {
            long now_ms = System.currentTimeMillis();
            int secSinceStart = Math.round((now_ms-wait_start_ms)/1000);
            if (secSinceStart>=wait_sec) {
                submit(1);
                return;
            }
            progressBar.setSelection(secSinceStart);
        }
        catch (Throwable t) { t.printStackTrace(); }

        dialogShell.getDisplay().timerExec(1000, ()->updateProgress());
    }

    @Override protected void onDialogClose()
    {
        submit(0);
    }
}
