package gov.anl.gmca.common.ui;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.State;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.HashMap;

public class WdgAppState
{
	protected static Shell shell;

	protected static HashMap<State,Control> controlMap = new HashMap<>();

	private static boolean isReadOnly = false;
	public static void open(boolean readOnly)
	{
		boolean ro_changed = (readOnly!=isReadOnly);
		boolean shellExists = (null!=shell && !shell.isDisposed());
		if (shellExists && !ro_changed)
		{
			shell.moveAbove(GUI.getShell());
			shell.setFocus();
			return;
		}
		if (shellExists && ro_changed)
		{
			Log.note("closing existing shell");
			shell.dispose();
		}
		isReadOnly=readOnly;
		
		shell = new Shell(GUI.getDisplay());
		shell.setText("Application state");
		shell.setLayout(new FillLayout());
		shell.addTraverseListener(new TraverseListener()
		{
			@Override public void keyTraversed(TraverseEvent e)
			{
				if (e.detail==SWT.TRAVERSE_ESCAPE) shell.dispose();
			}
		});
		
		Group stateGroup = new Group(shell, SWT.NONE);
		GridLayout gl = new GridLayout(3,false);
		stateGroup.setLayout(gl);
		stateGroup.setText("States");
		
		for (State state : State.values())
		{
			if (isReadOnly)
			{
				Label label = new Label(stateGroup, SWT.BORDER);
				label.setText(state.toString().toLowerCase());
				controlMap.put(state,label);
			}
			else
			{
				Button button = new Button(stateGroup, SWT.PUSH);
				button.setText(state.toString().toLowerCase());
				button.setEnabled(!readOnly);
				controlMap.put(state,button);
				
				final State tState = state;
				button.addSelectionListener(new SelectionListener()
				{
					@Override public void widgetDefaultSelected(SelectionEvent se) {}
					@Override public void widgetSelected(SelectionEvent se)
					{
						AppState.setState(tState, !AppState.getBusyMap().get(tState));
					}
				});
			}
			Control control = controlMap.get(state);
			GridData gd = new GridData(GridData.FILL_BOTH);
			control.setLayoutData(gd);
		}

		final ListenerList.Listener<AppState> listener = new ListenerList.Listener<AppState>()
		{
			@Override public void handleEvent(AppState object, Object data)
			{
				updateButtons();
			}
		};
		
		AppState.stateLL.addListener(listener);
		shell.addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent e)
			{
				AppState.stateLL.removeListener(listener);
				shell = null;
			}
		});
		
		updateButtons();
		
		shell.pack();
		shell.open();
	}
	
	protected static void updateButtons()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			for (State state : State.values())
			{
				Color bgColor = AppState.getBusyMap().get(state)?GUI.COLOR_LIGHT_RED:GUI.COLOR_LIGHT_GREEN;
				controlMap.get(state).setBackground(bgColor);
			}
		}});
	}
}
