package gov.anl.gmca.common.ui;

import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.ValidateDirPrefix;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Disk;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.io.IOException;

// This class directly uses the DataDir class to get the base data directory.
// Only the directory suffix is displayed in the directory widget.
//
// Currently there is no way to use this widget without using the common data
// directory.
//
public class WdgControlDir extends WdgControl
{
	protected Text prefixText, dirText;
	protected Button dirButton, dataDirButton;
	protected Group fileDirGroup;
	protected Composite internalParent;
	
	public abstract static class ModelInterface extends WdgControl.ModelInterface
	{
		public PrefixDir getPrefixDir() { return null; }
	}
	
	public static class PrefixDir
	{
		public PrefixDir(String prefix, String dir) { this.prefix=prefix; this.dir=dir; }
		public String prefix;
		public String dir;
	}
	
	public WdgControlDir(Composite parent, int style, String groupTitle, int verticalSpacing, ModelInterface _mi)
	{
		super(parent, style, _mi);

		GridLayout internalLayout = new GridLayout(4,false);
		internalLayout.horizontalSpacing=0;
		internalLayout.verticalSpacing=0;
		internalLayout.marginWidth=0;
		internalLayout.marginHeight=0;
		
		///////////////////////////////////// enclosing group /////////////////////////////////////
		
		internalParent = this;
		if (null != groupTitle)
		{
			setLayout(new FillLayout());
    		fileDirGroup = new Group(this,SWT.NONE);
    		fileDirGroup.setBackground(GUI.BGCOLOR);
    		fileDirGroup.setText(groupTitle);
    		fileDirGroup.setLayout(internalLayout);
    		internalParent = fileDirGroup;
		}
		else
		{
			setLayout(internalLayout);
		}
		
		internalParent.setBackground(GUI.BGCOLOR);
		
		////////////////////////////////////////// prefix ///////////////////////////////////////////
		
		Label prefixLabel = new Label(internalParent,SWT.NONE);
		prefixLabel.setText("Prefix: ");
		prefixLabel.setFont(GUI.FONT_DEFAULT);
		prefixLabel.setBackground(GUI.BGCOLOR);
		prefixLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));

		prefixText = new Text(internalParent,SWT.BORDER);
		prefixText.setTextLimit(19);
		prefixText.setFont(GUI.FONT_DEFAULT);
		prefixText.setText("");
		prefixText.setEnabled(false);
		prefixText.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1));
		
		prefixText.addVerifyListener(new VerifyListener()
		{
			@Override public void verifyText(VerifyEvent ve)
			{
				if (ve.character==SWT.NONE || ve.character==SWT.BS || ve.character==SWT.DEL || ve.character==SWT.LF) return;
			
				String newText = prefixText.getText()+ve.character;
				if (!ValidateDirPrefix.isValidFilename(newText))
				{
					Log.note("Only letters, numbers, dashes and underscores are allowed in filenames");
					ve.doit=false;return;
				}
			}
		});

		prefixText.addTraverseListener(new TraverseListener()
		{
			@Override public void keyTraversed(TraverseEvent te)
			{
				if (te.detail == SWT.TRAVERSE_ESCAPE)
				{
					// TODO: support Object in getDeviceValue() and then return it here
					// so it can be reset with the escape key
				}
				else if
				(
					te.detail	==	SWT.TRAVERSE_RETURN 		||
					te.detail	==	SWT.TRAVERSE_TAB_NEXT 		||
					te.detail	==	SWT.TRAVERSE_TAB_PREVIOUS
				)
				{
					sendToDevice();
				}
			}
		});
		
		prefixText.addFocusListener(new FocusListener()
		{
			@Override public void focusLost(FocusEvent e)
			{
				sendToDevice();
			}
			@Override public void focusGained(FocusEvent e) {}
		});
		
		Composite fillerComp = new Composite(internalParent, SWT.NONE);
		GridData gd = new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false);
		gd.widthHint=1;
		gd.heightHint=1;
		fillerComp.setLayoutData(gd);
		
		////////////////////////////////////////// directory //////////////////////////////////////
		
		Label dirLabel = new Label(internalParent,SWT.NONE);
		dirLabel.setText("Dir: ");
		dirLabel.setFont(GUI.FONT_DEFAULT);
		dirLabel.setBackground(GUI.BGCOLOR);
		dirLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, true));
		
		dataDirButton = new Button(internalParent, SWT.PUSH);
		dataDirButton.setText(".../");
		dataDirButton.setEnabled(false);
		dataDirButton.setLayoutData(new GridData(SWT.LEFT, SWT.DEFAULT, false, false));
		dataDirButton.setFont(GUI.FONT_DEFAULT);
		dataDirButton.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0)
			{
				ExecTask.asyncExec("/usr/bin/caja " + DevDataDir.getPrimary(), "caja", false, false);
			}
		});

		dirText = new Text(internalParent,SWT.BORDER);
		dirText.setEnabled(false);
		dirText.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		dirText.setFont(GUI.FONT_DEFAULT);

		dirText.addVerifyListener(new VerifyListener()
		{
			@Override public void verifyText(VerifyEvent ve)
			{
				if (ve.character==SWT.NONE || ve.character==SWT.BS || ve.character==SWT.DEL || ve.character==SWT.LF) return;
				
				String newText = dirText.getText()+ve.character;
				if (!ValidateDirPrefix.isValidDir(newText))
				{
					Log.note("Only letters, numbers, dashes, underscores and forward slashes are allowed in directories");
					ve.doit=false;return;
				}
			}
		});

		dirText.setToolTipText(dirText.getText());
		dirText.addModifyListener(new ModifyListener(){ public void modifyText(ModifyEvent me)
		{
			updateDirColor();
		}});

		dirText.addTraverseListener(new TraverseListener()
		{
			@Override public void keyTraversed(TraverseEvent te)
			{
				if (te.detail == SWT.TRAVERSE_ESCAPE)
				{
					// TODO: support Object in getDeviceValue() and then return it here
					// so it can be reset with the escape key
				}
				else if
				(
					te.detail	==	SWT.TRAVERSE_RETURN 		||
					te.detail	==	SWT.TRAVERSE_TAB_NEXT 		||
					te.detail	==	SWT.TRAVERSE_TAB_PREVIOUS
				)
				{
					sendToDevice();
				}
			}
		});

		dirText.addFocusListener(new FocusListener()
		{
			@Override public void focusLost(FocusEvent e)
			{
				sendToDevice();
			}
			@Override public void focusGained(FocusEvent e) {}
		});

		dirButton = new Button(internalParent,SWT.PUSH);
		dirButton.setEnabled(false);
		dirButton.setFont(GUI.FONT_DEFAULT);
		dirButton.setImage(GUI.IMAGE_FOLDER);
		dirButton.setLayoutData(new GridData(SWT.RIGHT, SWT.DEFAULT, false, false));

		dirText.addControlListener(new ControlListener()
		{
			@Override public void controlResized(ControlEvent ce)
			{
				((GridData)dirButton.getLayoutData()).heightHint = dirText.getSize().y;
				((GridData)dataDirButton.getLayoutData()).heightHint = dirText.getSize().y;
				internalParent.layout();
			}
			@Override public void controlMoved(ControlEvent arg0) {}
		});
		
		// open the directory selection dialog and set the directory if requested
		dirButton.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0)
			{
				NBDirectoryDialog dd = new NBDirectoryDialog(GUI.getShell());
				String widgetDir = ((WdgControlDir.ModelInterface)mi).getPrefixDir().dir;
				dd.setFilterPath(DevDataDir.addToPrimary(widgetDir));
				String fullDir = dd.openStr();
				if (null != fullDir)
				{
					try
					{
						fullDir = (new java.io.File(fullDir)).getCanonicalPath().toString();
					}
					catch (IOException ioe) { ioe.printStackTrace(); }
					
					String subDir = checkNewFullDir(fullDir);
					if (null != subDir)
					{
						setDir(subDir);
						sendToDevice();
					}
				}
			}
		});
		
		addListeners();
		
		//////////////////////////////////// end widgets ////////////////////////////////////
	}
	
	private ListenerList.Listener<DevDataDir> dataDirListener;
	private void addListeners()
	{
		dataDirListener = new ListenerList.Listener<DevDataDir>()
		{
			@Override public void handleEvent(DevDataDir object, Object data)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					handleDirChange(DevDataDir.getPrimary());
				}});
			}
		};
		
		DevDataDir.llDirChange.addListener(dataDirListener);
		handleDirChange(DevDataDir.getPrimary());
		
		addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent de)
			{
				removeListeners();
			}
		});
	}
	
	private void removeListeners()
	{
		DevDataDir.llDirChange.removeListener(dataDirListener);
	}

	private void handleDirChange(String newDataDir)
	{
		dataDirButton.setToolTipText(newDataDir);
		updateDirColor();
	}
	
	private String checkNewFullDir(String newDir)
	{
		String commonDir = DevDataDir.getPrimary();
		
		if (newDir.indexOf(commonDir)!=0)
		{
			Util.popupError("Directory error", "The chosen directory:\n\n"+newDir+"\n\nis not a subdirectory of the common directory:\n\n"+commonDir+"\n\nTo change the common data directory please ask your host.");
			return null;
		}
		
		if (!ValidateDirPrefix.isValidDir(newDir))
		{
			Util.popupError("Directory error", "The chosen directory:\n\n"+newDir+"\n\ncontains invalid characters. Only letters, numbers, dashes, underscores and forward slashes are allowed.");
			return null;
		}
		
		if (newDir.equals(commonDir))
		{
			return "";
		}
		else
		{
			String subDir = newDir.substring(commonDir.length()+1);
			return subDir;
		}
	}
	
	protected void sendToDevice()
	{
		String prefix = prefixText.getText().length() != 0 ? prefixText.getText() : ((WdgControlDir.ModelInterface)mi).getPrefixDir().prefix;
		String dir = dirText.getText();
		mi.setValue(new PrefixDir(prefix,dir));
	}
	
	@Override
	public void setEnabled(boolean enable)
	{
		prefixText.setEnabled(enable);
		dirButton.setEnabled(enable);
		dataDirButton.setEnabled(enable);
		
		dirText.setEnabled(enable);
		updateDirColor();
	}
	
	public void setPrefix(String newPrefix)
	{
		prefixText.setText(newPrefix);
	}
	
	public void setDir(String newDir)
	{
		dirText.setText(newDir);
		dirText.setSelection(newDir.length());
		dirText.setToolTipText(newDir);
		updateDirColor();
	}
	
	protected void updateDirColor()
	{
		boolean enabled = dirText.getEnabled();
		if (!enabled)
		{
			dirText.setForeground(GUI.COLOR_DISABLED_GRAY);
			return;
		}
		
		String currentText = dirText.getText();
		String currentFullDir = DevDataDir.addToPrimary(currentText);
		
		// text is red if the directory doesn't exist
		Color fgColor = GUI.COLOR_BLACK;
		try
		{
			boolean isDir = Disk.isDirectory(currentFullDir);
			//System.out.println("Checked "+currentFullDir+", isDir="+isDir);
			if (!isDir)
			{
				fgColor = GUI.COLOR_RED;
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		dirText.setForeground(fgColor);
	}

	@Override
	public boolean updateModel()
	{
		return true;
	}
}
