package gov.anl.gmca.common.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgEnableDisable extends Composite
{
	public ListenerList<WdgEnableDisable> llUserSelection;
	private Boolean selection = null;
	private MouseListener enableMouseListener, disableMouseListener;
	private MouseTrackListener enableTrackListener, disableTrackListener;
	private Label enableLabel, disableLabel;
	
	public WdgEnableDisable(Composite parent, String enableString, String disableString)
	{
		super(parent, SWT.NONE);
		llUserSelection = new ListenerList<WdgEnableDisable>(this);
		
		GridLayout gl = new GridLayout(2,true);
		gl.marginHeight=0;
		gl.marginWidth=0;
		setLayout(gl);
		
		disableLabel = new Label(this, SWT.CENTER|SWT.BORDER);
		disableLabel.setFont(GUI.FONT_11_BOLD);
		disableLabel.setText(disableString);
		GridData gd = new GridData();
		gd.widthHint = 100;
		disableLabel.setLayoutData(gd);
		
		enableLabel = new Label(this, SWT.CENTER|SWT.BORDER);
		enableLabel.setFont(GUI.FONT_11_BOLD);
		enableLabel.setText(enableString);
		gd = new GridData();
		gd.widthHint = 100;
		enableLabel.setLayoutData(gd);
		
		connectListeners();
	}
	
	public Boolean getSelection()
	{
		return selection;
	}
	
	public void setSelection(boolean enable)
	{
		selection = enable;
		
		if (enable)
		{
			enableLabel.setBackground(GUI.COLOR_GREEN);
			enableLabel.setForeground(GUI.COLOR_WHITE);
			disableLabel.setBackground(GUI.COLOR_LIGHT_GRAY);
			disableLabel.setForeground(GUI.COLOR_DARK_GRAY);
		}
		else
		{
			enableLabel.setBackground(GUI.COLOR_LIGHT_GRAY);
			enableLabel.setForeground(GUI.COLOR_DARK_GRAY);
			disableLabel.setBackground(GUI.COLOR_RED);
			disableLabel.setForeground(GUI.COLOR_WHITE);
		}
	}
	
	public void select(boolean enable)
	{
		setSelection(enable);
		llUserSelection.notifyListeners(enable);
	}
	
	private void connectListeners()
	{
		enableMouseListener = new MouseListener()
		{
			@Override public void mouseDoubleClick(MouseEvent arg0) {}
			@Override public void mouseUp(MouseEvent arg0) {}
			@Override public void mouseDown(MouseEvent arg0)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					select(true);
				}});
			}
		};
		enableLabel.addMouseListener(enableMouseListener);
		
		disableMouseListener = new MouseListener()
		{
			@Override public void mouseDoubleClick(MouseEvent arg0) {}
			@Override public void mouseUp(MouseEvent arg0) {}
			@Override public void mouseDown(MouseEvent arg0)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					select(false);
				}});
			}
		};
		disableLabel.addMouseListener(disableMouseListener);
		
		enableTrackListener = new MouseTrackListener()
		{
			@Override public void mouseEnter(MouseEvent arg0)
			{
			}

			@Override public void mouseExit(MouseEvent arg0)
			{
			}

			@Override public void mouseHover(MouseEvent arg0) {}
		};
		enableLabel.addMouseTrackListener(enableTrackListener);

		disableTrackListener = new MouseTrackListener()
		{
			@Override public void mouseEnter(MouseEvent arg0)
			{
			}

			@Override public void mouseExit(MouseEvent arg0)
			{
			}

			@Override public void mouseHover(MouseEvent arg0) {}
		};
		disableLabel.addMouseTrackListener(disableTrackListener);
		
		addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent arg0)
			{
				removeListeners();
			}
		});
	}
	
	private void removeListeners()
	{
		enableLabel.removeMouseListener(enableMouseListener);
		disableLabel.removeMouseListener(disableMouseListener);
		enableLabel.removeMouseTrackListener(enableTrackListener);
		disableLabel.removeMouseTrackListener(disableTrackListener);
	}
}
