package gov.anl.gmca.common.ui;

import gov.anl.gmca.analysis.helpers.StrategyResults;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.DevRunList;
import gov.anl.gmca.common.DevRunList.DevRun;
import gov.anl.gmca.common.DevRunList.RUN_LABEL_SETTING;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.common.ModelRunList.ModelRun;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_ROBOT;
import gov.anl.gmca.io.PVLookup.PV_RUN;
import gov.anl.gmca.io.PVLookup.PV_SCR_RUN;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterRunList;
import gov.anl.gmca.spotfinder.SFData.SFType;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

import java.util.ArrayList;

public class WdgRunTabs extends Composite implements ModelRunList.Listener
{
	/////////////////////////////////////////////////// Listener ////////////////////////////////////////
	
	public abstract static class Listener
	{
		public abstract void update();
	}
	
	protected ArrayList<Listener> listenerList = new ArrayList<Listener>();
	public void addListener(Listener listener) { listenerList.add(listener); listener.update(); }
	public void removeListener(Listener listener) { while (listenerList.remove(listener)) {}}
	public void notifyListeners() { for (Listener l : listenerList) { l.update(); } }
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////

	public boolean remoteOpInProgress = false;
	public boolean getRemoteOpInProgress() { return remoteOpInProgress; }
	public void setRemoteOpInProgress() { remoteOpInProgress = true; }
	
	public GTabFolder runTabFolder;
	protected ModelRunList runList;
	protected DevRunList runListDev;
	protected WdgRunTabFactory runTabFactory;
	protected boolean areTabsInitialized = false;
	protected boolean isTabAdditionInProgress = false;
	
	public interface WdgRunTabFactory
	{
		public Composite createRunTab(Composite parent, int style, int runIndex, ModelRun modelRun, DevRun devRun, WdgRunTabs runTabs, GTabItem tabItem);
	}
	
	public WdgRunTabs(Composite parent, int style, final ModelRunList runList, DevRunList runListDev, WdgRunTabFactory runTabFactory)
	{
		super(parent, style);
		this.runList=runList;
		this.runListDev=runListDev;
		this.runTabFactory=runTabFactory;
		setLayout(new FillLayout());
		
		runTabFolder = new GTabFolder(this, style);
		runTabFolder.setEnabled(false);
		runTabFolder.setFont(GUI.FONT_COLLECT_GROUP);
		runTabFolder.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				// don't let the widget change tabs itself
				runTabFolder.setSelection(runList.getCurrentRunIndex());
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		runTabFolder.addMouseListener(new MouseListener()
		{
			@Override public void mouseDown(MouseEvent e)
			{
				handleUserClick();
			}
			@Override public void mouseDoubleClick(MouseEvent e) {}
			@Override public void mouseUp(MouseEvent e) {}
		});
	}
	
	public void connectListeners()
	{
		runList.addListener(this);
	}
	
	public void setTabColor(GTabItem item, Color color)
	{
		// TODO: this is not supported yet
		// item.setColor(color);
	}
	
	public void setTabText(GTabItem item, String text)
	{
		if (item.getText().equals(text)) return;
		
		item.setText(text);
		
		// a temporary fix to get the TabFolder to redraw after changing
		// labels, which may change the width of the labels and the tab contents
		if (runTabFolder.getItemCount()>=2)
		{
			int selection = runTabFolder.getSelectionIndex();
    		runTabFolder.setSelection(1);
    		runTabFolder.setSelection(0);
    		runTabFolder.setSelection(selection);
		}
	}
	
	public void updateTabs()
	{
		try
		{
    		runList.setRunCount(runListDev.getRunCount());
    		
			runList.notifyListeners();
			runList.notifyCurrentRunListeners();
			
			for (int i=0;i<=runList.getRunCount();i++)
			{
	    		ModelRasterRun r = (ModelRasterRun)runList.getRun(i);
	    		r.llRunChanged.notifyListeners();
	    		//r.dev().sendNotification();
	    		Log.note("updated raster run "+i);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	// model reports a new run count or current run
	@Override
	public void runListChanged(ModelRunList runList)
	{
		//System.out.println("runListChanged: runCount="+runList.getRunCount()+" currentRun="+runList.getCurrentRunIndex());
		
		final int runCount = runList.getRunCount();
		final int currentRunIndex = runList.getCurrentRunIndex();
		
		Util.safeSyncExec(new Runnable(){public void run()
		{
			configureTabs(runCount,currentRunIndex);
		}});
	}
	
	private void updateStrategyResults(boolean useImplicitStrategy, int destRunIndex) {
		if (!useImplicitStrategy) {
			return;
		}
			
		if (Epics.getString(PV_ROBOT.mounted).equals("none")) {
			Log.note("nothing mounted, skipping strategy result checking.");
			return;
		}

		String fileRoot0 = Epics.getString(PV_SCR_RUN.fileroot.run(0));
		String dirSuffix0 = Epics.getString(PV_RUN.directory256.run(0));
		String fileRootCurr = Epics.getString(PV_RUN.fileroot.run(destRunIndex));

		if (fileRoot0.equals(fileRootCurr)) {
			Log.note("same prefix, skipping strategy result checking.");
			return;
		}

		Log.note("Loading strategy results.");
		ModelCollectRun r = (ModelCollectRun) ModelCollectRunList.get().getRun(destRunIndex);
		StrategyResults sr = new StrategyResults(fileRoot0);
		
		if (sr.getOscStart() != null) {
			r.dev().setFileroot(fileRoot0, false);
			r.dev().setDirectorySuffix(dirSuffix0, false);
			r.dev().setStartAngle(sr.getOscStart(), false);
			r.dev().setDelta(sr.getOscDelta(), false);
			r.dev().setDistance(sr.getDetectorDistance(), false);
			r.dev().setStartFrameLabel(1, false);
			r.fid.changeEndAngle(sr.getOscEnd()); // send update
		}
	}
	
	protected void handleUserClick()
	{
		if (!areTabsInitialized) return;
		
		final int newTabIndex = runTabFolder.getSelectionIndex();
		int currentTabs = runTabFolder.getItemCount();
		
		// is it the star tab?
		if ( (currentTabs-1) == newTabIndex && runTabFolder.getItem(newTabIndex).getText().equals(" * "))
		{
			if (newTabIndex == 16)
			{
				Util.popupOk("Tab limit", "You have reached the limit of 16 run tabs.");
				return;
			}
			
			remoteOpInProgress = true; notifyListeners();
			
			// send the device events
			//System.out.println("Star tab clicked, setting run count="+(newTabIndex)+" currentRun="+newTabIndex);
			runListDev.getRunDev(newTabIndex-1).copyRunContents(newTabIndex,RUN_LABEL_SETTING.INCREMENT_SRC,true,false);
			runListDev.setRunCount(newTabIndex,false);
			runListDev.setCurrentRun(newTabIndex,true);
			
			// QX: experimental, use implicit strategy result if available when creating a new run, TODO testing
			updateStrategyResults(false, newTabIndex);
		}
		else
		{
			// no local change needed, it's done automatically
			//System.out.println("Current run = "+newTabIndex);

			// when a new tab is being added, the star tab is deleted and the
			// last existing tab becomes the selected tab, causing a selection event
			//
			// this event is ignored here
			if (!isTabAdditionInProgress)
			{
				runListDev.setCurrentRun(newTabIndex,true);
				runList.getRun(newTabIndex).notifyExistingRunClick();
			}
		}
	}
	
	// user clicked delete on a run tab
	public void deleteTab(int tabIndex, SFType sfType, boolean update)
	{
		remoteOpInProgress = true; notifyListeners();

		// the last tab is actually deleted, but to make it look like the
		// chosen tab was deleted, all tabs after that one have their contents copied "up"
		for (int srcTab = tabIndex+1; srcTab <= runList.getRunCount(); srcTab++)
		{
			runListDev.getRunDev(srcTab).copyRunContents(srcTab-1,RUN_LABEL_SETTING.SAME,false,true);
		}

		// change current run and run count
		int newRunCount = runList.getRunCount()-1;
		if (newRunCount<0)
		{
			Log.error("Attempt to delete the last run cancelled.");
			newRunCount=0;
		}
		if (runList.getCurrentRunIndex() > newRunCount)
		{
			runListDev.setCurrentRun(newRunCount, false);
		}
		runListDev.setRunCount(newRunCount, false);
		if (update) runListDev.sendUpdate();
	}
	
	public void configureTabs(int runCount, int currentRunIndex)
	{
		String listDesc = "";
		if (runList instanceof ModelCollectRunList)
		{
			listDesc = "collect";
		}
		else {
			ModelRasterRunList rl = (ModelRasterRunList)runList;
			listDesc="raster-"+rl.getListType().toString();
		}
		System.out.println("### configureTabs count="+runCount+" currentIndex="+currentRunIndex+" type="+listDesc);
		
		if (runCount<0) return;
		
		isTabAdditionInProgress = true;
		
		// don't count the star tab
		int currentTabCount = (runTabFolder.getItemCount()-1) - (areTabsInitialized?1:0);
		
		if (currentTabCount != runCount)
		{
			if (areTabsInitialized) removeStarTab();

			// add or remove tabs to equal runCount
			if (currentTabCount > runCount)
			{
				for (int i=currentTabCount; i>runCount; i--)
				{
					removeTab(i);
				}
			}
			else if (currentTabCount < runCount)
			{
				for (int i=currentTabCount+1;i<=runCount;i++)
				{
					addTab(i);
				}
			}
			
			addStarTab();
			areTabsInitialized = true;
		}
		
		//System.out.println("Setting tab selection: "+(currentRun+0));
		runTabFolder.setSelection(currentRunIndex);

		remoteOpInProgress = false; notifyListeners();
		
		isTabAdditionInProgress = false;
	}
	
	protected void removeStarTab()
	{
		int starTabNumber = runTabFolder.getItemCount()-1;
		GTabItem ti = runTabFolder.getItem(starTabNumber);
		ti.dispose();
	}
	
	protected void addStarTab()
	{
		GTabItem starItem = new GTabItem(runTabFolder, SWT.NONE);
		starItem.setText(" * ");
	}
	
	protected void addTab(int runIndex)
	{
		System.out.println("Creating tab "+runIndex);
		
		GTabItem item = new GTabItem(runTabFolder, SWT.NONE);
		item.setText("...");
		Composite itemComp = runTabFactory.createRunTab(runTabFolder, SWT.NONE, runIndex, runList.getRun(runIndex), runListDev.getRunDev(runIndex),this,item);
		item.setControl(itemComp);
	}
	
	protected void removeTab(int i)
	{
		System.out.println("Deleting tab "+i);

		GTabItem ti = runTabFolder.getItem(i);
		ti.getControl().dispose();
		ti.dispose();
	}
	
	@Override
	public void setEnabled(boolean enable)
	{
		runTabFolder.setEnabled(enable);
	}
}
