/**
 * @author spothineni
 * 
 * This class reads the image header from image server incase of not pilatus
 * For Pilatus beam center is read from PV
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.diffimage.MarImageHeader;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;

import java.util.StringTokenizer;

public class CmdGetBeamCenter {
	
	private MarImageHeader currentHeader;
	
	public MarImageHeader initHeader (String frameName) {
		
		currentHeader = new MarImageHeader(frameName);
			
		if(DetectorConfig.isDectris()) {
			/* in case of Pilatus beam center is directly read from PV */
			String beam_xy = Epics.getString(PV_BLUICE.beam_xy);
			boolean hasComma = beam_xy.contains(",");
			String delimiters = hasComma ? ", " : " ";
			StringTokenizer str = new StringTokenizer(beam_xy, delimiters);

			currentHeader.displayOriginX = Double.parseDouble(str.nextToken())* 0.172;
			currentHeader.displayOriginY = Double.parseDouble(str.nextToken())* 0.172;
			currentHeader.detector ="PILATUS3";
			currentHeader.overload = 1930680;
			currentHeader.size1 = (int)(DetectorConfig.getDimensionsMM().x/0.172);
			currentHeader.size2 = (int)((DetectorConfig.getDimensionsMM().y+0.5)/0.172);
			currentHeader.jpegPixelSize = 0.172;
			
		} else {
				/* read image header from image server */
				currentHeader.getHeader();
				if(currentHeader.displayOriginX == 0.0 || currentHeader.displayOriginX == 0.0) {
					currentHeader.displayOriginX = 150.0;
					currentHeader.displayOriginY = 150.0;
				}
			}
		
		currentHeader.displayOriginX = Math.floor(currentHeader.displayOriginX*100)/100.0;
		currentHeader.displayOriginY = Math.floor(currentHeader.displayOriginY*100)/100.0;
		
		return currentHeader;
	}
}
