/**
 * @author spothineni
 * 
 * This class similar to CmdDataProc, but runs when user clicked reprocess button 
 * on Reprocessing sub-tab
 * CmdDataProc, CmdReprocessing can be combined instead of duplicating, for simplicity
 * they kept separate for now
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.common.Command;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.dataproc.ModelDataProc.DataProcField;
import gov.anl.gmca.diffimage.MarImageHeader;
import gov.anl.gmca.gridengine.CheckSubmitHost;
import gov.anl.gmca.gridengine.DRMAA;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.Disk;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.multistrategy.DevMultiStrategy;
import gov.anl.gmca.syslog.Log;
import org.ggf.drmaa.DrmaaException;

import java.io.*;
import java.util.ArrayList;
import java.util.StringTokenizer;

public class CmdReprocessing extends Command {

	protected static boolean THREAD_RUNNING = false;
	protected int sampNumber;
	protected boolean update = true;
	protected boolean software = true;
	protected String firstImg = "1", lastImg = "2";
	protected int noofenergies = 1;
	protected ExecTask task;
	protected String unitCell = "";
	protected int spaceGroup = 1;
	protected DRMAA drmaa;
	protected int invOn = 0;

	protected static MarImageHeader currentHeader = null;
	protected DevDataProc devDataProc = new DevDataProc();
	private ModelDataProcParams dataprocResult;
	public static String tempLogDir = ".JBluIce";
	
	@Override
	public void run() {

		THREAD_RUNNING = true;
		/* Always increment new run for Analysis tab for re-processing */
		sampNumber = DevDataProc.getRowCount() + 1;

		dataprocResult = new ModelDataProcParams();
		
		ModelAimlessSummary modelAimlessSummary = new ModelAimlessSummary();

		if (ModelDataProcList.get().getCurrentResult() == null) {
			Log.warn("No data processing result available for reprocessing");
			return;
		}
		
		/* decide on work directory */
		String processingDir = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.IMAGEDIR)+"/"+ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.SAMPLE_NAME);
		dataprocResult.inverseOn = (ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.INVERSEON).equals("1"))?true:false;
		
		String collectTypetoProcessing = ModelDataProcList.get().getCurrentResult().getField(DataProcField.COLLECTTYPE);
		String overlap_proc = ModelDataProcList.get().getCurrentResult().getField(DataProcField.OVERLAP);
		
		dataprocResult.software = this.software;
		
		if(dataprocResult.inverseOn) {
			invOn = 1;
			dataprocResult.software = false;
		}
		else {
			invOn = 0;
		}
		if(Integer.parseInt(overlap_proc) != 0 && collectTypetoProcessing.equals("Raster")) {
			dataprocResult.software = false;
		}
		
		/* processing pipeline */
		dataprocResult.workDir = processingDir + "_" + (dataprocResult.software?"fast_dp":"GMCAproc") + "_Reprocess" + sampNumber;
		
		/* find which software used fast_dp/XDS */
		int isFastDP = 0;

		if (dataprocResult.software) {
			dataprocResult.log = dataprocResult.workDir + "/"+ CmdDataProc.tempLogDir
					 + "/fast_dp.log";

			isFastDP = 0;
		} else {
			dataprocResult.log = dataprocResult.workDir + "/"+ CmdDataProc.tempLogDir
								+ "/GMCAproc.LOG";
			dataprocResult.software = false;
			isFastDP = 1;
		}
		
		/* read from cache of runs of Analsysis tab, current selected run is read */
		dataprocResult.imageDir = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.IMAGEDIR);
		dataprocResult.sampleName = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.SAMPLE_NAME);
		dataprocResult.firstImage = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.FIRSTFRAME);
		dataprocResult.images = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.IMAGES);
		dataprocResult.collectType = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.COLLECTTYPE);
		
		dataprocResult.images = dataprocResult.images.substring(0,
				dataprocResult.images.lastIndexOf("#")+1);
		
		dataprocResult.images += " "+firstImg + " " + lastImg;
		String imageDir = dataprocResult.imageDir;
		dataprocResult.state = "PROCESSING";
		dataprocResult.imageSet = firstImg + "-" + lastImg;
		
		/* format unit cell with commas for fast_dp */
		String fdUnitCell = "";
		if(!unitCell.equals("")) {
			StringTokenizer tokens = new StringTokenizer(unitCell);
			fdUnitCell+=tokens.nextToken();
			if(tokens.countTokens()+1 == ModelReprocessing.UnitCell.length) {
				while(tokens.hasMoreElements())
					fdUnitCell+=","+tokens.nextToken();
			}
		}
		String fdSpaceGroup = "";
		if(!fdUnitCell.equals(""))
			fdSpaceGroup = DevMultiStrategy.get().getSpacegroupFromNumber(spaceGroup);
		
		if(isFastDP == 1)
			fdSpaceGroup = Integer.toString(spaceGroup);
		
		devDataProc.setResult(sampNumber, dataprocResult, this.update);
		
		currentHeader = (new CmdGetBeamCenter()).initHeader((imageDir + "/" + dataprocResult.firstImage)
				.replace("//", "/"));

		dataprocResult.scalingsoftware = "AIMLESS";
		
		/* Check the working directory exists or not 
		 */
		File dirFile = new File(dataprocResult.workDir);
		if (!dirFile.exists()) {

			if (!dirFile.mkdirs()) {
				THREAD_RUNNING = false;
				Log.error("Couldn't create directory " + dataprocResult.workDir);
				return;
			}
		}
		Log.note("XDS data reduction in progress, check the dir. "
				+ dataprocResult.workDir);
		
		/* write user changed XDS parameters (from Settings tab) into a separate file *
		 * .xdsparams
		 * run_XDS.csh script will read these parameters and writes inot XDS.INP
		 */
		if(ModelReprocessing.takeSettings && ModelXDSSettings.changedValues.size() > 0) {
			try {
				BufferedWriter runxdsOut = new BufferedWriter(new FileWriter(
					dataprocResult.workDir + "/" + ".xdsparams", false));
				
				runxdsOut.write("\n!XDS parameters from Settings tab of JBluIce\n");
				for(ModelXDSSettings.DefaultParam param : ModelXDSSettings.changedValues.keySet()) {
					String val = ModelXDSSettings.changedValues.get(param);
					if(val == null || val.equals(param.getValue()))
						continue;
					if(param.getXDSParam().equals("FRIEDELS_LAW")) {
						runxdsOut.write("FRIEDEL'S_LAW"+"="+ModelXDSSettings.changedValues.get(param)+"\n");
						continue;
					}
					if(!val.equals(""))
						runxdsOut.write(param.getXDSParam()+"="+ModelXDSSettings.changedValues.get(param)+"\n");
				}
				runxdsOut.close();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}
		
		String data_dir_prefix = "";
		/* decide is it is NFS and replace it to corresponding directory */
		String ppu_prefix = DBConfig.get("ppu_ramdisk_prefix");
		data_dir_prefix = imageDir.replace("/mnt", DBConfig
				.get("processing_data_dir").toString().equals("NFS")?(ppu_prefix+"/mnt"):"/mnt");
		
		/* test/fake data */
		if (Main.FAKE_DATA) {
			
			dataprocResult.sampleName = "xdstest1_1";
			dataprocResult.firstImage = "xdstest1_1_00001.cbf";
			
			data_dir_prefix = "/mnt/sahre1/user0/testdataxds";
			
			dataprocResult.firstImage = DBConfig
					.get("processing_test_firstframe");
			
			data_dir_prefix = DBConfig
					.get("processing_test_dir");
			
			if(dataprocResult.firstImage.contains("cbf")) {
				dataprocResult.sampleName = dataprocResult.firstImage.substring(0, dataprocResult.firstImage.lastIndexOf('_'));
			} else {
				dataprocResult.sampleName = dataprocResult.firstImage.substring(0, dataprocResult.firstImage.lastIndexOf('.'));
			}
			/*update beamX, beamY */
			currentHeader = (new CmdGetBeamCenter()).initHeader(data_dir_prefix+"/"+dataprocResult.firstImage);
			
			/* fake data file*/
			File fakedataFile = new File(imageDir+"/"+"testdataset");
			
			if(fakedataFile.exists()) {
				ArrayList<String> lines = Disk.readText(imageDir+"/"+"testdataset");
				try {
					if(lines.size() >= 3) {
						dataprocResult.sampleName = lines.get(0);
						dataprocResult.firstImage = lines.get(1);
						data_dir_prefix = lines.get(2);
				
						/* udpate beamX, beamY */
						currentHeader =(new CmdGetBeamCenter()).initHeader(data_dir_prefix+"/"+dataprocResult.firstImage);
					} else {
						Log.error("Test dataset file doesn't have enough data, taking defaults");
					}
				
				} catch (Exception e) {
					System.out.println(e.getMessage());
				}
			}
		}
		
		/* write script file for data processing */
		try {
			/* process XDS */
			BufferedWriter runxdsOut = new BufferedWriter(new FileWriter(
					dataprocResult.workDir + "/" + "run_XDS.sh", false));

			String command = Main.getAppDir()
					+ "/tools/scripts/dataproc/run_XDS.csh";

			String isScala = "aimless";

			/* treat vector with overlaps as normal collection */
			if (collectTypetoProcessing.equals("Vector")) {
				if (overlap_proc.equals("0"))
					dataprocResult.collectType = "Standard";
				collectTypetoProcessing = "Standard";
			}
			
			/* TODO write XML file instead, too many parameters at commandline */ 
			runxdsOut.write("# Running fast_dp/XDS in parallel, works only on 23ID beamlines envitonment @APS\n");
			runxdsOut.write("# Usage : run_XDS.csh <file_prefix> <working_dir> " +
							"<number of energies> <first_image> <fast_dp> <scala/aimless> " +
							"<beamX> <beamY>" +
							"<firstImage> <lastImage> <isInverseBeam> <imageSet> <collectType>\n");
			
			String commandParams = "";
			if(!fdUnitCell.equals("")) {
				commandParams +=" \""+fdUnitCell+"\" "+fdSpaceGroup;
				BufferedWriter runxdsOut1 = new BufferedWriter(new FileWriter(
						dataprocResult.workDir + "/" + ".xdsparams", false));
				
				runxdsOut1.write("\n!XDS parameters from reprocessing tab of JBluIce\n");
				runxdsOut1.write("SPACE_GROUP_NUMBER= "+spaceGroup+"\n");
				runxdsOut1.write("UNIT_CELL_CONSTANTS= "+fdUnitCell.replace(',',' ')+"\n");
				runxdsOut1.write("INCLUDE_RESOLUTION_RANGE= "+ModelReprocessing.RESOLUTION_LOW 
														+" "+ModelReprocessing.RESOLUTION_HIGH+"\n");
				runxdsOut1.close();
			}
			runxdsOut.write(command +" "+dataprocResult.sampleName+" "+dataprocResult.workDir +" "+
						noofenergies+" "+dataprocResult.firstImage+" "+dataprocResult.firstImage+" "+isFastDP+" "+isScala+" "+
							currentHeader.displayOriginX+" "+currentHeader.displayOriginY+" "+
							firstImg+" "+lastImg+" "+invOn+" "+0+" "+
							dataprocResult.imageSet+" "+collectTypetoProcessing+ " " + data_dir_prefix+" "+commandParams);



			runxdsOut.close();
			Runtime.getRuntime().exec(
					"chmod u+x " + dataprocResult.workDir + "/" + "run_XDS.sh");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		/*check for gridengine environment and quit if not a submit host */
		boolean isSubmitHost = CheckSubmitHost.get().isSubmitHost();
		
		System.out.println("Gridengine env detected: "+isSubmitHost);
		if(!isSubmitHost) {
			quitWithException(new Exception());
			return;
		}
		
		/* submit the processing job to Grid Engine using DRMAA 
		 * waits 900 seconds before it deletes the job
		 * 
		 */
		//if (true) { // force local
		if (!isSubmitHost) {
			task = ExecTask.new_ExecTask(dataprocResult.workDir + "/" + "run_XDS.sh", "dataReProcessing", false, false);
			task.start();
			task.waitFor();
		} else {
			try {
				drmaa = new DRMAA(dataprocResult.workDir + "/" + "run_XDS.sh",
						dataprocResult.workDir, WdgDataProc.TIMEWAIT_FOR_PROCESSINGJOB); 
				drmaa.start();
				int jobStatus = drmaa.waitFor();
	
			} catch (NoClassDefFoundError  e) {
				quitWithException(new Exception());
				return;
			} catch (UnsatisfiedLinkError e) {
				quitWithException(new Exception());
				return;
			} catch (DrmaaException e) {
				quitWithException(e);
				return;
			} catch (Exception e) {
				quitWithException(e);
				return;
			} 
		}
		/* read the sub-directory where the aimless log files are stored */
		BufferedReader aimlessbr;
		String aimlesslogFile = dataprocResult.workDir  +"/"+tempLogDir + "/subdirfile";
		try {
			aimlessbr = new BufferedReader(new FileReader(aimlesslogFile));
			dataprocResult.subdir = aimlessbr.readLine();
			
		} catch (FileNotFoundException e) {
				System.out.println("logfile file not found " + e.getMessage());
			
		} catch (IOException e) {
				System.out.println("logfile file not found " + e.getMessage());
		}
		
		/* save the log files into parameters which are writeen to MySQL */
		dataprocResult.scalaLog = dataprocResult.subdir+ "/"+dataprocResult.sampleName + "_aimless.log";
		
		if(dataprocResult.workDir.contains("fast_dp")) {
			dataprocResult.log = dataprocResult.subdir+"/fast_dp.log";
		} else {
			dataprocResult.log = dataprocResult.subdir+"/GMCAproc.LOG";
		}
		
		/* read the <prefix>_aimless log for summary of data quality params */
		try {
			modelAimlessSummary.processResult(dataprocResult.workDir,
				dataprocResult);
		} catch (Exception e) {
			e.printStackTrace();
		}

		dataprocResult.truncateLog = dataprocResult.subdir+"/"
		+ dataprocResult.sampleName + "_ctruncate.log";

		
		if (dataprocResult.warning.equals("processing"))
			dataprocResult.warning = "DONE";
		if (dataprocResult.state.equals("PROCESSING"))
			dataprocResult.state = "DONE";
		
		devDataProc.setDataProcResult(sampNumber, dataprocResult,
				dataprocResult.imageSet, true);
		
		/* save the new result to Excel spreadsheet */
		String savePath = DevDataDir.getPrimary();
		devDataProc.saveAllResults(savePath + "/dataProcessingResults.xls");

		THREAD_RUNNING = false;
		Log.note("Data processing finished");
	}
	
	private void quitWithException(Exception e) {
		THREAD_RUNNING = false;
		dataprocResult.warning = "GridEngine/DRMAA exception for submitting processing job\n report this to your host";
		dataprocResult.state = "ERROR";
		e.printStackTrace();
		devDataProc.setDataProcResult(sampNumber, dataprocResult,
				dataprocResult.imageSet, true);
		return;
	}
}
