/**
 *  @author spothineni
 *  
 *  this class runs MySQL queries on DataProcResults table form user_data database
 *  somehow dataprocessing thread locks are ended up this this class because it just created
 *  only once, refer to lock and lock1 objects which are used in InitDataprocessing
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.dataproc.ModelDataProc.DataProcField;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_ANALYSIS;
import gov.anl.gmca.log.db.CollectEvents;
import gov.anl.gmca.log.db.DataProcEvents;
import gov.anl.gmca.log.db.DataProcEvents.DataProcParam;
import gov.anl.gmca.syslog.Log;
import gov.aps.jca.dbr.DBR;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

public class DevDataProc {
	protected static final String tableName = "DataProcResults";
	protected static final boolean UPDATE_CURRENT_INDEX_ON_RESULT_CREATION = true;
	protected static final boolean SHOW_DATAPROC_TAB_ON_RESULT_CREATION = true;
	
	public static Object lock = new Object(); // lock for processing thread
	public static Object lock1 = new Object(); // lock for writing log thread
	
	protected static boolean doesTableExist() {
		Database.Result result = Database.exec(DBName.USERDATA,
				"show tables like '" + tableName + "'");
		boolean exists = (result.numRows() > 0);
		return exists;
	}

	public synchronized static boolean verifyTableExists() {
		if (!doesTableExist()) {
			System.out.println("Creating DataProc results table");
			String createCmd = "CREATE TABLE `" + tableName + "` ("
					+ "`sampleNumber` int(11) NOT NULL,"
					+ "`sampleName` varchar(30) default '',"
					+ "`imageSet` varchar(256) default '1-5',"
					+ "`state` varchar(30) default '',"
					+ "`software` varchar(30) default '',"
					+ "`selected` boolean default false,"
					+ "`scalingsoftware` varchar(30) default '',"
					+ "`collectType` varchar(250) default '',"
					+ "`overlap` int(11) default 0,"
					+ "`inverseOn` boolean default false,"
					+ "`firstFrame` varchar(250) default '',"
					+ "`workdir` varchar(250) default '',"
					+ "`subdir` varchar(250) default '',"
					+ "`imagedir` varchar(250) default '',"
					+ "`images` varchar(512) default '',"
					+ "`unitcell` varchar(2048) default '',"
					+ "`lowresolution` varchar(30) default '',"
					+ "`lowresolution_inner` varchar(30) default '',"
					+ "`lowresolution_outer` varchar(30) default '',"
					+ "`highresolution` varchar(30) default '',"
					+ "`highresolution_inner` varchar(30) default '',"
					+ "`highresolution_outer` varchar(30) default '',"
					+ "`rmerge` varchar(30) default '',"
					+ "`rmerge_inner` varchar(30) default '',"
					+ "`rmerge_outer` varchar(30) default '',"
					+ "`rpim` varchar(30) default '',"
					+ "`rpim_inner` varchar(30) default '',"
					+ "`rpim_outer` varchar(30) default '',"
					+ "`cchalf` varchar(30) default '',"
					+ "`cchalf_inner` varchar(30) default '',"
					+ "`cchalf_outer` varchar(30) default '',"
					+ "`completeness` varchar(30) default '',"
					+ "`completeness_inner` varchar(30) default '',"
					+ "`completeness_outer` varchar(30) default '',"
					+ "`anom_completeness` varchar(30) default '',"
					+ "`anom_completeness_inner` varchar(30) default '',"
					+ "`anom_completeness_outer` varchar(30) default '',"
					+ "`multiplicity` varchar(30) default '',"
					+ "`multiplicity_inner` varchar(30) default '',"
					+ "`multiplicity_outer` varchar(30) default '',"
					+ "`anom_multiplicity` varchar(30) default '',"
					+ "`anom_multiplicity_inner` varchar(30) default '',"
					+ "`anom_multiplicity_outer` varchar(30) default '',"
					+ "`isigma` varchar(30) default '',"
					+ "`isigma_inner` varchar(30) default '',"
					+ "`isigma_outer` varchar(30) default '',"
					+ "`spacegroup` varchar(2048) default '',"
					+ "`log` varchar(512) default '',"
					+ "`scalalog` varchar(512) default '',"
					+ "`truncatelog` varchar(512) default '',"
					+ " `warning` varchar(5120) default '',"
					+ " `timestamp` varchar(512) default '');";

			// createCmd +=
			// " PRIMARY KEY (`sampleNumber`)) DEFAULT CHARSET=utf8";
			Log.note(createCmd);
			Database.exec(DBName.USERDATA, createCmd);
		}

		return doesTableExist();
	}

	protected boolean connected = false;
	protected Epics.EpicsValueMonitor dbUpdatedMonitor;

	public static DevDataProc get() {
		return inst;
	}

	protected static DevDataProc inst = null;

	public DevDataProc() {
		inst = this;
	}

	public void verifyConnected() {
		if (connected)
			return;

		updateAllRows();

		Epics.addMonitor(PV_ANALYSIS.currentDataProcIndex,
				new Epics.EpicsValueMonitor() {
					@Override
					public void valueChanged(String pvName, DBR value) {
						ModelDataProcList.get().setCurrentDataProcIndex(
								Epics.dbrToInt(value));
						ModelDataProcList.get().ll.notifyListeners();
					}
				});

		dbUpdatedMonitor = new Epics.EpicsValueMonitor() {
			@Override
			public void valueChanged(String pvName, DBR value) {
				int intVal = Epics.dbrToInt(value);
				if (-1 != intVal)
					handlePVUpdate(intVal);
			}
		};
		Epics.addMonitor(PV_ANALYSIS.dataprocUpdated, dbUpdatedMonitor);

		connected = true;
	}

	public void setResult(int sampleNumber, ModelDataProcParams dataprocResult,
			boolean update) {
		verifyTableExists();
		boolean newResultCreated = false;
		String software = dataprocResult.software?"fast_dp":"GMCAproc";
		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber+" and software='"+software+"'");
		// if (0==result.numRows())
		{
			Database.exec(
					DBName.USERDATA,
					"insert into "
							+ tableName
							+ " (sampleNumber, sampleName, state, selected, software, workdir, imageDir, images, imageSet, log, warning) "
							+ "values (" + sampleNumber + ", '"
							+ dataprocResult.sampleName + "', '"
							+ dataprocResult.state + "', "
							+ update+ ", '"
							+ software + "', '"
							+ dataprocResult.workDir + "', '"
							+ dataprocResult.imageDir + "', '"
							+ dataprocResult.images + "', '"
							+ dataprocResult.imageSet + "', '"
							+ dataprocResult.log + "', '"
							+ dataprocResult.warning + "')");
			newResultCreated = true;
		}
		/*
		 * else { Database.exec( DBName.USERDATA, "update "+tableName+" "+
		 * "set state='"+dataprocResult.state+"' "+
		 * ", sampleName='"+dataprocResult.sampleName+"' "+
		 * ", workdir='"+dataprocResult.workDir+"' "+
		 * ", software='"+dataprocResult.software+"' "+
		 * ", scalingsoftware='"+dataprocResult.scalingsoftware+"' "+
		 * ", images='"+dataprocResult.images+"' "+
		 * ", warning='"+dataprocResult.warning+"' "+
		 * "where sampleNumber="+sampleNumber ); }
		 */
		if (update) {

			// update the GUI directly
			updateRow(sampleNumber, true);
			if (newResultCreated && UPDATE_CURRENT_INDEX_ON_RESULT_CREATION) {
				int last = ModelDataProcList.get().getDataProcList().size() - 1;
				DevDataProc.setCurrentDataProcIndex(last);
			}
			// send the PV update signal
			sendRowUpdate(sampleNumber);
		}
	}
	protected void setSoftware(int sampleNumber, ModelDataProcParams dataprocResult, String imageSet, boolean update) {
		verifyTableExists();

		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber+" and imageSet='"+ imageSet + "'");
		
		if(result.numRows() < 1) {
			System.out.print("ERROR: No results found to set the software: DevDataProc:setSoftware");
			return;
		}
		
		ModelDataProc dataProcResult;
		dataProcResult = sqlRowToResult(result.getRow(0));
		for(HashMap<String, String> row:result.getRows()) {
			dataProcResult.pipelines.add(row.get("software"));
		}
		
		ModelDataProcList.get().setRow(dataProcResult, sampleNumber, true);
		ModelDataProcList.get().ll.notifyListeners(sampleNumber);
	}
	private void setSelectedFalse (int sampleNumber) {
		verifyTableExists();
		boolean newResultCreated = false;
		
		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber);
		if (result.numRows() != 0)
		{
			Database.exec( DBName.USERDATA, "update "+tableName+" "+
					  "set selected= false"+
					  " where sampleNumber="+sampleNumber); 
			newResultCreated = true;
		}
	}
	public void updateWarning (int sampleNumber, ModelDataProcParams dataprocResult, String imageSet, boolean update) {
		verifyTableExists();
		boolean newResultCreated = false;
		String software = dataprocResult.software?"fast_dp":"GMCAproc";
		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber+" and imageSet='"+ imageSet + "'"+ " and software='"+software+"'");
		if (result.numRows() != 0)
		{
			Database.exec( DBName.USERDATA, "update "+tableName+" "+
					  "set warning= IFNULL(CONCAT(warning, '"+dataprocResult.warning+"'), '') "+
					  " where sampleNumber="+sampleNumber+" and imageSet='"+ imageSet + "'"); 
			newResultCreated = true;
		}
		
		if (update && newResultCreated) {

			// update the GUI directly
			updateRow(sampleNumber, true, imageSet, software);
			if (newResultCreated && UPDATE_CURRENT_INDEX_ON_RESULT_CREATION) {
				int last = ModelDataProcList.get().getDataProcList().size() - 1;
				DevDataProc.setCurrentDataProcIndex(last);
			}
		}
	}
	protected void updateRowSoftware(int row, boolean update, String software, String imageSet) {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + row +" and imageSet='"+ imageSet+"' and software='" + software + "'");
		if (result.numRows() < 1)
			return;
		
		setSelectedFalse(row);
		Database.exec( DBName.USERDATA, "update "+tableName+" "+
				  "set selected=true"+
				  " where sampleNumber="+row+" and imageSet='"+ imageSet+"' and software='" + software + "'"); 
		
		ModelDataProc dataProcResult;
		dataProcResult = sqlRowToResult(result.getRow(result.numRows() - 1));
		dataProcResult.imageSets = getImageSets(result.getRow(result.numRows() - 1));
		dataProcResult.pipelines = getPipelines(result.getRow(result.numRows() - 1));
		ModelDataProcList.get().setRow(dataProcResult, row, true);
		ModelDataProcList.get().ll.notifyListeners(row);
	}
	public void setDataProcResult(int sampleNumber,
			ModelDataProcParams dataprocResult, String imageSet, boolean update) {
		verifyTableExists();

		boolean newResultCreated = false;
		String software = dataprocResult.software?"fast_dp":"GMCAproc";
		
		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber + " and imageSet='" + imageSet + "'"+ " and software='" + software + "'");
		if (0 == result.numRows()) {
			return;
			/*Database.exec(DBName.USERDATA, "insert into `" + tableName
					+ "` (sampleNumber,state) " + "values (" + sampleNumber
					+ ",'DONE')");
			newResultCreated = true;*/
		}
		
		String updateCmd = "update `" + tableName + "` set ";
		String DATE_FORMAT_NOW = "yyyy-MM-dd-HH:mm:ss";
		Date date = new Date();
		SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
		String timestamp = sdf.format(date);
		
		updateCmd += "state='" + dataprocResult.state;
		updateCmd += "', images='" + dataprocResult.images;
		updateCmd += "', software='" + software;
		updateCmd += "', imageSet='" + dataprocResult.imageSet;
		updateCmd += "', scalingsoftware='" + dataprocResult.scalingsoftware;
		updateCmd += "', collectType='" + dataprocResult.collectType;
		updateCmd += "', inverseOn='" + (dataprocResult.inverseOn?"1":"0");
		updateCmd += "', overlap=" + dataprocResult.overlap;
		updateCmd += ", firstFrame='" + dataprocResult.firstImage;
		updateCmd += "', warning=\"" + dataprocResult.warning;
		updateCmd += "\", lowresolution='" + dataprocResult.lowresolution;
		updateCmd += "', lowresolution_inner='"
				+ dataprocResult.lowresolution_inner;
		updateCmd += "', lowresolution_outer='"
				+ dataprocResult.lowresolution_outer;
		updateCmd += "', highresolution='" + dataprocResult.highresolution;
		updateCmd += "', highresolution_inner='"
				+ dataprocResult.highresolution_inner;
		updateCmd += "', highresolution_outer='"
				+ dataprocResult.highresolution_outer;
		updateCmd += "', unitcell='" + dataprocResult.unitCell;
		updateCmd += "', spacegroup='" + dataprocResult.spaceGroup;
		updateCmd += "', rmerge='" + dataprocResult.rMerge;
		updateCmd += "', rpim='" + dataprocResult.rPim;
		updateCmd += "', cchalf='" + dataprocResult.cchalf;
		updateCmd += "', completeness='" + dataprocResult.completeness;
		updateCmd += "', anom_completeness='"
				+ dataprocResult.anom_completeness;
		updateCmd += "', multiplicity='" + dataprocResult.multiplicity;
		updateCmd += "', anom_multiplicity='"
				+ dataprocResult.anom_multiplicity;
		updateCmd += "', isigma='" + dataprocResult.iSigma;
		updateCmd += "', rmerge_inner='" + dataprocResult.rMerge_inner;
		updateCmd += "', rpim_inner='" + dataprocResult.rPim_inner;
		updateCmd += "', cchalf_inner='" + dataprocResult.cchalf_inner;
		updateCmd += "', completeness_inner='"
				+ dataprocResult.completeness_inner;
		updateCmd += "', anom_completeness_inner='"
				+ dataprocResult.anom_completeness_inner;
		updateCmd += "', multiplicity_inner='"
				+ dataprocResult.multiplicity_inner;
		updateCmd += "', anom_multiplicity_inner='"
				+ dataprocResult.anom_multiplicity_inner;
		updateCmd += "', isigma_inner='" + dataprocResult.iSigma_inner;
		updateCmd += "', rmerge_outer='" + dataprocResult.rMerge_outer;
		updateCmd += "', rpim_outer='" + dataprocResult.rPim_outer;
		updateCmd += "', cchalf_outer='" + dataprocResult.cchalf_outer;
		updateCmd += "', completeness_outer='"
				+ dataprocResult.completeness_outer;
		updateCmd += "', anom_completeness_outer='"
				+ dataprocResult.anom_completeness_outer;
		updateCmd += "', multiplicity_outer='"
				+ dataprocResult.multiplicity_outer;
		updateCmd += "', anom_multiplicity_outer='"
				+ dataprocResult.anom_multiplicity_outer;
		updateCmd += "', isigma_outer='" + dataprocResult.iSigma_outer;
		updateCmd += "', scalalog='" + dataprocResult.scalaLog;
		updateCmd += "', log='" + dataprocResult.log;
		updateCmd += "', subdir='" + dataprocResult.subdir;
		updateCmd += "', truncatelog='" + dataprocResult.truncateLog;
		updateCmd += "', timestamp='" + timestamp;
		
		if (update) {
			setSelectedFalse(sampleNumber);
			updateCmd += "', selected=true";
		} else {
			updateCmd += "'";
		}
		updateCmd += " where sampleNumber=" + sampleNumber + " and imageSet='"
				+ imageSet + "'"+ " and software='" + software + "'";

		Database.exec(DBName.USERDATA, updateCmd);
		
		if (update) {
			// send the PV update signal
			// updateRow(sampleNumber);
			if (newResultCreated && UPDATE_CURRENT_INDEX_ON_RESULT_CREATION) {
				// int last =
				// ModelDataProcList.get().getDataProcList().size()-1;
				// DevDataProc.setCurrentDataProcIndex(last);
			}
			// send the PV update signal
			sendRowUpdate(sampleNumber);
		}
		/* update GM/CA database */
		if (imageSet.contains("Final_") && !dataprocResult.state.equals("PROCESSING"))
			updateDataProcEvent(sampleNumber, software);
	}
	/* get row for selected imageSet index*/
	protected void updateRow(int row, int index, boolean update) {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + row);

		ModelDataProc dataProcResult;
		dataProcResult = sqlRowToResult(result.getRow(index));
		dataProcResult.imageSets = getImageSets(result.getRow(index));
		dataProcResult.pipelines = getPipelines(result.getRow(index));
		ModelDataProcList.get().setRow(dataProcResult, row, true);
		ModelDataProcList.get().ll.notifyListeners(row);
	}
	/* get row for selected imageSet */
	protected void updateRow(int row, String imageSet, boolean update) {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + row+" and imageSet='"+imageSet+"'");
		
		if(result.numRows() < 1) {
			Log.error("The selected image set doesn't have any result in MySQl table");
			return;
		}
		ModelDataProc dataProcResult;
		dataProcResult = sqlRowToResult(result.getRow(0));
		dataProcResult.imageSets = getImageSets(result.getRow(0));
		dataProcResult.pipelines = getPipelines(result.getRow(0));
		ModelDataProcList.get().setRow(dataProcResult, row, true);
		ModelDataProcList.get().ll.notifyListeners(row);
	}
	protected void updateRow(int row, boolean update) {
		verifyTableExists();
		
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + row+" and selected=true");
		if (result.numRows() < 1)
			return;
		
		ModelDataProc dataProcResult;
		dataProcResult = sqlRowToResult(result.getRow(result.numRows() - 1));
		dataProcResult.imageSets = getImageSets(result.getRow(result.numRows() - 1));
		dataProcResult.pipelines = getPipelines(result.getRow(result.numRows() - 1));
		ModelDataProcList.get().setRow(dataProcResult, row, true);
		ModelDataProcList.get().ll.notifyListeners(row);
	}
	
	protected void updateRow(int row, boolean update, String imageSet, String software) {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + row +
				" and imageSet='" + imageSet + "'"+ " "
						+ "and selected=true");
		if (result.numRows() < 1)
			return;
		ModelDataProc dataProcResult;
		dataProcResult = sqlRowToResult(result.getRow(result.numRows() - 1));
		dataProcResult.imageSets = getImageSets(result.getRow(0));
		dataProcResult.pipelines = getPipelines(result.getRow(0));
		ModelDataProcList.get().setRow(dataProcResult, row, true);
		ModelDataProcList.get().ll.notifyListeners(row);
	}

	protected void updateAllRows() {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " group by sampleNumber");
		
		/*SELECT m1.*
				FROM messages m1 LEFT JOIN messages m2
 				ON (m1.name = m2.name AND m1.id < m2.id)
				WHERE m2.id IS NULL;
		 * 
		 * 
		 * final Database.Result result = Database.exec(DBName.USERDATA,
		 *
				"select * from " + tableName + " tab1 LEFT JOIN "+ tableName + " tab2"+
						"on (tab1.sampleNumber = tab2.sampleNumber)";
		*/
		
		System.out.println("Number of rows in the DataProcResults table "
				+ result.numRows());
		ModelDataProcList.get().clearResults();
		
		if (result == null || result.numRows() < 1) {
			ModelDataProcList.get().ll.notifyListeners(-2);
			return;
		}
		for (HashMap<String, String> row : result.getRows()) {
			ModelDataProc dataProcResult = sqlRowToResult(row);
			dataProcResult.imageSets = getImageSets(row);
			dataProcResult.pipelines = getPipelines(row);
			if (null != dataProcResult)
				ModelDataProcList.get().setRow(dataProcResult,
						Integer.parseInt(row.get("sampleNumber")), false);
		}
		ModelDataProcList.get().setCurrentDataProcIndex(
				result.numRows()-1);
		ModelDataProcList.get().ll.notifyListeners(-2);
	}
	
	private ArrayList<String> getImageSets(HashMap<String, String> row) {
		final Database.Result result3 = Database.exec(
				DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber="
						+ row.get("sampleNumber"));

		int i = 0;
		ArrayList<String> imageSets = new ArrayList<String>();
		for (HashMap<String, String> row2 : result3.getRows()) {
			for (String sqlCol : row2.keySet()) {
				if (sqlCol.equals("imageSet")) {
					imageSets.add(row2.get(sqlCol));
					i++;
				}
			}
		}
		return (imageSets);
	}
	private ArrayList<String> getPipelines(HashMap<String, String> row) {
		final Database.Result result3 = Database.exec(
				DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber="
						+ row.get("sampleNumber")+" and imageSet='" + row.get("imageSet") + "'");

		int i = 0;
		ArrayList<String> pipelines = new ArrayList<String>();
		for (HashMap<String, String> row2 : result3.getRows()) {
			for (String sqlCol : row2.keySet()) {
				if (sqlCol.equals("software")) {
					pipelines.add(row2.get(sqlCol));
					i++;
				}
			}
		}
		return (pipelines);
	}
	
	
	protected ModelDataProc sqlRowToResult(HashMap<String, String> row) {
		HashMap<DataProcField, String> fieldMap = new HashMap<DataProcField, String>();

		for (String sqlCol : row.keySet()) {
			for (DataProcField field : DataProcField.values()) {
				if (field.getDBString().equals(sqlCol)) {
					fieldMap.put(field, row.get(sqlCol));
					// Log.note(sqlCol + " "+ row.get(sqlCol));
				}
			}
		}
		return new ModelDataProc(fieldMap);
	}

	protected void updateDataProcEvent(int num, String pipeline) {
		System.out.println("Writing data processing results into GMCA database");
		HashMap<DataProcParam, String> fieldMapStr = new HashMap<DataProcParam, String>();
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + num+ " and software='" + pipeline + "'");
		if (result.numRows() < 1)
			return;
		HashMap<String, String> row = result.getRow(result.numRows() - 1);

		for (String sqlCol : row.keySet()) {
			for (DataProcParam fieldStr : DataProcParam.values()) {
				if (fieldStr.getDBField().fieldName.equals(sqlCol)) {
					fieldMapStr.put(fieldStr, row.get(sqlCol));
				}
			}
		}
		final Database.Result result1 = Database.exec(DBName.GMCADATA,
				"select * from " + CollectEvents.GMCA_COLLECT_DATA_TABLE);
		fieldMapStr.put(DataProcParam.COLLECT_NUMBER,
				Integer.toString(result1.numRows()));
		DataProcEvents.writeEvent(fieldMapStr);
	}

	public static int getRowCount() {
		int count = 0;
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " group by sampleNumber");
		try {
		if (result != null)
			count = result.numRows();
		} catch (NullPointerException e) {
			e.getMessage();
		}
		return count;
	}

	protected void handlePVUpdate(int pvValue) {
		verifyTableExists();
		if (pvValue >= 0) {
			System.out.println("Updating row " + pvValue);
			updateRow(pvValue, true);
		} else if (-2 == pvValue) {
			System.out.println("Updating all rows " + pvValue);
			updateAllRows();
		}
	}

	protected void sendRowUpdate(int sampleNumber) {
		Epics.putInteger(PV_ANALYSIS.dataprocUpdated, -1);
		Epics.putInteger(PV_ANALYSIS.dataprocUpdated, sampleNumber);
	}

	public static int getRows() {
		int count;
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);
		count = result.numRows();
		return count;
	}

	public static void setCurrentDataProcIndex(int i) {
		//Epics.putInteger(PV_ANALYSIS., i);
		ModelDataProcList.get().setCurrentDataProcIndex(
				i);
		ModelDataProcList.get().ll.notifyListeners();
	}

	public void deleteAllResults() {
		Log.note("Deleting all strategy results");

		verifyTableExists();

		setCurrentDataProcIndex(0);

		Database.exec(DBName.USERDATA, "delete from " + tableName);
		sendRowUpdate(-2);
	}

	public void saveAllResults(String excelFile) {
		// WritableWorkbook workbook = Workbook.createWorkbook(new
		// File(excelFile));
		Workbook wbs;
		ModelDataProcList resultList = ModelDataProcList.get();
		try {
			OutputStream inp = new FileOutputStream(excelFile);

			wbs = new HSSFWorkbook();

			Sheet s = wbs.createSheet();
			// declare a row object reference
			Row r = null;
			// declare a cell object reference
			Cell c = null;
			int i = 1;
			r = s.createRow(0); // write the column names

			final Database.Result result = Database.exec(DBName.USERDATA,
					"select * from " + tableName);
			if (result.numRows() < 1) {
				inp.close();
				return;
			}
			for (DataProcField field : DataProcField.values()) {

				c = r.createCell(field.ordinal());
				c.setCellValue(field.getDBString());
			}
			// write data
			int k = 1;
			int j = 0;
			for (HashMap<String, String> row : result.getRows()) {

				Row r1 = s.createRow(k);
				j = 0;
				for (DataProcField field : DataProcField.values()) {
					c = r1.createCell(j);
					for (String sqlCol : row.keySet()) {
						if (field.getDBString().equals(sqlCol))
							c.setCellValue(row.get(sqlCol));
					}
					j++;
				}
				k++;
			}
			wbs.write(inp);
			inp.close();
		} catch (IOException ex) {
			Log.error("Excel export failed :" + ex.getMessage());
		}
	}

	public String getWorkingDir(int sampNumber) {
		final Database.Result result3 = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber="
						+ sampNumber);
		String workDir = "";
		for (HashMap<String, String> row2 : result3.getRows()) {
			for (String sqlCol : row2.keySet()) {
				if (sqlCol.equals("workdir")) {
					workDir = row2.get(sqlCol);
				}
			}
		}

		return workDir;
	}
}
