/**
 * @author spothineni
 * 
 * InitDataProcessing initiates a thread for data processing 
 * it watches PV PV_FPE.lastfilename 
 * 
 * All the input data read from ModelCollectRun class
 * 
 * It only submits one job at a time, except for final porcessing
 * it submits both fast_dp and GMCAproc threads in a spawn of 15 secs
 * 
 * if the 15 secs gap not maintained DRMAA exception is coming, it can be investiagted further
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.analysis.helpers.MiscUtils;
import gov.anl.gmca.collect.DevFPE;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.diffimage.MarImageHeader;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.io.Disk;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;
import gov.anl.gmca.io.hw.Epics;
import gov.anl.gmca.log.TabLog;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Point2D;
import gov.anl.gmca.util.Util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.StringTokenizer;

public class InitDataProcessing implements Runnable {

	public static double LOOKFORETA = 45.0;
	public static boolean SUBMITTINGJOB = false;

	private boolean isDataRunExists = false;
	private double frameWidth_c = 1.0;
	private int proc_numdegrees = 1;
	private int procXDS = 1;

	public static String frame_name = "";
	private MarImageHeader currentHeader = null;
	private String currentRunStr = "", imageDir;
	private CmdDataProc cmdDataProc = null, cmdDataProc1 = null;
	private String firstFrame = "";

	public int sampNumber;
	public int runNumber = 1;
	
	private ModelCollectRun currRun;

	public InitDataProcessing() {}

	public void initParams() {
		ModelRunList runlist = ModelCollectRunList.get();
		ModelCollectRun currRun = ((ModelCollectRun) runlist.getRun(runlist.getCurrentRunIndex()));
		if (currRun != null) {
			procXDS = currRun.getXDSProcMode().ordinal();
			currentRunStr = Integer.toString(this.runNumber);
			imageDir = currRun.getFullDirectory();
			frameWidth_c = currRun.getDelta();
			proc_numdegrees = Integer.parseInt(DBConfig.get("processing_mindegrees"));
		}
	}

	public void stopThread(int step) {
		System.out.println("Analysis: Stoping processing thread for run :" + this.runNumber + " for step : " + step);
		TabLog.stopLog[this.runNumber] = true;
		TabLog.countLog[this.runNumber] = 0;
	}

	private String[] getEPICSBeamCenter_mm() {
		String beamXY = Epics.getString(PV_BLUICE.beam_xy);
		String delimiters = beamXY.contains(",") ? ", " : " ";

		StringTokenizer st = new StringTokenizer(beamXY, delimiters);
		double beamX_px = Double.parseDouble(st.nextToken());
		double beamY_px = Double.parseDouble(st.nextToken());

		Point2D p2d = DetectorConfig.getPixelSizeMM();
		double _beamX_mm = beamX_px * p2d.x;
		double _beamY_mm = beamY_px * p2d.y;
		String beamX_mm = String.format("%.3f", _beamX_mm);
		String beamY_mm = String.format("%.3f", _beamY_mm);

		String[] beam_mm = new String[2];
		beam_mm[0] = beamX_mm;
		beam_mm[1] = beamY_mm;
		return beam_mm;
	}

	@Override
	public void run() {
		/* get current run values */

		int localCountLog = 0;
		int procCount = 0;
		double min_angle_processing = 0.04;
		if (DBConfig.get("processing_parallel_min_angle") != null)
			min_angle_processing = Double.parseDouble(DBConfig.get("processing_parallel_min_angle"));

		initParams();
		System.out.println("Analysis: Starting of the thread for run : ");
		/* return for screening image */

		synchronized (DevDataProc.lock) {
			/* flag for incrementing result number showed in analysis tab */
			isDataRunExists = false;

			/* return for zero tab */
			if (currentRunStr.equals("0") || procXDS == 0) {

				stopThread(1);
				return;
			}

			while (!TabLog.stopLog[runNumber]) {
				if (TabLog.countLog[runNumber] < 1) {
					TabLog.countLog[runNumber]++;
					localCountLog++;
					procCount++;

				} else {
					if (localCountLog < TabLog.countLog[runNumber]) {

						localCountLog++;
						procCount++;
					}
				}
				if (TabLog.stopLog[runNumber]) {
					TabLog.stopLog[runNumber] = true;
					break;
				}
				Util.sleep(50);
				/* Submit XDS job for every proc_numdegrees degrees of data */
				if (DevUserConfig.processingPipeline < 3 && !frame_name.contains("_chk.")
						&& frameWidth_c > min_angle_processing) {
					if (frameWidth_c != 0.0) {

						int noofdegrees = (int) ((localCountLog * (double) (frameWidth_c + 0.05) * 10.0 / 10.0));
						int procCountdegrees = (int) ((procCount * (double) (frameWidth_c + 0.05) * 10.0 / 10.0));

						if (procCountdegrees >= proc_numdegrees && !isDataRunExists && noofdegrees >= proc_numdegrees) {
							procCount = 0;
							procCountdegrees = 0;

							final int finalCount = localCountLog;
							Thread finalProcess = new Thread() {
								public void run() {
									if (!CmdDataProc.THREAD_RUNNING) {

										if (DevFPE.get().getETA() > LOOKFORETA) {
											// if ETA < 45 don't submit job
											sampNumber = DevDataProc.getRowCount() + 1;
											SUBMITTINGJOB = true;
											System.out.println("Analysis: Submitting XDS job first time");
											submitXDSJob(finalCount, 0, true);
											isDataRunExists = true;
											SUBMITTINGJOB = false;
										} else {
											System.out.println("Analysis: ETA less than 45 sec, XDS first job not submitted");
										}
									}
								}
							};
							finalProcess.start();

						} else if (procCountdegrees >= proc_numdegrees && noofdegrees >= proc_numdegrees) {

							procCount = 0;
							procCountdegrees = 0;

							final int finalCount = localCountLog;
							Thread finalProcess = new Thread() {
								public void run() {
									if (!CmdDataProc.THREAD_RUNNING && !SUBMITTINGJOB) {

										/* store all results 2013-2 */
										if (DevFPE.get().getETA() > LOOKFORETA) {
											System.out.println("Analysis: Continue submitting XDS job");
											submitXDSJob(finalCount, 0, false);
										} else
											System.out.println("Analysis: ETA less than 45 sec, XDS job not submitted");

									}
								}
							};
							finalProcess.start();
						}
					}
				} else {
					System.out.println ("something wrong data processing not started");
				}
			}

			/* write XDS file if no. images more than 10 */
			if (DevUserConfig.processingPipeline < 3 && !frame_name.contains("_chk.")) {

				final int finalCount = ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).getEndFrameLabel();

				if (finalCount < 2)
					return;
				Thread finalProcess = new Thread() {
					public void run() {
						while (CmdDataProc.THREAD_RUNNING) {
							Util.sleep(500);
						}
						// if (finalCount >= proc_numdegrees - 1) {
						Log.note("Analysis: Submitting XDS job at the end of data collection");

						/* store all results 2013-2 */
						if (!isDataRunExists)
							sampNumber = DevDataProc.getRowCount() + 1;

						submitXDSJob(finalCount, 1, true);

						// }
					}
				};

				finalProcess.start();
				try {
					finalProcess.join();
				} catch (InterruptedException e) {

					e.printStackTrace();
				}
			}
			stopThread(2);
			Util.sleep(5000);
		}
		System.out.println("Analysis: Data processing thread finished for current dataset");
		TabLog.countLog[runNumber] = 0;
	}

	public synchronized void submitXDSJob(int numimages, int finalProc, boolean update) {

		final int _finalProc = finalProc;
		/* submit XDS job */
		/* String create sub directory for XDS */
		firstFrame = ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).seq().getFrameListCopy()
				.get(0).cbfFilename;
		imageDir = ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).getFullDirectory();

		currentHeader = (new CmdGetBeamCenter()).initHeader(imageDir + "/" + firstFrame);

		Thread finalProcess = new Thread() {
			public void run() {
				_submitXDSJob(_finalProc, update);
			}
		};
		finalProcess.start();
	}

	public void _submitXDSJob(int finalProc, boolean update) {

		/* String create sub directory for XDS */
		firstFrame = ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).seq().getFrameListCopy()
				.get(0).cbfFilename;
		int totalFrames = ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).seq().getFrameListCopy()
				.size();
		String lastFrame = ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).seq().getFrameListCopy()
				.get(totalFrames - 1).cbfFilename;
		/*
		 * String pre_framename =
		 * ((ModelCollectRun)ModelCollectRunList.get().getRun(this.runNumber))
		 * .getFileRoot()+"_"+ ModelCollectRunList.get().getCurrentRunIndex();
		 */
		String pre_framename = ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).getFileRoot() + "_"
				+ ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).getRunLabel();
		boolean inverseOn = ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).getInverseOn();
		int startFrameLabel = ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).getStartFrameLabel();

		int noEnergies = ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).getEnergies().size();
		imageDir = ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).getFullDirectory();

		int overlap_proc = ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).getOverlap();

		String collectTypetoProcessing = "Standard";

		if (((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber))
				.getScanMode() == ModelCollectRun.ScanMode.VECTOR)
			collectTypetoProcessing = "Vector";
		else if (((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber))
				.getScanMode() == ModelCollectRun.ScanMode.RASTER)
			collectTypetoProcessing = "Raster";

		String xdsDir = pre_framename + "_GMCAproc";

		/* change the working directory for fast_dp */
		int isFastDP = DevUserConfig.processingPipeline;

		if (collectTypetoProcessing.equals("Vector") && overlap_proc == 0) {
			collectTypetoProcessing = "Standard";
		}

		/* find which software used fast_dp/GMCAproc */
		String log = "", secondarylog = "";
		boolean software = true;
		String secondXDSDir = "";

		if ((isFastDP == 0 || isFastDP == 2) && !inverseOn && overlap_proc == 0
				&& collectTypetoProcessing.equals("Standard")) {
			software = true;
			xdsDir = pre_framename + "_fast_dp";
			secondXDSDir = pre_framename + "_GMCAproc";
			log = MiscUtils.getProcessDir(imageDir, MiscUtils.analysisSubDirectory) + "/" + xdsDir + "/" + CmdDataProc.tempLogDir + "/fast_dp.log";
			secondarylog = MiscUtils.getProcessDir(imageDir, MiscUtils.analysisSubDirectory) + "/" + secondXDSDir + "/" + CmdDataProc.tempLogDir + "/GMCAproc.LOG";
		} else {
			xdsDir = pre_framename + "_GMCAproc";
			secondXDSDir = pre_framename + "_fast_dp";
			software = false;
			log = MiscUtils.getProcessDir(imageDir, MiscUtils.analysisSubDirectory) + "/" + xdsDir + "/" + CmdDataProc.tempLogDir + "/GMCAproc.LOG";
		}

		File fastdpDirFile = new File(MiscUtils.getProcessDir(imageDir, MiscUtils.analysisSubDirectory) + "/" + xdsDir);
		if (!fastdpDirFile.exists()) {

			if (!fastdpDirFile.mkdirs()) {
				System.out.println("Couldn't create processing directory " + MiscUtils.getProcessDir(imageDir, MiscUtils.analysisSubDirectory) + "/" + xdsDir);
				return;
			}
		}

		String firstImage = Integer.toString(startFrameLabel);
		String lastImage = Integer
				.toString(((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).getNextFrame());

		int lastImageNo = startFrameLabel
				+ ((ModelCollectRun) ModelCollectRunList.get().getRun(this.runNumber)).getNextFrame() - 1;

		if (finalProc != 0)
			lastImage = Integer.toString(lastImageNo + startFrameLabel);

		if (lastImageNo < 2) {
			Log.error("Processing job not submitted");
			return;
		}
		if (lastImageNo <= startFrameLabel) {
			Log.error("Processing job not submitted " + lastImageNo + " " + startFrameLabel);
			System.out.println("Thread interference for submitting processing job to Grid Engine");
			return;
		}
		String imageSet = "";
		if (finalProc == 0)
			imageSet = firstImage + "-" + lastImage;
		else
			imageSet = "Final_" + firstImage + "-" + lastImageNo;

		String data_dir_prefix = "";

		String ppu_prefix = DBConfig.get("ppu_ramdisk_prefix");
		data_dir_prefix = imageDir.replace("/mnt",
				DBConfig.get("processing_data_dir").toString().equals("NFS") ? (ppu_prefix + "/mnt") : "/mnt");

		/* test data */
		if (Main.FAKE_DATA) {

			firstFrame = DBConfig.get("processing_test_firstframe");
			data_dir_prefix = DBConfig.get("processing_test_dir");
			if (firstFrame.contains("cbf")) {
				pre_framename = firstFrame.substring(0, firstFrame.lastIndexOf('_'));
			} else {
				pre_framename = firstFrame.substring(0, firstFrame.lastIndexOf('.'));
			}

			/* udpate beamX, beamY */
			currentHeader = (new CmdGetBeamCenter()).initHeader(data_dir_prefix + "/" + firstFrame);

			File fakedataFile = new File(imageDir + "/" + "testdataset");

			if (fakedataFile.exists()) {
				ArrayList<String> lines = Disk.readText(imageDir + "/" + "testdataset");
				try {
					if (lines.size() >= 3) {
						pre_framename = lines.get(0);
						firstFrame = lines.get(1);
						data_dir_prefix = lines.get(2);

						/* udpate beamX, beamY */
						currentHeader = (new CmdGetBeamCenter()).initHeader(data_dir_prefix + "/" + firstFrame);
					} else {
						Log.error("Test dataset file doesn't have enough data, taking defaults");
					}

				} catch (Exception e) {
					System.out.println(e.getMessage());
				}
			}
		}

		if (Main.SIM_LEVEL >= 1) {

			ExecTask.new_ExecTask("touch " + firstFrame.substring(0, firstFrame.lastIndexOf('.') - 1) + "log" + "",
					"Evince", false, false).start();

			int i = Integer.parseInt(firstImage);
			int j = Integer.parseInt(lastImage);
			;
			String _prefix = firstFrame.substring(0, firstFrame.indexOf("."));
			int MAXIMG = 360;
			if (firstFrame.contains(".cbf")) {
				_prefix = firstFrame.substring(0, firstFrame.lastIndexOf("_"));
				MAXIMG = 4500;
			}
			String frame = "", framecp = "";
			if (j <= MAXIMG) {
				for (; i < j; i++) {
					if (firstFrame.contains(".cbf")) {
						frame = _prefix + "_" + String.format("%05d", i) + ".cbf";
						framecp = "xdstest1_1_" + String.format("%05d", i) + ".cbf";
						// dataprocResult.imageDir =
						// "/mnt/sahre1/user0/testdataxds /";
					} else {
						frame = _prefix + "." + String.format("%05d", i);
						framecp = "sdrc_nat4_1." + String.format("%04d", i);
					}
					Log.warn("cp -fp /mnt/share1/user0/testdataxds /" + framecp + " " + imageDir + "/" + frame);
					ExecTask.syncExec("cp -f /mnt/share1/user0/testdataxds /" + framecp + " " + imageDir + "/" + frame,
							null, false);
				}
			}
			currentHeader.displayOriginX = 207.948;
			currentHeader.displayOriginY = 221.02;
		}

		String beamX_mm, beamY_mm;
		String[] beam_mm = getEPICSBeamCenter_mm();
		beamX_mm = beam_mm[0];
		beamY_mm = beam_mm[1];

		try {
			/* process XDS */
			BufferedWriter runxdsOut = new BufferedWriter(
					new FileWriter(MiscUtils.getProcessDir(imageDir, MiscUtils.analysisSubDirectory) + "/" + xdsDir + "/" + "run_XDS.sh", false));

			String command = Main.getAppDir() + "/tools/scripts/dataproc/run_XDS.csh";

			String isScala = "aimless";

			/* treat vector with overlaps as normal collection */
			if (collectTypetoProcessing.equals("Vector")) {
				if (overlap_proc == 0)
					collectTypetoProcessing = "Standard";
			}

			// String beamXY = Epics.getString(PV_BLUICE.beam_xy);
			// String delimiters = beamXY.contains(",")?", ":" ";
			// StringTokenizer st = new StringTokenizer(beamXY, delimiters);
			// double beamX_px = Double.parseDouble(st.nextToken());
			// double beamY_px = Double.parseDouble(st.nextToken());
			// !! incorrect code below, using eiger dimension here which is not
			// applicable to pilatus
			// double _beamX_mm = beamX_px * 0.075;
			// double _beamY_mm = beamY_px * 0.075;
			// String beamX_mm = String.format("%.3f", _beamX_mm);
			// String beamY_mm = String.format("%.3f", _beamY_mm);

			/*
			 * TODO write XML file instead, too many parameters at commandline
			 */
			runxdsOut.write("# Running fast_dp/XDS in parallel, works only on 23ID beamlines envitonment @APS\n");
			runxdsOut.write("# Usage : run_XDS.csh <file_prefix> <working_dir> "
					+ "<number of energies> <first_imagefullname> <last_imagefullname> <fast_dp> <scala/aimless> "
					+ "<beamX> <beamY>" + "<firstImageNo> <lastImageNo> <isInverseBeam> <imageSet> <collectType>\n");
			runxdsOut.write(command + " " + pre_framename + " " + String.format("%s/%s", MiscUtils.getProcessDir(imageDir, MiscUtils.analysisSubDirectory),xdsDir) + " " + noEnergies + " "
					+ firstFrame + " " + lastFrame + " " + (!software ? 1 : 0) + " " + isScala + " " + beamX_mm + " "
					+ beamY_mm + " " + firstImage + " " + lastImage + " " + (inverseOn ? 1 : 0) + " " + finalProc + " "
					+ imageSet + " " + collectTypetoProcessing + " " + data_dir_prefix);
			runxdsOut.close();
			Runtime.getRuntime().exec("chmod u+x " + MiscUtils.getProcessDir(imageDir, MiscUtils.analysisSubDirectory) + "/" + xdsDir + "/" + "run_XDS.sh");
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}

		if (procXDS > 0) {
			cmdDataProc = new CmdDataProc();
			cmdDataProc.workDir = String.format("%s/%s", MiscUtils.getProcessDir(imageDir, MiscUtils.analysisSubDirectory), xdsDir);
			cmdDataProc.imageDir = imageDir;
			cmdDataProc.sampNumber = sampNumber;
			cmdDataProc.software = software;
			cmdDataProc.firstImage = firstFrame;
			cmdDataProc.inverseOn = inverseOn;
			cmdDataProc.overlap = Integer.toString(overlap_proc);
			cmdDataProc.collectType = collectTypetoProcessing;
			cmdDataProc.sampleName = pre_framename;
			if (finalProc == 0)
				cmdDataProc.images = imageDir + "/" + pre_framename + ".##### " + firstImage + " " + lastImage;
			else
				cmdDataProc.images = imageDir + "/" + pre_framename + ".##### " + firstImage + " " + lastImageNo;

			cmdDataProc.firstImgNo = firstImage;
			cmdDataProc.lastImgNo = lastImage;
			cmdDataProc.imageSet = imageSet;
			cmdDataProc.log = log;
			cmdDataProc.second_pipeline_update = true;
			cmdDataProc.interface_update = update;
			cmdDataProc.start();

			if (DevUserConfig.processingPipeline == 2 && finalProc == 1 && !inverseOn && overlap_proc == 0
					&& !collectTypetoProcessing.equals("Raster")) {

				String secondProcessDir = MiscUtils.getProcessDir(imageDir, MiscUtils.analysisSubDirectory) + "/" + secondXDSDir;
				fastdpDirFile = new File(secondProcessDir);
				
				if (!fastdpDirFile.exists()) {

					if (!fastdpDirFile.mkdirs()) {
						System.out.println("Couldn't create processing directory " + secondProcessDir);
						return;
					}
				}

				try {
					/* process XDS */
					BufferedWriter runxdsOut = new BufferedWriter(
							new FileWriter(secondProcessDir + "/" + "run_XDS.sh", false));

					String command = Main.getAppDir() + "/tools/scripts/dataproc/run_XDS.csh";

					String isScala = "aimless";

					/* treat vector with overlaps as normal collection */
					if (collectTypetoProcessing.equals("Vector")) {
						if (overlap_proc == 0)
							collectTypetoProcessing = "Standard";
					}

					/*
					 * TODO write XML file instead, too many parameters at
					 * commandline
					 */
					runxdsOut.write(
							"# Running fast_dp/XDS in parallel, works only on 23ID beamlines envitonment @APS\n");
					runxdsOut.write("# Usage : run_XDS.csh <file_prefix> <working_dir> "
							+ "<number of energies> <first_imagefullname> <last_imagefullname> <fast_dp> <scala/aimless> "
							+ "<beamX> <beamY>"
							+ "<firstImageNo> <lastImageNo> <isInverseBeam> <imageSet> <collectType>\n");
					runxdsOut.write(command + " " + pre_framename + " " + MiscUtils.getProcessDir(imageDir, MiscUtils.analysisSubDirectory) + "/" + secondXDSDir + " "
							+ noEnergies + " " + firstFrame + " " + lastFrame + " " + (software ? 1 : 0) + " " + isScala
							+ " " + beamX_mm + " " + beamY_mm + " " + firstImage + " " + lastImage + " "
							+ (inverseOn ? 1 : 0) + " " + finalProc + " " + imageSet + " " + collectTypetoProcessing
							+ " " + data_dir_prefix);
					runxdsOut.close();
					Runtime.getRuntime().exec("chmod u+x " + secondProcessDir + "/" + "run_XDS.sh");
				} catch (IOException e) {
					e.printStackTrace();
					return;
				}
				Util.sleep(15000);

				cmdDataProc1 = new CmdDataProc();
				cmdDataProc1.workDir = secondProcessDir;
				cmdDataProc1.imageDir = imageDir;
				cmdDataProc1.sampNumber = sampNumber;
				cmdDataProc1.software = !software;
				cmdDataProc1.firstImage = firstFrame;
				cmdDataProc1.inverseOn = inverseOn;
				cmdDataProc1.overlap = Integer.toString(overlap_proc);
				cmdDataProc1.collectType = collectTypetoProcessing;
				cmdDataProc1.sampleName = pre_framename;
				if (finalProc == 0)
					cmdDataProc1.images = imageDir + "/" + pre_framename + ".##### " + firstImage + " " + lastImage;
				else
					cmdDataProc1.images = imageDir + "/" + pre_framename + ".##### " + firstImage + " " + lastImageNo;
				cmdDataProc1.firstImgNo = firstImage;
				cmdDataProc1.lastImgNo = lastImage;
				cmdDataProc1.imageSet = imageSet;
				cmdDataProc1.log = secondarylog;
				cmdDataProc1.second_pipeline_update = false;
				cmdDataProc1.interface_update = false;
				cmdDataProc1.start();
			}
		}
	}
}
