/**
 * @author spothineni
 * 
 * This is the model for aimless.log summary
 */

package gov.anl.gmca.dataproc;

import java.util.ArrayList;
import java.util.HashMap;

public class ModelDataProc implements Comparable<ModelDataProc> {
	
	public ArrayList<String> imageSets = new ArrayList<String>();
	public ArrayList<String> pipelines = new ArrayList<String>();
	
	public static enum DataProcField {
							// db string // user string // display lines
				SAMPLE_NUMBER("sampleNumber", "Sample #", 3, true),
				SAMPLE_NAME("sampleName","Sample Name", 2, true), 
				IMAGE_SET("imageSet", "Image Set", 2, true), 
				STATE("state", "State", 2, true), 
				COLLECTTYPE("collectType","Collect Type", 2, true), 
				OVERLAP("overlap", "Overlap", 2, true), 
				INVERSEON("inverseOn", "Inverse Beam", 2, true), 
				FIRSTFRAME("firstFrame", "First Image", 2, true), 
				SOFTWARE("software","Software", 2, true), 
				SCALINGSOFTWARE("scalingsoftware","Scaling Software", 2, true), 
				WORKDIR("workdir", "Proc. Dir", 2, true), 
				SUBDIR("subdir", "Dir", 2, true),
				IMAGEDIR("imagedir", "Image Dir", 2, true),
				WARNING("warning", "Status", 3, true),
				IMAGES("images", "Images", 2, true), 
				UNIT_CELL("unitcell", "Unit Cell", 2, true), 
				SPACE_GROUP("spacegroup", "Space Group", 2, true), 
				LOWRESOLUTION("lowresolution","Low Res.", 1, true), 
				LOWRESOLUTION_INNER("lowresolution_inner","Low Res.", 0, false), 
				LOWRESOLUTION_OUTER("lowresolution_outer","Low Res.", 0, false), 
				HIGHRESOLUTION("highresolution", "High Res.", 1, true), 
				HIGHRESOLUTION_INNER("highresolution_inner", "High Res.", 0, false),
				HIGHRESOLUTION_OUTER("highresolution_outer", "High Res.", 0, false), 
				RMERGE("rmerge","Rmerge", 1, true), 
				RMERGE_INNER("rmerge_inner", "Rmerge", 0, false),
				RMERGE_OUTER("rmerge_outer", "Rmerge", 0, false), 
				RPIM("rpim", "Rpim", 1, true), 
				RPIM_INNER("rpim_inner", "Rpim", 0, false), 
				RPIM_OUTER("rpim_outer", "Rpim", 0, false), 
				CCHALF("cchalf", "CC1/2", 1, true),
				CCHALF_INNER("cchalf_inner", "CC1/2", 0, false),
				CCHALF_OUTER("cchalf_outer", "CC1/2", 0, false), 
				COMPLETE("completeness","Completeness", 1, true),
				COMPLETE_INNER("completeness_inner","Completeness", 0, false), 
				COMPLETE_OUTER("completeness_outer","Completeness", 0, false), 
				ANOM_COMPLETE("anom_completeness","Anom.Completeness", 1, true), 
				ANOM_COMPLETE_INNER("anom_completeness_inner", "Anom.Completeness", 0, false), 
				ANOM_COMPLETE_OUTER("anom_completeness_outer", "Anom.Completeness", 0, false), 
				MULTIPLICITY("multiplicity", "Multiplicity", 1, true), 
				MULTIPLICITY_INNER("multiplicity_inner", "Multiplicity", 0, false),
				MULTIPLICITY_OUTER("multiplicity_outer", "Multiplicity", 0, false),
				ANOM_MULTIPLICITY("anom_multiplicity", "Anom.Multiplicity", 1, true), 
				ANOM_MULTIPLICITY_INNER("anom_multiplicity_inner", "Anom.Multiplicity", 0, false), 
				ANOM_MULTIPLICITY_OUTER("anom_multiplicity_outer", "Anom.Multiplicity", 0, false), 
				ISIGMA("isigma", "I/Sigma", 1, true), 
				ISIGMA_INNER("isigma_inner","I/Sigma", 0, false), 
				ISIGMA_OUTER("isigma_outer", "I/Sigma", 0, false),
				LOG("log", "XDS Log", 1, true), 
				SCALALOG("scalalog", "Scala Log", 1, true), 
				TRUNCATELOG("truncatelog", "Truncate Log", 1, true);

		protected String dbString, userString;
		protected int displayLines;
		protected boolean exportable;

		public String getDBString() {
			return dbString;
		}

		public String getUserString() {
			return userString;
		}

		public int getDisplayLines() {
			return displayLines;
		}
		public boolean getExportable() {
			return exportable;
		}

		DataProcField(String dbString, String userString, int displayLines, boolean exportable) {
			this.dbString = dbString;
			this.userString = userString;
			this.displayLines = displayLines;
			this.exportable = exportable;
		}
	}

	public static enum DataProcState {
		INACTIVE, ERROR, PROCESSING, DONE;

		public static DataProcState stringToState(String stateString) {
			if (null == stateString)
				return null;

			for (DataProcState state : DataProcState.values()) {
				if (stateString.toLowerCase().equals(
						state.toString().toLowerCase()))
					return state;
			}

			return null;
		}
	}

	public DataProcState getState() {
		return DataProcState.stringToState(getField(DataProcField.STATE));
	}

	protected HashMap<DataProcField, String> fieldMap = new HashMap<DataProcField, String>();

	public void setField(DataProcField field, String value) {
		fieldMap.put(field, value);
	}

	public String getField(DataProcField field) {
		return fieldMap.get(field);
	}

	public HashMap<DataProcField, String> getAllFields() {
		return fieldMap;
	}

	protected Integer resultID = null;

	public Integer getID() {
		return resultID;
	}

	public void setID(int id) {
		resultID = id;
	}

	public ModelDataProc(HashMap<DataProcField, String> fieldMap) {
		this.fieldMap = new HashMap<DataProcField, String>(fieldMap);
		String sampleNumber = fieldMap.get(DataProcField.SAMPLE_NUMBER);
		if (null != sampleNumber) {
			try {
				resultID = Integer.parseInt(sampleNumber);
			} catch (NumberFormatException nfe) {
			}
		}
	}

	@Override
	public int compareTo(ModelDataProc r2) {
		return getID() - r2.getID();
	}
}
