/**
 * @author spothineni
 * 
 * This should be POJO for data processing parameters
 * but the reading aimless.log from ModelAimlessSummary duplicated hear
 * this need to be changed.
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;

import java.io.*;
import java.util.StringTokenizer;

public class ModelDataProcParams {

	public String warning = "processing";
	public boolean software = true;
	public String scalingsoftware = "AIMLESS";
	public String workDir = "";
	public String subdir = "";
	public String imageDir = "";
	public String sampleName = "";
	public String images = "";
	public String scalaParams = "";
	public String imageSet = "1-10";
	public String firstImage = "";
	public String collectType = "";
	public boolean inverseOn = false;

	public String spaceGroup = "";
	public String unitCell = "";
	public String rMerge = "";
	public String mosaicity = "";
	public String completeness = "";
	public String anom_completeness = "";
	public String multiplicity = "";
	public String anom_multiplicity = "";
	public String iSigma = "";

	public String resolution = "";
	public String lowresolution = "";
	public String lowresolution_inner = "";
	public String lowresolution_outer = "";

	public String highresolution = "";
	public String highresolution_inner = "";
	public String highresolution_outer = "";
	public String cchalf = "";
	public String rPim = "";

	public String rMerge_inner = "";
	public String mosaicity_inner = "";
	public String completeness_inner = "";
	public String anom_completeness_inner = "";
	public String multiplicity_inner = "";
	public String anom_multiplicity_inner = "";
	public String iSigma_inner = "";
	public String cchalf_inner = "";
	public String rPim_inner = "";

	public String rMerge_outer = "";
	public String mosaicity_outer = "";
	public String completeness_outer = "";
	public String anom_completeness_outer = "";
	public String multiplicity_outer = "";
	public String anom_multiplicity_outer = "";
	public String iSigma_outer = "";
	public String cchalf_outer = "";
	public String rPim_outer = "";

	public String log = "";
	public String scalaLog = "";
	public String aimlessLog = "";
	public String truncateLog = "";

	public String state = "";
	public String overlap = "0";
	
	public ModelDataProcParams(){
		
	}
	
	public void processResult(String _workDir) throws Exception {
		String outputFile = "", xdsLog = "";
		int pos1, pos2, pos3, pos4;
		workDir = _workDir;
		/* Read XDS log file */
		int isFastDP = DevUserConfig.processingPipeline;

		if (isFastDP == 0)
			xdsLog = workDir + "/fast_dp.log";
		else
			xdsLog = workDir + "/GMCAproc.LOG";

		outputFile = workDir + "/" + sampleName + "_scala.log";
		truncateLog = workDir + "/" + sampleName + "_ctruncate.log";
		scalaLog = outputFile;

		File xdsFile = new File(xdsLog);
		if (!xdsFile.exists()) {
			Log.error(xdsLog + " file not found");
			state = "ERROR";
			warning = xdsLog + " doesnt exists check data!!!";
			return;
		} else {
			String xdsContent = readLogFile(xdsLog);
			if (isFastDP == 0) {
				if (xdsContent.contains("error:")) {
					pos1 = xdsContent.lastIndexOf("error:");
					warning = xdsContent.substring(pos1);
					state = "ERROR";
					return;
				}
			} else {
				if (xdsContent.contains("!!! ERROR ")) {
					pos1 = xdsContent.lastIndexOf("!!! ERROR ");
					warning = xdsContent.substring(pos1,pos1+2500).replace("\n", " ");
					state = "ERROR";
					return;
				}
			}
		}

		Log.note("reading " + outputFile);

		File scalaFile = new File(outputFile);

		if (!scalaFile.exists()) {
			Log.note(scalaLog
					+ " doesnt exists,scaling not successful, check XDS log file for errors!!!");
			warning = scalaLog
					+ " doesnt exists, scaling not successful, check XDS log file for errors!!!";
			return;
		}

		File truncateFile = new File(outputFile);

		if (!truncateFile.exists()) {
			Log.note(truncateLog
					+ " doesnt exists,scaling not successful, check data!!!");
			warning = truncateLog
					+ " doesnt exists, scaling not successful, check data!!!";
		}

		/* Read scala log file */
		String content = readLogFile(outputFile);

		scalaParams = "N/A";
		pos1 = content.indexOf("Overall  InnerShell  OuterShell");

		pos2 = content.indexOf("Average unit cell:", pos1);
		if (pos2 < 0 || pos1 < 0) {
			warning = "Average unit cell doesnt find in scala log";
			state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);
		String unitcell = content.substring(pos2 + 20, pos3);
		StringTokenizer tok = new StringTokenizer(unitcell);
		unitCell = tok.nextToken() + " ";
		unitCell = unitCell + " " + tok.nextToken();
		unitCell = unitCell + " " + tok.nextToken();
		unitCell = unitCell + " " + tok.nextToken();
		unitCell = unitCell + " " + tok.nextToken();
		unitCell = unitCell + " " + tok.nextToken();

		pos2 = content.indexOf("Space group:", pos1);
		if (pos2 < 0 || pos1 < 0) {
			warning = "Space group doesnt find in scala log";
			state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);
		spaceGroup = content.substring(pos2 + 13, pos3);

		pos2 = content.indexOf("Maximum resolution:", pos1);
		if (pos2 < 0 || pos1 < 0) {
			warning = "Space group doesnt find in scala log";
			state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);
		resolution = content.substring(pos2 + 24, pos3);

		pos2 = content.indexOf("High resolution limit", pos1);
		if (pos2 < 0 || pos1 < 0) {
			warning = "High resolution limit doesnt find in scala log";
			state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String highres = content.substring(pos2 + 24, pos3);
		tok = new StringTokenizer(highres);
		String highres_ovearall = tok.nextToken();
		String highres_innershell = tok.nextToken();
		String highres_outershell = tok.nextToken();

		highresolution = highres_ovearall;
		highresolution_inner = highres_innershell;
		highresolution_outer = highres_outershell;

		pos2 = content.indexOf("Low resolution limit", pos1);
		if (pos2 < 0 || pos1 < 0) {
			warning = "Low resolution limit doesnt find in scala log";
			state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String lowres = content.substring(pos2 + 20, pos3);
		tok = new StringTokenizer(lowres);
		String lowres_ovearall = tok.nextToken();
		String lowres_innershell = tok.nextToken();
		String lowres_outershell = tok.nextToken();

		lowresolution = lowres_ovearall;
		lowresolution_inner = lowres_innershell;
		lowresolution_outer = lowres_outershell;
		// resolution = highres_ovearall;

		pos2 = content.indexOf("Rmerge ", pos1);
		if (pos2 < 0 || pos1 < 0) {
			warning = "Rmerge doesnt find in scala log";
			state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String rmerge = content.substring(pos2 + 10, pos3);
		tok = new StringTokenizer(rmerge);
		String rmerge_ovearall = tok.nextToken();
		String rmerge_innershell = tok.nextToken();
		String rmerge_outershell = tok.nextToken();

		rMerge = rmerge_ovearall;
		rMerge_inner = rmerge_innershell;
		rMerge_outer = rmerge_outershell;

		pos2 = content.indexOf("Completeness", pos1);
		if (pos2 < 0 || pos1 < 0) {
			warning = "Completeness doesnt find in scala log";
			state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String complete = content.substring(pos2 + 15, pos3);
		tok = new StringTokenizer(complete);
		String complete_ovearall = tok.nextToken();
		String complete_innershell = tok.nextToken();
		String complete_outershell = tok.nextToken();

		completeness = complete_ovearall;
		completeness_inner = complete_innershell;
		completeness_outer = complete_outershell;

		pos2 = content.indexOf("Anomalous completeness", pos1);
		if (pos2 < 0 || pos1 < 0) {
			warning = "Completeness doesnt find in scala log";
			state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String anom_complete = content.substring(pos2 + 25, pos3);
		tok = new StringTokenizer(anom_complete);
		String anom_complete_ovearall = tok.nextToken();
		String anom_complete_innershell = tok.nextToken();
		String anom_complete_outershell = tok.nextToken();

		anom_completeness = anom_complete_ovearall;
		anom_completeness_inner = anom_complete_innershell;
		anom_completeness_outer = anom_complete_outershell;

		pos2 = content.indexOf("Multiplicity", pos1);
		if (pos2 < 0 || pos1 < 0) {
			warning = "Completeness doesnt find in scala log";
			state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String multi = content.substring(pos2 + 15, pos3);
		tok = new StringTokenizer(multi);
		String multi_ovearall = tok.nextToken();
		String multi_innershell = tok.nextToken();
		String multi_outershell = tok.nextToken();

		multiplicity = multi_ovearall;
		multiplicity_inner = multi_innershell;
		multiplicity_outer = multi_outershell;

		pos2 = content.indexOf("Anomalous multiplicity", pos1);
		if (pos2 < 0 || pos1 < 0) {
			warning = "Completeness doesnt find in scala log";
			state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String anom_multi = content.substring(pos2 + 25, pos3);
		tok = new StringTokenizer(anom_multi);
		String anom_multi_ovearall = tok.nextToken();
		String anom_multi_innershell = tok.nextToken();
		String anom_multi_outershell = tok.nextToken();

		anom_multiplicity = anom_multi_ovearall;
		anom_multiplicity_inner = anom_multi_innershell;
		anom_multiplicity_outer = anom_multi_outershell;

		pos2 = content.indexOf("Mean((I)/sd(I))", pos1);
		if (pos2 < 0 || pos1 < 0) {
			warning = "Completeness doesnt find in scala log";
			state = "ERROR";
			return;
		}
		pos3 = content.indexOf("\n", pos2);

		String isigma = content.substring(pos2 + 15, pos3);
		tok = new StringTokenizer(isigma);
		String isigma_ovearall = tok.nextToken();
		String isigmai_innershell = tok.nextToken();
		String isigmai_outershell = tok.nextToken();

		iSigma = isigma_ovearall;
		iSigma_inner = isigmai_innershell;
		iSigma_outer = isigmai_outershell;

		scalaParams = content.substring(pos1, pos2 + 20);

		// Generate Graphs

		pos1 = content.indexOf("N 1/d^2 Dmin(A) Rmrg");
		pos2 = content.indexOf("$$", pos1);
		pos3 = content.indexOf("$$", pos2 + 10);

		pos4 = content.indexOf("\n", pos1);
		String header = content.substring(pos1, pos4);

		String resolutionTable = content.substring(pos2 + 3, pos3);
		if (resolutionTable.equals("") || resolutionTable == null)
			return;

		BufferedWriter out = new BufferedWriter(new FileWriter(workDir
				+ "/resolutionTable1.txt", false));

		out.write(header);
		out.write("\n");
		out.write(resolutionTable);
		out.close();

		/* write png files */

		writeGnuplotPNG("Rmerge vs Resolution", "Resolution (A)", "Rmerge", 3,
				4, workDir + "/resolutionTable1.txt", workDir
						+ "/rmergevsresolution");

		writeGnuplotPNG("Isigma vs Resolution", "Resolution (A)", "I/Sigma", 3,
				11, workDir + "/resolutionTable1.txt", workDir
						+ "/isigmavsresolution");

		pos1 = content.indexOf("N 1/resol^2 Dmin   Nmeas");
		pos2 = content.indexOf("$$", pos1);
		pos3 = content.indexOf("$$", pos2 + 10);

		pos4 = content.indexOf("\n", pos1);
		header = content.substring(pos1, pos4);

		resolutionTable = content.substring(pos2 + 3, pos3);
		if (resolutionTable.equals("") || resolutionTable == null)
			return;

		out = new BufferedWriter(new FileWriter(workDir
				+ "/resolutionTable2.txt", false));

		out.write(header);
		out.write("\n");
		out.write(resolutionTable);
		out.close();

		writeGnuplotPNG("Completeness vs Resolution", "Resolution (A)",
				"Completeness", 3, 8, workDir + "/resolutionTable2.txt",
				workDir + "/completevsresolution");

		writeGnuplotPNG("Multiplicity vs Resolution", "Resolution (A)",
				"Multiplicity", 3, 9, workDir + "/resolutionTable2.txt",
				workDir + "/multivsresolution");

		writeGnuplotPNG("Rmeas vs Resolution", "Resolution (A)", "Rmeas", 3,
				14, workDir + "/resolutionTable2.txt", workDir
						+ "/rmeasvsresolution");
	}

	protected void writeGnuplotPNG(String title, String xaxis, String yaxis,
			int col1, int col2, String fileName, String pngFile) {
		BufferedWriter plotIntBS;
		try {
			plotIntBS = new BufferedWriter(new FileWriter(workDir
					+ "/vsresolution.sh", false));

			/* write gnupplot file */
			plotIntBS.append("set title \"" + title + "\"\n");
			plotIntBS.append("set xlabel \"" + xaxis + "\"\n");
			if (xaxis.equals("Resolution (A)"))
				plotIntBS.append("set xrange [" + highresolution + ":"
						+ highresolution_inner + "] reverse\n");
			plotIntBS.append("set ylabel \"" + yaxis + "\"\n");
			plotIntBS.append("set terminal png size 320, 240\n");
			plotIntBS.append("unset key\n");
			plotIntBS.append("set output \"" + pngFile + ".png\"" + "\n");
			plotIntBS.append("plot \"" + fileName + "\" using " + col1 + ":"
					+ col2 + "  with linespoints");
			if (plotIntBS != null)
				plotIntBS.close();

			Util.sleep(100);
			ExecTask task = ExecTask.new_ExecTask("/usr/bin/gnuplot " + workDir
					+ "/vsresolution.sh", "", false, false);
			task.start();
			task.waitFor();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	protected String readLogFile(String logFile) {

		byte[] buffer = new byte[(int) new File(logFile).length()];
		BufferedInputStream f = null;
		try {
			f = new BufferedInputStream(new FileInputStream(logFile));
			f.read(buffer);
			f.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			if (f != null)
				try {
					f.close();
				} catch (IOException ignored) {
				}
		}

		return new String(buffer);
	}
}
