/**
 * @author spothineni
 * 
 * Help sub-tab of Analysis tab GUI
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.Bullet;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.GlyphMetrics;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;


public class WdgProcHelp extends Composite implements
		ListenerList.Listener<ModelDataProcList>, PaintListener {
	
	private Composite helpComp;
	protected Button helpButton, videoButton;
	public static String XDS_VERSION = "XDS";
	public static String CCP4_VERSION = "CCP4";
	public static String POINTLESS_VERSION = "POINTLESS";
	public static String AIMLESS_VERSION = "AIMLESS";
	
	public WdgProcHelp(Composite parent, int style) {
		super(parent, style);
		setLayout(new GridLayout(1, true));
		
		helpComp = new Composite(this, SWT.NONE);
		helpComp.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);

		helpComp.setLayoutData(gd);
		helpComp.setFont(GUI.FONT_COLLECT_GROUP);
		
		helpButton = new Button(helpComp, SWT.PUSH);
		helpButton.setText("Help");
		helpButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		
		createHelp();
		updateTable();
	}

	protected void createHelp() {
		
		
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1);
		gd.widthHint = 630;
		
		GridLayout actionGrid = new GridLayout(1, false);
		helpComp.setLayout(actionGrid);

		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 2;
		gd.minimumWidth = 120;
		helpButton.setLayoutData(gd);
		
		helpButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}

			@Override
			public void widgetSelected(SelectionEvent arg0) {
				
				String filename = Main.getAppDir()+"/java/docs/autoprocessing.pdf";
				ExecTask.new_ExecTask("firefox " + filename + "", "firefox",
						false, false).start();
		}});
		
		videoButton = new Button(helpComp, SWT.PUSH);
		videoButton.setText("Video Tutorial");
		videoButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 2;
		videoButton.setLayoutData(gd);
		
		videoButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}

			@Override
			public void widgetSelected(SelectionEvent arg0) {
				
				String filename = "https://www.gmca.aps.anl.gov/video/html5-guides/analysis-vjs.html";
				ExecTask.new_ExecTask("firefox " + filename + "", "firefox",
						false, false).start();
		}});
		
		// create the styled text for processing help text
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 8;
		StyledText helpWidget = new StyledText(helpComp, SWT.BORDER | SWT.WRAP);
		helpWidget.setText("Auto processing use following software with either fast_dp or GMCAproc\n"
				+ "\n"
				+ "XDS\t\t\t\t(Autoindexing, Integration and scaling)\n"
				+ "POINTLESS\t(spacegroup determination)\n"
				+ "AIMLESS\t\t(Merging & Statistics)\n"
				+ "CTRUNCATE\t(Structure factor amplitudes)\n\n"
				+ "To switch between fast_dp and GMCAproc select it from JBluIce top menu Tools -> Options -> Background data processing.\n\n"
				+ "Selecting 'both' option will run both fast_dp and GMCAproc pipelines only at the end of data processing.\n\n"
				+ "Double clicking on the row of All Results tab will load corresponding processing results to Data quality sub-tab.\n\n"
				+ "To enable and disable parameters set on Settings tab use the check box 'Use above XDS parameters for auto-processing' "
				+ "above 'Set defaults' button on the Settings tab.\n\n"
				+ "To reprocess the data use Reprocessing tab after selecting corresponding dataset on Data quality tab or "
				+ "double clicking on All Results tab.\n\n"
				+ "To reprocess the data in the command line with different XDS parameters, put them in .xdsparams text file in "
				+ "the dataprocessing directory and run script ./run_XDS.sh in the same directory");
		
		helpWidget.setLayoutData(gd);
		
		StyleRange style0 = new StyleRange();
		style0.metrics = new GlyphMetrics(0, 0, 40);
		Bullet bullet0 = new Bullet (style0);
		helpWidget.setLayoutData(gd);
		
		int tip1 = 7;
		helpWidget.setLineBullet(tip1, 1, bullet0);
		helpWidget.setLineJustify(tip1, 1, true);
		helpWidget.setLineBackground(tip1, 1, GUI.BGCOLOR);
		
		int tip2 = 9;
		helpWidget.setLineBullet(tip2, 1, bullet0);
		helpWidget.setLineJustify(tip2, 1, true);
		
		int tip3 = 11;
		helpWidget.setLineBullet(tip3, 1, bullet0);
		helpWidget.setLineJustify(tip3, 1, true);
		helpWidget.setLineBackground(tip3, 1, GUI.BGCOLOR);
		addPaintListener(this);
		
		int tip4 = 13;
		helpWidget.setLineBullet(tip4, 1, bullet0);
		helpWidget.setLineJustify(tip4, 1, true);
		
		int tip5 = 15;
		helpWidget.setLineBullet(tip5, 1, bullet0);
		helpWidget.setLineJustify(tip5, 1, true);
		helpWidget.setLineBackground(tip5, 1, GUI.BGCOLOR);
		
		int tip6 = 17;
		helpWidget.setLineBullet(tip6, 1, bullet0);
		helpWidget.setLineJustify(tip6, 1, true);
		
		addPaintListener(this);
	}

	public void updateTable() {}
	
	public void connectListeners() {}

	@Override
	public void handleEvent(ModelDataProcList object, Object data) {}

	@Override
	public void paintControl(PaintEvent e) {}
}
