/**
 * @author spothineni
 * 
 * Reprocessing sub-tab of Analysis tab
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.dataproc.ModelDataProc.DataProcField;
import gov.anl.gmca.dataproc.ModelDataProc.DataProcState;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.multistrategy.DevMultiStrategy;
import gov.anl.gmca.strategy.ModelStrategyResult;
import gov.anl.gmca.strategy.ModelStrategyResultList;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.HashMap;

public class WdgReprocessing extends Composite implements
		ListenerList.Listener<ModelDataProcList>, PaintListener, ClientListener {

	protected Group crystalGroup;
	protected Spinner firstImgText, lastImgText;
	protected Font font = GUI.FONT_11;
	protected Combo valueGroups, softCombo;
	protected Button procButton, xdsButton, settingsButton;
	protected Combo cellCombo;
	protected static final ModelDataProc.DataProcField[] displayFields = {
			ModelDataProc.DataProcField.SAMPLE_NAME,
			ModelDataProc.DataProcField.IMAGEDIR,
			ModelDataProc.DataProcField.WORKDIR,
			ModelDataProc.DataProcField.UNIT_CELL,
			ModelDataProc.DataProcField.SPACE_GROUP, };
	protected WdgUnitCell cell = null;
	protected HashMap<DataProcField, Text> fieldTexts = new HashMap<DataProcField, Text>();
	
	private Composite logComp;
	public WdgReprocessing(Composite parent, int style) {
		super(parent, style);
		setLayout(new GridLayout(1, true));
		
		Client.addListener(this);
		
		logComp = new Composite(parent, SWT.NONE);
		logComp.setBackground(GUI.BGCOLOR);
		logComp.setLayout(new GridLayout(4, false));
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		logComp.setLayoutData(gd);
		
		createButoons();
		createParams();
	}

	protected void createButoons() {
		
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, false);
		gd.horizontalSpan = 4;
		
		Label warn = new Label(logComp, SWT.WRAP);
		warn.setBackground(GUI.BGCOLOR);
		warn.setForeground(GUI.COLOR_RED);
		warn.setText("Don't reprocess during data collection, it may slow down current auto porcessing.");
		warn.setLayoutData(gd);
		
		procButton = new Button(logComp, SWT.PUSH);
		procButton.setText("Reprocess");
		procButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.widthHint = 150;
		procButton.setLayoutData(gd);
		procButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				
				CmdReprocessing cmdReprocess = new CmdReprocessing();
				cmdReprocess.software = softCombo.getText().equals("fast_dp")?true:false;
				cmdReprocess.firstImg = firstImgText.getText();
				cmdReprocess.lastImg = lastImgText.getText();
				cmdReprocess.unitCell = ModelReprocessing.UNIT_CELL;
				cmdReprocess.spaceGroup = ModelReprocessing.SPACE_GROUP;
				cmdReprocess.start();
			}
		});

		xdsButton = new Button(logComp, SWT.PUSH);
		xdsButton.setText("Open xdsGUI");
		xdsButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.widthHint = 150;
		xdsButton.setLayoutData(gd);
		xdsButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				if (ModelDataProcList.get().getCurrentResult() != null) {
					String workDir = ModelDataProcList.get().getCurrentResult()
							.getField(DataProcField.WORKDIR);
					ExecTask.new_ExecTask("xdsGUI " + workDir + "", "xdsGUI",
							false, false).start();
				} else {
					Log.warn("No dataprocessing results are available");
					ExecTask.new_ExecTask("xdsGUI", "xdsGUI", false, false)
							.start();
				}
			}
		});

		String items[] = { "get CELL from Strategy", "get CELL from processing" };
		cellCombo = new Combo(logComp, SWT.PUSH);
		cellCombo.setItems(items);
		
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.widthHint = 200;
		
		cellCombo.select(0);
		cellCombo.setLayoutData(gd);
		cellCombo.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				try {
					String unitCell = "", spaceGroup = "";
					int spNo = 0;
					if (cellCombo.getSelectionIndex() == 0) {
						if (ModelStrategyResultList.get().getCurrentResult() == null) {
							Log.warn("Strategy result not available");
							return;
						}
						unitCell = ModelStrategyResultList
								.get()
								.getCurrentResult()
								.getField(
										ModelStrategyResult.StrategyField.UNIT_CELL);
						spaceGroup = ModelStrategyResultList
								.get()
								.getCurrentResult()
								.getField(
										ModelStrategyResult.StrategyField.SPACE_GROUP_SH);
						spNo = DevMultiStrategy.get().getNumberFromSpacegroup(
								spaceGroup);
						Log.warn("Unit Cell : " + unitCell + " Space Group : "
								+ spaceGroup);
					} else {
						if (ModelDataProcList.get().getCurrentResult() == null) {
							Log.warn("Processing result not available");
							return;
						}
						unitCell = ModelDataProcList
								.get()
								.getCurrentResult()
								.getField(ModelDataProc.DataProcField.UNIT_CELL);
						spaceGroup = ModelDataProcList
								.get()
								.getCurrentResult()
								.getField(
										ModelDataProc.DataProcField.SPACE_GROUP);

						spNo = DevMultiStrategy.get().getNumberFromSpacegroup(
								spaceGroup);
						Log.warn("Unit Cell : " + unitCell + " Space Group : "
								+ spaceGroup);
					}
					if (!unitCell.equals("") && !spaceGroup.equals("")) {
						cell.setUnitCell(unitCell);
						cell.setSpaceGroup(spaceGroup, spNo);
					}
				} catch (Exception e) {
					System.out.println(e.getMessage());
				}
			}
		});
	}

	protected void createParams() {
		
		GridData gd;
		Composite labelComp = new Composite(this, SWT.BORDER);
		labelComp.setBackground(GUI.BGCOLOR);
		labelComp.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 6,
				1));
		GridLayout gl = new GridLayout(6, false);
		gl.verticalSpacing = 5;
		gl.horizontalSpacing = 15;
		labelComp.setLayout(gl);
		
		for (ModelDataProc.DataProcField field : displayFields) {
			
			if (field == ModelDataProc.DataProcField.UNIT_CELL) {
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.horizontalSpan = 2;
				Label labelImgSet = new Label(labelComp, SWT.NONE);
				labelImgSet.setBackground(GUI.BGCOLOR);
				labelImgSet.setFont(font);
				labelImgSet.setText("ImageSet : ");
				labelImgSet.setLayoutData(gd);

				gd = new GridData(SWT.LEFT, SWT.CENTER, false, false);
				Label labelFirst = new Label(labelComp, SWT.NONE);
				labelFirst.setBackground(GUI.BGCOLOR);
				labelFirst.setFont(font);
				labelFirst.setText("First image");
				labelFirst.setLayoutData(gd);

				gd = new GridData(SWT.LEFT, SWT.CENTER, false, false);
				gd.widthHint = 120;
				firstImgText = new Spinner(labelComp, SWT.BORDER);
				firstImgText.setIncrement(1);
				firstImgText.setMaximum(99999);
				firstImgText.setMinimum(1);
				firstImgText.setLayoutData(gd);

				gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
				Label labelLast = new Label(labelComp, SWT.NONE);
				labelLast.setBackground(GUI.BGCOLOR);
				labelLast.setFont(font);
				labelLast.setText("Last image");
				labelLast.setLayoutData(gd);

				gd = new GridData(SWT.LEFT, SWT.CENTER, false, false);
				gd.widthHint = 120;
				lastImgText = new Spinner(labelComp, SWT.BORDER);
				lastImgText.setIncrement(1);
				lastImgText.setMaximum(99999);
				lastImgText.setMinimum(1);
				lastImgText.setLayoutData(gd);

				Label lblSoftware = new Label(labelComp, SWT.NONE);
				lblSoftware.setText("Software : ");
				lblSoftware.setFont(font);
				lblSoftware.setBackground(GUI.BGCOLOR);
				gd = new GridData(SWT.FILL, SWT.CENTER, true, true);
				gd.horizontalSpan = 2;
				lblSoftware.setLayoutData(gd);

				gd = new GridData(SWT.LEFT, SWT.DEFAULT, true, false);
				gd.horizontalSpan = 4;
				gd.grabExcessHorizontalSpace = true;
				gd.widthHint = 150;
				String items[] = { "fast_dp", "GMCAProc" };

				softCombo = new Combo(labelComp, SWT.READ_ONLY);
				softCombo.setLayoutData(gd);
				softCombo.setItems(items);
				softCombo.select(1);
				softCombo.addSelectionListener(new SelectionListener() {
					@Override
					public void widgetDefaultSelected(SelectionEvent se) {
					}

					@Override
					public void widgetSelected(SelectionEvent se) {
						updateLabels();
					}
				});
				
				/*  build cell parameters defined in MultiCrystalStrategy */
				gd = new GridData(SWT.FILL, SWT.FILL, true, true);
				gd.horizontalSpan = 4;
				cell = new WdgUnitCell(this);
				cell.setLayoutData(gd);
				
				break;
			}
			if (field.getDisplayLines() > 0) {
				gd = new GridData(SWT.FILL, SWT.CENTER, true, true);
				gd.horizontalSpan = 2;
				gd.minimumWidth = 150;
				Label nameLabel = new Label(labelComp, SWT.NONE);
				nameLabel.setBackground(GUI.BGCOLOR);
				nameLabel.setFont(font);
				nameLabel.setText(null == field ? "" : field.getUserString()
						+ ":");
				nameLabel.setLayoutData(gd);
			
				Text valueText = (field.getDisplayLines() > 2) ? new Text(
						labelComp, SWT.WRAP | SWT.V_SCROLL | SWT.MULTI
								| SWT.BORDER) : new Text(labelComp, SWT.WRAP
						| SWT.MULTI);

				valueText.setFont(font);
				valueText.setBackground(GUI.BGCOLOR);
				gd = new GridData(SWT.FILL, SWT.CENTER, true, true);
				if (field.getDisplayLines() > 1)
					gd.horizontalSpan = 4;
				if (field.getDisplayLines() > 2) {
					GC gc = new GC(labelComp);
					gc.setFont(font);
					int textHeight = gc.textExtent("1234567890").y;
					gd.heightHint = textHeight * field.getDisplayLines();
				}

				valueText.setEditable(false);
				valueText.setLayoutData(gd);

				if (null != field)
					fieldTexts.put(field, valueText);
				
				//valueText.setBackground(GUI.BGCOLOR);
			}
		}
		
		addPaintListener(this);
	}

	public void connectListeners() {
		//Log.note("Connection Listeners");
		ModelDataProcList.get().ll.addListener(this);
		// updateLabels();
	}

	protected void updateLabels() {
		GUI.asyncExec(new Runnable() {
			public void run() {
				ModelDataProcList resultList = ModelDataProcList.get();

				// position in list
				int listSize = resultList.getDataProcList().size();
				
				// fields
				ModelDataProc currentResult = resultList.getCurrentResult();
				
				
				if (null != currentResult) {
					
					String sampleName = currentResult
							.getField(ModelDataProc.DataProcField.SAMPLE_NAME);
					
					if(sampleName.endsWith("mcs")) {
						return;
					}
					
					for (DataProcField field : currentResult.getAllFields()
							.keySet()) {
						 
						Text text = fieldTexts.get(field);

						if (null != text) {
							String value = currentResult.getField(field);

							if (null == value) {
								text.setText("---");
							} else {
								if (!value.equals("")) {
									if(field == ModelDataProc.DataProcField.WORKDIR) {
										text.setText(ModelReprocessing.getProcDir(softCombo.getText()));
									} else {
										text.setText(value);
									}
								}
								else
									text.setText("---");
							}
						}
					}
					try {
						String unitCell = currentResult
								.getField(ModelDataProc.DataProcField.UNIT_CELL);
						cell.setUnitCell(unitCell);
						String spaceGroup = currentResult
								.getField(ModelDataProc.DataProcField.SPACE_GROUP);
						int spNo = DevMultiStrategy.get()
								.getNumberFromSpacegroup(spaceGroup);
						cell.setSpaceGroup(spaceGroup, spNo);
						cell._setResolution(
								Double.parseDouble(currentResult
										.getField(ModelDataProc.DataProcField.LOWRESOLUTION)),
								Double.parseDouble(currentResult
										.getField(ModelDataProc.DataProcField.HIGHRESOLUTION)));

						String soft = currentResult
								.getField(ModelDataProc.DataProcField.SOFTWARE);
						String imageSet = currentResult
								.getField(ModelDataProc.DataProcField.IMAGE_SET);
						String[] image = imageSet.split("_");
						String[] image1;
						if (image[0].equals("Final")) {
							image1 = image[1].split("-");
							firstImgText.setSelection(Integer
									.parseInt(image1[0]));
							lastImgText.setSelection(Integer
									.parseInt(image1[1]));
						} else {
							image1 = image[0].split("-");
							firstImgText.setSelection(Integer
									.parseInt(image1[0]));
							lastImgText.setSelection(Integer
									.parseInt(image1[1]));
						}
					} catch (Exception e) {
						System.out.println(e.getMessage());
					}
				}

			}
		});
		updateEnabled();
	}

	protected void updateEnabled()
	{
		ModelDataProc currentResult = ModelDataProcList.get().getCurrentResult();
		final DataProcState state = (null == currentResult) ? null : currentResult.getState();
		
		GUI.asyncExec(new Runnable(){public void run()
		{
    		boolean enableButtons = (null != state && state == DataProcState.DONE);
    		boolean isMaster = Client.isMaster();
    		cellCombo.setEnabled(isMaster);
    		procButton.setEnabled(isMaster);
    		xdsButton.setEnabled(isMaster);
    		//valueGroups.setEnabled(isMaster);
    		softCombo.setEnabled(isMaster);
    		//settingsButton.setEnabled(isMaster);
    	
		}});

	}

	@Override
	public void handleEvent(ModelDataProcList object, Object data) {
		updateLabels();
	}

	@Override
	public void paintControl(PaintEvent e) {

	}

	@Override
	public void masterChanged(boolean isMaster) {
		updateEnabled();
	}
}
