/**
 * @author spothineni
 * 
 * Units cell GUI,
 * 
 * this class should be further simplified by just reading unitcell in one text box
 * instead of muliptl text boxes
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.multistrategy.DevMultiStrategy;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.StringTokenizer;

public class WdgUnitCell extends Composite implements
		ListenerList.Listener<ModelReprocessing>, PaintListener, ClientListener {

	protected String unitCell = "";
	protected String spaceGroup = "";

	protected int spaceGroupNo = 1;
	protected Label strSpaceGroup;

	protected Composite expandComp;
	protected Group crystalGroup;
	protected Text spaceGroupText, lowResLimitEntry, highResLimitEntry;
	protected Button multiCrystButton, xdsfileBrowse, runXDSButton,
			btnDefaultRes;
	protected Text[] unitCellText;
	protected Font font = GUI.FONT_11;

	public WdgUnitCell(Composite parent) {
		super(parent, SWT.BORDER);
		setBackground(GUI.BGCOLOR);
		setLayout(new GridLayout(1, false));

		expandComp = new Composite(this, SWT.NONE);
		expandComp.setBackground(GUI.BGCOLOR);
		GridLayout gl1 = new GridLayout(7, false);
		expandComp.setLayout(gl1);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, false);
		expandComp.setLayoutData(gd);

		createCell();
	}

	protected void createCell() {

		crystalGroup = new Group(expandComp, SWT.NONE);
		crystalGroup.setBackground(GUI.BGCOLOR);
		GridLayout actionGrid = new GridLayout(13, false);
		crystalGroup.setLayout(actionGrid);

		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 7;

		crystalGroup.setLayoutData(gd);
		crystalGroup.setFont(GUI.FONT_COLLECT_GROUP);
		crystalGroup.setText("Cell Parameters");

		Label lblUnitcell = new Label(crystalGroup, SWT.NONE);
		lblUnitcell.setText("Unit cell : ");
		lblUnitcell.setFont(font);
		lblUnitcell.setBackground(GUI.BGCOLOR);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 4;
		lblUnitcell.setLayoutData(gd);

		unitCellText = new Text[ModelReprocessing.UnitCell.length];
		for (int i = 0; i < ModelReprocessing.UnitCell.length; i++) {

			gd = new GridData(SWT.RIGHT, SWT.DEFAULT, true, false);
			Label labelCell = new Label(crystalGroup, SWT.NONE);
			labelCell.setBackground(GUI.BGCOLOR);
			labelCell.setText(ModelReprocessing.UnitCell[i]);
			labelCell.setLayoutData(gd);

			gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
			gd.horizontalSpan = 2;

			unitCellText[i] = new Text(crystalGroup, SWT.BORDER);
			unitCellText[i].setLayoutData(gd);

			unitCellText[i].addListener(SWT.Verify, new Listener() {
				public void handleEvent(Event e) {
					Util.allowOnlyAlpha(e);
				}
			});

			unitCellText[i].addFocusListener(new FocusListener() {
				@Override
				public void focusGained(FocusEvent arg0) {
				}

				public void focusLost(FocusEvent arg0) {
					setUnitCell();
				}
			});
			unitCellText[i].addKeyListener(new KeyListener() {
				@Override
				public void keyPressed(KeyEvent arg0) {
				}

				public void keyReleased(KeyEvent arg0) {
					if (arg0.keyCode == SWT.CR) {
						setUnitCell();
					}
				}
			});
			if (i == 2) {
				Label lblBlanck = new Label(crystalGroup, SWT.NONE);
				lblBlanck.setText("          ");
				lblBlanck.setBackground(GUI.BGCOLOR);
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.horizontalSpan = 4;
				lblBlanck.setLayoutData(gd);
			}
		}

		gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		gd.horizontalSpan = 4;
		Label labelSpaceGroup = new Label(crystalGroup, SWT.NONE);
		labelSpaceGroup.setBackground(GUI.BGCOLOR);
		labelSpaceGroup.setFont(font);
		labelSpaceGroup.setText("Space Group Number : ");
		labelSpaceGroup.setLayoutData(gd);

		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.widthHint = 70;

		spaceGroupText = new Text(crystalGroup, SWT.BORDER);
		spaceGroupText.setLayoutData(gd);

		spaceGroupText.addListener(SWT.Verify, new Listener() {
			public void handleEvent(Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		spaceGroupText.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent arg0) {
			}

			public void focusLost(FocusEvent arg0) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						String spaceGroup = spaceGroupText.getText();
						setSpaceGroup(spaceGroup);
					}
				});
			}
		});
		spaceGroupText.addKeyListener(new KeyListener() {
			@Override
			public void keyPressed(KeyEvent arg0) {
			}

			public void keyReleased(KeyEvent arg0) {
				if (arg0.keyCode == SWT.CR) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							String spaceGroup = spaceGroupText.getText();
							setSpaceGroup(spaceGroup);
						}
					});
				}
			}
		});

		gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		gd.horizontalSpan = 2;
		strSpaceGroup = new Label(crystalGroup, SWT.NONE);
		strSpaceGroup.setBackground(GUI.BGCOLOR);
		strSpaceGroup.setFont(font);
		strSpaceGroup.setLayoutData(gd);

		gd = new GridData(SWT.FILL, SWT.CENTER, true, true);
		gd.horizontalSpan = 6;
		Link link = new Link(crystalGroup, SWT.NONE);
		link.setBackground(crystalGroup.getBackground());
		link.setText("<a>Show space group numbers</a>");
		link.setLayoutData(gd);

		link.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			@Override
			public void widgetSelected(SelectionEvent arg0) {
				String filename = Main.getAppDir()+"/docs/spacegroupnumbers.htm";
				ExecTask.new_ExecTask("firefox " + filename + "", "firefox",
						false, false).start();
			}
		});

		gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		gd.horizontalSpan = 4;
		Label lblResLimit = new Label(crystalGroup, SWT.NONE);
		lblResLimit.setText("Res. Limit:");
		lblResLimit.setFont(font);
		lblResLimit.setBackground(GUI.BGCOLOR);
		lblResLimit.setLayoutData(gd);
		lblResLimit
				.setToolTipText("Resolution limits for processing by DISTL.signal_strength");

		/*
		 * gd = new GridData(SWT.FILL, SWT.CENTER, false, false);
		 * gd.horizontalSpan = 9; Composite resLimitsComp = new
		 * Composite(crystalGroup, SWT.BORDER); resLimitsComp.setLayoutData(gd);
		 * resLimitsComp.setBackground(GUI.BGCOLOR); GridLayout gl = new
		 * GridLayout(12,false); gl.marginHeight=0; gl.marginWidth=0;
		 * gl.horizontalSpacing=0; resLimitsComp.setLayout(gl);
		 */

		gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
		Label lowLabel = new Label(crystalGroup, SWT.NONE);
		lowLabel.setText("L");
		lowLabel.setBackground(GUI.BGCOLOR);
		lowLabel.setLayoutData(gd);

		gd = new GridData(SWT.LEFT, SWT.CENTER, true, false);
		gd.horizontalSpan = 3;
		gd.widthHint = 70;

		lowResLimitEntry = new Text(crystalGroup, SWT.BORDER);
		lowResLimitEntry.setLayoutData(gd);

		lowResLimitEntry.addListener(SWT.Verify, new Listener() {
			public void handleEvent(Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		lowResLimitEntry.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent arg0) {
			}

			public void focusLost(FocusEvent arg0) {
				setResolution();
			}
		});
		lowResLimitEntry.addKeyListener(new KeyListener() {
			@Override
			public void keyPressed(KeyEvent arg0) {
			}

			public void keyReleased(KeyEvent arg0) {
				if (arg0.keyCode == SWT.CR) {
					setResolution();
				}
			}
		});

		gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
		Label highLabel = new Label(crystalGroup, SWT.NONE);
		highLabel.setText("H");
		highLabel.setBackground(GUI.BGCOLOR);
		highLabel.setLayoutData(gd);

		gd = new GridData(SWT.LEFT, SWT.CENTER, false, false);
		gd.horizontalSpan = 3;
		gd.widthHint = 70;

		highResLimitEntry = new Text(crystalGroup, SWT.BORDER);
		highResLimitEntry.setLayoutData(gd);

		highResLimitEntry.addListener(SWT.Verify, new Listener() {
			public void handleEvent(Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		highResLimitEntry.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent arg0) {
			}

			public void focusLost(FocusEvent arg0) {
				setResolution();
			}
		});
		highResLimitEntry.addKeyListener(new KeyListener() {
			@Override
			public void keyPressed(KeyEvent arg0) {
			}

			public void keyReleased(KeyEvent arg0) {
				if (arg0.keyCode == SWT.CR) {
					setResolution();
				}
			}
		});

		Label angLabel = new Label(crystalGroup, SWT.NONE);
		angLabel.setText("\u212B");
		angLabel.setBackground(GUI.BGCOLOR);
		angLabel.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false));
		
		gd = new GridData(SWT.FILL, SWT.TOP, true, false);
		gd.horizontalSpan = 4;
		
		Button settingsButton = new Button(expandComp, SWT.CHECK | SWT.BORDER);
		settingsButton.setText("Use other XDS parameters not mentioned above from Settings tab for Reprocessing");
		settingsButton.setFont(GUI.FONT_COLLECT_GROUP);
		settingsButton.setLayoutData(gd);
		settingsButton.setBackground(GUI.BGCOLOR);
		settingsButton.setSelection(true);

		settingsButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						ModelReprocessing.takeSettings = settingsButton.getSelection();
					}
				});
			}
		});
	}

	public void setSpaceGroup(String sg, int spNo) {

		if (sg.equals("") || sg == null) {
			// Log.error("Spacegroup# must be between 1 and 266");
			return;
		}
		try {
			if (spNo > 0 && spNo <= 266) {
				spaceGroupText.setText(Integer.toString(spNo));
				strSpaceGroup.setText(sg);
				ModelReprocessing.SPACE_GROUP = spNo;
			} else {
				if (spNo != 0)
					System.out.println("Spacegroup# must be between 1 and 266");
			}
		} catch (NumberFormatException ne) {
			System.out.println(ne.getMessage());
		}
	}

	public void setSpaceGroup(String sg) {
		final String spg = sg;

		if (spg.equals("") || spg == null) {
			// Log.error("Spacegroup# must be between 1 and 266");
			return;
		}
		try {
			final int spaceGroupNo = Integer.parseInt(spg);
			ModelReprocessing.SPACE_GROUP = spaceGroupNo;
			String spgStr = DevMultiStrategy.get().getSpacegroupFromNumber(
					spaceGroupNo);

			if (spaceGroupNo > 0 && spaceGroupNo <= 266) {
				Log.warn("Space Group : " + spgStr);
				strSpaceGroup.setText(spgStr);
			} else {
				// Log.error("Spacegroup# must be between 1 and 266");
			}
		} catch (NumberFormatException ne) {
			System.out.println(ne.getMessage());
		}
	}

	public void setUnitCell(String unitCell) {
		final String cell = unitCell;
		ModelReprocessing.UNIT_CELL = unitCell;

		StringTokenizer strTok = new StringTokenizer(cell);
		int i = 0;
		while (strTok.hasMoreElements()) {
			unitCellText[i].setText(strTok.nextToken());
			i++;
		}

	}

	public void setUnitCell() {
		GUI.asyncExec(new Runnable() {
			public void run() {
				String unitCell = "";
				for (int i = 0; i < ModelReprocessing.UnitCell.length; i++) {
					unitCell += unitCellText[i].getText() + " ";
				}
				StringTokenizer strTok = new StringTokenizer(unitCell);

				if (strTok.countTokens() == ModelReprocessing.UnitCell.length) {
					ModelReprocessing.UNIT_CELL = unitCell;
					System.out.println("Unit Cell changed by user " + unitCell);
				} else {
					Log.error("Unit Cell : " + unitCell + " not valid");
				}
			}
		});
	}

	public void setResolution() {
		String lowStr = lowResLimitEntry.getText();
		String highStr = highResLimitEntry.getText();

		if (lowStr.equals("") || lowStr == null || highStr.equals("")
				|| highStr == null || lowStr.equals("-") || highStr.equals("-")) {

			return;
		}

		double low = Double.parseDouble(lowStr);
		double high = Double.parseDouble(highStr);

		if (low > 50.0) {
			Log.warn("Low resolution should be less than 50.0");

			return;
		}
		if (high < 0.0) {
			Log.warn("High resolution should be greater than 0.0");

			return;
		}
		if (low < high) {
			Log.warn("Value of Low Res. > High Res");

			return;
		}

		_setResolution(low, high);
	}

	public void _setResolution(double low, double high) {

		ModelReprocessing.RESOLUTION_LOW = low;
		ModelReprocessing.RESOLUTION_HIGH = high;
		final double lowFinal = low;
		final double highFinal = high;
		lowResLimitEntry.setText(Double.toString(lowFinal));
		highResLimitEntry.setText(Double.toString(highFinal));
		//Log.note("High resolution limit " + high + " Low resolution limit "
				//+ low);
	}

	public void setImages(String first, String last) {

	}

	@Override
	public void handleEvent(ModelReprocessing object, Object data) {

	}

	@Override
	public void masterChanged(boolean isMaster) {

	}

	@Override
	public void paintControl(PaintEvent arg0) {

	}

}
