/**
 * @author spothineni
 * 
 * GUI for Settings sub-tab of Analysis tab
 */

package gov.anl.gmca.dataproc;

import gov.anl.gmca.common.ui.NBMessageBox;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.multistrategy.DevMultiStrategy;
import gov.anl.gmca.strategy.ModelStrategyResult;
import gov.anl.gmca.strategy.ModelStrategyResultList;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.EventObject;

public class WdgXDSSettings extends Composite 
			implements ClientListener, ListenerList.Listener<ModelXDSSettings> {
	
	protected TableViewer tview;
	protected Table table;
	protected TableViewerColumn col2;
	protected Button defaultButton;
	protected Button default1Button;
	protected Button settingsButton;
	protected Combo cellCombo;
	
	private Composite logComp;
	public WdgXDSSettings(Composite parent, int style) {
		
		super(parent, style);

		GridLayout gl = new GridLayout(1, true);
		gl.marginWidth = 0;
		gl.marginHeight = 0;
		gl.horizontalSpacing = 0;
		gl.verticalSpacing = 3;
		setLayout(gl);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		setBackground(GUI.BGCOLOR);
		setLayoutData(gd);
		
		logComp = new Composite(parent, SWT.NONE);
		logComp.setBackground(GUI.BGCOLOR);
		GridLayout gl1 = new GridLayout(1, true);
		gl1.marginWidth = 0;
		gl1.marginHeight = 0;
		logComp.setLayout(gl1);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		logComp.setLayoutData(gd);
		
		Client.addListener(this);
		
		createParams();
	}

	private void createParams() {
		
		new ModelXDSSettings();
		new DevXDSSettings();
		DevXDSSettings.get().verifyConnected();
		ModelXDSSettings.ll.addListener(this);
		
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, false);
		gd.horizontalSpan = 2;
		
		Label warn = new Label(logComp, SWT.WRAP);
		warn.setBackground(GUI.BGCOLOR);
		warn.setForeground(GUI.COLOR_RED);
		warn.setText("In this tab you can change the XDS default parameters for JBluIce auto processing, "
				+ "Please change only when you are obsolutely sure how the parameter "
				+ "affect the XDS processing. (Double-click on value column to change a parameter)");
		warn.setLayoutData(gd);
		
		gd = new GridData(SWT.FILL, SWT.FILL, true, false);
		gd.horizontalSpan = 2;
		Label warn1 = new Label(logComp, SWT.WRAP);
		warn1.setBackground(GUI.BGCOLOR);
		warn1.setForeground(GUI.COLOR_GREEN);
		warn1.setText("To reset a parameter to default value after you changed it, click on the corresponding checkbox and click Set Default button");
		warn1.setLayoutData(gd);
		
		gd = new GridData(SWT.CENTER, SWT.FILL, true, false);
		gd.horizontalSpan = 2;
		Link xdsLink = new Link(logComp, SWT.NONE);
		xdsLink.setLayoutData(gd);
		xdsLink.setText("<a href=\"http://xds.mpimf-heidelberg.mpg.de/html_doc/xds_parameters.html\">"
				+ "Click here to see the description  of XDS parameters here</a>");
		
		xdsLink.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			@Override
			public void widgetSelected(SelectionEvent arg0) {
				String filename = "http://xds.mpimf-heidelberg.mpg.de/html_doc/xds_parameters.html";
				ExecTask.new_ExecTask("firefox " + filename + "", "firefox",
						false, false).start();
			}
		});
		
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 2;
		
		/* define the TableViewer */
		tview = new TableViewer(logComp, SWT.CHECK | SWT.MULTI | SWT.H_SCROLL
		      | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER);
		createColumns(logComp, tview);
		
		final Table table = tview.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		table.setLayoutData(gd);
		
		tview.setContentProvider(new ArrayContentProvider());
		tview.setInput(ModelXDSSettings.params);
		
		gd = new GridData(SWT.FILL, SWT.FILL, true, false);
		gd.horizontalSpan = 2;
		
		settingsButton = new Button(logComp, SWT.CHECK | SWT.BORDER);
		settingsButton.setText("Use above XDS parameters for auto-processing");
		settingsButton.setFont(GUI.FONT_COLLECT_GROUP);
		settingsButton.setLayoutData(gd);
		settingsButton.setBackground(GUI.BGCOLOR);
		settingsButton.setSelection(true);

		settingsButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						ModelXDSSettings.takeXDSSettings = settingsButton.getSelection();
						
					}
				});
			}
		});
		
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		
		default1Button = new Button(logComp, SWT.PUSH);
		default1Button.setText("Set Defaults");
		default1Button.setEnabled(false);
		default1Button.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.widthHint = 150;
		default1Button.setLayoutData(gd);
		default1Button.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				try {
					int changed = ModelXDSSettings.params.size();
					for (int i=0;i<changed;i++) {
						TableItem item = table.getItem(i);
						if(item.getChecked()) {
							
							String param = item.getText(0);
							for(ModelXDSSettings.DefaultParam pa : ModelXDSSettings.params) {
								if(pa.getXDSParam().equals(param)) {
									Log.warn("XDS parameter "+pa.getXDSParam() +" value reset to "+ pa.getValue());
									ModelXDSSettings.get().setXDSParamvalue(pa, pa.getValue());
								}
							}
							item.setChecked(false);
							tview.setInput(ModelXDSSettings.params);
						}
					}
					//tview.setInput(ModelXDSSettings.params);
				}catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		
		defaultButton = new Button(logComp, SWT.PUSH);
		defaultButton.setText("Set All Defaults");
		defaultButton.setEnabled(false);
		defaultButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.widthHint = 150;
		defaultButton.setLayoutData(gd);
		defaultButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				
				NBMessageBox messageBox = new NBMessageBox(GUI.getShell(),
						SWT.OK | SWT.CANCEL | SWT.ICON_WARNING);
				messageBox
						.setMessage("All the previously set XDS parameters will reset to defaults,\n"
								+ "Please click OK only when you need it");
				if (messageBox.open() != SWT.OK) {
					return;
				}
				
				ModelXDSSettings.get().resetXDSParameters();
			}
		});
		
		String items[] = { "get CELL from Strategy", "get CELL from processing" };
		cellCombo = new Combo(logComp, SWT.PUSH);
		cellCombo.setItems(items);
		
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 2;
		gd.widthHint = 200;
		
		cellCombo.select(0);
		cellCombo.setLayoutData(gd);
		cellCombo.setEnabled(false);
		cellCombo.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				try {
					String unitCell = "", spaceGroup = "";
					int spNo = 0;
					if (cellCombo.getSelectionIndex() == 0) {
						if (ModelStrategyResultList.get().getCurrentResult() == null) {
							Log.warn("Strategy result not available");
							return;
						}
						unitCell = ModelStrategyResultList
								.get()
								.getCurrentResult()
								.getField(
										ModelStrategyResult.StrategyField.UNIT_CELL);
						spaceGroup = ModelStrategyResultList
								.get()
								.getCurrentResult()
								.getField(
										ModelStrategyResult.StrategyField.SPACE_GROUP_SH);
						spNo = DevMultiStrategy.get().getNumberFromSpacegroup(
								spaceGroup);
						Log.warn("Unit Cell : " + unitCell + " Space Group : "
								+ spaceGroup);
					} else {
						if (ModelDataProcList.get().getCurrentResult() == null) {
							Log.warn("Processing result not available");
							return;
						}
						unitCell = ModelDataProcList
								.get()
								.getCurrentResult()
								.getField(ModelDataProc.DataProcField.UNIT_CELL);
						spaceGroup = ModelDataProcList
								.get()
								.getCurrentResult()
								.getField(
										ModelDataProc.DataProcField.SPACE_GROUP);

						spNo = DevMultiStrategy.get().getNumberFromSpacegroup(
								spaceGroup);
						Log.warn("Unit Cell : " + unitCell + " Space Group : "
								+ spaceGroup);
					}
					if (!unitCell.equals("") && !spaceGroup.equals("")) {
						ModelXDSSettings.get().setXDSParamvalue(ModelXDSSettings.DefaultParam.UNIT_CELL_CONSTANTS, unitCell);
						ModelXDSSettings.get().setXDSParamvalue(ModelXDSSettings.DefaultParam.SPACE_GROUP_NUMBER, Integer.toString(spNo));
						tview.setInput(ModelXDSSettings.params);
					}
				} catch (Exception e) {
					System.out.println(e.getMessage());
				}
			}
		});
	}
	private void createColumns (final Composite parent, final TableViewer viewer) {
		String columnNames[] = {"XDS Parameter", "Value"};
		int[] bounds = {400, 100};
		
		/* setting columns */
	    TableViewerColumn col1 = createTableViewerColumn(columnNames[0], bounds[0], 0);
	    col1.setLabelProvider(new CellLabelProvider() {
	        @Override
	        public void update(ViewerCell cell) {
	          cell.setText(((ModelXDSSettings.DefaultParam)cell.getElement()).getXDSParam());
	        }
	      });
	    
	   col2 = createTableViewerColumn(columnNames[1], bounds[1], 1);
	   col2.setLabelProvider(new CellLabelProvider() {
	        @Override
	        public void update(ViewerCell cell) {
	          
	          ModelXDSSettings.DefaultParam param= (ModelXDSSettings.DefaultParam)cell.getElement();
	          String val = ModelXDSSettings.paramValues.get(param);
	           
	          cell.setText(val);
	         
	          if(val!= null && !val.equals(param.getValue())) {
	        	 cell.setBackground(GUI.COLOR_GREEN);
	          } else {
	        	 cell.setBackground(null);
	          }
	  
	  
	        }
	    });
	   
	   /* make values column editable */
	   col2.setEditingSupport(new EditingSupport(viewer) {

           @Override
           protected void setValue(Object element, Object value) {
        	   String val = (String)value;
        	   if(val.equals(" ") || val.equals(""))
        		   return;
        	   ModelXDSSettings.get().setXDSParamvalue(((ModelXDSSettings.DefaultParam) element), (String)value);
        	   viewer.update(element, null);

           }

           @Override
           protected Object getValue(Object element) {
        	   return (ModelXDSSettings.paramValues.get((ModelXDSSettings.DefaultParam) element));
           }

           @Override
           protected CellEditor getCellEditor(Object element) {
               TextCellEditor editor = new TextCellEditor(viewer.getTable());
        	   return editor;
           }

           @Override
           protected boolean canEdit(Object element) {
        	   if(Client.isMaster())
        		   return true;
        	   else
        		   return false;
           }
       });
	   TableViewerFocusCellManager focusCellManager = new TableViewerFocusCellManager(viewer, new FocusCellOwnerDrawHighlighter(viewer));

	   ColumnViewerEditorActivationStrategy activationSupport = new ColumnViewerEditorActivationStrategy(viewer) {
		    protected boolean isEditorActivationEvent(ColumnViewerEditorActivationEvent event) {
		        // Enable editor only with mouse double click
		        if (event.eventType == ColumnViewerEditorActivationEvent.MOUSE_DOUBLE_CLICK_SELECTION) {
		            EventObject source = event.sourceEvent;
		            if (source instanceof MouseEvent && ((MouseEvent)source).button == 3)
		                return false;

		            return true;
		        }

		        return false;
		    }
		};
		TableViewerEditor.create(viewer, focusCellManager, activationSupport, ColumnViewerEditor.TABBING_HORIZONTAL | 
			    ColumnViewerEditor.TABBING_MOVE_TO_ROW_NEIGHBOR | 
			    ColumnViewerEditor.TABBING_VERTICAL |
			    ColumnViewerEditor.KEYBOARD_ACTIVATION);
	}
	private TableViewerColumn createTableViewerColumn(String title, int bound, final int colNumber) {
	    final TableViewerColumn viewerColumn = new TableViewerColumn(tview,
	        SWT.NONE);
	    final TableColumn column = viewerColumn.getColumn();
	    column.setText(title);
	    column.setWidth(bound);
	    column.setResizable(true);
	    column.setMoveable(true);
	    return viewerColumn;
	}
	@Override
	public void masterChanged(boolean isMaster) {
		
		GUI.asyncExec(new Runnable() {public void run() {
				//defaultButton.setEnabled(Client.isMaster());
				default1Button.setEnabled(Client.isMaster());
				cellCombo.setEnabled(Client.isMaster());
				
		}});

	}

	@Override
	public void handleEvent(ModelXDSSettings object, Object data) {
		GUI.asyncExec(new Runnable() {public void run() {
			tview.setInput(ModelXDSSettings.params);
		}});
	}
}
