/**
 * This class created a script for combing multiple datasets 
 * exported to CombineDatasets tab and runs XSCALE.
 * 
 * The script basically generates a XSCALE.INP and runs xscale_par command
 */
package gov.anl.gmca.dataproc.combinedatasets;

import gov.anl.gmca.common.Command;
import gov.anl.gmca.dataproc.DevDataProc;
import gov.anl.gmca.dataproc.ModelAimlessSummary;
import gov.anl.gmca.dataproc.ModelDataProcParams;
import gov.anl.gmca.dataproc.combinedatasets.ModelCombineDatasets.HKLDataset;
import gov.anl.gmca.diffimage.MarImageHeader;
import gov.anl.gmca.gridengine.CheckSubmitHost;
import gov.anl.gmca.gridengine.DRMAA;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.multistrategy.ModelReferenceData;
import gov.anl.gmca.syslog.Log;

import java.io.*;
import java.util.HashMap;

public class CmdCombineDatasets extends Command {

	public String workDir = "";
	public String datasets = "";
	ExecTask task;
	int startFrameLabel1;
	int startFrameLabel2;

	protected static MarImageHeader currentHeader = null;

	private DevCombineDatasets devCombineDatasets = new DevCombineDatasets();
	private ModelDataProcParams dataprocResult;
	private DevDataProc devDataProc = new DevDataProc();
	@Override
	public void run() {

		ModelCombineDatasets.DIR = this.workDir;
		ModelCombineDatasets.STATE = "PROCESSING";
		dataprocResult = new ModelDataProcParams();
		ModelAimlessSummary modelAimlessSummary = new ModelAimlessSummary();
		
		File dirFile = new File(this.workDir);
		if (!dirFile.exists()) {

			if (!dirFile.mkdirs()) {
				Log.error("Couldn't create directory " + workDir);
				return;
			}
		}
		ModelCombineDatasets.RESULTFILE = "";
		ModelCombineDatasets.WARNING = "Merging with XSCALE...";

		devCombineDatasets.setResult(true);
		
		int sampNumber = DevDataProc.getRowCount() + 1;
		dataprocResult.workDir = workDir;
		dataprocResult.state = "PROCESSING";
		dataprocResult.log =  workDir+"/"+"XSCALE/XSCALE.LOG";
		dataprocResult.imageSet = "Combining multiple datasets";
		dataprocResult.sampleName = "mcs";
		dataprocResult.scalaLog = workDir+"/"+"POINTLESS_AIMLESS/mcs_aimless.html";
		
		for (int i = 0; i < ModelCombineDatasets.listHKLSets.size(); i++) {
			HashMap<HKLDataset, String> field = ModelCombineDatasets.get().listHKLSets.get(i);
			String hklinfile = field.get(HKLDataset.HKLFILE);
			int selected = Integer.parseInt(field.get(HKLDataset.SELECT));
			
			if(selected == 0) {
				continue;
			}
			
			int lastslash = hklinfile.lastIndexOf('/');
			int lastslash1 = 0,lastslash2 = 0,lastslash3 = 0;
			if(lastslash > 1) {
				lastslash1 = hklinfile.substring(0, lastslash).lastIndexOf("/");
			}
			if(lastslash1 > 1) {
				lastslash2 = hklinfile.substring(0, lastslash1).lastIndexOf("/");
			}
			if(lastslash2 > 1) {
				lastslash3 = hklinfile.substring(0, lastslash2).lastIndexOf("/");
			}
			if(lastslash3 > 1 && lastslash1 > 1) {
				if(i==0) {
					dataprocResult.images += hklinfile.substring(lastslash3, lastslash1);
				} else {
					dataprocResult.images += ", "+hklinfile.substring(lastslash3, lastslash1);
				}
			}
		}
		
		/*dataprocResult.images = ModelCombineDatasets.get().listHKLSets
				.get(0).get(HKLDataset.HKLFILE);*/
		
		devDataProc.setResult(sampNumber, dataprocResult, true);
		
		/* process XSCALE */
		try {

			BufferedWriter runxdsOut = new BufferedWriter(new FileWriter(
					workDir + "/" + "run_combinedatasets.sh", false));
			runxdsOut.write("#!/bin/bash -f \n");
			runxdsOut
					.write("# Running XSCALE : works only on GM/CA @ APS computing environment\n");

			runxdsOut.write("cd " + workDir + "\n");
			runxdsOut.write("mkdir XSCALE\n");
			runxdsOut.write("cd XSCALE\n");
			runxdsOut.write("echo \"OUTPUT_FILE= XDS_ASCII.AHKL"
					+ "\" > XSCALE.INP\n");
			runxdsOut.write("echo \"MERGE= FALSE\" >> XSCALE.INP\n");
			runxdsOut.write("echo \"FRIEDEL'S_LAW= FALSE\" >> XSCALE.INP\n");

			String infile;
			String isScala = "aimless";
		
			for (int i = 0; i < ModelCombineDatasets.listHKLSets.size(); i++) {

				HashMap<HKLDataset, String> field = ModelCombineDatasets.get().listHKLSets
						.get(i);
				if (!field.get(HKLDataset.SELECT).equals("1"))
					continue;
				infile = field.get(HKLDataset.HKLFILE);
				runxdsOut.write("set timestamp=`date +%d-%m-%Y-%k-%M-%S`\n");
				runxdsOut
						.write("mv XDS_ASCII.AHKL XDS_ASCII.AHKL_$timestamp\n");
				runxdsOut.write("mv " + isScala + ".log " + isScala
						+ ".log_$timestamp\n");
				runxdsOut.write("rm -rf " + i + ".HKL\n");
				runxdsOut.write("ln -s " + infile + " " + i + ".HKL\n");
				runxdsOut.write("echo \"INPUT_FILE= " + i + ".HKL"
						+ "\" >> XSCALE.INP\n");
				runxdsOut.write("echo \"INCLUDE_RESOLUTION_RANGE= "
						+ ModelCombineDatasets.LowRes + " "
						+ ModelCombineDatasets.HighRes + "\" >> XSCALE.INP\n");
			}
			runxdsOut.write("/mnt/software/px64/XDS/xscale_par > XSCALE.LOG\n");

			runxdsOut.write("cp -p XDS_ASCII.AHKL XDS_ASCII.HKL"
					+ " >& /dev/null\n");
			runxdsOut.write(Main.getAppDir()
					+ "/tools/scripts/multistrategy/scale.csh " + workDir + " "
					+ isScala + " mcs\n");
			runxdsOut.write("rm -rf " + workDir + "/XDS_ASCII.HKL"
					+ " >& /dev/null\n");
			runxdsOut.close();

			File setExec = new File(workDir + "/" + "run_combinedatasets.sh");
			setExec.setExecutable(true);

		} catch (IOException e1) {
			e1.printStackTrace();
		}

		int jobStatus = 0;
		boolean isSubmitHost = CheckSubmitHost.get().isSubmitHost();
		
		System.out.println("Gridengine env detected: "+isSubmitHost);
		if (!isSubmitHost) {

			executeLocally();

		} else {
			try {
				Log.note("XSCALE Merging in progress, check the dir. "
						+ workDir);
				DRMAA drmaa = null;
				boolean skip = true;
				try {
					drmaa = new DRMAA(workDir + "/" + "run_combinedatasets.sh", workDir,
							300);
				} catch (UnsatisfiedLinkError e) {
					e.printStackTrace();
					executeLocally();
					skip = false;
				}
				catch (Exception e) {
					
					exitWithError();
					e.printStackTrace();
					return;
				} // wait 300 sec before quitting job
				if(skip) {
					drmaa.start();
					jobStatus = drmaa.waitFor();
				}
			} catch (Exception e1) {
				exitWithError();
				e1.printStackTrace();
			}
		}

		try {
			processResult(workDir);
		} catch (Exception e) {
			e.printStackTrace();
		}

		ModelCombineDatasets.RESULTFILE = this.workDir + "/XSCALE/XDS_ASCII.AHKL";

		if (ModelCombineDatasets.WARNING.equals("Merging with XSCALE..."))
			ModelCombineDatasets.WARNING = "DONE";
		if (ModelCombineDatasets.STATE.equals("PROCESSING"))
			ModelCombineDatasets.STATE = "DONE";
		
		
		if (dataprocResult.warning.contains("Processing") || dataprocResult.warning.contains("processing"))
			dataprocResult.warning = "DONE";
		if (dataprocResult.state.equals("PROCESSING"))
			dataprocResult.state = "DONE";
		
		dataprocResult.subdir = workDir + "/POINTLESS_AIMLESS";
		
		/* read aimless log of combined files */
		try {
			modelAimlessSummary.processResult(workDir, dataprocResult);
		} catch (Exception e) {
				e.printStackTrace();
		}
		
		devDataProc.setDataProcResult(sampNumber, dataprocResult,
				"Combining multiple datasets", true);
		
		/* load to ReferenceData as next crystal */
		ModelReferenceData.loadHKLFile(ModelCombineDatasets.RESULTFILE, true);
		/*
		 * ModelReferenceData refData = new ModelReferenceData(); refData.PATH =
		 * ModelCombineDatasets.RESULTFILE; refData.isHighRes = true;
		 * refData.start();
		 */

		devCombineDatasets.setResult(true);
		Log.note("Multiple Datasets Merging finished");
	}
	private void executeLocally() {
		Log.warn("Grid Engine not available, strategy job running locally");
		task = ExecTask.new_ExecTask("bash " + workDir + "/"
				+ "run_combinedatasets.sh " + workDir, "Multi Crystal strategy :",
				false, false);
		task.start();
		task.waitFor();
	}
	private void exitWithError() {
		ModelCombineDatasets.WARNING = "GridEngine/DRMAA exception";
		ModelCombineDatasets.STATE = "ERROR";
		/* load to ReferenceData as next crystal */
		ModelReferenceData.loadHKLFile(ModelCombineDatasets.RESULTFILE, true);
	}
	private void processResult(String workDir2) {
		
		String xscaleLog = workDir + "/XSCALE/XSCALE.LOG";

		File xscaleFile = new File(xscaleLog);
		if (!xscaleFile.exists()) {
			Log.error(xscaleLog + " not found");
			ModelCombineDatasets.STATE = "ERROR";
			ModelCombineDatasets.WARNING = xscaleFile
					+ " doesnt exists check data!!!";
			return;
		} else {
			String xdsContent = readLogFile(xscaleLog);

			if (xdsContent.contains("!!! ERROR ")) {
				int pos1 = xdsContent.lastIndexOf("!!! ERROR ");
				ModelCombineDatasets.WARNING = xdsContent.substring(pos1,
						xdsContent.indexOf("\n", pos1));
				ModelCombineDatasets.STATE = "ERROR";
				return;
			}
		}
	}

	public static String readLogFile(String logFile) {

		byte[] buffer = new byte[(int) new File(logFile).length()];
		BufferedInputStream f = null;
		try {
			f = new BufferedInputStream(new FileInputStream(logFile));
			f.read(buffer);
			f.close();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			if (f != null)
				try {
					f.close();
				} catch (IOException ignored) {
				}
		}

		return new String(buffer);
	}
}
