/**
 * Model for CombineDatasets tab
 */

package gov.anl.gmca.dataproc.combinedatasets;

import gov.anl.gmca.common.ui.NBMessageBox;
import gov.anl.gmca.dataproc.ModelDataProc.DataProcField;
import gov.anl.gmca.dataproc.ModelDataProcList;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.multistrategy.DevMultiStrategy;
import gov.anl.gmca.multistrategy.ModelMultiStrategyParams;
import gov.anl.gmca.multistrategy.ModelReferenceData;
import gov.anl.gmca.strategy.ModelStrategyResult.StrategyField;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.apache.http.entity.mime.content.ContentBody;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;

import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;

public class ModelCombineDatasets implements IStructuredContentProvider,
		ITableLabelProvider, ContentBody {

	public ListenerList<ModelCombineDatasets> ll;;
	protected static ModelCombineDatasets inst = null;

	public enum HKLDataset {
					SELECT				("Selection", 		 50), 
					HKLFILE				("HKLFile", 		 300), 
					RESOLUTION			("Resolution",		 80), 
					SPACEGROUP			("Spacegroup", 		 90),
					UNIT_CELL 			("Unitcell", 		 300),
					COMPLETE			("Completeness",	 90),
					RMERGE				("Rmege", 			 60),
					RPIM				("Rpim", 			 50),
					CCHALF				("CChalf", 			 50),
					ISIGMA				("ISigma", 			 60),
					MULTIPLICITY		("Multiplicity",	 50),
					ANOM_COMPLETE		("AnomCompleteness", 50),
					ANOM_MULTIPLICITY	("AnomMulti", 		 50);

		protected String datasetStr;
		protected int width;

		public String getDatasetString() {
			return datasetStr;
		}

		public int getWidth() {
			return width;
		}

		HKLDataset(String datasetStr, int width) {
			this.datasetStr = datasetStr;
			this.width = width;
		}
	}
	public static HashMap<HKLDataset, String> fMap = new HashMap<HKLDataset, String>();

	public static String STATE = "";
	public static String LowRes = "50.0";
	public static String HighRes = "1.0";
	public static String WARNING = "";
	public static String DIR = "";
	public static String RESULTFILE = "";
	public static String UNIT_CELL = "";

	public static ModelCombineDatasets get() {
		return inst;
	}

	public ModelCombineDatasets() {
		inst = this;
		ll = new ListenerList<ModelCombineDatasets>(this);
	}

	protected HashMap<HKLDataset, String> fieldMap = new HashMap<HKLDataset, String>();
	public static ArrayList<HashMap<HKLDataset, String>> listHKLSets = new ArrayList<HashMap<HKLDataset, String>>();

	public void setField(HKLDataset dataset, String value) {
		fieldMap.put(dataset, value);
	}

	public String getField(HKLDataset dataset) {
		return fieldMap.get(dataset);
	}

	public HashMap<HKLDataset, String> getAllFields() {
		return fieldMap;
	}

	protected Integer resultID = null;

	public Integer getID() {
		return resultID;
	}

	public void setID(int id) {
		resultID = id;
	}

	public ModelCombineDatasets(HashMap<HKLDataset, String> fieldMap) {
		this.fieldMap = new HashMap<HKLDataset, String>(fieldMap);
		String sampleNumber = fieldMap.get(StrategyField.SAMPLE_NUMBER);
		if (null != sampleNumber) {
			try {
				resultID = Integer.parseInt(sampleNumber);
			} catch (NumberFormatException nfe) {
			}
		}
	}

	public static boolean isHKLFileExists(String hklFile) {

		for (int i = 0; i < listHKLSets.size(); i++) {

			HashMap<HKLDataset, String> field = ModelCombineDatasets.listHKLSets
					.get(i);
			if (field.get(HKLDataset.HKLFILE).equals(hklFile))
				return true;
		}
		return false;
	}

	public static void updateList(HashMap<HKLDataset, String> fieldM) {
		String selectStr = fieldM.get(ModelCombineDatasets.HKLDataset.SELECT);
		String selectHKLFile = fieldM.get(ModelCombineDatasets.HKLDataset.HKLFILE);
		int select = Integer.parseInt(selectStr);
		DevCombineDatasets.get().updateSelectRow(select, selectHKLFile);
		DevCombineDatasets.get().sendRowUpdate(0);
	}

	public void removeRow(String hklFile) {
		DevCombineDatasets.get().removeRow(hklFile);
	}

	public static void parseHKLFile(String path, String resolution) {

		if (path == null)
			return;
		if (path.equals("") || path.trim().equals("") || path == null)
			return;

		/* check if the file is XDS_ASCII.HKL */
		if (path.lastIndexOf(".") == -1) {
			Log.error("Please select XDS_ASCII.HKL");
			return;
		}
		
		String extension = path.substring(path.lastIndexOf("."), path.length());

		if (extension.contains(".sca") || extension.contains(".SCA")) {
			
			path = readScaFile(path);
			//DevReferenceData.get().setResult(true);
		}
		
		String content = ModelMultiStrategyParams.readLogFile(path, true);
		if (!content.contains("!FORMAT=XDS_ASCII")) {
			Log.error(path + " is not in XDS_ASCII.HKL file format");
			NBMessageBox messageBox = new NBMessageBox(GUI.getShell(),
					SWT.OK | SWT.ICON_ERROR);
			messageBox.setMessage(path+" \nis not in XDS_ASCII.HKL file format");
			messageBox.open();
			return;
		}
		boolean isExists = isHKLFileExists(path);
		if (isExists) {
			
			Log.error("XDS_ASCII.HKL file already in the list");
			NBMessageBox messageBox = new NBMessageBox(GUI.getShell(),
					SWT.OK | SWT.ICON_WARNING);
			messageBox.setMessage(path+" file already in the list");
			messageBox.open();
			return;
		}

		fMap.put(HKLDataset.SELECT, "1");
		fMap.put(HKLDataset.HKLFILE, path);
		/*
		 * UNIT_CELL 		("Unitcell", 100),
					COMPLETENESS	("Completeness", 50),
					RMEGRGE			("Rmege", 50),
					RPIM			("RPIM", 50),
					CCHALF			("CChalf", 50),
					ISIGMA			("ISigma", 50),
					MUMITIPLICITY	("Multiplicity", 50),
					ANOMCOMPLETENESS("AnomCompleteness", 50),
					ANOMMULTI		("AnomMulti", 50);
		 */
		if(!resolution.equals("")) {
			String unitcell = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.UNIT_CELL);
			fMap.put(HKLDataset.UNIT_CELL, unitcell);
		
			String complete = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.COMPLETE);
			fMap.put(HKLDataset.COMPLETE, complete);
		
			String rmerge = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.RMERGE);
			fMap.put(HKLDataset.RMERGE, rmerge);
		
			String rpim = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.RPIM);
			fMap.put(HKLDataset.RPIM, rpim);
		
			String isigma = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.ISIGMA);
			fMap.put(HKLDataset.ISIGMA, isigma);
		
			String cchalf = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.CCHALF);
			fMap.put(HKLDataset.CCHALF, cchalf);
		
			String multi = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.MULTIPLICITY);
			fMap.put(HKLDataset.MULTIPLICITY, multi);
		
			String anomcomplete = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.ANOM_COMPLETE);
			fMap.put(HKLDataset.ANOM_COMPLETE, anomcomplete);
		
			String anommulti = ModelDataProcList.get().getCurrentResult()
				.getField(DataProcField.ANOM_MULTIPLICITY);
			fMap.put(HKLDataset.ANOM_MULTIPLICITY, anommulti);
		} else {
			fMap.put(HKLDataset.UNIT_CELL, "");
			fMap.put(HKLDataset.COMPLETE, "");
			fMap.put(HKLDataset.RMERGE, "");
			fMap.put(HKLDataset.RPIM, "");
			fMap.put(HKLDataset.ISIGMA, "");
			fMap.put(HKLDataset.CCHALF, "");
			fMap.put(HKLDataset.MULTIPLICITY, "");
			fMap.put(HKLDataset.ANOM_COMPLETE, "");
			fMap.put(HKLDataset.ANOM_MULTIPLICITY, "");
		}
		/* find the resolution */
		int pos1, pos2;
		StringTokenizer tok;
		pos1 = content.indexOf("!INCLUDE_RESOLUTION_RANGE=");
		if (pos1 > 1) {
			pos2 = content.indexOf("\n", pos1);

			tok = new StringTokenizer(content.substring(pos1, pos2));
			tok.nextToken();
			tok.nextToken();
			fMap.put(HKLDataset.RESOLUTION, tok.nextToken());
		} else {
			fMap.put(HKLDataset.RESOLUTION, resolution);
		}

		/* find the spacegroup */

		pos1 = content.indexOf("!SPACE_GROUP_NUMBER=");
		if (pos1 > 1) {
			pos2 = content.indexOf("\n", pos1);

			tok = new StringTokenizer(content.substring(pos1, pos2));
			tok.nextToken();

			/* fidn the spaceGroup from number */
			String spaceGroupNo = tok.nextToken();
			String spaceGroup = DevMultiStrategy.get().getSpacegroupFromNumber(
					Integer.parseInt(spaceGroupNo));
			fMap.put(HKLDataset.SPACEGROUP, spaceGroupNo + "(" + spaceGroup
					+ ")");

			listHKLSets.add(fMap);
		}

		DevCombineDatasets.get().setResultDatasets(true, fMap);
	}
	public static String readScaFile(String scaFile) {

		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(scaFile)));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			Log.error(scaFile + " not found");
			e.printStackTrace();
		}
		// Skip 2 lines.
		Log.note("Converting .sca file to .HKL of XDS");
		String sgNo = "1";
		try {
			for (int i = 0; i < 2; i++)
				reader.readLine();
			String cell = reader.readLine();
			
			ModelCombineDatasets.UNIT_CELL = ModelReferenceData.getUnitCell(cell);
			StringTokenizer tok = new StringTokenizer(cell);
	
			for (int i = 0; i < ModelReferenceData.UnitCell.length; i++)
				tok.nextToken();

			sgNo = tok.nextToken();
			// Log.error(sgNo);
			String spaceGrouopNo = Integer.toString(DevMultiStrategy.get().getNumberFromSpacegroup(sgNo));
			fMap.put(HKLDataset.SPACEGROUP, spaceGrouopNo);
			sgNo = spaceGrouopNo;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			Log.error("Reading " + scaFile + " failed");
			e.printStackTrace();
		}

		/* Convert .sca file to XDS_ASCII.HKL file */
		String HKLFile = scaFile.substring(0, scaFile.lastIndexOf("."));
		String content = "";

		BufferedWriter out = null;
		try {
			out = new BufferedWriter(new FileWriter(HKLFile + ".HKL", false));
			out.write("!FORMAT=XDS_ASCII\n");
			out.write("!UNIT_CELL_CONSTANTS= " + ModelCombineDatasets.UNIT_CELL
					+ "\n");
			out.write("!SPACE_GROUP_NUMBER= " + sgNo + "\n");
			out.write("!NUMBER_OF_ITEMS_IN_EACH_DATA_RECORD= 5\n");
			out.write("!ITEM_H=1\n");
			out.write("!ITEM_K=2\n");
			out.write("!ITEM_L=3\n");
			out.write("!ITEM_IOBS=4\n");
			out.write("!ITEM_SIGMA(IOBS)=5\n");
			out.write("!END_OF_HEADER");
			out.write("\n");
			while ((content = reader.readLine()) != null) {
				out.write(content + "\n");
			}
			out.write("!END_OF_DATA");
			out.close();

		} catch (IOException e1) {
			Log.note("Writing .sca to .HKL failed");
			e1.printStackTrace();
		}
		try {
			reader.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		fMap.put(HKLDataset.HKLFILE, HKLFile + ".HKL");
		
		return(HKLFile + ".HKL");
	}

	public void deleteAll() {
		new Thread(new Runnable() {
			public void run() {
				boolean okToDelete = Util
						.popupOkCancel("Delete?",
								"OK to delete ALL multi startegy results?  This action cannot be undone.");
				if (okToDelete)
					DevCombineDatasets.get().deleteAllResults();
			}
		}).start();
	}

	@Override
	public void dispose() {
	}

	@Override
	public void inputChanged(Viewer arg0, Object arg1, Object arg2) {
	}

	@Override
	public void addListener(ILabelProviderListener arg0) {
	}

	@Override
	public boolean isLabelProperty(Object arg0, String arg1) {
		return false;
	}

	@Override
	public void removeListener(ILabelProviderListener arg0) {
	}

	@Override
	public String getCharset() {
		return null;
	}

	@Override
	public long getContentLength() {
		return 0;
	}

	@Override
	public String getMediaType() {
		return null;
	}

	@Override
	public String getMimeType() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String getSubType() {
		return null;
	}

	@Override
	public String getTransferEncoding() {
		return null;
	}

	@Override
	public String getFilename() {
		return null;
	}

	@Override
	public Image getColumnImage(Object arg0, int arg1) {
		return null;
	}

	@Override
	public String getColumnText(Object arg0, int arg1) {
		return null;
	}

	@Override
	public Object[] getElements(Object arg0) {
		return null;
	}
	@Override
	public void writeTo(OutputStream arg0) throws IOException {}
}
