/**
 * cache for CombineDatasets results
 */

package gov.anl.gmca.dataproc.combinedatasets;

import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;

public class ModelCombineDatasetsList {

	public ListenerList<ModelCombineDatasetsList> ll;

	protected ArrayList<ModelCombineDatasets> resultList = new ArrayList<ModelCombineDatasets>();

	public ArrayList<ModelCombineDatasets> getMultiMergeList() {
		return new ArrayList<ModelCombineDatasets>(resultList);
	}

	public ModelCombineDatasets getResultID(int requestedID) {
		for (ModelCombineDatasets result : getMultiMergeList()) {
			if (result.getID() == requestedID)
				return result;
		}
		return null;
	}

	public ModelCombineDatasets getResultIndex(int index) {
		try {
			return resultList.get(index);
		} catch (IndexOutOfBoundsException ioobe) {
			return null;
		}
	}

	public ModelCombineDatasets getCurrentResult() {
		return getResultIndex(getCurrentMultiMergeIndex());
	}

	public void clearResults() {
		resultList = new ArrayList<ModelCombineDatasets>();
	}

	public void setRow(ModelCombineDatasets result, int sampleNo, boolean update) {
		for (int i = 0; i < resultList.size(); i++) {
			if (resultList.get(i).getID() == sampleNo) {
				resultList.set(i, result);
				return;
			}
		}

		resultList.add(result);

		// Collections.sort(resultList);

		if (update)
			ll.notifyListeners(result.getID());
	}

	protected int currentMultiMergeIndex = 0;
	protected int showMultiStrategy = 1;

	public void setMultiStrategy(int show) {
		showMultiStrategy = show;
	}

	public int getMultiStrategy() {
		return showMultiStrategy;
	}

	public void setCurrentMultiMergeIndex(int i) {
		currentMultiMergeIndex = i;
	}

	public int getCurrentMultiMergeIndex() {
		return currentMultiMergeIndex;
	}

	protected static ModelCombineDatasetsList inst = null;

	public static ModelCombineDatasetsList get() {
		return inst;
	}

	public ModelCombineDatasetsList() {
		inst = this;
		ll = new ListenerList<ModelCombineDatasetsList>(this);
	}

	public void deleteAll() {
		new Thread(new Runnable() {
			public void run() {
				boolean okToDelete = Util.popupOkCancel(
						"Delete?",
						"OK to delete ALL ("
								+ resultList.size()
								+ ") strategy results?  This action cannot be undone.");
				if (okToDelete)
					DevCombineDatasets.get().deleteAllResults();
			}
		}).start();
	}
}