package gov.anl.gmca.diffimage;

import gov.anl.gmca.collect.Filenames;
import gov.anl.gmca.util.Util;
import hdf.hdf5lib.H5;
import hdf.hdf5lib.HDF5Constants;

import java.nio.file.Path;
import java.util.regex.Matcher;

public class HDF5DataFile
{
	private static final int RETRIES=3, RETRY_MS=1500;
	private Integer firstImageNumber, lastImageNumber, imagesPerDataFile=null;
	public Integer dataFileNumber;
	public String prefix, directory;
	public Path path;
	public HDF5MasterFile masterFile;
	
	public int getFirstImageNumber() throws Exception
	{
		if (null==firstImageNumber) calcImageNumbers();
		return firstImageNumber;
	}
	
	public int getLastImageNumber() throws Exception
	{
		if(null==lastImageNumber) calcImageNumbers();
		return lastImageNumber;
	}
	
	public synchronized int getImageCount() throws Exception
	{
		if (null!=imagesPerDataFile) return imagesPerDataFile;
		
		for (int i=0;i<RETRIES;i++)
		{
			try
			{
				long file_id = H5.H5Fopen(path.toString(), HDF5Constants.H5F_ACC_RDONLY, HDF5Constants.H5P_DEFAULT);
				
				String datasetName = "data"; //Util.sprintf("data_%06d", dataFileNumber);
				long[] dims = HDF5Image.getDatasetDims(file_id, "/entry/data", datasetName);
				int imageCount = (int)dims[0];
				return imageCount;
			}
			catch (Exception e) { e.printStackTrace(); }
			Util.sleep(RETRY_MS);
		}
		throw new Exception("Failed to read images per data file");
	}
	
	private void calcImageNumbers() throws Exception
	{
		firstImageNumber = getImageCount() * (dataFileNumber-1) + 1;
		lastImageNumber = firstImageNumber + getImageCount() - 1;
	}
	
	public HDF5DataFile(Path path)
	{
		this.path = path;
		this.directory = path.getParent().toString();
		
		String filename = path.getFileName().toString();
		Matcher matcher = Filenames.HDF5DataFilePattern.matcher(filename);
		if (matcher.matches())
		{
			prefix = matcher.group(1);
			dataFileNumber = Integer.parseInt(matcher.group(2));
		}
	}
	
	public void setMasterFile(HDF5MasterFile masterFile)
	{
		this.masterFile = masterFile;
	}
	
	public String getMasterFilePath()
	{
		return directory+"/"+prefix+"_master.h5";
	}

	public String getOwner()
	{
		return null;
	}
}
