package gov.anl.gmca.diffimage;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.io.Disk.File;
import gov.anl.gmca.util.Util;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.StringTokenizer;

public class MarImageHeader
{
	public double overload;
	public int size1,size2;
	public double wavelength, distance;
	public double displayOriginX, displayOriginY;
	public double jpegPixelSize;
	public double exposureTime;
	public double phi;
	public String detector;
	public double osc_range;
	
	public int httpResponseCode = -1;
	public String httpResponseText = "";

	private String path;
    
    public MarImageHeader(String path) { this.path=path; }

	/* This is the format of the header returned by the image server.  It's returned
	 * as the HTML content of the request (not as HTTP headers).
	  
	 * 	OVERLOAD_CUTOFF       65535
	 *	SIZE1                 4096
	 *	SIZE2                 4096failed, retry 
	 *	PIXEL_SIZE            0.073242
	 *	PIXEL SIZE            0.073242
	 *	DETECTOR              MARCCD300
	 *	DETECTOR_TYPE         0
	 *	WAVELENGTH            1.0332
	 *	DISTANCE              250.002
	 *	BEAM_CENTER_X         150.293
	 *	BEAM_CENTER_Y         149.414
	 *	TIME                  0.5
	 *	EXPOSURE TIME         0.5
	 *	PHI                   0.0000
	 *	OSC_START             0.0000
	 *	OSC_END               1.0000
	 *	OSC_RANGE             1.0000
	 *	OSCILLATION RANGE     1.0000
	 */
	public int getHeader() {

		boolean waitForGFS = false;
		String host = DBConfig.get("app_imgsrvIP");
		String port = "14003";
		String command = "getHeader";
		String userName = "gmca";
		String sessionId = "3282111111111111111";

		String urlString = "http://" + host + ":" + port + "/" + command + "?fileName=" + path + "&userName=" + userName
				+ "&sessionId=" + sessionId;
		System.out.println(urlString);
		try {
			if (waitForGFS) {
				// download
				File headerFile = new File(path);
				int sleepmax = 100;
				while (!headerFile.exists()) { // wait max of 5 sec till the
												// image available
					if (!headerFile.exists()) {
						// System.out.println(image + " not available at this
						// time");
					}
					Util.sleep(100);
					sleepmax = sleepmax + 100;
					if (sleepmax > 5000)
						break;
				}
			}

			URL url = new URL(urlString);
			URLConnection connection = url.openConnection();

			try {
				BufferedReader reader = new BufferedReader(new InputStreamReader(connection.getInputStream()));

				String line = null;
				StringBuilder sb = new StringBuilder();
				while ((line = reader.readLine()) != null) {

					sb.append(line);
					sb.append('\n');

					StringTokenizer st = new StringTokenizer(line, " ");

					String key = st.nextToken().trim();
					if (key == null || key.equals(""))
						continue;
					String val = st.nextToken().trim();
					if (val == null || val.equals(""))
						continue;

					if (key.contains("OVERLOAD_CUTOFF"))
						overload = Double.valueOf(val);
					if (key.contains("SIZE1"))
						size1 = Integer.valueOf(val);
					if (key.contains("SIZE2"))
						size2 = Integer.valueOf(val);
					if (key.contains("PIXEL_SIZE"))
						jpegPixelSize = Double.valueOf(val);

					if (key.equals("DETECTOR")) {
						while (st.hasMoreTokens()) {
							val += " " + st.nextToken().trim(); 
						}
						detector = val;
                                        }

					if (key.contains("WAVELENGTH"))
						wavelength = Double.valueOf(val);
					
					if (key.contains("DISTANCE"))
						distance = Double.valueOf(val);

					if (key.contains("EXPOSURE")) {
						val = st.nextToken().trim();
						exposureTime = Double.valueOf(val);
					}

					if (key.contains("BEAM_CENTER_X"))
						displayOriginX = Double.valueOf(val);
					if (key.contains("BEAM_CENTER_Y"))
						displayOriginY = Double.valueOf(val);
					
					if (key.contains("PHI"))
						phi = Double.valueOf(val);
					if (key.contains("OSC_RANGE"))
						osc_range = Double.valueOf(val);
					
					
				}

				httpResponseText = sb.toString();
				reader.close();
			} catch (Exception e) {
				System.out.println("ERROR: Reading the header (" + e.getMessage() + ")");
				return 1;
			}

		} catch (java.net.ConnectException ce) {
			System.out.println("ERROR: Couldn't connect to image server (" + ce.getMessage() + ")");
			return 1;
		} catch (java.io.IOException ioe) {
			String msg = ioe.getMessage();
			String httpCodeMsg = "Server returned HTTP response code: ";
			if (msg.indexOf(httpCodeMsg) == 0) {
				try {
					httpResponseCode = Integer.parseInt(msg.substring(httpCodeMsg.length(), httpCodeMsg.length() + 3));
				} catch (NumberFormatException nfe) {
					nfe.printStackTrace();
				}
				msg = "HTTP " + httpResponseCode;
			}
			System.out.println("ERROR: IO Error loading diffraction image (" + msg + ")");
			return 2;
		} catch (Exception e) {
			e.printStackTrace();
			return 3;
		}
		return 0;
	}
}
