/* DRMAA session
 * 
 * code snippet explanation found here
 * http://gridscheduler.sourceforge.net/howto/drmaa_java.html
 *
 */

package gov.anl.gmca.gridengine;

import gov.anl.gmca.syslog.Log;
import org.ggf.drmaa.DrmaaException;
import org.ggf.drmaa.JobTemplate;
import org.ggf.drmaa.Session;
import org.ggf.drmaa.SessionFactory;

import java.util.Collections;

public class DRMAA extends Thread {

	public static SessionFactory factory = null;
	public static Session session = null;
	public int waittime = 180;
	String jobid = null;
	String scriptString = "";
	String workDir = "";
	int status = 0;

	public DRMAA(String _scriptString, String _workDir, int _waittime)
			throws UnsatisfiedLinkError, Exception {
		setName("DRMAA");
		scriptString = _scriptString;
		workDir = _workDir;
		waittime = _waittime;
		/****************************************************************/
		/* initializing DRMAA session to submit jobs to Sun Grid engine */
		/****************************************************************/
		if (factory == null)
			factory = SessionFactory.getFactory();
		if (session == null) {
			try {
				session = factory.getSession();
				session.init("");
			} catch (NoClassDefFoundError e) {
				throw e;
			} catch (UnsatisfiedLinkError e) {
				throw e;
			} catch (DrmaaException e) {
				throw e;
			}
		}
		/****************************************************************/
	}

	/* wait for job id to finish */
	public int waitFor() throws InterruptedException, DrmaaException {

		/* wait few 3 (?) sec till job is submitted to SGE */
		try {
			Thread.sleep(3000);
		} catch (InterruptedException e1) {
			throw new InterruptedException();
		}

		/*
		 * status 16 when waiting in the queue, 32 when running, 48 when it is
		 * finished (based on obesvation) alternative may exist. session.wait()
		 * has problems with multiple threads. quit after 3 minutes
		 */
		int timer = 1;
		while (status < 48) {

			try {
				status = session.getJobProgramStatus(jobid);
			} catch (DrmaaException e) {
				e.printStackTrace();
				throw e;
			}
			// Log.note("status: "+ status);

			try {
				Thread.sleep(1000);
			} catch (InterruptedException e1) {
				throw new InterruptedException();
			}
			timer++;
			if (timer > waittime) {
				Log.error("Grid Engine processing timed out while running : "
						+ scriptString);
				try {
					session.control(jobid, Session.TERMINATE);
				} catch (DrmaaException e) {
					e.printStackTrace();
					throw e;
				}
				return (1);
			}
		}
		return 0;
	}

	/* submit job to SGE */
	@Override
	public void run() {
		// TODO Auto-generated method stub
		try {
			JobTemplate jt = session.createJobTemplate();
			jt.setRemoteCommand(scriptString);
			jt.setArgs(Collections.singletonList("5"));
			jt.setWorkingDirectory(workDir);

			jobid = session.runJob(jt);

			System.out.println(scriptString
					+ " job has been submitted with id " + jobid);

			// session.wait(jobid, Session.TIMEOUT_WAIT_FOREVER);

			session.deleteJobTemplate(jt);
			// session.exit();
		} catch (DrmaaException e) {
			e.printStackTrace();
		}
	}
}
