package gov.anl.gmca.gui;

import gov.anl.gmca.analysis.TabAnalysisNG;
import gov.anl.gmca.collect.EnergyCheckThread;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.ModelCollectRunList.CollectState;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.embed.TabJetCollect;
import gov.anl.gmca.embed.TabSONICC;
import gov.anl.gmca.gui.state.JBServer;
import gov.anl.gmca.gui.state.SWServer;
import gov.anl.gmca.hutch.TabHutch;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Disk.File;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.Interpreter;
import gov.anl.gmca.io.sim.dev.SimDeviceInit;
import gov.anl.gmca.syslog.TextLog;
import gov.anl.gmca.users.DevUserTable;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;

public class Main
{
	public		static int				APP_LIFETIME			= Integer.MAX_VALUE;
	public 		static boolean			STILL_SINGLE_ARM_NOT_IMPLEMENTED = false;
	public		static boolean			SHOW_SF_GRAPH			= true;
	public		static int				START_TAB_INDEX			= 0;
	public		static boolean			START_AS_MASTER			= false;
	public 		static int 				SIM_LEVEL 				= 0;
	public		static boolean			USE_REAL_DISK			= false;
	public		static boolean			FAKE_DATA				= false;
	public		static boolean			SIM_FPE					= false;
	public		static boolean 			DISABLE_APS_PVS 		= false;
	public		static boolean			USE_SERVER				= false;
	public		static boolean			SHOW_JET_MODE			= false;
	public		static boolean			SHOW_MESH_MODE			= true;
	public 		static boolean			SONICC_TEST_MODE		= false;
	public 		static final int 		PROGRESS_MAX 			= 140;
	protected 	static final int 		EPICS_QUEUE_SIZE_TARGET = 20;
	
	public static GUI gui = null;
	protected static SplashScreen splash = null;
	public static void setProgress(int i, String currentTask)
	{
		if (null != splash) splash.setProgress(i, currentTask);
	}
	
	public static String getAppDir() { return System.getProperty("application.home"); }
	
	protected static void processArgs(String[] args)
	{
		try
		{
    		for (int i=0; i<args.length; i++)
    		{
    			if (args[i].equals("sim")) 		{ System.out.println("Sim level 1"); SIM_LEVEL = 1; }
    			if (args[i].equals("sim2")) 	{ System.out.println("Sim level 2"); SIM_LEVEL = 2; }
    			if (args[i].equals("sim3")) 	{ System.out.println("Sim level 3 (Simulated EPICS only)"); SIM_LEVEL = 3; }
				if (args[i].equals("--sim")) 	{ System.out.println("Sim level 5 (Full beamline sim)"); DBConfig.sim5 = true; }
				if (args[i].equals("simfpe")) 	{ System.out.println("Sim FPE"); SIM_FPE = true; }
    			if (args[i].equals("realdisk")) { System.out.println("Using real disk access"); USE_REAL_DISK = true; }
    			if (args[i].equals("fakedata")) { System.out.println("Overwriting image files with fake data"); FAKE_DATA = true; }
    			if (args[i].equals("noaps"))	{ System.out.println("Disabling APS PVs"); DISABLE_APS_PVS = true; }
    			if (args[i].equals("lifetime"))	{ APP_LIFETIME = Integer.parseInt(args[i+1]); System.out.println("App lifetime "+APP_LIFETIME+" sec"); }
    			if (args[i].equals("starttab")) { START_TAB_INDEX = Integer.parseInt(args[i+1]); System.out.println("Starting with tab index "+START_TAB_INDEX); }
    			if (args[i].equals("startmaster")) { System.out.println("Starting as master"); START_AS_MASTER = true; }
    			if (args[i].equals("sfgraph"))  { System.out.println("Showing SpotFinder Graph"); SHOW_SF_GRAPH = true; }
    			if (args[i].equals("bluice_server"))  { System.out.println("Using JBluIce server"); USE_SERVER = true; }
				if (args[i].equals("jetcollect"))  { System.out.println("Enabling Jet Collect mode"); SHOW_JET_MODE = true; }
				if (args[i].equals("mesh"))		{ System.out.println("Enabling Mesh Collect mode"); SHOW_MESH_MODE = true; }
				if (args[i].equals("sonicc-test")) { System.out.println("Running SONICC in test mode"); SONICC_TEST_MODE = true; }
    		}
		}
		catch (ArrayIndexOutOfBoundsException oobe) { oobe.printStackTrace(); }
		catch (NumberFormatException nfe) { nfe.printStackTrace(); }
	}

	public static void closeSplash()
	{
		splash.close();
		while (GUI.getDisplay().readAndDispatch()) {}
	}

	public static void main(String[] args)
	{
		startSysLog(args);
		addShutdownHook();
		
		processArgs(args);
		
		splash = new SplashScreen();
		splash.open();

		verify_tempdir();
		
		// EPICS is needed when connecting to the database
		setProgress(10, "EPICS");
		Epics.connect();

		setProgress(15, "JBluIce server");
		if (!JBServer.get().init() || !SWServer.get().init()) {
			System.exit(-1);
		}
		
		setProgress(20, "SQL Database");
		Database.init();

		// print startup info now that SQL is loaded
		System.out.println("Starting JBluIce-EPICS "+Util.getVersionString());
		System.out.println("Hostname="+Client.getShortHostname());
		System.out.println("Beamline="+(Util.getBeamlineNumberFromHostname()+1));
		System.out.println("Workstation="+Util.getWorkstationNumberFromHostname());
		System.out.println("XDG_CONFIG_HOME is ["+System.getenv().get("XDG_CONFIG_HOME")+"]");
		System.out.println("GTK_THEME is ["+System.getenv().get("GTK_THEME")+"]");
		
		// guarantee devices are created before any GUI elements are made (IPC triggers tab creation)
		setProgress(25, "Creating Devices");
		if (Main.SIM_LEVEL>=2) SimDeviceInit.init();
		DeviceInit.init();
		
		setProgress(30, "Shell");
		gui = new GUI();
		
		createStartupTasks();
		gui.mainLoop();

		System.out.println("Java: exited main loop");
		
		Epics.disconnectMonitors();

		System.exit(0);
	}
	
	private static long appStartTime = System.currentTimeMillis();

	private static void createStartupTasks()
	{
		final Runnable onStartupCompletion = new Runnable(){public void run()
		{
			if (START_AS_MASTER) DevUserTable.get().becomeMaster();
			else
			{
				if (ModelCollectRunList.get().getCollectState()!=CollectState.INACTIVE)
				{
					Main.setProgress(135, "Crash recovery, please wait...");
					ModelCollectRunList.get().handleCollectInProgressOnStartup();
				}
			}
			
			splash.close();
			splash = null;
			
			Interpreter.connect();
			
			long currentTime = System.currentTimeMillis();
			double startupTime = (currentTime-appStartTime)/1000.0;
			Util.printf("Total startup time: %.2f sec", startupTime);
			
			if (APP_LIFETIME < Integer.MAX_VALUE && APP_LIFETIME >= 0)
			{
				new Thread(new Runnable(){public void run()
				{
					System.out.println("WARNING: Sleeping "+APP_LIFETIME+" seconds and then shutting down.");
					Util.sleep(APP_LIFETIME * 1000);
					Main.shutdown();
				}
				}, "App Lifetime").start();
			}
			
			GUI.getShell().setVisible(true);
			GUI.appState = GUI.RUNNING;
			EnergyCheckThread.start();
			ExecTask.new_ExecTask("/mnt/software/px/bin/albula --help", "albula", false, false).start();
			GUI.get().monitor = GUIThreadMonitor.start(GUI.get());
			TabHutch.get().setDefaultFocus();
		}};
		
		final long startTime = System.currentTimeMillis();
		final Runnable waitForEpics = new Runnable(){public void run()
		{
			int queueSize;
			if ((queueSize=Epics.getEpicsTaskCount()) > EPICS_QUEUE_SIZE_TARGET)
			{
				long now = System.currentTimeMillis() - startTime;
				long msSinceStart = now - startTime;
				long msSinceLastUpdate = now - lastUpdateMs;

				if (msSinceStart > 5000)
				{
					GUI.asyncExec(onStartupCompletion);
					return;
				}
				
				if (msSinceLastUpdate > 500)
				{
					setProgress(130, "Waiting for EPICS ("+queueSize+" pending)");
					lastUpdateMs = now;
				}
				
				GUI.asyncExec(this);
			}
			else
			{
				GUI.asyncExec(onStartupCompletion);
			}
		}};
		GUI.asyncExec(waitForEpics);
	}
	private static long lastUpdateMs = 0;

	private static String tempDirString = null;
	public static String getTempDir() { return tempDirString; }
	
	public static void verify_tempdir()
	{
		tempDirString = System.getProperty("user.home")+"/bluice_temp";//_bl"+Util.getBeamlineNumberFromHostname();
		File tempDir = new File(tempDirString);
		if (!tempDir.exists())
		{
			tempDir.mkdir();
		}
		else
		{
			if (!tempDir.canWrite())
			{
				tempDir.setWritable(true);
			}
		}
	}
	
	protected static void startSysLog(String[] args)
	{
		boolean logToFile = true;
		
		for (String arg : args)
		{
			if (arg.equals("--console")) logToFile = false;
		}
		
		TextLog.init(logToFile);
	}
	
    public static void shutdown()
    {
    	System.out.println("Unregistering client");
		DevUserTable.get().unregisterClient();
		
		System.out.println("Java: setting atExit=true");
		Util.atExit(true);

		System.out.println("Stopping Python process");

		TabAnalysisNG analysisTab = TabAnalysisNG.get();
		if (null != analysisTab) analysisTab.save();

		System.out.println("Java: Calling System.exit(0)");
		System.exit(0);
	}
    
    private static void addShutdownHook()
    {
    	Runtime.getRuntime().addShutdownHook(new Thread(new Runnable(){public void run()
    	{
    		if (!Util.atExit())
    		{
    			System.out.println("ERROR: Unexpected shutdown");
        		DevUserTable.get().unregisterClient();
    		}
    	}}));
    }
}
