package gov.anl.gmca.gui;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.client_server.ui.DlgServerView;
import gov.anl.gmca.common.client_server.ui.DlgStreamWriterView;
import gov.anl.gmca.common.ui.WdgEpicsState;
import gov.anl.gmca.gui.options.WdgControlPanel;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.PVLookup.*;
import gov.anl.gmca.raster.util.ClearSampleState;
import gov.anl.gmca.sample.PiezoDialog;
import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.screening.ui.TabScreening;
import gov.anl.gmca.strategy.ModelStrategyResultList;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;


public class MenuActions
{
	public final static String acroreadPath = "/usr/local/bin/firefox";
	protected final static String firefoxPath = "/usr/local/bin/firefox";
	protected final static String marccdepicsPath = "/usr/local/bin/marccdepics";
	protected final static String terminalPath = "/usr/bin/gnome-terminal";
	protected final static String medmPath = "/usr/local/bin/";
	
	public static void setAttenuationByFactors() {}
	public static void setAttenuationByPercentage() {}
	
	public static void cameraUndo() { Epics.put(PV_BLUICE.pointclick, "undo"); }
	public static void cameraRedo() { Epics.put(PV_BLUICE.pointclick, "redo"); }
	
	public static void reloadSQL() {}
	
	public static void showEpicsState()
	{
		WdgEpicsState.open();
	}
	
	public static void setCommandPromptVisible(boolean visible)
	{
		WdgMain.setCommandPromptVisible(visible);
	}
	public static void openAbout()
	{
		WdgAbout about = new WdgAbout(GUI.getShell(), SWT.NONE);
		about.open();
	}
	
	protected static void openPDF(String pdfPath)
	{
	        Log.note("Opening PDF: "+pdfPath);
		ExecTask.new_ExecTask(acroreadPath+" "+pdfPath, "firefox", true).start();
	}
	
	public static void openNewfeaturesManual()	{ openPDF("http://www.gmca.anl.gov/userprogram/docs/NewFeatures_combined.pdf");     }
	public static void openBeamlineOpsManual()	{ openPDF("http://www.gmca.anl.gov/userprogram/docs/beamline_operations.pdf"); 		}
	public static void openBackupManual()		{ openPDF("http://www.gmca.anl.gov/userprogram/docs/Backup_manual.pdf"); 			}
	public static void openScreeningManual()	{ openPDF("http://www.gmca.anl.gov/userprogram/docs/ScreeningTab_UserManual.pdf"); 	}
	
	public static void openBrowser(String url)
	{
		ExecTask.new_ExecTask(firefoxPath+" "+url, "firefox", true).start();
	}
	public static void openFlScanManual()		{ openBrowser("https://www.gmca.aps.anl.gov/video/html5-guides/scan-vjs.html"); 	}
	public static void openGMCASite()			{ openBrowser("http://www.gmca.anl.gov/"); }
	public static void openStaffList()			{ openBrowser("http://www.gmca.anl.gov/staff/index.html"); }
	public static void openSoftware()			{ openBrowser("http://www.gmca.anl.gov/computing/software_crystallography.html"); }
	public static void openVideoTutorials()		{ openBrowser("http://www.gmca.anl.gov/video/"); }
	
	public static void openSpreadsheetInstructions() { openBrowser("http://www.gmca.anl.gov/facilities/automounter.html#instructions");}
	public static void unloadResetSampleOnGonio() { 
		if(!Client.isMaster()) { Log.warn("This client is not master");return;}
		if(Util.popupOkCancel("Clear the mounted sample", "ATTENTION!!!! The mounted sample will be cleared, set to none. \nIf the sample is already mounted, please manually remove it."))
			Epics.put(PV_ROBOT.cmd, "unload reset");
	}
	public static void clearStrategyResults() { 
		if(!Client.isMaster()) { Log.warn("This client is not master");return;}
		//ModelStrategyResultList.get().save(System.getProperty("user.home")+"/strategyResults.xls");
		ModelStrategyResultList.get().save(DevDataDir.getPrimary()+"/strategyResults.xls");
		ModelStrategyResultList.get().deleteAll(); 
	}
	public static void setProcessingXDS(int isXDS) { 
		if(!Client.isMaster()) { Log.warn("This client is not master"); return;}
		
			Epics.putInteger(PV_RUNS.bgProc, isXDS);
	}
	public static void setStrategyProgram(int programVal) {
		if(!Client.isMaster()) { Log.warn("This client is not master");return;}
		Epics.putInteger(PV_SCR.strategyProgram, programVal);
	}
	public static void openMarccdEpics() { ExecTask.new_ExecTask(marccdepicsPath, "MarccdEpics", true).start(); }
	public static void openTerminal() { ExecTask.new_ExecTask(terminalPath, "Terminal", true).start(); }
	public static void openMedm() { ExecTask.new_ExecTask(medmPath+DBConfig.get("app_beamlineID"), "MEDM", true).start(); }
	
	public static void showScreeningSampleWindow() {
		TabScreening.sampleDialog.messageDialog.open();
		TabScreening.sampleDialog.messageDialog.setVisible(true);
	}
	
	public static void piezoControls () { 
		
		final PiezoDialog piezoDialog=new PiezoDialog(GUI.getShell(),SWT.NONE);
		piezoDialog.create();
		piezoDialog.messageDialog.setVisible(true);
		/*piezoButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) {
				// TODO Auto-generated method stub
				GUI.asyncExec(new Runnable(){public void run()
				{
					piezoDialog.messageDialog.open();
					piezoDialog.messageDialog.setVisible(true);
				}
				});
			}
		});*/
	}

	public static void setScanModeAdaptive()	{ DevScan.get().setAdaptiveScan(true, true); }
	public static void setScanModeFixed()		{ DevScan.get().setAdaptiveScan(false, true); }
	
	public static void clearSampleState()
	{
		if (!Client.isMaster())
		{
			Log.note("Client is not master, sample-related clearing aborted.");
			return;
		}
		ClearSampleState.clearAllRelatedState("", true, true);
	}

	public static void setClearRuns(boolean selection) {
		if(!Client.isMaster()) { Log.warn("This client is not master");return;}
		
		if(selection)
			Epics.putInteger(PV_R_RUNS.scrClear, 1);
		else
			Epics.putInteger(PV_R_RUNS.scrClear, 0);
		
	}

	public static void openOptionsWindow()
	{
		WdgControlPanel.get().open();
	}

	public static void openServerView()
	{
		DlgServerView.get().open();
	}

	public static void openCounterView()
	{
		DlgStreamWriterView.get().open();
	}
}
