package gov.anl.gmca.gui;

import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

public class SplashScreen
{
	protected Shell splash = null;
	
	protected ProgressBar progress = null;
	protected static final int progressHeight = 5;
	
	protected Label taskLabel = null;
	protected static final int taskHeight = 20;
	
	protected Button cancelButton = null;
	protected static final int cancelHeight = 23;
	
	public void setProgress(final int i, final String currentTask)
	{
		Util.safeSyncExec(new Runnable(){public void run()
		{
			progress.setSelection(i);
			taskLabel.setText(currentTask);
			GUI.processMessages(100);
		}});
	}
	
	public SplashScreen()
	{
		GridData gd;
		
	    // create the shell
	    splash = new Shell(GUI.getDisplay(), SWT.NO_TRIM);
	    splash.setSize(GUI.IMAGE_LOGO.getBounds().width, GUI.IMAGE_LOGO.getBounds().height+progressHeight+taskHeight);
	    GridLayout gl = new GridLayout(2,false);
	    gl.marginHeight=0;
	    gl.marginWidth=0;
	    gl.verticalSpacing=0;
	    gl.horizontalSpacing=0;
	    splash.setLayout(gl);

	    // add the image label to the shell
	    Label label = new Label(splash, SWT.NONE);
	    label.setImage(GUI.IMAGE_LOGO);
	    label.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
	    
	    Composite taskComp = new Composite(splash, SWT.LEFT);
	    gl = new GridLayout(1,false);
	    gl.marginHeight=3;
	    taskComp.setLayout(gl);
	    taskComp.setBackground(GUI.COLOR_DARK_BLUE);
	    gd = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
	    gd.heightHint = taskHeight;
	    taskComp.setLayoutData(gd);
	    
	    taskLabel = new Label(taskComp, SWT.LEFT);
	    taskLabel.setBackground(GUI.COLOR_DARK_BLUE);
	    taskLabel.setForeground(GUI.COLOR_WHITE);
	    taskLabel.setText(" Initializing... ");
	    taskLabel.setFont(GUI.FONT_SPLASH);
	    gd = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
	    taskLabel.setLayoutData(gd);
	    
	    cancelButton = new Button(splash, SWT.PUSH);
	    cancelButton.setText("Cancel");
	    cancelButton.setFont(GUI.FONT_10);
	    gd = new GridData(SWT.DEFAULT, SWT.FILL, false, true, 1, 1);
	    gd.heightHint = cancelHeight;
	    cancelButton.setLayoutData(gd);
	    cancelButton.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				Main.shutdown();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
	    

	    progress = new ProgressBar(splash, SWT.NONE);
	    progress.setMinimum(0);
	    progress.setMaximum(Main.PROGRESS_MAX);
	    progress.setSelection(0);
	    gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
	    progress.setLayoutData(gd);
	    
	    // center the shell and open
	    Rectangle splashRect = splash.getBounds();
	    Rectangle displayRect = GUI.getDisplay().getBounds();
	    int x = (displayRect.width - splashRect.width) / 2;
	    int y = (displayRect.height - splashRect.height) / 2;
	    splash.setLocation(x, y);
	}
	
	public void open()
	{
	    splash.open();
	}
	
	public void close()
	{
		splash.close();
	}
}
