package gov.anl.gmca.gui;

import gov.anl.gmca.io.Interpreter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;

import java.util.ArrayList;

public class WdgCommandPrompt extends Composite
{
	protected Text text;
	protected String textContents = "";
	protected ArrayList<String> history = new ArrayList<String>();
	protected int historyIndex = 0;
	
	public WdgCommandPrompt(Composite parent)
	{
		super(parent, SWT.BORDER);
		setLayout(new FillLayout());
		
		text = new Text(this,SWT.BORDER);
		text.addModifyListener(new ModifyListener()
		{
			@Override public void modifyText(ModifyEvent me)
			{
				textContents = text.getText();
			}
		});
		text.addTraverseListener(new TraverseListener()
		{
			@Override public void keyTraversed(TraverseEvent te)
			{
				if (SWT.TRAVERSE_RETURN == te.detail)
				{
					if (0 != textContents.length())
					{
						history.add(textContents);
						historyIndex = history.size();
					}
					
					final String contentCopy = textContents;
					
					new Thread(new Runnable(){public void run()
					{
						Interpreter.process(contentCopy);
					}}, "Interpreter").start();
					
					text.setText("");
				}
			}
		});

		text.addKeyListener(new KeyListener()
		{
			@Override public void keyPressed(KeyEvent ke)
			{
				if (ke.keyCode == SWT.ARROW_DOWN)
				{
					historyIndex++;
					recallHistory(historyIndex);
				}
				else if (ke.keyCode == SWT.ARROW_UP)
				{
					historyIndex--;
					recallHistory(historyIndex);
				}
			}
			@Override public void keyReleased(KeyEvent arg0) {}
		});
	}
	
	protected void recallHistory(int index)
	{
		if (historyIndex >= history.size()) historyIndex = history.size();
		if (historyIndex < 0) historyIndex = 0;
		
		if (historyIndex == history.size())
		{
			text.setText("");
		}
		else
		{
			text.setText(history.get(historyIndex));
		}
		text.setSelection(text.getCharCount());
	}
}
