package gov.anl.gmca.gui;

import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.EpicsDevice;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.sample.DevCollimatorCtrl;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ProcessHandling;
import gov.anl.gmca.util.ProcessHandling.Signal;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import java.util.ArrayList;

public class WdgEmergencyStop extends Composite
{
	protected Button button;
	protected Thread scriptStopThread = null;
	
	protected static ArrayList<String> processNames = new ArrayList<String>();
	public static void registerProcess(String processName)
	{
		if (!processNames.contains(processName)) processNames.add(processName);
	}
	
	public WdgEmergencyStop(Composite parent, int style)
	{
		super(parent, style);
		setLayout(new FillLayout());
		
		button = new Button(this, style);
		button.setText("EMERGENCY STOP");
		button.setBackground(GUI.COLOR_RED_BUTTON);
		button.setFont(GUI.FONT_STATUS_BUTTON);
		button.setEnabled(false);
		
		button.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				emergencyStop();
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(final boolean isMaster)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					button.setEnabled(isMaster);
				}});
			}
		});
	}
	
	protected void emergencyStop()
	{
		EpicsDevice.abortAll();
		
		if (null != ModelCollectRunList.get())
		{
			ModelCollectRunList.get().abortCollection();
		}
		
		if (null != ModelRasterState.get())
		{
			ModelRasterState.get().getCurrentRunModel().abort();
		}

		if (null != ModelScan.get())
		{
			ModelScan.get().abortScan();
		}
		
		if (null != DevCollimatorCtrl.get())
		{
			DevCollimatorCtrl.get().abort();
		}
		
		if (null == scriptStopThread || !scriptStopThread.isAlive())
		{
			scriptStopThread = new Thread(new Runnable(){public void run()
			{
				// find pids of running scripts
				ArrayList<Integer> runningScriptPids = new ArrayList<Integer>();
				for (String processName : processNames)
				{
					ArrayList<Integer> pidList = ProcessHandling.getChildPids(processName, true);
					System.out.println("Process "+processName+" has pids "+pidList);
					runningScriptPids.addAll(pidList);
				}

	    		// send SIGINT
				for (Integer pid : runningScriptPids)
				{
					ProcessHandling.kill(pid, Signal.INT);
				}

				// monitor them
	    		long startTime = System.currentTimeMillis();
	    		while (0 != runningScriptPids.size())
	    		{
	    			// on timeout, send SIGKILL and exit
	    			long now = System.currentTimeMillis();
	    			long timeSinceStart = now-startTime;
	    			if (timeSinceStart > 10000)
	    			{
	    				for (Integer pid : runningScriptPids)
	    				{
	    					ProcessHandling.kill(pid, Signal.KILL);
	    				}
	    				return;
	    			}

	    			// otherwise, update pid list and wait
	    			ArrayList<Integer> stillAlivePids = new ArrayList<Integer>();
					for (Integer pid : runningScriptPids)
					{
						if (ProcessHandling.isPidAlive(pid))
						{
							stillAlivePids.add(pid);
						}
					}
					runningScriptPids = stillAlivePids;
	    			
	    			Util.sleep(500);
	    		}
			}}, "Script stop");
			scriptStopThread.start();
		}
		///// stop Autocentering
		//Epics.putInteger(Epics.addBID("bi:autoCenterRun"),0);
		/////////
	}
	
	@Override
	public void setEnabled(boolean enable)
	{
		button.setEnabled(enable);
	}
}
