package gov.anl.gmca.gui;

import gov.anl.gmca.analysis.TabAnalysisNG;
import gov.anl.gmca.collect.DevCollectRunList;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.ui.TabCollect;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.ui.NBMessageBox;
import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.common.ui.WdgAppState;
import gov.anl.gmca.dataproc.ModelDataProcList;
import gov.anl.gmca.dataproc.TabAnalysis;
import gov.anl.gmca.gui.options.DevUserConfig;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.hutch.TabHutch;
import gov.anl.gmca.log.TabLog;
import gov.anl.gmca.raster.DevRasterRun;
import gov.anl.gmca.raster.DevRasterRunList;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.raster.ui.TabRaster;
import gov.anl.gmca.sample.TabSample;
import gov.anl.gmca.scan.ui.TabScan;
import gov.anl.gmca.screening.ui.TabScreening;
import gov.anl.gmca.embed.*;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.syslog.WdgLog;
import gov.anl.gmca.users.DevUserTable;
import gov.anl.gmca.users.TabUsers;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.io.File;
import java.util.ArrayList;

public class WdgMain implements SelectionListener, 
								ListenerList.Listener<ModelDataProcList>
{
	private static int analysisTabIndex;

	public static enum TAB { Hutch, Sample, Screening, Raster, Scan, Collect, SONICC, JetCollect, AnalysisNG, Analysis, Users, Log };

	public static void setVisibleTab(TAB tab)
	{
		if (tab != TAB.AnalysisNG) {
			get().ctFolder.setSelection(tab.ordinal());
		} else {
			get().ctFolder.setSelection(analysisTabIndex);
		}
		get().handleTabChange();
	}

	protected static WdgMain inst = null;
	public static WdgMain get() { return inst; }

	private WdgChecksWarning checkLabel;
	public WdgLog getLogWidget() { return logBar; }
	private WdgLog logBar;
	private WdgStatus wdgStatus;
	protected Menu toolsMenu;
	protected CTabFolder ctFolder;
	protected ArrayList<Tab> tabs;
	protected static WdgCommandPrompt commandPrompt = null;
	protected static Composite statusComp = null;
	public ListenerList<WdgMain> llTabChanged;

	public static void setCommandPromptVisible(boolean visible)
	{
		((GridData)commandPrompt.getLayoutData()).exclude = !visible;
		commandPrompt.setVisible(visible);
		GUI.shell.layout(true);
		statusComp.layout(true);
		if (visible) commandPrompt.setFocus();
	}
	
	public WdgMain()
	{
		inst = this;
		
		GUI.shell = new Shell(GUI.getDisplay());
		llTabChanged = new ListenerList<WdgMain>(this);
		
		GridLayout shellLayout = new GridLayout();
		shellLayout.marginHeight=0;
		shellLayout.marginWidth=0;
		shellLayout.horizontalSpacing=0;
		shellLayout.verticalSpacing=0;
		
		String beamline = DBConfig.get("app_beamlineName");
		String versionString = Util.getVersionString();
		String simString = (Main.SIM_LEVEL>0) ? " *Simulation Level "+Main.SIM_LEVEL+"*" : "";
		String noapsString = (Main.DISABLE_APS_PVS) ? " *APS PVs Disabled *" : "";
		if (Main.FAKE_DATA) simString += " *TEST DATA MODE* ";
		if (Main.SONICC_TEST_MODE) simString += " *SONICC TEST MODE* ";
		if (Main.SIM_FPE) simString += "*SIM FPE* ";
		
		GUI.shell.setText("JBluIce-EPICS: Beamline "+beamline+" "+versionString+simString+noapsString);
		GUI.shell.setBackground(GUI.BGCOLOR);
		GUI.shell.setImage(GUI.IMAGE_LOGO);
		
		GUI.shell.setLayout(shellLayout);
		GUI.shell.pack();
		GUI.shell.setSize(1036, 899);
		GUI.shell.setLocation(150, 50);
		
		//////////////////////////////////////////// main window layout ///////////////////////////////////////

		// menu
		makeMenu();
		
		// divider
		Composite blueDivider = new Composite(GUI.shell, SWT.NONE);
		//blueDivider.setBackground(GUI.COLOR_LIGHT_BLUE4);
		blueDivider.setBackground(GUI.COLOR_BLUE_BUTTON);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint = 8;
		blueDivider.setLayoutData(gd);
		
		// tabs
		final CTabFolder ctFolder = makeTabFolder(GUI.shell);
		
		// sash between main area and log window
		Sash sash = new Sash(GUI.shell, SWT.HORIZONTAL);
		sash.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		sash.setBackground(GUI.COLOR_GRAY);
		sash.addListener(SWT.Selection, new Listener()
		{
			public void handleEvent(Event event)
			{
				// header height + folder height + log height + status height = total height
				// -> log height = total height - status height - folder height - header height
				// where folder height is set by the sash
				GridData data = (GridData)logBar.getLayoutData();
				data.heightHint = GUI.shell.getSize().y - statusComp.getSize().y - event.y - 66;
				//Log.note(data.heightHint+"="+GUI.shell.getSize().y+"-"+statusComp.getSize().y+"-"+event.y);
				GUI.asyncExec(()->{GUI.shell.layout();});
			}
		});

		// log window
		logBar = new WdgLog(GUI.shell, SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 20;
		logBar.setLayoutData(gd);

		// status bars
		final Composite statusComp = makeStatusBars(GUI.shell);

		checkLabel = new WdgChecksWarning(GUI.shell);

		///////////////////////////////////////////////// listeners ////////////////////////////////////////////
		ModelDataProcList.get().ll.addListener(this);
		DevUserConfig.get().ll.notifyListeners();
		
		GUI.shell.addListener(SWT.Close, new Listener()
		{
			public void handleEvent(Event event)
			{
				if (ModelCollectRunList.get().isCollectThreadAlive())
				{
					Util.popupError("Action not possible", "This client can't close during collection.  Please pause collection first.");
					event.doit = false;
					return;
				}
				
				System.out.println("Java: close listener");
				
				// TODO: This dialog takes a second or two to open, but the delay is in open() itself
				// Making a custom pre-created dialog would fix this
                int style = SWT.APPLICATION_MODAL | SWT.YES | SWT.NO;
                NBMessageBox messageBox = new NBMessageBox(GUI.shell, style);
                messageBox.setText("Exit");
                messageBox.setMessage("Close JBluIce-EPICS?");
                event.doit = messageBox.open() == SWT.YES;
                if (!event.doit) return; 

				Main.shutdown();

		    	event.doit = true;
			}
		});

		GUI.getDisplay().addFilter(SWT.Traverse, new Listener()
		{
			@Override public void handleEvent(Event event)
			{
				boolean ctrlDown = ((event.stateMask & SWT.CTRL) != 0);
				
				if (event.keyCode == SWT.TAB && ctrlDown)
				{
					event.doit = false;
					
					int currentIndex = ctFolder.getSelectionIndex();
					int newIndex = (currentIndex + 1) % ctFolder.getItemCount();
					ctFolder.setSelection(newIndex);
				}
			}
		});
	}
	
	protected Menu makeMenu()
	{
		Menu menu = new Menu(GUI.shell, SWT.BAR);

		// File
		MenuItem fileHeader = new MenuItem(menu, SWT.CASCADE);
		fileHeader.setText("&File");

		Menu fileMenu = new Menu(GUI.shell, SWT.DROP_DOWN);
		fileHeader.setMenu(fileMenu);

		/*MenuItem fileLoadScan = new MenuItem(fileMenu, SWT.PUSH);
		fileLoadScan.setText("&Load scan");
		fileLoadScan.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				TabScan scanTab = TabScan.get();
				if (null != scanTab)
				{
					NBFileDialog fd = new NBFileDialog(GUI.getShell(),SWT.NONE);
					fd.setFilterPath(ModelScan.get().getFullDirectory());
					fd.setFilterExtensions(new String[]{"*.bip"});
					String fullpath = fd.openStr();
					if (null != fullpath)
					{
						scanTab.loadScan(fullpath);
					}
				}
			}
		});*/

		MenuItem fileExit = new MenuItem(fileMenu, SWT.PUSH);
		fileExit.setText("E&xit");
		fileExit.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) { GUI.getShell().close(); }
		});

		// Network
		MenuItem networkHeader = new MenuItem(menu, SWT.CASCADE);
		networkHeader.setText("&Network");

		Menu networkMenu = new Menu(GUI.shell, SWT.DROP_DOWN);
		networkHeader.setMenu(networkMenu);

		MenuItem networkBecomeMaster = new MenuItem(networkMenu, SWT.PUSH);
		networkBecomeMaster.setText("Become &Master");
		networkBecomeMaster.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) { DevUserTable.get().becomeMaster(); }
		});

		MenuItem networkBecomeMasterByForce = new MenuItem(networkMenu,	SWT.PUSH);
		networkBecomeMasterByForce.setText("Become Master by &Force");
		networkBecomeMasterByForce.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) { DevUserTable.get().becomeMasterByForce(); }
		});

		MenuItem networkBecomeSlave = new MenuItem(networkMenu, SWT.PUSH);
		networkBecomeSlave.setText("Become &Slave");
		networkBecomeSlave.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) { DevUserTable.get().becomeSlave(true); }
		});

		// Tools
		
		MenuItem toolsHeader = new MenuItem(menu, SWT.CASCADE);
		toolsHeader.setText("&Tools");

		toolsMenu = new Menu(GUI.shell, SWT.DROP_DOWN);
		toolsHeader.setMenu(toolsMenu);
		
		MenuItem optionsWindow = new MenuItem(toolsMenu, SWT.PUSH);
		optionsWindow.setText("&Options");
		optionsWindow.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openOptionsWindow(); }
		});
		
		final MenuItem optionsServerView = new MenuItem(toolsMenu, SWT.PUSH);
		optionsServerView.setText("&Server view");
		optionsServerView.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openServerView(); }
		});

		if (DetectorConfig.isEiger()) {
			final MenuItem optionsCounterView = new MenuItem(toolsMenu, SWT.PUSH);
			optionsCounterView.setText("&Counter view");
			optionsCounterView.addSelectionListener(new SelectionListener()
			{
				public void widgetDefaultSelected(SelectionEvent se) {}
				public void widgetSelected(SelectionEvent se)
				{
					MenuActions.openCounterView();
				}
			});
		}

		final MenuItem optionsCommandPrompt = new MenuItem(toolsMenu, SWT.CHECK);
		optionsCommandPrompt.setText("&Command Prompt");
		optionsCommandPrompt.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.setCommandPromptVisible(optionsCommandPrompt.getSelection()); }
		});

		MenuItem screeningStatusWindow = new MenuItem(toolsMenu, SWT.PUSH);
		screeningStatusWindow.setText("Screening sample window");
		screeningStatusWindow.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.showScreeningSampleWindow(); }
		});
		
		MenuItem marccdepicsWindow = new MenuItem(toolsMenu, SWT.PUSH);
		marccdepicsWindow.setText("Marccdepics");
		marccdepicsWindow.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openMarccdEpics(); }
		});
		
		MenuItem medmWindow = new MenuItem(toolsMenu, SWT.PUSH);
		medmWindow.setText("MEDM Toolbar");
		medmWindow.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openMedm(); }
		});
		
		MenuItem terminalWindow = new MenuItem(toolsMenu, SWT.PUSH);
		terminalWindow.setText("Terminal");
		terminalWindow.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openTerminal(); }
		});
		
		MenuItem appStateViewer = new MenuItem(toolsMenu, SWT.PUSH);
		appStateViewer.setText("Application state");
		appStateViewer.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { WdgAppState.open(true); }
		});

		// Help
		MenuItem helpHeader = new MenuItem(menu, SWT.CASCADE);
		helpHeader.setText("&Help");

		Menu helpMenu = new Menu(GUI.shell, SWT.DROP_DOWN);
		helpHeader.setMenu(helpMenu);
		
		MenuItem newfeaturesManual = new MenuItem(helpMenu, SWT.PUSH);
		newfeaturesManual.setText("What's New");
		newfeaturesManual.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openNewfeaturesManual(); }
		});
		
		MenuItem helpBeamlineManual = new MenuItem(helpMenu, SWT.PUSH);
		helpBeamlineManual.setText("Beamline &Operations Manual");
		helpBeamlineManual.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openBeamlineOpsManual(); }
		});

		MenuItem helpScanManual = new MenuItem(helpMenu, SWT.PUSH);
		helpScanManual.setText("&Fluorescence Scanning Manual");
		helpScanManual.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openFlScanManual(); }
		});
		
		MenuItem helpScreeningManual = new MenuItem(helpMenu, SWT.PUSH);
		helpScreeningManual.setText("&Screening Manual");
		helpScreeningManual.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openScreeningManual(); }
		});

		MenuItem helpBackupManual = new MenuItem(helpMenu, SWT.PUSH);
		helpBackupManual.setText("&Backup Manual");
		helpBackupManual.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openBackupManual(); }
		});
		
		new MenuItem(helpMenu, SWT.SEPARATOR);

		MenuItem helpGMCAWebSite = new MenuItem(helpMenu, SWT.PUSH);
		helpGMCAWebSite.setText("&GM/CA Web Site");
		helpGMCAWebSite.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openGMCASite(); }
		});

		MenuItem helpGMCAStaffList = new MenuItem(helpMenu, SWT.PUSH);
		helpGMCAStaffList.setText("GM/CA &Staff List");
		helpGMCAStaffList.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openStaffList(); }
		});

		new MenuItem(helpMenu, SWT.SEPARATOR);

		MenuItem helpSoftware = new MenuItem(helpMenu, SWT.PUSH);
		helpSoftware.setText("Crystallographic Software");
		helpSoftware.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openSoftware(); }
		});

		MenuItem helpVideo = new MenuItem(helpMenu, SWT.PUSH);
		helpVideo.setText("Video Tutorials");
		helpVideo.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openVideoTutorials(); }
		});

		new MenuItem(helpMenu, SWT.SEPARATOR);

		MenuItem helpAbout = new MenuItem(helpMenu, SWT.PUSH);
		helpAbout.setText("&About");
		helpAbout.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent se) {}
			public void widgetSelected(SelectionEvent se) { MenuActions.openAbout(); }
		});

		GUI.shell.setMenuBar(menu);
		
		return menu;
	}

	protected CTabFolder makeTabFolder(Composite parent)
	{
		ctFolder = new CTabFolder(parent, SWT.BORDER);
		ctFolder.setFont(GUI.FONT_HUTCH_GROUP);
		ctFolder.setMinimumCharacters(12);
		ctFolder.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;
		gd.verticalAlignment = SWT.FILL;
		gd.horizontalAlignment = SWT.FILL;
		ctFolder.setLayoutData(gd);

		tabs = new ArrayList<Tab>();
		
		int progress = 40;
		for (TAB tab : TAB.values())
		{
			Main.setProgress(progress, tab.toString()+" Tab");
			progress += 8;
			
			switch (tab)
			{
			case Hutch: 	tabs.add(new TabHutch(ctFolder, SWT.NONE));    	break;
			case Sample:	tabs.add(new TabSample(ctFolder, SWT.NONE,0)); 	break;
			case Screening:	tabs.add(new TabScreening(ctFolder, SWT.NONE));	break;
			case Raster:	tabs.add(new TabRaster(ctFolder, SWT.NONE));   	break;
			case SONICC:
				if (DBConfig.getBoolean("python_show_sonicc"))
					tabs.add(new TabSONICC(ctFolder, SWT.NONE));
				break;
			case Scan:		tabs.add(new TabScan(ctFolder, SWT.NONE));     	break;
			case Collect:	tabs.add(new TabCollect(ctFolder, SWT.NONE));  	break;
			case JetCollect:
				if (DBConfig.getBoolean("python_show_jet"))
					tabs.add(new TabJetCollect(ctFolder, SWT.NONE));
				break;
			case AnalysisNG:
				tabs.add(new TabAnalysisNG(ctFolder, SWT.NONE));
				analysisTabIndex = tabs.size() - 1;
				break;
			case Analysis:	
				if (DBConfig.getBoolean("processing_enable_analysis_legacy")) {
					tabs.add(new TabAnalysis(ctFolder, SWT.NONE));
				}
				break;

			case Users:		tabs.add(new TabUsers(ctFolder, SWT.NONE));    	break;
			case Log:		tabs.add(new TabLog(ctFolder, SWT.NONE));		break;
			}
		}

		for (Tab tab : tabs)
		{
			GUI.tabMap.put(tab.getTabName(), tab);
		}

		// TODO: find a better place for this.  it needs to be done outside the TabCollect constructor
		// because it uses getCollectTab() in a GUI thread
		Main.setProgress(120, "Loading collect runs");
		DevCollectRunList.get().connect();
		
		Main.setProgress(130, "Loading raster runs");
		while (true)
		{
			boolean allConnected = true;
			DevRasterRunList runlistDev = DevRasterRunList.get(RasterListType.INTER);
			if (!runlistDev.connected()) allConnected=false;
			for (int i=0;i<=runlistDev.getRunCount();i++)
			{
				DevRasterRun runDev = (DevRasterRun)runlistDev.getRunDev(i);
				if (!runDev.connected()) allConnected=false;
			}
			if (Main.SIM_LEVEL>=2) allConnected=true;
			if (allConnected) break;
			Util.sleep(100);
			Log.note("Waiting for raster runs to connect");
			
			// TODO: remove this temporary fix for sim2 mode
			if (Main.SIM_LEVEL==2) break;
		}
		TabRaster.get().createTabs();

		for (Tab tab : tabs)
		{
			tab.connectListeners();
			
			CTabItem item = new CTabItem(ctFolder, SWT.NONE);
			item.setText("   " + tab.getTabName() + "   ");
			item.setControl(tab);
		}
		
		TabRaster.get().runTabs.updateTabs();
		ctFolder.addSelectionListener(this);
		
		// calling setSelection below is not enough to trigger the callback, so
		// these must be done the first time
		lastTab = tabs.get(Main.START_TAB_INDEX);
		tabs.get(Main.START_TAB_INDEX).notifyIsVisible(true);
		
		ctFolder.setSelection(Main.START_TAB_INDEX);
		
		return ctFolder;
	}
	
	public void widgetDefaultSelected(SelectionEvent arg0) {}
	
	protected Tab lastTab = null;

	// on tab change, notify the tabs
	@Override
	public void widgetSelected(SelectionEvent se)
	{
		if (!(se.getSource() instanceof CTabFolder)) { Log.error("Unexpected source object: "+se.getSource()); return; }
		
		handleTabChange();
	}
	
	private void handleTabChange()
	{
		CTabItem cti = ctFolder.getSelection();
		Tab selectedTab = (Tab)cti.getControl();
		
		if (null != lastTab) lastTab.notifyIsVisible(false);
		
		// setting Analysis tab image to null after selection
		if(selectedTab.getTabName().equals("Analysis")) {
			cti.setImage(null);
		}
		selectedTab.notifyIsVisible(true);

		lastTab = selectedTab;
		
		llTabChanged.notifyListeners(selectedTab);
	}

	protected Composite makeStatusBars(Composite parent)
	{
		GridLayout compLayout = new GridLayout();
		compLayout.marginHeight=0;
		compLayout.marginWidth=0;
		compLayout.horizontalSpacing=0;
		compLayout.verticalSpacing=0;
		GridData gd;
		
		statusComp = new Composite(parent, SWT.NONE);
		statusComp.setBackground(GUI.COLOR_STATUS_BG);
		statusComp.setLayout(compLayout);
		statusComp.setLayoutData(new GridData(SWT.FILL, SWT.BOTTOM, true, false));
		
		commandPrompt = new WdgCommandPrompt(statusComp);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.exclude = true;
		commandPrompt.setLayoutData(gd);
		commandPrompt.setVisible(false);
		
		Composite statusBarBorder = new Composite(statusComp, SWT.BORDER);
		statusBarBorder.setBackground(GUI.COLOR_STATUS_BG);
		GridLayout gl = new GridLayout(1,false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		statusBarBorder.setLayout(gl);
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.verticalAlignment = SWT.BOTTOM;
		statusBarBorder.setLayoutData(gd);

		wdgStatus = new WdgStatus(statusBarBorder, SWT.NONE);
		gd = new GridData();
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		gd.heightHint = 56;
		wdgStatus.setLayoutData(gd);
		
		return statusComp;
	}

	@Override
	public void handleEvent(ModelDataProcList object, Object data) {
		settabColor("Analysis");
	}

	private void settabColor(String tabName) {
		GUI.asyncExec(new Runnable(){public void run()
		{
			CTabItem cti = ctFolder.getItem(6);
			cti.setImage(GUI.imagegreen);
		}});
	}
}
