package gov.anl.gmca.gui;

import gov.anl.gmca.collect.DevFPE;
import gov.anl.gmca.collect.ETA;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.collect.ModelCollectRunList.CollectState;
import gov.anl.gmca.collect.ui.TabCollect;
import gov.anl.gmca.common.EpicsDevice.State;
import gov.anl.gmca.common.ModelRunList.CurrentRunListener;
import gov.anl.gmca.common.ModelRunList.ModelRun;
import gov.anl.gmca.gui.WdgStatusElement.InterpretStatus;
import gov.anl.gmca.hutch.DevADC;
import gov.anl.gmca.hutch.DevAPSStatus;
import gov.anl.gmca.hutch.DevHutchShutter;
import gov.anl.gmca.hutch.DevShutter;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.ui.TabRaster;
import gov.anl.gmca.sample.DevInOut;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.DevUserTable;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import gov.aps.jca.CAException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackAdapter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

public class WdgStatus extends Composite
{
	/*  top   */ protected WdgStatusElement swAPSCurrent, swAPSShutters, swEndstationSecured, swAShutter, swEndstationShutter, swIZero;
	/* bottom */ protected WdgStatusElement swETA, swUndulator, swHarmonic, swMono, swControl, swShutter;
				 protected WdgCmdStatus		swBIStatus;
	
	protected Color black = GUI.getDisplay().getSystemColor(SWT.COLOR_BLACK);
	protected Color red = GUI.getDisplay().getSystemColor(SWT.COLOR_RED);
	private static WdgStatus inst = null;
	
	public static void setETAToolTip(String text)
	{
		inst.swETA.status.setToolTipText(text);
	}
	
	public WdgStatus(Composite parent, int style)
	{
		super(parent,style);
		inst=this;
		
		setBackground(GUI.COLOR_STATUS_BG);
		
		setLayout(new FillLayout(SWT.VERTICAL));
		
		/////////////////////////////////////////////////////////////////////////////////////////////////////
		//
		//
		//                                       top bar
		//
		//
		/////////////////////////////////////////////////////////////////////////////////////////////////////
		
		Composite compTop = new Composite(this, SWT.NONE);
		compTop.setBackground(GUI.COLOR_STATUS_BG);
		GridLayout glTop = new GridLayout(10,false);
		glTop.marginHeight=1;
		compTop.setLayout(glTop);
		
		int statusMargin = 19;
		
		swAPSCurrent = new WdgStatusElement(
				compTop,
				" APS Current ",
				null,
				"",
				54+statusMargin,
				true,
				null,
				new InterpretStatus() {
					@Override public boolean isValOK(Object value)  { return (Double)value > 50; }
					@Override public String getString(Object value) { return Util.printDouble((Double)value,1); }
				},
				null);
		
		DevAPSStatus.llCurrent.addListener(new ListenerList.Listener<DevAPSStatus>()
		{
			@Override public void handleEvent(DevAPSStatus object, Object data)
			{
				swAPSCurrent.setValue(data);
			}
		});
		swAPSCurrent.setValue(DevAPSStatus.getCurrent());
		
		swAPSShutters = new WdgStatusElement(
				compTop,
				" Shutter Permit ",
				null,
				"",
				75+statusMargin,
				true,
				null,
				new InterpretStatus() {
					@Override public boolean isValOK(Object value)  { return (Boolean)value; }
					@Override public String getString(Object value) { return (Boolean)value ? "Enabled" : "Disabled"; }
				},
				null);
		
		DevAPSStatus.llShutterPermit.addListener(new ListenerList.Listener<DevAPSStatus>()
		{
			@Override public void handleEvent(DevAPSStatus object, Object data)
			{
				swAPSShutters.setValue(data);
			}
		});
		swAPSShutters.setValue(DevAPSStatus.isShutterPermitEnabled());
				
		swAShutter = new WdgStatusElement(
				compTop,
				" A Shutter ",
				"Click to open the A shutter if it is closed",
				"",
				58+statusMargin,
				true,
				null,
				new InterpretStatus() {
					@Override public boolean isValOK(Object value)  { if (null==DevHutchShutter.get("aShutter")) return true;  return (Boolean)value; }
					@Override public String getString(Object value) { if (null==DevHutchShutter.get("aShutter")) return "N/A"; return (Boolean)value ? "Open" : "Closed"; }
				},
				(null==DevHutchShutter.get("aShutter"))?null:
				new Runnable()
				{
					@Override public void run()
					{
						if (!Client.isMaster()) { Log.warn("Client is not master, A shutter click ignored"); return; }
						
						DevHutchShutter aShutter = DevHutchShutter.get("aShutter");
						if (aShutter.isOpen())
						{
							Log.warn("Closing the A shutter is not supported by JBluIce-EPICS.");
						}
						else
						{
							Log.note("Sending command to open A shutter");
							try
							{
								aShutter.open();
							}
							catch (CAException cae)
							{
								Log.error("Couldn't open A shutter: "+cae.getMessage());
							}
						}
					}
				});
		
		if (null!=DevHutchShutter.get("aShutter"))
		{
			DevHutchShutter.get("aShutter").ll.addListener(new ListenerList.Listener<DevHutchShutter>()
			{
				@Override public void handleEvent(DevHutchShutter object, Object data)
				{
					swAShutter.setValue(data);
				}
			});
			swAShutter.setValue(DevHutchShutter.get("aShutter").isOpen());
		}
		else
		{
			swAShutter.setValue(false); // doesn't matter what value is sent, N/A is displayed
		}
		
		swEndstationShutter = new WdgStatusElement(
				compTop,
				" Endstation Shutter ",
				"Click to open or close the endstation shutter",
				"",
				58+statusMargin,
				true,
				null,
				new InterpretStatus() {
					@Override public boolean isValOK(Object value)  { return (Boolean)value; }
					@Override public String getString(Object value) { return (Boolean)value ? "Open" : "Closed"; }
				},
				new Runnable()
				{
					@Override public void run()
					{
						if (!Client.isMaster()) { Log.warn("Client is not master, endstation shutter click ignored"); return; }
						DevHutchShutter esShutter = DevHutchShutter.get("endStation");
						if (esShutter.isOpen())
						{
							Log.note("Closing endstation shutter");
							try
							{
								esShutter.close();
							}
							catch (CAException cae)
							{
								Log.error("Couldn't close endstation shutter: "+cae.getMessage());
							}
						}
						else
						{
							boolean searched = DevAPSStatus.isEndstationSecure();
							
							if (searched)
							{
    							Log.note("Sending command to open endstation shutter");
    							try
    							{
        							esShutter.open();
    							}
    							catch (CAException cae)
    							{
    								Log.error("Couldn't open endstation shutter: "+cae.getMessage());
    							}
							}
							else
							{
								Util.popupError("Station not searched", "Either the hutch door still OPEN or Searching not finished");
							}
						}
					}
				});
		
		DevHutchShutter.get("endStation").ll.addListener(new ListenerList.Listener<DevHutchShutter>()
		{
			@Override public void handleEvent(DevHutchShutter object, Object data)
			{
				swEndstationShutter.setValue(data);
			}
		});
		swEndstationShutter.setValue(DevHutchShutter.get("endStation").isOpen());
		
		swEndstationSecured = new WdgStatusElement(
				compTop,
				" Endstation Secure ",
				"Indicates whether the hutch door is closed and the station has been searched",
				"",
				40+statusMargin,
				true,
				null,
				new InterpretStatus() {
					@Override public boolean isValOK(Object value)  { return (Boolean)value; }
					@Override public String getString(Object value) { return (Boolean)value ? "Yes" : "No"; }
				},
				null);
		
		DevAPSStatus.llEndStationSecure.addListener(new ListenerList.Listener<DevAPSStatus>()
		{
			@Override public void handleEvent(DevAPSStatus object, Object data)
			{
				swEndstationSecured.setValue(data);
			}
		});
		swEndstationSecured.setValue(DevAPSStatus.isEndstationSecure());
						
		/////////////////////////////////////////////////////////////////////////////////////////////////////
		//
		//
		//                                     bottom bar
		//
		//
		/////////////////////////////////////////////////////////////////////////////////////////////////////
		
		Composite compBottom = new Composite(this, SWT.NONE);
		compBottom.setBackground(GUI.COLOR_STATUS_BG);
		GridLayout glBottom = new GridLayout(13,false);
		glBottom.marginHeight=1;
		compBottom.setLayout(glBottom);

		swBIStatus 	= new WdgCmdStatus(compBottom, " State:", "Ready", 125);
		
		swETA		= new WdgStatusElement(
					compBottom,
					" ETA:",
					null,
					"---",
					65,
					false,
					null,
					new InterpretStatus()
					{
						@Override public boolean isValOK(Object value) { return true; }
						@Override public String getString(Object value)
						{
							int totalSec = (int)Math.round((Double)value);
							if (totalSec<0)
							{
								totalSec=-totalSec;
								swETA.setOkStatusColor(GUI.COLOR_BLUE2);
							}
							else
							{
								swETA.setOkStatusColor(GUI.COLOR_BLACK);
							}
							if (totalSec == 0) return "---";
							return Util.readableSeconds(totalSec);
						}
					},
					null);
		
		swETA.status.addMouseTrackListener(new MouseTrackAdapter()
		{
			@Override public void mouseEnter(MouseEvent e)
			{
				Control currentControl = WdgMain.get().ctFolder.getSelection().getControl();
				if (currentControl==TabRaster.get())
				{
					swETA.status.setToolTipText(ETA.getRasterTooltip());
				}
				else if (currentControl==TabCollect.get())
				{
					swETA.status.setToolTipText(ETA.getCollectTooltip());
				}
				else
				{
					swETA.status.setToolTipText("Select the collect or raster tab to see the ETA");
				}
			}
		});
		
		WdgEmergencyStop emergencyButton = new WdgEmergencyStop(compBottom, SWT.PUSH);
		GridData gd = new GridData();
		gd.grabExcessHorizontalSpace=false;
		gd.grabExcessVerticalSpace=true;
		emergencyButton.setLayoutData(gd);
		emergencyButton.setEnabled(false);
		
		swMono = new WdgEnergyStatus(compBottom, " Mono:", "---", 95);
		
		swIZero = new WdgStatusElement(
				compBottom,
				" IZero:",
				null,
				"",
				65,
				false,
				null,
				new InterpretStatus() {
					@Override public boolean isValOK(Object value)  { return true; }
					@Override public String getString(Object value) { return Util.printDouble((Double)value,2)+" V"; }
				},
				null);
		
		DevADC.llIZero.addListener(new ListenerList.Listener<DevADC>()
		{
			@Override public void handleEvent(DevADC object, Object data)
			{
				swIZero.setValue((Double)data);
			}
		});
		swIZero.setValue(DevADC.getIZero());
		
		swControl 	= new WdgStatusElement(
				compBottom,
				" Control:",
				null,
				"Passive",
				70,
				false,
				null,
				new InterpretStatus() {
					@Override public boolean isValOK(Object value)  { return (!(Boolean)value); }
					@Override public String getString(Object value) { return (Boolean)value?"Active":"Passive"; }
				},
				new Runnable()
				{
					@Override public void run()
					{
						if (!Client.isMaster())
						{
							DevUserTable.get().becomeMaster();
						}
						else
						{
							DevUserTable.get().becomeSlave(true);
						}
					}
				});
		
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				swControl.setValue(isMaster);
			}
		});
		swControl.setValue(Client.isMaster());
		
		swShutter = new WdgStatusElement(
				compBottom,
				" Shutter:",
				null,
				"---",
				60,
				false,
				null,
				new InterpretStatus() {
					@Override public boolean isValOK(Object value)  { return (Boolean)value; }
					@Override public String getString(Object value) { return (Boolean)value ? "Open" : "Closed"; }
				},
				new Runnable()
				{
					@Override public void run()
					{
						if (!Client.isMaster()) { Log.warn("Client is not master, fast shutter click ignored"); return; }
						DevShutter fastShutter = DevShutter.get("shutter");
						if (fastShutter.getIsOpen()) 
						{
							if (CollectState.INACTIVE != ModelCollectRunList.get().getCollectState())
							{
								Log.note("Sending FPE abort on shutter close click");
								DevFPE.get().abort();
							}
							
							fastShutter.close(); 
						}
						else 
						{
							if(DevInOut.getStatus("Pindiode").equals("OUT") && DevInOut.getStatus("Beamstop").equals("OUT"))
							{
								Log.error("Both beamstop and pindiode are not in place");
								return;
							}
							fastShutter.open();
						}
					}
				});
		
		DevShutter.get("shutter").addListener(new DevShutter.Listener()
		{
			@Override public void handleStateChange(DevShutter shutter, State state) {}
			@Override public void handleOpenChange(DevShutter shutter, boolean open)
			{
				swShutter.setValue(open);
			}
		});
		swShutter.setValue(DevShutter.get("shutter").getIsOpen());
		
		connectListeners();
	}

	public void connectListeners()
	{
		DevFPE.get().etaListener.addListener(new ListenerList.Listener<DevFPE>()
    	{
    		@Override public void handleEvent(DevFPE object, Object data)
    		{
    			updateETA();
    		}
    	});
		
		ModelCollectRunList.get().addCurrentRunListener(new CurrentRunListener()
		{
			@Override public void currentRunChanged(ModelRun run)
			{
				updateETA();
			}
		});
		
		ModelRasterState.get().llCurrentRun.addListener(new ListenerList.Listener<ModelRasterState>()
    	{
    		@Override public void handleEvent(ModelRasterState object, Object data)
    		{
    			updateETA();
    		}
    	});
		
		WdgMain.get().llTabChanged.addListener(new ListenerList.Listener<WdgMain>()
		{
			@Override public void handleEvent(WdgMain object, Object data)
			{
				updateETA();
			}
		});
		
		updateETA();
	}
	
	private volatile double lastETA = -1.0;
	private void updateETA()
	{
		double etaToSet = 0.0;
		double activeETA = (double)ETA.calcActiveETA();
		if (0.0 != activeETA)
		{
			etaToSet = activeETA;
		}
		else
		{
			Control currentControl = WdgMain.get().lastTab;
			if (currentControl==TabCollect.get())
			{
				etaToSet = -1.0*ETA.calcCollectETA().totalSeconds;
			}
			else if (currentControl==TabRaster.get())
			{
				etaToSet = -1.0*ETA.calcRasterETA().totalSeconds;
			}
		}
		if (etaToSet != lastETA)
		{
			final double f_etaToSet = etaToSet;
			GUI.asyncExec(new Runnable(){public void run()
			{
				swETA.setValue(f_etaToSet);
				lastETA = f_etaToSet;
			}});
		}
	}
}
