package gov.anl.gmca.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.MouseTrackListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgStatusElement
{
	public Label lbl, status, units;
	private Color okStatusColor = GUI.COLOR_BLACK;
	public void setOkStatusColor(Color c) { okStatusColor=c; }

	interface InterpretStatus
	{
		public boolean isValOK(Object value);
		public String getString(Object value);
	}
	
	public void setValue(Object value)
	{
		currentValue = value;
		GUI.asyncExec(new Runnable(){public void run()
		{
			updateStatus();
		}});
	}
	
	public WdgStatusElement(Composite parent, String labelName, String tooltipText, String initialStatus, int statusWidth, boolean isInverse, String unitsString, InterpretStatus interpretStatus, final Runnable onClick)
	{
		GridData gd;
		
		this.initialStatus		= initialStatus;
		this.isInverse			= isInverse;
		this.interpretStatus	= interpretStatus;
		this.isClickable		= (null!=onClick);
		
		lbl = new Label(parent,SWT.RIGHT);
		lbl.setText(labelName);
		lbl.setFont(GUI.FONT_STATUS_LABEL);
		lbl.setBackground(GUI.COLOR_STATUS_BG);
		gd = new GridData(SWT.FILL, SWT.CENTER, true, true);
		lbl.setLayoutData(gd);

		status = new Label(parent,SWT.BORDER);
		status.setFont(GUI.FONT_STATUS_DISPLAY);
		status.setAlignment(SWT.CENTER);
		gd = new GridData();
		gd.grabExcessHorizontalSpace=false;
		gd.grabExcessVerticalSpace=true;
		gd.widthHint = statusWidth;
		status.setLayoutData(gd);
		updateStatus();
		
		if (null != tooltipText)
		{
			status.setToolTipText(tooltipText);
		}
		
		if (isClickable)
		{
    		status.addMouseListener(new MouseListener()
    		{
    			@Override public void mouseDoubleClick(MouseEvent arg0) {}
    			@Override public void mouseDown(MouseEvent arg0){ isMouseDown = true; 	updateStatus(); }
    			@Override public void mouseUp(MouseEvent me)	{ isMouseDown = false;	updateStatus();	if (isMouseHovering && isInBounds(me.x,me.y)) onClick.run();	}
    		});
			// this listener tracks motion while the mouse is down - MouseTrackListener does not
    		status.addMouseMoveListener(new MouseMoveListener()
    		{
				@Override public void mouseMove(MouseEvent me)
				{
					isMouseHovering = isInBounds(me.x,me.y);
					updateStatus();
				}
			});
    		// but MouseTrackListener is needed for when the mouse leaves the area, since MouseMove won't be called
    		status.addMouseTrackListener(new MouseTrackListener()
    		{
				@Override public void mouseEnter(MouseEvent arg0) {}
				@Override public void mouseExit(MouseEvent arg0)
				{
					isMouseHovering = false;
					updateStatus();
				}
				@Override public void mouseHover(MouseEvent arg0) {}
    		});
		}
		
		if (null != unitsString)
		{
			units = new Label(parent,SWT.NONE);
			units.setText(unitsString);
			units.setFont(GUI.FONT_STATUS_UNITS);
			units.setBackground(GUI.COLOR_STATUS_BG);
			gd = new GridData(SWT.FILL, SWT.CENTER, true, true);
			units.setLayoutData(gd);
		}
	}
	
	// these variables determine the colors and text of the status widget
	protected boolean 			isMouseHovering = false;
	protected boolean			isMouseDown		= false;
	protected Object			currentValue 	= null;
	protected String			initialStatus	= "";
	protected InterpretStatus	interpretStatus	= null;
	protected boolean			isInverse		= false;
	protected boolean			isClickable		= false;
	
	// isInverse==true: green bg if status is OK, red bg otherwise, always white fg
	// isInverse==false: always normal bg color, black fg if ok, red otherwise
	protected void updateStatus()
	{
		Color bg,fg;

		// interpret text first to set colors 
		String text = (null==currentValue)?initialStatus:interpretStatus.getString(currentValue);
		
		// determine base colors 
		boolean isStatusOK = (null==currentValue)?true:interpretStatus.isValOK(currentValue);
		if (isInverse)
		{
			fg = GUI.COLOR_WHITE;
			bg = isStatusOK?GUI.COLOR_GREEN:GUI.COLOR_RED;
		}
		else
		{
			fg = isStatusOK?okStatusColor:GUI.COLOR_RED;
			bg = GUI.COLOR_STATUS_BG;
		}

		// add hover highlight
		if (isMouseHovering && isClickable)
		{
    		if (GUI.COLOR_GREEN 	== bg)	bg = GUI.COLOR_LIGHT_GREEN;
    		if (GUI.COLOR_RED		== bg)	bg = GUI.COLOR_LIGHT_RED;
    		if (GUI.COLOR_STATUS_BG	== bg)	bg = GUI.COLOR_WHITE;
		}
		
		// override colors on click
		if (isMouseDown && isMouseHovering && isClickable)
		{
			fg = GUI.COLOR_WHITE;
			bg = GUI.COLOR_DARK_GRAY;
		}
		
		status.setForeground(fg);
		status.setBackground(bg);
		status.setText(text);
	}
	
	protected boolean isInBounds(int x, int y)
	{
		return (x>=0 && y>=0 && x<status.getBounds().width && y<status.getBounds().height);
	}
}