package gov.anl.gmca.gui.options;

import gov.anl.gmca.gui.GUI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import java.util.ArrayList;

public class WdgControlOption
{
	public static interface OptionHandler { public void handleSelection(int i); }

	private Label label;
	private ArrayList<Button> buttons = new ArrayList<Button>();
	
	public WdgControlOption(Composite parent, String text, String[] options, final OptionHandler oh, boolean onUserTab)
	{
		GridData gd;
		
		label = new Label(parent, SWT.NONE);
		label.setText(text);
		if (onUserTab) label.setFont(GUI.FONT_10_BOLD);
		gd = onUserTab ? new GridData(GridData.FILL_HORIZONTAL) : new GridData();
		label.setLayoutData(gd);
		
		Composite radioComp = new Composite(parent, SWT.NONE);
		if (!onUserTab) radioComp.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		GridLayout gl = new GridLayout(1,false);
		gl.marginWidth=0;
		gl.marginHeight=0;
		gl.verticalSpacing=0;
		radioComp.setLayout(gl);
		
		for (int i=0; i<options.length; i++)
		{
			Button button = new Button(radioComp, SWT.RADIO);
			button.setText(options[i]);
			button.setData(true);
			buttons.add(button);
			final int buttonIndex=i;
			button.addSelectionListener(new SelectionListener()
			{
				@Override public void widgetSelected(SelectionEvent e)
				{
					oh.handleSelection(buttonIndex);
				}
				@Override public void widgetDefaultSelected(SelectionEvent e) {}
			});
		}
		buttons.get(0).setSelection(true);
		
		if (onUserTab)
		{
			Composite spacer = new Composite(parent, SWT.NONE);
			gd = new GridData();
			gd.heightHint = 18;
			spacer.setLayoutData(gd);
		}
	}

	public void select(int selectedIndex)
	{
		for (int i=0; i<buttons.size(); i++)
		{
			buttons.get(i).setSelection(i==selectedIndex);
		}
	}

	// enable/disable a single choice
	public void setEnabled(int choiceIndex, boolean enable)
	{
		Button b = buttons.get(choiceIndex);
		b.setEnabled(enable);
		b.setData(enable); // save if this button was specifically disabled
	}
	
	// enable/disable all sub-widgets
	public void setEnabled(boolean enable)
	{
		for (Button button : buttons)
		{
			Boolean data = (Boolean)button.getData();
			if (null==data)
			{
				button.setEnabled(enable);
			}
			else
			{
				button.setEnabled(data && enable);
			}
		}
	}
}

