package gov.anl.gmca.gui.state;

import com.github.wnameless.json.flattener.FlattenMode;
import com.github.wnameless.json.flattener.JsonFlattener;
import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.Unirest;
import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.client_server.StateEvents;
import gov.anl.gmca.common.client_server.StateMap;
import gov.anl.gmca.common.client_server.ui.DlgServerView;
import gov.anl.gmca.hutch.TabHutch;
import gov.anl.gmca.server.HTTP;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable;
import gov.anl.gmca.util.Util;

import java.util.Map;
import java.util.UUID;

public class JBServer extends Server
{
    ////////////////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////// Singleton and init() //////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////

    private static String uuid;
    private static JBServer inst = new JBServer();
    public static JBServer get() { return inst; }

    public boolean init()
    {
        uuid = UUID.randomUUID().toString();
        restartCmd = "jbs-restart";
        serverName = "JBluIce server";
        url = Util.getServerURL();
        return startReaderThread();
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////// update() ////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////

    public synchronized boolean update()
    {
        if (Util.atExit()) return false;

        try
        {
            // HTTP request
            HttpResponse<String> resp = Unirest.post(url+"/state")
                    .field("host", ModelUserTable.Client.getShortHostname())
                    .field("isMaster", ModelUserTable.Client.isMaster()?"1":"0")
                    .field("pid", ""+ModelUserTable.Client.getPid())
                    .field("username", Util.getUserLogin())
                    .field("clientID", uuid)
                    .asString();
            if (200 != resp.getStatus())
            {
                Log.error("JBluIce server /state returned code " + resp.getStatus());
                return false;
            }

            // get JSON text
            String jsonText=resp.getBody();
            DlgServerView.get().jsonChange(jsonText);

            // convert to object and map
            Map<String,Object> newFlatMap = new JsonFlattener(jsonText).withFlattenMode(FlattenMode.KEEP_ARRAYS).flattenAsMap();
            StateMap newObjMap = HTTP.gson.fromJson(jsonText, StateMap.class);

            // update references
            StateMap oldState = StateMap.current;
            StateMap.current = newObjMap;

            Map<String,Object> oldFlatMap = StateMap.currentMap;
            StateMap.currentMap = newFlatMap;

            AppState.handleStateUpdate();
            if (null!=TabHutch.get() && null!=TabHutch.get().wdgAlignMiniBeam)
                TabHutch.get().wdgAlignMiniBeam.stateChanged();

            // find differences
            if (null != oldFlatMap)
                for (String key : newFlatMap.keySet())
                {
                    String oldVal = StateMap.getKey(oldFlatMap, key);
                    String newVal = StateMap.getKey(newFlatMap, key);
                    if (!oldVal.equals(newVal)) {
                        StateEvents.callListeners(key, newVal);
                        DlgServerView.get().statusChange(key+"="+newVal);
                    }
                }
        }
        catch (Exception e) { e.printStackTrace(); return false; }
        return true;
    }

}
