package gov.anl.gmca.gui.state;

import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.Unirest;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.client_server.CounterState;
import gov.anl.gmca.common.client_server.ui.DlgStreamWriterView;
import gov.anl.gmca.gui.*;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.server.HTTP;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;

public class SWServer extends Server
{
    ////////////////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////// Singleton and init() //////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////

    private static SWServer inst = new SWServer();
    public static SWServer get() { return inst; }

    public boolean init()
    {
        if (!DetectorConfig.isEiger()) return true;

        restartCmd = "sw-restart";
        serverName = "StreamWriter server";
        url = "http://"+DBConfig.get("streamwriter_ip")+":"+DBConfig.get("streamwriter_port");
        return startReaderThread();
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////// update() ////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////

    public synchronized boolean update()
    {
        if (!DetectorConfig.isEiger() || Main.SIM_FPE) return true;
        try
        {
            // Load JSON state
            HttpResponse<String> resp = Unirest.get(url+"/counter").asString();
            if (200 != resp.getStatus()) {
                Log.error("StreamWriter /counter response error: " + resp.getStatus());
                return false;
            }
            String jsonText = resp.getBody();
            if (GUIStatus.inGUIMode) DlgStreamWriterView.get().jsonChange(jsonText);

            // fill counter state object
            CounterState newState = HTTP.gson.fromJson(jsonText, CounterState.class);
            CounterState oldState = CounterState.current;
            CounterState.current = newState;

            // check for changes
            // TODO: avoid checking each variable individually
            boolean change = false;
            if (null!=oldState) {
                if (newState.counter.latestCounter != oldState.counter.latestCounter) {
                    change = true;
                    if (GUIStatus.inGUIMode) DlgStreamWriterView.get().statusChange("latestCounter=" + newState.current.counter.latestCounter);
                }
                if (newState.counter.completeCounter != oldState.counter.completeCounter) {
                    change = true;
                    if (GUIStatus.inGUIMode) DlgStreamWriterView.get().statusChange("completeCounter=" + newState.current.counter.completeCounter);
                }
                if (newState.counter.streamInProgress != oldState.counter.streamInProgress) {
                    change = true;
                    if (GUIStatus.inGUIMode) DlgStreamWriterView.get().statusChange("streamInProgress=" + newState.current.counter.streamInProgress);
                }
            }
            if (change)
            {
                lastCounterChangeMs = System.currentTimeMillis();
                System.out.println("Counter: "+jsonText);
                llChange.notifyListeners(newState.current.counter);
            }
        }
        catch (Exception e) { e.printStackTrace(); return false; }
        return true;
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////// Change listeners ////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////

    public static ListenerList<CounterState.CounterInfo> llChange = new ListenerList<>(null);
    public static long lastCounterChangeMs = 0;
    public static long elapsedSinceCounterChangeMs() { return System.currentTimeMillis()-lastCounterChangeMs; }
}
