package gov.anl.gmca.hutch;

import gov.anl.gmca.common.Command;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.DevMove.MoveCmd;
import gov.anl.gmca.common.DevMove.MoveResult;
import gov.anl.gmca.common.DevMove.ResultType;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.sample.DevCollimatorCtrl;
import gov.anl.gmca.sample.DevCollimatorCtrl.CCMove;
import gov.anl.gmca.sample.DevInOut.InOut;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import gov.aps.jca.CAException;

public class CmdEnterHutch extends Command
{
	public static ListenerList<CmdEnterHutch> listenerList = new ListenerList<CmdEnterHutch>(null);
	public static Thread thread = null;
	public static boolean isRunning() { return (null != thread && thread.isAlive()); }
	
	public CmdEnterHutch() {}
	
	@Override
	public void run()
	{
		thread = Thread.currentThread();
		
		System.out.println("Notifying enter hutch listeners");
		listenerList.notifyListeners();
		
	    Log.note("Enter hutch procedure started...");

	    // close endstation shutter
    	DevHutchShutter esShutter = DevHutchShutter.get("endStation");
	    try
	    {
	    	esShutter.close();
	    }
	    catch (CAException cae)
	    {
	    	Log.error("Couldn't close endstation shutter: "+cae.getMessage());
	    }
	    
	    CCMove move = DevCollimatorCtrl.get().newMove();
	    move.setDest("collimator",		InOut.OUT);
	    move.setDest("backlight",		InOut.OUT);
	    move.setDest("beamstop",		InOut.OUT);
	    move.setDest("pindiode",		InOut.OUT);
	    move.setDest("ringlight_value",	"off");
	    move.start();
	    
	    // detector to detector_back_pos
	    double distance = Double.parseDouble(DBConfig.get("detector_back_pos"));
    	MoveCmd detCmd = DevPmacMotor.get("detector_z").startMove(distance);
    	
    	esShutter.waitFor();
	    MoveResult ccResult = move.waitFor();
    	MoveResult detResult = detCmd.waitFor();
    	
    	boolean success = true;
    	if (esShutter.isOpen())						{ Log.error("Enter hutch: Couldn't close endstation shutter"); 	success = false; }
    	if (ResultType.SUCCESS != ccResult.type)	{ Log.error("Enter hutch: Collimator move failed"); 			success = false; }
    	if (ResultType.SUCCESS != detResult.type)	{ Log.error("Enter hutch: Detector move failed");				success = false; }
    	if (!success)								{ Log.error("Enter hutch procedure encountered errors."); }
    	if (success)								{ Log.note("Enter hutch procedure completed successfully."); }

		//Util.sleep(5000);
		Log.note("Enter hutch procedure complete.");

    	// TODO: find a better way to do this that doesn't potentially hold up the GUI thread
    	GUI.asyncExec(new Runnable(){public void run()
    	{
    		while (null != thread && thread.isAlive()) Util.sleep(10);
    		listenerList.notifyListeners();
    	}});
	}
}
