package gov.anl.gmca.hutch;

import gov.anl.gmca.collect.DevBinMode.BinMode;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.gui.*;
import gov.anl.gmca.util.Point2D;

public class DetectorConfig
{
	public static enum DETECTOR_TYPE { EIGER, PILATUS, MAR };
	
	private static final Object[][] DETECTOR_SPECS =
	{
		// serial number	36um bin#	72um bin#	144um bin#	Det Type				width x height (mm)		pixel WxH (mm),		ImgSrv ID
		{"015",			    1,			2,			4,			DETECTOR_TYPE.MAR,		300.0,	300.0,			0.03662, 0.03662,	"MARCCD300"},  // currently ID-B
		{"013",			    null,		1,			2,			DETECTOR_TYPE.MAR,		300.0,	300.0,			0.03662, 0.03662,	"MARCCD300"},  // currently ID-D
		{"0006",			null,		1,			2,			DETECTOR_TYPE.MAR,		225.0,	225.0,			0.03662, 0.03662,	"MARCCD225"},  // currently BM
		{"2-0106",			null,		null,		null,		DETECTOR_TYPE.PILATUS,	83.8,	70.0,			0.17200, 0.17200,	null},   	   // Pilatus 200k
		{"60-0122",			null,		null,		null,		DETECTOR_TYPE.PILATUS,	423.636,434.644,		0.17200, 0.17200,	"PILATUS3 6M"},// Pilatus 6M
		{"P2MF",			null,		null,		null,		DETECTOR_TYPE.PILATUS,	253.7,	288.788,		0.17200, 0.17200,	"PILATUS 2M-F"},
		{"E-32-0104",		null,		null,		null,		DETECTOR_TYPE.EIGER,	311.2,	327.8,			0.07500, 0.07500,	"EIGER"},
		{"DP-P073-04",		null,		null,		null,		DETECTOR_TYPE.EIGER,	155.1,	162.2,			0.07500, 0.07500,	"EIGER"},
	};
	
	public static DetectorConfig inst = null;
	public static DetectorConfig get() { return inst; }
	
	public DetectorConfig()
	{
		inst=this;
	}
	
	// if the ID is matched, the size is returned via lookup table
	// otherwise the image path is used and the header is read via the server
	public static Point2D getImageSizeMM(String imgSrvDetectorID, String imgPath)
	{
		for (Object[] row : DETECTOR_SPECS)
		{
			String id = (String)row[9];
			if (null!=id && id.equals(imgSrvDetectorID))
			{
				return new Point2D((double)row[5],(double)row[6]);
			}
		}
		return null; // TODO: implement header lookup
	}
	
	private static synchronized Object[] getDetectorRow()
	{
		if (Main.SIM_FPE) return DETECTOR_SPECS[6];

		String serial = DBConfig.get("detector_serial");
		for (Object[] detectorRow : DETECTOR_SPECS)
		{
			if (((String)detectorRow[0]).equals(serial))
			{
				return detectorRow;
			}
		}
		throw new IllegalStateException("Invalid detector serial: "+serial);
	}
	
	public static Integer getBinArgument(BinMode mode)
	{
		Object[] detectorRow = getDetectorRow();
		int colIndex = mode.ordinal()+1;
		return (Integer)detectorRow[colIndex];
	}

	public static Point2D getPixelSizeMM()
	{
		Object[] detectorRow = getDetectorRow();
		return new Point2D((double)detectorRow[7],(double)detectorRow[8]);
	}
	
	public static DETECTOR_TYPE getType()
	{
		Object[] detectorRow = getDetectorRow();
		return (DETECTOR_TYPE)(detectorRow[4]);
	}
	
	public static boolean isDectris()	{ return getType() == DETECTOR_TYPE.EIGER || getType() == DETECTOR_TYPE.PILATUS; }
	public static boolean isEiger() 	{ return getType() == DETECTOR_TYPE.EIGER; 		}
	public static boolean isPilatus() 	{ return getType() == DETECTOR_TYPE.PILATUS; 	}
	public static boolean isMAR()		{ return getType() == DETECTOR_TYPE.MAR; 		}
	
	public static double getHeightWidthRatio()
	{
		return getDimensionsMM().y / getDimensionsMM().x;
	}
	
	public static Point2D getDimensionsMM()
	{
		Object[] detectorRow = getDetectorRow();
		return new Point2D((Double)detectorRow[5],(Double)detectorRow[6]);
	}
}
