package gov.anl.gmca.hutch;

import gov.anl.gmca.common.DevScript;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;
import gov.aps.jca.dbr.DBR;

public class DevAutocenter extends DevScript
{
	public enum CENTER_MODE { LOOP, CRYSTAL };
	public static CENTER_MODE mode = CENTER_MODE.LOOP;

	protected static String loop_center_cmd = "";
	protected static String crystal_center_cmd = "";

	protected static DevAutocenter inst = null;
	public static DevAutocenter get() { return inst; }
	
	// for now both will run crystal centering
	public void syncExecLoop() { init(loop_center_cmd); mode=CENTER_MODE.LOOP; _syncExec(); }
	public void syncExecCrystal() { init(crystal_center_cmd); mode=CENTER_MODE.CRYSTAL; _syncExec(); }
	
	private void init(String cmd)
	{
		task = ExecTask.new_ExecTask(cmd, "autocenter", true);
	}
	
	public DevAutocenter()
	{
		super("autocenter");
		inst = this;

		// read SQL config
		Database.Result pmaResult = Database.exec(Database.DBName.CONFIG, "select * from Scripts");
		for (int i=0;i<pmaResult.numRows();i++) {
			String id = pmaResult.getRow(i).get("ID");
			String path = pmaResult.getRow(i).get("Path");
			String args = pmaResult.getRow(i).get("Arguments");
			if (id.equals("loop_center")) {
				loop_center_cmd = path+" "+args;
			} else if (id.equals("crystal_center")) {
				crystal_center_cmd = path+" "+args;
			}
		}

		Epics.addMonitor(PV_BLUICE.autoCenterRun, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
			int intVal = Epics.dbrToInt(value);
			if (0 == intVal) stopExec();
			}
		});
	}
	
	// override default _syncExec() to add PV changes
	@Override public void _syncExec()
	{
		Epics.putInteger(PV_BLUICE.autoCenterRun, 1);
		super._syncExec();
		Epics.putInteger(PV_BLUICE.autoCenterRun, 0);
	}
	
	@Override public void stopExec()
	{
		if (null == task || !task.isRunning()) return;
		
		task.asyncStop();
		
		Epics.putInteger(PV_BLUICE.autoCenterRun,0);
	}
}
