package gov.anl.gmca.hutch;

import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;
import gov.anl.gmca.raster.util.ClearSampleState;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.aps.jca.dbr.DBR;

// PV changes from 0 to 1 on pin unmount
public class DevPinSensor
{
	public static ListenerList<DevPinSensor> llDismount = new ListenerList<DevPinSensor>(null);
	
	public static void init()
	{
		Epics.addMonitor(PV_BLUICE.nopin, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				if (Epics.dbrToInt(value)==1 && Client.isMaster())
				{
					handleFlag();
				}
			}
		});
		
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				if (isMaster && Epics.getInteger(PV_BLUICE.nopin)==1)
				{
					handleFlag();
				}
			}
		});
	}
	
	private static void handleFlag()
	{
		clearState();
		llDismount.notifyListeners();
		 
		String msg = "The pin sensor detected a sample dismount.\n";
		ClearSampleState.clearAllRelatedState(msg, false, false);
	}

	// calling this will clear the nopin PV
	// nopin can't be set directly because of an issue with callbacks not being processed in that case
	public static void clearState()
	{
	    Epics.putInteger(PV_BLUICE.nopin$B, 1);
	}
}
