package gov.anl.gmca.hutch;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.client_server.RPC;
import gov.anl.gmca.common.client_server.StateMap;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import java.text.Format;
import java.text.SimpleDateFormat;

public class WdgAlignMiniBeam extends Composite /*implements PaintListener, ControlListener*/
{
	private WdgAlignPlot hMirrorPlot, vMirrorPlot;
	private Label lblRunTime, lblEnergy;
	private Button autoalignButton;
	private int widgetWidth, widgetHeight;
	private Format dateFormat;

	public WdgAlignMiniBeam(Composite parent)
	{
		super(parent, SWT.NONE);
		setBackground(GUI.BGCOLOR);

		dateFormat = new SimpleDateFormat("E MMM d, h:mma");
		
		GridLayout gl = new GridLayout(2,false);
		//gl.horizontalSpacing=0;
		//gl.verticalSpacing=0;
		gl.marginWidth=8;
		gl.marginBottom=8;
		setLayout(gl);
		
		vMirrorPlot = new WdgAlignPlot(this, SWT.VERTICAL);
		vMirrorPlot.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
		
		hMirrorPlot = new WdgAlignPlot(this, SWT.HORIZONTAL);
		hMirrorPlot.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));

		Composite lblComp = new Composite(this, SWT.BORDER);
		lblComp.setBackground(GUI.COLOR_LIGHT_GRAY);
		gl = new GridLayout(1,false);
		lblComp.setLayout(gl);
		lblComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		lblRunTime = new Label(lblComp, SWT.LEFT);
		lblRunTime.setFont(GUI.FONT_11);
		lblRunTime.setBackground(lblRunTime.getParent().getBackground());
		lblRunTime.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		lblRunTime.setText("Last run: N/A");
		
		lblEnergy = new Label(lblComp, SWT.LEFT);
		lblEnergy.setFont(GUI.FONT_11);
		lblEnergy.setBackground(lblRunTime.getParent().getBackground());
		lblEnergy.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		lblEnergy.setText("Last energy: N/A");
		
		autoalignButton = new Button(this, SWT.PUSH);
		GridData gd = new GridData(SWT.RIGHT, SWT.FILL, false, false);
		gd.widthHint = 200;
		gd.heightHint = 50;
	 	autoalignButton.setBackground(GUI.COLOR_BLUE_BUTTON);
	 	autoalignButton.setText("Optimize Beam");
	 	autoalignButton.setImage(GUI.IMAGE_OPTIMIZE);
	 	autoalignButton.setFont(GUI.FONT_11);
	 	autoalignButton.setLayoutData(gd);
	 	autoalignButton.setEnabled(false);

		connectListeners();
	}

	public void stateChanged()
	{
		StateMap.StateOptimizeBeam optimize = StateMap.current.optimizeBeam;
		handleGraphData(optimize.hGraph, SWT.HORIZONTAL);
		handleGraphData(optimize.vGraph, SWT.VERTICAL);
		updateLabels(optimize.lastEnergy_keV, optimize.lastRunTime_unix);
		updateEnableState(optimize.cmdRunning);
	}

	private static boolean graphEquals(StateMap.StateOptimizeBeamGraph a, StateMap.StateOptimizeBeamGraph b)
	{
		if (a.pointCount != b.pointCount) return false;
		for (int i=0; i<a.pointCount; i++) {
			if (a.xPoints[i] != b.xPoints[i]) return false;
			if (a.yPoints[i] != b.yPoints[i]) return false;
		}
		return true;
	}

	StateMap.StateOptimizeBeamGraph lastHGraph, lastVGraph;
	private void handleGraphData(StateMap.StateOptimizeBeamGraph graph, int orientation)
	{
		if (orientation==SWT.HORIZONTAL) {
			if (null==lastHGraph || !graphEquals(lastHGraph, graph)) {
				lastHGraph = graph;
				GUI.asyncExec(()->{
					hMirrorPlot.data = graph;
					hMirrorPlot.redraw();
				});
			}
		} else {
			if (null==lastVGraph || !graphEquals(lastVGraph, graph)) {
				lastVGraph = graph;
				GUI.asyncExec(()->{
					vMirrorPlot.data = graph;
					vMirrorPlot.redraw();
				});
			}
		}
	}
	
	private void connectListeners()
	{
		//StateEvents.addListener("optimizeBeam", (key, value) -> stateChanged());

		autoalignButton.addSelectionListener(new SelectionListener()
	 	{
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				//DevAlignMinibeam.get().asyncExec();
				RPC.sendCommand("cmd:OptimizeBeam");
			}
	 	});

		AppState.addListener(AppState.Operation.OP_ALIGN_MINIBEAM, new AppState.Listener(){@Override public void stateChanged(Operation op, boolean allowed) { GUI.asyncExec(new Runnable(){public void run()
	 	{
	 		updateEnableState(StateMap.current.optimizeBeam.cmdRunning);
	 	}});}});
		
		Client.addListener(new ClientListener(){ @Override public void masterChanged(boolean isMaster)
		{
			updateEnableState(StateMap.current.optimizeBeam.cmdRunning);
		}});

		updateEnableState(StateMap.current.optimizeBeam.cmdRunning);
	}

	private Boolean getEnableState(Boolean cmdRunning)
	{
		return
			Client.isMaster() &&
			!cmdRunning &&
			AppState.isOperationAllowed(Operation.OP_ALIGN_MINIBEAM);
	}

	Boolean lastEnabled = null;
	private void updateEnableState(Boolean cmdRunning)
	{
		if (lastEnabled != getEnableState(cmdRunning)) {
			GUI.asyncExec(()->{
				Boolean enable = getEnableState(cmdRunning);
				autoalignButton.setEnabled(enable);
				lastEnabled = enable;
			});
		}
	}

	private Double lastLastEnergy = null;
	private Long lastLastRunTime = null;
	private void updateLabels(Double lastEnergy_keV, Long lastRunTime_unix)
	{
		if (lastLastEnergy==lastEnergy_keV && lastLastRunTime==lastRunTime_unix) return;
		lastLastEnergy = lastEnergy_keV;
		lastLastRunTime = lastRunTime_unix;

		GUI.asyncExec(new Runnable()
		{
			@Override public void run()
			{
				String str = dateFormat.format(lastRunTime_unix*1000);
				lblRunTime.setText("Last run: "+str);

				if (lastEnergy_keV>0.0)
				{
					String str2 = Util.sprintf("Last energy: %.3f keV", lastEnergy_keV);
					lblEnergy.setText(str2);
				}
			}
		});
	}

	/*private void handleData(AlignData data)
	{
		final WdgAlignPlot plotLayer = (data.orientation==SWT.VERTICAL?vMirrorPlot:hMirrorPlot);
		plotLayer.data = data;

		GUI.asyncExec(new Runnable(){public void run()
		{
			plotLayer.redraw();
		}});
	}*/

	/*@Override public void paintControl(PaintEvent e)
	{
		// background
		e.gc.setBackground(GUI.COLOR_WHITE);
		e.gc.fillRectangle(e.x, e.y, e.width, e.height);

		// border and divider
		e.gc.setForeground(GUI.COLOR_BLACK);
		e.gc.drawRectangle(0, 0, widgetWidth-1, widgetHeight-1);
		e.gc.drawLine(0, widgetHeight/2, widgetWidth-1, widgetHeight/2);

		// draw each graph
		vMirrorPlot.draw(e.gc);
		hMirrorPlot.draw(e.gc);
	}*/
}
