package gov.anl.gmca.hutch;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.DevScript;
import gov.anl.gmca.common.client_server.RPC;
import gov.anl.gmca.common.client_server.StateMap;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

public class WdgAutomation extends Composite
{
	private Button enterHutchButton;
	private Button autoButton, stopButton, loopButton;
	private CmdEnterHutch enterHutchCmd = null;
	private static final int BUTTON_WIDTH = 158;
	
	public WdgAutomation(Composite parent, int style)
	{
		super(parent,style);
		setLayout(new FillLayout());
		setBackground(GUI.BGCOLOR);
		
		Group group = new Group(this,SWT.None);
		group.setText("Automation");
		group.setFont(GUI.FONT_HUTCH_GROUP);
		group.setBackground(GUI.BGCOLOR);
		group.setLayout(new GridLayout(1,false));
		
		GridData gd = new GridData();
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		gd.widthHint = BUTTON_WIDTH;
		loopButton = new Button(group, SWT.PUSH);
		loopButton.setText("Loop Centering");
		loopButton.setFont(GUI.FONT_11);
		loopButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		loopButton.setEnabled(false);
		loopButton.setLayoutData(gd);
		loopButton.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				//DevAutocenter.get().asyncExecLoop();
				RPC.sendCommand("cmd:AutoCenter", "mode:loop");
			}
	    });
		
		gd = new GridData();
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		gd.widthHint = BUTTON_WIDTH;
		autoButton = new Button(group, SWT.PUSH);
		autoButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		autoButton.setText("Crystal Centering");
		autoButton.setFont(GUI.FONT_11);
		autoButton.setEnabled(false);
		autoButton.setLayoutData(gd);
		autoButton.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				//DevAutocenter.get().asyncExecCrystal();
				RPC.sendCommand("cmd:AutoCenter", "mode:crystal");
			}
	    });
		
		gd = new GridData();
		gd.heightHint = 30;
		gd.horizontalAlignment = SWT.CENTER;
		stopButton = new Button(group, SWT.PUSH);
		stopButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		stopButton.setImage(GUI.IMAGE_STOP);
		stopButton.setText("Stop");
		stopButton.setFont(GUI.FONT_11);
		stopButton.setEnabled(false);
		stopButton.setLayoutData(gd);
		stopButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					//DevAutocenter.get().stopExec();
					RPC.sendCommand("cmd:StopAutoCenter");
				}});
			}
		});
	
		addSeparator(group); // -------------------------------------------------
		
		gd = new GridData();
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		gd.widthHint = BUTTON_WIDTH;
		enterHutchButton = new Button(group,SWT.PUSH);
		enterHutchButton.setText("Enter Hutch");
		enterHutchButton.setImage(GUI.IMAGE_ENTER);
		enterHutchButton.setFont(GUI.FONT_11);
		enterHutchButton.setLayoutData(gd);
		enterHutchButton.setEnabled(false);
		enterHutchButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		enterHutchButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				/*if (null != enterHutchCmd && enterHutchCmd.isAlive())
				{
					Log.error("Enter hutch is already running");
					return;
				}*/
				
				//enterHutchCmd = new CmdEnterHutch();
				//enterHutchCmd.start();
				RPC.sendCommand("cmd:EnterHutch");
			}
		});
		
	 	connectListeners();
	}
	
	private void addSeparator(Group group)
	{
		GridData gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		Label separator3 = new Label(group, SWT.SEPARATOR | SWT.HORIZONTAL);
		separator3.setLayoutData(gd);
	}
	
	private void connectListeners()
	{
	 	AppState.addListener(AppState.Operation.OP_AUTOCENTER, new AppState.Listener(){@Override public void stateChanged(Operation op, boolean allowed) { GUI.asyncExec(new Runnable(){public void run()
	 	{
	 		updateEnableState(); 
	 	}});}});
	 	
	 	AppState.addListener(AppState.Operation.OP_ALIGN_MINIBEAM, new AppState.Listener(){@Override public void stateChanged(Operation op, boolean allowed) { GUI.asyncExec(new Runnable(){public void run()
	 	{
	 		updateEnableState(); 
	 	}});}});
	 	
	 	AppState.addListener(AppState.Operation.OP_ENTER_HUTCH, new AppState.Listener(){@Override public void stateChanged(Operation op, boolean allowed) { GUI.asyncExec(new Runnable(){public void run()
	 	{
	 		updateEnableState(); 
	 	}});}});
	 	
		Client.addListener(new ClientListener(){ @Override public void masterChanged(boolean isMaster)
		{
			updateEnableState();
		}});
	 	
	 	DevAutocenter.get().ll.addListener(new ListenerList.Listener<DevScript>(){@Override public void handleEvent(DevScript object, Object data)
		{
			updateEnableState();
		}});
	}
	
	private void updateEnableState()
	{
		final boolean isEnterHutchRunning = (null != enterHutchCmd && enterHutchCmd.isAlive());
		final boolean isAutocenterRunning 	= StateMap.current.autoCenter.cmdRunning;
		
		final boolean enableCenter 			= (Client.isMaster() && !isAutocenterRunning && AppState.isOperationAllowed(Operation.OP_AUTOCENTER));
		final boolean enableStopCenter 		= (Client.isMaster() && isAutocenterRunning);
		final boolean enableEH 				= (Client.isMaster() && !isEnterHutchRunning && AppState.isOperationAllowed(Operation.OP_ENTER_HUTCH));

		GUI.asyncExec(new Runnable(){public void run()
		{
			loopButton.setEnabled(enableCenter);
			autoButton.setEnabled(enableCenter);
			stopButton.setEnabled(enableStopCenter);
			
			enterHutchButton.setEnabled(enableEH);
		}});
	}
}
