package gov.anl.gmca.hutch;

import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.Unirest;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import java.io.InputStream;

/*
 * After creating this widget, call the three public set functions to set the parameters it depends on.
 * They should be called at startup and whenever they change.
 */
public class WdgResPredictor extends Composite implements PaintListener
{
	protected Composite comp;
	protected Group group;

	protected double detectorZ_mm = Double.NaN;
	protected double beamEnergy_eV = Double.NaN;
	protected double detector2t_deg = Double.NaN;
	public void setDetectorZ_mm(double d, boolean doRedraw)		{ detectorZ_mm = d; 	if (doRedraw) GUI.asyncExec(new Runnable(){public void run(){ comp.redraw(); }}); }
	public void setBeamEnergy_eV(double d, boolean doRedraw) 	{ beamEnergy_eV = d; 	if (doRedraw) GUI.asyncExec(new Runnable(){public void run(){ comp.redraw(); }}); }
	public void setDetector2t_deg(double d, boolean doRedraw) 	{ detector2t_deg = d; 	if (doRedraw) GUI.asyncExec(new Runnable(){public void run(){ comp.redraw(); }}); }
	
	protected Font resTextFont;
	protected Color bgColor, circleColor;
	protected int border;
	
	public WdgResPredictor(Composite parent, int style, Color bgColor, Color circleColor, Font resTextFont, int border)
	{
		super(parent,style);
		this.bgColor = bgColor;
		this.circleColor = circleColor;
		this.border = border;
		this.resTextFont = resTextFont;
		setBackground(GUI.BGCOLOR);
		
		setLayout(new FillLayout());
		
		group = new Group(this,SWT.None);
		group.setText("Resolution Predictor");
		group.setFont(parent.getFont());
		group.setBackground(GUI.BGCOLOR);
		group.setLayout(new GridLayout(1,false));
		
		comp = new Composite(group,SWT.NONE);
		GridData gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;
		gd.verticalAlignment = SWT.FILL;
		gd.horizontalAlignment = SWT.FILL;
		comp.setLayoutData(gd);

		comp.addPaintListener(this);
	}
	
	public void setGroupFont(Font font)
	{
		group.setFont(font);
	}

	@Override
	public void paintControl(PaintEvent pe)
	{
		// update width/height
		Rectangle bounds = comp.getBounds();
		int width = bounds.width;
		int height = bounds.height;
		
		GC gc = pe.gc;
		Image img = null;
		try
		{
			String url = Util.sprintf(Util.getServerURL()+"/respredictor");
			HttpResponse<InputStream> resp = Unirest.get(url)
					.queryString("width", width)
					.queryString("height", height)
					.queryString("border", border)
					.queryString("detector_z_mm", detectorZ_mm)
					.queryString("detector_2t_deg", detector2t_deg)
					.queryString("energy_eV", beamEnergy_eV)
					.asBinary();
			if (200 != resp.getStatus())
			{
				Log.error("Server error when reading res predictor: " + resp.getStatus());
				return;
			}
			InputStream is=resp.getBody();
			img = new Image(pe.display, is);
			gc.drawImage(img, 0, 0);
		}
		catch (Exception e) { e.printStackTrace(); }
		finally {
			if (null!=img && !img.isDisposed()) img.dispose();
		}
	}
}
