package gov.anl.gmca.io;

import gov.anl.gmca.gui.Main;
import gov.anl.gmca.util.Util;

import java.nio.file.Path;
import java.nio.file.attribute.GroupPrincipal;
import java.nio.file.attribute.PosixFileAttributes;
import java.nio.file.attribute.UserPrincipal;
import java.util.ArrayList;
import java.util.Calendar;

// provides home directory access
//
public abstract class Disk
{
	// can be a file or dir
	public static class File
	{
		private String fullPath;
		
		public String getName()				{ return Disk.getName(fullPath); 			}
		public String getCanonicalPath() throws IOException { return Disk.getCanonicalPath(fullPath);	}
		public boolean isDirectory() 		{ return Disk.isDirectory(fullPath); 		}
		public boolean isFile() 			{ return !isDirectory(); 					}
		public boolean exists()				{ return Disk.exists(fullPath); 			}
		public String getAbsolutePath() 	{ return Disk.getAbsolutePath(fullPath);	}
		public boolean mkdir()				{ return Disk.mkdir(fullPath); 				}
		public boolean mkdirs()				{ return Disk.mkdirs(fullPath); 			}
		public boolean canRead()			{ return Disk.canRead(fullPath);			}
		public boolean canWrite()			{ return Disk.canWrite(fullPath);			}
		public boolean canExec()			{ return Disk.canExec(fullPath);			}
		public void setReadable(boolean b)	{ Disk.setReadable(fullPath, b);			}
		public void setWritable(boolean b)	{ Disk.setWritable(fullPath, b);			}
		public void createNewFile()	throws IOException		{ Disk.createNewFile(fullPath);				}
		public File(String path)			{ this.fullPath=path; }
	}
	
	public static class IOException extends Exception
	{
		private static final long serialVersionUID = -7552907365591936547L;
		public IOException(String message) {this.message=message;}
		public String getMessage() { return message; }
		public String toString() { return message; }
		private String message = "";
		public void printStackTrace() { Util.printStackTrace(); }
	}
	
	///////////////////////////////////////////////////////////////////////////////////////
	
	protected static Disk inst = null;
	public static Disk get()
	{
		if (null == inst)
		{
    		if (Main.SIM_LEVEL==2 && !Main.USE_REAL_DISK)
    		{
    			inst = new gov.anl.gmca.io.sim.Disk();
    		}
    		else
    		{
    			inst = new gov.anl.gmca.io.hw.Disk();
    		}
		}
		return inst;
	}
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////// Public methods /////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	public static boolean	copyFile(String src, String dest)	{ return get().i_copyFile(src,dest); 			}
	public static String[]	getDir(String path)					{ return get().i_getDir(path); 					}
	public static boolean	isDirectory(String path)			{ return get().i_isDirectory(path); 			}
	public static String	getName(String path)				{ return get().i_getName(path);					}
	public static String	getAbsolutePath(String path)		{ return get().i_getAbsolutePath(path); 		}
	public static String	getCanonicalPath(String path)		{ return get().i_getCanonicalPath(path); 		}
	public static Calendar	getLastModified(String path)		{ return get().i_getLastModified(path);			}
	public static boolean	exists(String path)					{ return get().i_exists(path); 					}
	public static boolean	mkdir(String path)					{ return get().i_mkdir(path); 					}
	public static boolean	mkdirs(String path)					{ return get().i_mkdirs(path); 					}
	public static boolean	chown(String path, UserPrincipal user, GroupPrincipal group) { return get().i_chown(path, user, group); }
	public static boolean	chmod(String path, String mode)		{ return get().i_chmod(path,mode);				}
	public static boolean	ln(String dest, String src)			{ return get().i_ln(dest,src);					}
	public static boolean	mv(String src, String dest)			{ return get().i_mv(src, dest);					}
	public static boolean	delete(String path)					{ return get().i_delete(path); 					}
	public static boolean	delete(String[] paths)				{ return get().i_delete(paths); 				}
	public static boolean	canRead(String path)				{ return get().i_canRead(path); 				}
	public static boolean	canWrite(String path)				{ return get().i_canWrite(path); 				}
	public static boolean	canExec(String path)				{ return get().i_canExec(path); 				}
	public static ArrayList<String> readText(String path)		{ return get().i_readText(path);				}
	public static void 		setWritable(String fullPath, boolean writable)					{ get().i_setWritable(fullPath, writable);	}
	public static boolean	writeText(String path, ArrayList<String> contents) 				{ return get().i_writeText(path, contents);	}
	public static void 		appendText(String fullPath, ArrayList<String> contentsToAppend)	{ get().i_appendText(fullPath, contentsToAppend); }
	public static void 		createNewFile(String path) throws IOException					{ get().i_createNewFile(path); }
	public static void 		setReadable(String path, boolean readable) 						{ get().i_setReadable(path, readable); }
	public static PosixFileAttributes	getPosixAttributes(Path path) { return get().i_getPosixAttributes(path); }
	public static long		getFileSizeInBytes(String path)			{ return get().i_getFileSizeInBytes(path); }
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////// Begin interface ////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	protected abstract boolean 				i_copyFile(String src, String dest);
	protected abstract String[]				i_getDir(String relPath);
	protected abstract boolean				i_isDirectory(String path);
	protected abstract String				i_getName(String path);
	protected abstract String				i_getCanonicalPath(String path);
	protected abstract String				i_getAbsolutePath(String path);
	protected abstract Calendar				i_getLastModified(String path);
	protected abstract boolean				i_exists(String path);
	protected abstract boolean				i_mkdir(String path);
	protected abstract boolean				i_mkdirs(String path);
	protected abstract boolean				i_chown(String path, UserPrincipal user, GroupPrincipal group);
	protected abstract boolean				i_chmod(String path, String mode);
	protected abstract boolean				i_ln(String dest, String src);
	protected abstract boolean				i_mv(String src, String dest);
	protected abstract boolean				i_delete(String path);
	protected abstract boolean				i_delete(String[] paths);
	protected abstract boolean				i_canRead(String path);
	protected abstract boolean				i_canWrite(String path);
	protected abstract boolean				i_canExec(String path);
	protected abstract void					i_setWritable(String path, boolean writable);
	protected abstract ArrayList<String>	i_readText(String path);
	protected abstract boolean				i_writeText(String path, ArrayList<String> contents);
	protected abstract boolean				i_appendText(String path, ArrayList<String> contents);
	protected abstract void					i_createNewFile(String path) throws IOException;
	protected abstract void					i_setReadable(String path, boolean readable);
	protected abstract PosixFileAttributes	i_getPosixAttributes(Path path);
	protected abstract long					i_getFileSizeInBytes(String path);
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////// utility methods ///////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	public static String concatDir(String dir1, String dir2)
	{
		return dir1+"/"+dir2;
	}
	
	public static String concatDirToFile(String dir, String filename)
	{
		return dir+"/"+filename;
	}
}
