package gov.anl.gmca.io;

import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;
import gov.aps.jca.dbr.DBRType;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.*;

public abstract class Epics
{
	protected static Epics inst = null;
	protected static Epics get()
	{
		if (null == inst)
		{
    		if (Main.SIM_LEVEL>=2)	inst = new gov.anl.gmca.io.sim.Epics();
    		else  					inst = new gov.anl.gmca.io.hw.Epics();
		}
		return inst;
	}
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////// String-based static redirectors ////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static void 		pvs_addMonitor(final String pvName, final EpicsMonitor em) 	{ get().i_addMonitor(pvName,em); }
	public static boolean 	pvs_removeMonitor(String pvName, EpicsMonitor em) 			{ return get().i_removeMonitor(pvName,em); }
	
	public static int 		pvs_getCacheInteger(String pvName) 							{ return get().i_getCacheInteger(pvName); }
	public static String 	pvs_getCacheString(String pvName) 						{ return get().i_getCacheString(pvName, -1); }
	
	public static double 	pvs_getDouble(String pvName) 								{ return get().i_getDouble(pvName); }
	public static int 		pvs_getInteger(String pvName) 								{ return get().i_getInteger(pvName); }
	public static short 	pvs_getEnum(String pvName) 									{ return get().i_getEnum(pvName); }
	public static boolean	pvs_getBoolean(String pvName) 								{ return get().i_getBoolean(pvName); }

	public static double[]	pvs_getDoubleArray(String pvName) 							{ return get().i_getDoubleArray(pvName); }
	public static int[] 	pvs_getIntegerArray(String pvName) 							{ return get().i_getIntegerArray(pvName); }
	public static short[] 	pvs_getEnumArray(String pvName) 							{ return get().i_getEnumArray(pvName); }
	public static String 	pvs_getString(String pvName) 								{ return get().i_getString(pvName, -1); }
	public static String 	pvs_getString(String pvName, int decimals)					{ return get().i_getString(pvName, decimals); }
	public static DBR 		pvs_getDBR(String pvName) 									{ return get().i_getDBR(pvName); }
	public static DBR 		pvs_getCacheDBR(String pvName) 								{ return get().i_getCacheDBR(pvName); }
	
	public static void 		pvs_putInteger(String pvName, int value) 					{ get().i_putInteger(pvName,value); }
	public static void 		pvs_putBoolean(String pvName, boolean value) 				{ get().i_putBoolean(pvName,value); }
	public static void 		pvs_putDouble(String pvName, double value) 					{ get().i_putDouble(pvName,value); }
	
	public static void 		pvs_putDoubleArray(String pvName, double[] values) 			{ get().i_putDoubleArray(pvName,values); }
	public static void 		pvs_putIntegerArray(String pvName, int[] values) 			{ get().i_putIntegerArray(pvName,values); }
	public static void 		pvs_put(String pvName, String[] value) 						{ get().i_put(pvName,value); }
	public static void 		pvs_put(String pvName, String value) 						{ get().i_put(pvName,value); }

	public static boolean 	pvs_isConnected(String pvName) 								{ return get().i_isConnected(pvName); }
	public static boolean 	pvs_checkConnection(String pvName, int timeoutMs)			{ return get().i_checkConnection(pvName, timeoutMs); }
	public static void 		pendIO(int ms)												{ get().i_pendIO(ms); }
	public static void 		poll() 														{ get().i_poll(); }
	public static void		flushIO()													{ get().i_flushIO(); }
	public static int		getEpicsTaskCount()											{ return get().i_getEpicsTaskCount(); }
	
	public static void 		connect() 													{ get().i_connect(); }
	public static void 		startPollThread() 											{ get().i_startPollThread(); }
	public static void 		disconnectMonitors() 										{ get().i_disconnectMonitors(); }
	public static void 		disablePV(PVName pvName, Object value)						{ get().i_disablePV(pvName, value); }
	
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////// Enum-based static redirectors /////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public static void 		addMonitor(final PVName pvName, final EpicsMonitor em) 	{ pvs_addMonitor(pvName.toString(), em); }
	public static boolean 	removeMonitor(PVName pvName, EpicsMonitor em) 			{ return pvs_removeMonitor(pvName.toString(), em); }
	
	public static int		getCacheInteger(PVName pvName)							{ return pvs_getCacheInteger(pvName.toString()); }
	public static String	getCacheString(PVName pvName)							{ return pvs_getCacheString(pvName.toString()); }
	
	public static double 	getDouble(PVName pvName) 								{ return pvs_getDouble(pvName.toString()); }
	public static int 		getInteger(PVName pvName) 								{ return pvs_getInteger(pvName.toString()); }
	public static short 	getEnum(PVName pvName) 									{ return pvs_getEnum(pvName.toString()); }
	public static boolean	getBoolean(PVName pvName) 								{ return pvs_getBoolean(pvName.toString()); }

	public static double[]	getDoubleArray(PVName pvName) 							{ return pvs_getDoubleArray(pvName.toString()); }
	public static int[] 	getIntegerArray(PVName pvName) 							{ return pvs_getIntegerArray(pvName.toString()); }
	public static short[] 	getEnumArray(PVName pvName) 							{ return pvs_getEnumArray(pvName.toString()); }
	public static String 	getString(PVName pvName) 								{ return pvs_getString(pvName.toString()); }
	public static String 	getString(PVName pvName, int decimals)					{ return pvs_getString(pvName.toString(), decimals); }
	public static DBR 		getDBR(PVName pvName) 									{ return pvs_getDBR(pvName.toString()); }
	public static DBR 		getCacheDBR(PVName pvName) 								{ return pvs_getCacheDBR(pvName.toString()); }
	
	public static void 		putInteger(PVName pvName, int value) 					{ pvs_putInteger(pvName.toString(), value); }
	public static void 		putBoolean(PVName pvName, boolean value) 				{ pvs_putBoolean(pvName.toString(), value); }
	public static void 		putDouble(PVName pvName, double value) 					{ pvs_putDouble(pvName.toString(), value); }
	
	public static void 		putDoubleArray(PVName pvName, double[] values) 			{ pvs_putDoubleArray(pvName.toString(), values); }
	public static void 		putIntegerArray(PVName pvName, int[] values) 			{ pvs_putIntegerArray(pvName.toString(), values); }
	public static void 		put(PVName pvName, String[] value) 						{ pvs_put(pvName.toString(), value); }
	public static void 		put(PVName pvName, String value) 						{ pvs_put(pvName.toString(), value); }

	public static boolean 	isConnected(PVName pvName) 								{ if (Main.SIM_LEVEL>=2) return true; return pvs_isConnected(pvName.toString()); }
	public static boolean	checkConnection(PVName pvName, int timeoutMs)			{ return pvs_checkConnection(pvName.toString(), timeoutMs); }
	public static String[]	getAllPVNames()											{ return get().i_getPVNames(); }
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////// begin interface ////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	protected abstract void 		i_addMonitor(final String pvName, final EpicsMonitor em);
	protected abstract boolean 		i_removeMonitor(String pvName, EpicsMonitor em);
	
	protected abstract int			i_getCacheInteger(String pvName);
	
	protected abstract double 		i_getDouble(String pvName);
	protected abstract int 			i_getInteger(String pvName);
	protected abstract short 		i_getEnum(String pvName);
	protected abstract boolean		i_getBoolean(String pvName);

	protected abstract double[]		i_getDoubleArray(String pvName);
	protected abstract int[] 		i_getIntegerArray(String pvName);
	protected abstract short[] 		i_getEnumArray(String pvName);
	protected abstract String 		i_getString(String pvName, int decimals);
	protected abstract String 		i_getCacheString(String pvName, int decimals);
	protected abstract DBR 			i_getDBR(String pvName);
	protected abstract DBR 			i_getCacheDBR(String pvName);
	
	protected abstract void 		i_putInteger(String pvName, int value);
	protected abstract void 		i_putBoolean(String pvName, boolean value);
	protected abstract void 		i_putDouble(String pvName, double value);
	
	protected abstract void 		i_putDoubleArray(String pvName, double[] values);
	protected abstract void 		i_putIntegerArray(String pvName, int[] values);
	protected abstract void 		i_put(String pvName, String[] value);
	protected abstract void 		i_put(String pvName, String value);

	protected abstract boolean 		i_isConnected(String pvName);
	protected abstract boolean 		i_checkConnection(String pvName, int timeoutMs);
	protected abstract void 		i_pendIO(int ms);
	protected abstract void 		i_poll();
	protected abstract void			i_flushIO();
	
	protected abstract void 		i_connect();
	protected abstract void 		i_startPollThread();
	protected abstract void 		i_disconnectMonitors();
	protected abstract int			i_getEpicsTaskCount();
	protected abstract void			i_disablePV(PVName pvName, Object defaultValue);
	protected abstract String[]		i_getPVNames();

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////// end interface //////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	protected 	static Semaphore 							imonMapLock 			= new Semaphore(1);
	protected 	static Map<String,ArrayList<EpicsMonitor>> 	imonMap 				= new HashMap<String,ArrayList<EpicsMonitor>>();
	public		static ThreadPoolExecutor 					executor;
	public 		static int 									getExecutorQueueSize() 	{ return inst.executor.getQueue().size(); }
	 
	public Epics()
	{
		ThreadFactory factory =  new ThreadFactory()
		{
			@Override public Thread newThread(Runnable r)
			{
				return new Thread(r, "Epics callback");
			}
		};
		executor = new ThreadPoolExecutor(1,1,0,TimeUnit.MILLISECONDS,new LinkedBlockingQueue<Runnable>(), factory);
	}
	
	public static String addBID(String suffix)
	{
		return Util.getBID()+":"+suffix;
	}
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////// Listener ////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 * When registering a PV listener, an EpicsMonitor must be specified.
	 * 
	 * It is recommended to use an in-line class created for each listener so the handler code is
	 * close to the creation code.  Using a single handler for many PV events makes understanding
	 * it less clear, and also relies on inefficient string comparisons.
	 */
	public static abstract class EpicsMonitor
	{
		public abstract void connectionChanged(String pvName, boolean isConnected);
		public abstract void valueChanged(String pvName, DBR value);
		public boolean createThread = true;
	}
	
	public static abstract class EpicsValueMonitor extends EpicsMonitor
	{
		public void connectionChanged(String pvName, boolean isConnected) {};
	}
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////// dbrTo[type] static redirectors ///////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	public static boolean 	dbrToBoolean(DBR dbr) 		{ return (1==dbrToInt(dbr));			}
	public static int 		dbrToInt(DBR dbr)			{ return get().i_dbrToInt(dbr); 		}
	public static double 	dbrToDouble(DBR dbr)		{ return get().i_dbrToDouble(dbr); 		}
	public static int[] 	dbrToIntArray(DBR dbr)		{ return get().i_dbrToIntArray(dbr); 	}
	public static double[] 	dbrToDoubleArray(DBR dbr)	{ return get().i_dbrToDoubleArray(dbr); }
	public static String	dbrToString(DBR dbr)		{ return dbrToString(dbr,-1);			}
	public static String	dbrToString(DBR dbr, int decimalsToKeep) { return get().i_dbrToString(dbr,decimalsToKeep); }
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////// dbrTo[type] interface ///////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	protected abstract int 			i_dbrToInt(DBR dbr);
	protected abstract double 		i_dbrToDouble(DBR dbr);
	protected abstract int[] 		i_dbrToIntArray(DBR dbr);
	protected abstract double[] 	i_dbrToDoubleArray(DBR dbr);
	protected abstract String		i_dbrToString(DBR dbr, int decimalsToKeep);
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////// dbrTo[type] default implementation ////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	protected static int d_dbrToInt(DBR dbr)
	{
		if (DBRType.INT == dbr.getType())		return ((int[])(dbr.getValue()))[0];
		if (DBRType.SHORT == dbr.getType())		return (((short[])(dbr.getValue()))[0]);
		if (DBRType.ENUM == dbr.getType())		return (((short[])(dbr.getValue()))[0]);
		if (DBRType.BYTE == dbr.getType())		return (((byte[])(dbr.getValue()))[0]);
		if (DBRType.DOUBLE == dbr.getType())	return (int)(((double[])(dbr.getValue()))[0]);
		if (DBRType.FLOAT == dbr.getType())		return (int)(((float[])(dbr.getValue()))[0]);
		if (DBRType.STRING == dbr.getType())
		{
			return Integer.parseInt(((String[])(dbr.getValue()))[0]);
		}
		System.out.println("ERROR: Couldn't parse dbr in dbrToInt(), returning MAX_VALUE");
		return Integer.MAX_VALUE;
	}
	
	public static double d_dbrToDouble(DBR dbr)
	{
		if (DBRType.INT == dbr.getType())		return (double)((int[])(dbr.getValue()))[0];
		if (DBRType.SHORT == dbr.getType())		return (double)(((short[])(dbr.getValue()))[0]);
		if (DBRType.ENUM == dbr.getType())		return (double)(((short[])(dbr.getValue()))[0]);
		if (DBRType.BYTE == dbr.getType())		return (double)(((byte[])(dbr.getValue()))[0]);
		if (DBRType.DOUBLE == dbr.getType())	return (((double[])(dbr.getValue()))[0]);
		if (DBRType.FLOAT == dbr.getType())		return (((float[])(dbr.getValue()))[0]);
		if (DBRType.STRING == dbr.getType())
		{
			return Double.parseDouble(((String[])(dbr.getValue()))[0]);
		}
		System.out.println("ERROR: Couldn't parse dbr in dbrToInt(), returning NaN");
		return Double.NaN;
	}
	
	public static int[] d_dbrToIntArray(DBR dbr)
	{
		int[] ret = new int[dbr.getCount()];

		for (int i=0; i<ret.length; i++)
		{
			if 		(DBRType.INT == dbr.getType())		ret[i] = ((int[])dbr.getValue())[i];
			else if (DBRType.SHORT == dbr.getType())	ret[i] = ((short[])dbr.getValue())[i];
			else if (DBRType.ENUM == dbr.getType())		ret[i] = ((short[])dbr.getValue())[i];
			else if (DBRType.BYTE == dbr.getType())		ret[i] = ((byte[])dbr.getValue())[i];
			else if (DBRType.DOUBLE == dbr.getType())	ret[i] = (int)((double[])dbr.getValue())[i];
			else if (DBRType.FLOAT == dbr.getType())	ret[i] = (int)((float[])dbr.getValue())[i];
			else
			{
				System.out.println("ERROR: Couldn't parse dbr in dbrToDoubleArray(), returning null");
				return null;
			}
		}
		return ret;
	}
	
	public static double[] d_dbrToDoubleArray(DBR dbr)
	{
		double[] ret = new double[dbr.getCount()];

		for (int i=0; i<ret.length; i++)
		{
			if (DBRType.INT == dbr.getType())			ret[i] = ((int[])dbr.getValue())[i];
			else if (DBRType.SHORT == dbr.getType())	ret[i] = ((short[])dbr.getValue())[i];
			else if (DBRType.ENUM == dbr.getType())		ret[i] = ((short[])dbr.getValue())[i];
			else if (DBRType.BYTE == dbr.getType())		ret[i] = ((byte[])dbr.getValue())[i];
			else if (DBRType.DOUBLE == dbr.getType())	ret[i] = ((double[])dbr.getValue())[i];
			else if (DBRType.FLOAT == dbr.getType())	ret[i] = ((float[])dbr.getValue())[i];
			else
			{
				System.out.println("ERROR: Couldn't parse dbr in dbrToDoubleArray(), returning null");
				return null;
			}
		}
		return ret;
	}
	
	public static String d_dbrToString(DBR dbr, int decimalsToKeep)
	{
		if (null == dbr) return "null";
		if (decimalsToKeep < 0) decimalsToKeep = 0;
		
		DBRType type = dbr.getType();
		if (type==DBRType.ENUM)
		{
			short s = ((short[])(dbr.getValue()))[0];
			return ""+s;
		}
		else if (type==DBRType.DOUBLE)
		{
			double[] array = ((double[])(dbr.getValue()));
			if (array.length==0) return "";
			
			if (array.length==1)
			{
				double d = array[0];
				if (decimalsToKeep >= 0)
				{
					String ret = Util.sprintf("%.0"+decimalsToKeep+"f", d);
					return ret;
				}
				return  ""+d;
			}
			
			String ret = "";
			for (double d : array)
			{
				ret += Util.sprintf("%.0"+decimalsToKeep+"f ", d);
			}
			return ret;
		}
		else if (type==DBRType.FLOAT)
		{
			float f = ((float[])(dbr.getValue()))[0];
			if (decimalsToKeep >= 0)
			{
				f = Math.round(f*(float)Math.pow(10,decimalsToKeep))/(float)Math.pow(10,decimalsToKeep);
			}
			return  ""+f;
		}
		else if (type==DBRType.INT)
		{
			int[] i = (int[])(dbr.getValue());
			String ret = "";
			for (int j=0; j<i.length; j++) { ret += i[j]+" "; }
			if (i.length>0) ret = ret.substring(0, ret.length()-1);
			return ret;
		}
		else if (type==DBRType.STRING)
		{
			String str = ((String[])(dbr.getValue()))[0];
			return str;
		}
		else if (type==DBRType.BYTE)
		{
			byte[] val = ((byte[])dbr.getValue());
			String str = new String("");
			for (byte b : val) { if (0==b) break; str += (char)b; }
			return str;
		}
		else if (type==DBRType.SHORT)
		{
			short i = ((short[])(dbr.getValue()))[0];
			return ""+i;
		}
		else
		{
			Log.note("Type "+type+" not supported.");
			return "";
		}
	}
}
