package gov.anl.gmca.io.hw;

import java.io.*;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.LinkOption;
import java.nio.file.Path;
import java.nio.file.attribute.GroupPrincipal;
import java.nio.file.attribute.PosixFileAttributeView;
import java.nio.file.attribute.PosixFileAttributes;
import java.nio.file.attribute.UserPrincipal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

// TODO: if needed, cache file objects
public class Disk extends gov.anl.gmca.io.Disk
{
	@Override protected boolean i_copyFile(String src, String dest)
	{
		gov.anl.gmca.io.hw.ExecTask cpTask = new gov.anl.gmca.io.hw.ExecTask("/bin/cp "+src+" "+dest, "cp", true);
		cpTask.start();
		cpTask.waitFor();
		return (cpTask.exitValue==0);
	}
	
	@Override protected String[] i_getDir(String path)
	{
		return (new java.io.File(path)).list();
	}

	@Override protected boolean i_isDirectory(String path)
	{
		if(path == null)
			return false;
		return (new java.io.File(path)).isDirectory();
	}

	@Override protected String i_getName(String path)
	{
		return (new java.io.File(path)).getName();
	}

	@Override protected String i_getCanonicalPath(String path)
	{
		try
		{
			return (new java.io.File(path)).getCanonicalPath();
		}
		catch (java.io.IOException ioe)
		{
			ioe.printStackTrace();
			return (new java.io.File(path)).getAbsolutePath();
		}
	}

	@Override protected boolean i_exists(String path)
	{
		return (new java.io.File(path)).exists();
	}

	@Override protected boolean i_mkdir(String path)
	{
		return (new java.io.File(path)).mkdir();
	}
	
	@Override protected boolean i_chown(String path, UserPrincipal user, GroupPrincipal group)
	{
		try
		{
			// get file "view"
			java.io.File f = new java.io.File(path);
			PosixFileAttributeView view = Files.getFileAttributeView(f.toPath(), PosixFileAttributeView.class, LinkOption.NOFOLLOW_LINKS);

			// get principal objects for user and group
			//UserPrincipalLookupService lookupService = FileSystems.getDefault().getUserPrincipalLookupService();
			//GroupPrincipal groupPrincipal = lookupService.lookupPrincipalByGroupName(group);
			//UserPrincipal userPrincipal = lookupService.lookupPrincipalByName(user);

			// set user and group owners using view
			view.setOwner(user);
			view.setGroup(group);
		}
		catch (Exception e) { e.printStackTrace(); return false; }
		return true;
	}

	@Override protected boolean i_chmod(String path, String mode)
	{
		gov.anl.gmca.io.hw.ExecTask chmodTask = new gov.anl.gmca.io.hw.ExecTask
		(
			"/usr/bin/chmod "+mode+" "+path, "chmod", true
		);
		chmodTask.start();
		chmodTask.waitFor();
		return (chmodTask.exitValue==0);
	}
	
	@Override protected boolean i_mkdirs(String path)
	{
		return (new java.io.File(path)).mkdirs();
	}

	@Override protected boolean i_mv(String src, String dest)
	{
		Path srcPath = FileSystems.getDefault().getPath(src);
		Path destPath = FileSystems.getDefault().getPath(dest);
		try {
			java.nio.file.Files.move(srcPath, destPath);
		}
		catch (Exception e) { e.printStackTrace(); return false; }
		return true;
	}

	@Override protected boolean i_delete(String path)
	{
		try
		{
			java.nio.file.Files.delete(FileSystems.getDefault().getPath(path));
		}
		catch (java.io.IOException e) { e.printStackTrace(); return false; }
		return true;
	}
	
	@Override protected boolean i_delete(String[] paths)
	{
		try
		{
			// I don't see a more efficient way to do this
			for (String path : paths)
				java.nio.file.Files.delete(FileSystems.getDefault().getPath(path));
		}
		catch (java.io.IOException e) { e.printStackTrace(); return false; }
		return true;
	}

	@Override protected boolean i_canRead(String path)
	{
		return (new java.io.File(path)).canRead();
	}

	@Override protected ArrayList<String> i_readText(String path)
	{
		try
		{
			java.io.File file = new java.io.File(path);
			BufferedReader br = new BufferedReader(new FileReader(file));
			
			ArrayList<String> ret = new ArrayList<String>();
			String line;
			while ((line = br.readLine()) != null)
			{
				ret.add(line);
			}

			br.close();
			return ret;
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return null;
		}
	}

	@Override protected String i_getAbsolutePath(String path)
	{
		return (new java.io.File(path)).getAbsolutePath();
	}

	@Override protected boolean i_canWrite(String path)
	{
		return (new java.io.File(path)).canWrite();
	}

	@Override protected boolean i_canExec(String path)
	{
		return (new java.io.File(path)).canExecute();
	}
	
	@Override protected void i_setWritable(String path, boolean writable)
	{
		(new java.io.File(path)).setWritable(writable);
	}

	@Override protected boolean i_writeText(String path, ArrayList<String> contents)
	{
		PrintWriter pw = null;
		boolean success = true;
		try
		{
			pw = new PrintWriter(new FileOutputStream(path));
			for (String line : contents)
			{
				pw.println(line);
			}
		}
		catch (FileNotFoundException fnfe)
		{
			System.out.println("File not found exception for path: "+path);
			fnfe.printStackTrace();
			success = false;
		}
		finally
		{
			if (null!=pw) pw.close();
		}
		return success;
	}

	@Override protected boolean i_appendText(String path, ArrayList<String> contents)
	{
		try
		{
			FileWriter fw = new FileWriter(path, true);
			PrintWriter pw = new PrintWriter(fw);
	
			for (String line : contents)
			{
				pw.append(line);
			}
			
			fw.close();
			pw.close();
		}
		catch (java.io.IOException ioe)
		{
			ioe.printStackTrace();
			return false;
		}
		return true;
	}

	@Override protected void i_createNewFile(String path) throws IOException
	{
		try
		{
			(new java.io.File(path)).createNewFile();
		}
		catch (java.io.IOException ioe)
		{
			throw new IOException(ioe.getMessage());
		}
	}

	@Override protected void i_setReadable(String path, boolean readable)
	{
		(new java.io.File(path)).setReadable(readable);
	}

	@Override protected Calendar i_getLastModified(String path)
	{
		if (!exists(path)) return null;
		long epochMs = (new java.io.File(path)).lastModified();
		Calendar ret = new GregorianCalendar();
		ret.setTimeInMillis(epochMs);
		System.out.println(path+" "+epochMs+" "+ret.toString());
		return ret;
	}

	@Override protected PosixFileAttributes i_getPosixAttributes(Path path)
	{
		try
		{
			PosixFileAttributes attributes = java.nio.file.Files.readAttributes(path, PosixFileAttributes.class, LinkOption.NOFOLLOW_LINKS);
			return attributes;
		}
		catch (Exception e) { e.printStackTrace(); return null; }
	}
	
	@Override protected long i_getFileSizeInBytes(String path)
	{
		return (new java.io.File(path)).length();
	}

	@Override protected boolean i_ln(String dest, String src)
	{
		if (Disk.exists(src)) return false;
		
		Path link = FileSystems.getDefault().getPath(src);
		Path target = FileSystems.getDefault().getPath(dest);
		try
		{
			Files.createSymbolicLink(link, target);
		}
		catch (Exception e) { e.printStackTrace(); return false; }	
		return true;
	}
}
