package gov.anl.gmca.io.hw;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.util.HashSet;
import java.util.concurrent.Semaphore;

public class Web extends gov.anl.gmca.io.Web
{
	private static final int TIMEOUT_MS = 10000;
	private HashSet<HttpGet> activeHttpGets = new HashSet<HttpGet>();
	private Semaphore httpGetLock = new Semaphore(1);
	
	public class HWWebConnection extends WebConnection
	{
		@Override public void abort()
		{
			httpGetLock.acquireUninterruptibly();
			try
			{
				for (HttpGet get : activeHttpGets)
				{
					get.abort();
				}
			}
			catch (Exception e) { e.printStackTrace(); }
			finally { httpGetLock.release(); }
		}
	}
	
	@Override public WebConnection openConnection(final String url, int timeoutMs)
	{
		final HWWebConnection wc = new HWWebConnection();
		
		wc.thread = new Thread(new Runnable(){public void run()
		{
			HttpGet httpget = new HttpGet(url);

			httpGetLock.acquireUninterruptibly();
			activeHttpGets.add(httpget);
			httpGetLock.release();
			
			CloseableHttpResponse response = null;
			try
			{
				CloseableHttpClient httpclient = null;
				if (0==timeoutMs) {
					httpclient = HttpClients.createDefault();
				} else {
					RequestConfig config = RequestConfig.custom()
							.setConnectTimeout(timeoutMs)
							.setConnectionRequestTimeout(timeoutMs)
							.setSocketTimeout(timeoutMs).build();
					httpclient =
							HttpClientBuilder.create().setDefaultRequestConfig(config).build();
				}

				response = httpclient.execute(httpget);
				
				// read headers
				Header[] headerArray = response.getAllHeaders();
				for (Header header : headerArray)
				{
					wc.headers.put(header.getName(), header.getValue());
				}
				
				// read response
				HttpEntity entity = response.getEntity();
				if (null != entity)
				{
					try
					{
						wc.content = EntityUtils.toByteArray(entity);
					}
					catch (Exception e) { e.printStackTrace(); }
				}
				
				wc.responseCode = response.getStatusLine().getStatusCode();
			}
			catch (ClientProtocolException e) { e.printStackTrace(); }
			catch (IOException e) { e.printStackTrace(); }
			finally
			{
				if (null != response)
				{
					try { response.close(); }
					catch (IOException ioe) { ioe.printStackTrace(); }
					activeHttpGets.remove(httpget);
				}
			}
		}});
		wc.thread.start();
		
		return wc;
	}
}
