package gov.anl.gmca.io.sim;

import gov.anl.gmca.io.Resources;
import gov.anl.gmca.log.db.DevEvents;
import gov.anl.gmca.log.db.RasterEvent;
import gov.anl.gmca.log.db.RasterEvent.RasterParam;
import gov.anl.gmca.log.db.ScanEvent;
import gov.anl.gmca.log.db.ScanEvent.ScanParam;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.StringTokenizer;

// Loads an SQL dump created by the "mysqldump" command
//
// Additionally may add its own test data
//
public class Database extends gov.anl.gmca.io.Database
{
	protected class SQLTable
	{
		public SQLTable(ArrayList<HashMap<String,String>> t) { contents = t; }
		ArrayList<HashMap<String,String>> contents = new ArrayList<HashMap<String,String>>();
	}
	
	protected HashMap<String, SQLTable> db = new HashMap<String, SQLTable>();
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////// interface implementation //////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////
	
	@Override protected boolean i_create(DBName dbName, String tableName, ArrayList<DBField> fields)
	{
		String queryString = gov.anl.gmca.io.hw.Database.q_create(tableName,fields);
		System.out.println(queryString);
		
		ArrayList<HashMap<String,String>> rows = new ArrayList<HashMap<String,String>>();
		SQLTable table = new SQLTable(rows);
		db.put(tableName, table);
		return true;
	}
	
	@Override protected boolean i_delete(DBName dbName, String tableName, WhereList whereList)
	{
		String queryString = gov.anl.gmca.io.hw.Database.q_delete(tableName,whereList);
		System.out.println(queryString);
		
		ArrayList<HashMap<String,String>>
			contents = db.get(tableName).contents,
			newContents = new ArrayList<HashMap<String,String>>();
		
		for (HashMap<String,String> row : contents)
		{
			boolean matches = true;
			for (Where where : whereList.whereList)
			{
				String rowVal = row.get(where.fieldName);
				
				if (!where.valueMatches(rowVal))
				{
					matches=false;
				}
			}
			
			if (!matches)
			{
				newContents.add(row);
			}
		}
		
		db.get(tableName).contents = newContents;
		
		return true;
	}
	
	@Override protected Result i_insert(DBName dbName, String tableName, DBRows rows)
	{
		String queryString = gov.anl.gmca.io.hw.Database.q_insert(tableName,rows);
		System.out.println(queryString);
		
		SQLTable table = db.get(tableName);
		if (null == table) throw new IllegalArgumentException("Table doesn't exist: "+tableName);

		for (int col=0;col<rows.columns.length;col++)
		{
			HashMap<String,String> rowMap = new HashMap<String,String>();
			for (int row=0;row<rows.values.length;row++)
			{
				rowMap.put(rows.columns[col],rows.values[row][col]);
			}
			table.contents.add(rowMap);
		}
		
		return new Result(RESULT_ERROR.NONE);
	}
	
	@Override protected Result i_insert(DBName dbName, String tableName, ArrayList<HashMap<String,String>> rows)
	{
		String queryString = gov.anl.gmca.io.hw.Database.q_insert(tableName,rows);
		System.out.println(queryString);
		
		SQLTable table = db.get(tableName);
		if (null == table) throw new IllegalArgumentException("Table doesn't exist: "+tableName);

		for (HashMap<String,String> row : rows)
		{
			table.contents.add(row);
		}
		
		return new Result(RESULT_ERROR.NONE);
	}
	
	@Override protected boolean i_update(DBName dbName, String tableName, HashMap<String,String> setMap, WhereList whereList)
	{
		String queryString = gov.anl.gmca.io.hw.Database.q_update(tableName,setMap,whereList);
		System.out.println(queryString);
		
		ArrayList<HashMap<String,String>> contents = db.get(tableName).contents;
		
		for (HashMap<String,String> row : contents)
		{
			boolean matches = true;
			for (Where where : whereList.whereList)
			{
				String rowVal = row.get(where.fieldName);
				
				if (!where.valueMatches(rowVal))
				{
					matches=false;
				}
			}
			
			if (matches)
			{
				for (String key : setMap.keySet())
				{
					row.put(key, setMap.get(key));
				}
			}
		}
		
		return true;
	}
	
	@Override protected Result i_select(DBName dbName, String tableName, ArrayList<String> requestedColumns, WhereList whereList, String orderBy)
	{
		ArrayList<HashMap<String,String>> ret = new ArrayList<HashMap<String,String>>();
		
		for (HashMap<String,String> row : db.get(tableName).contents)
		{
			boolean matches = true;
			for (Where where : whereList.whereList)
			{
				String rowVal = row.get(where.fieldName);
				
				if (!where.valueMatches(rowVal))
				{
					matches=false;
				}
			}
			
			if (matches) ret.add(row);
		}

		if (null != orderBy)
		{
			// TODO: sort results by orderBy column
		}
		
		return new Result(ret);
	}
	
	@Override protected Result i_exec(DBName dbName, String query)
	{
		System.out.println("SQL: db="+dbName+" query="+query);
		
		if (query.toLowerCase().indexOf("select * from")==0)
		{
			StringTokenizer st = new StringTokenizer(query);
			st.nextToken(); // select
			st.nextToken(); // *
			st.nextToken(); // from
			StringTokenizer st2 = new StringTokenizer(st.nextToken(),"`");
			String tableName = st2.nextToken();
			
			SQLTable ret = db.get(tableName);
			if (null!=ret)
				return new Result(ret.contents);
			else
				return new Result(new ArrayList<HashMap<String,String>>());
		}
		else if (query.toLowerCase().indexOf("select ")==0)
		{
			StringTokenizer st = new StringTokenizer(query," ,'=");
			st.nextToken(); // select
			
			HashSet<String> requestedColumns = new HashSet<String>();
			String token;
			while ( !(token=st.nextToken()).equals("from"))
			{
				requestedColumns.add(token);
			}

			String tableName = st.nextToken();

			HashMap<String,String> whereMap = null;
			if (st.hasMoreTokens() && st.nextToken().toLowerCase().equals("where"))
			{
				whereMap = new HashMap<String,String>();
				while (st.hasMoreTokens())
				{
					String key = st.nextToken();
					String value = st.nextToken();
					whereMap.put(key, value);
					if (st.hasMoreTokens()) st.nextToken(); // and
				}
			}
			
			return constructResult(requestedColumns, tableName, whereMap);
		}
		
		if (query.indexOf("unduflag") >= 0)
		{
			ArrayList<HashMap<String,String>> ret = new ArrayList<HashMap<String,String>>();
			ret.add(new HashMap<String,String>());
			ret.get(0).put("unduflag", "0");
			return new Result(ret);
		}
		
		StringTokenizer st = new StringTokenizer(query);
		if (	st.hasMoreTokens() && st.nextToken().toLowerCase().equals("show") &&
				st.hasMoreTokens() && st.nextToken().toLowerCase().equals("tables") &&
				st.hasMoreTokens() && st.nextToken().toLowerCase().equals("like"))
		{
			String tableName = st.nextToken();
			tableName = tableName.substring(1, tableName.length()-2);
			
			ArrayList<HashMap<String,String>> ret = new ArrayList<HashMap<String,String>>();
			ret.add(new HashMap<String,String>());
			ret.get(0).put("table", tableName);
			return new Result(ret);
		}
		
		return null;
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////// helper methods ///////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////
	
	protected String getNextLine(BufferedReader reader)
	{
		while (true)
		{
			String lineBuffer;
			try
			{
				lineBuffer = reader.readLine();
			}
			catch (IOException e) { return null; }
			
			if (null == lineBuffer) return null;
			
			// return line buffer if any non-white space found
			for (int i=0; i<lineBuffer.length(); i++)
			{
				char c = lineBuffer.charAt(i);
				if (c != ' ' && c != '\t' && c != '\n' && c != '\r') return lineBuffer;
			}
			
			return "";
		}
	}
	
	private String[][] RasterResults = new String[][]
    {
	 {  "listType",	"runNumber","cellNumber",	"state","fluoValue","hitNumber","spot_total",	"in_res_total",	"bragg_candidates",	"ice_rings","method1_res",	"method2_res",	"max_unit_cell",	"pct_saturation",	"in_res_ovrld_spots"},
	 {	"inter",	"1",		"0",			"done",	"",			"",			"5",			"5",			"5",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"1",			"done",	"",			"",			"7",			"7",			"7",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"2",			"done",	"",			"",			"2",			"2",			"2",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"3",			"done",	"",			"",			"3",			"3",			"3",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"4",			"done",	"",			"",			"9",			"9",			"9",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"5",			"done",	"",			"",			"7",			"7",			"7",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"6",			"done",	"",			"",			"12",			"12",			"12",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"7",			"done",	"",			"",			"15",			"15",			"15",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"8",			"done",	"",			"",			"5",			"5",			"5",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"9",			"done",	"",			"",			"6",			"6",			"6",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"10",			"done",	"",			"",			"14",			"14",			"14",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"11",			"done",	"",			"",			"19",			"19",			"19",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"12",			"done",	"",			"",			"4",			"4",			"4",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"13",			"done",	"",			"",			"2",			"2",			"2",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"14",			"done",	"",			"",			"3",			"3",			"3",				"0",		"",				"",				"",					"",					"",					},
	 {	"inter",	"1",		"15",			"done",	"",			"",			"8",			"8",			"8",				"0",		"",				"",				"",					"",					"",					}
	};
	
	private String[][] SilTable = new String[][]
    {
		 {"sampleNumber", "selected", "ordered", "screened", "Port", "CrystalID","Directory", "Comment", "workdir", "image1", "image2", "Images", "Score","Resolution", "Protein", "Mosaicity", "UnitCell", "BravaisLattice", "Rmsd","IceRings", "warning", "timestamp"},
		 {"0","0","0","0","A1","A1","A1","","","","","","","","","","","","","","",""},
		 {"1","0","0","0","A2","A2","A2","","","","","","","","","","","","","","",""},
		 {"2","0","0","0","A3","A3","A3","","","","","","","","","","","","","","",""},
		 {"3","0","0","0","A4","A4","A4","","","","","","","","","","","","","","",""},
		 {"4","0","0","0","A5","A5","A5","","","","","","","","","","","","","","",""},
		 {"5","0","0","0","A6","A6","A6","","","","","","","","","","","","","","",""},
		 {"6","0","0","0","A7","A7","A7","","","","","","","","","","","","","","",""},
		 {"7","0","0","0","A8","A8","A8","","","","","","","","","","","","","","",""},
		 {"8","0","0","0","A9","A9","A9","","","","","","","","","","","","","","",""},
		 {"9","0","0","0","A10","A10","A10","","","","","","","","","","","","","","",""},
		 {"10","0","0","0","A11","A11","A11","","","","","","","","","","","","","","",""},
		 {"11","0","0","0","A12","A12","A12","","","","","","","","","","","","","","",""},
		 {"12","0","0","0","A13","A13","A13","","","","","","","","","","","","","","",""},
		 {"13","0","0","0","A14","A14","A14","","","","","","","","","","","","","","",""},
		 {"14","0","0","0","A15","A15","A15","","","","","","","","","","","","","","",""},
		 {"15","0","0","0","A16","A16","A16","","","","","","","","","","","","","","",""},
		 {"16","0","0","0","B1","B1","B1","","","","","","","","","","","","","","",""},
		 {"17","0","0","0","B2","B2","B2","","","","","","","","","","","","","","",""},
		 {"18","0","0","0","B3","B3","B3","","","","","","","","","","","","","","",""},
		 {"19","0","0","0","B4","B4","B4","","","","","","","","","","","","","","",""},
		 {"20","0","0","0","B5","B5","B5","","","","","","","","","","","","","","",""},
		 {"21","0","0","0","B6","B6","B6","","","","","","","","","","","","","","",""},
		 {"22","0","0","0","B7","B7","B7","","","","","","","","","","","","","","",""},
		 {"23","0","0","0","B8","B8","B8","","","","","","","","","","","","","","",""},
		 {"24","0","0","0","B9","B9","B9","","","","","","","","","","","","","","",""},
		 {"25","0","0","0","B10","B10","B10","","","","","","","","","","","","","","",""},
		 {"26","0","0","0","B11","B11","B11","","","","","","","","","","","","","","",""},
		 {"27","0","0","0","B12","B12","B12","","","","","","","","","","","","","","",""},
		 {"28","0","0","0","B13","B13","B13","","","","","","","","","","","","","","",""},
		 {"29","0","0","0","B14","B14","B14","","","","","","","","","","","","","","",""},
		 {"30","0","0","0","B15","B15","B15","","","","","","","","","","","","","","",""},
		 {"31","0","0","0","B16","B16","B16","","","","","","","","","","","","","","",""},
		 {"32","0","0","0","C1","C1","C1","","","","","","","","","","","","","","",""},
		 {"33","0","0","0","C2","C2","C2","","","","","","","","","","","","","","",""},
		 {"34","0","0","0","C3","C3","C3","","","","","","","","","","","","","","",""},
		 {"35","0","0","0","C4","C4","C4","","","","","","","","","","","","","","",""},
		 {"36","0","0","0","C5","C5","C5","","","","","","","","","","","","","","",""},
		 {"37","0","0","0","C6","C6","C6","","","","","","","","","","","","","","",""},
		 {"38","0","0","0","C7","C7","C7","","","","","","","","","","","","","","",""},
		 {"39","0","0","0","C8","C8","C8","","","","","","","","","","","","","","",""},
		 {"40","0","0","0","C9","C9","C9","","","","","","","","","","","","","","",""},
		 {"41","0","0","0","C10","C10","C10","","","","","","","","","","","","","","",""},
		 {"42","0","0","0","C11","C11","C11","","","","","","","","","","","","","","",""},
		 {"43","0","0","0","C12","C12","C12","","","","","","","","","","","","","","",""},
		 {"44","0","0","0","C13","C13","C13","","","","","","","","","","","","","","",""},
		 {"45","0","0","0","C14","C14","C14","","","","","","","","","","","","","","",""},
		 {"46","0","0","0","C15","C15","C15","","","","","","","","","","","","","","",""},
		 {"47","0","0","0","C16","C16","C16","","","","","","","","","","","","","","",""},
		 {"48","0","0","0","D1","D1","D1","","","","","","","","","","","","","","",""},
		 {"49","0","0","0","D2","D2","D2","","","","","","","","","","","","","","",""},
		 {"50","0","0","0","D3","D3","D3","","","","","","","","","","","","","","",""},
		 {"51","0","0","0","D4","D4","D4","","","","","","","","","","","","","","",""},
		 {"52","0","0","0","D5","D5","D5","","","","","","","","","","","","","","",""},
		 {"53","0","0","0","D6","D6","D6","","","","","","","","","","","","","","",""},
		 {"54","0","0","0","D7","D7","D7","","","","","","","","","","","","","","",""},
		 {"55","0","0","0","D8","D8","D8","","","","","","","","","","","","","","",""},
		 {"56","0","0","0","D9","D9","D9","","","","","","","","","","","","","","",""},
		 {"57","0","0","0","D10","D10","D10","","","","","","","","","","","","","","",""},
		 {"58","0","0","0","D11","D11","D11","","","","","","","","","","","","","","",""},
		 {"59","0","0","0","D12","D12","D12","","","","","","","","","","","","","","",""},
		 {"60","0","0","0","D13","D13","D13","","","","","","","","","","","","","","",""},
		 {"61","0","0","0","D14","D14","D14","","","","","","","","","","","","","","",""},
		 {"62","0","0","0","D15","D15","D15","","","","","","","","","","","","","","",""},
		 {"63","0","0","0","D16","D16","D16","","","","","","","","","","","","","","",""},
		 {"64","0","0","0","E1","E1","E1","","","","","","","","","","","","","","",""},
		 {"65","0","0","0","E2","E2","E2","","","","","","","","","","","","","","",""},
		 {"66","0","0","0","E3","E3","E3","","","","","","","","","","","","","","",""},
		 {"67","0","0","0","E4","E4","E4","","","","","","","","","","","","","","",""},
		 {"68","0","0","0","E5","E5","E5","","","","","","","","","","","","","","",""},
		 {"69","0","0","0","E6","E6","E6","","","","","","","","","","","","","","",""},
		 {"70","0","0","0","E7","E7","E7","","","","","","","","","","","","","","",""},
		 {"71","0","0","0","E8","E8","E8","","","","","","","","","","","","","","",""},
		 {"72","0","0","0","E9","E9","E9","","","","","","","","","","","","","","",""},
		 {"73","0","0","0","E10","E10","E10","","","","","","","","","","","","","","",""},
		 {"74","0","0","0","E11","E11","E11","","","","","","","","","","","","","","",""},
		 {"75","0","0","0","E12","E12","E12","","","","","","","","","","","","","","",""},
		 {"76","0","0","0","E13","E13","E13","","","","","","","","","","","","","","",""},
		 {"77","0","0","0","E14","E14","E14","","","","","","","","","","","","","","",""},
		 {"78","0","0","0","E15","E15","E15","","","","","","","","","","","","","","",""},
		 {"79","0","0","0","E16","E16","E16","","","","","","","","","","","","","","",""},
		 {"80","0","0","0","F1","F1","F1","","","","","","","","","","","","","","",""},
		 {"81","0","0","0","F2","F2","F2","","","","","","","","","","","","","","",""},
		 {"82","0","0","0","F3","F3","F3","","","","","","","","","","","","","","",""},
		 {"83","0","0","0","F4","F4","F4","","","","","","","","","","","","","","",""},
		 {"84","0","0","0","F5","F5","F5","","","","","","","","","","","","","","",""},
		 {"85","0","0","0","F6","F6","F6","","","","","","","","","","","","","","",""},
		 {"86","0","0","0","F7","F7","F7","","","","","","","","","","","","","","",""},
		 {"87","0","0","0","F8","F8","F8","","","","","","","","","","","","","","",""},
		 {"88","0","0","0","F9","F9","F9","","","","","","","","","","","","","","",""},
		 {"89","0","0","0","F10","F10","F10","","","","","","","","","","","","","","",""},
		 {"90","0","0","0","F11","F11","F11","","","","","","","","","","","","","","",""},
		 {"91","0","0","0","F12","F12","F12","","","","","","","","","","","","","","",""},
		 {"92","0","0","0","F13","F13","F13","","","","","","","","","","","","","","",""},
		 {"93","0","0","0","F14","F14","F14","","","","","","","","","","","","","","",""},
		 {"94","0","0","0","F15","F15","F15","","","","","","","","","","","","","","",""},
		 {"95","0","0","0","F16","F16","F16","","","","","","","","","","","","","","",""}
	};
	
	private String[][] SpotFinderResults = new String[][]
    {
		{"sfType","runIndex","frameIndex","state","spot_total","bragg_candidates","res_estimate","int_signal"},
		{"COLLECT","1","0", "DONE","522","304","4.12","0"},
		{"COLLECT","1","1", "DONE","492","272","4.27","0"},
		{"COLLECT","1","2", "DONE","417","223","4.5","0"},
		{"COLLECT","1","3", "DONE","344","183","4.56","0"},
		{"COLLECT","1","4", "DONE","383","211","4.6","0"},
		{"COLLECT","1","5", "DONE","333","202","4.39","0"},
		{"COLLECT","1","6", "DONE","423","254","4.7","0"},
		{"COLLECT","1","7", "DONE","397","248","4.14","0"},
		{"COLLECT","1","8", "DONE","317","182","4.4","0"},
		{"COLLECT","1","9", "DONE","365","188","4.24","0"},
		{"COLLECT","1","10","DONE","423","252","4.32","0"},
		{"COLLECT","1","11","DONE","352","221","4.31","0"},
		{"COLLECT","1","12","DONE","389","239","4.35","0"},
		{"COLLECT","1","13","DONE","380","201","4.4","0"},
		{"COLLECT","1","14","DONE","381","235","4.21","0"},
		{"COLLECT","1","15","DONE","445","276","4.45","0"},
		{"COLLECT","1","16","DONE","401","245","4.53","0"},
		{"COLLECT","1","17","DONE","425","248","4.46","0"},
		{"COLLECT","1","18","DONE","371","104","4.52","0"},
		{"COLLECT","1","19","DONE","427","243","4.46","0"},
		{"COLLECT","1","20","DONE","382","223","4.08","0"},
		{"COLLECT","1","21","DONE","352","107","4.37","0"},
		{"COLLECT","1","22","DONE","451","122","4.16","0"},
		{"COLLECT","1","23","DONE","374","124","4.44","0"},
		{"COLLECT","1","24","DONE","357","142","4.25","0"},
		{"COLLECT","1","25","DONE","350","139","4.29","0"},
		{"COLLECT","1","26","DONE","318","110","4.49","0"},
		{"COLLECT","1","27","DONE","347","205","4.33","0"},
		{"COLLECT","1","28","DONE","395","104","4","0"},
		{"COLLECT","1","29","DONE","290","91","4.39","0"},
		{"COLLECT","1","30","DONE","356","82","4.16","0"},
		{"COLLECT","1","31","DONE","367","211","4.09","0"},
		{"COLLECT","1","32","DONE","319","85","4.13","0"},
		{"COLLECT","1","33","DONE","296","87","3.99","0"},
		{"COLLECT","1","34","DONE","275","90","4.06","0"},
		{"COLLECT","1","35","DONE","376","93","4.23","0"},
		{"COLLECT","1","36","DONE","404","89","4.25","0"},
		{"COLLECT","1","37","DONE","353","84","4.24","0"},
		{"COLLECT","1","38","DONE","282","115","4.3","0"},
		{"COLLECT","1","39","DONE","269","85","4.07","0"},
		{"COLLECT","1","40","DONE","282","79","4.17","0"},
		{"COLLECT","1","41","DONE","292","83","4.05","0"},
		{"COLLECT","1","42","DONE","304","86","4.17","0"},
		{"COLLECT","1","43","DONE","297","84","4.13","0"},
		{"COLLECT","1","44","DONE","333","86","4.26","0"},
		{"COLLECT","1","45","DONE","288","85","3.96","0"},
		{"COLLECT","1","46","DONE","334","92","4.07","0"},
		{"COLLECT","1","47","DONE","337","85","4.16","0"},
		{"COLLECT","1","48","DONE","265","83","4.31","0"},
		{"COLLECT","1","49","DONE","305","70","4","0"},
		{"COLLECT","1","50","DONE","264","83","4.17","0"},
		{"COLLECT","1","51","DONE","305","70","3.99","0"},
		{"COLLECT","1","52","DONE","326","79","3.93","0"},
		{"COLLECT","1","53","DONE","235","81","8.33","0"},
		{"COLLECT","1","54","DONE","264","93","4.29","0"},
		{"COLLECT","1","55","DONE","357","83","4.08","0"},
		{"COLLECT","1","56","DONE","261","80","4.1","0"},
		{"COLLECT","1","57","DONE","266","94","7.24","0"},
		{"COLLECT","1","58","DONE","343","84","7.81","0"},
		{"COLLECT","1","59","DONE","321","86","7.62","0"},
		{"COLLECT","1","60","DONE","266","97","4.19","0"},
		{"COLLECT","1","61","DONE","295","75","3.91","0"},
		{"COLLECT","1","62","DONE","343","89","4.14","0"},
		{"COLLECT","1","63","DONE","294","86","8.54","0"},
		{"COLLECT","1","64","DONE","303","94","9.08","0"},
		{"COLLECT","1","65","DONE","338","93","8.54","0"},
		{"COLLECT","1","66","DONE","291","83","8.38","0"},
		{"COLLECT","1","67","DONE","268","87","5.54","0"},
		{"COLLECT","1","68","DONE","305","97","4.03","0"},
		{"COLLECT","1","69","DONE","259","96","6.65","0"},
		{"COLLECT","1","70","DONE","336","89","7.19","0"},
		{"COLLECT","1","71","DONE","352","101","8.19","0"},
		{"COLLECT","1","72","DONE","275","98","8.08","0"},
		{"COLLECT","1","73","DONE","293","91","6.63","0"},
		{"COLLECT","1","74","DONE","357","102","7.79","0"},
		{"COLLECT","1","75","DONE","325","92","8.17","0"},
		{"COLLECT","1","76","DONE","283","99","4.96","0"},
		{"COLLECT","1","77","DONE","335","107","8.66","0"},
		{"COLLECT","1","78","DONE","336","104","6.04","0"},
		{"COLLECT","1","79","DONE","375","110","4.18","0"},
		{"COLLECT","1","80","DONE","273","97","5.57","0"},
		{"COLLECT","1","81","DONE","305","94","7.93","0"},
		{"COLLECT","1","82","DONE","284","105","8.94","0"},
		{"COLLECT","1","83","DONE","295","112","8.09","0"},
		{"COLLECT","1","84","DONE","305","109","8.23","0"},
		{"COLLECT","1","85","DONE","314","107","8.07","0"},
		{"COLLECT","1","86","DONE","341","107","8.24","0"},
		{"COLLECT","1","87","DONE","351","98","8.57","0"},
		{"COLLECT","1","88","DONE","365","102","6.4","0"},
		{"COLLECT","1","89","DONE","318","103","8.37","0"}
    };
	
	private String[][] Macros = new String[][]
    {
		{"ID","rasterMode","prefix","dir","beamWidth","beamHeight","cellWidth","cellHeight","exposureTime","delta","ccdDistance","attenuation","processingEnabled","energy","ROI"}
	};
	
	private String[][] StrategyResults = new String[][]
    {
 		{"sampleNumber","sampleName",	"solutionNumber",	"good",	"oscstart",	"anom_oscstart","oscend",	"anom_oscend",	"continuous",	"complete",	"anom_complete","oscdelta",	"detectordistance",	"detectorwarning",																																			"spaceGroup_short",	"spaceGroup",												"resolution",	"unitcell",								"rmsd",	"beamx",	"beamy"},
 		{"1",			"test_1",		"2",				"yes",	"104.00",	"85.00",		"190.10",	"210.45",		"335.90",		"98.53%",	"96.96%",		"0.70",		"477.9",			"The calculated detector distance (477.9 mm) is longer than collected detector distance.  Full detector surface will be used at this detector distance.",	"P121",				"(2)  P121 P1211-->57.62,89.26,62.83,90.00,109.65,90.00",	"3.41 A",		"57.62 89.26 62.83 90.00 109.65 90.00",	"N/A",	"150.10",	"150.01"}
 	};
	
	private String[][] MultiStrategyResults = new String[][]
    {
 		{"sampleNumber", "state", "workdir", "referencedata", "images", "unitcell", "lowresolution", "highresolution", "spacegroup", "displaytext", "scalalog", "xplanlog", "warning", "timestamp"},
		{ "1", "DONE", "/mnt/share1/user0/sudhir/testxds/strategy", "", "/mnt/share1/user0/sudhir/testxds/sdrc_nat4_1.???? 135", "45.80    48.48    78.13  90.075  87.099  90.300", "", "", "1", "************************************************************************\n\n            GUIDE FOR OPTIMAL SELECTION OF DATA COLLECTION STRATEGY\n\n ************************************************************************\n\n Assuming that the currently processed data satisfy\n\n SPACE_GROUP_NUMBER=    1\n UNIT_CELL_CONSTANTS=    45.80    48.48    78.13  90.075  87.099  90.300\n FRIEDELS_LAW=TRUE  \n STARTING_ANGLE=   174.000\n STARTING_FRAME=   135\n\n you would collect a maximum of data for the following combinations\n of starting spindle_angle and total range of rotation (degrees):\n\n starting at     total rotation     completeness        multiplicity of\n spindle_angle        range          of data (%)        measurements\n\n      80.00           30.00            24.53                 1.09\n     120.00           60.00            40.06                 1.33\n      70.00           90.00            53.68                 1.49\n      60.00          120.00            66.83                 1.60\n      30.00          150.00            78.03                 1.71\n       0.00          180.00            80.31                 1.99\n\n************************************************************************\n", "", "", "DONE", "2013-01-15-16:57:22"}
	};
	private String[][] HKLDatasets = new String[][]
	{
			{"Select", "HKLFile", "Resolution", "Spacegroup"},
			{"1", "/mnt/share1/user0/naga/xds1/XDS_ASCII.HKL", "1.348", "18"},
	};
	private String[][] MultiStrategyMergeDatasets = new String[][]
	{
			{"merge_result", "warning", "dir", "state" },
			{"/mnt/share1/user0/naga/xds1/XDS_ASCII.HKL /mnt/share1/user0/naga/xds2/XDS_ASCII.HKL", "/mnt/share1/user0/nagatest/XDS_ASCII.HKL", "DONE", "/mnt/share1/user0/nagatest","DONE"}
	};
	
	private String[][] MultiStrategyReferenceData = new String[][]
	{
		{"isReference", "reference_data", "unit_cell", "images", "spaceGroup", "lowRes", "highRes" },
		{"1", "/mnt/share1/user0/sudhir/testxds/secondXtalStrategy/XDS_ASCII.HKL", "48.644 51.740 83.319 90.000 92.776 90.000", "/mnt/share1/user0/softtest/sudhir/18012013/F1/F1_19_scr.0046 /mnt/share1/user0/softtest/sudhir/18012013/F1/F1_19_scr.0047","3", "50.0", "2.267" }
	};

	private String[][] DataProcResults = new String[][]
	{
		{"sampleNumber","sampleName","state","workdir","images","unitcell","lowresolution","lowresolution_inner","lowresolution_outer","highresolution","highresolution_inner","highresolution_outer","rmerge","rmerge_inner","rmerge_outer","completeness","completeness_inner","completeness_outer","multiplicity","multiplicity_inner","multiplicity_outer","isigma","isigma_inner","isigma_outer","spacegroup","scalalog","truncatelog","warning","timestamp"}
	};
	
	private String[][] AutoindexResults = new String[][]
    {
		{"sampleNumber","sampleName","state",	"workdir",			"images",					"score","resolution",	"mosaicity","warning",																					"timestamp"},
		{"1",			"test_1",	"DONE",		"/mnt/share2/test1","test_1.0001 test_1.0091",	"",		"",				"N/A",		"Spotfinder Problem: Too few Bragg spots (15) in image 1 to construct resolution profile\n","2012-05-18-10:25:00"},
	};

	// these sites are from an imagined 3 pass raster
	// original 40x40 grid is a 5x5, or 200x200 total
	// pass 1 40x40: found one multi-cell hit (80x40) and one single-cell hit (40x40)
	// pass 2 10x10: found two hits in the single-cell hit from pass 1
	// pass 3 40x80: ortho for pass 1 hit 1 (found single-cell hit)
	// pass 4 10x80: ortho for pass 2 hit 1 (found single-cell hit)
	// pass 5 10x80: ortho for pass 2 hit 2 (found a two-cell hit)
	//
	// Coarse grid: *=hit  .=non-hit
	//
	// .....
	// .**..
	// .....
	// ...*.
	// .....
	//
	protected double sampleX=-.0025, sampleY=-.034, sampleZ=3.81;
	protected String[][] RasterSites =
	{
			{ "site_index",	"inter_run","inter_cell",	"fine_run",	"ortho_run",	"hit_number",	"motor_x",			"motor_y",			"motor_z",			"motor_o",	"size_x",	"size_y",	"size_z" },
			/*{ "1",			"-1",		"-1",			"-1",		"3",			"1",			""+sampleX,			""+(sampleY-.04),	""+(sampleZ+.02),	"0",		"80",		"40",		"40"},
			{ "2",			"-1",		"-1",			"2",		"4",			"3",			""+sampleX,			""+sampleY,			""+sampleZ,			"0",		"10",		"10",		"10"},
			{ "3",			"-1",		"-1",			"2",		"5",			"1",			""+(sampleX+.01),	""+sampleY,			""+(sampleZ-.01),	"0",		"10",		"10",		"20"},
			{ "4",			"0",		"3",			"-1",		"-1",			"-1",			""+(sampleX+.02),	""+(sampleY-.01),	""+(sampleZ-.03),	"0",		"10",		"10",		"20"}*/
	};
	
	private SQLTable convertToSQLTable(String[][] table)
	{
		ArrayList<HashMap<String,String>> results = new ArrayList<HashMap<String,String>>();
		
		for (int rowIndex = 1; rowIndex < table.length; rowIndex++)
		{
			HashMap<String,String> row = new HashMap<String,String>();
			
			for (int colIndex=0; colIndex<table[0].length; colIndex++)
			{
				row.put(table[0][colIndex], table[rowIndex][colIndex]);
			}
			
			results.add(row);
		}

		return new SQLTable(results);
	}
	
	@Override protected void i_init()
	{
		loadSnapshot();
		
		// raster events
		DevEvents.createTable(RasterEvent.GMCA_RASTER_DATA_TABLE, RasterParam.values());
		ArrayList<HashMap<String,String>> rows = new ArrayList<HashMap<String,String>>();
		HashMap<String,String> row = new HashMap<String,String>();
		for (RasterParam param : RasterParam.values())
		{
			row.put(param.dbString, "0");
		}
		rows.add(row);
		db.put(RasterEvent.GMCA_RASTER_DATA_TABLE, new SQLTable(rows));

		// scan events
		DevEvents.createTable(ScanEvent.GMCA_SCAN_DATA_TABLE, ScanParam.values());
		rows = new ArrayList<HashMap<String,String>>();
		row = new HashMap<String,String>();
		for (ScanParam param : ScanParam.values())
		{
			row.put(param.dbString, "0");
		}
		rows.add(row);
		db.put(ScanEvent.GMCA_SCAN_DATA_TABLE, new SQLTable(rows));
		
		// non-event tables
		db.put("RasterSites", 		convertToSQLTable(RasterSites));
		db.put("RasterResults", 	convertToSQLTable(RasterResults));
		db.put("SpotFinderResults",	convertToSQLTable(SpotFinderResults));
		db.put("Macros",			convertToSQLTable(Macros));
		db.put("StrategyResults",	convertToSQLTable(StrategyResults));
		db.put("MultiStrategyResults",	convertToSQLTable(MultiStrategyResults));
		db.put("MultiStrategyMergeDatasets", convertToSQLTable(MultiStrategyMergeDatasets));
		db.put("MultiStrategyReferenceData",	convertToSQLTable(MultiStrategyReferenceData));
		db.put("AutoindexResults",	convertToSQLTable(AutoindexResults));
		db.put("DataProcResults",	convertToSQLTable(DataProcResults));
		db.put("HKLDatasets",	convertToSQLTable(HKLDatasets));
	}
	
	private void loadSnapshot()
	{
		BufferedReader reader = Resources.get().getReader("data/blc2004.sql");
		
		String line = null;
		String currentTable = null;
		ArrayList<String> currentHeaders = null;

		while ( null != (line = getNextLine(reader)) )
		{
			StringTokenizer st = new StringTokenizer(line," `()',;");
			
			if (st.hasMoreTokens() && st.nextToken().equals("CREATE") && 
				st.hasMoreTokens() && st.nextToken().equals("TABLE"))
			{
				currentTable = st.nextToken();
				
				currentHeaders = new ArrayList<String>();
				while (true)
				{
					line = getNextLine(reader);
					if (line.charAt(0)==')') break;
					
					StringTokenizer st2 = new StringTokenizer(line," `");
					String value = st2.nextToken();
					if (!value.equals("PRIMARY"))
					{
						currentHeaders.add(value);
					}
				}
				
				continue;
			}
			
			st = new StringTokenizer(line," `,;");
			if (st.hasMoreTokens() && st.nextToken().equals("INSERT") && 
				st.hasMoreTokens() && st.nextToken().equals("INTO"))
			{
				line = line.substring(line.indexOf("VALUES")+7);

				ArrayList<HashMap<String,String>> table = new ArrayList<HashMap<String,String>>();
				HashMap<String,String> row = new HashMap<String,String>();
				boolean inParen = false;
				boolean inQuotes = false;
				String currentValue = new String("");
				int headerIndex = 0;
				for (int i=0; i<line.length(); i++)
				{
					char c = line.charAt(i);
					if (c == '(' && !inQuotes)
					{
						inParen = true;
						continue;
					}
					
					if (c == ')' && !inQuotes)
					{
						if (headerIndex == currentHeaders.size()) headerIndex = 0;
						String header = currentHeaders.get(headerIndex++);
						
						row.put(header, currentValue);
						table.add(row);
						row = new HashMap<String,String>();
						
						currentValue = new String("");
						inParen = false;
						continue;
					}

					if (c == '\'')
					{
						inQuotes = !inQuotes;
						continue;
					}
					
					if (c == ',' && !inQuotes && inParen)
					{
						if (headerIndex == currentHeaders.size()) headerIndex = 0;
						String header = currentHeaders.get(headerIndex++);
						
						row.put(header, currentValue);
						
						currentValue = new String("");
						continue;
					}
					
					if (inParen) currentValue += c;
				}
				
				db.put(currentTable, new SQLTable(table));
				
				continue;
			}
		}
	}
	
	// takes a list of requested columns, a table name, and a list of where clauses
	// returns a "table" with values taken from the stored SQL tables
	private Result constructResult(HashSet<String> requestedColumns, String tableName, HashMap<String,String> whereMap)
	{
		SQLTable table = db.get(tableName);
		ArrayList<HashMap<String,String>> contents = new ArrayList<HashMap<String,String>>();
		
		// go through each row of the table
		for (HashMap<String,String> row : table.contents)
		{
			// make a new row with only the requested columns
			HashMap<String,String> newRow = new HashMap<String,String>();
			for (String col : row.keySet())
			{
				if (requestedColumns.contains(col))
				{
					newRow.put(col, row.get(col));
				}
			}
			
			// check wheremap to see if this row should be returned
			boolean rowOK = true;
			if (null != whereMap) // whereMap is optional
			{
				for (String key : whereMap.keySet())
				{
					if (!row.containsKey(key) || !row.get(key).equals(whereMap.get(key)))
					{
						rowOK = false;
					}
				}
			}
			if (!rowOK) continue;
			
			// add the row
			contents.add(newRow);
		}
		
		return new Result(contents);
	}
}
