package gov.anl.gmca.io.sim.dev;

import gov.anl.gmca.collect.DevBinMode.BinMode;
import gov.anl.gmca.collect.ModelCollectRun.OptionTab;
import gov.anl.gmca.collect.ModelCollectRun.ScanMode;
import gov.anl.gmca.collect.ModelCollectRun.SetupStep;
import gov.anl.gmca.collect.ModelCollectRunList.CollectLeftTab;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.io.PVLookup.*;
import gov.anl.gmca.io.sim.Epics;
import gov.anl.gmca.raster.ModelRasterRun.RunState;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.scan.ModelScan.EDGE_IND;

import java.util.HashMap;

public class SimDeviceInit
{
	private static final boolean USE_TEST_DATA = true;
	
    protected static HashMap<String,HashMap<String,String>> pmaMap = new HashMap<String,HashMap<String,String>>();
	
	public static HashMap<String,String> getPmacRow(String id)
	{
		return pmaMap.get(id);
	}
	
	public static void init()
	{
    	///////////////////////////////////////////////////////// Misc. PVs //////////////////////////////////////////////////////

		// beam align info
		//Epics.putInteger(PV_BLUICE.alignLastRunTime, (int)(System.currentTimeMillis()/1000L));
		//Epics.putDouble(PV_BLUICE.alignLastEnergy, 12.667);
		
		Epics.put(PV_BLUICE.data_dir, "/mnt/beegfs/user0/23IDB_2017_01_01");

		new SimHutchShutter("endStation", true);
		
		// CCD bin modes
		Epics.putInteger(PV_RUNS.binMode, BinMode.UM_72.ordinal());
		Epics.putInteger(PV_R_RUNS.binMode, BinMode.UM_144.ordinal());
		
		// measured beam size
		Epics.putDouble(PV_BLUICE.beamsize_h_act1$VAL, 116.25);
		Epics.putDouble(PV_BLUICE.beamsize_v_act1$VAL, 31.6083);
		
		// beamstop in
		Epics.putInteger(PV_PMAC21.bi21_VAL, 1);
		
		// endstation secure
		Epics.putInteger(PV_MISC.stationSearched, 1);
		
		// 101.3mA beam and permit enabled
		Epics.putDouble(PV_MISC.apsCurrent, 101.3);
		Epics.putInteger(PV_MISC.apsShutterStat, 1);
		
		// scan tab
		Epics.putIntegerArray(PV_SCAN.displayedElements, new int[EDGE_IND.values().length]);
		
		Epics.putInteger(PV_R_RUN.autoFind.run(RasterListType.AUTO, 0), 1);

		// gonio limits
		Epics.putDouble(PV_PMAC.RqsPos_DRVL.motor("gonio_omega"), -1000.0);
		Epics.putDouble(PV_PMAC.RqsPos_DRVH.motor("gonio_omega"), 1000.0);
		
		// camera calibration
		Epics.putDouble(PV_BLUICE.px2mm_h_lo$VAL, 0.0146247);
		Epics.putDouble(PV_BLUICE.px2mm_v_lo$VAL, 0.0158263);
		Epics.putDouble(PV_BLUICE.px2mm_h_hi$VAL, 0.000354143);
		Epics.putDouble(PV_BLUICE.px2mm_v_hi$VAL, 0.000382211);

		// robot
		Epics.put(PV_ROBOT.sta, "SMR: Ready");

		// collect run 0
		Epics.putInteger(PV_RUN.runLabel.run(0), 		0);
		Epics.putInteger(PV_RUN.scanMode.run(0), 		ScanMode.SINGLE.ordinal());
		Epics.putInteger(PV_RUN.optionTab.run(0), 		OptionTab.COLLECT_PARAM.ordinal());
		Epics.put		(PV_RUN.fileroot.run(0),		"A1");
		Epics.put		(PV_RUN.directory256.run(0),	"A1");

		Epics.putDouble (PV_RUN.distance.run(0), 		800.0);
		Epics.putDouble (PV_RUN.beamstop_mm.run(0), 	30.5);
		Epics.putDouble (PV_RUN.atten_factor.run(0), 	1.0);
		Epics.putDouble (PV_RUN.delta.run(0), 			1.0);
		Epics.putDouble (PV_RUN.exposureTime.run(0), 	1.0);
		Epics.putDouble (PV_RUN.wedgeSize.run(0), 		180.0);
		Epics.putDouble (PV_RUN.energy1.run(0), 		12.67);
		Epics.putInteger(PV_RUN.numEnergy.run(0), 		1);
		
		Epics.putDouble (PV_RUN.startAngle.run(0), 		0.0);
		Epics.putDouble (PV_RUN.endAngle.run(0), 		1.0);
		Epics.putInteger(PV_RUN.startFrameLabel.run(0), 1);
		
		if (!USE_TEST_DATA)
		{
			// collect runs
			Epics.putInteger(PV_RUNS.runCount, 				0);
			Epics.putInteger(PV_RUNS.currentRun, 			0);
			Epics.putInteger(PV_RUNS.currentLeftTab, 		CollectLeftTab.DIFFRACTION.ordinal());
		}
		else
		{
			// collect runs
			Epics.putInteger(PV_RUNS.runCount, 				1);
			Epics.putInteger(PV_RUNS.currentRun, 			1);
			Epics.putInteger(PV_RUNS.currentLeftTab, 		CollectLeftTab.VECTOR.ordinal());
			
			Epics.put		(PV_FPE.viewDir256,				System.getenv("HOME"));
			Epics.put		(PV_FPE.viewFile256,			"A1_000001.cbf");
			
			// collect run 1
			Epics.putInteger(PV_RUN.runLabel.run(1), 		1);
			Epics.putInteger(PV_RUN.runStatus.run(1),		RunState.COMPLETE.ordinal());
			Epics.putInteger(PV_RUN.nextFrame.run(1), 		5);
			Epics.putInteger(PV_RUN.scanMode.run(1), 		ScanMode.SINGLE.ordinal());
			Epics.putInteger(PV_RUN.optionTab.run(1), 		OptionTab.COLLECT_PARAM.ordinal());
			Epics.putInteger(PV_RUN.setupStep.run(1),		SetupStep.COLLECT_PARAM.ordinal());
			Epics.putInteger(PV_RUN.helicalMode.run(1),		1);
			Epics.put		(PV_RUN.fileroot.run(1),		"A1");
			Epics.put		(PV_RUN.directory256.run(1),	"A1");
			Epics.putInteger(PV_RUN.dbUpdated.run(1),		0);
			
			Epics.putInteger(PV_RUN.inverseOn.run(1), 		0);
			Epics.putInteger(PV_RUN.framesPerSite.run(1), 	1);
			Epics.putDouble (PV_RUN.stepSize_um.run(1), 	10.0);
			Epics.putDouble (PV_RUN.beamWidth_um.run(1), 	43.0);
	
			Epics.putDouble (PV_RUN.distance.run(1), 		800.0);
			Epics.putDouble (PV_RUN.beamstop_mm.run(1), 	30.5);
			Epics.putDouble (PV_RUN.atten_factor.run(1), 	1.0);
			Epics.putDouble (PV_RUN.delta.run(1), 			1.0);
			Epics.putDouble (PV_RUN.exposureTime.run(1), 	1.0);
			Epics.putDouble (PV_RUN.wedgeSize.run(1), 		180.0);
			Epics.putDouble (PV_RUN.energy1.run(1), 		12.67);
			Epics.putInteger(PV_RUN.numEnergy.run(1), 		1);
			
			Epics.putDouble (PV_RUN.startAngle.run(1), 		0.0);
			Epics.putDouble (PV_RUN.endAngle.run(1), 		90.0);
			Epics.putInteger(PV_RUN.startFrameLabel.run(1), 1);
			
			Epics.putDouble (PV_RUN.pt1_x_mm.run(1),		-0.0025);
			Epics.putDouble (PV_RUN.pt1_y_mm.run(1),		-0.0340);
			Epics.putDouble (PV_RUN.pt1_z_mm.run(1),		 3.8100);
			Epics.putInteger(PV_RUN.pt1_set.run(1),		 1);
			
			Epics.putDouble (PV_RUN.pt2_x_mm.run(1),		-0.0025);
			Epics.putDouble (PV_RUN.pt2_y_mm.run(1),		-0.0401);
			Epics.putDouble (PV_RUN.pt2_z_mm.run(1),		 3.7667);
			Epics.putInteger(PV_RUN.pt2_set.run(1),			 1);
			
			Epics.putInteger(PV_RUN.potVecSiteCount.run(1),	 5); // fits vector length
			
			int[] run1selected = new int[1000];
			run1selected[0]=1;
			run1selected[1]=1;
			run1selected[2]=1;
			run1selected[3]=1;
			run1selected[4]=1;
			Epics.putIntegerArray(PV_RUN.selectedSites.run(1), run1selected);
			
			Epics.putInteger(PV_RUN.runLabel.run(2), 		2);
			Epics.putInteger(PV_RUN.scanMode.run(2), 		ScanMode.RASTER.ordinal());
			Epics.putInteger(PV_RUN.optionTab.run(2), 		OptionTab.SITE_SETUP.ordinal());
			Epics.put		(PV_RUN.fileroot.run(2),		"A1");
			Epics.put		(PV_RUN.directory256.run(2),	"A1");
	
			Epics.putInteger(PV_RUN.framesPerSite.run(2), 	1);
			Epics.putDouble (PV_RUN.stepSize_um.run(2), 	10.0);
	
			Epics.putDouble (PV_RUN.distance.run(2), 		800.0);
			Epics.putDouble (PV_RUN.beamstop_mm.run(2), 	30.5);
			Epics.putDouble (PV_RUN.atten_factor.run(2), 	1.0);
			Epics.putDouble (PV_RUN.delta.run(2), 			1.0);
			Epics.putDouble (PV_RUN.exposureTime.run(2), 	1.0);
			Epics.putDouble (PV_RUN.wedgeSize.run(2), 		180.0);
			Epics.putDouble (PV_RUN.energy1.run(2), 		12.67);
			Epics.putInteger(PV_RUN.numEnergy.run(2), 		1);
		
			Epics.putDouble (PV_RUN.startAngle.run(2), 		0.0);
			Epics.putDouble (PV_RUN.endAngle.run(2), 		5.0);
			Epics.putInteger(PV_RUN.startFrameLabel.run(2), 1);
			
			/*int[] run2selected = new int[1000];
			run2selected[0]=1;
			run2selected[1]=1;
			run2selected[2]=1;
			Epics.putIntegerArray(PV_RUN.selectedSites.run(2), run2selected);*/
		}
		
		Epics.putInteger(PV_R_RUN.runLabel          .run(RasterListType.INTER,0), 	0);
		Epics.putInteger(PV_R_RUN.runStatus         .run(RasterListType.INTER,0), 	USE_TEST_DATA ? RunState.COMPLETE.ordinal() : RunState.INACTIVE.ordinal());
		Epics.putDouble (PV_R_RUN.cellWidth_um      .run(RasterListType.INTER,0),	40.0);
		Epics.putDouble (PV_R_RUN.cellHeight_um     .run(RasterListType.INTER,0),	40.0);
		Epics.putDouble (PV_R_RUN.beamWidth_um      .run(RasterListType.INTER,0),	40.0);
		Epics.putDouble (PV_R_RUN.beamHeight_um     .run(RasterListType.INTER,0),	40.0);
		Epics.putDouble (PV_R_RUN.exposure_sec      .run(RasterListType.INTER,0),	1.0);
		Epics.putDouble (PV_R_RUN.delta_deg         .run(RasterListType.INTER,0),	1.0);
		Epics.putDouble (PV_R_RUN.distance_mm       .run(RasterListType.INTER,0),	800.0);
		Epics.putDouble (PV_R_RUN.atten_factor      .run(RasterListType.INTER,0),	1.0);
		Epics.putDouble (PV_R_RUN.lowResLimit_A		.run(RasterListType.INTER,0),	Double.POSITIVE_INFINITY);
		Epics.putDouble (PV_R_RUN.highResLimit_A	.run(RasterListType.INTER,0),	0.0);
		
		Epics.putInteger(PV_R_RUN.runLabel          .run(RasterListType.AUTO,0), 	0);
		Epics.putInteger(PV_R_RUN.runStatus         .run(RasterListType.AUTO,0), 	RunState.INACTIVE.ordinal());
		Epics.putDouble (PV_R_RUN.cellWidth_um      .run(RasterListType.AUTO,0),	40.0);
		Epics.putDouble (PV_R_RUN.cellHeight_um     .run(RasterListType.AUTO,0),	40.0);
		Epics.putDouble (PV_R_RUN.beamWidth_um      .run(RasterListType.AUTO,0),	40.0);
		Epics.putDouble (PV_R_RUN.beamHeight_um     .run(RasterListType.AUTO,0),	40.0);
		Epics.putDouble (PV_R_RUN.exposure_sec      .run(RasterListType.AUTO,0),	1.0);
		Epics.putDouble (PV_R_RUN.delta_deg         .run(RasterListType.AUTO,0),	1.0);
		Epics.putDouble (PV_R_RUN.distance_mm       .run(RasterListType.AUTO,0),	800.0);
		Epics.putDouble (PV_R_RUN.atten_factor      .run(RasterListType.AUTO,0),	1.0);
		
		if (!USE_TEST_DATA)
		{
			Epics.putInteger(PV_R_RUNS.currentRun,			0);
			Epics.putInteger(PV_R_RUNS.runCount,			0);
		}
		else
		{
			Epics.putInteger(PV_R_RUNS.currentRun,			1);
			Epics.putInteger(PV_R_RUNS.runCount,			1);
			
			Epics.putIntegerArray(PV_R_RUN.selectedCells.run(RasterListType.INTER,0), 	new int[]{0xFFFF});
			
			Epics.putInteger(PV_R_RUN.runLabel        		.run(RasterListType.INTER,1), 	1);
			Epics.putInteger(PV_R_RUN.runStatus       		.run(RasterListType.INTER,1), 	RunState.COMPLETE.ordinal());
			Epics.putDouble (PV_R_RUN.cellWidth_um    		.run(RasterListType.INTER,1),	20.0);
			Epics.putDouble (PV_R_RUN.cellHeight_um   		.run(RasterListType.INTER,1),	20.0);
			Epics.putDouble (PV_R_RUN.beamWidth_um    		.run(RasterListType.INTER,1),	20.0);
			Epics.putDouble (PV_R_RUN.beamHeight_um   		.run(RasterListType.INTER,1),	20.0);
			Epics.putDouble (PV_R_RUN.exposure_sec    		.run(RasterListType.INTER,1),	1.0);
			Epics.putDouble (PV_R_RUN.delta_deg       		.run(RasterListType.INTER,1),	1.0);
			Epics.putDouble (PV_R_RUN.distance_mm     		.run(RasterListType.INTER,1),	800.0);
			Epics.putDouble (PV_R_RUN.atten_factor    		.run(RasterListType.INTER,1),	1.0);
			Epics.putInteger(PV_R_RUN.isCenterSet	  		.run(RasterListType.INTER,1),	1);
			Epics.putDouble( PV_R_RUN.center_x_mm	  		.run(RasterListType.INTER,1),	SimPmacMotor.getMotorInitPos("sample_x"));
			Epics.putDouble( PV_R_RUN.center_y_mm	  		.run(RasterListType.INTER,1),	SimPmacMotor.getMotorInitPos("sample_y"));
			Epics.putDouble( PV_R_RUN.center_z_mm	  		.run(RasterListType.INTER,1),	SimPmacMotor.getMotorInitPos("sample_z"));
			Epics.putDouble( PV_R_RUN.gridBottomOffset_um	.run(RasterListType.INTER,1),	40.0);
			Epics.putDouble( PV_R_RUN.gridTopOffset_um		.run(RasterListType.INTER,1),	-40.0);
			Epics.putDouble( PV_R_RUN.gridLeftOffset_um		.run(RasterListType.INTER,1),	-40.0);
			Epics.putDouble( PV_R_RUN.gridRightOffset_um	.run(RasterListType.INTER,1),	40.0);
			Epics.putIntegerArray(PV_R_RUN.selectedCells	.run(RasterListType.INTER,1), 	new int[]{16,0x7FFF8000});
			Epics.putDoubleArray (PV_R_RUN.contours			.run(RasterListType.INTER,1),	new double[]
	        {
				1, 			// # of contours
				4, 			// # of points in this contour
				-10,-10, 	// x1,y1
				+10,-10,	// x2,y2
				+10,+10,	// x3,y3
				-10,+10,	// x4,y4
			});
		}
		
		Epics.put		(PV_SCAN.prefix,				"test");
		Epics.put		(PV_SCAN.directory,				"/");
		Epics.putDouble	(PV_SCAN.spectrumTime,			1.0);
		Epics.putDouble	(PV_SCAN.time,					1.0);
		Epics.putInteger(PV_SCAN.currentTab,			1);
		Epics.putInteger(PV_SCAN.roiLow,				500);
		Epics.putInteger(PV_SCAN.roiHigh,				600);
		
    	//////////////////////////////////////////////////////// PMAC motors /////////////////////////////////////////////////////
    	
    	Result pmaResult = Database.exec(DBName.CONFIG, "select * from PmacMotorAssembly where name in ('sample_x','sample_y','sample_z','gonio_omega','detector_z','detector_2t','beamstop_z','beam_size_x','beam_size_y','mo_En_E','camera_zoom','camera_zoom_2')");
		for (int i=0;i<pmaResult.numRows();i++)
		{
			String devName = pmaResult.getRow(i).get("name");
			
			if (null == pmaMap.get(devName))
			{
				pmaMap.put(devName, pmaResult.getRow(i));
				new SimPmacMotor(devName);
			}
		}
		
		///////////////////////////////////////////////////////// shutters ///////////////////////////////////////////////////////
		
    	pmaResult = Database.exec(DBName.CONFIG, "select * from EpicsShutter");
		for (int i=0;i<pmaResult.numRows();i++)
		{
			HashMap<String,String> row = pmaResult.getRow(i);
			
			String devName = row.get("name");
			
			// some filters are turned off if they're not installed
			boolean enable = true; //row.get("enable").equals("1");
			
			boolean startOpen = !devName.equals("shutter"); // start fast shutter closed
			if (enable) new SimShutter(devName, startOpen);
		}
		
		////////////////////////////////////////////////////////// servers ///////////////////////////////////////////////////////
	
		new SimCamera();
		new SimPnC();
		new SimFPE();
		new SimSampleServer();
		new SimSampleServer();
	}
}
