package gov.anl.gmca.io.sim.dev;

import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_FPE;
import gov.anl.gmca.io.PVLookup.PV_SCAN;
import gov.anl.gmca.io.Resources;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Point2D;
import gov.anl.gmca.util.Util;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.StringTokenizer;

public class SimEdgeScan implements Runnable
{
	private static final String fakeDataFile = "/data/fake_EDGE_Se_K.dat";
	private static boolean FAST_SIM = false;
	private static boolean simStopRequested = false;
	
	@Override public void run()
	{
		simStopRequested = false;
		
		ArrayList<Point2D> fakePoints = new ArrayList<Point2D>();
		
		Epics.put(PV_FPE.cmd_output256, "setGraphXLimit 12628.50 12687.50");
		
		int putCount=5;
		BufferedReader br;
		try
		{
			br = Resources.get().getReader(fakeDataFile);
			
			br.readLine();
			br.readLine();
			
			ArrayList<Double> fakeArray = null;
			while (true)
			{
				String line = br.readLine();
				if (null == line) break;
				
				StringTokenizer st = new StringTokenizer(line, " ");
				if (2 != st.countTokens()) continue;
				
				double energy = Double.parseDouble(st.nextToken());
				double value = Double.parseDouble(st.nextToken());
				
				fakePoints.add(new Point2D(energy,value));

				fakeArray = new ArrayList<Double>();
				fakeArray.add(1.0);
				fakeArray.add((double)fakePoints.size());
				for (Point2D pt : fakePoints)
				{
					fakeArray.add(pt.x);
				}
				for (Point2D pt : fakePoints)
				{
					fakeArray.add(pt.y);
				}

				if (!FAST_SIM)
				{
					putCount--;
					if (0==putCount)
					{
    					Epics.putDoubleArray(PV_SCAN.fastscanData, Util.toPrimitiveArray(fakeArray));
    					Util.sleep(200);
    					while (true)
    					{
    						double status = Epics.getDouble(PV_SCAN.fastscanData);
    						if (0.0 == status) break;
    						Util.sleep(10);
    					}
    					putCount = 5;
					}
				}
				
				if (simStopRequested)
				{
					Log.note("Stop requested, ending edge scan");
					break;
				}
			}
			
			if (FAST_SIM && !simStopRequested)
			{
				Epics.putDoubleArray(PV_SCAN.fastscanData, Util.toPrimitiveArray(fakeArray));
				Util.sleep(20);
				while (0 != Epics.getDouble(PV_SCAN.fastscanData)) Util.sleep(10);
			}

			// send finish signal
			Epics.putDoubleArray(PV_SCAN.fastscanData, new double[]{3,0,0,0});
			Util.sleep(20);
			while (0 != Epics.getDouble(PV_SCAN.fastscanData)) Util.sleep(10);
			
			br.close();
		}
		catch (FileNotFoundException e)		{ e.printStackTrace(); }
		catch (IOException e)				{ e.printStackTrace(); }
	}
}
