package gov.anl.gmca.io.sim.dev;

import gov.anl.gmca.io.PVLookup.PV_PMAC;
import gov.anl.gmca.io.sim.Epics;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;

import java.util.HashMap;

public class SimPmacMotor
{
	protected String devName;
	public String getDeviceName() { return devName; }
	
	private static Object[][] initMotorStates =
	{
		// dev name			ActPos		speed		RqsBkl
		// ----------------------------------------------------
		{ "gonio_omega",	0.0,		90.0,		0.0 		},
		{ "beamstop_z",		30.5,		2.94,		0.01 		},
		{ "beam_size_x",	0.0426,		1.04,		0.014 		},
		{ "beam_size_y",	0.0426, 	1.16,		0.014 		},
		{ "camera_zoom",	6.0,		3.84,		0.000658 	},
		{ "camera_zoom_2",	14.0,		1.37,		0.000685 	},
		{ "detector_2t",	0.0,		0.3,		0.001819 	},
		{ "detector_z",		800.0,		24.6,		0.998 		},
		{ "mo_En_E",		12.67,		0.0786,		0.00506		},
		{ "sample_x",		-.0025,		2.06,		0.008 		},
		{ "sample_y",		-.034,		2.06,		0.008 		},
		{ "sample_z",		3.81,		10.0,		0.005 		}
	};
	
	public static Double getMotorInitPos(String motorID)
	{
		for (Object[] row : initMotorStates)
		{
			String id = (String)row[0];
			if (id.equals(motorID)) return (Double)row[1];
		}
		return null;
	}
	
	public SimPmacMotor(final String devName)
	{
		this.devName = devName;
		
		Epics.putInteger	(PV_PMAC.Homed.motor(devName), 		1);
		Epics.putInteger	(PV_PMAC.Motion.motor(devName), 	3); 	// "position"
		Epics.putInteger	(PV_PMAC.AmpEna.motor(devName),	 	1);
		Epics.putInteger	(PV_PMAC.AutoStart.motor(devName),	0); 	// 0=auto, no extra start cmd needed
		Epics.putInteger	(PV_PMAC.InPos.motor(devName), 		1);
		Epics.putInteger	(PV_PMAC.RunPrg.motor(devName),		0); 	// 0=not running

		for (Object[] initState : initMotorStates)
		{
			if (devName.equals((String)initState[0]))
			{
    			Epics.putDouble	(PV_PMAC.ActPos.motor(devName),		(Double)initState[1]);
    			Epics.putDouble	(PV_PMAC.RqsPos.motor(devName),		(Double)initState[1]);
    			Epics.putDouble	(PV_PMAC.speed.motor(devName),		(Double)initState[2]);
    			Epics.putDouble	(PV_PMAC.RqsBkl.motor(devName),		(Double)initState[3]);
			}
		}
		
		Epics.addMonitor(PV_PMAC.RqsPos.motor(devName), new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				Double rqsPos = Epics.dbrToDouble(value);
				Double ActPos = Epics.getDouble(PV_PMAC.ActPos.motor(devName));
				Double dist = Math.abs(rqsPos - ActPos);
				if (dist > 0.001)
				{
					startMove(rqsPos);
				}
			}
		});
	}
	
	protected HashMap<String,String> pvMap = new HashMap<String,String>();
	protected void regPV(String id, String pvName) { pvMap.put(id,pvName); }
	
	protected void startMove(final double rqsPos)
	{
		new Thread(new Runnable(){public void run()
		{
			Epics.putInteger(PV_PMAC.InPos.motor(devName), 	0);
			Epics.putInteger(PV_PMAC.RunPrg.motor(devName),	1);
			
			Double ActPos = Epics.getDouble(PV_PMAC.ActPos.motor(devName));
			int dir = (rqsPos > ActPos) ? 1 : -1;
			Double speed = dir * Epics.getDouble(PV_PMAC.speed.motor(devName));
			
			double pos = ActPos;
			while (true)
			{
				boolean stop=false;
				if (dir==-1)
				{
					if (pos < rqsPos) { pos = rqsPos; stop=true; }
				}
				else
				{
					if (pos > rqsPos) { pos = rqsPos; stop=true; }
				}

				System.out.println("stop="+stop);
				
				Util.sleep(100);
				Epics.putDouble(PV_PMAC.ActPos.motor(devName),	pos);
				pos += speed/10.0;
				
				if (stop) break;
			}
			
			Epics.putInteger(PV_PMAC.InPos.motor(devName), 	1);
			Epics.putInteger(PV_PMAC.RunPrg.motor(devName),	0);
		}}, "sim PMAC move "+devName).start();
	}
}
