package gov.anl.gmca.log;

import gov.anl.gmca.analysis.helpers.FileListToTemplateStr;
import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.dataproc.DevDataProc;
import gov.anl.gmca.diffimage.MarImageHeader;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.PVLookup.*;
import gov.anl.gmca.log.db.CollectEvents;
import gov.anl.gmca.log.db.CollectEvents.CollectParam;
import gov.anl.gmca.log.db.CollectZeroTabEvents;
import gov.anl.gmca.screening.ui.TabScreening;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Writes pre experimental log with data collection parameters
 */


public class WritePreExperimentalLog implements Runnable {

	public static double beamx = 0.0, beamy = 0.0;
	public String currentRunStr;

	protected String logfile;
	
	protected String gnuplotfile;
	protected String logDir;
	protected String imageDir;

	protected String firstFrame;
	
	protected String frame_name = "";

	public volatile int countLog = 0;
	public volatile boolean stopLog = false;
	
	int noFrames = 0;
	int countInt = 0;
	int countIntNoaudit = 0;

	private MarImageHeader currentHeader;

	protected BufferedWriter out;
	protected ArrayList<Double> Int0 = new ArrayList<Double>();
	protected ArrayList<Double> IntBS = new ArrayList<Double>();
	protected ArrayList<Integer> Frames = new ArrayList<Integer>();

	/* GMCA database object */
	public HashMap<CollectParam, String> fieldMap = new HashMap<CollectParam, String>();

	public WritePreExperimentalLog() {
	
	}

	public void appendLog(int first) {
		
		try {
			out = new BufferedWriter(new FileWriter(logfile, true));
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		try {
			String _I0String = Epics.getString(PV_FRAMEAUDIT.msg);
			int isFrameAudit = Epics.getInteger(PV_FRAMEAUDIT.sts);
			if (isFrameAudit == 1) {
				
				if (noFrames == 0 || TabLog.scrLog) {
					// get image header to read BEAM_X. BEAM_Y
					try {
						writeBeamCenter(out);
					} catch (IOException e1) {
						
						e1.printStackTrace();
					}
					out.write("\n\nINDIVIDUAL_INTENSITIES\n");
					out.append("FrameNo\t\tI0_MAX\t\tI0_SUM\t\tI0_gain\t\tI0_off\t\tI0_bkg\t\tBS_MAX\t\tBS_SUM\t\tBS_gain\t\tBS_off\t\tBS_bkg");
			
				}
				out.append("\n");

				String[] _I0Strings = _I0String.split(" ");
				int frameNo = 1, ind = 0;
				String frame;
				for (int i = 0; i < _I0Strings.length; i++) {

					if (!_I0Strings[i].equals(" ") && !_I0Strings[i].equals("")
							&& _I0Strings[i] != null) {
						String[] _I0Stringseach = _I0Strings[i].split("=");
						frame = _I0Stringseach[1];

						if (_I0Stringseach[0].equals("Frame")) {
							ind = frame.lastIndexOf('.');
							frameNo = Integer.valueOf(frame.substring(ind + 1,
									frame.length()));
							out.append(frameNo + "\t\t");
							Frames.add(frameNo);
						} else {
							out.append(_I0Stringseach[1] + "\t\t");
						}
						if (_I0Stringseach[0].equals("I0_sum")) {
							Int0.add(Double.parseDouble(_I0Stringseach[1]));
							if (i == 0)
								fieldMap.put(CollectParam.IZERO,
										_I0Stringseach[1]);

						} else if (_I0Stringseach[0].equals("BS_sum")) {
							IntBS.add(Double.parseDouble(_I0Stringseach[1]));
							if (i == 0)
								fieldMap.put(CollectParam.ACTIVE_BS,
										_I0Stringseach[1]);
						} else {

						}
					}
				}
				countInt++;
				noFrames = countInt;
			} else {
				
				writeBeamCenter(out);
				out.write("\n\nFrameaudit not running");
				countIntNoaudit++;
				noFrames = countIntNoaudit;
			}
			out.close();

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void writeBeamCenter(BufferedWriter out) throws IOException {
		// get image header to read BEAM_X. BEAM_Y
		String firstFrameForHeader = ModelCollectRunList.get().getCurrentRunModel()
				.seq().getFrameListCopy().get(0).hdfFilename;
		
		if(TabLog.scrLog) {
			firstFrameForHeader = frame_name;
		}
		
		currentHeader = new MarImageHeader(firstFrameForHeader);
		try {
			if(DetectorConfig.isDectris()) {
				/* in case of Pilatus beam center is directly read from PV */
				String beam_xy = Epics.getString(PV_BLUICE.beam_xy);
				StringTokenizer str = new StringTokenizer(beam_xy);
					
				currentHeader.displayOriginX = Double.parseDouble(str.nextToken())* 0.172;
				currentHeader.displayOriginY = Double.parseDouble(str.nextToken())* 0.172;
				currentHeader.detector ="PILATUS3";
				currentHeader.overload = 1930680;
				currentHeader.size1 = (int)(DetectorConfig.getDimensionsMM().x/0.172);
				currentHeader.size2 = (int)((DetectorConfig.getDimensionsMM().y+0.5)/0.172);
				currentHeader.jpegPixelSize = 0.172;
				
			} else {
					currentHeader.getHeader();
					if(currentHeader.displayOriginX == 0.0 || currentHeader.displayOriginX == 0.0) {
						currentHeader.displayOriginX = 150.0;
						currentHeader.displayOriginY = 150.0;
					}
				}
			
		//}
		
		beamx = currentHeader.displayOriginX;
		beamy = currentHeader.displayOriginY;
		
		Formatter fmt = new Formatter();
		fmt.format("\n%-20s\t\t%.2f %s", "BEAM_X", beamx,
				"(BEAM_X, BEAM_Y) reverse for HKL2000");
		fmt.format("\n%-20s\t\t%.2f %s", "BEAM_Y", beamy, "");

		fieldMap.put(CollectEvents.CollectParam.BEAMX, Double.toString(beamx));
		fieldMap.put(CollectEvents.CollectParam.BEAMY, Double.toString(beamy));
	
		out.write(fmt.toString());
		fmt.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw e;
		}
		
	}
	
	public void stopThread() {
		this.stopLog = true;
		this.countLog = 0;
	}
	
	@Override
	public void run() {
		int localCountLog = 0;
		if (TabLog.scrLog) {
			writeLog();
			TabLog.scrLog = false;
			stopThread();
			return;
		}
		
		synchronized (DevDataProc.lock1) {
			/* write Log */

			int isFrameaudit = Epics.getInteger(PV_FRAMEAUDIT.sts);

			while (!stopLog) {
				if (this.countLog < 1) {
					this.countLog++;
					localCountLog++;
					try { writeLog(); } catch (Exception e) { e.printStackTrace(); return; }
					if (currentRunStr.equals("0")) {
						/* write collect parameters to MySQL for run 0 */
						CollectZeroTabEvents.writeEvent(fieldMap);
						stopThread();
						return;

					}
				} else {
					if(isFrameaudit != 1)
						continue;
					if (localCountLog < this.countLog) {

						appendLog(0); /* write intensities */
						localCountLog++;
					}
				}
				if (this.stopLog) {
					stopLog = true;
				}
				Util.sleep(10);
			}
			
			this.countLog = 0;
			/*int NUMIMAGES = ModelCollectRunList.get().getCurrentRunModel()
					.seq().getFrameListCopy().size();*/
			
			int NUMIMAGES = Epics.getInteger(PV_RUN.nextFrame.run(ModelCollectRunList.get().getCurrentRunModel().getRunIndex()));
			/* write collect parameters to MySQL for non-zero run*/
			fieldMap.put(CollectEvents.CollectParam.NUMIMAGES, Integer.toString(NUMIMAGES));
			CollectEvents.writeEvent(fieldMap);

			//clear fieldMap
			fieldMap = new HashMap<CollectParam, String>();

			// QX write datasets.txt file at DevDataDir.getPrimary(), accumulatively
			try {
				FileListToTemplateStr f = new FileListToTemplateStr(Integer.valueOf(currentRunStr));
				f.getTemplateStrAsFile(true);
				Log.note(String.format("saved dataset templates into %s/datasets.txt, num of images: %d", DevDataDir.getPrimary(), NUMIMAGES));
			} catch (Exception e) {
				e.printStackTrace();
				Log.note("failed to save dataset templates into datasets.txt");
			}			
			
			
			
			/* don't write intensities PNG if if no. if images <= 2 */
			if (localCountLog < 3 || isFrameaudit == 1) {
				return;
			}
			
			try {
				/* write intensities in a temporary file */
				BufferedWriter plotInt0 = new BufferedWriter(
						new FileWriter(gnuplotfile, true));
				plotInt0.write("FrameNo\t\tI0_SUM\t\tBS_SUM\n");
				for (int i = 0; i < Int0.size(); i++) {
					plotInt0.append(Frames.get(i) + "\t\t" + Int0.get(i)
							+ "\t\t" + IntBS.get(i) + "\n");
				}
				if (plotInt0 != null)
					plotInt0.close();

				BufferedWriter plotIntBS = new BufferedWriter(
							new FileWriter(gnuplotfile + ".sh", true));
					/* write gnupplot file */
					plotIntBS.append("set title \"" + logfile + "\"\n");
					plotIntBS.append("set xlabel \"FrameNo\"\n");
					plotIntBS.append("set ylabel \"Intensity\"\n");
					plotIntBS.append("set terminal png\n");
					plotIntBS.append("set key right nobox\n");
					plotIntBS.append("set output \"" + gnuplotfile + ".png\""
							+ "\n");
					plotIntBS
							.append("plot \""
									+ gnuplotfile
									+ "\" using 1:2  with linespoints title \"I0 intensities\", \""
									+ gnuplotfile
									+ "\" using 1:3  with linespoints title \"Active Beamstop intensities\"");
					if (plotIntBS != null)
						plotIntBS.close();
					if (out != null)
						out.close();

				} catch (IOException e) {
					System.out
							.println("ERROR: Writing I0, active beamstop intensity plot failed");
				}
				Util.sleep(100);
				System.setProperty("GDFONTPATH", "/usr/share/fonts/liberation");
				System.setProperty("GNUPLOT_DEFAULT_GDFONT",
						"LiberationSans-Regular");

				ExecTask task = ExecTask.new_ExecTask("/usr/bin/gnuplot "
						+ gnuplotfile + ".sh", "", false, false);
				task.start();
				task.waitFor();
				this.countLog = 0;

				/* delete temporary files */

				File f1 = new File(gnuplotfile + ".sh");
				if (f1.exists()) {
					f1.delete();
				}
				File f2 = new File(gnuplotfile);
				if (f2.exists()) {
					f2.delete();
				}
		}
	}

	public void getFrameName() {
		/* Get file name depends on Frameaudit running or not */
		int isFrameAudit = Epics.getInteger(PV_FRAMEAUDIT.sts);

		String[] _I0Strings;
		String[] _I0Stringseach;

		if (isFrameAudit == 1) {
			String _I0String = Epics.getString(PV_FRAMEAUDIT.msg);
			System.out.println("frameaudit : " + _I0String);
			_I0Strings = _I0String.split(" ");
			for (int i = 0; i < _I0Strings.length; i++) {
				// System.out.println ("frameaudit : "+ _I0Strings[i]);
				if (!_I0Strings[i].equals(" ") && !_I0Strings[i].equals("")
						&& _I0Strings[i] != null) {
					_I0Stringseach = _I0Strings[i].split("=");
					if (_I0Stringseach[0].equals("Frame"))
						frame_name = _I0Stringseach[1];
				}
			}
		} else {
			frame_name = Epics.getString(PV_FPE.lastfilename);
		}
	}

	public void writeLog() {

		String run;
		/* Get file name depends on Frameaudit running or not */
		getFrameName();
		String _wedgeSize = "10";
		int noEnergies = 1;
		 int procXDS = 1;
		/* get the general parameters available from PVs */
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
		Date date = new Date();

		String _date = dateFormat.format(date);

		String beamline = DBConfig.get("app_beamlineID");
		String beamlineID = "";
		if (beamline.equals("23i")) {
			beamlineID = "23ID-D";
		}
		if (beamline.equals("23o")) {
			beamlineID = "23ID-B";
		}
		if (beamline.equals("23b")) {
			beamlineID = "23BM";
		}

		String _user = TabLog.userName;
		fieldMap.put(CollectEvents.CollectParam.USER_NAME, _user);
		
		String mounted = Epics.getString(PV_ROBOT.mounted);
		
		String _project = "Project_"+mounted;

		String _sample = mounted;
		String _sampleX = TabLog.sampleXSize;
		String _sampleY = TabLog.sampleYSize;
		String _sampleZ = TabLog.sampleZSize;

		fieldMap.put(CollectEvents.CollectParam.XTAL_X, _sampleX);
		fieldMap.put(CollectEvents.CollectParam.XTAL_Y, _sampleY);
		fieldMap.put(CollectEvents.CollectParam.XTAL_Z, _sampleZ);

		String _apsCurrent = Epics.getString(PV_MISC.apsCurrent);
		double _izero = Epics.getDouble(PV_MISC.localIZero);

		fieldMap.put(CollectEvents.CollectParam.APS_CURRENT, _apsCurrent);
		fieldMap.put(CollectEvents.CollectParam.IZERO,
				Double.toString(Math.round(_izero * 100.0) / 100.0));

		String _energy = String.format("%.3f",
				Epics.getDouble(PV_PMAC.ActPos.motor("mo_En_E")));
		fieldMap.put(CollectEvents.CollectParam.MONO_ENERGY, _energy);

		double _dcm = Epics.getDouble(PV_MISC.dcm);
		fieldMap.put(CollectEvents.CollectParam.DCM,
				Double.toString(Math.round(_dcm * 100.0) / 100.0));

		String _twotheta = Epics.getString(PV_PMAC.ActPos.motor("detector_2t"));
		fieldMap.put(CollectEvents.CollectParam.TWOTHETA, _twotheta);

		int _collimator = Epics.getInteger(PV_COLLIMATOR.ActivePreset);
		String _colliStr = "ScatterGuard";

		if (_collimator == 0) {
			_colliStr = "None";
		} else if (_collimator == 1) {
			_colliStr = "ScatterGuard";
		} else if (_collimator == 2) {
			_colliStr = "20um";
		} else if (_collimator == 3) {
			_colliStr = "10um";
		} else if (_collimator == 4) {
			_colliStr = "5um";
		}
		fieldMap.put(CollectEvents.CollectParam.COLLIMATOR, _colliStr);

		double _guardslity = Epics.getDouble(PV_PMAC.ActPos
				.motor("beam_size_x"));
		double _guardslitz = Epics.getDouble(PV_PMAC.ActPos
				.motor("beam_size_y"));

		fieldMap.put(CollectEvents.CollectParam.SLIT_WIDTH,
				Double.toString(Math.round(_guardslity * 1000.0)));
		fieldMap.put(CollectEvents.CollectParam.SLIT_HEIGHT,
				Double.toString(Math.round(_guardslitz * 1000.0)));

		String _beamstopDistance = Epics.getString(PV_PMAC.ActPos
				.motor("beamstop_z"));

		fieldMap.put(CollectEvents.CollectParam.BEAMSTOP, _beamstopDistance);

		double _gonio_H = Epics.getDouble(PV_PMAC.ActPos.motor("sample_z"));
		double _crystalx = Epics.getDouble(PV_PMAC.ActPos.motor("sample_x"));
		double _crystaly = Epics.getDouble(PV_PMAC.ActPos.motor("sample_y"));

		fieldMap.put(CollectEvents.CollectParam.XTAL_X,
				Double.toString(Math.round(_crystalx * 1000.0) / 1000.0));
		fieldMap.put(CollectEvents.CollectParam.XTAL_Y,
				Double.toString(Math.round(_crystaly * 1000.0) / 1000.0));
		fieldMap.put(CollectEvents.CollectParam.XTAL_Z,
				Double.toString(Math.round(_gonio_H * 1000.0) / 1000.0));

		double _actBeamSizeX = Epics.getDouble(PV_BLUICE.beamsize_h_act1$VAL) / 1000.0;
		double _actBeamSizeY = Epics.getDouble(PV_BLUICE.beamsize_v_act1$VAL) / 1000.0;

		fieldMap.put(CollectEvents.CollectParam.BEAM_SIZE,
				Double.toString(Math.round(_actBeamSizeX * 1000.0)) + "X"
						+ Double.toString(Math.round(_actBeamSizeY * 1000.0)));

		run = currentRunStr;
		if (run == null) {
			Log.error("Current run is not defined, unable to generate log.");
			return;
		}
		int currentRun = Integer.parseInt(run);
		
		if(!currentRunStr.equals("-1"))
			frame_name = ((ModelCollectRun)ModelCollectRunList.get().getRun(currentRun))
				.seq().getFrameListCopy().get(0).cbfFilename;
		
		/* find if the image is from screening */
		if (frame_name.contains(TabScreening.isScr)) {
			currentRun = 0;
			run = "0";
			currentRunStr = "0";
		}

		String logdir;
		double _omegaStart = 0.0;
		double _omegaEnd = 0.0;
		int _inverseBeam = 0;
		double energy1 = 0.0;
		double energy2 = 0.0;
		double energy3 = 0.0;

		String _spindle;
		double stepSize = 0.0;
		int framesPerSite = 0, overlap = 0;
		int reverseDir = 0;

		int numsites = 1;

		PVName pn = PV_RUN.scanMode.run(currentRun);
		int scanMode = Epics.getInteger(pn);
		String cType = Epics.getString(pn);

		double _exposureTime;

		double _frameWidth;
		String _detectorDistance = "0.0";
		String _attenuation = "0.0";
		double _beamsizex = 0.0, _beamsizey = 0.0;
		
		/* get the run parameters available from PVs */
		if (!frame_name.contains(TabScreening.isScr)) {

			String _prefix = Epics.getString(PV_RUN.fileroot.run(currentRun));

			fieldMap.put(CollectEvents.CollectParam.PREFIX, _prefix);
			String _startFrameLabel = Epics.getString(PV_RUN.startFrameLabel
					.run(currentRun));
			
			firstFrame = ((ModelCollectRun)ModelCollectRunList.get().getRun(currentRun))
					.seq().getFrameListCopy().get(0).cbfFilename;

			_spindle = Epics.getString(PV_RUN.axisMotor.run(currentRun));
			_omegaStart = Math.round((Epics.getDouble(PV_RUN.startAngle
					.run(currentRun))) * 100.0) / 100.0;
			_omegaEnd = Math.round(Epics.getDouble(PV_RUN.endAngle
					.run(currentRun)) * 100.0) / 100.0;
			
			_beamsizex = Math.round(Epics.getDouble(PV_RUN.beamWidth_um.run(currentRun)))/1000.0;
			_beamsizey = Math.round(Epics.getDouble(PV_RUN.beamHeight_um.run(currentRun)))/ 1000.0;
			
			fieldMap.put(CollectEvents.CollectParam.OMEGA_START,
					Double.toString(_omegaStart));
			fieldMap.put(CollectEvents.CollectParam.OMEGA_END,
					Double.toString(_omegaEnd));

			// _frameWidth = Epics.getString(PV_RUN.delta.run(currentRun));
			double frameW = Epics.getDouble(PV_RUN.delta.run(currentRun));
			/*
			 * calculate no of frames for XDS file, this only works for normal
			 * data collection, vector and MAD data collection need input from
			 * Collect tab
			 */
			_frameWidth = frameW;

			fieldMap.put(CollectEvents.CollectParam.FRAME_WIDTH,
					Double.toString(frameW));

			_exposureTime = Epics
					.getDouble(PV_RUN.exposureTime.run(currentRun));
			_wedgeSize = Epics.getString(PV_RUN.wedgeSize.run(currentRun), 2);
			_inverseBeam = Epics.getInteger(PV_RUN.inverseOn.run(currentRun));

			energy1 = Epics.getDouble(PV_RUN.energy1.run(currentRun));
			energy2 = Epics.getDouble(PV_RUN.energy2.run(currentRun));
			energy3 = Epics.getDouble(PV_RUN.energy3.run(currentRun));
			
			_detectorDistance = Epics.getString(PV_RUN.distance.run(currentRun), 2);
			fieldMap.put(CollectEvents.CollectParam.DET_DISTANCE, _detectorDistance);
			_attenuation = Epics.getString(PV_BLUICE.attenuation);
			fieldMap.put(CollectEvents.CollectParam.ATTENUATION, _attenuation);
			
			fieldMap.put(CollectEvents.CollectParam.EXP_TIME,
					Double.toString(_exposureTime));
			fieldMap.put(CollectEvents.CollectParam.WEDGE, _wedgeSize);
			fieldMap.put(CollectEvents.CollectParam.INV_BEAM,
					Integer.toString(_inverseBeam));

			fieldMap.put(CollectEvents.CollectParam.ENERGY1,
					Double.toString(Math.round(energy1 * 1000.0) / 1000.0));
			if (energy2 != 0.0 && !Double.isNaN(energy2)) {
				noEnergies++;
				fieldMap.put(CollectEvents.CollectParam.ENERGY2,
						Double.toString(Math.round(energy2 * 1000.0) / 1000.0));
			}
			if (energy3 != 0.0 && !Double.isNaN(energy2)) {
				fieldMap.put(CollectEvents.CollectParam.ENERGY3,
						Double.toString(Math.round(energy1 * 1000.0) / 1000.0));
				noEnergies++;
			}

			PVName dirPV = PV_RUN.directory256.run(currentRun);
			procXDS = Epics.getInteger(PV_RUN.xdsProc.run(currentRun));

			String xdsProc = "NATIVE";

			if (procXDS == 1)
				xdsProc = "NATIVE";
			else if (procXDS == 2)
				xdsProc = "ANOMALOUS";
			else
				xdsProc = "NONE";

			fieldMap.put(CollectEvents.CollectParam.XDSPROC, xdsProc);

			logdir = Epics.getString(dirPV);

			fieldMap.put(CollectEvents.CollectParam.NUMSITES, "1");
			fieldMap.put(CollectEvents.CollectParam.STEP_SIZE, "0.0");


			if (scanMode >= 1 ) {
				/* In vector mode */

				stepSize = Epics.getDouble(PV_RUN.stepSize_um.run(currentRun));

				overlap = Epics.getInteger(PV_RUN.overlap.run(currentRun));

				numsites = ModelCollectRunList.get().getCurrentRunModel()
						.getSelectedSiteCount();
				reverseDir = Epics.getInteger(PV_RUN.reverseDir.run(currentRun));
				if (scanMode == 1)
					fieldMap.put(CollectEvents.CollectParam.STEP_SIZE, Double
							.toString(Math.round(stepSize * 100.0) / 100.0));

				framesPerSite = Epics.getInteger(PV_RUN.framesPerSite.run(currentRun));
				fieldMap.put(CollectEvents.CollectParam.FRAMESPERSITE, 	Double.toString(framesPerSite));

				fieldMap.put(CollectEvents.CollectParam.OVERLAP,
						Integer.toString(overlap));
				fieldMap.put(CollectEvents.CollectParam.NUMSITES,
						Integer.toString(numsites));
			}
		} else {
			String _prefix = Epics.getString(PV_SCR_RUN.fileroot
					.run(currentRun));

			firstFrame = frame_name;

			_spindle = Epics.getString(PV_SCR_RUN.axisMotor.run(currentRun));
			_omegaStart = Math.round((Epics.getDouble(PV_SCR_RUN.startAngle
					.run(currentRun))) * 100.0) / 100.0;
			_omegaEnd = Math.round(Epics.getDouble(PV_SCR_RUN.endAngle
					.run(currentRun)) * 100.0) / 100.0;
			_frameWidth = Epics.getDouble(PV_SCR_RUN.delta.run(currentRun));

			/*
			 * calculate no of frames for XDS file, this only works for normal
			 * data collection, vector and MAD data collection need input from
			 * Collect tab
			 */

			_exposureTime = Epics.getDouble(PV_SCR_RUN.exposureTime
					.run(currentRun));
			_wedgeSize = Epics.getString(PV_SCR_RUN.wedgeSize.run(currentRun));
			_inverseBeam = Epics.getInteger(PV_SCR_RUN.inverseOn
					.run(currentRun));

			PVName dirPV = PV_SCR_RUN.directory256.run(currentRun);
			logdir = Epics.getString(dirPV);
		}

		//collect/ModelCollectRun.java:	public enum ScanMode 	{ SINGLE, VECTOR, RASTER, IMPORT, MESH, JET };
		if (scanMode == 0)
			cType = "Standard";
		else if (scanMode == 1)
			cType = "Vector";
		else if (scanMode == 2)
			cType = "Raster";
		else if (scanMode == 3)
			cType = "Import";
		else if (scanMode == 4)
			cType = "Mesh";
		else if (scanMode == 5)
			cType = "Jet";
		else
			cType = "Unknown";

		fieldMap.put(CollectEvents.CollectParam.COLLECT_TYPE, cType);

		/* Decide log file name depends on the run, for screening use run 0 */
		if(frame_name == null || frame_name.equals("")) {
			System.out.println("ERROR: Frame name empty while writing pre experimental log");
			countLog = 0;
			return;
		}
		String logprefix = frame_name.substring(0, frame_name.lastIndexOf("."));
		if(frame_name.contains(".cbf"))
			logprefix = frame_name.substring(0, frame_name.lastIndexOf("_"));
		logDir = DevDataDir.getPrimary() + "/" + logdir + "/" + logprefix + "_Logs/";
		imageDir = DevDataDir.getPrimary() + "/" + logdir;

		fieldMap.put(CollectEvents.CollectParam.ROOT_DIR, imageDir);

		fieldMap.put(CollectEvents.CollectParam.FIRST_FRAME, frame_name);

		File dirFile = new File(logDir);
		if (!dirFile.exists()) {

			if (!dirFile.mkdirs()) {
				Log.error("Couldn't create directory " + logDir);
			}
		}

		if (!run.equals("0")) {
			logfile = logDir + "/" + frame_name + "_" + _omegaStart + "-"
					+ _omegaEnd + ".txt";
			
			gnuplotfile = logDir + "/" + frame_name + "_" + _omegaStart + "-"
					+ _omegaEnd + ".sh";
		} else {
			logfile = logDir + "/" + frame_name + ".txt";
			gnuplotfile = logDir + "/" + frame_name + ".sh";
		}

		
		// QX write datasets.txt file at DevDataDir.getPrimary(), accumulatively
		/* write @ beginning
		try {
			FileListToTemplateStr f = new FileListToTemplateStr(currentRun);
			f.getTemplateStrAsFile(true);
			Log.note(String.format("saved dataset templates into %s/datasets.txt", DevDataDir.getPrimary()));
		} catch (Exception e) {
			e.printStackTrace();
			Log.note("failed to save dataset templates into datasets.txt");
		}
		*/
		
		/* write log file , use formatting */
		Formatter fmt;
		try {
			out = new BufferedWriter(new FileWriter(logfile, false));

			fmt = new Formatter();
			fmt.format("%-20s\t\t%s", "DATE", _date);
			fmt.format("\n%-20s\t\t%s", "USER_NAME", _user);
			fmt.format("\n%-20s\t\t%s", "PROJECT_NAME", _project);
			fmt.format("\n%-20s\t\t%s\n", "BEAMLINE", beamlineID);
			out.write(fmt.toString());

			fmt = new Formatter();
			fmt.format("\n%-20s\t\t%s", "SAMPLE", _sample);
			fmt.format("\n%-20s\t\t%s", "SAMPLE_SIZE", _sampleX + "X"
					+ _sampleY + "X" + _sampleZ + " (mm)");
			out.write(fmt.toString());
			out.write("\n");

			fmt = new Formatter();
			fmt.format("\n%-20s\t\t%.3f %s", "IZERO", _izero, " V");
			fmt.format("\n%-20s\t\t%.3f %s", "APS_CURRENT",
					Double.parseDouble(_apsCurrent), " V");
			fmt.format("\n%-20s\t\t%.3f %s", "DCM", _dcm, " V");
			fmt.format("\n%-20s\t\t%.2f %s\n", "ATTENUATION",
					Double.parseDouble(_attenuation), " factors");
			fmt.format("\n%-20s\t\t%s %s\n", "MONO_ENERGY", _energy, " KeV");
			out.write(fmt.toString());
			
			/* check the user input from collect tab and decide which collimator 
			 * TODO what happens for 1um in the future?
			 * */
			fmt = new Formatter();
			if(_beamsizex > 0.02) {
				fmt.format("\n%s\t\t\t%s", "Collimator", "Scatterguard");
			} else if (_beamsizex == 0.02) {
				fmt.format("\n%s\t\t\t%s", "Collimator", "20um");
			}else if (_beamsizex == 0.01) {
				fmt.format("\n%s\t\t\t%s", "Collimator", "10um");
			}else if (_beamsizex == 0.005) {
				fmt.format("\n%s\t\t\t%s", "Collimator", "5um");
			} else {
				fmt.format("\n%s\t\t\t%s", "Collimator", "Not known");
			}
			
			/* when minibeam is not used user input from colelct tab is written as 
			 * slit width slit height
			 * TODO for minibeam it is decided in the jbluice runtime settings
			 */
			if(_beamsizex > 0.02) {
				fmt.format("\n%-20s\t\t%.3f %s", "SLIT_WIDTH", _beamsizex, "mm");
				fmt.format("\n%-20s\t\t%.3f %s", "SLIT_HEIGHT", _beamsizey, "mm");
			} else if (_guardslity > 100){
				fmt.format("\n%-20s\t\t%.3f %s", "SLIT_WIDTH", _guardslity, "mm");
				fmt.format("\n%-20s\t\t%.3f %s", "SLIT_HEIGHT", _guardslitz, "mm");
			} else {
				fmt.format("\n%-20s\t\t%.3f %s", "SLIT_WIDTH", 100.0, "mm");
				fmt.format("\n%-20s\t\t%.3f %s", "SLIT_HEIGHT", 100.0, "mm");
			}

			out.write(fmt.toString());
			out.write("\n");

			fmt = new Formatter();
			fmt.format("\n%-20s\t\t%.3f %s", "CRYSTALX", _crystalx, "mm");
			fmt.format("\n%-20s\t\t%.3f %s", "CRYSTALY", _crystaly, "mm");
			fmt.format("\n%-20s\t\t%.3f %s", "CRYSTALZ", _gonio_H, "mm");
			out.write(fmt.toString());
			out.write("\n");

			fmt = new Formatter();
			fmt.format("\n%-20s\t\t%.3f %s", "DETECTOR_DISTANCE",
					Double.parseDouble(_detectorDistance), "mm");
			fmt.format("\n%-20s\t\t%.3f %s", "2THETA",
					Double.valueOf(_twotheta), "deg");
			fmt.format("\n%-20s\t\t%.3f %s", "BEAMSTOP_DISTANCE",
					Double.parseDouble(_beamstopDistance), "mm");
			out.write(fmt.toString());
			out.write("\n");

			fmt = new Formatter();
			fmt.format("\n%-20s\t\t%s", "SPINDLE", _spindle);
			if (frame_name.contains("_scr.")) {
				// int angle =
				// Integer.parseInt(frame_name.substring(frame_name.lastIndexOf('.')+1));
				double angle = Epics.getDouble(PV_SCR_RUN.startAngle.run(0));
				fmt.format("\n%-20s\t\t%.2f %s", "OMEGA_START", angle, "deg");
			} else {
				fmt.format("\n%-20s\t\t%.2f %s", "OMEGA_START",
						Double.valueOf(_omegaStart), "deg");
				fmt.format("\n%-20s\t\t%.2f %s", "OMEGA_END",
						Double.valueOf(_omegaEnd), "deg");
			}

			fmt.format("\n%-20s\t\t%.2f %s", "FRAME_WIDTH", _frameWidth, "deg");
			fmt.format("\n%-20s\t\t%.2f %s", "EXPOSURE_TIME", _exposureTime,
					"sec");

			if (_inverseBeam == 0) {
				
				fmt.format("\n%-20s\t\t%s", "INVERSE_BEAM", "NO");
			} else {
				
				fmt.format("\n%-20s\t\t%s", "INVERSE_BEAM", "YES");
				fmt.format("\n%-20s\t\t%.2f %s", "WEDGE_SIZE",
						Double.parseDouble(_wedgeSize), "deg");
			}

			if (!frame_name.contains(TabScreening.isScr)) {
				fmt.format("\n%-20s\t\t%.6f %s", "ENERGY1", energy1, "KeV");

				if (noEnergies > 1)
					fmt.format("\n%-20s\t\t%.6f %s", "ENERGY2", energy2, "KeV");
				if (noEnergies > 2)
					fmt.format("\n%-20s\t\t%.6f %s", "ENERGY3", energy3, "KeV");
			}

			out.write(fmt.toString());
			out.write("\n");

			fmt = new Formatter();
			fmt.format("\n%-20s\t\t%s", "FIRST_FRAME", frame_name);
			fmt.format("\n%-20s\t\t%s\n", "DIRECTORY", logdir);
			out.write(fmt.toString());

			if (scanMode >= 1 && !frame_name.contains("_scr.")) {
				if (scanMode == 1)
					out.write("\nVECTOR_COLLECTON_PARAMETERS\n");
				if (scanMode == 2)
					out.write("\nRASTER_COLLECTON_PARAMETERS\n");

				fmt = new Formatter();
				if (scanMode == 1)
					fmt.format("\n%-20s\t\t%.2f %s", "STEPSIZE", stepSize, "um");
				fmt.format("\n%-20s\t\t%d %s", "FRAMES_PER_SITE",
						framesPerSite, "frames");
				fmt.format("\n%-20s\t\t%d %s", "NUMSITES", numsites, "sites");
				fmt.format("\n%-20s\t\t%d %s", "OVERLAP", overlap, "frames");
				if (reverseDir == 0)
					fmt.format("\n%-20s\t\t%s %s", "DIRECTION", "Normal", "");
				else
					fmt.format("\n%-20s\t\t%s %s", "DIRECTION", "Reverse", "");
				out.write(fmt.toString());
			}

			out.write("\n\nCOMMENTS\n");

			out.write(WdgLogDataCollection.comments);
			out.write("\n");
			out.close();
			Util.sleep(100);
			int isFrameAudit = Epics.getInteger(PV_FRAMEAUDIT.sts);
			//if(isFrameAudit != 1)
				//appendLog(1); /* write intensities */
			//writeXDSInput();

		} catch (IOException e) {
			Log.error("Failed to write Log file : " + e.getMessage());
		}
	}

}