package gov.anl.gmca.log.db;

import gov.anl.gmca.common.SQLDataType;
import gov.anl.gmca.common.Units;
import gov.anl.gmca.io.Database.DBField;
import gov.anl.gmca.log.db.DevEvents.EventParam;

import java.util.ArrayList;
import java.util.HashMap;

public class DataProcEvents {

	public static final String GMCA_DATAPROC_DATA_TABLE = "DataProcEvents";
	public static enum DataProcParam implements EventParam
	{
							// db string				// units			// datatype						// allowNull	// primary key
		SAMPLE_NUMBER		("sampleNumber",			Units.NONE,			SQLDataType.INT,				false,			false),
		DATAPROC_NUMBER		("dataprocNumber",			Units.NONE,			SQLDataType.AUTOINC,				false,			true),
		COLLECT_NUMBER		("collectNumber",			Units.NONE,			SQLDataType.INT,				false,			false),
		SAMPLE_NAME			("sampleName",				Units.NONE,			SQLDataType.STRING,				false,			false),
		STATE				("state",					Units.NONE,			SQLDataType.STRING,				false,			false),
		SOFTWARE			("software",				Units.NONE,			SQLDataType.STRING,				false,			false),
		SCA_SOFTWARE		("scalingsoftware",			Units.NONE,			SQLDataType.STRING,				false,			false),
		ROOT_DIR			("workdir",					Units.NONE,			SQLDataType.STRING,				false,			false),
		IMAGES				("images",					Units.NONE,			SQLDataType.STRING,				false,			false),
		UNITCELL			("unitcell",				Units.NONE,			SQLDataType.STRING,				false,			false),
		LOWRES				("lowresolution",			Units.NONE,			SQLDataType.STRING,				false,			false),
		LOWRES_INNER		("lowresolution_inner",		Units.NONE,			SQLDataType.STRING,				false,			false),
		LOWRES_OUTER		("lowresolution_outer",		Units.NONE,			SQLDataType.STRING,				false,			false),
		HIGHRES				("highresolution",			Units.NONE,			SQLDataType.STRING,				false,			false),
		HIGHRES_INNER		("highresolution_inner",	Units.NONE,			SQLDataType.STRING,				false,			false),
		HIGHRES_OUTER		("highresolution_outer",	Units.NONE,			SQLDataType.STRING,				false,			false),
		RMERGE				("rmerge",					Units.NONE,			SQLDataType.STRING,				false,			false),
		RMERGE_INNER		("rmerge_inner",			Units.NONE,			SQLDataType.STRING,				false,			false),
		RMERGE_OUTER		("rmerge_outer",			Units.NONE,			SQLDataType.STRING,				false,			false),
		COMPLETE			("completeness",			Units.NONE,			SQLDataType.STRING,				false,			false),
		COMPLETE_INNER		("completeness_inner",		Units.NONE,			SQLDataType.STRING,				false,			false),
		COMPLETE_OUTER		("completeness_outer",		Units.NONE,			SQLDataType.STRING,				false,			false),
		ANOM_COMPLETE		("anom_completeness",		Units.NONE,			SQLDataType.STRING,				false,			false),
		ANOM_COMPLETE_INNER	("anom_completeness_inner",	Units.NONE,			SQLDataType.STRING,				false,			false),
		ANOM_COMPLETE_OUTER	("anom_completeness_outer",	Units.NONE,			SQLDataType.STRING,				false,			false),
		MULTI				("multiplicity",			Units.NONE,			SQLDataType.STRING,				false,			false),
		MULTI_INNER			("multiplicity_inner",		Units.NONE,			SQLDataType.STRING,				false,			false),
		MULTI_OUTER			("multiplicity_outer",		Units.NONE,			SQLDataType.STRING,				false,			false),
		ANOM_MULTI			("anom_multiplicity",		Units.NONE,			SQLDataType.STRING,				false,			false),
		ANOM_MULTI_INNER	("anom_multiplicity_inner",	Units.NONE,			SQLDataType.STRING,				false,			false),
		ANOM_MULTI_OUTER	("anom_multiplicity_outer",	Units.NONE,			SQLDataType.STRING,				false,			false),
		ISIGMA				("isigma",					Units.NONE,			SQLDataType.STRING,				false,			false),
		ISIGMA_INNER		("isigma_inner",			Units.NONE,			SQLDataType.STRING,				false,			false),
		ISIGMA_OUTER		("isigma_outer",			Units.NONE,			SQLDataType.STRING,				false,			false),
		SPA_GRP				("spacegroup",				Units.NONE,			SQLDataType.STRING,				false,			false),
		SCALA_LOG			("scalalog",				Units.NONE,			SQLDataType.STRING,				false,			false),
		TRUNCATE_LOG		("truncatelog",				Units.NONE,			SQLDataType.STRING,				false,			false),
		WARNING				("warning",					Units.NONE,			SQLDataType.STRING,				false,			false),
		TIMESTAMP			("timestamp",				Units.TIMESTAMP,	SQLDataType.TIMESTAMP,			false, 			false);
		
		public String dbString;
		public Units units;
		public SQLDataType dataType;
		public boolean allowNull;
		public boolean isPrimaryKey;
		
		DataProcParam(String dbString, Units units, SQLDataType dataType, boolean allowNull, boolean isPrimaryKey)
		{
			this.dbString=dbString;
			this.units=units;
			this.dataType=dataType;
			this.allowNull=allowNull;
			this.isPrimaryKey=isPrimaryKey;
		}
		
		public DBField getDBField() { return new DBField(dbString, dataType, allowNull, isPrimaryKey, null); }
	};
	
	public static void writeEvent(HashMap<DataProcParam,String> fieldMap)
	{
		// first create a beamline event
		//int beamlineEventNumber = BeamlineEvent.writeEvent();

		// then create the raster event
		ArrayList<HashMap<String,String>> rows = new ArrayList<HashMap<String,String>>(); 
		HashMap<String,String> row = new HashMap<String,String>();
		for (DataProcParam param : DataProcParam.values())
		{
			switch (param)
			{
			case 	SAMPLE_NUMBER:				DevEvents.addInt	(row, param, DevEvents.getSampleNumber());	break;
			case 	DATAPROC_NUMBER:			DevEvents.addInt	(row, param, 0);	break;
			case 	COLLECT_NUMBER:				
			case	SAMPLE_NAME	:
			case	STATE	:
			case	SOFTWARE	:
			case	SCA_SOFTWARE	:
			case	ROOT_DIR	:
			case	IMAGES	:
			case	UNITCELL	:
			case	LOWRES	:
			case	LOWRES_INNER	:
			case	LOWRES_OUTER	:
			case	HIGHRES	:
			case	HIGHRES_INNER	:
			case	HIGHRES_OUTER	:
			case	RMERGE	:
			case	RMERGE_INNER	:
			case	RMERGE_OUTER	:
			case	COMPLETE	:
			case	COMPLETE_INNER	:
			case	COMPLETE_OUTER	:
			case	ANOM_COMPLETE	:
			case	ANOM_COMPLETE_INNER	:
			case	ANOM_COMPLETE_OUTER	:
			case	MULTI	:
			case	MULTI_INNER	:
			case	MULTI_OUTER	:
			case	ANOM_MULTI	:
			case	ANOM_MULTI_INNER	:
			case	ANOM_MULTI_OUTER	:
			case	ISIGMA	:
			case	ISIGMA_INNER	:
			case	ISIGMA_OUTER	:
			case	SPA_GRP	:
			case	SCALA_LOG	:
			case	TRUNCATE_LOG	:
			case	WARNING			:
										DevEvents.addString	(row, param, fieldMap.get(param)); break;
			case TIMESTAMP:				DevEvents.addString	(row, param, SQLDataType.getCurrentTimeString()); break;
			default:					throw new IllegalArgumentException("Unknown DataProcEvents field: "+param);
			}
		}
		rows.add(row);
		DevEvents.addEvent(GMCA_DATAPROC_DATA_TABLE, DataProcParam.values(), rows);
	}
}
