package gov.anl.gmca.log.db;

import gov.anl.gmca.common.SQLDataType;
import gov.anl.gmca.common.Units;
import gov.anl.gmca.hutch.DevAPSStatus;
import gov.anl.gmca.io.Database.DBField;
import gov.anl.gmca.log.db.DevEvents.EventParam;
import gov.anl.gmca.log.db.DevFrameauditMsg.FrameauditParam;
import gov.anl.gmca.screening.ModelScreeningTasks;

import java.util.ArrayList;
import java.util.HashMap;

public class ScreeningCollectEvents {
public static final String GMCA_SCREENING_DATA_TABLE = "ScreeningCollectEvents";

	public static enum ScreeningCollectParam implements EventParam
	{
							// db string			// units			// datatype						// allowNull	// primary key
		SAMPLE_NUMBER		("sampleNumber",		Units.NONE,			SQLDataType.INT,				false,			false),
		SCREENCOLLECT_NUMBER("screenCollectNumber",	Units.NONE,			SQLDataType.AUTOINC,			false,			true),
		IMAGES				("Images",				Units.NONE,			SQLDataType.STRING,				false,			false),
		PREFIX				("prefix",				Units.NONE,			SQLDataType.STRING,				false,			false),
		WORK_DIR			("workdir",				Units.NONE,			SQLDataType.STRING,				false,			false),
		COLLIMATOR			("collimator",			Units.NONE,			SQLDataType.STRING,				false,			false),
		NUMFRAMES			("numframes",			Units.NONE,			SQLDataType.STRING,				false,			false),
		SCORE				("Score",				Units.NONE,			SQLDataType.STRING,				false,			false),
		RESOLUTION			("Resolution",			Units.A,			SQLDataType.STRING,				false,			false),
		PROTEIN				("Protein",				Units.NONE,			SQLDataType.STRING,				false,			false),
		MOSAICITY			("Mosaicity",			Units.DEG,			SQLDataType.STRING,				false,			false),
		UNITCELL			("UnitCell",			Units.NONE,			SQLDataType.STRING,				false,			false),
		BRAVISLATTICE		("BravaisLattice",		Units.NONE,			SQLDataType.STRING,				false,			false),
		RMSD				("Rmsd",				Units.NONE,			SQLDataType.STRING,				false,			false),
		ICERINGS			("IceRings",			Units.NONE,			SQLDataType.STRING,				false,			false),
		WARNING				("warning",				Units.NONE,			SQLDataType.STRING,				false,			false),
		DET_DISTANCE		("detectorDistance",	Units.MM,			SQLDataType.STRING,				false,			false),
		DELTA				("delta",				Units.DEG,			SQLDataType.STRING,				false,			false),
		EXP_TIME			("exposureTime",		Units.SEC,			SQLDataType.STRING,				false,			false),
		BEAMSIZE			("beamsize",			Units.UM,			SQLDataType.STRING,				false,			false),
		IZERO				("izero",				Units.V,			SQLDataType.STRING,				false,			false),
		ACTIVE_BS			("active_bs",			Units.V,			SQLDataType.STRING,				false,			false),
		APS_CURRENT			("apsCurrent",			Units.NONE,			SQLDataType.STRING,				false,			false),
		DCM					("dcm",					Units.V,			SQLDataType.STRING,				false,			false),
		ATTENUATION			("attenuation",			Units.NONE,			SQLDataType.STRING,				false,			false),
		ENERGY				("energy",				Units.KEV,			SQLDataType.STRING,				false,			false),
		BEAMSTOP			("beamstop",			Units.MM,			SQLDataType.STRING,				false,			false),
		TIMESTAMP			("timestamp",			Units.TIMESTAMP,	SQLDataType.TIMESTAMP,			false, 			false);
		
		public String dbString;
		public Units units;
		public SQLDataType dataType;
		public boolean allowNull;
		public boolean isPrimaryKey;
		
		ScreeningCollectParam(String dbString, Units units, SQLDataType dataType, boolean allowNull, boolean isPrimaryKey)
		{
			this.dbString=dbString;
			this.units=units;
			this.dataType=dataType;
			this.allowNull=allowNull;
			this.isPrimaryKey=isPrimaryKey;
		}
		
		public DBField getDBField() { return new DBField(dbString, dataType, allowNull, isPrimaryKey, null); }
	};
	
	public static void writeEvent(HashMap<ScreeningCollectParam,String> fieldMap)
	{
		// first create a beamline event
		//int beamlineEventNumber = BeamlineEvent.writeEvent();

		// then create the raster event
		ArrayList<HashMap<String,String>> rows = new ArrayList<HashMap<String,String>>(); 
		HashMap<String,String> row = new HashMap<String,String>();
		for (ScreeningCollectParam param : ScreeningCollectParam.values())
		{
			switch (param)
			{
			case 	SAMPLE_NUMBER:					DevEvents.addInt	(row, param, DevEvents.getSampleNumber());	break;
			case 	SCREENCOLLECT_NUMBER:			DevEvents.addInt	(row, param, 0);	break;
			case	IMAGES	:
			case	PREFIX	:
			case	WORK_DIR	:
			case	COLLIMATOR	:
			case	NUMFRAMES	:
			case	SCORE	:
			case	RESOLUTION	:
			case	PROTEIN	:
			case	MOSAICITY	:
			case	UNITCELL	:
			case	BRAVISLATTICE	:
			case	RMSD	:
			case	ICERINGS	:
			case	WARNING	:
										DevEvents.addString	(row, param, fieldMap.get(param)); break;
			case	DET_DISTANCE	:   DevEvents.addDouble(row, param, ModelScreeningTasks.detectorDistance); break;
			case	DELTA	:			DevEvents.addDouble(row, param, ModelScreeningTasks.delta); break;
			case	EXP_TIME	:		DevEvents.addDouble(row, param, ModelScreeningTasks.exposureTime); break;
			case	BEAMSIZE	:		DevEvents.addString	(row, param, ModelScreeningTasks.beamsize); break;
			case 	IZERO:				String s_i0 = DevFrameauditMsg.fieldMap.get(FrameauditParam.I0_SUM); DevEvents.addDouble(row, param, Double.parseDouble(s_i0==null?"0":s_i0)); break;
			case 	ACTIVE_BS:			String s_bs = DevFrameauditMsg.fieldMap.get(FrameauditParam.BS_SUM); DevEvents.addDouble(row, param, Double.parseDouble(s_bs==null?"0":s_bs)); break;
			case 	APS_CURRENT:		DevEvents.addDouble	(row, param, DevAPSStatus.getCurrent());		break;
			/**/case DCM:				DevEvents.addDouble	(row, param, 0);		break;
			case	ATTENUATION	:		DevEvents.addDouble(row, param, ModelScreeningTasks.attenuation); break;
			case	ENERGY	:			DevEvents.addDouble(row, param, ModelScreeningTasks.energy); break;
			case	BEAMSTOP	:		DevEvents.addDouble(row, param, ModelScreeningTasks.beamstop); break;

			case TIMESTAMP:				DevEvents.addString	(row, param, SQLDataType.getCurrentTimeString()); break;
			default:					throw new IllegalArgumentException("Unknown screening collect event field: "+param);
			}
		}
		rows.add(row);
		DevEvents.addEvent(GMCA_SCREENING_DATA_TABLE, ScreeningCollectParam.values(), rows);
	}
}
