package gov.anl.gmca.log.db;

import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.SQLDataType;
import gov.anl.gmca.common.Units;
import gov.anl.gmca.io.Database.DBField;
import gov.anl.gmca.log.db.DevEvents.EventParam;
import gov.anl.gmca.screening.ModelScreening;

import java.util.ArrayList;
import java.util.HashMap;

public class ScreeningEvents {
	
public static final String GMCA_SCREENING_DATA_TABLE = "ScreeningEvents";
	
	public static enum ScreeningParam implements EventParam
	{
							// db string		// units			// datatype						// allowNull	// primary key
		SAMPLE_NUMBER		("sampleNumber",	Units.NONE,			SQLDataType.AUTOINC,			false,			true),
		USER_NAME			("userName",		Units.NONE,			SQLDataType.STRING,				false,			false),
		HOST_NAME			("hostName",		Units.NONE,			SQLDataType.STRING,				false,			false),
		PUCK_TYPE			("puckType",		Units.NONE,			SQLDataType.STRING,				false,			false),
		ROBOT_SUCCESS		("robotSuccess",	Units.NONE,			SQLDataType.BOOL,				false,			false),
		ROBOT_ERROR			("robotError",		Units.NONE,			SQLDataType.STRING,				false,			false),
		CENTER_TYPE			("centerType",		Units.NONE,			SQLDataType.STRING,				false,			false),
		WASH_TYPE			("washType",		Units.NONE,			SQLDataType.STRING,				false,			false),
		ROOT_DIR			("rootDir",			Units.NONE,			SQLDataType.STRING,				false,			false),
		PORT				("Port",			Units.NONE,			SQLDataType.STRING,				false,			false),
		DIR					("Directory",		Units.NONE,			SQLDataType.STRING,				false,			false),
		COMMENT				("comment",			Units.NONE,			SQLDataType.STRING,				false,			false),
		TIMESTAMP			("timestamp",		Units.TIMESTAMP,	SQLDataType.TIMESTAMP,			false, 			false);
		
		public String dbString;
		public Units units;
		public SQLDataType dataType;
		public boolean allowNull;
		public boolean isPrimaryKey;
		
		ScreeningParam(String dbString, Units units, SQLDataType dataType, boolean allowNull, boolean isPrimaryKey)
		{
			this.dbString=dbString;
			this.units=units;
			this.dataType=dataType;
			this.allowNull=allowNull;
			this.isPrimaryKey=isPrimaryKey;
		}
		
		public DBField getDBField() { return new DBField(dbString, dataType, allowNull, isPrimaryKey, null); }
	};
	
	public static void writeEvent(ModelScreening scr)
	{
		// first create a beamline event
		//int beamlineEventNumber = BeamlineEvent.writeEvent();

		// then create the raster event
		ArrayList<HashMap<String,String>> rows = new ArrayList<HashMap<String,String>>(); 
		HashMap<String,String> row = new HashMap<String,String>();
		for (ScreeningParam param : ScreeningParam.values())
		{
			switch (param)
			{
			case SAMPLE_NUMBER:			DevEvents.addInt	(row, param, DevEvents.getSampleNumber()+1);	break;
			case USER_NAME:				DevEvents.addString	(row, param, System.getProperty("user.name"));	break;
			case HOST_NAME:				DevEvents.addString	(row, param, "");	break;
			case PUCK_TYPE:				DevEvents.addString	(row, param, ModelScreening.getPuckType());	break;
			case ROBOT_SUCCESS:			DevEvents.addBool	(row, param, ModelScreening.robotSuccess);	break;
			case ROBOT_ERROR:			DevEvents.addString	(row, param, ModelScreening.robotError);	break;
			case CENTER_TYPE:			DevEvents.addString	(row, param, ModelScreening.centerType);	break;
			case WASH_TYPE:				DevEvents.addString	(row, param, ModelScreening.washType);	break;
			case ROOT_DIR:				DevEvents.addString	(row, param, DevDataDir.getPrimary()+"/"+ModelScreening.currentRootDir);	break;
			case PORT:					DevEvents.addString	(row, param, ModelScreening.currentPort);	break;
			case DIR:					DevEvents.addString	(row, param, ModelScreening.currentSubDir);	break;
			case COMMENT:				DevEvents.addString	(row, param, "");	break;
			case TIMESTAMP:				DevEvents.addString	(row, param, SQLDataType.getCurrentTimeString()); break;
			default:					throw new IllegalArgumentException("Unknown screening event field: "+param);
			}
		}
		rows.add(row);
		DevEvents.setSampleNumber(DevEvents.getSampleNumber()+1);
		DevEvents.addEvent(GMCA_SCREENING_DATA_TABLE, ScreeningParam.values(), rows);
	}
}
