/**
 * handles mysql queries for MultiStrategyResults table
 */

package gov.anl.gmca.multistrategy;

import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_STRATEGY;
import gov.anl.gmca.multistrategy.ModelMultiStrategy.MultiStrategyField;
import gov.anl.gmca.syslog.Log;
import gov.aps.jca.dbr.DBR;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

public class DevMultiStrategy {
	protected static final String tableName = "MultiStrategyResults";
	protected static final boolean UPDATE_CURRENT_INDEX_ON_RESULT_CREATION = true;
	protected static final boolean SHOW_MultiStrategy_TAB_ON_RESULT_CREATION = true;

	protected static boolean doesTableExist() {
		Database.Result result = Database.exec(DBName.USERDATA,
				"show tables like '" + tableName + "'");
		boolean exists = (result.numRows() > 0);
		return exists;
	}

	public synchronized static boolean verifyTableExists() {
		if (!doesTableExist()) {
			System.out.println("Creating MultiStrategy results table");
			String createCmd = "CREATE TABLE `" + tableName + "` ("
					+ "`sampleNumber` int(11) NOT NULL,"
					+ "`state` varchar(30) default '',"
					+ "`workdir` varchar(250) default '',"
					+ "`referencedata` varchar(512) default '',"
					+ "`images` varchar(512) default '',"
					+ "`unitcell` varchar(512) default '',"
					+ "`lowresolution` varchar(30) default '',"
					+ "`highresolution` varchar(30) default '',"
					+ "`spacegroup` varchar(512) default '',"
					+ "`displaytext` varchar(250) default '',"
					+ "`strategy` varchar(2048) default '',"
					+ "`scalalog` varchar(512) default '',"
					+ "`xplanlog` varchar(512) default '',"
					+ "`warning` varchar(5120) default '',"
					+ "`timestamp` varchar(512) default '',";

			createCmd += " PRIMARY KEY (`sampleNumber`)) DEFAULT CHARSET=utf8";
			Log.note(createCmd);
			Database.exec(DBName.USERDATA, createCmd);
		}

		return doesTableExist();
	}

	protected boolean connected = false;
	protected Epics.EpicsValueMonitor dbUpdatedMonitor;

	public static DevMultiStrategy get() {
		return inst;
	}

	protected static DevMultiStrategy inst = null;

	public DevMultiStrategy() {
		inst = this;
	}

	public void verifyConnected() {
		if (connected)
			return;

		updateAllRows();

		Epics.addMonitor(PV_STRATEGY.currentMultiStrategyIndex,
				new Epics.EpicsValueMonitor() {
					@Override
					public void valueChanged(String pvName, DBR value) {
						ModelMultiStrategyList.get()
								.setCurrentMultiStrategyIndex(
										Epics.dbrToInt(value));
						ModelMultiStrategyList.get().ll.notifyListeners();
					}
				});

		dbUpdatedMonitor = new Epics.EpicsValueMonitor() {
			@Override
			public void valueChanged(String pvName, DBR value) {
				if (pvName.equals(PV_STRATEGY.multistrategyUpdated.toString())) {
					int intVal = Epics.dbrToInt(value);
					
					if (-1 != intVal) {
						handlePVUpdate(intVal);
					}
				}
				if (pvName.equals(PV_STRATEGY.showMultiStrategy.toString())) {
					ModelMultiStrategyList.get().setMultiStrategy(
							Epics.dbrToInt(value));
					ModelMultiStrategyList.get().ll.notifyListeners();
				}
			}
		};
		Epics.addMonitor(PV_STRATEGY.multistrategyUpdated, dbUpdatedMonitor);
		Epics.addMonitor(PV_STRATEGY.showMultiStrategy, dbUpdatedMonitor);

		connected = true;
	}

	public void setResult(int sampleNumber,
			ModelMultiStrategyParams MultiStrategyResult, boolean update) {
		verifyTableExists();
		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber);
		if (0 == result.numRows()) {
			Database.exec(
					DBName.USERDATA,
					"insert into "
							+ tableName
							+ " (sampleNumber, state, workdir, images, referencedata, warning) "
							+ "values (" + sampleNumber + ", '"
							+ MultiStrategyResult.state + "', '"
							+ MultiStrategyResult.workDir + "', '"
							+ MultiStrategyResult.images + "', '"
							+ MultiStrategyResult.referencedata + "', '"
							+ MultiStrategyResult.warning + "')");
			newResultCreated = true;
		} else {
			Database.exec(DBName.USERDATA, "update " + tableName + " "
					+ "set state='" + MultiStrategyResult.state + "' "
					+ ", workdir='" + MultiStrategyResult.workDir + "' "
					+ ", images='" + MultiStrategyResult.images + "' "
					+ ", referencedata='" + MultiStrategyResult.referencedata
					+ "' " + ", warning='" + MultiStrategyResult.warning + "' "
					+ "where sampleNumber=" + sampleNumber);
		}
		if (update) {

			// update the GUI directly
			updateRow(sampleNumber);
			if (newResultCreated && UPDATE_CURRENT_INDEX_ON_RESULT_CREATION) {
				int last = ModelMultiStrategyList.get().getMultiStrategyList()
						.size() - 1;
				DevMultiStrategy.setCurrentMultiStrategyIndex(last);
			}
			// send the PV update signal
			sendRowUpdate(sampleNumber);
		}
	}

	public void setMultiStrategyResult(int sampleNumber,
			ModelMultiStrategyParams MultiStrategyResult, boolean update) {
		verifyTableExists();

		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber);
		if (0 == result.numRows()) {
			Database.exec(DBName.USERDATA, "insert into `" + tableName
					+ "` (sampleNumber,state) " + "values (" + sampleNumber
					+ ",'DONE')");
			newResultCreated = true;
		}

		String updateCmd = "update `" + tableName + "` set ";
		String DATE_FORMAT_NOW = "yyyy-MM-dd-HH:mm:ss";
		Date date = new Date();
		SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
		String timestamp = sdf.format(date);

		updateCmd += "state='" + MultiStrategyResult.state;
		updateCmd += "', warning=\"" + MultiStrategyResult.warning;
		updateCmd += "\", lowresolution='" + MultiStrategyResult.lowresolution;
		updateCmd += "', highresolution='" + MultiStrategyResult.highresolution;
		updateCmd += "', unitcell='" + MultiStrategyResult.unitCell;
		updateCmd += "', spacegroup='" + MultiStrategyResult.spaceGroup;
		updateCmd += "', displaytext='" + MultiStrategyResult.displaytext;
		updateCmd += "', strategy='" + MultiStrategyResult.strategy;
		updateCmd += "', timestamp='" + timestamp;
		updateCmd += "' where sampleNumber=" + sampleNumber;

		Database.exec(DBName.USERDATA, updateCmd);

		if (update) {
			// send the PV update signal
			// updateRow(sampleNumber);
			if (newResultCreated && UPDATE_CURRENT_INDEX_ON_RESULT_CREATION) {
				// int last =
				// ModelMultiStrategyList.get().getMultiStrategyList().size()-1;
				// DevMultiStrategy.setCurrentMultiStrategyIndex(last);
			}
			// send the PV update signal
			sendRowUpdate(sampleNumber);
		}
	}

	protected void updateRow(int row) {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + row);
		ModelMultiStrategy MultiStrategyResult;
		if(result.numRows() < 1)
			return;
		MultiStrategyResult = sqlRowToResult(result.getRow(0));
		ModelMultiStrategyList.get().setRow(MultiStrategyResult, row, true);
		ModelMultiStrategyList.get().ll.notifyListeners(row);
	}

	protected void updateAllRows() {
		verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);
		System.out.println("Number of rows " + result.numRows());
		ModelMultiStrategyList.get().clearResults();
		ModelMultiStrategy MultiStrategyResult;

		for (HashMap<String, String> row : result.getRows()) {
			MultiStrategyResult = sqlRowToResult(row);
			if (null != MultiStrategyResult)
				ModelMultiStrategyList.get().setRow(MultiStrategyResult,
						Integer.parseInt(row.get("sampleNumber")), false);
		}
		ModelMultiStrategyList.get().ll.notifyListeners(-2);
	}

	protected ModelMultiStrategy sqlRowToResult(HashMap<String, String> row) {
		HashMap<MultiStrategyField, String> fieldMap = new HashMap<MultiStrategyField, String>();
		for (MultiStrategyField field : MultiStrategyField.values()) {
			boolean isStrat = true;
			for (String sqlCol : row.keySet()) {
				if (field.getDBString().equals(sqlCol)) {
					fieldMap.put(field, row.get(sqlCol));
					// Log.note(sqlCol + " "+ row.get(sqlCol));
					isStrat = false;
				}
			}
			if (isStrat)
				fieldMap.put(field, "");
		}
		return new ModelMultiStrategy(fieldMap);
	}

	public int getRowCount() {
		int count;
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);
		count = result.numRows();
		return count;
	}

	protected void handlePVUpdate(int pvValue) {
		verifyTableExists();
		if (pvValue >= 0) {
			System.out.println("Updating row " + pvValue);
			updateRow(pvValue);
		} else if (-2 == pvValue) {
			System.out.println("Updating all rows " + pvValue);
			updateAllRows();
		}
	}

	protected void sendRowUpdate(int sampleNumber) {
		Epics.putInteger(PV_STRATEGY.multistrategyUpdated, -1);
		Epics.putInteger(PV_STRATEGY.multistrategyUpdated, sampleNumber);
	}

	public static int getRows() {
		int count;
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);
		count = result.numRows();
		return count;
	}

	public static void setCurrentMultiStrategyIndex(int i) {
		Epics.putInteger(PV_STRATEGY.currentMultiStrategyIndex, i);
	}

	public void deleteAllResults() {
		Log.note("Deleting all strategy results");

		verifyTableExists();

		setCurrentMultiStrategyIndex(0);

		Database.exec(DBName.USERDATA, "delete from " + tableName);
		sendRowUpdate(-2);
	}

	public void saveAllResults(String excelFile) {
		// WritableWorkbook workbook = Workbook.createWorkbook(new
		// File(excelFile));
		Workbook wbs;
		ModelMultiStrategyList resultList = ModelMultiStrategyList.get();
		try {
			OutputStream inp = new FileOutputStream(excelFile);

			wbs = new HSSFWorkbook();

			Sheet s = wbs.createSheet();
			// declare a row object reference
			Row r = null;
			// declare a cell object reference
			Cell c = null;
			int i = 1;
			r = s.createRow(0); // write the column names
			for (MultiStrategyField field : MultiStrategyField.values()) {

				c = r.createCell(field.ordinal());
				c.setCellValue(field.getDBString());
			}
			// write data
			for (ModelMultiStrategy cell : new ArrayList<ModelMultiStrategy>(
					resultList.getMultiStrategyList())) {

				Row r1 = s.createRow(i);
				for (MultiStrategyField field : cell.getAllFields().keySet()) {
					String value = cell.getField(field);
					c = r1.createCell(field.ordinal());
					if (value != null)
						c.setCellValue(value);
					else
						c.setCellValue("");
				}
				i++;
			}
			wbs.write(inp);
			inp.close();
		} catch (IOException ex) {
			Log.error("Excel export failed :" + ex.getMessage());
		}
	}

	/*
	 * Converting spacegroup number to space group. The mapping is in the
	 * follwoing file CLIBD environment variable
	 * /mnt/software/px/CCP4/ccp4-6.3.0/lib/data/symop.lib
	 */
	public String getSpacegroupFromNumber(int spgNo) {

		String spacegroup = "";
		// TODO: sim mode file replacement should be done in gov.anl.gmca.io.Disk
		String spgFile = (Main.SIM_LEVEL>0 ? Main.getAppDir()+"/data" : System.getenv("CLIBD"))+"/symop.lib";

		List<String> lines = null;
		try {
			lines = Files.readAllLines(Paths.get(spgFile),
					Charset.defaultCharset());
		} catch (IOException e) {
			e.printStackTrace();
		}
		for (String line : lines) {
			if (line.contains("'") && line != null) {

				String num = line.split(" ")[0];
				if (num.equals(Integer.toString(spgNo))) {
					spacegroup = line.split(" ")[3];
				}
			}
		}
		return spacegroup;
	}

	/*
	 * Converting spacegroup number to space group. The mapping is in the
	 * follwoing file CLIBD environment variable
	 * /mnt/software/px/CCP4/ccp4-6.3.0/lib/data/symop.lib
	 */
	public int getNumberFromSpacegroup(String spg) {

		String spacegroup = "";
		spg = spg.toUpperCase();
		// TODO: sim mode file replacement should be done in gov.anl.gmca.io.Disk
		String spgFile = (Main.SIM_LEVEL>0 ? Main.getAppDir()+"/data" : System.getenv("CLIBD"))+"/symop.lib";
		
		List<String> lines = null;
		try {
			lines = Files.readAllLines(Paths.get(spgFile),
					Charset.defaultCharset());
		} catch (IOException e) {
			e.printStackTrace();
		}
		for (String line : lines) {
			if (spg.contains(" ")) {
				if (line.contains(spg)) {
					spacegroup = line.split(" ")[0];
					break;
				}
			} else {
				if (line.contains("'") && line != null) {

					String num = line.split(" ")[3];
					if (num.equals(spg)) {
						spacegroup = line.split(" ")[0];
						break;
					}
				}
			}
		}
		if (spacegroup.equals(""))
			return 0;
		else
			return Integer.parseInt(spacegroup);
	}

	public String getWorkingDir(int sampNumber) {
		final Database.Result result3 = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber="
						+ sampNumber);
		String workDir = "";
		for (HashMap<String, String> row2 : result3.getRows()) {
			for (String sqlCol : row2.keySet()) {
				if (sqlCol.equals("workdir")) {
					workDir = row2.get(sqlCol);
				}
			}
		}

		return workDir;
	}
}
