/**
 * All results table of multi crystal strategy
 * 
 */

package gov.anl.gmca.multistrategy;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.multistrategy.ModelMultiStrategy.MultiStrategyField;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import java.util.ArrayList;

public class WdgMultiStrategyTable extends Composite implements
		ListenerList.Listener<ModelMultiStrategyList>, PaintListener {
	protected ArrayList<TableColumn> columnList = new ArrayList<TableColumn>();
	protected Table table = null;

	public WdgMultiStrategyTable(Composite parent, int style) {
		super(parent, style);

		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = 0;
		gl.marginHeight = 0;
		gl.horizontalSpacing = 0;
		gl.verticalSpacing = 3;
		setLayout(gl);

		createTable();
		updateTable();
	}

	protected void createTable() {
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1);
		gd.widthHint = 500;
		gd.heightHint = 500;
		table = new Table(this, SWT.NONE);
		table.setFont(GUI.FONT_DEFAULT);
		table.setLinesVisible(true);
		table.setLayoutData(gd);

		columnList.clear();
		for (MultiStrategyField field : MultiStrategyField.values()) {
			if (field == MultiStrategyField.SAMPLE_NUMBER)
				continue;
			TableColumn tc = new TableColumn(table, SWT.CENTER);
			tc.setText(field.dbString);
			tc.setWidth(130);
			columnList.add(tc);
		}
		table.setHeaderVisible(true);
		table.addMouseListener(new MouseListener() {
			public void mouseDoubleClick(MouseEvent arg0) {
				if (!Client.isMaster())
					return;
				int index = table.getSelectionIndex();
				DevMultiStrategy.setCurrentMultiStrategyIndex(index);
			}

			@Override
			public void mouseDown(MouseEvent e) {
			}

			@Override
			public void mouseUp(MouseEvent e) {
			}
		});
		addPaintListener(this);
	}

	public void updateTable() {
		GUI.asyncExec(new Runnable() {
			public void run() {
				ModelMultiStrategyList resultList = ModelMultiStrategyList
						.get();

				table.removeAll();
				for (ModelMultiStrategy cell : new ArrayList<ModelMultiStrategy>(
						resultList.getMultiStrategyList())) {

					TableItem ti = new TableItem(table, SWT.NONE);

					for (MultiStrategyField field : cell.getAllFields()
							.keySet()) {
						if (field == MultiStrategyField.SAMPLE_NUMBER)
							continue;
						if (field == MultiStrategyField.WARNING
								|| field == MultiStrategyField.DISPLAYTEXT) {
							String value = cell.getField(field);
							if (value.length() > 20) {
								value = value.substring(0, 20);
								value = value.concat("...");
							}
							if (value != null)
								ti.setText(field.ordinal() - 1, value);
							else
								ti.setText(field.ordinal() - 1, "");
							continue;
						}
						String value = cell.getField(field);
						if (value != null)
							ti.setText(field.ordinal() - 1, value);
						else
							ti.setText(field.ordinal() - 1, "");
					}
				}
			}
		});
	}

	public void connectListeners() {
		ModelMultiStrategyList.get().ll.addListener(this);
		updateTable();
	}

	@Override
	public void handleEvent(ModelMultiStrategyList object, Object data) {
		// TODO Auto-generated method stub
		updateTable();
	}

	@Override
	public void paintControl(PaintEvent e) {
		// TODO Auto-generated method stub

	}
}
