/**
 * GUI for ReferenceData sub-tab of multi crystal tab
 */

package gov.anl.gmca.multistrategy;

import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.ui.NBFileDialog;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.io.File;
import java.util.StringTokenizer;

public class WdgReferenceData extends Composite implements
		ListenerList.Listener<ModelReferenceData>, PaintListener,
		ClientListener {

	protected Label dirLabel, dirLabelVal;
	protected Table table = null;
	protected Composite expandComp, crstalComposite, fileComp;
	protected Group crystalGroup;
	protected Text spaceGroupText, lowResLimitEntry, highResLimitEntry;
	protected Text xdsfileText, minRangeText;
	protected Button multiCrystButton, xdsfileBrowse, runXDSButton,
			btnDefaultRes, addButton, removeButton;
	protected Text[] unitCellText;
	protected List fileList;

	public WdgReferenceData(Composite parent, int style) {
		
		super(parent, style);
		ModelReferenceData.get().ll.addListener(this);
		DevReferenceData.get().verifyConnected();
		Client.addListener(this);

		GridLayout gl = new GridLayout(7, false);
		gl.marginWidth = 0;
		gl.marginHeight = 0;
		gl.horizontalSpacing = 0;
		gl.verticalSpacing = 3;
		setLayout(gl);

		expandComp = new Composite(this, SWT.NONE);
		expandComp.setBackground(GUI.BGCOLOR);
		GridLayout gl1 = new GridLayout(7, false);
		expandComp.setLayout(gl1);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		expandComp.setLayoutData(gd);

		createReferenceData();
	}

	protected void createReferenceData() {
		GridData gd;

		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 3;

		multiCrystButton = new Button(expandComp, SWT.CHECK | SWT.BORDER);
		multiCrystButton.setText("Partial data available?");
		multiCrystButton.setFont(GUI.FONT_COLLECT_GROUP);
		multiCrystButton.setLayoutData(gd);
		multiCrystButton.setBackground(GUI.BGCOLOR);
		multiCrystButton.setToolTipText("check this button if you want to use partial data for strategy\nuncheck if not");
		
		multiCrystButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						int isReference = multiCrystButton.getSelection() ? 1
								: 0;
						ModelReferenceData.ISREFERENCE = isReference;
						DevReferenceData.get().setResult(false);
						updateButtons();
					}
				});
			}
		});
		
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint = 20;
		gd.horizontalSpan = 5;

		xdsfileText = new Text(expandComp, SWT.BORDER);
		xdsfileText.setLayoutData(gd);
		xdsfileText.setToolTipText(xdsfileText.getText());

		if (Main.SIM_LEVEL >= 1) {
			xdsfileText
					.setText("/mnt/staffhome/spothineni/testxds/secondXtalStrategy/XDS_ASCII.HKL");
		}
		/* Browse for XDS_ASCII.HKL */
		gd = new GridData();
		gd.heightHint = 30;
		gd.horizontalSpan = 2;

		xdsfileBrowse = new Button(expandComp, SWT.PUSH);
		xdsfileBrowse.setText("Browse");
		xdsfileBrowse.setLayoutData(gd);
		xdsfileBrowse.setToolTipText("Select XDS_ASCII.HKL of XDS or .sca of HKL2000");
		
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 7;
		
		Label labelData = new Label(expandComp, SWT.NONE);
		labelData.setBackground(GUI.BGCOLOR);
		labelData.setFont(GUI.FONT_8_FIXED);
		labelData.setText("(load XDS_ASCII.HKL of XDS or .sca of HKL2000)");
		labelData.setLayoutData(gd);
		
		crystalGroup = new Group(expandComp, SWT.NONE);
		crystalGroup.setBackground(GUI.BGCOLOR);
		GridLayout actionGrid = new GridLayout(13, false);
		crystalGroup.setLayout(actionGrid);

		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 7;

		crystalGroup.setLayoutData(gd);
		crystalGroup.setFont(GUI.FONT_COLLECT_GROUP);
		crystalGroup.setText("Crystal Parameters (optional)");

		Label lblUnitcell = new Label(crystalGroup, SWT.NONE);
		lblUnitcell.setText("Unit cell : ");
		lblUnitcell.setBackground(GUI.BGCOLOR);
		gd = new GridData();
		gd.horizontalSpan = 4;
		lblUnitcell.setLayoutData(gd);

		unitCellText = new Text[ModelReferenceData.UnitCell.length];
		for (int i = 0; i < ModelReferenceData.UnitCell.length; i++) {

			gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
			gd.horizontalAlignment = SWT.RIGHT;
			Label labelCell = new Label(crystalGroup, SWT.NONE);
			labelCell.setBackground(GUI.BGCOLOR);
			labelCell.setText(ModelReferenceData.UnitCell[i]);
			labelCell.setLayoutData(gd);

			gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
			gd.horizontalSpan = 2;

			unitCellText[i] = new Text(crystalGroup, SWT.BORDER);
			unitCellText[i].setLayoutData(gd);

			unitCellText[i].addListener(SWT.Verify, new Listener() {
				public void handleEvent(Event e) {
					Util.allowOnlyAlpha(e);
				}
			});

			unitCellText[i].addFocusListener(new FocusListener() {
				@Override
				public void focusGained(FocusEvent arg0) {
				}

				public void focusLost(FocusEvent arg0) {
					setUnitCell();
				}
			});
			unitCellText[i].addKeyListener(new KeyListener() {
				@Override
				public void keyPressed(KeyEvent arg0) {
				}

				public void keyReleased(KeyEvent arg0) {
					if (arg0.keyCode == SWT.CR) {
						setUnitCell();
					}
				}
			});
			if (i == 2) {
				Label lblBlanck = new Label(crystalGroup, SWT.NONE);
				lblBlanck.setText("          ");
				lblBlanck.setBackground(GUI.BGCOLOR);
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.horizontalSpan = 4;
				lblBlanck.setLayoutData(gd);
			}
		}

		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 4;
		Label labelSpaceGroup = new Label(crystalGroup, SWT.NONE);
		labelSpaceGroup.setBackground(GUI.BGCOLOR);
		labelSpaceGroup.setText("Space Group Number : ");
		labelSpaceGroup.setLayoutData(gd);

		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.widthHint = 70;
		gd.horizontalSpan = 4;

		spaceGroupText = new Text(crystalGroup, SWT.BORDER);
		spaceGroupText.setLayoutData(gd);

		/*
		 * spaceGroupText.addListener (SWT.Verify, new Listener () { public void
		 * handleEvent (Event e) { Util.allowOnlyAlpha(e); } });
		 */
		spaceGroupText.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent arg0) {
			}

			public void focusLost(FocusEvent arg0) {
				setSpaceGroup();
			}
		});
		spaceGroupText.addKeyListener(new KeyListener() {
			@Override
			public void keyPressed(KeyEvent arg0) {
			}

			public void keyReleased(KeyEvent arg0) {
				if (arg0.keyCode == SWT.CR) {
					setSpaceGroup();
				}
			}
		});

		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, true);
		gd.horizontalSpan = 5;
		gd.horizontalAlignment = SWT.CENTER;
		Link link = new Link(crystalGroup, SWT.NONE);
		link.setBackground(crystalGroup.getBackground());
		link.setText("<a>Show space group numbers</a>");
		link.setLayoutData(gd);

		link.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			@Override
			public void widgetSelected(SelectionEvent arg0) {
				String filename = Main.getAppDir()+"/docs/spacegroupnumbers.htm";
				ExecTask.new_ExecTask("firefox " + filename + "", "firefox",
						false, false).start();
			}
		});

		fileComp = new Composite(expandComp, SWT.NONE);
		fileComp.setBackground(GUI.BGCOLOR);
		GridLayout gl1 = new GridLayout(7, false);
		fileComp.setLayout(gl1);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 7;
		gd.verticalSpan = 5;
		fileComp.setLayoutData(gd);

		gd = new GridData(SWT.FILL, SWT.CENTER, false, false);
		Label labelMinRange = new Label(fileComp, SWT.NONE);
		labelMinRange.setBackground(GUI.BGCOLOR);
		labelMinRange.setText("Min. rotation range : ");
		labelMinRange.setLayoutData(gd);

		gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
		gd.horizontalSpan = 2;
		gd.widthHint = 70;

		minRangeText = new Text(fileComp, SWT.BORDER);
		minRangeText.setLayoutData(gd);

		minRangeText.addListener(SWT.Verify, new Listener() {
			public void handleEvent(Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		minRangeText.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent arg0) {
			}

			public void focusLost(FocusEvent arg0) {
				setMinRange();
			}
		});
		minRangeText.addKeyListener(new KeyListener() {
			@Override
			public void keyPressed(KeyEvent arg0) {
			}

			public void keyReleased(KeyEvent arg0) {
				if (arg0.keyCode == SWT.CR) {
					setMinRange();
				}
			}
		});
		
		/*gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 7;
		
		Label minrangeData = new Label(expandComp, SWT.NONE);
		minrangeData.setBackground(GUI.BGCOLOR);
		minrangeData.setFont(GUI.FONT_8_FIXED);
		minrangeData.setText("(sets XDS parameter TOTAL_SPINDLE_ROTATION_RANGES)");
		minrangeData.setLayoutData(gd);*/
		
		Group imageGroup = new Group(expandComp, SWT.NONE);
		imageGroup.setBackground(GUI.BGCOLOR);
		GridLayout aGrid = new GridLayout(7, false);
		imageGroup.setLayout(aGrid);

		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 7;

		imageGroup.setLayoutData(gd);
		imageGroup.setFont(GUI.FONT_COLLECT_GROUP);
		imageGroup.setText("Test images of new crystal :");

		gd = new GridData(SWT.LEFT, SWT.CENTER, false, false);
		dirLabel = new Label(imageGroup, SWT.NONE);
		dirLabel.setBackground(GUI.BGCOLOR);
		dirLabel.setText("Directory : ");
		dirLabel.setLayoutData(gd);

		gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		gd.horizontalSpan = 6;
		dirLabelVal = new Label(imageGroup, SWT.WRAP);
		dirLabelVal.setBackground(GUI.BGCOLOR);
		dirLabelVal.setLayoutData(gd);

		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.horizontalSpan = 6;
		gd.heightHint = 150;
		gd.widthHint = 350;
		gd.verticalSpan = 4;

		fileList = new List(imageGroup, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
		fileList.setLayoutData(gd);

		if (Main.SIM_LEVEL >= 1) {
			fileList.add("/mnt/staffhome/spothineni/testxds/sdrc_nat4_1.0135");
			fileList.add("/mnt/staffhome/spothineni/testxds/sdrc_nat4_1.0180");
		}
		gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		gd.horizontalSpan = 1;

		addButton = new Button(imageGroup, SWT.PUSH);

		addButton.setText("Add");
		addButton.setLayoutData(gd);
		addButton.setToolTipText("Add test image for the list");

		addButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						Shell fileShell = new Shell(SWT.NONE);
						NBFileDialog dialog = new NBFileDialog(fileShell,
								SWT.OPEN);
						String path1 = DevDataDir.getPrimary();
						if (fileList.getItemCount() > 0) {
							path1 = dirLabelVal.getText();
						}

						dialog.setFilterPath(path1);
						dialog.setFilterExtensions(new String[] { "*.*" });

						String path = dialog.openStr();
						if (path != null && !path.equals("")) {
							String extension = path.substring(
									path.lastIndexOf(".") + 1, path.length());
							
							if (!extension.matches("[0-9][0-9][0-9][0-9][0-9]") && !extension.equalsIgnoreCase("cbf")) {
								Log.error(path
										+ " unsupported file format");
								return;
							}
							String currDir = path.substring(0,
									path.lastIndexOf("/"));
							String prefix = path.substring(
									path.lastIndexOf("/") + 1,
									path.lastIndexOf("_"));

							/* compare directory with first file */
							if (fileList.getItemCount() > 0) {
								String firstFile = fileList.getItem(0);
								String firstFileDir = dirLabelVal.getText();

								if (!firstFileDir.contains(currDir)) {
									Log.error("Directory of the file must same as the first file in the list");
									return;
								}
								String firstPrefix = firstFile.substring(0,
										firstFile.lastIndexOf("_"));

								if (!firstPrefix.contains(prefix)) {
									Log.error("Prefix must be same as the first file in the list");
									return;
								}
							}
							fileList.add(path.substring(
									path.lastIndexOf("/") + 1, path.length()));

							dirLabelVal.setText(currDir);

							ModelReferenceData.IMAGES = "";
							int i = 0;
							for (i = 0; i < fileList.getItemCount() - 1; i++)
								ModelReferenceData.IMAGES += dirLabelVal
										.getText()
										+ "/"
										+ fileList.getItem(i)
										+ " ";
							if (fileList.getItemCount() > 0)
								ModelReferenceData.IMAGES += dirLabelVal
										.getText() + "/" + fileList.getItem(i);
							DevReferenceData.get().setResult(false);
						}
						// Log.note(path);
					}
				});

			}
		});

		gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		gd.horizontalSpan = 1;

		removeButton = new Button(imageGroup, SWT.PUSH);

		removeButton.setText("Remove");
		removeButton.setLayoutData(gd);
		removeButton.setToolTipText("Remove image from the list");

		removeButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						int i = 0;
						String[] selectStr = fileList.getSelection();

						for (String str : selectStr) {
							fileList.remove(str);
						}

						ModelReferenceData.IMAGES = "";

						for (i = 0; i < fileList.getItemCount() - 1; i++)
							ModelReferenceData.IMAGES += dirLabelVal.getText()
									+ "/" + fileList.getItem(i) + " ";
						if (fileList.getItemCount() > 0)
							ModelReferenceData.IMAGES += dirLabelVal.getText()
									+ "/" + fileList.getItem(i);
						
						DevReferenceData.get().setResult(false);
					}
				});

			}
		});

		xdsfileBrowse.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						Shell fileShell = new Shell(SWT.NONE);
						NBFileDialog dialog = new NBFileDialog(fileShell,
								SWT.OPEN);
						String path1 = DevDataDir.getPrimary();
						dialog.setFilterPath(path1 + "/");
						dialog.setFilterExtensions(new String[] { "*HKL",
								"*.hkl", "*.ahkl", "*.AHKL", "*.sca", "*.SCA","*.*"});

						String path = dialog.openStr();

						/*
						 * ModelReferenceData refData = new
						 * ModelReferenceData(); refData.PATH = path;
						 * refData.isHighRes = false; refData.start();
						 */
						ModelReferenceData.loadHKLFile(path, false);
					}
				});

			}
		});

		gd = new GridData(SWT.CENTER, SWT.CENTER, false, false);
		gd.horizontalSpan = 7;

		runXDSButton = new Button(expandComp, SWT.PUSH);
		runXDSButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		runXDSButton.setText("Run XDS for XPLAN");
		runXDSButton.setLayoutData(gd);
		runXDSButton.setToolTipText("Run XDS for generating startegy with XPLAN module");

		runXDSButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {

				if (fileList.getItemCount() == 0) {
					Log.error("File list for startegy is empty");
					return;
				}

				CmdMultiStrategy cmdMultiStrategy = new CmdMultiStrategy();
				String firstFile = fileList.getItem(0);
				String dir = dirLabelVal.getText();

				if (dir.equals("") || dir == null) {
					Log.error(" directory string problem.");
				}
				Log.note("Image directory : " + dir);

				if (multiCrystButton.getSelection()) {
					String xdsFile = xdsfileText.getText();
					File XDSFILE = new File(xdsFile);
					
					if(xdsFile.isEmpty()) {
						Log.error("Partial data file not selected");
						return;
					}
					
					if (!XDSFILE.exists()) {
						Log.error(xdsFile + " partial data file doesn't exist");
						return;
					}

					cmdMultiStrategy.referencedata = xdsFile;
				}
				cmdMultiStrategy.workDir = dir;
				cmdMultiStrategy.start();
			}
		});

		gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		gd.horizontalSpan = 4;
		Label lblResLimit = new Label(crystalGroup, SWT.LEFT);
		lblResLimit.setText("Res. Limit:");
		lblResLimit.setBackground(GUI.BGCOLOR);
		lblResLimit.setLayoutData(gd);
		lblResLimit
				.setToolTipText("Resolution limits for processing by DISTL.signal_strength");

		gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
		gd.horizontalSpan = 7;
		Composite resLimitsComp = new Composite(crystalGroup, SWT.NONE);
		resLimitsComp.setLayoutData(gd);
		resLimitsComp.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(12, false);
		gl.marginHeight = 0;
		gl.marginWidth = 0;
		gl.horizontalSpacing = 0;
		resLimitsComp.setLayout(gl);

		gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
		Label lowLabel = new Label(resLimitsComp, SWT.NONE);
		lowLabel.setText("L");
		lowLabel.setBackground(GUI.BGCOLOR);
		lowLabel.setLayoutData(gd);

		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.horizontalSpan = 3;
		gd.widthHint = 70;

		lowResLimitEntry = new Text(resLimitsComp, SWT.BORDER);
		lowResLimitEntry.setLayoutData(gd);

		lowResLimitEntry.addListener(SWT.Verify, new Listener() {
			public void handleEvent(Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		lowResLimitEntry.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent arg0) {
			}

			public void focusLost(FocusEvent arg0) {
				setResolution();
			}
		});
		lowResLimitEntry.addKeyListener(new KeyListener() {
			@Override
			public void keyPressed(KeyEvent arg0) {
			}

			public void keyReleased(KeyEvent arg0) {
				if (arg0.keyCode == SWT.CR) {
					setResolution();
				}
			}
		});

		gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
		Label highLabel = new Label(resLimitsComp, SWT.NONE);
		highLabel.setText("H");
		highLabel.setBackground(GUI.BGCOLOR);
		highLabel.setLayoutData(gd);

		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.horizontalSpan = 3;
		gd.widthHint = 70;

		highResLimitEntry = new Text(resLimitsComp, SWT.BORDER);
		highResLimitEntry.setLayoutData(gd);

		highResLimitEntry.addListener(SWT.Verify, new Listener() {
			public void handleEvent(Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		highResLimitEntry.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent arg0) {
			}

			public void focusLost(FocusEvent arg0) {
				setResolution();
			}
		});
		highResLimitEntry.addKeyListener(new KeyListener() {
			@Override
			public void keyPressed(KeyEvent arg0) {
			}

			public void keyReleased(KeyEvent arg0) {
				if (arg0.keyCode == SWT.CR) {
					setResolution();
				}
			}
		});

		Label angLabel = new Label(resLimitsComp, SWT.LEFT);
		angLabel.setText("\u212B");
		angLabel.setBackground(GUI.BGCOLOR);
		angLabel.setLayoutData(new GridData());

		addPaintListener(this);
	}

	public void updateLabels() {

		GUI.asyncExec(new Runnable() {
			public void run() {

				if (ModelReferenceData.ISREFERENCE == 1)
					multiCrystButton.setSelection(true);
				else
					multiCrystButton.setSelection(false);

				xdsfileText.setText(ModelReferenceData.REFERENCE_DATA);

				xdsfileText.setSelection(ModelReferenceData.REFERENCE_DATA
						.length());
				String spaceGroup = DevMultiStrategy
						.get()
						.getSpacegroupFromNumber(ModelReferenceData.SPACE_GROUP);

				spaceGroupText.setText(Integer
						.toString(ModelReferenceData.SPACE_GROUP));
				// spaceGroupText.setText(Integer.toString(ModelReferenceData.SPACE_GROUP));
				minRangeText.setText(Integer
						.toString(ModelReferenceData.MINRANGE));
				String unitCell = ModelReferenceData.UNIT_CELL;

				if (unitCell != null) {
					if (!unitCell.equals("") && unitCell != null) {
						// Log.note(unitCell);
						StringTokenizer tok = new StringTokenizer(unitCell);
						if (tok.countTokens() == ModelReferenceData.UnitCell.length) {
							for (int i = 0; i < ModelReferenceData.UnitCell.length; i++) {
								unitCellText[i].setText(tok.nextToken());
							}
						}
					}
				}
				String images = ModelReferenceData.IMAGES;

				fileList.removeAll();

				if (images != null) {
					if (!images.equals("")) {
						StringTokenizer tok = new StringTokenizer(images);

						while (tok.hasMoreTokens()) {
							String image = tok.nextToken();
							int last = image.lastIndexOf("/");
							dirLabelVal.setText(image.substring(0, last));
							fileList.add(image.substring(last + 1,
									image.length()));
						}
					}
				}
				lowResLimitEntry.setText(Double
						.toString(ModelReferenceData.RESOLUTION_LOW));
				highResLimitEntry.setText(Double
						.toString(ModelReferenceData.RESOLUTION_HIGH));
			}
		});
		updateButtons();
	}

	public void updateButtons() {
		final boolean isReference = ModelReferenceData.ISREFERENCE == 1
				&& Client.isMaster();
		final boolean isMaster = Client.isMaster();
		GUI.asyncExec(new Runnable() {
			public void run() {

				multiCrystButton.setEnabled(isMaster);
				xdsfileText.setEnabled(isReference);
				xdsfileBrowse.setEnabled(isReference);
				// crystalGroup.setEnabled(isReference);
				for (int i = 0; i < ModelReferenceData.UnitCell.length; i++) {
					unitCellText[i].setEnabled(isReference);
				}
				spaceGroupText.setEnabled(isReference);
				minRangeText.setEnabled(isMaster);
				highResLimitEntry.setEnabled(isReference);
				lowResLimitEntry.setEnabled(isReference);

				runXDSButton.setEnabled(isMaster);
				addButton.setEnabled(isMaster);
				removeButton.setEnabled(isMaster);

			}
		});
	}

	public void setSpaceGroup() {

		GUI.asyncExec(new Runnable() {
			public void run() {
				String sg = spaceGroupText.getText();
				if (sg.equals("") || sg == null) {
					Log.error("Spacegroup# must be between 1 and 266");
					DevReferenceData.get().setResult(true);
					return;
				}
				try {
					int spaceGroupNo = Integer.parseInt(sg);
					if (spaceGroupNo > 0 && spaceGroupNo <= 266) {
						ModelReferenceData.SPACE_GROUP = spaceGroupNo;
						DevReferenceData.get().setResult(true);
					} else {
						Log.error("Spacegroup# must be between 1 and 266");
						DevReferenceData.get().setResult(true);
					}
				} catch (Exception e) {
					System.out.println(sg);
				}
			}
		});
	}

	public void setUnitCell() {
		GUI.asyncExec(new Runnable() {
			public void run() {
				String unitCell = "";
				for (int i = 0; i < ModelReferenceData.UnitCell.length; i++) {
					unitCell += unitCellText[i].getText() + " ";
				}
				StringTokenizer strTok = new StringTokenizer(unitCell);

				if (strTok.countTokens() == ModelReferenceData.UnitCell.length) {
					ModelReferenceData.UNIT_CELL = unitCell;
					System.out.println("Unit Cell changed by user " + unitCell);
					DevReferenceData.get().setResult(false);
				} else {
					Log.error("Unit Cell : " + unitCell + " not valid");
					DevReferenceData.get().setResult(true);
				}
			}
		});
	}

	public void setMinRange() {
		GUI.asyncExec(new Runnable() {
			public void run() {
				String minrange = minRangeText.getText();
				if (minrange.equals("") || minrange == null) {
					Log.error("Min Range must be between 0 and 30");
					DevReferenceData.get().setResult(true);
					return;
				}
				int minrangeNo = Integer.parseInt(minrange);
				if (minrangeNo > 0 && minrangeNo < 30) {
					ModelReferenceData.MINRANGE = minrangeNo;
					DevReferenceData.get().setResult(false);
				} else {
					Log.error("Min Range must be between 0 and 30");
					DevReferenceData.get().setResult(true);
				}
			}
		});
	}

	public void setResolution() {
		String lowStr = lowResLimitEntry.getText();
		String highStr = highResLimitEntry.getText();

		if (lowStr.equals("") || lowStr == null || highStr.equals("")
				|| highStr == null || lowStr.equals("-") || highStr.equals("-")) {
			DevReferenceData.get().setResult(true);
			return;
		}

		double low = Double.parseDouble(lowStr);
		double high = Double.parseDouble(highStr);

		if (low > 50.0) {
			Log.warn("Low resolution should be less than 50.0");
			DevReferenceData.get().setResult(true);
			return;
		}
		if (high < 0.0) {
			Log.warn("High resolution should be greater than 0.0");
			DevReferenceData.get().setResult(true);
			return;
		}
		if (low < high) {
			Log.warn("Value of Low Res. > High Res");
			DevReferenceData.get().setResult(true);
			return;
		}

		_setResolution(low, high);
	}

	public void _setResolution(double low, double high) {
		ModelReferenceData.RESOLUTION_LOW = low;
		ModelReferenceData.RESOLUTION_HIGH = high;
		Log.note("High resolution limit " + high + " Low resolution limit "
				+ low);
		DevReferenceData.get().setResult(false);
	}

	@Override
	public void paintControl(PaintEvent e) {
	}

	@Override
	public void masterChanged(boolean isMaster) {

		updateButtons();
	}

	@Override
	public void handleEvent(ModelReferenceData object, Object data) {

		updateLabels();
		updateButtons();
	}
}
