package gov.anl.gmca.raster;

import gov.anl.gmca.common.DevEpicsUpdate;
import gov.anl.gmca.common.DevRunList;
import gov.anl.gmca.common.DevRunList.RUN_LABEL_SETTING;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.PVLookup.PV_R_RUN;
import gov.anl.gmca.io.PVLookup.PV_R_RUNS;
import gov.anl.gmca.raster.ModelRasterRun.ROIMode;
import gov.anl.gmca.raster.ModelRasterRun.RefinementType;
import gov.anl.gmca.raster.ModelRasterRun.RunState;
import gov.anl.gmca.raster.ModelRasterRun.RunType;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.util.Point2D;
import gov.anl.gmca.util.Point4D;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;

public class DevRasterRun extends DevEpicsUpdate implements DevRunList.DevRun
{
	protected ModelRasterRun run;
	public ModelRasterRun getRun() { return run; }
	protected int runIndex;
	protected RasterListType rType;
	
	public DevRasterRun(int _runIndex, ModelRasterRun run)
	{
		this.run=run;
		runIndex=_runIndex;
		rType = run.getRunList().getListType();
		
		for (PV_R_RUN pvName : PV_R_RUN.values())
		{
			registerPV(pvName.run(rType, runIndex));
		}
		setUpdatePV(PV_R_RUN.updated.run(rType, _runIndex));
	}

	// PV connection is not done in the constructor to give the caller time to set up listeners
	// before events are sent
	public void connect()
	{
		registrationComplete();
	}
	
	@Override
	public void sendNotification()
	{
		run.setRunState(			RunState.values()[Epics.getInteger(PV_R_RUN.runStatus.run(rType,runIndex))]);
		run.setRunType(				RunType.values()[Epics.getInteger(PV_R_RUN.runType.run(rType,runIndex))]);
		run.setNextFrame(			Epics.getInteger(PV_R_RUN.nextFrame.run(rType,runIndex)));
		run.setRunLabel(			Epics.getInteger(PV_R_RUN.runLabel.run(rType,runIndex)));
		run.setFileRoot(			Epics.getString(PV_R_RUN.fileroot.run(rType,runIndex)));
		run.setBeamWidth_um(		Epics.getDouble(PV_R_RUN.beamWidth_um.run(rType,runIndex)));
		run.setBeamHeight_um(		Epics.getDouble(PV_R_RUN.beamHeight_um.run(rType,runIndex)));
		run.setStartFrame(			Epics.getInteger(PV_R_RUN.startFrameLabel.run(rType,runIndex)));
		run.setAutoFind(			Epics.getBoolean(PV_R_RUN.autoFind.run(rType,runIndex)));
		run.setLoopCentered(		Epics.getBoolean(PV_R_RUN.loopCentered.run(rType,runIndex)));
		run.setROIMode(				ROIMode.values()[Epics.getInteger(PV_R_RUN.roiMode.run(rType,runIndex))]);
		run.setExposureTime(		Epics.getDouble(PV_R_RUN.exposure_sec.run(rType,runIndex)));
		run.setDelta(				Epics.getDouble(PV_R_RUN.delta_deg.run(rType,runIndex)));
		run.setDetectorDist(		Epics.getDouble(PV_R_RUN.distance_mm.run(rType,runIndex)));
		run.setAttenuation(			Epics.getDouble(PV_R_RUN.atten_factor.run(rType,runIndex)));
		run.setEnergy(				Epics.getDouble(PV_R_RUN.energy_keV.run(rType,runIndex)));
		run.grid().setHOffsets_um(	Epics.getDouble(PV_R_RUN.gridLeftOffset_um.run(rType,runIndex)),Epics.getDouble(PV_R_RUN.gridRightOffset_um.run(rType,runIndex)));
		run.grid().setVOffsets_um(	Epics.getDouble(PV_R_RUN.gridTopOffset_um.run(rType,runIndex)),Epics.getDouble(PV_R_RUN.gridBottomOffset_um.run(rType,runIndex)));
		run.grid().setCellWidth_um(	Epics.getDouble(PV_R_RUN.cellWidth_um.run(rType,runIndex)));
		run.grid().setCellHeight_um(Epics.getDouble(PV_R_RUN.cellHeight_um.run(rType,runIndex)));
		run.setDirectorySuffix(    		Epics.getString(PV_R_RUN.directory256.run(rType,runIndex)));
		run.setReferenceRun(		Epics.getInteger(PV_R_RUN.referenceRun.run(rType,runIndex)));
		run.setReferenceHit(		Epics.getInteger(PV_R_RUN.referenceHit.run(rType,runIndex)));
		run.setRefinementType(		RefinementType.values()[Epics.getInteger(PV_R_RUN.refinementType.run(rType,runIndex))]);
		run.setHideRunParameters(	Epics.getBoolean(PV_R_RUN.hideRunParameters.run(rType,runIndex)));
		run.setLoopMeasurement_um(	Epics.getInteger(PV_R_RUN.loopMeasurement_um.run(rType, runIndex)));
		run.setLowResLimit_A(		Epics.getDouble(PV_R_RUN.lowResLimit_A.run(rType, runIndex)));
		run.setHighResLimit_A(		Epics.getDouble(PV_R_RUN.highResLimit_A.run(rType, runIndex)));
		run.setSONICCBin(			Epics.getInteger(PV_R_RUN.soniccBin.run(rType, runIndex))==1);

		double[] refPosWithFlag = Epics.getDoubleArray(PV_R_RUN.soniccRefPos.run(rType, runIndex));
		Point4D soniccRefPos = null;
		if (refPosWithFlag[0]==1) soniccRefPos = new Point4D(refPosWithFlag[1],refPosWithFlag[2],refPosWithFlag[3],refPosWithFlag[4]);
		run.setSONICCRefPos(soniccRefPos);

		//////////////////////////////// zoom,center,offset ///////////////////////////////
		
		Double savedZoom = Epics.getDouble(PV_R_RUN.savedZoom.run(rType, runIndex));
		if (savedZoom < 0.001) savedZoom = null;
		
		Point4D center = null;
		if (Epics.getBoolean(PV_R_RUN.isCenterSet.run(rType,runIndex)))
		{
			center = new Point4D
			(
				Epics.getDouble(PV_R_RUN.center_x_mm.run(rType,runIndex)),
				Epics.getDouble(PV_R_RUN.center_y_mm.run(rType,runIndex)),
				Epics.getDouble(PV_R_RUN.center_z_mm.run(rType,runIndex)),
				Epics.getDouble(PV_R_RUN.center_omega_deg.run(rType,runIndex))
			);
		}
		run.grid().setCenter(center);
		
		Point4D userPosition = null;
		if (Epics.getBoolean(PV_R_RUN.userOffsetSet.run(rType,runIndex)))
		{
			userPosition = new Point4D
			(
				Epics.getDouble(PV_R_RUN.userOffsetX.run(rType,runIndex)),
				Epics.getDouble(PV_R_RUN.userOffsetY.run(rType,runIndex)),
				Epics.getDouble(PV_R_RUN.userOffsetZ.run(rType,runIndex)),
				Epics.getDouble(PV_R_RUN.userOffsetOmega.run(rType,runIndex))
			);
		}
		run.setUserPosition(userPosition);
		
		////////////////////////////////// contours //////////////////////////////////
		
		double[] contourArray = Epics.getDoubleArray(PV_R_RUN.contours.run(rType,runIndex));
		ArrayList<ModelContour> contours = new ArrayList<ModelContour>();
		
		int arrayIndex = 1;
		int contourCount = (int)Math.round(contourArray[0]);
		
		if (contourCount > 9)
		{
			System.out.println("WARNING: Contour array reports > 9 contours");
			String s = "Contour array as read: ";
			for (int i=0;i<5;i++) {
				s += i+"="+contourArray[i]+" ";
			}
			System.out.println(s);
			
			System.out.println("caget "+PV_R_RUN.contours.run(rType, runIndex).toString());
			ExecTask.syncExec("caget "+PV_R_RUN.contours.run(rType,runIndex), "caget", true);
		}
		
		for (int contourIndex=0; contourIndex<contourCount; contourIndex++)
		{
			ModelContour c = new ModelContour();
			
			int pointTotal = (int)contourArray[arrayIndex++];
			
			for (int pointIndex = 0; pointIndex < pointTotal; pointIndex++)
			{
				double x = contourArray[arrayIndex++];
				double y = contourArray[arrayIndex++];
				Point2D point = new Point2D(x,y);
				c.points.add(point);
			}
			
			contours.add(c);
		}
		run.grid().setContours(contours);

		/////////////////////////////// selected cells /////////////////////////////

		int[] selectedArray = Epics.getIntegerArray(PV_R_RUN.selectedCells.run(rType,runIndex));
		int cellNumber = 0;
		int cellCount = selectedArray[0];
		ArrayList<Boolean> newSelectedCells = new ArrayList<Boolean>();
		int selectedCellCount=0;
		for (int i=1; i<selectedArray.length; i++)
		{
			if (cellNumber >= cellCount) break;
			
			for (int bit=30; bit>=0; bit--)
			{
				if (cellNumber >= cellCount) break;
				
				boolean bSelected = (0 != (selectedArray[i] & 1<<bit));
				newSelectedCells.add(Boolean.valueOf(bSelected));
				if (bSelected) selectedCellCount++;
				
				cellNumber++;
			}
		}
		selectedCells = newSelectedCells;
		run.grid().setSelectedCellCount(selectedCellCount);

		//////////////////////////////////// end ///////////////////////////////////
		
		run.handleChange();
	}
	
	protected ArrayList<Boolean> selectedCells = new ArrayList<Boolean>();
	public ArrayList<Boolean> getSelectedCells() { return new ArrayList<Boolean>(selectedCells); }
	public boolean isCellSelected(int cellNumber)
	{
		if (cellNumber >= selectedCells.size()) return false;
		return selectedCells.get(cellNumber);
	}

	// these set PVs directly
	public void setRunStatus(RunState state, boolean update)		{ run.setRunState(state);			run.handleChange();	Epics.putInteger	(PV_R_RUN.runStatus.run(rType,runIndex),		run.getRunState().ordinal());		if(update)sendUpdate();	}
	public void setRunType(RunType type, boolean update) 			{ run.setRunType(type);				run.handleChange();	Epics.putInteger	(PV_R_RUN.runType.run(rType,runIndex),			run.getRunType().ordinal());		if(update)sendUpdate();	}
	public void setRunLabel(int label, boolean update)				{ run.setRunLabel(label);			run.handleChange();	Epics.putInteger	(PV_R_RUN.runLabel.run(rType,runIndex),			run.getRunLabel());					if(update)sendUpdate();	}
	public void setROIMode(ROIMode mode, boolean update)			{ run.setROIMode(mode);				run.handleChange();	Epics.putInteger	(PV_R_RUN.roiMode.run(rType,runIndex), 			run.getROIMode().ordinal());		if(update)sendUpdate();	}
	public void setEnergy_keV(double d, boolean update)				{ run.setEnergy(d);					run.handleChange();	Epics.putDouble		(PV_R_RUN.energy_keV.run(rType,runIndex),		run.getEnergy());					if(update)sendUpdate();	}
	public void setReferenceRun(int i, boolean update)				{ run.setReferenceRun(i);			run.handleChange();	Epics.putInteger	(PV_R_RUN.referenceRun.run(rType,runIndex),		run.getReferenceRun());				if(update)sendUpdate();	}
	public void setReferenceHit(int i, boolean update)				{ run.setReferenceHit(i);			run.handleChange();	Epics.putInteger	(PV_R_RUN.referenceHit.run(rType,runIndex),		run.getReferenceHit());				if(update)sendUpdate();	}
	public void setLoopMeasurement_um(int i, boolean update)		{ run.setLoopMeasurement_um(i);		run.handleChange();	Epics.putInteger	(PV_R_RUN.loopMeasurement_um.run(rType,runIndex),	run.getLoopMeasurement_um());	if(update)sendUpdate();	}
	public void setHideRunParameters(boolean b, boolean update)		{ run.setHideRunParameters(b);		run.handleChange();	Epics.putInteger	(PV_R_RUN.hideRunParameters.run(rType,runIndex),	run.getHideRunParameters()?1:0);if(update)sendUpdate();	}
	
	// these use static methods
	public void setAttenuation_factors(double d, boolean update)	{ run.setAttenuation(d);			run.handleChange();					s_setAttenuation_factors(run.getRunList().getListType(), run.getRunIndex(), run.getAttenuation());				if(update)sendUpdate();	}
	public void setBeamWidth_um(double d, boolean update)			{ d=Math.round(d); run.setBeamWidth_um(d);	run.handleChange();			s_setBeamHeight_um		(run.getRunList().getListType(), run.getRunIndex(), run.getBeamWidth_um());				if(update)sendUpdate();	}
	public void setBeamHeight_um(double d, boolean update)			{ d=Math.round(d); run.setBeamHeight_um(d);	run.handleChange();			s_setBeamWidth_um		(run.getRunList().getListType(), run.getRunIndex(), run.getBeamHeight_um());			if(update)sendUpdate();	}
	public void setCellWidth_um(double d, boolean update)			{ d=Math.round(d); run.grid().setCellWidth_um(d);	run.handleChange();	s_setCellWidth_um		(run.getRunList().getListType(), run.getRunIndex(), run.grid().getCellWidth_um());		if(update)sendUpdate();	}
	public void setCellHeight_um(double d, boolean update)			{ d=Math.round(d); run.grid().setCellHeight_um(d);	run.handleChange();	s_setCellHeight_um		(run.getRunList().getListType(), run.getRunIndex(), run.grid().getCellHeight_um());		if(update)sendUpdate();	}
	public void setDelta(double deg, boolean update)				{ run.setDelta(deg);				run.handleChange();					s_setDelta				(run.getRunList().getListType(), run.getRunIndex(), run.getDelta());					if(update)sendUpdate();	}
	public void setDirectory(String dir, boolean update)			{ run.setDirectorySuffix(dir);			run.handleChange();					s_setDirectory			(run.getRunList().getListType(), run.getRunIndex(), run.getDirectorySuffix());				if(update)sendUpdate();	}
	public void setDistance_mm(double d, boolean update)			{ run.setDetectorDist(d);			run.handleChange();					s_setDistance_mm		(run.getRunList().getListType(), run.getRunIndex(), run.getDetectorDist());				if(update)sendUpdate();	}
	public void setAutoFind(boolean b, boolean update)				{ run.setAutoFind(b);				run.handleChange();					s_setAutoFind			(run.getRunList().getListType(), run.getRunIndex(), run.getAutoFind());					if(update)sendUpdate();	}
	public void setLoopCentered(boolean b, boolean update)			{ run.setLoopCentered(b);			run.handleChange();					s_setLoopCentered		(run.getRunList().getListType(), run.getRunIndex(), run.getLoopCentered());				if(update)sendUpdate();	}
	public void setExposureTime(double sec, boolean update) 		{ run.setExposureTime(sec);			run.handleChange();					s_setExposureTime		(run.getRunList().getListType(), run.getRunIndex(), run.getExposureTime());				if(update)sendUpdate();	}
	public void setFileroot(String root, boolean update)			{ run.setFileRoot(root);			run.handleChange();					s_setFileroot			(run.getRunList().getListType(), run.getRunIndex(), run.getFileRoot());					if(update)sendUpdate();	}
	public void setGridHOffsets_um(double a, double b, boolean update)	{ run.grid().setHOffsets_um(a,b);	run.handleChange();				s_setGridHOffsets		(run.getRunList().getListType(), run.getRunIndex(), run.grid().getLeftOffset_um(), 	run.grid().getRightOffset_um()); 	if(update)sendUpdate(); }
	public void setGridVOffsets_um(double a, double b, boolean update)	{ run.grid().setVOffsets_um(a,b);	run.handleChange();				s_setGridVOffsets		(run.getRunList().getListType(), run.getRunIndex(), run.grid().getTopOffset_um(), 	run.grid().getBottomOffset_um()); 	if(update)sendUpdate(); }
	public void setStartFrameLabel(int label, boolean update)		{ run.setStartFrame(label);			run.handleChange();					s_setStartFrameLabel	(run.getRunList().getListType(), run.getRunIndex(), run.getStartFrame());				if(update)sendUpdate();	}
	public void setNextFrame(int frame, boolean update)				{ run.setNextFrame(frame);			run.handleChange();					s_setNextFrame			(run.getRunList().getListType(), run.getRunIndex(), run.getNextFrame());				if(update)sendUpdate();	}
	public void setRefinementType(RefinementType rt, boolean update){ run.setRefinementType(rt);		run.handleChange();					s_setRefinementType		(run.getRunList().getListType(), run.getRunIndex(), run.getRefinementType());			if(update)sendUpdate();	}
	public void setLowResLimit_A(double d, boolean update)			{ run.setLowResLimit_A(d);			run.handleChange();					s_setLowResLimit_A		(run.getRunList().getListType(), run.getRunIndex(), run.getLowResLimit_A());			if(update)sendUpdate();	}
	public void setHighResLimit_A(double d, boolean update)			{ run.setHighResLimit_A(d);			run.handleChange();					s_setHighResLimit_A		(run.getRunList().getListType(), run.getRunIndex(), run.getHighResLimit_A());			if(update)sendUpdate();	}
	public void setSONICCBin(boolean b, boolean update)				{ run.setSONICCBin(b);				run.handleChange();					s_setSONICCBin			(run.getRunList().getListType(), run.getRunIndex(), run.getSONICCBin());				if(update)sendUpdate();	}
	public void setSONICCRefPos(Point4D p, boolean update)			{ run.setSONICCRefPos(p);			run.handleChange();					s_setSONICCRefPos		(run.getRunList().getListType(), run.getRunIndex(), run.getSONICCRefPosArray());		if(update)sendUpdate();	}
	public void setSavedZoom(Double d, boolean update)				{ run.setSavedZoom(d);				run.handleChange();					s_setSavedZoom			(run.getRunList().getListType(), run.getRunIndex(), run.getSavedZoom());				if(update)sendUpdate();	}

	protected static String getPVName(RasterListType type, int runIndex, String pvSuffix)
	{
		return Epics.addBID("bi:"+(type==RasterListType.AUTO?"a":"")+"r_r"+runIndex+":"+pvSuffix);
	}
	
	public void setUserPosition(Point4D pt, boolean update)
	{
		run.setUserPosition(pt);
		run.handleChange();
		
		Epics.putDouble	(PV_R_RUN.userOffsetX.run(rType,runIndex), 		null==pt?0:run.getUserPosition().x);
		Epics.putDouble	(PV_R_RUN.userOffsetY.run(rType,runIndex), 		null==pt?0:run.getUserPosition().y);
		Epics.putDouble	(PV_R_RUN.userOffsetZ.run(rType,runIndex), 		null==pt?0:run.getUserPosition().z);
		Epics.putDouble	(PV_R_RUN.userOffsetOmega.run(rType,runIndex),	null==pt?0:run.getUserPosition().omega);
		Epics.putInteger(PV_R_RUN.userOffsetSet.run(rType,runIndex), 	null==pt?0:1);
		
		if (update) sendUpdate();
	}
	
	public void setCenter(Point4D center, boolean update)
	{
		run.grid().setCenter(center);
		run.handleChange();
		
		// TODO: temporarily allow null directly
		Point4D runCenter = center==null?null:run.grid().getCenter();
		
		s_setCenter(run.getRunList().getListType(), run.getRunIndex(), runCenter);
		
		if(update)sendUpdate();	
	}
	
	public void setContours(ArrayList<ModelContour> contours, boolean update)
	{
		run.grid().setContours(contours);
		run.handleChange();
		
		s_setContours(run.getRunList().getListType(), run.getRunIndex(), contours);
		
		if(update)sendUpdate();	
	}

	// boolean array is packed into an int array using 31 bits per int, starting with the highest non-sign bit
	public void setSelectedCells(ArrayList<Boolean> selectedCells, boolean update)
	{
		// first send the array to the models
		this.selectedCells = selectedCells;
		run.handleChange();
		
		// now send it to EPICS
		s_setSelectedCells(run.getRunList().getListType(), run.getRunIndex(), selectedCells);
		
		if(update)sendUpdate();	
	}
	
	@Override
	public void copyRunContents(int destRunIndex, RUN_LABEL_SETTING runLabelSetting, boolean resetState, boolean doUpdate)
	{
		boolean addOneToRunLabel;
		switch (runLabelSetting)
		{
		case SAME:			addOneToRunLabel = false; break;
		case INCREMENT_SRC: addOneToRunLabel = true;  break;
		default:	throw new IllegalArgumentException("Unsupported label setting: "+runLabelSetting);
		}
		
		System.out.println("Copying run index "+runIndex+" contents to run index "+destRunIndex+" addOne="+addOneToRunLabel+" update="+doUpdate);
		
		// JCA doesn't have a generic ch.put() function, so converting to and from a String
		// allows a simple loop here.  The alternative is to have a line for each PV.
		for (PV_R_RUN runPV : PV_R_RUN.values())
		{
			switch (runPV)
			{
			case runStatus:			if (resetState) 	Epics.putInteger(PV_R_RUN.runStatus.run(rType,destRunIndex), 		0); break;
			case isCenterSet:		if (resetState) 	Epics.putInteger(PV_R_RUN.isCenterSet.run(rType,destRunIndex),		0);	break;
			case center_x_mm:		if (resetState)		Epics.putInteger(PV_R_RUN.center_x_mm.run(rType,destRunIndex),		0);	break;
			case center_y_mm:		if (resetState) 	Epics.putInteger(PV_R_RUN.center_y_mm.run(rType,destRunIndex),		0);	break;
			case center_z_mm:		if (resetState) 	Epics.putInteger(PV_R_RUN.center_z_mm.run(rType,destRunIndex),		0);	break;
			case center_omega_deg:	if (resetState) 	Epics.putInteger(PV_R_RUN.center_omega_deg.run(rType,destRunIndex),	0);	break;
			case nextFrame:			if (resetState) 	Epics.putInteger(PV_R_RUN.nextFrame.run(rType,destRunIndex),		0);	break;
			case soniccBin:			if (resetState)		Epics.putInteger(PV_R_RUN.soniccBin.run(rType,destRunIndex),		0); break;
			case contours:
			{
				if (resetState)	Epics.putInteger	(PV_R_RUN.contours.run(rType,destRunIndex), 0);
				else			Epics.putDoubleArray(PV_R_RUN.contours.run(rType,destRunIndex),	Epics.getDoubleArray(PV_R_RUN.contours.run(rType,runIndex)));
				break;
			}	
			case selectedCells:
			{
				if (resetState)	Epics.putInteger		(PV_R_RUN.selectedCells.run(rType,destRunIndex), 	0);
				else			Epics.putIntegerArray	(PV_R_RUN.selectedCells.run(rType,destRunIndex),	Epics.getIntegerArray(PV_R_RUN.selectedCells.run(rType,runIndex)));
				break;
			}	
			default:
			{
    			String srcValue = Epics.getString(runPV.run(rType, runIndex),5);
    			Epics.put(runPV.run(rType, destRunIndex),srcValue);
			}}
		}
		
		if (addOneToRunLabel)
		{
			Epics.putInteger(PV_R_RUN.runLabel.run(rType, destRunIndex), Epics.getInteger(PV_R_RUN.runLabel.run(rType,runIndex))+1);
		}
		
		if (doUpdate) ((DevRasterRun)run.runList.devRunList.getRunDev(destRunIndex)).sendUpdate();
		
		// TODO: update needed?
		//if (doUpdate) DevRasterRunResults.sendUpdate(run.getRunList().getListType(), destRunIndex);
	}
	
	@Override
	public void stopEpicsMonitors()
	{
		super.stopEpicsMonitors();
	}

	// copy selectedCells to the non-run-specific PV used by the fl Raster script to
	// know which cells are left to raster
	public void sendNonRasteredCellsToScript()
	{
		ArrayList<Boolean> selectedCellsNotRastered = getSelectedCells();
		for (int i=0;i<run.getNextFrame();i++)
		{
			selectedCellsNotRastered.set(i, false);
		}
		int[] packedArray = packArray(selectedCellsNotRastered);
		Epics.putIntegerArray(PV_R_RUNS.selectedCells, packedArray);
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////// static methods /////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	public static int s_getRunLabel				(RasterListType type, int index)									{ return Epics.getInteger		(PV_R_RUN.runLabel.run(type,index)); 			}
	public static void s_setAttenuation_factors	(RasterListType type, int index, double d) 							{ Epics.putDouble		(PV_R_RUN.atten_factor.run(type,index), 		d); 	}
	public static void s_setBeamHeight_um		(RasterListType type, int index, double d) 							{ Epics.putDouble		(PV_R_RUN.beamWidth_um.run(type,index), 		d); 	}
	public static void s_setBeamWidth_um		(RasterListType type, int index, double d) 							{ Epics.putDouble		(PV_R_RUN.beamHeight_um.run(type,index), 		d); 	}
	public static void s_setCellWidth_um		(RasterListType type, int index, double d) 							{ Epics.putDouble		(PV_R_RUN.cellWidth_um.run(type,index), 		d); 	}
	public static void s_setCellHeight_um		(RasterListType type, int index, double d) 							{ Epics.putDouble		(PV_R_RUN.cellHeight_um.run(type,index), 		d); 	}
	public static void s_setDelta				(RasterListType type, int index, double d) 							{ Epics.putDouble		(PV_R_RUN.delta_deg.run(type,index), 			d); 	}
	public static void s_setDirectory			(RasterListType type, int index, String dir) 						{ Epics.put				(PV_R_RUN.directory256.run(type,index), 		dir); 	}
	public static void s_setDistance_mm			(RasterListType type, int index, double d) 							{ Epics.putDouble		(PV_R_RUN.distance_mm.run(type,index), 			d); 	}
	public static void s_setAutoFind			(RasterListType type, int index, boolean b) 						{ Epics.putInteger		(PV_R_RUN.autoFind.run(type,index), 			b?1:0); }
	public static void s_setLoopCentered		(RasterListType type, int index, boolean b) 						{ Epics.putInteger		(PV_R_RUN.loopCentered.run(type,index), 		b?1:0); }
	public static void s_setExposureTime		(RasterListType type, int index, double d) 							{ Epics.putDouble		(PV_R_RUN.exposure_sec.run(type,index), 		d); 	}
	public static void s_setFileroot			(RasterListType type, int index, String root) 						{ Epics.put				(PV_R_RUN.fileroot.run(type,index), 			root); 	}
	public static void s_setGridHOffsets		(RasterListType type, int index, double min, double max) 			{ Epics.putDouble		(PV_R_RUN.gridLeftOffset_um.run(type,index), 	min); 	Epics.putDouble		(PV_R_RUN.gridRightOffset_um.run(type,index), 	max);	}
	public static void s_setGridVOffsets		(RasterListType type, int index, double min, double max) 			{ Epics.putDouble		(PV_R_RUN.gridTopOffset_um.run(type,index), 	min); 	Epics.putDouble		(PV_R_RUN.gridBottomOffset_um.run(type,index), max); 	}
	public static void s_setStartFrameLabel		(RasterListType type, int index, int label) 						{ Epics.putInteger		(PV_R_RUN.startFrameLabel.run(type,index), 		label); }
	public static void s_setNextFrame			(RasterListType type, int index, int frame) 						{ Epics.putInteger		(PV_R_RUN.nextFrame.run(type,index), 			frame); }
	public static void s_setReferenceHit		(RasterListType type, int index, int i)								{ Epics.putInteger		(PV_R_RUN.referenceHit.run(type,index), 		i); 	}
	public static void s_setReferenceRun		(RasterListType type, int index, int i)								{ Epics.putInteger		(PV_R_RUN.referenceRun.run(type,index), 		i); 	}
	public static void s_setRefinementType		(RasterListType type, int index, RefinementType refType)			{ Epics.putInteger		(PV_R_RUN.refinementType.run(type,index), 		refType.ordinal()); }
	public static void s_setRunLabel			(RasterListType type, int index, int runLabel)						{ Epics.putInteger		(PV_R_RUN.runLabel.run(type,index), 			runLabel); }
	public static void s_setRunState			(RasterListType type, int index, RunState runState)					{ Epics.putInteger		(PV_R_RUN.runStatus.run(type,index), 			runState.ordinal()); }
	public static void s_setLowResLimit_A		(RasterListType type, int index, double d)							{ Epics.putDouble		(PV_R_RUN.lowResLimit_A.run(type,index), 		d); 	}
	public static void s_setHighResLimit_A		(RasterListType type, int index, double d)							{ Epics.putDouble		(PV_R_RUN.highResLimit_A.run(type,index), 		d); 	}
	public static void s_setSONICCBin			(RasterListType type, int index, boolean b)							{ Epics.putInteger		(PV_R_RUN.soniccBin.run(type,index), 			b?1:0); 	}
	public static void s_setSONICCRefPos		(RasterListType type, int index, double[] d)						{ Epics.putDoubleArray	(PV_R_RUN.soniccRefPos.run(type,index), 		d); 	}
	public static void s_setSavedZoom			(RasterListType type, int index, Double d)							{ Epics.putDouble		(PV_R_RUN.savedZoom.run(type,index), 			d==null?0:d); 	}
	
	public static void s_setCenter				(RasterListType type, int index, Point4D center)
	{
		if (null == center)
    	{
    		Epics.putDouble(	PV_R_RUN.center_x_mm.run(type,index),		0.0);
    		Epics.putDouble(	PV_R_RUN.center_y_mm.run(type,index),		0.0);
    		Epics.putDouble(	PV_R_RUN.center_z_mm.run(type,index),		0.0);
    		Epics.putDouble(	PV_R_RUN.center_omega_deg.run(type,index),	0.0);
    		Epics.putInteger(	PV_R_RUN.isCenterSet.run(type,index),		0);
    	}
    	else
    	{
    		Epics.putDouble(	PV_R_RUN.center_x_mm.run(type,index),		center.x);
    		Epics.putDouble(	PV_R_RUN.center_y_mm.run(type,index),		center.y);
    		Epics.putDouble(	PV_R_RUN.center_z_mm.run(type,index),		center.z);
    		Epics.putDouble(	PV_R_RUN.center_omega_deg.run(type,index),	center.omega);
    		Epics.putInteger(	PV_R_RUN.isCenterSet.run(type,index),		1);
    	}
	}
	
	public static void s_setContours(RasterListType type, int index, ArrayList<ModelContour> contours)
	{
		ArrayList<Double> arrayToWrite = new ArrayList<Double>();
		
		// # of contours
		arrayToWrite.add((double)contours.size());
		
		for (ModelContour c : contours)
		{
			// # of points in this contour
			arrayToWrite.add((double)c.points.size());
			
			for (Point2D p : c.points)
			{
				// point X, Y
				arrayToWrite.add(p.x);
				arrayToWrite.add(p.y);
			}
		}
		
		double[] doubleArray = new double[arrayToWrite.size()];
		String arrayString = "";
		for (int i=0; i<arrayToWrite.size(); i++)
		{
			doubleArray[i] = arrayToWrite.get(i);
			arrayString += Util.printDouble(doubleArray[i],1)+" ";
		}
		System.out.println("Writing contours for run "+index+": "+arrayString);
		
		Epics.putDoubleArray(PV_R_RUN.contours.run(type,index), doubleArray);
	}
	
	public static void s_setSelectedCells(RasterListType type, int index, ArrayList<Boolean> selectedCells)
	{
		int[] packedArray = packArray(selectedCells);
		Epics.putIntegerArray(PV_R_RUN.selectedCells.run(type,index), packedArray);
	}
	
	protected static int[] packArray(ArrayList<Boolean> list)
	{
		int[] packedArray = new int[(int)Math.ceil(list.size()/31.0)+1];
		packedArray[0] = list.size();
		
		int bitIndex=30, arrayIndex=1;
		for (int i=0; i < list.size(); i++)
		{
			if (list.get(i))
			{
				packedArray[arrayIndex] |= 1<<bitIndex;
			}
			bitIndex--;
			if (bitIndex<0)
			{
				bitIndex=30;
				arrayIndex++;
			}
		}
		return packedArray;
	}
}
