package gov.anl.gmca.raster;

import gov.anl.gmca.common.DevEpicsUpdate;
import gov.anl.gmca.common.DevRunList;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.io.PVLookup.PV_R_RUNS;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;

import java.util.ArrayList;
import java.util.HashMap;

public class DevRasterRunList extends DevEpicsUpdate implements DevRunList
{
	protected ModelRasterRunList runList;
	protected RasterListType type;
	protected PVName countPV, currentPV;
	
	protected ArrayList<DevRasterRun> runDevs = new ArrayList<DevRasterRun>();
	
    public DevRun getRunDev(int runIndex) { if (runIndex>=runDevs.size()) return null; return (DevRun)runDevs.get(runIndex); }
	
	protected static HashMap<RasterListType,DevRasterRunList> instMap = new HashMap<RasterListType,DevRasterRunList>();
	public static DevRasterRunList get(RasterListType type) { return instMap.get(type); }
	
	public DevRasterRunList(ModelRasterRunList _runList)
	{
		runList=_runList;
		runList.devRunList = this;
		type = _runList.getListType();

		instMap.put(_runList.listType, this);

		PVName updatePV = null;
		
		switch (type)
		{
		case INTER:
			countPV = 	PV_R_RUNS.runCount;
			currentPV = PV_R_RUNS.currentRun;
			updatePV = 	PV_R_RUNS.updated;
			break;
		case AUTO:
			countPV = 	PV_R_RUNS.autoRunCount;
			currentPV = PV_R_RUNS.autoCurrentRun;
			updatePV = 	PV_R_RUNS.autoUpdated;
			break;
		default:
			throw new IllegalArgumentException("Raster list type "+type+" not supported");
		}

		setUpdatePV(updatePV);
		registerPV(countPV);
		registerPV(currentPV);
		//registrationComplete();
	}

	public int getCurrentRunIndex() { return Epics.getInteger(currentPV); }
	public int getRunCount()
	{
		int runCount = Epics.getInteger(countPV);
		return runCount;
	}

	@Override
	protected void sendNotification()
	{
		int runCount 	= getRunCount();
		int currentRun 	= getCurrentRunIndex();
		
		// TODO: This should be moved to ModelCollectRunList.validateAndFix()
		if (currentRun>runCount)
		{
			setCurrentRun(runCount,true);
			return;
		}
		
		// set these two variables in an order so runCount is never less than currentRun
		if (runList.getCurrentRunIndex() > runCount)
		{
			runList.setCurrentRun(currentRun);
			runList.setRunCount(runCount);
		}
		else
		{
			runList.setRunCount(runCount);
			runList.setCurrentRun(currentRun);
		}
		
		runList.notifyListeners();
		//runList.notifyCurrentRunListeners();
	}
	
    @Override public boolean setRunCount(int runCount, boolean sendUpdate)
    {
    	if (runCount<0) { System.out.println("Ignoring setRunCount "+runCount); if (sendUpdate) sendUpdate(); return true; }
    	
    	//if (!runList.validateRunCount(runCount)) return false;

    	// see collect list for why this can't be done in interactive mode
    	if (runList.getListType() == RasterListType.AUTO)
    	{
    		runList.setRunCount(runCount);
    		runList.notifyListeners();
    	}
    	
		Epics.putInteger(countPV, runCount);
		if (sendUpdate) sendUpdate();
		
    	return true;
    }
    
    @Override public boolean setCurrentRun(int currentRun, boolean sendUpdate)
    {
    	if (currentRun<0) { System.out.println("Ignoring setCurrentRun "+currentRun); if (sendUpdate) sendUpdate(); return true; }
    	
    	// this returns false when the star tab is clicked, because the callback didn't come
    	// back yet to update the runCount
    	//if (!runList.validateCurrentRun(currentRun)) return false;
    	
    	if (runList.getListType() == RasterListType.AUTO)
    	{
    		runList.setCurrentRun(currentRun);
    		runList.notifyListeners();
    	}
    	
		Epics.putInteger(currentPV, currentRun);
		if (sendUpdate) sendUpdate();
		
		return true;
    }

	public void connect()
	{
		registrationComplete();
	}
}
