package gov.anl.gmca.raster;

import gov.anl.gmca.collect.DevBinMode.BinMode;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;
import gov.anl.gmca.io.PVLookup.PV_R_RUNS;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.raster.ModelRasterState.RasterState;
import gov.aps.jca.dbr.DBR;

public class DevRasterState
{
	protected ModelRasterState model;
	
	protected static DevRasterState inst = null;
	public static DevRasterState get() { return inst; }
	
	public DevRasterState(final ModelRasterState model)
	{
		inst = this;
		this.model = model;

		// pre-connect
		Epics.getDouble(PV_BLUICE.beamsize_h_act1$VAL);
		Epics.getDouble(PV_BLUICE.beamsize_v_act1$VAL);
		
		Epics.addMonitor(PV_R_RUNS.rasterState, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				model.setRasterState(RasterState.values()[Epics.dbrToInt(value)]);
				model.llCurrentRun.notifyListeners();
			}
		});
		Epics.addMonitor(PV_R_RUNS.autoMode, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				model.setAutoMode(RasterListType.values()[Epics.dbrToInt(value)]);
				model.llCurrentRun.notifyListeners();
			}
		});
		Epics.addMonitor(PV_R_RUNS.binMode, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				model.setBinMode(BinMode.values()[Epics.dbrToInt(value)]);
				model.llCurrentRun.notifyListeners();
			}
		});
		Epics.addMonitor(PV_R_RUNS.fixResLimits, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				model.setFixLimits(Epics.dbrToInt(value)==1);
				model.llCurrentRun.notifyListeners();
			}
		});
	}
	
	public void setRasterState(RasterState state)
	{
		model.setRasterState(state);
		Epics.putInteger(PV_R_RUNS.rasterState, state.ordinal());
	}
	
	public void setAutoMode(RasterListType autoMode)
	{
		model.setAutoMode(autoMode);
		Epics.putInteger(PV_R_RUNS.autoMode, autoMode.ordinal());
	}
	
	public void setBinMode(BinMode mode)
	{
		model.setBinMode(mode);
		Epics.putInteger(PV_R_RUNS.binMode, mode.ordinal());
	}

	public void setFixResLimits(boolean fixLimits)
	{
		model.setFixLimits(fixLimits);
		Epics.putInteger(PV_R_RUNS.fixResLimits, fixLimits?1:0);
	}
}
