package gov.anl.gmca.raster.calc;

import gov.anl.gmca.raster.ModelRasterGrid;
import gov.anl.gmca.raster.ModelRasterGrid.CellSizeMethod;
import gov.anl.gmca.sample.ModelBeamsize;
import gov.anl.gmca.util.Point2D;

public class CellCalculator
{
	public static Point2D getValidCellSize(double requestedCellWidth_um, double requestedCellHeight_um, CellSizeMethod cellSizeMethod)
	{
		// if in mini-beam range, set to valid collimator size
		if (requestedCellWidth_um < 7.5) requestedCellWidth_um = 5.0;
		else if (requestedCellWidth_um < 15.0) requestedCellWidth_um = 10.0;
		else if (requestedCellWidth_um < 25.0) requestedCellWidth_um = 20.0;
		
		if (requestedCellHeight_um < 7.5) requestedCellHeight_um = 5.0;
		else if (requestedCellHeight_um < 15.0) requestedCellHeight_um = 10.0;
		else if (requestedCellHeight_um < 25.0) requestedCellHeight_um = 20.0;
		
		// if in mini-beam range, cell width and height must match 
		if ((requestedCellWidth_um > 20.0 && requestedCellHeight_um <= 20.0) ||
			(requestedCellWidth_um <= 20.0 && requestedCellHeight_um > 20.0) ||
			(requestedCellWidth_um <= 20.0 && requestedCellHeight_um <= 20.0 && Math.abs(requestedCellWidth_um-requestedCellHeight_um)>=0.001))
		{
			switch (cellSizeMethod)
			{
			case USE_WIDTH:
				requestedCellHeight_um = requestedCellWidth_um;
				break;
			case USE_HEIGHT:
				requestedCellWidth_um = requestedCellHeight_um;
				break;
			case USE_SMALLER:
				double smallerDim = Math.min(requestedCellWidth_um, requestedCellHeight_um);
				requestedCellWidth_um = smallerDim;
				requestedCellHeight_um = smallerDim;
				break;
			}
		}
		
		// constrain cell height to max
		Point2D maxBeam = ModelBeamsize.getActualBeamsize_um();
		if (requestedCellHeight_um > maxBeam.y) requestedCellHeight_um = maxBeam.y;
		
		return new Point2D(requestedCellWidth_um, requestedCellHeight_um);
	}
	
	public static Point2D getDefaultCellSize(double referenceWidth_um, double referenceHeight_um, CellSizeMethod cellSizeMethod)
	{
		Point2D maxBeam = ModelBeamsize.getActualBeamsize_um();

		// start with total width / 4
		double cellWidth_um = Math.round(Math.min(maxBeam.x, referenceWidth_um/ModelRasterGrid.DEFAULT_GRID_COLUMNS));
		if (referenceWidth_um <= 20.0) cellWidth_um = 5.0;

		// start with total height / 4
		double cellHeight_um = Math.round(Math.min(maxBeam.y, referenceHeight_um/ModelRasterGrid.DEFAULT_GRID_ROWS));

		// add beam size constraints
		Point2D validSize = getValidCellSize(cellWidth_um, cellHeight_um, cellSizeMethod);
		return validSize;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////// constructor //////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////
	
	private ModelRasterGrid grid;
	public CellCalculator(ModelRasterGrid grid) { this.grid = grid; }
}
