package gov.anl.gmca.raster.calc;

import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.raster.ModelContour;
import gov.anl.gmca.raster.ModelRasterGrid;
import gov.anl.gmca.util.Point2D;
import gov.anl.gmca.util.Point4D;

public class RasterGeometry
{
	private ModelRasterGrid grid;
	public RasterGeometry(ModelRasterGrid grid) { this.grid = grid; }
	
	protected static class Rectangle
	{
		public double xmin, ymin, xmax, ymax;
		public Rectangle(double xmin, double xmax, double ymin, double ymax)
		{
			this.xmin=xmin; this.xmax=xmax; this.ymin=ymin; this.ymax=ymax;
		}
		public Rectangle(Rectangle other) { this.xmin=other.xmin; this.xmax=other.xmax; this.ymin=other.ymin; this.ymax=other.ymax; }
		public Point2D size() { return new Point2D(xmax-xmin,ymax-ymin); }
	}

	public void saveOrigin()
	{
		grid.setCenter(new Point4D
		(
				DevPmacMotor.get("sample_x").getPosition(),
				DevPmacMotor.get("sample_y").getPosition(),
				DevPmacMotor.get("sample_z").getPosition(),
				DevPmacMotor.get("gonio_omega").getPosition()
		));
	}

	
	public static Rectangle getContainingRectangle(ModelContour contour)
	{
		Point2D p = contour.points.get(0);
		Rectangle cr = new Rectangle(p.x,p.x,p.y,p.y);
		for (Point2D point : contour.points)
		{
			if (point.x < cr.xmin) 	cr.xmin	= point.x;
			if (point.x > cr.xmax)	cr.xmax	= point.x;
			if (point.y < cr.ymin)	cr.ymin	= point.y;
			if (point.y > cr.ymax)	cr.ymax	= point.y;
		}
		
		return cr;
	}
	
	protected static Rectangle getContainingRectangle(Rectangle existingRect, Rectangle newRect)
	{
		if (null==existingRect) return new Rectangle(newRect);
		
		Rectangle ret = new Rectangle(existingRect);
		if (newRect.xmin < ret.xmin) ret.xmin = newRect.xmin;
		if (newRect.xmax > ret.xmax) ret.xmax = newRect.xmax;
		if (newRect.ymin < ret.ymin) ret.ymin = newRect.ymin;
		if (newRect.ymax > ret.ymax) ret.ymax = newRect.ymax;
		return ret;
	}

	public static Point4D getCurrentMotorPositions()
	{
		return new Point4D
		(
			DevPmacMotor.get("sample_x").getPosition(),
			DevPmacMotor.get("sample_y").getPosition(),
			DevPmacMotor.get("sample_z").getPosition(),
			DevPmacMotor.get("gonio_omega").getPosition()
		);
	}
}
