package gov.anl.gmca.raster.cam;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.ModelRasterCell;
import gov.anl.gmca.raster.ModelRasterGrid;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.cam.ModelRasterVideo.VideoState;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;

public class RasterMenu
{
	private MenuItem itemPnC, itemCenterCell, itemSaveSnapshot, itemCreateSite, itemCreateHit;
	private ModelRasterVideo model;
	private Menu menu;
	private int lastMouseX, lastMouseY;
	
	public RasterMenu(final ModelRasterVideo model)
	{
		this.model = model;

		model.cam.addMouseListener(new MouseListener()
		{
			@Override public void mouseDoubleClick(MouseEvent arg0) {}
			@Override public void mouseDown(MouseEvent me)
			{
				lastMouseX=me.x; lastMouseY=me.y;
			}
			@Override public void mouseUp(MouseEvent arg0) {}
		});
		
		menu = new Menu(GUI.getShell(), SWT.POP_UP);
		menu.addMenuListener(new MenuAdapter()
		{
			@Override public void menuShown(MenuEvent me)
			{
				onMenuOpen();
			}
		});
		
		model.llStateChange.addListener(new ListenerList.Listener<ModelRasterVideo>()
		{
			@Override public void handleEvent(ModelRasterVideo object, Object data)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					updateEnabled();
				}});
			}
		});
	}
	
	private void onMenuOpen()
	{
		if (null!=itemPnC && 			!itemPnC.isDisposed()) 			itemPnC.dispose();
		if (null!=itemCenterCell && 	!itemCenterCell.isDisposed()) 	itemCenterCell.dispose();
		if (null!=itemSaveSnapshot && 	!itemSaveSnapshot.isDisposed()) itemSaveSnapshot.dispose();
		if (null!=itemCreateSite && 	!itemCreateSite.isDisposed()) 	itemCreateSite.dispose();
		if (null!=itemCreateHit &&		!itemCreateHit.isDisposed())	itemCreateHit.dispose();
		
		boolean moveAllowed = AppState.isOperationAllowed(Operation.MOVE_SAMPLE);

		itemPnC = new MenuItem(menu, SWT.PUSH);
		itemPnC.setText("Center this point");
		itemPnC.addSelectionListener(new SelectionAdapter()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				model.cam.pointAndClick(lastMouseX,lastMouseY);
			}
		});
		itemPnC.setEnabled(moveAllowed);
		
		ModelRasterGrid currentGrid = ModelRasterState.get().getCurrentRunModel().grid();
		int cellIndex=-1;
		ModelRasterCell clickedCell = null;
	    for (ModelRasterCell cell : currentGrid.getGridCalc().getCellListCopy())
	    {
			Rectangle bounds = model.getCellScreenBounds(cell);
			if (null != bounds && bounds.contains(lastMouseX-model.cam.drawX, lastMouseY-model.cam.drawY))
			{
				cellIndex = cell.index;
				clickedCell = cell;
			}
	    }
	    ModelRasterCell _clickedCell = clickedCell;
	    if (cellIndex>=0)
	    {
			itemCenterCell = new MenuItem(menu, SWT.PUSH);
			itemCenterCell.setText("Center cell "+(cellIndex+1));
			itemCenterCell.addSelectionListener(new SelectionAdapter()
		    {
				@Override public void widgetSelected(SelectionEvent e)
				{
					if (null!=_clickedCell)
					{
						Log.note("Centering cell "+(_clickedCell.index+1));
						_clickedCell.center();
					}
				}
			});
			itemCenterCell.setEnabled(moveAllowed);
			
			itemCreateHit = new MenuItem(menu, SWT.PUSH);
			itemCreateHit.setText("Create site from cell "+(_clickedCell.index+1));
			itemCreateHit.addSelectionListener(new SelectionAdapter()
		    {
				@Override public void widgetSelected(SelectionEvent e)
				{
					_clickedCell.addSiteFromCell();
				}
			});
			itemCreateHit.setEnabled(moveAllowed);
	    }
		
		itemSaveSnapshot = new MenuItem(menu, SWT.PUSH);
		itemSaveSnapshot.setText("Save snapshot");
		itemSaveSnapshot.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				model.cam.saveScreenshot(null, true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		itemCreateSite = new MenuItem(menu, SWT.PUSH);
		itemCreateSite.setText("Interactive hit selection");
		itemCreateSite.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				model.changeState(VideoState.CREATE_SITE);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		itemCreateSite.setEnabled(moveAllowed);
	}
	
	public void updateEnabled()
	{
		boolean showMenu = 
		(
			model.getState() == VideoState.POINT_AND_CLICK ||
			model.getState() == VideoState.VIEW_IMAGES
		);
		setEnabled(showMenu);
	}
	
	public void setEnabled(boolean enable)
	{
		model.cam.setMenu(enable ? menu : null);
	}
}
