package gov.anl.gmca.raster.cam;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.raster.ModelContour;
import gov.anl.gmca.raster.ModelRasterGrid;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.cam.ModelRasterVideo.VideoState;
import gov.anl.gmca.raster.ui.TabRaster;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Point2D;
import gov.anl.gmca.util.Point4D;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.graphics.Point;

import java.util.ArrayList;

public class VCCreatePolygon
{
	public 		static enum 	ContourType 					{ POLYGON, RECTANGLE; 		}
	protected 	ContourType 	contourType 					= ContourType.POLYGON;
	public 		ContourType 	getContourType() 				{ return contourType; 		}
	
	protected	double			rectangleWidth_um;
	protected	double			rectangleHeight_um;
	protected	double			rectangleOffset_um;
	public		double			getRectangleWidth_um()			{ return rectangleWidth_um; }
	public		double			getRectangleHeight_um()			{ return rectangleHeight_um; }
	public		double			getRectangleOffset_um()			{ return rectangleOffset_um; }
	
	private		boolean			inMode							= false;
	
	protected static final double MIN_RECTANGLE_DIMENSION_UM = 1.0;
	protected static final int POLYGON_ARRAY_SIZE = 211;

	protected ModelContour currentContour = new ModelContour();
	public ModelContour getCurrentContour() { return currentContour; }
	
	protected ModelRasterVideo model;
	public DrawCreatePolygon draw;
	public VCCreatePolygon(ModelRasterVideo model)
	{
		this.model=model;
		
		model.llStateChange.addListener(new ListenerList.Listener<ModelRasterVideo>()
		{
			@Override public void handleEvent(final ModelRasterVideo vc, Object data)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					handleModelChange(vc);
				}});
			}
		});
	}

	public void handleModelChange(ModelRasterVideo mrv)
	{
		boolean enteringPolygonMode = mrv.getState()==VideoState.CREATE_CONTOUR;
		if (enteringPolygonMode && !inMode)
		{
			Log.note("Entering polygon creation mode");
			startCreation();
		}
		if (!enteringPolygonMode && inMode)
		{
			Log.note("Leaving polygon creation mode");
		}
		inMode = enteringPolygonMode;
	}
	
	private void startCreation()
	{
		contourType = ContourType.POLYGON;
		currentContour = new ModelContour();
		model.llStateChange.notifyListeners();
	}

	public void endPolygon()
	{
		currentContour = new ModelContour();
		TabRaster.get().cam.model.changeToDefaultState();
	}
	
	protected boolean arrayHasMoreSpace(int elementsRequested)
	{
		int spaceLeft = POLYGON_ARRAY_SIZE-1; // space for elements
		
		ArrayList<ModelContour> contourList = new ArrayList<ModelContour>(ModelRasterState.get().getCurrentRunModel().grid().getContours());
		spaceLeft -= contourList.size();
		for (ModelContour c : contourList)
		{
			if (null != c.points) spaceLeft -= c.points.size();
		}
		if (null != currentContour && null != currentContour.points) spaceLeft -= currentContour.points.size();
		
		return (spaceLeft >= elementsRequested);
	}
	
	public void setContourType(ContourType ct)
	{
		if (ct==ContourType.RECTANGLE && !arrayHasMoreSpace(6)) // 5 points + 1 new contour
		{
			Util.popupOk("Limit reached","You have reached the limit for adding new polygons");
			return;
		}
		
		contourType = ct;
		currentContour = new ModelContour();
		if (ct == ContourType.RECTANGLE)
		{
			rectangleWidth_um = ModelRasterState.get().getCurrentRunModel().grid().getCellWidth_um();
			rectangleHeight_um = ModelRasterState.get().getCurrentRunModel().grid().getCellHeight_um();
			rectangleOffset_um = 0.0;
			updateRectangleContour();
		}
		model.llStateChange.notifyListeners();
	}
	
	public void setRectangleWidth(double width_um)
	{
		if (contourType!=ContourType.RECTANGLE)
		{
			System.out.println("setRectangleWidth called with contour type="+contourType);
			return;
		}
		rectangleWidth_um=width_um;
		updateRectangleContour();
		model.llStateChange.notifyListeners();
	}
	
	public void setRectangleHeight(double height_um)
	{
		if (contourType!=ContourType.RECTANGLE)
		{
			System.out.println("setRectangleHeight called with contour type="+contourType);
			return;
		}
		rectangleHeight_um=height_um;
		updateRectangleContour();
		model.llStateChange.notifyListeners();
	}
	
	public void setRectangleOffset(double offset_um)
	{
		if (contourType!=ContourType.RECTANGLE)
		{
			System.out.println("setRectangleOffset called with contour type="+contourType);
			return;
		}
		rectangleOffset_um=offset_um;
		updateRectangleContour();
		model.llStateChange.notifyListeners();
	}
	
	protected void updateRectangleContour()
	{
		// get the grid center
		ModelRasterGrid currentGrid = ModelRasterState.get().getCurrentRunModel().grid();
		Point4D gridCenter = currentGrid.getCenter();
		
		// set the center if it isn't set
		if (null == gridCenter)
		{
			currentGrid.run().dev().setSavedZoom(DevPmacMotor.get("camera_zoom_2").getPosition(), false);
			currentGrid.run().dev().setCenter(model.cam.getImageMotorPos(), true);
			gridCenter = currentGrid.getCenter();
		}

		// calculate the rectangle corners
		double[] currentCenterOffset_um = getGridCenterPx(gridCenter);
		currentCenterOffset_um[0] = model.cam.px2mmH(currentCenterOffset_um[0])*1000.0;
		currentCenterOffset_um[1] = model.cam.px2mmH(currentCenterOffset_um[1])*1000.0;
		
		double xmin = -currentCenterOffset_um[0]-(rectangleWidth_um/2.0);
		double xmax = -currentCenterOffset_um[0]+(rectangleWidth_um/2.0);
		double ymin = -currentCenterOffset_um[1]-(rectangleHeight_um/2.0)-rectangleOffset_um;
		double ymax = -currentCenterOffset_um[1]+(rectangleHeight_um/2.0)-rectangleOffset_um;
		
		// create the rectangle contour
		currentContour = new ModelContour();
		currentContour.points.add(new Point2D(xmin,ymin));
		currentContour.points.add(new Point2D(xmax,ymin));
		currentContour.points.add(new Point2D(xmax,ymax));
		currentContour.points.add(new Point2D(xmin,ymax));
		currentContour.points.add(new Point2D(xmin,ymin));
	}
	
	// coordinates are saved as visual offsets in mm from the grid center
	public void addContourPoint(int xPixels, int yPixels)
	{
		if (ContourType.RECTANGLE == contourType)
		{
			Log.note("Points may not be added in rectangle mode");
			return;
		}
		
		if (!arrayHasMoreSpace(1))
		{
			Util.popupOk("Limit reached", "You have reached the limit of polygon points");
			return;
		}
		
		/*if (DBConfig.getBoolean("raster_single_arm") && currentContour.points.size()>=2)
		{
			Util.popupOk("Polygons not supported", "Due to a temporary limitation, only rectangular grids are supported with the Eiger.\nSelect two points to define the diagonal of the rectangle and click Done.");
			return;
			}*/
		
		// get the grid center
		ModelRasterGrid currentGrid = ModelRasterState.get().getCurrentRunModel().grid();
		Point4D gridCenter = currentGrid.getCenter();
		
		// set the center if it isn't set
		if (null == gridCenter)
		{
			currentGrid.run().dev().setSavedZoom(DevPmacMotor.get("camera_zoom_2").getPosition(), false);
			currentGrid.run().dev().setCenter(model.cam.getImageMotorPos(), true);
			gridCenter = currentGrid.getCenter();
		}
		
		// calculate the on-screen center
		double[] gridCenterPx = getGridCenterPx(gridCenter);
		
		// calculate the new point offset in terms of the grid center
		Point screenSize = new Point(model.cam.displayWidth,model.cam.displayHeight);
		int relXPixels = xPixels - screenSize.x/2;
		int relYPixels = yPixels - screenSize.y/2;
		double xMM = model.cam.px2mmH(relXPixels-gridCenterPx[0]);
		double yMM = model.cam.px2mmV(relYPixels-gridCenterPx[1]);
		
		// add the point in microns
		currentContour.points.add(new Point2D(xMM*1000.0, yMM*1000.0));
		
		// notify listeners
		model.llStateChange.notifyListeners();
	}
	
	protected double[] getGridCenterPx(Point4D gridCenter)
	{
		// get the stored motor positions for this frame
		Point4D mtrCurrentCenter = model.cam.getImageMotorPos();
		mtrCurrentCenter.omega = gridCenter.omega;

		// convert grid center and current motor center to screen coordinates
		Point2D scrGridCenter = model.cam.motorMMtoCurrentScreenPx(gridCenter);
		Point2D scrCurrentCenter = model.cam.motorMMtoCurrentScreenPx(mtrCurrentCenter);

		// return the difference
		double[] gridCenterPx = new double[]
		{
			scrGridCenter.x-scrCurrentCenter.x,
			scrGridCenter.y-scrCurrentCenter.y
        };
		return gridCenterPx;
	}
	
	public void completeContour()
	{
		ModelContour completedContour = currentContour;
		model.changeToDefaultState();
		currentContour = new ModelContour();
		
		if (ContourType.RECTANGLE == contourType)
		{
			ModelRasterState.get().getCurrentRunModel().grid().addContour(completedContour);
			model.llStateChange.notifyListeners();
		}
		else if (completedContour.points.size() == 1)
		{
			Log.note("One point was entered and expanded to a "+Util.printDouble(MIN_RECTANGLE_DIMENSION_UM,1)+" \u00B5m rectangle");
			
			Point2D p = completedContour.points.get(0);
			
			double xmin = p.x-(MIN_RECTANGLE_DIMENSION_UM/2.0);
			double xmax = p.x+(MIN_RECTANGLE_DIMENSION_UM/2.0);
			double ymin = p.y-(MIN_RECTANGLE_DIMENSION_UM/2.0);
			double ymax = p.y+(MIN_RECTANGLE_DIMENSION_UM/2.0);
			
			ModelContour rectangleContour = new ModelContour();
			rectangleContour.points.add(new Point2D(xmin,ymin));
			rectangleContour.points.add(new Point2D(xmax,ymin));
			rectangleContour.points.add(new Point2D(xmax,ymax));
			rectangleContour.points.add(new Point2D(xmin,ymax));
			
			ModelRasterState.get().getCurrentRunModel().grid().addContour(rectangleContour);
			model.llStateChange.notifyListeners();
		}
		else if (completedContour.points.size() == 2)
		{
			String msg = "Two points were entered and expanded to a rectangle.";
			Point2D p1 = completedContour.points.get(0);
			Point2D p2 = completedContour.points.get(1);
			
			double xmin=Math.min(p1.x,p2.x);
			double xmax=Math.max(p1.x,p2.x);
			double ymin=Math.min(p1.y,p2.y);
			double ymax=Math.max(p1.y,p2.y);
			
			Log.note("width="+(xmax-xmin)+" height="+(ymax-ymin));
			
			// ensure the rectangle has the minimum width/height
			boolean widthExpanded=false, heightExpanded=false;
			if ((xmax-xmin)<MIN_RECTANGLE_DIMENSION_UM)
			{
				double xmid = (xmin+xmax)/2.0;
				xmin=xmid-(MIN_RECTANGLE_DIMENSION_UM/2.0);
				xmax=xmid+(MIN_RECTANGLE_DIMENSION_UM/2.0);
				widthExpanded=true;
			}
			if ((ymax-ymin)<MIN_RECTANGLE_DIMENSION_UM)
			{
				double ymid = (ymin+ymax)/2.0;
				ymin=ymid-(MIN_RECTANGLE_DIMENSION_UM/2.0);
				ymax=ymid+(MIN_RECTANGLE_DIMENSION_UM/2.0);
				heightExpanded=true;
			}
			
			if (widthExpanded && !heightExpanded) msg += " (width ";
			else if (!widthExpanded && heightExpanded) msg += " (height ";
			else if (widthExpanded && heightExpanded) msg += "(width and height ";
			if (widthExpanded || heightExpanded) msg += "increased to "+Util.printDouble(MIN_RECTANGLE_DIMENSION_UM,1)+" \u00B5m)";
			Log.note(msg);
			
			ModelContour rectangleContour = new ModelContour();
			rectangleContour.points.add(new Point2D(xmin,ymin));
			rectangleContour.points.add(new Point2D(xmax,ymin));
			rectangleContour.points.add(new Point2D(xmax,ymax));
			rectangleContour.points.add(new Point2D(xmin,ymax));
			
			ModelRasterState.get().getCurrentRunModel().grid().addContour(rectangleContour);
			model.llStateChange.notifyListeners();
		}
		else if (completedContour.points.size() >= 3)
		{
			ModelRasterState.get().getCurrentRunModel().grid().addContour(completedContour);
			model.llStateChange.notifyListeners();
		}
		else
		{
			Log.warn("Polygon incomplete, will not be added.");
			model.llStateChange.notifyListeners();
		}
	}

	public void removeLastPoint()
	{
		if (0 == currentContour.points.size())
		{
			Log.note("No points to remove");
			return;
		}
		currentContour.points.remove(currentContour.points.size()-1);
		if (0 == currentContour.points.size())
		{
			model.changeToDefaultState();
		}
	}
}
