package gov.anl.gmca.raster.cam;

import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.WidgetFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgCreateSite extends Composite
{
	protected final static Color buttonBgColor = GUI.COLOR_LIGHT_GRAY;
	protected final static Color bgColor = GUI.COLOR_LIGHT_GRAY;
	protected final static Color fgColor = GUI.COLOR_BLACK;
	
	private final static String initialText = "Click create to begin";
	private final static String createText = "Left click/drag to add cells, right click/drag to remove";

	private Label labelMessage, label2, currentHitLabel, ofLabel;
	private Button btnCreate, btnDelete, btnDone, backButton, forwardButton;
	private Composite editComp, interiorComp;
	private ModelRasterVideo model;

	public WdgCreateSite(Composite parent, ModelRasterVideo model)
	{
		super(parent, SWT.NONE);
		
		this.model=model;
		final ModelRasterVideo fModel = model;
		
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth=1;
		gl.marginHeight=1;
		setLayout(gl);
		setBackground(GUI.COLOR_BLACK);
		
		interiorComp = new Composite(this, SWT.NONE);
		interiorComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		interiorComp.setBackground(GUI.COLOR_GRAY);
		gl = new GridLayout(4,false);
		gl.marginRight = 0;
		gl.marginBottom = 0;
		gl.horizontalSpacing = 0;
		gl.marginHeight=2;
		interiorComp.setLayout(gl);
		
		labelMessage = WidgetFactory.newLabel(interiorComp, SWT.NONE, "");
		labelMessage.setBackground(GUI.COLOR_GRAY);
		labelMessage.setForeground(fgColor);
		labelMessage.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		
		btnCreate = WidgetFactory.newButton(interiorComp, SWT.PUSH, "Create");
		btnCreate.setBackground(bgColor);
		WidgetFactory.setForeground(btnCreate, fgColor, GUI.COLOR_BLACK);
		btnCreate.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 2));
		btnCreate.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				fModel.vcCreateSite.handleCreateButton();
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		btnDelete = WidgetFactory.newButton(interiorComp, SWT.PUSH, "Delete");
		//btnDelete.setBackground(buttonBgColor);
		btnDelete.setBackground(bgColor);
		WidgetFactory.setForeground(btnDelete, fgColor, GUI.COLOR_BLACK);
		btnDelete.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 2));
		btnDelete.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				fModel.vcCreateSite.deleteHit();
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		btnDone = WidgetFactory.newButton(interiorComp, SWT.PUSH, "Done");
		//btnDone.setBackground(buttonBgColor);
		btnDone.setBackground(bgColor);
		WidgetFactory.setForeground(btnDone, fgColor, GUI.COLOR_BLACK);
		btnDone.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 2));
		btnDone.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				fModel.vcCreateSite.handleDone();
			}
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
		});
		
		editComp = new Composite(interiorComp, SWT.NONE);
		editComp.setBackground(GUI.COLOR_GRAY);
		editComp.setForeground(fgColor);
		gl = new GridLayout(5, false);
		gl.marginWidth=0;
		gl.marginHeight=0;
		editComp.setLayout(gl);
		editComp.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		
		label2 = WidgetFactory.newLabel(editComp, SWT.NONE, "Currently editing hit ");
		label2.setBackground(GUI.COLOR_GRAY);
		label2.setForeground(fgColor);
		
		backButton = new Button(editComp, SWT.PUSH);
		backButton.setBackground(bgColor);
		WidgetFactory.setForeground(backButton, fgColor, GUI.COLOR_BLACK);
		backButton.setText("<<");
		backButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				fModel.vcCreateSite.goToPrevHit();
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		currentHitLabel = WidgetFactory.newLabel(editComp, SWT.NONE, " ? ");
		currentHitLabel.setBackground(GUI.COLOR_GRAY);
		currentHitLabel.setForeground(fgColor);
		
		forwardButton = new Button(editComp, SWT.PUSH);
		forwardButton.setBackground(bgColor);
		WidgetFactory.setForeground(forwardButton, fgColor, GUI.COLOR_BLACK);
		forwardButton.setText(">>");
		forwardButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				fModel.vcCreateSite.goToNextHit();
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
		
		ofLabel = WidgetFactory.newLabel(editComp, SWT.NONE, " of ?");
		ofLabel.setBackground(GUI.COLOR_GRAY);
		ofLabel.setForeground(fgColor);
		
		updateWidgets();
		
		model.llStateChange.addListener(new ListenerList.Listener<ModelRasterVideo>()
		{
			@Override public void handleEvent(ModelRasterVideo object, Object data)
			{
				updateWidgets();
			}
		});
	}
	
	private void updateWidgets()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			int currentHit=model.vcCreateSite.getCurrentHit();
			int hitCount=model.vcCreateSite.getHitCount();
			
			if (0==currentHit)
			{
				labelMessage.setText(initialText);
				
				WdgControl.setControlVisible(editComp, false);
				((GridData)btnDelete.getLayoutData()).verticalSpan=1;
				((GridData)btnCreate.getLayoutData()).verticalSpan=1;
				((GridData)btnDone.getLayoutData()).verticalSpan=1;
				getParent().getParent().layout(true);
				
				btnDelete.setEnabled(false);
				btnCreate.setEnabled(true);
				btnDone.setEnabled(true);
				btnDone.setText("Cancel");
				backButton.setEnabled(false);
				forwardButton.setEnabled(false);
			}
			else
			{
				labelMessage.setText(createText);
				
				currentHitLabel.setText(" "+currentHit+" ");
				ofLabel.setText(" of "+hitCount);
				
				WdgControl.setControlVisible(editComp, true);
				((GridData)btnDelete.getLayoutData()).verticalSpan=2;
				((GridData)btnCreate.getLayoutData()).verticalSpan=2;
				((GridData)btnDone.getLayoutData()).verticalSpan=2;
				getParent().getParent().layout(true);
				
				btnDelete.setEnabled(true);
				btnCreate.setEnabled(true);
				btnDone.setEnabled(true);
				btnDone.setText("Done");
				backButton.setEnabled(currentHit!=1);
				forwardButton.setEnabled(currentHit<hitCount);
			}
		}});
	}
}
