package gov.anl.gmca.raster.cam;

import gov.anl.gmca.common.ui.GButton;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgMaximize;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.ModelRasterRun.RunType;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.ModelRasterState.FluoDataParameter;
import gov.anl.gmca.raster.ModelRasterState.NumberingParameter;
import gov.anl.gmca.raster.ui.TabRaster;
import gov.anl.gmca.spotfinder.DevSFType;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_OUTPUT;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_TYPE;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import java.util.ArrayList;
import java.util.HashMap;

public class WdgRasterToolbar extends Composite
{
	private WdgRasterCamera cam;
	private GridLayout glToolbar;
	private WdgMaximize wdgMaximize;
	
	private static enum HeatMapColoringSONICC { None, SONICC };
	private Boolean lastShowDiff = null;
	private SPOTFINDER_TYPE lastSFType = null;
	private ArrayList<SPOTFINDER_OUTPUT> coloringOptions;
	private HashMap<NumberingParameter,GButton> numberMap = new HashMap<>();
	private HashMap<SPOTFINDER_OUTPUT,GButton> coloringMapDiff = new HashMap<>();
	private HashMap<HeatMapColoringSONICC,GButton> coloringMapFluo = new HashMap<>();
	
	public WdgRasterToolbar(Composite parent)
	{
		super(parent, SWT.NONE);
		//setBackground(GUI.COLOR_DARK_GRAY5);

		coloringOptions = new ArrayList<>();
		for (SPOTFINDER_OUTPUT output : SPOTFINDER_OUTPUT.values()) coloringOptions.add(output);
		coloringOptions.add(null);

		glToolbar = new GridLayout(13,false);
		glToolbar.marginHeight = 0;
		glToolbar.horizontalSpacing = 2;
		glToolbar.marginWidth = 0;
		setLayout(glToolbar);

		GButton b1 = new GButton(this, SWT.NONE);
		b1.setText("Numbering: ");
		for (NumberingParameter nm : NumberingParameter.values())
		{
			GButton b = new GButton(this, SWT.BORDER);
			b.setText(Util.capitalizeFirstLetters(nm.toString()));
			numberMap.put(nm, b);
			b.llSelected.addListener(new ListenerList.Listener<GButton>()
			{
				@Override public void handleEvent(GButton object, Object data)
				{
					handleNumberingChange(nm);
				}
			});
		}
		handleNumberingChange(NumberingParameter.CELL);
		
		Label l = new Label(this, SWT.NONE);
		l.setText("        ");
		
		GButton b2 = new GButton(this, SWT.NONE);
		b2.setText("Coloring: ");
		for (SPOTFINDER_OUTPUT output : coloringOptions)
		{
			GButton b = addColoringButton(null!=output?output.shortName:"None");
			coloringMapDiff.put(output, b);
			b.llSelected.addListener((object,data)->handleDiffColoringChange(output, true));
		}
		handleDiffColoringChange(DevSFType.getConfig(SFRunType.RASTER).id.defaultOutput, true);

		for (HeatMapColoringSONICC hm : HeatMapColoringSONICC.values())
		{
			GButton b = addColoringButton(hm.toString());
			coloringMapFluo.put(hm, b);
			b.llSelected.addListener((object,data)-> handleSONICCColoringChange(hm));
		}
		handleSONICCColoringChange(HeatMapColoringSONICC.SONICC);
		setDiffParamsVisible();
		
		Composite spacer = new Composite(this, SWT.NONE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 0;
		spacer.setLayoutData(gd);
		
		wdgMaximize = new WdgMaximize(this, SWT.NONE);
		wdgMaximize.setToolTipText("Maximize raster video");
		gd = new GridData();
		wdgMaximize.setLayoutData(gd);
	}
	
	private GButton addColoringButton(String text)
	{
		GButton b = new GButton(this, SWT.BORDER);
		b.setText(text);
		return b;
	}
	
	public void connectListeners(WdgRasterCamera cam)
	{
		this.cam=cam;
		
		DevSFType.get(SFRunType.RASTER).llChange.addListener(new ListenerList.Listener<DevSFType>()
		{
			@Override public void handleEvent(DevSFType object, Object data)
			{
				setDiffParamsVisible();
			}
		});
		
		// raster state change
		ModelRasterState.get().llCurrentRun.addListener(new ListenerList.Listener<ModelRasterState>()
		{
			@Override public void handleEvent(ModelRasterState state, Object data)
			{
				System.out.println("Raster state change listener in toolbar");
				setDiffParamsVisible();
				GUI.asyncExec(()->handleDiffColoringChange(state.getDiffDisplayParameter(), false));
			}
		});
		
		// maximize
		wdgMaximize.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				boolean isMaximized = TabRaster.get().getCameraMaximized();
				
				TabRaster.get().setCameraMaximized(!isMaximized);
				isMaximized = !isMaximized;
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	}
	
	private void setDiffParamsVisible()
	{
		GUI.asyncExec(new Runnable() {public void run()
		{
			// return if no relevant changes
			Boolean showDiff = RunType.DIFFRACTION == ModelRasterState.get().getCurrentRunModel().getRunType();
			SPOTFINDER_TYPE currentSF = DevSFType.getConfig(SFRunType.RASTER).id;
			boolean typeChange = showDiff!=lastShowDiff;
			if (showDiff==lastShowDiff && currentSF==lastSFType) return;
			lastShowDiff=showDiff; lastSFType=currentSF;

			if (typeChange) {
				if (showDiff) handleNumberingChange(NumberingParameter.CELL);
				else          handleNumberingChange(NumberingParameter.NONE);
			}

			// update visible widgets
			for (SPOTFINDER_OUTPUT output : coloringOptions)
			{
				GButton b = coloringMapDiff.get(output);
				boolean showButton = showDiff;
				if (null!=output) {
					SPOTFINDER_TYPE sfType = output.sf_type;
					if (sfType != currentSF) showButton = false;
				}
				WdgControl.setControlVisible(b, showButton);
			}
			for (HeatMapColoringSONICC h : HeatMapColoringSONICC.values())
			{
				GButton b = coloringMapFluo.get(h);
				WdgControl.setControlVisible(b, !showDiff);
			}
			
			// update layout
			glToolbar.numColumns = showDiff ? (8+DevSFType.getConfig(SFRunType.RASTER).id.getOutputs().size()+1) : (8+ HeatMapColoringSONICC.values().length);
			layout(true);

		}});
	}
	
	private void handleNumberingChange(NumberingParameter np)
	{
		for (NumberingParameter m : NumberingParameter.values())
		{
			GButton b = numberMap.get(m);
			setColors(b, m==np);
			WdgControl.setControlVisible(b, m==NumberingParameter.NONE||null==lastShowDiff||lastShowDiff);
		}
		
		ModelRasterState.get().setNumberingParameter(np);
	}
	
	private void handleDiffColoringChange(SPOTFINDER_OUTPUT selectedOutput, boolean update)
	{
		for (SPOTFINDER_OUTPUT output : coloringOptions)
		{
			boolean colorOutput = output==selectedOutput;
			GButton b = coloringMapDiff.get(output);
			setColors(b, colorOutput);
		}
		if (update) ModelRasterState.get().setDiffDisplayParameter(selectedOutput);
	}
	
	private void handleSONICCColoringChange(HeatMapColoringSONICC hm)
	{
		for (HeatMapColoringSONICC h : HeatMapColoringSONICC.values())
		{
			GButton b = coloringMapFluo.get(h);
			setColors(b, h==hm);
		}
		ModelRasterState.get().setSONICCDisplay(hm == HeatMapColoringSONICC.SONICC? ModelRasterState.SONICCDisplay.SONICC: ModelRasterState.SONICCDisplay.NONE);
	}
	
	private void setColors(GButton b, boolean selected)
	{
		b.setInteriorBackground(b.getBackground());
		b.setForeground(GUI.COLOR_BLACK);
		b.setBorderColor(GUI.COLOR_SWT_BORDER);
		b.setUnderline(selected);
	}
}
