package gov.anl.gmca.raster.cam;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.ui.CButton;
import gov.anl.gmca.common.ui.WdgSegmentedToggle;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.cam.VCViewImages.ViewMode;
import gov.anl.gmca.sample.TabSample;
import gov.anl.gmca.spotfinder.SFData.SFType;
import gov.anl.gmca.spotfinder.SFLog;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.ListenerList.Listener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

public class WdgViewImages extends Composite
{
	private Composite labelComp;
	private ModelRasterVideo model;
	private Button btnSpreadsheet, btnViewer, plus90Button, minus90Button;
	private WdgSegmentedToggle modeToggle;
	private Label label;
	private Shell browserShell = null;

	public WdgViewImages(Composite parent, ModelRasterVideo model)
	{
		super(parent, SWT.NONE);
		this.model=model;
		
		createWidgets(parent);
		connectListeners();
	}
	
	private void connectListeners()
	{
		ModelRasterState.get().llCurrentRun.addListener(new ListenerList.Listener<ModelRasterState>()
		{
			@Override public void handleEvent(ModelRasterState object, Object data)
			{
				updateWidgets();
			}
		});
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				updateWidgets();
			}
		});
		modeToggle.llClicked.addListener(new Listener<WdgSegmentedToggle>()
		{
			@Override public void handleEvent(WdgSegmentedToggle object, Object data)
			{
				int selectedIndex = modeToggle.getSelectedIndex();
				model.vcViewImages.setMode(ViewMode.values()[selectedIndex], true);
			}
		});
		model.vcViewImages.llViewMode.addListener(new Listener<VCViewImages>()
		{
			@Override public void handleEvent(VCViewImages object, Object data)
			{
				updateWidgets();
			}
		});
		AppState.addListener(Operation.MOVE_SAMPLE, new AppState.Listener()
		{
			@Override public void stateChanged(Operation op, boolean allowed)
			{
				updateWidgets();
			}
		});
		updateWidgets();
	}
	
	private int lastContourCount=-1;
	private ViewMode lastMode=null;
	private boolean lastIsMaster=false;
	private boolean lastEnableRotate=true;
	private void updateWidgets()
	{
		ViewMode mode = (ViewMode)model.vcViewImages.getMode();
		if (mode==null) return;
		ModelRasterRun run = ModelRasterState.get().getCurrentRunModel();
		final int contourCount = run.grid().getContours().size();
		boolean isMaster = Client.isMaster();
		boolean enableRotate = AppState.isOperationAllowed(Operation.MOVE_SAMPLE);
		if (lastContourCount!=contourCount || lastMode!=mode || lastIsMaster!=isMaster || lastEnableRotate != enableRotate)
		{
			lastContourCount = contourCount;
			lastMode = mode;
			lastIsMaster = isMaster;
			lastEnableRotate = enableRotate;
			GUI.asyncExec(new Runnable(){public void run()
			{
				Color bgColor = mode==ViewMode.MOVE?GUI.COLOR_LIGHT_RED:GUI.COLOR_LIGHT_GREEN;
				labelComp.setBackground(bgColor);
				label.setBackground(bgColor);
				
				String lblText = "";
				if (mode==ViewMode.VIEW)
				{
					if (isMaster)
					{
						lblText = contourCount>0
						?"Click or drag to view cell images"
						:"Click + on a raster run to start";
					}
					else lblText = "Become master to view images";
				}
				else
				{
					if (isMaster)
						lblText = "Click to move the sample";
					else
						lblText = "Become master to move the sample";
				}
				
	    		label.setText("   "+lblText);
	    		label.setForeground(GUI.COLOR_BLACK);
	    		labelComp.layout(true);
	    		modeToggle.setSelection(mode.ordinal());
	    		
	    		plus90Button.setEnabled(enableRotate);
	    		minus90Button.setEnabled(enableRotate);
			}});
		}
	}
	
	private void createWidgets(Composite parent)
	{
		setBackground(GUI.COLOR_BLACK);
		
		GridLayout gl = new GridLayout(2, false);
		gl.marginWidth=1;
		gl.marginHeight=1;
		gl.horizontalSpacing=0;
		setLayout(gl);
		
		labelComp = new Composite(this, SWT.NONE);
		labelComp.setBackground(GUI.COLOR_DARK_GREEN);
		gl = new GridLayout(7,false);
		gl.marginWidth=5;
		gl.marginHeight=2;
		labelComp.setLayout(gl);
		labelComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		modeToggle = new WdgSegmentedToggle(labelComp);
		modeToggle.setText("Move","View");
		modeToggle.setLayoutData(new GridData(SWT.LEFT, SWT.DEFAULT, false, false));
		modeToggle.setSelection(0);
		
		label = new Label(labelComp, SWT.LEFT);
		label.setBackground(GUI.COLOR_DARK_GREEN);
		label.setForeground(GUI.COLOR_WHITE);
		label.setText(" ");
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));

		plus90Button = new Button(labelComp, SWT.PUSH);
		plus90Button.setText("+90");
		plus90Button.setBackground(GUI.COLOR_BLUE_BUTTON);
		GridData gd = new GridData();
		gd.heightHint = 27;
		gd.widthHint = 45;
		plus90Button.setLayoutData(gd);
		plus90Button.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				DevPmacMotor.get("gonio_omega").startRelativeMove(90.0);
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0){}
		});
		plus90Button.setToolTipText("Rotate sample +90 degrees");
		
		minus90Button = new Button(labelComp, SWT.PUSH);
		minus90Button.setText("-90");
		minus90Button.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData();
		gd.heightHint = 27;
		gd.widthHint = 45;
		minus90Button.setLayoutData(gd);
		minus90Button.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				DevPmacMotor.get("gonio_omega").startRelativeMove(-90.0);
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0){}
		});
		minus90Button.setToolTipText("Rotate sample -90 degrees");
		
		CButton spreadsheetButton = new CButton(labelComp);
		spreadsheetButton.setImage(GUI.IMAGE_SPREADSHEET);
		spreadsheetButton.ll.addListener(new ListenerList.Listener<CButton>()
		{
			@Override public void handleEvent(CButton object, Object data)
			{
				SFType sfType;
				switch (ModelRasterState.get().getAutoMode())
				{
				case AUTO: sfType = SFType.AUTO_RASTER; break;
				case INTER: sfType = SFType.INTER_RASTER; break;
				default: throw new IllegalStateException("Unknown state");
				}
				SFLog.openLog(sfType, ModelRasterState.get().getCurrentRunModel().getRunIndex());
			}
		});
		spreadsheetButton.setToolTipText("View results in a spreadsheet");
		
		CButton viewerButton = new CButton(labelComp);
		viewerButton.setImage(GUI.IMAGE_DIFFRACTION);
		viewerButton.ll.addListener(new ListenerList.Listener<CButton>()
		{
			@Override public void handleEvent(CButton object, Object data)
			{
				TabSample.get().get(0).createSamplePeelout();
				TabSample.get().get(1).allCams.showDiffViewer();
			}
		});
		viewerButton.setToolTipText("Open diffraction viewer");
		
		CButton helpButton = new CButton(labelComp);
		helpButton.setImage(GUI.IMAGE_KEYBOARD);
		helpButton.ll.addListener(new ListenerList.Listener<CButton>()
		{
			@Override public void handleEvent(CButton object, Object data)
			{
				String url = Main.getAppDir()+"/data/raster_help.html";
				//MenuActions.openBrowser(url);
				
				// create browser shell
				boolean createNewShell = (null==browserShell || browserShell.isDisposed());
				if (createNewShell)
				{
					browserShell = new Shell(GUI.getShell(), SWT.DIALOG_TRIM);
					browserShell.setLayout(new FillLayout());
				}
				
				// set browser size
				Point browserSize = new Point(600, 800);
				browserShell.setSize(browserSize);
				
				// center browser on JBluIce
				Point bluiceLoc = GUI.getShell().getLocation();
				Point bluiceSize = GUI.getShell().getSize();
				Point sizeDelta = new Point(bluiceSize.x-browserSize.x,bluiceSize.y-browserSize.y);
				Point browserLoc = new Point(bluiceLoc.x + sizeDelta.x/2, bluiceLoc.y + sizeDelta.y/2);
				browserShell.setLocation(browserLoc);
				
				// create the browser
				if (createNewShell)
				{
					Browser browser = new Browser(browserShell, SWT.NONE);
					browser.setUrl(url);
					browserShell.open();
				}
			}
		});
		helpButton.setToolTipText("Show keyboard and mouse help");
	}		
}
