package gov.anl.gmca.raster.ui;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgRunTabs;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.DBRasterHits;
import gov.anl.gmca.raster.DevRasterRunList;
import gov.anl.gmca.raster.ModelRasterRunList;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.cam.*;
import gov.anl.gmca.raster.cam.ModelRasterVideo.VideoState;
import gov.anl.gmca.sample.DBCamera;
import gov.anl.gmca.sample.WdgCamera;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.WidgetFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

public class TabRaster extends Tab
{
	protected static TabRaster inst = null;
	public static TabRaster get() { return inst; }
	
	// three widgets that make up the raster tab
	public 		WdgRunTabs 			runTabs;
	private		WdgRasterModeBar	modeBar;
	protected 	WdgCreatePolygon 	createPolygonBar;
	protected	WdgCreateSite		createSiteBar;
	protected	WdgViewImages		viewImagesBar;
	protected 	WdgRasterButtons 	controlButtons;
	protected 	WdgRasterOptions 	rasterOptions;
	protected 	WdgDiffViewButtons 	diffViewButtons;
	public		WdgRasterCamera 	cam;
	private		Composite			leftComp;
	private		CTabFolder			lowerTabs;
	public		WdgCellInfo			cellInfo;
	public		WdgRasterResultsTable resultsTable;
	private		Point				mouseDragStart = null;
	private		int					dragHeightStart = 300;
				SashForm			rightForm, lowerForm;
	
	public TabRaster(Composite parent, int style)
	{
		super(parent,style);
		inst = this;

		Log.note("TabRaster: start");
		
		GridLayout gl = new GridLayout(2, false);
		setLayout(gl);
		
		setBackground(GUI.BGCOLOR);
		
		GridData gd;
		
		leftComp = new Composite(this, SWT.NONE);
		leftComp.setBackground(GUI.BGCOLOR);
		GridLayout leftCompLayout = new GridLayout(1,false);
		leftCompLayout.marginWidth=0;
		leftCompLayout.marginHeight=0;
		leftComp.setLayout(leftCompLayout);
		gd = new GridData(GridData.FILL_VERTICAL);
		gd.widthHint = 360;
		leftComp.setLayoutData(gd);
		
		Composite commonControlsComp = setupCommonControls(leftComp);
		commonControlsComp.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));

		runTabs = new WdgRunTabs(leftComp, SWT.LEFT, ModelRasterRunList.get(ModelRasterState.RasterListType.INTER),
				DevRasterRunList.get(ModelRasterState.RasterListType.INTER), new WdgRunTabInter.Factory());
		runTabs.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		rightForm = new SashForm(this, SWT.VERTICAL|SWT.SMOOTH);
		rightForm.setBackground(GUI.BGCOLOR);
		rightForm.setLayoutData(new GridData(GridData.FILL_BOTH));
		rightForm.setSashWidth(8);
		
		Composite cameraComp = new Composite(rightForm, SWT.BORDER);
		//cameraComp.setLayoutData(new GridData(GridData.FILL_BOTH));
		gl = new GridLayout(1,false);
		gl.marginWidth=gl.marginHeight=gl.verticalSpacing=gl.horizontalSpacing=0;
		cameraComp.setLayout(gl);
		
		WdgRasterToolbar toolbar = new WdgRasterToolbar(cameraComp);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		toolbar.setLayoutData(gd);
		
		Composite divider = new Composite(cameraComp, SWT.NONE);
		divider.setBackground(GUI.COLOR_SWT_BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint=1;
		divider.setLayoutData(gd);
		
		cam = new WdgRasterCamera(cameraComp, SWT.NONE, DBCamera.getImageURL(WdgCamera.CameraType.HIGHRES, WdgCamera.CameraSize.LARGE), WdgCamera.CameraType.HIGHRES, VideoState.VIEW_IMAGES);
		cam.setBackground(GUI.BGCOLOR);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		cam.setLayoutData(gd);
		
		toolbar.connectListeners(cam);
		
		modeBar = new WdgRasterModeBar(cameraComp, cam.model);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		modeBar.setLayoutData(gd);
		
		lowerForm = new SashForm(rightForm, SWT.HORIZONTAL|SWT.SMOOTH);
		lowerForm.setBackground(GUI.BGCOLOR);
		lowerForm.setSashWidth(8);
		//lowerForm.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		lowerTabs = new CTabFolder(lowerForm, SWT.TOP|SWT.BORDER);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		lowerTabs.setLayoutData(gd);
		gl = new GridLayout(1,false);
		gl.marginWidth=gl.marginHeight=0;
		lowerTabs.setLayout(gl);

		resultsTable = new WdgRasterResultsTable(lowerTabs);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		resultsTable.setLayoutData(gd);
		CTabItem tab0 = new CTabItem(lowerTabs, SWT.NONE);
		tab0.setText("  SpotFinder Results  ");
		tab0.setControl(resultsTable);
		
		cellInfo = new WdgCellInfo(lowerTabs, cam);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		cellInfo.setLayoutData(gd);
		CTabItem tab1 = new CTabItem(lowerTabs, SWT.NONE);
		tab1.setText("  SpotFinder Output  ");
		tab1.setControl(cellInfo);
		
		lowerTabs.setSelection(0);
		
		WdgRasterSiteContainer siteList = new WdgRasterSiteContainer(lowerForm, SWT.BORDER);
		
		rightForm.setWeights(new int[]{67,33});
		lowerForm.setWeights(new int[]{67,33});
		
		Log.note("TabRaster: done");
	}
	
	private boolean isCameraMaximized = false;
	public boolean getCameraMaximized() { return isCameraMaximized; }
	public void setCameraMaximized(boolean maximize)
	{
		WdgControl.setControlVisible(leftComp, !maximize);
		layout(true);
		
		isCameraMaximized = maximize;
	}
	
	protected Composite setupCommonControls(Composite parent)
	{
		Composite commonControlsComp = new Composite(parent,SWT.NONE);
		commonControlsComp.setBackground(GUI.BGCOLOR);
		commonControlsComp.setLayout(WidgetFactory.newGridLayout(2, false, 0, 0));
		commonControlsComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		GridData gd;
		
		controlButtons = new WdgRasterButtons(commonControlsComp, SWT.NONE);
		gd = new GridData(GridData.FILL_BOTH);
		gd.widthHint = 90;
		controlButtons.setLayoutData(gd);
		
		rasterOptions = new WdgRasterOptions(commonControlsComp);
		gd = new GridData(GridData.FILL_BOTH);
		gd.widthHint = 110;
		//gd.verticalSpan = 2;
		rasterOptions.setLayoutData(gd);
		
		/*diffViewButtons = new WdgDiffViewButtons(commonControlsComp);
		gd = new GridData(GridData.FILL_BOTH);
		gd.widthHint = 100;
		diffViewButtons.setLayoutData(gd);*/

		return commonControlsComp;
	}
	
	// called during app startup to create run tabs
	public void createTabs()
	{
		new DBRasterHits();
	}
	
	@Override
	public void notifyIsVisible(boolean isVisible)
	{
		cam.notifyIsVisible(isVisible);
	}
	
	@Override
	public void connectListeners()
	{
		runTabs.connectListeners();
		controlButtons.connectListeners();
		//diffViewButtons.connectListeners();
		rasterOptions.connectListeners();
		resultsTable.connectListeners();
		cam.model.vcViewImages.connectListeners();

		AppState.addListener(AppState.Operation.START_RASTER, new AppState.Listener()
		{
			@Override public void stateChanged(AppState.Operation op, boolean allowed)
			{
				updateEnableTabs();
			}
		});

		ModelRasterState.get().llCurrentRun.addListener(new ListenerList.Listener<ModelRasterState>()
		{
			@Override public void handleEvent(final ModelRasterState state, Object data)
			{
				updateEnableTabs();
			}
		});

		ModelUserTable.Client.addListener(new ModelUserTable.ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				updateEnableTabs();
			}
		});

		TabRaster.get().cam.model.llStateChange.addListener(new ListenerList.Listener<ModelRasterVideo>()
		{
			@Override public void handleEvent(ModelRasterVideo model, Object data)
			{
				updateEnableTabs();
			}
		});
	}

	protected void updateEnableTabs()
	{
		VideoState state = TabRaster.get().cam.model.getState();
		final boolean enableRunTabs =
		(
			AppState.isOperationAllowed(AppState.Operation.START_RASTER) &&
				ModelRasterState.get().getRasterState() == ModelRasterState.RasterState.INACTIVE &&
				ModelUserTable.Client.isMaster() &&
				(state == VideoState.POINT_AND_CLICK || state == VideoState.VIEW_IMAGES)
		);

		GUI.asyncExec(new Runnable(){public void run()
		{
			runTabs.setEnabled(enableRunTabs);
		}});
	}
	
	@Override public String getTabName() { return "Raster"; }

	// called from GUI thread
	public void setViewerMode(boolean viewMode)
	{
		final GridData leftCompGD = (GridData)leftComp.getLayoutData();
		
		if (viewMode)
		{
			leftCompGD.widthHint = 0;
			layout(true);
		}
		else
		{
			leftCompGD.widthHint = 350;
			layout(true);
		}
	}
}
