package gov.anl.gmca.raster.ui;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.raster.sonicc.CmdBinSONICC;
import gov.anl.gmca.raster.DevRasterRun;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterRun.RunType;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

public class WdgControlMode extends WdgControl implements SelectionListener
{
	protected Button modeDiffractionRadio, modeSONICCRadio, modeFluoRadio;
	protected ModelRasterRun run;
	protected DevRasterRun dev;

	public WdgControlMode(Composite parent, int style, final ModelRasterRun run, DevRasterRun _dev)
	{
		super(parent, style, null);
		setLayout(new FillLayout());
		this.run=run;
		this.dev=_dev;
		
		Group rasterModeGroup = new Group(this,SWT.NONE);
		rasterModeGroup.setBackground(GUI.BGCOLOR);
		rasterModeGroup.setText(" Raster mode ");

		GridLayout gl = new GridLayout(3,false);
		rasterModeGroup.setLayout(gl);
		
		modeDiffractionRadio = new Button(rasterModeGroup,SWT.RADIO);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		modeDiffractionRadio.setLayoutData(gd);
		modeDiffractionRadio.setText("Diffraction");
		modeDiffractionRadio.setBackground(GUI.BGCOLOR);
		modeDiffractionRadio.addSelectionListener(this);

		modeFluoRadio = new Button(rasterModeGroup,SWT.RADIO);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		modeFluoRadio.setLayoutData(gd);
		modeFluoRadio.setText("Fluor.");
		modeFluoRadio.setBackground(GUI.BGCOLOR);
		modeFluoRadio.addSelectionListener(this);

		modeSONICCRadio = new Button(rasterModeGroup,SWT.RADIO);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		modeSONICCRadio.setLayoutData(gd);
		modeSONICCRadio.setText("SONICC");
		modeSONICCRadio.setBackground(GUI.BGCOLOR);
		modeSONICCRadio.addSelectionListener(this);
		
		modeDiffractionRadio.setSelection(RunType.DIFFRACTION == run.getRunType());
		modeFluoRadio.setSelection(RunType.FLUO == run.getRunType());
		modeSONICCRadio.setSelection(RunType.SONICC == run.getRunType());

		setEnabled(false);
		
		final ListenerList.Listener<ModelRasterRun> runListener = new ListenerList.Listener<ModelRasterRun>()
		{
			@Override public void handleEvent(ModelRasterRun object, Object data)
			{
				handleRunChange();
			}
		};
		run.llRunChanged.addListener(runListener);
		
		addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent e)
			{
				run.llRunChanged.removeListener(runListener);
			}
		});
	}
	
	@Override
	public void setEnabled(boolean enable)
	{
		boolean allowSONICC = DBConfig.getBoolean("sonicc_enable") || Main.SONICC_TEST_MODE;

		modeDiffractionRadio.setEnabled(enable);
		modeFluoRadio.setEnabled(false);
		modeSONICCRadio.setEnabled(enable && allowSONICC);
	}

	// device reports a change
	private void handleRunChange()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
    		switch (run.getRunType())
    		{
    		case DIFFRACTION:
    			modeDiffractionRadio.setSelection(true);
				modeFluoRadio.setSelection(false);
    			modeSONICCRadio.setSelection(false);
    			break;

			case FLUO:
				modeDiffractionRadio.setSelection(false);
				modeFluoRadio.setSelection(true);
				modeSONICCRadio.setSelection(false);
				break;

			case SONICC:
    			modeDiffractionRadio.setSelection(false);
				modeFluoRadio.setSelection(false);
    			modeSONICCRadio.setSelection(true);
    			break;
    		}
		}});
	}

	// radio button was clicked
	@Override
	public void widgetSelected(SelectionEvent se)
	{
		RunType newType = null;
		
		// Two events are sent on each click, one for the deselected radio button and
		// one for the selected one.  Ignore the deselection event.
		if (se.getSource() instanceof Button && !((Button)se.getSource()).getSelection()) return;
		
		if (se.getSource() == modeDiffractionRadio)
		{
			newType = RunType.DIFFRACTION;
		}
		else if (se.getSource() == modeFluoRadio)
		{
			newType = RunType.FLUO;
		}
		else if (se.getSource() == modeSONICCRadio)
		{
			newType = RunType.SONICC;
		}
		else
		{
			System.out.println("ERROR: Unknown selection event source: "+se);
			return;
		}

		if (run.getRunType()==RunType.SONICC && newType != RunType.SONICC && run.getSONICCBin()) {
			boolean okToUnbin = Util.popupOkCancel("Unbin","Unbin SONICC data?");
			if (!okToUnbin) {
				handleRunChange(); // reset radio buttons
				return;
			}
			CmdBinSONICC.unbin(run, false);
		}

		// make the local change first - this will be much faster than waiting for the callback
		((WdgRunTabInter)getParent()).reconfigureWidgets(newType);
		
		// tell the device, which will end up causing a redundant call to change the widgets
		// on this client, but it's OK because it will be ignored
		dev.setRunType(newType,false);
		
		// redistribute cells and send run dev update
		run.grid().getGridCalc().fitGridToContour(false, true);
	}

	@Override public void widgetDefaultSelected(SelectionEvent arg0) {}

	@Override
	public boolean updateModel()
	{
		return true;
	}
}
