package gov.anl.gmca.raster.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.ModelRasterRun;
import gov.anl.gmca.raster.ModelRasterRun.RunState;
import gov.anl.gmca.raster.ModelRasterRunList;
import gov.anl.gmca.raster.ModelRasterState;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

public class WdgDiffViewButtons extends Composite
{
	private Button openButton;
	
	public WdgDiffViewButtons(Composite parent)
	{
		super(parent, SWT.NONE);
		
		setLayout(new FillLayout());
		
		Group buttonsGroup = new Group(this,SWT.NONE);
		buttonsGroup.setFont(GUI.FONT_COLLECT_GROUP);
		buttonsGroup.setText("View");
		
		buttonsGroup.setLayout(new GridLayout(1,false));
		buttonsGroup.setBackground(GUI.BGCOLOR);
		
		openButton = new Button(buttonsGroup,SWT.NONE);
		openButton.setText("Open Viewer");
		openButton.setFont(GUI.FONT_DEFAULT);
		openButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		openButton.setEnabled(false);
		openButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		openButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent se)
			{
				TabRaster.get().setViewerMode(true);
			}
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	}
	
	public void connectListeners()
	{
		// update on raster state change
		ModelRasterState.get().llCurrentRun.addListener(new ListenerList.Listener<ModelRasterState>()
		{
			@Override public void handleEvent(ModelRasterState object, Object data)
			{
				updateButtonState();
			}
		});
		
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				updateButtonState();
			}
		}); 
	}
	
	protected void updateButtonState()
	{
		boolean runStarted = false;
		for (ModelRasterRun run : ModelRasterRunList.get(RasterListType.INTER).getAllRunsCopy())
		{
			if (run.getRunState() != RunState.INACTIVE) runStarted = true;
		}
		for (ModelRasterRun run : ModelRasterRunList.get(RasterListType.AUTO).getAllRunsCopy())
		{
			if (run.getRunState() != RunState.INACTIVE) runStarted = true;
		}
		
		// open button is enabled if there are some results
		// for now, disable in slave mode
		//boolean enableOpen = runStarted && Client.isMaster();
		boolean enableOpen = Client.isMaster();
		
		final boolean f_enableOpen=enableOpen;
		GUI.asyncExec(new Runnable(){public void run()
		{
			openButton.setEnabled(f_enableOpen);
		}});
	}
}
