package gov.anl.gmca.raster.ui;

import gov.anl.gmca.common.ui.NBMessageBox;
import gov.anl.gmca.common.ui.NBMessageBox.FieldType;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.ModelRasterRun.RunState;
import gov.anl.gmca.raster.util.DevMacroList;
import gov.anl.gmca.raster.util.ModelMacroList;
import gov.anl.gmca.raster.util.ModelMacroList.Macro;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ActionResult;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import gov.anl.gmca.util.WidgetFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import java.util.ArrayList;
import java.util.Collections;

public class WdgRasterMacro extends Composite
{
	protected Combo settingsCombo;
	protected ListenerList.Listener<ModelMacroList> macroListListener;
	protected Button btnLoad, btnDelete, btnSave;
	
	public WdgRasterMacro(Composite parent)
	{
		super(parent, SWT.NONE);
		
		setLayout(new FillLayout());
		
		Group settingsGroup = new Group(this, SWT.NONE);
		settingsGroup.setBackground(GUI.BGCOLOR);
		settingsGroup.setText("Macros");
		GridLayout gl = new GridLayout(4,false);
		gl.horizontalSpacing=0;
		settingsGroup.setLayout(gl);
		
		settingsCombo = new Combo(settingsGroup, SWT.READ_ONLY);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		settingsCombo.setLayoutData(gd);
		settingsCombo.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				select(settingsCombo.getText());
			}
		});
		
		btnLoad = WidgetFactory.newButton(settingsGroup, SWT.PUSH, "L");
		btnLoad.setToolTipText("Load the selected macro");
		btnLoad.setLayoutData(new GridData());
		btnLoad.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				load();
			}
		});
		
		btnDelete = WidgetFactory.newButton(settingsGroup, SWT.PUSH, "X");
		btnDelete.setToolTipText("Delete the selected macro");
		btnDelete.setLayoutData(new GridData());
		btnDelete.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				delete();
			}
		});
		
		btnSave = WidgetFactory.newButton(settingsGroup, SWT.PUSH, "+");
		btnSave.setToolTipText("Save current settings as a new macro");
		btnSave.setLayoutData(new GridData());
		btnSave.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				save();
			}
		});
		
		macroListListener = new ListenerList.Listener<ModelMacroList>()
		{
			@Override public void handleEvent(ModelMacroList object, Object data)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					updateMacroList();
				}});
			}
		};
		
		ModelMacroList.get().ll.addListener(macroListListener);
		
		addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent de)
			{
				ModelMacroList.get().ll.removeListener(macroListListener);
			}
		});
		
		updateMacroList();
	}
	
	private void select(String macroName)
	{
		DevMacroList.get().setCurrentMacro(macroName);
	}
	
	private void load()
	{
		ModelMacroList.get().loadMacro(settingsCombo.getText());
	}

	private String getMacroName(String promptText)
	{
		// ask the user for a macro name
		NBMessageBox namePrompt = new NBMessageBox(GUI.getShell(), SWT.ICON_QUESTION | SWT.OK | SWT.CANCEL);
		namePrompt.setText("Macro name");
		namePrompt.setMessage(promptText);
		namePrompt.setFieldType(FieldType.TEXT);
		int nameResult = namePrompt.open();
		if (nameResult != SWT.OK)
		{
			Log.note("Macro save cancelled.");
			return null;
		}
		else
		{
			return namePrompt.getText();
		}
	}
	
	private boolean macroExists(String macroName)
	{
		boolean exists = false;
		for (Macro macro : ModelMacroList.get().getAllMacros())
		{
			if (macro.ID.equals(macroName)) { exists = true; break; }
		}
		return exists;
	}
	
	private void save()
	{
		String promptText = "Please enter a name for your macro:";
		String errorText = "";
		boolean attemptSave = true;
		
		while (true)
		{
			// get name
			String macroName = getMacroName(promptText);
    		if (null == macroName) return;
    		attemptSave = true;

    		// does it exist?
    		if (macroExists(macroName))
    		{
    			// ask to overwrite
    			boolean overwrite = Util.popupYesNo("Macro exists", "The macro \""+macroName+"\" already exists.  Overwrite?");
    			if (overwrite)
    			{
    				// delete existing
    				DevMacroList.get().removeMacro(macroName);
    			}
    			else
    			{
    				// don't overwrite
    				errorText = "Overwrite declined.";
    				attemptSave = false;
    			}
    		}
    		
    		if (attemptSave)
    		{
    			// try to save
    			ActionResult saveResult = ModelMacroList.get().saveMacro(macroName);
    			if (saveResult.succeeded()) return;
    			errorText = saveResult.getError();
    		}
    		
			// if there was an error, save the error text for the next prompt
			promptText = errorText+"\nPlease enter a new name for your macro:";
		}
	}
	
	private void delete()
	{
		ModelMacroList.get().deleteMacro(settingsCombo.getText());
	}
	
	// called in gui thread
	private void updateMacroList()
	{
		// get sorted macro list
		ArrayList<Macro> macroListCopy = ModelMacroList.get().getAllMacros();
		Collections.sort(macroListCopy);
		
		// get sorted combo list
		ArrayList<String> comboItemsCopy = new ArrayList<String>();
		for (String item : settingsCombo.getItems()) comboItemsCopy.add(item);
		Collections.sort(comboItemsCopy);
		
		// compare lists
		boolean listsEqual = true;
		if (macroListCopy.size() != comboItemsCopy.size())
		{
			listsEqual = false;
		}
		else
		{
			for (int i=0; i<macroListCopy.size(); i++)
			{
				if (!macroListCopy.get(i).ID.equals(comboItemsCopy.get(i)))
				{
					listsEqual = false; break;
				}
			}
		}
		
		// update combo if needed
		if (!listsEqual)
		{
			settingsCombo.removeAll();
			for (Macro macro : macroListCopy)
			{
				settingsCombo.add(macro.ID);
			}
		}

		updateCurrentMacro();
		updateWidgetState();
	}
	
	private void updateCurrentMacro()
	{
		String currentMacro = ModelMacroList.get().getCurrentMacro();
		
		for (int i=0; i<settingsCombo.getItems().length; i++)
		{
			if (settingsCombo.getItem(i).equals(currentMacro))
			{
				settingsCombo.select(i);
			}
		}
	}
	
	protected boolean globalEnable = false;
	protected RunState state = RunState.INACTIVE;
	
	public void setEnabled(boolean enable) { this.globalEnable = enable; updateWidgetState(); }
	public void updateState(RunState state) { this.state = state; updateWidgetState(); }
	
	protected void updateWidgetState()
	{
		String currentMacro = ModelMacroList.get().getCurrentMacro();
		boolean currentMacroIsEmpty = (null == currentMacro || currentMacro.length()==0);
		
		settingsCombo	.setEnabled(globalEnable && !currentMacroIsEmpty);
		btnLoad			.setEnabled(globalEnable && !currentMacroIsEmpty);
		btnDelete		.setEnabled(globalEnable && !currentMacroIsEmpty);
		btnSave			.setEnabled(globalEnable);
	}
}
