package gov.anl.gmca.raster.ui;

import gov.anl.gmca.collect.DevCollectSettings;
import gov.anl.gmca.common.DevMove;
import gov.anl.gmca.common.DevMoveFloat;
import gov.anl.gmca.common.EpicsDevice;
import gov.anl.gmca.common.EpicsDevice.State;
import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgControlCheckbox;
import gov.anl.gmca.common.ui.WdgControlDir;
import gov.anl.gmca.common.ui.WdgControlDir.PrefixDir;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.common.ui.WdgControlEntry.EntryType;
import gov.anl.gmca.common.ui.WdgControlEntry.ValidateResult;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevAttenuation;
import gov.anl.gmca.hutch.DevEnergy;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.hutch.ModelPmacMotor;
import gov.anl.gmca.raster.*;
import gov.anl.gmca.raster.ModelRasterRun.ROIMode;
import gov.anl.gmca.raster.ModelRasterRun.RunState;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.raster.cam.ModelRasterVideo.VideoState;
import gov.anl.gmca.raster.sonicc.CmdBinSONICC;
import gov.anl.gmca.sample.DBBeamSize;
import gov.anl.gmca.sample.DBBeamSize.BeamSize;
import gov.anl.gmca.sample.ModelBeamsize;
import gov.anl.gmca.spotfinder.DevSFType;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Point2D;
import gov.anl.gmca.util.Util;
import gov.anl.gmca.util.WidgetFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;

// Common widgets between the interactive and auto raster tabs are setup here
// Widgets are tied to the run passed to the constructor - the auto tab is linked to a setup run
public abstract class WdgRasterRunTab extends Composite implements ClientListener, ModelRasterRunList.Listener
{
	protected int tabIndex;
	protected boolean isAuto;
	protected ModelRasterRun run;
	protected DevRasterRun devRun;
	protected ArrayList<WdgControl> nonGridControls = new ArrayList<WdgControl>();
	protected ArrayList<WdgControl> gridControls = new ArrayList<WdgControl>();

	protected Composite paramComp, resLimitsComp, resLimitsButtonComp;
	protected Composite timeDeltaComp, spacerComp, cellSizeComp;
	protected Group expDeltaGroup;

	protected Label beamSizeLabel, beamSizeUnits, cellSizeLabel, cellSizeUnits;
	protected Label lblPolygons, lblLoopZ, lblLoopZUnits, lblLoopCenteredUnits, attenLabel, attenUnitLabel;
	protected Label timeUnitLabel, distLabel, distUnitLabel, binLabel, binUnits;
	protected Button binButton, unbinButton;
	protected Composite polygonControlComp, beamSizeComp;
	protected Label lblContourCount, deltaUnitLabel, energyLabel, energyUnits, autoFindLabel, lblLoopCentered;
	protected Label cellWidthLabel, cellHeightLabel, timeLabel, deltaLabel, lblResLimit;
	protected Button btnAddContour, btnRemoveContour, btnClearContours, checkboxFixLimits;
	protected Combo beamsizeCombo, channelCombo, soniccDisplayCombo;
	protected WdgControlEntry cellWidthEntry, cellHeightEntry, beamWidthEntry, beamHeightEntry, timeEntry, deltaEntry;
	protected WdgControlEntry distanceEntry, attenuationEntry, energyEntry, loopMeasurementEntry, lowResLimitEntry, highResLimitEntry;
	protected WdgControlDir controlDir;
	protected WdgControlROI roi;
	protected WdgControlCheckbox autoFindCheckbox, loopCenteredCheckbox;
	protected WdgRasterMacro wdgMacro;
	
	public WdgRasterRunTab(Composite parent, int style, ModelRasterRun run, DevRasterRun devRun)
	{
		super(parent, style);
		this.run=run;
		this.devRun=devRun;
		isAuto = run.getRunList().getListType()==RasterListType.AUTO;
	}
	
	public static enum PW
	{
		DirPrefix,
		AutoFind,
		Contours,
		CellAndBeamsize,
		Time,
		Delta,
		Distance,
		Attenuation,
		LoopMeasurement,
		Energy,
		ROI,
		ResLimits,
		Macros,
		CellAndBeamsize3pass,
		SONICC_Controls
	};
	
	protected Composite setupParameterWidgets(Composite parent, PW... _setupWidgets)
	{
		ArrayList<PW> setupWidgets = new ArrayList<PW>();
		for (PW pw : _setupWidgets) setupWidgets.add(pw);
		
		paramComp = new Composite(parent,SWT.NONE);
		GridLayout paramLayout = new GridLayout(5,false);
		paramLayout.marginWidth = 0;
		paramLayout.verticalSpacing = 0;
		paramLayout.marginHeight = 0;
		paramComp.setLayout(paramLayout);
		paramComp.setBackground(GUI.BGCOLOR);
		paramComp.setLayoutData(new GridData(SWT.CENTER, SWT.FILL, true, true));

		for (PW pw : setupWidgets)
		{
			switch (pw)
			{
			case DirPrefix:
	    		controlDir = new WdgControlDir(paramComp, SWT.NONE, null, 0, new WdgControlDir.ModelInterface()
	    		{
	    			@Override public void setValue(Object value)
	    			{
	    				WdgControlDir.PrefixDir prefixDir = (WdgControlDir.PrefixDir)value;
	    				devRun.setFileroot(prefixDir.prefix,false);
	    				devRun.setDirectory(prefixDir.dir,false);
	    				devRun.sendUpdate();
	    			}
	    			@Override public PrefixDir getPrefixDir()
	    			{
	    				return new WdgControlDir.PrefixDir(run.getFileRoot(), run.getDirectorySuffix());
	    			}
	    		});
	    		controlDir.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 5, 1));

	    		spacerComp = new Composite(paramComp, SWT.NONE);
	    		spacerComp.setBackground(GUI.BGCOLOR);
	    		GridData gd1 = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 5, 1);
	    		gd1.heightHint = 3;
	    		spacerComp.setLayoutData(gd1);
				break;
				
			case AutoFind:
	    		autoFindLabel = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "AutoFind: ");
	    		autoFindLabel.setLayoutData(newGDLeftLabel());
	    		autoFindCheckbox = new WdgControlCheckbox(paramComp, SWT.NONE, new WdgControlEntry.ModelInterface()
	    		{
	    			@Override public void setValue(Object value)
	    			{
	    				if (value == Boolean.FALSE)
	    				{
	    					devRun.setAutoFind(false, true); return;
	    				}
	    				else
	    				{
		    				int polygonCount = run.grid().getContours().size();
		    				if (polygonCount > 0)
		    				{
		    					boolean okToDelete = Util.popupOkCancel("Delete polygons?", "Turning on autoFind will delete the "+polygonCount+" existing polygons.  OK to delete?");
		    					if (okToDelete)
		    					{
		    						devRun.setContours(new ArrayList<ModelContour>(), false);
		    						devRun.setCenter(null, false);
		    						devRun.setSelectedCells(new ArrayList<Boolean>(), false);
			    					devRun.setLoopCentered(false, false);
			    					devRun.setAutoFind(true, false);
			    					run.grid().getGridCalc().fitGridToContour(false, true);
		    					}
		    					else
		    					{
		    						autoFindCheckbox.setSelected(false);
		    					}
		    				}
		    				else
		    				{
		    					devRun.setLoopCentered(false, false);
		    					devRun.setAutoFind(true, true);
		    				}
	    				}
	    			}
	    		});
	    		autoFindCheckbox.setLayoutData(newGDEntry());
	    		WidgetFactory.newLabel(paramComp, SWT.LEFT, "").setLayoutData(newGDRightLabel());
	    		
	    		lblLoopCentered = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Loop centered: ");
	    		lblLoopCentered.setLayoutData(newGDLeftLabel());
	    		loopCenteredCheckbox = new WdgControlCheckbox(paramComp, SWT.NONE, new WdgControlEntry.ModelInterface()
	    		{
	    			@Override public void setValue(Object value) { devRun.setLoopCentered((Boolean)value, true); }
	    		});
	    		loopCenteredCheckbox.setLayoutData(newGDEntry());
	    		lblLoopCenteredUnits = WidgetFactory.newLabel(paramComp, SWT.LEFT, "");
	    		lblLoopCenteredUnits.setLayoutData(newGDRightLabel());
				break;
				
			case Contours:
				lblPolygons = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Polygons: ");
				lblPolygons.setLayoutData(newGDLeftLabel());
	    		lblContourCount = WidgetFactory.newLabel(paramComp, SWT.CENTER, "---");
	    		lblContourCount.setLayoutData(newGDEntry());
	    		
	    		polygonControlComp = new Composite(paramComp, SWT.RIGHT);
	    		GridData gd = newGDRightLabel();
	    		polygonControlComp.setLayoutData(gd);
	    		polygonControlComp.setLayout(new FillLayout());
	    		btnAddContour = WidgetFactory.newButton(polygonControlComp, SWT.PUSH, "+");
	    		btnAddContour.setToolTipText("Add a new polygon");
	    		btnAddContour.setBackground(GUI.COLOR_LIGHT_GRAY);
	    		btnAddContour.setFont(GUI.FONT_COLLECT_BUTTON2);
	    		btnAddContour.addSelectionListener(new SelectionListener()
	    		{
	    			@Override public void widgetSelected(SelectionEvent se)
	    			{
	    				TabRaster.get().cam.model.changeState(VideoState.CREATE_CONTOUR);
	    			}
	    			@Override public void widgetDefaultSelected(SelectionEvent se) {}
	    		});
	    		btnRemoveContour = WidgetFactory.newButton(polygonControlComp, SWT.PUSH, "-");
	    		btnRemoveContour.setToolTipText("Delete the last polygon");
	    		btnRemoveContour.setBackground(GUI.COLOR_LIGHT_GRAY);
	    		btnRemoveContour.setFont(GUI.FONT_COLLECT_BUTTON2);
	    		btnRemoveContour.addSelectionListener(new SelectionListener()
	    		{
	    			@Override public void widgetSelected(SelectionEvent se)
	    			{
	    				run.grid().removeLastContour();
	    			}
	    			@Override public void widgetDefaultSelected(SelectionEvent se) {}
	    		});
	    		btnClearContours = WidgetFactory.newButton(polygonControlComp, SWT.PUSH, "C");
	    		btnClearContours.setToolTipText("Delete all polygons");
	    		btnClearContours.setBackground(GUI.COLOR_LIGHT_GRAY);
	    		btnClearContours.setFont(GUI.FONT_COLLECT_BUTTON2);
	    		btnClearContours.addSelectionListener(new SelectionListener()
	    		{
	    			@Override public void widgetSelected(SelectionEvent se)
	    			{
	    				run.grid().clearAllContours(true, true);
	    			}
	    			@Override public void widgetDefaultSelected(SelectionEvent se) {}
	    		});
				
				break;
				
			case CellAndBeamsize:

				if (Util.beamSizeCalculated()) {
					beamSizeLabel = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Beam size: "); beamSizeLabel.setLayoutData(newGDLeftLabel());
					createBeamSizeDropdown(paramComp);
					beamSizeUnits = WidgetFactory.newLabel(paramComp, SWT.LEFT, "\u00B5m"); beamSizeUnits.setLayoutData(newGDRightLabel());

					cellSizeLabel = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Cell size: "); cellSizeLabel.setLayoutData(newGDLeftLabel());
					createCellSize(paramComp);
					cellSizeUnits = WidgetFactory.newLabel(paramComp, SWT.LEFT, "\u00B5m"); cellSizeUnits.setLayoutData(newGDRightLabel());
				} else {
					cellSizeLabel = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Cell size: "); cellSizeLabel.setLayoutData(newGDLeftLabel());
					createCellSize(paramComp);
					cellSizeUnits = WidgetFactory.newLabel(paramComp, SWT.LEFT, "\u00B5m"); cellSizeUnits.setLayoutData(newGDRightLabel());

					beamSizeLabel = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Beam size: "); beamSizeLabel.setLayoutData(newGDLeftLabel());
					createBeamSizeFields(paramComp);
					beamSizeUnits = WidgetFactory.newLabel(paramComp, SWT.LEFT, "\u00B5m"); beamSizeUnits.setLayoutData(newGDRightLabel());
				}
				break;

			case CellAndBeamsize3pass:
				
				// full width composite
				Composite threePassComp = new Composite(paramComp, SWT.NONE);
				GridLayout gl = new GridLayout(10,false);
				gl.horizontalSpacing = 1;
				gl.verticalSpacing = 2;
				threePassComp.setLayout(gl);
				threePassComp.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 5, 1));
				threePassComp.setBackground(GUI.BGCOLOR);
				threePassComp.setBackgroundMode(SWT.INHERIT_FORCE);
				
				// fill top left
				new Label(threePassComp, SWT.NONE);
				
				// headers
				Label coarseLabel = WidgetFactory.newLabel(threePassComp, SWT.CENTER, "Coarse");
				coarseLabel.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));
				Label fineLabel = WidgetFactory.newLabel(threePassComp, SWT.CENTER, "Fine");
				fineLabel.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));
				Label orthoLabel = WidgetFactory.newLabel(threePassComp, SWT.CENTER, "Ortho");
				orthoLabel.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));
				
				// cell label
				Label cellLabel = WidgetFactory.newLabel(threePassComp, SWT.RIGHT, "Cell ");
				gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
				gd.minimumWidth = 50;
				cellLabel.setLayoutData(gd);
				
				// coarse cell dim
				WdgControlEntry cellCoarseWidthEntry = new WdgControlEntry(threePassComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface(){ @Override public void setValue(Object value) {} });
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.minimumWidth = 35;
				cellCoarseWidthEntry.setLayoutData(gd);
				
				WidgetFactory.newLabel(threePassComp, SWT.NONE, "x");
				
				WdgControlEntry cellCoarseHeightEntry = new WdgControlEntry(threePassComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface(){ @Override public void setValue(Object value) {} });
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.minimumWidth = 35;
				cellCoarseHeightEntry.setLayoutData(gd);

				// fine cell dim
				WdgControlEntry cellFineWidthEntry = new WdgControlEntry(threePassComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface(){ @Override public void setValue(Object value) {} });
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.minimumWidth = 35;
				cellFineWidthEntry.setLayoutData(gd);
				
				WidgetFactory.newLabel(threePassComp, SWT.NONE, "x");
				
				WdgControlEntry cellFineHeightEntry = new WdgControlEntry(threePassComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface(){ @Override public void setValue(Object value) {} });
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.minimumWidth = 35;
				cellFineHeightEntry.setLayoutData(gd);

				// ortho cell dim
				WdgControlEntry cellOrthoWidthEntry = new WdgControlEntry(threePassComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface(){ @Override public void setValue(Object value) {} });
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.minimumWidth = 35;
				cellOrthoWidthEntry.setLayoutData(gd);
				
				WidgetFactory.newLabel(threePassComp, SWT.NONE, "x");
				
				WdgControlEntry cellOrthoHeightEntry = new WdgControlEntry(threePassComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface(){ @Override public void setValue(Object value) {} });
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.minimumWidth = 35;
				cellOrthoHeightEntry.setLayoutData(gd);

				// beam label
				Label beamLabel = WidgetFactory.newLabel(threePassComp, SWT.RIGHT, "Beam ");
				gd = new GridData(SWT.FILL, SWT.CENTER, true, false);
				gd.minimumWidth = 50;
				beamLabel.setLayoutData(gd);
				
				// coarse cell dim
				WdgControlEntry beamCoarseWidthEntry = new WdgControlEntry(threePassComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface(){ @Override public void setValue(Object value) {} });
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.minimumWidth = 35;
				beamCoarseWidthEntry.setLayoutData(gd);
				
				WidgetFactory.newLabel(threePassComp, SWT.NONE, "x");
				
				WdgControlEntry beamCoarseHeightEntry = new WdgControlEntry(threePassComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface(){ @Override public void setValue(Object value) {} });
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.minimumWidth = 35;
				beamCoarseHeightEntry.setLayoutData(gd);

				// fine beam dim
				WdgControlEntry beamFineWidthEntry = new WdgControlEntry(threePassComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface(){ @Override public void setValue(Object value) {} });
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.minimumWidth = 35;
				beamFineWidthEntry.setLayoutData(gd);
				
				WidgetFactory.newLabel(threePassComp, SWT.NONE, "x");
				
				WdgControlEntry beamFineHeightEntry = new WdgControlEntry(threePassComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface(){ @Override public void setValue(Object value) {} });
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.minimumWidth = 35;
				beamFineHeightEntry.setLayoutData(gd);

				// ortho beam dim
				WdgControlEntry beamOrthoWidthEntry = new WdgControlEntry(threePassComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface(){ @Override public void setValue(Object value) {} });
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.minimumWidth = 35;
				beamOrthoWidthEntry.setLayoutData(gd);
				
				WidgetFactory.newLabel(threePassComp, SWT.NONE, "x");
				
				WdgControlEntry beamOrthoHeightEntry = new WdgControlEntry(threePassComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface(){ @Override public void setValue(Object value) {} });
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.minimumWidth = 35;
				beamOrthoHeightEntry.setLayoutData(gd);

				break;
				
			case Time:
	    		timeLabel = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Time: "); timeLabel.setLayoutData(newGDLeftLabel());
	    		timeEntry = new WdgControlEntry(paramComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
	    		{
	    			@Override public void setValue(Object value)	{ devRun.setExposureTime((Double)value,true); 	}
	    			@Override public Object getModelValue() 		{ return run.getExposureTime();			 		}
	    			@Override public int getDecimals() 				{ return 3; 									}
	    			@Override public Limits getLimits()
	    			{
	    				double pvMin = DevCollectSettings.getRasterExposureLimits().getLower();
	    				double pvMax = DevCollectSettings.getRasterExposureLimits().getUpper();
	    				double speedBasedMin = run.grid().getGridCalc().getMinExposureTime_sec(run.grid().getCellWidth_um());
	    				if (pvMin>speedBasedMin)
	    				{
	    					return new Limits(pvMin, pvMax, "exposure time");
	    				}
	    				else
	    				{
	    					return new Limits(speedBasedMin, pvMax, "exposure time due to top horizontal motor speed");
	    				}
	    			}
	    		});
	    		timeEntry.setLayoutData(newGDEntry());
	    		timeUnitLabel = WidgetFactory.newLabel(paramComp, SWT.LEFT, "sec"); timeUnitLabel.setLayoutData(newGDRightLabel());
	    		break;

			case Delta:
	    		deltaLabel = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Delta: ");
	    		deltaLabel.setLayoutData(newGDLeftLabel());
	    		deltaEntry = new WdgControlEntry(paramComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
	    		{
	    			@Override public void setValue(Object value)	{ devRun.setDelta((Double)value,true);		 	}
	    			@Override public Object getModelValue() 		{ return run.getDelta();				 		}
	    			@Override public Limits getLimits() 			{ return DevCollectSettings.getRasterDeltaLimits(); }
	    			@Override public int getDecimals() 				{ return 3; 									}
	    			@Override public ValidateResult validate(String s)
	    			{
	    				return super.validate(s);
	    			};
	    		});
	    		deltaEntry.setLayoutData(newGDEntry());
	    		deltaUnitLabel = WidgetFactory.newLabel(paramComp, SWT.LEFT, "deg");
	    		deltaUnitLabel.setLayoutData(newGDRightLabel());
	    		break;

			case Distance:
	    		distLabel = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Distance: "); distLabel.setLayoutData(newGDLeftLabel());
	    		distanceEntry = new WdgControlEntry(paramComp, SWT.NONE, EntryType.RW_COMBO,  new WdgControlEntry.ModelInterface()
	    		{
	    			@Override public void setValue(Object value)	{ run.changeDetectorDist((Double)value); 							}
	    			@Override public Object getModelValue() 		{ return run.getDetectorDist();										}
	    			@Override public EpicsDevice getDevice()		{ return DevPmacMotor.get("detector_z");							}
	    			@Override public Object getDeviceValue() 		{ return DevPmacMotor.get("detector_z").getPosition(); 				}
	    			@Override public Limits getLimits() 			{ return DevPmacMotor.get("detector_z").getLimits(); 				}
	    			@Override public int getDecimals() 				{ return 3; 														}
	    			@Override public boolean doHighlight(double d)  { return Math.abs((Double)getDeviceValue()-d)>0.01; }
	    			@Override public double[] getFloatChoices()		{ return ModelPmacMotor.getDetectorDistanceOptions();				}
	    		});
	    		distanceEntry.setLayoutData(newGDEntry());
	    		distUnitLabel = WidgetFactory.newLabel(paramComp, SWT.LEFT, "mm"); distUnitLabel.setLayoutData(newGDRightLabel());
	    		break;
	    		
			case Attenuation:
	    		attenLabel = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Attenuation: "); attenLabel.setLayoutData(newGDLeftLabel());
	    		attenuationEntry = new WdgControlEntry(paramComp, SWT.NONE, EntryType.RW_COMBO,  new WdgControlEntry.ModelInterface()
	    		{
	    			@Override public void setValue(Object value)	{ devRun.setAttenuation_factors(((Double)value).intValue(), true); 	}
	    			@Override public Object getModelValue()			{ return run.getAttenuation();										}
	    			@Override public EpicsDevice getDevice()		{ return DevAttenuation.get();										}
	    			@Override public Object getDeviceValue() 		{ return DevAttenuation.get().getPosition();						}
	    			@Override public Limits getLimits() 			{ return DevAttenuation.get().getLimits();							}
	    			@Override public int getDecimals() 				{ return 2;					 										}
	    			@Override public double[] getFloatChoices()		{ return new double[]{1,5,10,20,50,100,1000};					}
	    			@Override public boolean doHighlight(double w)  { double d=(Double)getDeviceValue(); return (Math.abs(w-d)/d)>0.05;	}
	    		});
	    		attenuationEntry.setLayoutData(newGDEntry());
	    		attenUnitLabel = WidgetFactory.newLabel(paramComp, SWT.LEFT, "factor"); attenUnitLabel.setLayoutData(newGDRightLabel());
	    		break;
				
			case LoopMeasurement:
	    		lblLoopZ = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Loop Z: ");
	    		lblLoopZ.setLayoutData(newGDLeftLabel());

	    		loopMeasurementEntry = new WdgControlEntry(paramComp, SWT.NONE, EntryType.TEXT,  new WdgControlEntry.ModelInterface()
	    		{
	    			@Override public void setValue(Object value)		{ devRun.setLoopMeasurement_um((Integer)value, true); 	}
	    			@Override public Object getModelValue() 			{ return run.getLoopMeasurement_um();				 	}
	    			@Override public Limits getLimits() 				{ return new Limits(5,1000,"loop Z"); 					}
	    			@Override public int getDecimals() 					{ return 0; 											}
	    			@Override public ValidateResult validate(String s)
	    			{
	    				try
	    				{
	    					Integer.parseInt(s);
	    				}
	    				catch (NumberFormatException nfe) { return new ValidateResult(false, "Invalid integer "+s); }
	    				return super.validate(s);
	    			}
	    		});
	    		loopMeasurementEntry.setLayoutData(newGDEntry());
	    		
	    		lblLoopZUnits = WidgetFactory.newLabel(paramComp, SWT.LEFT, "\u00B5m");
	    		lblLoopZUnits.setLayoutData(newGDRightLabel());
	    		
	    		break;
				
			case Energy:
	    		energyLabel = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Energy: ");
	    		energyLabel.setLayoutData(newGDLeftLabel());
	    		energyEntry = new WdgControlEntry(paramComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
	    		{
	    			@Override public void setValue(Object value)		{ devRun.setEnergy_keV((Double)value,true);						}
	    			@Override public Object getModelValue()				{ return run.getEnergy();										}
	    			@Override public EpicsDevice getDevice()			{ return DevPmacMotor.get("mo_En_E");							}
	    			@Override public Object getDeviceValue() 			{ return DevPmacMotor.get("mo_En_E").getPosition();				}
	    			@Override public Limits getLimits() 				{ return DevPmacMotor.get("mo_En_E").getLimits();				}
	    			@Override public ValidateResult validate(String s)	{ if (s.equals("")) return new ValidateResult(true); try { Double.parseDouble(s); return new ValidateResult(true); } catch (NumberFormatException nfe) { return new ValidateResult(false, s+" is not a valid number."); } }
	    			@Override public int getDecimals() 					{ return 4; 													}
	    		});
	    		energyEntry.setLayoutData(newGDEntry());
	    		energyUnits = WidgetFactory.newLabel(paramComp, SWT.LEFT, "keV");
	    		energyUnits.setLayoutData(newGDRightLabel());
	    		break;
			
			case ROI:
	    		roi = new WdgControlROI(paramComp, SWT.NONE, new WdgControlEntry.ModelInterface()
	    		{
	    			@Override public void setValue(Object value) { devRun.setROIMode((ROIMode)value,true);	}
	    		});
	    		roi.setLayoutData(new GridData(SWT.CENTER, SWT.DEFAULT, true, false, 5, 1));
	    		break;

			case SONICC_Controls:
				binButton = new Button(paramComp, SWT.NONE);
				binButton.setBackground(GUI.COLOR_BLUE_BUTTON);
				binButton.setText("Bin data");
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
				binButton.setLayoutData(gd);
				binButton.addSelectionListener(new SelectionListener()
				{
					@Override
					public void widgetSelected(SelectionEvent selectionEvent)
					{
						new CmdBinSONICC(run, new Point2D(20,20)).start();
					}
					@Override public void widgetDefaultSelected(SelectionEvent selectionEvent) {}
				});

				Composite spacer = new Composite(paramComp, SWT.NONE);
				spacer.setBackground(GUI.BGCOLOR);
				gd = new GridData(GridData.FILL_HORIZONTAL);
				gd.heightHint=1;
				spacer.setLayoutData(gd);

				unbinButton = new Button(paramComp, SWT.NONE);
				unbinButton.setBackground(GUI.COLOR_BLUE_BUTTON);
				unbinButton.setText("Unbin data");
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1);
				unbinButton.setLayoutData(gd);
				unbinButton.addSelectionListener(new SelectionListener()
				{
					@Override
					public void widgetSelected(SelectionEvent selectionEvent)
					{
						CmdBinSONICC.unbin(run, true);
					}
					@Override public void widgetDefaultSelected(SelectionEvent selectionEvent) {}
				});

				channelCombo = new Combo(paramComp, SWT.READ_ONLY);
				channelCombo.add("Channel A");
				channelCombo.add("Channel B");
				channelCombo.add("Channel C");
				gd = new GridData(GridData.FILL_HORIZONTAL); gd.horizontalSpan=5;
				channelCombo.setLayoutData(gd);
				channelCombo.addSelectionListener(new SelectionListener()
				{
					@Override
					public void widgetSelected(SelectionEvent selectionEvent)
					{
						String[] choices = {"A","B","C"};
						WdgSONICCTab.channelString = choices[channelCombo.getSelectionIndex()];
					}
					@Override public void widgetDefaultSelected(SelectionEvent selectionEvent) {}
				});
				channelCombo.select(0);

				soniccDisplayCombo = new Combo(paramComp, SWT.READ_ONLY);
				soniccDisplayCombo.add("None");
				soniccDisplayCombo.add("Highlight 1");
				soniccDisplayCombo.add("Highlight 2");
				soniccDisplayCombo.add("Heatmap 1");
				soniccDisplayCombo.add("Heatmap 2");
				gd = new GridData(GridData.FILL_HORIZONTAL); gd.horizontalSpan=5;
				soniccDisplayCombo.setLayoutData(gd);
				soniccDisplayCombo.addSelectionListener(new SelectionListener()
				{
					@Override
					public void widgetSelected(SelectionEvent selectionEvent)
					{
						WdgSONICCTab.displayIndex = soniccDisplayCombo.getSelectionIndex();
					}
					@Override public void widgetDefaultSelected(SelectionEvent selectionEvent) {
					}
				});
				soniccDisplayCombo.select(2);

				break;

			case ResLimits:
	    		lblResLimit = WidgetFactory.newLabel(paramComp, SWT.RIGHT, "Res. Limit: ");
	    		lblResLimit.setLayoutData(newGDLeftLabel());
	    		lblResLimit.setToolTipText("Resolution limits for processing by DISTL.signal_strength");
	    		
	    		resLimitsComp = new Composite(paramComp, SWT.NONE);
	    		resLimitsComp.setLayoutData(newGDEntry());
	    		resLimitsComp.setBackground(GUI.BGCOLOR);
	    		gl = new GridLayout(4,false);
	    		gl.marginHeight=0;
	    		gl.marginWidth=0;
	    		gl.horizontalSpacing=0;
	    		resLimitsComp.setLayout(gl);
	    		WidgetFactory.newLabel(resLimitsComp, SWT.NONE, "L").setLayoutData(new GridData());
	    		lowResLimitEntry = new WdgControlEntry(resLimitsComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
	    		{
	    			@Override public void setValue(Object value)
	    			{
	    				double newLowRes = (value instanceof String && ((String)value).equals("\u221e"))?Double.POSITIVE_INFINITY:(Double)value;
	    				run.dev().setLowResLimit_A(newLowRes, false);
	    				if (newLowRes < run.getHighResLimit_A())
	    				{
	    					Log.note(Util.sprintf("Requested low res. limit %.1f\u212b < high res. limit %.1f\u212b, setting high res. limit to low res. limit", newLowRes, run.getHighResLimit_A()));
	    					run.dev().setHighResLimit_A(newLowRes, false);
	    				}
	    				run.dev().sendUpdate();
	    			}
	    			@Override public Object getModelValue() 		{ return run.getLowResLimit_A(); }
	    			@Override public int getDecimals() 				{ return 1;	}
	    			@Override public Limits getLimits()				{ return new Limits(1.0,50.0, "resolution limit"); }
	    		});
	    		lowResLimitEntry.setLayoutData(new GridData(SWT.FILL,SWT.DEFAULT,true,false));
	    		//nonGridControls.add(lowResLimitEntry);
	    		WidgetFactory.newLabel(resLimitsComp, SWT.NONE, " H").setLayoutData(new GridData());
	    		highResLimitEntry = new WdgControlEntry(resLimitsComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
	    		{
	    			@Override public void setValue(Object value)
	    			{
	    				double newHighRes = (value instanceof String && ((String)value).equals("\u221e"))?Double.POSITIVE_INFINITY:(Double)value;
	    				run.dev().setHighResLimit_A(newHighRes, false);
	    				if (newHighRes > run.getLowResLimit_A())
	    				{
	    					Log.note(Util.sprintf("Requested high res. limit %.1f\u212b > low res. limit %.1f\u212b, setting low res. limit to high res. limit", newHighRes, run.getLowResLimit_A()));
	    					run.dev().setLowResLimit_A(newHighRes, false);
	    				}
	    				run.dev().sendUpdate();
	    			}
	    			@Override public Object getModelValue() 		{ return run.getHighResLimit_A(); }
	    			@Override public int getDecimals() 				{ return 1; }
	    			@Override public Limits getLimits()				{ return new Limits(1.0,50.0, "resolution limit"); }
	    		});
	    		highResLimitEntry.setLayoutData(new GridData(SWT.FILL,SWT.DEFAULT,true,false));
	    		//nonGridControls.add(highResLimitEntry);
	    		
	    		resLimitsButtonComp = new Composite(paramComp, SWT.NONE);
	    		resLimitsButtonComp.setBackground(GUI.BGCOLOR);
	    		resLimitsButtonComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	    		gl = new GridLayout(2,false);
	    		gl.marginHeight=0;
	    		gl.marginWidth=0;
	    		resLimitsButtonComp.setLayout(gl);
	    		WidgetFactory.newLabel(resLimitsButtonComp, SWT.LEFT, "\u212B").setLayoutData(new GridData());

	    		checkboxFixLimits = WidgetFactory.newButton(resLimitsButtonComp, SWT.CHECK, "Fix");
	    		checkboxFixLimits.setToolTipText("Disallow automatic updates to the limit fields\n(This setting is applied to all raster runs)");
	    		checkboxFixLimits.setBackground(GUI.BGCOLOR);
	    		checkboxFixLimits.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, true));
	    		checkboxFixLimits.addSelectionListener(new SelectionListener()
				{
					@Override public void widgetSelected(SelectionEvent e)
					{
						DevRasterState.get().setFixResLimits(checkboxFixLimits.getSelection());
					}
					@Override public void widgetDefaultSelected(SelectionEvent e) {}
				});
				break;
	    		
			case Macros:
				
	    		Composite fillComp = new Composite(paramComp, SWT.NONE);
	    		gd = new GridData(SWT.FILL, SWT.FILL, true, true, 5, 1);
	    		fillComp.setLayoutData(gd);
	    		fillComp.setBackground(GUI.BGCOLOR);
	    		
				wdgMacro = new WdgRasterMacro(paramComp);
				wdgMacro.setLayoutData(new GridData(SWT.FILL, SWT.BOTTOM, true, false, 5, 1));
				
	    		break;
			}
		}
		
		WdgControl[] nonGridControlList = new WdgControl[]
        {
			controlDir,distanceEntry,attenuationEntry,energyEntry,roi,
			autoFindCheckbox,loopCenteredCheckbox,
			controlDir,deltaEntry,timeEntry,
        };
		for (WdgControl control : nonGridControlList)
		{
			if (null != control) nonGridControls.add(control);
		}
		WdgControl[] gridControlList = new WdgControl[]
        {
				cellWidthEntry, cellHeightEntry, beamWidthEntry, beamHeightEntry,
				loopMeasurementEntry
        };
		for (WdgControl control : gridControlList)
		{
			if (null != control) gridControls.add(control);
		}
		
		//////////////////////////////////////////////// end widgets //////////////////////////////////////

		Client.addListener(this);
		run.getRunList().addListener(this);
		
		final ListenerList.Listener<ModelRasterRun> runListener = new ListenerList.Listener<ModelRasterRun>()
		{
			@Override public void handleEvent(ModelRasterRun run, Object data)
			{
				handleRunChange(run);
			}
		};
		run.llRunChanged.addListener(runListener);
		
		final ListenerList.Listener<ModelRasterState> stateListener = new ListenerList.Listener<ModelRasterState>()
		{
			@Override public void handleEvent(ModelRasterState object, Object data)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					updateEnableState();
					updateOtherFields();
				}});
			}
		};
		ModelRasterState.get().llCurrentRun.addListener(stateListener);

		if (Util.beamSizeCalculated()) {
			DevMoveFloat.Listener energyMoveListener = new DevMoveFloat.Listener() {
				@Override
				public void handleStateChange(State state) {
				}

				@Override
				public void handleActPosChange(DevMoveFloat dev, double newPos) {
					if (!dev.isMoving()) return;
					if (run.getRunState() != RunState.INACTIVE) return;
					GUI.asyncExec(() ->
							updateBeamSizeDisplay(false)
					);
				}

				@Override
				public void handleRqsPosChange(double newPos) {
				}

				@Override
				public void handleUnitsChange(String units) {
				}
			};
			DevPmacMotor.get("mo_En_E").addListener(energyMoveListener);

			ListenerList.Listener<DevMove> energyStopListener = new ListenerList.Listener<DevMove>() {
				@Override
				public void handleEvent(DevMove object, Object data) {
					if (run.getRunState() != RunState.INACTIVE) return;
					GUI.asyncExec(() ->
							updateBeamSizeDisplay(false)
					);
				}
			};
			DevEnergy.getDevice().llMoveStartStop.addListener(energyStopListener);

			addDisposeListener(new DisposeListener()
			{
				@Override public void widgetDisposed(DisposeEvent e)
				{
					DevPmacMotor.get("mo_En_E").removeListener(energyMoveListener);
					DevEnergy.getDevice().llMoveStartStop.removeListener(energyStopListener);
				}
			});
		}

		final WdgRasterRunTab runTab = this;
		addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent e)
			{
				Client.removeListener(runTab);
				run.llRunChanged.removeListener(runListener);
				run.getRunList().removeListener(runTab);
				ModelRasterState.get().llCurrentRun.removeListener(stateListener);
			}
		});
		
		updateVisibleState();
		
		return paramComp;
	}

	private void createCellSize(Composite parent)
	{
		cellSizeComp = new Composite(paramComp, SWT.NONE);
		cellSizeComp.setLayoutData(newGDEntry());
		cellSizeComp.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(3,false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		cellSizeComp.setLayout(gl);

		cellWidthEntry = new WdgControlEntry(cellSizeComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.grid().changeCellWidth_um((Double)value); 	}
			@Override public Object getDeviceValue() {
				if (!Util.beamSizeCalculated()) return null;
				BeamSize bs = DBBeamSize.getBeamSize((int)run.getBeamWidth_um(), DevEnergy.getDevice().getPosition());
				return bs.width_um;
			}
			@Override public Object getModelValue() 		{ return run.grid().getCellWidth_um();	 		}
			@Override public int getDecimals() 				{ return 0; 									}
			@Override public Limits getLimits()
			{
				double lowLimit = 2.0;
				double highLimit = 1000.0;
				String upperLimitDescription = "maximum cell width";

				double gridWidth = Math.ceil(run.grid().getMinGridWidth_um());
				if (gridWidth < highLimit && gridWidth > 1.0)
				{
					highLimit = gridWidth;
					upperLimitDescription = "selected grid width";
				}

				return new Limits(lowLimit, "minimum cell width", highLimit, upperLimitDescription);
			}
		});
		cellWidthEntry.setLayoutData(new GridData(SWT.FILL,SWT.DEFAULT,true,false));
		WidgetFactory.newLabel(cellSizeComp, SWT.NONE, "x").setLayoutData(new GridData());
		cellHeightEntry = new WdgControlEntry(cellSizeComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.grid().changeCellHeight_um((Double)value);}
			@Override public Object getModelValue() 		{ return run.grid().getCellHeight_um(); 		}
			@Override public Object getDeviceValue() {
				if (!Util.beamSizeCalculated()) return null;
				BeamSize bs = DBBeamSize.getBeamSize((int)run.getBeamWidth_um(), DevEnergy.getDevice().getPosition());
				return bs.height_um;
			}
			@Override public int getDecimals() 				{ return 0; 									}
			@Override public Limits getLimits()
			{
				double lowLimit = 2.0;
				double highLimit = 1000.0;
				String upperDescription = "maximum cell height";

				// upper limit should not be greater than grid height
				double gridHeight = Math.ceil(run.grid().getMinGridHeight_um());
				if (gridHeight < highLimit && gridHeight > 1.0)
				{
					highLimit = gridHeight;
					upperDescription = "selected grid height";
				}

				return new Limits(lowLimit, "minimum cell height",highLimit,upperDescription);
			}

			@Override public ValidateResult validate(String s)
			{
				// first do default validation - is it a number? is it within standard limits?
				ValidateResult defaultVR =  WdgControlEntry.defaultValidate(s,getLimits());
				if (!defaultVR.isValid) return defaultVR;

				return new ValidateResult(true);
			}
		});
		cellHeightEntry.setLayoutData(new GridData(SWT.FILL,SWT.DEFAULT,true,false));
	}

	private void createBeamSizeDropdown(Composite parent)
	{
		beamsizeCombo = new Combo(parent, SWT.READ_ONLY);
		beamsizeCombo.setLayoutData(newGDEntry());
		for (int i=0;i<DBBeamSize.colli_sizes.length;i++) beamsizeCombo.add("");
		updateBeamSizeDisplay(false);
		beamsizeCombo.select(0);
		beamsizeCombo.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent selectionEvent)
			{
				int selected = beamsizeCombo.getSelectionIndex();
				int beamsize = DBBeamSize.colli_sizes[selected];
				if (beamsize != run.getBeamWidth_um()) {
					run.dev().setBeamWidth_um(beamsize, false);
					run.dev().setBeamHeight_um(beamsize, false);

					BeamSize bs = DBBeamSize.getBeamSize(beamsize, DevPmacMotor.get("mo_En_E").getPosition());
					run.dev().setCellWidth_um(bs.width_um, false);
					run.dev().setCellHeight_um(bs.height_um, false);
					run.grid().getGridCalc().fitGridToContour(false, true); // sends update
				}
			}
			@Override public void widgetDefaultSelected(SelectionEvent selectionEvent) {}
		});
	}

	private void createBeamSizeFields(Composite parent)
	{
		beamSizeComp = new Composite(paramComp, SWT.NONE);
		beamSizeComp.setLayoutData(newGDEntry());
		beamSizeComp.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(3,false);
		gl.marginHeight=0;
		gl.marginWidth=0;
		beamSizeComp.setLayout(gl);
		beamWidthEntry = new WdgControlEntry(beamSizeComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.grid().changeBeamWidth_um((Double)value, true); 	}
			@Override public Object getModelValue() 		{ return run.getBeamWidth_um(); 		}
			@Override public int getDecimals() 				{ return 0; 									}
			@Override public Limits getLimits()
			{
				double lowLimit = 5.0;
				double highLimit = ModelBeamsize.getActualBeamsize_um().x;
				String upperLimitDescription = "maximum beam width";

				double gridWidth = run.grid().getMinGridWidth_um();
				if (gridWidth < highLimit && gridWidth > 1.0)
				{
					highLimit = gridWidth;
					upperLimitDescription = "selected grid width";
				}

				return new Limits(lowLimit, "minimum beam width", highLimit, upperLimitDescription);
			}
		});
		beamWidthEntry.setLayoutData(new GridData(SWT.FILL,SWT.DEFAULT,true,false));
		WidgetFactory.newLabel(beamSizeComp, SWT.NONE, "x").setLayoutData(new GridData());
		beamHeightEntry = new WdgControlEntry(beamSizeComp, SWT.NONE, EntryType.TEXT, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)	{ run.grid().changeBeamHeight_um((Double)value, true);}
			@Override public Object getModelValue() 		{ return run.getBeamHeight_um(); 		}
			@Override public int getDecimals() 				{ return 0; 									}
			@Override public Limits getLimits()
			{
				double lowLimit = 5.0;
				double highLimit = ModelBeamsize.getActualBeamsize_um().y;
				String upperDescription = "maximum beam height";

				// upper limit should not be greater than grid height
				double gridHeight = run.grid().getMinGridHeight_um();
				if (gridHeight < highLimit && gridHeight > 1.0)
				{
					highLimit = gridHeight;
					upperDescription = "selected grid height";
				}

				return new Limits(lowLimit, "minimum beam height",highLimit,upperDescription);
			}

			@Override public ValidateResult validate(String s)
			{
				// first do default validation - is it a number? is it within standard limits?
				ValidateResult defaultVR =  WdgControlEntry.defaultValidate(s,getLimits());
				if (!defaultVR.isValid) return defaultVR;
				return new ValidateResult(true);
			}
		});
		beamHeightEntry.setLayoutData(new GridData(SWT.FILL,SWT.DEFAULT,true,false));
	}

	// This can be removed by using JFace's GridDataFactory.copy()
	protected GridData newGDEntry()
	{
		GridData gd = new GridData();
		gd.widthHint = 260;
		gd.grabExcessHorizontalSpace = true;
		return gd;
	}
	protected GridData newGDLeftLabel()
	{
		GridData gd = new GridData();
		gd.widthHint = isAuto?130:115;
		gd.horizontalSpan = 2;
		return gd;
	}
	protected GridData newGDRightLabel()
	{
		GridData gd = new GridData();
		gd.widthHint = isAuto?70:70;
		gd.horizontalSpan = 2;
		return gd;
	}
	
	public void gridChanged(final ModelRasterGrid grid)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			if (isDisposed()) return;
			
    		if (null != cellWidthEntry) cellWidthEntry.setText(grid.getCellWidth_um());
    		if (null != cellHeightEntry) cellHeightEntry.setText(grid.getCellHeight_um());
		}});
	}
	
	private boolean manualControlsVisible = true, resLimitsVisible = true;

	public void updateBeamSizeDisplay(boolean updateCellSize)
	{
		ArrayList<BeamSize> sizes = DBBeamSize.getBeamSizes(DevEnergy.getDevice().getPosition());
		for (int i=0;i<sizes.size();i++) {
			BeamSize size = sizes.get(i);
			String text = Util.sprintf("%d x %d", Math.round(size.width_um), Math.round(size.height_um));
			beamsizeCombo.setItem(i, text);
		}
		int colli_index = updateBeamsizeDropdownSelect();
		if (updateCellSize) {
			run.dev().setCellWidth_um(Math.round(sizes.get(colli_index).width_um), false);
			run.dev().setCellHeight_um(Math.round(sizes.get(colli_index).height_um), true);
		}
		if (null!=cellWidthEntry) cellWidthEntry.updateHighlight();
		if (null!=cellHeightEntry) cellHeightEntry.updateHighlight();
	}

	private int updateBeamsizeDropdownSelect()
	{
		int colli_index = DBBeamSize.getColliIndex((int)run.getBeamWidth_um());
		if (beamsizeCombo.getSelectionIndex()!=colli_index) beamsizeCombo.select(colli_index);
		return colli_index;
	}

	protected void updateVisibleState()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			boolean showResLimits = DevSFType.get(DevSFType.SFRunType.RASTER)._getConfig().id != DevSFType.SPOTFINDER_TYPE.DOZOR;
			if (run.getRunType() != ModelRasterRun.RunType.DIFFRACTION) showResLimits = false;
			if (showResLimits != resLimitsVisible) {
				WdgControl.setControlVisible(lblResLimit, showResLimits);
				WdgControl.setControlVisible(resLimitsComp, showResLimits);
				WdgControl.setControlVisible(resLimitsButtonComp, showResLimits);
				WdgControl.setControlVisible(checkboxFixLimits, showResLimits);
				layout(true);
				resLimitsVisible = showResLimits;
			}
		}});
	}
	
	protected void updateEnableState()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			if (isDisposed()) return;
			VideoState state = TabRaster.get().cam.model.getState();
			
    		// enable flags
    		boolean gEnable =
    		(
    			Client.isMaster() &&
    			run.getRunState() == RunState.INACTIVE &&
    			(VideoState.POINT_AND_CLICK == state || VideoState.VIEW_IMAGES == state)
    		);
    		boolean enableGridControls = gEnable && !run.getAutoFind();
    		boolean enableResLimits =
				Client.isMaster() &&
				run.getRunState()!=RunState.RASTERING;

    		// sonicc controls
			if (null!=binButton) binButton.setEnabled(Client.isMaster()&&!run.getSONICCBin());
			if (null!=unbinButton) unbinButton.setEnabled(Client.isMaster()&&run.getSONICCBin());

    		// non-grid controls
    		for (WdgControl control : nonGridControls)
    		{
    			if (!control.isDisposed()) control.setEnabled(gEnable);
    		}
    		if (null != lowResLimitEntry) lowResLimitEntry.setEnabled(enableResLimits);
    		if (null != highResLimitEntry) highResLimitEntry.setEnabled(enableResLimits);
    		
    		if (null != wdgMacro && !wdgMacro.isDisposed())	wdgMacro.setEnabled(gEnable);
    		if (null != checkboxFixLimits && !checkboxFixLimits.isDisposed()) checkboxFixLimits.setEnabled(gEnable);
    		
    		// grid controls
    		for (WdgControl control : gridControls)
    		{
    			if (!control.isDisposed()) control.setEnabled(enableGridControls);
    		}
    		if (null!=beamsizeCombo && !beamsizeCombo.isDisposed()) { beamsizeCombo.setEnabled(enableGridControls); }
    		if (!btnAddContour.isDisposed()) 	btnAddContour.setEnabled(enableGridControls);
    		if (!btnRemoveContour.isDisposed()) btnRemoveContour.setEnabled(enableGridControls);
    		if (!btnClearContours.isDisposed()) btnClearContours.setEnabled(enableGridControls);
		}});
	}
	
	protected void updateOtherFields()
	{
		if (null != lblContourCount) lblContourCount.setText(Util.intToWord(run.grid().getContours().size()));
		
		if (null != timeEntry) timeEntry.setText(run.getExposureTime());
		if (null != deltaEntry) deltaEntry.setText(run.getDelta());
		if (null != roi) roi.setSelection(run.getROIMode());
		if (null != loopMeasurementEntry) loopMeasurementEntry.setText(run.getLoopMeasurement_um());
		
		if (null != controlDir) controlDir.setPrefix(run.getFileRoot());
		if (null != controlDir) controlDir.setDir(run.getDirectorySuffix());
		
		if (null != distanceEntry) distanceEntry.setText(run.getDetectorDist());
		if (null != attenuationEntry) attenuationEntry.setText(run.getAttenuation());
		if (null != energyEntry) energyEntry.setText(run.getEnergy());
		
		if (null != autoFindCheckbox) autoFindCheckbox.setSelected(run.getAutoFind());
		if (null != loopCenteredCheckbox) loopCenteredCheckbox.setSelected(run.getLoopCentered());
		
		if (null != lowResLimitEntry) lowResLimitEntry.setText(run.getLowResLimit_A());
		if (null != highResLimitEntry) highResLimitEntry.setText(run.getHighResLimit_A());
		if (null != checkboxFixLimits) checkboxFixLimits.setSelection(ModelRasterState.get().getFixLimits());
		
		//if (null != wdgMacro) wdgMacro.updateState(run.getRunState());
		
		updateVisibleState();
		updateEnableState();
	}
	
	protected void updateCellAndBeamsize(ModelRasterRun run)
	{
		boolean hideSizes =
		(
			run.getRunIndex()==0 &&
			run.getRunList().getListType()==RasterListType.AUTO &&
			run.getAutoFind()
		);
		
		Object oBeamX = hideSizes?"auto":run.getBeamWidth_um();
		Object oBeamY = hideSizes?"auto":run.getBeamHeight_um();
		Object oCellX = hideSizes?"auto":run.grid().getCellWidth_um();
		Object oCellY = hideSizes?"auto":run.grid().getCellHeight_um();
		
		if (null != beamWidthEntry) 	beamWidthEntry.setText	(oBeamX);
		if (null != beamHeightEntry) 	beamHeightEntry.setText (oBeamY);
		if (null != beamsizeCombo) 		updateBeamSizeDisplay(false);

		if (null != cellWidthEntry) 	cellWidthEntry.setText	(oCellX);
		if (null != cellWidthEntry) 	cellHeightEntry.setText	(oCellY);
	}

	protected void updateCellAndBeamsize()
	{
		updateCellAndBeamsize(run);
	}
	
	protected void handleRunChange(final ModelRasterRun run)
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			if (isDisposed()) return;
			
			updateCellAndBeamsize();
			updateOtherFields();
			if (null!=cellWidthEntry) cellWidthEntry.updateHighlight();
			if (null!=cellHeightEntry) cellHeightEntry.updateHighlight();
		}});
	}
}
