package gov.anl.gmca.raster.ui;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.raster.ModelRasterSiteList;
import gov.anl.gmca.raster.ModelRasterSiteList.Site;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;
import java.util.HashMap;

public class WdgRasterSiteList extends Composite implements ListenerList.Listener<ModelRasterSiteList>
{
	public static class SiteProvider implements ILazyContentProvider
	{
		private TableViewer viewer;
		private ArrayList<Site> frames;
		
		public SiteProvider(TableViewer viewer) { this.viewer=viewer; }
		@Override public void dispose() {}
		@SuppressWarnings("unchecked")
		@Override public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
			frames = (ArrayList<Site>)newInput;
		}

		@Override public void updateElement(int index)
		{
			viewer.replace(frames.get(index), index);
		}
	}
	
	public static enum ListType { COLLECT_LIST, COLLECT_SELECTED, RASTER_LIST };
	public ListenerList<WdgRasterSiteList> llButtonClicked;
	private ListType lt;
	private Object[][] headers;
	private TableViewer tableViewer;
	private ArrayList<Site> siteList;
	public ArrayList<Site> getSiteList()
	{
		if (Thread.currentThread()==GUI.getDisplay().getThread()) return siteList;
		throw new IllegalStateException("Must be called from GUI thread");
	}
	private ModelCollectRun run;
	protected Table table;
	public Table getTable() { return table; }
	private HashMap<Button,Site> buttonSiteMap = new HashMap<>();
	private HashMap<Object,Button> buttonMap = new HashMap<>();
	private HashMap<Object,TableEditor> editorMap = new HashMap<>();
	
	public WdgRasterSiteList(Composite parent, int style, Object[][] headers, ListType lt, ModelCollectRun run)
	{
		super(parent, style);
		this.headers=headers;
		this.lt=lt;
		this.run=run;
		this.llButtonClicked = new ListenerList<>(this);
		
		createWidgets();
		addControlListener(new ControlAdapter()
		{
			@Override public void controlResized(ControlEvent ce)
			{
				redistributeColumns();
			}
		});
		addDisposeListener(new DisposeListener()
		{
			@Override public void widgetDisposed(DisposeEvent e)
			{
				for (Button b : buttonMap.values()) b.dispose(); buttonMap = new HashMap<>();
				for (TableEditor _e : editorMap.values()) _e.dispose(); editorMap = new HashMap<>();
				buttonSiteMap = new HashMap<>();
				table.dispose();
				table = null;
			}
		});
		
		ModelRasterSiteList.get().ll.addListener(this);
		updateResults();
	}
	
	@Override public void setMenu(Menu menu)
	{
		table.setMenu(menu);
	}
	
	private void createWidgets()
	{
		setLayout(new FillLayout());
		
		tableViewer = new TableViewer(this, SWT.VIRTUAL|SWT.MULTI);
		table = tableViewer.getTable();
		table.setFont(GUI.FONT_DEFAULT);
		for (int i=0;i<headers.length;i++)
		{
		    TableViewerColumn viewerColumn = new TableViewerColumn(tableViewer,SWT.CENTER);
			TableColumn column = viewerColumn.getColumn();
			column.setText((String)headers[i][0]);
			column.setWidth((Integer)headers[i][1]);
			column.setResizable(true);
			column.setMoveable(true);
			final int columnIndex=i;
			viewerColumn.setLabelProvider(new CellLabelProvider()
			{
		        @Override public void update(ViewerCell cell)
		        {
		        	Site site = (Site)cell.getElement();
		        	String cellText = null;
		        	switch (columnIndex)
		        	{
		        	case 0:
		        	{
		        		if (lt==ListType.RASTER_LIST)
		        			cellText = ""+site.index;
		        		else
		        			updateButton(cell, site);
		        		break;
		        	}
		        	case 1:	cellText = site.getSource().id; break;
		        	case 2:	cellText = site.getSizeString(); break;
		        	default: throw new IllegalStateException();
		        	}
		        	if (null != cellText) cell.setText(cellText);
			    }
			});
		}
		table.setHeaderVisible(true);
		tableViewer.setContentProvider(new SiteProvider(tableViewer));
	}

	private void updateButton(ViewerCell cell, Site site)
	{
		TableItem item = (TableItem)cell.getItem();
		Button button;
        if(buttonMap.containsKey(cell.getElement()))
        {
        	button = buttonMap.get(cell.getElement());
        }
        else
        {
        	button = new Button((Composite)cell.getViewerRow().getControl(),SWT.NONE);
        	button.addSelectionListener(new SelectionListener()
        	{
				@Override public void widgetSelected(SelectionEvent arg0)
				{
					llButtonClicked.notifyListeners(site);
				}
				@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			});
        	buttonMap.put(cell.getElement(), button);
        	buttonSiteMap.put(button, site);

        	boolean siteSelected = run.isSiteSelected(site.index);
        	switch (lt)
        	{
        	case COLLECT_LIST:
        		button.setEnabled(tableEnabled && !siteSelected);
        		button.setText("Select");
        		break;
        	case COLLECT_SELECTED:
        		button.setEnabled(tableEnabled);
        		button.setText("Deselect");
        		break;
        	case RASTER_LIST:
        		break;
        	}
        }
        GUI.asyncExec(new Runnable(){public void run()
        {
	        TableEditor editor = new TableEditor(item.getParent());
	        editor.grabHorizontal = true;
	        editor.grabVertical = true;
	        editor.setEditor(button, item, cell.getColumnIndex());
	        //editor.layout();
	        editorMap.put(cell.getElement(), editor);
        }});
	}
	
	private boolean tableEnabled = false;
	@Override public void setEnabled(boolean enable)
	{
		tableEnabled = enable;
		for (Button b : buttonMap.values()) b.setEnabled(enable);
	}
	
	@Override public void handleEvent(final ModelRasterSiteList _siteList, final Object data)
	{
		updateResults();
	}
	
	public void updateResults()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			if (null == table) return;
			
			// dispose of table buttons
			for (Button button : buttonMap.values()) button.dispose(); buttonMap = new HashMap<>();
			for (TableEditor _e : editorMap.values()) _e.dispose(); editorMap = new HashMap<>();
			buttonSiteMap = new HashMap<>();
			
			// get the current list of sites
			if (lt==ListType.COLLECT_SELECTED)
				siteList = run.getSelectedRasterSiteObjects();
			else
			{
				siteList = ModelRasterSiteList.get().getSiteListCopy();
			
				/*if (lt==ListType.COLLECT_LIST)
				{
					String s1 = "Loaded "+siteList.size()+" sites";
					for (Site site : siteList)
					{
						s1 += " "+(null==site?"null":site.index);
					}
					Log.note(s1);
				}*/
			}
			
			// send the list to the tableviewer
			tableViewer.setItemCount(siteList.size());
			tableViewer.setInput(siteList);
			tableViewer.refresh();
			
			// update button enable/disable
			for (Button b : buttonSiteMap.keySet())
			{
				Site s = buttonSiteMap.get(b);
				b.setEnabled(run.isSiteSelected(s.index));
			}
		}});
	}
	
	private void redistributeColumns()
	{
		int totalWidth = getSize().x-15;
		
		if (null==siteList || siteList.size()==0)
		{
			int w0 = (Integer)headers[0][1], w1 = (Integer)headers[1][1];
			table.getColumn(0).setWidth(w0);
			table.getColumn(1).setWidth(w1);
			table.getColumn(2).setWidth(totalWidth-w0-w1);
			return;
		}
		
		// update source and size column widths
		int sourceMaxPx=1, sizeMaxPx=1;
		GC gc = new GC(this);
		for (Site site : siteList)
		{
			int sourceWidthPx = 0+10;
			sourceMaxPx = Math.max(sourceMaxPx, gc.textExtent(""+sourceWidthPx).x);
			int sizeWidthPx = 0+10;
			sizeMaxPx = Math.max(sizeMaxPx, gc.textExtent(""+sizeWidthPx).x);
		}
		gc.dispose();
		
		// for wide layout
		int totalStandardWidth = 0;
		for (Object[] colData : headers) { totalStandardWidth += (Integer)colData[1]; }
		double multiplier = (double)totalWidth / (double)totalStandardWidth;
		
		int w0 = (Integer)headers[0][1], w1 = (Integer)headers[1][1];
		table.getColumn(0).setWidth(w0);
		table.getColumn(1).setWidth(w1);
		table.getColumn(2).setWidth(totalWidth-w0-w1);
	}
}
