package gov.anl.gmca.sample;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.DevMove;
import gov.anl.gmca.common.ui.CCombo;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevAlignMinibeam;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.PVLookup;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.StringTokenizer;

public class CRLDialog {

    public Shell messageDialog = null;
    Label crlhPos, crlvPos, crlxPos, lastAlignLbl;
    CCombo comboMove, comboMoveX;
    Button moveUpButton, moveDownButton, moveLeftButton, moveRightButton, focusNegButton, focusPosButton;
    boolean vSynced=false, hSynced=false;

    public CRLDialog (Shell _parent, int style)
    {
        messageDialog = new Shell(_parent, SWT.DIALOG_TRIM);
        messageDialog.setText("CRL controls");
        GridLayout gl = new GridLayout(2,false);
        gl.marginWidth = 10;
        gl.marginHeight = 10;
        gl.verticalSpacing = 10;
        messageDialog.setLayout(gl);

        int buttonWidth=27, buttonHeight=27;

        Group moveGroup = new Group(messageDialog, SWT.NONE);
        moveGroup.setText("Move CRL H/V");
        moveGroup.setBackground(messageDialog.getBackground());
        moveGroup.setLayout(new GridLayout(2,false));
        moveGroup.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false));

        comboMove = new CCombo(moveGroup,SWT.None);
        comboMove.setItems(new String[]{"1.0", "2.0", "5.0", "10.0", "20.0", "50.0", "100.0"});
        comboMove.setText("1.0");
        comboMove.setFont(GUI.FONT_DEFAULT);
        comboMove.setBackground(GUI.COLOR_WHITE);
        GridData gd = new GridData();
        gd.minimumWidth = 200;
        gd.horizontalAlignment = SWT.FILL;
        comboMove.setLayoutData(gd);
        comboMove.addListener (SWT.Verify, new Listener () {
            public void handleEvent (Event e) {
                Util.allowOnlyAlpha(e);
            }
        });
        Label lblUnits = new Label(moveGroup,SWT.None);
        lblUnits.setText("\u03BCm");
        lblUnits.setBackground(messageDialog.getBackground());

        Composite moveComp = new Composite(moveGroup, SWT.NONE);
        moveComp.setSize(buttonWidth*3, buttonHeight*3);
        moveComp.setBackground(messageDialog.getBackground());
        moveComp.setLayoutData(new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false, 2, 1));

        moveUpButton = new Button(moveComp, SWT.ARROW|SWT.UP);
        moveUpButton.setBackground(GUI.COLOR_BLUE_BUTTON);
        moveUpButton.setSize(buttonWidth,buttonHeight);
        moveUpButton.setLocation(buttonWidth,0);

        moveUpButton.addSelectionListener(new SelectionListener()
        {
            @Override public void widgetSelected(SelectionEvent e)
            {
                handleArrows(0,1);
            }
            @Override public void widgetDefaultSelected(SelectionEvent e) {}
        });

        moveDownButton = new Button(moveComp, SWT.ARROW|SWT.DOWN);
        moveDownButton.setBackground(GUI.COLOR_BLUE_BUTTON);
        moveDownButton.setSize(buttonWidth,buttonHeight);
        moveDownButton.setLocation(buttonWidth,buttonHeight*2);
        moveDownButton.addSelectionListener(new SelectionListener()
        {
            @Override public void widgetSelected(SelectionEvent e)
            {
                handleArrows(0,-1);
            }
            @Override public void widgetDefaultSelected(SelectionEvent e) {}
        });

        moveLeftButton = new Button(moveComp, SWT.ARROW|SWT.LEFT);
        moveLeftButton.setBackground(GUI.COLOR_BLUE_BUTTON);
        moveLeftButton.setSize(buttonWidth,buttonHeight);
        moveLeftButton.setLocation(0,buttonHeight);
        moveLeftButton.addSelectionListener(new SelectionListener()
        {
            @Override public void widgetSelected(SelectionEvent e)
            {
                handleArrows(1,0);
            }
            @Override public void widgetDefaultSelected(SelectionEvent e) {}
        });

        moveRightButton = new Button(moveComp, SWT.ARROW|SWT.RIGHT);
        moveRightButton.setBackground(GUI.COLOR_BLUE_BUTTON);
        moveRightButton.setSize(buttonWidth,buttonHeight);
        moveRightButton.setLocation(buttonWidth*2,buttonHeight);
        moveRightButton.addSelectionListener(new SelectionListener()
        {
            @Override public void widgetSelected(SelectionEvent e) { handleArrows(-1,0); }
            @Override public void widgetDefaultSelected(SelectionEvent e) {}
        });

        Group posComp = new Group(messageDialog, SWT.NONE);
        posComp.setText("Positions");
        posComp.setBackground(messageDialog.getBackground());
        gd = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 2);
        posComp.setLayoutData(gd);
        posComp.setLayout(new GridLayout());

        crlhPos = new Label(posComp, SWT.NONE);
        crlhPos.setFont(GUI.FONT_10_FIXED);
        crlhPos.setText("CRL_H:     ---     ");

        crlvPos = new Label(posComp, SWT.NONE);
        crlvPos.setFont(GUI.FONT_10_FIXED);
        crlvPos.setText("CRL_V:     ---     ");

        crlxPos = new Label(posComp, SWT.NONE);
        crlxPos.setFont(GUI.FONT_10_FIXED);
        crlxPos.setText("CRL_X:     ---     ");

        lastAlignLbl = new Label(posComp, SWT.NONE);
        lastAlignLbl.setFont(GUI.FONT_10_FIXED);
        lastAlignLbl.setText("\nLast alignment:\n\n");

        Label infoLbl = new Label(posComp, SWT.NONE);
        infoLbl.setFont(GUI.FONT_10_FIXED);
        infoLbl.setText(
            "\n"+
            "< Outboard\n"+
            "> Inboard\n"+
            "- Upstream\n"+
            "+ Downstream");

        Group focusGroup = new Group(messageDialog, SWT.NONE);
        focusGroup.setText("Move CRL Focus");
        focusGroup.setBackground(messageDialog.getBackground());
        focusGroup.setLayout(new GridLayout(2,false));
        focusGroup.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false));

        comboMoveX = new CCombo(focusGroup,SWT.None);
        comboMoveX.setItems(new String[]{"1.0", "2.0", "5.0", "10.0", "20.0"});
        comboMoveX.setText("1.0");
        comboMoveX.setFont(GUI.FONT_DEFAULT);
        comboMoveX.setBackground(GUI.COLOR_WHITE);
        gd = new GridData();
        gd.minimumWidth = 200;
        gd.horizontalAlignment = SWT.FILL;
        comboMoveX.setLayoutData(gd);
        comboMoveX.addListener (SWT.Verify, new Listener () {
            public void handleEvent (Event e) {
                Util.allowOnlyAlpha(e);
            }
        });
        Label lblUnitsX = new Label(focusGroup,SWT.None);
        lblUnitsX.setText("mm");
        lblUnitsX.setBackground(messageDialog.getBackground());

        Composite moveCompX = new Composite(focusGroup, SWT.NONE);
        moveCompX.setLayout(new GridLayout(2,false));
        moveCompX.setBackground(messageDialog.getBackground());
        moveCompX.setLayoutData(new GridData(SWT.DEFAULT, SWT.DEFAULT, false, false, 1, 1));

        focusNegButton = new Button(moveCompX, SWT.NONE);
        focusNegButton.setText("-");
        focusNegButton.setBackground(GUI.COLOR_BLUE_BUTTON);
        focusNegButton.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
        focusNegButton.addSelectionListener(new SelectionListener()
        {
            @Override public void widgetSelected(SelectionEvent e)
            {
                handleFocus(-1);
            }
            @Override public void widgetDefaultSelected(SelectionEvent e) {}
        });

        focusPosButton = new Button(moveCompX, SWT.NONE);
        focusPosButton.setText("+");
        focusPosButton.setBackground(GUI.COLOR_BLUE_BUTTON);
        focusPosButton.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
        focusPosButton.addSelectionListener(new SelectionListener()
        {
            @Override public void widgetSelected(SelectionEvent e)
            {
                handleFocus(1);
            }
            @Override public void widgetDefaultSelected(SelectionEvent e) {}
        });

        /////////////////////////////////////////////////// Listeners ///////////////////////////////////////////////

        ListenerList.Listener<DevMove> motorListener = (a1,a2)->updateWidgets();
        ClientListener clientListener = (isMaster) -> updateWidgets();
        Epics.EpicsValueMonitor historyMonitor = new Epics.EpicsValueMonitor()
        {
            @Override
            public void valueChanged(String pvName, DBR value)
            {
                updateWidgets();
            }
        };
        Epics.addMonitor(PVLookup.PV_BLUICE.AlignBeamHistory, historyMonitor);
        Client.addListener(clientListener);
        DevPmacMotor.get("crl_h").llMoveStartStop.addListener(motorListener);
        DevPmacMotor.get("crl_v").llMoveStartStop.addListener(motorListener);
        DevPmacMotor.get("crl_x").llMoveStartStop.addListener(motorListener);
        updateWidgets();

        messageDialog.pack();
        messageDialog.open();
        messageDialog.addDisposeListener((a)->{
            Epics.removeMonitor(PVLookup.PV_BLUICE.AlignBeamHistory, historyMonitor);
            Client.removeListener(clientListener);
            DevPmacMotor.get("crl_h").llMoveStartStop.removeListener(motorListener);
            DevPmacMotor.get("crl_v").llMoveStartStop.removeListener(motorListener);
            DevPmacMotor.get("crl_x").llMoveStartStop.removeListener(motorListener);
        });
    }

    public void updateWidgets()
    {
        GUI.asyncExec(()-> {

            boolean hEnable = Client.isMaster() && !DevPmacMotor.get("crl_h").isMoving();
            boolean vEnable = Client.isMaster() && !DevPmacMotor.get("crl_v").isMoving();
            boolean xEnable = Client.isMaster() && !DevPmacMotor.get("crl_x").isMoving();
            moveUpButton.setEnabled(vEnable);
            moveDownButton.setEnabled(vEnable);
            moveLeftButton.setEnabled(hEnable);
            moveRightButton.setEnabled(hEnable);
            focusNegButton.setEnabled(xEnable);
            focusPosButton.setEnabled(xEnable);

            String txt = String.format("CRL_H: %8.4f mm", DevPmacMotor.get("crl_h").getPosition());
            crlhPos.setText(txt);
            String txt2 = String.format("CRL_V: %8.4f mm", DevPmacMotor.get("crl_v").getPosition());
            crlvPos.setText(txt2);
            String txt3 = String.format("CRL_X: %8.1f mm", DevPmacMotor.get("crl_x").getPosition());
            crlxPos.setText(txt3);

            String alignString = Epics.getString(PVLookup.PV_BLUICE.AlignBeamHistory);
            String hString="", vString="";
            StringTokenizer st = new StringTokenizer(alignString, " ");
            while (st.hasMoreTokens()) {
                String t = st.nextToken();
                if (t.startsWith("H=")) hString = t;
                else if (t.startsWith("V=")) vString = t;
            }

            lastAlignLbl.setText(
                "\nLast alignment:\n"+
                hString+"\n"+
                vString
            );
        });
    }

    public void handleArrows (int xScale, int yScale) {
        try {
            double moveSize = Double.parseDouble(comboMove.getText()) / 1000.0;
            String motor;
            if (xScale != 0) {
                motor = "crl_h";
                moveSize *= xScale;
            } else {
                motor = "crl_v";
                moveSize *= yScale;
            }
            double moveSize_f = moveSize;
            if (xScale != 0 && !hSynced) {
                hSynced = true;
                new Thread(()->{
                    DevPmacMotor.get(motor).syncAssembly();
                    DevPmacMotor.get(motor).startRelativeMove(moveSize_f);
                },"crl_h sync and move").start();
            } else if (yScale != 0 && !vSynced) {
                vSynced = true;
                new Thread(()->{
                    DevPmacMotor.get(motor).syncAssembly();
                    DevPmacMotor.get(motor).startRelativeMove(moveSize_f);
                },"crl_v sync and move").start();
            } else {
                DevPmacMotor.get(motor).startRelativeMove(moveSize);
            }
        } catch (NumberFormatException nfe) {
            Util.asyncPopupError("Invalid input", String.format("'%s' is not a valid step size", comboMove.getText()));
        }
    }

    public void handleFocus(int scale) {
        try {
            double moveSize = (Double.parseDouble(comboMoveX.getText()) / 1.0) * scale;
            DevPmacMotor.get("crl_x").startRelativeMove(moveSize);
        } catch (NumberFormatException nfe) {
            Util.asyncPopupError("Invalid input", String.format("'%s' is not a valid step size", comboMoveX.getText()));
        }
    }
}
