package gov.anl.gmca.sample;

import com.google.common.primitives.Doubles;
import gov.anl.gmca.common.*;
import gov.anl.gmca.io.hw.Database;
import gov.anl.gmca.util.Util;
import org.apache.commons.math3.analysis.interpolation.LinearInterpolator;
import org.apache.commons.math3.analysis.polynomials.PolynomialSplineFunction;
import org.jdbi.v3.core.Jdbi;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

public class DBBeamSize
{
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////// public interface /////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public static class BeamSize
    {
        public double energy_keV;
        public int colli_um;
        public double width_um, height_um;
        public BeamSize(double a, int b, double c, double d) { energy_keV=a; colli_um=b; width_um=c; height_um=d; }
        public BeamSize(){}
    }

    // uses linear interpolation
    public static BeamSize getBeamSize(int colli_um, double energy_keV)
    {
        int colli_index = getColliIndex(colli_um);
        if (-1==colli_index) return null;
        energy_keV = Doubles.constrainToRange(energy_keV, minEnergy_keV, maxEnergy_keV);

        BeamSize ret = new BeamSize();
        ret.energy_keV = energy_keV;
        ret.colli_um = colli_um;
        ret.width_um = width_interp[colli_index].value(energy_keV);
        ret.height_um = height_interp[colli_index].value(energy_keV);
        return ret;
    }

    public static ArrayList<BeamSize> getBeamSizes(double energy_keV)
    {
        ArrayList<BeamSize> ret = new ArrayList<>();
        for (int colli_size_um : colli_sizes) {
            ret.add(getBeamSize(colli_size_um, energy_keV));
        }
        return ret;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////// end public interface ///////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public static class BEAM_SIZE_MEASUREMENT implements Serializable
    {
        private Double energy_keV;
        private int colli_um;
        private Double width_um, height_um;

        public Double getEnergy_keV() { return energy_keV; }
        public void setEnergy_keV(Double energy_keV) { this.energy_keV = energy_keV; }

        public Integer getColli_um() { return colli_um; }
        public void setColli_um(Integer colli_um) { this.colli_um = colli_um; }

        public Double getWidth_um() { return width_um; }
        public void setWidth_um(Double width_um) { this.width_um = width_um; }

        public Double getHeight_um() { return height_um; }
        public void setHeight_um(Double height_um) { this.height_um = height_um; }
    }

    private static List<BEAM_SIZE_MEASUREMENT> measurements = new ArrayList<>();

    public static int[] colli_sizes = {100,20,10,5};
    public static int getColliIndex(int colli_um) {
        int colli_index = -1;
        for (int i=0;i<colli_sizes.length;i++) {
            if (colli_sizes[i]==colli_um) {
                colli_index=i; break;
            }
        }
        return colli_index;
    }

    private static PolynomialSplineFunction[] width_interp = new PolynomialSplineFunction[4];
    private static PolynomialSplineFunction[] height_interp = new PolynomialSplineFunction[4];

    private static double minEnergy_keV = Double.MAX_VALUE, maxEnergy_keV = Double.MIN_VALUE;
    public static void init()
    {
        String sqlUrl = DBConfig.sim5 ? "jdbc:sqlite:"+Database.sqlLitePath : "jdbc:mysql://"+ Database.getSqlHost()+"/blc2004?user=dhs";
        Jdbi jdbi = Jdbi.create(sqlUrl);
        measurements = jdbi.withHandle(handle -> {
            return handle.createQuery("select * from BeamSize order by energy_keV")
                .mapToBean(BEAM_SIZE_MEASUREMENT.class)
                .list();
        });

        for (BEAM_SIZE_MEASUREMENT m : measurements)
        {
            if (m.energy_keV < minEnergy_keV) minEnergy_keV = m.energy_keV;
            if (m.energy_keV > maxEnergy_keV) maxEnergy_keV = m.energy_keV;
        }

        for (int i=0;i<4;i++)
        {
            ArrayList<Double> energies = new ArrayList<>(), widths = new ArrayList<>(), heights = new ArrayList<>();
            for (BEAM_SIZE_MEASUREMENT m : measurements)
            {
                if (m.colli_um==colli_sizes[i]) {
                    energies.add(m.energy_keV);
                    widths.add(m.width_um);
                    heights.add(m.height_um);
                }
            }

            width_interp[i] = new LinearInterpolator().interpolate(Util.toPrimitiveArray(energies), Util.toPrimitiveArray(widths));
            height_interp[i] = new LinearInterpolator().interpolate(Util.toPrimitiveArray(energies), Util.toPrimitiveArray(heights));
        }
    }
}
