package gov.anl.gmca.sample;

import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.syslog.Log;
import org.eclipse.swt.graphics.Point;

import java.util.HashMap;

public class DBCamera
{
	public static enum ZoomDirection { IN, OUT };
	
	/////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////// stat and def classes /////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////
	
    protected static class CameraStat
    {
    	public String url, ip;
    	public int width, height;
    	
    	public CameraStat(String _url, String _ip, int _width, int _height)
    	{
    		url = _url;
    		ip = _ip;
    		width = _width;
    		height = _height;
    	}
    }
    
    protected static class CameraDef
    {
    	public WdgCamera.CameraType type;
    	public WdgCamera.CameraSize size;
    	
    	public CameraDef(WdgCamera.CameraType _type, WdgCamera.CameraSize _size)
    	{
    		type = _type;
    		size = _size;
    	}
    	
    	@Override
    	public boolean equals(Object o)
    	{
    		if (!(o instanceof CameraDef)) return false;
    		
    		CameraDef rhs = (CameraDef)o;
    		if (type != rhs.type) return false;
    		if (size != rhs.size) return false;
    		return true;
    	}

    	@Override
    	public int hashCode() { return 0; }
    }
    
	/////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////

    protected static HashMap<CameraDef,CameraStat> cameraMap = new HashMap<CameraDef,CameraStat>();
    protected static HashMap<String,WdgCamera.CameraType> cameraTypeStrings = null;
    
    protected static String hutchControlURL;
	public static String getHutchControlURL() { return hutchControlURL; }

    public static Point getCameraDimensions(WdgCamera.CameraType type, WdgCamera.CameraSize size)
	{
		CameraStat cs = cameraMap.get(new CameraDef(type,size));
		if (null == cs) return null;

		int height=0;
		if (WdgCamera.CameraSize.SMALL == size) height = 240;
		if (WdgCamera.CameraSize.LARGE == size) height = 480;
		
		return new Point(cs.width, height);
	}
	
	public static String getImageURL(WdgCamera.CameraType type, WdgCamera.CameraSize size)
	{
		CameraStat cs = cameraMap.get(new CameraDef(type,size));
		if (null == cs) return null;
		return cs.url;
	}

	public static String getPointAndClickURL(WdgCamera.CameraType type, WdgCamera.CameraSize size, int x, int y)
	{
		CameraStat cs = cameraMap.get(new CameraDef(type,size));
		if (null == cs) return null;
		
		String url = "http://"+cs.ip+"/axis-cgi/com/ptz.cgi?imagewidth="+cs.width+"&imageheight="+cs.height+"&camera=1&center=?"+x+","+y;
		return url;
	}
	
	public static String getZoomURL(WdgCamera.CameraType type, WdgCamera.CameraSize size, ZoomDirection dir)
	{
		CameraStat cs = cameraMap.get(new CameraDef(type,size));
		if (null == cs) return null;
		
		int zoomAmount = dir==ZoomDirection.IN ? 2500 : -2500;
		String url = "http://"+cs.ip+"/axis-cgi/com/ptz.cgi?camera=1&rzoom="+zoomAmount;
		return url;
	}
	
	public static void update()
	{
		if (null == cameraTypeStrings)
		{
			cameraTypeStrings = new HashMap<String,WdgCamera.CameraType>();
			cameraTypeStrings.put("LowRes",WdgCamera.CameraType.LOWRES);
	        cameraTypeStrings.put("HighRes",WdgCamera.CameraType.HIGHRES);
	        cameraTypeStrings.put("Hutch",WdgCamera.CameraType.HUTCH);
			cameraTypeStrings.put("APSStatus",WdgCamera.CameraType.HUTCH);
		}
        
 		Result pmaResult = Database.exec(DBName.CONFIG, "select * from CameraIP");
    	for (int i=0;i<pmaResult.numRows();i++)
		{
			String name = pmaResult.getRow(i).get("name");
			String type = pmaResult.getRow(i).get("type");
			String ip = pmaResult.getRow(i).get("ip");
			
			int cameraNo=1;
			
			if (type.equals("axis1")) { cameraNo=1; type="axis"; }
			if (type.equals("axis2")) { cameraNo=2; type="axis"; }
			
			String baseURL=null,smallURL=null,largeURL=null;
			int smallWidth=0,largeWidth=0,smallHeight=0,largeHeight=0;
			
			if (type.equals("axis"))
			{
				baseURL = "http://"+ip+"/axis-cgi/jpg/image.cgi"
					+"?camera="+cameraNo
					+"&compression=25";
				smallURL=baseURL+"&resolution=352x240";
				largeURL=baseURL+"&resolution=704x480";
				smallWidth=352; smallHeight=240;
				largeWidth=704; largeHeight=480;
			}
			else if (type.contains("iqeye"))
			{
				baseURL = "http://"+ip+"/now.jpg";
				smallURL=baseURL+"?ds=8";
				largeURL=baseURL+"?ds=4";
				smallWidth=320; smallHeight=200;
				largeWidth=640; largeHeight=400; // TODO: verify the 200/400 heights
			}
			else
			{
				Log.error("Unknown camera type "+type);
				continue;
			}

			if (!cameraTypeStrings.containsKey(name))
			{
				Log.error("Error loading CameraIP table, unknown camera type "+name);
				continue;
			}

			WdgCamera.CameraType eType = cameraTypeStrings.get(name);
			cameraMap.put(new CameraDef(eType, WdgCamera.CameraSize.SMALL), new CameraStat(smallURL,ip,smallWidth,smallHeight));
			cameraMap.put(new CameraDef(eType, WdgCamera.CameraSize.LARGE), new CameraStat(largeURL,ip,largeWidth,largeHeight));
			
			// the hutch camera has a third URL, for changing positions
			hutchControlURL = "http://"+ip+"/axis-cgi/com/ptz.cgi";
		}
    }
}
