package gov.anl.gmca.sample;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevAlignMinibeam;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Shell;

public class PiezoDialog implements ClientListener {
	
	public Shell messageDialog = null;
	protected Shell parent = null;
	Thread alignmentThread = null;
	String Message = null;
	Button autoalignButton;
	GridData gd = null;
	TabSample peelOutTab;
	public PiezoDialog (Shell _parent, int style)
	{
		parent = _parent;
	}
	public void create()
	{
		Client.addListener(this);
        messageDialog = new Shell(parent, SWT.DIALOG_TRIM);
        messageDialog.setText("Piezo Mirror Controls");
        GridLayout gl = new GridLayout(2,false);
        gl.marginWidth = 10;
        gl.marginHeight = 10;
        gl.verticalSpacing = 10;
        messageDialog.setLayout(gl);
 	
	 	new WdgSampleSlider(messageDialog, SWT.NONE, "Vertical Mirror Piezo", "verticalpiezo",10);
	 	new WdgSampleSlider(messageDialog, SWT.NONE, "Horizontall Mirror Piezo","horizontalpiezo", 10);
		
	 	gd = new GridData();
	 	gd.horizontalSpan = 2;
		
	 	autoalignButton = new Button(messageDialog, SWT.PUSH);
	 	autoalignButton.setBackground(GUI.COLOR_BLUE_BUTTON);
	 	autoalignButton.setText("Optimize Beam");
	 	autoalignButton.setLayoutData(gd);
	 	if(Client.isMaster())
	 		autoalignButton.setEnabled(true);
	 	else
	 		autoalignButton.setEnabled(false);
	 	autoalignButton.setFont(GUI.FONT_SAMPLE_BUTTONS);
	 	autoalignButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) { }
			public void widgetSelected(SelectionEvent arg0) {
				if (null != alignmentThread && alignmentThread.isAlive())
				{
					Log.error("Alignment script is already running");
					return;
				}
				alignmentThread = new Thread(new Runnable(){public void run()
				{
					updateEnableState();
					
				    Log.note("Optimize minibeam script procedure started...");
				    DevAlignMinibeam.get().syncExec();
				    
			    	// TODO: find a better way to do this that doesn't potentially hold up the GUI thread
			    	GUI.asyncExec(new Runnable(){public void run()
			    	{
			    		while (null != alignmentThread && alignmentThread.isAlive()) Util.sleep(10);
			    		updateEnableState();
			    	}});
				}}, "optimize minibeam monitor");
				alignmentThread.start();
				}
	 	});
	 	messageDialog.pack();
		messageDialog.addShellListener(new ShellAdapter() {
            public void shellClosed(ShellEvent e) {
                e.doit = false;
                GUI.asyncExec(new Runnable(){public void run()
				{
                	//TabSample.
                	messageDialog.setVisible(false);
				}
				});
            }
		});
		
		// TODO: disableExceptions
		AppState.addListener(AppState.Operation.OP_ALIGN_MINIBEAM, new AppState.Listener()
	 	{
	 		@Override
			public void stateChanged(Operation op, boolean allowed) {
				updateEnableState();
			}
	 	});
        
	}
	public void updateEnableState()
	{
		// TODO: disableExceptions
		boolean isalignmentRunning = (null != alignmentThread && alignmentThread.isAlive());
		final boolean enable = (Client.isMaster() && !isalignmentRunning && AppState.isOperationAllowed(Operation.OP_ALIGN_MINIBEAM));
		GUI.asyncExec(new Runnable(){public void run()
		{
			autoalignButton.setEnabled(enable);
		}});
	}
	
	@Override
	public void masterChanged(boolean isMaster) {
		// TODO Auto-generated method stub
		
			updateEnableState();
	}
	
}
