/**
 * @author spothineni
 * 
 * Sample tab GUI is initated in this class
 */

package gov.anl.gmca.sample;

import gov.anl.gmca.common.*;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.EpicsDevice.State;
import gov.anl.gmca.common.ui.NBFileDialog;
import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.*;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.PVLookup.*;
import gov.anl.gmca.sample.DBBeamSize.BeamSize;
import gov.anl.gmca.sample.DevCollimatorCtrl.CCMove;
import gov.anl.gmca.sample.DevInOut.InOut;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ProcessHandling;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.ArrayList;
public class TabSample extends Tab implements ClientListener
{
	
	public WdgSamplePeelOut peeloutTab;
	public WdgAllCameras allCams;
	public WdgSampleInOut collimatorConstruct, beamstopConstruct, pindiodeConstruct;
	private GrpWdgMove moveGroup;
	public DevInOut initCollimator;
	protected Epics.EpicsValueMonitor monitor = null;
	
	public Composite messageComp, topComp, tabComp, bottomComp;
	
	public static Button peeloutButton;
	public Button snapshotButton, beamControlButton, crystalcenterButton,loopcenterButton;
	public Button threeclickcenterButton, stopButton, autoButton;
	
	public Combo comboColli, combocomboBeamsize, comboBeamWidth, comboBeamHeight;;
	public Label horizontalColliPos;
	
	protected double[] omegaChoices = { 0, 90, 180, 270 };
	static double beamsizeboxX = 0.3, beamsizeboxY = 0.3;
	
	static int isCircle = 0;
	private int sampleDetach;
	boolean isAuto = false;
	
	private static ArrayList<TabSample> instArray = new ArrayList<TabSample>();
	public static ArrayList<TabSample> get() { return instArray; }
	
	ExecTask runningTask = null;
	static Thread autocenteringThread = null;
	Thread autoThread = null, moveThread = null, watchThread;
	PVName newIn, newOut;
	GridData gd;
	
	public TabSample(Composite parent, int style, int sampleDetach)
	{
		super(parent,style);
		this.sampleDetach=sampleDetach;
		this.moveGroup=new GrpWdgMove();
		instArray.add(this);
		
		System.out.println("TabSample: start");
		
		setLayout(new GridLayout(1, false));
		setBackground(GUI.BGCOLOR);

		messageComp = createMessageComp();
		topComp = createTopComp();
		tabComp = createTabComp();
		bottomComp = createBottomComp();
		showExteriorControls(true);

		connectListeners();
		
		System.out.println("TabSample: done");
	}
	
	public void showExteriorControls(boolean showExterior)
	{
		WdgControl.setControlVisible(messageComp, !showExterior);
		WdgControl.setControlVisible(topComp, showExterior);
		WdgControl.setControlVisible(bottomComp, showExterior);
		layout(true);
	}
	
	private Composite createMessageComp()
	{
		Composite messageComp = new Composite(this, SWT.NONE);
		messageComp.setBackground(getParent().getBackground());
		messageComp.setLayout(new FillLayout());
		messageComp.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		Label label = new Label(messageComp, SWT.BORDER | SWT.CENTER);
		label.setText("Sample controls are hidden when the diffraction viewer is selected - click to show");
		label.addMouseListener(new MouseListener()
		{
			@Override public void mouseDoubleClick(MouseEvent arg0) {}
			@Override public void mouseUp(MouseEvent arg0) {}
			@Override public void mouseDown(MouseEvent arg0)
			{
				boolean showExterior = Util.popupYesNo("Show controls?", "Show sample controls?");
				if (showExterior) showExteriorControls(true);
			}
		});
		
		return messageComp;
	}

	private void updateBeamSizeDisplay()
	{
		int selection = comboColli.getSelectionIndex();
		ArrayList<BeamSize> sizes = DBBeamSize.getBeamSizes(DevEnergy.getDevice().getPosition());
		for (int i=0;i<sizes.size();i++) {
			BeamSize size = sizes.get(i);
			String text = Util.sprintf("%d x %d \u00B5m", Math.round(size.width_um), Math.round(size.height_um));
			comboColli.setItem(i+1, text);
		}
		if (comboColli.getSelectionIndex()!=selection) comboColli.select(selection);
	}

	@Override public void connectListeners()
	{
		Client.addListener(this);
		allCams.connectListeners();
		if (null != peeloutTab) peeloutTab.connectListeners();

		if (Util.beamSizeCalculated()) {
			DevPmacMotor.get("mo_En_E").addListener(new DevMoveFloat.Listener() {
				@Override
				public void handleStateChange(State state) {
				}

				@Override
				public void handleActPosChange(DevMoveFloat dev, double newPos) {
					if (!dev.isMoving()) return;
					GUI.asyncExec(() -> updateBeamSizeDisplay());
				}

				@Override
				public void handleRqsPosChange(double newPos) {
				}

				@Override
				public void handleUnitsChange(String units) {
				}
			});
			DevEnergy.getDevice().llMoveStartStop.addListener((object, data) ->
			{
				GUI.asyncExec(() -> updateBeamSizeDisplay());
			});
		}

		snapshotButton.addSelectionListener(new SelectionListener()
	    {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) {
				// TODO Auto-generated method stub
				NBFileDialog dialog = new NBFileDialog(new Shell(GUI.getDisplay()), SWT.SAVE);

			   	String savePath = "";
			    //savePath = 
			    
			    if (savePath.equals("") || savePath == null)
			    	savePath = DevDataDir.getPrimary();
			    
			    dialog.setFilterPath(savePath); 
			    dialog.setFileName("snapshot");
			    
			    try {
			    	Util.take2Snapshot(dialog.openStr());
			    } catch (Exception e) {
			    	Log.error("ERROR: "+e.getMessage());
			    }
				
			}
	    	
	    });
		// TODO: disableExceptions
	  
	    AppState.addListener(AppState.Operation.OP_AUTOCENTER, new AppState.Listener()
	 	{
	 		@Override
			public void stateChanged(Operation op, boolean allowed) {
	    		updateButtonState();
	    		
	    	}
	    });
	    AppState.addListener(AppState.Operation.MOVE_COLL_CTRL, new AppState.Listener()
	 	{
	 		@Override
			public void stateChanged(Operation op, boolean allowed) {
	 			updatecomboColliState();
	    	}
	    });
	  
		monitor = new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String pvName, DBR value) {
				final String _pvName=pvName;
				final DBR _value=value;

				if(_pvName.equals(PV_COLLIMATOR.RunPrg.toString())) {
					
					GUI.asyncExec(new Runnable(){public void run()
					{
						if (Epics.dbrToInt(_value) != 1 && Client.isMaster() && AppState.isOperationAllowed(Operation.MOVE_COLL_CTRL)) {
							comboColli.setEnabled(true);
							collimatorConstruct.deviceButton.setEnabled(true);
						}
							
						else {
							comboColli.setEnabled(false);
							collimatorConstruct.deviceButton.setEnabled(false);
						}
								
					}});
				}
				if(_pvName.equals(PV_COLLIMATOR.ActivePreset.toString())) {
					
					GUI.asyncExec(new Runnable(){public void run()
					{
						int select = Epics.dbrToInt(_value);
						setPresetsPVs(select);
						setColliBeamsizeBox(select);
					}
					});
				}
				if(_pvName.equals(PV_BLUICE.beamsize_h.toString())) {
					
					GUI.asyncExec(new Runnable(){public void run()
					{
						int isUserSpecified = Epics.getInteger(PV_BLUICE.beamsizeOption);
						if (isUserSpecified == 2)
							setBeamsizeOption(2);
					}
					});
				}
				if(_pvName.equals(PV_BLUICE.beamsize_v.toString())) {
					
					GUI.asyncExec(new Runnable(){public void run()
					{
						int isUserSpecified = Epics.getInteger(PV_BLUICE.beamsizeOption);
						if (isUserSpecified == 2) {
							setBeamsizeOption(2);
						}
					}
					});	
				}
				if(_pvName.equals(PV_PMAC.ActPos.motor("beam_size_y").toString())) {
					
					GUI.asyncExec(new Runnable(){public void run()
					{
						int preset = Epics.getInteger(PV_COLLIMATOR.ActivePreset);
						if(preset == 1)
							setBeamsizeOption(6);
					}
					});	
				}
				
				if(_pvName.equals(PV_PMAC.ActPos.motor("beam_size_x").toString())) {
					
					GUI.asyncExec(new Runnable(){public void run()
					{
						int preset = Epics.getInteger(PV_COLLIMATOR.ActivePreset);
						if(preset == 1)
							setBeamsizeOption(6);
					}
					});	
				}	
				if(_pvName.equals(PV_BLUICE.beamsizeOption.toString())) {
					
					GUI.asyncExec(new Runnable(){public void run()
					{
						int beamsizeOption = Epics.dbrToInt(_value);
						
						setBeamsizeOption(beamsizeOption);
					}
					});
	
				}
				if(_pvName.equals(PV_BLUICE.pointclick.toString())) {
					int auto = Epics.getInteger(PV_BLUICE.autoCenterRun);
					if(auto != 1)
						resetThreeClick();
				}
				if(_pvName.equals(PV_ROBOT.sta.toString())) {
					if(Client.isMaster())
					{
						String robotStatus = Epics.dbrToString(_value);
						int robotPause = Epics.getInteger(PV_ROBOT.pause);
						if (!robotStatus.toLowerCase().contains("smr: ready") &&
								!robotStatus.toLowerCase().contains("warm_up")    &&
								!robotStatus.toLowerCase().contains("pin_base warm-up") &&
								robotPause != 1)
						{
							GUI.asyncExec(new Runnable(){public void run()
							{
								collimatorConstruct.deviceButton.setEnabled(false);
								comboColli.setEnabled(false);
							}});
						}
						else {
							GUI.asyncExec(new Runnable(){public void run()
							{
								collimatorConstruct.deviceButton.setEnabled(true);
								comboColli.setEnabled(true);
							}});
						}
					}
				}
				if(_pvName.equals(PV_BLUICE.autoCenterRun.toString())) {
					final int auto = Epics.dbrToInt(_value);
					
						GUI.asyncExec(new Runnable(){public void run()
						{
							if(Client.isMaster())
							{
								if (auto == 0) {
									autoButton.setEnabled(true);
									threeclickcenterButton.setEnabled(true);
									stopAutocentering();
								}
								else {
									autoButton.setEnabled(false);
									threeclickcenterButton.setEnabled(false);
									stopButton.setEnabled(true);
								}
							}
							else {
								if (auto == 0) {
									stopAutocentering();
						}
					}
						}});
					}
			}

		};	
		
		Epics.addMonitor(PV_BLUICE.beamsizeOption, monitor);
		Epics.addMonitor(PV_COLLIMATOR.ActivePreset, monitor);
		Epics.addMonitor(PV_PMAC.ActPos.motor("collimator_v"), monitor);
		Epics.addMonitor(PV_PMAC.ActPos.motor("collimator_h"), monitor);
		Epics.addMonitor(PV_BLUICE.currentCollimatorSet, monitor); //
		Epics.addMonitor(PV_BLUICE.beamsize_h, monitor);
		Epics.addMonitor(PV_BLUICE.beamsize_v, monitor);
		Epics.addMonitor(PV_COLLIMATOR.RunPrg, monitor);
		Epics.addMonitor(PV_BLUICE.pointclick, monitor);
		Epics.addMonitor(PV_ROBOT.sta, monitor);
		Epics.addMonitor(PV_BLUICE.autoCenterRun, monitor);
		Epics.addMonitor(PV_PMAC.ActPos.motor("beam_size_x"), monitor);
		Epics.addMonitor(PV_PMAC.ActPos.motor("beam_size_y"), monitor);
	}

	private Composite createTopComp()
	{
		Composite topComp = new Composite(this,SWT.NONE);
		topComp.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(8,false);
		
		topComp.setLayout(gl);
		gd = new GridData(SWT.CENTER,SWT.DEFAULT,true,false);
		topComp.setLayoutData(gd);
		
		gd = new GridData();
		gd.widthHint = 100;
		gd.heightHint = 40;
	    Button startButton = new Button(topComp, SWT.PUSH);
	    startButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		startButton.setText("Start");
		startButton.setLayoutData(gd);
		startButton.setFont(GUI.FONT_HUTCH_BUTTON);
		moveGroup.setStartButton(startButton);
		
		gd = new GridData();
		gd.widthHint = 100;
	    gd.heightHint = 40;
		Button cancelButton = new Button(topComp,SWT.PUSH);
		cancelButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		cancelButton.setFont(GUI.FONT_HUTCH_BUTTON);
		cancelButton.setText("Cancel");
		cancelButton.setLayoutData(gd);
		moveGroup.setCancelButton(cancelButton);
		
		// Motor widgets
		gd = new GridData();
		gd.widthHint = 180;
		gd.verticalAlignment = SWT.FILL;
		WdgMotor wdgOmega = new WdgMotor(topComp, SWT.None, moveGroup, "Omega", null, "gonio_omega", GUI.BGCOLOR, omegaChoices, 3);
		wdgOmega.setSize(70,90);
		wdgOmega.setLayoutData(gd);
		wdgOmega.setDisplayUnits("deg");
		
		double[] beamSizeXChoices =	{ .025, .05, .075, .1, .2, 1, 2, 5 },
			beamSizeYChoices = 	beamSizeXChoices;
		
		constructColli(topComp, SWT.NONE);
		
		gd = new GridData();
		gd.widthHint = 125;
		gd.verticalAlignment = SWT.FILL;
		WdgMotor wdgBeamWidth = new WdgMotor(topComp, SWT.None, moveGroup, "Slit Width", null, "beam_size_x", GUI.BGCOLOR, beamSizeXChoices, 3);
		
		wdgBeamWidth.setSize(40,50);
		wdgBeamWidth.setLayoutData(gd);
		wdgBeamWidth.setDisplayUnits("mm");
		
		gd = new GridData();
		Label lblX = new Label(topComp, SWT.None);
		lblX.setText("x");
		
		lblX.setSize(15,20);
		lblX.setBackground(GUI.BGCOLOR);
		
		lblX.setLayoutData(gd);

		gd = new GridData();
		gd.widthHint = 125;
		gd.verticalAlignment = SWT.FILL;
		WdgMotor wdgBeamHeight = new WdgMotor(topComp, SWT.None, moveGroup, "Slit Height", null, "beam_size_y", GUI.BGCOLOR, beamSizeYChoices, 3);
		wdgBeamHeight.setDisplayUnits("mm");
		wdgBeamHeight.setSize(40,50);
		wdgBeamHeight.setLayoutData(gd);
		
		Group popupGroup = new Group(topComp, SWT.NONE);
		popupGroup.setBackground(getParent().getBackground());
		popupGroup.setText("Peelout");
		gd = new GridData();
		gd.verticalAlignment = GridData.FILL;
		popupGroup.setLayoutData(gd);
		popupGroup.setLayout(new GridLayout(1,false));
		
		if(sampleDetach != 1) {
			gd = new GridData();
			gd.heightHint = 30;
			peeloutButton = new Button(popupGroup, SWT.PUSH);
			peeloutButton.setBackground(GUI.COLOR_BLUE_BUTTON);
			peeloutButton.setText("SampleWindow");
			peeloutButton.setLayoutData(gd);
			peeloutButton.setFont(GUI.FONT_SAMPLE_BUTTONS);
			
			peeloutTab =new WdgSamplePeelOut(GUI.getShell(),SWT.NONE);
			peeloutTab.create(0);
			peeloutTab.peeloutShell.setVisible(false);
			peeloutButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) { }
			public void widgetSelected(SelectionEvent arg0) {
				createSamplePeelout();
			}
			
			});
		}
	
		gd = new GridData();
		gd.heightHint = 30;
		gd.verticalAlignment = SWT.CENTER;
		beamControlButton = new Button(popupGroup, SWT.PUSH);
		beamControlButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		beamControlButton.setText("Beam Controls");
		beamControlButton.setLayoutData(gd);
		beamControlButton.setFont(GUI.FONT_SAMPLE_BUTTONS);

		boolean showPiezo = DBConfig.get("beam_align_device").toLowerCase().contains("kbm");
		if (showPiezo) {
			final PiezoDialog piezoDialog = new PiezoDialog(getParent().getShell(), SWT.NONE);
			piezoDialog.create();
			piezoDialog.messageDialog.setVisible(false);
			beamControlButton.addSelectionListener(new SelectionListener()
			{

				@Override
				public void widgetDefaultSelected(SelectionEvent arg0)
				{
				}

				@Override
				public void widgetSelected(SelectionEvent arg0)
				{
					// TODO Auto-generated method stub
					GUI.asyncExec(new Runnable()
					{
						public void run()
						{
							piezoDialog.messageDialog.open();
							piezoDialog.messageDialog.setVisible(true);
						}
					});
				}
			});
		} else {
			beamControlButton.addSelectionListener(new SelectionListener()
			{
				@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
				@Override public void widgetSelected(SelectionEvent arg0) { GUI.asyncExec(() -> new CRLDialog(getParent().getShell(), SWT.NONE)); }
			});
		}
		
		return topComp;
	}

	private Composite createTabComp()
	{
		Composite tabComp = new Composite(this,SWT.NONE);
		tabComp.setBackground(getParent().getBackground());
		GridLayout tableGrid = new GridLayout(3,false);
		
		tabComp.setLayout(tableGrid);
		gd = new GridData(SWT.FILL,SWT.FILL,true,true);
		
		tabComp.setLayoutData(gd);
		
		allCams = new WdgAllCameras(tabComp,SWT.MULTI,WdgCamera.CameraSize.LARGE, moveGroup);
		GridData gd = new GridData(SWT.FILL,SWT.FILL,true,true);
		allCams.setLayoutData(gd);
		
		return tabComp;
	}

	private Composite createBottomComp()
	{
		Composite bottomComp = new Composite(this,SWT.NONE);
		bottomComp.setBackground(GUI.BGCOLOR);
		GridLayout gl1 = new GridLayout(6,false);
		gl1.horizontalSpacing = 15;
		gl1.marginWidth=0;
		gl1.marginHeight=0;
		bottomComp.setLayout(gl1);
		gd = new GridData(SWT.CENTER,SWT.FILL,true,false);
		bottomComp.setLayoutData(gd);
		
		constructCentering(bottomComp,SWT.NONE);
		
		Group groupPindiode = new Group(bottomComp, SWT.NONE);
		groupPindiode.setBackground(getParent().getBackground());
		groupPindiode.setText("Pindiode");
		gd = new GridData();
		gd.horizontalSpan = 2;
		gd.verticalAlignment = GridData.BEGINNING;
		groupPindiode.setLayoutData(gd);
		groupPindiode.setLayout(new GridLayout(3,false));
		
		PVName actposPV = PV_MISC.pindiode_actpos;
		PVName rqsMem1PV = PV_MISC.pindiode_mem1;
		PVName rqsMem2PV = PV_MISC.pindiode_mem2;
		PVName motionPV = PV_MISC.pindiode_run;
		
		new DevInOut("Pindiode",actposPV,rqsMem1PV,rqsMem2PV,motionPV);
		
		pindiodeConstruct = new WdgSampleInOut(groupPindiode,SWT.NONE, "Pindiode",actposPV,rqsMem1PV,rqsMem2PV,motionPV);
		
		Group groupBeamstop = new Group(bottomComp, SWT.NONE);
		groupBeamstop.setBackground(getParent().getBackground());
		groupBeamstop.setText("Beamstop");
		gd = new GridData();
		gd.horizontalSpan = 2;
		gd.verticalAlignment = GridData.BEGINNING;
		groupBeamstop.setLayoutData(gd);
		groupBeamstop.setLayout(new GridLayout(3,false));
		
		new DevInOut("Beamstop",PV_PMAC21.bo14_VAL, PV_PMAC21.bi21_VAL);
		beamstopConstruct = new WdgSampleInOut(groupBeamstop,SWT.NONE, "Beamstop",PV_PMAC21.bo14_VAL, PV_PMAC21.bi21_VAL);
		
		gd = new GridData();
		gd.heightHint = 50;
		//gd.verticalAlignment = GridData.CENTER;
		gd.verticalAlignment = GridData.END;
		snapshotButton = new Button(bottomComp, SWT.PUSH);
	    snapshotButton.setBackground(GUI.COLOR_BLUE_BUTTON);
	    snapshotButton.setText("Take Snapshot");
	    snapshotButton.setLayoutData(gd);
	    snapshotButton.setFont(GUI.FONT_SAMPLE_BUTTONS);
	    
	    return bottomComp;
	}
	
	public void constructCentering(Composite parent, int style) {
		
		Group centerGroup = new Group(parent, SWT.NONE);
		centerGroup.setBackground(getParent().getBackground());
		centerGroup.setText("Centering");
		centerGroup.setLayout(new GridLayout(6,false));
		
		gd = new GridData(SWT.FILL,SWT.DEFAULT,true,false, 6, 1);
		
		gd = new GridData();
		gd.heightHint = 30;
	    autoButton = new Button(centerGroup, SWT.PUSH);
	    autoButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		autoButton.setText("Auto Centering");
		autoButton.setEnabled(false);
		autoButton.setLayoutData(gd);
		autoButton.addSelectionListener(new SelectionListener()
	    {
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) { }
			public void widgetSelected(SelectionEvent arg0) {
					
					final boolean isCrystalCenter = loopcenterButton.getSelection();
					if (null != autocenteringThread && autocenteringThread.isAlive())
					{
						Log.error("Autocentering script is already running");
						return;
					}
					autocenteringThread = new Thread(new Runnable(){ public void run() {
						
						updateButtonState();
						
						// TODO: if this is correct, fix the variable name
						if(isCrystalCenter) 
							DevAutocenter.get().syncExecLoop();
						else
							DevAutocenter.get().syncExecCrystal();
						
						GUI.asyncExec(new Runnable(){public void run()
				    	{
				    		while (null != autoThread && autoThread.isAlive()) Util.sleep(10);
				    		updateButtonState();
				    	}});
					}}, "autoCentering");
					autocenteringThread.start();
				}
	    });
		gd = new GridData();
		gd.heightHint = 30;
	    loopcenterButton = new Button(centerGroup, SWT.RADIO);
	    loopcenterButton.setBackground(getParent().getBackground());
	    loopcenterButton.setText("Loop");
	    loopcenterButton.setLayoutData(gd);
	    loopcenterButton.setSelection(true);
	    loopcenterButton.setEnabled(false);
	    
	    gd = new GridData();
		gd.heightHint = 30;
	    crystalcenterButton = new Button(centerGroup, SWT.RADIO);
	    crystalcenterButton.setBackground(getParent().getBackground());
	    crystalcenterButton.setText("Crystal");
	    crystalcenterButton.setLayoutData(gd);
	    crystalcenterButton.setEnabled(false);
	    
	    gd = new GridData();
		gd.heightHint = 30;
		gd.horizontalAlignment = SWT.CENTER;
	    threeclickcenterButton = new Button(centerGroup, SWT.PUSH);
	    threeclickcenterButton.setBackground(GUI.COLOR_BLUE_BUTTON);
	    threeclickcenterButton.setText("3Click Centering");
	    threeclickcenterButton.setEnabled(false);
	    threeclickcenterButton.setLayoutData(gd);
	    threeclickcenterButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) {
				GUI.asyncExec(new Runnable(){public void run()
				{
					threeclickcenterButton.setEnabled(false);
				}});
				WdgCamera.threeClickStep = 1;
			}
	    	
	    });
	    gd = new GridData();
		gd.heightHint = 30;
		gd.horizontalSpan = 2;
		gd.horizontalAlignment = SWT.CENTER;
	    stopButton = new Button(centerGroup, SWT.PUSH);
	    stopButton.setBackground(GUI.COLOR_BLUE_BUTTON);
	    stopButton.setImage(GUI.IMAGE_STOP);
	    stopButton.setText("Stop");
	    stopButton.setEnabled(false);
	    stopButton.setLayoutData(gd);
	    stopButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) {
					stopAutocentering ();
			}
	    	
	    });
	}
	public void stopAutocentering () {
		int isAutoRunning =  Epics.getInteger(PV_BLUICE.autoCenterRun);
		if(isAutoRunning == 1) {
			//ProcessHandling.killall("collimatorCtrl.pl", ProcessHandling.Signal.INT);
			ProcessHandling.killall("autoraster_xrec.pl", ProcessHandling.Signal.INT);
			DevAutocenter.get().stopExec();
		}
			
		if (isAutoRunning!=0)
		{
			Epics.putInteger(PV_BLUICE.autoCenterRun,0);
		}
		
		GUI.asyncExec(new Runnable(){public void run()
		{
			if(Client.isMaster())
				autoButton.setEnabled(true);
			stopButton.setEnabled(false);
		}});
	}
	public void setBeamsizeOption (int beamsize) {
		
		double gaurdslitX = Epics.getDouble(PV_PMAC.ActPos.motor("beam_size_x"));
		double gaurdslitY = Epics.getDouble(PV_PMAC.ActPos.motor("beam_size_y"));
		//comboBeamsize.select(beamsize);
		if (beamsize == 0) {
			beamsizeboxX = 0.0;
			beamsizeboxY = 0.0;
		} else if (beamsize == 1) {
			beamsizeboxX = gaurdslitX;
			beamsizeboxY = gaurdslitY;
			isCircle = 0;
		} else if (beamsize == 2) {
			beamsizeboxX = Epics.getDouble(PV_BLUICE.beamsize_h);
			beamsizeboxY = Epics.getDouble(PV_BLUICE.beamsize_v);
			isCircle = 0;
		} else if (beamsize == 3) {
			beamsizeboxX = 0.005;
			beamsizeboxY = 0.005;
			isCircle = 1;
		} else if (beamsize == 4) {
			beamsizeboxX = 0.01;
			beamsizeboxY = 0.01;
			isCircle = 1;
		} else if (beamsize  == 5) {
			beamsizeboxX = 0.02;
			beamsizeboxY = 0.02;
			isCircle = 1;
		} else if (beamsize == 6) {
			
			if ( DBConfig.get("app_beamlineID").equals("23o")) {
				//these PVs updated by sergey's script
				beamsizeboxX = Epics.getDouble(PV_BLUICE.beamsize_h_act1$VAL)/1000.0;
				beamsizeboxY = Epics.getDouble(PV_BLUICE.beamsize_v_act1$VAL)/1000.0;
				
				if (beamsizeboxX == 0.0 || beamsizeboxX == 0.0) {
					beamsizeboxX = Double.valueOf(DBConfig.get("beamsizebox_H"));
					beamsizeboxY = Double.valueOf(DBConfig.get("beamsizebox_V"));
				}
				
				if(gaurdslitX < beamsizeboxX) {
					//comboBeamsize.select(1);
					beamsizeboxX = gaurdslitX;
					
				}
				if(gaurdslitY < beamsizeboxY) {
					//comboBeamsize.select(1);
					beamsizeboxY = gaurdslitY;
				}
			} else if (  DBConfig.get("app_beamlineID").equals("23i") ) {

				//these PVs updated by sergey's script
				beamsizeboxX = Epics.getDouble(PV_BLUICE.beamsize_h_act1$VAL)/1000.0;
				beamsizeboxY = Epics.getDouble(PV_BLUICE.beamsize_v_act1$VAL)/1000.0;
				
				if (beamsizeboxX == 0.0 || beamsizeboxX == 0.0) {
					beamsizeboxX = Double.valueOf(DBConfig.get("beamsizebox_H"));
					beamsizeboxY = Double.valueOf(DBConfig.get("beamsizebox_V"));
				}
				if(gaurdslitX < beamsizeboxX) { {
					
				}
				beamsizeboxX = gaurdslitX;
					
				}
				if(gaurdslitY < beamsizeboxY)
					beamsizeboxY = gaurdslitY;
			} else {
				beamsizeboxX = 0.100; 
				beamsizeboxY = 0.100; 
			}
			isCircle = 0;
		} else {
			beamsizeboxX = 0.100; 
			beamsizeboxY = 0.100; 
		}
	}

	public void setColliBeamsizeBox(int select) {
		if ( DBConfig.get("app_collimatorSet").equals("TRIPLE")) {
			
			String selectColli = comboColli.getText();
			if(selectColli.equals("5um") ||selectColli.equals("20um")) {
				Epics.putInteger(PV_BLUICE.beamsizeOption, 3);
			} else if (selectColli.equals("10um")) {
				Epics.putInteger(PV_BLUICE.beamsizeOption, 4);
			} else if (selectColli.equals("Scatter Guard")) {
				Epics.putInteger(PV_BLUICE.beamsizeOption, 5);
			} else if (select == 0) {
				//Epics.putInteger(Epics.addBID("bi:beamsizeOption"), 3);
			} else {
			Log.error("Wrong Collimator selection");
		}
		} else {
		
			if(select == 4) {
				Epics.putInteger(PV_BLUICE.beamsizeOption, 3);
			} else if (select == 3) {
				Epics.putInteger(PV_BLUICE.beamsizeOption, 4);
			} else if (select == 2) {
				Epics.putInteger(PV_BLUICE.beamsizeOption, 5);
			} else if (select == 1) {
				Epics.putInteger(PV_BLUICE.beamsizeOption, 6);
			} else if (select == 0) {
				//Epics.putInteger(Epics.addBID("bi:beamsizeOption"), 3);
			} else {
				Log.error("Wrong Collimator selection");
			}
		}
	}
	
	public void createSamplePeelout()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			if(peeloutTab.visible == 0) {
				Log.note("creating peelout Sample Tab");
				peeloutTab.peeloutShell.open();
				peeloutTab.peelOutTab.notifyIsVisible(true);
			} else {
				peeloutTab.peeloutShell.setVisible(true);
				peeloutTab.peeloutShell.forceActive();
				peeloutTab.peelOutTab.notifyIsVisible(true);
			}
		}});
	}
	
	public void setPresetsPVs(int value) {
		
		comboColli.select(value);
		
		if ( value == 0 ) { newIn = null; newOut = null; }
		if ( value == 1 ) { newIn = PV_MISC.collimator_mem2; newOut = PV_MISC.collimator_mem1; }
		if ( value == 2 ) { newIn = PV_MISC.collimator_mem3; newOut = PV_MISC.collimator_mem1; }
		if ( value == 3 ) { newIn = PV_MISC.collimator_mem4; newOut = PV_MISC.collimator_mem1;; }
		if ( DBConfig.get("app_collimatorSet").equals("QUAD")) {
			if ( value == 4 ) { newIn = PV_MISC.collimator_mem5; newOut = PV_MISC.collimator_mem1;}
		}
		if(initCollimator!=null)
			initCollimator.changeInOutConfig(newIn, newOut, null, 0.0);
	}
	
	public void constructColli(Composite parent, int style) {
		
		Group groupColli = new Group(parent, SWT.NONE);
		groupColli.setBackground(getParent().getBackground());
		groupColli.setText("Collimator Controls");
		groupColli.setFont(GUI.FONT_MOTOR_GROUP);
		//groupColli.setForeground(GUI.COLOR_RED);
		gd = new GridData();
		gd.verticalAlignment = GridData.FILL;
		groupColli.setLayoutData(gd);
		groupColli.setLayout(new GridLayout(5,false));
		
		int tripleCollimatorSet = Epics.getInteger(PV_BLUICE.currentCollimatorSet);
	
		final ArrayList<String> colliList = new ArrayList<String>();
		if ( DBConfig.get("app_collimatorSet").equals("TRIPLE")) {
			colliList.add("None");
			colliList.add("Scatetr Guard");
		
			if (tripleCollimatorSet == 1) {
				colliList.add("20um");
				colliList.add("10um");
				
			} else {
				colliList.add("10um");
				colliList.add("5um");
			}
			
		} else {
			colliList.add("None");
			colliList.add("ScatterGuard");
			colliList.add("20um");
			colliList.add("10um");
			colliList.add("5um");
		}
	
		
		comboColli = new Combo(groupColli,SWT.READ_ONLY);
		
		for(int i=0;i<colliList.size();i++) {
			comboColli.add(colliList.get(i),i);
		}
		if (Util.beamSizeCalculated()) updateBeamSizeDisplay();

		comboColli.setFont(GUI.FONT_MOTOR_RQSPOS);
		gd = new GridData();
		gd.horizontalSpan = 5;
		gd.horizontalAlignment = SWT.FILL;
		comboColli.setEnabled(false);
		comboColli.setFont(GUI.FONT_MOTOR_GROUP);
		comboColli.setToolTipText("Choosing None means no collimator/no scatterguard and \nlarge background scatter");
		comboColli.setLayoutData(gd);

		comboColli.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) { }
			public void widgetSelected(SelectionEvent arg0) {
				GUI.asyncExec(new Runnable(){public void run()
				{
					int select = comboColli.getSelectionIndex();
					if(DevInOut.deviceStatus.get("Collimator").equals("IN")) {
						comboColli.setEnabled(false);
						collimatorConstruct.deviceButton.setEnabled(false);
					}
					Epics.putInteger(PV_COLLIMATOR.ActivePreset, select);
					/* set Beamsize Box with respect to collimator */
					setColliBeamsizeBox(select);
					
					/* collimator in */
					String move;
					if (select == 0) move = "OUT";
					else move = "IN";
					if(DevInOut.deviceStatus.get("Collimator").equals("IN")) {

						// set GS for scatterguard
						DevMove.MoveCmd _gsxMove=null, _gsyMove=null;
						if (select==1 ) {
							DevPmacMotor devGSX = DevPmacMotor.get("beam_size_x");
							DevPmacMotor devGSY = DevPmacMotor.get("beam_size_y");
							boolean moveNeeded = false;
							if (devGSX.getPosition()<0.2 && devGSX.isMoveNeededTo(0.2)) moveNeeded=true;
							if (devGSY.getPosition()<0.2 && devGSY.isMoveNeededTo(0.2)) moveNeeded=true;
							if (moveNeeded) {
								_gsxMove = devGSX.startMove(0.2);
								_gsyMove = devGSY.startMove(0.2);
							}
						}
						DevMove.MoveCmd gsxMove=_gsxMove, gsyMove=_gsyMove;

					    CCMove ccMove = DevCollimatorCtrl.get().newMove();
					    ccMove.setDest("collimator",		InOut.IN);
					    ccMove.start();
					    
						watchThread  = new Thread(new Runnable(){public void run()
						{
							while (null != moveThread && moveThread.isAlive()) Util.sleep(10);
							if (null!=gsxMove) while (!gsxMove.isComplete()) Util.sleep(10);
							if (null!=gsyMove) while (!gsyMove.isComplete()) Util.sleep(10);
							updatecomboColliState();
						}}, "TabSample comboColli move monitor");
						watchThread.start();
					}
				}
				
			});
			}
		});
		PVName actposPV = PV_PMAC.ActPos.motor("collimator_v");
		PVName motionPV = PV_COLLIMATOR.RunPrg;
		
		int preset = Epics.getInteger(PV_COLLIMATOR.ActivePreset);
		setPresetsPVs(preset);
		initCollimator = new DevInOut("Collimator",actposPV, newIn, newOut, motionPV);
		Util.sleep(20);
		collimatorConstruct = new WdgSampleInOut(groupColli,SWT.NONE, "Collimator",actposPV, newIn, newOut,motionPV);
		
		collimatorConstruct.deviceButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) {
				boolean state = collimatorConstruct.deviceButton.getEnabled();
				if(Client.isMaster()) {
					int select = comboColli.getSelectionIndex();
					if (select != 0)
						comboColli.setEnabled(state);
				}
			}
			
		});
	}
	
	public void resetThreeClick() {
		
		if(!Client.isMaster())
			return;
		if(WdgCamera.threeClickStep > 3 || WdgCamera.threeClickStep < 1) {
			WdgCamera.threeClickStep = 0;
			GUI.asyncExec(new Runnable(){public void run()
			{
				threeclickcenterButton.setEnabled(true);
			}});
		}
		else {
			GUI.asyncExec(new Runnable(){public void run()
			{
				threeclickcenterButton.setEnabled(false);
			}});
		}
	}
	
	@Override
	public void notifyIsVisible(boolean isVisible)
	{
		allCams.notifyIsVisible(isVisible);
	}
	
	public static void create()
	{
		GUI.asyncExec(new Runnable()
		{
			public void run()
			{
				TabSample ts;
				try {
					ts = new TabSample(new Shell(GUI.getDisplay()),SWT.NONE,0);
					GUI.tabMap.put(ts.getTabName(),ts);
				} catch (Exception e){
					e.printStackTrace();
				}
			}
		});
	}
	
	public void updatecomboColliState() {
		// TODO: disableExceptions
		boolean iscolliRunning = (null != moveThread && moveThread.isAlive());
		final boolean enable = (Client.isMaster() && !iscolliRunning && AppState.isOperationAllowed(Operation.MOVE_COLL_CTRL));
		GUI.asyncExec(new Runnable(){public void run()
		{
			comboColli.setEnabled(enable);
			collimatorConstruct.deviceButton.setEnabled(enable);
			beamstopConstruct.deviceButton.setEnabled(enable);
			pindiodeConstruct.deviceButton.setEnabled(enable);
		}});
	}
	
	public void updateButtonState() {
		
		boolean isautocenteringRunning = (null != autocenteringThread && autocenteringThread.isAlive());
		final int isAuto = Epics.getInteger(PV_BLUICE.autoCenterRun);
		final boolean enable = (Client.isMaster() && isAuto == 0 && AppState.isOperationAllowed(Operation.OP_AUTOCENTER));
		GUI.asyncExec(new Runnable(){public void run()
		{
			autoButton.setEnabled(enable);
			loopcenterButton.setEnabled(enable);
			crystalcenterButton.setEnabled(enable);
			threeclickcenterButton.setEnabled(enable);
			if (Client.isMaster() )
    		{
				if(enable) {
					stopButton.setEnabled(false);
				}
				else {
					if(isAuto == 1)
						stopButton.setEnabled(true);
				}
    		}
		}});
	}
	
	public void updateEnableState()
	{
		
		GUI.asyncExec(new Runnable(){public void run()
		{
			if (Client.isMaster()) {
				updateButtonState();
				updatecomboColliState();
			}
		}});
	}
	
	@Override
	public void masterChanged(boolean isMaster) {
		
			updateEnableState();
			updateButtonState();
	}
	
	@Override
	public String getTabName() { return "Sample"; }
	
}
