package gov.anl.gmca.sample;

import gov.anl.gmca.collect.viewer.WdgDiffControls;
import gov.anl.gmca.collect.viewer.WdgDiffTab;
import gov.anl.gmca.common.ui.CCombo;
import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.GrpWdgMove;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;
import gov.anl.gmca.io.PVLookup.PV_COLLIMATOR;
import gov.anl.gmca.sample.WdgBeamsizeBox.BEAMSIZE_OPTION;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.HashMap;

public class WdgAllCameras extends Composite
{
	protected 	Color 					bgColor = GUI.COLOR_ALLCAM_BG;
	protected 	WdgCamera 				lowResCam, highResCam, hutchCam, toolsCam, apsStatus;
	protected 	WdgDiffTab 				diffTabContents;
	public 		Label 					fullHeightLabel, fullWidthLabel;
	public 		CCombo 					comboBeamWidth, comboBeamHeight;
	protected 	CTabFolder 				ctFolder;
	protected 	CTabItem 				lowResTab, highResTab, hutchTab, toolsTab, diffTab, apsTab;
	protected 	WdgCameraControls 		controls, controlsHighres, controlsTools;
	protected 	Tab.VisibilityListener 	lastViewer = null;
	public 		WdgDiffControls 		diffControls;
	public 		HashMap<BEAMSIZE_OPTION, Button> buttonGroup = new HashMap<BEAMSIZE_OPTION, Button>();
	private		int						controlsWidth;
	private		WdgCamera.CameraSize	cameraSize;
	private		GrpWdgMove 				moveGroup;
	
	public WdgAllCameras(Composite parent, int style, WdgCamera.CameraSize cameraSize,GrpWdgMove moveGroup)
	{
		super(parent,style);
		controlsWidth = (cameraSize != WdgCamera.CameraSize.SMALL) ? 270 : 135;
		this.cameraSize = cameraSize;
		this.moveGroup = moveGroup;
		
		setBackground(GUI.BGCOLOR);
		setLayout(new FillLayout());
		
		ctFolder = new CTabFolder(this, SWT.BORDER);
		ctFolder.setFont(GUI.FONT_SAMPLE_TABS);
		
		ctFolder.setSelectionBackground(bgColor);
		ctFolder.setSelectionBackground
		(
			new Color[]
			{
				GUI.getDisplay().getSystemColor(SWT.COLOR_WHITE),
				new Color(GUI.getDisplay(), 184,211,238),
				new Color(GUI.getDisplay(), 136,182,227),
				new Color(GUI.getDisplay(), 136,182,227)
			},
			new int[] {25,50,100},
			true
		);
		
		createLowResTab(ctFolder);
		createHighResTab(ctFolder);
		createHutchTab(ctFolder);
		createAPSTab(ctFolder);
		
		if (cameraSize != WdgCamera.CameraSize.SMALL)
		{
			createToolsTab(ctFolder);
			createDiffTab(ctFolder);
		}
		
		connectListeners();
	}
	
	private void createLowResTab(CTabFolder ctFolder)
	{
		//////////////////////////////////// Low res tab ///////////////////////////////
		
		Composite compLowRes = new Composite(ctFolder,SWT.NONE);
		compLowRes.setBackground(GUI.BGCOLOR);
		compLowRes.setLayout(new GridLayout(2,false));
		
		controls = new WdgCameraControls(compLowRes, SWT.NONE, moveGroup, cameraSize, 0);
		controls.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.DEFAULT, SWT.CENTER, false, true);
		gd.widthHint = controlsWidth;
		controls.setLayoutData(gd);
		
		lowResCam = new WdgCamera(compLowRes,SWT.None, DBCamera.getImageURL(WdgCamera.CameraType.LOWRES, cameraSize), WdgCamera.CameraType.LOWRES, cameraSize);
		gd = new GridData(SWT.FILL,SWT.FILL,true,true);
		lowResCam.setLayoutData(gd);
		
		lowResTab = new CTabItem(ctFolder, SWT.NONE);
		lowResTab.setText(" Sample: Low Res ");
		lowResTab.setControl(compLowRes);
	}
	
	private void createHighResTab(CTabFolder ctFolder)
	{
		/////////////////////////////////// High res tab //////////////////////////
		
		Composite compHighRes = new Composite(ctFolder,SWT.NONE);
		compHighRes.setBackground(GUI.BGCOLOR);
		compHighRes.setLayout(new GridLayout(3,false));
		
		controlsHighres = new WdgCameraControls(compHighRes, SWT.NONE,moveGroup, cameraSize, 1);
		controlsHighres.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.DEFAULT, SWT.CENTER, false, true);
		gd.widthHint = controlsWidth;
		controlsHighres.setLayoutData(gd);
		
		highResCam = new WdgCamera(compHighRes,SWT.None, DBCamera.getImageURL(WdgCamera.CameraType.HIGHRES, cameraSize), WdgCamera.CameraType.HIGHRES, cameraSize);
		highResCam.setSize(DBCamera.getCameraDimensions(WdgCamera.CameraType.HIGHRES, cameraSize));
		gd = new GridData(SWT.FILL,SWT.FILL,true,true);
		highResCam.setLayoutData(gd);
	
		highResTab = new CTabItem(ctFolder, SWT.NONE);
		highResTab.setText(" Sample: High Res ");
		highResTab.setControl(compHighRes);
		
		Group beasizeButtonGroup = new Group(compHighRes,SWT.NONE);
		GridLayout paramGrid = new GridLayout(1,false);
		beasizeButtonGroup.setBackground(GUI.BGCOLOR);
		beasizeButtonGroup.setLayout(paramGrid);
		beasizeButtonGroup.setText("Display");
		gd = new GridData();
		gd.widthHint = 85;
		gd.horizontalAlignment = SWT.TOP;
		gd.minimumHeight = 300;
		beasizeButtonGroup.setLayoutData(gd);
		
		for (BEAMSIZE_OPTION colliButton : BEAMSIZE_OPTION.values()) {
			
			gd = new GridData();
			gd.widthHint = 65;
			
			final Button buttonColli = new Button(beasizeButtonGroup, SWT.CHECK);
			final String colliOpt = colliButton.getColli();
			buttonColli.setText(colliOpt);
			buttonColli.setForeground(colliButton.getColor());
			buttonColli.setBackground(GUI.BGCOLOR);
			buttonColli.setLayoutData(gd);	
			final int ord = colliButton.ordinal();
			buttonColli.addListener(SWT.Selection, new Listener() {
				@Override
				public void handleEvent(Event arg0) {
					highResCam.redraw();
					if(buttonColli.getSelection())
						Epics.putInteger(PV_BLUICE.beamsize.getBeamsizePV(ord),1);
					else
						Epics.putInteger(PV_BLUICE.beamsize.getBeamsizePV(ord),0);
					if(colliOpt.equals("User")) {
							comboBeamWidth.setEnabled(buttonColli.getSelection());
							comboBeamHeight.setEnabled(buttonColli.getSelection());
							highResCam.redraw();		
					}
				}
		   	});
			
			buttonGroup.put(colliButton, buttonColli);
			
			if(colliButton.getColli().equals("Full")) {
				
				gd = new GridData();
				buttonColli.setText("FWHM");
				gd.horizontalIndent = 20;
				fullWidthLabel = new Label(beasizeButtonGroup, SWT.NONE);
				fullWidthLabel.setLayoutData(gd);
				fullWidthLabel.setText("H:");
				fullWidthLabel.setBackground(GUI.BGCOLOR);
				
				gd = new GridData();
				gd.horizontalIndent = 20;
				fullHeightLabel = new Label(beasizeButtonGroup, SWT.NONE);
				fullHeightLabel.setLayoutData(gd);
				fullHeightLabel.setText("V:");
				fullHeightLabel.setBackground(GUI.BGCOLOR);
				
				double widthVal = Math.round(Epics.getDouble(PV_BLUICE.beamsize_h_act1$VAL))/1000.0;
				double heightVal = Math.round(Epics.getDouble(PV_BLUICE.beamsize_v_act1$VAL))/1000.0;
				fullWidthLabel.setText("H: " + Double.toString(widthVal));
				fullHeightLabel.setText("V: " + Double.toString(heightVal));
			}			
		}
		
		gd = new GridData();
		Label beamWidthLabel = new Label(beasizeButtonGroup, SWT.NONE);
		beamWidthLabel.setLayoutData(gd);
		beamWidthLabel.setText("H:");
		beamWidthLabel.setBackground(GUI.BGCOLOR);
		
		String[] widthList =	{ "0.025", "0.05","0.075","0.1", "0.2","1.0","2.0","5.0" };
		String[] heightList =	{ "0.025", "0.05","0.075","0.1", "0.2","1.0","2.0","5.0" };
	
		comboBeamWidth = new CCombo(beasizeButtonGroup,SWT.None);
		comboBeamWidth.setItems(widthList);
		comboBeamWidth.setFont(GUI.FONT_MOTOR_RQSPOS);
		comboBeamWidth.setEnabled(false);
		gd = new GridData();
		gd.widthHint = 60;
		gd.horizontalAlignment = SWT.FILL;
		
		comboBeamWidth.setLayoutData(gd);
		comboBeamWidth.addListener (SWT.Verify, new Listener () {
			public void handleEvent (Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		comboBeamWidth.addKeyListener(new KeyListener() {

			@Override
			public void keyPressed(KeyEvent arg0) {}
			public void keyReleased(KeyEvent arg0) {
				if(arg0.keyCode == SWT.CR)
					handleUserspecifiedBeamsizeChange();
				//highResCam.redraw();
			}
			
		});
		comboBeamWidth.addSelectionListener(new SelectionListener()
		{

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) { }
			public void widgetSelected(SelectionEvent arg0) {
				// TODO Auto-generated method stub
				handleUserspecifiedBeamsizeChange();
				highResCam.redraw();
			}
			
		});
		gd = new GridData();
		Label beamHeightLabel = new Label(beasizeButtonGroup, SWT.NONE);
		beamHeightLabel.setLayoutData(gd);
		beamHeightLabel.setText("V:");
		beamHeightLabel.setBackground(GUI.BGCOLOR);
		
		comboBeamHeight = new CCombo(beasizeButtonGroup,SWT.None);
		comboBeamHeight.setBackground(GUI.COLOR_WHITE);
		comboBeamHeight.setItems(heightList);
		comboBeamHeight.setFont(GUI.FONT_MOTOR_RQSPOS);
		comboBeamHeight.setEnabled(false);
		
		gd = new GridData();
		gd.widthHint = 60;
		gd.horizontalAlignment = SWT.FILL;
		comboBeamHeight.setLayoutData(gd);
		comboBeamHeight.addListener (SWT.Verify, new Listener () {
			public void handleEvent (Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		comboBeamHeight.addKeyListener(new KeyListener() {

			@Override
			public void keyPressed(KeyEvent arg0) {}
			public void keyReleased(KeyEvent arg0) {
				if(arg0.keyCode == SWT.CR)
					handleUserspecifiedBeamsizeChange();
			}
			
		});
		comboBeamHeight.addSelectionListener(new SelectionListener()
		{

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) { }
			public void widgetSelected(SelectionEvent arg0) {
				// TODO Auto-generated method stub
				handleUserspecifiedBeamsizeChange();
			}
		});
	}

	private void createHutchTab(CTabFolder ctFolder)
	{
		Composite compHutch = new Composite(ctFolder,SWT.NONE);
		compHutch.setBackground(GUI.BGCOLOR);
		compHutch.setLayout(new GridLayout(2,false));

		WdgHutchControls hutchControls = new WdgHutchControls(compHutch, SWT.NONE);
		hutchControls.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.DEFAULT, SWT.CENTER, false, true);
		gd.widthHint = controlsWidth;
		hutchControls.setLayoutData(gd);
		
		hutchCam = new WdgCamera(compHutch,SWT.None, DBCamera.getImageURL(WdgCamera.CameraType.HUTCH, cameraSize), WdgCamera.CameraType.HUTCH, cameraSize);
		hutchCam.setSize(DBCamera.getCameraDimensions(WdgCamera.CameraType.HUTCH, cameraSize));
		gd = new GridData(SWT.FILL,SWT.FILL,true,true);
		hutchCam.setLayoutData(gd);
		
		hutchTab = new CTabItem(ctFolder, SWT.NONE);
		hutchTab.setText(" View Hutch ");
		hutchTab.setControl(compHutch);
	}

	private void createAPSTab(CTabFolder ctFolder)
	{
		Composite compAPS = new Composite(ctFolder,SWT.NONE);
		compAPS.setBackground(GUI.BGCOLOR);
		compAPS.setLayout(new FillLayout());

		String apsStatusURL = "https://www3.aps.anl.gov/aod/blops/plots/smallStatusPlot.png";
		apsStatus = new WdgCamera(compAPS, SWT.NONE, apsStatusURL, WdgCamera.CameraType.APSSTATUS, cameraSize);
		apsStatus.setSize(new Point(600,490));

		apsTab = new CTabItem(ctFolder, SWT.NONE);
		apsTab.setText(" APS Status ");
		apsTab.setControl(compAPS);
	}
	
	private void createToolsTab(CTabFolder ctFolder)
	{
		/////////////////////////////////// Tools tab /////////////////////////
		
		Composite compTools = new Composite(ctFolder,SWT.NONE);
		compTools.setBackground(GUI.BGCOLOR);
		compTools.setLayout(new GridLayout(2,false));
		
		controlsTools = new WdgCameraControls(compTools, SWT.NONE,moveGroup, cameraSize, 3);
		controlsTools.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.DEFAULT, SWT.CENTER, false, true);
		gd.widthHint = controlsWidth;
		controlsTools.setLayoutData(gd);
		
		toolsCam = new WdgCamera(compTools,SWT.None, DBCamera.getImageURL(WdgCamera.CameraType.HIGHRES, cameraSize), WdgCamera.CameraType.HIGHRES, cameraSize);
		toolsCam.setSize(DBCamera.getCameraDimensions(WdgCamera.CameraType.HIGHRES, cameraSize));
		gd = new GridData(SWT.FILL,SWT.FILL,true,true);
		toolsCam.setLayoutData(gd);
	
		toolsTab = new CTabItem(ctFolder, SWT.NONE);
		toolsTab.setText(" Sample: Tools ");
		toolsTab.setControl(compTools);
	}

	private void createDiffTab(CTabFolder ctFolder)
	{
		diffTabContents = new WdgDiffTab(ctFolder, false);
		GridData gd = new GridData(SWT.FILL,SWT.FILL,true,true);
		diffTabContents.setLayoutData(gd);
		
		diffTab = new CTabItem(ctFolder, SWT.NONE);
		diffTab.setText(" Diff. Image ");
		diffTab.setControl(diffTabContents);
	}
	
	protected void connectListeners()
	{
		diffTabContents.connectListeners();
		controls.connectListeners();
		controlsHighres.connectListeners();
		controlsTools.connectListeners();
		
		Epics.addMonitor(PV_BLUICE.beamsize_h_act1$VAL, new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String pvName, final DBR value)
			{
				GUI.asyncExec(new Runnable() {public void run()
	 			{
					double val = Math.round(Epics.dbrToDouble(value))/1000.0;
					fullWidthLabel.setText("H: " + Double.toString(val));
	 			}});
			}
			
		});
		
		Epics.addMonitor(PV_BLUICE.beamsize_v_act1$VAL, new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String pvName, final DBR value)
			{
				GUI.asyncExec(new Runnable() {public void run()
	 			{
					double val = Math.round(Epics.dbrToDouble(value))/1000.0;
					fullHeightLabel.setText("V: " + Double.toString(val));
	 			}});
			}
			
		});
		for (final BEAMSIZE_OPTION colliButton : BEAMSIZE_OPTION.values()) {
			
		Epics.addMonitor(PV_BLUICE.beamsize.getBeamsizePV(colliButton.ordinal()), new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String pvName, final DBR value)
			{
				final String pv = pvName; 
				GUI.asyncExec(new Runnable() {public void run()
	 			{	
					Button colliB = buttonGroup.get(colliButton);
					if(Epics.dbrToInt(value)== 1)
						colliB.setSelection(true);
					else
						colliB.setSelection(false);
					
					if(colliButton == BEAMSIZE_OPTION.USER_DEFINED) {
						comboBeamWidth.setEnabled(colliB.getSelection());
						comboBeamHeight.setEnabled(colliB.getSelection());
					}
	 			}});
			}
			
		});
		}
		
		Epics.addMonitor(PV_BLUICE.beamsize_h, new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String pvName, final DBR value)
			{
				GUI.asyncExec(new Runnable() {public void run()
	 			{
					double val = Math.round(Epics.dbrToDouble(value)*1000.0)/1000.0;
					comboBeamWidth.setText(Double.toString(val));
					highResCam.redraw();
	 			}});
			}
		});
		
		Epics.addMonitor(PV_BLUICE.beamsize_v, new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String pvName, final DBR value)
			{				
				GUI.asyncExec(new Runnable() {public void run()
	 			{
					double val = Math.round(Epics.dbrToDouble(value)*1000.0)/1000.0;
					comboBeamHeight.setText(Double.toString(val));
					highResCam.redraw();
	 			}});
			}
			
		});
		
		Epics.addMonitor(PV_COLLIMATOR.ActivePreset, new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String pvName, final DBR value)
			{
				
				GUI.asyncExec(new Runnable() {public void run()
	 			{	
				for( BEAMSIZE_OPTION colliButton : BEAMSIZE_OPTION.values()) {
					
					Button colliB = buttonGroup.get(colliButton);;
					if(Epics.dbrToInt(value) == 1 && colliButton == BEAMSIZE_OPTION.FULL_BEAM)
						colliB.setForeground(GUI.COLOR_RED);	
					else if (Epics.dbrToInt(value) == 2 && colliButton == BEAMSIZE_OPTION.TWENTY_MICRONS)
						colliB.setForeground(GUI.COLOR_RED);
					else if (Epics.dbrToInt(value) == 3 && colliButton == BEAMSIZE_OPTION.TEN_MICRONS)
						colliB.setForeground(GUI.COLOR_RED);
					else if (Epics.dbrToInt(value) == 5 && colliButton == BEAMSIZE_OPTION.FIVE_MICRONS)
						colliB.setForeground(GUI.COLOR_RED);
					else
						colliB.setForeground(colliButton.getColor());
				}
	 			}});
			}
		});
		
		ctFolder.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0)
			{
				if (!(arg0.item instanceof CTabItem)) { Log.error("Unexpected widget: "+arg0.item); return; }
				CTabItem newTab = (CTabItem)arg0.item;
				
				((TabSample)getParent().getParent()).showExteriorControls(diffTab!=newTab);
			
				if (lowResTab == newTab)
				{
					if (null != lastViewer) lastViewer.notifyIsVisible(false);
					//lowResCam.bLoadImages.set(true);
					lowResCam.notifyIsVisible(true);
					lastViewer = lowResCam;
				}
				else if (highResTab == newTab)
				{
					if (null != lastViewer) lastViewer.notifyIsVisible(false);
					highResCam.notifyIsVisible(true);
					lastViewer = highResCam;
				}
				else if (hutchTab == newTab)
				{
					if (null != lastViewer) lastViewer.notifyIsVisible(false);
					hutchCam.notifyIsVisible(true);
					lastViewer = hutchCam;
				}
				else if (apsTab == newTab)
				{
					if (null != lastViewer) lastViewer.notifyIsVisible(false);
					apsStatus.notifyIsVisible(true);
					lastViewer = apsStatus;
				}
				else if (diffTab == newTab)
				{
					if (null != lastViewer) lastViewer.notifyIsVisible(false);
					diffTabContents.notifyIsVisible(true);
					lastViewer = diffTabContents;
				}
				else if (toolsTab == newTab)
				{
					if (null != lastViewer) lastViewer.notifyIsVisible(false);
					toolsCam.notifyIsVisible(true);
					lastViewer = toolsCam;
				}
				else
				{
					Log.error("Unknown video tab: "+newTab);
				}
			}
		});
		
		// don't set the low res cam load images flag to true here, it will
		// be set when WdgAllCameras.notifyIsVisible(true) is called
		lastViewer = lowResCam;
		ctFolder.setSelection(0);
	}
	
	public void handleUserspecifiedBeamsizeChange () {
		Epics.putDouble(PV_BLUICE.beamsize_h, Double.valueOf(comboBeamWidth.getText()));
		Epics.putDouble(PV_BLUICE.beamsize_v, Double.valueOf(comboBeamHeight.getText()));
	}
	public void notifyIsVisible(boolean isVisible)
	{
		CTabItem selectedTab = ctFolder.getSelection();
		if (selectedTab == lowResTab) { lowResCam.notifyIsVisible(isVisible); }
		if (selectedTab == highResTab) { highResCam.notifyIsVisible(isVisible); }
		if (selectedTab == hutchTab) { hutchCam.notifyIsVisible(isVisible); }
		if (selectedTab == apsTab) { apsStatus.notifyIsVisible(isVisible); }
		if (selectedTab == diffTab) { diffTabContents.notifyIsVisible(isVisible); }
	}
	
	public void showDiffViewer()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			ctFolder.setSelection(diffTab);
			TabSample.get().get(1).showExteriorControls(false);
			TabSample.get().get(1).allCams.diffTabContents.setSFGraphVisible(false, true);
			notifyIsVisible(true);
		}});
	}
}
