package gov.anl.gmca.sample;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_A_FLOW;
import gov.anl.gmca.io.PVLookup.PV_A_SHIFT;
import gov.anl.gmca.io.PVLookup.PV_ROBOT;
import gov.anl.gmca.screening.ui.WdgRinseWash;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

public class WdgAnnealingControls  implements ClientListener {
	
	GridData gd;
	Button goWash, cancelShiftButton, annealShiftButton;
	public Combo comboWash,comboShiftAnneal, comboAnnealOption, comboMoveAnneal, comboRinseWash,comboRinseWashTime;
	Spinner comboWashTimes;
	protected Epics.EpicsValueMonitor monitor = null;
	Thread washThread = null;
	static double omega, crystalX, crystalY, horizon;
	boolean isRinse = false;
	public WdgAnnealingControls (Composite parent, int style) {
		
		Client.addListener(this);
		
		String[] washOptions = { "wash", "rinse"};
		String[] annealingOptions = {"Flow", "Shift" };
		String[] annealList =	{ "1", "2", "3", "4", "5", "6", "7", "8", "9", "10" };
		String[] annealShiftList = {"1.00", "2.00", "3.00", "4.00", "5.00"};
		String[] annealMoveList = {"5.00", "10.00", "15.00", "20.00", "25.00", "30.00", "35.00", "40.00"};
		
		Group washGroup = new Group(parent, SWT.NONE);
		washGroup.setBackground(GUI.COLOR_BLUE_BUTTON);
		washGroup.setLayout(new GridLayout(3,false));
		washGroup.setText("Wash/Rinse by Robot");
		
		gd = new GridData();
		gd.widthHint = 250;
		gd.horizontalSpan = 2;
		washGroup.setLayoutData(gd);	
		
		WdgRinseWash rinsewash = new WdgRinseWash(washGroup,style);
		
		Group shiftGroup = new Group(parent, SWT.NONE);
		shiftGroup.setBackground(GUI.COLOR_BLUE_BUTTON);
		shiftGroup.setLayout(new GridLayout(2,false));
		shiftGroup.setText("Annealing by Shift/Flow Control");
		gd = new GridData();
		gd.widthHint = 250;
		gd.horizontalSpan = 2;
		shiftGroup.setLayoutData(gd);
		
		Group groupShiftTime = new Group(shiftGroup, SWT.NONE);
		groupShiftTime.setBackground(GUI.COLOR_BLUE_BUTTON);
		groupShiftTime.setText("Shift(mm)");
		gd = new GridData();
		groupShiftTime.setLayoutData(gd);
		groupShiftTime.setLayout(new GridLayout(1,false));
		
		comboShiftAnneal = new Combo(groupShiftTime,SWT.None);
		comboShiftAnneal.setItems(annealShiftList);
		comboShiftAnneal.setText("4.75");
		comboShiftAnneal.setFont(GUI.FONT_MOTOR_RQSPOS);
		gd = new GridData();
		gd.widthHint = 80;
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		comboShiftAnneal.setLayoutData(gd);
		comboShiftAnneal.setEnabled(false);
		
		Group groupMoveTime = new Group(shiftGroup, SWT.NONE);
		groupMoveTime.setBackground(GUI.COLOR_BLUE_BUTTON);
		groupMoveTime.setText("Time(sec)");
		gd = new GridData();
		groupMoveTime.setLayoutData(gd);
		groupMoveTime.setLayout(new GridLayout(1,false));
		
		comboMoveAnneal = new Combo(groupMoveTime,SWT.None);
		comboMoveAnneal.setItems(annealMoveList);
		comboMoveAnneal.setFont(GUI.FONT_MOTOR_RQSPOS);
		comboMoveAnneal.setText("10.0");
		gd = new GridData();
		gd.widthHint = 80;
		gd.horizontalAlignment = SWT.CENTER;
		gd.grabExcessHorizontalSpace = true;
		comboMoveAnneal.setLayoutData(gd);
		
		comboAnnealOption = new Combo(shiftGroup,SWT.READ_ONLY);
		comboAnnealOption.setItems(annealingOptions);
		comboAnnealOption.setFont(GUI.FONT_MOTOR_RQSPOS);
		comboAnnealOption.setText("Flow");
		gd = new GridData();
		gd.widthHint = 80;
		gd.horizontalAlignment = SWT.CENTER;
		gd.horizontalSpan = 2;
		gd.grabExcessHorizontalSpace = true;
		comboAnnealOption.setLayoutData(gd);
		
		comboAnnealOption.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) {
				if (comboAnnealOption.getText().equals("Flow")) {
					comboShiftAnneal.setEnabled(false);
				} else {
					comboShiftAnneal.setEnabled(true);
				}
				
			}
			
		});		
		
		gd = new GridData();
		gd.widthHint = 100;
		annealShiftButton = new Button(shiftGroup, SWT.PUSH);
		annealShiftButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		annealShiftButton.setText("Anneal");
		annealShiftButton.setLayoutData(gd);
		gd.grabExcessHorizontalSpace = true;
		annealShiftButton.setEnabled(false);
		annealShiftButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) { }
			public void widgetSelected(SelectionEvent arg0) {
				double annealTime = Double.valueOf(comboMoveAnneal.getText());
				double annealShift = Double.valueOf(comboShiftAnneal.getText());
				if (comboAnnealOption.getText().equals("Shift")) {
					annealShiftButton.setEnabled(false);
					Epics.putDouble(PV_A_SHIFT.A_Time, annealTime);
					Epics.putDouble(PV_A_SHIFT.inboard, annealShift);
					Epics.putInteger(PV_A_SHIFT.Anneal, 1);
				} else if (comboAnnealOption.getText().equals("Flow")) {
					double annealFlowTime = Double.valueOf(comboMoveAnneal.getText());
					annealShiftButton.setEnabled(false);
					Epics.putDouble(PV_A_FLOW.A_Time, annealFlowTime);
					Epics.putInteger(PV_A_FLOW.Anneal, 1);
				} else {
					Log.warn("Wrong anenaling option");
				}
			}
			
		});
		
		gd = new GridData();
		gd.widthHint = 100;
		cancelShiftButton = new Button(shiftGroup, SWT.PUSH);
		cancelShiftButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		cancelShiftButton.setText("Cancel");
		cancelShiftButton.setLayoutData(gd);
		gd.grabExcessHorizontalSpace = true;
		cancelShiftButton.setEnabled(false);
		cancelShiftButton.addSelectionListener(new SelectionListener () {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) { }
			public void widgetSelected(SelectionEvent arg0) {
				
				if (comboAnnealOption.getText().equals("Shift")) {
					Epics.putInteger(PV_A_SHIFT.Cancel, 1);
				} else if (comboAnnealOption.getText().equals("Flow")) {
					Epics.putInteger(PV_A_FLOW.Cancel, 1);
				} else {
					Log.warn("Wrong anenaling option");
				}
				
			}
			
		});
		
		AppState.addListener(AppState.Operation.OP_ANNEALING, new AppState.Listener()
	 	{
	 		@Override
			public void stateChanged(Operation op, boolean allowed) {	
	 			updateEnableState();
	 		}
		});
		monitor = new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String pvName, DBR value)
			{
				
				final String _pvName=pvName;
				final DBR _value = value;

				if (_pvName.equals(PV_A_FLOW.Anneal.toString())) {
					GUI.asyncExec(new Runnable(){public void run()
					{
						int pvVal = Epics.dbrToInt(_value);
						if(pvVal == 1) {
							if (Client.isMaster() )
				    		{
								annealShiftButton.setEnabled(false);
								cancelShiftButton.setEnabled(true);
				    		}
						} else {
							if (Client.isMaster() )
				    		{
								annealShiftButton.setEnabled(true);
								cancelShiftButton.setEnabled(false);
				    		}
						}
						
					}
					});
				} else if (_pvName.equals(PV_A_FLOW.Cancel.toString())) {
					GUI.asyncExec(new Runnable(){public void run()
					{
						int pvVal = Epics.dbrToInt(_value);
						if(pvVal == 1) {
							if (Client.isMaster() )
				    		{
								//cancelButton.setEnabled(false);
				    		}
						} else {
							if (Client.isMaster() )
				    		{
								//cancelButton.setEnabled(true);
				    		}
						}
						
					}
					});
				} else if (_pvName.equals(PV_A_SHIFT.Anneal.toString())) {
					GUI.asyncExec(new Runnable(){public void run()
					{
						int pvVal = Epics.dbrToInt(_value);
						if(pvVal == 1) {
							if (Client.isMaster() )
				    		{
								annealShiftButton.setEnabled(false);
								cancelShiftButton.setEnabled(true);
				    		}
						} else {
							if (Client.isMaster() )
				    		{
								annealShiftButton.setEnabled(true);
								cancelShiftButton.setEnabled(false);
				    		}
						}
						
					}
					});
				} else if (_pvName.equals(PV_A_SHIFT.Cancel.toString())) {
					GUI.asyncExec(new Runnable(){public void run()
					{
						int pvVal = Epics.dbrToInt(_value);
						if(pvVal == 1) {
							if (Client.isMaster() )
				    		{
								cancelShiftButton.setEnabled(false);
				    		}
						} else {
							if (Client.isMaster() )
				    		{
								cancelShiftButton.setEnabled(true);
				    		}
						}
						
					}
					});
				} else if (_pvName.equals(PV_A_FLOW.A_Time.toString())) {
					GUI.asyncExec(new Runnable(){public void run()
					{
						if(Client.isMaster())
							comboMoveAnneal.setText(Epics.dbrToString(_value));
					}
					});
				} else if (_pvName.equals(PV_A_SHIFT.A_Time.toString())) {
					GUI.asyncExec(new Runnable(){public void run()
					{
						if(Client.isMaster())
							comboMoveAnneal.setText(Epics.dbrToString(_value));
					}
					});
				}
			};
		
		};
		Epics.addMonitor(PV_A_FLOW.Anneal, monitor);
		Epics.addMonitor(PV_A_FLOW.Cancel, monitor);
		
		Epics.addMonitor(PV_A_SHIFT.Anneal, monitor);
		Epics.addMonitor(PV_A_SHIFT.Cancel, monitor);
		
		Epics.addMonitor(PV_A_FLOW.A_Time, monitor);
		Epics.addMonitor(PV_A_SHIFT.A_Time, monitor);
		
	}
	
	
	public void updateEnableState()
	{
		final String mounted = Epics.getString(PV_ROBOT.pause);
		GUI.asyncExec(new Runnable(){public void run()
		{
			// TODO: disableExceptions
    		if (Client.isMaster()&& AppState.isOperationAllowed(Operation.OP_ANNEALING))
    		{
    			annealShiftButton.setEnabled(true);
    		}
    		else
    		{
    			cancelShiftButton.setEnabled(false);
    			annealShiftButton.setEnabled(false);
    		}
		}});
	}

	@Override
	public void masterChanged(boolean isMaster) {
		// TODO Auto-generated method stub
		GUI.asyncExec(new Runnable(){public void run()
		{
			updateEnableState();
		}});
	}
}
