package gov.anl.gmca.sample;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevEnergy;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_BLUICE;
import gov.anl.gmca.io.PVLookup.PV_PMAC;
import gov.anl.gmca.sample.DBBeamSize.BeamSize;
import gov.anl.gmca.sample.WdgCamera.CameraType;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.SwingUtil;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.graphics.Color;

import java.awt.*;
import java.awt.geom.Rectangle2D;

public class WdgBeamsizeBox
{
	public ListenerList<WdgBeamsizeBox> redrawLL;
	private static final java.awt.Color BORDER_COLOR = new java.awt.Color(128,0,0);
	
	public enum BEAMSIZE_OPTION
	{
		FIVE_MICRONS 	("5u", 		GUI.COLOR_THICK_ORANGE), 
		TEN_MICRONS  	("10u", 	GUI.COLOR_LIGHT_BLUE6), 
		TWENTY_MICRONS 	("20u", 	GUI.COLOR_BROWNRED), 
		SLITS 			("Slits", 	GUI.COLOR_DARK_CYAN), 
		FULL_BEAM 		("Full", 	GUI.COLOR_MAGENTA),
		USER_DEFINED 	("User", 	GUI.COLOR_DARK_GREEN);
		
		public String colli;
		public Color color;
		
		public String getColli() { return colli; };
		public Color getColor() { return color; };
		
		BEAMSIZE_OPTION (String colli,Color color)
		{
			this.colli = colli;
			this.color = color;
		}
	}
	
	protected Epics.EpicsValueMonitor monitor 	= null;
	
	public static enum colliSelection { redraw5um, redraw10um, redraw20um , redrawGS , redrawFull; }

	public WdgBeamsizeBox()
	{
		redrawLL = new ListenerList<WdgBeamsizeBox>(this);
		
		monitor = new Epics.EpicsValueMonitor()
		{
			@Override
			public void valueChanged(String pvName, DBR value) {
				// TODO Auto-generated method stub
				final String _pvName = pvName;
				final DBR _value = value;
				if(_pvName.equals(PV_BLUICE.beamsizeOption.toString()) ||
						_pvName.equals(PV_BLUICE.px2mm_v_lo$VAL.toString()) ||
						_pvName.equals(PV_BLUICE.px2mm_h_lo$VAL.toString()) ||
						_pvName.equals(PV_PMAC.ActPos.motor("beam_size_x").toString()) ||
						_pvName.equals(PV_PMAC.ActPos.motor("beam_size_y").toString()) ||
						_pvName.equals(PV_BLUICE.beamsize_h.toString()) ||
						_pvName.equals(PV_BLUICE.beamsize_v.toString())) 
				{
					redrawLL.notifyListeners();
				}
			}
		};
		
		Epics.addMonitor(PV_PMAC.ActPos.motor("beam_size_x"), monitor);
		Epics.addMonitor(PV_PMAC.ActPos.motor("beam_size_y"), monitor);
		Epics.addMonitor(PV_BLUICE.beamsize_h, monitor);
		Epics.addMonitor(PV_BLUICE.beamsize_v, monitor);
		Epics.addMonitor(PV_BLUICE.px2mm_v_lo$VAL, monitor);
		Epics.addMonitor(PV_BLUICE.px2mm_h_lo$VAL, monitor);
		Epics.addMonitor(PV_BLUICE.px2mm_v_hi$VAL, monitor);
		Epics.addMonitor(PV_BLUICE.px2mm_h_hi$VAL, monitor);
		
		Epics.addMonitor(PV_BLUICE.beamsizeOption, monitor);
	}
	
	// returns the size in pixels of the drawn beam size
	private void drawBeamSize(Graphics2D g2, WdgCamera cam, int xmid, int ymid, double width_mm, double height_mm, java.awt.Color color, boolean drawRectangle)
	{
    	if (g2==null) return;
    	
		double width_px = cam.mm2pxH(width_mm);
		double height_px = cam.mm2pxV(height_mm);
		int halfwidth_px = (int)(width_px/2.0);
		int halfheight_px = (int)(height_px/2.0);
		int x = xmid-halfwidth_px;
		int y = ymid-halfheight_px;
		
		g2.setColor(color);
		if (drawRectangle)
			g2.drawRect(x, y, (int)width_px, (int)height_px);
		else
			g2.drawOval(x, y, (int)width_px,(int)height_px);
	}
	
	public void draw(Graphics2D g2, int _imgWidth, int _imgHeight, WdgCamera cam)
	{
    	if (cam.camType != CameraType.HIGHRES) return;
    	
		int xmid = _imgWidth/2;
    	int ymid = _imgHeight/2;
    	int flag_circle = TabSample.isCircle;
    	
    	// user beam size options
		for(BEAMSIZE_OPTION colliOption : BEAMSIZE_OPTION.values()) {
			
			// only draw if this option is selected
			boolean isSelected = 1==Epics.getInteger(PV_BLUICE.beamsize.getBeamsizePV(colliOption.ordinal()));
			if (!isSelected) continue;
			
			if (colliOption==BEAMSIZE_OPTION.FIVE_MICRONS) {
				drawBeamSize(g2, cam, xmid, ymid, 0.005, 0.005, GUI.COLOR_THICK_ORANGE_AWT, false);
			} 
			if (colliOption==BEAMSIZE_OPTION.TEN_MICRONS) {
				drawBeamSize(g2, cam, xmid, ymid, 0.010, 0.010, GUI.COLOR_LIGHT_BLUE6_AWT, false);
			} 
			if (colliOption==BEAMSIZE_OPTION.TWENTY_MICRONS) {
				drawBeamSize(g2, cam, xmid, ymid, 0.020, 0.020, GUI.COLOR_BROWNRED_AWT, false);
			}
			if (colliOption==BEAMSIZE_OPTION.SLITS) {
				double guardslitX = Epics.getDouble(PV_PMAC.ActPos.motor("beam_size_x"));
				double guardslitY = Epics.getDouble(PV_PMAC.ActPos.motor("beam_size_y"));
				drawBeamSize(g2, cam, xmid, ymid, guardslitX, guardslitY, GUI.COLOR_BROWNRED_AWT, true);
			}
			//Log.note("beamWidth "+beamWidthMM + " beamHeight "+beamHeightMM);
			if (colliOption==BEAMSIZE_OPTION.FULL_BEAM) {
				double beamWidthFull = Epics.getDouble(PV_BLUICE.beamsize_h_act1$VAL)/1000.0; //converting to mm
				double beamHeightFull = Epics.getDouble(PV_BLUICE.beamsize_v_act1$VAL)/1000.0;
				
				if (beamWidthFull == 0.0 || beamHeightFull == 0.0) {
					beamWidthFull = Double.valueOf(DBConfig.get("beamsizebox_H"));
					beamHeightFull = Double.valueOf(DBConfig.get("beamsizebox_V"));
				}
				drawBeamSize(g2, cam, xmid, ymid, beamWidthFull, beamHeightFull, GUI.COLOR_MAGENTA_AWT, true);
			}
			if (colliOption==BEAMSIZE_OPTION.USER_DEFINED) {
				double guardslitX = Epics.getDouble(PV_BLUICE.beamsize_h);
				double guardslitY = Epics.getDouble(PV_BLUICE.beamsize_v);
				drawBeamSize(g2, cam, xmid, ymid, guardslitX, guardslitY, GUI.COLOR_DARK_GREEN_AWT, true);
			}
		}
		
		// current beam size

		double beamWidthMM=0, beamHeightMM=0;
		boolean drawRectangle=false;
		g2.setFont(GUI.FONT_DEFAULT_AWT);
		if (Util.beamSizeCalculated()) {
			int colli_um = DevCollimatorCtrl.getCollimatorSize().size_um;
			double energy_keV = DevEnergy.getDevice().getPosition();
			BeamSize beamsize = DBBeamSize.getBeamSize(colli_um, energy_keV);
			if (null!=beamsize) {
				beamWidthMM = beamsize.width_um / 1000.0;
				beamHeightMM = beamsize.height_um / 1000.0;
				drawRectangle = false;
			}
		} else {
			beamWidthMM = TabSample.beamsizeboxX;
			beamHeightMM = TabSample.beamsizeboxY;
			drawRectangle = flag_circle==0;
		}
		if (0!=beamWidthMM) {
			drawBeamSize(g2, cam, xmid, ymid, beamWidthMM, beamHeightMM, GUI.COLOR_DARK_GREEN_AWT, drawRectangle);

			// beam size messages
			int beamWidth = (int) cam.mm2pxH(beamWidthMM);
			int beamHeight = (int) cam.mm2pxV(beamHeightMM);
			int beamsizeOption = Epics.getInteger(PV_BLUICE.beamsizeOption);
			if (beamsizeOption != 0) {
				if (beamWidth < 2 || beamHeight < 2) {
					SwingUtil.drawStringWithBorder(g2, "Beamsize box too small", java.awt.Color.red, BORDER_COLOR, 10, 20);
				}
				if (beamWidth > _imgWidth || beamHeight > _imgHeight) {
					SwingUtil.drawStringWithBorder(g2, "Beamsize box too large", java.awt.Color.red, BORDER_COLOR, 10, 20);
				}
			}
		}

		/* draw scale bar */
		
		// constants
		int marginX=10,marginY=10,lineSpacing=3;
    	int scalebarHeightPx = 3;
    	int scalebarWidthMaxPx = (int)((double)_imgWidth/6.0);
    	int scalebarWidthMinPx = 20;
    	double bar20widthPx = (int)cam.mm2pxH(0.020);

    	// optimal scale bar size
    	double scalebarWidthMM = 0.020;
    	if 		(bar20widthPx > scalebarWidthMaxPx) scalebarWidthMM = 0.010;
    	else if (bar20widthPx < scalebarWidthMinPx) scalebarWidthMM = 0.050;
    	else										scalebarWidthMM = 0.020;
    	int scalebarWidthPx = (int)cam.mm2pxH(scalebarWidthMM);

    	// text dimensions
    	String text = (int)(scalebarWidthMM*1000.0)+"\u00B5m";
		Rectangle2D extent = g2.getFontMetrics().getStringBounds(text, g2);
		int textBaseHeight = (int)extent.getY();
		int textWidth = (int)(extent.getWidth());
		int textHeight = (int)(extent.getHeight());

    	// scale bar and text positions
    	int rectX = (int)(_imgWidth-marginX-scalebarWidthPx);
    	int rectY = (int)(_imgHeight-marginY-textHeight-lineSpacing-scalebarHeightPx);
    	int strX = (int)(_imgWidth-marginX-textWidth);
    	int strY = (int)(_imgHeight-marginY-textHeight-textBaseHeight);

    	// draw scale bar and text
    	g2.setColor(BORDER_COLOR);
    	g2.fillRect(rectX-1, rectY-1, scalebarWidthPx+2, scalebarHeightPx+2);
    	g2.setColor(java.awt.Color.red);
    	g2.fillRect(rectX, rectY, scalebarWidthPx, scalebarHeightPx);
    	SwingUtil.drawStringWithBorder(g2, text, java.awt.Color.red, BORDER_COLOR, strX, strY);
    	
    	//showUserClick (gc);
	}
}
