package gov.anl.gmca.sample;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.Limits;
import gov.anl.gmca.common.ui.CCombo;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.common.ui.WdgControlEntry.EntryType;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.hutch.DevPmacMotor;
import gov.anl.gmca.hutch.GrpWdgMove;
import gov.anl.gmca.hutch.WdgMotor;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.PVLookup.PV_PMAC21;
import gov.anl.gmca.sample.WdgSampleRotate.GetRotateN;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

public class WdgCameraControls extends Composite implements ClientListener
{	
	protected double[] dropdownLowZoomChoices = { 1, 2, 4, 6, 8, 10, 12};
	protected double[] dropdownHighZoomChoices = { 1, 2, 4, 6, 8, 10, 12, 14, 15};
	protected double rotateNValue = 1.0;
	protected WdgSampleRotate rotate;
	GridData gd = new GridData();
	Button moveRightButton, moveUpButton, moveLeftButton, moveDownButton;
	CCombo comboMove;
	final WdgControlEntry rotateNEntry;
	int tab;
	WdgCamera.CameraSize cameraSize;
	WdgSampleInOut backlightConstruct;
	public WdgCameraControls(Composite parent, int style,GrpWdgMove moveGroup, WdgCamera.CameraSize _cameraSize, int _tab)
	{
		super(parent,style);
		cameraSize = _cameraSize;
		tab = _tab;
		Client.addListener(this);
		setBackgroundMode(SWT.INHERIT_FORCE);
		
		setLayout(new GridLayout(1,false));
	
		Composite actionComp = new Composite(this,SWT.NONE);
		actionComp.setBackground(parent.getBackground());
		GridLayout gl = new GridLayout(2,false);
		
		actionComp.setLayout(gl);
		gd = new GridData();
		gd.verticalAlignment = SWT.FILL;
		actionComp.setLayoutData(gd);
		
		// tab=0 for lowres tab=1 for highres
		if ( tab == 0 ) {
			WdgMotor wdgZoom = new WdgMotor(actionComp,SWT.NONE,moveGroup, "LowRes Zoom", null, "camera_zoom", parent.getBackground(), dropdownLowZoomChoices, 2);
			if (cameraSize != WdgCamera.CameraSize.SMALL) 
			{
				gd = new GridData(SWT.FILL,SWT.FILL,true,true);
				gd.horizontalSpan = 2;
			}
			else {
				gd = new GridData();
				gd.widthHint = 125;
				gd.horizontalSpan = 2;
			}
				
			wdgZoom.setLayoutData(gd);
		} else if ( tab == 1 || tab == 3) {
			WdgMotor wdgZoom = new WdgMotor(actionComp,SWT.NONE,moveGroup, "HighRes Zoom",null, "camera_zoom_2", parent.getBackground(), dropdownHighZoomChoices, 2);
			gd = new GridData();
			if (cameraSize != WdgCamera.CameraSize.SMALL) 
			{
				gd = new GridData(SWT.FILL,SWT.FILL,true,true);
				gd.horizontalSpan = 2;
			}
			else {
				gd = new GridData();
				gd.widthHint = 125;
				gd.horizontalSpan = 2;
			}
			wdgZoom.setLayoutData(gd);
		}
	
		Group nGroup = new Group(actionComp, SWT.NONE);
		nGroup.setBackground(parent.getBackground());
		nGroup.setText("Rotate +-n");
		nGroup.setFont(GUI.FONT_DEFAULT);
		gd = new GridData(SWT.FILL,SWT.FILL,true,true);
		nGroup.setLayoutData(gd);
		
		nGroup.setLayout(new GridLayout(2,false));
		
		rotateNEntry = new WdgControlEntry(nGroup, SWT.NONE, EntryType.RW_COMBO, new WdgControlEntry.ModelInterface()
		{
			@Override public void setValue(Object value)
			{
				rotateNValue = (Double)value;
			}
			@Override public Object getModelValue()
			{
				return rotateNValue;
			}
			@Override public double[] getFloatChoices()
			{
				return new double[]{.5,1,5,10,30,45,60};
			}
			@Override public int getDecimals() 				{ return 2; 		}
			@Override public Limits getLimits() 			{ return DevPmacMotor.get("gonio_omega").getLimits(); }
		});
		rotateNEntry.setText(1.0);
		gd = new GridData();
		gd.widthHint = 80;
		rotateNEntry.setLayoutData(gd);
		rotateNEntry.setEnabled(true);
		
		Label nLabel = new Label(nGroup, SWT.NONE);
		nLabel.setText("deg");
		nLabel.setBackground(parent.getBackground());
		gd = new GridData();
		nLabel.setLayoutData(gd);
		
		rotate = new WdgSampleRotate(nGroup, SWT.VERTICAL, new GetRotateN()
		{
			@Override public double getRotateN() { return rotateNValue; }
		});
		rotate.setLayoutData(new GridData(SWT.LEFT, SWT.DEFAULT, true, false, 3, 1));
	
		int arrowX = 120;
		int arrowY = 150;
		int buttonWidth = 27;
		int buttonHeight = 27;
		int margin = -2;
		
		Group groupMove = new Group(actionComp, SWT.NONE);
		groupMove.setBackground(parent.getBackground());
		groupMove.setText("Move XY");
		groupMove.setFont(GUI.FONT_DEFAULT);
		gd = new GridData();
		gd.widthHint = 115;
		gd.heightHint = 140;
		groupMove.setLayoutData(gd);
		groupMove.setLayout(new GridLayout(2,false));
		
		String[] itemList =	{ "1.0", "5.0","10.0","100.0" };
		
		comboMove = new CCombo(groupMove,SWT.None);
		comboMove.setItems(itemList);
		comboMove.setText("1.0");
		comboMove.setFont(GUI.FONT_DEFAULT);
		comboMove.setBackground(GUI.COLOR_WHITE);
		gd = new GridData();
		gd.widthHint = 70;
		gd.horizontalAlignment = SWT.FILL;
		comboMove.setLayoutData(gd);
		comboMove.addListener (SWT.Verify, new Listener () {
			public void handleEvent (Event e) {
				Util.allowOnlyAlpha(e);
			}
		});
		Label lblUnits = new Label(groupMove,SWT.None);
		lblUnits.setText("um");
		lblUnits.setBackground(parent.getBackground());

		// move
		
		Composite moveComp = new Composite(groupMove, SWT.NONE);
		moveComp.setBackground(parent.getBackground());
		moveComp.setSize(buttonWidth*3, buttonHeight*3);
		moveComp.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		gd = new GridData();
		gd.horizontalAlignment = SWT.CENTER;
		gd.verticalAlignment = SWT.CENTER;
		moveComp.setLayoutData(gd);
		
		moveUpButton = new Button(moveComp, SWT.ARROW|SWT.UP);
		moveUpButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		moveUpButton.setSize(buttonWidth,buttonHeight);
		moveUpButton.setLocation(buttonWidth,0);
		gd = new GridData();
		moveUpButton.setLayoutData(gd);
		
		moveUpButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				handleArrows(0,1);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		moveDownButton = new Button(moveComp, SWT.ARROW|SWT.DOWN);
		moveDownButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		moveDownButton.setSize(buttonWidth,buttonHeight);
		moveDownButton.setLocation(buttonWidth,buttonHeight*2);
		gd = new GridData();
		moveDownButton.setLayoutData(gd);
		moveDownButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				handleArrows(0,-1);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});

		moveLeftButton = new Button(moveComp, SWT.ARROW|SWT.LEFT);
		moveLeftButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		moveLeftButton.setSize(buttonWidth,buttonHeight);
		moveLeftButton.setLocation(0,buttonHeight);
		gd = new GridData();
		moveLeftButton.setLayoutData(gd);
		moveLeftButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				handleArrows(-1,0);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
		
		moveRightButton = new Button(moveComp, SWT.ARROW|SWT.RIGHT);
		moveRightButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		moveRightButton.setSize(buttonWidth,buttonHeight);
		moveRightButton.setLocation(buttonWidth*2,buttonHeight);
		gd = new GridData();
		moveRightButton.setLayoutData(gd);
		moveRightButton.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetSelected(SelectionEvent e)
			{
				handleArrows(1,0);
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});

		
		if (cameraSize != WdgCamera.CameraSize.SMALL) 
		{
			if(tab == 0 || tab == 1) {
				
				if (tab == 0 )
					new WdgSampleSlider(actionComp,SWT.NONE,"LowRes Intensity", "lowres", 5);
				else
					new WdgSampleSlider(actionComp,SWT.NONE,"HighRes Intensity", "highres", 5);
		
				//new WdgSampleSlider(actionComp,SWT.NONE,"Ringlight Intensity", "ring", 5);
			
				Group groupBacklight = new Group(actionComp, SWT.NONE);
				groupBacklight.setBackground(parent.getBackground());
				groupBacklight.setText("Backlight");
				gd = new GridData(SWT.FILL,SWT.FILL,true,true);
				gd.horizontalSpan = 2;
				groupBacklight.setLayoutData(gd);
				groupBacklight.setLayout(new GridLayout(3,false));
				
				new DevInOut("Backlight",PV_PMAC21.bo13_VAL, PV_PMAC21.bi20_VAL );
				backlightConstruct = new WdgSampleInOut(groupBacklight,SWT.NONE, "Backlight",PV_PMAC21.bo13_VAL,PV_PMAC21.bi20_VAL);
				new WdgSampleSlider(actionComp,SWT.NONE,"Backlight Intensity","back", 5);
			
			} else if (tab == 3) {
				WdgAnnealingControls controlsTools;
				if (!Util.getBID().equals("23b"))
					controlsTools = new WdgAnnealingControls(actionComp, SWT.NONE);
			}
		}
		
		AppState.addListener(Operation.MOVE_COLL_CTRL, new AppState.Listener()
		{
			@Override public void stateChanged(Operation op, boolean allowed)
			{
		    	updateEnableState();
			}
		});
		AppState.addListener(Operation.MOVE_SAMPLE, new AppState.Listener()
		{
			@Override public void stateChanged(Operation op, boolean allowed)
			{
		    	updateEnableState();
			}
		});
		
	}
	public void handleArrows (int xAxis, int yAxis) {
		
		int resX = 0,resY = 0;
		int isHighRes = 0;
		if(!Client.isMaster()) {
			Log.error("Can't move XY, this client is not master");
			return;
		}
		if(comboMove.getText() == null || comboMove.getText().equals(""))
			return;
		if (tab == 0) {
			Point lowResCamSize = DBCamera.getCameraDimensions(WdgCamera.CameraType.LOWRES, cameraSize);
			resX = lowResCamSize.x;
			resY = lowResCamSize.y;
			isHighRes = 0;
		} else if (tab == 1 || tab == 3) {
			Point highResCamSize = DBCamera.getCameraDimensions(WdgCamera.CameraType.HIGHRES, cameraSize);
			resX = highResCamSize.x;
			resY = highResCamSize.y;
			isHighRes = 1;
		} else {
				
		}
	
		String resStr =xAxis*Double.valueOf(comboMove.getText())+"um "+ yAxis*Double.valueOf(comboMove.getText())+"um ";
		String movCmd = "caput "+DBConfig.get("app_beamlineID")+":bi:pointclick " + isHighRes +" "+ resStr+resX+" "+resY;
		
		System.out.println(movCmd);
		ExecTask runningTask = ExecTask.new_ExecTask(movCmd,"Move ",true);
		runningTask.start();
		runningTask.waitFor();		
	}
	
	public void updateEnableState()
	{
		final boolean isSampleMove = Client.isMaster() && AppState.isOperationAllowed(Operation.MOVE_SAMPLE);
		final boolean isColli = Client.isMaster() && AppState.isOperationAllowed(Operation.MOVE_COLL_CTRL);
		GUI.asyncExec(new Runnable(){public void run()
		{
			if(backlightConstruct!=null)
				backlightConstruct.deviceButton.setEnabled(isColli);
			
			moveRightButton.setEnabled(isSampleMove);
			moveUpButton.setEnabled(isSampleMove);
			moveLeftButton.setEnabled(isSampleMove);
			moveDownButton.setEnabled(isSampleMove);
			
			/* check for backlight availability for optimize beam */
			if(backlightConstruct!=null)
					backlightConstruct.deviceButton.setEnabled(isSampleMove);	
    		
		}});
	}

	@Override
	public void masterChanged(boolean isMaster) {
		// TODO Auto-generated method stub		
			updateEnableState();		
	}
	
	protected void connectListeners()
	{
		rotate.connectListeners();
		updateEnableState();
	}
}
