package gov.anl.gmca.sample;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.io.Web;
import gov.anl.gmca.sample.DBCamera.ZoomDirection;
import gov.anl.gmca.sample.WdgCamera.CameraSize;
import gov.anl.gmca.sample.WdgCamera.CameraType;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import java.util.ArrayList;

public class WdgHutchControls extends Composite
{
	public WdgHutchControls(Composite parent, int style)
	{
		super(parent,style);
		setLayout(new GridLayout(1,false));
		
		ArrayList<String> presetList = getHutchCameraPresets();
		GridData gd;
		for (final String preset : presetList)
		{
			final Button button = new Button(this, SWT.PUSH);
			button.setText(preset);
			button.setFont(GUI.FONT_DEFAULT);
			gd = new GridData();
			gd.grabExcessHorizontalSpace = true;
			gd.horizontalAlignment = SWT.FILL;
			button.setLayoutData(gd);
			
			button.addSelectionListener(new SelectionListener()
			{
				public void widgetSelected(SelectionEvent se)
				{
					String queryURL = DBCamera.getHutchControlURL()+"?gotoserverpresetname="+preset;
					Util.getURLText(queryURL, "Hutch camera move request: ");
				}
				public void widgetDefaultSelected(SelectionEvent arg0) { }
			});
		}
		
		Composite spacer = new Composite(this, SWT.NONE);
		spacer.setBackground(GUI.BGCOLOR);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint = 50;
		spacer.setLayoutData(gd);
		
		addZoomButton(ZoomDirection.IN);
		addZoomButton(ZoomDirection.OUT);
	}
	
	private void addZoomButton(ZoomDirection dir)
	{
		Button button = new Button(this, SWT.PUSH);
		button.setText("Zoom "+dir.toString().toLowerCase());
		button.setFont(GUI.FONT_DEFAULT);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		button.setLayoutData(gd);
		
		button.addSelectionListener(new SelectionListener()
		{
			public void widgetSelected(SelectionEvent se)
			{
				// construct URL
				String url=DBCamera.getZoomURL(CameraType.HUTCH, CameraSize.LARGE, dir);
				
				// load URL
				Log.note("Sending zoom "+(dir.toString().toLowerCase())+" command to hutch camera");
				System.out.println(url);
				Web.get().openConnection(url);
			}
			public void widgetDefaultSelected(SelectionEvent arg0) { }
		});
	}
	
	protected ArrayList<String> getHutchCameraPresets()
	{
		ArrayList<String> ret = new ArrayList<String>();
		
		if (Main.SIM_LEVEL<2)
		{
    		String queryURL = DBCamera.getHutchControlURL()+"?query=presetposall";
    		ArrayList<String> result = Util.getURLText(queryURL,null);
    		for (String line : result)
    		{
    			if (line.indexOf("presetposno") == 0)
    			{
    				String preset = line.substring(13); 
    				ret.add(preset);
    			}
    		}
		}
		
		return ret;
	}
}
