package gov.anl.gmca.scan;

import gov.anl.gmca.common.Command;
import gov.anl.gmca.scan.ModelScan.ElementEdge;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Calendar;

public abstract class CmdScan extends Command
{
	protected volatile boolean stopRequested = false;
	public void requestStop() { stopRequested = true; }
	
	@Override
	public abstract void run();
	
    protected static SimpleDateFormat timestampFormat = new SimpleDateFormat("MMM_dd_yy_HH_mm_ss");
	
	public static String createDirAndGeneratePrefixWithDir(ElementEdge ee, boolean isEdgeScan)
	{
		Calendar now = Calendar.getInstance();
		
		// make the directory if needed
		File selectedDir = new File(ModelScan.get().getFullDirectory());
		boolean dirOK = true;
		if (!selectedDir.exists())
		{
			if (!selectedDir.mkdirs())
			{
				Log.error("Couldn't create "+selectedDir.getAbsolutePath()+", using ~/bluice_temp.");
				dirOK = false;
			}
		}
		if (!selectedDir.canWrite())
		{
			Log.error("Can't write to "+selectedDir.getAbsolutePath()+", using ~/bluice_temp.");
			dirOK = false;
		}
		if (!dirOK)
		{
			selectedDir = new File(System.getProperty("user.home")+"/bluice_temp");
		}
		
		System.out.println("Writing to "+selectedDir.getAbsolutePath());
		
		String timeStamp = timestampFormat.format(now.getTime());
		int pid = Util.getPid();
		String userPrefix = ModelScan.get().getPrefix();
		
		String scanTypeLabel = isEdgeScan?"_EDGE_":"_FLUOR_";
		
		String filePrefix =
			selectedDir.getAbsolutePath()+"/"+
			userPrefix+"_"+pid+
			scanTypeLabel+
			timeStamp+"_"+
			ee.element+"_"+ee.edge;
		
		System.out.println("prefix="+filePrefix);
		
		return filePrefix;
	}
}
