package gov.anl.gmca.scan;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.hutch.DevUndulator;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.io.PVLookup.PV_SCAN;
import gov.anl.gmca.scan.ModelScan.ScanPlotMode;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.ActionResult;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;

public class DevEdgeScan
{
	protected static boolean FAST_SIM = false;
	protected static boolean simStopRequested = false;
	public static ListenerList<DevEdgeScan> llData = new ListenerList<DevEdgeScan>(null);
	
	protected 	static final String scriptFile 		= "edgeScan.pl";
	
	protected static ExecTask scanTask = null;
	
	public DevEdgeScan()
	{
		Epics.addMonitor(PV_SCAN.fastscanData, new Epics.EpicsValueMonitor()
		{
			@Override public void valueChanged(String pvName, DBR value)
			{
				llData.notifyListeners(Epics.dbrToDoubleArray(value));
			}
		});
	}
	
	public static ActionResult run()
	{
		return runScript();
	}
	
	public static void stop()
	{
		if (null != scanTask && scanTask.isRunning())
		{
			scanTask.asyncStop();
		}
		if (Main.SIM_LEVEL>=1)
		{
			simStopRequested = true;
		}
	}
	
	protected static ActionResult runScript()
	{
		if (null != scanTask)
		{
			return new ActionResult("Scan script already running, ignoring.", true);
		}
		
		DevScan.get().setScanPlotMode(ScanPlotMode.EDGE, true);
		
		// TODO: pass the edge time in, maybe to run()
		double scanTimeSec = ModelScan.get().getEdgeTime();
		String mcaMode = "auto";
		String idFlag = (null == DevUndulator.getDevice())?"noID":DBConfig.get("edgeScan_idFlag");
		String edgeShiftFlag = ModelScan.get().getAdaptiveScan()?"adaptive":"user";
		String cmd =
			Util.getPerlPrefix()+scriptFile+" "+
			ModelScan.get().getElementEdge()+" "+
			"count="+scanTimeSec+" "+
			"mca="+mcaMode+" "+
			"edgeShift="+edgeShiftFlag+" "+
			"bluice nognuplot nochooch nologfile "+
			"scan="+DBConfig.get("edgeScan_scanFlag")+" "+
			DBConfig.get("edgeScan_fakeFlag")+" "+
			idFlag;
		
		Log.note("Running script: "+cmd);
		
		scanTask = ExecTask.new_ExecTask(cmd, scriptFile, true);
		scanTask.start();
		scanTask.waitFor();
		int exitVal = scanTask.exitValue;
		scanTask = null;
		
		if (0 == exitVal)	return ActionResult.SUCCESS;
		else				return new ActionResult(scriptFile+" returned "+exitVal, true);
	}
	
	public static void setData(double[] d)
	{
		Epics.putDoubleArray(PV_SCAN.fastscanData, d);
	}
}
