package gov.anl.gmca.scan;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.WdgMain;
import gov.anl.gmca.gui.WdgMain.TAB;
import gov.anl.gmca.scan.ModelScan.ScanPlotMode;
import gov.anl.gmca.scan.ModelScan.TRACE_ID;
import gov.anl.gmca.scan.ui.ModelGraph;
import gov.anl.gmca.scan.ui.ModelGraph.Trace;
import gov.anl.gmca.scan.ui.WdgPlotSelect;
import gov.anl.gmca.util.Point2D;
import gov.anl.gmca.util.Util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;

// ported from BIWGraph.tcl

public class FileBIP
{
	public static void load(ModelGraph graph, String fullpath)
	{
		// read the cif file from disk
		FileCIF cif = new FileCIF(fullpath);

		// load global parameters
		String plotModeString 		 = cif.getValue(0,"_graph_info.mode");
		String graphTitle 			 = cif.getValue(0, "_graph_title.text");
		String xLabel 				 = cif.getValue(0,"_graph_axes.xLabel");
		String roiLowString 		 = cif.getValue(0,"_graph_info.roiLow");
		String roiHighString 		 = cif.getValue(0,"_graph_info.roiHigh");
		String inflectionString 	 = cif.getValue(0,"_graph_info.inflection");
		String peakString 			 = cif.getValue(0,"_graph_info.peak");
		String remoteString 		 = cif.getValue(0,"_graph_info.remote");
		String infl_fP_String 		 = cif.getValue(0,"_graph_info.infl_fP");
		String infl_fPP_String 		 = cif.getValue(0,"_graph_info.infl_fPP");
		String peak_fP_String 		 = cif.getValue(0,"_graph_info.peak_fP");
		String peak_fPP_String 		 = cif.getValue(0,"_graph_info.peak_fPP");
		String spectrumEnergyString	 = cif.getValue(0,"_graph_info.spectrumEnergy");
		String spectrumElementString = cif.getValue(0,"_graph_info.spectrumElement");

		// get number of traces
		int traceCount = cif.getBlockCount();

		// track which traces to show, then send the info to the device all at once
		HashMap<TRACE_ID,Boolean> showTrace = new HashMap<TRACE_ID,Boolean>();
		for (TRACE_ID id : TRACE_ID.values()) showTrace.put(id, false);
		
		// create the traces
		for (int traceIndex = 1; traceIndex <= traceCount; traceIndex++)
		{
			// get trace name and label
			int subTraceNumber = 1; // multiple sub-traces not supported
			String subTraceName 	= cif.getValue(traceIndex, "_sub_trace.name", 		subTraceNumber);
			String subTraceYLabel 	= cif.getValue(traceIndex, "_sub_trace.yLabels", 	subTraceNumber);

			// check which trace this is
			TRACE_ID traceID = null;
			for (TRACE_ID id : TRACE_ID.values())
			{
				if (id.toString().equals(subTraceName))
				{
					traceID = id;
					break;
				}
			}
			if (null == traceID) { System.out.println("ERROR: unsupported trace "+subTraceName); continue; }

			// get the number of data for the trace
			int dataCount = cif.getValueCount(traceIndex,"_sub_trace.x");
			if (0 == dataCount) continue;
			
			// read in data for the sub-traces
			ArrayList<Point2D> points = new ArrayList<Point2D>();
			for (int dataIndex = 1; dataIndex <= dataCount; dataIndex++)
			{
				// get x for this data index
				double x = Double.parseDouble(cif.getValue(traceIndex, "_sub_trace.x", dataIndex));
				
				// get y for this data index
				double y = Double.parseDouble(cif.getValue(traceIndex, "_sub_trace.y1", dataIndex));
				
				// add the point to the trace
				//System.out.println("addPoint("+x+","+y+")");
				points.add(new Point2D(x,y));
			}

			// determine graph bounds
			double xMin=points.get(0).x, xMax=points.get(0).x;
			double yMin=points.get(0).y, yMax=points.get(0).y;
			
			for (Point2D pt : points)
			{
				if (pt.x<xMin) xMin=pt.x;
				if (pt.x>xMax) xMax=pt.x;
				if (pt.y<yMin) yMin=pt.y;
				if (pt.y>yMax) yMax=pt.y;
			}
			
			DevScan.get().setDefaultXMin(xMin, false);
			DevScan.get().setDefaultXMax(xMax, false);
			DevScan.get().setDefaultYMin(traceID, yMin, false);
			DevScan.get().setDefaultYMax(traceID, yMax, false);
			
			DevScan.get().setGraphYMin(traceID, yMin, false);
			DevScan.get().setGraphYMax(traceID, yMax, false);
			DevScan.get().setGraphXMin(xMin, false);
			DevScan.get().setGraphXMax(xMax, false);

			DevScan.get().setGraphYLabel(traceID, subTraceYLabel, false);
			DevScan.get().setGraphTrace(traceID, points, false);
			showTrace.put(traceID, true);
		}

		// general scan info
		DevScan.get().setGraphTitle(null==graphTitle?"":graphTitle, false);
		DevScan.get().setGraphXLabel(null==xLabel?"":xLabel, false);
		DevScan.get().setRightTab(1, false); // show plot

		// set mode-dependent values
		ScanPlotMode mode = ScanPlotMode.fromString(plotModeString);
		DevScan.get().setScanPlotMode(mode, false);
		switch (mode)
		{
		case EDGE:
			// chooch values
			try
			{
				DevScan.get().setROILow(Double.NaN, false);
				DevScan.get().setROIHigh(Double.NaN, false);
				DevScan.get().setInflection(Double.parseDouble(inflectionString), false);
				DevScan.get().setPeak(Double.parseDouble(peakString), false);
				DevScan.get().setRemote(Double.parseDouble(remoteString), false);
				DevScan.get().setDefaultInflection(Double.parseDouble(inflectionString), false);
				DevScan.get().setDefaultPeak(Double.parseDouble(peakString), false);
				DevScan.get().setDefaultRemote(Double.parseDouble(remoteString), false);
				DevScan.get().setInfl_fP(Double.parseDouble(infl_fP_String), false);
				DevScan.get().setInfl_fPP(Double.parseDouble(infl_fPP_String), false);
				DevScan.get().setPeak_fP(Double.parseDouble(peak_fP_String), false);
				DevScan.get().setPeak_fPP(Double.parseDouble(peak_fPP_String), false);
				DevScan.get().setSpectrumEnergy(Double.NaN, false);
			}
			catch (NullPointerException npe) { npe.printStackTrace(); }
			catch (NumberFormatException nfe) { nfe.printStackTrace(); }
			
			// show all 3 plots
			WdgPlotSelect.showTraces(new TRACE_ID[]{TRACE_ID.edge, TRACE_ID.fPrime, TRACE_ID.fDoublePrime}, TRACE_ID.fPrime, TRACE_ID.fDoublePrime, false);
			break;
		case SPECTRUM:
			// ROI
			try
			{
				DevScan.get().setInflection(Double.NaN, false);
				DevScan.get().setPeak(Double.NaN, false);
				DevScan.get().setRemote(Double.NaN, false);
				DevScan.get().setROILow(Double.parseDouble(roiLowString), false);
				DevScan.get().setROIHigh(Double.parseDouble(roiHighString), false);
				DevScan.get().setSpectrumEnergy(Double.parseDouble(spectrumEnergyString), false);
				ModelScan.get().showOnlyIndicatorForElement(spectrumElementString);
			}
			catch (NullPointerException npe) { npe.printStackTrace(); }
			catch (NumberFormatException nfe) { nfe.printStackTrace(); }
			
			// show "edge" trace
			WdgPlotSelect.showTraces(new TRACE_ID[]{TRACE_ID.edge}, TRACE_ID.edge, null, false);
			break;
		case NONE:
			WdgPlotSelect.showTraces(new TRACE_ID[]{}, null, null, false);
			break;
		}

		// update!
		DevScan.get().sendUpdate();

		// show scan tab in this client
		GUI.asyncExec(new Runnable(){public void run()
		{
			WdgMain.setVisibleTab(TAB.Scan);
		}});
	}
	
	public static void saveFile(String fullPath, ModelGraph graph, ModelScan scan)
	{
		// open the file for writing
		File file = new File(fullPath);
		PrintWriter pw = null;
		try
		{
			pw = new PrintWriter(new FileOutputStream(file));
		}
		catch (FileNotFoundException e) { e.printStackTrace(); return; }
		
		// write information about the graph title 
		pw.println();
		pw.println("# general info");
		pw.println("_graph_info.mode            \""+scan.getScanPlotMode()+"\"");
		pw.println("_graph_title.text           \""+graph.getTitle()+"\"");
		pw.println("_graph_axes.xLabel          \""+graph.getXLabel()+"\"");
		pw.println("_graph_info.roiLow          \""+Util.printDouble(scan.getROILow(),4)+"\"");
		pw.println("_graph_info.roiHigh         \""+Util.printDouble(scan.getROIHigh(),4)+"\"");
		pw.println("_graph_info.inflection      \""+Util.printDouble(scan.getInflection(),4)+"\"");
		pw.println("_graph_info.peak            \""+Util.printDouble(scan.getPeak(),4)+"\"");
		pw.println("_graph_info.remote          \""+Util.printDouble(scan.getRemote(),4)+"\"");
		pw.println("_graph_info.infl_fP         \""+Util.printDouble(scan.getInfl_fP(),4)+"\"");
		pw.println("_graph_info.infl_fPP        \""+Util.printDouble(scan.getInfl_fPP(),4)+"\"");
		pw.println("_graph_info.peak_fP         \""+Util.printDouble(scan.getPeak_fP(),4)+"\"");
		pw.println("_graph_info.peak_fPP        \""+Util.printDouble(scan.getPeak_fPP(),4)+"\"");
		pw.println("_graph_info.spectrumEnergy  \""+Util.printDouble(scan.getSpectrumEnergy(),4)+"\"");
		pw.println("_graph_info.spectrumElement \""+scan.getElementEdge().element+"\"");
		
		// write out the traces
		for (Trace trace : graph.getTraces())
		{
			saveTrace(pw, trace, trace.getID().toString());
		}

		pw.println();
		pw.close();
	}
	
	protected static void saveTrace(PrintWriter pw, Trace trace, String traceName)
	{
		pw.println();
		pw.println("# data for trace "+traceName);
		pw.println("data_");
		pw.println("_trace.name 	"+traceName);
		pw.println("_trace.hide		0");
		pw.println();
		pw.println("loop_");
		pw.println("_sub_trace.name");
		pw.println("_sub_trace.yLabels");
		pw.println(trace.getID()+" \""+trace.getYLabel()+"\"");
		pw.println();
		
		pw.println("loop_");
		pw.println("_sub_trace.x");
		pw.println("_sub_trace.y1");

		for (Point2D point : trace.getPoints())
		{
			pw.println(point.x+" "+point.y);
		}
	}
}
