package gov.anl.gmca.scan.ui;

import gov.anl.gmca.common.ui.NBFileDialog;
import gov.anl.gmca.common.ui.NBMessageBox;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Disk;
import gov.anl.gmca.sample.ImageViewer;
import gov.anl.gmca.scan.CmdScan;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.scan.ModelScan.ScanPlotMode;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.MenuListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;

public class ContextMenu
{
	private static ContextMenu inst;
	public static ContextMenu get() { if (null==inst) inst=new ContextMenu(); return inst; }

	public void addMenu(WdgGraph wdg)
	{
		Menu menu = new Menu(GUI.getShell(), SWT.POP_UP);
	    
		//MenuItem itemTitle = new MenuItem(menu, SWT.NONE);
		//itemTitle.setText("Scan Menu");
		
		//MenuItem itemDivider = new MenuItem(menu, SWT.NONE);
		//itemDivider.setText("---------------");
		
		final MenuItem itemZoomOut = new MenuItem(menu, SWT.PUSH);
		itemZoomOut.setText("Zoom out");
		itemZoomOut.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				ModelScan.get().zoomOut();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
	    
		final MenuItem itemShowAll = new MenuItem(menu, SWT.PUSH);
		itemShowAll.setText("Show all");
	    itemShowAll.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				ModelScan.get().showAll();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
	    
		final MenuItem fileLoadScan = new MenuItem(menu, SWT.PUSH);
		fileLoadScan.setText("Load scan");
		fileLoadScan.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				TabScan scanTab = TabScan.get();
				if (null != scanTab)
				{
					NBFileDialog fd = new NBFileDialog(GUI.getShell(),SWT.NONE);
					fd.setFilterPath(ModelScan.get().getFullDirectory());
					fd.setFilterExtensions(new String[]{"*.bip"});
					String fullpath = fd.openStr();
					if (null != fullpath)
					{
						scanTab.loadScan(fullpath);
					}
				}
			}
		});
	    
	    MenuItem itemSaveSnapshot = new MenuItem(menu, SWT.PUSH);
	    itemSaveSnapshot.setText("Save to PNG");
	    itemSaveSnapshot.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				saveScreenshot();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
	    
	    MenuItem itemPrint = new MenuItem(menu, SWT.PUSH);
	    itemPrint.setText("Print");
	    itemPrint.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				boolean okToPrint = Util.popupOkCancel("Print", "Send current plot to the default printer?");
				if (okToPrint)
				{
					Log.note("sending plot to printer");
					TabScan.get().getGraphWidget().printScreenshot();
				}
				else
				{
					Log.note("print aborted");
				}
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
	    
	    /*final MenuItem itemShowPDF = new MenuItem(menu, SWT.PUSH);
	    itemShowPDF.setText("Open Chooch PDF");
	    itemShowPDF.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				ModelScan.get().openChoochPDF();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});*/
	    
		final MenuItem itemHelp = new MenuItem(menu, SWT.PUSH);
		itemHelp.setText("Graph controls");
		itemHelp.addSelectionListener(new SelectionListener()
	    {
			@Override public void widgetSelected(SelectionEvent e)
			{
				final NBMessageBox mb = new NBMessageBox(GUI.getShell(), SWT.NONE);
				mb.setText("Graph controls");
				mb.setMessage
				(
					"To zoom in, drag with the left mouse\n"+
					"button inside the graph area to define\n"+
					"new view boundaries.\n"+
					"\n"+
					"To zoom out, use the 'zoom out' or 'show all'\n"+
					"options in the context menu.\n"//+
					//"\n"+
					//"To pan, hold control while dragging with the\n"+
					//"left mouse button down."
				);
				new Thread(new Runnable(){public void run(){mb.open();}}, "Scan help popup").start();
			}
			@Override public void widgetDefaultSelected(SelectionEvent e) {}
		});
	    
	    menu.addMenuListener(new MenuListener()
	    {
			@Override public void menuShown(MenuEvent e)
			{
				//String pdfPath = ModelScan.get().getChoochPDFPath();
				//boolean pdfAvailable = Disk.get().exists(pdfPath);
				//itemShowPDF.setEnabled(pdfAvailable);
				
				itemShowAll.setEnabled(ModelScan.get().canZoomOut());
				itemZoomOut.setEnabled(ModelScan.get().canZoomOut());
				fileLoadScan.setEnabled(Client.isMaster());
			}
			@Override public void menuHidden(MenuEvent e) {}
		});
	    
	    wdg.setMenu(menu);
	}
	
	private void saveScreenshot()
	{
		String imgPath = ModelScan.get().getFullDirectory();
		
		NBFileDialog fileDialog = new NBFileDialog(GUI.getShell(), SWT.SAVE);
		fileDialog.setFilterPath(imgPath);
		String filename = fileDialog.openStr();

		// if cancel clicked, choose a filename that doesn't exist
		if (null == filename)
		{
			String defaultPrefixWithDir;
			if (ModelScan.get().getScanPlotMode()==ScanPlotMode.EDGE || ModelScan.get().getScanPlotMode()==ScanPlotMode.SPECTRUM)
			{
				defaultPrefixWithDir = CmdScan.createDirAndGeneratePrefixWithDir(ModelScan.get().getElementEdge(), ModelScan.get().getScanPlotMode()==ScanPlotMode.EDGE);
			}
			else
			{
				defaultPrefixWithDir = imgPath+"/scan";
			}
			
			int suffix=1;
			String prefixWithDir = defaultPrefixWithDir;
			while (Disk.exists(prefixWithDir+".png"))
			{
				if (suffix==1000)
				{
					Log.error("Couldn't find a unique filename to save the scan screenshot (tried up to "+filename+"), aborting.");
					return;
				}
				prefixWithDir = Util.sprintf(defaultPrefixWithDir+"_%05d",suffix++);
			}
			
			filename = prefixWithDir+".png";
		}
		// but if they did specify one, add a suffix if needed
		else if (filename.indexOf('.')<0)
		{
			filename += ".png";
		}

		final String f_filename = filename;
		new Thread(new Runnable(){public void run()
		{
			// now write!
			TabScan.get().getGraphWidget().saveScreenshot(f_filename);
			ImageViewer.show(f_filename);
		}}).start();
	}
}
