package gov.anl.gmca.scan.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.ui.ModelGraph.Trace;
import gov.anl.gmca.util.Point2D;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

public class LayerPlot extends Layer
{
	private boolean lineAA = true;
	public void setLineAA(boolean b) { lineAA = b; }
	
	public LayerPlot(ModelGraph graph)
	{
		super(graph);
	}

	@Override protected Rectangle _getBounds()
	{
		return graph.getGraphBounds();
	}
	
	@Override
	protected void _update(GC gc)
	{
		drawBackground(gc);
		
		/*if (0 == graph.getTrace().size())
		{
			String message = graph.getMessage();
			if (null == message) message = "No plot data";
			
			drawMessage(gc, message);
			drawBorder(gc);
			return;
		}*/

		gc.setAntialias(SWT.OFF);
		
		// draw guidelines
		if (graph.getDrawGuideLines())
		{
			if (graph.drawGuideLines)
			{
				gc.setForeground(GUI.COLOR_GRAY);
				gc.setLineDash(new int[]{1,1});
				gc.setLineStyle(SWT.LINE_CUSTOM);
				
    			for (Double x : graph.getXMajorTicks())
    			{
    				Integer screenX = graph.getScreenCoordX(x);
    				if (null != screenX) gc.drawLine(screenX, 0, screenX, getSize().height);
    			}
			
    			for (Trace trace : graph.getTraces())
    			{
    				if (trace.getVisible())
    				{
	        			for (Double y : trace.getYMajorTicks())
	        			{
	        				Integer screenY = trace.getScreenCoordY(y);
	            			if (null!=screenY) gc.drawLine(0, screenY, getSize().width, screenY);
	        			}
    				}
    			}
    			
				gc.setForeground(GUI.COLOR_BLACK);
				gc.setLineStyle(SWT.LINE_SOLID);
			}
		}
		
		gc.setAntialias(lineAA?SWT.ON:SWT.OFF);
		
		// draw points
		gc.setLineWidth(1);
		int ptSize = 6;
		Point lastPoint = null;
		for (Trace trace : graph.getTraces())
		{
			//System.out.println("Drawing trace "+trace.getName()+" with "+trace.getPoints().size()+" points");
			
			if (!trace.getVisible()) continue;
			
			gc.setForeground(trace.getID().getColor());
			gc.setBackground(trace.getID().getColor());
			
			for (Point2D p2d : trace.getPoints())
			{
				Point screenPt = trace.getScreenCoord(p2d.x,p2d.y);
				if (null != screenPt)
				{
	    			gc.fillOval(screenPt.x-ptSize/2, screenPt.y-ptSize/2, ptSize, ptSize);
	    			
	    			if (null != lastPoint)
	    			{
	    				gc.drawLine(lastPoint.x, lastPoint.y, screenPt.x, screenPt.y);
	    			}
	    			lastPoint = screenPt;
				}
			}
			
			lastPoint = null;
		}
		
		gc.setAntialias(SWT.OFF);
		
		drawBorder(gc);
	}

	protected void drawBackground(GC gc)
	{
		gc.setBackground(graph.getEnabled()?GUI.COLOR_WHITE:GUI.COLOR_LIGHT_GRAY);
		gc.fillRectangle(getSize());
	}
	
	protected void drawMessage(GC gc, String message)
	{
		gc.setFont(GUI.FONT_GRAPH_MESSAGE);
		
		Point extent = gc.textExtent(message);
		
		int xcenter = graph.getXScaleBounds().width/2;
		int ycenter = graph.getLeftYScaleBounds().height/2;

		int x = xcenter - extent.x/2;
		int y = ycenter - extent.y/2;
		
		gc.drawText(message, x, y);
	}
	
	protected void drawBorder(GC gc)
	{
		gc.setLineStyle(SWT.LINE_SOLID);
		gc.setForeground(GUI.COLOR_BLACK);
		gc.drawRectangle(
				0,
				0,
				_getBounds().width-1,
				_getBounds().height-1);

		gc.setForeground(GUI.COLOR_GRAY);
		gc.drawRectangle(new Rectangle
		(
				1,
				1,
				_getBounds().width-3,
				_getBounds().height-3
		));
	}
	
}
