package gov.anl.gmca.scan.ui;

import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.scan.ModelScan.ScanPlotMode;
import gov.anl.gmca.scan.ModelScan.TRACE_ID;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.graphics.Point;

public class ListenerEdgeDisplay implements MouseListener, MouseMoveListener
{
	private static ListenerEdgeDisplay inst;
	public static ListenerEdgeDisplay get() { if (null==inst) inst=new ListenerEdgeDisplay(); return inst; }

	private boolean drag = false;
	
	public void addListeners(WdgGraph wdg)
	{
		wdg.addMouseListener(this);
		wdg.addMouseMoveListener(this);
	}

	private void updateDisplay(int screenX, int screenY)
	{
		// update bar location
		Double graphX = ModelGraph.get().getGraphCoordX(screenX-ModelGraph.get().getGraphBounds().x, false);
		if (graphX==null) return;
		
		WdgGraph.get().layerOverlays.edgeDisplayX = graphX;
		
		// update hover text
		Double fP = ModelGraph.get().getTraceValueAt(TRACE_ID.fPrime, graphX);
		Double fPP = ModelGraph.get().getTraceValueAt(TRACE_ID.fDoublePrime, graphX);
		WdgGraph.get().layerOverlays.hoverScreenPt = new Point(screenX,screenY);
		WdgGraph.get().layerOverlays.hoverText =
			"Energy = "+Util.printDouble(graphX,1)+" eV\n"+
			"f' = "+Util.printDouble(fP,2)+"\n"+
			"f\" = "+Util.printDouble(fPP,2);
		
		// redraw overlays only
		WdgGraph.get().redraw();
	}
	
	@Override public void mouseMove(MouseEvent e)
	{
		if (drag) updateDisplay(e.x,e.y);
	}


	@Override public void mouseDown(MouseEvent e)
	{
		boolean ctrlDown = 0!=(e.stateMask & SWT.CTRL);
		if (ctrlDown && ModelScan.get().getScanPlotMode() == ScanPlotMode.EDGE)
		{
			drag=true;
			updateDisplay(e.x,e.y);
		}
	}

	@Override public void mouseUp(MouseEvent e)
	{
		drag=false;
		WdgGraph.get().layerOverlays.edgeDisplayX = null;
		WdgGraph.get().layerOverlays.hoverText = "";
	}
	
	@Override public void mouseDoubleClick(MouseEvent e) {}
}
