package gov.anl.gmca.scan.ui;

import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.scan.FileBIP;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.scan.ModelScan.ScanListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

public class TabScan extends Tab
{
	protected static final Color BGCOLOR = GUI.COLOR_LIGHT_BLUE2;
	
	public WdgGraph getGraphWidget() { return graphWidget; }
	
	public WdgGraph graphWidget = null;
	public ModelGraph graphModel = null;
	protected ModelScan scanModel;
	protected DevScan scanDev;
	
	CTabFolder rightTabFolder;
	
	private static TabScan inst = null;
	public static TabScan get() { return inst; }
	
	public TabScan(Composite parent, int style)
	{
		super(parent,style);
		inst = this;
		
		scanModel = new ModelScan();
		scanDev = new DevScan(scanModel);
		scanModel.setDevice(scanDev);
		
		setLayout(new GridLayout(2,false));
		setBackground(GUI.BGCOLOR);

		GridData gd;
		
		////////////////////////// left side controls /////////////////////////////
		
		Composite controls = new WdgScanControls(this,SWT.NONE,scanModel,scanDev);
		controls.setBackground(GUI.BGCOLOR);
		gd = new GridData();
		gd.widthHint = WdgScanControls.CONTROLS_WIDTH;
		gd.verticalAlignment = SWT.FILL;
		gd.grabExcessVerticalSpace = true;
		controls.setLayoutData(gd);

		//////////////////////////// right side tabs //////////////////////////
		
		rightTabFolder = new CTabFolder(this, SWT.TOP|SWT.BORDER);
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.verticalAlignment = SWT.FILL;
		gd.grabExcessVerticalSpace = true;
		rightTabFolder.setLayoutData(gd);
		
		rightTabFolder.setFont(GUI.FONT_SCAN_TAB);
		rightTabFolder.setBackground(GUI.BGCOLOR);

		//////////////////////////// periodic table tab //////////////////////////
		
		Composite compPeriodicTable = new Composite(rightTabFolder, SWT.NONE);
		compPeriodicTable.setLayout(new GridLayout(1,false));
		compPeriodicTable.setBackground(GUI.COLOR_LIGHT_BLUE2);
		
		WdgPeriodicTable periodicTable = new WdgPeriodicTable(compPeriodicTable,SWT.NONE);
		periodicTable.setBackground(GUI.COLOR_LIGHT_BLUE2);
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;
		gd.horizontalAlignment = SWT.CENTER;
		gd.verticalAlignment = SWT.CENTER;
		periodicTable.setLayoutData(gd);

		CTabItem tabPeriodicTable = new CTabItem(rightTabFolder, SWT.NONE);
		tabPeriodicTable.setText("  Periodic Table  ");
		tabPeriodicTable.setControl(compPeriodicTable);
		
		//////////////////////////// plot tab ///////////////////////////////////////
		
		Composite compTabPlot = new Composite(rightTabFolder,SWT.NONE);
		compTabPlot.setBackground(GUI.BGCOLOR);
		compTabPlot.setBackground(GUI.COLOR_LIGHT_BLUE2);
		compTabPlot.setLayout(new GridLayout(1,false));
		
		//////// graph
		
		graphModel = new ModelGraph();
		
		graphWidget = new WdgGraph(compTabPlot, SWT.NONE, graphModel);
		graphWidget.setBackground(GUI.COLOR_LIGHT_BLUE2);
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.verticalAlignment = SWT.FILL;
		gd.grabExcessVerticalSpace = true;
		graphWidget.setLayoutData(gd);

		/////// controls

		WdgPlotControls graphControls = new WdgPlotControls(compTabPlot, SWT.NONE);
		graphControls.setBackground(GUI.COLOR_LIGHT_BLUE2);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		graphControls.setLayoutData(gd);
		
		CTabItem tabPlot = new CTabItem(rightTabFolder, SWT.NONE);
		tabPlot.setText("  Plot  ");
		tabPlot.setControl(compTabPlot);

		//////////////////////////// chooch tab ///////////////////////////////////////

		/*Composite compTabChooch = new Composite(rightTabFolder,SWT.NONE);
		compTabChooch.setBackground(GUI.BGCOLOR);
		compTabChooch.setBackground(GUI.COLOR_LIGHT_BLUE2);
		compTabChooch.setLayout(new GridLayout(1,false));
		
		choochWidget = new WdgChooch(compTabChooch);
		choochWidget.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		CTabItem tabChooch = new CTabItem(rightTabFolder, SWT.NONE);
		tabChooch.setText("  Chooch  ");
		tabChooch.setControl(compTabChooch);*/
		
		////////////////////////////// end tabs ///////////////////////////////////////
		
		rightTabFolder.setSelection(0);
		
		rightTabFolder.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent se) {}
			@Override public void widgetSelected(SelectionEvent se)
			{
				DevScan.get().setRightTab(rightTabFolder.getSelectionIndex(), true);
			}
		});
		
		ModelScan.get().addListener(new ScanListener()
		{
			@Override public void scanChanged(final ModelScan settings)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					if (2 != settings.getRightTab()) // chooch tab is disabled for now
					{
						rightTabFolder.setSelection(settings.getRightTab());
					}
				}});
			}
		});
		
		scanDev.connect();
	}
	
	public void loadScan(String fullpath)
	{
		FileBIP.load(graphModel, fullpath);
	}
	
	@Override
	public String getTabName() { return "Scan"; }
}
