package gov.anl.gmca.scan.ui;

import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.scan.ModelScan.AutoStep;
import gov.anl.gmca.scan.ModelScan.ScanListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import java.util.HashMap;

public class WdgAutoProgress extends Composite
{
	protected static HashMap<AutoStep, WdgStep> stepMap = new HashMap<AutoStep, WdgStep>();
	
	protected static class WdgStep extends Composite
	{
		protected static final Color UNHIGHLIGHTED_FG 	= GUI.COLOR_DARK_GRAY;
		protected static final Color UNHIGHLIGHTED_BG	= GUI.BGCOLOR;
		protected static final Color HIGHLIGHTED_FG 	= GUI.COLOR_LIGHT_GREEN;
		protected static final Color HIGHLIGHTED_BG 	= GUI.COLOR_DARK_GREEN;
		
		protected AutoStep step;
		protected String text;
		protected Label label;
		
		public WdgStep(Composite parent, AutoStep step, String text)
		{
			super(parent, SWT.NONE);
			
			this.step=step; this.text=text; stepMap.put(step, this);
			
			setBackground(UNHIGHLIGHTED_BG);
			setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
			GridLayout gl = new GridLayout(1,false);
			gl.marginHeight = 2;
			setLayout(gl);
			
			label = new Label(this, SWT.CENTER);
			label.setFont(GUI.FONT_SCAN_STATUS);
			label.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
			label.setForeground(UNHIGHLIGHTED_FG);
			label.setBackground(UNHIGHLIGHTED_BG);
			label.setText(text);
		}
		
		public void setHighlight(boolean highlight)
		{
			Color fg = highlight ? HIGHLIGHTED_FG : UNHIGHLIGHTED_FG;
			Color bg = highlight ? HIGHLIGHTED_BG : UNHIGHLIGHTED_BG;
			
	        setBackground(bg);
	        label.setBackground(bg);
	        label.setForeground(fg);
		}
	}
	
	protected AutoStep lastStep = AutoStep.NONE;
	
	WdgAutoProgress(Composite parent, int style)
	{
		super(parent, style);
		
    	GridData gd = new GridData(SWT.CENTER, SWT.DEFAULT, false, false);
    	gd.widthHint = WdgScanControls.CONTROLS_WIDTH - 50;
    	setLayoutData(gd);

    	setLayout(new FillLayout());
    	Group group = new Group(this, SWT.NONE);
    	
    	GridLayout gl = new GridLayout(1,false);
		gl.marginHeight = 5;
		gl.marginWidth = 5;
		gl.verticalSpacing = 5;
		group.setLayout(gl);
		
    	newLabel2(group, "Progress");
 
    	new WdgStep(group, AutoStep.MOVE_ENERGY,	"Move energy to edge");
    	new WdgStep(group, AutoStep.OPTIMIZE, 		"Optimize fluor. signal");
    	new WdgStep(group, AutoStep.SPECTRUM, 		"Take spectrum");
    	new WdgStep(group, AutoStep.EDGE_SCAN, 		"Do edge scan");
    	new WdgStep(group, AutoStep.RETRACT, 		"Retract fluor. detector");
    	
        ModelScan.get().addListener(new ScanListener()
        {
			@Override public void scanChanged(final ModelScan settings)
			{
				GUI.asyncExec(new Runnable(){public void run()
				{
					AutoStep currStep = settings.getAutoStep();
					
					if (lastStep != currStep)
					{
						if (AutoStep.NONE != lastStep && AutoStep.STOPPING != lastStep) stepMap.get(lastStep).setHighlight(false);
						if (AutoStep.NONE != currStep && AutoStep.STOPPING != currStep) stepMap.get(currStep).setHighlight(true);
					}
					
					lastStep = currStep;
				}});
			}
		});
	}
	
	protected Label newLabel2(Composite parent, String text)
	{
		Label label = new Label(parent, SWT.NONE);
		label.setLayoutData(new GridData(SWT.CENTER, SWT.DEFAULT, false,false));
		label.setFont(GUI.FONT_SCAN_STEP);
		label.setText(text);
		return label;
	}
}
