package gov.anl.gmca.scan.ui;

import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgControlEntry;
import gov.anl.gmca.common.ui.WdgControlEntry.EntryType;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.scan.ModelScan;
import gov.anl.gmca.scan.ModelScan.AutoStep;
import gov.anl.gmca.scan.ModelScan.InterStep;
import gov.anl.gmca.scan.ModelScan.ScanListener;
import gov.anl.gmca.scan.ModelScan.ScanPlotMode;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class WdgPlotControls extends Composite
{
	private WdgEdgeIndicators	edgeIndicators;
	private WdgPlotSelect		plotSelect;
	private WdgROISelect 		roiSelect;
	private WdgChoochSelect 	choochSelect;
	
	WdgPlotControls(Composite parent, int style)
	{
		super(parent, style);
		setLayout(new GridLayout(1,false));
		
		edgeIndicators 	= new WdgEdgeIndicators(this);
		plotSelect 		= new WdgPlotSelect(this);
		roiSelect 		= new WdgROISelect(this);
		choochSelect 	= new WdgChoochSelect(this);
		
		connectListeners();
	}

	////////////////////////////////////// widget creation helpers //////////////////////////////////////////
	
	public static Composite newSpacer(Composite parent)
	{
		Composite spacerComp = new Composite(parent, SWT.NONE);
		spacerComp.setBackground(parent.getBackground());
		GridData gd = new GridData(SWT.DEFAULT, SWT.DEFAULT, true, false);
		gd.widthHint = 10;
		gd.heightHint = 1;
		spacerComp.setLayoutData(gd);
		return spacerComp;
	}
	
	public static WdgControlEntry newEntry(Composite parent, String labelText, String unitText, Color fgColor, WdgControlEntry.ModelInterface mi)
	{
		newSpacer(parent);
		
		Label lblName = new Label(parent, SWT.RIGHT);
		lblName.setText(labelText);
		lblName.setForeground(fgColor);
		lblName.setBackground(parent.getBackground());
		lblName.setFont(GUI.FONT_SCAN_TEXT);
		lblName.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
		
		WdgControlEntry entry = new WdgControlEntry(parent, SWT.NONE, EntryType.TEXT, mi);
		GridData gd = new GridData(SWT.DEFAULT, SWT.CENTER, false, false);
		gd.widthHint = 90;
		entry.setLayoutData(gd);
		
		Label lblUnits = new Label(parent, SWT.NONE);
		lblUnits.setText(unitText);
		lblUnits.setForeground(fgColor);
		lblUnits.setBackground(parent.getBackground());
		lblUnits.setFont(GUI.FONT_SCAN_TEXT);
		lblUnits.setLayoutData(new GridData(SWT.DEFAULT, SWT.CENTER, false, false));
			
		return entry;
	}
	
	///////////////////////////////////////// update methods ////////////////////////////////////////////////
	
	private void connectListeners()
	{
		ModelScan.get().addListener(new ScanListener()
		{
			@Override public void scanChanged(final ModelScan settings)
			{
    			updateFields();
    			updateState();
				updateVisibleState();
			}
		});
		
		ModelGraph.get().listenerList.addListener(new ListenerList.Listener<ModelGraph>()
    	{
    		@Override public void handleEvent(ModelGraph graph, Object data)
    		{
    			updateFields();
    			updateState();
    		}
    	});
		
		Client.addListener(new ClientListener()
		{
			@Override public void masterChanged(boolean isMaster)
			{
				updateState();
			}
		});
		
		updateState();
		updateFields();
		updateVisibleState();
	}
	
	private volatile ScanPlotMode displayedMode = null;
	
	private void updateVisibleState()
	{
		final ScanPlotMode currentMode = ModelScan.get().getScanPlotMode();
		if (displayedMode == currentMode) return;
		displayedMode = currentMode;
		
		GUI.asyncExec(new Runnable(){public void run()
		{
			WdgControl.setControlVisible(plotSelect, 	currentMode == ScanPlotMode.EDGE);
			WdgControl.setControlVisible(choochSelect,	currentMode == ScanPlotMode.EDGE);
			WdgControl.setControlVisible(roiSelect, 	currentMode == ScanPlotMode.SPECTRUM);
			WdgControl.setControlVisible(edgeIndicators,currentMode == ScanPlotMode.SPECTRUM);

			layout(true);
			getParent().layout(true);
		}});
	}
	
	private void updateFields()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
			roiSelect.updateFields();
			choochSelect.updateFields();
			plotSelect.updateFields();
		}});
	}
	
	public void updateState()
	{
		GUI.asyncExec(new Runnable(){public void run()
		{
    		boolean isMaster = Client.isMaster();
    		
    		boolean scanInProgress = 
    			(ModelScan.get().getAutoStep() != AutoStep.NONE ||
    			 ModelScan.get().getInterStep() != InterStep.IDLE);
    		
			roiSelect.updateState		(isMaster, scanInProgress);
			choochSelect.updateState	(isMaster, scanInProgress);
			plotSelect.updateState		(isMaster, scanInProgress);
		}});
	}
}
