/**
 *  
 * This class is a part of Web-Ice, reads a spreadsheet and converts to xml file
 * Sudhir : Reading xls and xlsx with Apache POI
 * 
 */
package gov.anl.gmca.screening;

import gov.anl.gmca.syslog.Log;
import org.apache.poi.ss.usermodel.*;
import org.w3c.dom.*;

import java.io.*;
import java.util.Hashtable;

/**
 * Reads excel spreadsheet and writes it out as xml.
 */
public class CmdExcel2Xml {
	// Name of the sheet that contains crystal data
	private String sheetName = "Sheet1";
	private String validChars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_";
	//String portFirstChar = "ABCDEFGHIJKLMNOPQR"; //"ABCDEF";// "ABCDEFGHIJKL";
	String portFirstChar = validChars.substring(validChars.indexOf('A'), validChars.indexOf('A') + ModelScreening.NoofPucks);
	int maxPort = 16;
	String directoryChars = "_-\\/";
	String crystalIdChars = "_-";
	int rowCount = 0;

	public CmdExcel2Xml() {
	}

	public boolean isXlsx(String fname) {
		String fileName = fname;
		String fname2 = "";
		String ext = "";
		int mid = fileName.lastIndexOf(".");
		fname2 = fileName.substring(0, mid);
		ext = fileName.substring(mid + 1, fileName.length());
		if (ext.equalsIgnoreCase("xlsx"))
			return true;
		else
			return false;
	}

	/**
	 * Convert excel file into xml file.
	 */
	public void convert(String input, String output) throws Exception {
		Workbook wbs;
		InputStream inp = new FileInputStream(input);
		wbs = WorkbookFactory.create(inp);
		/*
		 * if(isXlsx(input)) wbs = new XSSFWorkbook(inp); else { POIFSFileSystem
		 * fs = new POIFSFileSystem(inp); wbs = new HSSFWorkbook(fs); }
		 */
		StringBuffer warnings = new StringBuffer();
		writeXml(wbs, output);
		inp.close();
		// validateSrcXml(input, warnings);
	}

	/**
	 * Create a workbook from an excel file.
	 */
	/*
	 * private Workbook readExcel(String filename) throws Exception { if
	 * (filename == null) throw new Exception("Null excel filename"); if
	 * (filename.length() == 0) throw new
	 * Exception("Zero length excel filename");
	 * 
	 * Workbook workbook = Workbook.getWorkbook(new File(filename));
	 * 
	 * return workbook; }
	 */
	/**
	 * Make sure port contains a 2-3 letter code, [A-L][1-16].
	 */
	public String validatePort(Element rowNode, Hashtable<String, Integer> lookup,
			StringBuffer warnings) throws Exception {
		
		int row = Integer.parseInt(rowNode.getAttribute("row").trim());

		// Validate Port. Can't fix this field.
		// If something is wrong we need to throw an exception.
		NodeList nodeList = rowNode.getElementsByTagName("Port");
		if ((nodeList == null) || (nodeList.getLength() != 1)) {
			ModelSilTable.throwException = true;
			warnings.append("Missing Port in row " + row + "\n");
		}
		Node port = nodeList.item(0);
		Text text = (Text) port.getFirstChild();
		if (text == null) {
			ModelSilTable.throwException = true;
			warnings.append("Invalid Port in row " + row + "\n");
		}

		// Port must have 2 characters [A-L][1-8]
		String value = text.getNodeValue();
		if ((value == null) || (value.length() == 0)) {
			ModelSilTable.throwException = true;
			warnings.append("Empty Port value in row " + row + "\n");
			return value;
		}

		if (value.length() > 3) {
			ModelSilTable.throwException = true;
			warnings.append("Invalid Port " + value + " in row " + row + "\n");
			return value;
		}

		if (portFirstChar.indexOf(value.charAt(0)) < 0) {
			ModelSilTable.throwException = true;
			warnings.append("Invalid Port " + value + " in row " + row + "\n");
			return value;
		}

		try {
			int num = Integer.parseInt(value.substring(1));
			if ((num < 0) || (num > maxPort)) {
				ModelSilTable.throwException = true;
				warnings.append("Invalid Port " + value + " in row " + row
						+ "\n");
				return value;
			}
		} catch (NumberFormatException e) {
			ModelSilTable.throwException = true;
			warnings.append("Invalid Port " + value + " in row " + row + "\n");
			return value;
		}

		// Make sure it's unqiue
		Integer otherRow = null;
		if ((otherRow = (Integer) lookup.get(value)) != null) {
			ModelSilTable.throwException = true;
			warnings.append("Port " + value + " in row " + row
					+ " duplicates value in row " + otherRow + "\n");
			return value;
		}
		// Save the new Port in the loopup table.
		lookup.put(value, row);

		return value;

	}

	/**
	 * Replace bad characters with _. Replace an empty CrystalID with Port
	 * throws an exception if CrystalID is not unique.
	 */
	public String validateCrystalID(Element rowNode, String defaultId,
			Hashtable<String, Integer> lookup, StringBuffer warnings) throws Exception {
		int row = Integer.parseInt(rowNode.getAttribute("row").trim());

		// If CrystalID field is missing then
		// create one with the default value
		NodeList nodeList = rowNode.getElementsByTagName("CrystalID");
		if ((nodeList == null) || (nodeList.getLength() != 1)) {
			Document doc = rowNode.getOwnerDocument();
			Element node = doc.createElement("CrystalID");
			Text text = doc.createTextNode(defaultId);
			node.appendChild(text);
			rowNode.appendChild(node);
			warnings.append("Replaced empty CrystalID in row " + row + " with "
					+ defaultId + "\n");
			return defaultId;
		}

		// CrystalID missing value then set default value.
		Node node = nodeList.item(0);
		Text text = (Text) node.getFirstChild();
		if (text == null) {
			Document doc = rowNode.getOwnerDocument();
			text = doc.createTextNode(defaultId);
			node.appendChild(text);
			warnings.append("Replaced empty CrystalID in row " + row + " with "
					+ defaultId + "\n");
			return defaultId;
		}

		// Replace an empty value with Port
		String value = text.getNodeValue();
		if ((value == null) || (value.length() == 0) || value.equals("null")) {
			text.setNodeValue(defaultId);
			warnings.append("Replaced empty CrystalID in row " + row + " with "
					+ defaultId + "\n");
			return defaultId;
		}
		
		//sudhir: check for length of the prefix
		if (value.length() > ModelScreening.LENGTH_SCREENING_PREFIX) {
			ModelSilTable.throwException = true;
			warnings.append("Number of characters for CrystalID " + value + " in row " + row + " should be <="
					+ ModelScreening.LENGTH_SCREENING_PREFIX +".\n");
			return value;
		}
		
		// Replace bad characters with underscores.
		StringBuffer buf = new StringBuffer();
		boolean modified = false;
		char ch;
		for (int i = 0; i < value.length(); ++i) {
			ch = value.charAt(i);
			if (Character.isLetterOrDigit(ch)
					|| (crystalIdChars.indexOf(ch) >= 0)) {
				buf.append(ch);
			} else {
				buf.append('_');
				modified = true;
			}
		}

		// Report the change
		if (modified) {
			warnings.append("Replaced CrystalID in row " + row + " from "
					+ value + " with " + buf.toString()
					+ ": removed bad characters\n");
			value = buf.toString();
			text.setNodeValue(value);
		}

		// Make sure it's unqiue
		// If not then append this row's port (which is unique) to
		// the CrystalID.
		Integer otherRow = null;
		if ((otherRow = (Integer) lookup.get(value)) != null) {
			String newValue = value + "_" + defaultId;
			warnings.append("Replaced CrystalID in row " + row + " from "
					+ value + " with " + newValue
					+ ": duplicated value in row " + row + "\n");
			value = newValue;
			text.setNodeValue(value);
		}

		// Save the new CrystalID in the lookup table.
		lookup.put(value, row);

		return value;
	}

	/**
	 * Replace bad characters with _.
	 */
	public String validateDirectory(Element rowNode, StringBuffer warnings)
			throws Exception {
		int row = Integer.parseInt(rowNode.getAttribute("row").trim());

		// Validate Port. Can't fix this field.
		// If something is wrong we need to throw an exception.
		NodeList nodeList = rowNode.getElementsByTagName("Directory");
		if ((nodeList == null) || (nodeList.getLength() != 1))
			return "";
		Node node = nodeList.item(0);
		Text text = (Text) node.getFirstChild();
		if (text == null)
			return "";

		String value = text.getNodeValue();

		// Replace bad characters with underscores.
		StringBuffer buf = new StringBuffer();
		boolean modified = false;
		char ch;
		// Dir must be relative path
		// First pass:
		// 1. Remove non-alphanum leading chars
		// 2. Replace illigal chars with _
		for (int i = 0; i < value.length(); ++i) {
			ch = value.charAt(i);
			if (Character.isLetterOrDigit(ch)) {
				buf.append(ch);
			} else {
				if (buf.length() == 0) {
					// Ignore all leading chars which are not alphanum
				} else {
					if (directoryChars.indexOf(ch) >= 0) {
						buf.append(ch); // only append allowed chars
					} else {
						buf.append('_'); // Replaced illigal chars with _
					}
				}
			}
		}
		// Second pass:
		// Remove non-alphanum chars that immediately follow slash
		String dir1 = buf.toString();
		buf.delete(0, buf.length());
		boolean marked = false;
		for (int i = 0; i < dir1.length(); ++i) {
			ch = dir1.charAt(i);
			if (ch == '/' && !marked) {
				// Mark first slash
				marked = true;
				buf.append(ch);
			} else {
				if (marked) {
					// This char follows a slash
					if (Character.isLetterOrDigit(ch)) {
						buf.append(ch);
						marked = false;
					}
				} else {
					if (Character.isLetterOrDigit(ch)
							|| (directoryChars.indexOf(ch) >= 0)) {
						buf.append(ch);
					}
				}
			}

		}
		String dir2 = buf.toString();

		if (dir2.length() == 0)
			throw new Exception("Directory in row " + row
					+ " contains sequence of illegal characters: " + value);

		// Report the change
		if (!dir2.equals(value)) {
			warnings.append("Replaced Directory in row " + row + " from "
					+ value + " to " + dir2 + "\n");
			value = dir2;
			text.setNodeValue(value);
		}

		return value;
	}

	/**
	 * Parse a workbook and write out contents into an xml file.
	 */
	private void writeXml(Workbook workbook, String output) throws Exception {
		if (workbook == null)
			throw new Exception("Null workbook");

		if (output == null)
			throw new Exception("Null xml filename");

		if (output.length() == 0)
			throw new Exception("Zero length xml filename");

		FileOutputStream out = new FileOutputStream(output);
		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(out,
				"UTF8"));

		try {

			Sheet s = workbook.getSheetAt(0);// workbook.getSheet(sheetName);

			if (s == null)
				throw new Exception("Cannot get sheet " + sheetName
						+ " from this workbook");

			bw.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			bw.write("<Data>\n");

			int count = 0;

			// First row contains column names

			Row row = s.getRow(0);
			int noofrows = s.getPhysicalNumberOfRows();

			int numColumns = row.getPhysicalNumberOfCells();

			if (row == null)
				throw new Exception("Cannot get row 0 from sheet " + sheetName);

			String colNames[] = new String[numColumns];
			Cell[] cells = new Cell[numColumns];

			boolean foundPort = false;
			boolean foundCrystalID = false;

			// Get column names from first row
			for (int i = 0; i < numColumns; ++i) {
				cells[i] = row.getCell(i);
				if (cells[i] == null) {
					Log.warn("Column name " + i + " is null");
					continue;
				}
				String cname = cells[i].getStringCellValue();
				if (cells[i].getCellType() != Cell.CELL_TYPE_BLANK) {
					String colName = cname.trim();
					// Make sure column name contains only allowed characters.
					validateColumnName(colName);
					colNames[i] = colName;
					if (colName.equals("Port")
							|| colName.equals("CurrentPosition"))
						foundPort = true;
					else if (colName.equals("CrystalID")
							|| colName.equals("XtalID"))
						foundCrystalID = true;
				}
			}

			// Make sure we have at least the required columns
			// which are Port, CrystalID
			if (!foundPort) {
				throw new Exception(
						"The first row does not contain Port column");
			}
			if (!foundCrystalID)
				throw new Exception(
						"The first row does not contain CrystalID column");

			// Loop over rows in this sheet

			Row tempRow;
			for (int i = 1; i < noofrows; i++) {

				tempRow = s.getRow(i);
				if (tempRow == null) {
					throw new Exception("Empty row found at row " + i
							+ " No empty rows allowed in between samples");
				}
				Cell cell = tempRow.getCell(0);
				if (cell == null || cell.getCellType() == Cell.CELL_TYPE_BLANK)
					continue;
				int excelRow = i + 1;
				// bw.write("	<Row number=\"" + (i+1) + "\">\n");
				bw.write("<Crystal row=\"" + i + "\" excelRow=\"" + excelRow
						+ "\" selected=\"" + 0 + "\">\n");

				// Loop over columns in this row
				for (int j = 0; j < numColumns; ++j) {
					// In case this row has more
					// columns than the fist row.
					// Then ignore this extra column.
					if (j >= numColumns)
						continue;

					cell = tempRow.getCell(j);

					String colName = colNames[j];
					String cellContent = "";
					if (cell != null) {
						int celltype = cell.getCellType();
						if (j == 0 && celltype == Cell.CELL_TYPE_BLANK) {
							rowCount++;
						}
						if (celltype == Cell.CELL_TYPE_STRING) {
							if (cell.getStringCellValue() != null)
								cellContent = xmlEncode(cell
										.getStringCellValue());
							else
								cellContent = "";
						} else if (celltype == Cell.CELL_TYPE_NUMERIC) {
							cellContent = Double.toString(cell
									.getNumericCellValue());
						} else {
							cellContent = "";
						}

					} else {
						if (j == 0)
							rowCount++;
					}
					// bw.write("		<" + colName + ">" + cellContent + "</" +
					// colName + ">\n");
					bw.append("<" + colName + ">" + cellContent + "</"
							+ colName + ">\n");
				}

				// bw.write("	</Row>\n");
				bw.write("</Crystal>\n");
			}

			bw.write("</Data>\n");
			bw.flush();

		} catch (Exception e) {
			throw new Exception("Cannot convert excel data to xml: "
					+ e.getMessage());
		} finally {
			bw.close();
		}

	}

	/**
	 * XML encode cell content so that character like '&' is converted to &amp;
	 */
	static private String xmlEncode(String rawStr) {
		String ret = rawStr.replace("&", "&amp;");
		ret = ret.replace("\"", "&quot;");
		ret = ret.replace("'", "&apos;");
		ret = ret.replace("<", "&lt;");
		ret = ret.replace(">", "&gt;");

		return ret;

	}

	/**
	 * Replace xml escaped characters like '&amp;' with real char '&'.
	 */
	static public String xmlDecode(String xmlStr) {
		String ret = xmlStr.replace("&quot;", "\"");
		ret = ret.replace("&apos;", "'");
		ret = ret.replace("&lt;", "<");
		ret = ret.replace("&gt;", ">");
		ret = ret.replace("&amp;", "&");
		return ret;
	}

	/**
	 * Column name cannot contain space.
	 */
	private void validateColumnName(String n) throws Exception {
		if (n.length() == 0)
			throw new Exception("Column name is empty");

		for (int i = 0; i < n.length(); ++i) {
			if (validChars.indexOf(n.charAt(i)) < 0)
				throw new Exception(
						"Column name ('"
								+ n
								+ "') must only contain alpha-numeric characters without spaces");
		}

	}
}
