/**
 * 
 * @author spothineni
 * sends commands to robot server
*  to load and unload samples.
*/
package gov.anl.gmca.screening;

import gov.anl.gmca.hutch.DevPinSensor;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.io.PVLookup.PV_ROBOT;
import gov.anl.gmca.io.PVLookup.PV_R_RUNS;
import gov.anl.gmca.io.PVLookup.PV_SCR;
import gov.anl.gmca.raster.util.ClearSampleState;
import gov.anl.gmca.sample.DevCollimatorCtrl;
import gov.anl.gmca.sample.DevCollimatorCtrl.CCMove;
import gov.anl.gmca.util.Util;

public class CmdRobot implements Runnable {

	String Cmd = "";
	
	public CmdRobot() { }
	
	public CmdRobot(String cmdRobo) {
		Cmd = cmdRobo;
	}

	/*
	 * load returns 0, when unload current mounted sample failed returns 1, when
	 * load failed return 2, success return 3, when robot not Ready return 4,
	 * already mounted
	 */

	public int load(int index, String params) {
	
		System.out.println("load " + index + " " + params);
		PVName mountedPV = PV_ROBOT.mounted;
		PVName mountPV = PV_ROBOT.cmd;
		PVName statusPV = PV_ROBOT.pause;

		String mounted = Epics.getString(mountedPV);

		int status = Epics.getInteger(statusPV);
		// return if Robot not ready
		if (status != 0) {
			return (3);
		}

		Epics.put(PV_SCR.CurrentTask, ModelScreening.indexToLabel(index)
				+ " MountNextXtal");
		Util.sleep(30);
		if (mounted.compareTo(CmdScreening.getMounted(index)) == 0) {
			System.out.println(ModelScreening.indexToLabel(index)
					+ " already mounted ");
			Epics.put(PV_SCR.CurrentTask, "SampleLoaded");
			Util.sleep(100);
			return (4);
		}
		
		/* move backlight IN collimator out */
		CCMove ccMove = DevCollimatorCtrl.get().newMove();
		ccMove.setArguments("OUT backlightIN bluice");
		ccMove.start().waitFor();
		
		Epics.put(mountPV, "load " + params);
		
		/* monitor unload if there sample already mounted */
		if (mounted.compareTo("none") != 0) {
			
			Util.sleep(30);
			if (monitorUnmountLocal(mounted)) {
				//pass
			} else {
				return (0);
			}
			Epics.put(PV_SCR.CurrentTask, "Unloaded");
			Util.sleep(30);
		}

		PVName statusPV2 = PV_ROBOT.sta;
		String status2 = Epics.getString(statusPV2);
		status2 = status2.substring(9);

        String crystal = ModelScreening.indexToLabel(index);
        String mt = Epics.getString(mountedPV);
        while (!(status2.compareTo("sample " + crystal + " loaded") == 0 || mt.equals(crystal))) {
		//while (status2.compareTo("sample " + ModelScreening.indexToLabel(index) + " loaded") != 0) {
			// Log.note("waiting load " + params);
			status2 = Epics.getString(statusPV2);
			status2 = status2.substring(9);
			mt = Epics.getString(mountedPV);
			if (CmdScreening.stopThread)
				return (1);
			if (status2.substring(0, 3).equals("ERR")) {
				return (1);
			}

			Util.sleep(50);// sleep for 100 ms
		}
		Epics.put(PV_SCR.CurrentTask, "SampleLoaded");
		Util.sleep(100);
		return (2);
	}
	
	public boolean monitorUnmountLocal(String mounted) {
		String status = "";
		status = Epics.getString(PV_ROBOT.sta);
		status = status.substring(9);
		int count = 0;
		Util.sleep(1000);
		String compareStr = "";
		
		compareStr = "pin " + mounted + " in the Dewar";

		while (status.compareTo(compareStr) != 0) {
			status = Epics.getString(PV_ROBOT.sta$1);
			String status1 = Epics.getString(PV_ROBOT.sta);
			status = status.substring(9);

			/*
			 * if (CmdScreening.stopThread) return false;
			 */

			if (status.substring(0, 3).equals("ERR")
					|| status1.contains("pause"))
				return (false);

			Util.sleep(50);// sleep for 100 ms
		}
		return (true);
	}
	
	public boolean monitorUnmount(String mounted) {
		String status = "";
		status = Epics.getString(PV_ROBOT.sta);
		status = status.substring(9);
		int count = 0;
		Util.sleep(1000);
		String compareStr = "";

		if (Cmd.contains("rinse") || Cmd.contains("wash"))
			compareStr = "sample " + mounted + " loaded";
		if (Cmd.contains("unload"))
			compareStr = "pin " + mounted + " in the Dewar";

		while (status.compareTo(compareStr) != 0) {
			status = Epics.getString(PV_ROBOT.sta$1);
			String status1 = Epics.getString(PV_ROBOT.sta);
			status = status.substring(9);

			/*
			 * if (CmdScreening.stopThread) return false;
			 */

			if (status.substring(0, 3).equals("ERR")
					|| status1.contains("pause"))
				return (false);

			Util.sleep(50);// sleep for 100 ms
		}
		return (true);
	}

	public boolean unload() {

		CCMove ccMove = DevCollimatorCtrl.get().newMove();
		ccMove.setArguments("OUT backlightIN bluice");
		ccMove.start().waitFor();

		PVName pvName = PV_ROBOT.cmd;
		PVName mountedPV = PV_ROBOT.mounted;
		String mounted = Epics.getString(mountedPV);
		Util.sleep(30);
		Epics.put(pvName, Cmd);
		if (monitorUnmount(mounted)) {
		} else {
			return false;
		}
		// clearRuns();
		Epics.put(PV_SCR.CurrentTask, Cmd);
		Util.sleep(30);
		return true;
	}

	public void putRobotStatus() {

	}

	public void clearRuns() {
		boolean clear = Epics.getBoolean(PV_R_RUNS.scrClear);
		if (clear) {
			ClearSampleState.clearAllRelatedState("Screening raster clear : ",
					false, false);
			DevPinSensor.clearState();
		}
	}

	public int setRobotStatus(int put) {

		PVName status = PV_ROBOT.pause;
		Epics.putInteger(status, put);
		return 0;
	}

	/*
	 * public String getRobotCmd() {
	 * 
	 * }
	 */

	@Override
	public void run() {

		CCMove ccMove = DevCollimatorCtrl.get().newMove();
		ccMove.setArguments("OUT backlightIN bluice");
		ccMove.start().waitFor();

		PVName pvName = PV_ROBOT.cmd;
		PVName mountedPV = PV_ROBOT.mounted;
		String mounted = Epics.getString(mountedPV);
		Util.sleep(30);

		Epics.put(pvName, Cmd);

		if (monitorUnmount(mounted)) {
			// clearRuns ();
		}
		if (Cmd.equals("unload"))
			Epics.put(PV_SCR.CurrentTask, "Unloaded");
		if (Cmd.contains("rinse"))
			Epics.put(PV_SCR.CurrentTask, "Rinsed");
		if (Cmd.contains("wash"))
			Epics.put(PV_SCR.CurrentTask, "Washed");
		// Epics.put(Epics.addBID("bi:scr:CurrentTask"),Cmd);
		Util.sleep(30);

	}
}
