/**
 * Determines if a warm-up is needed, and prevents multiple warm-up commands from being sent
 * before the warm-up occurs
 */

package gov.anl.gmca.screening;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_ROBOT;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;

public class DevRobotWarmup {
	public void sendWarmupIfNeeded(int operationLengthSec) {
		double marginSec = Double.parseDouble(DBConfig
				.get("robotWarmupMarginSec"));
		int secToNextWarmup = getSecondsToNextWarmup();
		if (operationLengthSec + marginSec >= secToNextWarmup) {
			sendWarmup();
		}
	}

	private int getSecondsToNextWarmup() {
		// no warmup PV at BM
		if (Util.getBID().equals("23b"))
			return Integer.MAX_VALUE;

		return Epics.getInteger(PV_ROBOT.w_tim);
	}

	private void sendWarmup() {
		double w_tim = Epics.getDouble(PV_ROBOT.w_tim);
		if (w_tim>=99999.0 && w_tim <= 99999.999) {
			Log.note("Skipping warmup, timeout = "+w_tim);
			return;
		}
		boolean isReady = Epics.getInteger(PV_ROBOT.pause)==0;
		if (!isReady) {
			Log.note("Skipping warmup, robot is not ready");
			return;
		}

		Log.note("Sending robot warmup command");
		Epics.put(PV_ROBOT.cmd, "warm-up");
	}

	private static DevRobotWarmup inst;

	public static DevRobotWarmup get() {
		if (null == inst)
			new DevRobotWarmup();
		return inst;
	}

	public DevRobotWarmup() {
		inst = this;
		// connectMonitors();
	}

	// ///////////////// methods for preventing multiple warm-ups from being
	// sent ///////////////

	/*
	 * private void connectMonitors() { Epics.addMonitor(PV_ROBOT.sta, new
	 * Epics.EpicsValueMonitor() {
	 * 
	 * @Override public void valueChanged(String pvName, DBR value) {
	 * handleRobotStatus(Epics.dbrToString(value)); } }); }
	 */

	/*
	 * public void sendWarmupIfNotPending() { if (!isWarmupPending())
	 * sendWarmup(); }
	 */

	/*
	 * private void handleRobotStatus(String status) { if
	 * (status.contains("warm_up") || status.contains("warm-up")) {
	 * Epics.putInteger(PV_BLUICE.robotWarmupPending, 0); } }
	 */

	/*
	 * private boolean isWarmupPending() { return
	 * Epics.getInteger(PV_BLUICE.robotWarmupPending)==1; }
	 */
}
