/**
 * @author spothineni
 * 
 * Handles MySQL queries for SilTable
 * 
 * It will insert and updates the spreadsheet into MySQL table
 * 
 */

package gov.anl.gmca.screening;

import gov.anl.gmca.io.Database;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_SCR;
import gov.anl.gmca.log.db.ScreeningCollectEvents;
import gov.anl.gmca.log.db.ScreeningCollectEvents.ScreeningCollectParam;
import gov.anl.gmca.sample.DevCollimatorCtrl;
import gov.anl.gmca.strategy.DevStrategyResults;
import gov.anl.gmca.strategy.ModelAutoindexResult;
import gov.anl.gmca.strategy.ModelStrategyResult;
import gov.anl.gmca.strategy.StrategyParams;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.StringTokenizer;

public class DevSilTable {
	protected static final String tableName = "SilTable";
	protected static final boolean UPDATE_CURRENT_INDEX_ON_RESULT_CREATION = true;
	protected static final boolean SHOW_STRATEGY_TAB_ON_RESULT_CREATION = true;

	protected static boolean doesTableExist() {
		Database.Result result = Database.exec(DBName.USERDATA,
				"show tables like '" + tableName + "'");
		boolean exists = (result.numRows() > 0);
		return exists;
	}

	public synchronized static boolean verifyTableExists() {
		if (!doesTableExist()) {
			System.out.println("Creating SIL results table");
			String createCmd = "CREATE TABLE `" + tableName + "` ("
					+ "`sampleNumber` int(11) NOT NULL,"
					+ "`selected` TINYINT default 0,"
					+ "`ordered` int(11) default 0,"
					+ "`screened` TINYINT default 0,"
					+ "`Port` varchar(30) NOT NULL,"
					+ "`CrystalID` varchar(250) NOT NULL,"
					+ "`Directory` varchar(250) NOT NULL,"
					+ "`Comment` varchar(250) NOT NULL,"
					+ "`workdir` varchar(250) default '',"
					+ "`image1` varchar(100) default '',"
					+ "`image2` varchar(100) default '',"
					+ "`Images` varchar(250) default '',"
					+ "`Score` varchar(30) default '',"
					+ "`Resolution` varchar(30) default '',"
					+ "`Protein` varchar(250) default '',"
					+ "`Mosaicity` varchar(50) default '',"
					+ "`UnitCell` varchar(50) default '',"
					+ "`BravaisLattice` varchar(250) default '',"
					+ "`Rmsd` varchar(50) default '',"
					+ "`IceRings` varchar(50) default '',"

					+ "`Metal` varchar(10) default '',"
					+ "`Spacegroup` varchar(30) default '',"
					+ "`ModelPath` varchar(90) default '',"
					+ "`SequencePath` varchar(80) default '',"
					
					+ "`warning` varchar(5120) default '',"
					+ "`currentPrefix` varchar(150) default '',"
					+ "`selectedPrefix` varchar(150) default '',"
					+ "`prefixes` varchar(5120) default '',"
					+ "`timestamp` varchar(512) default '',";

			createCmd += " PRIMARY KEY (`sampleNumber`)) DEFAULT CHARSET=utf8";
			Database.exec(DBName.USERDATA, createCmd);
		}
		return doesTableExist();
	}

	protected boolean connected = false;
	protected Epics.EpicsValueMonitor dbUpdatedMonitor;
	protected String pvBase;

	public static DevSilTable get() {
		return inst;
	}

	protected static DevSilTable inst = null;

	public DevSilTable() {
		inst = this;
		pvBase = Epics.addBID("bi:scr:");
		verifyTableExists();
	}

	public void verifyConnected() {
		if (connected)
			return;

		connected = true;
	}

	public void setRow(ModelRowData row, boolean update) {
		
		//verifyTableExists();
		//long startTime0 = System.nanoTime();

		boolean newResultCreated = false;
		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ row.row);
		if (0 == result.numRows()) {
			Database.exec(
					DBName.USERDATA,
					"insert into "
							+ tableName
							+ " (sampleNumber, ordered, Port,"
							+ "CrystalID,Directory,"
							+ "Comment,Resolution, Score, Images, IceRings, UnitCell, Mosaicity, Rmsd, "
							+ "BravaisLattice, Protein,"
							+ "Metal, Spacegroup, ModelPath, SequencePath"
							+ ") "
							+ "values (" + row.row + ", " + 0 + ", '" + row.col[1]
							+ "', '" + row.col[2] + "', '" + row.col[3]
							+ "', '" + row.col[4] + "', '" + row.col[6]
							+ "', '" + row.col[7] + "', '" + row.col[9]
							+ "', '" + row.col[10] + "', '" + row.col[11]
							+ "', '" + row.col[12] + "', '" + row.col[13]
							+ "', '" + row.col[14] + "', '" + row.col[8] 
							+ "', '" + row.col[15] + "', '" + row.col[16] + "', '" + row.col[17] + "', '" + row.col[18] 
									+ "')");
		} else {
			
			/*String prefixes = "";
			if(row.prefixes.size() == 1) {
				prefixes = row.prefixes.get(0);
			} else {
				for(String pre:row.prefixes) {
					prefixes = pre + " " + prefixes;
				}
			}*/
			String updateCmd = "update " + tableName + " "
					+ "set selected='" + row.selected + "' " + ", ordered='"
					+ row.order + "' " + ", screened='" + row.screened + "' "
					+ ", Port='" + row.col[1] + "' " + ", CrystalID='"
					+ row.col[2] + "' " + ", Directory='" + row.col[3] + "' "
					+ ", Comment='" + row.col[4] + "' " + ", Protein='"
					+ row.col[8] + "' " + ", Resolution='" + row.col[6] + "' "
					+ ", Score='" + row.col[7] + "' " + ", Images='"
					+ row.col[9] + "' " + ", IceRings='" + row.col[10] + "' "
					+ ", UnitCell='" + row.col[11] + "' " + ", Mosaicity='"
					+ row.col[12] + "' " + ", Rmsd='" + row.col[13] + "' "
					+ ", BravaisLattice='" + row.col[14] + "' "
						
					+ ", Metal='" + row.col[15] + "' "
					+ ", Spacegroup='" + row.col[16] + "' "
					+ ", ModelPath='" + row.col[17] + "' "
					+ ", SequencePath='" + row.col[18] + "' ";
			
			/*if(!row.currentPrefix.isEmpty())
				updateCmd += ", currentPrefix='" + row.currentPrefix + "' ";
			
			if(!prefixes.isEmpty()) {
				updateCmd += ", prefixes= '" + prefixes + "'";
			}*/
			
			updateCmd += " where sampleNumber=" + row.row;
			Database.exec(DBName.USERDATA, updateCmd);
		}
		
		//long startTime = System.nanoTime();
		//Log.note("DB query updates took " + (startTime - startTime0)/1000000 + " milsec.");

		// send the PV update signal
		if (update) {
			sendRowUpdate(row.row);
		}
		//long endTime = System.nanoTime();
		//Log.note("PV updates took " + (endTime - startTime)/1000000 + " milsec.");
	}
	
	public void clearRow(ModelRowData row, boolean update) {
		
		//verifyTableExists();
		
		boolean newResultCreated = false;
		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ row.row);
		if (0 == result.numRows()) {
			Database.exec(
					DBName.USERDATA,
					"insert into "
							+ tableName
							+ " (sampleNumber,Port,CrystalID,Directory,Comment,Resolution, Score, Images, IceRings, UnitCell, Mosaicity, Rmsd, BravaisLattice, Protein) "
							+ "values (" + row.row + ", '" + row.col[1]
							+ "', '" + row.col[2] + "', '" + row.col[3]
							+ "', '" + row.col[4] + "', '" + row.col[6]
							+ "', '" + row.col[7] + "', '" + row.col[9]
							+ "', '" + row.col[10] + "', '" + row.col[11]
							+ "', '" + row.col[12] + "', '" + row.col[13]
							+ "', '" + row.col[14] + "', '" + row.col[8] + "')");
		} else {
			
			String updateCmd = "update " + tableName + " "
					+ "set selected='" + row.selected + "' " + ", ordered='"
					+ row.order + "' " + ", screened='" + row.screened + "' "
					+ ", Port='" + row.col[1] + "' " + ", CrystalID='"
					+ row.col[2] + "' " + ", Directory='" + row.col[3] + "' "
					+ ", Comment='" + "' " + ", Protein='"
					+ "' " + ", Resolution='"  + "' "
					+ ", Score='"  + "' " + ", Images='"
					+ "' " + ", IceRings='" +  "' "
					+ ", UnitCell='"  + "' " + ", Mosaicity='"
					+ "' " + ", Rmsd='"  + "' "
					+ ", BravaisLattice='"  + "' "
					+ ", currentPrefix='"  + "' "
					+ ", prefixes='"  + "' ";
			
			updateCmd += " where sampleNumber=" + row.row;
			Database.exec(DBName.USERDATA, updateCmd);
					
		}
		// send the PV update signal
		if (update)
			sendRowUpdate(row.row);
	}
	
	public void setResult(int sampleNumber, ModelAutoindexResult autoindexResult) {
		//verifyTableExists();

		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber);
		if (0 == result.numRows()) {
			Database.exec(DBName.USERDATA, "insert into " + tableName
					+ " (sampleNumber, workdir, image1, image2, warning) "
					+ "values (" + sampleNumber + "', '"
					+ autoindexResult.workDir + "', '" + autoindexResult.image1
					+ "', '" + autoindexResult.image2 + "','"
					+ autoindexResult.warning + "')");
			newResultCreated = true;
		} else {
			Database.exec(DBName.USERDATA, "update " + tableName + " "
					+ "set workdir='" + autoindexResult.workDir + "' "
					+ ", image1='" + autoindexResult.image1 + "' "
					+ ", image2='" + autoindexResult.image2 + "' "
					+ "where sampleNumber=" + sampleNumber);
		}

		/* update GM/CA database */

		if (autoindexResult.state.equals("ERROR")
				|| autoindexResult.state.equals("DONE"))
			updateScreenCollectEvent(sampleNumber);
	}

	public void setResult(int sampleNumber,
			ModelAutoindexResult autoindexResult,
			StrategyParams strategyResult, boolean update) {
		//verifyTableExists();

		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber);
		
		String prefixSet = result.get(0, "prefixes");
		if (0 == result.numRows()) {
			Database.exec(DBName.USERDATA, "insert into `" + tableName
					+ "` (sampleNumber,state) " + "values (" + sampleNumber
					+ ",'DONE')");
			newResultCreated = true;
		}
		
		String updateCmd = "update `" + tableName + "` set ";
		String DATE_FORMAT_NOW = "yyyy-MM-dd-HH:mm:ss";
		Date date = new Date();
		SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
		String timestamp = sdf.format(date);

		updateCmd += "Score='" + autoindexResult.score;
		updateCmd += "', BravaisLattice='" + strategyResult.spaceGroup_short;
		updateCmd += "', Resolution='" + strategyResult.predictedResolution;
		updateCmd += "', Rmsd='" + strategyResult.rmsd;
		updateCmd += "', image1='" + autoindexResult.image1;
		updateCmd += "', image2='" + autoindexResult.image2;
		updateCmd += "', Images='" + autoindexResult.image1 + " "
				+ autoindexResult.image2;
		updateCmd += "', UnitCell='" + strategyResult.unitCellA + " "
				+ strategyResult.unitCellB + " " + strategyResult.unitCellC
				+ " " + strategyResult.unitCellAlpha + " "
				+ strategyResult.unitCellBeta + " "
				+ strategyResult.unitCellGamma;
		updateCmd += "', Mosaicity='" + autoindexResult.mosaicity + " deg";
		updateCmd += "', warning='" + autoindexResult.warning;
		updateCmd += "', currentPrefix='" + autoindexResult.sampleName;
		
		if(!prefixSet.contains(autoindexResult.sampleName))
			updateCmd += "', prefixes=concat('" +  autoindexResult.sampleName+" ', prefixes)";
		else 
			updateCmd += "'";
		
		updateCmd += ", timestamp='" + timestamp;
		updateCmd += "' where sampleNumber=" + sampleNumber;
		// Log.note(updateCmd);

		Database.exec(DBName.USERDATA, updateCmd);

		if (update) {
			/* send the PV update signal */
			sendRowUpdate(sampleNumber);

			/* update GM/CA database */
			updateScreenCollectEvent(sampleNumber);
		}
	}

	public void setResult(int sampleNumber,
			ModelStrategyResult modelStrategyResult, boolean update) {
		//verifyTableExists();

		boolean newResultCreated = false;

		Database.Result result = Database.exec(DBName.USERDATA,
				"select * from `" + tableName + "` where sampleNumber="
						+ sampleNumber);
		if (0 == result.numRows()) {
			Database.exec(DBName.USERDATA, "insert into `" + tableName
					+ "` (sampleNumber,state) " + "values (" + sampleNumber
					+ ",'DONE')");
			newResultCreated = true;
		}

		String updateCmd = "update `" + tableName + "` set ";
		String DATE_FORMAT_NOW = "yyyy-MM-dd-HH:mm:ss";
		Date date = new Date();
		SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
		String timestamp = sdf.format(date);

		updateCmd += "Score='"
				+ modelStrategyResult
						.getField(ModelStrategyResult.StrategyField.SCORE);
		updateCmd += "', currentPrefix='"
				+ modelStrategyResult
						.getField(ModelStrategyResult.StrategyField.SAMPLE_NAME);
		updateCmd += "', Resolution='"
				+ modelStrategyResult
						.getField(ModelStrategyResult.StrategyField.RESOLUTION);
		updateCmd += "', BravaisLattice='"
				+ modelStrategyResult
						.getField(ModelStrategyResult.StrategyField.SPACE_GROUP_SH);
		updateCmd += "', UnitCell='"
				+ modelStrategyResult
						.getField(ModelStrategyResult.StrategyField.UNIT_CELL);
		updateCmd += "', Rmsd='"
				+ modelStrategyResult
						.getField(ModelStrategyResult.StrategyField.RMSD);
		updateCmd += "', warning='"
				+ modelStrategyResult
						.getField(ModelStrategyResult.StrategyField.WARNING);
		updateCmd += "', timestamp='" + timestamp;
		updateCmd += "' where sampleNumber=" + sampleNumber;
		// Log.note(updateCmd);

		Database.exec(DBName.USERDATA, updateCmd);

		if (update) {
			/* send the PV update signal */
			// Log.error("Debugging DevSilTable setResult modelStrategyResult ");
			sendRowUpdate(sampleNumber);
		}
	}

	protected void updateRow(int row, ModelSilTable model) {
		//verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + row);
		
		if (result.numRows() > 0) {

			HashMap<String, String> tableRow = result.getRow(0);
			ModelRowData currentRow = new ModelRowData(row);;
			for (String key : tableRow.keySet()) {
				int i=0;
				for (Object o:currentRow.col) {
					if(key.equals(currentRow.ColumnNames[i])) {
						currentRow.col[i] = tableRow.get(key);
					}
					i++;
				}
				if (key.equals("selected"))
					currentRow.selected = Integer.valueOf(tableRow.get(key));
				if (key.equals("ordered"))
					currentRow.order = Integer.valueOf(tableRow.get(key));
				if (key.equals("screened"))
					currentRow.screened = Integer.valueOf(tableRow.get(key));
				if(key.equals("currentPrefix")) {
					currentRow.currentPrefix = tableRow.get(key);
				}
				if(key.equals("prefixes")) {
					String prefix = tableRow.get(key).trim();
					StringTokenizer tokens = new StringTokenizer(prefix);
					currentRow.prefixes = new ArrayList<String>();
					
					while(tokens.hasMoreTokens()) {
						currentRow.prefixes.add(tokens.nextToken());
					}	
				}
			}
			model.data.put(row, currentRow);
		}
	}
	
	public void updateStrategy(ModelSilTable model, String port, String prefix) {
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + DevStrategyResults.tableName + " where screeningSample='"+port+ "' and sampleName='"+prefix+"'");
		
		if(result.numRows() < 1)
			return;
		
		int totalresults = result.numRows();
		int sampleNumber = Integer.parseInt(result.get(totalresults-1, "sampleNumber"));
		
		final Database.Result result1 = Database.exec(DBName.USERDATA,
				"select * from " + DevStrategyResults.resultTableName + " where sampleNumber="
						+ sampleNumber + " and good='yes'");
		
		HashMap<String, String> autoindexResult = result.getRow(totalresults-1);
		
		String updateCmd = "update `" + tableName + "` set ";
		String DATE_FORMAT_NOW = "yyyy-MM-dd-HH:mm:ss";
		Date date = new Date();
		SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
		String timestamp = sdf.format(date);

		updateCmd += "Score='" + autoindexResult.get("score");
		updateCmd += "', currentPrefix='" + prefix;
		updateCmd += "', Mosaicity='" + autoindexResult.get("mosaicity") + " deg";
		updateCmd += "', Images='" + autoindexResult.get( "images");
		if(result1.numRows() > 0) {
			HashMap<String, String> strategyResult = result1.getRow(0);
			updateCmd += "', UnitCell='" + strategyResult.get("unitcell");
			updateCmd += "', Rmsd='" + strategyResult.get("rmsd");
			updateCmd += "', Resolution='" + strategyResult.get("resolution");
			updateCmd += "', BravaisLattice='" + strategyResult.get("spaceGroup_short");
			
		} else {
			updateCmd += "', UnitCell='N/A" ;
			updateCmd += "', Rmsd='N/A" ;
			updateCmd += "', Resolution='N/A" ;
			updateCmd += "', BravaisLattice='";
		}
		
		updateCmd += "', warning='" + autoindexResult.get("warning");
		updateCmd += "', timestamp='" + timestamp;
		updateCmd += "' where sampleNumber=" + CmdScreening.labelToindex(port);
		// Log.note(updateCmd);

		Database.exec(DBName.USERDATA, updateCmd);
		sendRowUpdate(CmdScreening.labelToindex(port));	
	}
	
	protected void updateAllRows(ModelSilTable model, boolean update) {
		//verifyTableExists();

		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " order by ordered");
		if (result.numRows() > 0) {
			int i = 0, sampleOrder = 0;
			for (HashMap<String, String> row : result.getRows()) {
				ModelRowData modelRow = new ModelRowData(i);
				for (String key : row.keySet()) {
					modelRow.setColumn(model.getColumnIndex(key), row.get(key));
					if (key.equals("sampleNumber")) {
						modelRow.row = Integer.parseInt(row.get(key));
						sampleOrder = Integer.parseInt(row.get(key));
					}
					if (key.equals("selected"))
						modelRow.selected = Integer.parseInt(row.get(key));
					if (key.equals("ordered")) {
						modelRow.order = Integer.parseInt(row.get(key));
					}
					if (key.equals("screened"))
						modelRow.screened = Integer.parseInt(row.get(key));
					if(key.equals("currentPrefix")) {
						modelRow.currentPrefix = row.get(key);
					}
					if(key.equals("selectedPrefix")) {
						modelRow.selectedPrefix = row.get(key);
					}
					if(key.equals("prefixes")) {
						String prefix = row.get(key).trim();
						if(!prefix.isEmpty()) {
							StringTokenizer tokens = new StringTokenizer(prefix);
							modelRow.prefixes = new ArrayList<String>();
						
							while(tokens.hasMoreTokens())
								modelRow.prefixes.add(tokens.nextToken());
						}
					}
					
				}

				model.data.put(sampleOrder, modelRow);
				i++;
			}

			if (update) {
				sendRowUpdate(-2);
			}
		}
	}

	protected void updateScreenCollectEvent(int num) {

		HashMap<ScreeningCollectParam, String> fieldMapStr = new HashMap<ScreeningCollectParam, String>();
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName + " where sampleNumber=" + num);
		if (result.numRows() < 1)
			return;
		for (HashMap<String, String> row : result.getRows()) {
			for (String sqlCol : row.keySet()) {
				for (ScreeningCollectParam fieldStr : ScreeningCollectParam
						.values()) {
					if (fieldStr.getDBField().fieldName.equals(sqlCol)) {
						fieldMapStr.put(fieldStr, row.get(sqlCol));
					}
				}
			}
		}
		fieldMapStr.put(ScreeningCollectParam.COLLIMATOR, DevCollimatorCtrl
				.getCollimatorSize().toString());
		fieldMapStr.put(ScreeningCollectParam.PREFIX,
				ModelScreening.currentPrefix);
		fieldMapStr.put(ScreeningCollectParam.NUMFRAMES,
				Integer.toString(CmdScreening.getTotalImagesScreen()));
		ScreeningCollectEvents.writeEvent(fieldMapStr);
	}

	protected int getRowCount() {
		int count;
		final Database.Result result = Database.exec(DBName.USERDATA,
				"select * from " + tableName);
		count = result.numRows();
		return count;
	}

	protected void sendRowUpdate(int sampleNumber) {
		Epics.putInteger(PV_SCR.silUpdated, -1);
		Epics.putInteger(PV_SCR.silUpdated, sampleNumber);
	}

	public void deleteAllResults() {
		System.out.println("Deleting all silTable results");

		//verifyTableExists();

		Database.delete(DBName.USERDATA, tableName, null);

		sendRowUpdate(-2);
	}	
}
