/**
 * @author spothineni
 * 
 * This is a Model for crystal list of the spreadsheet
 * for screening
 * 
 * CrystalList is linkedilist of crystal 0 1 2 3 5 6 .....95
 * 
 * TODO I think the crystalList should be list of Integer
 * please change the String -> Integer and no need to call parseInt in that case
 */

package gov.anl.gmca.screening;

import gov.anl.gmca.syslog.Log;

import java.util.LinkedList;
import java.util.List;

public class ModelCrystalList {
	
	/* carefully change this list to String to Integer */
	List<String> crystalList;

	public ModelCrystalList() {
		crystalList = new LinkedList<String>();
	}

	public int addCrystalToList(String element) {
		if (isExist(Integer.parseInt(element)) != 1)
			crystalList.add(element);

		return (crystalList.size());
	}

	public void removeCrystalFromList(String element) {
		crystalList.remove(element);
	}

	public List<String> getAll() {
		return crystalList;
	}

	public int getNoofCrystals() {
		if (crystalList == null) {
			return 0;
		}
		return (crystalList.size());
	}

	public String getElement(int index) {
		if (index >= getNoofCrystals() || index < 0)
			return null;
		else
			return ((String) crystalList.get(index));
	}

	public int getIndex(String element) {
		int i;
		for (i = 0; i < crystalList.size(); i++) {
			String temp;
			temp = getElement(i);
			if (temp.equals(element)) {
				return i;
			}

		}
		return -1;
	}

	public int isExist(int index) {
		int i;
		for (i = 0; i < crystalList.size(); i++) {
			String temp;
			temp = getElement(i);
			if (temp.equals(Integer.toString(index))) {
				return 1;
			}

		}
		return 0;
	}
	
	//QX move an element to a new position , otherwise add first, 10/27/2017
	public String toString() {
		return String.join(",", crystalList);
	}
	
	public void deleteAll() {
		crystalList.clear();
	}
	
	public int addOrMove(String element, int position) { // element is string of index
		if (getIndex(element) != -1) {
			removeCrystalFromList(element);
		}
		
		// append to the end if empty or requested position is not legal
		if (crystalList.size() == 0 || position >= crystalList.size()) {
			crystalList.add(element);
		} else {
			crystalList.add(position, element);
		}
		return crystalList.size();		
	}
	
	// other utilities
	public void addACrystal(String requestedCrystal) { // A1
		int crystalIndex = CmdScreening.labelToindex(requestedCrystal);
		if (!Pucks.isPuckOwnerFromPinIndex(crystalIndex)) {
			Log.error("cannot move crystal since it belongs to another user.");
			return;
		}
		
		addCrystalToList(String.valueOf(crystalIndex));
	}
	
	public void addPuck(char puckLabel) {
		if (!Pucks.isPuckOwner(puckLabel)) {
			Log.error("cannot add puck since it belongs to another user.");
			return;
		}
		
		int[] indices = ModelScreening.puckLabelToPinIndices(puckLabel);
		for (int i = 0; i< indices.length; i++) {
			addCrystalToList(String.valueOf(indices[i]));
		}
	}
	
	public void addPuck(int puckIndex) {
		if (!Pucks.isPuckOwnerFromPuckIndex(puckIndex)) {
			Log.error("cannot add puck since it belongs to another user.");
			return;
		}
		
		int pinsPerPuck = ModelScreening.getPinsPerPuck();
		for (int i=0; i < pinsPerPuck; i++) {
			int index = puckIndex * pinsPerPuck + i - 1;
			addCrystalToList(String.valueOf(index));
		}	
	}
	
	public void addAllPucks() {
		for (int p = 0; p < ModelScreening.NoofPucks; p++) {
			if (Pucks.isPuckOwnerFromPuckIndex(p)) {
				for (int i = 0; i <= ModelScreening.getPinsPerPuck(); i++) {
					addCrystalToList(String.valueOf(p * ModelScreening.getPinsPerPuck() + i - 1));
				}
			}
		}
	}
}
