/**
 * @author spothineni
 * This class is collection of static variables used for Screening spreadsheet
 * and few method for Pin label index conversion
 * 
 */

package gov.anl.gmca.screening;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PVName;
import gov.anl.gmca.io.PVLookup.PV_ROBOT;

public class ModelScreening {

	/***** puck details change for BAM2 or move to MySql *********/
	// insert into blc2004.Config (ID, Value, TypeInfo) values ('screening_noofpucks', '6', 'i');
	// insert into blc2004.Config (ID, Value, TypeInfo) values ('screening_enable_puck_ownership_check', '1', 'b');
	// insert into blc2004.Config (ID, Value, TypeInfo) values ('processing_enable_autoproc', '1', 'b');
	
	public static int NoofPucks = getNoOfPucks();
	
	public static int ALSPuckSamples = 16, RigakuPuckSamples = 12;
	/************************************************************/
	public static String currentPort = "";
	public static String currentSubDir = "";
	public static String currentRootDir = "";
	public static String currentPrefix = "";
	public static String washType = "";
	public static String centerType = "";
	public static String comment = "";
	public static boolean robotSuccess = true;
	public static String mountedPin = "none";
	public static String robotError = "";
	static String validChars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_";
	public static char[] puckLabels = validChars.substring(validChars.indexOf('A'), validChars.indexOf('A') + ModelScreening.NoofPucks).toCharArray();
	public static int crystalStartIndex = 0;
	public static String currentTaskStr = "";
	public static boolean pauseCollect = false;
	public static boolean preIncrement = true;
	
	public static int screeningStatus = 0;
	public static int LENGTH_SCREENING_PREFIX = 19;

	private static Integer getNoOfPucks() {
		Integer noOfPucks;
		try {
			noOfPucks = Integer.valueOf(DBConfig.get("screening_noofpucks"));
		} catch (Exception e) {
			noOfPucks = 6;
		}
		return noOfPucks;		
	}
	
	public ModelScreening() {

	}

	public static String getRobotServerFormat(int index) {

		int pinsPerPuck = getPinsPerPuck();
		int puckNumber = index / pinsPerPuck;
		int pinNumber = index % pinsPerPuck;

		String puckLetter = Character.toString(puckLabels[puckNumber]);
		PVName puckTypePV = PV_ROBOT.puckType;
		String puckCode = "";
		int puckType = Epics.getInteger(puckTypePV);
		if (puckType == 1) {
			puckCode = "R";
			pinsPerPuck = RigakuPuckSamples;
		} else if (puckType == 0) {
			puckCode = "A";
			pinsPerPuck = ALSPuckSamples;
		}
		String params = puckLetter + " " + puckCode + " "
				+ Integer.toString(pinNumber + 1);

		return (params);
	}

	public static String getPuckType() {
		String puckType = "";
		String puckCode = "";

		puckType = Epics.getString(PV_ROBOT.puckType);

		if (puckType.equals("0")) {
			puckCode = "A";
		} else if (puckType.equals("1")) {
			puckCode = "R";
		} else {
			puckCode = "ERROR";
		}
		return (puckCode);
	}

	public static int getPinsPerPuck() {
		int puckType = 0;
		int pinsPerPuck = ALSPuckSamples;
		PVName puckTypePV = PV_ROBOT.puckType;

		puckType = Epics.getInteger(puckTypePV);
		if (puckType == 1) {
			pinsPerPuck = RigakuPuckSamples;
		} else if (puckType == 0) {
			pinsPerPuck = ALSPuckSamples;
		} else {
			pinsPerPuck = 0;
		}
		return (pinsPerPuck);
	}

	public static int labelToindex_(String label) {
		String pinNumber = label.substring(2);
		int puckNumber = 1;
		int pinIndex = 1;
		for (int i = 0; i < NoofPucks; i++) { // 6 pucks
			if (puckLabels[i] == label.charAt(0))
				puckNumber = i;
		}
		pinIndex = puckNumber * getPinsPerPuck() + Integer.valueOf(pinNumber);
		return (pinIndex - 1);
	}

	public static String indexToLabel(int pinIndex) {

		int pinsPerPuck = getPinsPerPuck();
		int puckNumber = pinIndex / pinsPerPuck;
		int pinNumber = pinIndex % pinsPerPuck;

		String label = Character.toString(puckLabels[puckNumber]);
		label = label.concat(Integer.toString(pinNumber + 1));

		return (label);
	}

	public static int getCurrentMountedIndex() {
		String mount = Epics.getString(PV_ROBOT.mounted);
		if (mount.equals("None"))
			return -1;
		int sampleNo = CmdScreening.labelToindex(mount);

		return sampleNo;
	}

	//QX utility methods for converting
	public static int puckLabelToIndex(char puckLabel) {
		int puckIndex = validChars.indexOf(puckLabel) - validChars.indexOf('A');
		if (puckIndex <= getNoOfPucks() -1 && puckIndex >= 0 ) {
			return puckIndex;
		} else {
			return -1;
		}
	}
	
	public static int[] puckLabelToPinIndices(char puckLabel) {
		int pinsPerPuck = getPinsPerPuck();
		int[] indices = new int[pinsPerPuck];

		int puckNumber = puckLabelToIndex(puckLabel);
		if (puckNumber == -1) {
			return null;
		}
		
		for (int i=0; i < pinsPerPuck; i++) {
			indices[i] = puckNumber * pinsPerPuck + i - 1;
		}
		return indices;
	}
}
