package gov.anl.gmca.screening;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.io.Database.DBName;
import gov.anl.gmca.io.Database.Result;
import gov.anl.gmca.io.PVLookup.PV_SCR;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.hw.Database;

import java.util.HashSet;
import java.util.Map;

//
// QX TODO this needs to be improved to allow in memory check instead of mySQL operation
//
public class Pucks {

	public Pucks() {
		Result r = Database.exec(DBName.USERDATA, "SHOW TABLES LIKE 'Pucks';");
		if (r.numRows() < 1) {
			String createTable = "CREATE TABLE IF NOT EXISTS Pucks(position varchar(1), type varchar (10) NOT NULL, name varchar (20), owner varchar(30), PRIMARY KEY (position));";
			Database.exec(DBName.USERDATA, createTable);
		}
		
		r = Database.exec(DBName.USERDATA, "select * from Pucks;");
		if (r.numRows() < 6 || r.numRows() != ModelScreening.NoofPucks) {
			Database.exec(DBName.USERDATA, "truncate Pucks;");
			for (int i = 0; i < ModelScreening.NoofPucks; i++) {
				char p = ModelScreening.puckLabels[i];
				Database.exec(DBName.USERDATA, String.format(
						"insert into Pucks (position, type, name, owner) values ('%s', 'ALS', 'LAB001', 'username');",
						p));
			}
		}
	}
	
	public static void update(String position, String type, String name, String owner) {
		Database.exec(DBName.USERDATA,
				String.format("update Pucks set type='%s', name='%s', owner='%s' where position='%s';", type,
						name, owner, position));
	}

	public static Result queryPucks() {
		Result r = Database.exec(DBName.USERDATA, "select position, type, name, owner from Pucks;");
		return r;
	}
	
	public static String[] getPuckSelectLabels() {
		String[] puckSelectLabels = new String[ModelScreening.NoofPucks];
		for(int i = 0; i < ModelScreening.NoofPucks; i++) {
			char puckLabel = ModelScreening.puckLabels[i];
			puckSelectLabels[i] = getPucknameFromPuckLabel(puckLabel);
		}
		return puckSelectLabels;
	}
	
	public static String getPucknameFromPinIndex(int pinIndex) {
		char puckLabel = ModelScreening.indexToLabel(pinIndex).charAt(0);
		return getPucknameFromPuckLabel(puckLabel);
	}

	public static String getPuckNameFromPuckIndex(int puckIndex) {
		char puckLabel = ModelScreening.puckLabels[puckIndex];
		return getPucknameFromPuckLabel(puckLabel);
	}

	public static String getPucknameFromPuckLabel(char puckLabel) {
		boolean enableOwnershipCheck = Epics.getBoolean(PV_SCR.enablePuckOwnership);

		if (enableOwnershipCheck) {
			Result r = Database.exec(DBName.USERDATA, "select name from Pucks where position='" + puckLabel + "';");
			if (r.numRows() == 1) {
				String puckName = r.getRow(0).get("name");
				if (puckName.replaceAll(" ", "").length() > 0 ) {
					return String.format("%s", puckName);
				} else {
					return String.format("%s%s","Puck", puckLabel);
				}
			}
		}
		return String.format("%s%s","Puck", puckLabel);
	}
	
	public static boolean isPuckOwner(char puckLabel) {
		boolean enableOwnershipCheck = Epics.getBoolean(PV_SCR.enablePuckOwnership);
		if (!enableOwnershipCheck) {
			return true;
		}

		String username = System.getProperty("user.name");
		Result r = Database.exec(DBName.USERDATA, "select owner from Pucks where position='" + puckLabel + "';");
		try {
			String puckOwner = r.getRow(0).get("owner");
			//if (puckOwner.equals(username)) {
			if (puckOwner.equals(username) || puckOwner.equals("") || puckOwner.equals("*")) {
				return true;
			} else {
				return false;
			}
		} catch (Exception e) {
			return false;
		}
	}

	public static boolean isPuckOwnerFromPinIndex(int pinIndex) {
		char puckLabel = ModelScreening.indexToLabel(pinIndex).charAt(0);
		return isPuckOwner(puckLabel);
	}

	public static boolean isPuckOwnerFromPuckIndex(int puckIndex) {
		char puckLabel = ModelScreening.puckLabels[puckIndex];
		return isPuckOwner(puckLabel);
	}
	
	// check each port and make sure user does not define any illegal ports
	public static void portCheck() {
		Result result = Database.exec(DBName.USERDATA, String.format("select position from Pucks where user='%s';", System.getProperty("user.name")));
		if (result.numRows() < 1 ) {
			return;
		}
		
		HashSet<String> positions = new HashSet<>();
		for (Map<String, String> row: result.getRows()) {
			positions.add(row.get("position"));
		}
		
		result = Database.exec(DBName.USERDATA, "select sampleNumber, Port from SilTable;");
		
		for (Map<String, String> row: result.getRows()) {
			String port = row.get("Port");
			int pinIndex = Integer.valueOf(row.get("sampleNumber"));
			String puckname = ModelScreening.indexToLabel(pinIndex).substring(0,1);
			if (!port.startsWith(puckname) || !positions.contains(port.substring(0, 1))) {
				//something is wrong
			}
		}
		
	}
	
}
