/**
 * @author spothineni
 * 
 * Screening tab GUI for spreadsheet, and history
 * 
 * Handles GUI for top half of screening tab.  Includes table construction, editing, and updating.
 * 
 * Contains PV listeners for table and spreadsheet field.
 * 
 * - mouse handler for checking samples
 * - 
 * 
 * DevSilTable loads the SilTable table from SQL
 * - contains the sample order, checkbox, and status (color) of each sample
 * - allows editing of any field and sends new value to SQL
 * - task list and history
 * - puck selectors (highlight no checkbox = partially selected)
 * - 
 */

package gov.anl.gmca.screening.ui;

import gov.anl.gmca.analysis.autoProcessData;
import gov.anl.gmca.analysis.helpers.WdgProtectedRunnableDialog;
import gov.anl.gmca.collect.DevCollectRun;
import gov.anl.gmca.collect.DevCollectRunList;
import gov.anl.gmca.collect.DevCollectSettings;
import gov.anl.gmca.collect.Filenames;
import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.ValidateDirPrefix;
import gov.anl.gmca.common.ui.NBFileDialog;
import gov.anl.gmca.common.ui.NBMessageBox;
import gov.anl.gmca.common.ui.Tab;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.gui.MenuActions;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_ROBOT;
import gov.anl.gmca.io.PVLookup.PV_RUNS;
import gov.anl.gmca.io.PVLookup.PV_RUN;
import gov.anl.gmca.io.PVLookup.PV_SCR;
import gov.anl.gmca.io.PVLookup.PV_SCR_RUN;
import gov.anl.gmca.raster.DevRasterRun;
import gov.anl.gmca.raster.DevRasterRunList;
import gov.anl.gmca.raster.ModelRasterState.RasterListType;
import gov.anl.gmca.sample.DevCryoJetMonitor;
import gov.anl.gmca.scan.DevScan;
import gov.anl.gmca.screening.*;
import gov.anl.gmca.strategy.CmdCollectStrategy;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.Util;
import gov.aps.jca.dbr.DBR;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Matcher;

public class TabScreening extends Tab implements ClientListener {
	public static String isScr = "_scr";
	public static WdgSampleInfoPeelOut sampleDialog;
	public static ModelCrystalList crystalList = new ModelCrystalList();
	protected static WdgScreeningTasks screeningTasks;
	protected Epics.EpicsValueMonitor monitor = null;
	private ModelSilTable model = null;

	public Table table;
	protected Composite spreadsheetComp, actionSelectCrystals;
	protected TableItem removeSample, currentItem;
	protected TableCursor cursor;
	protected TableViewer tview;
	protected Button[] crystalSelected;
	protected Button editButton, exportButton, browseButton, btnMoreTools, peelButton; /*loadButton, clearButton, clearColorButton,*/ 
	protected Button[] puckSelect = new Button[ModelScreening.NoofPucks];
	protected Label labelMounted, puckMountedLabel;
	protected Label[] labelCrystal = null;
	protected Label[] sampleHistory = new Label[10];
	protected Text spreadsheetText, usernameField, passwordField;
	protected NBFileDialog dialog;
	protected List taskList = null;
	private WdgEditPucks editPucks = null;
	Integer[] editableColumns = {2,3,4};
	
	public static String[] ActionNames = { "MountNextXtal", "CenterXtal", "Paused4Center", "JpegSnap", "Collect", "Collect",
			"Collect", "Paused4Inspection" };

	String prefix, dir, dir0, spreadsheetName = "", samp;

	boolean manualSelection = false, 
			isWebice = false, 
			useWebice = false, 
			spreadsheetLoaded = false;

	int NoofPucks = ModelScreening.NoofPucks;
	int pinsPerPuck = ModelScreening.getPinsPerPuck();

	final int[] screenedCrystals = new int[NoofPucks * pinsPerPuck];
	int[] countCheck = new int[ModelScreening.NoofPucks];
	int[] countUnCheck = new int[ModelScreening.NoofPucks];
	int counter, currentSample, removePin;
	int[] selected = new int[200];
	int previousPin = -1, commandIssued = 0, flagLabelCrystal = 0, selectionCount = 0;

	public void updateTable() {
		tview = new TableViewer(table);
		//tview.setContentProvider(model);
		//tview.setContentProvider(new ScreenContentProvider(tview));
		tview.setContentProvider(new LazyScreenContentProvider(tview));
		tview.setLabelProvider(model);
		tview.setUseHashlookup(true);
		tview.setInput(model);
		tview.setItemCount(model.getRowCount());
	}

	public TabScreening(Composite parent, int style) {
		super(parent, style);
		Log.note("TabScreening: start");
		Client.addListener(this);
		setLayout(new GridLayout(1, false));
		setBackground(GUI.BGCOLOR);

		new Pucks();

		GridData gd;
		spreadsheetComp = new Composite(this, SWT.NONE);
		spreadsheetComp.setBackground(GUI.BGCOLOR);
		GridLayout gl = new GridLayout(5, true);

		spreadsheetComp.setLayout(gl);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		spreadsheetComp.setLayoutData(gd);

		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint = 20;
		spreadsheetText = new Text(spreadsheetComp, SWT.BORDER);
		spreadsheetText.setLayoutData(gd);
		spreadsheetText.setToolTipText("This is the current spreadsheet loaded");

		/* Browse for spreadsheet */
		gd = new GridData();
		gd.heightHint = 30;
		gd.horizontalAlignment = SWT.FILL;
		browseButton = new Button(spreadsheetComp, SWT.PUSH);
		browseButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		browseButton.setText("Import spreadsheet");
		browseButton.setLayoutData(gd);
		browseButton.setToolTipText("Import spreadsheet");

		gd = new GridData();
		gd.heightHint = 30;
		gd.horizontalAlignment = SWT.CENTER;
		editButton = new Button(spreadsheetComp, SWT.CHECK | SWT.BORDER);
		editButton.setImage(GUI.imageedit);
		editButton.setBackground(GUI.BGCOLOR);
		editButton.setText("Edit");
		editButton.setLayoutData(gd);

		gd = new GridData();
		gd.heightHint = 30;
		gd.horizontalAlignment = SWT.FILL;
		exportButton = new Button(spreadsheetComp, SWT.PUSH);
		exportButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		exportButton.setText("Export spreadsheet");
		exportButton.setLayoutData(gd);
		exportButton.setToolTipText("Export table into excel sheet");
		exportButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				// exporting
				if (spreadsheetLoaded) {
					try {
						exportSpreadsheet();
					} catch (Exception e1) {
						e1.printStackTrace();
					}
				}
			}
		});

		gd = new GridData();
		gd.heightHint = 30;
		gd.horizontalAlignment = SWT.FILL;
		peelButton = new Button(spreadsheetComp, SWT.PUSH);
		peelButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		peelButton.setText("Mini Screening Control");
		peelButton.setLayoutData(gd);
		peelButton.setToolTipText("Display separate window for mounted sample");
		peelButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				try {
					MenuActions.showScreeningSampleWindow();
				} catch (Exception e1) {
					e1.printStackTrace();
				}
			}
		});

		browseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						String fileSpreadsheet = selectspreadsheetFile();
						handleBrowseButton(fileSpreadsheet);
					}
				});
			}
		});

		/* MiniScreening popup extra window for showing BIG sample characters */
		sampleDialog = new WdgSampleInfoPeelOut(parent.getShell(), SWT.NONE);
		sampleDialog.create();
		sampleDialog.messageDialog.setVisible(false);

		try {
			model = new ModelSilTable();
		} catch (Exception e) {
			e.printStackTrace();
		}

		table = new Table(spreadsheetComp, SWT.VIRTUAL | SWT.BORDER | SWT.CHECK | SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL);
		table.setLinesVisible(true);
		table.addMouseListener(new MouseListener() {

			@Override
			public void mouseDoubleClick(MouseEvent arg0) {
			}

			@Override
			public void mouseDown(MouseEvent arg0) {

				if (arg0.button == 3) {
					if (!spreadsheetLoaded)
						return;

					int index = table.getSelectionIndex();
					final TableItem currentItem = table.getItem(index);

					int directory = model.getColumnIndex("Directory");
					dir = currentItem.getText(directory);
					dir0 = ModelScreening.currentRootDir;

					Menu popupMenu = new Menu(table);

					MenuItem refreshItem = new MenuItem(popupMenu, SWT.NONE);
					refreshItem.setText("Update the view");
					MenuItem exportItem = new MenuItem(popupMenu, SWT.NONE);
					exportItem.setText("Export to a file");

					refreshItem.addSelectionListener(new SelectionListener() {
						public void widgetDefaultSelected(SelectionEvent arg0) {
						}

						public void widgetSelected(SelectionEvent arg0) {
							GUI.asyncExec(new Runnable() {
								public void run() {
									loadSpreadsheet();
								}
							});
						}
					});

					exportItem.addSelectionListener(new SelectionListener() {
						public void widgetDefaultSelected(SelectionEvent arg0) {}

						public void widgetSelected(SelectionEvent arg0) {
							exportSpreadsheet();
						}
					});

					table.setMenu(popupMenu);
				}
			}

			public void mouseUp(MouseEvent arg0) {}
		});

		table.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {

				if (event.detail == SWT.CHECK) {
					final TableItem item = (TableItem) event.item;
					int idx = table.indexOf(item);

					if (!Client.isMaster()) {
						if (crystalList.isExist(idx) == 1)
							item.setChecked(true);
						else {
							item.setChecked(false);
							item.setText("");
						}
						return;
					}
					commandIssued = 1;

					if (!Pucks.isPuckOwnerFromPinIndex(idx)) {
						Log.warn("You can only select crystals that you are the owner.");
						item.setChecked(false);
						return;
					}

					if (item.getChecked() == true) {
					} else {
						crystalList.removeCrystalFromList(Integer.toString(idx));
						item.setText("");
						item.setImage(1, null);
						item.setChecked(false);
						if (crystalList.getNoofCrystals() == 0) {
							ModelScreening.crystalStartIndex = 0;
							manualSelection = false;
						}
					}

					changeNumbering(0);

					setCrystal(idx, true);
					updateTaskProgress();

				} else {
					table.deselectAll();
				}
			}
		});

		cursor = new TableCursor(table, SWT.NONE);
		editButton.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event arg0) {
				if (spreadsheetLoaded) {
					int isSil = Epics.getInteger(PV_SCR.SILID);

					// update tag image for editable columns
					if (editButton.getSelection()) {

						if (isSil != 0) {
							for (int h = 2; h < model.getColumnCount(); h++) {
								TableColumn editcol = table.getColumn(h);
								String colName = model.getColumnName(h);
								if (colName.equals("CrystalID") || 
										colName.equals("Directory") || 
										colName.equals("Comment"))
									editcol.setImage(GUI.imageedit);
							}
							cursor.setVisible(true);
						} else {
							/*
							for (int h = 2; h < model.getColumnCount(); h++) {
								TableColumn editcol = table.getColumn(h);
								editcol.setImage(GUI.imageedit);
							}
							*/
							for (int x : editableColumns) {
								TableColumn editcol = table.getColumn(x);
								editcol.setImage(GUI.imageedit);
							}
							cursor.setVisible(true);
						}

						// remove tag image
					} else {
						cursor.setVisible(false);
						for (int h = 2; h < model.getColumnCount(); h++) {
							TableColumn editcol = table.getColumn(h);
							editcol.setImage(null);
						}
					}
				}
			}
		});

		enableEdit();
		
		

		if (model != null) {
			for (int i = 0; i < model.getColumnCount(); i++) {
				TableColumn tc = new TableColumn(table, SWT.CHECK);
				tc.setText(model.getColumnName(i));

				tc.setWidth(ModelRowData.columnWidths[i]);
				tc.setAlignment(SWT.CENTER);
			}
		}
		table.setHeaderVisible(true);

		updateTable();

		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = 4;
		table.setLayoutData(gd);

		gd = new GridData(SWT.FILL, SWT.FILL, true, true);

		CTabFolder ctFolder = new CTabFolder(spreadsheetComp, SWT.NONE);
		ctFolder.setFont(GUI.FONT_SAMPLE_TABS);
		ctFolder.setMinimumCharacters(12);
		ctFolder.setBackground(GUI.BGCOLOR);
		ctFolder.setLayoutData(gd);
		ctFolder.setSimple(false);

		ctFolder.setSelectionBackground(new Color[] { GUI.getDisplay().getSystemColor(SWT.COLOR_WHITE),
				new Color(GUI.getDisplay(), 184, 211, 238), new Color(GUI.getDisplay(), 136, 182, 227),
				new Color(GUI.getDisplay(), 136, 182, 227) }, new int[] { 25, 50, 100 }, true);

		Composite tasklistComp = new Composite(ctFolder, SWT.NONE);
		tasklistComp.setBackground(GUI.BGCOLOR);
		tasklistComp.setLayout(new GridLayout(1, false));
		
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		taskList = new List(tasklistComp, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
		taskList.setLayoutData(gd);

		taskList.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event e) {
				setProgress();
			}
		});

		CTabItem tasklistTab = new CTabItem(ctFolder, SWT.NONE);
		tasklistTab.setText("Tasklist");
		tasklistTab.setControl(tasklistComp);

		Composite historyComp = new Composite(ctFolder, SWT.NONE);
		historyComp.setBackground(GUI.COLOR_WHITE);
		historyComp.setLayout(new GridLayout(1, false));

		for (int i = 0; i < 10; i++) {
			gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);

			sampleHistory[i] = new Label(historyComp, SWT.NONE);
			sampleHistory[i].setBackground(GUI.COLOR_WHITE);
			sampleHistory[i].setLayoutData(gd);
			String history = Epics.pvs_getString(Epics.addBID("R:mnt-" + i));
			sampleHistory[i].setText("-" + i + ": " + history);
		}

		CTabItem historyTab = new CTabItem(ctFolder, SWT.MULTI | SWT.V_SCROLL);
		historyTab.setText("History");
		historyTab.setControl(historyComp);

		ctFolder.setSelection(0);

		actionSelectCrystals = new Composite(this, SWT.NONE);
		actionSelectCrystals.setBackground(GUI.BGCOLOR);
		GridLayout puckSelectionGrid = new GridLayout(12, false);
		actionSelectCrystals.setLayout(puckSelectionGrid);

		gd = new GridData(SWT.CENTER, SWT.CENTER, true, false);
		gd.minimumHeight = 40;
		actionSelectCrystals.setLayoutData(gd);

		String[] puckNames = Pucks.getPuckSelectLabels();
		for (int i = 0; i < NoofPucks; i++) {
			gd = new GridData();
			gd.grabExcessVerticalSpace = true;
			puckSelect[i] = new Button(actionSelectCrystals, SWT.CHECK);
			puckSelect[i].setText("Posn " + ModelScreening.puckLabels[i]);
			puckSelect[i].setToolTipText(puckNames[i]);
			puckSelect[i].setBackground(GUI.BGCOLOR);
			puckSelect[i].setLayoutData(gd);
			
			if (!Client.isMaster()) {
				puckSelect[i].setEnabled(false);
			}
		}

		for (int i = 0; i < NoofPucks; i++) {
			final int tempi = i;

			puckSelect[i].addListener(SWT.Selection, new Listener() {

				@Override
				public void handleEvent(Event arg0) {

					if (!spreadsheetLoaded) {
						Log.error("Spreadsheet not loaded, please load it first.");
						return;
					}

					final boolean selectCryx = puckSelect[tempi].getSelection();

					final String puckType = ModelScreening.getPuckType();
					final int noofSamplesPerPuck;

					if (puckType.equals("A"))
						noofSamplesPerPuck = ModelScreening.ALSPuckSamples;
					else
						noofSamplesPerPuck = ModelScreening.RigakuPuckSamples;

					GUI.asyncExec(new Runnable() {
						public void run() {
							for (int j = 0; j < noofSamplesPerPuck; j++) {
								int calcCryx = noofSamplesPerPuck * tempi + j;

								TableItem item = table.getItem(calcCryx);

								item.setChecked(selectCryx);

								if (selectCryx)
									crystalList.addCrystalToList(Integer.toString(calcCryx));
								else {
									item.setImage(1, null);
									crystalList.removeCrystalFromList(Integer.toString(calcCryx));
									item.setText("");
								}
								setCrystal(calcCryx, false);
							}

							// QX bottleneck setAllCrystals take too long to run ~133 milli sec per puck
							// selection

							// save to db, this method only run in 17 msec
							setAllCrystalsFast();

							// update model in memory
							model.setAllCrystals();
							changeNumbering(0);
						}
					});
					updateTaskProgress();
				}

			});
		}

		
		/// Aggregated tools added by QX
		gd = new GridData();

		btnMoreTools = new Button(actionSelectCrystals, SWT.BORDER);
	    btnMoreTools.setBackground(GUI.COLOR_BLUE_BUTTON);
	    //gd.widthHint = 80;
	    btnMoreTools.setLayoutData(gd);
	    btnMoreTools.setText("Xtra tools");

		gd = new GridData();
	    Menu moreToolsMenu = new Menu(btnMoreTools);
	    MenuItem addAllCrystalsItem = new MenuItem(moreToolsMenu, SWT.NONE);
	    addAllCrystalsItem.setText("Add all crystals");
	    addAllCrystalsItem.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				if (!Client.isMaster()) {
					Log.error("Become master and try again.");
					return;
				}
				if (!spreadsheetLoaded) {
					Log.error("Spreadsheet not loaded, please load it first.");
					return;
				}
				
				checkAll();
				setAllCrystalsFast(); // update db
				model.setAllCrystals(); // update model and notify slaves
				changeNumbering(0); // reset numbering
				//updateTaskProgress(); // update task list
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	    
	    MenuItem moveCrystalItem = new MenuItem(moreToolsMenu, SWT.NONE);
	    moveCrystalItem.setText("Add/Move a crystal to top");
	    moveCrystalItem.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				Log.note("rearrange crystal order");
				if (!Client.isMaster()) {
					Log.error("Become master and try again.");
					return;
				}
				if (!spreadsheetLoaded) {
					Log.error("Spreadsheet not loaded, please load it first.");
					return;
				}
				moveCrystals();
			}

			// TODO
			// problem 1: need to check what to do with the crystal already mounted !!
			private void moveCrystals() {
				WdgCrystalMove dlg = new WdgCrystalMove(parent.getShell());
				if (dlg.open() == Window.OK) {
					String requestedCrystal = dlg.getCrystal().replaceAll(" ", "");

					// check the user own the port
					if (!validateInputs(requestedCrystal)) {
						Log.error("invalid input, please input the port name, such as A1.");
						return;
					}

					String currentMounted = Epics.getString(PV_ROBOT.mounted);
					if (currentMounted.equals(requestedCrystal)) {
						return;
					}

					//Log.note("crystalList=" + crystalList.toString());
					int crystalIndex = CmdScreening.labelToindex(requestedCrystal);
					if (!Pucks.isPuckOwnerFromPinIndex(crystalIndex)) {
						Log.error("cannot move crystal since it belongs to another user.");
						return;
					}

					crystalList.addOrMove(String.valueOf(crystalIndex), 0);
					/* if mounted crystal is off list, so not necessary to check? */
					/*
					if (currentMounted.equals("none")) {
						crystalList.addOrMove(String.valueOf(crystalIndex), 0);
					} else {
						crystalList.addOrMove(String.valueOf(crystalIndex), 1);
					}
					*/
					
					//Log.note("crystalList=" + crystalList.toString());
					setAllCrystalsFast(); // update db
					model.setAllCrystals(); // update model and notify slaves
					changeNumbering(0); // reset numbering
					//updateTaskProgress(); // update task list, this is done in the SilTable (-2) listener code
				}
			}

			// check the input is valid
			private boolean validateInputs(String requestedCrystal) {
			
				if (requestedCrystal.length() > 3 || requestedCrystal.length() < 2) {
					return false;
				}

				String puckLocation = requestedCrystal.substring(0, 1);
				if (!new String(ModelScreening.puckLabels).contains(puckLocation)) {
					return false;
				}

				String crystalLocation = requestedCrystal.substring(1);
				try {
					int crystal = Integer.valueOf(crystalLocation);
					if (crystal > ModelScreening.getPinsPerPuck() || crystal <= 0) {
						return false;
					}
				} catch (Exception e) {
					return false;
				}

				return true;
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	    
	    MenuItem clearSelectionsItem = new MenuItem(moreToolsMenu, SWT.NONE);
	    clearSelectionsItem.setText("Clear all crystal selections");
	    clearSelectionsItem.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				if (!spreadsheetLoaded) {
					Log.error("Spreadsheet not loaded, please load it first.");
					return;
				}
				
				if (!Client.isMaster()) {
					Log.error("Become master and try again.");
					return;
				}

				final NBMessageBox messageBox = new NBMessageBox(GUI.getShell(),
						SWT.OK | SWT.CANCEL | SWT.ICON_WARNING);
				messageBox.setMessage("All the selection of the  crystals will be lost.\nIt will take few seconds \"Task Progress\" to be cleared\nexcept the mounted sample tasks");
				GUI.asyncExec(new Runnable() {
					public void run() {
						if (messageBox.open() == SWT.OK) {
							uncheckAll();
							for (int j = 0; j < ModelScreening.NoofPucks; j++) {
								puckSelect[j].setSelection(false);
							}
							setCrystalList();

							updateTaskProgress();
							changeNumbering(0);
						}
					}
				});
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	    
	    MenuItem clearResultsItem = new MenuItem(moreToolsMenu, SWT.NONE);
	    clearResultsItem.setText("Clear all results");
	    clearResultsItem.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				if (!spreadsheetLoaded) {
					Log.error("Spreadsheet not loaded, please load it first.");
					return;
				}
				
				if (!Client.isMaster()) {
					Log.error("Become master and try again.");
					return;
				}
				
				boolean[] clearPucks = new boolean[ModelScreening.NoofPucks];

				WdgClearColorCode dlgColorcode = new WdgClearColorCode(GUI.getShell(), SWT.NONE, "");

				GUI.asyncExec(new Runnable() {
					public void run() {
						if (dlgColorcode.open(clearPucks) == SWT.OK) {
							updateTaskProgress();
							changeNumbering(0);
							try {
								clearColorCode(clearPucks);
							} catch (IOException e) {

								e.printStackTrace();
							}
						}
					}
				});
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
	    
	    MenuItem puckEditItem = new MenuItem(moreToolsMenu, SWT.NONE);
	    puckEditItem.setText("Puck assignment");
	    puckEditItem.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent arg0) {
				// QX edit pucks
				//if (Client.isMaster()) {
				GUI.asyncExec(new Runnable() {
					@Override
					public void run() {
						Shell shell = parent.getShell();
						WdgProtectedRunnableDialog dlg = new WdgProtectedRunnableDialog(shell);

						if (dlg.open() == Window.OK) {
							if (dlg.isPasswordOK()) {
								editPucks = new WdgEditPucks(shell, true);
								if (editPucks.open() != Window.OK) {
									return;
								}

								//afterPuckEdit();
							}
						}
					}
				});
				//} else {
				//	Log.warn("Sorry, only staff as master can edit the puck table.");
				//}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {}
		});
    
		btnMoreTools.setMenu(moreToolsMenu);
		
		// left click to activate menu (by default, right click)
		btnMoreTools.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				if (!spreadsheetLoaded) {
					Log.error("Spreadsheet not loaded, please load it first.");
					return;
				}
				Rectangle bounds = btnMoreTools.getBounds();
				Point point = btnMoreTools.getParent().toDisplay(bounds.x, bounds.y + bounds.height);
				moreToolsMenu.setLocation(point);
				moreToolsMenu.setVisible(true);
			}
		});

		gd = new GridData();
		Label puckMounted = new Label(actionSelectCrystals, SWT.NONE);
		puckMounted.setBackground(GUI.BGCOLOR);
		puckMounted.setFont(GUI.FONT_COLLECT_BUTTON2);
		puckMounted.setText("PuckType : ");
		puckMounted.setLayoutData(gd);

		gd = new GridData(SWT.CENTER, SWT.CENTER, false, false);
		gd.widthHint = 60;
		puckMountedLabel = new Label(actionSelectCrystals, SWT.NONE);
		puckMountedLabel.setBackground(GUI.BGCOLOR);
		puckMountedLabel.setForeground(GUI.COLOR_BLUE);
		puckMountedLabel.setFont(GUI.FONT_14_BOLD);

		int puck = Epics.getInteger(PV_ROBOT.puckType);

		if (puck == 0)
			puckMountedLabel.setText("ALS");
		else
			puckMountedLabel.setText("Rigaku");

		puckMountedLabel.setLayoutData(gd);

		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.CENTER;
		gd.widthHint = 200;
		Label labelMount = new Label(actionSelectCrystals, SWT.NONE);
		labelMount.setFont(GUI.FONT_COLLECT_BUTTON2);
		labelMount.setBackground(GUI.BGCOLOR);
		labelMount.setText("Mounted : ");

		gd = new GridData();
		gd.widthHint = 50;
		gd.horizontalAlignment = SWT.CENTER;
		labelMounted = new Label(actionSelectCrystals, SWT.BORDER | SWT.CENTER);
		labelMounted.setFont(GUI.FONT_14_BOLD);
		labelMounted.setBackground(GUI.BGCOLOR);
		labelMounted.setForeground(GUI.COLOR_BLUE);
		String mounted = Epics.getString(PV_ROBOT.mounted);

		labelMounted.setText(mounted);
		labelMounted.setLayoutData(gd);

		// //////////////////////////////Construct actions pane////////////////
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		screeningTasks = new WdgScreeningTasks(this, SWT.BORDER);
		screeningTasks.setLayoutData(gd);
		GUI.asyncExec(new Runnable() {
			public void run() {
				updateWidgetState();
			}
		});
		// ////////////////////////////////////////////////////////////////////

		// initialize arrays
		for (int i = 0; i < ModelScreening.NoofPucks * ModelScreening.getPinsPerPuck(); i++) {
			screenedCrystals[i] = 0;
		}

		AppState.addListener(AppState.Operation.START_SCREENING, new AppState.Listener() {
			@Override
			public void stateChanged(Operation op, boolean allowed) {

				GUI.asyncExec(new Runnable() {
					public void run() {
						updateWidgetState();
					}
				});
			}
		});

		monitor = new Epics.EpicsValueMonitor() {
			@Override
			public void valueChanged(String pvName, DBR value) {
				final String _pvName = pvName;
				final DBR _value = value;

				if (_pvName.equals(PV_ROBOT.puckType.toString())) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							if (Epics.dbrToInt(_value) == 0)
								puckMountedLabel.setText("ALS");
							else
								puckMountedLabel.setText("Rigaku");

							if (model != null && pinsPerPuck != model.getRowCount() && model.getRowCount() != 0) {

								Log.error("Puck type mismatch ALS/Rigaku or Spreadsheet not compatible \n"
										+ "\n See Instructions http://www.gmca.anl.gov/facilities/automounter.html#instructions");

								displayMessageBox("Puck type mismatch ALS/Rigaku or Spreadsheet not compatible \n"
										+ "\n See Instructions http://www.gmca.anl.gov/facilities/automounter.html#instructions");

								model.deleteAllTable();
								model.data.clear();
								updateTable();
							}

						}
					});
				}
				
				if (_pvName.equals(PV_ROBOT.mounted.toString())) {

					final String mounted = Epics.dbrToString(_value);

					GUI.asyncExec(new Runnable() {
						public void run() {
							String Pin = "";
							for (int i = 0; i < 10; i++) {
								Pin = Epics.pvs_getString(Epics.addBID("R:mnt-" + i));
								sampleHistory[i].setText("-" + i + ": " + Pin);
							}
							labelMounted.setText(mounted);
							ModelScreening.mountedPin = mounted;

							if (!spreadsheetLoaded)
								return;
							if (mounted.equals("none")) {

								ModelScreening.robotSuccess = true;
								if (previousPin > -1) {
									TableItem previousItem = table.getItem(previousPin);
									previousItem.setBackground(null);
									previousItem.setImage(1, null);
								}
								return;
							}

							if (!spreadsheetLoaded)
								return;

							try {
								showMounted(mounted);
								changeNumbering(0);
							} catch (Exception e) {
							}
						}
					});
				}
				
				if (_pvName.equals(PV_SCR.spreadsheetStatus.toString())) {
					int status = 0;
					try {
						status = Epics.dbrToInt(_value);
					} catch (Exception e) {
						e.printStackTrace();
					}

					if (status == 1) {
						GUI.asyncExec(new Runnable() {
							public void run() {
								loadSpreadsheet();
								Util.sleep(100);

								String Pin = Epics.getString(PV_ROBOT.mounted);
								if (crystalList.getNoofCrystals() > 0) {
									if (Pin.equals("none")) {
										table.setTopIndex(Integer.parseInt(crystalList.getElement(0)));
										return;
									} else
										table.setTopIndex(CmdScreening.labelToindex(Pin));

								}
								changeNumbering(0);
								showMounted(Pin);
							}
						});
					}
				}

				if (_pvName.equals(PV_SCR.CurrentTask.toString())) {
					final String status = Epics.dbrToString(_value);
					System.out.println(status);
					if (status.equals(""))
						return;

					if (status.equals("Robot not ready")) {
						if (!Client.isMaster()) {
							return;
						}
						Log.error("Robot not ready");
						GUI.asyncExec(new Runnable() {
							public void run() {
								NBMessageBox messageBox = new NBMessageBox(GUI.getShell(), SWT.OK | SWT.ICON_ERROR);
								messageBox.setMessage("Robot not ready");
								messageBox.open();
							}
						});
					} else if (status.equals("SampleLoaded")) {
						// do nothing
					} else if (status.equals("Unloaded")) {
						String mounted = Epics.getString(PV_ROBOT.mnt$0);
						Log.note("Unloaded " + mounted);
						if (!mounted.equals("none")) {
							removePin = CmdScreening.labelToindex(mounted);

							if (spreadsheetLoaded) {
								GUI.asyncExec(new Runnable() {
									public void run() {
										removeSample = table.getItem(removePin);
										crystalList.removeCrystalFromList(Integer.toString(removePin));
										removeSample.setBackground(null);
										removeSample.setChecked(false);
										removeSample.setText("");
										removeSample.setImage(1, null);

										Util.sleep(100);
										changeNumbering(1);
										setCrystal(removePin, true);
										updateTaskProgress();
										saveExcelFile();
									}
								});
							}
						}
					} else if (status.equals("RenameGroup")) {

						if (!Client.isMaster())
							return;

						String mou = Epics.getString(PV_ROBOT.mounted);
						ModelScreening.currentPort = mou;
						currentSample = CmdScreening.labelToindex(mou);

						if (model == null)
							return;

						if (CmdScreening.group == 0)
							CmdScreening.group++;

						GUI.asyncExec(new Runnable() {
							public void run() {
								TableItem currentItem = table.getItem(currentSample);

								prefix = Epics.getString(PV_SCR.prefix);

								if (prefix.equals("")) {
									prefix = ModelScreening.indexToLabel(currentSample);
								}

								int st = prefix.lastIndexOf("_");
								int grp = 1;
								String groupPre;
								boolean success = true;

								if (st > 0) {
									groupPre = prefix.substring(st + 1);
									try {
										grp = Integer.parseInt(groupPre);

									} catch (NumberFormatException nfe) {
										success = false;
									}
									if (success) {
										if (ModelScreening.preIncrement)
											grp++;
										else
											ModelScreening.preIncrement = true;

										prefix = prefix.substring(0, st);

										prefix = prefix.concat("_" + Integer.toString(grp));

									} else {
										prefix = prefix.concat("_" + CmdScreening.group);
									}
								} else {
									prefix = prefix.concat("_" + CmdScreening.group);
								}

								int directory = model.getColumnIndex("Directory");
								dir = currentItem.getText(directory);
								ModelScreening.currentSubDir = dir;
								dir0 = ModelScreening.currentRootDir;
								if (dir0 == null || dir0.equals("null")) {
									dir0 = "";
								}

								ModelScreening.currentRootDir = dir0;
								ModelScreening.currentPrefix = prefix;
								Epics.put(PV_SCR.prefix, ValidateDirPrefix.validateFileName(prefix));

								Epics.put(PV_SCR_RUN.fileroot.run(0), prefix);
                                Epics.put(PV_RUN.fileroot.run(0), prefix);

                                String cDir = "";
								if (dir.equals("")) {
									cDir = dir0.replace("//", "/");
								} else {
									if (dir0 != null && !dir0.isEmpty())
										cDir = (dir0 + "/" + dir).replace("//", "/");
									else
										cDir = dir;
								}
								putR0Dir(cDir);

								WdgScreeningTasks.updateRuns();
							}
						});
					} else if (status.equals("Unload") || status.equals("Completed")
							|| status.equals("UnloadLastSample")) {

					} else if (status.contains("Recollect")) {
						return;
					} else {
						String Sample = "";
						int space = status.indexOf(' ');
						if (space > 0)
							Sample = status.substring(0, space);
						else
							return;

						final int pin = CmdScreening.labelToindex(Sample);
						String status1 = status.substring(space + 1);
						if (!spreadsheetLoaded)
							return;

						setProgress();

						if (status1.equals("UnloadFailed")) {

							GUI.asyncExec(new Runnable() {
								public void run() {
									TableItem item_ = table.getItem(pin);
									item_.setImage(0, GUI.imagered);
									item_.setBackground(null);

									Epics.put(PV_SCR.prefix, "");
									try {
										TableItem itemC = table.getItem(Integer.parseInt(crystalList.getElement(0)));
										itemC.setBackground(null);
									} catch (Exception e) {

									}
								}
							});

							if (!Client.isMaster())
								return;

							screenedCrystals[pin] = 1;
							model.setCrystalScreened(pin, 1, "", true);
							Log.error("Unloading sample failed");

							GUI.asyncExec(new Runnable() {
								public void run() {

									NBMessageBox messageBox = new NBMessageBox(GUI.getShell(), SWT.OK | SWT.ICON_ERROR);
									messageBox.setMessage(
											"Unload Failed, Please check robot..\n" + ModelScreening.robotError);
									messageBox.open();

								}
							});
						}

						if (status1.equals("SampleLoadFailed")) {

							GUI.asyncExec(new Runnable() {
								public void run() {

									TableItem item_ = table.getItem(pin);
									item_.setImage(0, GUI.imagered);
									item_.setBackground(null);

									if (!Client.isMaster())
										return;

									Log.error("Sample Load failed");
									screenedCrystals[pin] = 1;
									model.setCrystalScreened(pin, 1, "", true);

									/* interpret robot errors */
									String solution = "";
									if (ModelScreening.robotError.contains("no pin@gonio")) {
										solution = "There is no sample mounted on gonio. To clear the sample \n"
												+ "on the top menu Tools -> Actions -> Clear mounted sample";
									} else if (ModelScreening.robotError.contains("unmount:pin")) {
										solution = "warm-up the gripper and press Start button to try again";
									} else if (ModelScreening.robotError.contains("recover:pin@gonio")) {
										solution = "Pin sensor detects a pin mounted on gonio, manually remove the pin to proceed \nor check pin sensor";
									} else if (ModelScreening.robotError.contains("check_pin:empty")) {
										solution = "There might be no sample in this position of the puckproceed with next sample, ";
									} else if (ModelScreening.robotError.contains("get_sample:svd")) {
										solution = "The sample might be tilted, open the dewar and make sure the sample sitting properly,\n"
												+ "If multiple errors occur re-seat the puck";
									} else if (ModelScreening.robotError.contains("cool_down > 300s")) {
										solution = "Cool down took more than 5 minutes, check the liquid nitrogen levels in the dewar,\n"
												+ "or check with your host for a solution if the problem continues";
									} else {
										solution = "";
									}

									Util.asyncPopupError("Warning", "Sample Load Failed , please check robot.\n"
											+ ModelScreening.robotError + "\n" + solution);
								}
							});

						}
						
						if (status1.equals("MountNextXtal")) {
							Util.sleep(100);

							currentSample = CmdScreening.labelToindex(Sample);

							GUI.asyncExec(new Runnable() {
								public void run() {
									TableItem currentItem = table.getItem(currentSample);
									currentItem.setBackground(GUI.COLOR_THICK_ORANGE);
								}
							});
							if (!Client.isMaster())
								return;
							WdgScreeningTasks.globalPrefix = "";
							if (model == null)
								return;
							if (!spreadsheetLoaded)
								return;

							GUI.asyncExec(new Runnable() {
								public void run() {
									try {
										TableItem currentItem = table.getItem(currentSample);
										int crystalId = model.getColumnIndex("CrystalID");
										prefix = currentItem.getText(crystalId);
										if (prefix.equals("")) {
											prefix = ModelScreening.indexToLabel(currentSample);
										}
										samp = prefix;
										ModelScreening.currentPort = prefix;
										prefix = prefix.concat("_0");
										currentItem.setBackground(GUI.COLOR_THICK_ORANGE);
										int directory = model.getColumnIndex("Directory");
										dir = currentItem.getText(directory);

										ModelScreening.currentSubDir = dir;
										ModelScreening.currentPrefix = prefix;
										dir0 = ModelScreening.currentRootDir;

										Epics.put(PV_SCR.prefix, ValidateDirPrefix.validateFileName(prefix));

										changeNumbering(1);
									} catch (Exception e) {
									}

									if (dir0 == null || dir0.equals("null"))
										dir0 = "";

									// validate and write directory name
									String cDir = "";
									if (dir.equals("")) {
										cDir = dir0.replace("//", "/");
									} else {
										if (dir0 != null && !dir0.isEmpty())
											cDir = (dir0 + "/" + dir).replace("//", "/");
										else
											cDir = dir;
									}
									putR0Dir(cDir);

									// write sample name into raster, collect and scan tabs
									if (!WdgScreeningTasks.globalPrefix.equals(prefix)) {
										putR0Samp(samp);
									} else {
										System.out.println("Screening warning:raster/collect/scan prefix might not be updated");
									}

									WdgScreeningTasks.updateRuns();
								}
							});
						}

						if (status1.equals("Paused4Center")) {
						}

						if (status1.equals("Paused4Inspection")) {

							final String sample = Sample;

							if (Client.isMaster())
								if (CmdScreening.manualRepeatFlag <= 0)
									setColorCode(sample);

						}
						if (status1.contains("Collect") || status1.contains("CenterXtal")) {
						}
					}
				}

				if (_pvName.equals(PV_SCR.silUpdated.toString())) {
					if (commandIssued == 1) {
						commandIssued = 0;
						return;
					}

					if (model.getRowCount() != pinsPerPuck * ModelScreening.NoofPucks)
						return;

					for (int i = 0; i < NoofPucks; i++) {
						countCheck[i] = 0;
						countUnCheck[i] = 0;
					}
					counter = 0;

					int row = Epics.dbrToInt(_value);
					if (row == -1) {
						return;
					} else if (row == -2) { // update everything

						selectionCount = 0;

						for (int j = 0; j < model.getRowCount(); ++j) {
							setSilUpdated(j, row);
						}

						if (!Client.isMaster()) { // slave updates
							model.setAllCrystals(false); // QX update must be false, otherwise infinite loop
							TreeMap<Integer, Integer> orderID = new TreeMap<Integer, Integer>();
							for (int j = 1; j <= selectionCount; ++j) {
								for (int i = 0; i < model.getRowCount(); i++) {
									if (model.data.get(i).selected == 1) {
										// crystalList.addCrystalToList(Integer.toString(i));
										// QX previous version does not honor existing order, e.g. when restarting
										// bluice
										orderID.put(model.data.get(i).order, i);
									}
								}
							}

							crystalList.deleteAll(); // TODO this can be problematic if other process is accessibling crystalList at the same time
							for (Map.Entry<Integer, Integer> entry : orderID.entrySet()) {
								int id = entry.getValue();
								crystalList.addCrystalToList(Integer.toString(id));
							}
							
							GUI.asyncExec(new Runnable() {

								@Override
								public void run() {
									// renumbering
									for (int j = 0; j < crystalList.getNoofCrystals(); j++) {
										try {
											TableItem _item = table
													.getItem(Integer.parseInt(crystalList.getElement(j)));
											_item.setText(Integer.toString(j + 1));
										} catch (Exception e) {
											e.printStackTrace();
											Log.error("Resetting numbering in spreadsheet of screening tab");
										}
									}

									// clear numbering
									for (int i = 0; i < pinsPerPuck * NoofPucks; i++) {
										TableItem item = table.getItem(i);
										if (!item.getChecked()) {
											item.setText("");
										}
									}
								}
							});
						}

						updatePuckSelection();
						updateTaskProgress();

						return;
					} else { // update row
						int getSelect = model.getCrystalSelection(row);
						if (getSelect == 1)
							crystalList.addCrystalToList(Integer.toString(row));
						else
							crystalList.removeCrystalFromList(Integer.toString(row));

						setSilUpdated(row, row);

						updatePuckSelection();
						updateTaskProgress();
					}
					
					
				}

				if (_pvName.equals(PV_SCR.pucksUpdated.toString())) {
					
					int val = Epics.dbrToInt(_value);
					if (Client.isMaster() && val == 1) {
					
						Log.note("DEBUG PV_SCR.pucksUpdated= " + val + " isMaster="+Client.isMaster()) ;
						GUI.asyncExec(new Runnable() {
							public void run() {
								afterPuckEdit();
								Log.note("master updated after pucks update");
							}
						});
					}
				}

				if (_pvName.equals(PV_SCR.enablePuckOwnership.toString())) {
					
					int val = Epics.dbrToInt(_value);
					Log.note("DEBUG PV_SCR.enablePuckOwnership = " + val + " isMaster="+Client.isMaster()) ;

					if (Client.isMaster() && val == 0) {
					
						GUI.asyncExec(new Runnable() {
							public void run() {
								for (int k = 0; k < NoofPucks; k++) {
									puckSelect[k].setEnabled(true);
								}		

								Log.note("puck ownership disabled, remove selection restrictions.");
							}
						});
					}
				}

				
			};
		};

		Epics.addMonitor(PV_ROBOT.puckType, monitor);
		Epics.addMonitor(PV_ROBOT.mounted, monitor);
		Epics.addMonitor(PV_SCR.spreadsheetStatus, monitor);

		Epics.addMonitor(PV_SCR.CurrentTask, monitor);
		Epics.addMonitor(PV_SCR.spreadsheetFile, monitor);
		Epics.addMonitor(PV_SCR.spreadsheetDir256, monitor);

		Epics.addMonitor(PV_SCR.silUpdated, monitor);
		Epics.addMonitor(PV_SCR.pucksUpdated, monitor);
		Epics.addMonitor(PV_SCR.enablePuckOwnership, monitor);

		System.out.println("Screening params database");

		/* initiate cryojetmonitor */
		new DevCryoJetMonitor();

		Log.note("TabScreening: done");
	}

	public void setEditorValue(Text text1) {
		TableItem row = cursor.getRow();
		if (row == null) {
			return;
		}
		final int rowIndex = table.indexOf(row);
		int column = cursor.getColumn();
		if (column < 2) {
			text1.dispose();
			return;
		}

		String commentTxt = text1.getText();
		commentTxt = commentTxt.replaceAll("\'", "");
		row.setText(column, commentTxt);

		System.out.println("Spreadsheet edit row :" + rowIndex + " column :" + column);
		boolean isCrystalIDLength = (commentTxt.length() > ModelScreening.LENGTH_SCREENING_PREFIX) && column == 2;
		if (isCrystalIDLength) { // F8_1234567891234567890
			NBMessageBox messageBox = new NBMessageBox(GUI.getShell(), SWT.OK | SWT.ICON_WARNING | SWT.CANCEL);
			messageBox.setMessage(
					"Number of characters for CrystalID should be less than " + ModelScreening.LENGTH_SCREENING_PREFIX);
			messageBox.open();

			commentTxt = (String) model.getValueAt(rowIndex, column);
		}

		model.setValueAt(commentTxt, rowIndex, column, true);

		Util.sleep(10);
		saveExcelFile();
		text1.dispose();
		cursor.setSelection(row, column);
	}

	private void afterPuckEdit() {
		String[] puckNames = Pucks.getPuckSelectLabels();
		for (int k = 0; k < NoofPucks; k++) {
			puckSelect[k].setToolTipText(puckNames[k]);
			puckSelect[k].setEnabled(true);
			if (!Pucks.isPuckOwnerFromPuckIndex(k)) {
				puckSelect[k].setEnabled(false);
			}
		}

		// remove existing selections not belonging to the current users
		ArrayList<Integer> crystalsUnset = new ArrayList<>();
		for (int i = 0; i < pinsPerPuck * NoofPucks; i++) {
			if (!Pucks.isPuckOwnerFromPinIndex(i)) {
				TableItem item = table.getItem(i);
				if (item.getChecked()) {
					item.setChecked(false);
					item.setText("");
					item.setImage(1, null);
					crystalList.removeCrystalFromList(Integer.toString(i));

					crystalsUnset.add(i);
				}
			}
		}
		model.unsetCrystalSelections(crystalsUnset, true);

		// not necessary?
		for (int j = 0; j < ModelScreening.NoofPucks; j++) {
			if (!Pucks.isPuckOwnerFromPuckIndex(j)) {
				puckSelect[j].setSelection(false);
				puckSelect[j].setBackground(GUI.BGCOLOR);
			}
		}

		Epics.putInteger(PV_SCR.silUpdated, -1);
		Epics.putInteger(PV_SCR.silUpdated, -2);
	}
	
	public void setSilUpdated(int row, int callbackRow) {
		final int pinscount = pinsPerPuck * NoofPucks;
		final int getSelect = model.getCrystalSelection(row);
		final int ordered = model.data.get(row).order;
		final int screened = model.data.get(row).screened;
		final int tempi = row;

		selectionCount++;
		GUI.asyncExec(new Runnable() {
			public void run() {
				TableItem item = table.getItem(tempi);
				if (getSelect == 1) {
					if (tempi < pinscount) {
						item.setChecked(true);
						item.setText(Integer.toString(ordered));
					}
				} else {
					if (tempi < pinscount) {
						item.setChecked(false);
						if (ModelScreening.mountedPin.contains("none"))
							item.setBackground(null);
						item.setText("");
					}
				}

				Image tempImage = null;
				if (screened == 3)
					tempImage = GUI.imagegreen;
				if (screened == 2)
					tempImage = GUI.imageblue;
				if (screened == 1)
					tempImage = GUI.imagered;
				if (screened == 0)
					tempImage = null;
				if (tempi < pinscount)
					item.setImage(0, tempImage);

				for (int j = 1; j < model.getColumnCount(); ++j) {
					String str = (String) model.data.get(tempi).col[j];

					if (j == model.getColumnIndex("Set")) {
						updateComboPrefixColumn(row, callbackRow, item, model.data.get(tempi).currentPrefix,
								model.data.get(tempi).prefixes);
					} else {
						if (str != null && !str.equals("null"))
							item.setText(j, str);
						else
							item.setText(j, "");
					}
				}
				changeNumbering(0);
			}
		});
	}
	
	public void updatePuckSelection() {
		counter = 0;
		for (int i = 0; i < NoofPucks; i++) {
			countCheck[i] = 0;
			countUnCheck[i] = 0;
		}

		GUI.asyncExec(new Runnable() {
			public void run() {

				for (int i = 0; i < model.getRowCount(); i++) {
					if (model.data.get(i).selected == 1) {
						final int pinscount = pinsPerPuck * NoofPucks;
						final int tempi = i;

						if (tempi < pinscount) {
							int puck = tempi / pinsPerPuck;
							if (puck < NoofPucks) {
								puckSelect[puck].setBackground(GUI.COLOR_THICK_GREEN);
								countCheck[puck]++;
							}
							if (countCheck[puck] == pinsPerPuck)
								puckSelect[puck].setSelection(true);
						}
					} else {
						final int pinscount = pinsPerPuck * NoofPucks;
						final int tempi = i;

						if (tempi < pinscount) {

							int puck = tempi / pinsPerPuck;
							countUnCheck[puck]++;

							if (countUnCheck[puck] == pinsPerPuck) {
								puckSelect[puck].setBackground(GUI.BGCOLOR);
							} else
								puckSelect[puck].setSelection(false);

						}
					}
				}
			}
		});
	}

	public void uncheckAll() {
		for (int i = 0; i < pinsPerPuck * NoofPucks; i++) {
			TableItem item = table.getItem(i);
			item.setChecked(false);
			item.setText("");
			item.setImage(1, null);
			crystalList.removeCrystalFromList(Integer.toString(i));
		}

		for (int i = 0; i < NoofPucks; i++) {
			clearPuckSelection(i);
		}
	}

	public void checkAll() {
		for (int i = 0; i < pinsPerPuck * NoofPucks; i++) {
			TableItem item = table.getItem(i);
			item.setChecked(true);
			if (Pucks.isPuckOwnerFromPinIndex(i)) {
				crystalList.addCrystalToList(Integer.toString(i));
			}
		}
	}

	public void clearColorCode(boolean[] clearPucks) throws IOException {
		System.out.println("Clearing the results: user pressed clear results button");
		for (int j = 0; j < NoofPucks; j++) {
			if (clearPucks[j]) {
				for (int i = j * pinsPerPuck; i < (j + 1) * pinsPerPuck; i++) {
					TableItem item = table.getItem(i);
					item.setImage(0, null);
					screenedCrystals[i] = '0';
					model.clearCrystalScreened(i, 0, "", true);
				}
			}
		}
		model.setAllCrystals();
	}

	public void setProgress() {
		GUI.asyncExec(new Runnable() {
			public void run() {
				try {
					String status = Epics.getString(PV_SCR.CurrentTask);
					int[] selected = taskList.getSelectionIndices();
					int inde = taskList.indexOf(status);
					if (status.equals("Unloaded") && AppState.isOperationAllowed(Operation.START_SCREENING)) {
						inde = 1;
					}
					int indexSelected = selected[0];
					/*
					 * if (inde < 0) {
					 * 
					 * taskList.setSelection(0,indexSelected); taskList.setTopIndex(indexSelected);
					 * } else {
					 */
					if (inde > 0) {
						taskList.setToolTipText(status);
						taskList.setSelection(0, inde);
						taskList.setTopIndex(inde);
					}
					// taskList.setFocus();
				} catch (Exception e) {

				}
			}
		});
	}

	public synchronized void changeNumbering(int previous) {
		if (crystalList.getNoofCrystals() < 1 ) {
			return;
		}

		if (!manualSelection) {
			for (int j = 0; j < crystalList.getNoofCrystals(); j++) {
				try {
					TableItem _item = table.getItem(Integer.parseInt(crystalList.getElement(j)));
					_item.setText(Integer.toString(j + 1));
				} catch (Exception e) {
					e.printStackTrace();
					Log.warn("Setting numbering in spreadsheet of screening tab, crystalList=" + crystalList.toString() + " size=" + crystalList.getNoofCrystals());
				}
			}
		} else {
			int i, j = 0;
			int startIndex = crystalList.getIndex(Integer.toString(ModelScreening.crystalStartIndex - previous));

			for (i = startIndex; i < crystalList.getNoofCrystals(); i++) {
				try {
					TableItem _item = table.getItem(Integer.parseInt(crystalList.getElement(i)));
					_item.setText(Integer.toString(j + 1));
					j++;
				} catch (Exception e) {

				}
			}
			for (i = 0; i < startIndex; i++) {
				try {

					TableItem _item = table.getItem(Integer.parseInt(crystalList.getElement(i)));
					_item.setText(Integer.toString(j + 1));
					j++;
				} catch (Exception e) {

				}
			}
		}
		resetMounted();
	}

	public void clearPuckSelection(int puck) {
		final int NoofPins = pinsPerPuck * puck;

		GUI.asyncExec(new Runnable() {
			public void run() {
				for (int i = NoofPins; i < NoofPins + pinsPerPuck; i++) {
					TableItem item = table.getItem(i);
					item.setText("");
					item.setImage(1, null);
				}
			}
		});

	}

	@Override
	public void masterChanged(boolean isMaster) {
		GUI.asyncExec(new Runnable() {
			public void run() {
				ModelScreening.screeningStatus = 0;
				updateScreeningState();
				updateWidgetState();
				DBConfig.update();
				if (editPucks != null) {
					// if an edit puck dialog is open, close it for the client
					// for synchronization purpose
					// TODO probably better to update the view upon change in the puck table
					editPucks.close();
				}

			}
		});
	}

	public static void startScreening() {

		String mou = Epics.getString(PV_ROBOT.mounted);

		/* Check if ateleast one sample selected */
		if (crystalList.getNoofCrystals() == 0 && mou.equals("none")) {
			NBMessageBox messageBox = new NBMessageBox(GUI.getShell(), SWT.OK | SWT.ICON_WARNING | SWT.CANCEL);
			messageBox.setMessage("Please select at least one sample");
			messageBox.open();
			return;
		}
		/* check cryojet temperature & flow */
		if (DevCollectSettings.getEnableCryojetMonitor()) {
			if (DevCryoJetMonitor.getInstance().getCryoJetFlow() <= 0.0) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						NBMessageBox messageBox = new NBMessageBox(GUI.getShell(), SWT.OK | SWT.ICON_WARNING);
						messageBox.setMessage("Cryojet flow shows as zero, Please report this to your host");
						messageBox.open();
					}
				});
			}
			if (DevCryoJetMonitor.getInstance().getCryoJetTemp() >= 115.0) {
				GUI.asyncExec(new Runnable() {
					public void run() {
						Log.error("Cryojet temperature >= 115.0");
						NBMessageBox messageBox = new NBMessageBox(GUI.getShell(), SWT.OK | SWT.ICON_WARNING);
						messageBox.setMessage("Cryojet temperature >= 115K, Please report this to your host");
						messageBox.open();
					}
				});
			}
		}
		screeningTasks.startScreening(crystalList);

	}

	public void showMounted(String mountedPin) {

		if (mountedPin.equals("none") || mountedPin.equals("")) {
			return;
		}
		if (!spreadsheetLoaded)
			return;
		final int pinIndex = CmdScreening.labelToindex(mountedPin);

		GUI.asyncExec(new Runnable() {
			public void run() {
				if (previousPin > 0) {
					TableItem previousItem = table.getItem(previousPin);
					previousItem.setBackground(null);
					previousItem.setImage(1, null);
				}
				if (pinIndex > 2)
					table.setTopIndex(pinIndex - 1);
				TableItem mountedItem = table.getItem(pinIndex);
				mountedItem.setBackground(GUI.COLOR_THICK_GREEN);
				if (crystalList.getNoofCrystals() > 0) {
					crystalList.removeCrystalFromList(Integer.toString(pinIndex));
					mountedItem.setChecked(false);
					mountedItem.setText("");
					mountedItem.setImage(1, null);
					if (Client.isMaster()) {
						setCrystal(pinIndex, true);
						setAllCrystalsFast();
					}
					changeNumbering(0);
					updatePuckSelection();
				}
				previousPin = pinIndex;
			}
		});
	}

	public void resetMounted() {
		String mou = Epics.getString(PV_ROBOT.mounted);
		if (mou == null || mou.length() == 0 || mou.equals("none")) {
			return;
		}
		int samMount = CmdScreening.labelToindex(mou);
		try {
			TableItem _item = table.getItem(samMount);
			_item.setText("");
		} catch (Exception e) {
			Log.error("Screening : Index out of bound exception resetMounted()");
		}
	}

	public void setCrystalList() {

		int[] selectedXs = new int[pinsPerPuck * NoofPucks];
		for (int i = 0; i < pinsPerPuck * NoofPucks; i++) {
			TableItem item_ = table.getItem(i);

			if (item_.getChecked() == true) {
				selectedXs[i] = 1;
				if (crystalList.isExist(i) == 1) {
					int ordered = Integer.parseInt(crystalList.getElement(i));
					model.setCrystalSelection(i, 1, ordered + 1, true);
				}
			} else {
				selectedXs[i] = 0;
				model.setCrystalSelection(i, 0, 0, true);
			}
		}
	}

	public void setCrystal(int crys, boolean update) {
		commandIssued = 1;

		int ordered = crystalList.getNoofCrystals();
		TableItem item_ = table.getItem(crys);

		if (item_.getChecked() == true)
			model.setCrystalSelection(crys, 1, ordered, update);
		else
			model.setCrystalSelection(crys, 0, 0, update);
	}

	public void setAllCrystalsFast() {
		model.setCrystalSelections(crystalList, false);
	}

	public void updateTaskProgress_(String sample_) {

		double ang1 = Epics.getDouble(PV_SCR.AngleImage0);
		double ang2 = Epics.getDouble(PV_SCR.AngleImage45);
		double ang3 = Epics.getDouble(PV_SCR.AngleImage90);

		int numang1 = Epics.getInteger(PV_SCR.diffimages0);
		int numang2 = Epics.getInteger(PV_SCR.diffimages45);
		int numang3 = Epics.getInteger(PV_SCR.diffimages90);

		int angle1 = (int) Math.ceil(ang1) + 1;
		int angle2 = (int) Math.ceil(ang2) + 1;
		int angle3 = (int) Math.ceil(ang3) + 1;
		String sample = sample_;

		final String prefix = sample_;
		int i;
		for (int j = 0; j < 8; j++) {
			if (screeningTasks.checkAction[j].getSelection()) {

				final String task = prefix.concat(" " + ActionNames[j]);

				if (j == 3) {
					if (WdgScreeningTasks.takeJpeg) {
						String jpegSnap = sample.concat(" JPEGsnapshot" + angle1);
						if (screeningTasks.checkAction[4].getSelection())
							taskList.add(jpegSnap);

						jpegSnap = sample.concat(" JPEGsnapshot" + angle2);
						if (screeningTasks.checkAction[5].getSelection())
							taskList.add(jpegSnap);

						jpegSnap = sample.concat(" JPEGsnapshot" + angle3);
						if (screeningTasks.checkAction[6].getSelection())
							taskList.add(jpegSnap);
					}
				} else if (j == 4) {
					// String rotate = sample.concat(" Rotate"+angle1);
					String collect1 = sample.concat(" Collect" + CmdScreening.getFrameNumber(0, ang1, 0));
					// taskList.add(rotate);
					taskList.add(collect1);

					for (i = 1; i < numang1; i++) {
						double tmp = angle1 + i;
						// rotate = sample.concat(" Rotate"+angle1);
						collect1 = sample.concat(" Collect" + CmdScreening.getFrameNumber(0, tmp, i));
						// taskList.add(rotate);
						taskList.add(collect1);
					}				
				} else if (j == 5) {

					//String rotate = sample.concat(" Rotate" + angle2);
					String collect2 = sample.concat(" Collect" + CmdScreening.getFrameNumber(1, ang2, 0));
					// taskList.add(rotate);
					taskList.add(collect2);
					for (i = 1; i < numang2; i++) {
						int tmp = angle2 + i;
						//rotate = sample.concat(" Rotate" + angl);
						collect2 = sample.concat(" Collect" + CmdScreening.getFrameNumber(1, tmp, i));
						// taskList.add(rotate);
						taskList.add(collect2);
					}
				} else if (j == 6) {
					//String rotate = sample.concat(" Rotate" + angle3);
					String collect3 = sample.concat(" Collect" + CmdScreening.getFrameNumber(2, ang3, 0));

					// taskList.add(rotate);
					taskList.add(collect3);

					for (i = 1; i < numang3; i++) {
						int tmp = angle3 + i;
						//rotate = sample.concat(" Rotate" + angl);
						collect3 = sample.concat(" Collect" + CmdScreening.getFrameNumber(2, tmp, i));
						// taskList.add(rotate);
						taskList.add(collect3);
					}
				} else {
					taskList.add(task);
				}
			}
		}
		setProgress();
	}

	public void updateTaskProgress() {

		GUI.asyncExec(new Runnable() {
			public void run() {
				taskList.removeAll();
				taskList.add("Task Progress");
				taskList.select(0);

				String Pin = Epics.getString(PV_ROBOT.mounted);
				if (!Pin.equals("none"))
					updateTaskProgress_(Pin);
				for (int i = 0; i < crystalList.getNoofCrystals(); i++) {
					prefix = (String) model.data.get(Integer.parseInt(crystalList.getElement(i))).col[1];
					final String sample = prefix;
					updateTaskProgress_(sample);
				}
			}
		});
	}

	private void updateComboPrefixColumn(int row, int callbackrow, TableItem item, String currentPrefix,
			ArrayList<String> prefixes) {

		if (callbackrow == -2 && prefixes.size() < 1) {

			CCombo combotemp = (CCombo) item.getData("COMBO");
			item.setData(null);
			if (combotemp != null) { // && !combotemp.isDisposed() need to check
				if (!combotemp.isDisposed()) {
					combotemp.removeAll();
					combotemp.setText("");
				}
				TableEditor editor = (TableEditor) item.getData("ComboEditor");
				editor.getEditor().dispose();
				item.setText(model.getColumnIndex("Set"), "");
			} else {
				item.setText(model.getColumnIndex("Set"), "");
			}
			return;
		}
		if (prefixes.size() < 1) {
			return;
		}
		if (prefixes.size() == 1) {

			item.setText(model.getColumnIndex("Set"), prefixes.get(0));
			return;
		}
		TableEditor editor = new TableEditor(table);
		CCombo combo = (CCombo) item.getData("COMBO");
		if (combo == null) {
			combo = new CCombo(table, SWT.BORDER);
			combo.setEditable(false);
			item.setData("COMBO", combo);
			item.setData("ComboEditor", editor);
		}
		combo.removeAll();
		final CCombo fCombo = combo;
		combo.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {

			}

			@Override
			public void widgetSelected(SelectionEvent arg0) {
				if (!model.data.get(row).currentPrefix.equals(fCombo.getText()) && !fCombo.getText().equals(""))
					model.updateRowResults((String) model.data.get(row).col[2], fCombo.getText());
			}

		});

		for (String pre : prefixes) {
			if (!pre.equals("null"))
				combo.add(pre);
		}
		combo.setText(currentPrefix);
		editor.grabHorizontal = true;
		editor.grabVertical = true;
		combo.clearSelection();
		editor.setEditor(combo, item, model.getColumnIndex("Set"));
	}

	private void setColorCode(String Sample) {

		String sampleName = Epics.getString(PV_SCR.prefix);
		int sampleIndex = CmdScreening.labelToindex(Sample);

		int row = sampleIndex;
		int group = CmdScreening.group;

		// when group is 0 it will be processed but won't be stored in SIL
		if (CmdScreening.group == 0)
			group = 1;

		String dir = CmdScreening.imageDirectory;
		String img1 = CmdScreening.image1;
		String img2 = CmdScreening.image2;
		String img3 = CmdScreening.image3;

		String analysisImg1 = "";
		String analysisImg2 = "";

		String snapImg1 = "";
		String snapImg2 = "";
		boolean oneImage = false;
		boolean noImage = false;
		if (img1.equals("") && img2.equals("") && img3.equals("")) {
			screenedCrystals[sampleIndex] = 2;
			Log.warn("No images collected for the sample " + Sample);
			oneImage = true;
			noImage = true;
		} else if (!img1.equals("") && img2.equals("") && img3.equals("")) {

			screenedCrystals[sampleIndex] = 2;
			Log.warn("Only one image collected, At least two images needed for autoindexing " + Sample);
			oneImage = true;
		} else if (img1.equals("") && !img2.equals("") && img3.equals("")) {

			screenedCrystals[sampleIndex] = 2;
			Log.warn("Only one image collected, At least two images needed for autoindexing " + Sample);
			oneImage = true;
		} else if (img1.equals("") && img2.equals("") && !img3.equals("")) {

			screenedCrystals[sampleIndex] = 2;
			Log.warn("Only one image collected, At least two images needed for autoindexing " + Sample);
			oneImage = true;
		} else if (!img1.equals("") && !img2.equals("") && img3.equals("")) {

			screenedCrystals[sampleIndex] = 3;
			analysisImg1 = img1;
			analysisImg2 = img2;
			snapImg1 = CmdScreening.snapImage1;
			snapImg2 = CmdScreening.snapImage2;
		} else if (!img1.equals("") && img2.equals("") && !img3.equals("")) {

			screenedCrystals[sampleIndex] = 3;
			analysisImg1 = img1;
			analysisImg2 = img3;
			snapImg1 = CmdScreening.snapImage1;
			snapImg2 = CmdScreening.snapImage3;

		} else if (img1.equals("") && !img2.equals("") && !img3.equals("")) {
			screenedCrystals[sampleIndex] = 3;
			analysisImg1 = img2;
			analysisImg2 = img3;
			snapImg1 = CmdScreening.snapImage2;
			snapImg2 = CmdScreening.snapImage3;
		} else if (!img1.equals("") && !img2.equals("") && !img3.equals("")) {

			screenedCrystals[sampleIndex] = 3;
			analysisImg1 = img1;
			analysisImg2 = img2;
			snapImg1 = CmdScreening.snapImage1;
			snapImg2 = CmdScreening.snapImage3;
		} else {
			screenedCrystals[sampleIndex] = 0;
			return;
		}
		final int indexScreened = screenedCrystals[sampleIndex];
		final String Sample_final = Sample;
		final int currentSample1 = CmdScreening.labelToindex(Sample_final);
		GUI.asyncExec(new Runnable() {
			public void run() {
				currentItem = table.getItem(currentSample);

				System.out.println("setScreened " + indexScreened);
				if (indexScreened == 0)
					currentItem.setImage(0, null);
				else if (indexScreened == 1)
					currentItem.setImage(0, GUI.imagered);
				else if (indexScreened == 2)
					currentItem.setImage(0, GUI.imageblue);
				else if (indexScreened == 3)
					currentItem.setImage(0, GUI.imagegreen);
			}
		});
		// Log.error(sampleName);
		if (indexScreened == 3)
			model.setCrystalScreened(currentSample1, indexScreened, ModelScreening.currentPrefix, true);
		else
			model.setCrystalScreened(currentSample1, indexScreened, "", true);
		
		// TODO test new pipeline hookup
		Integer bgProcMode = Epics.getInteger(PV_RUNS.bgProc);
		if (Client.isMaster() && !noImage && bgProcMode != 0) {
			Log.note("start new screening analysis job for " + ModelScreening.currentPrefix + ", dir= " + dir);
			autoProcessData p = new autoProcessData(dir, Util.getBID(), ModelScreening.currentPrefix,
					String.valueOf(sampleIndex), ModelScreening.currentPrefix, 0, null,
					autoProcessData.PROCESSING_MODE.AFTER_SCREENING);
			new Thread(p).start();
		}
		// end of test new pipeline hookup

		/* Exit if only one or no images collected */
		if (oneImage) {
			oneImage = false;
			return;
		}
		/*
		 * GUI.asyncExec(new Runnable() { public void run() { currentItem =
		 * table.getItem(currentSample);
		 * 
		 * updateComboPrefixColumn(currentItem, ModelScreening.currentPrefix); } });
		 */

		// TODO: instead of getting the frame label from the HDF5 filename, move the
		// CmdStrategy call to a place where the frame label is available
		// Log.note("Debug: analysisImg1= "+ analysisImg1);
		// Log.note("Debug: analysisImg2= "+ analysisImg2);
		String cbfImg1;
		String cbfImg2;
		if (DetectorConfig.isEiger() && analysisImg1.endsWith(".h5")) {
			cbfImg1 = getCBFFilenameFromMasterFileName(analysisImg1);
			cbfImg2 = getCBFFilenameFromMasterFileName(analysisImg2);
		} else {
			cbfImg1 = analysisImg1;
			cbfImg2 = analysisImg2;
		}

		// old pipeline
		CmdCollectStrategy cmdStrategy = new CmdCollectStrategy(dir, analysisImg1, analysisImg2, cbfImg1, cbfImg2);
		cmdStrategy.setScreening(true, sampleIndex, ModelScreening.currentPrefix);
		cmdStrategy.start();
	}

	private static String getCBFFilenameFromMasterFileName(String filename) {
		Matcher matcher = Filenames.HDF5MasterFilePattern.matcher(filename);
		if (matcher.matches()) {
			String prefix = matcher.group(1);
			int frameLabel = Integer.parseInt(matcher.group(3));
			return String.format("%s_0_scr_%06d.cbf", prefix, frameLabel);
		}
		return "";
	}

	private void loadSpreadsheet() {

		String spreadsheetFile = Epics.getString(PV_SCR.spreadsheetFile);

		if (!spreadsheetFile.equals("")) {

			spreadsheetName = Epics.getString(PV_SCR.spreadsheetDir256) + spreadsheetFile;
			try {
				File spreadsheet = new File(spreadsheetName);
				if (!spreadsheet.canRead()) {
					Log.warn("Spreadsheet file not readable " + spreadsheetName);
					return;
				}

				try {
					model.loadSilFromMysql();
				} catch (Exception e) {

					e.printStackTrace();
				}

				if (model.getRowCount() > 0)
					spreadsheetLoaded = true;

				int rows = pinsPerPuck * NoofPucks;

				if (model.getRowCount() != rows) {
					if (Client.isMaster()) {
						displayMessageBox("Puck type mismatch ALS/Rigaku");
					}
					return;
				}

				updateTable();
				spreadsheetText.setText(spreadsheetName);
				spreadsheetText.setSelection(spreadsheetName.length());
				changeNumbering(0);

				final String mounted = Epics.getString(PV_ROBOT.mounted);

				showMounted(mounted);

			} catch (Exception e) {

				e.printStackTrace();
			}
		}
	}

	public void handleBrowseButton(String fileSpreadsheet) {

		String spreadsheetSuccess = "";
		if (fileSpreadsheet == null)
			return;

		File sheetFile = new File(fileSpreadsheet);
		if (!sheetFile.canRead()) {
			displayMessageBox("Screening directory" + fileSpreadsheet + " don't have read permissions");
			return;
		}
		try {
			spreadsheetSuccess = model.loadSilFromSpreadsheet(fileSpreadsheet, 0);
		} catch (Exception e) {
			e.printStackTrace();
			Log.note("Spreadsheet load failed : " + e.getMessage());
			new WdgDlgMessage(GUI.getShell(), SWT.NONE, e.getMessage()).open(1);

			return;
		}
		if (spreadsheetSuccess == "1") {
			displayMessageBox(
					"This spreadsheet is not compatible Compare your spreadsheet with the template available at \n http://www.gmca.anl.gov/facilities/automounter.html#instructions");
			return;
		} else if (spreadsheetSuccess == "2") {
			return;
		}

		int rows = pinsPerPuck * NoofPucks;

		if (model.getRowCount() != rows) {
			Log.error("Puck type mismatch ALS/Rigaku or Spreadsheet not compatible \n"
					+ "\n See Instructions http://www.gmca.anl.gov/facilities/automounter.html#instructions");
			// GUI.asyncExec(new Runnable(){public void run() {
			displayMessageBox("Puck type mismatch ALS/Rigaku or Spreadsheet not compatible \n"
					+ "\n See Instructions http://www.gmca.anl.gov/facilities/automounter.html#instructions");
			// }});
			return;
		}
		spreadsheetText.setText(fileSpreadsheet);
		puttoSpreadsheetFile(spreadsheetText.getText());
		Epics.putInteger(PV_SCR.spreadsheetStatus, 0);
		Epics.putInteger(PV_SCR.spreadsheetStatus, 1);
		updateTable();
		Log.note("Spreadsheet loaded");
		updateTaskProgress();
		final String mounted = Epics.getString(PV_ROBOT.mounted);
		showMounted(mounted);
	}

	public String selectspreadsheetFile() {
		dialog = new NBFileDialog(GUI.getShell(), SWT.OPEN);
		String path1 = System.getProperty("user.home");
		dialog.setFilterPath(path1);
		dialog.setFilterExtensions(new String[] { "*.xls", "*.xlsx", "*.*" });
		String path = dialog.openStr();
		return path;
	}

	public void putR0Samp(String localSamp) {
		if (localSamp == null || localSamp.equals("") || localSamp.equals(" ")) {
			System.out.println("Screening debug: sample " + localSamp);
			return;
		}
		try {
			localSamp = ValidateDirPrefix.validateFileName(localSamp);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println("Screening debug: sample " + localSamp);
		// screening
		Epics.put(PV_SCR_RUN.fileroot.run(0), prefix);

		// collect run 0
		DevCollectRun run0 = (DevCollectRun) DevCollectRunList.getIndex(0);
		run0.setFileroot(localSamp, false);

		// scan
		DevScan.get().setPrefix(localSamp + "_scan", false);

		// raster run 0s
		System.out.println("Screening debug: sample " + localSamp);
		((DevRasterRun) DevRasterRunList.get(RasterListType.INTER).getRunDev(0)).setFileroot(localSamp + "_raster",
				false);
		((DevRasterRun) DevRasterRunList.get(RasterListType.AUTO).getRunDev(0)).setFileroot(localSamp + "_raster",
				false);
	}

	public void putR0Dir(String R0Dir) {
		if (R0Dir.equals("") || R0Dir == null || R0Dir.equals(" "))
			return;
		try {
			R0Dir = ValidateDirPrefix.validateDir(R0Dir);
		} catch (Exception e) {
			e.printStackTrace();
		}

		// screening
		Epics.put(PV_SCR_RUN.directory256.run(0), R0Dir + "/screen");

		// collect run 0
		DevCollectRun run0 = (DevCollectRun) DevCollectRunList.getIndex(0);
		run0.setDirectorySuffix(R0Dir + "/collect", false);

		// scan
		DevScan.get().setDirectory(R0Dir + "/scan", false);

		// raster run 0s
		((DevRasterRun) DevRasterRunList.get(RasterListType.INTER).getRunDev(0)).setDirectory(R0Dir + "/raster", false);
		((DevRasterRun) DevRasterRunList.get(RasterListType.AUTO).getRunDev(0)).setDirectory(R0Dir + "/raster", false);
	}

	public void displayMessageBox(String Message) {
		NBMessageBox messageBox = new NBMessageBox(GUI.getShell(), SWT.OK | SWT.ICON_WARNING);
		messageBox.setMessage(Message);
		messageBox.open();
	}

	public void saveExcelFile() {

		String str_ = spreadsheetText.getText();
		if (str_ == null || str_.equals("")) {
			Log.error("Spreadsheet file name returned empty");
			return;
		}
		String saveDir = DevDataDir.getPrimary();
		File dirFile = new File(saveDir);
		/* create directory if does's exist */
		if (!dirFile.exists()) {
			if (!dirFile.mkdirs()) {
				Log.error("ERROR: Couldn't create directory " + saveDir);
				return;
			}
		}

		if (!dirFile.canWrite()) {
			Log.error("Spreadsheet file is not auto-saved : " + saveDir + " not writable");
			return;
		}

		String saved_file = str_.substring(str_.lastIndexOf('/') + 1, str_.length());
		saved_file = saved_file.substring(0, saved_file.lastIndexOf('.'));
		try {
			model.downloadSil(saveDir + "/" + saved_file + "_saved.xls");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void exportSpreadsheet() {

		NBFileDialog dialog = new NBFileDialog(GUI.getShell(), SWT.SAVE);
		dialog.setFilterNames(new String[] { "Excel Files", "All Files (*.*)" });
		dialog.setFilterExtensions(new String[] { "*.xls" });
		dialog.setFileName("results.xls");

		String savePath = "";

		savePath = DevDataDir.getPrimary();
		if (savePath.equals("") || savePath == null) {
			savePath = System.getProperty("user.home");
		}

		dialog.setFilterPath(savePath);

		try {
			String fileName = dialog.openStr();
			Log.note("Saving spreadsheet to " + fileName);
			model.downloadSil(fileName);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void puttoSpreadsheetFile(String File) {

		if (File.equals("") || File == null || File.equals(" "))
			return;

		int slash = File.lastIndexOf('/');
		if (slash < 1)
			return;

		String fileName = File.substring(slash + 1, File.length());

		String Directory = File.substring(0, slash + 1);

		if (!fileName.equals(""))
			Epics.put(PV_SCR.spreadsheetFile, fileName);

		if (!Directory.equals(""))
			Epics.put(PV_SCR.spreadsheetDir256, Directory);

		WdgScreeningTasks.updateRuns();
	}

	public void updateScreeningState() {
		if (Client.isMaster()) {
			int status = Epics.getInteger(PV_SCR.ScreeningStatus);
			if (status == 1)
				Epics.putInteger(PV_SCR.ScreeningStatus, 0);
		}
	}

	public void updateWidgetState() {
		boolean isMaster = Client.isMaster();
		boolean isScreening = Client.isMaster() && AppState.isOperationAllowed(Operation.START_SCREENING);
		editButton.setEnabled(isMaster);

		screeningTasks.updateButtons();
		exportButton.setEnabled(isScreening);
		browseButton.setEnabled(isScreening);
		spreadsheetText.setEnabled(isScreening);
		btnMoreTools.setEnabled(true);
		//clearButton.setEnabled(isScreening);
		//clearColorButton.setEnabled(isScreening);


		String[] puckNames = Pucks.getPuckSelectLabels();
		for (int k = 0; k < NoofPucks; k++) {
			puckSelect[k].setToolTipText(puckNames[k]);
			puckSelect[k].setEnabled(isScreening);
			if (Pucks.isPuckOwnerFromPuckIndex(k) && Client.isMaster()) {
				puckSelect[k].setEnabled(true);
			} else {
				puckSelect[k].setEnabled(false);
			}
		}
	}
	
	private void enableEdit() {
		/* create a TableCursor to navigate around the table */
		final ControlEditor editor = new ControlEditor(cursor);
		editor.grabHorizontal = true;
		editor.grabVertical = true;
		cursor.setVisible(false);
		cursor.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (editButton.getSelection()) {
					TableItem row = cursor.getRow();
					int rowIndex = table.indexOf(row);

					if (!Pucks.isPuckOwnerFromPinIndex(rowIndex)) {
						Log.warn("you cannot edit this row since ownership check failed.");
						return;
					}

					int column = cursor.getColumn();
					//Log.note("row x column selected = " +  rowIndex + " " + column);

					if (isWebice) {
						if (column < 2 || column > 4) {
							// cursor.setVisible(false);
							return;
						}
					} else {
						if (column < 2) {
							// cursor.setVisible(false);
							return;
						}
					}

					//Log.note("xx = " +  Arrays.asList(editableColumns).contains(column));
					if (Arrays.asList(editableColumns).contains(column) ) {
						Log.note("row x column selected == " +  rowIndex + " " + column);
					
						cursor.setVisible(true);
						final Text text = new Text(cursor, SWT.BORDER);
						text.setText(row.getText(column));
	
						text.addFocusListener(new FocusListener() {
	
							@Override
							public void focusGained(FocusEvent arg0) {
								int column = cursor.getColumn();
								if (column < 2) {
									text.dispose();
								}
							}
	
							public void focusLost(FocusEvent arg0) {
								setEditorValue(text);
							}
						});
						text.addKeyListener(new KeyAdapter() {
	
							@Override
							public void keyPressed(KeyEvent arg0) {
	
								if (arg0.character == SWT.CR) {
									setEditorValue(text);
								}
							}
	
							@Override
							public void keyReleased(KeyEvent arg0) {
							}
	
						});
						text.addTraverseListener(new TraverseListener() {
							@Override
							public void keyTraversed(TraverseEvent te) {
								if (SWT.TRAVERSE_RETURN == te.detail) {
									te.doit = false;
								} else {
									setEditorValue(text);
								}
							}
						});
						editor.setEditor(text);
						text.setFocus();
					}
				}
			}

		});
		
	}
	
	public String getTabName() {
		return "Screening";
	}

}// class end
