/**
 * @author spothineni
 * 
 * handles Start/Pause buttons
 * 
 */

package gov.anl.gmca.screening.ui;

import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.Epics;
import gov.anl.gmca.io.PVLookup.PV_ROBOT;
import gov.anl.gmca.io.PVLookup.PV_SCR;
import gov.anl.gmca.screening.CmdScreening;
import gov.anl.gmca.screening.ModelScreening;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.aps.jca.dbr.DBR;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

public class WdgScreeningButtons implements ClientListener {
	public Button startButton, resumeButton;
	Epics.EpicsValueMonitor monitor = null;

	public WdgScreeningButtons(Composite parent, int style) {

		Client.addListener(this);
		GridData gd;
		Group screenButtonGroup = new Group(parent, SWT.NONE);
		GridLayout gs = new GridLayout(2, false);
		screenButtonGroup.setBackground(GUI.BGCOLOR);
		screenButtonGroup.setLayout(gs);
		gd = new GridData();
		gd.horizontalAlignment = GridData.CENTER;
		gd.horizontalSpan = 4;
		gd.verticalIndent = 10;

		screenButtonGroup.setLayoutData(gd);

		gd = new GridData();
		gd.widthHint = 95;
		resumeButton = new Button(screenButtonGroup, SWT.NONE);
		resumeButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		resumeButton.setFont(GUI.FONT_HUTCH_BUTTON);
		resumeButton.setText("Resume");
		resumeButton.setLocation(125, 20);
		resumeButton.setSize(95, 40);
		resumeButton.setEnabled(false);
		resumeButton.setLayoutData(gd);
		resumeButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}

			public void widgetSelected(SelectionEvent arg0) {

				System.out
						.println("Resume button clicked by user on Mini Screening tab");
				if (CmdScreening.manualRepeatFlag > 0) {
					if (WdgScreeningTasks.incrementGroup && ModelScreening.preIncrement)
						CmdScreening.group++;
					WdgScreeningTasks.incrementGroup = true;
				} else {

				}
				if (!CmdScreening.stopThread) {
					Epics.putInteger(PV_SCR.ScreeningStatus, 1);
					if (null == WdgScreeningTasks.screeningThread
							|| !WdgScreeningTasks.screeningThread.isAlive())
						TabScreening.startScreening();
				} else
					TabScreening.startScreening();
			}
		});

		AppState.addListener(AppState.Operation.START_SCREENING,
				new AppState.Listener() {
					@Override
					public void stateChanged(Operation op, boolean allowed) {

						GUI.asyncExec(new Runnable() {
							public void run() {
								updateButtons();
							}
						});
					}
				});

		// TODO: (optional) this object can be broken up in to individual
		// monitors for each PV specified below -MH
		monitor = new Epics.EpicsValueMonitor() {
			@Override
			public void valueChanged(String pvName, DBR value) {
				final String _pvName = pvName;
				final DBR _value = value;
				String status;
				if (_pvName.equals(PV_ROBOT.pause.toString())) {
					GUI.asyncExec(new Runnable() {
						public void run() {
							updateButtons();
						}
					});
				}

				if (_pvName.equals(PV_SCR.ScreeningStatus.toString())) {

					GUI.asyncExec(new Runnable() {
						public void run() {
							updateButtons();
						}
					});
				}
			}
		};
		Epics.addMonitor(PV_ROBOT.pause, monitor);
		Epics.addMonitor(PV_SCR.ScreeningStatus, monitor);
		updateButtons();
	}

	public void updateButtons() {

		if (Client.isMaster()
				&& AppState.isOperationAllowed(Operation.START_SCREENING)) {
			int statusfinal = Epics.getInteger(PV_SCR.ScreeningStatus);
			if (statusfinal == 2) {
				// if(ScreeningActions.screeningStatus == 1)
				resumeButton.setEnabled(true);
				// else
				// resumeButton.setEnabled(false);
			} else if (statusfinal == 1) {

				resumeButton.setEnabled(false);
			} else if (statusfinal == 0) {

				resumeButton.setEnabled(false);
			}
		} else {
			resumeButton.setEnabled(false);
		}
	}

	@Override
	public void masterChanged(boolean isMaster) {
		// TODO Auto-generated method stub
		GUI.asyncExec(new Runnable() {
			public void run() {
				updateButtons();
			}
		});
	}
}
