package gov.anl.gmca.server;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.io.*;
import org.apache.commons.cli.*;
import gov.anl.gmca.util.Util;

public class Main
{
	public static CommandLine commandLine;

	public static void main(String[] args)
	{
		DBConfig.serverMode = true;
		parseArgs(args);

		if (commandLine.hasOption("sim")) {
			System.out.println("Starting simulated beamline");
			SimBeamline.start();
		}

		System.out.println("Starting JBluIce server");
		gov.anl.gmca.gui.Main.verify_tempdir();

		System.out.println("Connecting EPICS");
		Epics.connect();

		System.out.println("Loading configuration");
		DBConfig.connectListener();
		DBConfig.update();

		System.out.println("Initializing database");
		Database.init();
		
		System.out.println("Creating devices");
		DeviceInit.init();

		System.out.println("Entering main loop");
		mainLoop();
	}

	public static void parseArgs(String[] args)
	{
		Options options = new Options();
		options.addOption("sim", false, "Start and use simulated beamline");
		options.addOption("console", false, "Write log to console");

		CommandLineParser parser = new BasicParser();
		try {
			commandLine = parser.parse(options, args);
		} catch (ParseException e) { e.printStackTrace(); }
	}

	public static void mainLoop()
	{
		HTTP.init();
		while (true)
		{
			try { Thread.sleep(100); } catch (Exception e) { e.printStackTrace(); }
		}
	}
}
