package gov.anl.gmca.spotfinder;

import gov.anl.gmca.collect.HeaderCheck;
import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.Redis;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_OUTPUT;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_TYPE;
import gov.anl.gmca.spotfinder.SFData.SFType;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.atomic.AtomicBoolean;

public abstract class CmdSpotFinder implements Runnable
{
	public static final int TIMEOUT_DEFAULT_MS = 30000;

	// parameters
	protected SFType sfType;
	protected SFRunType sfRunType;
	protected double lowLimit_A, highLimit_A;
	protected String directory, prefix;
	protected int timeoutMs;
	protected ArrayList<String> imageNames;
	protected ArrayList<HeaderCheck.Values> headerValueList;
	protected int runIndex, imageIndex, runLabel;
	protected boolean isLocal;
	protected int retriesLeft;
	protected SPOTFINDER_TYPE sf_type;
	public boolean saveToRedis = true;
	
	// results
	ArrayList<String> cmdOutput = new ArrayList<>();
	public SFData.SFImageData results = null;
	boolean sfFailed = false;
	
	// per-thread stop flag
	protected AtomicBoolean stopFlag = new AtomicBoolean(false);
	protected AtomicBoolean doneFlag = new AtomicBoolean(false);
	
	public void requestStop()
	{
		System.out.println("Stopping spotfinder thread for batch starting with image "+imageNames.get(0));
		stopFlag.set(true);
	}
	
	public void waitFor()
	{
		while (!doneFlag.get()) Util.sleep(50);
	}

	///////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////// run method ////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////
	
	@Override public void run()
	{
		try
		{
			process();
		}
		catch (Exception e) { e.printStackTrace(); }
		finally
		{
			doneFlag.set(true);
			synchronized(SpotFinderQueue.cmdList)
			{
				SpotFinderQueue.cmdList.remove(this);
			}
		}
	}
	
	private void process()
	{
		if (stopFlag.get()) return;
		
		if (!init()) return;

		if (stopFlag.get()) return;

		runSpotFinder(); // this is overridden by the local or remote class
	}

	///////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////// interface //////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////
	
	abstract protected void runSpotFinder();
	
	///////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////// main methods ////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////
	
	protected CmdSpotFinder(SFType sfType, SFRunType sfRunType, int runIndex, int imageIndex, String dir, ArrayList<String> imgNames, String prefix, int runLabel, double lowLimit_A, double highLimit_A, int retries, SPOTFINDER_TYPE sf_type, ArrayList<HeaderCheck.Values> headerValueList, int timeoutMs)
	{
		if (-1==retries) retries = 3;

		this.sfType			= sfType;
		this.sfRunType		= sfRunType;
		this.runIndex		= runIndex;
		this.imageIndex 	= imageIndex;
		this.lowLimit_A 	= lowLimit_A;
		this.highLimit_A 	= highLimit_A;
		this.directory 		= dir;
		this.imageNames		= imgNames;
		this.prefix			= prefix;
		this.runLabel		= runLabel;
		this.retriesLeft	= retries;
		this.sf_type		= sf_type;
		this.headerValueList = headerValueList;
		this.timeoutMs		= timeoutMs;
	}
	
	// returns false if the command should exit
	protected boolean init()
	{
		if (stopFlag.get()) return false;
		return true;
	}
}
