package gov.anl.gmca.spotfinder;

import gov.anl.gmca.common.Redis;
import gov.anl.gmca.spotfinder.CmdSpotFinderRemote.*;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.apache.commons.lang3.ArrayUtils;
import redis.clients.jedis.Jedis;

import java.nio.ByteBuffer;
import java.nio.FloatBuffer;

public class DBImageStats
{
    public static ListenerList<DBImageStats> llAnyChange = new ListenerList<>(null);

    public static float max(FloatBuffer data, float ifEmpty)
    {
        float ret = ifEmpty;
        for (int i=0;i<data.capacity();i++) {
            if (data.get(i)>ret) ret=data.get(i);
        }
        return ret;
    }

    public static boolean isSame(FloatBuffer a, FloatBuffer b)
    {
        if (null==a&&null==b) return true;
        if (null!=a&&null==b) return false;
        if (null==a&&null!=b) return false;
        if (a.capacity()!=b.capacity()) return false;
        for (int i=0;i<a.capacity();i++) {
            if (a.get(i)!=b.get(i)) return false;
        }
        return true;
    }

    public static synchronized void delete(String relDir, String prefix, int runLabel, String output)
    {
        String key = Redis.image_stats(relDir, prefix, runLabel, output);
        System.out.println("Deleting key "+key);
        try (Jedis jedis = Redis.get()) {
            jedis.del(key);
        }
    }

    // set all spot finder output array values to -1 and result values to NOT_STARTED
    //
    // key should not include the output string
    //
    public static synchronized void initialize(String key, int elementCount)
    {
        float[] init_data = new float[elementCount], init_result = new float[elementCount];
        for (int i=0;i<elementCount;i++) { init_data[i]=-1; init_result[i] = RESULT.NOT_STARTED.ordinal(); }
        for (DevSFType.SPOTFINDER_OUTPUT output : DevSFType.SPOTFINDER_OUTPUT.values()) {
            DBImageStats.save(key+output.toString(), init_data);
            System.out.println("Initialized "+key+output.toString()+" to -1");
        }
        for (String sf : new String[]{"dials","dozor"}) {
            DBImageStats.save(key+sf+":result", init_result);
            System.out.println("Initialized "+key+sf+":result to "+init_result[0]);
        }
    }

    public static synchronized void save(String key, int index, float data)
    {
        try (Jedis jedis = Redis.get()) {
            jedis.setrange(key.getBytes(), index * 4, ByteBuffer.allocate(4).putFloat(data).array());
        }
    }

    public static synchronized void save(String key, float[] data)
    {
        ByteBuffer byteBuf = ByteBuffer.allocate(data.length*4);
        FloatBuffer floatBuf = byteBuf.asFloatBuffer();
        floatBuf.put(data);
        try (Jedis jedis = Redis.get()) {
            jedis.set(key.getBytes(), byteBuf.array());
        }
    }

    public static synchronized FloatBuffer load(String key)
    {
        try (Jedis jedis = Redis.get()) {
            byte[] returnVal = jedis.get(key.getBytes());
            if (null!=returnVal) {
                ByteBuffer returnByteBuffer = ByteBuffer.wrap(returnVal);
                FloatBuffer returnData = returnByteBuffer.asFloatBuffer();
                return returnData;
            } else {
                jedis.set(key,"");
                return ByteBuffer.wrap(new byte[]{}).asFloatBuffer();
            }
        }
    }
}
