package gov.anl.gmca.spotfinder;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.Multimap;
import gov.anl.gmca.common.ui.WdgControl;
import gov.anl.gmca.common.ui.WdgSpacer;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.spotfinder.DevSFOptions.SPOTFINDER_OPTION;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_TYPE;
import gov.anl.gmca.util.ListenerList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.HashMap;

public class DlgSFOptions
{
	private static DlgSFOptions collectInst=null, rasterInst=null;
	public static DlgSFOptions get(SFRunType sfRunType)
	{
		switch (sfRunType)
		{
			case COLLECT:
				if (null == collectInst) collectInst = new DlgSFOptions(SFRunType.COLLECT);
				return collectInst;
			case RASTER:
				if (null == rasterInst) rasterInst = new DlgSFOptions(SFRunType.RASTER);
				return rasterInst;
			default:
				return null;
		}
	}

	private Shell shell;
	private Composite headerComp, divider, divider2;
	private boolean inModifyEvent = false;
	public Shell getShell() { return shell; }

	private SFRunType sfRunType;
	private HashMap<SPOTFINDER_OPTION,Text> fieldMap = new HashMap<>();
	private Multimap<SPOTFINDER_TYPE,Composite> compMap = ArrayListMultimap.create();
	public HashMap<SPOTFINDER_OPTION,String> currentValues = new HashMap<>();
	private HashMap<SPOTFINDER_TYPE,Boolean> hasOptions = new HashMap<>();

	public DlgSFOptions(SFRunType sfRunType)
	{
		this.sfRunType = sfRunType;
		loadFromDevice(false);
	}

	private void loadFromDevice(boolean sendToGUI)
	{
		HashMap<String,String> devValues = DevSFOptions.getValues(sfRunType);
		for (SPOTFINDER_OPTION option : DevSFOptions.options) {
			if (devValues.containsKey(option.getUrl_key())) {
				currentValues.put(option, devValues.get(option.getUrl_key()));
			} else {
				currentValues.put(option, "");
			}
		}
		if (sendToGUI) sendMapToGUI();
	}

	private void sendMapToGUI()
	{
		GUI.asyncExec(()->{
			inModifyEvent=true;
			try {
				for (SPOTFINDER_OPTION option : DevSFOptions.options) {
					fieldMap.get(option).setText(currentValues.get(option));
				}
			} catch (Exception e) { e.printStackTrace(); }
			finally { inModifyEvent = false; }
		});
	}
	
	private void createWidgets()
	{
		fieldMap = new HashMap<>();
		compMap = ArrayListMultimap.create();

		GridLayout gl = new GridLayout(1,false);
		gl.marginHeight=gl.marginWidth=0;
		gl.verticalSpacing=gl.horizontalSpacing=0;
		gl.marginLeft=0;
		shell.setLayout(gl);

		// table headers
		headerComp = new Composite(shell, SWT.NONE);
		headerComp.setBackground(GUI.COLOR_LIGHT_GRAY);
		gl = new GridLayout(3,false);
		headerComp.setLayout(gl);

		Label l = new Label(headerComp, SWT.CENTER);
		l.setFont(GUI.FONT_12);
		l.setText("Parameter");
		GridData gd = new GridData();
		gd.widthHint = 270;
		l.setLayoutData(gd);

		l = new Label(headerComp, SWT.CENTER);
		l.setFont(GUI.FONT_12);
		l.setText("Value");
		gd = new GridData();
		gd.widthHint = 100;
		l.setLayoutData(gd);

		l = new Label(headerComp, SWT.CENTER);
		l.setFont(GUI.FONT_12);
		l.setText("Description");
		gd = new GridData();
		gd.widthHint = 420;
		l.setLayoutData(gd);

		divider = new Composite(shell, SWT.NONE);
		divider.setBackground(GUI.COLOR_DARK_GRAY);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint=1;
		divider.setLayoutData(gd);

		// table contents
		int row=0;
		for (SPOTFINDER_OPTION option : DevSFOptions.options)
		{
			Composite comp = new Composite(shell, SWT.NONE);
			if (row++%2==0) comp.setBackground(GUI.COLOR_LIGHT_GRAY2); else comp.setBackground(GUI.COLOR_LIGHT_GRAY);
			gl = new GridLayout(3,false);
			//gl.marginHeight=gl.marginWidth=0;
			comp.setLayout(gl);
			compMap.put(SPOTFINDER_TYPE.fromString(option.getSpotfinder_id()), comp);

			l = new Label(comp, SWT.LEFT);
			l.setText(option.getUrl_key());
			gd = new GridData();
			gd.widthHint = 260;
			l.setLayoutData(gd);

			Text t = new Text(comp, SWT.BORDER);
			t.setTextLimit(40);
			if (currentValues.containsKey(option)) t.setText(currentValues.get(option));
			gd = new GridData();
			gd.widthHint = 100;
			t.setLayoutData(gd);
			fieldMap.put(option, t);
			t.addModifyListener((modifyEvent)-> { if (!inModifyEvent) saveGUIValues();});

			String helpText = option.getHelp_text();
			Label h = new Label(comp, SWT.LEFT|SWT.WRAP);
			if (null!=helpText) {
				helpText = helpText.replace("\\n","\n");
				h.setText(helpText);
			}
			gd = new GridData();
			gd.widthHint = 420;
			h.setLayoutData(gd);
		}

		divider2 = new Composite(shell, SWT.NONE);
		divider2.setBackground(GUI.COLOR_DARK_GRAY);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
		gd.heightHint=1;
		divider2.setLayoutData(gd);

		// Add a row with a note for any SF types with no options
		for (SPOTFINDER_TYPE sfType : SPOTFINDER_TYPE.values())
		{
			boolean hasOption = false;
			for (SPOTFINDER_OPTION option : DevSFOptions.options)
			{
				if (option.getSpotfinder_id().equalsIgnoreCase(sfType.name()))
				{
					hasOption = true;
				}
			}
			if (!hasOption)
			{
				Composite noteComp = new Composite(shell, SWT.NONE);
				gl = new GridLayout();
				gl.marginWidth=gl.marginHeight=10;
				noteComp.setLayout(gl);

				l = new Label(noteComp, SWT.LEFT);
				l.setFont(GUI.FONT_12);
				l.setText(sfType.name()+" has no options configurable in JBluIce.");
				compMap.put(sfType, noteComp);
			}
			hasOptions.put(sfType,hasOption);
		}

		new WdgSpacer(shell, SWT.VERTICAL, 1, 1);
		
		Composite buttonComp = new Composite(shell, SWT.NONE);
		gl = new GridLayout(4,false);
		buttonComp.setLayout(gl);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan=3;
		buttonComp.setLayoutData(gd);
		
		new WdgSpacer(buttonComp, SWT.HORIZONTAL, 2, 1);
		
		Button btnReset = new Button(buttonComp, SWT.PUSH);
		btnReset.setText("Reset");
		btnReset.setBackground(GUI.COLOR_BLUE_BUTTON);
		btnReset.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				for (SPOTFINDER_OPTION option : fieldMap.keySet()) {
					fieldMap.get(option).setText("");
				}
				saveGUIValues();
			}
		});
		
		Button btnClose = new Button(buttonComp, SWT.PUSH);
		btnClose.setText("Close");
		btnClose.setBackground(GUI.COLOR_BLUE_BUTTON);
		btnClose.addSelectionListener(new SelectionListener()
		{
			@Override public void widgetDefaultSelected(SelectionEvent arg0) {}
			@Override public void widgetSelected(SelectionEvent arg0)
			{
				shell.close();
				shell.dispose();
			}
		});

		connectListeners();
	}

	private void updateVisibleOptions(boolean calcLayout)
	{
		SPOTFINDER_TYPE currentSFType = DevSFType.getConfig(sfRunType).id;
		for (SPOTFINDER_TYPE sfType : SPOTFINDER_TYPE.values())
		{
			boolean show = sfType==currentSFType;
			for (Composite comp : compMap.get(sfType))
			{
				WdgControl.setControlVisible(comp, show);
			}
		}
		// show extra table components only if there are options
		boolean typeHasOptions = hasOptions.get(currentSFType);
		WdgControl.setControlVisible(headerComp, typeHasOptions);
		WdgControl.setControlVisible(divider, typeHasOptions);
		WdgControl.setControlVisible(divider2, typeHasOptions);

		if (calcLayout) {
			shell.layout(true);
			shell.pack(true);
		}

		saveGUIValues();
	}

	public void connectListeners()
	{
		DevSFType.get(sfRunType).llChange.addListener(new ListenerList.Listener<DevSFType>()
		{
			@Override public void handleEvent(DevSFType config, Object data)
			{
				GUI.asyncExec(() -> updateVisibleOptions(true));
			}
		});
		DevSFOptions.getListenerList(sfRunType).addListener((dev,data)->{
			loadFromDevice(true);
		});
	}

	public HashMap<SPOTFINDER_OPTION,String> getCurrentEntries()
	{
		HashMap<SPOTFINDER_OPTION,String> ret = new HashMap<>();
		SPOTFINDER_TYPE sfType = DevSFType.getConfig(sfRunType).id;
		for (SPOTFINDER_OPTION option : DevSFOptions.options) {
			if (option.getSpotfinder_id().equalsIgnoreCase(sfType.name())) {
				ret.put(option,currentValues.get(option));
			}
		}
		return ret;
	}

	public void saveGUIValues()
	{
		SPOTFINDER_TYPE sfType = DevSFType.getConfig(sfRunType).id;

		// for each option
		boolean change = false;
		for (SPOTFINDER_OPTION option : DevSFOptions.options)
		{
			// if the option applies to the current spotfinder
			if (option.getSpotfinder_id().equalsIgnoreCase(sfType.name()))
			{
				// update the map with the current value
				String value = fieldMap.get(option).getText().trim();
				if (!value.equals(currentValues.get(option))) {
					currentValues.put(option, value);
					change = true;
				}
			}
		}
		if (change) DevSFOptions.setSettings(sfRunType, currentValues);
	}

	public void open()
	{
		if (null!=shell && !shell.isDisposed())
		{
			shell.moveAbove(GUI.getShell());
			shell.setFocus();
			return;
		}
		
		shell = new Shell(GUI.getShell(), SWT.SHELL_TRIM);
		shell.setText((sfRunType==SFRunType.RASTER?"Raster":"Collect")+" SpotFinder Settings");

		shell.addTraverseListener(new TraverseListener()
		{
			@Override public void keyTraversed(TraverseEvent e)
			{
				if (e.detail == SWT.TRAVERSE_ESCAPE)
				{
					shell.close();
					shell.dispose();
				}
			}
		});

		createWidgets();

		// center shell
		int centerX = GUI.getShell().getLocation().x + GUI.getShell().getSize().x/2;
		int centerY = GUI.getShell().getLocation().y + GUI.getShell().getSize().y/2;
		int x = centerX - 100;
		int y = centerY - 100;
		shell.setLocation(x,y);

		// TODO: eliminate visible resize
		shell.setVisible(false);
		shell.open();
		updateVisibleOptions(false);
		shell.layout(true);
		shell.pack(true);
	}
}
