package gov.anl.gmca.spotfinder;

import gov.anl.gmca.common.DBConfig;
import gov.anl.gmca.spotfinder.SFData.SFType;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

public class SpotFinderQueue
{
	public static final int MAX_REMOTE_THREADS = 150;
	
	// executes only a certain amount of threads at a time
	private static ThreadPoolExecutor remoteExecutor;
	
	// executor init
	public static void init()
	{
		remoteExecutor = new ThreadPoolExecutor(MAX_REMOTE_THREADS, MAX_REMOTE_THREADS, 0, TimeUnit.MILLISECONDS, new LinkedBlockingDeque<>());
		startMonitor();
	}
	
	// queue monitor
	private static Runnable monitor;
	public static int getActiveThreads() { return remoteExecutor.getActiveCount(); }
	public static void startMonitor()
	{
		if (null==monitor)
		{
			monitor = new Runnable(){public void run()
			{
				while (true)
				{
					if (null!=remoteExecutor) {
						int active = remoteExecutor.getActiveCount();
						int queueSize = remoteExecutor.getQueue().size();
						if (active>0 || queueSize>0) {
						    //Log.note("SF queue active="+active+" queued="+queueSize);
						}
					}
					Util.sleep(1000);
				}
			}};
			new Thread(monitor).start();
		}
	}
	
	// static list of running threads
	protected static ArrayList<CmdSpotFinder> cmdList = new ArrayList<CmdSpotFinder>();
	public static void stopActiveThreads(SFType sfType, int runIndex)
	{
		System.out.println("Stopping active SpotFinder threads for "+sfType.toString()+" run index "+runIndex+"...");
		synchronized(cmdList)
		{
			for (CmdSpotFinder cmd : cmdList)
			{
				if (sfType==cmd.sfType && runIndex==cmd.runIndex)
				{
					cmd.requestStop();
				}
			}
			for (CmdSpotFinder cmd : cmdList)
			{
				//cmd.waitFor();
			}
			Util.sleep(500);
		}
		//cmdList = new ArrayList<CmdSpotFinder>();
		System.out.println("Active SpotFinder threads stopped.");
	}
	
	public static void submitTask(CmdSpotFinder cmd)
	{
		if (DBConfig.getBoolean("spotfinder_enable_collect") || cmd.sfType!=SFType.COLLECT)
		{
			//System.out.println("Submitting thread for SpotFinder image "+cmd.imageName);
			
			synchronized(cmdList)
			{
				cmdList.add(cmd);
			}

			remoteExecutor.execute(cmd);
		}
	}
}
