package gov.anl.gmca.spotfinder;

import gov.anl.gmca.io.Database;
import gov.anl.gmca.gui.Main;
import gov.anl.gmca.hutch.DetectorConfig;
import gov.anl.gmca.spotfinder.DevSFOptions.SPOTFINDER_OPTION;
import gov.anl.gmca.spotfinder.DevSFType.SFConfig;
import gov.anl.gmca.spotfinder.DevSFType.SFRunType;
import gov.anl.gmca.spotfinder.DevSFType.SPOTFINDER_TYPE;
import gov.anl.gmca.util.Util;
import org.apache.http.client.utils.URIBuilder;

import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;

public class SpotFinderURL
{
	public static class SFURLInfo
	{
		public SFConfig sfConfig;
		public String remoteHost;
		public SFData.SFType sfType;
		public SFRunType sfRunType;
		public int frameIndex;
		public String imageDir;
		public ArrayList<String> cbfFilenames;
		public double highLimit_A, lowLimit_A;
		public int timeoutMs;
	}
        public static String server_location = null;
	
	public static ArrayList<String> generateURL(SFURLInfo info)
	{
		String fullDir = info.imageDir;
		ArrayList<String> cbfFilenames;
		cbfFilenames = info.cbfFilenames;
		if (server_location == null) {
		    Database.Result dbResult = Database.exec(Database.DBName.CONFIG, "select location from Locations where name='dials_http'");
		    server_location = dbResult.getRow(0).get("location");
		}

		ArrayList<String> cmd = new ArrayList<>();
		switch (info.sfConfig.id) {
			case DIALS_SPOTFINDER:
				cmd.add("curl");
				cmd.add("-s");
				cmd.add(server_location+fullDir+"/"+cbfFilenames.get(0));
				break;
			case DOZOR:
				String dozor_script_path = System.getProperty("application.home")+"/tools/scripts/dozor_mp.py";
				cmd.add("srun");

				// job name
				cmd.add("-J");
				cmd.add("dozor "+String.join(",", cbfFilenames));

				// timeout
				cmd.add("--export");
				cmd.add("TIMEOUT_MS="+info.timeoutMs);

				// first wrapper: create temp dir (add after removing temp dir creation from dozor script)
				//cmd.add(System.getProperty("application.home")+"/tools/scripts/run_in_tmp_dir.sh");

				// second wrapper: wait for all CBF images to be fully written
				cmd.add(System.getProperty("application.home")+"/tools/scripts/cbf_footer_check.py");

				// now run dozor script
				cmd.add(dozor_script_path);
				for (String filename : cbfFilenames) {
					cmd.add(String.format("%s/%s", fullDir, filename));
				}
				break;
		}
		return cmd;

		/*String urlSuffix = Util.sprintf(info.sfConfig.url, fullDir+"/"+cbfFilename);
		String urlBase = Util.sprintf("http://%s/%s",info.remoteHost,urlSuffix);
		try {
			URIBuilder ub = new URIBuilder(urlBase);
			HashMap<SPOTFINDER_OPTION, String> options = DlgSFOptions.get(info.sfRunType).getCurrentEntries();

			if (info.sfConfig.id == SPOTFINDER_TYPE.DIALS_SPOTFINDER)
				addDIALSResArguments(ub, info);

			for (SPOTFINDER_OPTION option : options.keySet()) {
				String value = options.get(option);
				if (value.length() > 0)
					ub.addParameter(option.getUrl_key(), options.get(option));
			}

			return ub.toString();
		}
		catch (URISyntaxException e) { e.printStackTrace(); return urlBase; }*/
	}

	private static void addDISTLResArguments(URIBuilder ub, SFURLInfo info)
	{
		if (info.highLimit_A>0.0)				      ub.addParameter("distl.res.outer",Util.sprintf("%.1f",info.highLimit_A));
		if (info.lowLimit_A<Double.POSITIVE_INFINITY) ub.addParameter("distl.res.inner",Util.sprintf("%.1f",info.lowLimit_A));
	}

	private static void addDIALSResArguments(URIBuilder ub, SFURLInfo info)
	{
		if (info.highLimit_A>0.0)				      ub.addParameter("spotfinder.filter.d_min",Util.sprintf("%.1f",info.highLimit_A));
		if (info.lowLimit_A<Double.POSITIVE_INFINITY) ub.addParameter("spotfinder.filter.d_max",Util.sprintf("%.1f",info.lowLimit_A));
	}
}
