/**
 * Cache for all data collection strategy results
 * 
 */

package gov.anl.gmca.strategy;

import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;

import java.util.ArrayList;
import java.util.Collections;

public class ModelStrategyResultList {
	public ListenerList<ModelStrategyResultList> ll;

	protected ArrayList<ModelStrategyResult> resultList = new ArrayList<ModelStrategyResult>();

	public ArrayList<ModelStrategyResult> getStrategyList() {
		return new ArrayList<ModelStrategyResult>(resultList);
	}

	public ModelStrategyResult getResultID(int requestedID) {
		for (ModelStrategyResult result : getStrategyList()) {
			if (result.getID() == requestedID)
				return result;
		}
		return null;
	}

	public ModelStrategyResult getResultIndex(int index) {
		try {
			return resultList.get(index);
		} catch (IndexOutOfBoundsException ioobe) {
			return null;
		}
	}

	public ModelStrategyResult getCurrentResult() {
		return getResultIndex(getCurrentStrategyIndex());
	}

	public void clearResults() {
		resultList = new ArrayList<ModelStrategyResult>();
	}

	public void setRow(ModelStrategyResult result, int sampleNo, boolean update) {
		for (int i = 0; i < resultList.size(); i++) {
			if (resultList.get(i).getID() == sampleNo) {
				resultList.set(i, result);
				return;
			}
		}

		resultList.add(result);

		Collections.sort(resultList);

		if (update)
			ll.notifyListeners(result.getID());
	}

	protected int currentStrategyIndex = 0;

	public void setCurrentStrategyIndex(int i) {
		currentStrategyIndex = i;
	}

	public int getCurrentStrategyIndex() {
		return currentStrategyIndex;
	}

	protected static ModelStrategyResultList inst = null;

	public static ModelStrategyResultList get() {
		return inst;
	}

	public ModelStrategyResultList() {
		inst = this;
		ll = new ListenerList<ModelStrategyResultList>(this);
	}

	public void deleteAll() {
		new Thread(new Runnable() {
			public void run() {
				boolean okToDelete = Util.popupOkCancel(
						"Delete?",
						"OK to delete ALL ("
								+ resultList.size()
								+ ") strategy results?  This action cannot be undone.");
				if (okToDelete)
					DevStrategyResults.get().deleteAllResults();
			}
		}, "ModelStrategyResultList deleteAll").start();
	}

	public void save(String excelname) {
		final String excelFile = excelname;
		new Thread(new Runnable() {
			public void run() {
				DevStrategyResults.get().saveAllResults(excelFile);
			}
		}, "ModelStrategyResultList save").start();

	}
}
