/**
 *
 *
 *
 *
 *
 *
 *
 *
 *
*/
package gov.anl.gmca.strategy;

import gov.anl.gmca.collect.ModelCollectRun;
import gov.anl.gmca.collect.ModelCollectRunList;
import gov.anl.gmca.common.AppState;
import gov.anl.gmca.common.AppState.Operation;
import gov.anl.gmca.common.DevDataDir;
import gov.anl.gmca.common.ModelRunList;
import gov.anl.gmca.common.ModelRunList.ModelRun;
import gov.anl.gmca.common.ui.NBFileDialog;
import gov.anl.gmca.gui.GUI;
import gov.anl.gmca.io.ExecTask;
import gov.anl.gmca.screening.CmdScreening;
import gov.anl.gmca.screening.DevSilTable;
import gov.anl.gmca.strategy.ModelStrategyResult.StrategyField;
import gov.anl.gmca.strategy.ModelStrategyResult.StrategyState;
import gov.anl.gmca.syslog.Log;
import gov.anl.gmca.users.ModelUserTable.Client;
import gov.anl.gmca.users.ModelUserTable.ClientListener;
import gov.anl.gmca.util.ListenerList;
import gov.anl.gmca.util.Util;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import java.util.HashMap;

public class WdgStrategy extends Composite implements
		ListenerList.Listener<ModelStrategyResultList>,
		ModelCollectRunList.Listener, ModelCollectRunList.CurrentRunListener,
		ClientListener {
	
	/**
	 * Time outs for processing
	 * 
	 */
	public static final int TIMEWAIT_FOR_STRATEYJOB = 180; //secs
	
	
	protected static final ModelStrategyResult.StrategyField[] displayFields = {
			ModelStrategyResult.StrategyField.WORKDIR,
			ModelStrategyResult.StrategyField.WARNING,
			ModelStrategyResult.StrategyField.IMAGES,
			ModelStrategyResult.StrategyField.SOFTWARE,
			ModelStrategyResult.StrategyField.SPACE_GROUP,
			ModelStrategyResult.StrategyField.OSC_START,
			ModelStrategyResult.StrategyField.ANOM_OSC_START,
			ModelStrategyResult.StrategyField.INV_OSC_START,
			ModelStrategyResult.StrategyField.MIN_END,
			ModelStrategyResult.StrategyField.MIN_ANOM_END,
			ModelStrategyResult.StrategyField.MIN_INV_END,
			ModelStrategyResult.StrategyField.OSC_END,
			ModelStrategyResult.StrategyField.ANOM_OSC_END,
			ModelStrategyResult.StrategyField.INV_OSC_END,
			ModelStrategyResult.StrategyField.COMPLETE,
			ModelStrategyResult.StrategyField.ANOM_COMPLETE,
			ModelStrategyResult.StrategyField.INV_COMPLETE,
			ModelStrategyResult.StrategyField.OSC_DELTA,
			ModelStrategyResult.StrategyField.DETECTOR_DIST,
			ModelStrategyResult.StrategyField.RESOLUTION,
			ModelStrategyResult.StrategyField.MOSAICITY,
			ModelStrategyResult.StrategyField.UNIT_CELL };

	protected HashMap<StrategyField, Text> fieldTexts = new HashMap<StrategyField, Text>();
	protected Composite labelComp, expandComp;
	protected Label positionLabel, stateLabel;
	protected Button exportToNewRunButton, exportToCurrentRunButton,
			solutionButton, deleteButton;
	protected Button startButton, prevButton, nextButton, lastButton;
	protected boolean dontUpdateSpacegroup = false;
	// sudhir
	protected Combo valueGroups;
	protected TabFolder ctFolder;
	protected Button nativeText, anomalousText, inverseText;
	WdgStrategyTable stratTable = null;
	protected static final boolean UPDATE_CURRENT_INDEX_ON_RESULT_CREATION = true;
	protected static final boolean SHOW_STRATEGY_TAB_ON_RESULT_CREATION = true;

	public WdgStrategy(Composite parent) {
		super(parent, SWT.NONE);
		setBackground(GUI.BGCOLOR);

		GridLayout gl = new GridLayout(1, true);
		setLayout(gl);

		expandComp = new Composite(this, SWT.NONE);
		expandComp.setBackground(GUI.BGCOLOR);
		GridLayout gl1 = new GridLayout(4, false);
		expandComp.setLayout(gl1);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		expandComp.setLayoutData(gd);

		createControls();
		createStrategyDisplay();
		createExportSolutionButtons();
	}

	public void connectListeners() {
		ModelStrategyResultList.get().ll.addListener(this);
		DevStrategyResults.get().verifyConnected();
		ModelCollectRunList.get().addListener(this);
		ModelCollectRunList.get().addCurrentRunListener(this); // needed to
																// track run
																// label
		Client.addListener(this);

		updateLabels();
		updateExportLabel();

		stratTable.updateTable();
		ctFolder.setSelection(0);
	}

	protected void createExportSolutionButtons() {
		exportToNewRunButton = new Button(expandComp, SWT.PUSH);
		exportToNewRunButton.setText("... Loading ...");
		exportToNewRunButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		GridData gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.widthHint = 150;
		exportToNewRunButton.setLayoutData(gd);
		exportToNewRunButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				int isAnom = 0;
				if (anomalousText.getSelection())
					isAnom = 1;
				if (inverseText.getSelection())
					isAnom = 2;
				ModelCollectRunList runList = ModelCollectRunList.get();

				int runCount = runList.getRunCount();
				if (runCount == 15) {
					Util.popupOk("Tab limit",
							"You have reached the limit of 16 run tabs.");
					return;
				}
				ModelStrategyResultList.get().getCurrentResult()
						.exportToRun(isAnom, true);
			}
		});

		exportToCurrentRunButton = new Button(expandComp, SWT.PUSH);
		exportToCurrentRunButton.setText("... Loading ...");
		exportToCurrentRunButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.widthHint = 150;
		exportToCurrentRunButton.setLayoutData(gd);
		exportToCurrentRunButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				int isAnom = 0;
				if (anomalousText.getSelection())
					isAnom = 1;
				if (inverseText.getSelection())
					isAnom = 2;
				ModelStrategyResultList.get().getCurrentResult()
						.exportToRun(isAnom, false);
			}
		});

		solutionButton = new Button(expandComp, SWT.PUSH);
		solutionButton.setText("Solution");
		solutionButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData(SWT.CENTER, SWT.DEFAULT, true, false);
		gd.widthHint = 150;
		solutionButton.setLayoutData(gd);
		solutionButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				String filename = ModelStrategyResultList.get()
						.getCurrentResult().getField(StrategyField.WORKDIR)
						+ "/LABELIT/labelit.out";

				// TODO: handle spaces

				ExecTask.new_ExecTask("nedit " + filename + "", "nedit", false,
						false).start();
			}
		});

		AppState.addListener(Operation.START_COLLECT, new AppState.Listener() {
			@Override
			public void stateChanged(Operation op, boolean allowed) {
				updateEnabled();
			}
		});
	}

	protected void createStrategyDisplay() {
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true, 4, 1);

		ctFolder = new TabFolder(expandComp, SWT.BORDER);
		ctFolder.setFont(GUI.FONT_SAMPLE_TABS);
		ctFolder.setLayoutData(gd);

		labelComp = new Composite(ctFolder, SWT.BORDER);
		labelComp.setBackground(GUI.COLOR_WHITE);
		labelComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 3,
				1));
		GridLayout gl = new GridLayout(4, false);
		gl.verticalSpacing = 5;
		gl.horizontalSpacing = 15;
		labelComp.setLayout(gl);
		
		TabItem strategyTab = new TabItem(ctFolder, SWT.NONE);
		strategyTab.setText("Strategy");
		strategyTab.setControl(labelComp);

		Composite allComp = new Composite(ctFolder, SWT.NONE);
		allComp.setBackground(GUI.COLOR_WHITE);
		allComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		GridLayout glall = new GridLayout(1, false);
		allComp.setLayout(glall);

		new GridData(SWT.FILL, SWT.FILL, true, true);
		stratTable = new WdgStrategyTable(allComp, SWT.NONE);
		stratTable.setLayoutData(gd);

		TabItem allTab = new TabItem(ctFolder, SWT.NONE);
		allTab.setText("All Results");
		allTab.setControl(allComp);
		
		Composite helpComp = new Composite(ctFolder, SWT.BORDER);
		helpComp.setBackground(GUI.COLOR_WHITE);
		helpComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		GridLayout glHelp = new GridLayout(1, false);
		helpComp.setLayout(glHelp);
		
		new GridData(SWT.FILL, SWT.FILL, true, true);
		WdgStrategyHelp procHelp = new WdgStrategyHelp(helpComp, SWT.NONE);
		procHelp.setLayoutData(gd);
		TabItem helpTab = new TabItem(ctFolder, SWT.NONE);
		helpTab.setText("Help");
		helpTab.setControl(helpComp);
		
		stateLabel = new Label(labelComp, SWT.BORDER | SWT.CENTER);
		gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 4, 1);
		stateLabel.setLayoutData(gd);
		stateLabel.setText("... Loading ...");

		Font font = GUI.FONT_11;

		for (ModelStrategyResult.StrategyField field : displayFields) {
			if (field.getDisplayLines() > -1) {
				Label nameLabel = new Label(labelComp, SWT.NONE);
				nameLabel.setBackground(GUI.COLOR_WHITE);
				nameLabel.setFont(font);
				if (field.getExportable())
					nameLabel.setForeground(GUI.COLOR_GREEN);
				nameLabel.setText(null == field ? "" : field.getUserString()
						+ ":");
			}

			if (field == ModelStrategyResult.StrategyField.SPACE_GROUP) {
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				gd.horizontalSpan = 3;
				valueGroups = new Combo(labelComp, SWT.READ_ONLY);
				valueGroups.setLayoutData(gd);

				valueGroups.addSelectionListener(new SelectionListener() {
					@Override
					public void widgetDefaultSelected(SelectionEvent se) {
					}

					@Override
					public void widgetSelected(SelectionEvent se) {
						DevStrategyResults
								.get()
								.updateRow(
										Integer.parseInt(ModelStrategyResultList
												.get()
												.getCurrentResult()
												.getField(
														ModelStrategyResult.StrategyField.SAMPLE_NUMBER)),
										valueGroups.getText(), true);
						String isGood = ModelStrategyResultList
								.get()
								.getCurrentResult()
								.getField(
										ModelStrategyResult.StrategyField.GOOD);
						DevStrategyResults
								.get()
								.updateRow(
										Integer.parseInt(ModelStrategyResultList
												.get()
												.getCurrentResult()
												.getField(
														ModelStrategyResult.StrategyField.SAMPLE_NUMBER)),
										valueGroups.getText(), true);

						String curr = ModelStrategyResultList
								.get()
								.getCurrentResult()
								.getField(
										ModelStrategyResult.StrategyField.SCREENING_SAMPLE);
						if (!curr.equals("") && curr != null
								&& !curr.equals("none")) {
							int currSamp = CmdScreening.labelToindex(curr);
							DevSilTable.get().setResult(
									currSamp,
									ModelStrategyResultList.get()
											.getCurrentResult(), true);
						}

						if (!isGood.equals("yes")) {

							String _workDir = ModelStrategyResultList
									.get()
									.getCurrentResult()
									.getField(
											ModelStrategyResult.StrategyField.WORKDIR);
							int _sampleNum = Integer
									.parseInt(ModelStrategyResultList
											.get()
											.getCurrentResult()
											.getField(
													ModelStrategyResult.StrategyField.SAMPLE_NUMBER));
							int _solutionNum = Integer
									.parseInt(ModelStrategyResultList
											.get()
											.getCurrentResult()
											.getField(
													ModelStrategyResult.StrategyField.SOLUTION));
							String _sampleName = ModelStrategyResultList
									.get()
									.getCurrentResult()
									.getField(
											ModelStrategyResult.StrategyField.SAMPLE_NAME);
							String _spaceGroup = ModelStrategyResultList
									.get()
									.getCurrentResult()
									.getField(
											ModelStrategyResult.StrategyField.SPACE_GROUP_SH);

							String _spaceGroupFull = valueGroups.getText();
							String _images = ModelStrategyResultList
									.get()
									.getCurrentResult()
									.getField(
											ModelStrategyResult.StrategyField.IMAGES);
							CmdUserStrategy cmdStrategy = new CmdUserStrategy(
									_workDir, _sampleNum, _solutionNum,
									_sampleName, _spaceGroup, _spaceGroupFull,
									_images);
							cmdStrategy.start();
						}

					}
				});
				gd = new GridData();
				Label blankText = new Label(labelComp, SWT.NONE);
				blankText.setBackground(GUI.COLOR_WHITE);
				blankText.setLayoutData(gd);

				gd = new GridData();
				gd.widthHint = 100;
				nativeText = new Button(labelComp, SWT.RADIO);
				nativeText.setText("Native");
				nativeText.setBackground(GUI.COLOR_WHITE);
				nativeText.setLayoutData(gd);
				nativeText.setSelection(true);

				gd = new GridData();
				gd.widthHint = 100;
				anomalousText = new Button(labelComp, SWT.RADIO);
				anomalousText.setText("Anomalous");
				anomalousText
						.setToolTipText("Anomalous Continuous: 2 times min. osc range");
				anomalousText.setBackground(GUI.COLOR_WHITE);
				anomalousText.setLayoutData(gd);

				gd = new GridData();
				gd.widthHint = 100;
				inverseText = new Button(labelComp, SWT.RADIO);
				inverseText.setText("Inverse");
				inverseText
						.setToolTipText("Anomalous Inverse : activates inverseBeam");
				inverseText.setBackground(GUI.COLOR_WHITE);
				inverseText.setLayoutData(gd);
			} else {
				Text valueText = (field.getDisplayLines() > 1) ? new Text(
						labelComp, SWT.WRAP | SWT.V_SCROLL | SWT.MULTI
								| SWT.BORDER) : new Text(labelComp, SWT.WRAP
						| SWT.SINGLE);

				valueText.setFont(font);
				gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false);
				if (field.getDisplayLines() > 0)
					gd.horizontalSpan = 3;
				if (field.getDisplayLines() > 1) {
					GC gc = new GC(labelComp);
					gc.setFont(font);
					int textHeight = gc.textExtent("1234567890").y;
					gd.heightHint = textHeight * field.getDisplayLines();
				}

				valueText.setEditable(false);
				valueText.setLayoutData(gd);
				if (field.getExportable())
					valueText.setForeground(GUI.COLOR_GREEN);
				if (null != field)
					fieldTexts.put(field, valueText);
				if (field == ModelStrategyResult.StrategyField.WARNING)
					valueText.setForeground(GUI.COLOR_RED);
				valueText.setBackground(GUI.COLOR_WHITE);
			}
		}

		ctFolder.setSelection(0);
	}

	protected void createControls() {
		Composite controlComp = new Composite(expandComp, SWT.NONE);
		controlComp.setBackground(GUI.BGCOLOR);
		GridData gd = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1);
		controlComp.setLayoutData(gd);
		GridLayout gl = new GridLayout(6, false);
		gl.horizontalSpacing = 0;
		controlComp.setLayout(gl);

		int clearButtonWidth = 80;
		int moveButtonWidth = 70;
		Font buttonFont = GUI.FONT_12;

		startButton = new Button(controlComp, SWT.PUSH);
		startButton.setText("|<<");
		startButton.setFont(buttonFont);
		startButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData();
		gd.widthHint = moveButtonWidth;
		startButton.setLayoutData(gd);
		startButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				dontUpdateSpacegroup = true;
				ModelStrategyResult currentResult = ModelStrategyResultList
						.get().getResultIndex(0);
				if (currentResult != null) {
					valueGroups.setText(currentResult
							.getField(ModelStrategyResult.StrategyField.SPACE_GROUP));
					DevStrategyResults
							.get()
							.updateRow(
									1,
									currentResult
											.getField(ModelStrategyResult.StrategyField.SPACE_GROUP),
									false);
					DevStrategyResults.setCurrentStrategyIndex(0);
				}
			}
		});

		prevButton = new Button(controlComp, SWT.PUSH);
		prevButton.setText("<");
		prevButton.setFont(buttonFont);
		prevButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData();
		gd.widthHint = moveButtonWidth;
		prevButton.setLayoutData(gd);
		prevButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				dontUpdateSpacegroup = true;
				int prev = ModelStrategyResultList.get()
						.getCurrentStrategyIndex() - 1;
				ModelStrategyResult currentResult = ModelStrategyResultList
						.get().getResultIndex(prev);
				if (currentResult != null) {
					valueGroups.setText(currentResult
							.getField(ModelStrategyResult.StrategyField.SPACE_GROUP));
					DevStrategyResults
							.get()
							.updateRow(
									prev,
									currentResult
											.getField(ModelStrategyResult.StrategyField.SPACE_GROUP),
									false);
					DevStrategyResults.setCurrentStrategyIndex(prev);
				}
			}
		});

		nextButton = new Button(controlComp, SWT.PUSH);
		nextButton.setText(">");
		nextButton.setFont(buttonFont);
		nextButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData();
		gd.widthHint = moveButtonWidth;
		nextButton.setLayoutData(gd);
		nextButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				dontUpdateSpacegroup = true;
				int next = ModelStrategyResultList.get()
						.getCurrentStrategyIndex() + 1;
				ModelStrategyResult currentResult = ModelStrategyResultList
						.get().getResultIndex(next);
				if (currentResult != null) {
					valueGroups.setText(currentResult
							.getField(ModelStrategyResult.StrategyField.SPACE_GROUP));
					DevStrategyResults
							.get()
							.updateRow(
									next,
									currentResult
											.getField(ModelStrategyResult.StrategyField.SPACE_GROUP),
									false);
					DevStrategyResults.setCurrentStrategyIndex(next);
				}
			}
		});

		lastButton = new Button(controlComp, SWT.PUSH);
		lastButton.setText(">>|");
		lastButton.setFont(buttonFont);
		lastButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData();
		gd.widthHint = moveButtonWidth;
		lastButton.setLayoutData(gd);
		lastButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				dontUpdateSpacegroup = true;
				int last = ModelStrategyResultList.get().getStrategyList()
						.size() - 1;
				ModelStrategyResult currentResult = ModelStrategyResultList
						.get().getResultIndex(last);
				if (currentResult != null) {
					valueGroups.setText(currentResult
							.getField(ModelStrategyResult.StrategyField.SPACE_GROUP));
					DevStrategyResults
							.get()
							.updateRow(
									last + 1,
									currentResult
											.getField(ModelStrategyResult.StrategyField.SPACE_GROUP),
									false);
					DevStrategyResults.setCurrentStrategyIndex(last);
				}
			}
		});

		positionLabel = new Label(controlComp, SWT.CENTER);
		positionLabel.setText("");
		positionLabel.setBackground(GUI.BGCOLOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.minimumWidth = 100;
		positionLabel.setLayoutData(gd);

		deleteButton = new Button(controlComp, SWT.PUSH);
		deleteButton.setText("Export All");
		deleteButton.setToolTipText("save all strategy results to excel");
		deleteButton.setBackground(GUI.COLOR_BLUE_BUTTON);
		gd = new GridData();
		gd.widthHint = clearButtonWidth;
		deleteButton.setLayoutData(gd);
		deleteButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent se) {
			}

			@Override
			public void widgetSelected(SelectionEvent se) {
				exportToSpreadsheet();
			}
		});
	}

	protected void setStateLabel(StrategyState ss) {
		if (null == ss) {
			stateLabel.setBackground(GUI.COLOR_LIGHT_GRAY);
			stateLabel.setForeground(GUI.COLOR_BLACK);
			stateLabel.setText("---");
			return;
		}

		switch (ss) {
		case INACTIVE:
			stateLabel.setBackground(GUI.COLOR_LIGHT_GRAY);
			stateLabel.setForeground(GUI.COLOR_BLACK);
			stateLabel.setText("Inactive");
			break;

		case PROCESSING:
			stateLabel.setBackground(GUI.COLOR_DARK_CYAN);
			stateLabel.setForeground(GUI.COLOR_WHITE);
			stateLabel.setText("Processing");
			break;

		case DONE:
			stateLabel.setBackground(GUI.COLOR_DARK_GREEN);
			stateLabel.setForeground(GUI.COLOR_WHITE);
			stateLabel.setText("Done");
			break;
		}
	}

	@Override
	public void handleEvent(ModelStrategyResultList resultList, Object data) {
		updateLabels();
		stratTable.updateTable();
		// ctFolder.setSelection(0);
	}

	protected void updateLabels() {
		GUI.asyncExec(new Runnable() {
			public void run() {
				ModelStrategyResultList resultList = ModelStrategyResultList
						.get();

				// position in list
				int listSize = resultList.getStrategyList().size();

				if (listSize >= 1) {
					positionLabel.setText((resultList.getCurrentStrategyIndex() + 1)
							+ " of " + listSize);
				} else {
					positionLabel.setText("---");
				}

				// fields
				ModelStrategyResult currentResult = resultList
						.getCurrentResult();
				if (null != currentResult) {
					setStateLabel(currentResult.getState());

					for (StrategyField field : currentResult.getAllFields()
							.keySet()) {
						if (field == ModelStrategyResult.StrategyField.SPACE_GROUP) {
							int currentSpaceGroup = valueGroups
									.getSelectionIndex();
							valueGroups.removeAll();
							for (Object s : currentResult.spaceGroups.toArray()) {
								if (valueGroups.indexOf((String) s) < 0) {
									valueGroups.add((String) s);
								}
							}
							if (dontUpdateSpacegroup) {
								dontUpdateSpacegroup = false;
								// currentSpaceGroup = 0;
							}
							valueGroups.setText(currentResult
									.getField(ModelStrategyResult.StrategyField.SPACE_GROUP));
							// valueGroups.select(currentSpaceGroup);
						}

						Text text = fieldTexts.get(field);
						if (null != text) {
							String value = currentResult.getField(field);

							if (null == value) {
								text.setText("---");
							} else {
								text.setText(value);

							}
						}
					}
				} else {
					exportToNewRunButton.setEnabled(false);
					exportToCurrentRunButton.setEnabled(false);

					setStateLabel(null);
					solutionButton.setEnabled(false);

					for (Text text : fieldTexts.values()) {
						if (null != text)
							text.setText("---");
					}
				}

				updateEnabled();

				labelComp.layout(true);
			}
		});
	}

	protected void updateEnabled() {
		ModelStrategyResult currentResult = ModelStrategyResultList.get()
				.getCurrentResult();
		final StrategyState state = (null == currentResult) ? null
				: currentResult.getState();

		GUI.asyncExec(new Runnable() {
			public void run() {
				boolean enableButtons = (null != state
						&& state == StrategyState.DONE && AppState
						.isOperationAllowed(Operation.START_COLLECT));

				solutionButton.setEnabled(enableButtons);
				valueGroups.setEnabled(null != state
						&& state != StrategyState.PROCESSING
						&& Client.isMaster());

				boolean zeroResults = ModelStrategyResultList.get()
						.getStrategyList().size() == 0;
				boolean atStart = ModelStrategyResultList.get()
						.getCurrentStrategyIndex() <= 0;
				boolean atEnd = ModelStrategyResultList.get()
						.getCurrentStrategyIndex() >= ModelStrategyResultList
						.get().getStrategyList().size() - 1;

				deleteButton.setEnabled(Client.isMaster() && !zeroResults);
				startButton.setEnabled(Client.isMaster() && !atStart
						&& !zeroResults);
				prevButton.setEnabled(Client.isMaster() && !atStart
						&& !zeroResults);
				nextButton.setEnabled(Client.isMaster() && !atEnd
						&& !zeroResults);
				lastButton.setEnabled(Client.isMaster() && !atEnd
						&& !zeroResults);

				exportToNewRunButton.setEnabled(enableButtons
						&& Client.isMaster() && !zeroResults);
				exportToCurrentRunButton.setEnabled(enableButtons
						&& Client.isMaster() && !zeroResults);
				updateExportLabel();
			}
		});
	}

	@Override
	public void runListChanged(ModelRunList _runList) {
		updateExportLabel();
	}

	@Override
	public void currentRunChanged(ModelRun run) {
		updateExportLabel();
	}

	protected void updateExportLabel() {
		final ModelCollectRunList runList = ModelCollectRunList.get();

		int runCount = runList.getRunCount();
		if (runCount >= 0) {
			ModelCollectRun lastRun = (ModelCollectRun) runList
					.getRun(runCount);
			if (null != lastRun) {
				final int nextRunLabel = lastRun.getRunLabel() + 1;
				final int currentRunLabel = runList.getCurrentRunModel()
						.getRunLabel();
				final boolean zeroResults = ModelStrategyResultList.get()
						.getStrategyList().size() == 0;
				ModelStrategyResult currentResult = ModelStrategyResultList
						.get().getCurrentResult();
				final StrategyState state = (null == currentResult) ? null
						: currentResult.getState();
				GUI.asyncExec(new Runnable() {
					public void run() {
						exportToNewRunButton.setText("Create run "
								+ nextRunLabel);

						exportToCurrentRunButton.setEnabled(0 != currentRunLabel
								&& null != state
								&& state == StrategyState.DONE
								&& !zeroResults
								&& Client.isMaster()
								&& AppState
										.isOperationAllowed(Operation.START_COLLECT));
						exportToCurrentRunButton.setText("Export to run "
								+ currentRunLabel);
					}
				});
			}
		}
	}

	protected void exportToSpreadsheet() {
		
		NBFileDialog dialog = new NBFileDialog(GUI.getShell(), SWT.SAVE);
		dialog.setFilterNames(new String[] { "Excel Files", "All Files (*.*)" });
		dialog.setFilterExtensions(new String[] { "*.xls" });
		dialog.setFileName("strategyResults.xls");
		
		String savePath = "";
		savePath = DevDataDir.getPrimary();
		
		if(savePath.equals("") || savePath == null)
			savePath = System.getProperty("user.home");
		
		dialog.setFilterPath(savePath);
		
		String fileName = dialog.openStr();
		
		if(fileName != null && !fileName.equals("null") && !fileName.equals("")) {
			Log.note("Saving to spreadsheet "+fileName);
			ModelStrategyResultList.get().save(fileName);
		} else {
			Log.warn("Spreadsheet not saved");
		}
	}

	@Override
	public void masterChanged(boolean isMaster) {
		updateEnabled();
	}
}
