package gov.anl.gmca.syslog;

import gov.anl.gmca.gui.GUI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.StringTokenizer;

public class WdgLog extends Composite
{
	protected StyledText text = null;
	protected static final int MAX_LINES = 500;
	
	protected int lineCount = 0;
	
	// TODO: why is this line needed?
	protected static Font courierFont = new Font(GUI.getDisplay(),"Courier",14,SWT.NONE);

	public WdgLog(Composite parent, int style)
	{
		super(parent,style);
		
		setLayout(new FillLayout());
		
		text = new StyledText(this, SWT.MULTI|SWT.V_SCROLL);
		text.setFont(GUI.FONT_LOG);
	}

	public static String currentTimestamp(boolean showMs)
	{
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss"+(showMs?".SSS":""));
		String timestamp = "["+sdf.format(cal.getTime())+"]";
		return timestamp;
	}

	public void println(String s, final int style)
	{
		final ArrayList<String> lines = new ArrayList<String>();
		StringTokenizer st = new StringTokenizer(s,"\r\n");
		while (st.hasMoreTokens()) lines.add(st.nextToken());

		String timestamp = currentTimestamp(false);

		GUI.asyncExec(new Runnable(){public void run()
		{
			for (String line : lines)
			{
    			StyleRange timestampRange = new StyleRange();
    			timestampRange.start = text.getCharCount();
    			timestampRange.length = timestamp.length()+1;
    			timestampRange.foreground = GUI.getDisplay().getSystemColor(SWT.COLOR_BLUE);
    
    			StyleRange messageRange = new StyleRange();
    			messageRange.start = text.getCharCount()+timestamp.length()+1;
    			messageRange.length = line.length()+1;
    			
    			if (Log.STYLE_NOTE 		== style) 	messageRange.foreground = GUI.COLOR_DARK_GREEN;
    			if (Log.STYLE_WARNING 	== style)	messageRange.foreground = GUI.COLOR_DARK_GRAY3;
    			if (Log.STYLE_ERROR 	== style)	messageRange.foreground = GUI.COLOR_RED;

    			text.append("\n"+timestamp+" "+line);
    			text.setStyleRange(timestampRange);
    			text.setStyleRange(messageRange);
    			
    			lineCount++;
			}
			
			while (lineCount >= MAX_LINES)
			{
				String line = text.getContent().getLine(0);
				text.getContent().replaceTextRange(0, line.length()+1, "");
				lineCount--;
			}
			
			text.setTopIndex(lineCount);
		}});
	}
}
