package gov.anl.gmca.util;

import gov.anl.gmca.gui.GUI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

/**
 * Convenience functions to create SWT objects more concisely.
 * 
 * Widgets have a default background color of GUI.BGCOLOR
 * 
 */
public class WidgetFactory
{
	public static Group newGroup(Composite parent, int style, String text)
	{
		Group group = new Group(parent, style);
		group.setText(text);
		group.setFont(GUI.FONT_DEFAULT);
		group.setBackground(GUI.BGCOLOR);
		return group;
	}
	
	public static GridLayout newGridLayout(int numColumns, boolean makeColumnsEqual, int marginWidth, int marginHeight) 
	{
		GridLayout gl = new GridLayout(numColumns, makeColumnsEqual);
		gl.marginWidth = marginWidth;
		gl.marginHeight = marginHeight;
		return gl;
	}
	
	public static Button newButton(Composite parent, int style, String text)
	{
		Button button = new Button(parent, style);
		button.setText(text);
		button.setFont(GUI.FONT_DEFAULT);
		button.setBackground(GUI.BGCOLOR);
		return button;
	}
	
	public static Button newButton(Composite parent, int style, String text, Font font, Color bgColor)
	{
		Button button = new Button(parent, style);
		button.setText(text);
		button.setFont(font);
		button.setBackground(bgColor);
		return button;
	}
	
	public static Label newLabel(Composite parent, int style, String text)
	{
		Label label = new Label(parent, style);
		label.setText(text);
		label.setFont(GUI.FONT_DEFAULT);
		label.setBackground(GUI.BGCOLOR);
		return label;
	}
	
	public static Label newLabel(Composite parent, int style, String text, String tooltip)
	{
		Label label = newLabel(parent, style, text);
		label.setToolTipText(tooltip);
		return label;
	}
	
	// we can't change GTK prelight colors directly, but we can update the colors that are
	// used during prelight on the fly
	// (prelight = mouse is over the button)
	public static void setForeground(final Button button, final Color standardFG, final Color prelightFG)
	{
		button.setForeground(standardFG);
		button.addListener(SWT.MouseEnter, new Listener()
		{
			@Override public void handleEvent(Event arg0)
			{
				button.setForeground(prelightFG);
			}
		});
		button.addListener(SWT.MouseExit, new Listener()
		{
			@Override public void handleEvent(Event arg0)
			{
				button.setForeground(standardFG);
			}
		});
	}
}
