current_state = null;

$(document).ready(function() {poll();});

function poll() {
    $.ajax({
        url: "/state",
        method: 'GET',
        error: function (xhr, status, error) {
            alert('There was an error loading server state:\n' + xhr.status + ' ' + error);
        },
        success: function (data) {
            handle_state_load(data);
            setTimeout(poll, 1000);
        },
        timeout: 10000
    });
}

var devices_initialized = false;
function handle_state_load(data) {
    current_state = JSON.parse(data);
    $('#status_list').empty();
    $('#command_list').empty();

    // devices list
    if (!devices_initialized) {
        $('#PmacMotorList').empty();
        var motorNames = Object.keys(current_state.motorList);
        add_to_list($('#PmacMotorList'), motorNames);
        devices_initialized = true;
    }

    // status of current device
    if (null == selected_device) {
        $('#status_list').append('<li>No device selected</li>');
    } else {
        var motor = current_state.motorList[selected_device];
        var motorAttributes = Object.keys(motor);
        motorAttributes.forEach(function (attribute) {
            var value = motor[attribute];
            var floatVal = parseFloat(value);
            if (!isNaN(floatVal)) value = floatVal.toFixed(3);
            $('#status_list').append('<li>' + attribute + ': ' + value + '</li>');
        });
    }

    // commands for current device
    var commandList = current_state.motorListCommands;
    for (var i=0; i<commandList.length; i++) {
        var command = commandList[i];
        $('#command_list').append('<li>'+command[0]+'</li>');
        if (command.length>1) {
            var cmd_ul = $('<ul></ul>');
            for (var j=1;j<command.length;j++) {
                var argument_tokens = command[j].split(',');
                cmd_ul.append('<li>'+argument_tokens[0]+'</li>');
            }
            $('#command_list').append(cmd_ul);
        }
    }

    // last load time display
    var last_load = moment().format('h:mm:ss a');
    $('#last_update_span').text('Last update: '+last_load);
}

function add_to_list(list_element, items) {
    items.forEach(function(selected_item){
        var item_element = $('<li id="'+selected_item+'">'+selected_item+'</li>');
        list_element.append(item_element);
        item_element.click(function(){select_device(selected_item)});
        item_element.css('cursor','pointer');
        if (selected_device==selected_item) { item_element.css('font-weight','bold'); }
    });
}

var selected_device = null;
function select_device(device_name) {
    if (null != selected_device) {
        $('#'+selected_device).css('font-weight','normal');
    }

    console.log('selected device '+device_name);
    selected_device = device_name;
    $('#'+device_name).css('font-weight','bold');

    poll();
}
