#include <jni_md.h>
#include <jni.h>
#include <dlfcn.h>
#include <string>
#include <iostream>
#include <dlfcn.h>
#include <fstream>
#include <vector>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <dirent.h>
#include <algorithm>
#include <unistd.h>
using namespace std;

#define JVM_PATH1   "/usr/java/latest/jre/lib/i386/client/libjvm.so"
#define JVM_PATH2   "/usr/java/latest/jre/lib/amd64/server/libjvm.so"
#define JVM_PATH3   "/usr/java/latest/lib/server/libjvm.so"
#define JVM_PATH4   "/mnt/software/java/jdk/lib/server/libjvm.so"

void destroyVM(JNIEnv *env, JavaVM *jvm)
{
   if (env->ExceptionOccurred())
   {
      env->ExceptionDescribe();
   }
   jvm->DestroyJavaVM();
}

typedef jint (JNICALL *pfnCreateJVM)(JavaVM** ppJvm, void** ppEnv, void* args);

// find all .jar files in subdirectories of the given path
vector<string> getLibs(string libDir)
{
  vector<string> libList;
  
  DIR *dir = opendir(libDir.c_str());
  struct dirent *pDir;
  while ( (pDir=readdir(dir)) != NULL)
  {
    string dirEntry(pDir->d_name);
    if (0==dirEntry.compare(".") || 0==dirEntry.compare("..")) // ignore these
    {
      continue;
    }

	string fullPath = libDir+"/"+dirEntry;
	DIR *tempDir = opendir(fullPath.c_str());
    if (tempDir) // call subdirs recursively
    {
	  closedir(tempDir);

      vector<string> subDirLibs = getLibs(libDir+"/"+dirEntry);
      libList.insert(libList.end(), subDirLibs.begin(), subDirLibs.end());
    }
	else if // add files if they end in ".jar"
    (
      dirEntry.length() > 4 &&
      0 == dirEntry.substr(dirEntry.length()-4).compare(".jar")
    )
    {
      libList.push_back(libDir+"/"+dirEntry);
    }
  }
  closedir(dir);
  
  return libList;
}

int run(string mainClassPath, int argc, char **argv)
{
	// load the JVM library
	bool mode64 = false;
	void *         libjvm = dlopen(JVM_PATH1,RTLD_NOW);
	if (!libjvm)
	{
		// couldn't load 32-bit JVM, try 64-bit
		libjvm = dlopen(JVM_PATH2,RTLD_NOW);
		mode64 = true;
	} 

        if (!libjvm) {
		libjvm = dlopen(JVM_PATH3,RTLD_NOW);
		mode64 = true;
        }


        if (!libjvm) {
        	const char* java_home = std::getenv("JAVA_HOME");
        	string jvm_path = string(java_home) + "/lib/server/libjvm.so";
		libjvm = dlopen(jvm_path.c_str(),RTLD_NOW);
		mode64 = true;
	}

	if (!libjvm)
	{
		cerr << "Error loading libjvm.so: " << dlerror() << endl;
		return 0;
	}

	// determine the exe path
	char exePathBuf[500];
	int length = readlink("/proc/self/exe", exePathBuf, sizeof(exePathBuf)-1);
	exePathBuf[length]='\0';
	string exePath(exePathBuf);

	// directory names
	size_t lastSlashIndex = exePath.find_last_of("/");
	string appDir(exePath.substr(0,lastSlashIndex));
	string javaDir(appDir+"/java");
	string libDir(javaDir+"/lib");
	string libDir32(javaDir+"/lib32");
	string libDir64(javaDir+"/lib64");
	string hdf5LibDir(libDir+"/hdf5");
	string hdf5PluginDir(hdf5LibDir+"/plugin");

	// GTK settings
	string config_home=string("XDG_CONFIG_HOME=");
	config_home += javaDir;
	config_home += "/xdg_config";
	putenv(const_cast<char*>(config_home.c_str()));
	unsetenv("SWT_GTK3"); // if set to 0, GTK2 would be used

	// Grid engine environment ////////////////////////////////////////////////////
	putenv(const_cast<char*>("SGE_ROOT=/opt/sge"));
	putenv(const_cast<char*>("SGE_QMASTER_PORT=6444"));
	putenv(const_cast<char*>("SGE_EXECD_PORT=6445"));
	//putenv(const_cast<char*>((string("HDF5_PLUGIN_PATH=")+hdf5PluginDir).c_str()));
	
/*
	string setLibraryPathString = 
		string("LD_LIBRARY_PATH=")+
		string(getenv("LD_LIBRARY_PATH"))+
	        //":"+hdf5LibDir+
		string(":/opt/sge/lib/lx-amd64");

	putenv(const_cast<char*>(setLibraryPathString.c_str()));
	
	cout << setLibraryPathString << endl;
*/
	
	// find libraries
	vector<string> libPaths = getLibs(libDir);
	vector<string> modeSpecificLibs = getLibs(mode64?libDir64:libDir32);
	libPaths.insert(libPaths.end(), modeSpecificLibs.begin(), modeSpecificLibs.end());
	sort(libPaths.begin(), libPaths.end());

	// construct the classpath
	string classPathArg("-Djava.class.path=");
	for (vector<string>::iterator it = libPaths.begin(); it != libPaths.end(); it++)
	{
	  classPathArg += *it + ":";
	}
    classPathArg += javaDir+"/bin";

	// set java.library.path
	string libPathArg("-Djava.library.path="+hdf5LibDir);
	string drmaa("-Djava.library.path=/opt/sge/lib/lx-amd64");

    // look for debug mode
	int debugMode = 0;
	for (int i=0;i<argc;i++) {
		if (0==strncmp(argv[i],"java-debug",10)) {
			debugMode = 1;
		}
	}

	// construct the VM arguments
	int nOptions = debugMode ? 8 : 7;
	JavaVMOption* vmOptions = new JavaVMOption[nOptions];
	string appHomeArg = string("-Dapplication.home=")+appDir;
	vmOptions[0].optionString = const_cast<char*>(appHomeArg.c_str());
	vmOptions[1].optionString = const_cast<char*>(classPathArg.c_str());
	vmOptions[2].optionString = const_cast<char*>(libPathArg.c_str());
	vmOptions[3].optionString = const_cast<char*>("-client");
	vmOptions[4].optionString = const_cast<char*>("-Xmx512M");
	vmOptions[5].optionString = const_cast<char*>("-Dorg.eclipse.swt.internal.gtk.cairoGraphics=false");
	vmOptions[6].optionString = const_cast<char*>(drmaa.c_str());
	if (debugMode) vmOptions[7].optionString = const_cast<char*>("-agentlib:jdwp=transport=dt_socket,server=y,suspend=n,address=localhost:5009");

	JavaVMInitArgs vm_args;
	vm_args.version 	= JNI_VERSION_1_6;
	vm_args.options 	= vmOptions;
	vm_args.nOptions = nOptions;
	vm_args.ignoreUnrecognized = JNI_TRUE;

	// create the JVM
	pfnCreateJVM fp_createJavaVM = NULL;
	fp_createJavaVM = (pfnCreateJVM)dlsym(libjvm, "JNI_CreateJavaVM");
	if (NULL == fp_createJavaVM)
	{
		cerr << "Error getting the address for JNI_CreateJavaVM()" << endl;
		return 0;
	}

	JNIEnv *env;
	JavaVM *jvm;
	jint VMStartupReturnValue = fp_createJavaVM(&jvm, (void**)&env, &vm_args);
	if (VMStartupReturnValue < 0)
	{
	   cerr << "Error creating VM" << endl;
	   destroyVM(env, jvm);
	   return 0;
	}

	// find the startup class
	jclass mainClass = env->FindClass(mainClassPath.c_str());
	if (mainClass == NULL)
	{
	  cerr << "couldn't find startup class" << endl;
	  destroyVM(env, jvm);
	  return 0;
	}

	// find the startup method
	string mainName = "main";
	string mainType = "([Ljava/lang/String;)V";
	jmethodID mainID = env->GetStaticMethodID(mainClass, mainName.c_str(), mainType.c_str());
	if (mainID == NULL)
	{
	   cerr << "Couldn't find startup method " << mainClassPath << "/" << mainName << endl;
	   destroyVM(env, jvm);
	   return 0;
	}

	// create the command-line argument array
	jclass jclassString = env->FindClass("java/lang/String");
	jobjectArray argArray = env->NewObjectArray(argc-1, jclassString, NULL);
	if (argArray == NULL)
	{
		cerr << "Error creating command-line arguments";
		destroyVM(env, jvm);
		return 0;
	}

	// set (pass) the arguments from the command line
	for (int i=1; i<argc; i++)
	{
		jstring arg = env->NewStringUTF(argv[i]);
		if (arg == NULL)
		{
			cerr << "Error creating command-line arguments";
			destroyVM(env, jvm);
			return 0;
		}
		env->SetObjectArrayElement(argArray, i-1, arg);
	}

	// call Main
	env->CallStaticVoidMethod(mainClass, mainID, argArray);

	// detach the main thread before exiting
	if (jvm->DetachCurrentThread() != 0)
	{
	   cerr << "Couldn't detach main thread";
	}

	// this call blocks until non-daemon threads exit
	jvm->DestroyJavaVM();
	
	return 0;
}
