#!/bin/csh
#
# Running XDS in parallel, works only on 23ID beamlines @APS
# using scaling part of autoxds script from SSRL by Ana Gonzalez ana@slac.stanford.edu
#
# Usage : scale.csh <file_prefix> 
#
# Ver 1 : Sudhir Babu Pothineni, 
# GM/CA CAT, APS, Argonne National Laboratory
# Ver 2 : runs fast_dp 09/11/2012
#
# proceed to scaling if XDS_ASCII.HKL available

set scale = $1
set prefix = $2
set params = $3
set xdsfilepath = $4
set scriptDir = `dirname $0`

echo
echo "Transforming output to mtz with POINTLESS"
if ($params == 18) then
        cd *GMCAproc*
endif

mkdir POINTLESS_AIMLESS
cd POINTLESS_AIMLESS
if ($params == 18) then
	echo "pointless -c XDSIN $xdsfilepath HKLOUT ${prefix}_xds.mtz <<EOF > ${prefix}_pointless.log" > pointless.sh
else
	echo "pointless XDSIN $xdsfilepath HKLOUT ${prefix}_xds.mtz <<EOF > ${prefix}_pointless.log" > pointless.sh
endif

echo "END" >> pointless.sh
echo "EOF" >> pointless.sh

source pointless.sh

echo
echo "POINTLESS finished. See ${prefix}_pointless.log"

echo
echo `date`
echo "merging data and generating statistics with $scale"

echo "$scale hklin ${prefix}_xds.mtz hklout ${prefix}_$scale.mtz \\" > $scale.sh
echo " <<EOF> ${prefix}_$scale.log" >> $scale.sh
echo "run 1 all" >> $scale.sh
echo "scales constant" >> $scale.sh
echo "anomalous on" >> $scale.sh
echo "cycles 0" >> $scale.sh
echo "sdcorrection norefine full 1.0 0.0 0.0" >> $scale.sh
echo "END" >> $scale.sh
echo "EOF" >> $scale.sh

source $scale.sh 

#write the logfile
echo `pwd` > ../../subdirfile
if (-e ../XDS_REINTEGRATE/GMCAproc.LOG) then
	cp -p ../XDS_REINTEGRATE/GMCAproc.LOG .
	cp -p ../XDS_REINTEGRATE/XDS_ASCII.HKL .
else
	cp -p ../XDS/GMCAproc.LOG .
	cp -p ../XDS/XDS_ASCII.HKL .
endif

## applying 2 sigma cutoff
#set isigma = 2
#echo "get resolution using $scriptDir/get_resolution_$scale.awk"
#set  resol = `awk -v isigma=$isigma -f $scriptDir/get_resolution_$scale.awk  ${prefix}_$scale.log`
#echo "$scale hklin ${prefix}_xds.mtz hklout ${prefix}_$scale.mtz \\" > $scale.sh
#echo " <<EOF> ${prefix}_$scale.log" >> $scale.sh
#echo "run 1 all" >> $scale.sh
#echo "anomalous on" >> $scale.sh
#echo "cycles 0" >> $scale.sh
#echo "sdcorrection norefine full 1.0 0.0 0.0" >> $scale.sh
#echo "RESOL $resol " >> $scale.sh
#echo "END" >> $scale.sh
#echo "EOF" >> $scale.sh

#source $scale.sh 

cp -p ${prefix}_$scale.log $scale.log

echo "$scale finished. See ${prefix}_$scale.log"

echo `date`

if ( -e ${prefix}_$scale.mtz ) then
	echo "Calculating structure factors with CTRUNCATE"

	echo "/mnt/software/px/CCP4/ccp4/bin/ctruncate -mtzin ${prefix}_$scale.mtz -mtzout ${prefix}_ctruncate.mtz -colano '/*/*/[I(+),SIGI(+),I(-),SIGI(-)]' <<EOF> ${prefix}_ctruncate.log" > ctruncate.sh
	echo "END" >> ctruncate.sh
	echo "EOF" >> ctruncate.sh

	source ctruncate.sh

	echo "CTRUNCATE finished. See ${prefix}_ctruncate.log"
endif

echo `date`
exit
