#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Wed Oct 26 10:36:19 2016

@author: qxu
"""
from __future__ import absolute_import
import logging
import os

logger = logging.getLogger(__name__)


class FileSweep(object):
    """ a fileset that represents a continuous series of image files, created
    by the generate_sweeps class"""
    count = 0  # NB variable not thread/multiprocessing safe
    
    def __init__(self, filelist, directory):
        self.__filelist = filelist
        self.__abs_path = directory
        self.nfiles = len(filelist)
        FileSweep.count += 1
        self.sweep_id = f"s{FileSweep.count:03d}_{self.get_prefix()}"

    def get_name_template(self, marker="#"):
        file1 = self.__filelist[0]
        return file1[0] + len(file1[1][:-1])*marker + '.' + file1[-1] 

    def get_seqlength(self):
        return len(self.__filelist[0][1]) - 1

    def get_filelist(self):
        return self.__filelist
    
    def get_absdir(self):
        return self.__abs_path

    def get_prefix(self):
        return self.__filelist[0][0]

    def get_suffix(self):
        return self.__filelist[0][-1]
        
    def get_all_files(self, abs_path=True):
        if abs_path:
            d = self.__abs_path
        else: 
            d = ''
        return [os.path.join(d, ''.join(f)) for f in self.__filelist ]
                
    def get_start_file_number(self):
        if 'master' in self.__filelist[0][1][:-1]:
            return 1
        return int(self.__filelist[0][1][:-1])

    def get_last_file_number(self):
        return int(self.__filelist[-1][1][:-1])

    def get_file_number_range(self):
        return self.get_start_file_number(), self.get_last_file_number()

    def get_sweep_id(self):
        start, end = self.get_file_number_range()
        prefix = self.get_prefix()
        return "{}{:05d}_{:05d}".format(prefix, start, end)

    def get_imageSet(self, mode="collect"):
        if mode == "screen":
            # for screening images: 1,2,3:prefix
            return "{}:{}".format(",".join(str(i) for i in self.get_all_file_numbers()), self.get_prefix())
        elif mode == "prefix":
            # for screening images: prefix:1,2,3
            return "{}:{}".format(self.get_prefix(), ",".join(str(i) for i in self.get_all_file_numbers()))
        elif mode == "redis":
            return "bluice:strategy:table#{}:{}:{}".format(self.get_prefix(),
                                                           self.get_start_file_number(),
                                                           self.get_last_file_number())
        else:
            # for collection images: 1:3:prefix
            return "{}-{}:{}".format(self.get_start_file_number(),
                                     self.get_last_file_number(),
                                     self.get_prefix())

    def get_all_file_numbers(self):
        return [int(x[1][:-1]) for x in self.__filelist if 'master.' not in x[1]]

    def get_xds_spot_ranges_for_screening(self):
        return [(i, i) for i in self.get_all_file_numbers()]

    def is_screening(self):
        if self.nfiles < 5:
            return True

        return False

    def get_first_file(self, abs_path=True):
        return self.get_file(0, abs_path)
        
    def get_file(self, i, abs_path=True):
        if i >= self.nfiles:
            return None
    
        if abs_path:
            d = self.__abs_path
        else: 
            d = ''
        
        return os.path.join(d, ''.join(self.__filelist[i]))
        
    def is_hdf5(self):
        return True if self.get_suffix() in ['h5', 'hdf5'] else False

    def is_cbf(self):
        return True if self.get_suffix() in ['cbf', 'CBF'] else False
    
    def get_id(self):
        return self.sweep_id

    def set_id(self, sid):
        self.sweep_id = sid

    def print_summary(self):
        logger.info("-" * 80)
        logger.info("sweep id {}".format(self.get_id()))
        logger.info("first image = {}".format(self.get_first_file()))
        logger.info("name template = {}".format(self.get_name_template(marker='#')))
        logger.info("abs dir = {}".format(self.get_absdir()))
        logger.info("nfiles = {}".format(self.nfiles))
        if self.is_cbf():
            logger.info("image format: cbf")
        elif self.is_hdf5():
            logger.info("image format: hdf5")
        else:
            logger.info("image format: other.")

    def __str__(self):
        return "file_sweep {}: {} {} {}".format(self.sweep_id,
                                                self.__filelist,
                                                self.get_first_file(),
                                                self.is_cbf())
