from __future__ import absolute_import

import logging
import os

from ..common import utils

logger = logging.getLogger(__name__)


class FileParser:
    def __init__(self, filename=None, lines=None):
        self.lines = []
        self.versions = []
        if filename and os.path.exists(filename):
            with open(filename, "r") as fh:
                self.lines = fh.readlines()
        elif lines:
            self.lines = lines

    def get_line(self, i):
        return self.lines[i]

    def get_lines(self, startIndex=0, endIndex=-1):
        if endIndex == -1:
            return self.lines[startIndex:]
        else:
            return self.lines[startIndex:endIndex + 1]

    def get_lines_startswith(self, str):
        return [line for line in self.lines if line.startswith(str)]

    def get_lines_aroundi(self, i, nLines, offset):
        start = i + offset
        end = start + nLines
        return self.lines[start:end]

    def get_lines_around(self, text,
                         nLines=1,
                         offset=0,
                         iMatch=1):
        # extract nLines lines around text with an offset
        nmatches = 0
        last = None
        for i, line in enumerate(self.lines):
            if text in line:
                nmatches += 1
                if nmatches == iMatch:
                    return self.get_lines_aroundi(i, nLines, offset)
                else:
                    last = self.get_lines_aroundi(i, nLines, offset)
        return last

    def get_lines_after(self, text,
                        nLines=1,
                        includeCurrentLine=False,
                        iMatch=1):
        offset = 0 if includeCurrentLine else 1
        return self.get_lines_around(text,
                                     nLines=nLines,
                                     offset=offset,
                                     iMatch=iMatch)

    def get_lines_contain(self, text):
        return [line for line in self.lines if text in line]

    def get_line_nums_contain(self, text):
        return [i for i, line in enumerate(self.lines) if text in line]

    def get_lines_not_contain(self, text):
        return [line for line in self.lines if text not in line]

    def get_blocks_between(self, text1, text2, skip1=0, skip2=0, iMatch=1, EOF=False):
        """
        # find text block defined by two anchor markers,
        # by default, only return first match, if iMatch = -1 will give
        # all matches
        # NB: if text2 is not present but EOF is True, return the block between text1 to the EOF
        # return: list of list
        """
        # last = None
        nmatches, i = 0, 0
        all_matches = []
        while i < len(self.lines):
            if text1 in self.lines[i]:
                j = i + 1
                while j < len(self.lines):
                    if text2 in self.lines[j] or (j == len(self.lines) - 1 and EOF):
                        nmatches += 1
                        all_matches.append(self.lines[i + skip1:j + 1 - skip2])
                        if nmatches == iMatch:
                            return [self.lines[i + skip1:j + 1 - skip2]]
                        # else:
                        #    last = self.lines[i+skip1:j+1-skip2]
                        i = j
                        break
                    j += 1
            i += 1
        return all_matches

    def get_numbers_from_lines(self, text, nLines=1, offset=0, iMatch=-1):
        lines = self.get_lines_around(text, nLines=nLines, offset=offset, iMatch=iMatch)
        res = []
        for line in lines:
            res.extend(utils.getNumbers(line))
        return res

    def __str__(self):
        return "".join(self.lines)


if __name__ == '__main__':
    pass
