from __future__ import absolute_import

import os

from .fileopener import FileOpener
from ..common.utils import getNumbers
from .spotxds import cal_exclude_ranges, merge_ranges

import logging

logger = logging.getLogger(__name__)


class IntegrationParser(FileOpener):
    def __init__(self, wdir=".", filename='INTEGRATE.LP'):
        self.log = filename
        super(IntegrationParser, self).__init__(wdir, filename)

    def get_refined_params_for_beam_reflections(self):
        """ get values of BEAM_DIVERGENCE, BEAM_DIVERGENCE_E.S.D. REFLECTING_RANGE, REFLECTING_RANGE_E.S.D."""
        res = self.parser.get_lines_after("SUGGESTED VALUES FOR INPUT PARAMETERS", nLines=2)
        out = ""

        if not res:
            return None

        for e in res:
            li = e.split()
            li.insert(2, ';')
            out += ''.join(li)
            out += ';'

        return out.strip(';') if out else None

    def get_redo_suggestions(self):
        """find # on which the integration fails, return (start, end) or new DATA_RANGE"""
        lines = self.parser.get_lines_contain("PROCESSING OF IMAGES ")
        if lines:
            start = getNumbers(lines[0])[0]
            last_batch_start = getNumbers(lines[-1])[0]
            last_batch_end = getNumbers(lines[-1])[-1]

            new_end = last_batch_end
            if self.parser.get_line_nums_contain(
                    "!!! ERROR !!! AUTOMATIC DETERMINATION OF SPOT SIZE PARAMETERS HAS FAILED."):
                new_end = int(last_batch_start) - 1
                print(f"new integration range: {start} {new_end}")

            out = f"DATA_RANGE= {start} {new_end}"
            opt = self.get_refined_params_for_beam_reflections()
            if opt:
                out += ";" + opt

            return out

    def get_exclude_range(self, max_threshold=1, start_threshold=0, min_frames=2, min_reject=0, min_gap=2, max_reject=50):
        frame_nstrong = self.parse_integrate_lp()
        ranges = cal_exclude_ranges(frame_nstrong,
                                    min_frames=min_frames,
                                    min_reject=min_reject,
                                    max_reject=max_reject,
                                    max_threshold=max_threshold,
                                    start_threshold=start_threshold)
        if ranges:
            return merge_ranges(ranges, min_gap=min_gap)
        else:
            return None

    def parse_integrate_lp(self):
        lp_file = os.path.join(self.wdir, self.log)
        frame_nstrong = []

        with open(lp_file, "r") as integrate_lp:
            lines = integrate_lp.readlines()
            in_section = False
            for line in lines:
                if "IMAGE IER  SCALE     NBKG NOVL NEWALD NSTRONG  NREJ   SIGMAB   SIGMAR" in line:
                    in_section = True
                    continue

                if "REFLECTIONS ACCEPTED FOR REFINEMENT" in line or len(line) < 10:
                    in_section = False
                    continue

                if in_section:
                    fields = line.split()
                    if len(fields) != 10:
                        print("parsing error: ", line)
                        continue

                    frame_no = fields[0]
                    n_strong = fields[-4]
                    if fields == "-1":
                        n_strong = 0

                    frame_nstrong.append([frame_no, int(n_strong)])

        # print(sorted(frame_npixels, key=itemgetter(0)))
        return frame_nstrong


if __name__ == '__main__':
    ilp = IntegrationParser(wdir='/mnt/beegfs/qxu/test1/gmcaproc/s001_196802_1_00001_00420/XDS-p1/')
    ilp.get_redo_suggestions()