from __future__ import absolute_import

import logging
from .fileopener import FileOpener
from ..common.utils import getNumbers

logger = logging.getLogger(__name__)


class MatthewsCoefParser(FileOpener):
    def __init__(self, wdir=".", filename='matthews_coef.log'):
        super(MatthewsCoefParser, self).__init__(wdir, filename)
        self.table = None
        self.estimated_asu = None
        self.nres = None

    def get_table(self):
        nresline = self.parser.get_lines_contain('Data line--- nres')
        self.nres = getNumbers(nresline[0])[0] if nresline else 25

        blocks = self.parser.get_blocks_between(text1='Nmol/asym  Matthews Coeff  %solvent       P(tot)',
                                                text2='</pre>', skip2=3)
        if blocks:
            self.table = blocks[0]

        return ''.join(self.table) +"**NB**: assuming {} aa per molecule".format(self.nres)

    def get_estimated_asu_content(self):
        if not self.table:
            self.get_table()

        maxdiff = 100
        nmol_at_50solvent = None
        if self.table:
            for line in self.table:
                fields = line.split()
                if len(fields) == 4:
                    nmol, matthews_coeff, solvent, prob = fields
                    diff = abs(float(solvent) - 50)
                    if diff < maxdiff:
                        maxdiff = diff
                        nmol_at_50solvent = nmol

        if not nmol_at_50solvent:
            logger.warning("matthew calculation produced empty results, probably small molecule or peptides?")
            return 25

        return int(nmol_at_50solvent) * int(self.nres)

if __name__ == '__main__':
    p = MatthewsCoefParser('/home/qxu/work/quickProcess', filename='XPLAN.LP.wREF')
    p.get_table()
