from __future__ import absolute_import

import matplotlib
import math
import os

import logging
logger = logging.getLogger(__name__)

matplotlib.use('Agg')
import matplotlib.pyplot as plt


def make_xy_plot(wdir,
                 x1, y,
                 x1label="Image No",
                 ylabel="",
                 outfig="fig.png",
                 markers=None):
    # change default font size
    matplotlib.rcParams.update({'font.size': 9})

    fig = plt.figure()
    fig.set_tight_layout(True)
    fig.set_dpi(100)
    fig.set_size_inches(3.2, 2.4)

    ax1 = fig.add_subplot(111)
    ax1.plot(x1, y, 'r-o', markersize=4)
    ax1.set_xlabel(x1label)
    ax1.set_ylabel(ylabel)

    if markers:
        for m in markers:
            ax1.axhline(y=m, color='b', alpha=0.2)

    ax1.grid(True)

    # plt.show()
    plt.savefig(os.path.join(wdir, outfig))
    plt.close('all')


def make_plot_vs_resolution(wdir, x1, y,
                            title="",
                            x1label=r"1/Resolution^2",
                            x2label=r"Resolution ($\AA$)",
                            ylabel="",
                            outfig="fig.png",
                            markers=None):
    # change default font size
    matplotlib.rcParams.update({'font.size': 9})

    fig = plt.figure()
    fig.set_tight_layout(True)
    fig.set_dpi(100)
    fig.set_size_inches(3.2, 2.4)

    ax1 = fig.add_subplot(111)
    ax1.plot(x1, y, 'r-o', markersize=4)
    ax1.set_xlabel(x1label)
    ax1.set_ylabel(ylabel)

    ax2 = ax1.twiny()
    xticks = ax1.get_xticks()
    x2ticks_label = []
    # print xticks

    for xt in xticks:
        if abs(xt) > 0.001:
            x = str(round(1.0 / math.sqrt(abs(xt)), 1))
        else:
            x = "inf"
        x2ticks_label.append(x)

    ax2.set_xlim(ax1.get_xlim())
    # ax2.set_xticks(xticks)
    ax2.set_xticklabels(x2ticks_label)
    ax2.set_xlabel(x2label)

    if markers:
        for m in markers:
            ax1.axhline(y=m, color='b', alpha=0.2)

    # plt.grid(b=True, which='major', color='k', linestyle='--')
    ax1.grid(True)

    # plt.show()
    plt.savefig(os.path.join(wdir, outfig))
    plt.close('all')


if __name__ == "__main__":
    x = [1,2,3,4,5]
    y = [1,2,3,4,5]
    make_xy_plot(".", x,y, x1label="x", ylabel="y")