from __future__ import absolute_import
from collections import Counter
import logging

logger = logging.getLogger(__name__)

# template for jquery dataTables plugin
template = \
    '''
    <html>
    <head>
        <title>Results</title>
    
        <link rel="stylesheet" type="text/css"
            href="https://cdn.datatables.net/1.10.16/css/jquery.dataTables.min.css">
        <link rel="stylesheet" type="text/css"
            href="https://cdn.datatables.net/responsive/2.2.0/css/responsive.dataTables.min.css">
        <link rel="stylesheet" type="text/css"
            href="https://cdn.datatables.net/buttons/1.4.2/css/buttons.dataTables.min.css">
        <link rel="stylesheet" type="text/css"
            href="https://cdn.datatables.net/fixedheader/3.1.3/css/fixedHeader.dataTables.min.css">
            
        <link rel="stylesheet" type="text/css" href="https://cdn.datatables.net/scroller/1.4.3/css/scroller.dataTables.css"/>
        <link rel="stylesheet" type="text/css" href="https://cdn.datatables.net/select/1.2.3/css/select.dataTables.css"/>
        <link rel="stylesheet" type="text/css" href="https://cdn.datatables.net/rowgroup/1.0.2/css/rowGroup.dataTables.min.css"/>
        <link rel="stylesheet" type="text/css" href="https://cdn.datatables.net/rowreorder/1.2.3/css/rowReorder.dataTables.min.css"/>
        
    
        <script type="text/javascript" language="javascript" 
            src="https://code.jquery.com/jquery-1.12.4.js">
        </script>
    
        <script type="text/javascript" language="javascript"
            src="https://cdn.datatables.net/1.10.16/js/jquery.dataTables.min.js">
        </script>
    
        <script type="text/javascript" language="javascript"
            src="https://cdn.datatables.net/responsive/2.2.0/js/dataTables.responsive.min.js">
        </script>
    
        <script type="text/javascript" language="javascript"
            src="https://cdn.datatables.net/buttons/1.4.2/js/dataTables.buttons.min.js">
        </script>
    
        <script type="text/javascript" language="javascript"
            src="https://cdn.datatables.net/buttons/1.4.2/js/buttons.colVis.min.js">
        </script>
        
        <script type="text/javascript" language="javascript"
            src="https://cdn.datatables.net/fixedheader/3.1.3/js/dataTables.fixedHeader.min.js">
        </script>	
        
        <script type="text/javascript" src="https://cdn.datatables.net/scroller/1.4.3/js/dataTables.scroller.js"></script>
        <script type="text/javascript" src="https://cdn.datatables.net/select/1.2.3/js/dataTables.select.js"></script>
        
        <script type="text/javascript" src="https://cdn.datatables.net/rowgroup/1.0.2/js/dataTables.rowGroup.min.js"></script>
        <script type="text/javascript" src="https://cdn.datatables.net/rowreorder/1.2.3/js/dataTables.rowReorder.min.js"></script>
    
        <!---
        
        <script type="text/javascript" src="http://getfirebug.com/releases/lite/1.2/firebug-lite-compressed.js"></script>
    
        --->
    
        <style>
            #dataprocstatus tbody tr:hover {
                background-color: yellow;
            }
            
            button.dt-button, div.dt-button, a.dt-button {
                background: #b8d3ee;
            }
        </style>
    
        <script type="text/javascript">
            $.fx.off = true; // animation off

    
            $(document).ready(
                    function() {
                        var processTable = $('#dataprocstatus').DataTable(
                                {
                                    responsive : true,
                                    dom : 'Blfrtip',
                                    paging : false,
                                    search : true,
                                    ordering: true,
                                    deferRender: false,
                                    stateSave: true,
                                    pageLength : -1,
                                    columnDefs: [ {'className': 'dt-center', 'targets': '_all'} ],
                                    lengthMenu : [[ 10, 25, 50, -1],	[ '10 rows', '25 rows', '50 rows', 'All rows'] ],
                                    fixedHeader: {
                                        header: true,
                                        footer: true
                                    },
                                    
                                    buttons : [
                                        'pageLength',
                                        {
                                            extend : 'colvis',
                                            text : 'Show/Hide Columns',
                                            collectionLayout : 'fixed two-column',
                                        },
                                    ],
                                });
                    });
        </script>
    </head>
    
    <body>
    <h2>Summary of Sweep/Run Processing Results</h2>
    <table id="dataprocstatus" class="display responsive nowrap" cellspacing="0" width="100%"> 
    {table}
    </table>
    </body>
    </html>
    '''


class Table:
    def __init__(self):
        self.nrow = 0
        self.ncol = 0
        self.colnames = []
        self.rownames = []
        self.data = []
        self.title = 'a generic table'

    def set_title(self, title):
        self.title = title

    def set_data(self, data):
        self.data = data

    def get_data(self):
        return self.data

    def set_headers(self, headers):
        if isinstance(headers, (list, tuple)):
            self.colnames = headers
        elif isinstance(headers, str):
            self.colnames = headers.split()
        else:
            logger.error("unknown headers, using a tuple, list, or space separated string")

        self.ncol = len(self.colnames)

    def add_data_by_line(self, line, hasRowname=False):
        alist = line.split()
        self.add_data_by_a_list(alist, hasRowname=hasRowname)

    def add_data_by_a_list(self, alist, hasRowname=False):
        if self.ncol < 1:
            logger.error("Table header must set before adding data")

        if hasRowname:
            self.rownames.append(alist[0])
            data = alist[1:]
        else:
            self.rownames.append(self.nrow)
            data = alist

        if len(data) != self.ncol:
            logger.error("Number of columns does not match header when adding data.")

        self.data.append(data)
        self.nrow += 1

    def get_table_col_by_name(self, colname):
        for i, name in enumerate(self.colnames):
            if colname in name:
                return [row[i] for row in self.data]
        return None

    def get_table_col_by_index(self, index):
        if index < self.ncol:
            return [row[index] for row in self.data]
        else:
            return None

    def get_table_row_by_index(self, index):
        if index < self.nrow:
            return self.data[index]
        else:
            return None

    def set_table_by_lines(self, lines, header=True, hasRowNames=False):
        # generate a table from the provided lines, assuming that the most table has a common # of fields
        # that occurs most frequently in the provided text
        tmpT = []
        for l in lines:
            tmpT.append(l.split())

        # most common # of fields
        self.ncol = Counter([len(e) for e in tmpT]).most_common()[0][0]

        # remove lines that may probably not belong to the table
        tmpT = [e for e in tmpT if len(e) == self.ncol]
        if header:
            header = tmpT[0]
            body = tmpT[1:]
        else:
            header = [str(i) for i in range(self.ncol)]
            body = tmpT

        self.set_headers(header)
        for line in body:
            self.add_data_by_a_list(line, hasRowname=hasRowNames)

    def format(self, sep='\t', tablefmt='simple', numalign=None, stralign=None):
        try:
            import tabulate
            return tabulate.tabulate(self.data, headers=self.colnames,
                                     tablefmt=tablefmt,
                                     numalign=numalign,
                                     stralign=stralign)
        except ImportError:
            output = [self.title, "-" * 80, sep.join(self.colnames)]
            output.extend(sep.join(row) for row in self.data)
            return '\n'.join(output)

    # generate html string styled by jquery/dataTables
    def htmlify(self):
        html_no_table_tags = self.format(tablefmt='html').replace("<table>", "").replace("</table>", "")
        return template.replace("{table}", html_no_table_tags)

    def print(self):
        logger.info(self.format(numalign='right', stralign='right'))

    def __str__(self):
        return str(self.data)
