
from __future__ import absolute_import
import logging
import os, shutil

logger = logging.getLogger(__name__)


class Xparm(object):
    """ deal with reading and rewrite XPARM.XDS and GXPARM.XDS"""
    def __init__(self):
        pass

    # need to rewrite this later
    def rewrite_gxparm_xds_with_new_params(self,
                                           new_start_angle,
                                           new_distance,
                                           new_wavelength,
                                           new_osc,
                                           src_dir='.',
                                           gxparm_file='GXPARM.XDS',
                                           dest_dir='.',
                                           new_filename='XPARM.XDS'):
        gxparm = os.path.join(src_dir, gxparm_file)

        if os.path.isfile(gxparm):
            dest_dir = os.path.abspath(dest_dir) if not os.path.isabs(dest_dir) else dest_dir
            if not os.path.isdir(dest_dir):
                os.makedirs(dest_dir)

            gxlines = open(gxparm).readlines()

            old_osc = float(gxlines[1][20:30])
            old_start_angle = float(gxlines[1][6:20])
            old_distance = float(gxlines[8][30:])
            old_wavelength = float(gxlines[2][:15])

            change = False
            if abs(float(new_start_angle) - old_start_angle) > 0.1:
                logger.info("start angle different from reference data")
                change = True

            if abs(float(new_distance) - old_distance) > 3.0:
                logger.info("distance different from reference data")
                change = True

            if abs(float(new_osc) - old_osc) > 0.01:
                logger.info("osc angle different from reference data")
                change = True

            if abs(float(new_wavelength) - old_wavelength) > 0.0001:
                logger.info("wavelength changed compared to the reference data")
                change = True

            if change:
                new_angle = '{:14.4f}'.format(new_start_angle)
                new_wavelength = '{:15.6f}'.format(new_wavelength)
                new_distance = '{:15.6f}'.format(new_distance)
                new_osc = '{:10.4f}'.format(new_osc)

                gxlines[1] = gxlines[1][:6] + new_angle + new_osc + gxlines[1][30:]
                gxlines[2] = new_wavelength + gxlines[2][15:]
                gxlines[8] = gxlines[8][:30] + new_distance + "\n"
                open(os.path.join(dest_dir, new_filename), 'w').write(''.join(gxlines))
            else:
                shutil.copy(gxparm, os.path.join(dest_dir, new_filename))
        else:
            logger.error('{} does not exist in {}.'.format(gxparm_file, src_dir))
