#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  1 12:15:01 2017

@author: qxu
"""

# NB: parsing log into db, this may have some risks when using with
# concurrent processing since the log can be garbled due to output 
# multiple processes

from __future__ import absolute_import
import logging
import os, sys, traceback, threading, multiprocessing
import random
import datetime
from concurrent_log_handler import ConcurrentRotatingFileHandler

from ..process.get_args import ARGS, namespace2dict
from ..common.pv import update_pipelineStatus_PV

logger = logging.getLogger(__name__)


class FileDBHandler(ConcurrentRotatingFileHandler):

    def __init__(self):
        pid = ARGS.processing_id if ARGS.processing_id else int(random.random() * 2 ** 10)
        hostname = ARGS.hostname if ARGS.hostname else os.getenv('HOSTNAME')
        logfilename = ARGS.logfile if ARGS.logfile else 'GMCAproc_{}_{}.log'.format(hostname, pid)
        path = ARGS.proc_dir if ARGS.proc_dir else os.getcwd()
        if not os.path.isdir(path):
            os.makedirs(path)
        self.baseFilename = os.path.abspath(os.path.join(path, logfilename))

        ConcurrentRotatingFileHandler.__init__(self, self.baseFilename)

        self.log2db = ARGS.log2db
        self.sampleName = ARGS.sampleName
        self.sampleNumber = ARGS.sampleNumber if ARGS.sampleNumber else 0
        self.hostname = hostname
        self.processing_id = pid
        self.pipeline = ','.join(ARGS.pipeline)
        self.processing_root_dir = ARGS.proc_dir
        self.username = ARGS.username

        if self.log2db:
            from .database import PipelineStatus, db

            logger.info("logging to database turned on")
            # check the entry exists
            try:
                self.pipelinestatus, _ = PipelineStatus.get_or_create(
                    processing_id=self.processing_id,
                    hostname=self.hostname,
                    sampleNumber=self.sampleNumber,
                    sampleName=self.sampleName)

                self.id = self.pipelinestatus.get_id()

                self.pipelinestatus.hostname = self.hostname
                self.pipelinestatus.processing_id = self.processing_id
                self.pipelinestatus.sampleName = self.sampleName
                self.pipelinestatus.imagedir = ARGS.data_dir
                self.pipelinestatus.command = ARGS.command
                self.pipelinestatus.sampeNumber = self.sampleNumber
                self.pipelinestatus.workdir = self.processing_root_dir
                self.pipelinestatus.pipeline = self.pipeline
                self.pipelinestatus.state = "STARTED"
                self.pipelinestatus.starttime = ARGS.starttime.isoformat()
                self.pipelinestatus.beamline = ARGS.beamline
                self.pipelinestatus.imageSet = ARGS.imageSet if ARGS.imageSet else 'All'
                self.pipelinestatus.logfile = "{}".format(self.baseFilename)

                with db.atomic():
                    self.pipelinestatus.save()

                logger.info("runtime info saved to PipelineStatus table, id={}."
                            .format(self.id))
            except Exception as e:
                logger.error("failed to get or create PipelineStatus")
                raise
        else:
            logger.info("logging to database turned off")

        self.queue = multiprocessing.Queue(-1)

        thrd = threading.Thread(target=self.db_write)
        thrd.daemon = True
        thrd.start()

    def db_write(self):
        while True:
            try:
                record = self.queue.get()

                if self.log2db:
                    from .database import PipelineStatus, db

                    msg = record.message

                    if record.levelname in ["ERROR", "CRITICAL", "WARNING", "WARN", "EXCEPTION"] \
                            or any(w in msg for w in ["Exception", "Traceback", ]):
                        self.pipelinestatus.warning += record.levelname + ": " + msg + "\n"

                        with db.atomic():
                            self.pipelinestatus.save()

            except (KeyboardInterrupt, SystemExit):
                raise
            except EOFError:
                break
            except AttributeError:
                raise
            except:
                traceback.print_exc(file=sys.stderr)

    def emit(self, record):
        ConcurrentRotatingFileHandler.emit(self, record)

        try:
            self.queue.put_nowait(record)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)

    def get_pipelinestatus_id(self):
        if self.log2db:
            return self.id
        else:
            logger.warning('pipeline status table primary key does not exist.')
            return None

    def get_pipelinestatus(self):
        if self.log2db:
            return self.pipelinestatus
        else:
            logger.warning('pipeline status table does not exist.')
            return None
