import json

import redis
from quickProcess.process.get_args import ARGS


def get_redis_server(beamline, type='utf-8'):
    beamline = beamline.lower()

    redis_host = '127.0.0.1'
    if "23i" in beamline:
        redis_host = '10.20.103.71'
    elif "23o" in beamline:
        redis_host = '10.20.103.138'
    elif "23b" in beamline:
        redis_host = '10.20.103.197'

    redis_port = 8003 if redis_host == '127.0.0.1' else 6379

    if type == 'utf-8':
        server = redis.Redis(host=redis_host, port=redis_port, charset="utf-8", decode_responses=True)
    else:
        server = redis.Redis(host=redis_host, port=redis_port)

    return server


def publish_to_redis(data, channel='analysis_channel'):
    if not ARGS.beamline:
        return

    r = get_redis_server(ARGS.beamline)
    # "bluice:sample:table#C14"
    data_s = json.dumps(data)
    r.publish(channel, data_s)


def message_listener(channel):
    def msg_handler(message):
        try:
            data_ = message.get('data')
            data_ = data_.decode()

            data_dict = json.loads(data_)
        except (IOError, json.JSONDecodeError):
            print(f"Unable to decode data: {message}, expects a json string")
            return

        if not isinstance(data_dict, dict):
            print(f"Unable to understand data field: {data_dict}, expects dict")
            return

        print(f"received message {message}, data = {data_dict}")

    rl = get_redis_server(ARGS.beamline)
    pubsub = rl.pubsub()
    pubsub.subscribe(**{channel: msg_handler})
    pubsub.run_in_thread(sleep_time=0.1, daemon=True)


def strategy_to_redis(data_dict, key=None):
    # send the result to redis & augment key to trigger pybluice update
    if not ARGS.beamline or not ARGS.redis_key:
        return

    if not key:
        key = ARGS.redis_key

    r = get_redis_server(ARGS.beamline)
    # send the result to redis
    for k, v in data_dict:
        if k in ['resolution',
                 'mosaicity',
                 'rmsd',
                 'unitcell',
                 'symm',
                 'osc_start',
                 'osc_end',
                 'osc_delta',
                 'score',
                 'distance']:
            r.hset(key, k, v)

    #  augment key to trigger pybluice update
    r.incr('bluice:sample:strategy_ver__s')


# ra = get_redis_server('23b',type='utf-8')
# for k in ra.keys():
#     print(k)
#
# sample_id = "A1"
# key = f"bluice:sample_table:{sample_id}"
# redis.hmset(key, {'resolution_from_spots': 2})

# channel = "analysis_channel"
# msg = json.dumps({'run': run_idx, 'state': state, 'filelist': filelist, 'n_images': n_images})
# rl.bin_srv.publish(channel, msg)


if __name__ == '__main__':
    r = get_redis_server('23b')
    di = {1: 2, 3: 4}
    print("strategy", di)
    r.hmset("strategy", mapping=di)
    do = r.hgetall("strategy")

    print(do)

    for k, v in di.items():
        r.hset("s", k, v)

    print(r.hgetall('s'))

    print(r.hgetall('23b#strategy'))
