#! /bin/awk -f

# convert XDS-style inputs into a MOSFLM matrix file
#								-James Holton 
#

BEGIN{if(! lambda) lambda=1;
      if(! pix)  pix=0.1024;
spindle_x = 1; spindle_y = 0; spindle_z = 0;
beam_x = 0; beam_y = 0; beam_z = 1;

}


/^WAVE/{lambda=$2}
/^BEAM/{beamx=$2;beamy=$3}
/^DIST/{dist=$NF}
/^TWOTHETA/{twotheta=$NF}
/^BEAM/{print;mos_beamy=$NF;mos_beamx=$(NF-1)}

/^SYMM/{SG=$2}
/^CELL/{a=$2;b=$3;c=$4;alpha=$5;beta=$6;gamma=$7}
/^LIMITS/{
   detsize=substr($0,index($0,"YMAX ")+5)+0
}


/QX= /{fpix=substr($0,index($0,"QX= ")+3)+0}
/NX= /{fpixels=substr($0,index($0,"NX= ")+3)+0}
/QY= /{spix=substr($0,index($0,"QY= ")+3)+0}
/NY= /{spixels=substr($0,index($0,"NY= ")+3)+0}

/ORGX= /{orgx=substr($0,index($0,"ORGX= ")+5)+0}
/ORGY= /{orgy=substr($0,index($0,"ORGY= ")+5)+0}


/^X-RAY_WAVELENGTH=/{
   test=substr($0,index($0,"=")+2);
   if(test+0>0)lambda=test
}
/^STARTING_FRAME=/{
   test=substr($0,index($0,"=")+2);
   if(int(test+0)==test) firstnum=test
}
/^STARTING_ANGLE=/{
   test=substr($0,index($0,"=")+2);
   phi0=test
}
/OSCILLATION_RANGE=/{
   test=substr($0,index($0,"=")+2);
   if(test+0!=0) osc=test
}
/DETECTOR_DISTANCE=/{
   test=substr($0,index($0,"=")+2);
   if(test+0!=0) dist=test
}

/SPACE_GROUP_NUMBER=/{
   test=substr($0,index($0,"=")+2);
   if(test+0>0)SGnum=test
}
/UNIT_CELL_CONSTANTS=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==6){
        a=w[1];b=w[2];c=w[3];alpha=w[4];beta=w[5];gamma=w[6];
    }
}


/INCIDENT_BEAM_DIRECTION=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==3){
        beam_x=w[1];beam_y=w[2];beam_z=w[3];
    }
}
/DIRECTION_OF_DETECTOR_X-AXIS=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==3){
        detX_x=w[1];detX_y=w[2];detX_z=w[3];
    }
}
/DIRECTION_OF_DETECTOR_Y-AXIS=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==3){
        detY_x=w[1];detY_y=w[2];detY_z=w[3];
    }
}
/ROTATION_AXIS=/{
    test=substr($0,index($0,"=")+2);
    n=split(test,w);
    if(n==3){
        spindle_x=w[1];spindle_y=w[2];spindle_z=w[3];
    }
}


/UNIT_CELL_A-AXIS=/{xds_a_x=$2;xds_a_y=$3;xds_a_z=$4}
/UNIT_CELL_B-AXIS=/{xds_b_x=$2;xds_b_y=$3;xds_b_z=$4}
/UNIT_CELL_C-AXIS=/{xds_c_x=$2;xds_c_y=$3;xds_c_z=$4}



END {
# scale factor to convert radians to degrees
RTD = 45.0/atan2(1,1);
DTR = 1/RTD;


if(firstnum=="") firstnum = 1;
if(lastnum=="") lastnum = firstnum;
if(phi0=="") phi0 = 0;
if(osc=="") osc = 1;
if(pix=="") pix = 0.1024;
if(pixels=="") pixels = 3096;
if(dist=="") dist = 250;
if(beam_z=="") beam_z = 1/lambda;
if(detX_x=="") detX_x=1;
if(detY_y=="") { detY_x=0; detY_y=cos(twotheta*DTR) ; detY_z=sin(twotheta*DTR) }
if(detZ_y=="") { detZ_x=0; detZ_y=-detY_z ; detZ_z=detY_y }
if(spindle_x=="") spindle_x=1;

if(mos_beamx != ""){
    orgx=mos_beamy/pix;
    orgy=(mos_beamx+dist*sin(twotheta*DTR)/cos(twotheta*DTR))/pix
}
if(orgx=="") orgx=pixels/2
if(orgy=="") orgy=pixels/2

# transform into mosflm coordinate system:
# "x" is x-ray beam and "z" is spindle axis
norm = sqrt(beam_x**2+beam_y**2+beam_z**2)
uxx = beam_x/norm ; uxy = beam_y/norm; uxz = beam_z/norm;
norm = sqrt(spindle_x**2+spindle_y**2+spindle_z**2)
uzx = spindle_x/norm; uzy = spindle_y/norm; uzz = spindle_z/norm;

# cross product to find y axis
uyx = uzy*uxz - uzz*uxy;
uyy = uzz*uxx - uzx*uxz;
uyz = uzx*uxy - uzy*uxx;
norm = sqrt(uyx**2+uyy**2+uyz**2)
uyx/=norm;uyy/=norm;uyz/=norm;
# make axes orthonormal by moving spindle
uzx = uxy*uyz - uxz*uyy;
uzy = uxz*uyx - uxx*uyz;
uzz = uxx*uyy - uxy*uyx;
norm = sqrt(uzx**2+uzy**2+uzz**2)
uzx/=norm;uzy/=norm;uzz/=norm;

determinant = uxx*(uyy*uzz-uyz*uzy) - uxy*(uyx*uzz-uyz*uzx) + uxz*(uyx*uzy-uyy*uzx)


a_x = xds_a_x*uxx+xds_a_y*uxy+xds_a_z*uxz;
a_y = xds_a_x*uyx+xds_a_y*uyy+xds_a_z*uyz;
a_z = xds_a_x*uzx+xds_a_y*uzy+xds_a_z*uzz;

b_x = xds_b_x*uxx+xds_b_y*uxy+xds_b_z*uxz;
b_y = xds_b_x*uyx+xds_b_y*uyy+xds_b_z*uyz;
b_z = xds_b_x*uzx+xds_b_y*uzy+xds_b_z*uzz;

c_x = xds_c_x*uxx+xds_c_y*uxy+xds_c_z*uxz;
c_y = xds_c_x*uyx+xds_c_y*uyy+xds_c_z*uyz;
c_z = xds_c_x*uzx+xds_c_y*uzy+xds_c_z*uzz;



# calculate direct-space unit cell lengths
a = sqrt(a_x^2+a_y^2+a_z^2);
b = sqrt(b_x^2+b_y^2+b_z^2);
c = sqrt(c_x^2+c_y^2+c_z^2);


# three cross products
a_cross_b_x = a_y*b_z - a_z*b_y;
a_cross_b_y = a_z*b_x - a_x*b_z;
a_cross_b_z = a_x*b_y - a_y*b_x;

b_cross_c_x = b_y*c_z - b_z*c_y;
b_cross_c_y = b_z*c_x - b_x*c_z;
b_cross_c_z = b_x*c_y - b_y*c_x;

c_cross_a_x = c_y*a_z - c_z*a_y;
c_cross_a_y = c_z*a_x - c_x*a_z;
c_cross_a_z = c_x*a_y - c_y*a_x;

# three dot products
a_dot_b = a_x*b_x + a_y*b_y + a_z*b_z;
a_dot_c = a_x*c_x + a_y*c_y + a_z*c_z;
b_dot_c = b_x*c_x + b_y*c_y + b_z*c_z;


# direct-space unit cell volume
V = a_x*b_cross_c_x + a_y*b_cross_c_y + a_z*b_cross_c_z;

# reciprocal-space cell volume
V_star = 1/V;

# reciprocal lattice vector "a_star" is defined as perpendicular to both b and c, and must also preserve volume
# converse is true for direct-space lattice: a is perpendicular to both b_star and c_star
# a = ( b cross c ) / V

a_star_x = b_cross_c_x/V
a_star_y = b_cross_c_y/V
a_star_z = b_cross_c_z/V

b_star_x = c_cross_a_x/V
b_star_y = c_cross_a_y/V
b_star_z = c_cross_a_z/V

c_star_x = a_cross_b_x/V
c_star_y = a_cross_b_y/V
c_star_z = a_cross_b_z/V

# calculate reciprocal cell vector lengths
a_star = sqrt(a_star_x^2+a_star_y^2+a_star_z^2);
b_star = sqrt(b_star_x^2+b_star_y^2+b_star_z^2);
c_star = sqrt(c_star_x^2+c_star_y^2+c_star_z^2);
# three dot products
a_star_dot_b_star = a_star_x*b_star_x + a_star_y*b_star_y + a_star_z*b_star_z;
a_star_dot_c_star = a_star_x*c_star_x + a_star_y*c_star_y + a_star_z*c_star_z;
b_star_dot_c_star = b_star_x*c_star_x + b_star_y*c_star_y + b_star_z*c_star_z;


# calculate direct-space cell angles
sinalpha = a_star*V/b/c;
sinbeta  = b_star*V/a/c;
singamma = c_star*V/a/b;
cosalpha = b_dot_c/b/c;
cosbeta  = a_dot_c/a/c;
cosgamma = a_dot_b/a/b;

if(sinalpha>1) sinalpha=1;
if(sinbeta>1) sinbeta=1;
if(singamma>1) singamma=1;
alpha = RTD*atan2(sinalpha,cosalpha);
beta  = RTD*atan2(sinbeta ,cosbeta);
gamma = RTD*atan2(singamma,cosgamma);


# reciprocal cell angles
sinalpha_star = a*V_star/b_star/c_star;
sinbeta_star  = b*V_star/a_star/c_star;
singamma_star = c*V_star/a_star/b_star;
cosalpha_star = b_star_dot_c_star/b_star/c_star;
cosbeta_star  = a_star_dot_c_star/a_star/c_star;
cosgamma_star = a_star_dot_b_star/a_star/b_star;

alpha_star = RTD*atan2(sinalpha_star,cosalpha_star);
beta_star  = RTD*atan2(sinbeta_star ,cosbeta_star);
gamma_star = RTD*atan2(singamma_star,cosgamma_star);


# make the cannonical "B" matrix
AX=a_star;
BX=b_star*cosgamma_star;
CX=c_star*cosbeta_star;
AY=0;
BY=b_star*singamma_star;
CY=c_star*(cosalpha_star-cosbeta_star*cosgamma_star)/singamma_star;
AZ=0;
BZ=0;
CZ=c_star*V/(a*b*c*singamma_star);

# make the inverse of the B matrix
nxx=(BY*CZ-CY*BZ)*V;
nxy=(BZ*CX-CZ*BX)*V;
nxz=(BX*CY-CX*BY)*V;
nyx=(AZ*CY-CZ*AY)*V;
nyy=(AX*CZ-CX*AZ)*V;
nyz=(AY*CX-CY*AX)*V;
nzx=(AY*BZ-BY*AZ)*V;
nzy=(AZ*BX-BZ*AX)*V;
nzz=(AX*BY-BX*AY)*V;

# multiply A by B^-1 matrix to get U matrix
Axx=a_star_x; Axy=b_star_x; Axz=c_star_x;
Ayx=a_star_y; Ayy=b_star_y; Ayz=c_star_y;
Azx=a_star_z; Azy=b_star_z; Azz=c_star_z;

uxx = Axx*nxx + Axy*nyx + Axz*nzx;
uxy = Axx*nxy + Axy*nyy + Axz*nzy;
uxz = Axx*nxz + Axy*nyz + Axz*nzz;
uyx = Ayx*nxx + Ayy*nyx + Ayz*nzx;
uyy = Ayx*nxy + Ayy*nyy + Ayz*nzy;
uyz = Ayx*nxz + Ayy*nyz + Ayz*nzz;
uzx = Azx*nxx + Azy*nyx + Azz*nzx;
uzy = Azx*nxy + Azy*nyy + Azz*nzy;
uzz = Azx*nxz + Azy*nyz + Azz*nzz;
determinant = uxx*(uyy*uzz-uyz*uzy) - uxy*(uyx*uzz-uyz*uzx) + uxz*(uyx*uzy-uyy*uzx)
#printf("GOTHERE %.15f\n", determinant)


# convert to mosflm A matrix
AX = a_star_x*lambda;
AY = a_star_y*lambda;
AZ = a_star_z*lambda;
BX = b_star_x*lambda;
BY = b_star_y*lambda;
BZ = b_star_z*lambda;
CX = c_star_x*lambda;
CY = c_star_y*lambda;
CZ = c_star_z*lambda;


    printf "%12.8f%12.8f%12.8f\n", AX+1e-9,BX+1e-9,CX+1e-9;
    printf "%12.8f%12.8f%12.8f\n", AY+1e-9,BY+1e-9,CY+1e-9;
    printf "%12.8f%12.8f%12.8f\n", AZ+1e-9,BZ+1e-9,CZ+1e-9;
    printf "%12.3f%12.3f%12.3f\n",0,0,0;
    printf "%12.8f%12.8f%12.8f\n", uxx,uxy,uxz;
    printf "%12.8f%12.8f%12.8f\n", uyx,uyy,uyz;
    printf "%12.8f%12.8f%12.8f\n", uzx,uzy,uzz;
    printf "%12.4f%12.4f%12.4f%12.4f%12.4f%12.4f\n",a,b,c,alpha,beta,gamma;
    printf "%12.3f%12.3f%12.3f\n",0,0,0;


}

