from __future__ import absolute_import
import argparse
import logging
import os, glob
import sys

from quickProcess.tasks.scriptjob import ScriptJob
from quickProcess.common.utils import getNumbers

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
logger = logging.getLogger(__name__)


def get_args():
    """ command line options """
    parser = argparse.ArgumentParser(prog='')
    parser.add_argument('--data_dir', '-i', default='/home/qxu/data-for-tests/23IDB_2017_03_07_dla',
                        required=False, help='parent directory for image files')
    parser.add_argument('--proc_dir', '-o', default=os.path.join('/tmp', os.getenv('USER'), 'processall'),
                        required=False, help='parent directory for all processing files')

    return parser.parse_args()


def walk(directory):
    if not os.path.isdir(directory):
        logger.info('{} does not exist'.format(directory))
        yield None

    for root, dirs, files in os.walk(directory):
        cbffiles = [file.endswith('.cbf') for file in files if not os.path.islink(os.path.join(root, file))]
        if cbffiles and sum(cbffiles) > 20:
            yield root


def process_it(data_dir,
               proc_dir='.',
               sampleName='sampleName',
               sampleNumber='sampleNumber',
               pipeline='autoproc',
               dry_run=False):
    wdir = proc_dir
    if not os.path.isdir(wdir):
        os.makedirs(wdir)

    job = ScriptJob(wdir, script_name='{}.sh'.format(pipeline))
    job.update_cmd("module load anaconda/python3\n"
                   "export PYTHONPATH=/home/qxu/work/quickProcess:$PYTHONPATH\n"
                   "python -m quickProcess.process"
                   " --data_dir={} --proc_dir={}"
                   " --pipeline={} "
                   " --highres=1.89"
                   " --start=5"
                   " --nproc=2 --njobs=1 --log2db"
                   " --sampleName={} --sampleNumber={}"
                   .format(data_dir, wdir, pipeline, sampleName, sampleNumber))
    if dry_run:
        job.write_script()
    else:
        job.run_sync()


def main():
    sys.path.append('/home/qxu/work/quickProcess')
    args = get_args()
    pipelines = ['gmcaproc', 'xia2_dials']
    count = 0
    maxcount = 1000
    for dir in walk(args.data_dir):
        print("processing: ", dir)
        subdir, id = dir.split(os.sep)[-2:]
        sampleName = str(subdir)
        sampleNumber = getNumbers(id)
        if not sampleNumber:
            sampleNumber = 0
        else:
            sampleNumber = sampleNumber[0]
        for pipeline in pipelines:
            proc_dir = os.path.join(args.proc_dir, subdir, id)
            process_it(dir,
                       proc_dir=proc_dir,
                       pipeline=pipeline,
                       sampleName=sampleName,
                       sampleNumber=sampleNumber, dry_run=False)
        count += 1
        if count >= maxcount:
            raise SystemExit(1)


if __name__ == '__main__':
    main()
