import os

pilatus_template = \
"""
photon_energy = {energy} ; in eV
adu_per_photon = 1.0
clen = {distance}  ; in m
coffset = 0.
res = 5814.0  ; in pixels

;data = /entry_1/data_1/data ; /entry_1/data_1/data
;dim0 = %
;dim1 = ss
;dim2 = fs

0/min_fs = 0
0/max_fs = 2462
0/min_ss = 0
0/max_ss = 2526
0/corner_x = -{xbeam} 
0/corner_y = -{ybeam}
0/fs = x
0/ss = y


bad_beamstop/min_fs = {xmin}
bad_beamstop/min_ss = {ymin}
bad_beamstop/max_fs = {xmax}
bad_beamstop/max_ss = {ymax}

bad_v1/min_fs = 487
bad_v1/min_ss = 0
bad_v1/max_fs = 493
bad_v1/max_ss = 2526
bad_v2/min_fs = 981
bad_v2/min_ss = 0
bad_v2/max_fs = 987
bad_v2/max_ss = 2526
bad_v3/min_fs = 1475
bad_v3/min_ss = 0
bad_v3/max_fs = 1481
bad_v3/max_ss = 2526
bad_v4/min_fs = 1969
bad_v4/min_ss = 0
bad_v4/max_fs = 1975
bad_v4/max_ss = 2526

bad_h1/min_fs = 0
bad_h1/min_ss = 195
bad_h1/max_fs = 2462
bad_h1/max_ss = 211
bad_h2/min_fs = 0
bad_h2/min_ss = 407
bad_h2/max_fs = 2462
bad_h2/max_ss = 423
bad_h3/min_fs = 0
bad_h3/min_ss = 619
bad_h3/max_fs = 2462
bad_h3/max_ss = 635
bad_h4/min_fs = 0
bad_h4/min_ss = 831
bad_h4/max_fs = 2462
bad_h4/max_ss = 847
bad_h5/min_fs = 0
bad_h5/min_ss = 1043
bad_h5/max_fs = 2462
bad_h5/max_ss = 1059
bad_h6/min_fs = 0
bad_h6/min_ss = 1255
bad_h6/max_fs = 2462
bad_h6/max_ss = 1271
bad_h7/min_fs = 0
bad_h7/min_ss = 1467
bad_h7/max_fs = 2462
bad_h7/max_ss = 1483
bad_h8/min_fs = 0
bad_h8/min_ss = 1679
bad_h8/max_fs = 2462
bad_h8/max_ss = 1695
bad_h9/min_fs = 0
bad_h9/min_ss = 1891
bad_h9/max_fs = 2462
bad_h9/max_ss = 1907
bad_ha/min_fs = 0
bad_ha/min_ss = 2103
bad_ha/max_fs = 2462
bad_ha/max_ss = 2119
bad_hb/min_fs = 0
bad_hb/min_ss = 2315
bad_hb/max_fs = 2462
bad_hb/max_ss = 2331
"""

eiger_template = \
"""
photon_energy = {energy} ; in eV
adu_per_photon = 1.0
clen = {distance}  ; in m
coffset = 0.
res = 13333.3  ; in pixels

;data = /entry_1/data_1/data ; /entry_1/data_1/data
;dim0 = %
;dim1 = ss
;dim2 = fs

0/min_fs = 0
0/max_fs = 4149
0/min_ss = 0
0/max_ss = 4370
0/corner_x = -{xbeam}
0/corner_y = -{ybeam}
0/fs = x
0/ss = y


bad_beamstop/min_fs = {xmin}
bad_beamstop/min_ss = {ymin}
bad_beamstop/max_fs = {xmax}
bad_beamstop/max_ss = {ymax}


bad_v1/min_fs = 1030
bad_v1/min_ss = 0
bad_v1/max_fs = 1039
bad_v1/max_ss = 4370
bad_v2/min_fs = 2070
bad_v2/min_ss = 0
bad_v2/max_fs = 2079
bad_v2/max_ss = 4370
bad_v3/min_fs = 3110
bad_v3/min_ss = 0
bad_v3/max_fs = 3119
bad_v3/max_ss = 4370

bad_h1/min_fs = 0
bad_h1/min_ss = 514
bad_h1/max_fs = 4149
bad_h1/max_ss = 550
bad_h2/min_fs = 0
bad_h2/min_ss = 1065
bad_h2/max_fs = 4149
bad_h2/max_ss = 1101
bad_h3/min_fs = 0
bad_h3/min_ss = 1616
bad_h3/max_fs = 4149
bad_h3/max_ss = 1652
bad_h4/min_fs = 0
bad_h4/min_ss = 2167
bad_h4/max_fs = 4149
bad_h4/max_ss = 2203
bad_h5/min_fs = 0
bad_h5/min_ss = 2718
bad_h5/max_fs = 4149
bad_h5/max_ss = 2754
bad_h6/min_fs = 0
bad_h6/min_ss = 3269
bad_h6/max_fs = 4149
bad_h6/max_ss = 3305
bad_h7/min_fs = 0
bad_h7/min_ss = 3820
bad_h7/max_fs = 4149
bad_h7/max_ss = 3856
"""


class Geometry:
    """
    simpler Geometry generation for Pilatus 6M and Eiger 16M, use other Geometry class
    for new detectors
    """
    def __init__(self,
                 header=None,
                 image=None,
                 wdir="."):
        self.h_dict = header
        self.image = image
        self.wdir = wdir

        if not self.h_dict:
            if os.path.exists(self.image):
                from .cbf_header import PilatusHeader
                self.h_dict = PilatusHeader(self.image).header_dict
            else:
                raise Exception("unable to generate geometry file, since header & image file are not both defined")

    def generate(self):
        distance = self.h_dict['Detector_distance']
        energy = round(12398.42 / self.h_dict['Wavelength'], 2)
        xbeam = self.h_dict['Beam_x']
        ybeam = self.h_dict['Beam_y']
        detector = self.h_dict['Detector_identifier']
        return Geometry.generate_real(self.wdir, detector, energy, distance, xbeam, ybeam)

    @staticmethod
    def generate_real(wdir, detector, energy, distance, xbeam, ybeam):
        def get_beamstop_shadow(x_beam, y_beam, radius=100):
            # radius of beam stop in pixels
            x_beam = float(x_beam)
            y_beam = float(y_beam)
            return tuple([0, int(y_beam - radius), int(x_beam + radius), int(y_beam + radius)])

        template = pilatus_template if 'pilatus' in detector.lower() else eiger_template
        xmin, ymin, xmax, ymax = get_beamstop_shadow(xbeam, ybeam, radius=80)
        geometry = template.format(energy=energy,
                                   distance=distance,
                                   xbeam=xbeam,
                                   ybeam=ybeam,
                                   xmin=xmin,
                                   ymin=ymin,
                                   xmax=xmax,
                                   ymax=ymax
                                   )

        geometry_file = os.path.join(wdir, "geometry.geom")
        with open(geometry_file, 'w') as fh:
            fh.write(geometry)

        return geometry_file

