from __future__ import absolute_import
import os
import logging

from .scriptjob import ScriptJob

logger = logging.getLogger(__name__)


class Fast_EP(ScriptJob):
    def __init__(self,
                 wdir='.',
                 script_name='fast_ep.sh',
                 script_text=None,
                 nproc=1,
                 machines=2,
                 runner='sge',
                 pe_name='smp',
                 space_group=None,
                 trace=False,
                 nsites=None,
                 native=None,
                 mode="basic",
                 atom=None,
                 rlims=None,
                 sge_project=None,
                 hklin_dir='.',
                 hklin_name='fast_dp.mtz',
                 logfile='fastep.log'):

        super(Fast_EP, self).__init__(wdir,
                                     script_name,
                                     script_text,
                                     nproc,
                                     runner,
                                     pe_name)
        self.hklin_name = hklin_name
        self.hklin_dir = hklin_dir
        self.space_group = "spg={}".format(space_group) if space_group else ""
        self.nsites = "nsites={}".format(nsites) if nsites else ""
        self.rlims = "rlims={}".format(rlims) if rlims else ""
        self.trace = "trace={}".format(trace) if trace else ""
        self.mode = "mode={}".format(mode) if mode else ""
        self.native = "native={}".format(native) if native and os.path.exists(native) else ""
        self.atom = "atom={}".format(atom) if atom else ""
        self.sge_project = "sge_project={}".format(sge_project) if sge_project else ""
        self.machines = "machines={}".format(machines)
        self.cpu = "cpu={}".format(nproc)
        self.hklin = os.path.join(self.hklin_dir, self.hklin_name)
        if not os.path.exists(self.hklin):
            raise RuntimeError("mtz file does not exist: {}".format(self.hklin))

        self.data = "data={}".format(self.hklin_name)

        self.options = list()
        self.options.append(self.data)
        self.options.append(self.space_group)
        self.options.append(self.trace)
        self.options.append(self.nsites)
        self.options.append(self.native)
        self.options.append(self.mode)
        self.options.append(self.sge_project)
        self.options.append(self.atom)
        self.options.append(self.rlims)
        self.options.append(self.machines)
        self.options.append(self.cpu)

        self.logfile = logfile

    def cmd_run_fastep(self):
        if os.path.exists(os.path.join(self.wdir, self.hklin_name)):
            os.unlink(os.path.join(self.wdir, self.hklin_name))

        px = "/mnt/software/px"  # os.environ('PX')
        fast_ep_root = "{}/fast_ep".format(px)
        fast_ep = "{}/bin/fast_ep".format(fast_ep_root)

        cmd = ""
        cmd += "export PX={}\nsource {}/CCP4/ccp4/bin/ccp4.setup-sh\n".format(px,px)
        if self.machines and self.runner == 'sge':
            cmd += "export SGE_ROOT=/opt/sge\nexport SGE_CELL=default\nexport DRMAA_LIBRARY_PATH=/opt/sge/lib/lx-amd64/libdrmaa.so\n"

        if self.machines and self.runner == 'slurm':
            cmd += "export DRMAA_LIBRARY_PATH=/usr/lib64/libdrmaa.so\n"

        cmd += "export FAST_EP_ROOT={}\n".format(fast_ep_root)

        cmd += "ln -s {} {}\n".format(self.hklin, self.hklin_name)
        cmd += "{} {}\n".format(fast_ep, " ".join(self.options))
        self.script_text = cmd
        return self

    def report(self):
        pass

    def run(self, async_flag=False):
        logger.info("run fast_ep for SAD phasing.")
        if async_flag:
            self.cmd_run_fastep().run_sync()
        else:
            return self.cmd_run_fastep().run_sync(timeout=1800)

        self.report()

"""
running on sge, requires installation of drmaa into ccp4 python

export PX=/mnt/software/px
source /mnt/software/px/CCP4/ccp4/bin/ccp4.setup-sh
echo $CCP4
/mnt/software/px/CCP4/ccp4/libexec/pip install drmaa

export SGE_ROOT=/opt/sge
export SGE_CELL=default
export DRMAA_LIBRARY_PATH=/opt/sge/lib/lx-amd64/libdrmaa.so


# SLURM
export DRMAA_LIBRARY_PATH=/usr/lib64/libdrmaa.so
"""