from __future__ import absolute_import
import os
import logging

from .scriptjob import ScriptJob

logger = logging.getLogger(__name__)


class Truncate(ScriptJob):
    def __init__(self,
                 wdir='.',
                 script_name='truncate.sh',
                 script_text=None,
                 nproc=1,
                 runner='local',
                 pe_name='smp',
                 hklin='aimless.mtz',
                 hklout='truncate.mtz',
                 logfile='truncate.log'):

        super(Truncate, self).__init__(wdir,
                                       script_name,
                                       script_text,
                                       nproc,
                                       runner,
                                       pe_name)
        self.hklin = hklin
        self.hklout = hklout
        self.logfile = logfile
        self.colano = "-colano '/*/*/[I(+),SIGI(+),I(-),SIGI(-)]'"

    def cmd_run_ctruncate(self):
        hklin = os.path.join(self.wdir, self.hklin)
        if os.path.isfile(hklin):
            cmd = "ctruncate -mtzin {} -mtzout {} {} << eof >{}\n".format(self.hklin,
                                                                          self.hklout,
                                                                          self.colano,
                                                                          self.logfile)
            cmd += "eof\n"
            self.script_text = cmd
            return self
        else:
            logger.error("{} does not exist".format(self.hklin))

    def run_ctruncate(self, async_flag=False):
        logger.info("run ctruncate to generate F/SigF.")
        if async_flag:
            self.cmd_run_ctruncate().run_async()
        else:
            return self.cmd_run_ctruncate().run_sync(timeout=1800)