from __future__ import absolute_import
import os
import logging

from .scriptjob import ScriptJob
from ..config.config import CCP4_SETUP

logger = logging.getLogger(__name__)


class Xia2Report(ScriptJob):
    def __init__(self,
                 unmergedfile,
                 unmergedfile_dir='.',
                 wdir='.',
                 script_name='xia2_report.sh',
                 script_text=None,
                 nproc=2,
                 runner='local',
                 pe_name='smp',):
        super(Xia2Report, self).__init__(wdir,
                                         script_name,
                                         script_text,
                                         nproc,
                                         runner,
                                         pe_name)
        self.unmergedfile_dir = unmergedfile_dir
        self.unmergedfile = unmergedfile
        self.wdir = os.path.abspath(wdir)
        if not os.path.isdir(self.wdir):
            os.makedirs(self.wdir)

    def cmd_for_xia2_report(self):
        cmd = list(["unset HDF5_PLUGIN_PATH\n", CCP4_SETUP, "\n\n"])
        cmd.append("cd {}\n".format(self.wdir))
        cmd.append("cp {}/{} .\n".format(self.unmergedfile_dir, self.unmergedfile))
        cmd.append("xia2.report {} > xia2-report.log\n".format(self.unmergedfile))

        self.script_text = ''.join(cmd)
        return self

    def run_xia2_report(self, async_flag=False):
        logger.info("generate html report async using xia2.report.")
        if async_flag:
            self.cmd_for_xia2_report().run_async()
        else:
            self.cmd_for_xia2_report().run_sync()
