import datetime, os, time
import pexpect


class Mosflm():
    def __init__(self,
                 imagesweep,
                 wdir='.',
                 user_spacegroup=None,
                 user_unitcell=None,
                 highres=None,
                 ):
        self.wdir = os.path.abspath(wdir)
        self.tag = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
        self.spotod = os.path.join(self.wdir, 'mosflm_{}.spotod'.format(self.tag))
        self.summary = os.path.join(self.wdir, 'mosflm_{}.sum'.format(self.tag))
        self.coords = os.path.join(self.wdir, 'mosflm_{}.coords'.format(self.tag))
        self.logfile = os.path.join(self.wdir, 'mosflm_{}.log'.format(self.tag))

        print(" ".join(['/mnt/software/px/CCP4/ccp4/bin/ipmosflm',
                        'SPOTOD {}'.format(self.spotod),
                        'COORDS {}'.format(self.coords),
                        'SUMMARY {}'.format(self.summary)]))
        self.mosflm = pexpect.spawn(
            '/mnt/software/px/CCP4/ccp4/bin/ipmosflm SPOTOD {} COORDS {} SUMMARY {}'.format(self.summary, self.coords,
                                                                                            self.summary),
            encoding='utf-8', timeout=30)
        self.highres = highres
        self.spacegroup = user_spacegroup
        self.unitcell = user_unitcell
        self.mosflm.logfile_read = open(self.logfile, 'w')
        self.image_sweep = imagesweep

    def setup(self):
        self.mosflm.sendline("directory {}".format(self.image_sweep.get_absdir()))
        self.mosflm.sendline("template {}".format(self.image_sweep.get_name_template("#")))
        self.mosflm.sendline("findspots rmin 5 AUTORESLN AUTORING LOCAL ICE")
        self.mosflm.sendline("best on")
        for i in self.image_sweep.get_all_file_numbers():
            self.mosflm.sendline("image {}".format(i))

    def shutdown(self):
        if self.mosflm.isalive():
            self.mosflm.sendline('exit')
            self.mosflm.terminate()

    def setup_test(self):
        self.mosflm.sendline('directory /home/qxu/data-for-tests/jcsg-p3-pilatus-6m/SP16764A_4ghn')
        self.mosflm.sendline('template 184818_0_#####.cbf')
        self.mosflm.sendline('best on')
        #self.mosflm.sendline('image 1')
        #self.mosflm.sendline('image 2')
        self.mosflm.sendline('symm p321')
        self.mosflm.sendline('findspots rmin 5')

    def test(self):
        print('is alive ?', self.mosflm.isalive())
        self.setup_test()
        self.mosflm.sendline('autoindex dps image 1 2')
        self.mosflm.sendline('newmatrix auto.dat')
        self.mosflm.sendline('go')
        self.spacegroup = 'p321'
        if not self.spacegroup:
            index = self.mosflm.expect(['Storing space group from DPS autoindexing:(.*)\r\n', pexpect.EOF, pexpect.TIMEOUT])
        else:
            index = self.mosflm.expect(['Final cell (after refinement) is (.*)\r\n', pexpect.EOF, pexpect.TIMEOUT])

        if index == 0:
            print(self.mosflm.before)
        else:
            if self.mosflm.isalive():
                print("mosflm failed to generate results.")
            else:
                print('mosflm crashed.')

        # parse mosflm before
        # refine a solution and obtain p(x,y), choose which one to refine: penality <= 100?
        #self.mosflm.sendline('autoindex dps solution 1 sdcutoff 2.50  lattice 1')
        #self.mosflm.sendline('separation 0.68 0.68 update close')
        #self.mosflm.sendline('testgen start 127 end 247 angle 0.5')


if __name__ == '__main__':
    x = Mosflm("x").test()
