#!/bin/bash

# Check we're on ID-B
hostname=`hostname -a`
if [ ${hostname:0:3} != "bl2" ]
then
	echo "Must be run at ID-B"
	return -1
fi

# Get PV values
filename=`caget -t 23o:FPE:filename`
dir=`caget -tS 23o:FPE:directory256`
user=`caget -t 23o:bi:runs:currentUser`
energy_keV=`caget -t 23o:MO:E:ActPos`
omega_delta=`caget -t 23o:FPE:delta`
omega_start=`caget -t 23o:FPE:startAngle`
homedir=`bash -c "echo ~$user"`
fulldir="${homedir}/${dir}"
currentFrameNumber=`caget -t 23o:FPE:frameNumber`

# Get frame number from filename
frameSuffix=${filename##*_}
frameNumber=$((10#${frameSuffix%.cbf}))
if [ ${#frameNumber} == 0 ]
then
	frameNumber=1
fi

# Get prefix from filename
suffix_length="-${#frameSuffix}"
prefix=${filename::$suffix_length}

# Get SQL config
streamwriter_ip=`mysql -u dhs -h bl2upper blc2004 -s -N -e "select Value from Config where ID='streamwriter_ip'"`
streamwriter_port=`mysql -u dhs -h bl2upper blc2004 -s -N -e "select Value from Config where ID='streamwriter_port'"`
url="http://${streamwriter_ip}:${streamwriter_port}/set_config"

# Send HTTP POST
params="dir=$fulldir"
params="${params}&prefix=$prefix"
params="${params}&username=$user"
params="${params}&first_frame_label=$frameNumber"
params="${params}&counter_start_value=$currentFrameNumber"
params="${params}&energy_keV=$energy_keV"
params="${params}&continue_series=0"
params="${params}&omega_delta=$omega_delta"
params="${params}&dir=$fulldir"
params="${params}&omega_start=$omega_start"

echo "${url}" "${params}"

curl -v -X POST -d "${params}" "${url}"

