#!/bin/python

import subprocess
import socket
import sys
import os
import random

# Usage: ./testdata.py 30 45
# Where each argument is a cell count for a raster run

def get_sql_host():
    hostname = socket.gethostname()
    if hostname[:2] != 'bl':
        print("Unknown hostname format: "+socket.gethostname())
        sys.exit(-1)
    return hostname[:3]+'upper'

cell_counts = sys.argv[1:]
runs = zip(range(len(cell_counts)),cell_counts)

# open SQL connection and delete existing entries
mysql_conn = subprocess.Popen(('mysql', '-u', 'dhs', '-h', get_sql_host(), 'user_data'), stdin=subprocess.PIPE)
mysql_conn.stdin.write(b"delete from SpotFinderResults3;\n")

for run in runs:
    runIndex = run[0]
    cells = int(run[1])
    
    inHit = False
    lastSignal = 0.0
    for cell in range(0,cells):

        signal = 0.0

        if not inHit:
            startNew = random.random() < 0.05
            if startNew:
                signal = 0.5 + random.random() * 0.5
                inHit = True
            else:
                signal = random.random() * 0.05
        else:
            end = random.random() < 0.2
            if end:
                signal = random.random() * 0.05
                inHit = False
            else:
                signal = lastSignal + random.random() * 0.05
        lastSignal = signal

        spots_4a = int(round(signal * 30.0))
        spots_total = int(round(signal * 50.0))
        res_est = 5.0 - signal * 4.0;
        dozor_spots = spots_total * random.random()*0.1+0.5
        dozor_res = res_est * random.random()*0.1+0.5

        dials_data = '%d %d %.2f' % (spots_4a, spots_total, res_est)
        dozor_data = '%d %.2f' % (dozor_spots, dozor_res)

        statement = b"insert into SpotFinderResults3 values ('INTER_RASTER','%d','%d','DONE','null null null','%s','%s','null','null');" % (runIndex, cell, dials_data, dozor_data)
        mysql_conn.stdin.write(statement)
    
mysql_conn.stdin.close()
mysql_conn.wait()

