#!/usr/bin/perl -W
#--------------------------------------------------------------
# These are PMAC motion controller related subs for beamline controls Perl scripts.
#
#     				Author: Sergey Stepanov
#
# Version-2.0:  2001/02/05 using Pezca
# Version-3.0:  2005/03/28 using CA-monitors array
# Version-4.0:  2005/12/23 added calcmotorTime, merged runMotor & runMotors, added trackMotor
# Version-5.0:  2012/07/21 added getAssyDataFromConfigTcl
# Version-6.0:  2013/05/02 added getAssyDataFromTsub
# Version-6.1:  2014/03/28 switched most passing arrays by value to passing by pointer
# Version-6.2:  2014/11/18 switched trackNmotors to reference style and added returning a message
# Version-6.3:  2018/09/25 switched universal sendCmd/readRsp functions for motor, pmac, and assembly
# Version-6.4:  2019/02/12 added &axis_afterScan (positions axis to startpos,priorpos,maxpos,peakpos,minpos,valleypos,centroid,gaussian,lorentzian...)
# Version-6.5:  2019/02/21 added &generateComponentList (returns array of hashes with comprehensive information about all assemblies from configNN.tcl)
# Version-6.6:  2019/02/25 simplified subs 94-98 to get configNNN.tcl data from generateComponentList instead of parsing it directly
# Version-6.7:  2019/11/98 cleanup of ":" at the end of $assy,$axis,$motor,$pmac
#--------------------------------------------------------------
# List of functions:
# ^^^^^^^^^^^^^^^^^
#   1.             $status = &configAssy ($assy,[$autoMode]);						#autoMode=forceAuto(default)/checkAuto/forceManual
#   2.             $status = &pushAssyStart ($assy);
#   3.             $status = &setAssyAuto ($assy,[$invert]);
#   4.             $status = &setAssyManual ($assy);
#   5.          $autoState = &getAssyAuto ($assy,[\$status]);						#returns "Auto"/"Man" or -1; fills status, if passed
#   6.             $runprg = &getRunPrg ($assy,[\$status]);						#fills status, if passed
#   7.              $inpos = &getInPos ($assy,[\$status]);						#fills status, if passed
#   8.              $homed = &getHomed ($assy,[\$status]);                                  		#fills status, if passed
#   9.                $prg = &getMotionPrg($assy,['string',\$status]);					#fills status, if passed
#  10.               $pmac = &getPmac($assy,[\$status]);							#fills status, if passed
#  11.                $pcs = &getPcs($assy,[\$status]);							#fills status, if passed
#  12.          $accelTime = &getAssyAccelTime ($assy,[\$status]);					#returns acceleration time in ms
#  13.             $ActPos = &getAxisAct ($axis,[\$status]);						#fills status, if passed
#  14.             $RqsPos = &getAxisRqs ($axis,[\$status]);						#fills status, if passed
#  15.             $MemPos = &getAxisMem ($axis,[\$status]);						#fills status, if passed
#  16.             $status = &setAxisRqs ($axis,$rqsPos);						#ATTN: may start moving, if assy is in auto!
#  17.             $RqsBkl = &getAxisBkl ($axis,[\$status]);						#fills status, if passed
#  18.             $status = &setAxisBkl ($axis,$RqsBkl);
#  19.           $drvScale = &getDriveScale ($drive,[\$status]);						#fills status, if passed
#  20.             $drvEGU = &getDriveEGU ($drive,[\$status]);						#fills status, if passed
#  21.             $status = &setAssyMonitors ($assy);
#  22.             $status = &setAxisMonitors ($axis);
#  23.         $motionTime = &calcMotorTime($assy, $axis, $RqsPos);
#  24.         $motionTime = &calcNMotorsTime($assy, $nAxis, \@axis, \@RqsPos);
#  25.             $waited = &waitMotor ($assy,$wait_max,[$wait_start]);					#waited: value=how-long-waited, 0=timeout
#  26.             $waited = &waitMotor_runprg ($assy,$wait_max,[$tell]);				#waited: value=how-long-waited, 0=timeout
#  27.             $status = &runMotor ($assy, $axis, $RqsPos, [$autoMode,\$msg]);			#autoMode=forceAuto/checkAuto
#  28.             $status = &runNMotors ($assy, $nAxis, \@axis, \@RqsPos, [$autoMode,$msg]);		#autoMode=forceAuto/checkAuto
#  29.             $status = &startMotor ($axis, $RqsPos, [$assy, $autoMode]);				#to use autoMode here we need $assy
#  30.             $status = &startNMotors ($assy, $nAxis, \@axis, \@RqsPos, [$autoMode]);		#autoMode=forceAuto/checkAuto
#  31.             $status = &trackMotorFast ($assy,$motionTime,$axis,$RqsPos,$tolerance,[$inposflag]);	#inposflag=RunPrg|InPos
#  32.             $status = &trackMotor ($assy, $motionTime, $axis, [\$msg]);
#  33.             $status = &trackNMotors ($assy, $motionTime, $nAxis, \@axis, [\$msg]);		#also uses EXTRA_MOTION_TIME
#  34.             $AmpEna = &checkMotorAmp ($axis,[$printFlag,\$status]);
#  35. ($l_limit,$h_limit) = &checkMotorLimits ($axis, [$printFlag]);					#returns (0,0) if neither limit is raised
#  36.             $status = &clearAxisLimits ($assy, $axis, $step);					#pushes axis off limits if any is hit
#  37.       ($drvl,$drvh) = &getAxisDRVLH ($axis, [\$status]);						#fills status, if passed
#  38.             $status = &setMotionPrg($assy, $program);
#  39.             $status = &syncAssy($assy);
#  40. ($l_speed,$h_speed) = &calcAxisSpeed ($assy, $axis, $scanRange, $scanTime);
#  41.              $speed = &getAxisSpeed ($assy, $axis,[\$status]);					#fills status, if passed
#  42.            $h_speed = &getAxisHiSpeed ($assy, $axis,[\$status]);					#fills status, if passed
#  43.             $status = &setAxisHiSpeed ($assy, $axis);
#  44.             $status = &setAxisSpeed ($assy, $axis, $speed);
#  45.             $status = &setAxisSlewScan ($assy, $axis, $speed);
#  46.             $status = &restoreAxis ($assy, $axis);
#  47.           $ampstate = &getMtrAmpEnable($motor);							#0=Off, 1=On, -1=PVerr
#  48.           $ampfault = &getMtrAmpFaultErr($motor);					       		#0=OK, 1=fault, -1=PVerr
#  49.            $follerr = &getMtrFatalFolErr($motor);							#0=OK, 1=fault, -1=PVerr
#  50.          $accelTime = &getMtrAccelTime ($motor,[\$status]);					#returns acceleration time in ms
#  51.           $jogSpeed = &getMtrJogSpeed($motor,[\$status]);
#  52.           $prgSpeed = &getMtrPrgSpeed($motor,[\$status]);
#  53.             $status = &setMtrPrgSpeed($motor,$prgSpeed);
#  54.             $status = &abortAssy ($assy, [$restore, $axis]);
#  55.              $mtrNo = &getMtrNo($motor,[\$status]);						#fills status, if passed
#  56.              $mtrNo = &getMtrOutNo_Pmac($pmac,[\$status]);					#fills status, if passed
#  57.              $mtrNo = &getMtrOutNo_Pcs($assy,[\$status]);						#fills status, if passed
#  58.                       &configureMxx80_Pmac($pmac, $mtrNo);
#  59.                       &configureMxx80_Pcs($assy, $mtrNo);
#  60.                       &unlinkMotor_Pmac($pmac,$mtrNo);
#  61.                       &unlinkMotor_Pcs($assy,$mtrNo);
#  62.                       &unlinkMotor32_Pmac($pmac);
#  63.                       &unlinkMotor32_Pcs($assy);
#  64.                       &unlinkMtrOut($assy,$mtrNo);
#  65.             $linked = &isMtrOutLinked($assy,$mtrNo);						#1=yes, 0=no, -N=other assembly
#  66.            $formula = &getLinkedFormula_Pcs($assy,$mtrNo);
#  67.            $formula = &getLinkedFormula($pmac,$pcsNo,$mtrNo);
#  68.                       &linkMtrOut($assy,$mtrNo,$formula,['sync']);
#  69.                       &linkMtrOut_Pmac($pmac,$pcsNo,$mtrNo,$formula,['sync']);
#  70.      $formulaOffset = &determineFormulaOffset($assy, $axis, $formula, $cts2egu);			#determines mtrOut ActPos
#  71.      $formulaOffset = &determineFormulaOffset2($assy, $axis, $formula, $cts2egu);			#applies homez to mtrOut
#  72.                       &stopMtrOut_Pmac($pmac,$mtrNo);
#  73.                       &stopMtrOut_Pcs($assy,$mtrNo);
#  74.             $status = &sendMtrCmd($motor,$cmd,['nowait|wait|wait=nn.nnn']);			#default is "wait" or "1"
#  75.             $status = &sendPmacCmd($pmac,$cmd,['nowait|wait|wait=nn.nnn']);
#  76.             $status = &sendPcsCmd($assy,$cmd,['nowait|wait|wait=nn.nnn']);
#  77.             $status = &sendCmd ($pv, $cmd, ['nowait|wait|wait=nn.nnn']);
#  78.                $rsp = &readMtrRsp($motor,$cmd, [$filter,\$status]);
#  79.                $rsp = &readPmacRsp($pmac,$cmd, [$filter,\$status]);
#  80.                $rsp = &readPcsRsp($assy,$cmd, [$filter,\$status]);
#  81.                $rsp = &readRsp($pvcmd, $pvrsp, $cmd, [$filter,\$status]);
#  82.               $axis = &filterAxis($axis);								#returns bl:axis
#  83.               $assy = &filterAssy($assy);								#returns bl:assy
#  84.               $pmac = &filterPmac($pmac);								#returns bl:pmac
#  85. ($assy,$axis,$cts2egu,$formula,$mtrOut,$struckPs,$struckNg) =
#                            &readMotorList($keyword);							#only gmca-1 records
#  86. ($assy,$axis,$cts2egu,$motor,$encoderAddress) =
#                            &readMotorList2($keyword);							#only gmca-2 records (new fastscanning mode)
#  87. ($cts2egu,$formula,$mtrOut,$struckPs,$struckNg) =
#                            &searchMotorList($assy,$axis,[hardsync|scanrec|any]);			#only gmca-1 records
#  88. ($cts2egu,$mtr,$encoderAddr) =
#                            &searchMotorList2($assy,$axis);     					#only gmca-2 records (new fastscanning mode)
#
#  89.          $ESinTheta = &getESinTheta($assy,[\$status]);						#fills status, if passed
#  90.              $theta = &E2theta($assy,$energy,[$ESinTheta]);
#  91.             $energy = &theta2E($assy,$theta,[$ESinTheta]);
#  92.              $theta = &L2theta($assy,$lambda,[$ESinTheta]);
#  93.             $lambda = &theta2L($assy,$theta,[$ESinTheta]);
#  94.            $numAssy = &generateAssy2Pcs ($beamline,\%assy2pcs);					#reads    configNN.tcl, returns hash with all $assy->$pcs
#  95.            $numAssy = &generateComponentList($beamline,\@list);					#reads    configNN.tcl, returns array of hashes with all information about assy,pcs,pmac,axes,drives,motors,mtr_no
#  96.            $numAssy = &generateAssyList($beamline,\@pvList,[$returnFlag]);			#reads    configNN.tcl, returns assy+axes or assy if $returnFlag='assyOnly'
#  97.             $status = &getAssyList4Pmac ($beamline,$pmac,\@assyNames,\@assyNums); 		#searches configNN.tcl, returns assy names & PCS for given pmac
#  98.             $status = &searchAssyList($beamline,$assy,\@axes);					#searches configNN.tcl, returns @axes for given $assy
#  99.             $status = &searchConfigTcl ($beamline,$assy,$axis,                                    #searches configNN.tcl for given $assy and $axis
#                                             \$pmac,\$axisOrDrive,\$axisOrDriveIndex,			#        ($axisOrDrive: 1=axis, 0=drive)
#                                             \@axes,\@drives,\@motors,\@mtNumbers);			
# 100                $assy = &getAssy($motor,\$status);                                                  #searches configNN.tcl
# 101.             $status = &getAssyDataFromConfigTcl ($beamline,$assy,                                 #searches configNN.tcl for given $assy
#                                             \$pmac,\@axes,\@drives,\@motors,\@mtNumbers);		
# 102.             $status = &getAssyDataFromTsub($assy,\@axes,\@drives,\@motors);			#reads assembly tsub record
# 103.             $status = &getAssyAxesFromTsub($assy,\@axes);						#reads assembly tsub record
# 104.             $status = &getAssyDrivesFromTsub($assy,\@drives);					#reads assembly tsub record
# 105.             $status = &getAssyMotorsFromTsub($assy,\@motors);					#reads assembly tsub record
# 106.             $status = &getAssyByPCS($pmac,$pcs,\$assy);				       		#finds assy name for PMAC & PCS
# 107.             $status = &getMotorSrvStatus($motor,\@bits);
# 108.             $status = &getMotorSrvStat2($motor,\@bits);
# 109.             $status = &getAssyExeSts($assy,\@bits);
# 110.             $status = &getAssySts1($assy,\@bits);
# 111.             $status = &axis_afterScan($afterScan,\@xarr,\@yarr,$begin,$end,$axisIni,$axisPV,$assyPV);
#--------------------------------------------------------------
# use warnings::unused;
  use strict;
# no strict "vars";
# no strict "subs";
# no strict "refs";
# use Pezca;

#=================================================================

# sub configAssy ($assy,[$autoMode]) {
sub configAssy ($;$) {

  our $IGNORE_ERRORS;
  my ($script, $assy, $autoMode, $status, $inpos, $positioned);
  my ($wait_step, $wait_max, $wait_, $current_time, $currentMode, $txt);

  $script     = 'configAssy';
  $assy       = shift(@_);      if ($assy !~ /:$/) {$assy .= ':';}
  $autoMode   = shift(@_);
  $status     = 0;
  $inpos      = 0;			# 0=Moving, 1=Positioned
  $positioned = 0;			# how many times positioned
  $wait_step  = 0.1;
  $wait_max   = 3;
  $wait_      = 0;

### Possible values are: auto=forceAuto(default)/checkAuto/manual=forceManual
  if (!defined $autoMode) {$autoMode = 'forceAuto';}
  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS=0;}

  &setEpicsRetry();			# retry no more than 20 times

### caput nnID:XX:St:Abort 1;  (Abort previous motions)
  $status = Pezca::PutDoubleOldCa($assy.'Abort',1);
  if ($status) {
     &report_exit($script, $status, $assy.'Abort');
     return $status;
  }

### caput nnID:XX:St:AutoStart 0; (switch to Start=Auto)
  if    ($autoMode =~ /^Auto$/i ||
         $autoMode =~ /^forceAuto$/i) {
     $status = Pezca::PutDoubleOldCa($assy.'AutoStart',0);		#0=Auto, 1=Man
     $currentMode = 'Auto';
  }
  elsif ($autoMode =~ /^Man$/i ||
         $autoMode =~ /^Manual$/i ||
         $autoMode =~ /^forceMan$/i ||
         $autoMode =~ /^forceManual$/i) {
     $status = Pezca::PutDoubleOldCa($assy.'AutoStart',1);		#0=Auto, 1=Man
     $currentMode = 'Man';
  }
  elsif ($autoMode =~ /^checkAuto$/i) {
     ($status,$currentMode) = Pezca::GetString($assy.'AutoStart');
     if ($status) {$currentMode = 'Unknown';}
  }
  else {
     print STDOUT '!!! '.$script.': unrecognized AutoStart argument "'.$autoMode.'". Using "Auto"'."\n";
     $autoMode = 'Auto';
     $status = Pezca::PutDoubleOldCa($assy.'AutoStart',0);		#0=Auto, 1=Man
     $currentMode = 'Auto';
  }
  if ($status) {
     &report_exit($script, $status, $assy.'AutoStart');
     return $status;
  }
  if ($autoMode =~ /^checkAuto$/i) {
     print STDOUT 'Assembly='.$assy.' -- current AutoStart='.$currentMode."\n";
  }

### Set monitors on motion status:
  &setAssyMonitors($assy);

### Loop here for the assembly motions to stop (3 seconds at maximum):
  while ($positioned < 3 && $wait_ < $wait_max) {
     select(undef,undef,undef,$wait_step);	# sleep($wait_step);
     $wait_ += $wait_step;
     ($status,$inpos) = Pezca::GetDouble($assy.'InPos');
     if ($status) {
        &report_exit($script, $status, $assy.'InPos');
        return $status;
     }
     if ($inpos == 1) {
        $positioned++;
     }
     else {
        $positioned = 0;
        $status = Pezca::PutDoubleOldCa($assy.'Abort',1);
        if ($status) {
           &report_exit($script, $status, $assy.'Abort');
           return $status;
        }
     }
  }

  if($positioned < 3) {
     $current_time = scalar localtime;
     $txt  = '*** '.$script.': '.$current_time."\n";
     $txt .= '*** '.$script.': Unable to stop '.$assy.': is the amplifier ON?'."\n";
     if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     else                 {print STDOUT $txt; return 1;}
  }
  return 0;
}

#=================================================================

# sub pushAssyStart ($assy) {
sub pushAssyStart ($) {

  my ($script, $assy, $status);

  $script = 'pushAssyStart';
  $assy   = shift(@_);          if ($assy !~ /:$/) {$assy .= ':';}

### caput nnID:XX:St:Start 1;
  $status = Pezca::PutDoubleOldCa($assy.'Start',1);
  if ($status) {
     &report_exit($script, $status, $assy.'Start');
     return $status;
  }
  return 0;
}

#=================================================================

# sub setAssyAuto ($assy,[$invert]) {
sub setAssyAuto ($;$) {

  my ($script, $assy, $invert, $status);

  $script = 'setAssyAuto';
  $assy   = shift(@_);          if ($assy !~ /:$/) {$assy .= ':';}
  $invert   = shift(@_);
  if (!defined $invert)  {$invert = 0;}
  if ($invert ne '0')    {$invert = 1;}

### caput nnID:XX:St:AutoStart 0;
### Switch to Start=0 (Auto) or to Start=1 (Man):
  $status = Pezca::PutDoubleOldCa($assy.'AutoStart',$invert);
  if ($status) {
     &report_exit($script, $status, $assy.'AutoStart');
     return $status;
  }
  return 0;
}

#=================================================================

# sub setAssyManual ($assy) {
sub setAssyManual ($) {

  my ($assy, $status, $invert);
  $assy = shift(@_);          if ($assy !~ /:$/) {$assy .= ':';}

  $invert = 1;
  $status = &setAssyAuto($assy,$invert);
  return $status;
}

#=================================================================

# sub getAssyAuto ($assy,[\$status]) {
sub getAssyAuto ($;\$) {

### This function returns returns "Auto"/"Man" or -1

  my ($script, $assy, $statref, $status, $AutoStart);

  $script  = 'getAssyAuto';
  $assy    = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $statref = shift(@_);

  ($status, $AutoStart) = Pezca::GetDouble($assy.'AutoStart');
  if ($status) {
     &report_exit($script, $status, $assy.'AutoStart');
     return -1;
  }
  if (defined $statref) {$$statref = $status;}

  if ($AutoStart == 0) {return 'Auto';}
  else                 {return 'Man';}
}

#=================================================================

# sub getRunPrg ($assy,[\$status]) {
sub getRunPrg ($;\$) {

### This routine reads RunPrg flag

  my ($script, $assy, $statref, $status, $runprg);

  $script  = 'getRunPrg';
  $assy    = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $statref = shift(@_);

  ($status, $runprg) = Pezca::GetDouble($assy.'RunPrg');
  if ($status) {
     &report_exit($script, $status, $assy.'RunPrg');
     $runprg = 1;
  }
  if (defined $statref) {$$statref = $status;}
  return $runprg;
}

#=================================================================

# sub getInPos ($assy,[\$status]) {
sub getInPos ($;\$) {

### This routine reads InPos flag

  my ($script, $assy, $statref, $status, $inpos);

  $script  = 'getInPos';
  $assy    = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $statref = shift(@_);

  ($status, $inpos) = Pezca::GetDouble($assy.'InPos');
  if ($status) {
     &report_exit($script, $status, $assy.'InPos');
     $inpos = 0;
  }
  if (defined $statref) {$$statref = $status;}
  return $inpos;
}

#=================================================================

# sub getHomed ($assy,[\$status]) {
sub getHomed ($;\$) {

### This routine reads Homed flag

  my ($script, $assy, $statref, $status, $homed);

  $script  = 'getHomed';
  $assy    = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $statref = shift(@_);

  ($status, $homed) = Pezca::GetDouble($assy.'Homed');
  if ($status) {
     &report_exit($script, $status, $assy.'Homed');
     $homed = 0;
  }
  if (defined $statref) {$$statref = $status;}
  return $homed;
}

#=================================================================

# sub getMotionPrg($assy,['string',\$status]) {
sub getMotionPrg ($;$\$) {

### This routine reads Motion program flag

  my ($script, $assy, $string, $statref, $status, $prg);

  $script  = 'getMotionPrg';
  $assy    = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $string  = shift(@_);       if (!defined $string) {$string='';}
  $statref = shift(@_);

  if ($string !~ /^string$/i) {($status, $prg) = Pezca::GetDouble($assy.'Motion');}
  else                        {($status, $prg) = Pezca::GetString($assy.'Motion');}
  if ($status) {
     &report_exit($script, $status, $assy.'Motion');
     if ($string !~ /^string$/i) {$prg = 0;}
     else                        {$prg = 'None';}
  }
  if (defined $statref) {$$statref = $status;}
  return $prg;
}

#=================================================================

# sub getPmac($assy,[\$status]) {
sub getPmac($;\$) {						#returns $pmac (text string)

  my ($script, $assy, $statref, $status, $pmac);

  $script  = 'getPmac';
  $assy    = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $statref = shift(@_);

### Get pmac name for the assembly:
  ($status,$pmac) = Pezca::GetString($assy.'Pmac');
  if ($status) {
     &report_exit($script, $status, $assy.'Pmac');
     $pmac = '';
  }
  if (defined $statref) {$$statref = $status;}
  return $pmac;
}

#=================================================================

# sub getPcs($assy,[\$status]) {
sub getPcs($;\$) {						#returns $pcs number

  my ($script, $assy, $statref, $status, $pcs);

  $script  = 'getPcs';
  $assy    = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $statref = shift(@_);

### Get PCS number for the assembly:
  ($status,$pcs) = Pezca::GetDouble($assy.'Pcs');
  if ($status) {
     &report_exit($script, $status, $assy.'Pcs');
     $pcs = -1;
  }
  if (defined $statref) {$$statref = $status;}
  return $pcs;
}

#=================================================================

# sub getAssyAccelTime ($assy,[\$status])		#returns $accelTimes in ms
sub getAssyAccelTime ($;\$) {

### This function returns $ActualPos

  my ($script, $assy, $statref, $stat1, $stat2);
  my ($accelTime, $accelCurve);

  $script  = 'getAssyAccelTime';
  $assy    = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $statref = shift(@_);

### Query assembly acceleration time:
  ($stat1,$accelTime) = Pezca::GetDouble($assy.'AccelTime');
  if ($stat1) {&report_only($script, $stat1, $assy.'AccelTime'); $accelTime=0;}
  ($stat2,$accelCurve) = Pezca::GetDouble($assy.'AccelCurve');
  if ($stat2) {&report_exit($script, $stat2, $assy.'AccelCurve'); $accelCurve=0;}
  if ($accelTime < 2*$accelCurve) {$accelTime = 2*$accelCurve;}

  if (defined $statref) {$$statref = $stat1+$stat2;}
  return $accelTime;
}

#=================================================================

# sub getAxisAct ($axis,[\$status]) {
sub getAxisAct ($;\$) {

### This function returns $ActualPos

  my ($script, $axis, $statref, $status, $ActualPos);

  $script  = 'getAxisAct';
  $axis    = shift(@_);      if ($axis !~ /:$/) {$axis .= ':';}
  $statref = shift(@_);
### caget nnID:XX:X:ActPos; (get the Actual motor position)
  ($status, $ActualPos) = Pezca::GetDouble($axis.'ActPos');
  if ($status) {
     &report_exit($script, $status, $axis.'ActPos');
     $ActualPos = 0;
  }
  if (defined $statref) {$$statref = $status;}
  return $ActualPos;
}

#=================================================================

# sub getAxisRqs ($axis,[\$status]) {
sub getAxisRqs ($;\$) {

### This function returns $RqsPos

  my ($script, $axis, $statref, $status, $RqsPos);

  $script  = 'getAxisRqs';
  $axis    = shift(@_);      if ($axis !~ /:$/) {$axis .= ':';}
  $statref = shift(@_);
### caget nnID:XX:X:RqsPos; (get the Requested motor position)
  ($status, $RqsPos) = Pezca::GetDouble($axis.'RqsPos');
  if ($status) {
     &report_exit($script, $status, $axis.'RqsPos');
     $RqsPos = 0;
  }
  if (defined $statref) {$$statref = $status;}
  return $RqsPos;
}

#=================================================================

# sub getAxisMem ($axis,[\$status]) {
sub getAxisMem ($;\$) {

### This function returns $MemPos

  my ($script, $axis, $statref, $status, $MemPos);

  $script  = 'getAxisMem';
  $axis    = shift(@_);      if ($axis !~ /:$/) {$axis .= ':';}
  $statref = shift(@_);
### caget nnID:XX:X:RqsPos; (get the Requested motor position)
  ($status, $MemPos) = Pezca::GetDouble($axis.'MemPos');
  if ($status) {
     &report_exit($script, $status, $axis.'MemPos');
     $MemPos = 0;
  }
  if (defined $statref) {$$statref = $status;}
  return $MemPos;
}

#=================================================================

# sub setAxisRqs ($axis,$rqsPos) {
sub setAxisRqs ($$) {

### This function sets $RqsPos
### ATTENTION: if the assembly is in "auto", it may start moving!

  my ($script, $axis, $rqsPos, $status);

  $script = 'setAxisRqs';
  $axis   = shift(@_);      if ($axis !~ /:$/) {$axis .= ':';}
  $rqsPos = shift(@_);
  $status = Pezca::PutDoubleOldCa($axis.'RqsPos',$rqsPos);
  &report_exit($script, $status, $axis.'RqsPos');
  return $status;
}

#=================================================================

# sub getAxisBkl ($axis,[\$status]) {
sub getAxisBkl ($;\$) {

### This function returns $RqsBkl

  my ($script, $axis, $statref, $status, $RqsBkl);

  $script  = 'getAxisBkl';
  $axis    = shift(@_);      if ($axis !~ /:$/) {$axis .= ':';}
  $statref = shift(@_);
### caget nnID:XX:X:RqsBkl; (get the motor backlash)
  ($status, $RqsBkl) = Pezca::GetDouble($axis.'RqsBkl');
  if ($status) {
     &report_only($script, $status, $axis.'RqsBkl');
     $RqsBkl = 0;
  }
  if (defined $statref) {$$statref = $status;}
  return $RqsBkl;
}

#=================================================================

# sub setAxisBkl ($axis,$RqsBkl) {
sub setAxisBkl ($$) {

### This function sets $RqsBkl

  my ($script, $axis, $status, $RqsBkl);

  $script  = 'setAxisBkl';
  $axis    = shift(@_);      if ($axis !~ /:$/) {$axis .= ':';}
  $RqsBkl  = shift(@_);
### caput nnID:XX:X:RqsBkl; (set the motor backlash)
  $status = Pezca::SetDoubleOldCa($axis.'RqsBkl',$RqsBkl);
  if ($status) {
     &report_only($script, $status, $axis.'RqsBkl');
  }
  return $status;
}

#=================================================================

# sub getDriveScale ($drive,[\$status]) {
sub getDriveScale ($;\$) {

### This function returns $drvScale

  my ($script, $drive, $statref, $status, $drvScale, $pv);

  $script  = 'getDriveScale';
  $drive   = shift(@_);      if ($drive !~ /:$/) {$drive .= ':';}
  $statref = shift(@_);
### caget nnID:XX:X:Scale; (get scale factor from mtr to drv)
  $pv = $drive.'Scale';
  ($status, $drvScale) = Pezca::GetDouble($pv);
  if ($status) {
     &report_exit($script, $status, $pv);
     $drvScale = 1;
  }
  if (defined $statref) {$$statref = $status;}
  return $drvScale;
}

#=================================================================

# sub getDriveEGU ($drive,[\$status]) {
sub getDriveEGU ($;\$) {

### This function returns $drvEGU

  my ($script, $drive, $statref, $status, $drvEGU, $pv);

  $script  = 'getDriveEGU';
  $drive   = shift(@_);      if ($drive !~ /:$/) {$drive .= ':';}
  $statref = shift(@_);
### caget nnID:XX:X:EGU; (get EGU units of drv)
  $pv = $drive.'ActPos.EGU';
  ($status, $drvEGU) = Pezca::GetString($pv);
  if ($status) {
     &report_exit($script, $status, $pv);
     $drvEGU = '';
  }
  if (defined $statref) {$$statref = $status;}
  return $drvEGU;
}

#=================================================================

# sub setAssyMonitors ($assy) {
sub setAssyMonitors ($) {

  my ($assy, $status);

  $assy   = shift(@_);          if ($assy !~ /:$/) {$assy .= ':';}
  $status = 0;
  $status  += &addCaMonitor($assy.'InPos');
  $status  += &addCaMonitor($assy.'RunPrg');
  return $status;
}

#=================================================================

# sub setAxisMonitors ($axis) {
sub setAxisMonitors ($) {

  my ($axis, $status);

  $axis   = shift(@_);      if ($axis !~ /:$/) {$axis .= ':';}
  $status = 0;
  $status += &addCaMonitor($axis.'AmpEna');
  $status += &addCaMonitor($axis.'NgLimSet');
  $status += &addCaMonitor($axis.'PsLimSet');
  $status += &addCaMonitor($axis.'ActPos');			#added 2005/07/20
# $status += &addCaMonitor($axis.'RqsPos');		#	added 2016/03/30, but then commented out (was causing issues with tracking)
  return $status;
}

#=================================================================

# sub calcMotorTime ($assy, $axis, $RqsPos) {
sub calcMotorTime ($$$) {

### To be executed before a motor starts. Returns estimated $motionTime
### All Axes are assumed to belong to the same assembly!

  my ($assy, $axis, $RqsPos, $nAxis, $motionTime);

  $assy   = shift(@_);      if ($assy !~ /:$/) {$assy .= ':';}
  $axis   = shift(@_);      if ($axis !~ /:$/) {$axis .= ':';}
  $RqsPos = shift(@_);

  $nAxis = 1;
  $motionTime = &calcNMotorsTime ($assy, $nAxis, [$axis], [$RqsPos]);

  return $motionTime;
}

#=================================================================

# sub calcNMotorsTime ($assy, $nAxis, \@axis, \@RqsPos) {
sub calcNMotorsTime ($$\@\@) {

### To be executed before a motor starts. Returns estimated $motionTime
### All Axes are assumed to belong to the same assembly!

  our ($report, $MONO_ENERGY, $MONO_LAMBDA, $MONO_THETA);
  my ($script, $assy, $nAxis, $axisRef, $rqsRef);
  my ($naPassed,  $nrPassed, $RqsPos, $ActPos, $axis);
  my ($speed, $accelTime, $motionTime, $mt, $status, $ESinTheta, $i);

  $script   = 'calcNMotorsTime';
  $assy     = shift(@_);      if ($assy !~ /:$/) {$assy .= ':';}
  $nAxis    = shift(@_);
  $axisRef  = shift(@_);
  $rqsRef   = shift(@_);

  $naPassed = @$axisRef;
  $nrPassed = @$rqsRef;
  if ( $naPassed < $nAxis || $nrPassed < $nAxis ) {
     print '*** '.$script.': passed arrays are shorter than nAxis='.$nAxis."\n"
          .'*** '.$script.': @axis has '.$naPassed.', @rqs has '.$nrPassed."\n"
          .'*** '.$script.': @axis='."@$axisRef".')'."\n";
     return -1;
  }
  if (!defined $report) {$report = 0;}
### There are two exceptions at the beamline when axis speed is depends
### on the postion. Those are mono energy and mono wavelength. For them
### we have to calculate the speed of Bragg angle:
  if (!defined $MONO_ENERGY) {$MONO_ENERGY = 'MO:E';}		#name used at GMCA
  if (!defined $MONO_LAMBDA) {$MONO_LAMBDA = 'MO:L';}		#name used at GMCA
  if (!defined $MONO_THETA)  {$MONO_THETA  = 'MO:QB';}		#name used at GMCA

### Query assembly acceleration time:
  $accelTime = &getAssyAccelTime($assy, \$status);

  $RqsPos = 0.0;
  $ActPos = 0.0;                                        	#motor position at start
  $motionTime = 0.0;
### Calc motion time per motor and find maximum time:
  for ($i=0; $i<$nAxis; $i++) {
     $axis   = @$axisRef[$i];     if ($axis !~ /:$/) {$axis .= ':';}
     $RqsPos = @$rqsRef[$i];
     $ActPos = &getAxisAct($axis);				#Query position at motor start
     if ($axis =~/^.*$MONO_ENERGY\:?$/) {      			#Special case: mono energy
#       print STDOUT $script.': Axis='.$axis.' is Mono Energy -- using '.$MONO_THETA."\n";
        $ESinTheta = &getESinTheta($assy);
        $RqsPos = &E2theta($assy,$RqsPos,$ESinTheta);
        $ActPos = &E2theta($assy,$ActPos,$ESinTheta);
        $axis =~ s/$MONO_ENERGY/$MONO_THETA/;
     }
     elsif ($axis =~/^.*$MONO_LAMBDA\:?$/) {			#Special case: mono wavelength
#       print STDOUT $script.': Axis='.$axis.' is Mono Lambda -- using '.$MONO_THETA."\n";
        $ESinTheta = &getESinTheta($assy);
        $RqsPos = &L2theta($assy,$RqsPos,$ESinTheta);
        $ActPos = &L2theta($assy,$ActPos,$ESinTheta);
        $axis =~ s/$MONO_LAMBDA/$MONO_THETA/;
     }
     $speed = &getAxisSpeed($assy, $axis);
### Calculate motion time for this motor:
     if ($speed) {
        $mt = abs(($RqsPos - $ActPos) / $speed);
### Nothing at the beamline can move longer than 30min:
        if ($mt > 30*60) {$mt = 30*60;}
#       print STDOUT 'speed='.$speed.', time='.$mt."\n";
        if ($mt > $motionTime) {$motionTime = $mt;}
     } else {
        $mt = 0.0;
     }
     if ($report >= 2) {
        printf STDOUT '--- '.$script.': Axis=%s motionTime=%.1fs'."\n",$axis,$mt;
        printf STDOUT '              speed=%.4f rqs=%.2f act=%.2f'."\n",$speed,$RqsPos,$ActPos;
     }
  }
### Add acceleration + decceleration time:
  $motionTime += 0.002*$accelTime;		#recalculate $accelTime from milliseconds
### ATTENTION: in principle, we should also
###            add the backlash motion time!
### Add backlash acceleration:
  $motionTime += 0.002*$accelTime;		#recalculate $accelTime from milliseconds
  if ($report) {
     printf STDOUT 'Estimated motion time for assembly='.$assy.' is %.1fs'."\n",$motionTime;
  }
  return $motionTime;
}

#=================================================================

# sub waitMotor ($assy,$wait_max,[$wait_start]) {
sub waitMotor ($$;$) {

  my ($assy, $wait_max, $wait_start, $wait_step, $wait_);
  my ($status, $runprg, $inpos, $positioned);

  $assy       = shift(@_);      if ($assy !~ /:$/) {$assy .= ':';}
  $wait_max   = shift(@_);
  $wait_start = shift(@_);      if (!defined $wait_start) {$wait_start = 0.4;}
  $wait_step  = 0.1;
  $wait_      = 0;
  $positioned = 0;

  &setAssyMonitors ($assy);

  select(undef,undef,undef,$wait_start);			#sleep($wait_step);
  $wait_  += $wait_start;

  do {
     select(undef,undef,undef,$wait_step);			#sleep($wait_step);
     $wait_  += $wait_step;
     ($status,$inpos)  = Pezca::GetDouble($assy.'InPos');
     if ($status) {$inpos = 0;}					#presume worst scenario
     ($status,$runprg) = Pezca::GetDouble($assy.'RunPrg');
     if ($status) {$runprg = 1;}				#presume worst scenario
     if ($inpos == 1)  {$positioned++;}
     else              {$positioned = 0;};
#    print STDOUT 'waitMotor: runprg='.$runprg.' inpos='.$inpos.' positioned='.$positioned.' wait_='.$wait_.' wait_max='.$wait_max."\n";
  } while (($runprg == 1 || $positioned < 3) && $wait_ <= $wait_max);

  if ($wait_ > $wait_max) {return 0;}
  else                    {return $wait_;}

}

#=================================================================

# sub waitMotor_runprg ($assy,$wait_max,[$tellwaiting]) {
sub waitMotor_runprg ($$;$) {

  my ($script, $assy, $wait_max, $wait_start, $wait_step, $wait_);
  my ($status, $runprg, $tellwaiting, $was_running);

  $script      = 'waitMotor_runprg';
  $assy        = shift(@_);     if ($assy !~ /:$/) {$assy .= ':';}
  $wait_max    = shift(@_);
  $tellwaiting = shift(@_);     if (!defined $tellwaiting) {$tellwaiting = 'silent';}
  $wait_start  = 0.4;
  $wait_step   = 0.1;
  $wait_       = 0;
  $was_running = 0;

  &setAssyMonitors ($assy);

  select(undef,undef,undef,$wait_start);			#sleep($wait_step);
  $wait_  += $wait_start;

  do {
     select(undef,undef,undef,$wait_step);			#sleep($wait_step);
     $wait_ += $wait_step;
     ($status,$runprg) = Pezca::GetDouble($assy.'RunPrg');
     if ($runprg) {$was_running = 1;}
#    print STDOUT $script.': runprg='.$runprg.' wait_='.$wait_.' wait_max='.$wait_max."\n";
  } while ($runprg == 1 && $wait_ <= $wait_max);

  if ($tellwaiting =~ /^1$/ || $tellwaiting =~ /^tell$/i) {
     if ($was_running) {
        printf STDOUT '!!! '.$script.': waited for %1.0fs while assembly='.$assy
                                    .' was executing motion program'."\n",$wait_;
     }
  }

  if ($wait_ > $wait_max) {return 0;}
  else                    {return $wait_;}

}

#=================================================================

# sub runMotor ($assy, $axis, $RqsPos, [$autoMode,\$msg]) {
sub runMotor ($$$;$\$) {

  my ($assy, $axis, $RqsPos, $autoMode, $msgRef, $nAxis, $status);
  $assy     = shift(@_);     if ($assy !~ /:$/) {$assy .= ':';}
  $axis     = shift(@_);     if ($axis !~ /:$/) {$axis .= ':';}
  $RqsPos   = shift(@_);
  $autoMode = shift(@_);
  $msgRef   = shift(@_);

  $nAxis  = 1;
  $status = &runNMotors ($assy, $nAxis, [$axis], [$RqsPos], $autoMode, $msgRef);
  return $status;
}

#=================================================================

# sub runNMotors ($assy, $nAxis, \@axis, \@RqsPos, [$autoMode,\$msg]) {
sub runNMotors ($$\@\@;$\$) {

### All Axes are assumed to belong to the same assembly!

  our $report;
  my ($script, $assy, $nAxis, $axisRef, $rqsRef, $autoMode, $msgRef);
  my (@axis, @RqsPos, $naPassed, $nrPassed, $txt);
  my ($i, $act, $status, $motionTime);

  $script   = 'runNMotors';
  $assy     = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $nAxis    = shift(@_);
  $axisRef  = shift(@_);
  $rqsRef   = shift(@_);
  $autoMode = shift(@_);
  $msgRef   = shift(@_);

  $naPassed = @$axisRef;
  $nrPassed = @$rqsRef;
  if ( $naPassed < $nAxis || $nrPassed < $nAxis ) {
     $txt =  '*** '.$script.': passed arrays are shorter than nAxis='.$nAxis."\n"
            .'*** '.$script.': @axis has '.$naPassed.', @rqs has '.$nrPassed."\n"
            .'*** '.$script.': @axis='."@$axisRef".')'."\n";
     print STDOUT $txt;
     if (defined $msgRef) {$$msgRef = $txt;}
     return -1;
  }

  for ($i=0; $i<$nAxis; $i++) {
     $axis[$i]   = @$axisRef[$i];    if ($axis[$i] !~ /:$/) {$axis[$i] .= ':';}
     $RqsPos[$i] = @$rqsRef[$i];
  }

  if (!defined $report) {$report = 0;}
  if ($report >= 2) {
     print STDOUT $script.': moving assembly ['.$assy.'] of '.$nAxis.' axes:'."\n";
     for ($i=0; $i<$nAxis; $i++) {
        $act = &getAxisAct($axis[$i]);
        printf STDOUT ' '.$axis[$i].'  %.3g --> '.$RqsPos[$i]."\n",$act;
     }
  }

### Estimate motion time:
# $motionTime = &calcNMotorsTime($assy, $nAxis, \@axis, \@RqsPos);
  $motionTime = &calcNMotorsTime($assy, $nAxis, $axisRef, $rqsRef);

### Start motion:
# $status = &startNMotors ($assy, $nAxis, \@axis, \@RqsPos, $autoMode);
  $status = &startNMotors ($assy, $nAxis, $axisRef, $rqsRef, $autoMode);
  if ($status) {
     if (defined $msgRef) {$$msgRef = '*** '.$script.': error starting '.$assy."\n";}
     return $status;
  }

### Track motion until it is finished:
  $status = &trackNMotors($assy,$motionTime,$nAxis,\@axis,$msgRef);
  return $status;
}

#=================================================================

# sub startMotor ($axis, $RqsPos, [$assy, $autoMode]) {
sub startMotor ($$;$$) {

  my ($script, $axis, $RqsPos, $assy, $autoMode, $autoState, $status);

  $script   = 'startMotor';
  $axis     = shift(@_);        if ($axis !~ /:$/) {$axis .= ':';}
  $RqsPos   = shift(@_);
  $assy     = shift(@_);        if (!defined $assy) {$assy = '';}
  $autoMode = shift(@_);        if (!defined $autoMode) {$autoMode = '';}
  $autoState = 'Auto';						#presume by default

### AutoMode defines handling AutoSetup button. Possible cases:
###    0 or ''  - presume Setup=Auto and do nothing (just set RqsPos) -- DEFAULT
### 'forceAuto' - switch assembly to 'Auto' and then enter the RqsPos
### 'checkAuto' - check in the status of assembly and hit 'Start' if it is in 'Manual'.

  if (length($assy) > 0) {
     $status = &abortAssy($assy);				#abort previous motion
     if ($status) {return $status;}
  }

  if    ($autoMode =~ /^forceauto$/i) {
     if (length($assy) == 0) {return -1;}
     $status = &setAssyAuto($assy);
     if ($status) {return $status;}
  }
  elsif ($autoMode =~ /^checkauto$/i) {
     if (length($assy) == 0) {return -1;}
     $autoState = &getAssyAuto($assy);				#returns 'Auto'/'Man' or '-1'
     if ($autoState eq '-1') {return $autoState;}		#EZCA error
  }

  $status = &setAxisRqs ($axis,$RqsPos);
  if ($status) {return $status;}

  if ($autoState =~ /^Man$/i) {
### Start motion:
     if (length($assy) == 0) {return -1;}
     $status = &pushAssyStart($assy);
     if ($status) {return $status;}
  }

  return 0;
}

#=================================================================

# sub startNMotors ($assy, $nAxis, \@axis, \@RqsPos, [$autoMode]) {
sub startNMotors ($$\@\@;$) {

### All Axes are assumed to belong to the same assembly!

  our $report;
  my ($script, $assy, $nAxis, $axisRef, $rqsRef, $autoMode);
  my (@axis, @RqsPos, $naPassed, $nrPassed);
  my ($autoState, $status, $i);

  $script   = 'startNMotors';
  $assy     = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $nAxis    = shift(@_);
  $axisRef  = shift(@_);
  $rqsRef   = shift(@_);
  $autoMode = shift(@_);
### AutoMode is only used when one motor is driven.
### It defines handling AutoSetup button. Possible
### cases:
###     0       - presume Setup=Auto and do nothing (just set RqsPos) -- DEFAULT
### "forceAuto" - switch assembly to "Auto" and then enter the RqsPos
### "checkAuto" - check in the status of assembly and hit "Start" if it is in "Manual".
  if (!defined $autoMode) {$autoMode = 0;}
  $autoMode = lc ($autoMode);
  $autoState = 0;
  if (!defined $report) {$report = 0;}

  $naPassed = @$axisRef;
  $nrPassed = @$rqsRef;
  if ( $naPassed < $nAxis || $nrPassed < $nAxis ) {
     print '*** '.$script.': passed arrays are shorter than nAxis='.$nAxis."\n"
          .'*** '.$script.': @axis has '.$naPassed.', @rqs has '.$nrPassed."\n"
          .'*** '.$script.': @axis='."@$axisRef".')'."\n";
     return -1;
  }

  for ($i=0; $i<$nAxis; $i++) {
     $axis[$i]   = @$axisRef[$i];   if ($axis[$i] !~ /:$/) {$axis[$i] .= ':';}
     $RqsPos[$i] = @$rqsRef[$i];
  }

  $status = &abortAssy($assy);					#abort previous motion
  if ($status) {return $status;}

  if ($nAxis > 1) {
### Switch to Manual start with multiple motors (0=Auto, 1=Man):
     $status = &setAssyManual($assy);
     if ($status) {return $status;}
  } else {
### When motor is only one,
### autostart operations
### are optional:
     if    ($autoMode =~ /^forceauto$/i) {
        $status = &setAssyAuto($assy);
        if ($status) {return $status;}
     }
     elsif ($autoMode =~ /^checkauto$/i) {
        $autoState = &getAssyAuto($assy);			#returns "Auto"/"Man" or -1
        if ($autoState eq '-1') {return $autoState;}		#EZCA error
     }
  }

### Enter requested positions:
  for ($i=0; $i<$nAxis; $i++) {
     if ($report >= 2) {
### Report start of motion:
        if ($i == 0) {print STDOUT 'Moving ';}
        else         {print STDOUT ',  ';}
        print STDOUT $axis[$i].' to '.$RqsPos[$i];
        if ($i == $nAxis-1) {print STDOUT "\n";}
     }
     $status = &setAxisRqs ($axis[$i],$RqsPos[$i]);
     if ($status) {return $status;}
  }

  if ($nAxis > 1) {
### Switch back to Autostart:
     $status = &setAssyAuto($assy);
     if ($status) {return $status;}
  }

# if ($nAxis > 1 || (($nAxis == 1) && ($autoMode =~ /^checkauto$/i) && ($autoState =~ /^Man$/i))) {
  if ($nAxis > 1 || $autoState =~ /^Man$/i) {
### Start motion:
     $status = &pushAssyStart($assy);
     if ($status) {return $status;}
  }
  return 0;
}

#=================================================================

# sub trackMotorFast ($assy, $motionTime, $axis, $RqsPos, $tolerance, [$inposflag]) {
sub trackMotorFast ($$$$$;$) {

### Simplified procedure for tracking single motor only. Includes position tolerance.

  our ($IGNORE_ERRORS, $EXTRA_MOTION_TIME, $nonstop);
  my ($script, $assy, $motionTime, $axis, $RqsPos, $tolerance, $inposflag);
  my ($status, $wait_step, $report_step, $wait_, $wait_report);
  my ($ActPos, $diff, $runprg, $inpos, $ampEnabled, $limStatus, $maxTime);
  my (@ax, @dr, @mt, $mtr, $flag, $txt);

  $script     = 'trackMotorFast';
  $assy       = shift(@_);    if ($assy !~ /:$/) {$assy .= ':';}
  $motionTime = shift(@_);						#motion timeout
  $axis       = shift(@_);    if ($axis !~ /:$/) {$axis .= ':';}
  $RqsPos     = shift(@_);
  $tolerance  = shift(@_);						#acceptable deviation from RqsPos
  $inposflag  = shift(@_);						#'RunPrg' or 'InPos'
  if (!defined $inposflag) {$inposflag='RunPrg';}
  if    ($inposflag =~ /^runprg$/i) {$inposflag='RunPrg';}
  elsif ($inposflag =~ /^inpos$/i)  {$inposflag='InPos';}
  else	{
     print STDOUT '!!! '.$script.': incorrect InPos flag "'.$inposflag.'". Presuming "RunPrg"'."\n";
     $inposflag = 'RunPrg';
  }
  $wait_step   = 0.05;							#in seconds
  $report_step = 30.;							#in seconds
### Global variables (can be customized per
### motor if the defaults are not good):
  if (!defined $IGNORE_ERRORS) 		{$IGNORE_ERRORS=0;}
  if (!defined $nonstop)               	{$nonstop = 0;}
  if (!defined $EXTRA_MOTION_TIME)     	{$EXTRA_MOTION_TIME = 5;}	#seconds
  $maxTime = 2 * $motionTime + $EXTRA_MOTION_TIME;

### Set monitors on motion status:
  &setAssyMonitors($assy);
  &setAxisMonitors($axis);

  $wait_       = 0;
  $wait_report = 0;
### =============================Begin-tracking=============================
### Loop here for motor to finish motion ($maxTime seconds at maximum):
  do {
     select(undef,undef,undef,$wait_step);				#sleep($wait_step);
     $wait_        += $wait_step;
     $wait_report  += $wait_step;
     ($status,$ActPos) = Pezca::GetDouble($axis.'ActPos');
     if ($status) {
        print STDOUT '!!! '.$script.': quit tracking on '
                    .$axis.' position unreadable'."\n"; 
        return 1;
     }
     $diff = abs ($ActPos - $RqsPos);
     if ($wait_report > $report_step) {
        $wait_report = 0;
        printf STDOUT '  Moving ['.$assy.'] for ~ %1.0fs of %1.0fs,'
                     .' ActPos=%.3f, RqsPos=%.3f'."\n",
                      $wait_,$motionTime,$ActPos,$RqsPos;
     }
     if ($inposflag =~ /^RunPrg$/i) {
        ($status,$runprg) = Pezca::GetDouble($assy.'RunPrg');		#0=Idle,   1=Running
        if ($status) {$runprg = 1;}					# presume worst scenario
        $inpos = 1 - $runprg;
     } else {
        ($status,$inpos)  = Pezca::GetDouble($assy.'InPos');		#0=Moving, 1=Positioned
        if ($status) {$inpos = 0;}					#presume worst scenario
     }
     ($status,$ampEnabled) = Pezca::GetDouble($axis.'AmpEna');
     if ($status) {$ampEnabled = 0;}					#presume worst
  } while (($inpos == 0 || $diff > $tolerance) && 
           $ampEnabled                         && 
           $wait_ < $maxTime);
  if ($ampEnabled == 0) {
     printf STDOUT '!!! '.$script.': quit tracking on '.$axis
                         .' amplifier fault, ActPos=%.3f.'."\n",$ActPos;
     $status = &getAssyDataFromTsub($assy,\@ax,\@dr,\@mt);
     if ($status) {
        print STDOUT '!!! '.$script.': failed to get assembly data for '.$assy."\n";
     } else {
        foreach $mtr (@mt) {
           $flag = &getMtrAmpFaultErr($mtr);				#0=OK, 1=fault, -1=PVerr
           if ($flag == 1) {
              print STDOUT '!!! '.$script.': amplifier fault error for motor='
                                 .$mtr.' in assembly='.$assy."\n";
           }
           $flag = &getMtrFatalFolErr($mtr);				#0=OK, 1=fault, -1=PVerr
           if ($flag == 1) {
              print STDOUT '!!! '.$script.': fatal following error for motor='
                                 .$mtr.' in assembly='.$assy."\n";
           }
        }
     }
     return 1;
  }
### =============================End-of-tracking============================

### Timeout?
  if ($wait_ > $maxTime) {
     $txt = '!!! '.$script.': quit tracking '.$assy.' on motion timeout'."\n";
### Limit hit?:
     ($status,$limStatus) = Pezca::GetDouble($axis.'NgLimSet');
     if ($status) {$limStatus = 0;}
     if ($limStatus) {
        $txt .= sprintf '!!! '.$script.': Motor '.$axis
                       .' hit negative limit at ActPos=%.3f'."\n",$ActPos;
     }
     ($status,$limStatus) = Pezca::GetDouble($axis.'PsLimSet');
     if ($status) {$limStatus = 0;}
     if ($limStatus) {
        $txt .= sprintf '!!! '.$script.': Motor '.$axis
                        .' hit positive limit at ActPos=%.3f'."\n",$ActPos;
     }
     $txt .= sprintf '*** '.$script.': '.$assy.' did not arrive at '
                           .'RqsPos=%.3f after %1.0fs (ActPos=%.3f)'."\n",
                           $wait_,$RqsPos,$ActPos;
     if    ($nonstop == 2) {
        $txt .= '!!! '.$script.': ... continuing script'."\n";
     }
     elsif ($nonstop == 1) {
        $txt .= '!!! '.$script.': ... aborting motion & continuing script'."\n";
        &abortAssy($assy);
     }
     else {
        &abortAssy ($assy);
        if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     }
     print STDOUT $txt;
     return 1;
  }
  return 0;
}

#=================================================================

# sub trackMotor ($assy, $motionTime, $axis, [\$msg]) {
sub trackMotor ($$$;\$) {

  my ($assy, $motionTime, $axis, $nAxis, $msgRef, $status);

  $assy       = shift(@_);
  $motionTime = shift(@_);
  $axis       = shift(@_);
  $msgRef     = shift(@_);

  $nAxis  = 1;
  $status = &trackNMotors($assy,$motionTime,$nAxis,[$axis],$msgRef);
  return $status;
}

#=================================================================

# sub trackNMotors ($assy, $motionTime, $nAxis, \@axis, [\$msg]) {
sub trackNMotors ($$$\@;\$) {

### Returns: 0 = OK,
###          1 = amplifier fault
###          2 = hit any limit
###          4 = timeout
###          8 = out of InPos band

### All Axes are assumed to belong to the same assembly!

  our ($IGNORE_MOTION_ESTIMATE, $IGNORE_ERRORS, $EXTRA_MOTION_TIME);
  our ($MOTION_TIME_ALLOWANCE, $diff_max, $report);
  our ($nonstop, $motor_startup_time);
  my ($script, $assy, $motionTime, $nAxis, $axisRef, $msgRef);
  my (@axis, @Distance, $timeout, $outofband, $status, $txt);
  my (@RqsPos, @OldPos, @PrvPos, @ActPos, @limSign, @limStatus, @ampEnabled);
  my ($diff, $allAmpsEnabled, $hitAnyLimit, $inpos, $runprg, $positioned);
  my ($ap_, $d1, $d2, $current_time, $maxTime, $warnTime);
  my ($wait_, $wait_report, $wait_step, $wait_max, $report_step);
  my ($hasStarted, $stillAtStart, $returnStatus, $progressing);
  my (@ax, @dr, @mt, $mtr, $flag, $i);
#					# @RqsPos  - destination position
#			 		# @OldPos  - motors positions at start
#					# @ActPos  - actual positions
#					# @limSign - what limit switch to check per motor
#					# @limStatus - limit switch status per motor
#					# @ampEnabled - amplifier enabled status per motor

  $script     = 'trackNMotors';
  $assy       = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $motionTime = shift(@_);
  $nAxis      = shift(@_);
  $axisRef    = shift(@_);
  $msgRef     = shift(@_);

  &setAssyMonitors($assy);						#set monitors on the assembly common PVs
  for ($i=0; $i<$nAxis; $i++) {
     $axis[$i] = @$axisRef[$i];  if ($axis[$i] !~ /:$/) {$axis[$i] .= ':';}
     &setAxisMonitors($axis[$i]);					#set monitors on axis motors
     $OldPos[$i]   = &getAxisAct($axis[$i]);
     $PrvPos[$i]   = $OldPos[$i];
     $RqsPos[$i]   = &getAxisRqs($axis[$i]);
     $Distance[$i] = abs($RqsPos[$i] - $OldPos[$i]);
     if ($Distance[$i] < 0.001) {
        $limSign[$i] = 'none';
     }
     elsif ($RqsPos[$i] > $OldPos[$i]) {
        $limSign[$i] = 'Ps'; &addCaMonitor($axis[$i].'PsLimSet');
     }
     elsif ($RqsPos[$i] < $OldPos[$i]) {
        $limSign[$i] = 'Ng'; &addCaMonitor($axis[$i].'NgLimSet');
     }
     else {
       $limSign[$i] = 'unexpected';
     }
     $limStatus[$i]  = 0;
     $ampEnabled[$i] = 1;
  }
### Global variables (can be customized per
### motor if the defaults are not good):
  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS=0;}
  if (!defined $diff_max)              {$diff_max = 1.;            }
  if (!defined $report)                {$report   = 0;             }
  if (!defined $nonstop)               {$nonstop  = 0;             }
  if (!defined $motor_startup_time)    {$motor_startup_time    = 1;}	#seconds
  if (!defined $EXTRA_MOTION_TIME)     {$EXTRA_MOTION_TIME     = 5;}	#seconds
  if (!defined $MOTION_TIME_ALLOWANCE) {$MOTION_TIME_ALLOWANCE = 3;}	#seconds
  $wait_step   = 0.1;
  $report_step = 30.;							#in seconds
  $hitAnyLimit = 0;             					#none limit is hit so far
  $wait_       = 0;
  $wait_report = 0;
  $positioned  = 0;							#how many times positioned
  $hasStarted  = 0;							#motor has/(has not) started motion
  $outofband   = 0;
  $timeout     = 0;
  $maxTime     = 2 * $motionTime + $EXTRA_MOTION_TIME;
  $warnTime    = $motionTime + $MOTION_TIME_ALLOWANCE;

### =============================Begin-tracking=============================
### Loop here for motor to finish motion ($maxTime seconds at maximum):
### (MOTION is a label of the loop)
# MOTION:
  while (1) {
     select(undef,undef,undef,$wait_step);				#sleep($wait_step);
     $wait_        += $wait_step;
     $wait_report  += $wait_step;
     if ($wait_report > $report_step) {
        $wait_report = 0;
        printf STDOUT '  Moving [%s] for ~ %1.0fs of %1.0fs'."\n",
                         $assy,$wait_,$motionTime;
     }
     ($status,$inpos)  = Pezca::GetDouble($assy.'InPos');		#0=Moving, 1=Positioned
     if ($status) {$inpos = 0;}						#presume worst scenario
     ($status,$runprg) = Pezca::GetDouble($assy.'RunPrg');		#0=Idle,   1=Running
     if ($status) {$runprg = 1;}					#presume worst scenario
     if ($runprg == 1) {$hasStarted = 1;}
     if ($inpos == 1)  {$positioned++;}
     else              {$positioned = 0;}
     $stillAtStart   = 0;						#motors are still at start(1) or not (0)
     $allAmpsEnabled = 1;
     $progressing    = 0;

     for ($i=0; $i<$nAxis; $i++) {
### Check if at least one motor is still at start:
        ($status,$ActPos[$i]) = Pezca::GetDouble($axis[$i].'ActPos');
        if ($status) {$ActPos[$i] = $PrvPos[$i];}
#       ($status,$rqs) = Pezca::GetDouble($axis[$i].'RqsPos');
#       if ($status == 0) {$RqsPos[$i] = $rqs;}
#        $Distance[$i] = abs($RqsPos[$i] - $OldPos[$i]);
        if ($Distance[$i] > $diff_max) {
           $d1 = abs ($ActPos[$i] - $OldPos[$i]);
           $d2 = abs ($ActPos[$i] - $RqsPos[$i]);
           if ($d1 < $d2) {$stillAtStart = 1;}
        }
### Check if at least one motor is progressing:
        if    ($PrvPos[$i] < $ActPos[$i] && $ActPos[$i] < $RqsPos[$i]) {$progressing = 1;}
        elsif ($PrvPos[$i] > $ActPos[$i] && $ActPos[$i] > $RqsPos[$i]) {$progressing = 1;}
        $PrvPos[$i] = $ActPos[$i];
### Check if at least one motor has amplifier failure:
        ($status,$ampEnabled[$i]) = Pezca::GetDouble($axis[$i].'AmpEna');
        if ($status) {$ampEnabled[$i] = 0;}				#presume worst
        if ($ampEnabled[$i] != 1) {
           printf STDOUT '!!! '.$script.': Amplifier fault'
                             .' for motor %s at ActPos=%.3f'."\n",
                                $axis[$i],$ActPos[$i];
           $allAmpsEnabled = 0;
        }
### Check if at least one motor hit a limit:
        if    ($limSign[$i] =~ /^Ng$/i) {
           ($status,$limStatus[$i]) = Pezca::GetDouble($axis[$i].'NgLimSet');
           if ($status) {$limStatus[$i] = 1;}				#presume worst
           if ($limStatus[$i]) {
              printf STDOUT '!!! '.$script.': Motor %s hit Negative limit'
                                  .' at ActPos=%.3f'."\n",$axis[$i],$ActPos[$i];
              $hitAnyLimit++;
           }
        }
        elsif ($limSign[$i] =~ /^Ps$/i) {
           ($status,$limStatus[$i]) = Pezca::GetDouble($axis[$i].'PsLimSet');
           if ($status) {$limStatus[$i] = 1;}				#presume worst
           if ($limStatus[$i]) {
              printf STDOUT '!!! '.$script.': Motor %s hit Positive limit'
                                  .' at ActPos=%.3f'."\n",$axis[$i],$ActPos[$i];
              $hitAnyLimit++;
           }
        }
     }

     if ($runprg == 0 && $positioned >= 3 && $stillAtStart == 0 &&
         ($hasStarted != 0 || $wait_ > $motor_startup_time)) {last;}
     if ($wait_ > $maxTime && $progressing == 0) {
        print STDOUT '!!! '.$script.': quit tracking on motion timeout'."\n";
        last;
     }
     if ($allAmpsEnabled == 0) {
        print STDOUT '!!! '.$script.': quit tracking on amplifier failure'."\n"; 
        last;
     }
     if ($hitAnyLimit) {
        print STDOUT '!!! '.$script.': quit tracking on motor hit a limit'."\n"; 
        last;
     }
  }
### =============================End-of-tracking============================

### -----------------------------Begin-postprocessing-----------------------
### Motor(s) not started:
  if ($stillAtStart == 1 && $hasStarted == 0) {
     printf STDOUT  '!!! '.$script.': '.$assy.' motion not detected. Possible late start?'."\n",
                   '..flags: runprg='.$runprg.' positioned='.$positioned."\n",
                   '         stillAtStart='.$stillAtStart.' hasStarted='.$hasStarted."\n",
                   '         wait=%1.0fs motor_startup_time='.$motor_startup_time."s\n",$wait_;
  }
  $txt = '';
### Amplifier faults?:
  if (! $allAmpsEnabled) {
     $current_time = scalar localtime;
     $txt = '*** '.$script.': '.$current_time."\n";
     for ($i=0; $i<$nAxis; $i++) {
        if ($ampEnabled[$i] != 1) {
           $txt .= sprintf '*** '.$script.': amplifier failure for'
                               .' motor %s at ActPos=%.3f'."\n",
                               $axis[$i],$ActPos[$i];
        }
     }
     $status = &getAssyDataFromTsub($assy,\@ax,\@dr,\@mt);
     if ($status) {
        $txt .= '!!! '.$script.': failed to get assembly data for '.$assy."\n";
     } else {
        foreach $mtr (@mt) {
           $flag = &getMtrAmpFaultErr($mtr);				#0=OK, 1=fault, -1=PVerr
           if ($flag == 1) {
              $txt .= '!!! '.$script.': amplifier fault error for motor='
                            .$mtr.' in assembly='.$assy."\n";
           }
           $flag = &getMtrFatalFolErr($mtr);				#0=OK, 1=fault, -1=PVerr
           if ($flag == 1) {
              $txt .= '!!! '.$script.': fatal following error for motor='
                            .$mtr.' in assembly='.$assy."\n";
           }
        }
     }
     if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     else {
        print STDOUT $txt;
        if (defined $msgRef) {$$msgRef = $txt;}
        return 1;
     }
  }

### Limit hit?:
  if ($hitAnyLimit) {
     $current_time = scalar localtime;
     $txt = '!!! '.$script.': '.$current_time."\n";
     for ($i=0; $i<$nAxis; $i++) {
        if ($limStatus[$i]) {
           $txt .= sprintf '!!! '.$script.': Motor %s hit %s hard limit'
                               .' at ActPos=%.3f'."\n",
                               $axis[$i],$limSign[$i],$ActPos[$i];
        }
     }
#    if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     print STDOUT $txt;
  }

  if ($report >= 2) {
     printf STDOUT 'Real in-motion time for assembly='.$assy.' was %.1fs'."\n",$wait_;
  }

### Timeout?
  if ($wait_ > $maxTime) {
     $current_time = scalar localtime;
     if ($progressing == 0) {
        if (length($txt) == 0) {$txt  = '*** '.$script.': '.$current_time."\n";}
        $txt .= sprintf '*** '.$script.': '.$assy.' did not arrive '.
                        'at requested position(s) after %1.0fs'."\n", $wait_;
        for ($i=0; $i<$nAxis; $i++) {
           $ActPos[$i] = &getAxisAct($axis[$i]);
           $RqsPos[$i] = &getAxisRqs($axis[$i]);
           $txt .= sprintf '   axis=%s, rqs=%.3f,  act=%.3f'."\n",$axis[$i],$RqsPos[$i],$ActPos[$i];
        }
        $timeout      = 1;
        if ($nonstop == 2) {
           $txt .= '!!! '.$script.': ... continuing script'."\n";
        }
        elsif ($nonstop == 1) {
           $txt .= '!!! '.$script.': ... aborting motion and continuing script'."\n";
           &abortAssy ($assy);
        }
        else {
           &abortAssy ($assy);
           if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
           else  {
              $returnStatus = 0;					#timeout
              if (! $allAmpsEnabled) {$returnStatus += 1;}
              if ($hitAnyLimit)      {$returnStatus += 2;}
              if ($timeout)          {$returnStatus += 4;}
              if (defined $msgRef)   {$$msgRef = $txt;}
              if (length($txt) > 0)  {print STDOUT $txt."\n";}
#             else                   {print STDOUT '-- No message from '.$script.' for assembly='.$assy."\n";}
              return $returnStatus;
           }
        }
     }
     else {
        if (length($txt) == 0) {$txt  = '!!! '.$script.': '.$current_time."\n";}
        $txt .= sprintf '!!! '.$script.': Assembly '.$assy.' motion time '.
                        '%1.0fs exceeded the timeout, but it was not aborted '.
                        'since kept moving'."\n", $wait_;
     }
  }
  elsif ($wait_ > $warnTime) {						#warning level

     if (!defined $IGNORE_MOTION_ESTIMATE) {
        printf STDOUT '!!! '.$script.': Assembly '.$assy.' motion_time=%.1fs '.
                      'exceeded the estimate=%.1fs'."\n", $wait_, $motionTime;
     }
  }

### Are the end positions proper?
  for ($i=0; $i<$nAxis; $i++) {
     $ap_ = &getAxisAct($axis[$i]);					#Get the Actual motor position
     $diff = abs($ap_ - $RqsPos[$i]);					#check if it corresponds to RqsPos:
     if ($diff > $diff_max) {
        ($status,$inpos)  = Pezca::GetDouble($assy.'InPos');
        if ($status) {$inpos=0;}			#presume worst
        printf STDOUT '!!! '.$script.': '.$axis[$i].' pos.mismatch=%.3f'
                            .', inpos='.$inpos."\n",$diff;
#       &clearCaMonitor($axis[$i].'ActPos');
        $wait_ = 0;
        $wait_max = 4;
        print STDOUT '... re-reading '.$axis[$i].' actual position'
                    .' after '.$wait_max.'s ...'."\n";
        do {
           select(undef,undef,undef,$wait_step);			#sleep($wait_step);
           $wait_ += $wait_step;
           ($status,$inpos)  = Pezca::GetDouble($assy.'InPos');
           if ($status) {$inpos=0;}					#presume worst
           $ap_ = &getAxisAct($axis[$i]);				#Get the Actual motor position
           $diff = abs($ap_ - $RqsPos[$i]);				#check if it corresponds to RqsPos:
###        printf STDOUT '!!! '.$script.': '.$axis[$i]
###                     .' pos.mismatch = %.3f, inpos='.$inpos."\n",$diff;
        } while ($diff > $diff_max && $wait_ < $wait_max);

        if ($diff > $diff_max) {
           print STDOUT '... extra waiting for '.$axis[$i].' did not help ...'."\n";
           $current_time = scalar localtime;
           print  STDOUT '!!! '.$script.': '.$current_time."\n";
           printf STDOUT '!!! '.$script.': '.$axis[$i].' pos.mismatch = %.3f'."\n",$diff;
           printf STDOUT '!!! '.$script.': previous  = %.3f'."\n",$OldPos[$i];
           printf STDOUT '!!! '.$script.': requested = %.3f'."\n",$RqsPos[$i];
           printf STDOUT '!!! '.$script.': actual    = %.3f'."\n",$ap_;
           printf STDOUT '!!! '.$script.': allowance = %.3f'."\n",$diff_max;
           print  STDOUT '..flags: runprg='.$runprg.' positioned='.$positioned."\n",
                         '         stillAtStart='.$stillAtStart.' hasStarted='.$hasStarted."\n",
                         '         wait='.$wait_.'s motor_startup_time='.$motor_startup_time."s\n";
           $outofband = 1;
        }
        else {
           print  STDOUT '... '.$axis[$i].' finally arrived ...'."\n";
           print  STDOUT '!!! '.$script.': delayed '.$axis[$i].' pos.report was detected!'."\n";
           printf STDOUT '!!! '.$script.': extra waiting for pos.update was %.2fs'."\n\n",$wait_;
        }
     }
  }
  $returnStatus = 0;
  if (! $allAmpsEnabled) {$returnStatus += 1;}
  if ($hitAnyLimit)      {$returnStatus += 2;}
  if ($timeout)          {$returnStatus += 4;}
  if ($outofband)        {$returnStatus += 8;}
  if (defined $msgRef)   {$$msgRef = $txt;}
  if (length($txt) > 0)  {print STDOUT $txt."\n";}
# else                   {print STDOUT '-- No message from '.$script.' for assembly='.$assy."\n";}

  return $returnStatus;
### -----------------------------End-of-postprocessing----------------------
}

#=================================================================

# sub checkMotorAmp ($axis, [$printFlag, \$status]) {
sub checkMotorAmp ($;$\$) {

### This function returns $AmpEna status for $axis

  my ($script, $axis, $printFlag, $AmpEna, $statref, $status);

  $script    = 'checkMotorAmp';
  $axis      = shift(@_);       if ($axis !~ /:$/) {$axis .= ':';}
  $printFlag = shift(@_);	if (!defined $printFlag) {$printFlag=0;}
  $statref   = shift(@_);

### caget nnID:XX:X:AmpEna; (get the AmpEna status of a motor)
  ($status, $AmpEna) = Pezca::GetDouble($axis.'AmpEna');
  if ($status) {
     &report_exit($script, $status, $axis.'AmpEna');
     $AmpEna = 0;
  }
  if ($AmpEna != 1 && $printFlag != 0) {
     print STDOUT '*** Motor '.$axis.' amplifier failed/disabled.'."\n";
  }
  if (defined $statref) {$$statref = $status;}
  return $AmpEna;
}

#=================================================================

# sub checkMotorLimits ($axis, [$printFlag]) {
sub checkMotorLimits ($;$) {
### This function returns $l_limit and $h_limit status for $axis
### It returns (0,0) if neither limit flag is raised

  my ($script, $axis, $printFlag, $status, $l_limit, $h_limit);

  $script    = 'checkMotorLimits';
  $axis      = shift(@_);       if ($axis !~ /:$/) {$axis .= ':';}
  $printFlag = shift(@_);	if (!defined $printFlag) {$printFlag=0;}

### caget nnID:XX:X:NgLimSet; (get the NgLim status of a motor)
  ($status, $l_limit) = Pezca::GetDouble($axis.'NgLimSet');
  if ($status) {
     &report_exit($script, $status, $axis.'NgLimSet');
     $l_limit = 0;
  }
  if ($l_limit != 0 && $printFlag != 0) {
     print STDOUT '*** Motor '.$axis.' hit negative limit.'."\n";
  }

### caget nnID:XX:X:PsLimSet; (get the PsLim status of a motor)
  ($status, $h_limit) = Pezca::GetDouble($axis.'PsLimSet');
  if ($status) {
     &report_exit($script, $status, $axis.'PsLimSet');
     $h_limit = 0;
  }
  if ($h_limit != 0 && $printFlag != 0) {
     print STDOUT '*** Motor '.$axis.' hit positive limit.'."\n";
  }

  return ($l_limit,$h_limit);
}

#=================================================================

# sub clearAxisLimits ($assy, $axis, $step) {
sub clearAxisLimits ($$$) {
### Pushes axis off limits if any is raised

  my ($script, $assy, $axis, $step, $l_limit, $h_limit, $act1, $act2);

  $script = 'clearAxisLimits';
  $assy   = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $axis   = shift(@_);         if ($axis !~ /:$/) {$axis .= ':';}
  $step   = shift(@_);

  if    ($step < 0)  {$step = -$step;}
  elsif ($step == 0) {print STDOUT '*** '.$script.': zero step for ['.$axis.']'."\n"; return 1;}

  ($l_limit, $h_limit) = &checkMotorLimits($axis, 1);

  if    ($l_limit == 0 && $h_limit == 0) {return 0;}
  elsif ($l_limit != 0 && $h_limit != 0) {
     print STDOUT '*** '.$script.': cannot proceed because both limits are hit for ['.$axis.']'."\n";
     return 1;
  }
  elsif ($l_limit != 0) {
     print STDOUT $script.': pushing ['.$axis.'] off lower limit with step='.$step."\n";
     do {
        $act1 = &getAxisAct ($axis);
        &runMotor ($assy, $axis,  $act1+$step, 'checkauto');
        $act2 = &getAxisAct ($axis);
        ($l_limit, $h_limit) = &checkMotorLimits($axis, 0);
     } while ($l_limit != 0 && $act2 > $act1 && $h_limit == 0);
     if ($act2 <= $act1) {
         print STDOUT '*** '.$script.': motor ['.$axis.'] is not moving'
                          .' when pushing it off lower limit with step='.$step."\n";
         return 1;
     }
     if ($h_limit != 0) {
        print STDOUT '*** '.$script.': motor ['.$axis.'] hit upper limit'
                         .' while pushing it off lower limit with step='.$step."\n";
        return 1;
     }
  }
  elsif ($h_limit != 0) {
     print STDOUT $script.': pushing ['.$axis.'] off upper limit with step='.$step."\n";
     do {
        $act1 = &getAxisAct ($axis);
        &runMotor ($assy, $axis,  $act1-$step, 'checkauto');
        $act2 = &getAxisAct ($axis);
        ($l_limit, $h_limit) = &checkMotorLimits($axis, 0);
     } while ($h_limit != 0 && $act2 < $act1 && $l_limit == 0);
     if ($act2 >= $act1) {
         print STDOUT '*** '.$script.': motor ['.$axis.'] is not moving'
                          .' when pushing it off upper limit with step='.$step."\n";
         return 1;
     }
     if ($l_limit != 0) {
        print STDOUT '*** '.$script.': motor ['.$axis.'] hit lower limit'
                          .' while pushing it off upper limit with step='.$step."\n";
        return 1;
     }
  }
  return 0;
}

#=================================================================

# sub getAxisDRVLH ($axis,[\$status]);
sub getAxisDRVLH ($;\$) {
### This function returns DRVL and DRVH for requested motor

  our $report;
  my ($script, $axis, $statref, $status, $drvl, $drvh);

  $script  = 'getAxisDRVLH';
  $axis    = shift(@_);       if ($axis !~ /:$/) {$axis .= ':';}
  $statref = shift(@_);
  if (!defined $report) {$report = 0;}

  ($status,$drvl) = Pezca::GetDouble($axis.'RqsPos.DRVL');
  if ($status) {
     &report_exit($script,$status,$axis.'RqsPos.DRVL');
     $drvl = 0;
  }
  if (defined $statref) {$$statref = $status;}

  ($status,$drvh) = Pezca::GetDouble($axis.'RqsPos.DRVH');
  if ($status) {
     &report_exit($script,$status,$axis.'RqsPos.DRVH');
     $drvh = 0;
  }
  if (defined $statref) {$$statref += $status;}

  if ($report && $drvl == 0 && $drvh == 0) {
     print STDOUT '!!! Soft limits for '.$axis.' are not set.'."\n";
  }

  return ($drvl,$drvh);
}

#=================================================================

# sub setMotionPrg ($assy, $program) {
sub setMotionPrg ($$) {

### This routine sets motion program for a SERVO motor.

  my ($script, $assy, $program, $status, $current_time, $wait_settle);

  $script   = 'setMotionPrg';
  $assy     = shift(@_);        if ($assy !~ /:$/) {$assy .= ':';}
  $program  = shift(@_);

### Just in case abort motion:
  &abortAssy ($assy);						#abort with no restoring speed

### Switch Autostart to manual:
  $status = &setAssyManual ($assy);
  if ($status) {return $status;}

  select(undef,undef,undef,0.05);

### Change motion program (GMCA: 1=slew/2=backlash/3=position)
# print STDOUT 'Setting assembly='.$assy.' to motion program='.$program.'...'."\n";
  $status = Pezca::PutDoubleOldCa($assy.'Motion',$program);
  if ($status) {
     &report_exit($script, $status, $assy.'Motion');
     return $status;
  }

  select(undef,undef,undef,0.05);

### Restore Autostart to auto:
  $status = &setAssyAuto ($assy);
  if ($status) {return $status;}

### If motor is in "AutoStart, changing motion program
### causes its execution; so abort motion:
  &abortAssy ($assy);						#abort with no restoring speed

  $wait_settle = &waitMotor ($assy,8);				#sleep to settle (max=8s)
  if ($wait_settle == 0) {
     $current_time = scalar localtime;
     print STDOUT '!!! '.$script.': '.$current_time."\n";
     print STDOUT '!!! '.$script.': assembly '.$assy.' did not settle after changing motion prg.'."\n";
  }
# print STDOUT $script.': settling='.$wait_settle."s\n";
  return 0;
}

#=================================================================

### $status = &syncAssy($assy);
# sub syncAssy ($assy) {
sub syncAssy ($) {

### This routine synchronizes requested positions with their actual positions.

  my ($script, $assy, $status, $current_time, $wait_settle);

  $script = 'syncAssy';
  $assy   = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}

### Just in case abort motion:
  &abortAssy ($assy);						#abort with no restoring speed

### Switch Autostart to manual:
  $status = &setAssyManual ($assy);
  if ($status) {return $status;}

  select(undef,undef,undef,0.05);

### Change motion program (GMCA: 1=slew/2=backlash/3=position)
# print STDOUT 'Setting assembly='.$assy.' to motion program='.$program.'...'."\n";
  $status = Pezca::PutDoubleOldCa($assy.'Sync.PROC',1);
  if ($status) {
     &report_exit($script, $status, $assy.'Sync');
     return $status;
  }

  select(undef,undef,undef,0.05);

### Restore Autostart to auto:
  $status = &setAssyAuto ($assy);
  if ($status) {return $status;}

### Just in case abort motion:
  &abortAssy ($assy);						#abort with no restoring speed

  $wait_settle = &waitMotor ($assy,8);				#sleep to settle (max=8s)
  if ($wait_settle == 0) {
     $current_time = scalar localtime;
     print STDOUT '!!! '.$script.': '.$current_time."\n";
     print STDOUT '!!! '.$script.': assembly '.$assy.' did not settle after Act->Rqs sync.'."\n";
  }
# print STDOUT $script.': settling='.$wait_settle."s\n";
  return 0;
}

#=================================================================

# sub calcAxisSpeed ($assy, $axis, $scanRange, $scanTime) {
sub calcAxisSpeed ($$$$) {

### This routine determines and returns $l_speed and $h_speed
### for SERVO motor.

  our $IGNORE_ERRORS;
  my ($script, $assy, $axis, $scanRange, $scanTime);
  my ($l_speed, $h_speed, $current_time, $txt);

  $script    = 'calcAxisSpeed';
  $assy      = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $axis      = shift(@_);         if ($axis !~ /:$/) {$axis .= ':';}
  $scanRange = shift(@_);
  $scanTime  = shift(@_);
  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS=0;}

### Get h_speed:
  $h_speed = &getAxisHiSpeed ($assy, $axis);
  if ($h_speed == 0) {
     $current_time = scalar localtime;
     $txt  = '*** '.$script.': '.$current_time."\n";
     $txt .= '*** '.$script.': getAxisHiSpeed returned zero high speed for '.$axis."\n";
     &press_key_exit($txt);
  }
  $l_speed = $scanRange / $scanTime;
  $l_speed = abs $l_speed;
  printf STDOUT $axis.': Required low speed = %9.5f'."\n",$l_speed;
  printf STDOUT $axis.': High  speed  limit = %9.5f'."\n",$h_speed;
  if ($l_speed > $h_speed) {
     if ($l_speed > 1.01 * $h_speed) {
        $l_speed = $h_speed;
        $scanTime = $scanRange / $l_speed;
        $current_time = scalar localtime;
        $txt  = '*** '.$script.': '.$current_time."\n";
        $txt .= '*** '.$script.': Cannot move so fast.'."\n";
        $txt .= sprintf '*** '.$script.': Increase scan time to %d s'."\n", $scanTime;
        if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
        else                 {print STDOUT $txt;}
     } else {
        $l_speed = $h_speed;					#1% over is OK (just round it)!
     }
  }
  return ($l_speed,$h_speed);
}

#=================================================================

# sub getAxisSpeed ($assy, $axis, [\$status]) {
sub getAxisSpeed ($$;\$) {

### This routine determines current speed for SERVO motor/axis.

  my ($script, $assy, $axis, $statref, $speed, $status, $current_time);
  my (@axes, @drives, @motors, $nmotors, @prgSpeeds, @jogSpeeds, $i);
  my ($needsRestore, $factor, $ratio);

  $script   = 'getAxisSpeed';
  $assy     = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $axis     = shift(@_);       if ($axis !~ /:$/) {$axis .= ':';}
  $statref  = shift(@_);

### Read current program speed:
  ($status,$speed) = Pezca::GetDouble($axis.'speed');
  if ($status) {
     &report_exit($script, $status, $axis.'speed');
     $speed = 0;
  }
  if (defined $statref) {$$statref = $status;}

  if ($speed == 0.0) {						#never set before
### Find what motors are present in the assembly:
     $status = &getAssyDataFromTsub($assy,\@axes,\@drives,\@motors);
     if ($status) {
        print STDOUT '!!! '.$script.': error='.$status.' from getAssyDataFromTsub for motor='.$axis."\n";
        if (defined $statref) {$$statref = $status;}
        return 0;
     }
### Normally program speed maxes at jog speed. In case of BAM2 it may exceed
### jog speed. The code below restores program speed if it gets reduced by 
### calling TSUB. 
     $nmotors = @motors;
     $needsRestore = 0;
     $factor = 99999;
     for ($i=0; $i<$nmotors; $i++) {
        $jogSpeeds[$i] = &getMtrJogSpeed($motors[$i]);
        $prgSpeeds[$i] = &getMtrPrgSpeed($motors[$i]);
        if ($prgSpeeds[$i] > $jogSpeeds[$i]) {
           $needsRestore = 1;
           $ratio = abs($prgSpeeds[$i]/$jogSpeeds[$i]);
           if ($ratio < $factor) {$factor = $ratio;}
        }
     }
### Set speed to maximum (since "0" supposed to be maximum):
     $status = &setAxisSpeed($assy, $axis, 0.0);		#"0" sets maximum speed
     select(undef,undef,undef,0.12);				#sleep to propagate the change
### Read current program speed again:
     ($status,$speed) = Pezca::GetDouble($axis.'speed');
     if ($status) {
        &report_exit($script, $status, $axis.'speed');
        $speed = 0;
     }
     if (defined $statref) {$$statref += $status;}
  }
  if ($speed == 0.0) {
     $current_time = scalar localtime;
     print STDOUT '!!! '.$script.': '.$current_time."\n";
     print STDOUT '!!! '.$script.': failed to get non-zero speed for '.$axis."\n";
  }
  if ($needsRestore) {
     for ($i=0; $i<$nmotors; $i++) {setMtrPrgSpeed ($motors[$i],$prgSpeeds[$i]);}
     $speed = $speed * $factor;
  }
  return $speed;
}

#=================================================================

# sub getAxisHiSpeed ($assy, $axis, [\$status]) {
sub getAxisHiSpeed ($$;\$) {

### This routine determines h_speed for SERVO motor/axis.

  our $IGNORE_ERRORS;
  my ($script, $assy, $axis, $statref);
  my ($h_speed, $prg_speed, $status, $txt);

  $script   = 'getAxisHiSpeed';
  $assy     = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $axis     = shift(@_);       if ($axis !~ /:$/) {$axis .= ':';}
  $statref  = shift(@_);
  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS=0;}

### Read current program speed:
  $prg_speed = &getAxisSpeed($assy, $axis, \$status);
  if (defined $statref) {$$statref = $status;}

### Set program speed to maximum:
  &setAxisSpeed($assy, $axis, 0.0);				#"0" sets maximum speed
  select(undef,undef,undef,0.12);				#sleep to propagate the change

### Read the maximum speed:
  $h_speed = &getAxisSpeed($assy, $axis, \$status);
  if (defined $statref) {$$statref += $status;}

  if ($h_speed == 0.0) {
     $txt = '*** '.$script.': failed to set maximum speed for '.$axis."\n";
     if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     else                 {print STDOUT $txt;}
  }

  if (($prg_speed != 0.0) && ($prg_speed != $h_speed)) {	#then restore!
     &setAxisSpeed($assy, $axis, $prg_speed);
  }

  return $h_speed;
}

#=================================================================

# sub setAxisHiSpeed ($assy, $axis) {
sub setAxisHiSpeed ($$) {

### This routine determines h_speed for SERVO motor/axis.

  our $IGNORE_ERRORS;
  my ($script, $assy, $axis, $status);

  $script = 'setAxisHiSpeed';
  $assy   = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $axis   = shift(@_);       if ($axis !~ /:$/) {$axis .= ':';}
  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS=0;}

  $status = &setAxisSpeed($assy, $axis, 0.0);			#"0" sets maximum speed
  return $status;

### TEST: Read the maximum speed:
# ($status,$h_speed) = Pezca::GetDouble($axis.'speed');
# if ($status) {
#    &report_exit("setAxisHiSpeed", $status, $axis.'speed');
#    $h_speed = 0;
# }
#
# if ($h_speed == 0.0) {
#    $txt = '*** '.$script.': failed to set maximum speed for '.$axis."\n";
#    if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
#    else                 {print STDOUT $txt;}
# }
}

#=================================================================

# $status = &setAxisSpeed ($assy, $axis, $speed);
sub setAxisSpeed ($$$) {

### This routine sets speed for PMAC motor/axis.

  my ($script, $assy, $axis, $speed, $status);
  my ($speed_act, $attempt, $max_attempts);

  $script  = 'setAxisSpeed';
  $assy    = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $axis    = shift(@_);         if ($axis !~ /:$/) {$axis .= ':';}
  $speed   = shift(@_);
  $attempt = 0;
  $max_attempts = 5;

  do {
#    print STDOUT 'Setting motor='.$axis.' speed to '.$speed.'...'."\n";
### Set program speed-1 (enable the change):
     $status = Pezca::PutDoubleOldCa($assy.'speedSDIS',0);
     if ($status) {
        &report_exit($script, $status, $assy.'speedSDIS');
        return $status;
     }
     select(undef,undef,undef,0.12);				#sleep;

### Set program speed-2 (enter the value):
     $status = Pezca::PutDoubleOldCa($axis.'speed',$speed);
     if ($status) {
        &report_exit($script, $status, $axis.'speed');
        return $status;
     }
### Verify:
     select(undef,undef,undef,0.12);				#sleep;
### Read current program speed:
     ($status,$speed_act) = Pezca::GetDouble($axis.'speed');
     if ($status) {
        &report_exit($script, $status, $axis.'speed');
        $speed_act = 0;
     }
     if ($speed == 0 && $speed_act != 0) {return 0;}
     if ($speed != 0) {
        if (abs(($speed_act-$speed)/$speed) < 0.01) {return 0;}
     }
     $attempt++;
     print STDOUT '*** '.$script.': failed attempt '.$attempt.'/'.$max_attempts.' to set';
     if ($speed != 0) {printf STDOUT ' '.$axis.' speed to %.3g'."\n",$speed;}
     else             {print  STDOUT ' '.$axis.' speed to MAX'."\n";}
  } while ($attempt < $max_attempts);
  return 1;
}

#=================================================================

# sub setAxisSlewScan ($assy, $axis, $speed) {
sub setAxisSlewScan ($$$) {

### This routine sets slew speed and
### motion mode for SERVO motor.

  my ($assy, $axis, $speed, $status);

  $assy  = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $axis  = shift(@_);       if ($axis !~ /:$/) {$axis .= ':';}
  $speed = shift(@_);

### Motion modes (BIOCAT): 1=slew/2=backlash/3=position/4=scan/5=test
### Motion modes (GMCA):   1=slew/2=backlash/3=position
# print STDOUT 'Setting motor='.$motor.' to slew motion...'."\n";
  $status = &setMotionPrg($assy,1);
  if ($status) {return $status;}

### Set specified low speed:
  $status = &setAxisSpeed($assy, $axis, $speed);
  return $status;
}

#=================================================================

# sub restoreAxis ($assy, $axis) {
sub restoreAxis ($$) {

### This routine restores fast speed and
### "position" motion mode for SERVO motor.

  my ($assy, $axis, $status);

  $assy = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $axis = shift(@_);       if ($axis !~ /:$/) {$axis .= ':';}

### Motion modes:   1=slew/2=backlash/3=position
# print STDOUT 'Restoring axis='.$axis.' motion prg to normal...'."\n";
  &setMotionPrg($assy,3);

  $status = &setAxisHiSpeed($assy, $axis);
  return $status;
}

#=================================================================

# $ampstate = &getMtrAmpEnable ($motor);
# sub getMtrAmpEnable ($motor) {
sub getMtrAmpEnable ($) {					#returns 0=Off, 1=On, -1=PVerr

  my ($script, $motor, $pv, $flag, $status);

  $script = 'getMtrAmpEnable';
  $motor  = shift(@_);       if ($motor !~ /:$/) {$motor .= ':';}

  $pv = $motor.'AmpEna';

  ($status,$flag) = Pezca::GetDouble($pv);
  if ($status) {
     &report_only($script, $status, $pv);
     $flag = -1;
  }
  return $flag;
}

#=================================================================

# $ampfault = &getMtrAmpFaultErr ($motor);
# sub getMtrAmpFaultErr ($motor) {
sub getMtrAmpFaultErr ($) {					#returns 0=OK, 1=fault, -1=PVerr

  my ($script, $motor, $pv, $flag, $status);

  $script = 'getMtrAmpFaultErr';
  $motor  = shift(@_);       if ($motor !~ /:$/) {$motor .= ':';}

  $pv = $motor.'AmpFltErr';

  ($status,$flag) = Pezca::GetDouble($pv);
  if ($status) {
     &report_only($script, $status, $pv);
     $flag = -1;
  }
  return $flag;
}

#=================================================================

# $follerr = &getMtrFatalFolErr ($motor);
# sub getMtrFatalFolErr ($motor) {
sub getMtrFatalFolErr ($) {					#returns 0=OK, 1=fault, -1=PVerr

  my ($script, $motor, $pv, $flag, $status);

  $script = 'getMtrFatalFolErr';
  $motor  = shift(@_);       if ($motor !~ /:$/) {$motor .= ':';}

  $pv = $motor.'FatalFolErr';

  ($status,$flag) = Pezca::GetDouble($pv);
  if ($status) {
     &report_only($script, $status, $pv);
     $flag = -1;
  }
  return $flag;
}

#=================================================================

# sub getMtrAccelTime ($motor,[\$status]) {			#returns $accelTime
sub getMtrAccelTime ($;\$) {

  my ($script, $motor, $statref, $mtrNo, $current_time);
  my ($accelTime, $accelCurve, $stat1, $stat2);

  $script   = 'getMtrAccelTime';
  $motor    = shift(@_);       if ($motor !~ /:$/) {$motor .= ':';}
  $statref  = shift(@_);

  $mtrNo = &getMtrNo($motor,\$stat1);
  if ($stat1) {
     $current_time = scalar localtime;
     print STDOUT '*** '.$script.': '.$current_time."\n";
     print STDOUT '*** '.$script.': error requesting mtrNo for mtr='.$motor."\n";
     if (defined $statref) {$$statref = -1;}
     return 0;
  }

  if ($mtrNo < 1 || $mtrNo > 32) {
     $current_time = scalar localtime;
     print STDOUT '*** '.$script.': '.$current_time."\n";
     print STDOUT '*** '.$script.': incorrect motor number='.$mtrNo."\n";
     if (defined $statref) {$$statref = -2;}
     return 0;
  }

  $stat1 = 0;
  $accelTime = &readMtrRsp($motor, 'I'.$mtrNo.'20','numeric');
  if (length($accelTime) == 0) {
     $current_time = scalar localtime;
     print STDOUT '*** '.$script.': '.$current_time."\n";
     print STDOUT '*** '.$script.': error reading acceleration time (I'.$mtrNo.'20) for mtr='.$motor."\n";
     $stat1 = -1;
     $accelTime = 0;
  }

  $stat2 = 0;
  $accelCurve = &readMtrRsp($motor, 'I'.$mtrNo.'21','numeric');
  if (length($accelCurve) == 0) {
     $current_time = scalar localtime;
     print STDOUT '*** '.$script.': '.$current_time."\n";
     print STDOUT '*** '.$script.': error reading S-curve time (I'.$mtrNo.'21) for mtr='.$motor."\n";
     $stat1 = -1;
     $accelCurve = 0;
  }
  if ($accelTime < 2*$accelCurve) {$accelTime = 2*$accelCurve;}

  if (defined $statref) {$$statref = $stat1+$stat2;}
  return $accelTime;
}

#=================================================================

# sub getMtrJogSpeed ($motor,[\$status]) {
sub getMtrJogSpeed ($;\$) {			# returns $jogSpeed

  my ($script, $motor, $statref, $pv, $speed, $status);

  $script   = 'getMtrJogSpeed';
  $motor    = shift(@_);       if ($motor !~ /:$/) {$motor .= ':';}
  $statref  = shift(@_);

  $pv = $motor.'Ix22:AI';

### Determine the jog speed of motor (Ix22):
  $status = Pezca::PutDouble($pv.'.PROC',1);			#process the variable
  if ($status) {
     &report_only($script, $status, $pv.'.PROC');
  } else {
     select(undef,undef,undef,0.1);	# sleep 0.1sec
  }

  ($status, $speed) = Pezca::GetDouble($pv);
  if ($status) {
     &report_only($script, $status, $pv);
     $speed = 0;
  }
  if (defined $statref) {$$statref = $status;}
  return $speed;
}

#=================================================================

# sub getMtrPrgSpeed ($motor,[\$status]) {
sub getMtrPrgSpeed ($;\$) {			# returns $prgSpeed

  my ($script, $motor, $statref, $pv, $speed, $status);

  $script   = 'getMtrPrgSpeed';
  $motor    = shift(@_);       if ($motor !~ /:$/) {$motor .= ':';}
  $statref  = shift(@_);

  $pv = $motor.'Ix16:AI';

### Determine the prg speed of motor (Ix16):
  $status = Pezca::PutDouble($pv.'.PROC',1);			#process the variable
  if ($status) {
     &report_only($script, $status, $pv.'.PROC');
  } else {
     select(undef,undef,undef,0.1);	# sleep 0.1sec
  }

  ($status, $speed) = Pezca::GetDouble($pv);
  if ($status) {
     &report_only($script, $status, $pv);
     $speed = 0;
  }
  if (defined $statref) {$$statref = $status;}
  return $speed;
}

#=================================================================

# sub setMtrPrgSpeed ($motor,$prgSpeed) {
sub setMtrPrgSpeed ($$) {					#returns $status

  my ($script, $motor, $speed, $pv, $status);

  $script = 'setMtrPrgSpeed';
  $motor  = shift(@_);       if ($motor !~ /:$/) {$motor .= ':';}
  $speed  = shift(@_);

  $pv = $motor.'speed';

### Set the prg speed of motor (Ix16):
  $status = Pezca::PutDoubleOldCa($pv, $speed);
  &report_only($script, $status, $pv);
  return $status;
}

#=================================================================

# sub abortAssy ($assy, [$restore, $axis]) {
sub abortAssy ($;$$) {

### This routine aborts SERVO motor motion
### and optionally restores fast speed and
### "position" motion mode.

  my ($script, $assy, $restore, $axis, $status);

  $script  = 'abortAssy';
  $assy    = shift(@_);       if ($assy !~ /:$/) {$assy .= ':';}
  $restore = shift(@_);       if (!defined $restore) {$restore = 0;}

  $status = Pezca::PutDoubleOldCa($assy.'Abort',1);
  &report_exit($script, $status, $assy.'Abort');

  if ($restore eq '1' || $restore =~ /^restore$/i) {
     $axis = shift(@_);
     $status += &restoreAxis($assy, $axis);
  }
  return $status;
}

#=================================================================

# sub getMtrNo($motor,[\$status]) {
sub getMtrNo($;\$) {						#returns $mtrNo

  our $report;
  my ($script, $motor, $statref, $status, $mtrNo, $pmac);

  $script  = 'getMtrNo';
  $motor   = shift(@_);       if ($motor !~ /:$/) {$motor .= ':';}
  $statref = shift(@_);
  if (!defined $report)        {$report = 0;}

  &setEpicsMessages(0);
### Get motor number for the motor:
# print STDOUT $script.': getting motor number for m='.$motor."\n";
  ($status,$mtrNo) = Pezca::GetDouble($motor.'Mtr');
  if ($status) {
     if ($motor =~ /pmac\d{2}:mo:$/) {
        if ($report) {
           $pmac = $motor;
           $pmac =~ s/:mo:$//;
           print STDOUT '!!! '.$script.': no virtual motor in pmac='.$pmac."\n";
        }
     } else {
        &report_only($script, $status, $motor.'Mtr');
     }
     $mtrNo = -1;
  }
  if (defined $statref) {$$statref = $status;}
  return $mtrNo;
}

#=================================================================

# sub getMtrOutNo_Pmac($pmac,[\$status]) {
sub getMtrOutNo_Pmac($;\$) {					#returns $mtrOutNo or -1 if not present

  my ($script, $pmac, $statref, $status, $mtrOut, $mtrOutNo);

  $script  = 'getMtrOutNo_Pmac';
  $pmac    = shift(@_);         if ($pmac !~ /:$/) {$pmac .= ':';}
  $statref = shift(@_);

  if ($pmac =~ /23o:pmac11:/ ) {				#there is no mtrOut on pmac11 at ID-out
     if (defined $statref) {$$statref = 0;}
     return -1;
  }
  $mtrOut   = $pmac.'mo';					#name of mtrOut in pmac
  $mtrOutNo = &getMtrNo($mtrOut,\$status);
  if (defined $statref) {$$statref = $status;}

  return $mtrOutNo;
}

#=================================================================

# sub getMtrOutNo_Pcs($assy,[\$status]) {
sub getMtrOutNo_Pcs($;\$) {					#returns $mtrOutNo or -1 if not present

  my ($script, $assy, $statref, $status, $pmac, $mtrOutNo);

  $script  = 'getMtrOutNo_Pcs';
  $assy    = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $statref = shift(@_);

  $pmac     = &getPmac($assy);
  $mtrOutNo = &getMtrOutNo_Pmac($pmac,\$status);
  if (defined $statref) {$$statref = $status;}

  return $mtrOutNo;
}

#=================================================================

# sub configureMxx80_Pmac ($pmac, $mtrNo) {
  sub configureMxx80_Pmac ($$) {

### The PMAC has to be configured that two M-variables read the status word
### of PMAC output channel (e.g.motor #32). In this status word, bit #15 is
### equal to 1 when the motor is assigned to a coordinate system and 0 if not.
### Four bits #20-23 compose the coordinate system number. We assign the
### M-variables Mxx80 and Mxx81 (i.e. M3280 and M3281 in our example) to read
### this information (the choice of Mxx80 and Mxx81 is voluntary -- just
### deploy any unused  M-variable -- see PeWin or TURBO_SRM.pdf, Software
### Reference Manual).
### (this version is using pmac StrCmd/StrRsp)

  our $IGNORE_ERRORS;
  my ($script, $pmac, $mtrNo, $current_time, $string, $hexaddr, $txt);

  $script = 'configureMxx80_Pmac';
  $pmac   = shift(@_);         if ($pmac !~ /:$/) {$pmac .= ':';}
  $mtrNo  = shift(@_);

  if (!defined $IGNORE_ERRORS)       {$IGNORE_ERRORS=0;}

  if ($mtrNo < 1 || $mtrNo > 32) {
     $current_time = scalar localtime;
     $txt  = '*** '.$script.': '.$current_time."\n";
     $txt .= '*** '.$script.': incorrect motor number='.$mtrNo."\n";
     if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     else                 {print STDOUT $txt; return;}
  }

### Setting Mxx80 and Mxx81 to read status of channel $mtrNo;
### Here are the values for different motors:       *
###  #1=Y:$C0,   #2=Y:#140,  #3=Y:$1C0 , #4=Y:$240, *
###  #5=Y:$2C0,  #6=Y:$340,  #7=Y:$3C0,  #8=Y:$440, *
###  #9=Y:$4C0, #10=Y:$540, #11=Y:$5C0, #12=Y:$640, *
### #13=Y:$6C0, #14=Y:$740, #15=Y:$7C0, #16=Y:$840, *
### #17=Y:$8C0, #18=Y:$940, #19=Y:$9C0, #20=Y:$A40, *
### #21=Y:$AC0, #22=Y:$B40, #23=Y:$BC0, #24=Y:$C40, *
### #25=Y:$CC0, #26=Y:$D40, #27=Y:$DC0, #28=Y:$E40, *
### #29=Y:$EC0, #30=Y:$F40, #31=Y:$FC0, #32=Y:$1040 *
### -- or:  #N=Y:$hex, where  hex=0xC0+0x80*(N-1)   */

  $hexaddr = 0xC0 + 0x80*($mtrNo-1);

  $string = sprintf 'M%d80->Y:$%x,15,1', $mtrNo, $hexaddr;
  &sendPmacCmd ($pmac, $string);				#Sending the string:

  $string = sprintf 'M%d81->Y:$%x,20,4', $mtrNo, $hexaddr;
  &sendPmacCmd ($pmac, $string);				#Sending the string:

### Finished setting Mxx80 and Mxx81 to read status of channel $mtrNo
}

#=================================================================

# sub configureMxx80_Pcs ($assy, $mtrNo) {
  sub configureMxx80_Pcs ($$) {

### The PMAC has to be configured that two M-variables read the status word
### of PMAC output channel (e.g.motor #32). In this status word, bit #15 is
### equal to 1 when the motor is assigned to a coordinate system and 0 if not.
### Four bits #20-23 compose the coordinate system number. We assign the
### M-variables Mxx80 and Mxx81 (i.e. M3280 and M3281 in our example) to read
### this information (the choice of Mxx80 and Mxx81 is voluntary -- just
### deploy any unused  M-variable -- see PeWin or TURBO_SRM.pdf, Software
### Reference Manual).
### (this version is using assembly StrCmd/StrRsp)

  our $IGNORE_ERRORS;
  my ($script, $assy, $mtrNo, $current_time, $string, $hexaddr, $txt);

  $script = 'configureMxx80_Pcs';
  $assy   = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $mtrNo  = shift(@_);

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS=0;}

  if ($mtrNo < 1 || $mtrNo > 32) {
     $current_time = scalar localtime;
     $txt  = '*** '.$script.': '.$current_time."\n";
     $txt .= '*** '.$script.': incorrect motor number='.$mtrNo."\n";
     if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     else                 {print STDOUT $txt; return;}
  }

### Setting Mxx80 and Mxx81 to read status of channel $mtrNo;
### Here are the values for different motors:       *
###  #1=Y:$C0,   #2=Y:#140,  #3=Y:$1C0 , #4=Y:$240, *
###  #5=Y:$2C0,  #6=Y:$340,  #7=Y:$3C0,  #8=Y:$440, *
###  #9=Y:$4C0, #10=Y:$540, #11=Y:$5C0, #12=Y:$640, *
### #13=Y:$6C0, #14=Y:$740, #15=Y:$7C0, #16=Y:$840, *
### #17=Y:$8C0, #18=Y:$940, #19=Y:$9C0, #20=Y:$A40, *
### #21=Y:$AC0, #22=Y:$B40, #23=Y:$BC0, #24=Y:$C40, *
### #25=Y:$CC0, #26=Y:$D40, #27=Y:$DC0, #28=Y:$E40, *
### #29=Y:$EC0, #30=Y:$F40, #31=Y:$FC0, #32=Y:$1040 *
### -- or:  #N=Y:$hex, where  hex=0xC0+0x80*(N-1)   */

  $hexaddr = 0xC0 + 0x80*($mtrNo-1);

  $string = sprintf 'M%d80->Y:$%x,15,1', $mtrNo, $hexaddr;
  &sendPcsCmd ($assy, $string);					#Sending the string:

  $string = sprintf 'M%d81->Y:$%x,20,4', $mtrNo, $hexaddr;
  &sendPcsCmd ($assy, $string);					#Sending the string:

### Finished setting Mxx80 and Mxx81 to read status of channel $mtrNo
}

#=================================================================

# sub unlinkMotor_Pmac ($pmac, $mtrNo) {
sub unlinkMotor_Pmac ($$) {

### Unlink motor from the assembly it belongs to:
### (this version is using pmac StrCmd/StrRsp)

  our ($report, $IGNORE_ERRORS);
  my ($script, $pmac, $mtrNo, $str, $pcsNo, $txt);

  $script = 'unlinkMotor_Pmac';
  $pmac   = shift(@_);         if ($pmac !~ /:$/) {$pmac .= ':';}
  $mtrNo  = shift(@_);

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS=0;}
  if (!defined $report)        {$report = 0;}

  &configureMxx80_Pmac($pmac, $mtrNo);

### Ask if the PMAC out is currently assigned:
  $str = &readPmacRsp($pmac,'M'.$mtrNo.'80','integer');

  if ($str ne '0') {   			#if assigned
### Ask what coordinate system is currently associated with the PMAC out:
     $pcsNo = &readPmacRsp($pmac,'M'.$mtrNo.'81','integer');
### If non-digit or not in range:
     if ($pcsNo !~ /\d/ || $pcsNo < 0 || $pcsNo > 15) {
        $txt = '*** '.$script.': incorrect PCS='.$pcsNo.' for motor #'.$mtrNo.' in '.$pmac."\n";
        if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
        else                 {print STDOUT $txt; return;}
     }
     $pcsNo++;
     print STDOUT '!!! '.$script.': this sub may be unlinking motor from a moving assembly'."\n";
     print STDOUT 'Unlinking motor #'.$mtrNo.' from coordinate system &'.$pcsNo.' in '.$pmac.' ...'."\n";
### Remove the motor from the coordinate system:
     $str = '&'.$pcsNo.' #'.$mtrNo.'->0';
#    print STDOUT script.': sending ['.$str.'] to '.$pmac."\n";
     &sendPmacCmd($pmac,$str);
### Verifying:
#    $str = &readPmacRsp($pmac,'M'.$mtrNo.'80','integer');
#    if ($str ne '0') {
#       print STDOUT '*** '.$script.': Unlinking #'.$mtrNo.' from &'.$pcsNo.' in '.$pmac.' failed!'."\n";
#    }
  }
  else  {
     if ($report) {
        print STDOUT 'Motor #'.$mtrNo.' is not linked to any coordinate system in '.$pmac."\n";
     }
  }

  if ($report) {
     print STDOUT 'Done unlinking motor #'.$mtrNo.' in '.$pmac."\n";
  }
}

#=================================================================

# sub unlinkMotor_Pcs ($assy, $mtrNo) {
sub unlinkMotor_Pcs ($$) {

### Unlink motor from the assembly it belongs to:
### (this version is using assembly StrCmd/StrRsp)

  our ($report, $IGNORE_ERRORS);
  my ($script, $assy, $mtrNo, $str, $pcsNo, $pmac);
  my ($wait_max, $waited, $txt);

  $script   = 'unlinkMotor_Pcs';
  $assy     = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $mtrNo    = shift(@_);
  $wait_max = 300;						#how long to wait for assembly to stop

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS=0;}
  if (!defined $report)        {$report = 0;}

  $str   = '';
  $pcsNo = 0;
  $pmac  = &getPmac($assy);

  &configureMxx80_Pcs($assy, $mtrNo);

### Ask if the PMAC out is currently assigned:
  $str = &readPcsRsp($assy,'M'.$mtrNo.'80','integer');

  if ($str ne '0') {   						#if mtrout is assigned
### Ask what coordinate system is currently associated with the PMAC out:
     $pcsNo = &readPcsRsp($assy,'M'.$mtrNo.'81','integer');
### If non-digit or not in range:
     if ($pcsNo eq '' || $pcsNo !~ /\d/ || $pcsNo < 0 || $pcsNo > 15) {
        $txt = '*** '.$script.': incorrect PCS='.$pcsNo.' for motor #'.$mtrNo.' in '.$pmac."\n";
        if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
        else                 {print STDOUT $txt; return;}
     }
     $pcsNo++;
     $waited = &waitMotor_runprg ($assy,$wait_max,1);		#waited: value=how-long-waited, 0=timeout
     if ($waited == 0) {
        print STDOUT '!!! '.$script.': possibly unlinking motor from a moving assembly='.$assy."\n";
     }
     print STDOUT 'Unlinking motor #'.$mtrNo.' from coordinate system &'.$pcsNo.' in '.$pmac.' ...'."\n";
### Remove the motor from the coordinate system:
     $str = '&'.$pcsNo.' #'.$mtrNo.'->0';
#    print STDOUT $script.': sending ['.$str.'] to '.$assy."\n";
     &sendPcsCmd($assy,$str);
### Verifying:
#    $str = &readPcsRsp($assy,'M'.$mtrNo.'80','integer');
#    if ($str ne '0') {
#       print STDOUT '*** '.$script.': Unlinking #'.$mtrNo.' from &'.$pcsNo.' in '.$pmac.' failed!'."\n";
#    }
  }
  else  {
     if ($report) {
        print STDOUT 'Motor #'.$mtrNo.' is not linked to any coordinate system in '.$pmac."\n";
     }
  }

  if ($report) {print STDOUT 'Done unlinking motor #'.$mtrNo.' in '.$pmac."\n";}
}

#=================================================================

# sub unlinkMotor32_Pmac ($pmac) {
sub unlinkMotor32_Pmac ($) {

### Unlink motor #32 from all beamline PMACS
### (this version is using pmac StrCmd/StrRsp)

  our ($beamline, $unlinkMtrOutWanted);
  my ($pmac, $mtrNo, $pmacName, $name);

  $pmac = shift(@_);         if ($pmac !~ /:$/) {$pmac .= ':';}

  if (!defined $unlinkMtrOutWanted) {$unlinkMtrOutWanted = 0;}
  if ($unlinkMtrOutWanted == 0)     {return;}
  $mtrNo = 32;
  $pmacName = $pmac;
  $pmacName =~ s/^${beamline}:?//;
  $pmacName =~ s/://g;						#should not be needed
### We define the global variable $pmacName.'Unlinked' to let
### the script know that motor-32 in this PMAC is already unlinked.
  $name = $pmacName.'Unlinked';
  no strict 'refs';
  if (!defined  $$name) {
#    print STDOUT 'Unlinking motor #32 in '.$pmac."\n";
     &unlinkMotor_Pmac($pmac,$mtrNo);
     $$name = 1;
  }
# else  {
#    print STDOUT $pmac.' has been processed in the past.'."\n";
# }
  use strict 'refs';
}

#=================================================================

# sub unlinkMotor32_Pcs ($assy) {
sub unlinkMotor32_Pcs ($) {

### Unlink motor #32 from all beamline PMACS
### (this version is using assembly StrCmd/StrRsp)

  our ($beamline, $unlinkMtrOutWanted);
  my ($assy, $pmac, $mtrNo, $pmacName, $name);

  $assy = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}

  if (!defined $unlinkMtrOutWanted) {$unlinkMtrOutWanted = 0;}
  if ($unlinkMtrOutWanted == 0)     {return;}

  $mtrNo    = 32;
  $pmac     = &getPmac($assy);
  $pmacName = $pmac;
  $pmacName =~ s/^${beamline}:?//;
  $pmacName =~ s/://g;
### We define the global variable $pmacName.'Unlinked"' to let
### the script know that motor-32 in this PMAC is already unlinked.
  $name = $pmacName.'Unlinked';
  no strict 'refs';
  if (!defined  $$name) {
#    print STDOUT 'Unlinking motor #32 in '.$pmac."\n";
     &unlinkMotor_Pcs($assy,$mtrNo);
     $$name = 1;
  }
# else  {
#    print STDOUT $pmac.' has been processed in the past.'."\n";
# }
  use strict 'refs';
}

#=================================================================

# sub unlinkMtrOut ($assy, $mtrNo) {
sub unlinkMtrOut ($$) {

### Unlink motor from the assembly it belongs to:

  my ($assy, $mtrNo);

  $assy  = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $mtrNo = shift(@_);

  $assy =~ s/:$//;

### We can use one the two ways:

# my $pmac = &getPmac($assy);
# &unlinkMotor_Pmac($pmac, $mtrNo);

  &unlinkMotor_Pcs($assy, $mtrNo);
}

#=================================================================

# sub isMtrOutLinked ($assy, $mtrNo) {
sub isMtrOutLinked ($$) {

### Check is PMAC output channel (like channel#32) is linked to the current
### coordinate, another coordinate system or unlinked at all.
### Return values:
###  1 = yes, motor is assigned to current assembly
###  0 = no, mnotor is not assigned to anything
### -N = motor is assigned to different coordinate system N

  our ($report, $IGNORE_ERRORS);
  my ($script, $assy, $mtrNo, $pmac, $pcs);
  my ($mtrAsigned, $pcsLinked, $txt);

  $script = 'isMtrOutLinked';
  $assy   = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $mtrNo  = shift(@_);

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS=0;}
  if (!defined $report)        {$report = 0;}

  $pmac = &getPmac($assy);
  $pcs  = &getPcs($assy);

  &configureMxx80_Pcs($assy, $mtrNo);

### Ask if the PMAC out is currently assigned:
  $mtrAsigned = &readPcsRsp($assy,'M'.$mtrNo.'80','integer');
  if ($mtrAsigned eq '0') {return 0;}

### Ask what coordinate system is currently associated with the PMAC out:
  $pcsLinked = &readPcsRsp($assy,'M'.$mtrNo.'81','integer');
### If non-digit or not in range:
  if ($pcsLinked eq '' || $pcsLinked !~ /^[\d]*$/) {
     $txt = '*** '.$script.': illegal PCS request response: PCS='
          . $pcsLinked.' for motor #'.$mtrNo.' in '.$pmac."\n";
     if ($IGNORE_ERRORS) {print STDOUT $txt; return -99;}
     else                {&press_key_exit($txt);}
  }
  elsif ($pcsLinked < 0 || $pcsLinked > 15) {
### This is a problem only when motor is assigned:
     $txt = '*** '.$script.': PCS response not in allowed range: PCS='
          . $pcsLinked.' for motor #'.$mtrNo.' in '.$pmac."\n";
     if ($IGNORE_ERRORS) {print STDOUT $txt; return -100;}
     else                {&press_key_exit($txt);}
  }
  $pcsLinked++;
  if ($pcsLinked == $pcs) {return 1;}
  else                    {return -$pcsLinked;}
}

#=================================================================

# sub getLinkedFormula_Pcs ($assy, $mtrNo) {
sub getLinkedFormula_Pcs ($$) {

### Get formula how mtrNo is linked to assebly

  my ($script, $assy, $mtrNo, $pmac, $pcsNo, $formula);

  $script = 'getLinkedFormula_Pcs';
  $assy   = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $mtrNo  = shift(@_);

  $pmac  = &getPmac($assy);
  $pcsNo = &getPcs($assy);
  $formula = &getLinkedFormula($pmac,$pcsNo,$mtrNo);
  return $formula;
}

#=================================================================

# sub getLinkedFormula ($pmac, $pcsNo, $mtrNo) {
sub getLinkedFormula ($$$) {

### Get formula how mtrNo is linked to assyNo

  my ($pmac, $script, $pcsNo, $mtrNo, $str, $formula);

  $script = 'getLinkedFormula';
  $pmac   = shift(@_);         if ($pmac !~ /:$/) {$pmac .= ':';}
  $pcsNo  = shift(@_);
  $mtrNo  = shift(@_);

  $str = '&'.$pcsNo.' #'.$mtrNo.'->';
  $formula = &readPmacRsp ($pmac, $str);
  return $formula;
}

#=================================================================

# sub linkMtrOut ($assy, $mtrNo, $formula, [$sync]) {
sub linkMtrOut ($$$;$) {

### Link PMAC output channel (like channel#32) to the coordinate
### system of given motor according to the formula

  our ($report, $IGNORE_ERRORS);
  my ($script, $assy, $mtrNo, $formula, $sync, $wait_max, $waited);
  my ($pmac, $pcsNo, $mtrAsigned, $pcsPrevious, $str, $txt);

  $script   = 'linkMtrOut';
  $assy     = shift(@_);        if ($assy !~ /:$/) {$assy .= ':';}
  $mtrNo    = shift(@_);
  $formula  = shift(@_);
  $sync     = shift(@_);        if (!defined $sync) {$sync = 0;}

  $wait_max = 300;						#how long to wait for assembly to stop

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS=0;}
  if (!defined $report)        {$report = 0;}

  $pmac  = &getPmac($assy);
  $pcsNo = &getPcs($assy);

  &configureMxx80_Pcs($assy, $mtrNo);

### Ask if the PMAC out is currently assigned:
  $mtrAsigned = &readPcsRsp($assy,'M'.$mtrNo.'80','integer');
  if ($mtrAsigned ne '0') {$mtrAsigned = 1;}

  if ($mtrAsigned == 1) {
     ### Ask what coordinate system is currently associated with the PMAC out:
     $pcsPrevious = &readPcsRsp($assy,'M'.$mtrNo.'81','integer');
     ### If non-digit or not in range:
     if ($pcsPrevious eq '' || $pcsPrevious !~ /\d/ || $pcsPrevious < 0 || $pcsPrevious > 15) {
        $txt = '*** '.$script.': incorrect previous PCS='.$pcsPrevious.' for MTR=#'.$mtrNo.' in '.$pmac;
        if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
        else {
           print STDOUT $txt;
           $mtrAsigned = 0;
           $pcsPrevious = $pcsNo-1;
        }
     }
     $pcsPrevious++;
     if ($pcsPrevious != $pcsNo) {
        ### Remove the motor-OUT from the coordinate system:
        $waited = &waitMotor_runprg ($assy,$wait_max,'tell');	#waited: value=how-long-waited, 0=timeout
        if ($waited == 0) {
           print STDOUT '!!! '.$script.': possibly unlinking motor from a moving assembly='.$pcsPrevious."\n";
        }
        print STDOUT 'Unlinking MTR=#'.$mtrNo.' from PCS=&'.$pcsPrevious.' in '.$pmac.' ...'."\n";
        $str = '&'.$pcsPrevious.' #'.$mtrNo.'->0';
        if ($report) {print STDOUT $script.': sending ['.$str.'] to '.$assy."\n";}
        &sendPcsCmd($assy,$str);
        $mtrAsigned = 0;
     } else {
        ### Motor is already assigned
        # $mtrAsigned = 1;
     }
  }

  if ($mtrAsigned == 0) {
     ### Zero motor-OUT position:
     print STDOUT 'Applying "homez" to MTR=#'.$mtrNo.' in '.$pmac.' before linking it to '.$assy.'...'."\n";
     &sendPcsCmd($assy,'#'.$mtrNo.' homez');
     $waited = &waitMotor_runprg ($assy,$wait_max,'tell');	#waited: value=how-long-waited, 0=timeout
     if ($waited == 0) {
        print STDOUT '!!! '.$script.': possibly linking motor to a moving assembly='.$assy."\n";
     }
     print STDOUT 'Linking MTR=#'.$mtrNo.' to '.$assy.' (PCS=&'.$pcsNo.') in '.$pmac.'...'."\n";
     $str = '&'.$pcsNo.' #'.$mtrNo.'->'.$formula;
     if ($report) {print STDOUT 'Sending command "'.$str.'" to '.$assy."\n";}
     &sendPcsCmd($assy,$str);
     if ($sync =~ /^sync$/i || $sync =~ /^true$/i || $sync =~ /^yes$/i) {
        print STDOUT 'Syncing linked MTR=#'.$mtrNo.' in PCS=&'.$pcsNo.', '.$pmac.' ...'."\n";
        $str = '&'.$pcsNo.' R';
        &sendPmacCmd($pmac,$str,'nowait');			#run to current Rqs positions
        $waited = &waitMotor_runprg ($assy,$wait_max);		#waited: value=how-long-waited, 0=timeout
     }
  } else {
     print STDOUT '!!! '.$script.': No need to link MTR=#'.$mtrNo.' in '.$pmac.' ...'."\n";
     print STDOUT '!!! '.$script.': MTR=#'.$mtrNo.' is already assigned to PCS='.$pcsNo.' ...'."\n";
  }

### Verifying:
# $str = &readPmacRsp($pmac,'M'.$mtrNo.'80','integer');
# if ($str eq '0') {   						#if not assigned
#    print STDOUT '*** '.$script.': Linking MTR=#'.$mtrNo.' to PCS=&'.$pcsNo.' in '.$pmac.' failed!'."\n";
# }

  if ($report) {print STDOUT 'Done linking MTR=#'.$mtrNo.' to '.$assy.' (PCS=&'.$pcsNo.') in '.$pmac."\n";}
}

#=================================================================

# sub linkMtrOut_Pmac ($pmac, $pcsNo, $mtrNo, $formula, [$sync]) {
sub linkMtrOut_Pmac ($$$$;$) {

### Link PMAC output channel (like channel#32) to the coordinate
### system of given motor according to the formula

  our ($report, $IGNORE_ERRORS);
  my ($script, $pmac, $pcsNo, $mtrNo, $formula, $sync);
  my ($mtrAsigned, $pcsPrevious, $str, $txt);
  my ($assy, $waited, $wait_max, $status);

  $script   = 'linkMtrOut_Pmac';
  $pmac     = shift(@_);         if ($pmac !~ /:$/) {$pmac .= ':';}
  $pcsNo    = shift(@_);
  $mtrNo    = shift(@_);
  $formula  = shift(@_);
  $sync     = shift(@_);  if (!defined $sync) {$sync = 0;}

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS=0;}
  if (!defined $report)        {$report = 0;}

  $wait_max = 300;						#how long to wait for assembly to stop

  &configureMxx80_Pmac($pmac, $mtrNo);

### Ask if the PMAC out is currently assigned:
  $mtrAsigned = &readPmacRsp($pmac,'M'.$mtrNo.'80','integer');
  if ($mtrAsigned ne '0') {$mtrAsigned = 1;}

  if ($mtrAsigned == 1) {
     ### Ask what coordinate system is currently associated with the PMAC out:
     $pcsPrevious = &readPmacRsp($pmac,'M'.$mtrNo.'81','integer');
     ### If non-digit or not in range:
     if ($pcsPrevious !~ /\d/ || $pcsPrevious < 0 || $pcsPrevious > 15) {
        ### This is a problem only when motor is assigned:
        $txt = '*** '.$script.': incorrect previous PCS='.$pcsPrevious.' for MTR=#'.$mtrNo.' in '.$pmac;
        if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
        else {
           print STDOUT $txt;
           $mtrAsigned = 0;
           $pcsPrevious = $pcsNo-1;
        }
     }
     $pcsPrevious++;
     if ($pcsPrevious != $pcsNo) {
        ### Remove the motor-OUT from the coordinate system:
        print STDOUT 'Aborting motion of PCS=&'.$pcsPrevious.' in '.$pmac.' ...'."\n";
        $str = '&'.$pcsPrevious.' A';
        &sendPmacCmd($pmac,$str);				#abort motion
        print STDOUT 'Unlinking MTR=#'.$mtrNo.' from PCS=&'.$pcsPrevious.' in '.$pmac.' ...'."\n";
        $str = '&'.$pcsPrevious.' #'.$mtrNo.'->0';
        if ($report) {print STDOUT $script.': sending ['.$str.'] to '.$pmac."\n";}
        &sendPmacCmd($pmac,$str);
        $mtrAsigned = 0;
     } else {
        ### Motor is already assigned
        # $mtrAsigned = 1;
     }
  }

  if ($mtrAsigned == 0) {
     ### Zero motor-OUT position:
     print STDOUT 'Applying "homez" to MTR=#'.$mtrNo.' in '.$pmac.' ...'."\n";
     &sendPmacCmd($pmac,'#'.$mtrNo.' homez');
     print STDOUT 'Aborting motion of PCS=&'.$pcsNo.' in '.$pmac.' ...'."\n";
     $str = '&'.$pcsNo.' A';
     &sendPmacCmd($pmac,$str);					# abort motion
     select(undef,undef,undef,0.1);				# sleep 0.1sec
     print STDOUT 'Linking MTR=#'.$mtrNo.' to PCS=&'.$pcsNo.' in '.$pmac.'...'."\n";
     $str = '&'.$pcsNo.' #'.$mtrNo.'->'.$formula;
     if ($report) {print STDOUT 'Sending command "'.$str.'" to '.$pmac."\n";}
     &sendPmacCmd($pmac,$str);
     if ($sync =~ /^1$/ || $sync =~ /^sync$/i) {
        print STDOUT 'Syncing linked MTR=#'.$mtrNo.' in PCS=&'.$pcsNo.', '.$pmac.' ...'."\n";
        $str = '&'.$pcsNo.' R';
        &sendPmacCmd($pmac,$str,'nowait');			# abort motion
        $status = &getAssyByPCS($pmac,$pcsNo,\$assy);
        if (!$status) {$waited = &waitMotor_runprg ($assy,$wait_max);}
        else          {sleep($wait_max);}
     }
  } else {
     print STDOUT '!!! '.$script.': No need to link MTR=#'.$mtrNo.' in '.$pmac.' ...'."\n";
     print STDOUT '!!! '.$script.': MTR=#'.$mtrNo.' is already assigned to PCS='.$pcsNo.' ...'."\n";
  }

### Verifying:
# $str = &readPmacRsp($pmac,'M'.$mtrNo.'80','integer');
# if ($str eq '0') {   						#if not assigned
#    print STDOUT '*** '.$script.': Linking MTR=#'.$mtrNo.' to PCS=&'.$pcsNo.' in '.$pmac.' failed!'."\n";
# }

  if ($report) {print STDOUT 'Done linking MTR=#'.$mtrNo.' to PCS=&'.$pcsNo.' in '.$pmac."\n";}
}

#=================================================================

# sub determineFormulaOffset ($assy, $axis, $formula, $cts2egu) {
sub determineFormulaOffset ($$$) {

### This routine is for simple formulas like 0.5x, 2y, -6z.
### It attempts to detemine current leading motor and mtrOut
### positions and then calculate the formula offset so that
### mtrOut is linked to the leading motor with the matching
### coordinate (no need to synchronize motors position before
### scan which somethimes takes long time).

  our $report;
  my ($script, $assy, $axis, $formula, $cts2egu);
  my ($mtrOutName, $nMtr, $factor, $status, $formulaOffset);
  my ($scale, $ratio, $offset, $axisAct, $mtrAct, $pmac);

  $script     = 'determineFormulaOffset';
  $assy       = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $axis       = shift(@_);         if ($axis !~ /:$/) {$axis .= ':';}
  $formula    = shift(@_);
  $cts2egu    = shift(@_);
  $mtrOutName = 'mo';

  if (!defined $report) {$report = 0;}

  $nMtr = 0;
  if ($formula =~ /x/i) {$nMtr++;}
  if ($formula =~ /y/i) {$nMtr++;}
  if ($formula =~ /z/i) {$nMtr++;}
  if ($nMtr > 1) {
     if ($report) {
        print STDOUT $script.': formula ['.$formula.'] for '.$axis.' is not trivial.'."\n";
        print STDOUT $script.': no offset applied.'."\n";
     }
     return 0;
  }
  $factor = $formula;
  $factor =~ s/[xyz]//i;
  if    ($factor eq '' || $factor eq '+') {$factor = 1;}
  elsif ($factor eq '-')                  {$factor =-1;}
  if (! &is_decimal($factor)) {
     print STDOUT '*** '.$script.': failed extracting factor from formula ['.$formula.'] for '.$axis."\n";
     print STDOUT '*** '.$script.': no offset applied.'."\n";
     return 0;
  }
  if ($factor == 0) {
     print STDOUT '*** '.$script.': zero factor found in formula ['.$formula.'] for '.$axis."\n";
     print STDOUT '*** '.$script.': no offset applied.'."\n";
     return 0;
  }

### Verify scaling
  ($status, $scale) = Pezca::GetDouble($axis.'Scale');
  if ($status) {
     &report_only($script, $status, $axis.'Scale');
     return 0;
  }
  $ratio = ($cts2egu*$factor)/$scale;

  if ($ratio < 0.95 || $ratio > 1.05) {
     print  STDOUT '*** '.$script.': ratio of scale and cts2egu is not 1 for '.$axis."\n";
     printf STDOUT '*** '.$script.': expected (cts2egu='.$cts2egu.')*(factor='.$factor.')/(scale='.$scale.')=1, found %.3g'."\n",$ratio;
     printf STDOUT '*** '.$script.': consider changing cts2egu in motorList.dat to %.4tg'."\n", ($ratio*$scale)/$factor;
     print  STDOUT '*** '.$script.': no offset applied.'."\n";
     return 0;
  }

### Calculate formula offset:
  ($status, $offset) = Pezca::GetDouble($axis.'Offset');
  if ($status) {
     &report_only($script, $status, $axis.'Offset');
     return 0;
  }
  ($status, $axisAct) = Pezca::GetDouble($axis.'ActPos');
  if ($status) {
     &report_only($script, $status, $axis.'ActPos');
     return 0;
  }
### Position of axis in encoder counts:
  $axisAct  = ($axisAct - $offset) / $scale;

### Position of mtrOut:
  $pmac = &getPmac($assy);
  ($status, $mtrAct) = Pezca::GetDouble($pmac.$mtrOutName.'ActPos');
  if ($status) {
     &report_only($script, $status, $pmac.$mtrOutName.'ActPos');
     return 0;
  }
  $formulaOffset = int($mtrAct - $factor * $axisAct);
  if ($report) {
     print STDOUT $script.': calculated offset for '.$axis.' is '.$formulaOffset."\n";
  }
  return $formulaOffset;
}

#=================================================================

# sub determineFormulaOffset2 ($assy, $axis, $formula, $cts2egu) {
sub determineFormulaOffset2 ($$$) {

### This routine is for simple formulas like 0.5x, 2y, -6z.
### It sets mtrOut position counter to "0" by applying "homez".
### Then it attempts to detemine current leading motor position
### calculates the formula offset so that mtrOut is linked to
### the leading motor with the matching coordinate (no need to
### synchronize motors position before scan which somethimes
### takes long time).

  our $report;
  my ($script, $assy, $axis, $formula, $cts2egu);
  my ($mtrOutName, $nMtr, $factor, $status, $formulaOffset);
  my ($scale, $ratio, $offset, $axisAct, $pmac, $pv);

  $script     = 'determineFormulaOffset2';
  $assy       = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $axis       = shift(@_);         if ($axis !~ /:$/) {$axis .= ':';}
  $formula    = shift(@_);
  $cts2egu    = shift(@_);
  $mtrOutName = 'mo';

  if (!defined $report) {$report = 0;}

  $nMtr = 0;
  if ($formula =~ /x/i) {$nMtr++;}
  if ($formula =~ /y/i) {$nMtr++;}
  if ($formula =~ /z/i) {$nMtr++;}
  if ($nMtr > 1) {
     if ($report) {
        print STDOUT $script.': formula ['.$formula.'] for '.$axis.' is not trivial.'."\n";
        print STDOUT $script.': no offset applied.'."\n";
     }
     return 0;
  }
  $factor = $formula;
  $factor =~ s/[xyz]//i;
  if    ($factor eq '' || $factor eq '+') {$factor = 1;}
  elsif ($factor eq '-')                  {$factor =-1;}
  if (! &is_decimal($factor)) {
     print STDOUT '*** '.$script.': failed extracting factor from formula ['.$formula.'] for '.$axis."\n";
     print STDOUT '*** '.$script.': no offset applied.'."\n";
     return 0;
  }
  if ($factor == 0) {
     print STDOUT '*** '.$script.': zero factor found in formula ['.$formula.'] for '.$axis."\n";
     print STDOUT '*** '.$script.': no offset applied.'."\n";
     return 0;
  }

### Verify scaling
  ($status, $scale) = Pezca::GetDouble($axis.'Scale');
  if ($status) {
     &report_only($script, $status, $axis.'Scale');
     return 0;
  }
  $ratio = ($cts2egu*$factor)/$scale;

  if ($ratio < 0.95 || $ratio > 1.05) {
     print  STDOUT '*** '.$script.': ratio of scale and cts2egu is not 1 for '.$axis."\n";
     printf STDOUT '*** '.$script.': expected (cts2egu='.$cts2egu.')*(factor='.$factor.')/(scale='.$scale.')=1, found %.3g'."\n",$ratio;
     printf STDOUT '*** '.$script.': consider changing cts2egu in motorList.dat to %.4tg'."\n", ($ratio*$scale)/$factor;
     print  STDOUT '*** '.$script.': no offset applied.'."\n";
     return 0;
  }

### Calculate formula offset:
  ($status, $offset) = Pezca::GetDouble($axis.'Offset');
  if ($status) {
     &report_only($script, $status, $axis.'Offset');
     return 0;
  }
  ($status, $axisAct) = Pezca::GetDouble($axis.'ActPos');
  if ($status) {
     &report_only($script, $status, $axis.'ActPos');
     return 0;
  }
### Position of axis in encoder counts:
  $axisAct  = ($axisAct - $offset) / $scale;

### Zero position counter of mtrOut:
  $pmac = &getPmac($assy);      if ($pmac !~ /:$/) {$pmac .= ':';}
  $pv = $pmac.$mtrOutName.':StrCmd';
  $status = Pezca::PutString($pv,'homez');
  if ($status) {
     &report_only($script, $status, $pv);
     return 0;
  }
  select(undef,undef,undef,0.1);				#sleep 0.1sec

  $formulaOffset = -int($factor * $axisAct);
  if ($report) {
     print STDOUT $script.': calculated offset for '.$axis.' is '.$formulaOffset."\n";
  }
  return $formulaOffset;
}

#=================================================================

# sub stopMtrOut_Pmac ($pmac, $mtrNo) {
  sub stopMtrOut_Pmac ($$) {

### Stop PMAC output channel (like channel#32) in the PMAC hosting given motor $pmac
### (this version is using pmac StrCmd/StrRsp)

  our $IGNORE_ERRORS;
  my ($script, $pmac, $mtrNo, $current_time, $txt);

  $script = 'stopMtrOut_Pmac';
  $pmac   = shift(@_);         if ($pmac !~ /:$/) {$pmac .= ':';}
  $mtrNo  = shift(@_);

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS=0;}

  if ($mtrNo < 1 || $mtrNo > 32) {
     $current_time = scalar localtime;
     $txt  = '*** '.$script.': '.$current_time."\n";
     $txt .= '*** '.$script.': incorrect motor number='.$mtrNo."\n";
     if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     else                 {print STDOUT $txt; return;}
  }

### Stopping PMAC output channel:
  &sendPmacCmd($pmac,'#'.$mtrNo.' J/');
}

#=================================================================
# sub stopMtrOut_Pcs ($assy, $mtrNo) {
  sub stopMtrOut_Pcs ($$) {

### Stop PMAC output channel (like channel#32) in the PMAC hosting given motor $pmac
### (this version is using assembly StrCmd/StrRsp)

  our $IGNORE_ERRORS;
  my ($script, $assy, $mtrNo, $current_time, $txt);

  $script = 'stopMtrOut_Pcs';
  $assy   = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $mtrNo  = shift(@_);

  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS = 0;}

  if ($mtrNo < 1 || $mtrNo > 32) {
     $current_time = scalar localtime;
     $txt  = '*** '.$script.': '.$current_time."\n";
     $txt .= '*** '.$script.': incorrect motor number='.$mtrNo."\n";
     if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     else                 {print STDOUT $txt; return;}
  }

### Stopping PMAC output channel:
  &sendPcsCmd($assy,'#'.$mtrNo.' J/');
}

#=================================================================

# sub sendMtrCmd ($motor, $cmd, ['nowait|wait|wait=nn.nnn']) {
sub sendMtrCmd ($$;$) {

  my ($script, $motor, $cmd, $nowait, $status);

  $script = 'sendMtrCmd';
  $motor  = shift(@_);            if ($motor !~ /:$/) {$motor .= ':';}
  $cmd    = shift(@_);
  $nowait = shift(@_);
  
  $status = &sendCmd($motor.'StrCmd',$cmd,$nowait);
  if ($status) {
     print STDOUT '*** '.$script.': sending "'.$cmd.'" to motor=['.$motor
                 .'] failed. Error='.$status.' from sendCmd'."\n";
  }
  return $status;						#for the 'IGNORE_ERRORS' case
}

#=================================================================

# sub sendPmacCmd ($pmac, $cmd, ['nowait|wait|wait=nn.nnn']) {
sub sendPmacCmd ($$;$) {

  my ($script, $pmac, $cmd, $nowait, $status);

  $script = 'sendPmacCmd';
  $pmac   = shift(@_);         if ($pmac !~ /:$/) {$pmac .= ':';}
  $cmd    = shift(@_);
  $nowait = shift(@_);

  $status = &sendCmd($pmac.'StrCmd',$cmd,$nowait);
  if ($status) {
     print STDOUT '*** '.$script.': sending "'.$cmd.'" to pmac=['.$pmac
                 .'] failed. Error='.$status.' from sendCmd'."\n";
  }
  return $status;						#for the 'IGNORE_ERRORS' case
}

#=================================================================

# sub sendPcsCmd ($assy, $cmd, ['nowait|wait|wait=nn.nnn']) {
sub sendPcsCmd ($$;$) {

  my ($script, $assy, $cmd, $nowait, $status);

  $script   = 'sendPcsCmd';
  $assy     = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $cmd      = shift(@_);
  $nowait   = shift(@_);

  $status = &sendCmd($assy.'StrCmd',$cmd,$nowait);
  if ($status) {
     print STDOUT '*** '.$script.': sending "'.$cmd.'" to assy=['.$assy
                 .'] failed. Error='.$status.' from sendCmd'."\n";
  }
  return $status;						#for the 'IGNORE_ERRORS' case
}

#=================================================================

# sub sendCmd ($pv, $cmd, ['nowait|wait|wait=nn.nnn']) {
sub sendCmd ($$;$) {

  my ($script, $pv, $cmd, $nowait, $status);
  my ($ezcatimeout_default, $ezcatimeout_wait);

  $script = 'sendCmd';
  $pv     = shift(@_);
  $cmd    = shift(@_);
  $nowait = shift(@_);     if (!defined $nowait) {$nowait=0;}
# $ezcatimeout_wait = 0.5;					#default waiting time (s)
  $ezcatimeout_wait = 2;					#default waiting time (s)

  if    ($nowait =~ /^nowait$/i || $nowait =~ /^1$/) {$nowait=1;}
  elsif ($nowait =~ /^wait$/i   || $nowait =~ /^0$/) {$nowait=0;}
  elsif ($nowait =~ /^wait=/i) {
     $nowait = s/^wait=//i;
     if ($nowait =~ /^[+]?(?:\d+(?:\.\d*)?|\.\d+)$/) {		#no exponents!
        $ezcatimeout_wait = $nowait;
     } else {
        print STDOUT '!!! '.$script.': incorrect waiting time="'.$nowait.'"'."\n";
     }
     $nowait = 0;
  }
  else {
     print STDOUT '!!! '.$script.': incorrect waiting argument="'.$nowait.'"'."\n";
     $nowait = 0;
  }

# print STDOUT 'pv='.$pv.'  cmd='.$cmd."\n";
  if ($nowait) {
     $status = Pezca::PutStringOldCa($pv,$cmd);
  } else {
     $ezcatimeout_default = Pezca::GetTimeout();		#this is 50ms by default
     Pezca::SetTimeout($ezcatimeout_wait);			#set longer CA timeout
     $status = Pezca::PutString($pv,$cmd);
     Pezca::SetTimeout($ezcatimeout_default);			#restore CA timeout
     if ($status == 6) {
        print STDOUT '!!! '.$script.': received "no timely response"'."\n"
                           .'         PV = ['.$pv."\n"
                           .'        CMD = ['.$cmd.']'."\n"
                           .' CA timeout = '.$ezcatimeout_wait.'s'."\n";
     }
  }
  if ($status != 0 && $status != 6) {
     print STDOUT '*** '.$script.': sending "'.$cmd.'" to '.$pv.' failed.'."\n";
  }
  &report_only($script, $status, $pv, \*STDOUT, $cmd);
  select(undef,undef,undef,0.1);				#sleep 0.1sec
  return $status;						#for the 'IGNORE_ERRORS' case
}

#=================================================================

# sub readMtrRsp ($motor, $cmd, [$filter,\$status]) {
sub readMtrRsp ($$;$$) {					#returns $rsp and optionally fills $status

  my ($script, $motor, $cmd, $filter, $statref, $status, $rsp);

  $script  = 'readMtrRsp';
  $motor   = shift(@_);	         if ($motor !~ /:$/) {$motor .= ':';}
  $cmd     = shift(@_);
  $filter  = shift(@_);						#optional numeric/integer/hex/hexdirect filter
  $statref = shift(@_);						#optional pointer to status to be filled

  $status = 0;
  $rsp = &readRsp ($motor.'StrCmd', $motor.'StrRsp', $cmd, $filter, \$status);
  if ($status) {
     print STDOUT '*** '.$script.': reading response to "'.$cmd.'" from motor=['
                 .$motor.'] failed. Error='.$status.' from readRsp'."\n";
  }
  if (defined $statref) {$$statref = $status;}
  return $rsp;
}

#=================================================================

# sub readPmacRsp ($pmac, $cmd, [$filter,\$status]) {
sub readPmacRsp ($$;$$) {					#returns $rsp and optionally fills $status

  my ($script, $pmac, $cmd, $filter, $statref, $status, $rsp);

  $script  = 'readPmacRsp';
  $pmac    = shift(@_);         if ($pmac !~ /:$/) {$pmac .= ':';}
  $cmd     = shift(@_);
  $filter  = shift(@_);						#optional numeric/integer/hex/hexdirect filter
  $statref = shift(@_);						#optional pointer to status to be filled

  $status = 0;
  $rsp = &readRsp ($pmac.'StrCmd', $pmac.'StrRsp', $cmd, $filter, \$status);
  if ($status) {
     print STDOUT '*** '.$script.': reading response to "'.$cmd.'" from pmac=['
                 .$pmac.'] failed. Error='.$status.' from readRsp'."\n";
  }
  if (defined $statref) {$$statref = $status;}
  return $rsp;
}

#=================================================================

# sub readPcsRsp ($assy, $cmd, [$filter,\$status]) {
sub readPcsRsp ($$;$$) {					#returns $rsp and optionally fills $status

  my ($script, $assy, $cmd, $filter, $statref, $status, $rsp); 

  $script  = 'readPcsRsp';
  $assy    = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $cmd     = shift(@_);
  $filter  = shift(@_);						#optional numeric/integer/hex/hexdirect filter
  $statref = shift(@_);						#optional pointer to status to be filled

  $status = 0;
  $rsp = &readRsp ($assy.'StrCmd', $assy.'StrRsp', $cmd, $filter, \$status);
  if ($status) {
     print STDOUT '*** '.$script.': reading response to "'.$cmd.'" from assy=['
                 .$assy.'] failed. Error='.$status.' from readRsp'."\n";
  }
  if (defined $statref) {$$statref = $status;}
  return $rsp;
}

#=================================================================

# sub readRsp ($pvcmd, $pvrsp, $cmd, [$filter,\$status]) {
sub readRsp ($$$;$$) {						#returns $rsp and optionally fills $status

  my ($script, $pvcmd, $pvrsp, $cmd, $filter, $statref, $status, $rsp);

  $script  = 'readRsp';
  $pvcmd   = shift(@_);
  $pvrsp   = shift(@_);
  $cmd     = shift(@_);
  $filter  = shift(@_);						#optional numeric/integer/hex/hexdirect filter
  $statref = shift(@_);						#optional pointer to status to be filled
  if (!defined $filter)  {$filter = 'none';}

# print STDOUT 'pv(cmd)='.$pvcmd.'  pv(rsp)='.$pvrsp.'  cmd='.$cmd."\n";
  $status = &sendCmd($pvcmd,$cmd);
  if ($status) {
     if ($filter =~ /^numeric$/i ||
         $filter =~ /^integer$/i ||
         $filter =~ /^hex/i)         {$rsp = 0;}
     else                            {$rsp = '';}
     if (defined $statref) {$$statref = $status;}
     return $rsp;						#for the 'IGNORE_ERRORS' case
  }
  ($status, $rsp) = Pezca::GetString($pvrsp.'.RTYP');
  if ($rsp =~ /^waveform$/i) {					#PowerPmac
     ($status,$rsp) = &PezcaGetStringWF($pvrsp);  
  } else {							#TurboPmac
     ($status,$rsp) = Pezca::GetString($pvrsp);
  }
  if ($status) {
     &report_only($script, $status, $pvrsp);
     if ($filter =~ /^numeric$/i ||
         $filter =~ /^integer$/i ||
         $filter =~ /^hex/i)         {$rsp = 0;}
     else                            {$rsp = '';}
     if (defined $statref) {$$statref = $status;}
     return $rsp;						#for the 'IGNORE_ERRORS' case
  }
  if ($filter =~ /^numeric$/i) {				#test for numeric response
     if ($rsp eq '' || &is_float($rsp) != 1) {
        print STDOUT '!!! '.$script.': non-numeric response from '.$pvrsp."\n";
        print STDOUT '!!!             cmd='.$cmd.'  pv(cmd)='.$pvcmd."\n";
        print STDOUT '!!!             rsp='.$rsp."\n\n";
        $rsp = '';
        $status = -1;
     }
  }
  elsif ($filter =~ /^integer$/i) {				#test for integer response
     if ($rsp eq '' || &is_int($rsp) != 1) {
        print STDOUT '!!! '.$script.': non-integer response from '.$pvrsp."\n";
        print STDOUT '!!!             cmd='.$cmd.'  pv(cmd)='.$pvcmd."\n";
        print STDOUT '!!!             rsp='.$rsp."\n\n";
        $rsp = '';
        $status = -1;
     }
  }
  elsif ($filter =~ /^hex/i) {					#test for hex response
     if ($rsp eq '' || $rsp !~ /^\$[+-]?[0-9a-fA-F]+$/) {
        print STDOUT '!!! '.$script.': non-hex response from '.$pvrsp."\n";
        print STDOUT '!!!             cmd='.$cmd.'  pv(cmd)='.$pvcmd."\n";
        print STDOUT '!!!             rsp='.$rsp."\n\n";
        $rsp = '';
        $status = -1;
     }
     if ($filter !~ /^hexdirect/i) {
        ### Perl style for hex is 0xnnnn instead of $nnnn:
        if    ($rsp =~ /^\$/)  {$rsp =~ s/^\$/0x/;}
        elsif ($rsp =~ /^-\$/) {$rsp =~ s/^-\$/-0x/;}
        ### We can use hex function to convert hex to decimal:
#       $rsp = hex($rsp);
     }
  }
  if (defined $statref) {$$statref = $status;}
  return $rsp;
}

#=================================================================

# sub filterAxis($axis) {
sub filterAxis($) {						#returns filetered $axis

  our $beamline;
  my $axis = shift(@_);

  $axis =~ s/:ActPos$//;					#remove ':ActPos' at the end of line
  $axis =~ s/:RqsPos$//;					#remove ':RqsPos' at the end of line
  $axis =~ s/:$//;						#remove ':' at the end of line
  $axis =~ s/^://;						#remove ':' at the beginning of the line

# if ($axis !~ /^${beamline}/) {$axis = $beamline.':'.$axis;}
  if ($axis !~ /^${beamline}:|^23d:|^23i:|^23o:|^23b:/) {$axis = $beamline.':'.$axis;}

  return $axis;
}

#=================================================================

# sub filterAssy($assy) {
sub filterAssy($) {						#returns filetered $assy

  our $beamline;
  my $assy = shift(@_);

  $assy =~ s/:Abort$//;						#remove ':Abort' at the end of line
  $assy =~ s/:InPos$//;						#remove ':InPos' at the end of line
  $assy =~ s/:RunPrg$//;					#remove ':RunPrg' at the end of line
  $assy =~ s/:Start$//;						#remove ':Start' at the end of line
  $assy =~ s/:Ready$//;						#remove ':Ready' at the end of line
  $assy =~ s/:$//;						#remove ':' at the end of line
  $assy =~ s/^://;						#remove ':' at the beginning of the line

# if ($assy !~ /^${beamline}/) {$assy = $beamline.':'.$assy;}
  if ($assy !~ /^${beamline}:|^23d:|^23i:|^23o:|^23b:/) {$assy = $beamline.':'.$assy;}

  return $assy;
}

#=================================================================

# sub filterPmac($pmac) {
sub filterPmac($) {						#returns filetered $pmac

  our $beamline;
  my $pmac = shift(@_);

  $pmac =~ s/:StrCmd$//;					#remove ':StrCmd' at the end of line
  $pmac =~ s/:StrRsp$//;					#remove ':StrRsp' at the end of line
  $pmac =~ s/:mbx:SvoTm$//;					#remove ':mbx:SvoTm' at the end of line
  $pmac =~ s/:svo:SvoTm$//;					#remove ':svo:SvoTm' at the end of line
  $pmac =~ s/:bkg:SvoTm$//;					#remove ':bkg:SvoTm' at the end of line
  $pmac =~ s/:var:SvoTm$//;					#remove ':var:SvoTm' at the end of line
  $pmac =~ s/:$//;						#remove ':' at the end of line
  $pmac =~ s/^://;						#remove ':' at the beginning of the line

# if ($pmac !~ /^${beamline}/) {$pmac = $beamline.':'.$pmac;}
  if ($pmac !~ /^${beamline}:|^23d:|^23i:|^23o:|^23b:/) {$pmac = $beamline.':'.$pmac;}

  return $pmac;
}

#=================================================================

# ($assy,$axis,$cts2egu,$formula,$mtrOut,$struckPs,$struckNg) =
#                                                  &readMotorList($keyword);
# sub readMotorList($keyword) {
sub readMotorList($) {

### This sub reads file motorList.dat from the fast scans package
### and returns all parameters of a respective motor. Only "gmca-1"
### records are processed.

  our ($report, $IGNORE_ERRORS, $beamline);
  my ($script, $keyword, $motorlist, $handle, $line, @array, $narr);
  my ($i, $record, $recordType, $motorName, $group, $txt, $fastscans);
  my ($assy, $axis, $cts2egu, $formula, $mtrOut, $struckPs, $struckNg);

  $script  = 'readMotorList';
  $keyword = shift(@_);
  if (!defined $report)        {$report = 0;}
  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS = 0;}

  if (!defined $ENV{FASTSCANS}) {
     $txt = '*** '.$script.': Environment FASTSCANS is undefined'."\n\n";
     if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     else                 {print STDOUT $txt; return;}
  }
  $fastscans = $ENV{FASTSCANS}; $fastscans =~ s/\\/\//g; $fastscans =~ s/\/$//;
  $fastscans =~ s/\/bin$//;
  $motorlist = $fastscans.'/data/motorList_'.$beamline.'.dat';
  print STDOUT 'Opening DB file: '.$motorlist.' for assembly='.$keyword."\n";
  $handle = open (ML, '< '.$motorlist);
  if (!defined $handle) {
     $txt = '*** '.$script.': Error opening '.$motorlist."\n";
     if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     else                 {print STDOUT $txt; return;}
  }
  select ML; $|=1;
  $line = 0;
  while (<ML>) {
     $record = $_;
     chomp $record;						#trim CR
     $line++;
     if ($record !~ /^#/ && $record =~ /$keyword/) {
        @array = split ' ', $record;
        $narr = @array;
        if ($array[0] ne 'gmca-1') {next;}			#recordType
        if ($narr != 10) {
           $txt  = '*** '.$script.': Record length='.$narr.' is not 10 words as expected for "gmca-1".'."\n";
           $txt .= '*** '.$script.': ... at line='.$line.' of '.$motorlist."\n";
           $txt .= '*** '.$script.': Record split results:'."\n";
           for ($i=0; $i<$narr; $i++) {$txt .= $array[$i]."\n";}
           if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
           else                 {print STDOUT $txt; return;}
        }
        ($recordType,$motorName,$group,$assy,$axis,$cts2egu,$formula,$mtrOut,$struckPs,$struckNg) = @array;
        if ($motorName ne $keyword) {
           $txt  = '*** '.$script.': wrong keyword placement in record structure.'."\n\n";
           $txt .= '*** '.$script.': ... at line='.$line.' of '.$motorlist."\n";
           if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
           else                 {print STDOUT $txt; return;}
        }
        last;
     }
  }
  close(ML);
  if (!defined $motorName) {
    $txt = '*** '.$script.': record '.$keyword.' not found'."\n\n";
    if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
    else                 {print STDOUT $txt; return;}
  }
  $assy = $group.$assy;
  $axis = $group.$axis;
  $assy =~ s/:$//;						#remove ":" at the end of line
  $axis =~ s/:$//;						#remove ":" at the end of line

  if ($report) {
     print STDOUT '--- '.$script.' found:'."\n";
     print STDOUT '     assy='.$assy."\n";
     print STDOUT '     axis='.$axis."\n";
     print STDOUT '  cts2egu='.$cts2egu."\n";
     print STDOUT '  formula='.$formula."\n";
     print STDOUT '   mtrOut='.$mtrOut."\n";
     print STDOUT ' struckPs='.$struckPs."\n";
     print STDOUT ' struckNg='.$struckNg."\n";
  }
  return $assy,$axis,$cts2egu,$formula,$mtrOut,$struckPs,$struckNg;
}

#=================================================================

# ($assy,$axis,$cts2egu,$motor,$encoderAddress) = &readMotorList2($keyword);
# sub readMotorList2($keyword) {
sub readMotorList2($) {

### This sub reads file motorList.dat from the fast scans package
### and returns all parameters of a respective motor. Only "gmca-2"
### records (records for motors that can be scanned with a PLC
### advancing the Struck scaler at fixed encoder increments) are
### processed.

  our ($report, $IGNORE_ERRORS, $beamline);
  my ($script, $keyword, $motorlist, $handle, $line, @array, $narr);
  my ($i, $record, $recordType, $motorName, $group, $recType);
  my ($assy, $axis, $cts2egu, $mtr, $encAddr, $txt, $fastscans);

  $script  = 'readMotorList2';
  $recType = 'gmca-2';
  $keyword = shift(@_);
  if (!defined $report)        {$report = 0;}
  if (!defined $IGNORE_ERRORS) {$IGNORE_ERRORS = 0;}

  if (!defined $ENV{FASTSCANS}) {
     $txt = '*** '.$script.': Environment FASTSCANS is undefined'."\n\n";
     if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     else                 {print STDOUT $txt; return;}
  }
  $fastscans = $ENV{FASTSCANS}; $fastscans =~ s/\\/\//g; $fastscans =~ s/\/$//;
  $fastscans =~ s/\/bin$//;
  $motorlist = $fastscans.'/data/motorList_'.$beamline.'.dat';
  print STDOUT 'Opening DB file: '.$motorlist.' for assembly='.$keyword."\n";
  $handle = open (ML, '< '.$motorlist);
  if (!defined $handle) {
     $txt = '*** '.$script.': Error opening '.$motorlist."\n";
     if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
     else                 {print STDOUT $txt; return;}
  }
  select ML; $|=1;
  $line = 0;
  while (<ML>) {
     $record = $_;
     chomp $record;						#trim CR
     $line++;
     if ($record !~ /^#/ && $record =~ /$keyword/) {
        @array = split ' ', $record;
        $narr = @array;
        if ($array[0] ne $recType) {next;}			#recordType
        if ($narr != 8) {
           $txt  = '*** '.$script.': Record length='.$narr.' is not 8 words as expected for "'.$recType.'".'."\n";
           $txt .= '*** '.$script.': ... at line='.$line.' of '.$motorlist."\n";
           $txt .= '*** '.$script.': Record split results:'."\n";
           for ($i=0; $i<$narr; $i++) {$txt .= $array[$i]."\n";}
           if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
           else                 {print STDOUT $txt; return;}
        }
        ($recordType,$motorName,$group,$assy,$axis,$cts2egu,$mtr,$encAddr) = @array;
        if ($motorName ne $keyword) {
           $txt  = '*** '.$script.': wrong keyword placement in record structure.'."\n\n";
           $txt .= '*** '.$script.': ... at line='.$line.' of '.$motorlist."\n";
           if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
           else                 {print STDOUT $txt; return;}
        }
        if ($encAddr !~ /^\$/) {
           $txt  = '*** '.$script.': wrong encoder address in record "'.$record.'"'."\n\n";
           $txt .= '*** '.$script.': ... at line='.$line.' of '.$motorlist."\n";
           if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
           else                 {print STDOUT $txt; return;}
        }
        last;
     }
  }
  close(ML);
  if (!defined $motorName) {
    $txt = '*** '.$script.': record "'.$keyword.'" of type not found'."\n\n";
    if (!$IGNORE_ERRORS) {&press_key_exit($txt);}
    else                 {print STDOUT $txt; return;}
  }
  $assy = $group.$assy;
  $axis = $group.$axis;
  $mtr  = $group.$mtr;
  $assy =~ s/:$//;						#remove ":" at the end of line
  $axis =~ s/:$//;						#remove ":" at the end of line
  $mtr  =~ s/:$//;						#remove ":" at the end of line

  if ($report) {
     print STDOUT '--- '.$script.' found:'."\n";
     print STDOUT '     assy='.$assy."\n";
     print STDOUT '     axis='.$axis."\n";
     print STDOUT '  cts2egu='.$cts2egu."\n";
     print STDOUT '      mtr='.$mtr."\n";
     print STDOUT '  encAddr='.$encAddr."\n";
  }
  return $assy,$axis,$cts2egu,$mtr,$encAddr;
}

#=================================================================

# ($cts2egu,$formula,$mtrOut,$struckPs,$struckNg) =
#                                                  &searchMotorList($assy,$axis,[hardsync|scanrec|any]);
# sub searchMotorList($assy,$axis,[hardsync|scanrec|any]) {
sub searchMotorList($$;$) {

### This sub searches file motorList.dat from the fast scans package
### and returns all parameters of a respective assy and axis. Only
### "gmca-1" records are processed.


  our ($report, $beamline);
  my ($script, $assy, $axis, $mode,$i, $fastscans);
  my ($motorlist, $handle, $line, $linefound, $record, @arr, $narr);
  my ($as, $ax, $cts2egu, $formula, $mtrOut, $struckPs, $struckNg);

  $script  = 'searchMotorList';
  $assy = shift(@_);	$assy =~ s/:$//;			#here we need it for legacy calls
  $axis = shift(@_);	$axis =~ s/:$//;			#here we need it for legacy calls
  $mode = shift(@_);
  if (!defined $mode) {$mode='any';}
  if ($mode ne 'any' && $mode ne 'harsync' && $mode ne 'scanrec') {
     print STDOUT '!!! '.$script.': incorrect mode='.$mode.', taking "any".'."\n\n";
     $mode = 'any';
  }
  if (!defined $report) {$report = 0;}

### Remove 23i,23o,23b, or 23d:					<<==== GMCA SPECIFIC!
  if ($assy =~ /^23[a-z]{1}:/i) {$assy =~ s/^23[a-z]{1}://i};
  if ($axis =~ /^23[a-z]{1}:/i) {$axis =~ s/^23[a-z]{1}://i};

  if (!defined $ENV{FASTSCANS}) {
     print STDOUT '*** '.$script.': Environment FASTSCANS is undefined'."\n\n";
     return;
  }
  $fastscans = $ENV{FASTSCANS}; $fastscans =~ s/\\/\//g; $fastscans =~ s/\/$//;
  $fastscans =~ s/\/bin$//;
  $motorlist = $fastscans.'/data/motorList_'.$beamline.'.dat';
  print STDOUT 'Opening DB file: '.$motorlist.' for assy='.$assy.' & axis='.$axis."\n";
  $handle = open (ML, '< '.$motorlist);
  if (!defined $handle) {
     print STDOUT '*** '.$script.': Error opening '.$motorlist."\n";
     return;
  }
  select ML; $|=1;
  $line=0;
### Record structure:
###  gmca-1    WBS_Vcenter   WS:  Av: ZC:   5.E-5  .5x+.5y     32       2         3
### $recordType    $key    $group $as $ax $cts2egu $formula $mtrOut $struckPs $struckNg

  while (<ML>) {
     $record = $_;
     chomp $record;						#trim CR
     $line++;
     if ($record !~ /^#/ && $record !~ /^end/i) {
        @arr = split " ", $record;
        $narr = @arr;
        if ($arr[0] ne 'gmca-1') {next;}
        if ($narr != 10) {
           print STDOUT '*** '.$script.': Record length='.$narr.' is not 10 as expected'."\n";
           print STDOUT '*** '.$script.': ... at line='.$line.' of '.$motorlist."\n";
           print STDOUT '*** '.$script.': Record split results:'."\n";
           for ($i=0; $i<$narr; $i++) {print STDOUT $arr[$i]."\n";}
           return;
        }
### ($recordType,$key,$group,$as,$ax,$cts2egu,$formula,$mtrOut,$struckPs,$struckNg) = @arr;
###      0         1     2    3   4      5        6       7        8         9
        $as = $arr[2].$arr[3];  $as =~ s/:$//;			#remove ":" at the end of line
        $ax = $arr[2].$arr[4];  $ax =~ s/:$//;			#remove ":" at the end of line
        if    ($mode eq 'harsync' && ($arr[5] eq '1' || $arr[6] eq '0')) {next;}
        elsif ($mode eq 'scanrec' && ($arr[5] ne '1' || $arr[6] ne '0')) {next;}
        if (($assy eq $as) && ($axis eq $ax)) {
           if (!defined $linefound) {
              $linefound = $line;
              ($cts2egu,$formula,$mtrOut,$struckPs,$struckNg) = ($arr[5],$arr[6],$arr[7],$arr[8],$arr[9]);
           } else {
              print STDOUT '*** '.$script.': duplicate data for assy='.$assy.' & axis='.$axis."\n";
              print STDOUT '*** '.$script.': ... at lines '.$linefound.' & '.$line.' of '.$motorlist."\n";
              return;
           }
        }
     }
  }
  close(ML);
  if (defined $linefound) {
     if ($report) {
        print STDOUT '--- '.$script.' found (line='.$linefound.' of '.$motorlist.'):'."\n";
        print STDOUT '     assy='.$assy."\n";
        print STDOUT '     axis='.$axis."\n";
        print STDOUT '  cts2egu='.$cts2egu."\n";
        print STDOUT '  formula='.$formula."\n";
        print STDOUT '   mtrOut='.$mtrOut."\n";
        print STDOUT ' struckPs='.$struckPs."\n";
        print STDOUT ' struckNg='.$struckNg."\n";
     }
     return $cts2egu,$formula,$mtrOut,$struckPs,$struckNg;
  }
  else {
     print STDOUT '*** '.$script.': no "gmca-1" record in '.$motorlist
                 .' for assy='.$assy.' & axis='.$axis."\n\n";
     return;
  }
}

#=================================================================

# ($cts2egu,$mtr,$encAddr) = &searchMotorList2($assy,$axis);
# sub searchMotorList($assy,$axis) {
sub searchMotorList2($$) {

### This sub searches file motorList.dat from the fast scans package
### and returns all parameters of a respective assy and axis. Only
### "gmca-2" records (records for motors that can be scanned with a PLC
### advancing the Struck scaler at fixed encoder increments) are processed.


  our ($report, $beamline);
  my ($script, $assy, $axis, $i, $fastscans, $recType);
  my ($motorlist, $handle, $line, $linefound, $record, @arr, $narr);
  my ($as, $ax, $cts2egu, $mtr, $encAddr);

  $script  = 'searchMotorList';
  $recType = 'gmca-2';
  $assy = shift(@_);	$assy =~ s/:$//;			#here we need it for legacy calls
  $axis = shift(@_);	$axis =~ s/:$//;			#here we need it for legacy calls
  if (!defined $report) {$report = 0;}

### Remove 23i,23o,23b, or 23d:					<<==== GMCA SPECIFIC!
  if ($assy =~ /^23[a-z]{1}:/i) {$assy =~ s/^23[a-z]{1}://i};
  if ($axis =~ /^23[a-z]{1}:/i) {$axis =~ s/^23[a-z]{1}://i};

  if (!defined $ENV{FASTSCANS}) {
     print STDOUT '*** '.$script.': Environment FASTSCANS is undefined'."\n\n";
     return;
  }
  $fastscans = $ENV{FASTSCANS}; $fastscans =~ s/\\/\//g; $fastscans =~ s/\/$//;
  $fastscans =~ s/\/bin$//;
  $motorlist = $fastscans.'/data/motorList_'.$beamline.'.dat';
  print STDOUT 'Opening DB file: '.$motorlist.' for assy='.$assy.' & axis='.$axis."\n";
  $handle = open (ML, '< '.$motorlist);
  if (!defined $handle) {
     print STDOUT '*** '.$script.': Error opening '.$motorlist."\n";
     return;
  }
  select ML; $|=1;
  $line=0;
### Record structure:
###  gmca-1    WBS_Vcenter   WS:  Av: ZC:   5.E-5    mz:     $350A
### $recordType    $key    $group $as $ax $cts2egu $mtrOut $encAddr

  while (<ML>) {
     $record = $_;
     chomp $record;						#trim CR
     $line++;
     if ($record !~ /^#/ && $record !~ /^end/i) {
        @arr = split " ", $record;
        $narr = @arr;
        if ($arr[0] ne $recType) {next;}
        if ($narr != 8) {
           print STDOUT '*** '.$script.': Record length='.$narr.' is not 10 as expected'."\n";
           print STDOUT '*** '.$script.': ... at line='.$line.' of '.$motorlist."\n";
           print STDOUT '*** '.$script.': Record split results:'."\n";
           for ($i=0; $i<$narr; $i++) {print STDOUT $arr[$i]."\n";}
           return;
        }
### ($recordType,$key,$group,$as,$ax,$cts2egu,$formula,$mtrOut,$struckPs,$struckNg) = @arr;
###      0         1     2    3   4      5        6       7        8         9
        $as = $arr[2].$arr[3];  $as =~ s/:$//;			#remove ":" at the end of line
        $ax = $arr[2].$arr[4];  $ax =~ s/:$//;			#remove ":" at the end of line
        if (($assy eq $as) && ($axis eq $ax)) {
           if (!defined $linefound) {
              $linefound = $line;
              ($cts2egu,$mtr,$encAddr) = ($arr[5],$arr[2].$arr[6],$arr[7]);
              $mtr =~ s/:$//;					#remove ":" at the end of line
              if ($encAddr !~ /^\$/) {
                 print STDOUT '*** '.$script.': wrong encoder address in record "'.$record.'"'."\n\n";
                 print STDOUT '*** '.$script.': ... at line='.$line.' of '.$motorlist."\n";
                 return;
              }
           } else {
              print STDOUT '*** '.$script.': duplicate data for assy='.$assy.' & axis='.$axis."\n";
              print STDOUT '*** '.$script.': ... at lines '.$linefound.' & '.$line.' of '.$motorlist."\n";
              return;
           }
        }
     }
  }
  close(ML);
  if (defined $linefound) {
     if ($report) {
        print STDOUT '--- '.$script.' found (line='.$linefound.' of '.$motorlist.'):'."\n";
        print STDOUT '     assy='.$assy."\n";
        print STDOUT '     axis='.$axis."\n";
        print STDOUT '  cts2egu='.$cts2egu."\n";
        print STDOUT '      mtr='.$mtr."\n";
        print STDOUT '  encAddr='.$encAddr."\n";
     }
     return $cts2egu,$mtr,$encAddr;
  }
  else {
     print STDOUT '*** '.$script.': no "'.$recType.'" record in '.$motorlist
                        .' for assy='.$assy.' & axis='.$axis."\n\n";
     return;
  }
}

#=================================================================

# sub getESinTheta($assy,[\$status]) {
sub getESinTheta($;\$) {

### This routine returns ESinTheta for monochromator assembly:

  my ($script, $assy, $statref, $status, $ESinTheta);

  $script  = 'getESinTheta';
  $assy    = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $statref = shift(@_);

### Get ESinTheta:
  ($status,$ESinTheta) = Pezca::GetDouble($assy.'ESinTheta');
  if ($status) {
     &report_exit($script, $status, $assy.'ESinTheta');
     $ESinTheta = 0.;
  }
  if (defined $statref) {$$statref = $status;}
  return $ESinTheta;
}

#=================================================================

# sub E2theta($assy,$energy,[$ESinTheta]) {
sub E2theta($$;$) {

### This routine recalculates Energy into Theta for monochromator assembly

  my ($script, $assy, $energy, $ESinTheta, $sinTheta, $theta, $rad2degree);

  $script    = 'E2theta';
  $assy      = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $energy    = shift(@_);
  $ESinTheta = shift(@_);
### If ESinTheta is not passed, then get it:
  if (!defined $ESinTheta) {$ESinTheta = &getESinTheta($assy);}

  $rad2degree = 45./atan2(1.,1.);

  if ($energy > 0.) {$sinTheta = $ESinTheta / $energy;}
  else              {$sinTheta = 1.0;}

  if    ($sinTheta > 1.) {$theta = 90.;}
  elsif ($sinTheta < 0.) {$theta = 0.;}
  else                   {$theta = &asinSS($sinTheta) * $rad2degree;}
# else                   {$theta = rad2deg(&asinSS($sinTheta));}	#Math::Trig
# else                   {$theta = &rad2degrees(&asinSS($sinTheta));}	#common_subs.pl

# print STDOUT '-- '.$script.': energy='.$energy.' theta='.$theta."\n";
  return $theta;
}

#=================================================================

# sub theta2E($assy,$theta,[$ESinTheta]) {
sub theta2E($$;$) {

### This routine recalculates Theta into Energy for monochromator assembly

  my ($script, $assy, $theta, $ESinTheta, $energy);

  $script    = 'theta2E';
  $assy      = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $theta     = shift(@_);
  $ESinTheta = shift(@_);
### If ESinTheta is not passed, then get it:
  if (!defined $ESinTheta) {$ESinTheta = &getESinTheta($assy);}

# my $degree2rad = atan2(1.,1.)/45.;

  $energy = 1E+10;				#some big number

# if ($theta != 0.) {$energy = $ESinTheta / sin($theta*$degree2rad);}
# if ($theta != 0.) {$energy = $ESinTheta / sin(deg2rad($theta));}	#needs Math::Trig
  if ($theta != 0.) {$energy = $ESinTheta / sin(&degrees2rad($theta));}	#see common_subs.pl

# print STDOUT '-- '.$script.': theta='.$theta.' energy='.$energy."\n";
  return $energy;
}

#=================================================================

# sub L2theta($assy,$lambda,[$ESinTheta]) {
sub L2theta($$;$) {

### This routine recalculates Lambda into Theta for monochromator assembly

  my ($script, $assy, $lambda, $ESinTheta, $EvLambda, $rad2degree);
  my ($sinTheta, $theta);

  $script    = 'L2theta';
  $assy      = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $lambda    = shift(@_);
  $ESinTheta = shift(@_);
### If ESinTheta is not passed, then get it:
  if (!defined $ESinTheta) {$ESinTheta = &getESinTheta($assy);}

  $EvLambda = 12.398541;

  $rad2degree = 45./atan2(1.,1.);

  $sinTheta = ($ESinTheta/$EvLambda) * $lambda;

  if    ($sinTheta > 1.) {$theta = 90.;}
  elsif ($sinTheta < 0.) {$theta = 0.;}
  else                   {$theta = &asinSS($sinTheta) * $rad2degree;}
# else                   {$theta = rad2deg(&asinSS($sinTheta));}	#Math:Trig
# else                   {$theta = &rad2degrees(&asinSS($sinTheta));}	#common_subs.pl

# print STDOUT '-- '.$script.': lambda='.$lambda.' theta='.$theta."\n";
  return $theta;
}

#=================================================================

# sub theta2L($assy,$theta,[$ESinTheta]) {
sub theta2L($$;$) {

### This routine recalculates Theta into Energy for monochromator assembly

  my ($script, $assy, $theta, $ESinTheta, $EvLambda, $lambda);

  $script    = 'theta2L';
  $assy      = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $theta     = shift(@_);
  $ESinTheta = shift(@_);
### If ESinTheta is not passed, then get it:
  if (!defined $ESinTheta) {$ESinTheta = &getESinTheta($assy);}

  $EvLambda = 12.398541;

# my $degree2rad = atan2(1.,1.)/45.;

# $lambda = ($EvLambda/$ESinTheta) * sin($theta*$degree2rad);
# $lambda = ($EvLambda/$ESinTheta) * sin(deg2rad($theta));              #needs Math::Trig
  $lambda = ($EvLambda/$ESinTheta) * sin(&degrees2rad($theta));         #see common_subs.pl

# print STDOUT '-- '.$script.': theta='.$theta.' lambda='.$lambda."\n";
  return $lambda;
}

#=======================================================================

# sub generateAssy2Pcs ($beamline,\%assy2pcs) {			#fills hash with all: $assy->$pcs
  sub generateAssy2Pcs ($\%) {
#-----
# This sub parses config${beamline}.tcl in the respective TCL subdirectory of
# $epicsClientDir and fills in the %assy2pcs as a hash with $assy as keywords
# and PCS numbers as values.
# The function return value is the number of found assemblies
# ($nAssy) or -1 in case of error.
#-----
  our $PEZCA;				#make GLOBAL so that one can override
  my ($script, $bl, $hashRef, $cat, $epicsClientDir);
  my ($configtcl, $cfghandle, $str, $s, @arr);
  my ($i, $pcs, $assy, $nassy, $n);

  $script  = 'generateAssy2Pcs';
  $bl      = shift(@_);
  $hashRef = shift(@_);

  if (!defined $PEZCA) {
     if    (defined $ENV{PEZCA})     {$PEZCA = $ENV{PEZCA};}
     elsif (defined $ENV{FASTSCANS}) {$PEZCA = $ENV{FASTSCANS}; $PEZCA =~ s/fastscans$/pezca/i;}
     else                            {$PEZCA = '/unsupported/pezca';}
  }
  $epicsClientDir = $PEZCA;
  $epicsClientDir =~ s/\\/\//g;					#change \ to / (e.g. on Windows)
  $epicsClientDir =~ s/\/pezca\/?$//;				#remove /pezca

  if    ($bl =~ /^23/) {$cat = 'gmca';}
  elsif ($bl =~ /^21/) {$cat = 'ls';}
  elsif ($bl =~ /^18/) {$cat = 'bio';}
  else {
     print STDOUT '*** '.$script.': unsupported beamline='.$bl."\n";
     return -1;
  }
  $bl =~ s/:$//;
  $configtcl = $epicsClientDir.'/'.$cat.'App/tcl/config'.$bl.'.tcl';
  print STDOUT 'Reading IocPmacList from: '.$configtcl."\n";
  $cfghandle = open (CFG, '< '.$configtcl);
  if (!defined $cfghandle) {
    print STDOUT '*** '.$script.': Could not open '.$configtcl.' for reading.'."\n";
    return -1;
  }
# select CFG; $|=1;

### Read the part of config till 'set IocPmacList':
  $bl .= ':';
  while (<CFG>) {
     chomp ($str = $_);				#read line and trail CR
     $str =~ s/\#.*$//g;			#trail comments started by '#' ('.' matches any character, '*' means match 0 or more times, '$' matches end of string)
     $str =~ s/\s+/ /g;				#compress spaces ans tabs ('+' means match 1 or more times)
     $str =~ s/^\s+//;				#remove beginning-of-the-line spaces
     $str =~ s/\s+$//;				#remove end-of-the-line spaces
     if ($str =~ /^set IocPmacList/) {last;}
  }
  if ($str !~ /^set IocPmacList/) {
     print STDOUT '*** '.$script.': No IocPmacList in '.$configtcl."\n";
     close(CFG);
     return -1;
  }
### Now read IocPmacList:
# @list = ();
  $s = '';
  while (<CFG>) {
     chomp ($str = $_);				#read line and trail CR
     $str =~ s/\#.*$//g;			#trail comments started by '#' ('.' matches any character, '*' means match 0 or more times, '$' matches end of string)
     $str =~ s/\s+/ /g;				#compress spaces and tabs ('+' means match 1 or more times)
     $str =~ s/^\s+//;				#remove beginning-of-the-line spaces
     $str =~ s/\s+$//;				#remove end-of-the-line spaces
     if (length($str) == 0) {next;}		#if non-empty string
     if ($str =~ /regsub /) {next;}
     if ($str =~ /^set/) {last;}		#Next list is starting
     $s .= $str;				#merge all parts of the list
  }
  close(CFG);
  $s =~ s/^\s+//;				#remove again beginning-of-the-line spaces
  $s =~ s/\s+$//;				#remove again end-of-the-line spaces
  $s =~ s/ioc[a-z0-9]+://g;			#remove iocgmcaN
  $s =~ s/pmac[0-9]+://g;			#remove pmacN:
  $s =~ s/( ?{ ?)+/{/g;				#replace all multiple {{{ with {
  $s =~ s/( ?} ?)+/}/g;				#replace all multiple }}} with }
### Convert the Tcl list into nested Perl array structure
  $s =~ s/([^{}\s]+)/'$1',/g;			#quote words + comma
  $s =~ s/{/[/g;				#start list
  $s =~ s/}/],/g;				#end list + comma
  @arr = eval($s);				#creates nested Perl array (should be array of $nassy arrays each containing 2 elements)
  $nassy = @arr;
  if ($nassy == 0) {
     print STDOUT '*** '.$script.': No assemblies found in '.$configtcl."\n";
     return -1;
  }
  for ($i=0; $i<$nassy; $i++) {
     if (ref(${arr[$i]}) ne 'ARRAY') {
        print STDOUT '*** '.$script.': element '.$i.' of IocPmacList is not an array in line=['.$s.'] in '.$configtcl."\n";
        close(CFG);
        return -1;
     }
     $n = @{$arr[$i]};
     if ($n != 2) {
        print STDOUT '*** '.$script.': array-'.$i.' of IocPmacList contains '.$n.' elements in line=['.$s.'] in '.$configtcl."\n";
        close(CFG);
        return -1;
     }
     $pcs  =     ${arr[$i]}[0];
     $assy = $bl.${arr[$i]}[1];
     $$hashRef{$assy} = $pcs;			#return as hash
#    $list[$i]{$assy} = $pcs;			#return as array
  }
# @$hashRef = @list;				#return as array
  return $nassy;
}

#=======================================================================

# sub generateComponentList ($beamline,\@list) {
  sub generateComponentList ($\@) {
#-----
# This sub parses config${beamline}.tcl in the respective TCL subdirectory of
# $epicsClientDir and fills in the @list as one-dimensional array of hashes
# with keywords like [label, assy, type, pmac, pcs, naxes, axis1,... ndrives, drive1,... nmotors, motor1, mtr1_no,...]
# The function return value is the number of found assemblies
# ($nAssy) or -1 in case of error.
#-----
  our $PEZCA;				#make GLOBAL so that one can override
  my ($script, $bl, $listRef, @list, %assy2pcs, $cat, $epicsClientDir);
  my ($configtcl, $cfghandle, $str, $s, @arr, $a, $narr, $nassy);
  my ($found, $i, $glabel, $group, $assy, $naxes, $ndrives, $nmotors, $p, $n);

  $script  = 'generateComponentList';
  $bl      = shift(@_);
  $listRef = shift(@_);

  if (!defined $PEZCA) {
     if    (defined $ENV{PEZCA})     {$PEZCA = $ENV{PEZCA};}
     elsif (defined $ENV{FASTSCANS}) {$PEZCA = $ENV{FASTSCANS}; $PEZCA =~ s/fastscans$/pezca/i;}
     else                            {$PEZCA = '/unsupported/pezca';}
  }
  $epicsClientDir = $PEZCA;
  $epicsClientDir =~ s/\\/\//g;			#change \ to / (e.g. on Windows)
  $epicsClientDir =~ s/\/pezca\/?$//;		#remove /pezca

  if    ($bl =~ /^23/) {$cat = 'gmca';}
  elsif ($bl =~ /^21/) {$cat = 'ls';}
  elsif ($bl =~ /^18/) {$cat = 'bio';}
  else {
     print STDOUT '*** '.$script.': unsupported beamline='.$bl."\n";
     return -1;
  }
  $bl =~ s/:$//;
  $configtcl = $epicsClientDir.'/'.$cat.'App/tcl/config'.$bl.'.tcl';
  print STDOUT 'Reading ComponentList from: '.$configtcl."\n";
  $cfghandle = open (CFG, '< '.$configtcl);
  if (!defined $cfghandle) {
    print STDOUT '*** '.$script.': Could not open '.$configtcl.' for reading.'."\n";
    return -1;
  }
# select CFG; $|=1;

### Read the part of config till 'set ComponentList':
  $bl .= ':';
  while (<CFG>) {
     chomp ($str = $_);	                        #read line and trail CR
     $str =~ s/\#.*$//g;			#trail comments started by '#' ('.' matches any character, '*' means match 0 or more times, '$' matches end of string)
     $str =~ s/\s+/ /g;			        #compress spaces ans tabs ('+' means match 1 or more times)
     $str =~ s/^\s+//;			        #remove beginning-of-the-line spaces
     $str =~ s/\s+$//;			        #remove end-of-the-line spaces
     if ($str =~ /^set ComponentList/) {last;}
  }
  if ($str !~ /^set ComponentList/) {
     print STDOUT '*** '.$script.': No ComponentList in '.$configtcl."\n";
     close(CFG);
     return -1;
  }
### Now read ComponentList:
  $found = 0;
  @list = ();
  while (<CFG>) {
     chomp ($str = $_);	                        #read line and trail CR
     $str =~ s/\#.*$//g;			#trail comments started by '#' ('.' matches any character, '*' means match 0 or more times, '$' matches end of string)
     $str =~ s/\s+/ /g;				#compress spaces and tabs ('+' means match 1 or more times)
     $str =~ s/^\s+//;				#remove beginning-of-the-line spaces
     $str =~ s/\s+$//;				#remove end-of-the-line spaces
     if ($str =~ /^set IocPmacList/) {last;}	#Next list is starting
     $str =~ s/^\{//;				#remove '{' at the beginning-of-the-line
     $str =~ s/\}$//;				#remove '}' at the end-of-the-line
     $str =~ s/^\s+//;				#remove again beginning-of-the-line spaces
     $str =~ s/\s+$//;				#remove again end-of-the-line spaces
     if (length($str) == 0) {next;}		#if non-empty string
     if ($str =~ /regsub /) {next;}
### The line may describe either group or assembly:
### In the later case it is a nested Tcl list:
### Group line (2 elements):  "Beam Stop" BS:
### Assy  line (8 elements):  "Mono Positioning" Ps: XYPs pmac10: 08XY3 {} {Z:  Y: } {{ 5 mz: } { 6 my: }}
### Convert the above Tcl list (nested in the second case) into nested Perl array structure"
     $s = $str;
     $s =~ s/([^{}\s]+)/'$1',/g;		#quote words + comma
     $s =~ s/{/[/g;				#start list
     $s =~ s/}/],/g;				#end list + comma
     @arr = eval($s);				#creates nested Perl array (should not be more than 3 levels)
     $narr = @arr;
     if ($arr[0] =~ /^\"/ && $arr[0] !~ /\"$/) {	#if label is like "a b c", it is split on several array elements; we merge them here
        for ($i=1; $i<$narr; $i++) {
           $a = $arr[$i];
           $arr[0] = join(' ',$arr[0],$a);
           splice @arr,$i,1;
           $i--;
           $narr--;
           if ($a =~ /\"$/) {last;}
        }
     }
     $arr[0] =~ s/\"//g;				#remove quotes from label
     if ($narr != 2 && $narr != 8) {
        print STDOUT '*** '.$script.': unexpected line=['.$str.' in '.$configtcl."\n";
        close(CFG);
        return -1;
     }
     if ($narr == 2) {					#this line describes group, e.g. BS:
        $glabel = $arr[0];  $glabel =~ s/\s+$//;
        $group  = $arr[1];  $group  =~ s/\s+$//;
     }
     elsif ($narr == 8) {				#this line describes assembly
        if (!defined $glabel || !defined $group) {
           print STDOUT '*** '.$script.': undefined group for line=['.$str.'] in '.$configtcl."\n";
           close(CFG);
           return -1;
        }
        $arr[0] =~ s/\s+$//;
        $list[$found]{'label'} = $glabel.', '.$arr[0];
        $assy = $bl.$group.$arr[1];
        $list[$found]{'assy'}  = $assy;			#e.g. BS:Ps:
        $list[$found]{'type'}  = $arr[2];		#e.g. XyPS
        $list[$found]{'pmac'}  = $arr[3];		#e.g. pmac10:
        $list[$found]{'mem'}   = $arr[4];		#e.g. 08XY3
#       $list[$found]{'pcs'}   = &getPcs($assy);	#e.g. 14	#this is requested via EPICS, not from configNNN.tcl
        
        if (ref(${arr[5]}) ne 'ARRAY') {
           print STDOUT '*** '.$script.': axes are not an array in line=['.$str.'] in '.$configtcl."\n";
           close(CFG);
           return -1;
        }
        $naxes = @{$arr[5]};
#       print STDOUT 'assy='.$assy.' naxes='.$naxes."\n";
        $list[$found]{'naxes'} = $naxes;
        for ($i=0; $i<$naxes; $i++) {
           $list[$found]{'axis'.($i+1)} = $bl.$group.${arr[5]}[$i];
        }
        if (ref(${arr[6]}) ne 'ARRAY') {
           print STDOUT '*** '.$script.': drives are not an array in line=['.$str.'] in '.$configtcl."\n";
           close(CFG);
           return -1;
        }
        $ndrives = @{$arr[6]};
#       print STDOUT 'assy='.$assy.' ndrives='.$ndrives."\n";
        $list[$found]{'ndrives'} = $ndrives;
        for ($i=0; $i<$ndrives; $i++) {
           $list[$found]{'drive'.($i+1)} = $bl.$group.${arr[6]}[$i];
        }
        if (ref($arr[7]) ne 'ARRAY') {
           print STDOUT '*** '.$script.': motors+mtrNo are not an array'
                            .' in line=['.$str.'] in '.$configtcl."\n";
           close(CFG);
           return -1;
        }
        $nmotors = @{$arr[7]};					#this should be reference to an array with $nmotors elements, e.g. {{ 5 mz: } { 6 my: }}
#       print STDOUT 'assy='.$assy.' nmotors='.$nmotors."\n";
        if ($nmotors != $ndrives) {
           printf STDOUT '*** '.$script.': incorrect nmotors=%d'
                             .' (expecting %d) in line=['.$str.'] in '.
                             $configtcl."\n", $nmotors, $ndrives;
           close(CFG);
           return -1;
        }
        $list[$found]{'nmotors'} = $nmotors;
        for ($i=0; $i<$nmotors; $i++) {
           $p = ${arr[7]}[$i];					#this is reference to single motor-ntrNo pair
           if (ref($p) ne 'ARRAY') {
              print STDOUT '*** '.$script.': motors-'.$i.' is not an array'
                               .' in line=['.$str.'] in '.$configtcl."\n";
              close(CFG);
              return -1;
           }
           $list[$found]{'mtr'.($i+1).'_no'} =           $$p[0];
           $list[$found]{'motor'.($i+1)}     = $bl.$group.$$p[1];
        }
        $found++;
     }   
  }
  close(CFG);
  if ($found == 0) {
     print STDOUT '*** '.$script.': No assemblies found in '.$configtcl."\n";
     return -1;
  }  
  $nassy = &generateAssy2Pcs($bl,\%assy2pcs);
  for ($i=0; $i<$found; $i++) {
     $assy = $list[$i]{'assy'};
#    @arr = grep defined, map {$_->{$assy}} @assy2pcs;
#    $narr = @arr;
#    if ($narr != 1) {
#       print STDOUT '*** '.$script.': found '.$narr.' PCS for assy='
#                        .$assy.' in '.$configtcl."\n"; 
#       return -1;
#    }
#    $list[$i]{'pcs'} = $arr[0];
     if (!defined $assy2pcs{$assy}) {
        print STDOUT '*** '.$script.': no PCS data for assy='.$assy
                         .' in '.$configtcl."\n"; 
        return -1;
     }
     $list[$i]{'pcs'} = $assy2pcs{$assy};
  }
 
  @$listRef = @list;
  return $found;
}

#=======================================================================

# sub generateAssyList ($beamline,\@pvList,['returnflag']) {
  sub generateAssyList ($\@;$) {
#-----
# This sub uses data from config${beamline}.tcl parsed by generateComponentList 
# to fill in the pvList as one-dimensional array of strings, each consisting of
# assy name followed by axes names (as many as may exist) separated by spaces.
# The function return value is the number of found assemblies ($nAssy) or -1 in
# case of error.
# The calling program will need to parse each array element as:
# for ($i=0; $i<$nAssy; $i++) {($assy, @axes) = split ' ', $pvList[$i];}
# or:
# foreach $string (@pvList) {($assy, @axes) = split ' ', $string;}
#-----
  my ($script, $bl, $pvListRef, $returnFlag);
  my (@hash, $nassy, $naxes, $ndrives, $axis, $i, $j, $m);

  $script     = 'generateAssyList';
  $bl         = shift;
  $pvListRef  = shift;
  $returnFlag = shift;		if (!defined $returnFlag) {$returnFlag = 'assy+axes';}
  if ($returnFlag !~ /^assyOnly$/i) {$returnFlag = 'assy+axes';}

### Read ComponentList:
  $nassy = &generateComponentList ($bl,\@hash);
  if ($nassy == 0) {print STDOUT '*** '.$script.': No assemblies found.'."\n"; return -1;}
  for ($i=0; $i<$nassy; $i++) {
     @$pvListRef[$i] = $hash[$i]->{'assy'};
     if ($returnFlag !~ /^assyOnly$/i) {
        $naxes  = $hash[$i]->{'naxes'};
        $ndrives = $hash[$i]->{'ndrives'};
        if ($naxes > 0) {
           for ($j=0; $j<$naxes; $j++) {
              $m = $j+1;
              $axis = $hash[$i]->{'axis'.$m};
              @$pvListRef[$i] .= ' '.$axis;
           }
        } else {
           for ($j=0; $j<$ndrives; $j++) {
              $m = $j+1;
              $axis = $hash[$i]->{'drive'.$m};
              @$pvListRef[$i] .= ' '.$axis;
           }
        }
     }
  }
  return $nassy;
}

#=======================================================================

# sub getAssyList4Pmac ($beamline,$pmac,\@assyNames,\@assyNums) {
  sub getAssyList4Pmac ($$\@) {
#-----
# This sub uses data from config${beamline}.tcl parsed by generateComponentList
# to find lists of assemblies and their PCS for given $pmac.
# The function also returns $status: 0=OK, everything else is an error.
#-----
  my ($script, $bl, $pmac, $assyNamesRef, $assyNumsRef);
  my ($nassy, @hash, $i, @index);

  $script       = 'getAssyList4Pmac';
  $bl           = shift(@_);					#input
  $pmac         = shift(@_);					#input
  $assyNamesRef = shift(@_);					#output
  $assyNumsRef  = shift(@_);					#output

  @$assyNamesRef  = ();
  @$assyNumsRef   = ();

### Separate possible beamline and group from $assy:
  $bl   =~ s/:$//;				#trim ending ':'
  $pmac =~ s/^$bl://;
  if ($pmac !~ /:$/) {$pmac .= ':';}

  $nassy = &generateComponentList ($bl,\@hash);
  if ($nassy == 0) {
     print STDOUT '*** '.$script.': No assemblies found for beamline='.$bl."\n"; 
     return -1;
  }
  for ($i=0; $i<$nassy; $i++) {
     if ($hash[$i]->{'pmac'} eq $pmac) {
        push(@$assyNamesRef,$hash[$i]->{'assy'});
        push(@$assyNumsRef,$hash[$i]->{'pcs'});
     }             
  }
  if (@$assyNamesRef == 0) {
     print STDOUT '*** '.$script.': No assemblies found for pmac='.$pmac."\n";
     return -1;
  }
### Sort by PCS:
  @index = 0..$#$assyNumsRef;
  @index = sort { $$assyNumsRef[$a] <=> $$assyNumsRef[$b] } @index;
  @$assyNumsRef  = @$assyNumsRef[@index];
  @$assyNamesRef = @$assyNamesRef[@index];
  return 0;
}

#=======================================================================

# sub searchAssyList ($beamline,$assy,\@axes) {			#returns \@axes and $status for given $assy
  sub searchAssyList ($$\@) {
#-----
# This sub parses config${beamline}.tcl in the respective TCL subdirectory of
# $epicsClientDir and finds all axes for given assembly ($assy).
#-----
  my ($script, $bl, $assy, $axsRef);
  my (@hash, $nassy, $naxes, $ndrives, $i, $j, $m);

  $script = 'searchAssyList';
  $bl     = shift;
  $assy   = shift;
  $axsRef = shift;

  $bl   =~ s/:$//;						#trim ending ':'
  if ($assy !~ /^${bl}/) {$assy  = $bl.':'.$assy;}  
  if ($assy !~ /:$/) {$assy .= ':';}

  $nassy = &generateComponentList ($bl,\@hash);
  if ($nassy == 0) {
     print STDOUT '*** '.$script.': No assemblies found.'."\n"; 
     return -1;
  }
  for ($i=0; $i<$nassy; $i++) {
     if ($hash[$i]->{'assy'} eq $assy) {
        $naxes    = $hash[$i]->{'naxes'};
        $ndrives  = $hash[$i]->{'ndrives'};
        if ($naxes > 0) {
           for ($j=0; $j<$naxes;   $j++) {$m=$j+1;  @$axsRef[$j]=$hash[$i]->{'axis'.$m};}
        } else {
           for ($j=0; $j<$ndrives; $j++) {$m=$j+1;  @$axsRef[$j]=$hash[$i]->{'drive'.$m};}
        }
        return 0;
     }
  }
  print STDOUT '*** '.$script.': No data in TCL config for assy='.$assy."\n";
  return -1;
}

#=======================================================================

# sub searchConfigTcl ($beamline,$assy,$axis,
#                      \$pmac,\$axisOrDrive,\$axisOrDriveIndex,
#                      \@axes,\@drives,\@motors,\@mtNumbers) {
  sub searchConfigTcl ($$$\$\$\$\@\@\@\@) {
#-----
# This sub uses data from config${beamline}.tcl parsed by generateComponentList
# and returns ($pmac,@axes,@drives,@motors,@mtrNumbers) for given $assy and $axis.
# The function also returns $status: 0=OK, everything else is an error.
#-----
  my ($script, $bl, $assy, $axis, $pmacRef, $aodRef);
  my ($indxRef, $axsRef, $drvRef, $mtrRef, $numRef, $grp);
  my (@hash, $nassy, $naxes, $ndrives, $nmotors, $i, $j, $m);

  $script    = 'searchConfigTcl';
  $bl        = shift(@_);					#input
  $assy      = shift(@_);					#input
  $axis      = shift(@_);					#input
  $pmacRef   = shift(@_);					#output
  $aodRef    = shift(@_);					#output
  $indxRef   = shift(@_);					#output
  $axsRef    = shift(@_);					#output
  $drvRef    = shift(@_);					#output
  $mtrRef    = shift(@_);					#output
  $numRef    = shift(@_);					#output

  $$pmacRef = '';
  $$aodRef  = '';
  $$indxRef = '';
  @$axsRef  = ();
  @$drvRef  = ();
  @$mtrRef  = ();
  @$numRef  = ();

  $bl   =~ s/:$//;						#trim ending ':'
  if ($assy !~ /^${bl}/) {$assy  = $bl.':'.$assy;}  if ($assy !~ /:$/) {$assy .= ':';}
  if ($axis !~ /^${bl}/) {$axis  = $bl.':'.$axis;}  if ($axis !~ /:$/) {$axis .= ':';}

  $nassy = &generateComponentList ($bl,\@hash);
  if ($nassy == 0) {
     print STDOUT '*** '.$script.': No assemblies found.'."\n";
     return -1;
  }
  for ($i=0; $i<$nassy; $i++) {
     if ($hash[$i]->{'assy'} eq $assy) {
        $$pmacRef = $hash[$i]->{'pmac'};
        $naxes    = $hash[$i]->{'naxes'};
        $ndrives  = $hash[$i]->{'ndrives'};
        $nmotors  = $hash[$i]->{'nmotors'};
        for ($j=0; $j<$naxes;   $j++) {
           $m = $j+1;  
           @$axsRef[$j] = $hash[$i]->{'axis'.$m};
           if (@$axsRef[$j] eq $axis) {
              if (length($$aodRef) == 0) {
                 $$aodRef = 1;
                 $$indxRef = $j;
              } else {
                print STDOUT '*** '.$script.': duplicate TCL config data'
                                 .' for axis='.$axis.', assy='.$assy."\n";
                return -1;
              }
           }
        }
        for ($j=0; $j<$ndrives; $j++) {
           $m = $j+1;  
           @$drvRef[$j] = $hash[$i]->{'drive'.$m};
           if (@$drvRef[$j] eq $axis) {
              if (length($$aodRef) == 0) {
                 $$aodRef = 0;
                 $$indxRef = $j;
              } else {
                print STDOUT '*** '.$script.': duplicate TCL config data'
                                 .' for axis='.$axis.', assy='.$assy."\n";
                return -1;
              }
           }
        }
        for ($j=0; $j<$nmotors; $j++) {
           $m = $j+1;  
           @$mtrRef[$j] = $hash[$i]->{'motor'.$m};
           @$numRef[$j] = $hash[$i]->{'mtr'.$m.'_no'};
        }
        return 0;
     }
  }
  print STDOUT '*** '.$script.': No data in TCL config'
                   .' for assy='.$assy.', axis='.$axis."\n";
  return -1;
}

#=======================================================================

# sub getAssy($motor,\$status) {				#returns assy name
sub getAssy($;\$) {
#-----
# This sub uses data from config${beamline}.tcl parsed by generateComponentList
# and returns $assy for given $axis or $drive, or $motor
# The function also returns $status: 0=OK, everything else is an error.
#-----
  my ($script, $motor, $statusRef, $status, $bl, $assy);
  my (@hash, $nassy, $naxes, $ndrives, $nmotors, $i, $j, $m);
  my (@list, $nlist);

  $script    = 'getAssy';
  $motor     = shift(@_)	;				#input
  $statusRef = shift(@_);					#output

  $bl = substr($motor,0,3);
  if ($bl !~ /^23[iobd]$/) {
     print STDOUT '*** '.$script.': incorrect beamline='.$bl."\n";
     if (defined $statusRef) {$$statusRef=-1;}
     return undef;
  }

  $nassy = &generateComponentList ($bl,\@hash);
  if ($nassy == 0) {
     print STDOUT '*** '.$script.': No assemblies found for beamline='.$bl."\n"; 
     if (defined $statusRef) {$$statusRef=-1;}
     return undef;
  }
  @list = ();
  if ($motor !~ /:$/) {$motor .= ':';}
ASSY:
  for ($i=0; $i<$nassy; $i++) {
     $naxes    = $hash[$i]->{'naxes'};
     $ndrives  = $hash[$i]->{'ndrives'};
     $nmotors  = $hash[$i]->{'nmotors'};
     for ($j=0; $j<$naxes; $j++) {
        $m = $j+1;  
#       print STDOUT $hash[$i]->{'axis'.$m},"\n";
        if ($motor eq $hash[$i]->{'axis'.$m}) {
           push(@list,$hash[$i]->{'assy'});
           next ASSY;
        }
     }
     for ($j=0; $j<$ndrives; $j++) {
        $m = $j+1;  
#       print STDOUT $hash[$i]->{'drive'.$m},"\n";
        if ($motor eq $hash[$i]->{'drive'.$m}) {
           push(@list,$hash[$i]->{'assy'});
           next ASSY;
        }
     }
     for ($j=0; $j<$nmotors; $j++) {
        $m = $j+1;  
#       print STDOUT $hash[$i]->{'motor'.$m},"\n";
        if ($motor eq $hash[$i]->{'motor'.$m}) {
           push(@list,$hash[$i]->{'assy'});
           next ASSY;
        }
     }
  }
  $nlist = @list;
  if ($nlist == 0) {
     print STDOUT '*** '.$script.': No data in TCL config'
                      .' for motor/drive/axis='.$motor."\n";
     if (defined $statusRef) {$$statusRef=-1;}
     return undef;
  }
  elsif ($nlist > 1) {
     print STDOUT '*** '.$script.': Duplicate data in TCL config'
                      .' for motor/drive/axis='.$motor.": @list\n";
     if (defined $statusRef) {$$statusRef=-1;}
     return undef;
  }
  else {
     if (defined $statusRef) {$$statusRef=0;}
     return $list[0];
  }
}

#=======================================================================

# sub getAssyDataFromConfigTcl ($beamline,$assy,
#                        \$pmac,\@axes,\@drives,\@motors,\@mtNumbers) {
  sub getAssyDataFromConfigTcl ($$\$\@\@\@\@) {
#-----
# This sub uses data from config${beamline}.tcl parsed by generateComponentList
# and for given $assy returns ($pmac,@axes,@drives,@motors,@mtrNumbers).
# The function also returns $status: 0=OK, everything else is an error.
#-----
  my ($script, $bl, $assy, $pmacRef, $axsRef, $drvRef, $mtrRef, $numRef);
  my (@hash, $nassy, $naxes, $ndrives, $nmotors, $i, $j, $m);

  $script    = 'getAssyDataFromConfigTcl';
  $bl        = shift(@_);					#input
  $assy      = shift(@_);					#input
  $pmacRef   = shift(@_);					#output
  $axsRef    = shift(@_);					#output
  $drvRef    = shift(@_);					#output
  $mtrRef    = shift(@_);					#output
  $numRef    = shift(@_);					#output

  $$pmacRef = '';
  @$axsRef  = ();
  @$drvRef  = ();
  @$mtrRef  = ();
  @$numRef  = ();

  $bl   =~ s/:$//;						#trim ending ':'
  if ($assy !~ /^${bl}/) {$assy  = $bl.':'.$assy;}
  if ($assy !~ /:$/)     {$assy .= ':';}
  
  $nassy = &generateComponentList ($bl,\@hash);
  if ($nassy == 0) {
     print STDOUT '*** '.$script.': No assemblies found.'."\n"; 
     return -1;
  }
  for ($i=0; $i<$nassy; $i++) {
     if ($hash[$i]->{'assy'} eq $assy) {
        $$pmacRef = $hash[$i]->{'pmac'};
        $naxes    = $hash[$i]->{'naxes'};
        $ndrives  = $hash[$i]->{'ndrives'};
        $nmotors  = $hash[$i]->{'nmotors'};
        for ($j=0; $j<$naxes;   $j++) {$m=$j+1;  @$axsRef[$j]=$hash[$i]->{'axis'.$m};}
        for ($j=0; $j<$ndrives; $j++) {$m=$j+1;  @$drvRef[$j]=$hash[$i]->{'drive'.$m};}
        for ($j=0; $j<$nmotors; $j++) {$m=$j+1;  @$mtrRef[$j]=$hash[$i]->{'motor'.$m};
                                                 @$numRef[$j]=$hash[$i]->{'mtr'.$m.'_no'};}
        return 0;
     }
  }
  print STDOUT '*** '.$script.': No data in TCL config for assy='.$assy."\n";
  return -1;
}


#=================================================================

# sub getAssyDataFromTsub($assy,\@axes,\@drives,\@motors) {	#returns $status
sub getAssyDataFromTsub ($\@\@\@) {

### This routine reads assembly tsub records to find all
### axes, drives and motors which belong to the assembly

  my ($script, $assy, $axs_ref, $drv_ref, $mtr_ref, $status);
  my ($val, $tsub, $pv, $str, @inputs, $i, $d, @words, $nwords);

  $script  = 'getAssyDataFromTsub';
  $assy    = shift(@_);						#input: EPICS name of motors assembly
  $axs_ref = shift(@_);						#output: pointer to axes array to be filled
  $drv_ref = shift(@_);						#output: pointer to drives array to be filled
  $mtr_ref = shift(@_);						#output: pointer to motors array to be filled

  @$axs_ref = ();
  @$drv_ref = ();
  @$mtr_ref = ();
  $assy   = &filterAssy($assy);
  $val    = ':RqsPos';
  $tsub   = $assy.$val.'TsubMtr';				#tsub record name
  @inputs = qw(A B C D);

### Loop over 0=axes, 1=drives, 2=motors:
  for ($d=0; $d<3; $d++) {
INPUTS:
     foreach $i (@inputs) {
        if    ($d == 0) {$pv = $tsub.'.OUT'.$i;}		#axes
        elsif ($d == 1) {$pv = $tsub.'.OT'.$i.'1';}		#drives
        elsif ($d == 2) {$pv = $tsub.'.INP'.$i;}		#motors
        ($status,$str) = Pezca::GetString($pv);
        if ($status) {
           &report_only($script, $status, $pv);
           return $status;
        }
        if (length($str) == 0) {last INPUTS;}
        @words = split(/\s+/,$str);
        $nwords = @words;
        if ($nwords != 3) {
           print STDOUT '*** '.$script.': unexpected number of words='.$nwords
                        .' in str=['.$str.'] contained in pv='.$pv."\n\n";
           return 10;
        }
        elsif ($words[0] !~ /${val}$/) {
#          print STDOUT '!!! '.$script.': unexpected word-1=['.$words[0]
#                       .'] in str=['.$str.'] contained in pv='.$pv."\n"
#                       .'The word should contain ['.$val.'].'."\n\n";
           last INPUTS;
        }
        elsif ($words[1] ne 'PP' && $words[1] ne 'NPP') {
           print STDOUT '*** '.$script.': unexpected word-2=['.$words[1]
                        .'] in str=['.$str.'] contained in pv='.$pv."\n"
                        .'The workd should be "PP" or "NPP"'."\n\n";
           return 11;
        }
        elsif ($words[2] ne 'MS') {
           print STDOUT '*** '.$script.': unexpected word-3=['.$words[2]
                        .'] in str=['.$str.'] contained in pv='.$pv."\n"
                        .'The word should be "MS"'."\n\n";
           return 12;
        }
        $words[0] =~ s/${val}$//;				#remove ':RqsPos' at end
        if    ($d == 0) {push(@$axs_ref,$words[0]);}		#axes
        elsif ($d == 1) {push(@$drv_ref,$words[0]);}		#drives
        elsif ($d == 2) {push(@$mtr_ref,$words[0]);}		#motors
     }
  }
  return 0;
}


#=================================================================

# sub getAssyAxesFromTsub($assy,\@axes) {			#returns $status
sub getAssyAxesFromTsub ($\@) {

### This routine reads assembly tsub records to find all
### axes which belong to the assembly

  my ($script, $assy, $axs_ref, $status);
  my ($val, $tsub, $pv, $str, @inputs, $i, @words, $nwords);

  $script  = 'getAssyAxesFromTsub';
  $assy    = shift(@_);						#input: EPICS name of motors assembly
  $axs_ref = shift(@_);						#output: pointer to axes array to be filled

  @$axs_ref = ();
  $assy   = &filterAssy($assy);
  $val    = ':RqsPos';
  $tsub   = $assy.$val.'TsubMtr';				#tsub record name
  @inputs = qw(A B C D);

  foreach $i (@inputs) {
     $pv = $tsub.'.OUT'.$i;					#axes
     ($status,$str) = Pezca::GetString($pv);
     if ($status) {&report_only($script, $status, $pv); return $status;}
     if (length($str) == 0) {last;}
     @words = split(/\s+/,$str);
     $nwords = @words;
     if ($nwords != 3) {
        print STDOUT '*** '.$script.': unexpected number of words='.$nwords
                     .' in str=['.$str.'] contained in pv='.$pv."\n\n";
        return -1;
     }
     elsif ($words[0] !~ /${val}$/) {
#       print STDOUT '!!! '.$script.': unexpected word-1=['.$words[0]
#                    .'] in str=['.$str.'] contained in pv='.$pv."\n"
#                    .'The word should contain ['.$val.'].'."\n\n";
        last;
     }
     elsif ($words[1] ne 'PP' && $words[1] ne 'NPP') {
        print STDOUT '*** '.$script.': unexpected word-2=['.$words[1]
                     .'] in str=['.$str.'] contained in pv='.$pv."\n"
                     .'The workd should be "PP" or "NPP"'."\n\n";
        return -1;
     }
     elsif ($words[2] ne 'MS') {
        print STDOUT '*** '.$script.': unexpected word-3=['.$words[2]
                     .'] in str=['.$str.'] contained in pv='.$pv."\n"
                     .'The word should be "MS"'."\n\n";
        return -1;
     }
     $words[0] =~ s/${val}$//;					#remove ':RqsPos' at end
     push(@$axs_ref,$words[0]);					#axes
  }
  return 0;
}


#=================================================================

# sub getAssyDrivesFromTsub($assy,\@drives) {			#returns $status
sub getAssyDrivesFromTsub ($\@\@\@) {

### This routine reads assembly tsub records to find all
### drives which belong to the assembly

  my ($script, $assy, $drv_ref, $status);
  my ($val, $tsub, $pv, $str, @inputs, $i, @words, $nwords);

  $script  = 'getAssyDrivesFromTsub';
  $assy    = shift(@_);						#input: EPICS name of motors assembly
  $drv_ref = shift(@_);						#output: pointer to drives array to be filled

  @$drv_ref = ();
  $assy   = &filterAssy($assy);
  $val    = ':RqsPos';
  $tsub   = $assy.$val.'TsubMtr';				#tsub record name
  @inputs = qw(A B C D);

  foreach $i (@inputs) {
     $pv = $tsub.'.OT'.$i.'1';					#drives
     ($status,$str) = Pezca::GetString($pv);
     if ($status) {&report_only($script, $status, $pv); return $status;}
     if (length($str) == 0) {last;}
     @words = split(/\s+/,$str);
     $nwords = @words;
     if ($nwords != 3) {
        print STDOUT '*** '.$script.': unexpected number of words='.$nwords
                     .' in str=['.$str.'] contained in pv='.$pv."\n\n";
        return -1;
     }
     elsif ($words[0] !~ /${val}$/) {
#       print STDOUT '!!! '.$script.': unexpected word-1=['.$words[0]
#                    .'] in str=['.$str.'] contained in pv='.$pv."\n"
#                    .'The word should contain ['.$val.'].'."\n\n";
        last;
     }
     elsif ($words[1] ne 'PP' && $words[1] ne 'NPP') {
        print STDOUT '*** '.$script.': unexpected word-2=['.$words[1]
                     .'] in str=['.$str.'] contained in pv='.$pv."\n"
                     .'The workd should be "PP" or "NPP"'."\n\n";
        return -1;
     }
     elsif ($words[2] ne 'MS') {
        print STDOUT '*** '.$script.': unexpected word-3=['.$words[2]
                     .'] in str=['.$str.'] contained in pv='.$pv."\n"
                     .'The word should be "MS"'."\n\n";
        return -1;
     }
     $words[0] =~ s/${val}$//;					#remove ':RqsPos' at end
     push(@$drv_ref,$words[0]);					#drives
  }
  return 0;
}


#=================================================================

# sub getAssyMotorsFromTsub($assy,\@motors) {			#returns $status
sub getAssyMotorsFromTsub ($\@\@\@) {

### This routine reads assembly tsub records to find all
### motors which belong to the assembly

  my ($script, $assy, $mtr_ref, $status);
  my ($val, $tsub, $pv, $str, @inputs, $i, @words, $nwords);

  $script  = 'getAssyMotorsFromTsub';
  $assy    = shift(@_);						#input: EPICS name of motors assembly
  $mtr_ref = shift(@_);						#output: pointer to motors array to be filled

  @$mtr_ref = ();
  $assy   = &filterAssy($assy);
  $val    = ':RqsPos';
  $tsub   = $assy.$val.'TsubMtr';				#tsub record name
  @inputs = qw(A B C D);

  foreach $i (@inputs) {
     $pv = $tsub.'.INP'.$i;				 	#motors
     ($status,$str) = Pezca::GetString($pv);
     if ($status) {&report_only($script, $status, $pv); return $status;}
     if (length($str) == 0) {last;}
     @words = split(/\s+/,$str);
     $nwords = @words;
     if ($nwords != 3) {
        print STDOUT '*** '.$script.': unexpected number of words='.$nwords
                     .' in str=['.$str.'] contained in pv='.$pv."\n\n";
        return -1;
     }
     elsif ($words[0] !~ /${val}$/) {
#       print STDOUT '!!! '.$script.': unexpected word-1=['.$words[0]
#                    .'] in str=['.$str.'] contained in pv='.$pv."\n"
#                    .'The word should contain ['.$val.'].'."\n\n";
        last;
     }
     elsif ($words[1] ne 'PP' && $words[1] ne 'NPP') {
        print STDOUT '*** '.$script.': unexpected word-2=['.$words[1]
                     .'] in str=['.$str.'] contained in pv='.$pv."\n"
                     .'The workd should be "PP" or "NPP"'."\n\n";
        return -1;
     }
     elsif ($words[2] ne 'MS') {
        print STDOUT '*** '.$script.': unexpected word-3=['.$words[2]
                     .'] in str=['.$str.'] contained in pv='.$pv."\n"
                     .'The word should be "MS"'."\n\n";
        return -1;
     }
     $words[0] =~ s/${val}$//;					#remove ':RqsPos' at end
     push(@$mtr_ref,$words[0]);					#motors
  }
  return 0;
}


#=================================================================

# sub getAssyByPCS($pmac,$pcs,\$assy) {	#returns $status and fills $assy
sub getAssyByPCS ($$\$) {

  our ($beamline);
  my ($script, $pmac, $pcs, $assyRef, $status);
  my ($bl, @assys, @pcss, $i, $nassys);

  $script  = 'getAssyByPCS';
  $pmac    = shift(@_);						#input: pmac name
  $pcs     = shift(@_);						#input: PCS number
  $assyRef = shift(@_);						#output: assembly name

  $$assyRef = '';
  $status   = 0;

# $i = index($pmac,':');  if ($i <= 0) {return 1;}  $bl = substr($pmac,0,$i+1);
  $bl    = $beamline;
  @assys = ();
  @pcss  = ();
  $status = &getAssyList4Pmac ($bl,$pmac,\@assys,\@pcss);
  if ($status) {return 1;}
  $nassys = @assys;
  for ($i=0; $i<$nassys; $i++) {
     if ($pcss[$i] == $pcs) {
        $$assyRef = $assys[$i];
        if ($$assyRef !~ /^$bl/) {$$assyRef = $bl.':'.$$assyRef;}
        return 0;
     }
  }
  print STDOUT '!!! '.$script.': no assembly found for pmac='.$pmac.', pcs='.$pcs."\n";
  return 2;
}


#=================================================================

# $status = &getMotorSrvStatus($motor,\@bits);
sub getMotorSrvStatus ($\@) {

# [b0] Rapid max velocity select (Ixx90)
# [b1] Sign/magnitude servo ena (Ixx96)
# [b2] Software-capture ena (Ixx97 bit 0)
# [b3] Capture on error ena (Ixx97 bit 1)
# [b4] Pos follow ena (Ixx06 bit 0)
# [b5] Pos follow offset mode (Ixx06 bit 1)
# [b6] Communication enable (Ixx01 bit 0)
# [b7] Y-addr commute enc (Ixx01 bit 1)
# [b8] User written servo ena (Ixx59 bit 1)
# [b9] User-written pha ena (Ixx59 bit 1)
# [b10] Home search in progress
# [b11] Block request
# [b12] Abort deceleration in progress
# [b13] Desired velocity 0
# [b14] Data block error
# [b15] Dwell in progress
# [b16] Integration mode (Ixx34; 0 always)
# [b17] Move timer active
# [b18] Open loop mode
# [b19] Amplifier enabled
# [b20] Ext servo algo ena (Iyy00/Iyy50)
# [b21] Positive end limit set (soft or hard)
# [b22] Negative end limit set (soft or hard)
# [b23] Motor activated (Ixx00)

  my ($script, $motor, $pv, $word, $bitsref, $status, $i);

  $script  = 'getMotorSrvStatus';
  $motor   = shift(@_);         if ($motor !~ /:$/) {$motor .= ':';}
  $bitsref = shift(@_);

  $pv = $motor.'SvoSts';

  @$bitsref = ();

  ($status,$word) = Pezca::GetDouble($pv);
  if ($status) {
     &report_only($script, $status, $pv);
     return $status;
  }

  $word = &dec2bin($word,'r',24);				#convert integer into bits string
# @$bitsref = split(//,substr($word,0,24));
  @$bitsref = split(//,$word);

  return $status;
}


#=================================================================

# $status = &getMotorSrvStat2($motor,\@bits);
sub getMotorSrvStat2 ($\@) {

# [b0] In-position true
# [b1] Fatal following error exceeded
# [b2] Software-capture ena (Ixx97 bit 0)
# [b3] Amplifier fault error
# [b4] Backlash direction flag
# [b5] I2T Amplifier fault error
# [b6] Integrated fatal following error
# [b7] Trigger move
# [b8] Phasing searh error
# [b9] Motor Phase Request
# [b10] Home complete
# [b11] Stopped on position limit
# [b12] Desired position limit stop
# [b13] Foreground in-position
# [b14] (Reserved for future use)
# [b15] Assigned to C.S.
# [b16] CS Axis definition bit 0
# [b17] CS axis definition bit 1
# [b18] CS Axis definition bit 2
# [b19] CS Axis definition bit 3
# [b20] (CS-1) # bit 0 (LSB)
# [b21] (CS-1) # bit 1
# [b22] (CS-1) # bit 2
# [b23] (CS-1) # bit 3 (MSB)

  my ($script, $motor, $pv, $word, $bitsref, $status, $i);

  $script  = 'getMotorSrvStat2';
  $motor   = shift(@_);         if ($motor !~ /:$/) {$motor .= ':';}
  $bitsref = shift(@_);

  $pv = $motor.'Sts2';

  @$bitsref = ();

  ($status,$word) = Pezca::GetDouble($pv);
  if ($status) {
     &report_only($script, $status, $pv);
     return $status;
  }

  $word = &dec2bin($word,'r',24);				#convert integer into bits string
# @$bitsref = split(//,substr($word,0,24));
  @$bitsref = split(//,$word);

  return $status;
}


#=================================================================

# $status = &getAssyExeSts($assy,\@bits);
sub getAssyExeSts ($\@) {

# [b0] CIRCLE/SPLINE move mode
# [b1] CCW CIRCLE/RAPID move mode
# [b2] Cutter compensation on
# [b3] Cutter compensation left
# [b4] PVT/SPLINE move mode
# [b5] Segmented move stop request
# [b6] Segmented move acceleration in progress
# [b7] Segmented move in progress
# [b8] Pre Jog move flag
# [b9] Cutter compensation move buffered
# [b10] Cutter compensation move stop request
# [b11] Cutter compensation outside corner
# [b12] Dwell move buffered
# [b13] Synchronous M-variable one-shot
# [b14] End-of-block (/) stop in progress
# [b15] Delayed calculation flag
# [b16] Rotary buffer full
# [b17] In-position (all motors)
# [b18] Warning following error (any motor)
# [b19] Fatal following error (any motor)
# [b20] Amplifier fault error (any motor)
# [b21] Circle radius error
# [b22] Run time error (err code in Y:$002x14)
# [b23] Lookahead in progress

  my ($script, $assy, $pv, $word, $bitsref, $status, $i);

  $script  = 'getAssyExeSts';
  $assy    = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $bitsref = shift(@_);

  $pv = $assy.'PrgExeSts';

  @$bitsref = ();

  ($status,$word) = Pezca::GetDouble($pv);
  if ($status) {
     &report_only($script, $status, $pv);
     return $status;
  }

  $word = &dec2bin($word,'r',24);				#convert integer into bits string
# @$bitsref = split(//,substr($word,0,24));
  @$bitsref = split(//,$word);

  return $status;
}


#=================================================================

# $status = &getAssySts1($assy,\@bits);
sub getAssySts1 ($\@) {

# [b0] Program running
# [b1] Single step mode
# [b2] Continuous-motion mode
# [b3] Move-specified-by-time mode
# [b4] Continuous-motion request
# [b5] Radius vector incremental mode
# [b6] A-axis incremental mode
# [b7] A-axis used in feedrate calculations
# [b8] B-axis incremental mode
# [b9] B-axis used in feedrate calculations
# [b10] C-axis incremental mode
# [b11] C-axis used in feedrate calculations
# [b12] U-axis incremental mode
# [b13] U-axis used in feedrate calculations
# [b14] V-axis incremental mode
# [b15] V-axis used in feedrate calculations
# [b16] W-axis incremental mode
# [b17] W-axis used in feedrate calculations
# [b18] X-axis incremental mode
# [b19] X-axis used in feedrate calculations
# [b20] Y-axis incremental mode
# [b21] Y-axis used in feedrate calculations
# [b22] Z-axis incremental mode
# [b23] Z-axis used in feedrate calculations

  my ($script, $assy, $pv, $word, $bitsref, $status, $i);

  $script  = 'getAssySts1';
  $assy    = shift(@_);         if ($assy !~ /:$/) {$assy .= ':';}
  $bitsref = shift(@_);

  $pv = $assy.'Sts1';

  @$bitsref = ();

  ($status,$word) = Pezca::GetDouble($pv);
  if ($status) {
     &report_only($script, $status, $pv);
     return $status;
  }

  $word = &dec2bin($word,'r',24);				#convert integer into bits string
# @$bitsref = split(//,substr($word,0,24));
  @$bitsref = split(//,$word);

  return $status;
}


#=================================================================

#sub axis_afterScan($afterScan,\@xarr,\@yarr,$begin,$end,$axisIni,$axisPV,$assyPV) {    #returns status
sub axis_afterScan($\@\@$$$$$) {
### -----------------------------------------------------
### afterScan operations for &HDflyScan (strtuck_subs_v2.pl) and &SRflyScan (scanrec_subs.pl).
### Positions axis to startpos,priorpos,maxpos,peakpos,minpos,valleypos,centroid,gaussian,lorentzian and
### etc depending on the $autoScan flag:
###           0=stay, 1=startpos, 2=priorpos 
###           3=maxpos 4,14=peakpos(noisy), 5,15=peakpos(smooth)
###           6=minpos 7,17=valleypos(noisy), 8,18=valleypos(smooth)
###           9=centroid, 10,110=gaussian 11,111=lorentzian
### -----------------------------------------------------

  my ($script, $afterScan, $xname, $yname, $begin, $end);
  my ($axisIni, $axisPV, $assyPV);
  my ($npts, $axisDest, $I_range, $I_noise);
  my ($yesno, $msg, $status, $searchType, $discriminator);
  my ($y_max, $y_min, $pt_max, $pt_min, $pt_FWHM);
  my ($pt_left, $pt_right, $x_left, $x_right, $fit);
  my ($ybase, $yscale, $xwidth, $wfactor, $x_peak, $x_valley);

  $afterScan = shift(@_);  if (!defined $afterScan) {$afterScan=0;}     #valid options 0-11,14,15,17,18,110,111
  $xname     = shift(@_);
  $yname     = shift(@_);
  $begin     = shift(@_);
  $end       = shift(@_);
  $axisIni   = shift(@_);
  $axisPV    = shift(@_);         if ($axisPV !~ /:$/) {$axisPV .= ':';}
  $assyPV    = shift(@_);         if ($assyPV !~ /:$/) {$assyPV .= ':';}

  $script  = 'axis_afterScan';
  $I_noise = 4.;							#noise intensity variation threshold
  $npts    = @$yname;

  ($pt_min,$y_min,$pt_max,$y_max) = &arrayStatsRef(\@$yname);

  $I_range = $y_max-$y_min;

  if ($I_range <= $I_noise && $afterScan >= 3) {
     printf STDOUT '!!! '.$script.': noise-level ('.$I_range.'cts) intensity variation during the scan.'."\n"
                  .'!!! Driving '.$axisPV.' to initial coordinate=%.3f'."\n",$axisIni;
     $afterScan = 2;
  }
  
# $afterScan: 0=stay, 1=startpos, 2=priorpos 
#             3=maxpos 4,14=peakpos(noisy), 5,15=peakpos(smooth)
#             6=minpos 7,17=valleypos(noisy), 8,18=valleypos(smooth)
#             9=centroid, 10,110=gaussian 11,111=lorentzian
  $yesno = 1;
  if    ($afterScan == 14)  {$afterScan=4;  undef $yesno;}
  elsif ($afterScan == 15)  {$afterScan=5;  undef $yesno;}
  elsif ($afterScan == 17)  {$afterScan=7;  undef $yesno;}
  elsif ($afterScan == 18)  {$afterScan=8;  undef $yesno;}
  elsif ($afterScan == 110) {$afterScan=10; undef $yesno;}
  elsif ($afterScan == 111) {$afterScan=11; undef $yesno;}

  if ($afterScan == 10 || $afterScan == 11) {				#Gaussian or Lerentzian fit
     if (!defined &peakFit) {
        print STDOUT '!!! '.$script.': no peakFit function available.'."\n"
                    .'!!! Perhaps common_PDL.pl is not imported.'."\n"
                    .'!!! Reverting to afterScan="Noisy Peak".'."\n";
        $afterScan = 4;
     }
  }

  if    ($afterScan == 0) {						#0=stay
     $axisDest = $end;
     printf STDOUT $script.': leaving '.$axisPV.' at the end of scan coordinate=%.3f'."\n",$axisDest;
     sleep (2);
     $status = 0;
  }
  elsif ($afterScan == 1) {						#1=startpos (begin of scan interval)
     $axisDest = $begin;
     printf STDOUT $script.': driving '.$axisPV.' to the beginning of scan coordinate=%.3f'."\n",$axisDest;
     $status = &runMotor($assyPV,$axisPV,$axisDest,'forceAuto');
  }
  elsif ($afterScan == 2) {						#2=priorpos (initial position before scan)
     $axisDest = $axisIni;
     printf STDOUT $script.': driving '.$axisPV.' to the initial coordinate=%.3f'."\n",$axisDest;
     $status = &runMotor($assyPV,$axisPV,$axisDest,'forceAuto');
  }
  elsif ($afterScan == 3) {						#3=maxpos
     $axisDest = @$xname[$pt_max];
     printf STDOUT $script.': driving '.$axisPV.' to the I0-max coordinate=%.3f'."\n",$axisDest;
     $status = &runMotor($assyPV,$axisPV,$axisDest,'forceAuto');
  }
  elsif ($afterScan == 4 || $afterScan == 5) {				#4=peakpos(noisy),5=peakpos(smooth)
     if    ($afterScan == 4) {$searchType='noisy';}			#search half-peak from tails
     elsif ($afterScan == 5) {$searchType='smooth';}			#search half-peak from max.intensity
     ($y_max,$y_min,$pt_max,$pt_min,$pt_FWHM,
      $pt_left,$pt_right,$x_left,$x_right,$x_peak) = &findPeakRef($npts,\@$yname,'auto',$searchType);
     if ($pt_FWHM > 0) {
        $axisDest = 0.5*(&pt2coord($x_left,\@$xname) + &pt2coord($x_right,\@$xname));
        printf STDOUT $script.': driving '.$axisPV.' to '.$searchType.' I0-peak coordinate=%.3f'."\n",$axisDest;
     } else {
        if (!defined $yesno) {
           $msg = sprintf 'No '.$searchType.' I0-peak found during '.$axisPV.' scan. Do you want'."\n"
                         .'to drive '.$axisPV.' to the maximum I0 coordinate=%.3f ?',@$xname[$pt_max];
           $yesno = &zenity_question($msg);
        }
        print STDOUT '!!! '.$script.': no '.$searchType.' I0-peak found. '."\n";
        if ($yesno) {
           $axisDest = @$xname[$pt_max];
           printf STDOUT '!!! '.$script.': Driving '.$axisPV.' to the I0-max coordinate=%.3f'."\n",$axisDest;
        } else {
           $axisDest = $axisIni;
           printf STDOUT '!!! '.$script.': Driving '.$axisPV.' to the initial coordinate=%.3f'."\n",$axisDest;
        }
     }
     $status = &runMotor($assyPV,$axisPV,$axisDest,'forceAuto');
  }
  elsif ($afterScan == 6) {						#6=minpos
     $axisDest = @$xname[$pt_min];
     printf STDOUT $script.': driving '.$axisPV.' to the I0-min coordinate=%.3f'."\n",$axisDest;
     $status = &runMotor($assyPV,$axisPV,$axisDest,'forceAuto');
  }
  elsif ($afterScan == 7 || $afterScan == 8) {				#7=valleypos(noisy),8=valleypos(smooth)
     if    ($afterScan == 7) {$searchType='noisy';}			#search half-valey from tails
     elsif ($afterScan == 8) {$searchType='smooth';}			#search half-valey from min.intensity
     ($y_max,$y_min,$pt_max,$pt_min,$pt_FWHM,
      $pt_left,$pt_right,$x_left,$x_right,$x_valley) = &findValleyRef($npts,\@$yname,'auto',$searchType);
     if ($pt_FWHM > 0) {
        $axisDest = 0.5*(&pt2coord($x_left,\@$xname) + &pt2coord($x_right,\@$xname));
        printf STDOUT $script.': driving '.$axisPV.' to '.$searchType.' I0-valley coordinate=%.3f'."\n",$axisDest;
     } else {
        if (!defined $yesno) {
           $msg = sprintf 'No '.$searchType.' I0-valley found during '.$axisPV.' scan. Do you want'."\n"
                         .'to drive '.$axisPV.' to the minimum I0 coordinate=%.3f ?',@$xname[$pt_min];
           $yesno = &zenity_question($msg);
        }
        print STDOUT '!!! '.$script.': no '.$searchType.' I0-valley found. '."\n";
        if ($yesno) {
           $axisDest = @$xname[$pt_min];
           printf STDOUT '!!! '.$script.': Driving '.$axisPV.' to the I0-min coordinate=%.3f'."\n",$axisDest;
        } else {
           $axisDest = $axisIni;
           printf STDOUT '!!! '.$script.': Driving '.$axisPV.' to the initial coordinate=%.3f'."\n",$axisDest;
        }
     }
     $status = &runMotor($assyPV,$axisPV,$axisDest,'forceAuto');
  }
  elsif ($afterScan == 9) {						#9=centroid
     $discriminator = 0.1;						#only account points with intensity > 10% of max
     $axisDest = &findCentroidRef($npts,\@$xname,\@$yname,$discriminator);
     printf STDOUT $script.': driving '.$axisPV.' to the I0-centroid coordinate=%.3f'."\n",$axisDest;
     $status = &runMotor($assyPV,$axisPV,$axisDest,'forceAuto');
  }
  elsif ($afterScan == 10 || $afterScan == 11) {			#10=Gaussian 11=Lorentzian ask
     ### Gaussian fit is:   $y=$ybase+$yscale*exp(-0.5*(($x-$x_peak)/$xwidth)**2)
     ### Lorentzian fit is: $y=$ybase+$yscale/(1+((x-$x_peak)/$x_width)**2) 
     if    ($afterScan == 10) {$fit = 'Gaussian';   $wfactor=2.35482;}	#$FWHM=$xwidth*(2*sqrt(2*ln(2)))
     elsif ($afterScan == 11) {$fit = 'Lorentzian'; $wfactor=2.0;}	#$FWHM=$xwidth*2
     ($status,$ybase,$yscale,$x_peak,$xwidth) = &peakFit(\@$xname, \@$yname, $fit);
     if ($status == 0) {
        printf STDOUT $script.': '.$fit.' fit xcenter=%.3f  xwidth=%.3f  FWHM=%.3f'."\n",$x_peak,$xwidth,$xwidth*$wfactor;
        $axisDest = $x_peak;
        printf STDOUT $script.': driving '.$axisPV.' to the '.$fit.' fit of I0-peak coordinate=%.3f'."\n",$axisDest;
     } else {
        print STDOUT '*** '.$script.': '.$fit.' fitting failed with status='.$status."\n";
        if (!defined $yesno) {
           $msg = sprintf 'No '.$fit.' I0-peak found during '.$axisPV.' scan. Do you want'."\n"
                         .'to drive '.$axisPV.' to the maximum I0 coordinate=%.3f ?',@$xname[$pt_max];
           $yesno = &zenity_question($msg);
        }
        print STDOUT '!!! '.$script.': no '.$fit.' I0-peak found. '."\n";
        if ($yesno) {
           $axisDest = @$xname[$pt_max];
           printf STDOUT '!!! '.$script.': Driving '.$axisPV.' to the I0-max coordinate=%.3f'."\n",$axisDest;
        } else {
           $axisDest = $axisIni;
           printf STDOUT '!!! '.$script.': Driving '.$axisPV.' to the initial coordinate=%.3f'."\n",$axisDest;
        }
     }
     $status = &runMotor($assyPV,$axisPV,$axisDest,'forceAuto');
  }
  else {
     print STDOUT '!!! '.$script.': invalid option value afterScan='.$afterScan."\n";
     $status = -1;
  }
  return $status;
}
